C500-500 C520/C530 Main Application Code
==========

The file structure is as follows:

* app -> Application code, builds using autoconf
* config -> Configuration files
* lib -> Libraries required to build the app code. These are included using git subtree, at fixed versions
* manual -> Reference manual source code (Latex)
* opkg -> Scripts to be run on package install/removal
* scripts -> rc.d initialisation script
* build_* -> Out of tree build folder for all libraries, application and opkg for a particular target
* M0x -> Release binary folders
 * M01: C500 opkg
 * M02: C500 rpk
 * M03: PC build opkg
 * M04: PC build rpk


## Building: ##

* PATH must contain m68k-uclinux-gcc
* Building all targets (does clean before building):
 * `make all -j4`
* Building for C500 (incremental build of application, libraries are only built the first time):
 * `make m02 -j4`
* Building for PC (incremental build of application, libraries are only built the first time):
 * `make m04 -j4`


## Changing the version: ##

* Version is set in app/configure.ac


## The libraries: ##

These are included as git subtrees. The versions used in the build are specified in the Makefile.

```
  git subtree add --squash -P lib/libmodbus-20160327-2 git@chef.rinstrumau.local:opensource/libmodbus.git 20160327-2
  git subtree add --squash -P lib/libmodbus-20170106 git@chef.rinstrumau.local:opensource/libmodbus.git 20170106
  git subtree add --squash -P lib/librinhw-master git@chef.rinstrumau.local:rnd/librinhw.git master
  git subtree add --squash -P lib/L000-503-1.0.0 git@chef.rinstrumau.local:rnd/L000-503.git v1.0.0
  git subtree add --squash -P lib/setnetconfig git@chef.rinstrumau.local:rnd/setnetconfig.git master
```

To update a library subtree from chef:

```
  git subtree pull --squash -P lib/librinhw-master git@chef.rinstrumau.local:rnd/librinhw.git master
  git subtree pull --squash -P lib/setnetconfig git@chef.rinstrumau.local:rnd/setnetconfig.git master
```

When committing changes, commit separately to the library, so a commit doesn't contain
some library changes and some app changes. After making changes and committing push back using:

```
  git subtree push -P lib/librinhw-master git@chef.rinstrumau.local:rnd/librinhw.git master
  git subtree push -P lib/setnetconfig git@chef.rinstrumau.local:rnd/setnetconfig.git master
```


## The Manual ##

See [here](manual/README.md)


## Running C500 application on the PC (with C500 required for ADC and I2C devices): ##

Getting set up:
 - Build librinhw for M68k, and copy librinhw_remote_server to the C500
 - Build and install setnetconfig for PC:
     git clone git@chef.rinstrumau.local:rnd/setnetconfig.git && cd setnetconfig && ./autogen.sh && ./configure && make clean && make
     sudo make install
 - On PC, install u-boot tools: sudo apt install u-boot-tools
 - Create uboot config file /etc/fw_env.config:
# Configuration file for fw_(printenv/setenv) utility.
# Up to two entries are valid, in this case the redundant
# environment sector is assumed present.
# MTD device name       Device offset   Env. size       Flash sector size
/tmp/uboot.bin          0x0000          0x2000          0x2000
 - Set permissions: sudo chmod 664 /etc/fw_env.config
 - Create uboot.bin (repeat every boot): dd if=/dev/zero of=/tmp/uboot.bin bs=1 count=8K
 - Copy all files from C500 /etc/rinstrum to PC /home/<username>/rinstrum
 - Setup config file: cp config/weconfig_pc.xml /home/<username>/rinstrum/weconfig.xml
 - Create (empty) nvram file: dd if=/dev/zero of=/home/<username>/rinstrum/we21nvram.bin bs=1 count=32K
 - Edit /home/<username>/rinstrum/weconfig.xml paths to suit /home/<username>
 - Change C500 Makefile rule "all: m02" to "all: m04"
 - In C500 folder: "make clean && make configure && make"
 - Fix up the ADC factory database checksum:
 -   app/src/facdbtool/facdbtool -e guess -n little /home/<username>/rinstrum/factory1.rdb fixendian -o /home/<username>/rinstrum/factory1.tmp
 -   cp /home/<username>/rinstrum/factory1.tmp /home/<username>/rinstrum/factory1.rdb
 -   app/src/facdbtool/facdbtool -e guess -n little /home/<username>/rinstrum/factory2.rdb fixendian -o /home/<username>/rinstrum/factory2.rdb
 -   cp /home/<username>/rinstrum/factory2.tmp /home/<username>/rinstrum/factory2.rdb
 - Connect rinlink from PC USB to rear port on C500
 - Make sure you can access serial devices: sudo adduser <username> dialout

Running over serial (best):
 - Stop C500 application on C500: "/etc/rc.d/init.d/c500-500 stop"
 - On C500 run: "./librinhw_remote_server --remote=/dev/ttyS1 --syslog --stdout"
 - On PC run: "app/src/C500_500 --config=/home/<username>/rinstrum/weconfig.xml --syslog --stdout --remote=/dev/ttyUSB0"

Running over TCP (seems slow):
 - Stop C500 application on C500: "/etc/rc.d/init.d/c500-500 stop"
 - On C500 run: "./librinhw_remote_server --tcp=7020 --syslog --stdout"
 - On PC run: "app/src/C500_500 --config=/home/<username>/rinstrum/weconfig.xml --syslog --stdout --remote=<C500ipaddess>:7020"

Debugging tips:
 - To run under valgrind:
     "LD_LIBRARY_PATH=lib/libmodbus-20170106/src/.libs valgrind app/src/.libs/C500_500 --config=/home/<username>/rinstrum/weconfig.xml --syslog --stdout --remote=/dev/ttyUSB0"
 - Tricking the device mounter to see a device, since we don't have mdev on the PC:
     "env -i ACTION=add SUBSYSTEM=tty MDEV=ttyUSB1 app/src/devicenotify/devicenotify /tmp/rinpredevmount /tmp/rindevmount"
