#ifndef ACCESS_DOT_H
#define ACCESS_DOT_H 1

#include <time.h>
#include <rinhw/rio.h>
#include <rinblock/rb_dio.h>
#include "setpoint.h"


/** Fullscale on the analog output DAC */
#define ANALOG_OUTPUT_DAC_FULLSCALE 50000


/** Configure an IO pin as either and output or input */
void access_configure_iopin(uint8_t pin, enum eDIOType iotype);


/** Configure all IO from the database settings */
void access_dbconfigure_allio(void);


/**
 * Initalise IO & analog rinblocks, plus do a check that all rinblocks are running correctly
 *
 * @param headless is whether we are running without a display
 */
void init_access(bool headless);


/***************************************************
access_get_outputs()
returns the current state of the Accessory Bus outputs
**************************************************/
uint8_t access_get_outputs(void);

/***************************************************
access_set_outputs()
sets the current state of the Accessory Bus outputs
**************************************************/
void access_set_outputs(uint8_t outputs);

/***************************************************
access_get_inputs()
returns the current state of the Accessory Bus inputs
**************************************************/
uint8_t access_get_inputs(void);


enum ePulseCount
{
  PULSE_COUNT_1 = 0,
  PULSE_COUNT_2,
  PULSE_COUNT_NUM
};

/***************************************************
access_get_pulses()
returns the pulse count of the Accessory Bus inputs
**************************************************/
int32_t access_get_pulses(enum ePulseCount pulse);

/***************************************************
access_clear_pulses()
clears the pulse count of the Accessory Bus inputs
**************************************************/
void access_clear_pulses(enum ePulseCount pulse);


/** From the digital inputs, return a mask of any configured remote keys that are pressed */
unsigned long access_kbd_readhikeybits(void *hw_data);


/** Forcing the analog output when adjusting */
enum eAnalogForce
{
  ANALOGOUT_FORCE_NONE = 0,
  ANALOGOUT_FORCE_LOW,
  ANALOGOUT_FORCE_HIGH,
  ANALOGOUT_FORCE_NUM
};


/** Used to force analog out state low or high for adjustment, as per @ref enum eAnalogForce */
void access_force_analogue(enum eAnalogForce force_state);


/* Get the current forced state of the analog output */
enum eAnalogForce access_get_force_analogue(void);


/** Configure the analog output card with the database settings */
void access_dbconfigure_analog(void);


/**
 * Initialise the rinblocks, and create any necessary devices
 *
 * @param headless is whether run as a headless device (ie no lcd, buzzer or keyboard)
 * @param iicdev is the i2c device to use for the rinblock bus
 * @param adcdev is the adc device to create if a rinblock supporting an ADC is found. If NULL no device will be created
 * @param kbddev is the keyboard device to create if a rinblock supporting a keyboard is found. If NULL no device will be created
 * @param lcddev is the LCD device to create if a rinblock supporting a LCD is found. If NULL no device will be created
 * @param buzdev is the buzzer device to create if a rinblock supporting a buzzer is found. If NULL no device will be created
 * @param ser_rb1a_dev is the serial device to create for a serial rinblock A channel on port 1. If NULL no device will be created
 * @param ser_rb1b_dev is the serial device to create for a serial rinblock B channel on port 1. If NULL no device will be created
 * @param ser_rb2a_dev is the serial device to create for a serial rinblock A channel on port 2. If NULL no device will be created
 * @param ser_rb2b_dev is the serial device to create for a serial rinblock B channel on port 2. If NULL no device will be created
 * @param ser_frontrl_dev is the serial device to create for a rinlink serial on the display. If NULL no device will be created
 */
void init_rinblocks(bool headless, struct t_io_dev *iicdev, struct t_io_dev *adcdev, struct t_io_dev *kbddev, struct t_io_dev *lcddev, struct t_io_dev *buzdev,
                    struct t_io_dev *ser_rb1a_dev, struct t_io_dev *ser_rb1b_dev, struct t_io_dev *ser_rb2a_dev, struct t_io_dev *ser_rb2b_dev,
                    struct t_io_dev *ser_frontrl_dev);


/** Clean up rinblock devices */
void destroy_rinblocks(void);


/** Write outputs and read inputs to/from IO cards */
void access_io_cards(struct s_setpoint_running_info *spri, struct timespec *start);


/** Return if IO is present on the WE2111 */
bool access_io_is_present(void);

/** Set comms analogue out valeu */
void access_set_register_analogue(int32_t val);

/** Get comms analogue out value */
int32_t acces_get_register_analogue(void);

#endif /* #ifndef ACCESS_DOT_H */
