#include <limits.h>
#include <rinhw/rio.h>
#include <rinhw/adc_up.h>

#include "adc_8bit.h"


/** This is the rinblock ADC device we use on the WE2111. Don't use dev_adc */
struct t_io_dev dev_adc_rb;


/** Reads converted level on ADC channel, in millivolts or kelvin */
void adc_read_all(long results[NUM_ADC_CHANNELS], uint16_t *temperature_raw)
{
  uint16_t buffer[NUM_ADC_CHANNELS];

  /* Open the device */
  if (ropen(&dev_adc_rb, 0) < 0)
  {
    return;
  }

  /* Read ADC data from device */
  if (sizeof(buffer) == rread(&dev_adc_rb, buffer, sizeof(buffer), 0))
  {
    /* Do conversions for the channels */
    *temperature_raw = buffer[ADC_CH_TEMP];
    results[ADC_CH_TEMP] = WE21_TEMP_KELVIN_REAL_FROM_CNT(buffer[ADC_CH_TEMP]);
    results[ADC_CH_VIN] = WE21_PWR_MV_REAL_FROM_CNT(buffer[ADC_CH_VIN]);
    results[ADC_CH_SNS_PLUS] = WE21_SEN_POS_MV_REAL_FROM_CNT(buffer[ADC_CH_SNS_PLUS]);
    results[ADC_CH_SNS_MINUS] = WE21_SEN_NEG_MV_REAL_FROM_CNT(buffer[ADC_CH_SNS_MINUS]);
  }

  /* Close the device */
  rclose(&dev_adc_rb);
}
