/*
 * app_db.h
 *
 *  Created on: Apr 17, 2012
 *      Author: ian
 */

#ifndef APP_DB_H_
#define APP_DB_H_

#include <rinweigh/adcdatatypes.h>


/* Change log masks */


/** Change log mask for factory database elements */
#define APP_DB_CLMASK_FACTORY    0x00000001


/** Change log mask for trade relevant database elements */
#define APP_DB_CLMASK_TRADE      0x00000002


/** Change log mask for user (non-trade relevant) database elements */
#define APP_DB_CLMASK_USER       0x00000004


/** Change log mask for runtime database elements */
#define APP_DB_CLMASK_RUNTIME    0x00000008


/** Change log mask for database events */
#define APP_DB_CLMASK_DATABASE   0x00000010


/** Change log mask for licence database elements */
#define APP_DB_CLMASK_LICENCE    0x00000020


/** Keep a record of these elements in the change log */
#define APP_DB_CHANGELOG_MASK (APP_DB_CLMASK_TRADE | APP_DB_CLMASK_DATABASE)


/** Strings to describe each of the change log masks */
#define APP_DB_CLMASK_STRS { "Factory", "Trade", "User", "Runtime", "Database", "Licence" }


/**
 * Check if the software version has changed, and if so, bump the
 * calibration counters, put an entry in the change log
 * and save the user database (containing the current software version)
 *
 * NOTE: This can only be called after the change log, user database and
 * ADC thread/adcSuperStructure have been initialised
 */
void app_db_check_software_version(void);


/** Initialise and read all the databases. Call only once on startup. */
void app_init_and_read_databases(void);


/** Destroy all the databases */
void app_destroy_databases(void);


/** Save the databases, this function is used as a callback from the ADC library */
bool app_save_databases(AdcSuperStructure *ass, const AdcFactoryDatabase *afd, const AdcUserDatabase *aud, const AdcRuntimeDatabase *ard);


#endif /* APP_DB_H_ */
