/*
 * changelog.c
 *
 *  Created on: Apr 30, 2012
 *      Author: ian
 */
#include <time.h>
#include <rindb/db_changelog.h>
#include <rindb/db_changelog_reader.h>
#include "we_config.h"
#include "app_db.h"
#include "app_db_user.h"
#include "filebackup.h"
#include "adcthread.h"

#include "app_db_changelog.h"


/** Strings to describe each of the change log masks */
static const char *const app_db_changelog_mask_strings[] = APP_DB_CLMASK_STRS;

static const char *const app_db_changelog_header_rows[dbcleNum] =
{
  "CalCounter",
  "ConfigCounter",
  "Time",
  "DB Name",
  "DB Element Name",
  "DB Menu Name",
  "Index",
  "Old Value",
  "New Value",
  "Log Mask"
};


/** System change log */
struct s_db_changelog *app_changelog = NULL;


/** System change log reader */
struct s_db_changelog_reader *app_changelog_reader = NULL;


/** Create the database change log */
bool app_changelog_init(void)
{
  enum eDBError res;

  /* Create the change log */
  res = db_changelog_create(&app_changelog,
                            we_configuration.changelog_file,
                            WECFG_CHGLOG_MAX_FILESIZE,
                            APP_DB_CHANGELOG_MASK,
                            app_db_changelog_mask_strings,
                            WECFG_CHGLOG_EVENT_NAME,
                            WECFG_CHGLOG_EVENT_NAME,
                            WECFG_CHGLOG_INITIAL_ENTRY,
                            WECFG_CHGLOG_CLEARED_ENTRY,
                            app_db_changelog_header_rows,
                            &adcSuperStructure.adcUserDatabase.calCntOIML,
                            NULL,
                            time(NULL),
                            APP_DB_U_NAME,
                            APP_DB_CLMASK_DATABASE);

  if (dbeSuccess == res)
  {
    /* Create the change log reader */
    res = db_changelog_reader_create_filename(&app_changelog_reader, we_configuration.changelog_file, 1024);
  }

  return (dbeSuccess == res);
}


/** Shut down the change log */
bool app_changelog_destroy(void)
{
  /* Destroy the change log */
  (void) db_changelog_destroy(app_changelog);
  app_changelog = NULL;

  /* Destroy the change log reader */
  (void) db_changelog_reader_destroy(app_changelog_reader);
  app_changelog_reader = NULL;

  return true;
}


/** Determine if the changelog is full, in which case the databases can't be saved */
bool app_can_save_databases(void)
{
  enum eDBError res;

  res = db_changelog_is_full(app_changelog);
  return (dbeChangelogFull != res);
}


/** Update the change log with any changes to database */
bool app_changelog_update(struct s_db_database *database)
{
  db_changelog_update_database(app_changelog, time(NULL), database, database->db_database_version);

  db_database_update_previous(database);
  return true;
}


/** Back up the change log file to a file called "folder/WE2111_serno_date_time_changelog.csv" */
bool app_changelog_backup_file(const char *folder)
{
  char str[255];

  /* Create the filename */
  create_backup_filename(str, 255, folder, "changelog", ".csv");

  return (dbeSuccess == db_changelog_slave_copy(app_changelog, str));
}

