/*
 * app_db_defaults.c
 *
 *  Created on: Jun 16, 2012
 *      Author: ian
 */
#include <string.h>
#include "app_db_runtime.h"
#include "app_db_user.h"

#include "app_db_defaults.h"


/** WE2111 default values for the ADC user database */
static const AdcUserDatabase AppDefaultAdcUserDB =
{
  .zeroRaw = APP_DB_U_ADCZERORAW_DEFAULT,
  .spanRaw = APP_DB_U_ADCSPANRAW_DEFAULT,
  .spanCnt = APP_DB_U_ADCSPANCNT_DEFAULT,
  .linPts =
  {                                           /* int8u  linPts[ADC_MAX_NUM_LIN_PTS+1]; */
    APP_DB_U_ADCLINPTS_DEFAULT, APP_DB_U_ADCLINPTS_DEFAULT, APP_DB_U_ADCLINPTS_DEFAULT, APP_DB_U_ADCLINPTS_DEFAULT, APP_DB_U_ADCLINPTS_DEFAULT,
    APP_DB_U_ADCLINPTS_DEFAULT, APP_DB_U_ADCLINPTS_DEFAULT, APP_DB_U_ADCLINPTS_DEFAULT, APP_DB_U_ADCLINPTS_DEFAULT, APP_DB_U_ADCLINPTS_DEFAULT,
    APP_DB_U_ADCLINPTSLAST_DEFAULT
  },
  .linCor =
  {                                           /* int32s linCor[ADC_MAX_NUM_LIN_PTS+1]; */
    APP_DB_U_ADCLINCOR_DEFAULT, APP_DB_U_ADCLINCOR_DEFAULT, APP_DB_U_ADCLINCOR_DEFAULT, APP_DB_U_ADCLINCOR_DEFAULT, APP_DB_U_ADCLINCOR_DEFAULT,
    APP_DB_U_ADCLINCOR_DEFAULT, APP_DB_U_ADCLINCOR_DEFAULT, APP_DB_U_ADCLINCOR_DEFAULT, APP_DB_U_ADCLINCOR_DEFAULT, APP_DB_U_ADCLINCOR_DEFAULT,
    APP_DB_U_ADCLINCOR_DEFAULT
  },

  .scaleType = APP_DB_U_ADCSCALETYPE_DEFAULT,
  .syncFreqIdx = APP_DB_U_ADCSYNCFREQ_DEFAULT,
  .jitter = APP_DB_U_ADCJITTER_DEFAULT,
  .dpPos = APP_DB_U_ADCDPPOS_DEFAULT,
  .fullScale = { APP_DB_U_ADCFS0_DEFAULT, APP_DB_U_ADCFS1_DEFAULT, APP_DB_U_ADCFS2_DEFAULT },
  .resolution = { APP_DB_U_ADCRES0_DEFAULT, APP_DB_U_ADCRES1_DEFAULT, APP_DB_U_ADCRES2_DEFAULT },
  .units = APP_DB_U_ADCUNITS_DEFAULT,
  .calWeight = APP_DB_U_ADCCALWEIGHT_DEFAULT,
  .addTare = APP_DB_U_ADCADDTARE_DEFAULT,
  .zeroBand = APP_DB_U_ADCZEROBAND_DEFAULT,
  .motion = APP_DB_U_ADCMOTION_DEFAULT,
  .zeroTracking = APP_DB_U_ADCZEROTRACK_DEFAULT,
  .average = APP_DB_U_ADCAVERAGE_DEFAULT,
  .rangeOfZero = APP_DB_U_ADCRANGEOFZERO_DEFAULT,
  .approvalMode = APP_DB_U_ADCAPPROVMODE_DEFAULT,
  .zeroOnStart = APP_DB_U_ADCZEROONSTART_DEFAULT,
  .wire4Mode = APP_DB_U_ADC4WIRE_DEFAULT,
  .tempCalZero = APP_DB_U_ADCTEMPCALZERO_DEFAULT,
  .hiRes = APP_DB_U_ADCHIRES_DEFAULT,

  .qaEnable = APP_DB_U_ADCQAENABLE_DEFAULT,
  .qaDay = APP_DB_U_ADCQADAY_DEFAULT,
  .qaMonth = APP_DB_U_ADCQAMONTH_DEFAULT,
  .qaYear = APP_DB_U_ADCQAYEAR_DEFAULT,

  .calCntNTEPCalibration = APP_DB_U_ADCCCNTEPCAL_DEFAULT,
  .calCntNTEPSettings = APP_DB_U_ADCCCNTEPSET_DEFAULT,
  .calCntOIML = APP_DB_U_ADCCCOIML_DEFAULT,
  .calFlag = APP_DB_U_ADCCALFLAG_DEFAULT,

  .chksum = APP_DB_U_ADCCHKSUM_DEFAULT
};


/** WE2111 default values for the ADC runtime database */
static const AdcRuntimeDatabase AppDefaultAdcRTDb =
{
  .cntZero = APP_DB_R_ADCCNTZERO_DEFAULT,
  .cntTare = APP_DB_R_ADCCNTTARE_DEFAULT,
  .cntPTare = APP_DB_R_ADCCNTPTARE_DEFAULT,
  .hiRes = APP_DB_R_ADCHIRES_DEFAULT,
  .netMode = APP_DB_R_ADCNETMODE_DEFAULT,
  .printId = APP_DB_R_ADCPRINTID_DEFAULT,
  .dispMode = APP_DB_R_ADCDISPMODE_DEFAULT,
  .tempCalOn = APP_DB_R_ADCTEMPCALON_DEFAULT,
  .overloadCnt = APP_DB_R_ADCOVERLOADCNT_DEFAULT,

  .chksum = APP_DB_R_ADCCHKSUM_DEFAULT
};


/**
 * Set a user database to WE2111 defaults
 *
 * @param aud is the user database to be defaulted
 * @param preserve_cal_counters causes the calibration counters to preserved (and incremented)
 * @param cal_only causes only the calibration settings to be defaulted (zero, span, span count, linearisation points)
 */
static void AppUserDatabaseDefault(AdcUserDatabase *aud, bool preserve_cal_counters, bool cal_only)
{
  int32u calCntNTEPCalibration;
  int32u calCntNTEPSettings;
  int32u calCntOIML;
  int8u i;

  /* Save current cal counters */
  calCntNTEPCalibration = aud->calCntNTEPCalibration;
  calCntNTEPSettings = aud->calCntNTEPSettings;
  calCntOIML = aud->calCntOIML;

  if (cal_only)
  {
    /* Default only the calibration settings */
    aud->zeroRaw = APP_DB_U_ADCZERORAW_DEFAULT;
    aud->spanRaw = APP_DB_U_ADCSPANRAW_DEFAULT;
    aud->spanCnt = APP_DB_U_ADCSPANCNT_DEFAULT;
    for (i = 0; i < ADC_MAX_NUM_LIN_PTS; i++)
    {
      aud->linPts[i] = APP_DB_U_ADCLINPTS_DEFAULT;
      aud->linCor[i] = APP_DB_U_ADCLINCOR_DEFAULT;
    }
    aud->linPts[ADC_MAX_NUM_LIN_PTS] = APP_DB_U_ADCLINPTSLAST_DEFAULT;
    aud->linCor[ADC_MAX_NUM_LIN_PTS] = APP_DB_U_ADCLINCOR_DEFAULT;
  }
  else
  {
    /* Default the entire database */
    memset(aud, 0, sizeof(AdcUserDatabase));
    *aud = AppDefaultAdcUserDB;
  }

  /* Restore cal counters */
  if (preserve_cal_counters)
  {
    aud->calCntNTEPCalibration = calCntNTEPCalibration;
    aud->calCntNTEPSettings = calCntNTEPSettings;
    aud->calCntOIML = calCntOIML;

    /* And increment */
    aud->calCntNTEPCalibration++;
    if (!cal_only)
      aud->calCntNTEPSettings++;
    aud->calCntOIML++;
  }
}


/** Set a runtime database to defaults */
static void AppRuntimeDatabaseDefault(AdcRuntimeDatabase *ard)
{
  /* Both checksums are bad, or overwrite forced, so blow away database and re-initialise */
  memset(ard, 0, sizeof(AdcRuntimeDatabase));
  *ard = AppDefaultAdcRTDb;
}


/** Implement the WE2111 database defaults */
void AppDatabaseDefaultImplementer(AdcSuperStructure *ass,
                                   AdcFactoryDatabase *afd,
                                   AdcUserDatabase *aud,
                                   AdcRuntimeDatabase *ard,
                                   bool preserve_cal_counters,
                                   bool cal_only)
{
#if 0
  if (NULL != afd)
  {
    /* It's not possible for the application to default the factory database */
  }
#endif
  if (NULL != aud)
  {
    AppUserDatabaseDefault(aud, preserve_cal_counters, cal_only);
  }
  if (NULL != ard)
  {
    AppRuntimeDatabaseDefault(ard);
  }
}
