/*
 * app_db_factory.c
 *
 *  Created on: Apr 10, 2012
 *      Author: ian
 */
#include <stdio.h>
#include <rinweigh/adcinterface.h>
#include <rindb/db_element.h>
#include <rindb/db_database.h>
#include "app_db.h"
#include "indicator_model.h"

#include "app_db_factory.h"


/** Factory span calibration minimums */
static int32s app_db_f_adcscal_min[ADC_NUM_SYNC_FREQ] =
{
  APP_DB_F_ADCSCAL_MIN, APP_DB_F_ADCSCAL_MIN, APP_DB_F_ADCSCAL_MIN, APP_DB_F_ADCSCAL_MIN,
  APP_DB_F_ADCSCAL_MIN, APP_DB_F_ADCSCAL_MIN, APP_DB_F_ADCSCAL_MIN, APP_DB_F_ADCSCAL_MIN,
  APP_DB_F_ADCSCAL_MIN
};


/** Factory span calibration maximums */
static int32s app_db_f_adcscal_max[ADC_NUM_SYNC_FREQ] =
{
  APP_DB_F_ADCSCAL_MAX, APP_DB_F_ADCSCAL_MAX, APP_DB_F_ADCSCAL_MAX, APP_DB_F_ADCSCAL_MAX,
  APP_DB_F_ADCSCAL_MAX, APP_DB_F_ADCSCAL_MAX, APP_DB_F_ADCSCAL_MAX, APP_DB_F_ADCSCAL_MAX,
  APP_DB_F_ADCSCAL_MAX
};


/** Factory span calibration defaults */
static int32s app_db_f_adcscal_defaults[ADC_NUM_SYNC_FREQ] =
{
  APP_DB_F_ADCSCAL_DEFAULT0, APP_DB_F_ADCSCAL_DEFAULT1, APP_DB_F_ADCSCAL_DEFAULT2, APP_DB_F_ADCSCAL_DEFAULT3,
  APP_DB_F_ADCSCAL_DEFAULT4, APP_DB_F_ADCSCAL_DEFAULT5, APP_DB_F_ADCSCAL_DEFAULT6, APP_DB_F_ADCSCAL_DEFAULT7,
  APP_DB_F_ADCSCAL_DEFAULT8
};


/** Application factory database */
struct s_db_app_factory db_app_factory;


struct s_db_element app_factory_elements[] =
{
  DB_DECLARE_INT32_ARRAY_ELEM_SINGLE(        APP_DB_F_ADCZOFFS_STR,       NULL,        db_app_factory.adc_factory_database.zeroOffset,     ADC_NUM_SYNC_FREQ,        APP_DB_F_ADCZOFFS_MIN,      APP_DB_F_ADCZOFFS_MAX,       APP_DB_F_ADCZOFFS_DEFAULT,     DB_NO_MINIMUM_VERSION,         DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_FACTORY),
  DB_DECLARE_INT32_ARRAY_ELEM_SINGLE(        APP_DB_F_ADCZDRIFT_STR,      NULL,        NULL,                                               ADC_NUM_SYNC_FREQ,        APP_DB_F_ADCZDRIFT_MIN,     APP_DB_F_ADCZDRIFT_MAX,      APP_DB_F_ADCZDRIFT_DEFAULT,    DB_NO_MINIMUM_VERSION,         APP_DB_F_VERSION1,     APP_DB_CLMASK_FACTORY),   /* Old zero drift values, replaced in APP_DB_F_VERSION2_ZERO_DRIFT by adc_factory_database.zeroDriftPos and adc_factory_database.zeroDriftNeg */
  DB_DECLARE_INT32_ARRAY_ELEM_SINGLE(        APP_DB_F_ADCZDRIFTPOS_STR,   NULL,        db_app_factory.adc_factory_database.zeroDriftPos,   ADC_NUM_SYNC_FREQ,        APP_DB_F_ADCZDRIFT_MIN,     APP_DB_F_ADCZDRIFT_MAX,      APP_DB_F_ADCZDRIFT_DEFAULT,    APP_DB_F_VERSION2_ZERO_DRIFT,  DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_FACTORY),
  DB_DECLARE_INT32_ARRAY_ELEM_SINGLE(        APP_DB_F_ADCZDRIFTNEG_STR,   NULL,        db_app_factory.adc_factory_database.zeroDriftNeg,   ADC_NUM_SYNC_FREQ,        APP_DB_F_ADCZDRIFT_MIN,     APP_DB_F_ADCZDRIFT_MAX,      APP_DB_F_ADCZDRIFT_DEFAULT,    APP_DB_F_VERSION2_ZERO_DRIFT,  DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_FACTORY),
  DB_DECLARE_INT32_ARRAY_ELEM_INDIVIDUAL(    APP_DB_F_ADCSCAL_STR,        NULL,        db_app_factory.adc_factory_database.spanCal,        ADC_NUM_SYNC_FREQ,        app_db_f_adcscal_min,       app_db_f_adcscal_max,        app_db_f_adcscal_defaults,     DB_NO_MINIMUM_VERSION,         DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_FACTORY),
  DB_DECLARE_INT32_ARRAY_ELEM_SINGLE(        APP_DB_F_ADCSDRIFTPOS_STR,   NULL,        db_app_factory.adc_factory_database.spanDriftPos,   ADC_NUM_SYNC_FREQ,        APP_DB_F_ADCSDRIFT_MIN,     APP_DB_F_ADCSDRIFT_MAX,      APP_DB_F_ADCSDRIFT_DEFAULT,    DB_NO_MINIMUM_VERSION,         DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_FACTORY),
  DB_DECLARE_INT32_ARRAY_ELEM_SINGLE(        APP_DB_F_ADCSDRIFTNEG_STR,   NULL,        db_app_factory.adc_factory_database.spanDriftNeg,   ADC_NUM_SYNC_FREQ,        APP_DB_F_ADCSDRIFT_MIN,     APP_DB_F_ADCSDRIFT_MAX,      APP_DB_F_ADCSDRIFT_DEFAULT,    DB_NO_MINIMUM_VERSION,         DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_FACTORY),
  DB_DECLARE_UINT32_ELEM(                    APP_DB_F_ADCCALTEMP_STR,     NULL,        &db_app_factory.adc_factory_database.calTemp,                                 APP_DB_F_ADCCALTEMP_MIN,    APP_DB_F_ADCCALTEMP_MAX,     APP_DB_F_ADCCALTEMP_DEFAULT,   DB_NO_MINIMUM_VERSION,         DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_FACTORY),
  DB_DECLARE_UINT8_ELEM(                     APP_DB_F_ADCCALFLAG_STR,     NULL,        &db_app_factory.adc_factory_database.calFlag,                                 APP_DB_F_ADCCALFLAG_MIN,    APP_DB_F_ADCCALFLAG_MAX,     APP_DB_F_ADCCALFLAG_DEFAULT,   DB_NO_MINIMUM_VERSION,         DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_FACTORY),
  DB_DECLARE_UINT32_ELEM(                    APP_DB_F_ADCCHKSUM_STR,      NULL,        &db_app_factory.adc_factory_database.chksum,                                  APP_DB_F_ADCCHKSUM_MIN,     APP_DB_F_ADCCHKSUM_MAX,      APP_DB_F_ADCCHKSUM_DEFAULT,    DB_NO_MINIMUM_VERSION,         DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_FACTORY),

  DB_DECLARE_UINT32_ELEM(                    APP_DB_F_SERIALNO_STR,       NULL,        &db_app_factory.serial_no,                                                    APP_DB_F_SERIALNO_MIN,      APP_DB_F_SERIALNO_MAX,       APP_DB_F_SERIALNO_DEFAULT,     DB_NO_MINIMUM_VERSION,         DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_FACTORY),
  DB_DECLARE_UINT8_ELEM(                     APP_DB_F_ECONOMY_STR,        NULL,        &db_app_factory.economy,                                                      APP_DB_F_ECONOMY_MIN,       APP_DB_F_ECONOMY_MAX,        APP_DB_F_ECONOMY_DEFAULT,      DB_NO_MINIMUM_VERSION,         DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_FACTORY),
  DB_DECLARE_UINT32_ELEM(                    APP_DB_F_APPLICID_STR,       NULL,        &db_app_factory.application_id,                                               APP_DB_F_APPLICID_MIN,      APP_DB_F_APPLICID_MAX,       APP_DB_F_APPLICID_DEFAULT,     DB_NO_MINIMUM_VERSION,         DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_FACTORY),
  DB_DECLARE_UINT8_ELEM(                     APP_DB_F_TRADEENABLE_STR,    NULL,        &db_app_factory.trade_enable,                                                 APP_DB_F_TRADEENABLE_MIN,   APP_DB_F_TRADEENABLE_MAX,    APP_DB_F_TRADEENABLE_DEFAULT,  DB_NO_MINIMUM_VERSION,         DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_FACTORY),
  DB_DECLARE_STRING_ELEM(                    APP_DB_F_MODEL_STR,          NULL,        &db_app_factory.model_str[0],                       APP_DB_F_MODEL_MAXLEN,                                                             APP_DB_F_MODEL_DEFAULT,        DB_NO_MINIMUM_VERSION,         DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_FACTORY),
  DB_DECLARE_UINT8_ELEM(                     APP_DB_F_TESTRESULT_STR,     NULL,        &db_app_factory.test_result,                                                  APP_DB_F_TESTRESULT_MIN,    APP_DB_F_TESTRESULT_MAX,     APP_DB_F_TESTRESULT_DEFAULT,   APP_DB_F_VERSION3_TEST_RESULT, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_FACTORY),
  DB_DECLARE_UINT32_ELEM(                    APP_DB_F_MODELSUBTYPE_STR,   NULL,        &db_app_factory.model_subtype,                                                APP_DB_F_MODELSUBTYPE_MIN,  APP_DB_F_MODELSUBTYPE_MAX,   APP_DB_F_MODELSUBTYPE_DEFAULT, APP_DB_F_VERSION4_HW_VARIANT,  DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_FACTORY),
};


/** The database structure used by the database library */
struct s_db_database app_factory_database =
{
  .db_database_version = APP_DB_F_VERSION4_HW_VARIANT,
  .db_database_chksum_seed = APP_DB_F_CHECKSUM_SEED,
  .db_elements = app_factory_elements,
  .db_num_elements = DB_NUM_ELEMENTS(app_factory_elements),
  .db_name = APP_DB_F_NAME,
  .db_checksum_buffer = NULL
};
