/*
 * db_factory.h
 *
 *  Created on: Apr 10, 2012
 *      Author: ian
 */

#ifndef APP_DB_FACTORY_H_
#define APP_DB_FACTORY_H_

#include <stddef.h>
#include <stdint.h>
#include <limits.h>
#include <rinweigh/adcdatatypes.h>


#define APP_DB_MODEL_STR_LEN 7


/** Default for when application ID hasn't been set */
#define APPLICATION_ID_DEFAULT 99


/** Application factory database definition */
struct s_db_app_factory
{
  AdcFactoryDatabase adc_factory_database;  /*!< Factory database for ADC library */
  uint32_t serial_no;                       /*!< Indicator serial number */
  uint8_t economy;                          /*!< Economy bits, originally used to disable features, no longer used */
  uint32_t application_id;                  /*!< Application ID, not really used */
  uint8_t trade_enable;                     /*!< Is the indicator trade approved, not used */
  char model_str[APP_DB_MODEL_STR_LEN];     /*!< The model string WE2111, C520, C530 */
  uint8_t test_result;                      /*!< Result of tests, not used */
  uint32_t model_subtype;                   /*!< Indicator model subtype. Permits 2 types of C520, see @ref enum e_indicator_model_sub_type */
};


/** Application factory database */
extern struct s_db_app_factory db_app_factory;


/** The database structure used by the database library */
extern struct s_db_database app_factory_database;


#define APP_DB_F_NAME "factorydb"
#define APP_DB_F_CHECKSUM_SEED 0x12345678


/** Initial factory database version */
#define APP_DB_F_VERSION1 1


/** Version that removes adc_factory_database.zeroDrift and adds adc_factory_database.zeroDriftPos and adc_factory_database.zeroDriftNeg */
#define APP_DB_F_VERSION2_ZERO_DRIFT 2


/** Version that adds a flag to indicate unit has been tested */
#define APP_DB_F_VERSION3_TEST_RESULT 3


/** Version that adds a hardware variant to discern between old C520 with rear rinlink, and new C520 with 2 rear DB9s */
#define APP_DB_F_VERSION4_HW_VARIANT 4


/*
 * ADC database settings
 */


#define APP_DB_F_ADCZOFFS_STR "adczerooffset"
#define APP_DB_F_ADCZOFFS_MIN ADC_DB_FACT_ZEROOFFSET_MINIMUM
#define APP_DB_F_ADCZOFFS_MAX ADC_DB_FACT_ZEROOFFSET_MAXIMUM
#define APP_DB_F_ADCZOFFS_DEFAULT ADC_DB_FACT_ZEROOFFSET_DEFAULT

#define APP_DB_F_ADCZDRIFT_STR "adczerodrift"
#define APP_DB_F_ADCZDRIFTPOS_STR "adczerodriftpos"
#define APP_DB_F_ADCZDRIFTNEG_STR "adczerodriftneg"
#define APP_DB_F_ADCZDRIFT_MIN ADC_DB_FACT_ZERODRIFT_MINIMUM
#define APP_DB_F_ADCZDRIFT_MAX ADC_DB_FACT_ZERODRIFT_MAXIMUM
#define APP_DB_F_ADCZDRIFT_DEFAULT ADC_DB_FACT_ZERODRIFT_DEFAULT

#define APP_DB_F_ADCSCAL_STR "adcspancal"
#define APP_DB_F_ADCSCAL_MIN ADC_DB_FACT_SPANCAL_MINIMUM
#define APP_DB_F_ADCSCAL_MAX ADC_DB_FACT_SPANCAL_MAXIMUM
#define APP_DB_F_ADCSCAL_DEFAULT0 ADC_DB_FACT_SPANCAL_DEFAULT0
#define APP_DB_F_ADCSCAL_DEFAULT1 ADC_DB_FACT_SPANCAL_DEFAULT1
#define APP_DB_F_ADCSCAL_DEFAULT2 ADC_DB_FACT_SPANCAL_DEFAULT2
#define APP_DB_F_ADCSCAL_DEFAULT3 ADC_DB_FACT_SPANCAL_DEFAULT3
#define APP_DB_F_ADCSCAL_DEFAULT4 ADC_DB_FACT_SPANCAL_DEFAULT4
#define APP_DB_F_ADCSCAL_DEFAULT5 ADC_DB_FACT_SPANCAL_DEFAULT5
#define APP_DB_F_ADCSCAL_DEFAULT6 ADC_DB_FACT_SPANCAL_DEFAULT6
#define APP_DB_F_ADCSCAL_DEFAULT7 ADC_DB_FACT_SPANCAL_DEFAULT7
#define APP_DB_F_ADCSCAL_DEFAULT8 ADC_DB_FACT_SPANCAL_DEFAULT8

#define APP_DB_F_ADCSDRIFTPOS_STR "adcspandriftpos"
#define APP_DB_F_ADCSDRIFTNEG_STR "adcspandriftneg"
#define APP_DB_F_ADCSDRIFT_MIN ADC_DB_FACT_SPANDRIFT_MINIMUM
#define APP_DB_F_ADCSDRIFT_MAX ADC_DB_FACT_SPANDRIFT_MAXIMUM
#define APP_DB_F_ADCSDRIFT_DEFAULT ADC_DB_FACT_SPANDRIFT_DEFAULT

#define APP_DB_F_ADCCALTEMP_STR "adccaltemp"
#define APP_DB_F_ADCCALTEMP_MIN ADC_DB_FACT_CALTEMP_MINIMUM
#define APP_DB_F_ADCCALTEMP_MAX ADC_DB_FACT_CALTEMP_MAXIMUM
#define APP_DB_F_ADCCALTEMP_DEFAULT ADC_DB_FACT_CALTEMP_DEFAULT

#define APP_DB_F_ADCCALFLAG_STR "adcadccalflag"
#define APP_DB_F_ADCCALFLAG_MIN ADC_DB_FACT_CALFLAG_MINIMUM
#define APP_DB_F_ADCCALFLAG_MAX ADC_DB_FACT_CALFLAG_MAXIMUM
#define APP_DB_F_ADCCALFLAG_DEFAULT ADC_DB_FACT_CALFLAG_DEFAULT

#define APP_DB_F_ADCCHKSUM_STR "adcchksum"
#define APP_DB_F_ADCCHKSUM_MIN ADC_DB_FACT_CHKSUM_MINIMUM
#define APP_DB_F_ADCCHKSUM_MAX ADC_DB_FACT_CHKSUM_MAXIMUM
#define APP_DB_F_ADCCHKSUM_DEFAULT ADC_DB_FACT_CHKSUM_DEFAULT

/*
 * Application database settings
 */


#define APP_DB_F_SERIALNO_STR "serialno"
#define APP_DB_F_SERIALNO_MIN 0
#define APP_DB_F_SERIALNO_MAX UINT_MAX
#define APP_DB_F_SERIALNO_DEFAULT 0

#define APP_DB_F_ECONOMY_STR "economy"
#define APP_DB_F_ECONOMY_MIN 0
#define APP_DB_F_ECONOMY_MAX UCHAR_MAX
#define APP_DB_F_ECONOMY_DEFAULT 0

#define APP_DB_F_APPLICID_STR "applicid"
#define APP_DB_F_APPLICID_MIN 0
#define APP_DB_F_APPLICID_MAX INT_MAX
#define APP_DB_F_APPLICID_DEFAULT APPLICATION_ID_DEFAULT

#define APP_DB_F_TRADEENABLE_STR "tradeenable"
#define APP_DB_F_TRADEENABLE_MIN 0
#define APP_DB_F_TRADEENABLE_MAX 1
#define APP_DB_F_TRADEENABLE_DEFAULT 0

#define APP_DB_F_MODEL_STR "model"
#define APP_DB_F_MODEL_MAXLEN APP_DB_MODEL_STR_LEN
#define APP_DB_F_MODEL_DEFAULT "WE2111"

#define APP_DB_F_TESTRESULT_STR "testresult"
#define APP_DB_F_TESTRESULT_MIN 0x00
#define APP_DB_F_TESTRESULT_MAX 0xFF
#define APP_DB_F_TESTRESULT_DEFAULT 0xFF

#define APP_DB_F_MODELSUBTYPE_STR "modelsubtype"
#define APP_DB_F_MODELSUBTYPE_MIN 0
#define APP_DB_F_MODELSUBTYPE_MAX UINT_MAX
#define APP_DB_F_MODELSUBTYPE_DEFAULT ((uint32_t) imst_unset)  /*!< In an application, this is unset to discern when an old factory database is upgraded */


/**
 * Write the factory database to file
 *
 * @param write_ass_copy specifies whether the adcSuperStructure copy of the adcFactoryDatabase should be written
 * @param lock specifies whether adcSuperStructure should be locked before writing
 */
bool app_write_factory_database(bool write_ass_copy, bool lock);


/**
 * Read the factory database from file
 *
 * @param recreate will cause any corrupt/missing copies of the database to be re-created, only if an intact database has been read
 */
bool app_read_factory_database(bool recreate);


/** Default the factory database */
bool app_default_factory_database(void);


/** Initialise the factory database */
bool app_init_factory_database(void);


/** Destroy the factory database */
bool app_destroy_factory_database(void);


/** Default Indicator Model Number */
void app_factory_default_model_no(void);


#endif /* APP_DB_FACTORY_H_ */
