/*
 * app_db_licence.h
 *
 *  Created on: 10/07/2017
 *      Author: ian
 */

#ifndef APP_DB_LICENCE_H_
#define APP_DB_LICENCE_H_

#include <stdbool.h>
#include <stdint.h>
#include <limits.h>
#include <rindb/db_database.h>


#define APP_DB_L_LICENCECODES_NUM 50


/** Application licence database definition */
struct s_db_app_licence
{
  /* Licensing */
  uint32_t licence_codes[APP_DB_L_LICENCECODES_NUM];    /*!< List of licence codes */
  uint32_t num_licence_codes;                           /*!< The number of entries in the licence code list */
};


#define APP_DB_L_NAME "licencedb"
#define APP_DB_L_CHECKSUM_SEED 0x1E7F7C49


/** Initial licence database version */
#define APP_DB_L_VERSION1 1


#define APP_DB_L_LICENCECODES_STR "licencecodes"
#define APP_DB_L_LICENCECODES_MIN 0
#define APP_DB_L_LICENCECODES_MAX (UINT32_MAX)
#define APP_DB_L_LICENCECODES_DEFAULT 0

#define APP_DB_L_LICENCECODENUM_STR "numlicencecodes"
#define APP_DB_L_LICENCECODENUM_MIN 0
#define APP_DB_L_LICENCECODENUM_MAX APP_DB_L_LICENCECODES_NUM
#define APP_DB_L_LICENCECODENUM_DEFAULT 0


/** Licence application database structure (container of database variables) */
extern struct s_db_app_licence db_app_licence;


/** Licence application database description (used by db library) */
extern struct s_db_database app_licence_database;


/** Write the licence database to file */
bool app_write_licence_database(void);


/** Read the licence database from file */
bool app_read_licence_database(void);


/**
 * Default the licence database.
 *
 * @param old_userdb_passcodes if not NULL, these codes will be copied into the licence database
 * @param old_userdb_num_passcodes if not 0, this number of codes in old_userdb_passcodes will be copied into the licence database
 *
 * @return true on success
 */
bool app_default_licence_database(const uint32_t *old_userdb_passcodes, uint32_t old_userdb_num_passcodes);


/** Initialise the licence database */
bool app_init_licence_database(void);


/** Destroy the licence database */
bool app_destroy_licence_database(void);


#endif /* APP_DB_LICENCE_H_ */
