/*
 * app_db_runtime.c
 *
 *  Created on: Apr 18, 2012
 *      Author: ian
 */
#include <string.h>
#include <stdio.h>
#include <pthread.h>
#include <rindb/db_element.h>
#include <rindb/db_database.h>
#include <rindb/db_database_binbuf.h>
#include <rindb/db_buffer.h>
#include <rinweigh/adcinterface.h>
#include "adcthread.h"
#include "nv_ram.h"
#include "app_db.h"
#include "app_db_changelog.h"
#include "we_debug.h"

#include "app_db_runtime.h"


#define APP_RUNTIME_DB_BUFFER_SIZE (sizeof(struct s_db_app_runtime) * 2)


/** Default txt data */
static const t_print_text txt_print_text_default =
{
  /*
   *          1         2         3         4
   * 1234567890123456789012345678901234567890
   */
  { "         TXT LINE 1                    " },
  { "         TXT LINE 2                    " },
  { "         TXT LINE 3                    " },
  { "         TXT LINE 4                    " },
  { "         TXT LINE 5                    " },
  { "         TXT LINE 6                    " },
  { "         TXT LINE 7                    " },
  { "         TXT LINE 8                    " },
  { "         TXT LINE 9                    " }
};


static const char *const txt_print_text_default_ptrs[APP_DB_R_PRINTTEXT_NUMELEMS] =
{
  txt_print_text_default[0],
  txt_print_text_default[1],
  txt_print_text_default[2],
  txt_print_text_default[3],
  txt_print_text_default[4],
  txt_print_text_default[5],
  txt_print_text_default[6],
  txt_print_text_default[7],
  txt_print_text_default[8]
};


static char *const txt_print_text_ptrs[APP_DB_R_PRINTTEXT_NUMELEMS] =
{
  db_app_runtime.print_text[0],
  db_app_runtime.print_text[1],
  db_app_runtime.print_text[2],
  db_app_runtime.print_text[3],
  db_app_runtime.print_text[4],
  db_app_runtime.print_text[5],
  db_app_runtime.print_text[6],
  db_app_runtime.print_text[7],
  db_app_runtime.print_text[8]
};


static size_t txt_print_text_maxlengths[APP_DB_R_PRINTTEXT_NUMELEMS] =
{
  APP_DB_R_PRINTTEXT_MAXLEN,
  APP_DB_R_PRINTTEXT_MAXLEN,
  APP_DB_R_PRINTTEXT_MAXLEN,
  APP_DB_R_PRINTTEXT_MAXLEN,
  APP_DB_R_PRINTTEXT_MAXLEN,
  APP_DB_R_PRINTTEXT_MAXLEN,
  APP_DB_R_PRINTTEXT_MAXLEN,
  APP_DB_R_PRINTTEXT_MAXLEN,
  APP_DB_R_PRINTTEXT_MAXLEN
};


/** User runtime database */
struct s_db_app_runtime db_app_runtime;


struct s_db_element app_runtime_elements[] =
{
  /* Element type                            Element database name          Element menu name  Element data                                      Number of elements   Element minimum              Element maximum               Element default                 First appeared in             Last appeared in       Change log mask      Change log lookup */

  /* ADC Runtime Database */
  DB_DECLARE_INT32_ELEM(                     APP_DB_R_ADCCNTZERO_STR,       NULL,          &db_app_runtime.adc_runtime_database.cntZero,                              APP_DB_R_ADCCNTZERO_MIN,     APP_DB_R_ADCCNTZERO_MAX,      APP_DB_R_ADCCNTZERO_DEFAULT,    DB_NO_MINIMUM_VERSION,        DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_RUNTIME),
  DB_DECLARE_INT32_ELEM(                     APP_DB_R_ADCCNTTARE_STR,       NULL,          &db_app_runtime.adc_runtime_database.cntTare,                              APP_DB_R_ADCCNTTARE_MIN,     APP_DB_R_ADCCNTTARE_MAX,      APP_DB_R_ADCCNTTARE_DEFAULT,    DB_NO_MINIMUM_VERSION,        DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_RUNTIME),
  DB_DECLARE_INT32_ELEM(                     APP_DB_R_ADCCNTPTARE_STR,      NULL,          &db_app_runtime.adc_runtime_database.cntPTare,                             APP_DB_R_ADCCNTPTARE_MIN,    APP_DB_R_ADCCNTPTARE_MAX,     APP_DB_R_ADCCNTPTARE_DEFAULT,   DB_NO_MINIMUM_VERSION,        DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_RUNTIME),
  DB_DECLARE_UINT8_ELEM(                     APP_DB_R_ADCHIRES_STR,         NULL,          &db_app_runtime.adc_runtime_database.hiRes,                                APP_DB_R_ADCHIRES_MIN,       APP_DB_R_ADCHIRES_MAX,        APP_DB_R_ADCHIRES_DEFAULT,      DB_NO_MINIMUM_VERSION,        DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_RUNTIME),
  DB_DECLARE_UINT8_ELEM(                     APP_DB_R_ADCNETMODE_STR,       NULL,          &db_app_runtime.adc_runtime_database.netMode,                              APP_DB_R_ADCNETMODE_MIN,     APP_DB_R_ADCNETMODE_MAX,      APP_DB_R_ADCNETMODE_DEFAULT,    DB_NO_MINIMUM_VERSION,        DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_RUNTIME),
  DB_DECLARE_UINT32_ELEM(                    APP_DB_R_ADCPRINTID_STR,       NULL,          &db_app_runtime.adc_runtime_database.printId,                              APP_DB_R_ADCPRINTID_MIN,     APP_DB_R_ADCPRINTID_MAX,      APP_DB_R_ADCPRINTID_DEFAULT,    DB_NO_MINIMUM_VERSION,        DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_RUNTIME),
  DB_DECLARE_UINT8_ELEM(                     APP_DB_R_ADCDISPMODE_STR,      NULL,          &db_app_runtime.adc_runtime_database.dispMode,                             APP_DB_R_ADCDISPMODE_MIN,    APP_DB_R_ADCDISPMODE_MAX,     APP_DB_R_ADCDISPMODE_DEFAULT,   DB_NO_MINIMUM_VERSION,        DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_RUNTIME),
  DB_DECLARE_UINT8_ELEM(                     APP_DB_R_ADCTEMPCALON_STR,     NULL,          &db_app_runtime.adc_runtime_database.tempCalOn,                            APP_DB_R_ADCTEMPCALON_MIN,   APP_DB_R_ADCTEMPCALON_MAX,    APP_DB_R_ADCTEMPCALON_DEFAULT,  DB_NO_MINIMUM_VERSION,        DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_RUNTIME),
  DB_DECLARE_UINT32_ELEM(                    APP_DB_R_ADCOVERLOADCNT_STR,   NULL,          &db_app_runtime.adc_runtime_database.overloadCnt,                          APP_DB_R_ADCOVERLOADCNT_MIN, APP_DB_R_ADCOVERLOADCNT_MAX,  APP_DB_R_ADCOVERLOADCNT_DEFAULT,DB_NO_MINIMUM_VERSION,        DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_RUNTIME),
  DB_DECLARE_UINT32_ELEM(                    APP_DB_R_ADCCHKSUM_STR,        NULL,          &db_app_runtime.adc_runtime_database.chksum,                               APP_DB_R_ADCCHKSUM_MIN,      APP_DB_R_ADCCHKSUM_MAX,       APP_DB_R_ADCCHKSUM_DEFAULT,     APP_DB_R_VERSION2_CHKSUM,     DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_RUNTIME),

  /* Setpoints */
  DB_DECLARE_INT32_ARRAY_ELEM_SINGLE(        APP_DB_R_TARGETS_STR,          NULL,          db_app_runtime.targets,                               NUM_SETPOINTS,       APP_DB_R_TARGETS_MIN,        APP_DB_R_TARGETS_MAX,         APP_DB_R_TARGETS_DEFAULT,       DB_NO_MINIMUM_VERSION,        DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_RUNTIME),
  DB_DECLARE_INT32_ARRAY_ELEM_SINGLE(        APP_DB_R_PREFLIGHT_STR,        NULL,          db_app_runtime.preflight,                             NUM_SETPOINTS,       APP_DB_R_PREFLIGHT_MIN,      APP_DB_R_PREFLIGHT_MAX,       APP_DB_R_PREFLIGHT_DEFAULT,     DB_NO_MINIMUM_VERSION,        DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_RUNTIME),
  DB_DECLARE_INT32_ARRAY_ELEM_SINGLE(        APP_DB_R_HYSTERESIS_STR,       NULL,          db_app_runtime.hys,                                   NUM_SETPOINTS,       APP_DB_R_HYSTERESIS_MIN,     APP_DB_R_HYSTERESIS_MAX,      APP_DB_R_HYSTERESIS_DEFAULT,    DB_NO_MINIMUM_VERSION,        DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_RUNTIME),

  /* Recipe statistics */
  DB_DECLARE_INT32_ELEM(                     APP_DB_R_ACTTARGETS_STR,       NULL,          &db_app_runtime.stat_total,                                                APP_DB_R_ACTTARGETS_MIN,     APP_DB_R_ACTTARGETS_MAX,      APP_DB_R_ACTTARGETS_DEFAULT,    DB_NO_MINIMUM_VERSION,        DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_RUNTIME),
  DB_DECLARE_UINT32_ELEM(                    APP_DB_R_NUMBATCHES_STR,       NULL,          &db_app_runtime.stat_num_items,                                            APP_DB_R_NUMBATCHES_MIN,     APP_DB_R_NUMBATCHES_MAX,      APP_DB_R_NUMBATCHES_DEFAULT,    DB_NO_MINIMUM_VERSION,        DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_RUNTIME),
  DB_DECLARE_INT32_ELEM(                     APP_DB_R_GROSS_TOTAL_STR,      NULL,          &db_app_runtime.stat_gross_total,                                          APP_DB_R_ACTTARGETS_MIN,     APP_DB_R_ACTTARGETS_MAX,      APP_DB_R_ACTTARGETS_DEFAULT,    APP_DB_R_VERSION4_GN_TOTAL,   DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_RUNTIME),
  DB_DECLARE_INT32_ELEM(                     APP_DB_R_NET_TOTAL_STR,        NULL,          &db_app_runtime.stat_net_total,                                            APP_DB_R_ACTTARGETS_MIN,     APP_DB_R_ACTTARGETS_MAX,      APP_DB_R_ACTTARGETS_DEFAULT,    APP_DB_R_VERSION4_GN_TOTAL,   DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_RUNTIME),

  /* Printer text */
  DB_DECLARE_STRING_ARRAY_ELEM_INDIVIDUAL(   APP_DB_R_PRINTTEXT_STR,        NULL,          txt_print_text_ptrs,                                  APP_DB_R_PRINTTEXT_NUMELEMS,                      txt_print_text_maxlengths,    txt_print_text_default_ptrs,    APP_DB_R_VERSION3_PRINT_TEXT, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_RUNTIME),
};


static struct s_db_database app_runtime_database =
{
  .db_database_version = APP_DB_R_VERSION4_GN_TOTAL,
  .db_database_chksum_seed = APP_DB_R_CHECKSUM_SEED,
  .db_elements = app_runtime_elements,
  .db_num_elements = DB_NUM_ELEMENTS(app_runtime_elements),
  .db_name = APP_DB_R_NAME,
  .db_checksum_buffer = NULL
};


/* Used for database writing */
static struct s_db_database_interp app_runtime_ddi_binbuffer;
static struct s_db_buffer app_runtime_db_buffer;


/** NVRAM exclusive access semaphore */
static pthread_mutex_t app_runtime_mutex;


/** Get exclusive access to the runtime database */
void app_runtime_lock(void)
{
  pthread_mutex_lock(&app_runtime_mutex);
}


/** Release exclusive access to the runtime database */
void app_runtime_unlock(void)
{
  pthread_mutex_unlock(&app_runtime_mutex);
}


/**
 * Write the runtime database to NVRAM
 *
 * @param write_ass_copy specifies whether the adcSuperStructure copy of the adcRuntimeDatabase should be written
 * @param lock_runtime specifies whether db_app_runtime should be locked before writing
 * @param lock_ass specifies whether adcSuperStructure should be locked before writing
 */
bool app_write_runtime_database(bool write_ass_copy, bool lock_runtime, bool lock_ass)
{
  enum eDBError res;
  bool nvres1 = false;
  bool nvres2 = false;

  if (lock_runtime)
    app_runtime_lock();

  if (write_ass_copy)
  {
    /* Write the adcSuperStructure copy */
    if (lock_ass)
      AdcMutexLock(&adcSuperStructure);
    db_app_runtime.adc_runtime_database = adcSuperStructure.adcRuntimeDatabase;
    if (lock_ass)
      AdcMutexUnlock(&adcSuperStructure);
  }

  /* Write database to binary buffer */
  res = db_database_write(&app_runtime_database, &app_runtime_ddi_binbuffer, app_runtime_database.db_database_version);
  if (dbeSuccess != res)
  {
    rdebug_printf(LOG_ERR, "app_write_runtime_database: db_database_write() returned %d\n", res);
  }
  else
  {
    /* Write 2 copies of binary buffer to NVRAM */
    nvres1 = nvram_hw_write(app_runtime_db_buffer.buffer, NVRAM_RUNTIME_DB0_OFFSET, app_runtime_db_buffer.posn);
    if (!nvres1)
    {
      rdebug_printf(LOG_ERR, "app_write_runtime_database: nvram_hw_write() copy 1 failed\n");
    }
    nvres2 = nvram_hw_write(app_runtime_db_buffer.buffer, NVRAM_RUNTIME_DB1_OFFSET, app_runtime_db_buffer.posn);
    if (!nvres2)
    {
      rdebug_printf(LOG_ERR, "app_write_runtime_database: nvram_hw_write() copy 2 failed\n");
    }
  }

  /* Update the change log */
  (void) app_changelog_update(&app_runtime_database);

  if (lock_runtime)
    app_runtime_unlock();

  return (dbeSuccess == res && nvres1 && nvres2);
}


/** Read the runtime database from NVRAM */
bool app_read_runtime_database(void)
{
  enum eDBError res = dbeFailure;
  uint32_t version;
  bool nvres;

  app_runtime_lock();

  /* Try reading 1st copy of runtime database from NVRAM */
  nvres = nvram_hw_read(app_runtime_db_buffer.buffer, NVRAM_RUNTIME_DB0_OFFSET, app_runtime_db_buffer.length);
  if (nvres)
  {
    /* Read database settings from binary buffer */
    res = db_database_read(&app_runtime_database, &app_runtime_ddi_binbuffer, &version, false);
    if (dbeSuccess != res)
    {
      rdebug_printf(LOG_ERR, "app_read_runtime_database: db_database_read() copy 1 returned %d\n", res);
    }
  }
  else
  {
    rdebug_printf(LOG_ERR, "app_write_runtime_database: nvram_hw_read() copy 1 failed\n");
  }

  if (!nvres || dbeSuccess != res)
  {
    /* First copy failed, try reading 2st copy of runtime database from NVRAM */
    nvres = nvram_hw_read(app_runtime_db_buffer.buffer, NVRAM_RUNTIME_DB1_OFFSET, app_runtime_db_buffer.length);
    if (nvres)
    {
      /* Read database settings from binary buffer */
      res = db_database_read(&app_runtime_database, &app_runtime_ddi_binbuffer, &version, false);
      if (dbeSuccess != res)
      {
        rdebug_printf(LOG_ERR, "app_read_runtime_database: db_database_read() copy 2 returned %d\n", res);
      }
    }
    else
    {
      rdebug_printf(LOG_ERR, "app_write_runtime_database: nvram_hw_read() copy 2 failed\n");
    }
  }

  /* Update previous data, so change log will work */
  db_database_update_previous(&app_runtime_database);

  app_runtime_unlock();

  return (nvres && dbeSuccess == res);
}


/**
 * Default the runtime database
 *
 * @param default_adc_runtime_also if true will also default the ADC runtime variables (tare values, etc)
 */
bool app_default_runtime_database(bool default_adc_runtime_also)
{
  enum eDBError res;

  AdcMutexLock(&adcSuperStructure);
  app_runtime_lock();

  /* Default the whole database */
  res = db_database_default(&app_runtime_database);
  if (dbeSuccess != res)
  {
    rdebug_printf(LOG_ERR, "app_default_runtime_database: db_database_default() returned %d\n", res);
  }

  /* Either default the ADC runtime variables, or leave them unchanged */
  if (default_adc_runtime_also)
  {
    /* Use the ADC database defaulter for ADC values */
    AdcDefaultDatabases(&adcSuperStructure, false, NULL, NULL, &adcSuperStructure.adcRuntimeDatabase, false, false);
    db_app_runtime.adc_runtime_database = adcSuperStructure.adcRuntimeDatabase;
  }
  else
  {
    /* Don't overwrite the ADC runtime variables */
    db_app_runtime.adc_runtime_database = adcSuperStructure.adcRuntimeDatabase;
  }

  app_runtime_unlock();
  AdcMutexUnlock(&adcSuperStructure);

  return (dbeSuccess == res);
}


/** Initialise the runtime database */
bool app_init_runtime_database(void)
{
  enum eDBError res;
  int mutex_res;

  mutex_res = pthread_mutex_init(&app_runtime_mutex, NULL);

  app_runtime_lock();

  /* Initialise the database (creates previous data for some elements) */
  res = db_database_init(&app_runtime_database);
  if (dbeSuccess == res)
  {
    /* Create a buffer by malloc'ing a buffer of specified length */
    res = db_buffer_create_malloc(&app_runtime_db_buffer, APP_RUNTIME_DB_BUFFER_SIZE, APP_RUNTIME_DB_BUFFER_SIZE);
    if (dbeSuccess == res)
    {
      res = db_create_database_interp_binbuffer(&app_runtime_ddi_binbuffer, &app_runtime_db_buffer);
      if (dbeSuccess != res)
      {
        rdebug_printf(LOG_ERR, "app_init_runtime_database: db_create_database_interp_bin() returned %d\n", res);
      }
    }
    else
    {
      rdebug_printf(LOG_ERR, "app_init_runtime_database: db_buffer_create_malloc() returned %d\n", res);
    }
  }
  else
  {
    rdebug_printf(LOG_ERR, "app_init_runtime_database: db_database_init() returned %d\n", res);
  }

  app_runtime_unlock();

  return (0 == mutex_res && dbeSuccess == res);
}


/** Destroy the runtime database */
bool app_destroy_runtime_database(void)
{
  pthread_mutex_destroy(&app_runtime_mutex);
  (void) db_database_destroy(&app_runtime_database);
  (void) db_destroy_database_interp_binbuffer(&app_runtime_ddi_binbuffer);
  (void) db_buffer_destroy(&app_runtime_db_buffer);
  return true;
}
