/*
 * app_db_runtime.h
 *
 *  Created on: Apr 18, 2012
 *      Author: ian
 */

#ifndef APP_DB_RUNTIME_H_
#define APP_DB_RUNTIME_H_

#include <stddef.h>
#include <stdint.h>
#include <limits.h>
#include <rinweigh/adcdatatypes.h>
#include "db_setpoint.h"


#define APP_DB_PRINT_TXT_LEN 41
#define APP_DB_PRINT_TXT_NUM 9


/** Printer text storage */
typedef char t_print_text[APP_DB_PRINT_TXT_NUM][APP_DB_PRINT_TXT_LEN];


/** Application runtime database definition */
struct s_db_app_runtime
{
  /* ADC runtime database */
  AdcRuntimeDatabase adc_runtime_database;

  /* Setpoints */
  int32_t targets[NUM_SETPOINTS];    /*!< Target for each setpoint */
  int32_t preflight[NUM_SETPOINTS];  /*!< Preflight for each setpoint */
  int32_t hys[NUM_SETPOINTS];        /*!< Hysteresis for each setpoint */

  /* Statistics */
  int32_t stat_total;                /*!< Accumulated target */
  uint32_t stat_num_items;           /*!< Number of batches run */
  int32_t stat_gross_total;          /*!< Accumulated gross weight */
  int32_t stat_net_total;            /*!< Accumulated net weight */

  /* Printer text */
  t_print_text print_text;           /*!< Printer text used by TXT cmd, REMOTE_TXTPRN IO thumbwheel */
};


/** User runtime database */
extern struct s_db_app_runtime db_app_runtime;


#define APP_DB_R_NAME "runtimedb"
#define APP_DB_R_CHECKSUM_SEED 0x12345678


/** Initial runtime database version */
#define APP_DB_R_VERSION1 1


/** Runtime database version which added checksum AdcRuntimeDatabase.chksum */
#define APP_DB_R_VERSION2_CHKSUM 2


/** Runtime database version which added print text used by TXT cmd, REMOTE_TXTPRN IO thumbwheel) */
#define APP_DB_R_VERSION3_PRINT_TEXT 3


/** Runtime database version which added gross and net totals */
#define APP_DB_R_VERSION4_GN_TOTAL 4


/*
 * ADC database settings
 */


#define APP_DB_R_ADCCNTZERO_STR "adccntzero"
#define APP_DB_R_ADCCNTZERO_MIN (ADC_DB_RT_CNTZERO_MINIMUM)
#define APP_DB_R_ADCCNTZERO_MAX (ADC_DB_RT_CNTZERO_MAXIMUM)
#define APP_DB_R_ADCCNTZERO_DEFAULT (ADC_DB_RT_CNTZERO_DEFAULT)

#define APP_DB_R_ADCCNTTARE_STR "adccnttare"
#define APP_DB_R_ADCCNTTARE_MIN (ADC_DB_RT_CNTTARE_MINIMUM)
#define APP_DB_R_ADCCNTTARE_MAX (ADC_DB_RT_CNTTARE_MAXIMUM)
#define APP_DB_R_ADCCNTTARE_DEFAULT (ADC_DB_RT_CNTTARE_DEFAULT)

#define APP_DB_R_ADCCNTPTARE_STR "adccntptare"
#define APP_DB_R_ADCCNTPTARE_MIN (ADC_DB_RT_CNTPTARE_MINIMUM)
#define APP_DB_R_ADCCNTPTARE_MAX (ADC_DB_RT_CNTPTARE_MAXIMUM)
#define APP_DB_R_ADCCNTPTARE_DEFAULT (ADC_DB_RT_CNTPTARE_DEFAULT)

#define APP_DB_R_ADCHIRES_STR "adchires"
#define APP_DB_R_ADCHIRES_MIN (ADC_DB_RT_HIRES_MINIMUM)
#define APP_DB_R_ADCHIRES_MAX (ADC_DB_RT_HIRES_MAXIMUM)
#define APP_DB_R_ADCHIRES_DEFAULT (ADC_DB_RT_HIRES_DEFAULT)

#define APP_DB_R_ADCNETMODE_STR "adcnetmode"
#define APP_DB_R_ADCNETMODE_MIN (ADC_DB_RT_NETMODE_MINIMUM)
#define APP_DB_R_ADCNETMODE_MAX (ADC_DB_RT_NETMODE_MAXIMUM)
#define APP_DB_R_ADCNETMODE_DEFAULT (ADC_DB_RT_NETMODE_DEFAULT)

#define APP_DB_R_ADCPRINTID_STR "adcprintid"
#define APP_DB_R_ADCPRINTID_MIN (ADC_DB_RT_PRINTID_MINIMUM)
#define APP_DB_R_ADCPRINTID_MAX (ADC_DB_RT_PRINTID_MAXIMUM)
#define APP_DB_R_ADCPRINTID_DEFAULT (ADC_DB_RT_PRINTID_DEFAULT)

#define APP_DB_R_ADCDISPMODE_STR "adcdispmode"
#define APP_DB_R_ADCDISPMODE_MIN (ADC_DB_RT_DISPMODE_MINIMUM)
#define APP_DB_R_ADCDISPMODE_MAX (ADC_DB_RT_DISPMODE_MAXIMUM)
#define APP_DB_R_ADCDISPMODE_DEFAULT (ADC_DB_RT_DISPMODE_DEFAULT)

#define APP_DB_R_ADCTEMPCALON_STR "adctempcalon"
#define APP_DB_R_ADCTEMPCALON_MIN (ADC_DB_RT_TEMPCALON_MINIMUM)
#define APP_DB_R_ADCTEMPCALON_MAX (ADC_DB_RT_TEMPCALON_MAXIMUM)
#define APP_DB_R_ADCTEMPCALON_DEFAULT (ADC_DB_RT_TEMPCALON_DEFAULT)

#define APP_DB_R_ADCOVERLOADCNT_STR "adcoverloadcnt"
#define APP_DB_R_ADCOVERLOADCNT_MIN (ADC_DB_RT_OVERLOADCNT_MINIMUM)
#define APP_DB_R_ADCOVERLOADCNT_MAX (ADC_DB_RT_OVERLOADCNT_MAXIMUM)
#define APP_DB_R_ADCOVERLOADCNT_DEFAULT (ADC_DB_RT_OVERLOADCNT_DEFAULT)

#define APP_DB_R_ADCCHKSUM_STR "adcchksum"
#define APP_DB_R_ADCCHKSUM_MIN ADC_DB_RT_CHKSUM_MINIMUM
#define APP_DB_R_ADCCHKSUM_MAX ADC_DB_RT_CHKSUM_MAXIMUM
#define APP_DB_R_ADCCHKSUM_DEFAULT ADC_DB_RT_CHKSUM_DEFAULT


/*
 * Application database settings
 */


#define APP_DB_R_TARGETS_STR "apptargets"
#define APP_DB_R_TARGETS_MIN (-99999)
#define APP_DB_R_TARGETS_MAX (999999)
#define APP_DB_R_TARGETS_DEFAULT (0)

#define APP_DB_R_PREFLIGHT_STR "apppreflight"
#define APP_DB_R_PREFLIGHT_MIN (0)
#define APP_DB_R_PREFLIGHT_MAX (999999)
#define APP_DB_R_PREFLIGHT_DEFAULT (0)

#define APP_DB_R_HYSTERESIS_STR "apphysteresis"
#define APP_DB_R_HYSTERESIS_MIN (0)
#define APP_DB_R_HYSTERESIS_MAX (999999)
#define APP_DB_R_HYSTERESIS_DEFAULT (0)

#define APP_DB_R_ACTTARGETS_STR "appactualtargets"
#define APP_DB_R_ACTTARGETS_MIN (INT32_MIN)
#define APP_DB_R_ACTTARGETS_MAX (INT32_MAX)
#define APP_DB_R_ACTTARGETS_DEFAULT (0)

#define APP_DB_R_GROSS_TOTAL_STR "totalgrossweight"
#define APP_DB_R_NET_TOTAL_STR "totalnetweight"

#define APP_DB_R_NUMBATCHES_STR "appnumbatches"
#define APP_DB_R_NUMBATCHES_MIN (0)
#define APP_DB_R_NUMBATCHES_MAX (UINT32_MAX)
#define APP_DB_R_NUMBATCHES_DEFAULT (0)

#define APP_DB_R_PRINTTEXT_STR "printtext"
#define APP_DB_R_PRINTTEXT_NUMELEMS (APP_DB_PRINT_TXT_NUM)
#define APP_DB_R_PRINTTEXT_MAXLEN (APP_DB_PRINT_TXT_LEN)


/** Get exclusive access to the runtime database */
void app_runtime_lock(void);


/** Release exclusive access to the runtime database */
void app_runtime_unlock(void);


/**
 * Write the runtime database to NVRAM
 *
 * @param write_ass_copy specifies whether the adcSuperStructure copy of the adcRuntimeDatabase should be written
 * @param lock_runtime specifies whether db_app_runtime should be locked before writing
 * @param lock_ass specifies whether adcSuperStructure should be locked before writing
 */
bool app_write_runtime_database(bool write_ass_copy, bool lock_runtime, bool lock_ass);


/** Read the runtime database from NVRAM */
bool app_read_runtime_database(void);


/**
 * Default the runtime database
 *
 * @param default_adc_runtime_also if true will also default the ADC runtime variables (tare values, etc)
 */
bool app_default_runtime_database(bool default_adc_runtime_also);


/** Initialise the runtime database */
bool app_init_runtime_database(void);


/** Destroy the runtime database */
bool app_destroy_runtime_database(void);


#endif /* APP_DB_RUNTIME_H_ */
