/*
 * app_db_runtime_vars.h
 *
 *  Created on: Apr 19, 2012
 *      Author: ian
 */

#ifndef APP_DB_RUNTIME_VARS_H_
#define APP_DB_RUNTIME_VARS_H_

#include <stddef.h>
#include <stdbool.h>
#include <stdint.h>


/*
 * Setpoint data functions
 */


/** Set the hysteresis for a recipe target */
bool setpoint_hysteresis_set(uint8_t num, int32_t data);


/** Get the hysteresis for a recipe target */
int32_t setpoint_hysteresis_get(uint8_t num);


/** Set the preflight for a recipe target */
bool setpoint_preflight_set(uint8_t num, int32_t data);


/** Get the preflight for a recipe target */
int32_t setpoint_preflight_get(uint8_t num);


/** Set the target for a recipe target */
bool setpoint_target_set(uint8_t num, int32_t data);


/** Get the target for a recipe target */
int32_t setpoint_target_get(uint8_t num);


/*
 * Statistics
 */


/** Return total in final units for current recipe total */
int32_t statistics_get_total(void);


/** Return total in final units for net weight total */
int32_t statistics_get_net_total(void);


/** Return total in final units for gross weight total */
int32_t statistics_get_gross_total(void);


/** Returns number of items in the current recipe total */
uint32_t statistics_get_num_items(void);


/** Clear total sum for current recipe total */
void statistics_clear(void);


/**
 * Memory minus/undo action for totalising for current recipe total.
 *
 * @return true if something was undone
 */
bool statistics_undo_mem_plus(void);


/**
 * Memory plus action for totalising for current recipe total
 *
 * @param rdg is in final units
 * @param gross is boolean value for if the weight is gross
 */
void statistics_mem_plus(long rdg, bool gross);


/*
 * Text data functions
 */


/** Set the text in the NVRAM */
void text_set_text(size_t num, const char *str);


/** Get the text in the NVRAM */
void text_get_text(char *str, size_t maxlen, size_t num);


#endif /* APP_DB_RUNTIME_VARS_H_ */
