/*
 * app_save.c
 *
 *  Created on: 13 Jul. 2018
 *      Author: ian
 */
#include <stdbool.h>
#include <stdio.h>
#include <rinweigh/adcdatatypes.h>
#include <rinweigh/adcinterface.h>
#include <rinweigh/adcliveimpl_simple.h>
#include "app_db_changelog.h"
#include "app_db_user.h"
#include "we_config.h"
#include "adcthread.h"
#include "comms_task.h"
#include "serial_ports.h"
#include "serial_usb_host.h"
#include "access.h"
#include "device_finder_we2111.h"
#include "kbd_5000.h"
#include "unitswitch.h"

#include "app_save.h"


/** Re-initialise comms, buzzer, IO, analog, setpoints and device finder after a save has been performed */
void reinitialise_after_save(void)
{
  /* Re-configure all serial ports */
  comms_reconfigure(true, SERPORT_UNKNOWN);

  /* Re-configure modbus tcp */
  rinmodbus_configure_from_db();

  /* Setup buzzer on keypresses */
  setup_kbd_buzz();

  /* Setup keys on remote inputs */
  setup_remote_kbd();

  /* Re-configure setpoints and IO hardware */
  init_setpoints(&setpoint_running_info, &adcSuperStructure);
  access_dbconfigure_allio();
  access_dbconfigure_analog();

  /* Let device finder know about changes */
  update_device_finder_we2111();

  /* update unit switching */
  init_unit_switch();
}


/**
 * Save databases changes and network settings and re-initialise the application with the changes
 *
 * @param check_can_save true if the function should check that the changelog is not full before saving
 * @param net_config if not NULL then the network configuration will be set/saved with this configuration
 *
 * @return true if databases were saved
 */
bool app_save_databases_and_reinit(bool check_can_save, struct s_snc_net_config *net_config)
{
  bool result;

  if (!check_can_save || app_can_save_databases())
  {
    /* Save the requested databases */
    AdcMutexLock(&adcSuperStructure);
    AdcSaveDatabases(&adcSuperStructure, false, NULL, &adcSuperStructure.adcUserDatabase, &adcSuperStructure.adcRuntimeDatabase, &db_app_user.adc_user_database);

    /* Re-initialise from adcSuperStructure database copies */
    AdcReInit(&adcSuperStructure, false, NULL, NULL, NULL);
    AdcLiveWeighImplementerSimpleConfigure(&adcSuperStructure, adcSuperStructure.adcLiveWeigh, WECFG_LIVE_WEIGH_AVERAGING_MILLISECONDS);
    AdcMutexUnlock(&adcSuperStructure);

    if (NULL != net_config)
      net_ip_save_config(net_config, NULL);

    /* Re-initialise everything after save */
    reinitialise_after_save();
    result = true;
  }
  else
  {
    result = false;
  }
  return result;
}
