/*
 * apperror.c
 *
 *  Created on: Apr 20, 2016
 *      Author: ian
 */
#include <string.h>
#include <stdio.h>
#include "we_debug.h"
#include "status_led.h"

#include "apperror.h"


/** Current system errors */
static we_error_t sys_error = APP_ERR_NONE;


/* Latched system errors (ie errors since startup) */
static we_error_t latched_error = APP_ERR_NONE;


/** Get a 6 character error string Exxxxx */
void get_error_str(char *str, int maxlen)
{
  snprintf(str, maxlen, "E%05X", sys_error);
  str[maxlen - 1] = '\0';
}


/** Set the error led based on err */
static void set_error_led(we_error_t err)
{
  static we_error_t last_sys_error = APP_ERR_NONE;

  if (err != last_sys_error)
  {
    status_led_set_frequency((APP_ERR_NONE != err) ? STATUS_LED_ERROR_FREQ : STATUS_LED_NORMAL_FREQ);
    last_sys_error = err;
  }
}


/** Called if error err detected, also logs into syslog */
void set_error(we_error_t err, const char *format, ...)
{
  va_list arglist;

  /* Keep track of errors */
  sys_error |= err;
  latched_error |= err;

  /* Put an entry is syslog */
  if (NULL != format)
  {
    va_start(arglist, format);
    rdebug_vprintf(LOG_ERR, format, arglist);
    va_end(arglist);
  }

  /* Set the error LED */
  set_error_led(sys_error);
}


/** Called to clear error err, from the current errors, not the latched errors, also logs into syslog */
void clr_error(we_error_t err, const char *format, ...)
{
  va_list arglist;

  /* Keep track of errors */
  sys_error &= ~err;

  /* Put an entry is syslog */
  if (NULL != format)
  {
    va_start(arglist, format);
    rdebug_vprintf(LOG_ERR, format, arglist);
    va_end(arglist);
  }

  /* Set the error LED */
  set_error_led(sys_error);
}


/** Get the current system errors */
we_error_t get_current_errors(void)
{
  return sys_error;
}


/** Get the latched system errors (ie all errors since startup) */
we_error_t get_latched_errors(void)
{
  return latched_error;
}
