#ifndef APPERROR_DOT_H
#define APPERROR_DOT_H 1

#include <stdint.h>


/*
 * WE2111 Errors
 *
 * Note there can on by 20bits of error,
 * as we only have 5 digits to display errors
 * (the 6th being taken up by the E)
 *
 */


/** No error */
#define APP_ERR_NONE                  0x00000

/** Low input voltage error mask */
#define APP_ERR_LOW_12V               0x00001

/** High input voltage error mask */
#define APP_ERR_HIGH_12V              0x00002

/** Supply voltage error, low or high supply */
#define APP_ERR_POWER_SUPPLY          (APP_ERR_LOW_12V | APP_ERR_HIGH_12V)

/** Positive sense wire out */
#define APP_ERR_SENSE_POS             0x00004

/** Negative sense wire out */
#define APP_ERR_SENSE_NEG             0x00008

/** One or both of the sense wires are out */
#define APP_ERR_SENSE_BOTH            (APP_ERR_SENSE_POS | APP_ERR_SENSE_NEG)

/* #define ADC_ERR_TEMPERATURE 0x00000010 */

/** Module error (one or more of the modules has an error) */
#define APP_ERR_MODULE                0x00020

/** Tilt sensor error (not used on C500, but we match R400) */
#define APP_ERR_TILT                  0x00040

/** Resolution setup is out of range */
#define APP_ERR_RESOLUTION            0x00080

/** RTC not working */
#define APP_ERR_RTC                   0x00100

/* #define ADC_ERR_USER_DB     0x00000200 */
/* #define ADC_ERR_FACTORY_DB  0x00000400 */

/** Low excitation voltage */
#define APP_ERR_LOW_EXCITATION        0x00800

/** High excitation voltage */
#define APP_ERR_HIGH_EXCITATION       0x01000

/** Excitation error, high or low excitation voltage */
#define APP_ERR_EXCITATION            (APP_ERR_LOW_EXCITATION | APP_ERR_HIGH_EXCITATION)

/* #define ADC_ERR_ADC_MAXED   0x00002000 */
/* #define ADC_ERR_RUNTIME_DB  0x00004000 */

/** One of the loadcell related errors */
#define APP_ERR_LOADCELL              (APP_ERR_EXCITATION | APP_ERR_POWER_SUPPLY | ADC_ERR_TEMPERATURE | APP_ERR_SENSE_BOTH)

/** Bad ROM checksum */
#define APP_ERR_ROM                   0x08000

/* #define ADC_ERR_HARDWARE    0x00010000 */

/** One or more of the boards is reporting that it has not been factory tested */
#define APP_ERR_UNTESTED              0x20000


/** Type for storing WE2111 error mask */
typedef uint32_t we_error_t;


/** Get a 6 character error string Exxxxx */
void get_error_str(char *str, int maxlen);


/** Called if error err detected, also logs into syslog */
void set_error(we_error_t err, const char *format, ...);


/** Called to clear error err, from the current errors, not the latched errors, also logs into syslog */
void clr_error(we_error_t err, const char *format, ...);


/** Get the current system errors */
we_error_t get_current_errors(void);


/** Get the latched system errors (ie all errors since startup) */
we_error_t get_latched_errors(void);


#endif /* #ifndef APPERROR_DOT_H */
