#include <stdint.h>
#include <rinhw/rio.h>
#include <rinhw/buzzer.h>
#include "database.h"
#include "devices.h"
#include "app_db_user.h"

#include "buzz5000.h"


static void setup_tbuzzer(tBuzzer *buzzer, uint16_t milli, uint16_t freq)
{
  long temp = OS_TICKS_PER_SEC;
  temp *= milli;
  temp /= 1000;

  buzzer->time = (uint16_t) temp;
  buzzer->freq = freq;
}


/**************************************************************************/
void buzz(uint16_t milli, uint8_t freq)
{
  tBuzzer buzzer;

  /* Only buzz if enabled */
  if (db_app_user.buzzer_enable)
  {

    setup_tbuzzer(&buzzer, milli, BUZZER_RESONANT_FREQ);

    ropen(&dev_buz_we, 0);
    rwrite(&dev_buz_we, &buzzer, sizeof(buzzer), 0);
    rclose(&dev_buz_we);
  }
}


#define MAX_BUZZES 5

/**************************************************************************/
void buzz_times(uint16_t on_milli, uint16_t off_milli, uint8_t freq, uint8_t times)
{
  uint8_t i;
  tBuzzer buzzes[MAX_BUZZES * 2];

  /* Only buzz if enabled */
  if (db_app_user.buzzer_enable)
  {

    if (times > MAX_BUZZES)
    {
      times = MAX_BUZZES;
    }

    for (i = 0; i < times * 2; i += 2)
    {
      setup_tbuzzer(&buzzes[i], on_milli, BUZZER_RESONANT_FREQ);
      setup_tbuzzer(&buzzes[i + 1], off_milli, 0);
    }

    ropen(&dev_buz_we, 0);
    rwrite(&dev_buz_we, &buzzes, sizeof(tBuzzer) * times * 2, 0);
    rclose(&dev_buz_we);
  }
}


/***************************************************************************/
void beep_thrice(void)
{
  buzz_times(BEEP_TIME, BEEP_WAIT, BUZZ_NORMAL, 3);
}


/***************************************************************************/
void beep_twice(void)
{
  buzz_times(BEEP_TIME, BEEP_WAIT, BUZZ_NORMAL, 2);
}


/***************************************************************************/
void beep(void)
{
  buzz(BEEP_TIME, BUZZ_NORMAL);
}


/***************************************************************************/
void beep_long(void)
{
  buzz(500, BUZZ_NORMAL);
}


/***************************************************************************/
void beep_error(void)
{
  buzz_times(250, 125, BUZZ_NORMAL, 3);
}


/***************************************************************************/
bool is_beeping(void)
{
  bool res;
  tBuzzer BeepCur;

  res = false;

  if (db_app_user.buzzer_enable)
  {
    /* Read buzzer state */
    if (ropen(&dev_buz_we, 1) == 1)
    {
      if (rread(&dev_buz_we,&BeepCur, sizeof(BeepCur),1) != -1)
        res = (BeepCur.freq > 0);
      rclose(&dev_buz_we);
    }
  }

  return res;
}
