/*
 * cmdline.c
 *
 *  Created on: 28/05/2010
 *      Author: ian
 */
#include <stdio.h>
#include <getopt.h>
#include <rinhw/stdrinstrum.h>
#include "we_config.h"

#include "cmdline.h"


/** WE2111 configuration file */
const char *cmdline_config_file = WECFG_CONFIG_FILE;


/** Flag to indicate the program should run as a daemon */
bool cmdline_daemonise = false;


/** Display command line help */
bool cmdline_showhelp = false;


/** Flag to indicate the program should log error messages to syslog */
bool cmdline_logtosyslog = false;


/** Flag to indicate the program should log error messages to stdout */
bool cmdline_logtostdout = false;


/** Serial port device to use for remote devices, if NULL remote devices are not used */
const char *cmdline_remote_device = NULL;


/** Print out the command line options */
void print_commandlineopts(const char *program_name)
{
  fprintf(stdout, "Usage: %s [-options]\n"
                  "  -c <file>    --config <file>    Use configuration from <file>,\n"
                  "                                  default is '" WECFG_CONFIG_FILE "'\n"
                  "  -d           --daemon           Run as a daemon\n"
                  "  -h           --help             Print help\n"
                  "  -r <serial>  --remote <serial>  Use remote devices over <serial> port\n"
                  "                                  (/dev/ttySx) or TCP (host:port)\n"
                  "  -l           --syslog           Log error messages to syslog\n"
                  "  -s           --stdout           Log error messages to stdout\n",
          program_name);
}


/** Parse the command line options */
int parse_commandline(int argc, char *argv[])
{
  int c;
  int result = 0;
  static const char opts[] = "c:dhr:ls";
  static const struct option lopts[] =
  {
    { "config",        1, 0, 'c'  },
    { "daemon",        0, 0, 'd'  },
    { "help",          0, 0, 'h'  },
    { "remote",        1, 0, 'r'  },
    { "syslog",        0, 0, 'l'  },
    { "stdout",        0, 0, 's'  },
    { NULL,            0, 0, '\0' }
  };

  while (1)
  {
    c = getopt_long(argc, argv, opts, lopts, NULL);
    if (-1 == c)
    {
      break;
    }

    switch (c)
    {
      case 'c':
        cmdline_config_file = optarg;
        break;
      case 'd':
        cmdline_daemonise = true;
        break;
      case 'h':
        cmdline_showhelp = true;
        break;
      case 'r':
        cmdline_remote_device = optarg;
        break;
      case 'l':
        cmdline_logtosyslog = true;
        break;
      case 's':
        cmdline_logtostdout = true;
        break;
      default:
        result = -1;
        break;
    }
  }
  return result;
}
