/*
 * comms_auto.h
 *
 *  Created on: 03/12/2010
 *      Author: ian
 */

#ifndef COMMS_AUTO_H_
#define COMMS_AUTO_H_

#include <semaphore.h>
#include <stdint.h>
#include <rinhw/rio.h>
#include <rinweigh/adcdatatypes.h>
#include "iface_serial_config.h"
#include "serial_ports.h"
#include "sysclock.h"
#include "apperror.h"
#include "comms_send_buffer.h"


/** Automatic output communications structure */
struct s_comms_auto
{
  /* These are filled out when structure is created */
  struct t_io_dev *serdev;                /*!< Serial device to send the auto output to */
  bool is_incoming_udp;                   /*!< Port is an incoming UDP connection */
  struct s_comms_send_buffer *csb;        /*!< Send buffer to assemble the auto output into, so we can do a single write to serdev */
  enum eAutoOutFmt format;
  enum eAutoOutputSrc source;
  sem_t trigger_sem;                      /*!< Semaphore to trigger an auto output */
  uint8_t start_char;
  uint8_t end_char1;
  uint8_t end_char2;
  const char *custom_format_str;

  /* The rest are filled out when an auto output is sent */

  /* Formatting */
  bool token_lz;
  char token_dp;
  char token_plus_sign;                   /*!< Character used for plus sign, 0 means space */
  uint8_t token_digits;
  uint8_t token_error_mode;
  bool token_upper;

  /* Weight information */
  t_datetime datetime;
  we_error_t sys_errors;
  enum eAdcUnitSwitch displayed_units;    /*!< What units are currently being displayed (primary/secondary/tertiary) */
  int resolution_length;                  /*!< In NTEP this affects the width of zero padding in printed weights */
  AdcRdg rdg_displayed;
  AdcRdg rdg_gross;
  AdcRdg rdg_net;
  AdcRdg rdg_tare;
  AdcRdg rdg_total;
};


/** Send an automatic output */
void comms_auto_send(struct s_comms_auto *comms_auto);


/** Called when weight reading requested via Kp command -> This sends an automatic output to the port it was requested on, rather than the auto output port. */
void comms_auto_send_request(struct t_io_dev *serdev);


#endif /* COMMS_AUTO_H_ */
