/*
 * comms_bdxwfp.c
 *
 *  Created on: Jun 2, 2016
 *      Author: ian
 */
#include "serial_ports.h"
#include "indicator_model_features.h"

#include "comms_bdxwfp.h"


/** Convert enum eSerialPort to enum eBDXWFPSerialPort */
enum eBDXWFPSerialPort xlat_serial_port_to_bdxwfp(enum eSerialPort serial_port)
{
  enum eBDXWFPSerialPort bdxwfp_port;

  switch (serial_port)
  {
    case SERPORT_RL_FRONT:
      bdxwfp_port = BDXWFP_PORT_9_RL_FRONT;
      break;
    case SERPORT_SER1_RL_REAR:
      bdxwfp_port = BDXWFP_PORT_10_SER1_RL_REAR;
      break;
    case SERPORT_RB1A:
      bdxwfp_port = BDXWFP_PORT_0_RB1A;
      break;
    case SERPORT_RB1B:
      bdxwfp_port = BDXWFP_PORT_1_RB1B;
      break;
    case SERPORT_RB2A:
      bdxwfp_port = BDXWFP_PORT_2_RB2A;
      break;
    case SERPORT_RB2B:
      bdxwfp_port = BDXWFP_PORT_3_RB2B;
      break;
    case SERPORT_USBGADGET:
      bdxwfp_port = BDXWFP_PORT_6_USBGADGET;
      break;
    case SERPORT_USBHOST:
      bdxwfp_port = BDXWFP_PORT_7_USBHOST;
      break;
    case SERPORT_SER2:
      bdxwfp_port = BDXWFP_PORT_17_SER2;
      break;
    case SERPORT_USBLP:
      bdxwfp_port = BDXWFP_PORT_8_USBLP;
      break;
    case SERPORT_UDPOUT1:
      bdxwfp_port = BDXWFP_PORT_11_UDPOUT1;
      break;
    case SERPORT_UDPOUT2:
      bdxwfp_port = BDXWFP_PORT_12_UDPOUT2;
      break;
    case SERPORT_UDPIN1:
      bdxwfp_port = BDXWFP_PORT_13_UDPIN1;
      break;
    case SERPORT_UDPIN2:
      bdxwfp_port = BDXWFP_PORT_14_UDPIN2;
      break;
    case SERPORT_TCPNET:
      bdxwfp_port = BDXWFP_PORT_4_TCPNET;
      break;
    case SERPORT_TCPAUTO:
      bdxwfp_port = BDXWFP_PORT_5_TCPAUTO;
      break;
    case SERPORT_UDSNET:
      bdxwfp_port = BDXWFP_PORT_15_UDSNET;
      break;
    case SERPORT_UDSAUTO:
      bdxwfp_port = BDXWFP_PORT_16_UDSAUTO;
      break;

    default:

      /*
       * If it's one of the TCP connections, then return BDXWFP_PORT_4_TCPNET,
       * otherwise return something safe (BDXWFP_PORT_1_ONBOARD)
       */
      bdxwfp_port = (serial_port >= SERPORT_NETWORK_START && serial_port < SERPORT_NUM)
                     ? BDXWFP_PORT_4_TCPNET : BDXWFP_PORTNUM;
      break;
  }
  return bdxwfp_port;
}


/** Convert enum eBDXWFPSerialPort to enum eSerialPort */
enum eSerialPort xlat_bdxwfp_to_serial_port(enum eBDXWFPSerialPort bdxwfp_port)
{
  enum eSerialPort serial_port;

  switch (bdxwfp_port)
  {
    case BDXWFP_PORT_0_RB1A:
      serial_port = SERPORT_RB1A;
      break;
    case BDXWFP_PORT_1_RB1B:
      serial_port = SERPORT_RB1B;
      break;
    case BDXWFP_PORT_2_RB2A:
      serial_port = SERPORT_RB2A;
      break;
    case BDXWFP_PORT_3_RB2B:
      serial_port = SERPORT_RB2B;
      break;
    case BDXWFP_PORT_4_TCPNET:
      serial_port = SERPORT_TCPNET;
      break;
    case BDXWFP_PORT_5_TCPAUTO:
      serial_port = SERPORT_TCPAUTO;
      break;
    case BDXWFP_PORT_6_USBGADGET:
      serial_port = SERPORT_USBGADGET;
      break;
    case BDXWFP_PORT_7_USBHOST:
      serial_port = SERPORT_USBHOST;
      break;
    case BDXWFP_PORT_8_USBLP:
      serial_port = SERPORT_USBLP;
      break;
    case BDXWFP_PORT_9_RL_FRONT:
      serial_port = SERPORT_RL_FRONT;
      break;
    case BDXWFP_PORT_10_SER1_RL_REAR:
      serial_port = SERPORT_SER1_RL_REAR;
      break;
    case BDXWFP_PORT_11_UDPOUT1:
      serial_port = SERPORT_UDPOUT1;
      break;
    case BDXWFP_PORT_12_UDPOUT2:
      serial_port = SERPORT_UDPOUT2;
      break;
    case BDXWFP_PORT_13_UDPIN1:
      serial_port = SERPORT_UDPIN1;
      break;
    case BDXWFP_PORT_14_UDPIN2:
      serial_port = SERPORT_UDPIN2;
      break;
    case BDXWFP_PORT_15_UDSNET:
      serial_port = SERPORT_UDSNET;
      break;
    case BDXWFP_PORT_16_UDSAUTO:
      serial_port = SERPORT_UDSAUTO;
      break;
    case BDXWFP_PORT_17_SER2:
      serial_port = SERPORT_SER2;
      break;
    default:
      serial_port = SERPORT_UNKNOWN;
      break;
  }
  return serial_port;
}


/** Can this port be configured? */
bool bdxwfp_port_can_configure(enum eBDXWFPSerialPort bdxwfp_port)
{
  return ((BDXWFP_PORT_8_USBLP != bdxwfp_port &&
           BDXWFP_PORT_9_RL_FRONT != bdxwfp_port &&
           BDXWFP_PORT_10_SER1_RL_REAR != bdxwfp_port &&
           BDXWFP_PORT_15_UDSNET != bdxwfp_port &&
           BDXWFP_PORT_16_UDSAUTO != bdxwfp_port) ||
          (indicator_model_subtype_has_db9_serial(get_indicator_model_subtype()) &&
           (BDXWFP_PORT_10_SER1_RL_REAR == bdxwfp_port ||
            BDXWFP_PORT_17_SER2 == bdxwfp_port)));
}
