/*
 * comms_dispatch.c
 *
 *  Created on: 13/10/2010
 *      Author: ian
 */
#include "database.h"
#include "comms_hdlr_am.h"
#include "comms_hdlr_defaults.h"
#include "comms_hdlr_fpl.h"
#include "comms_hdlr_network.h"
#include "comms_hdlr_nz.h"
#include "comms_hdlr_passcode.h"
#include "comms_hdlr_rdb.h"
#include "comms_params.h"
#include "comms_errors.h"
#include "iface.h"

#include "comms_dispatch.h"


#define mAAA HBM_CMD_CHARS_TO_NUM('A', 'A', 'A')
#define mAAB 1
#define mACL 63
#define mADR 95
#define mAFT 149
#define mAOC HBM_CMD_CHARS_TO_NUM('A', 'O', 'C')
#define mASF 473
#define mBCA HBM_CMD_CHARS_TO_NUM('B', 'C', 'A')
#define mBDR HBM_CMD_CHARS_TO_NUM('B', 'D', 'R')
#define mBDX HBM_CMD_CHARS_TO_NUM('B', 'D', 'X')
#define mBRT HBM_CMD_CHARS_TO_NUM('B', 'R', 'T')
#define mBUZ HBM_CMD_CHARS_TO_NUM('B', 'U', 'Z')
#define mCDL 1441
#define mCFM HBM_CMD_CHARS_TO_NUM('C', 'F', 'M')
#define mCLK 1648
#define mCOF 1721
#define mCWT 1943
#define mDBG 2060
#define mDFT HBM_CMD_CHARS_TO_NUM('D', 'F', 'T')
#define mDPF HBM_CMD_CHARS_TO_NUM('D', 'P', 'F')
#define mDPS HBM_CMD_CHARS_TO_NUM('D', 'P', 'S')
#define mENU 3062
#define mESM HBM_CMD_CHARS_TO_NUM('E', 'S', 'M')
#define mESR HBM_CMD_CHARS_TO_NUM('E', 'S', 'R')
#define mETH HBM_CMD_CHARS_TO_NUM('E', 'T', 'H')
#define mFBT HBM_CMD_CHARS_TO_NUM('F', 'B', 'T')
#define mFOP HBM_CMD_CHARS_TO_NUM('F', 'O', 'P')
#define mFPL HBM_CMD_CHARS_TO_NUM('F', 'P', 'L')
#define mIAD 5411
#define mICR 5477
#define mIDM HBM_CMD_CHARS_TO_NUM('I', 'D', 'M')
#define mIDN HBM_CMD_CHARS_TO_NUM('I', 'D', 'N')
#define mLBT 7481
#define mLDN HBM_CMD_CHARS_TO_NUM('L', 'D', 'N')
#define mLDW HBM_CMD_CHARS_TO_NUM('L', 'D', 'W')
#define mLIC 7646
#define mLIV 7665
#define mLIW HBM_CMD_CHARS_TO_NUM('L', 'I', 'W')
#define mLRP HBM_CMD_CHARS_TO_NUM('L', 'R', 'P')
#define mLWN HBM_CMD_CHARS_TO_NUM('L', 'W', 'N')
#define mLWT HBM_CMD_CHARS_TO_NUM('L', 'W', 'T')
#define mMBS HBM_CMD_CHARS_TO_NUM('M', 'B', 'S')
#define mMDD 8193   /* MDD command used to dump calibration database as a binary blob, used to backup and restore cal. Use BCA instead */
#define mMTD 8609
#define mMSM HBM_CMD_CHARS_TO_NUM('M', 'S', 'M')
#define mMSV HBM_CMD_CHARS_TO_NUM('M', 'S', 'V')
#define mPCD HBM_CMD_CHARS_TO_NUM('P', 'C', 'D')
#define mPFT 10289
#define mPOR 10521
#define mPRS 10600
#define mPRT 10601
#define mPST 10627
#define mQAF 10821
#define mRBT 11537
#define mRDB HBM_CMD_CHARS_TO_NUM('R', 'D', 'B')
#define mRES 11614
#define mSTP 12677
#define mTAR 12861
#define mTAS 12862
#define mTAV 12865
#define mTDD 12925
#define mTST 13331
#define mTXT 13461
#define mUNT HBM_CMD_CHARS_TO_NUM('U', 'N', 'T')
#define mVAL 14207
#define mWFP HBM_CMD_CHARS_TO_NUM('W', 'F', 'P')
#define mWMD 15187
#define mZST 17387


/** Handle a HBM style command (eg IDN) */
uint8_t comms_handle_hbm_cmd(uint16_t cmd, struct s_comms_control *comms_ctrl, char *s)
{
  get_params(&comms_ctrl->comms_params, s);

  /* Block commands, but not queries for all but FPL when in the menus, which allows viewer to navigate the menus */
  if (edit_functions && !comms_ctrl->cmd_query && mFPL != cmd)
  {
    return MESS_ERROR;
  }

  /* Dispatch command */
  switch (cmd)
  {
#if 0
    case mAAA:
      return handle_AAA(comms_ctrl, s);
#endif
    case mAAB:
      return handle_AAB(comms_ctrl, s);
    case mACL:
      return handle_ACL(comms_ctrl, s);
    case mADR:
      return handle_ADR(comms_ctrl, s);
    case mAFT:
      return handle_AFT(comms_ctrl, s);
    case mAOC:
      return handle_AOC(comms_ctrl, s);
    case mASF:
      return handle_ASF(comms_ctrl, s);
    case mBCA:
      return handle_BCA(comms_ctrl, s);
    case mBDR:
      return handle_BDR(comms_ctrl, s);
    case mBDX:
      return handle_BDX(comms_ctrl, s);
    case mBRT:
      return handle_BRT(comms_ctrl, s);
    case mBUZ:
      return handle_BUZ(comms_ctrl, s);
    case mCDL:
      return handle_CDL(comms_ctrl, s);
    case mCFM:
      return handle_CFM(comms_ctrl, s);
    case mCLK:
      return handle_CLK(comms_ctrl, s);
    case mCOF:
      return handle_COF(comms_ctrl, s);
    case mCWT:
      return handle_CWT(comms_ctrl, s);
#if 0
    case mDBG:
      return handle_DBG(s);
#endif
    case mDFT:
      return handle_DFT(comms_ctrl, s);
    case mDPF:
      return handle_DPF(comms_ctrl, s);
    case mDPS:
      return handle_DPS(comms_ctrl, s);
    case mENU:
      return handle_ENU(comms_ctrl, s);
    case mESM:
      return handle_ESM(comms_ctrl, s);
    case mESR:
      return handle_ESR(comms_ctrl, s);
    case mETH:
      return handle_ETH(comms_ctrl, s);
    case mFBT:
      return handle_FBT(comms_ctrl, s);
    case mFOP:
      return handle_FOP(comms_ctrl, s);
    case mFPL:
      return handle_FPL(comms_ctrl, s);
    case mIAD:
      return handle_IAD(comms_ctrl, s);
    case mICR:
      return handle_ICR(comms_ctrl, s);
    case mIDM:
      return handle_IDM(comms_ctrl, s);
    case mIDN:
      return handle_IDN(comms_ctrl, s);
    case mLBT:
      return handle_LBT(comms_ctrl, s);
    case mLDN:
      return handle_LDN(comms_ctrl, s);
    case mLDW:
      return handle_LDW(comms_ctrl, s);
    case mLIC:
      return handle_LIC(comms_ctrl, s);
    case mLIV:
      return handle_LIV(comms_ctrl, s);
    case mLIW:
      return handle_LIW(comms_ctrl, s);
    case mLRP:
      return handle_LRP(comms_ctrl, s);
    case mLWN:
      return handle_LWN(comms_ctrl, s);
    case mLWT:
      return handle_LWT(comms_ctrl, s);
    case mMBS:
      return handle_MBS(comms_ctrl, s);
    case mMSM:
      return handle_MSM(comms_ctrl, s);
    case mMSV:
      return handle_MSV(comms_ctrl, s);
    case mMTD:
      return handle_MTD(comms_ctrl, s);
    case mPCD:
      return handle_DPF(comms_ctrl, s);
    case mPFT:
      return handle_PFT(comms_ctrl, s);
    case mPOR:
      return handle_POR(comms_ctrl, s);
    case mPRS:
      return handle_PRS(comms_ctrl, s);
    case mPRT:
      return handle_PRT(comms_ctrl, s);
    case mPST:
      return handle_PST(comms_ctrl, s);
    case mQAF:
      return handle_QAF(comms_ctrl, s);
    case mRBT:
      return handle_RBT(comms_ctrl, s);
    case mRDB:
      return handle_RDB(comms_ctrl, s);
    case mRES:
      return handle_RES(comms_ctrl, s);
    case mTAR:
      return handle_TAR(comms_ctrl, s);
    case mTAS:
      return handle_TAS(comms_ctrl, s);
    case mTAV:
      return handle_TAV(comms_ctrl, s);
    case mTDD:
      return handle_TDD(comms_ctrl, s);
    case mTST:
      return handle_TST(comms_ctrl, s);
    case mTXT:
      return handle_TXT(comms_ctrl, s);
    case mUNT:
      return handle_UNT(comms_ctrl, s);
    case mVAL:
      return handle_VAL(comms_ctrl, s);
    case mWFP:
      return handle_WFP(comms_ctrl, s);
    case mWMD:
      return handle_WMD(comms_ctrl, s);
    case mZST:
      return handle_ZST(comms_ctrl, s);
  }
  return MESS_ERROR;
}
