/*
 * comms_escape.h
 *
 *  Created on: Jun 11, 2012
 *      Author: ian
 */

#ifndef COMMS_ESCAPE_H_
#define COMMS_ESCAPE_H_

#include <stdlib.h>


/** An escaped dot (\.) is unescaped as 128 */
#define COMMS_ESCAPED_DOT_UNESCAPED (128)

/** An escaped plus (\+) is unescaped as 155 */
#define COMMS_ESCAPED_PLUS_UNESCAPED (155)

/** An escaped plus (\-) is unescaped as 156 */
#define COMMS_ESCAPED_MINUS_UNESCAPED (156)

/** An escaped upper case character (\A-\Z) is unescaped by adding 64 */
#define COMMS_ESCAPED_CHAR_TO_UNESCAPED(ch) (129 - 'A' + (ch))


/**
 * Escape a character as per the strange HBM comms character escape rules:
 * 0x00-0x1F,",0x97..0xFF  -> \nnn
 * 0x80..0x97 -> As per comms_escape_chars (\A, etc)
 * \ -> \\
 * Other printables -> Not escaped
 *
 * @param s is a string of length 5 characters or greater, that ch is escaped into
 * @param ch is the character to be escaped
 *
 * @return s
 */
char *comms_escape_char(char *s, char ch);


/**
 * Un-escape a character as per the strange HBM comms character escape rules:
 * \nnn -> char value
 * \\ -> \
 * \A, etc -> 129 as per comms_escape_chars
 * \. -> 128
 * \+ -> 155
 * \- -> 156
 * Other printables -> Not escaped
 *
 * @param unescstr is the output un-escaped string (always <= length of escstr, unless max_len gets in the way)
 * @param escstr is the escaped string to be decoded
 * @param max_len specifies the maximum capacity of unescstr including null terminating character
 */
void comm_unescape_str(char *unescstr, const char *escstr, size_t max_len);


#endif /* COMMS_ESCAPE_H_ */
