#include <setjmp.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <stdio.h>
#include <rinhw/rio.h>
#include <rinhw/lcd.h>
#include <rinweigh/adcdatabase.h>
#include <rinweigh/adcinterface.h>
#include <rinweigh/adcliveimpl_simple.h>
#include "serial_levels.h"
#include "asciidef.h"
#include "meas.h"
#include "database.h"
#include "backgrnd.h"
#include "version.h"
#include "iface.h"
#include "system.h"
#include "editor.h"
#include "kbd_5000.h"
#include "setpoint.h"
#include "comms_print.h"
#include "access.h"
#include "sysclock.h"
#include "buzz5000.h"
#include "modbus_old.h"
#include "comms_util.h"
#include "comms_params.h"
#include "comms_setup.h"
#include "comms_errors.h"
#include "comms_task.h"
#include "comms_task_listen.h"
#include "db_option.h"
#include "remote_cal.h"
#include "db_special.h"
#include "db_serial.h"
#include "db_serial_ip.h"
#include "devices.h"
#include "adcthread.h"
#include "app_db_factory.h"
#include "app_db_user.h"
#include "app_db_runtime_vars.h"
#include "comms_kbd.h"
#include "comms_escape.h"
#include "app_db_changelog.h"
#include "app_db_runtime.h"
#include "app_db_licence.h"
#include "iface_menus.h"
#include "comms_bdxwfp.h"
#include "display.h"
#include "licensing.h"
#include "edit_string.h"

#include "comms_hdlr_am.h"


/** Re-initialise setpoints and IO hardware */
void comms_reinit_setpoints(void)
{
  init_setpoints(&setpoint_running_info, &adcSuperStructure);
  access_dbconfigure_allio();
}


/** Re-initialise the adc and setpoints */
void comms_reinit_adc_and_setpoints(void)
{
  AdcReInit(&adcSuperStructure, true, NULL, NULL, NULL);
  comms_reinit_setpoints();
}


/** Test MultiDiv() */
uint8_t handle_AAB(struct s_comms_control *comms_ctrl, char *s)
{
  long tmp;

  if (param_ok(&CTRL_COMP, 0, 99999999L, -99999999L) &&
      param_ok(&CTRL_COMP, 1, 99999999L, -99999999L) &&
      param_ok(&CTRL_COMP, 2, 99999999L, -99999999L))
  {
    tmp = AdcMultDiv(COMPARAM(0), COMPARAM(1), COMPARAM(2));
    comms_net_rfprintf(comms_ctrl, "%ld" COMMS_TERMINATOR, tmp);
    return REPLY_SENT;
  }
  return MESS_ERROR;
}


/** Zero mux settings */
uint8_t handle_ACL(struct s_comms_control *comms_ctrl, char *s)
{
  uint8_t mbs, mbm;

  if (comms_ctrl->cmd_query)
  {
#if 0
    mbs = (mux_blocked & BLOCK_STEADY) ? 0 : 1;
    mbm = (mux_blocked & BLOCK_MOTION) ? 0 : 1;
#endif
    /* Not implemented on WE2111, just say they are turned off for backwards compatibility */
    mbs = 0;
    mbm = 0;
    comms_net_rfprintf(comms_ctrl, "%u,%u" COMMS_TERMINATOR, mbs, mbm);
    return REPLY_SENT;
  }

  if (adcSuperStructure.adcUserDatabase.approvalMode == ADC_APPROVAL_MODE_OIML)
  {
    return MESS_ERROR;
  }

  if (param_ok(&CTRL_COMP, 0, 1, 0))
  {
#if 0 /* Not implemented on WE2111, but accept command for backwards compatibility */
    if (0 == COMPARAM(0))
      mux_blocked |= BLOCK_STEADY;
    else
      mux_blocked &= ~BLOCK_STEADY;
#endif
  }
  if (param_ok(&CTRL_COMP, 1, 1, 0))
  {
#if 0 /* Not implemented on WE2111, but accept command for backwards compatibility */
    if (0 == COMPARAM(1))
      mux_blocked |= BLOCK_MOTION;
    else
      mux_blocked &= ~BLOCK_MOTION;
#endif
  }
  return MESS_OK;
}


static void send_ADR(struct s_comms_control *comms_ctrl)
{
  comms_net_rfprintf(comms_ctrl, "%u" COMMS_TERMINATOR,
                     db_app_user.address
                    );
}


/** Unit address */
uint8_t handle_ADR(struct s_comms_control *comms_ctrl, char *s)
{
  bool ok;
  long l;

  if (comms_ctrl->cmd_query)
  {
    send_ADR(comms_ctrl);
    return REPLY_SENT;
  }

  ok = true;
  if (param_str_ok(&CTRL_COMP, 1, 7, 1))
  {
    l = atol(COMPARAMSTR(1));
    if ((unsigned long) l != db_app_factory.serial_no)
      ok = false;
  }
  if (!ok)
  {
    return REPLY_SENT;
  }

  if (param_ok(&CTRL_COMP, 0, APP_DB_U_SERADDRESS_MAX, APP_DB_U_SERADDRESS_MIN))
  {
    db_app_user.address = (uint8_t) COMPARAM(0);
    return MESS_OK;
  }
  return MESS_ERROR;
}


/** Auto output format */
uint8_t handle_AFT(struct s_comms_control *comms_ctrl, char *s)
{
  char temp_str[10];
  const char *p;

  if (comms_ctrl->cmd_query)
  {
    comms_net_rfprintf(comms_ctrl, "\"");
    p = db_app_user.auto_fmt;
    while (*p)
    {
      comms_net_rfprintf(comms_ctrl, "%s", comms_escape_char(temp_str, *p));
      p++;
    }
    comms_net_rfprintf(comms_ctrl, "\"" COMMS_TERMINATOR);
    return REPLY_SENT;
  }

  if (param_str_ok(&CTRL_COMP, 0, APP_DB_AUTO_FMT_LEN * 5, 0))
  {
    comm_unescape_str(db_app_user.auto_fmt, COMPARAMSTR(0), APP_DB_AUTO_FMT_LEN);
    return MESS_OK;
  }
  return MESS_ERROR;
}


/** Analog output calibration */
uint8_t handle_AOC(struct s_comms_control *comms_ctrl, char *s)
{
  if (comms_ctrl->cmd_query)
  {
    comms_net_rfprintf(comms_ctrl, "%u,%u,%ld,%ld,%u,%u,%u" COMMS_TERMINATOR,
                       (unsigned) db_app_user.analog_output_type, (unsigned) db_app_user.analog_output_source,
                       (long) db_app_user.analog_zero_cal, (long) db_app_user.analog_span_cal,
                       (unsigned) access_get_force_analogue(),
                       (unsigned) db_app_user.analog_output_abs, (unsigned) db_app_user.analog_output_clip);
    return REPLY_SENT;
  }

  /* Param 0 - Analog output type */
  if (param_ok(&CTRL_COMP, 0, APP_DB_U_ANALOUTTYPE_MAX, APP_DB_U_ANALOUTTYPE_MIN))
  {
    db_app_user.analog_output_type = (uint8_t) COMPARAM(0);
  }

  /* Param 1 - Analog output source */
  if (param_ok(&CTRL_COMP, 1, APP_DB_U_ANALOUTSRC_MAX, APP_DB_U_ANALOUTSRC_MIN))
  {
    db_app_user.analog_output_source = (uint8_t) COMPARAM(1);
  }

  /* Param 2 - zero adjustment */
  if (param_ok(&CTRL_COMP, 2, APP_DB_U_ANALOUTZERO_MAX, APP_DB_U_ANALOUTZERO_MIN))
  {
    db_app_user.analog_zero_cal = COMPARAM(2);
  }

  /* Param 3 - span adjustment */
  if (param_ok(&CTRL_COMP, 3, APP_DB_U_ANALOUTSPAN_MAX, APP_DB_U_ANALOUTSPAN_MIN))
  {
    db_app_user.analog_span_cal = COMPARAM(3);
  }

  /* Param 4 - force analog */
  if (param_ok(&CTRL_COMP, 4, ANALOGOUT_FORCE_NUM - 1, 0))
  {
    access_force_analogue((enum eAnalogForce) COMPARAM(4));
  }

  /* Param 5 - absolute */
  if (param_ok(&CTRL_COMP, 5, APP_DB_U_ANALOUTABS_MAX, APP_DB_U_ANALOUTABS_MIN))
  {
    db_app_user.analog_output_abs = COMPARAM(5);
  }

  /* Param 6 - clip */
  if (param_ok(&CTRL_COMP, 6, APP_DB_U_ANALOUTCLIP_MAX, APP_DB_U_ANALOUTCLIP_MIN))
  {
    db_app_user.analog_output_clip = COMPARAM(6);
  }

  access_dbconfigure_analog();
  return MESS_OK;
}


static void send_ASF(struct s_comms_control *comms_ctrl)
{
  uint8_t av_idx;

  av_idx = xlat_adc_library_average_to_we2111(adcSuperStructure.adcUserDatabase.average);
  comms_net_rfprintf(comms_ctrl, "%u,%u" COMMS_TERMINATOR,
                     av_idx,
                     adcSuperStructure.adcUserDatabase.jitter
                    );
}


/** Averaging and jitter */
uint8_t handle_ASF(struct s_comms_control *comms_ctrl, char *s)
{
  if (comms_ctrl->cmd_query)
  {
    send_ASF(comms_ctrl);
    return REPLY_SENT;
  }

  /* Param 0 - averaging */
  if (param_ok(&CTRL_COMP, 0, NUM_AVERAGE_OPT - 1, 0))
  {
    adcSuperStructure.adcUserDatabase.average = xlat_we2111_average_to_adc_library(COMPARAM(0));
  }

  /* Param 1 - anti-jitter */
  if (param_ok(&CTRL_COMP, 1, 2, 0))
  {
    adcSuperStructure.adcUserDatabase.jitter = (uint8_t) COMPARAM(1);
  }

  AdcReInit(&adcSuperStructure, true, NULL, NULL, NULL);
  return MESS_OK;
}


/** The different types of BCA command */
enum eBCAQueryType { bqtBasic = 0, bqtFullscales, bqtResolutions, bqtLinPoints, bqtLinCorrections, bqtNum };


/** Send the backup calibration information */
static uint8_t send_BCA(struct s_comms_control *comms_ctrl, enum eBCAQueryType bqt)
{
  int i;
  uint8_t result;

  result = REPLY_SENT;
  switch (bqt)
  {
    case bqtBasic:
      comms_net_rfprintf(comms_ctrl, "%u,%ld,%ld,%ld,%u,%u,%u,%u,%ld,%ld,%u,%u" COMMS_TERMINATOR,
                         (unsigned) bqt,
                         (long) adcSuperStructure.adcUserDatabase.zeroRaw,
                         (long) adcSuperStructure.adcUserDatabase.spanRaw,
                         (long) adcSuperStructure.adcUserDatabase.spanCnt,
                         (unsigned) adcSuperStructure.adcUserDatabase.scaleType,
                         (unsigned) adcSuperStructure.adcUserDatabase.syncFreqIdx,
                         (unsigned) adcSuperStructure.adcUserDatabase.dpPos,
                         (unsigned) adcSuperStructure.adcUserDatabase.units,
                         (long) adcSuperStructure.adcUserDatabase.calWeight,
                         (long) adcSuperStructure.adcUserDatabase.addTare,
                         (unsigned) adcSuperStructure.adcUserDatabase.wire4Mode,
                         (unsigned) adcSuperStructure.adcUserDatabase.hiRes);
      break;

    case bqtFullscales:
      comms_net_rfprintf(comms_ctrl, "%u,", (unsigned) bqt);
      for (i = 0; i < ADC_MAX_NUM_INTERVAL - 1; i++)
      {
        comms_net_rfprintf(comms_ctrl, "%ld,", (long) adcSuperStructure.adcUserDatabase.fullScale[i]);
      }
      comms_net_rfprintf(comms_ctrl, "%ld" COMMS_TERMINATOR, (long) adcSuperStructure.adcUserDatabase.fullScale[i]);
      break;

    case bqtResolutions:
      comms_net_rfprintf(comms_ctrl, "%u,", (unsigned) bqt);
      for (i = 0; i < ADC_MAX_NUM_INTERVAL - 1; i++)
      {
        comms_net_rfprintf(comms_ctrl, "%u,", (unsigned) adcSuperStructure.adcUserDatabase.resolution[i]);
      }
      comms_net_rfprintf(comms_ctrl, "%u" COMMS_TERMINATOR, (unsigned) adcSuperStructure.adcUserDatabase.resolution[i]);
      break;

    case bqtLinPoints:
      comms_net_rfprintf(comms_ctrl, "%u,", (unsigned) bqt);
      for (i = 0; i < ADC_MAX_NUM_LIN_PTS; i++) /* No -1 because linPts[ADC_MAX_NUM_LIN_PTS+1] */
      {
        comms_net_rfprintf(comms_ctrl, "%u,", (unsigned) adcSuperStructure.adcUserDatabase.linPts[i]);
      }
      comms_net_rfprintf(comms_ctrl, "%u" COMMS_TERMINATOR, (unsigned) adcSuperStructure.adcUserDatabase.linPts[i]);
      break;

    case bqtLinCorrections:
      comms_net_rfprintf(comms_ctrl, "%u,", (unsigned) bqt);
      for (i = 0; i < ADC_MAX_NUM_LIN_PTS; i++) /* No -1 because linCor[ADC_MAX_NUM_LIN_PTS+1] */
      {
        comms_net_rfprintf(comms_ctrl, "%ld,", (long) adcSuperStructure.adcUserDatabase.linCor[i]);
      }
      comms_net_rfprintf(comms_ctrl, "%ld" COMMS_TERMINATOR, (long) adcSuperStructure.adcUserDatabase.linCor[i]);
      break;

    default:
      result = MESS_ERROR;
      break;
  }
  return result;
}


/** Set BCA basic settings */
static uint8_t handle_BCA_basic(struct s_comms_control *comms_ctrl)
{
  /* Param 1 - zeroRaw */
  if (param_ok(&CTRL_COMP, 1, APP_DB_U_ADCZERORAW_MAX, APP_DB_U_ADCZERORAW_MIN))
  {
    adcSuperStructure.adcUserDatabase.zeroRaw = COMPARAM(1);
  }

  /* Param 2 - spanRaw */
  if (param_ok(&CTRL_COMP, 2, APP_DB_U_ADCSPANRAW_MAX, APP_DB_U_ADCSPANRAW_MIN))
  {
    adcSuperStructure.adcUserDatabase.spanRaw = COMPARAM(2);
  }

  /* Param 3 - spanCnt */
  if (param_ok(&CTRL_COMP, 3, APP_DB_U_ADCSPANCNT_MAX, APP_DB_U_ADCSPANCNT_MIN))
  {
    adcSuperStructure.adcUserDatabase.spanCnt = COMPARAM(3);
  }

  /* Param 4 - scaleType */
  if (param_ok(&CTRL_COMP, 4, ADC_SCALE_TYPE_DUAL_RANGE, APP_DB_U_ADCSCALETYPE_MIN))
  {
    adcSuperStructure.adcUserDatabase.scaleType = (int8u) COMPARAM(4);
  }

  /* Param 5 - syncFreqIdx */
  if (param_ok(&CTRL_COMP, 5, APP_DB_U_ADCSYNCFREQ_MAX, APP_DB_U_ADCSYNCFREQ_MIN))
  {
    adcSuperStructure.adcUserDatabase.syncFreqIdx = (int8u) COMPARAM(5);
  }

  /* Param 6 - dpPos */
  if (param_ok(&CTRL_COMP, 6, APP_DB_U_ADCDPPOS_MAX, APP_DB_U_ADCDPPOS_MIN))
  {
    adcSuperStructure.adcUserDatabase.dpPos = (int8u) COMPARAM(6);
  }

  /* Param 7 - units */
  if (param_ok(&CTRL_COMP, 7, APP_DB_U_ADCUNITS_MAX, APP_DB_U_ADCUNITS_MIN))
  {
    adcSuperStructure.adcUserDatabase.units = (int8u) COMPARAM(7);
  }

  /* Param 8 - calWeight */
  if (param_ok(&CTRL_COMP, 8, APP_DB_U_ADCCALWEIGHT_MAX, APP_DB_U_ADCCALWEIGHT_MIN))
  {
    adcSuperStructure.adcUserDatabase.calWeight = COMPARAM(8);
  }

  /* Param 9 - addTare */
  if (param_ok(&CTRL_COMP, 9, APP_DB_U_ADCADDTARE_MAX, APP_DB_U_ADCADDTARE_MIN))
  {
    adcSuperStructure.adcUserDatabase.addTare = COMPARAM(9);
  }

  /* Param 10 - wire4Mode */
  if (param_ok(&CTRL_COMP, 10, APP_DB_U_ADC4WIRE_MAX, APP_DB_U_ADC4WIRE_MIN))
  {
    adcSuperStructure.adcUserDatabase.wire4Mode = (int8u) COMPARAM(10);
  }

  /* Param 11 - hiRes */
  if (param_ok(&CTRL_COMP, 11, APP_DB_U_ADCHIRES_MAX, APP_DB_U_ADCHIRES_MIN))
  {
    adcSuperStructure.adcUserDatabase.hiRes = (int8u) COMPARAM(11);
  }

  return MESS_OK;
}


/** Set BCA fullscales */
static uint8_t handle_BCA_fullscales(struct s_comms_control *comms_ctrl)
{
  int i;
  const long fs_mins[ADC_MAX_NUM_INTERVAL] = { APP_DB_U_ADCFS0_MIN, APP_DB_U_ADCFS1_MIN, APP_DB_U_ADCFS2_MIN };
  const long fs_maxs[ADC_MAX_NUM_INTERVAL] = { APP_DB_U_ADCFS0_MAX, APP_DB_U_ADCFS1_MAX, APP_DB_U_ADCFS2_MAX };

  for (i = 0; i < ADC_MAX_NUM_INTERVAL; i++)
  {
    if (param_ok(&CTRL_COMP, i + 1, fs_maxs[i], fs_mins[i]))
    {
      adcSuperStructure.adcUserDatabase.fullScale[i] = COMPARAM(i + 1);
    }
  }

  return MESS_OK;
}


/** Set BCA resolutions */
static uint8_t handle_BCA_resolutions(struct s_comms_control *comms_ctrl)
{
  int i;
  const uint8_t res_mins[ADC_MAX_NUM_INTERVAL] = { APP_DB_U_ADCRES0_MIN, APP_DB_U_ADCRES1_MIN, APP_DB_U_ADCRES2_MIN };
  const uint8_t res_maxs[ADC_MAX_NUM_INTERVAL] = { APP_DB_U_ADCRES0_MAX, APP_DB_U_ADCRES1_MAX, APP_DB_U_ADCRES2_MAX };

  for (i = 0; i < ADC_MAX_NUM_INTERVAL; i++)
  {
    if (param_ok(&CTRL_COMP, i + 1, res_maxs[i], res_mins[i]))
    {
      adcSuperStructure.adcUserDatabase.resolution[i] = (int8u) COMPARAM(i + 1);
    }
  }

  return MESS_OK;
}


/** Set BCA linearisation points */
static uint8_t handle_BCA_linpoints(struct s_comms_control *comms_ctrl)
{
  int i;

  for (i = 0; i <= ADC_MAX_NUM_LIN_PTS; i++) /* <= ADC_MAX_NUM_LIN_PTS because linPts[ADC_MAX_NUM_LIN_PTS+1] */
  {
    if (param_ok(&CTRL_COMP, i + 1, APP_DB_U_ADCLINPTS_MAX, APP_DB_U_ADCLINPTS_MIN))
    {
      adcSuperStructure.adcUserDatabase.linPts[i] = (int8u) COMPARAM(i + 1);
    }
  }
  return MESS_OK;
}


/** Set BCA linearisation correction */
static uint8_t handle_BCA_lincorrections(struct s_comms_control *comms_ctrl)
{
  int i;

  for (i = 0; i <= ADC_MAX_NUM_LIN_PTS; i++) /* <= ADC_MAX_NUM_LIN_PTS because linCor[ADC_MAX_NUM_LIN_PTS+1] */
  {
    if (param_ok(&CTRL_COMP, i + 1, APP_DB_U_ADCLINCOR_MAX, APP_DB_U_ADCLINCOR_MIN))
    {
      adcSuperStructure.adcUserDatabase.linCor[i] = COMPARAM(i + 1);
    }
  }
  return MESS_OK;
}


/** Backup and restore calibration */
uint8_t handle_BCA(struct s_comms_control *comms_ctrl, char *s)
{
  enum eBCAQueryType bqt;
  uint8_t result;

  /* Param 0 is what to send (0 = cal info, 1 = linearisation points) */
  bqt = bqtBasic;
  if (param_ok(&CTRL_COMP, 0, bqtNum - 1, 0))
  {
    bqt = (enum eBCAQueryType) COMPARAM(0);
  }

  if (comms_ctrl->cmd_query)
  {
    /* Query */
    result = send_BCA(comms_ctrl, bqt);
  }
  else
  {
    /* Command */
    if (comms_ctrl->passcode_ok)
    {
      switch (bqt)
      {
        case bqtBasic:
          result = handle_BCA_basic(comms_ctrl);
          break;
        case bqtFullscales:
          result = handle_BCA_fullscales(comms_ctrl);
          break;
        case bqtResolutions:
          result = handle_BCA_resolutions(comms_ctrl);
          break;
        case bqtLinPoints:
          result = handle_BCA_linpoints(comms_ctrl);
          break;
        case bqtLinCorrections:
          result = handle_BCA_lincorrections(comms_ctrl);
          break;
        default:
          result = MESS_ERROR;
          break;
      }
    }
    else
    {
      /* Passcode not ok */
      result = MESS_ERROR;
    }
  }
  return result;
}


/** Legacy BDR always sends the default settings */
static void send_BDR(struct s_comms_control *comms_ctrl)
{
  /* BDR is maintained for backward compatibility, without port or CTS setting */
  comms_net_rfprintf(comms_ctrl, "%u,%u,%u,%u,%u" COMMS_TERMINATOR,
                     DB_SERIAL_DEFAULT_BAUD_INDEX + 1,
                     DB_SERIAL_DEFAULT_PARITY,
                     DB_SERIAL_DEFAULT_DATA,
                     DB_SERIAL_DEFAULT_STOP,
                     DB_SERIAL_DEFAULT_USETERM
                    );
}


/** Legacy serial baud rate and bit settings (does nothing, maintained for backward compatibility) */
uint8_t handle_BDR(struct s_comms_control *comms_ctrl, char *s)
{
  if (comms_ctrl->cmd_query)
  {
    send_BDR(comms_ctrl);
    return REPLY_SENT;
  }

  /* Param 0 - baud rate */
  param_ok(&CTRL_COMP, 0, NUM_DB_BAUD_SETTINGS, 1);

  /* Param 1 - parity bits */
  param_ok(&CTRL_COMP, 1, 2, 0);

  /* Param 2 - data bits */
  param_ok(&CTRL_COMP, 2, 8, 7);

  /* Param 3 - stop bits */
  param_ok(&CTRL_COMP, 3, 2, 1);

  /* Param 4 - use terminations */
  param_ok(&CTRL_COMP, 4, 1, 0);

  return MESS_OK;
}


static void send_BDX(struct s_comms_control *comms_ctrl, enum eSerialPort serial_port)
{
  enum eBDXWFPSerialPort bdx_port;
  enum eSerialPortType porttype;
  enum eSerialBaudIndex baudidx;
  unsigned paritybits;
  unsigned databits;
  unsigned stopbits;
  unsigned terminations;
  unsigned usects;
  unsigned users232;
  int tcp_port;
  unsigned long tcp_timeout;
  struct in_addr udp_dest_ip;
  char udp_dest_ip_str[INET_ADDRSTRLEN];

  bdx_port = xlat_serial_port_to_bdxwfp(serial_port);
  baudidx = db_get_serial_baud_index(serial_port);

  porttype = db_get_serial_type(serial_port);
  paritybits = db_get_serial_paritybits(serial_port);
  databits = db_get_serial_databits(serial_port);
  stopbits = db_get_serial_stopbits(serial_port);
  terminations = db_get_serial_useterm(serial_port) ? 1 : 0;
  usects = db_get_serial_usects(serial_port) ? 1 : 0;
  users232 = db_get_serial_users232(serial_port) ? 1 : 0;
  tcp_port = db_get_serial_tcpport(serial_port);
  tcp_timeout = db_get_serial_tcp_timeout(serial_port);
  db_get_serial_dest_ipaddress(serial_port, &udp_dest_ip);
  (void) net_ip_inet_ntop_host_afinet(&udp_dest_ip, udp_dest_ip_str, INET_ADDRSTRLEN);

  /* The BDX command contains the port */
  comms_net_rfprintf(comms_ctrl, "%u,%u,%u,%u,%u,%u,%u,%u,%u,%d,%lu,\"%s\"" COMMS_TERMINATOR,
                     (unsigned) bdx_port,
                     (unsigned) porttype,
                     (unsigned) baudidx + 1,
                     paritybits,
                     databits,
                     stopbits,
                     terminations,
                     usects,
                     users232,
                     tcp_port,
                     tcp_timeout,
                     udp_dest_ip_str
                    );
}


static uint8_t save_BDX(struct s_comms_control *comms_ctrl, enum eSerialPort port)
{
  uint8_t settings;
  enum eSerialPortType porttype_min;
  enum eSerialPortType porttype_max;
  int tcpport_lowlimit;
  int tcpport_hilimit;
  struct in_addr udp_dest_ip;

  /* Existing settings */
  if (port < NUM_DB_SERIAL_SETTINGS)
  {
    settings = db_app_user.serial_bit_settings[port];
  }
  else
  {
    settings = DB_SERIAL_DEFAULT;
  }

  /* Param 1 - port type */
  porttype_min = db_get_serial_type_min(port);
  porttype_max = db_get_serial_type_max(port);
  if (param_ok(&CTRL_COMP, 1, porttype_max, porttype_min))
  {
    db_set_serial_type(port, (enum eSerialPortType) COMPARAM(1));
  }

  /* Param 2 - baud rate */
  if (param_ok(&CTRL_COMP, 2, NUM_DB_BAUD_SETTINGS, 1))
  {
    db_set_serial_baud_index(port, (enum eSerialBaudIndex) (COMPARAM(2) - 1));
  }

  /* Param 3 - parity bits */
  if (param_ok(&CTRL_COMP, 3, 2, 0))
  {
    settings &= ~DB_SERIAL_PARITY_MASK;
    settings |= db_get_bits_from_serial_paritybits(COMPARAM(3));
  }

  /* Param 4 - data bits */
  if (param_ok(&CTRL_COMP, 4, 8, 7))
  {
    settings &= ~DB_SERIAL_DATA_MASK;
    settings |= db_get_bits_from_serial_databits(COMPARAM(4));
  }

  /* Param 5 - stop bits */
  if (param_ok(&CTRL_COMP, 5, 2, 1))
  {
    settings &= ~DB_SERIAL_STOP_MASK;
    settings |= db_get_bits_from_serial_stopbits_from_bits(COMPARAM(5));
  }

  /* Param 6 - use terminations */
  if (param_ok(&CTRL_COMP, 6, 1, 0))
  {
    settings &= ~DB_SERIAL_USETERM_MASK;
    if (COMPARAM(6) == 1)
    {
      settings |= DB_SERIAL_USETERM;
    }
  }

  /* Param 7 - use CTS */
  if (param_ok(&CTRL_COMP, 7, 1, 0))
  {
    settings &= ~DB_SERIAL_USECTS_MASK;
    if (COMPARAM(7) == 1)
    {
      settings |= DB_SERIAL_USECTS;
    }
  }

  /* Param 8 - use RS232 */
  if (param_ok(&CTRL_COMP, 8, 1, 0))
  {
    settings &= ~DB_SERIAL_USERS232_MASK;
    if (COMPARAM(8) == 1)
    {
      settings |= DB_SERIAL_RS232;
    }
  }

  /* Param 9 - TCP listen port, only applies to SERPORT_TCPNET, SERPORT_TCPAUTO, SERPORT_UDPOUTx, SERPORT_UDPINx, db_set_serial_tcpport() will ignore all others */
  tcpport_lowlimit = comms_is_tcp_connection(port) ? DB_SERIAL_TCPPORT_LOW_LIMIT : DB_SERIAL_UDP_PORT_LOW_LIMIT;
  tcpport_hilimit = comms_is_tcp_connection(port) ? DB_SERIAL_TCPPORT_HIGH_LIMIT : DB_SERIAL_UDP_PORT_HIGH_LIMIT;
  if (param_ok(&CTRL_COMP, 9, tcpport_hilimit, tcpport_lowlimit))
  {
    db_set_serial_tcpport(port, COMPARAM(9));
  }

  /* Param 10 - TCP timeout, only applies to SERPORT_TCPNET, SERPORT_TCPAUTO, db_set_serial_tcp_timeout() will ignore all others */
  if (param_ok(&CTRL_COMP, 10, DB_SERIAL_TCPTIMEOUT_HIGH_LIMIT, DB_SERIAL_TCPTIMEOUT_LOW_LIMIT))
  {
    db_set_serial_tcp_timeout(port, COMPARAM(10));
  }

  /* Param 11 - UDP destination, only applies to SERPORT_UDPOUTx, db_set_serial_tcpport() will ignore all others */
  if (param_str_ok(&CTRL_COMP, 11, INET_ADDRSTRLEN, 1))
  {
    if (net_ip_inet_pton_host_afinet(COMPARAMSTR(11), &udp_dest_ip))
      db_set_serial_dest_ipaddress(port, &udp_dest_ip);
    else
      return MESS_ERROR;
  }

  /* Only save for those ports with settings */
  if (port < NUM_DB_SERIAL_SETTINGS)
  {
    db_app_user.serial_bit_settings[port] = settings;
  }

  if (comms_is_tcp_connection(port))
  {
    /* Re-configure TCP listening ports */
    comms_task_listen_reconfigure();
  }

  /* Reconfigure only the port specified to BDX */
  comms_reconfigure(false, port);
  return MESS_OK;
}


/** Extended serial baud rate and bit settings, all ports */
uint8_t handle_BDX(struct s_comms_control *comms_ctrl, char *s)
{
  enum eBDXWFPSerialPort bdx_port;
  enum eSerialPort serial_port;

  /* Param 0 is the port for BDX */
  bdx_port = BDXWFP_PORT_0_RB1A;
  if (param_ok(&CTRL_COMP, 0, BDXWFP_PORTNUM - 1, 0))
  {
    bdx_port = (enum eBDXWFPSerialPort) COMPARAM(0);
  }

  /*
   * We can only configure select ports:
   *   - the rinblocks (RB1A, RB1B, RB2A, RB2B)
   *   - TCP ports
   *   - USB host and slave
   *   - UDP ports
   *   - Serial 1 & 2 on a C520 with DB9s
   */
  if (!bdxwfp_port_can_configure(bdx_port))
    return MESS_ERROR;

  /* Convert to serial port */
  serial_port = xlat_bdxwfp_to_serial_port(bdx_port);
  if (SERPORT_UNKNOWN == serial_port)
  {
    return MESS_ERROR;
  }

  /* Query */
  if (comms_ctrl->cmd_query)
  {
    send_BDX(comms_ctrl, serial_port);
    return REPLY_SENT;
  }

  /* Save settings, baud is second parameter */
  return save_BDX(comms_ctrl, serial_port);
}


/** Adjust display brightness */
uint8_t handle_BRT(struct s_comms_control *comms_ctrl, char *s)
{

  /* Send response to query */
  if (comms_ctrl->cmd_query)
  {
    comms_net_rfprintf(comms_ctrl, "%u" COMMS_TERMINATOR,
                       db_app_user.brightness
                      );
    return REPLY_SENT;
  }

  /* Param 0 - display brightness */
  if (param_ok(&CTRL_COMP, 0, APP_DB_U_BRIGHTNESS_MAX, APP_DB_U_BRIGHTNESS_MIN))
  {
    db_app_user.brightness = COMPARAM(0);
    Dset_brightness();
  }
  return MESS_OK;
}


/** Enable/disable buzzer */
uint8_t handle_BUZ(struct s_comms_control *comms_ctrl, char *s)
{

  /* Send response to query */
  if (comms_ctrl->cmd_query)
  {
    comms_net_rfprintf(comms_ctrl, "%u" COMMS_TERMINATOR,
                       db_app_user.buzzer_enable
                      );
    return REPLY_SENT;
  }

  /* Param 0 - buzzer enable */
  if (param_ok(&CTRL_COMP, 0, APP_DB_U_BUZENABLE_MAX, APP_DB_U_BUZENABLE_MIN))
  {
    db_app_user.buzzer_enable = COMPARAM(0);
    setup_kbd_buzz();
  }
  return MESS_OK;
}


/** Perform user zero */
uint8_t handle_CDL(struct s_comms_control *comms_ctrl, char *s)
{
  uint8_t res;

  if (comms_ctrl->cmd_query)
  {
    return MESS_ERROR;
  }

  /* Check status before call, so it won't block */
  res = AdcSemiautoZero(&adcSuperStructure, true, false, true, false);
  if (ADC_OP_STATUS_SUCCESS == res)
  {
    /* Perform zero */
    res = AdcSemiautoZero(&adcSuperStructure, true, true, false, false);
  }

  /* Return result */
  return convert_adc_result_to_mess_result(res);
}


/** Clock date/time format */
uint8_t handle_CFM(struct s_comms_control *comms_ctrl, char *s)
{

  /* Send response to query */
  if (comms_ctrl->cmd_query)
  {
    comms_net_rfprintf(comms_ctrl, "%u,%u" COMMS_TERMINATOR,
                       db_app_user.time_fmt, db_app_user.date_fmt
                      );
    return REPLY_SENT;
  }

  /* Param 0 - time format */
  if (param_ok(&CTRL_COMP, 0, APP_DB_U_TIME_FMT_MAX, APP_DB_U_TIME_FMT_MIN))
  {
    db_app_user.time_fmt = COMPARAM(0);
  }

  /* Param 1 - time format */
  if (param_ok(&CTRL_COMP, 1, APP_DB_U_DATE_FMT_MAX, APP_DB_U_DATE_FMT_MIN))
  {
    db_app_user.date_fmt = COMPARAM(1);
  }
  return MESS_OK;
}


/** Print time and date */
void send_CLK_no_line_termination(struct s_comms_control *comms_ctrl, const t_datetime *t)
{
  comms_net_rfprintf(comms_ctrl, "%u,%u,%u,%u,%u,%u",
                 t->hour,
                 t->minute,
                 t->second,
                 t->date,
                 t->month,
                 t->year % 100
                );
}


uint8_t handle_CLK(struct s_comms_control *comms_ctrl, char *s)
{
  t_datetime dt;

  /* Get current date and time */
  if (!GetDateTime(&dt))
  {
    return MESS_ERROR;
  }

  /* Handle query */
  if (comms_ctrl->cmd_query)
  {
    send_CLK_no_line_termination(comms_ctrl, &dt);
    comms_net_rfprintf(comms_ctrl, COMMS_TERMINATOR);
    return REPLY_SENT;
  }

  /* Handle set date and time */
  if (param_ok(&CTRL_COMP, 0, 23, 0))
    dt.hour = (uint8_t) (COMPARAM(0));
  if (param_ok(&CTRL_COMP, 1, 59, 0))
    dt.minute = (uint8_t) (COMPARAM(1));
  if (param_ok(&CTRL_COMP, 2, 59, 0))
    dt.second = (uint8_t) (COMPARAM(2));
  if (param_ok(&CTRL_COMP, 3, 31, 1))
    dt.date = (uint8_t) (COMPARAM(3));
  if (param_ok(&CTRL_COMP, 4, 12, 1))
    dt.month = (uint8_t) (COMPARAM(4));
  if (param_ok(&CTRL_COMP, 5, 2097, 0))
    dt.year = (uint16_t) (COMPARAM(5));
  if (dt.year < 98)
    dt.year += 2000;
  else if (dt.year < 100)
    dt.year += 1900;
  else if (dt.year < 1998)
    return MESS_ERROR;

  /* Make sure date is ok */
  if (!check_date(dt.date, dt.month, dt.year))
    return MESS_ERROR;

  /* Set the date */
  if (SetDateTime(&dt))
  {
    return MESS_OK;
  }
  else
  {
    return MESS_ERROR;
  }
}


static void send_COF(struct s_comms_control *comms_ctrl)
{
  comms_net_rfprintf(comms_ctrl, "%u" COMMS_TERMINATOR,
                     db_app_user.cof_setting
                    );
}


/** Data output format (for MSV) */
uint8_t handle_COF(struct s_comms_control *comms_ctrl, char *s)
{
  if (comms_ctrl->cmd_query)
  {
    send_COF(comms_ctrl);
    return REPLY_SENT;
  }

  if (param_ok(&CTRL_COMP, 0, APP_DB_U_SERCOF_MAX, APP_DB_U_SERCOF_MIN))
  {
    db_app_user.cof_setting = (uint8_t) COMPARAM(0);
    return MESS_OK;
  }
  return MESS_ERROR;
}


/** Calibration weight */
uint8_t handle_CWT(struct s_comms_control *comms_ctrl, char *s)
{
  long fs;

  if (comms_ctrl->cmd_query)
  {
    comms_net_rfprintf(comms_ctrl, "%ld" COMMS_TERMINATOR,
                       adcSuperStructure.adcUserDatabase.calWeight
                      );
    return REPLY_SENT;
  }

  if (!comms_ctrl->passcode_ok)
  {
    return (MESS_ERROR);
  }

  fs = true_full_scale();
  if (!param_ok(&CTRL_COMP, 0, fs, min_span_value()))
    return MESS_ERROR;
  adcSuperStructure.adcUserDatabase.calWeight = COMPARAM(0);
  return MESS_OK;
}


static void send_ENU(struct s_comms_control *comms_ctrl)
{
  uint8_t weunits;

  weunits = xlat_adc_library_units_to_we2111(adcSuperStructure.adcUserDatabase.units);
  comms_net_rfprintf(comms_ctrl, "%u" COMMS_TERMINATOR, weunits);
}


/** Units of measurement */
uint8_t handle_ENU(struct s_comms_control *comms_ctrl, char *s)
{
  if (comms_ctrl->cmd_query)
  {
    send_ENU(comms_ctrl);
    return REPLY_SENT;
  }

  /* Needs full access */
  if (!comms_ctrl->passcode_ok)
  {
    return MESS_ERROR;
  }

  /* Param 0 - units */
  if (param_ok(&CTRL_COMP, 0, NUM_UNITS - 1, 0))
  {
    adcSuperStructure.adcUserDatabase.units = xlat_we2111_units_to_adc_library(COMPARAM(0));
  }
  return MESS_OK;
}


static void send_ESx(struct s_comms_control *comms_ctrl, bool latched_not_system, bool esr_not_esm)
{
  we_error_t err;
  const char *format;

  err = (latched_not_system) ? get_latched_errors() : get_current_errors();
  format = (esr_not_esm) ? "%05lX" COMMS_TERMINATOR : "%lu" COMMS_TERMINATOR;
  comms_net_rfprintf(comms_ctrl, format, (unsigned long) err);
}


/** Error status */
static uint8_t handle_ESx(struct s_comms_control *comms_ctrl, bool esr_not_esm)
{
  bool latched_not_system;

  if (comms_ctrl->cmd_query)
  {
    /* Default to system errors */
    latched_not_system = false;
    if (param_ok(&CTRL_COMP, 0, 1, 0))
    {
      latched_not_system = (0 != COMPARAM(0));
    }
    send_ESx(comms_ctrl, latched_not_system, esr_not_esm);
    return REPLY_SENT;
  }

  /* Can't set anything for this command */
  return MESS_ERROR;
}


/** Modbus error status (same as ESR but result sent in decimal) */
uint8_t handle_ESM(struct s_comms_control *comms_ctrl, char *s)
{
  return handle_ESx(comms_ctrl, false);
}


/** Error status */
uint8_t handle_ESR(struct s_comms_control *comms_ctrl, char *s)
{
  return handle_ESx(comms_ctrl, true);
}


static void send_xBT(struct s_comms_control *comms_ctrl, const uint8_t *button_funcs, size_t button)
{
  comms_net_rfprintf(comms_ctrl, "%u" COMMS_TERMINATOR, button_funcs[button - 1]);
}


/**
 * Front key & remote button settings
 *
 * @param comms_ctrl
 * @param button_fn is a database array of function settings (ie db_app_user.remote_f or db_app_user.front_key_function)
 * @param num_buttons is the number of buttons in button_fn
 * @param num_functions is the number of functions supported by each button_fn (ie NUM_REMOTE_KEY_FUNCTIONS or NUM_FRONTFUNC)
 * @param func_to_key is an array to convert a function key/remote button into a key press (ie remote_button_to_key_str or front_func_to_key_str)
 * */
uint8_t handle_xBT(struct s_comms_control *comms_ctrl, uint8_t *button_fn, size_t num_buttons, size_t num_functions,
                   const char *func_to_key)
{
  size_t button;
  char key;

  /* Param 0 - key/button (1-3 front function key, 1-8 remote key) */
  button = 1;
  if (param_ok(&CTRL_COMP, 0, num_buttons, 1))
  {
    button = COMPARAM(0);
  }

  if (comms_ctrl->cmd_query)
  {
    /* Query - send button function setting */
    send_xBT(comms_ctrl, button_fn, button);
    return REPLY_SENT;
  }

  /* Key function */
  if (param_ok(&CTRL_COMP, 1, num_functions - 1, 0))
  {
    /* Param 1 - Change function setting for button */
    button_fn[button - 1] = (uint8_t) COMPARAM(1);
    setup_remote_kbd();
  }
  else
  {
    /* Param 1 not present - emulate button (param 0) being pressed */
    key = func_to_key[button_fn[button - 1]];
    if (param_ok(&CTRL_COMP, 2, 1, 0))
    {
      if (1 == COMPARAM(2))
      {
        /* Simulate long press */
        key = toupper(key);
      }
    }
    comms_kbd_put_key_char(key);
  }
  return MESS_OK;
}


/** Handle the front button */
uint8_t handle_FBT(struct s_comms_control *comms_ctrl, char *s)
{
  return handle_xBT(comms_ctrl, db_app_user.front_key_function, NUM_FRONT_FUNCTION_KEYS, NUM_FRONTFUNC, front_func_to_key_str);
}


/** Digital outputs */
uint8_t handle_FOP(struct s_comms_control *comms_ctrl, char *s)
{
  bool ok;
  uint8_t tmp;
  int i;

  if (comms_ctrl->cmd_query)
  {
    tmp = access_get_io_states();
    for (i = 0; i < NUM_SETPOINTS - 1; i++)
    {
      comms_net_rfprintf(comms_ctrl, "%u,", ((tmp & (0x01 << i)) != 0) ? 1 : 0);
    }
    comms_net_rfprintf(comms_ctrl, "%u" COMMS_TERMINATOR, ((tmp & (0x01 << i)) != 0) ? 1 : 0);
    return REPLY_SENT;
  }

  ok = true;
  for (i = 0; i < NUM_SETPOINTS; i++)
  {
    if (param_ok(&CTRL_COMP, i, 1, 0))
    {
      if (!direct_drive_output(&setpoint_running_info, i, (0 != COMPARAM(i))))
      {
        ok = false;
      }
    }
  }

  return (ok) ? MESS_OK : MESS_ERROR;
}


static void send_IAD(struct s_comms_control *comms_ctrl, uint8_t range)
{
  const long fs = adcSuperStructure.adcUserDatabase.fullScale[range];
  const uint8_t res = adcSuperStructure.adcUserDatabase.resolution[range];
  const uint8_t hr = (adcSuperStructure.adcUserDatabase.hiRes) ? 1 : 0;
  const uint8_t slc = (db_app_user.sense_check) ? 1 : 0;

  comms_net_rfprintf(comms_ctrl, "%u,%ld,%u,%u,%u,%ld,%u" COMMS_TERMINATOR,
                     range + 1,
                     fs,
                     adcSuperStructure.adcUserDatabase.dpPos,
                     res + 1,
                     hr,
                     adcSuperStructure.adcUserDatabase.addTare,
                     slc
                    );
}


/** Full scale, decimal point, resolution, hi-res, additive tare */
uint8_t handle_IAD(struct s_comms_control *comms_ctrl, char *s)
{
  uint8_t range;

  /* Default range */
  range = (is_single_range()) ? 0 : 1;

  /* Param 0 is the range, can be used for both querying and setting */
  if (param_ok(&CTRL_COMP, 0, 2, 1))
  {
    range = (uint8_t) (COMPARAM(0) - 1);
  }

  /* Handle query */
  if (comms_ctrl->cmd_query)
  {
    send_IAD(comms_ctrl, range);
    return REPLY_SENT;
  }

  /* Needs full access to change */
  if (!comms_ctrl->passcode_ok)
  {
    return MESS_ERROR;
  }

  /* Param 1 - fullscale */
  if (param_ok(&CTRL_COMP, 1, 999999, 100))
  {
    if (range == 0)
      adcSuperStructure.adcUserDatabase.fullScale[0] = COMPARAM(1);
    else
      adcSuperStructure.adcUserDatabase.fullScale[1] = COMPARAM(1);
    if (adcSuperStructure.adcUserDatabase.addTare > true_full_scale())
      adcSuperStructure.adcUserDatabase.addTare = true_full_scale();
  }

  /* Param 2 - decimal point */
  if (param_ok(&CTRL_COMP, 2, 5, 0))
  {
    adcSuperStructure.adcUserDatabase.dpPos = (uint8_t) COMPARAM(2);
  }

  /* Param 3 - resolution */
  if (param_ok(&CTRL_COMP, 3, ADC_RESOLUTION_ONEHUNDRED + 1, ADC_RESOLUTION_ONE + 1))
  {
    if (range == 0)
      adcSuperStructure.adcUserDatabase.resolution[0] = COMPARAM(3) - 1;
    else
      adcSuperStructure.adcUserDatabase.resolution[1] = COMPARAM(3) - 1;
  }

  /* Param 4 - hi res */
  if (param_ok(&CTRL_COMP, 4, 1, 0))
  {
    adcSuperStructure.adcUserDatabase.hiRes = (0 != COMPARAM(4));
  }

  /* Param 5 - additive tare */
  if (param_ok(&CTRL_COMP, 5, true_full_scale(), 0L))
  {
    adcSuperStructure.adcUserDatabase.addTare = COMPARAM(5);
  }

  /* Param 6 - sense line check */
  if (param_ok(&CTRL_COMP, 6, APP_DB_U_SNSCHECK_MAX, APP_DB_U_SNSCHECK_MIN))
  {
    db_app_user.sense_check = (0 != COMPARAM(6));
  }

  comms_reinit_adc_and_setpoints();
  return MESS_OK;
}


static void send_ICR(struct s_comms_control *comms_ctrl)
{
  comms_net_rfprintf(comms_ctrl, "%u" COMMS_TERMINATOR,
                     AdcSyncFrequencies[adcSuperStructure.adcUserDatabase.syncFreqIdx] / 10);
}


/** Sync cal */
uint8_t handle_ICR(struct s_comms_control *comms_ctrl, char *s)
{
  uint16_t adc_sync_cal;

  if (comms_ctrl->cmd_query)
  {
    send_ICR(comms_ctrl);
    return REPLY_SENT;
  }

  /* Requires full passcode access */
  if (!comms_ctrl->passcode_ok)
  {
    return MESS_ERROR;
  }

  /* Set sync cal */
  if (param_ok(&CTRL_COMP, 0, AdcSyncFrequencies[ADC_NUM_SYNC_FREQ - 1] / 10, AdcSyncFrequencies[0] / 10))
  {
    /* Back and forwards to set to closest setting known by WE2111 */
    adc_sync_cal = COMPARAM(0) * 10;
    xlat_sync_freq_to_adc_library_sync_freq_idx(adc_sync_cal, &adcSuperStructure.adcUserDatabase.syncFreqIdx);
    return MESS_OK;
  }
  return MESS_ERROR;
}


static void send_IDN_IDM(struct s_comms_control *comms_ctrl, bool quote_serial_no)
{
  const char *format;

  format = (quote_serial_no) ?
             "\"%s\",\"%07lu\",\"%s\",\"%s\",%lu" COMMS_TERMINATOR :
             "\"%s\",%07lu,\"%s\",\"%s\",%lu" COMMS_TERMINATOR;

  comms_net_rfprintf(comms_ctrl, format,
                     db_app_user.id,
                     (unsigned long) db_app_factory.serial_no,
                     app_version_str,
                     db_app_factory.model_str,
                     (unsigned long) db_app_factory.model_subtype
                    );
}


static uint8_t handle_idm_idn_cmd(struct s_comms_control *comms_ctrl)
{
  /* Param 0, unit ID */
  if (param_str_charset_ok(&CTRL_COMP, 0, APP_DB_ID_STR_LEN - 1, 0, charset_alpha_numeric))
  {
    snprintf(db_app_user.id, APP_DB_ID_STR_LEN, "%s", COMPARAMSTR(0));
  }
  return MESS_OK;
}


/**
 * Unit identification for Modbus
 *
 * This command implements IDN in a way that follows the command convention
 */
uint8_t handle_IDM(struct s_comms_control *comms_ctrl, char *s)
{
  if (comms_ctrl->cmd_query)
  {
    send_IDN_IDM(comms_ctrl, false);
    return REPLY_SENT;
  }

  return handle_idm_idn_cmd(comms_ctrl);
}


/** Unit identification */
uint8_t handle_IDN(struct s_comms_control *comms_ctrl, char *s)
{
  if (comms_ctrl->cmd_query)
  {
    send_IDN_IDM(comms_ctrl, true);
    return REPLY_SENT;
  }

  return handle_idm_idn_cmd(comms_ctrl);
}


static void send_LBT(struct s_comms_control *comms_ctrl, uint8_t key)
{
  uint8_t tmp;

  tmp = db_app_user.front_key_permissions[key];
  comms_net_rfprintf(comms_ctrl, "%u" COMMS_TERMINATOR, (unsigned) tmp);
}


/** Button permission */
uint8_t handle_LBT(struct s_comms_control *comms_ctrl, char *s)
{
  uint8_t key, dat;

  /* Which key is it, when not specified use 0 */
  key = 0;
  if (param_ok(&CTRL_COMP, 0, NUM_FRONT_KEYS - 1, 0))
  {
    key = (uint8_t) COMPARAM(0);
  }

  /* Handle query */
  if (comms_ctrl->cmd_query)
  {
    send_LBT(comms_ctrl, key);
    return REPLY_SENT;
  }

  /* Needs full access to change */
  if (!comms_ctrl->passcode_ok)
  {
    return MESS_ERROR;
  }

  /* Modify permissions */
  if (param_ok(&CTRL_COMP, 1, front_key_permission_limits[key], FRONTKEY_PERMIS_DISABLED))
  {
    dat = (uint8_t) (COMPARAM(1));
    db_app_user.front_key_permissions[key] = dat;
    return MESS_OK;
  }
  return MESS_ERROR;
}


static unsigned int get_remote_cal_result(void)
{
  return (REMOTE_CAL_MODE_IDLE == get_remote_cal_mode()) ? get_last_cal_result() : REMOTE_CAL_RESULT_BUSY;
}


static void send_remote_cal(struct s_comms_control *comms_ctrl)
{
  const unsigned int result = get_remote_cal_result();

  comms_net_rfprintf(comms_ctrl, "%u" COMMS_TERMINATOR, result);
}


/** Send response to new zero/span calibration query */
static void send_LxN(struct s_comms_control *comms_ctrl, int32_t raw)
{
  const unsigned int result = get_remote_cal_result();
  const long rdg = ADC_RAW_TO_DUVV_ROUNDED(raw);

  comms_net_rfprintf(comms_ctrl, "%ld,%u" COMMS_TERMINATOR, rdg, result);
}


/** New form of zero calibration that doesn't require normal/direct mode */
uint8_t handle_LDN(struct s_comms_control *comms_ctrl, char *s)
{
  /* Handle query */
  if (comms_ctrl->cmd_query)
  {
    send_LxN(comms_ctrl, adcSuperStructure.adcUserDatabase.zeroRaw);
    return REPLY_SENT;
  }

  /* Needs full access to change */
  if (!comms_ctrl->passcode_ok)
  {
    return MESS_ERROR;
  }

  /* Can't start another cal if on is already in progress */
  if (get_remote_cal_mode() != REMOTE_CAL_MODE_IDLE)
  {
    return MESS_ERROR;
  }

  /* Direct zero calibration requires a mV/V parameter. If supplied then we are doing a direct cal. */
  if (param_ok(&CTRL_COMP, 0, ADC_RAW_TO_DUVV(APP_DB_U_ADCZERORAW_MAX), ADC_RAW_TO_DUVV(APP_DB_U_ADCZERORAW_MIN)))
  {
    /* Start direct zero calibration */
    setup_for_zero_direct_cal(COMPARAM(0));
  }
  else
  {
    /* Start normal zero calibration */
    setup_for_zero_cal();
  }

  return MESS_OK;
}


/** Send response to the zero calibration query */
static void send_LDW(struct s_comms_control *comms_ctrl)
{
  send_remote_cal(comms_ctrl);
}


/** Original zero calibration, nobbled so it only does normal calibration because scale type (normal/direct) no longer exists */
uint8_t handle_LDW(struct s_comms_control *comms_ctrl, char *s)
{
  /* Handle query */
  if (comms_ctrl->cmd_query)
  {
    send_LDW(comms_ctrl);
    return REPLY_SENT;
  }

  /* Needs full access to change */
  if (!comms_ctrl->passcode_ok)
  {
    return MESS_ERROR;
  }

  /* Can't start another cal if on is already in progress */
  if (get_remote_cal_mode() != REMOTE_CAL_MODE_IDLE)
  {
    return MESS_ERROR;
  }

  /* Start normal zero calibration */
  setup_for_zero_cal();
  return MESS_OK;
}


static void send_LIC(struct s_comms_control *comms_ctrl, uint8_t pt)
{
  long full_scale_point;
  long full_sale_cnt;
  long pt_val;
  long pt_pct;
  long lin_val;

  /* Work out full scale */
  AdcMutexLock(&adcSuperStructure);
  full_sale_cnt = AdcFinalToCounts(&adcSuperStructure, false, adcSuperStructure.adcUserDatabase.fullScale[(adcSuperStructure.adcUserDatabase.scaleType + 1) / 2], false);
  full_scale_point = AdcGetLinPt(&adcSuperStructure, false, full_sale_cnt);
  full_scale_point -= 128;

  pt_val = adcSuperStructure.adcUserDatabase.linPts[pt];
  pt_pct = ((200 * (pt_val - 128) / full_scale_point) + 1) / 2;

  lin_val = adcSuperStructure.adcUserDatabase.linCor[pt];
  AdcMutexUnlock(&adcSuperStructure);

  comms_net_rfprintf(comms_ctrl, "%ld,%ld" COMMS_TERMINATOR, pt_pct, lin_val);
}


/** Linearisation */
uint8_t handle_LIC(struct s_comms_control *comms_ctrl, char *s)
{
  uint8_t pt;
  long new_val;
/*  long cur_raw, limit; */

  /* Param 0 is the linearisation point, and must be specified */
  if (param_ok(&CTRL_COMP, 0, ADC_MAX_NUM_LIN_PTS, 1))
    pt = (uint8_t) (COMPARAM(0) - 1);
  else
    return (MESS_ERROR);

  /* Handle query */
  if (comms_ctrl->cmd_query)
  {
    send_LIC(comms_ctrl, pt);
    return REPLY_SENT;
  }

  /* Needs full access to change */
  if (!comms_ctrl->passcode_ok)
  {
    return MESS_ERROR;
  }

  /* Param 1 if not specified clears the linearisation point, otherwise sets it */
  if (!param_ok(&CTRL_COMP, 1, 999999L, 0L))
  {
    AdcClearLinPnt(&adcSuperStructure, true, false, pt);
  }
  else
  {
    new_val = COMPARAM(1);
    /* TODO new ADC library limits on linearisation */
    (void) AdcAddLinPnt(&adcSuperStructure, true, true, pt, new_val);
  }
  return MESS_OK;
}


static void send_LIV(struct s_comms_control *comms_ctrl, uint8_t sp)
{
  comms_net_rfprintf(comms_ctrl, "%u,%u,%u,1,%ld,%ld,%ld,%u,%u,%u" COMMS_TERMINATOR,
                     sp + 1,                                            /* Setpoint */
                     db_app_user.setp_active[sp],                       /* Type */
                     (db_app_user.setp_source[sp]) ? 2 : 1,             /* Source */
                     setpoint_target_get(sp),                           /* Target */
                     setpoint_preflight_get(sp),                        /* Preflight */
                     setpoint_hysteresis_get(sp),                       /* Hysteresis */
                     (db_app_user.setp_logic[sp]) ? 2 : 1,              /* Logic inversion */
                     (db_app_user.setp_lock[sp]) ? 1 : 0,               /* Lock */
                     db_app_user.setp_alarm[sp],                        /* Alarm */
                     db_app_user.setp_timing[sp],                       /* Timing */
                     db_app_user.setp_reset[sp]                         /* Reset */
                    );
}


/** Setpoint settings */
uint8_t handle_LIV(struct s_comms_control *comms_ctrl, char *s)
{
  uint8_t sp;
  int max_sp_type;

  /* Param 0 is setpoint, if not specified default to 0 */
  sp = 0;
  if (param_ok(&CTRL_COMP, 0, NUM_SETPOINTS, 1))
  {
    sp = (uint8_t) (COMPARAM(0) - 1);
  }

  /* Handle query */
  if (comms_ctrl->cmd_query)
  {
    send_LIV(comms_ctrl, sp);
    return REPLY_SENT;
  }

  /* Param 1, setpoint type */
  max_sp_type = SETP_TYPE_NUM - 1;
  if (param_ok(&CTRL_COMP, 1, max_sp_type, SETP_INACTIVE))
  {
    db_app_user.setp_active[sp] = (uint8_t) COMPARAM(1);
  }

  /* Param 2 - setpoint source */
  if (param_ok(&CTRL_COMP, 2, SETP_SOURCE_NUM, 1))
  {
    db_app_user.setp_source[sp] = (COMPARAM(2) == 2);
  }

  /* Param 3 - setpoint direction, no longer used */

  /* Param 4 - setpoint target */
  if (param_ok(&CTRL_COMP, 4, APP_DB_R_TARGETS_MAX, APP_DB_R_TARGETS_MIN))
  {
    setpoint_target_set(sp, COMPARAM(4));
  }

  /* Param 5 - setpoint preflight */
  if (param_ok(&CTRL_COMP, 5, APP_DB_R_PREFLIGHT_MAX, APP_DB_R_PREFLIGHT_MIN))
  {
    setpoint_preflight_set(sp, COMPARAM(5));
  }

  /* Param 6 - setpoint hysteresis */
  if (param_ok(&CTRL_COMP, 6, APP_DB_R_HYSTERESIS_MAX, APP_DB_R_HYSTERESIS_MIN))
  {
    setpoint_hysteresis_set(sp, COMPARAM(6));
  }

  /* Param 7 - setpoint logic inversion */
  if (param_ok(&CTRL_COMP, 7, SETP_LOGIC_NUM, 1))
  {
    db_app_user.setp_logic[sp] = (COMPARAM(7) == 2);
  }

  /* Param 8 - setpoint lock */
  if (param_ok(&CTRL_COMP, 8, SETP_LOCK_NUM - 1, 0))
  {
    db_app_user.setp_lock[sp] = (COMPARAM(8) == 1);
  }

  /* Param 9 - setpoint alarm */
  if (param_ok(&CTRL_COMP, 9, SETP_ALARM_NUM - 1, 0))
  {
    db_app_user.setp_alarm[sp] = (uint8_t) COMPARAM(9);
  }

  /* Param 10 - setpoint timing */
  if (param_ok(&CTRL_COMP, 10, SETP_TIMING_NUM - 1, 0))
  {
    db_app_user.setp_timing[sp] = (uint8_t) COMPARAM(10);
  }

  /* Param 11 - setpoint reset */
  if (param_ok(&CTRL_COMP, 11, SETP_RESET_NUM - 1, 0))
  {
    db_app_user.setp_reset[sp] = (uint8_t) COMPARAM(11);
  }

  /* Re-initialise setpointing */
  comms_reinit_setpoints();
  return MESS_OK;
}


static void send_LIW(struct s_comms_control *comms_ctrl)
{
  comms_net_rfprintf(comms_ctrl, "%lu,%lu,%u,%u,%u,%lu" COMMS_TERMINATOR,
                     db_app_user.setp_jog_on,         /* jog on time */
                     db_app_user.setp_jog_off,        /* jog off time */
                     db_app_user.setp_jog_set,        /* jog set */
                     db_app_user.setp_max_sets,       /* max sets */
                     db_app_user.setp_feeder,         /* feeder */
                     db_app_user.setp_delay_check     /* delay check */
                    );
}


/** Setpoint global settings */
uint8_t handle_LIW(struct s_comms_control *comms_ctrl, char *s)
{
  /* Handle query */
  if (comms_ctrl->cmd_query)
  {
    send_LIW(comms_ctrl);
    return REPLY_SENT;
  }

  /* Param 0 - jog on time */
  if (param_ok(&CTRL_COMP, 0, APP_DB_U_SETPJOGON_MAX, APP_DB_U_SETPJOGON_MIN))
  {
    db_app_user.setp_jog_on = (uint32_t) COMPARAM(0);
  }

  /* Param 1 - jog off time */
  if (param_ok(&CTRL_COMP, 1, APP_DB_U_SETPJOGOFF_MAX, APP_DB_U_SETPJOGOFF_MIN))
  {
    db_app_user.setp_jog_off = (uint32_t) COMPARAM(1);
  }

  /* Param 2 - jog set */
  if (param_ok(&CTRL_COMP, 2, APP_DB_U_SETPJOGSET_MAX, APP_DB_U_SETPJOGSET_MIN))
  {
    db_app_user.setp_jog_set = (uint8_t) COMPARAM(2);
  }

  /* Param 3 - max sets */
  if (param_ok(&CTRL_COMP, 3, APP_DB_U_SETPMAXSET_MAX, APP_DB_U_SETPMAXSET_MIN))
  {
    db_app_user.setp_max_sets = (uint8_t) COMPARAM(3);
  }

  /* Param 4 - feeder */
  if (param_ok(&CTRL_COMP, 4, APP_DB_U_SETPFEEDER_MAX, APP_DB_U_SETPFEEDER_MIN))
  {
    db_app_user.setp_feeder = (uint8_t) COMPARAM(4);
  }

  /* Param 5 - delay check */
  if (param_ok(&CTRL_COMP, 5, APP_DB_U_SETPDELAYCHECK_MAX, APP_DB_U_SETPDELAYCHECK_MIN))
  {
    db_app_user.setp_delay_check = (uint32_t) COMPARAM(5);
  }

  /* Re-initialise setpointing */
  comms_reinit_setpoints();
  return MESS_OK;
}


static uint8_t send_LRP(struct s_comms_control *comms_ctrl, size_t pkg_index)
{
  const char *pkg_name;
  size_t num_pkgs;
  bool is_licensed;
  char licence_code[LICENCECODE_LEN_C500 + 1];

  if (!app_licensing_comms_query(pkg_index, &num_pkgs,
                                 &pkg_name,
                                 &is_licensed,
                                 licence_code))
  {
    return MESS_ERROR;
  }

  comms_net_rfprintf(comms_ctrl, "%lu,%lu,\"%s\",%u,\"%s\"" COMMS_TERMINATOR,
                     (unsigned long) pkg_index,
                     (unsigned long) num_pkgs,
                     pkg_name,
                     (is_licensed) ? 1 : 0,
                     licence_code);
  return REPLY_SENT;
}


/** License a rin package */
uint8_t handle_LRP(struct s_comms_control *comms_ctrl, char *s)
{
  size_t pkg_index = 0;
  bool licence_not_unlicence;
  static size_t lrp_tries = 0;
  uint8_t result = MESS_ERROR;

  /* Needs full access for query or command */
  if (!comms_ctrl->passcode_ok)
  {
    return MESS_ERROR;
  }

  /* Param 0 - package index */
  if (param_ok(&CTRL_COMP, 0, APP_DB_L_LICENCECODES_NUM - 1, 0))
  {
    pkg_index = COMPARAM(0);
  }

  /* Handle query */
  if (comms_ctrl->cmd_query)
  {
    return send_LRP(comms_ctrl, pkg_index);
  }

  /* Param 2 - package name */
  if (param_str_ok(&CTRL_COMP, 2, 100, 0))
  {
    /* Param 3 - 1 = licence/0 = unlicence */
    if (param_ok(&CTRL_COMP, 3, 1, 0))
    {
      licence_not_unlicence = (COMPARAM(3) != 0);
      if (licence_not_unlicence)
      {
        /* Param 4 - licence code */
        if (param_str_ok(&CTRL_COMP, 4, LICENCECODE_LEN_C500, 0))
        {
          if (lrp_tries < WECFG_LICENSING_COMMS_TRIES)
          {
            /* Try to license the package with given code */
            if (app_licensing_comms_cmd(COMPARAMSTR(2), licence_not_unlicence, COMPARAMSTR(4)))
            {
              result = (app_write_licence_database()) ? MESS_OK : MESS_ERROR;
            }
            else
            {
              lrp_tries++;
              result = MESS_ERROR;
            }
          }
        }
      }
      else
      {
        /* Unlicense the package */
        result = (app_licensing_comms_cmd(COMPARAMSTR(2), licence_not_unlicence, "") &&
                  app_write_licence_database()) ? MESS_OK : MESS_ERROR;
      }
    }
  }

  return result;
}


/** New form of span calibration that doesn't require normal/direct mode */
uint8_t handle_LWN(struct s_comms_control *comms_ctrl, char *s)
{
  /* Handle query */
  if (comms_ctrl->cmd_query)
  {
    send_LxN(comms_ctrl, adcSuperStructure.adcUserDatabase.spanRaw);
    return REPLY_SENT;
  }

  /* Needs full access to change */
  if (!comms_ctrl->passcode_ok)
  {
    return MESS_ERROR;
  }

  /* Can't start another cal if on is already in progress */
  if (get_remote_cal_mode() != REMOTE_CAL_MODE_IDLE)
  {
    return MESS_ERROR;
  }

  /* Direct span calibration requires a mV/V parameter. If supplied then we are doing a direct cal. */
  if (param_ok(&CTRL_COMP, 0, ADC_RAW_TO_DUVV(APP_DB_U_ADCSPANRAW_MAX), ADC_RAW_TO_DUVV(APP_DB_U_ADCSPANRAW_MIN)))
  {
    /* Start direct span cal */
    setup_for_span_direct_cal(COMPARAM(0));
  }
  else
  {
    /* Start normal span calibration */
    setup_for_span_cal();
  }

  return MESS_OK;
}


/** Send response to the span calibration query */
static void send_LWT(struct s_comms_control *comms_ctrl)
{
  send_remote_cal(comms_ctrl);
}


/** Original span calibration, nobbled so it only does normal calibration because scale type (normal/direct) no longer exists */
uint8_t handle_LWT(struct s_comms_control *comms_ctrl, char *s)
{
  /* Handle query */
  if (comms_ctrl->cmd_query)
  {
    send_LWT(comms_ctrl);
    return REPLY_SENT;
  }

  /* Needs full access to change */
  if (!comms_ctrl->passcode_ok)
  {
    return MESS_ERROR;
  }

  /* Can't start another cal if on is already in progress */
  if (get_remote_cal_mode() != REMOTE_CAL_MODE_IDLE)
  {
    return MESS_ERROR;
  }

  /* Start normal span calibration */
  setup_for_span_cal();
  return MESS_OK;
}


/** Modbus settings */
uint8_t handle_MBS(struct s_comms_control *comms_ctrl, char *s)
{
  /* Handle query */
  if (comms_ctrl->cmd_query)
  {
    comms_net_rfprintf(comms_ctrl, "%u,%lu" COMMS_TERMINATOR,
                       (unsigned) db_app_user.modbus_endian, (unsigned long) db_app_user.modbus_tcpport);
    return REPLY_SENT;
  }

  /* Param 0 - Modbus endian setting */
  if (param_ok(&CTRL_COMP, 0, APP_DB_U_MODBUSENDIAN_MAX, APP_DB_U_MODBUSENDIAN_MIN))
  {
    db_app_user.modbus_endian = (uint8_t) COMPARAM(0);
  }

  /* Param 1 - Modbus TCP port setting */
  if (param_ok(&CTRL_COMP, 1, APP_DB_U_MODBUSTCPPORT_MAX, APP_DB_U_MODBUSTCPPORT_MIN))
  {
    db_app_user.modbus_tcpport = COMPARAM(1);
  }
  return MESS_OK;
}


/** Query weight and status readings for Modbus */
uint8_t handle_MSM(struct s_comms_control *comms_ctrl, char *s)
{
  enum eMSVSource source;

  if (!comms_ctrl->cmd_query)
  {
    return MESS_OK;
  }

  /* Param 0 - data source (gross/net/displayed/etc) */
  source = MSV_SRC_DISPLAYED;
  if (param_ok(&CTRL_COMP, 0, MSV_SRC_NUM, MSV_SRC_DISPLAYED))
  {
    source = (enum eMSVSource) COMPARAM(0);
  }

  comms_send_oiml_msm(comms_ctrl, source);
  return REPLY_SENT;
}


/** 0,2,4,6,8 are COF binary outputs */
static bool is_binary_COF(enum eCOFSetting cof)
{
  bool result;

  switch (cof)
  {
    case COF_SETTING0_BINARY4_MSB_FIRST:
    case COF_SETTING2_BINARY2_MSB_FIRST:
    case COF_SETTING4_BINARY4_LSB_FIRST:
    case COF_SETTING6_BINARY2_LSB_FIRST:
    case COF_SETTING8_BINARY4_MSB_FIRST_LSB_STATUS:
      result = true;
      break;

    default:
      result = false;
      break;
  }
  return result;
}


/** Query weight readings */
uint8_t handle_MSV(struct s_comms_control *comms_ctrl, char *s)
{
  if (!comms_ctrl->cmd_query)
  {
    /* check_mux_error(true); */
    return MESS_OK;
  }

  /* Param 0 - data source (gross/net/displayed/etc) */
  comms_ctrl->msv_source = MSV_SRC_DISPLAYED;
  if (param_ok(&CTRL_COMP, 0, MSV_SRC_NUM, MSV_SRC_DISPLAYED))
  {
    comms_ctrl->msv_source = (enum eMSVSource) COMPARAM(0);
  }

  /* Param 1 - how many records to send, if not specified then 1 */
  comms_ctrl->msv_cmd_streaming = true;
  if (param_ok(&CTRL_COMP, 1, 60000, 0))
  {
    comms_ctrl->msv_count = (int) COMPARAM(1);
  }
  else
  {
    comms_ctrl->msv_count = 1;
  }

  /* Figure out if we need to send an extra end (ie CRLF) - TODO this could probably be done elsewhere, and not require the global */
  if (is_binary_COF(db_app_user.cof_setting))
  {
    comms_ctrl->send_extra_line_termination = true;
  }
  else
  {
    comms_ctrl->send_extra_line_termination = (comms_ctrl->msv_count != 1);
  }

  return REPLY_SENT;
}


/** Motion settings */
uint8_t handle_MTD(struct s_comms_control *comms_ctrl, char *s)
{
  /* Handle query */
  if (comms_ctrl->cmd_query)
  {
    comms_net_rfprintf(comms_ctrl, "%u" COMMS_TERMINATOR, adcSuperStructure.adcUserDatabase.motion);
    return REPLY_SENT;
  }

  /* Needs full access to change */
  if (!comms_ctrl->passcode_ok)
  {
    return MESS_ERROR;
  }

  /* Param 0 - Motion setting */
  if (param_ok(&CTRL_COMP, 0, ADC_NUM_MOTION_DETECT - 1, 0))
  {
    adcSuperStructure.adcUserDatabase.motion = (uint8_t) COMPARAM(0);
    comms_reinit_adc_and_setpoints();
    return MESS_OK;
  }
  return MESS_ERROR;
}
