/*
 * comms_hdlr_am.h
 *
 *  Created on: 13/10/2010
 *      Author: ian
 */

#ifndef COMMS_HDLR_AM_H_
#define COMMS_HDLR_AM_H_

#include <stdint.h>
#include "sysclock.h"
#include "comms_util.h"


/** Re-initialise setpoints and IO hardware */
void comms_reinit_setpoints(void);


/** Re-initialise the adc and setpoints */
void comms_reinit_adc_and_setpoints(void);


/** Sync/gain cal */
/* uint8_t handle_AAA(struct s_comms_control *comms_ctrl, char *s); */


/** Test MultiDiv() */
uint8_t handle_AAB(struct s_comms_control *comms_ctrl, char *s);


/** Zero mux settings */
uint8_t handle_ACL(struct s_comms_control *comms_ctrl, char *s);


/** Unit address */
uint8_t handle_ADR(struct s_comms_control *comms_ctrl, char *s);


/** Auto output format */
uint8_t handle_AFT(struct s_comms_control *comms_ctrl, char *s);


/** Analog output calibration */
uint8_t handle_AOC(struct s_comms_control *comms_ctrl, char *s);


/** Averaging and jitter */
uint8_t handle_ASF(struct s_comms_control *comms_ctrl, char *s);


/** Backup and restore calibration */
uint8_t handle_BCA(struct s_comms_control *comms_ctrl, char *s);


/** Legacy serial baud rate and bit settings (does nothing, maintained for backward compatibility) */
uint8_t handle_BDR(struct s_comms_control *comms_ctrl, char *s);


/** Extended serial baud rate and bit settings, all ports */
uint8_t handle_BDX(struct s_comms_control *comms_ctrl, char *s);


/** Adjust display brightness */
uint8_t handle_BRT(struct s_comms_control *comms_ctrl, char *s);


/** Enable/disable buzzer */
uint8_t handle_BUZ(struct s_comms_control *comms_ctrl, char *s);


/** Perform user zero */
uint8_t handle_CDL(struct s_comms_control *comms_ctrl, char *s);


/** Clock date/time format */
uint8_t handle_CFM(struct s_comms_control *comms_ctrl, char *s);


/** Print time and date */
void send_CLK_no_line_termination(struct s_comms_control *comms_ctrl, const t_datetime *t);


/** Time and date */
uint8_t handle_CLK(struct s_comms_control *comms_ctrl, char *s);


/** Data output format (for MSV) */
uint8_t handle_COF(struct s_comms_control *comms_ctrl, char *s);


/** Calibration weight */
uint8_t handle_CWT(struct s_comms_control *comms_ctrl, char *s);


/** Dump ADC readings */
uint8_t handle_DBG(char *s);


/** Units of measurement */
uint8_t handle_ENU(struct s_comms_control *comms_ctrl, char *s);


/** Modbus error status (same as ESR but result sent in decimal) */
uint8_t handle_ESM(struct s_comms_control *comms_ctrl, char *s);


/** Error status */
uint8_t handle_ESR(struct s_comms_control *comms_ctrl, char *s);


/**
 * Front key & remote button settings
 *
 * @param comms_ctrl
 * @param button_fn is a database array of function settings (ie db_app_user.remote_f or db_app_user.front_key_function)
 * @param num_buttons is the number of buttons in button_fn
 * @param num_functions is the number of function supported by each button_fn (ie NUM_REMOTE_KEY_FUNCTIONS or NUM_FRONTFUNC)
 * @param func_to_key is an array to convert a function key/remote button into a key press (ie remote_button_to_key_str or front_func_to_key_str)
 * */
uint8_t handle_xBT(struct s_comms_control *comms_ctrl, uint8_t *button_fn, size_t num_buttons, size_t num_functions,
                   const char *func_to_key);


/** Handle the front button */
uint8_t handle_FBT(struct s_comms_control *comms_ctrl, char *s);


/** Digital outputs */
uint8_t handle_FOP(struct s_comms_control *comms_ctrl, char *s);


/** Display buffer */
uint8_t handle_FPL(struct s_comms_control *comms_ctrl, char *s);


/** Full scale, decimal point, resolution, hi-res, additive tare */
uint8_t handle_IAD(struct s_comms_control *comms_ctrl, char *s);


/** Sync cal */
uint8_t handle_ICR(struct s_comms_control *comms_ctrl, char *s);

/**
 * Unit identification for Modbus
 *
 * This command implements IDN in a way that follows the command convention
 */
uint8_t handle_IDM(struct s_comms_control *comms_ctrl, char *s);


/** Unit identification */
uint8_t handle_IDN(struct s_comms_control *comms_ctrl, char *s);


/** Button permission */
uint8_t handle_LBT(struct s_comms_control *comms_ctrl, char *s);


/** New form of zero calibration that doesn't require normal/direct mode */
uint8_t handle_LDN(struct s_comms_control *comms_ctrl, char *s);


/** Original zero calibration, nobbled so it only does normal calibration because scale type (normal/direct) no longer exists */
uint8_t handle_LDW(struct s_comms_control *comms_ctrl, char *s);


/** Linearisation */
uint8_t handle_LIC(struct s_comms_control *comms_ctrl, char *s);


/** Setpoint settings */
uint8_t handle_LIV(struct s_comms_control *comms_ctrl, char *s);


/** Setpoint global settings */
uint8_t handle_LIW(struct s_comms_control *comms_ctrl, char *s);


/** License a rin package */
uint8_t handle_LRP(struct s_comms_control *comms_ctrl, char *s);


/** New form of span calibration that doesn't require normal/direct mode */
uint8_t handle_LWN(struct s_comms_control *comms_ctrl, char *s);


/** Original span calibration, nobbled so it only does normal calibration because scale type (normal/direct) no longer exists */
uint8_t handle_LWT(struct s_comms_control *comms_ctrl, char *s);


/** Modbus settings */
uint8_t handle_MBS(struct s_comms_control *comms_ctrl, char *s);


/** Query weight and status readings for Modbus */
uint8_t handle_MSM(struct s_comms_control *comms_ctrl, char *s);


/** Query weight readings */
uint8_t handle_MSV(struct s_comms_control *comms_ctrl, char *s);


/** Motion settings */
uint8_t handle_MTD(struct s_comms_control *comms_ctrl, char *s);


#endif /* COMMS_HDLR_AM_H_ */
