/*
 * comms_hdlr_dft.c
 *
 *  Created on: 30 May 2018
 *      Author: ian
 */
#include <stdbool.h>
#include <stdint.h>
#include <rinweigh/adcinterface.h>
#include <rinweigh/adcliveimpl_simple.h>
#include "comms_util.h"
#include "comms_errors.h"
#include "ipinfo.h"
#include "we_config.h"
#include "app_db_changelog.h"
#include "app_db_user.h"
#include "app_db_runtime.h"
#include "app_db_licence.h"
#include "adcthread.h"
#include "iface_menus.h"
#include "app_save.h"
#include "licensing.h"

#include "comms_hdlr_defaults.h"


/** Do the actual defaulting of a particular database, but not saving or re-initialising. Don't call directly, call @ref comms_default_settings() instead. */
static bool comms_default_specific_database(enum e_comms_default_settings default_what, bool *should_reinit)
{
  bool result = false;
  struct s_snc_net_config default_net_config;

  *should_reinit = false;
  switch (default_what)
  {
    /* Default user database, except calibration */
    case cds_userdb:
      if (app_can_save_databases())
      {
        app_default_user_database(false);
        result = true;
        *should_reinit = true;
      }
      break;

    /* Default runtime database (all of it user & adc settings) */
    case cds_runtimedb:
      if (app_can_save_databases())
      {
        app_default_runtime_database(true);
        result = true;
        *should_reinit = true;
      }
      break;

    /* Default calibration */
    case cds_calibration:
      if (app_can_save_databases())
      {
        AdcDoDefaultCal(&adcSuperStructure, true, false);
        result = true;
        *should_reinit = true;
      }
      break;

    /* Default ethernet settings */
    case cds_network:
      net_ip_set_defaults(&default_net_config);
      result = net_ip_save_config(&default_net_config, NULL);
      break;

    /* Default licensing settings */
    case cds_licensingdb:
      result = app_default_licence_database(NULL, 0) &&
               app_write_licence_database();
      app_licensing_force_recheck();
      break;

    /* Default all databases not handled here, use comms_default_settings() instead */
    case cds_all:
    default:
      break;
  }
  return result;
}


/** Default the settings specified by default_what from a comms command */
bool comms_default_settings(enum e_comms_default_settings default_what)
{
  bool should_reinit = false;
  bool should_reinit1 = false;
  bool should_reinit2 = false;
  bool should_reinit3 = false;
  bool should_reinit4 = false;
  bool result;

  if (cds_all == default_what)
  {
    result = comms_default_specific_database(cds_userdb, &should_reinit1) &&
             comms_default_specific_database(cds_runtimedb, &should_reinit2) &&
             comms_default_specific_database(cds_calibration, &should_reinit3) &&
             comms_default_specific_database(cds_network, &should_reinit4);
    should_reinit = (should_reinit1 || should_reinit2 || should_reinit3 || should_reinit4);
  }
  else
  {
    result = comms_default_specific_database(default_what, &should_reinit);
  }

  if (should_reinit)
  {
    (void) app_save_databases_and_reinit(false, NULL);
  }
  return result;
}


/** A single command for setting defaults, more for final factory setup than customer use */
uint8_t handle_DFT(struct s_comms_control *comms_ctrl, char *s)
{
  enum e_comms_default_settings default_what;

  /* No query for this command */
  if (comms_ctrl->cmd_query)
    return MESS_ERROR;

  if (!param_ok(&CTRL_COMP, 0, 4, 0))
    return MESS_ERR_SYS;

  if (!comms_ctrl->passcode_ok)
    return MESS_ERROR;

  switch (COMPARAM(0))
  {
    /* Default user database, except calibration */
    case 0:
      default_what = cds_userdb;
      break;

    /* Default runtime database (all of it user & adc settings) */
    case 1:
      default_what = cds_runtimedb;
      break;

    /* Default calibration */
    case 2:
      default_what = cds_calibration;
      break;

    /* Default ethernet settings */
    case 3:
      default_what = cds_network;
      break;

    /* Default licensing settings */
    case 4:
      default_what = cds_licensingdb;
      break;

    default:
      return MESS_ERROR;
      break;
  }

  /* Do the defaulting */
  return comms_default_settings(default_what) ? MESS_OK : MESS_ERROR;
}
