/*
 * comms_hdlr_fpl.c
 *
 *  Created on: Mar 3, 2017
 *      Author: ian
 */
#include <stdbool.h>
#include <endian.h>
#include <stddef.h>
#include <stdint.h>
#include <string.h>
#include <stdio.h>
#include <rinhw/rio.h>
#include <rinhw/lcd.h>
#include <rinblock/rb_msg.h>
#include "comms_util.h"
#include "devices.h"
#include "comms_errors.h"
#include "comms_kbd.h"

#include "comms_hdlr_fpl.h"


/** Convert the contents of display buffer into a endian independent string */
bool display_buffer_to_string(char *displaystr, size_t displaystrlen, bool quoted)
{
  struct s_rb_c500_disp_write_14seg segs;
  long len;
  uint8_t data;
  size_t strpos;
  size_t bufpos;
  bool result;
  uint8_t *bufptr;
  size_t dpystrend;
#if (BYTE_ORDER != BIG_ENDIAN)
  size_t i;
#endif

  if (ropen(&dev_lcd_we, 0) < 0)
  {
    return false;
  }
  rioctl(&dev_lcd_we, IOCTL_LCD_ACCESS_OUTPUT);
  rlseek(&dev_lcd_we, 0, SEEK_SET);
  len = rfilelength(&dev_lcd_we);
  if (sizeof(segs) != len)
  {
    result = false;
    goto sf_fail;
  }

  rread(&dev_lcd_we, &segs, sizeof(segs), 0);

#if (BYTE_ORDER != BIG_ENDIAN)
  /* When running on the PC we need to swap the endian */
  for (i = 0; i < C500_DISPLAY_NUM_DIGITS; i++)
  {
    segs.digits[i].word = htobe16(segs.digits[i].word);
  }
  segs.annunciators.lword = htobe32(segs.annunciators.lword);
#endif

  if (quoted)
    strncpy(displaystr, "\"", displaystrlen);
  displaystr[displaystrlen - 1] = '\0';
  bufptr = (uint8_t *) &segs;
  strpos = (quoted) ? 1 : 0;
  dpystrend = (quoted) ? displaystrlen - 2 : displaystrlen - 1;
  for (bufpos = 0; bufpos < sizeof(segs) && strpos < dpystrend; bufpos++, strpos += 2)
  {
    data = bufptr[bufpos];
    snprintf(&displaystr[strpos], 3, "%02X", data);
  }
  if (quoted)
    strncat(displaystr, "\"", displaystrlen - 1);
  result = true;

sf_fail:
  rclose(&dev_lcd_we);
  return result;
}


/** Send contents of display buffer */
static uint8_t send_FPL(struct s_comms_control *comms_ctrl)
{
  /* FPL string is two chars for every byte in the display buffer plus two quotes plus one null terminator */
  #define FPL_STR_LEN (sizeof(struct s_rb_c500_disp_write_14seg) * 2 + 2 + 1)
  char fpl_str[FPL_STR_LEN];
  uint8_t res;

  res = (display_buffer_to_string(fpl_str, FPL_STR_LEN, true)) ? REPLY_SENT : MESS_ERROR;
  comms_net_rfprintf(comms_ctrl, "%s" COMMS_TERMINATOR, fpl_str);
  return res;
}


/** Display buffer */
uint8_t handle_FPL(struct s_comms_control *comms_ctrl, char *s)
{
  uint8_t key;
  bool longpress;

  /* Handle query */
  if (comms_ctrl->cmd_query)
  {
    return send_FPL(comms_ctrl);
  }

  /* Handle write keystroke, first param is key, second param is long press */
  if (param_ok(&CTRL_COMP, 0, WE_NUM_PHYSICAL_KEYS - 1, 1))
  {
    key = COMPARAM(0);
    longpress = (param_ok(&CTRL_COMP, 1, 1, 0) && COMPARAM(1) > 0);
    comms_kbd_put_key(key, longpress);
  }
  return MESS_OK;
}
