/*
 * comms_kbd.c
 *
 *  Created on: 13/10/2010
 *      Author: ian
 */
#include "kbd_5000.h"

#include "comms_kbd.h"


/** Current comms key */
static uint8_t comms_kbd_key = NO_KEY;


/** Send a key from the comms to the indicator using keycodes _KEY constants */
void comms_kbd_put_key(uint8_t key, bool longpress)
{
  /* Cal key not permitted */
  if (CAL_KEY == key)
    return;

  /* Handle long press */
  if (longpress)
    key += DEKEY;

  /* Save key */
  comms_kbd_key = key;
}


/** Send a key from the comms to the indicator using characters <STX>Kx00<ETX> commands */
void comms_kbd_put_key_char(char keych)
{
  switch (keych)
  {
    case 'Z':
      comms_kbd_key = DEZERO_KEY;
      break;
    case 'z':
      comms_kbd_key = ZERO_KEY;
      break;
    case 'T':
    case 't':
      comms_kbd_key = TARE_KEY;
      break;
    case 'G':
    case 'g':
      comms_kbd_key = GROSS_KEY;
      break;
    case 'P':
      comms_kbd_key = PRINT_KEY;
      break;
    case 'q':
      comms_kbd_key = PRINT_KEY;  /*NB 'q' is used to implement 'p' properly, but the original handling of KP and Kp is retained */
      break;
    case 'Q':
      comms_kbd_key = DE_PRINT_KEY;
      break;
    case 'U':
    case 'u':
      comms_kbd_key = UNDO_KEY;
      break;
    case 'C':
    case 'c':
      comms_kbd_key = CLR_TOTAL_KEY;
      break;
    case 'S':
    case 's':
      comms_kbd_key = SHOW_TOTAL_KEY;
      break;
    case '1':
      comms_kbd_key = SINGLE1_KEY;
      break;
    case '2':
      comms_kbd_key = SINGLE2_KEY;
      break;
    case '3':
      comms_kbd_key = SINGLE3_KEY;
      break;
    case '4':
      comms_kbd_key = SINGLE4_KEY;
      break;
    case '5':
      comms_kbd_key = SINGLE5_KEY;
      break;
    case '6':
      comms_kbd_key = SINGLE6_KEY;
      break;
    case 'H':
    case 'h':
      comms_kbd_key = MHOLD_KEY;
      break;
    case 'e':
      comms_kbd_key = PHOLD_KEY;
      break;
    case 'E':
      comms_kbd_key = DE_PHOLD_KEY;
      break;
    case 'f':
      comms_kbd_key = LHOLD_KEY;
      break;
    case 'F':
      comms_kbd_key = DE_LHOLD_KEY;
      break;
    case '.':
      comms_kbd_key = IDLE_CHG;
      break;
    case 'n':
      comms_kbd_key = UNITS_KEY;
      break;
  }
}


/** Translate a comms key into a normal WE2111 keystroke */
uint8_t comms_kbd_get_cmd(void)
{
  uint8_t result;

  result = comms_kbd_key;
  comms_kbd_key = NO_KEY;
  return result;
}
