/*
 * comms_net.h
 *
 *  Created on: 03/03/2011
 *      Author: ian
 */

#ifndef COMMS_NET_H_
#define COMMS_NET_H_

#include <stddef.h>
#include <stdbool.h>
#include <netinet/in.h>
#include "comms_util.h"
#include "comms_print_buffer.h"
#include "comms_errors.h"
#include "comms_udp.h"


/**
 * Maximum length of a comms_message.
 * This must be large enough to handle the longest comms command, currently AFT & PFT (xFT"<250chars>"\0) = 256
 *
 * I've made it double the size
 */
#define COMMS_NET_MAX_MESS_LEN 512


/** Structure for network comms */
struct s_comms_net
{
  char incoming_mesg[COMMS_NET_MAX_MESS_LEN];  /*!< The incoming message */
  size_t incoming_mesg_length;                 /*!< The current position in the incoming message */
  size_t last_examined_mesg_posn;              /*!< The last position we examined for the terminating characters */
  size_t start_of_mesg;                        /*!< Where the message begins (after STX, etc) */
  struct s_comms_control comms_ctrl;           /*!< The comms control structure for handling messages */
  struct timespec ts_last_rx_data;             /*!< Last time we received some data */
  bool is_udp;                                 /*!< UDP requires special handling */
  struct s_comms_udp_info udp_info;            /*!< If this is a UDP port, this is the local port and who we are communicating with. All members in host endian. */
};


/** Re-initialise parts of cn from the database settings */
void comms_net_reinit(struct s_comms_net *cn);


/** Create and initialise a new s_comms_net structure. udp_info in host endian. */
struct s_comms_net *comms_net_create(struct t_io_dev *serdev, struct s_comms_print_buffer *cpb, bool is_udp, const struct s_comms_udp_info *udp_info);


/** Clean up and destroy an existing s_comms_net structure */
void comms_net_destroy(struct s_comms_net *cn);


/** Does the work of listening and replying to network commands */
enum eCommsError comms_net_handler(struct s_comms_net *cn);


#endif /* COMMS_NET_H_ */
