/*
 * comms_params.h
 *
 *  Created on: 13/10/2010
 *      Author: ian
 */

#ifndef COMMS_PARAMS_H_
#define COMMS_PARAMS_H_

#include <setjmp.h>
#include <stdbool.h>
#include <stdint.h>
#include <stddef.h>


/** Maximum number of parameters, can't be more than 32 because of p_set & ps_set being t_comms_param_mask = uint32_t */
#define COMMS_PARAMS_MAX 20


/** Type for parameter set mask */
typedef uint32_t t_comms_param_mask;


/** Structure of parsed comms parameters */
struct s_comms_params
{
  t_comms_param_mask p_set;                     /*!< Bits set for each of the converted long parameters found */
  long params[COMMS_PARAMS_MAX];                /*!< Integer parameters */
  t_comms_param_mask ps_set;                    /*!< Bits set for each of the converted string parameters found */
  const char *param_str[COMMS_PARAMS_MAX];      /*!< String parameters */
  jmp_buf parse_error;                          /*!< Escape from parsing errors */
};


/**
 * Break string up into integer and string parameters. It modifies the string by replacing '"'s with '\0'.
 * longjmp()'s back to cp->parse_error if parsing error occurs
 *
 * @param cp is the comms parameters structure that the parameters are stored into
 * @param s is the string to be broken up into parameters. This gets modified, and must not be destroyed until
 *        cp is no longer required (cp points to elements in this string)
 */
void get_params(struct s_comms_params *cp, char *s);


/** Check string parameter */
bool param_str_ok(struct s_comms_params *cp, size_t p, size_t max_len, size_t min_len);


/** Check string parameter ensuring all characters fall within the character set */
bool param_str_charset_ok(struct s_comms_params *cp, size_t p, size_t max_len, size_t min_len, const char *character_set);


/** Check integer parameter */
bool param_ok(struct s_comms_params *cp, size_t p, long maxl, long minl);


#endif /* COMMS_PARAMS_H_ */
