/*
 * comms_print_auto.c
 *
 *  Created on: Mar 16, 2012
 *      Author: ian
 */
#include <rinweigh/adcinterface.h>
#include <rinweigh/adcdatabase.h>
#include "database.h"
#include "comms_print_trigger.h"
#include "traceprint.h"
#include "adcthread.h"
#include "iface_serial.h"
#include "app_db_user.h"

#include "comms_print_auto.h"


/** Check that the weight reading is ok to do an auto print */
static bool comms_print_auto_weight_ok(void)
{
  long l;
  bool result = false;

  if (!adcSuperStructure.adcDataWorking.dispRdg.info.status.bit.zeroBand)
  {
    /* Outside zero band */
    l = adcSuperStructure.adcDataWorking.dispRdg.rdg;
    if (l < 0)
      l = -l;

    /* Weight bigger than 10 divisions */
    result = (l > adcSuperStructure.adcUserDatabase.resolution[0] * 10);
  }
  return result;
}


/** Check if auto printing is enabled */
static bool comms_print_auto_enabled(void)
{
  switch (db_app_user.printer_mode)
  {
    case PA_SINGLE:
    case PA_DOUBLE:
    case PA_TIC:
    case PA_TOTAL:
      return true;
    default:
      return false;
  }
}


/** Run the auto printing state machine */
void comms_print_auto_check(struct s_comms_print_auto *cpa)
{
  switch (cpa->printer_state)
  {
    case apsWaitForZeroBand:
      if (adcSuperStructure.adcDataWorking.dispRdg.info.status.bit.zeroBand)
        cpa->printer_state = apsWaitForMotion;
      break;

    case apsWaitForMotion:
      if (adcSuperStructure.adcDataWorking.dispRdg.info.status.bit.motion)
        cpa->printer_state = apsWaitForStable;
      break;

    case apsWaitForStable:
      if (!adcSuperStructure.adcDataWorking.dispRdg.info.status.bit.motion)
        cpa->printer_state = apsCheckWeightOk;
      break;

    case apsCheckWeightOk:
      if (comms_print_auto_weight_ok() && comms_print_auto_enabled())
      {
        traceable_printout_initiate_standard_ui(false, false);
      }
      cpa->printer_state = apsWaitForZeroBand;
      break;

    default:
      cpa->printer_state = apsWaitForZeroBand;
      break;
  }
}


/** Initialise the auto printing */
void comms_print_auto_init(struct s_comms_print_auto *cpa)
{
  cpa->printer_state = apsWaitForZeroBand;
}
