/*
 * comms_print_trigger.c
 *
 *  Created on: 28/09/2011
 *      Author: ian
 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <pthread.h>
#include "comms_print.h"

#include "comms_print_init.h"


/** Re-initialise parts of cp from the database settings */
void comms_print_reinit(struct s_comms_print *cp)
{
  /* Nothing to be re-initalised from database */
}


/** Create and initialise a new s_comms_print structure */
struct s_comms_print *comms_print_create(struct t_io_dev *serdev, struct s_comms_print_buffer *cpb, bool is_incoming_udp)
{
  struct s_comms_print *cp;
  bool mutex_inited = false;
  bool sem_inited = false;

  /* Allocate new s_comms_print */
  cp = calloc(1, sizeof(struct s_comms_print));
  if (NULL == cp)
    goto cpc_fail;

  /* Initialise it */
  cp->serdev = serdev;
  cp->is_incoming_udp = is_incoming_udp;
  if (!comms_send_buffer_create(&cp->csb, 10) || NULL == cp->csb)
    goto cpc_fail;
  if (pthread_mutex_init(&cp->mutex, NULL) != 0)
    goto cpc_fail;
  mutex_inited = true;
  if (sem_init(&cp->trigger_sem, 0, 0) != 0)
    goto cpc_fail;
  sem_inited = true;
  cp->cpb = cpb;

  cp->potype = ptNone;
  cp->ptr_data = NULL;

  cp->print_finished_sem = NULL;
  cp->print_pos = 0;
  cp->prn_txt = 0;
  cp->last_print_weight = 0;
  cp->last_print_type = lpwtNone;
  return cp;

cpc_fail:
  if (NULL != cp)
  {
    if (NULL != cp->csb)
    {
      comms_send_buffer_destroy(cp->csb);
      cp->csb = NULL;
    }
    if (mutex_inited)
      pthread_mutex_destroy(&cp->mutex);
    if (sem_inited)
      sem_destroy(&cp->trigger_sem);
    free(cp);
    cp = NULL;
  }
  return NULL;
}


/** Clean up and destroy an existing s_comms_print structure */
void comms_print_destroy(struct s_comms_print *cp)
{
  if (NULL != cp)
  {
    comms_send_buffer_destroy(cp->csb);
    cp->csb = NULL;
    pthread_mutex_destroy(&cp->mutex);
    sem_destroy(&cp->trigger_sem);
    cp->print_finished_sem = NULL;
    /* Comms print buffer is not ours to destroy */
    cp->cpb = NULL;
    free(cp);
  }
}
