/*
 * comms_print_trigger.h
 *
 *  Created on: Mar 16, 2012
 *      Author: ian
 */

#ifndef COMMS_PRINT_TRIGGER_H_
#define COMMS_PRINT_TRIGGER_H_

#include <semaphore.h>
#include <rinhw/stdrinstrum.h>
#include <rinweigh/adcdatatypes.h>
#include "comms_print.h"


/** Determine if a printout is traceable */
bool comms_print_is_traceable(enum ePrintoutType potype);


/** Determine if a printout is initiated by the comms, rather than UI */
bool comms_print_is_comms_initiated(enum ePrintoutType potype);


/** Initiate a standard printout (traceable) */
bool comms_print_trigger_traceable_standard(const AdcTraceableWeight *atw, sem_t *print_finished_sem);


/** Initiate a total printout (non-traceable) */
bool comms_print_trigger_nontraceable_total(sem_t *print_finished_sem);


/** Initiate a memory minus printout (non-traceable) */
bool comms_print_trigger_nontraceable_memory_minus(void);


/** Initiate an abort printout (non-traceable) */
bool comms_print_trigger_nontraceable_abort(void);


/** Initiate a database configuration printout (non-traceable) */
bool comms_print_trigger_nontraceable_database_config(bool full);


/** Initiate a special remote format printout (traceable). s is a format string eg "\g". */
bool comms_print_trigger_traceable_special_remote(const char *s, const AdcTraceableWeight *atw, sem_t *print_finished_sem);


/** Initiate a raw data printout (non-traceable). s is null terminated raw data to be sent straight to the printer */
bool comms_print_trigger_nontraceable_raw_data(const char *s);


#endif /* COMMS_PRINT_TRIGGER_H_ */
