/*
 * comms_rincmd_support.c
 *
 *  Created on: 7 Apr 2016
 *      Author: rnduser
 */
#include <stdio.h>
#include <string.h>
#include <rinhw/kbd.h>
#include <rincmd/rc_public.h>
#include <rincmd/rc_streaming.h>
#include <rincmd/rc_parser.h>
#include <rinweigh/adcinterface.h>
#include <rinweigh/adcliveimpl_simple.h>

#include "adcthread.h"
#include "app_devicemount.h"
#include "setpoint.h"
#include "db_setpoint.h"
#include "iface.h"
#include "iface_file.h"
#include "iface_usb.h"
#include "kbd_5000.h"
#include "buzz5000.h"
#include "app_db_user.h"
#include "app_db_changelog.h"
#include "iface_menus.h"
#include "access.h"
#include "remote_cal.h"
#include "apperror.h"
#include "comms_hdlr_defaults.h"
#include "comms_hdlr_fpl.h"
#include "comms_hdlr_network.h"
#include "rc_gen_basic.h"
#include "reboot.h"
#include "app_save.h"

#include "comms_rincmd_support.h"


char rincmd_emptystring[2] = "";

#define CHECK_SINGLE_VAL(char1) ((val[0] == char1) && (len == 1 || (len == 2 && val[1] == '\0')))


/**
 * Calculate an int32_t value from the hexadecimal ASCII input string
 *
 * @param str String to convert
 * @param len Number of characters to convert. Must be <= 8 to fit in 32bits
 * @param value is a pointer to an unsigned long to set with the value of the conversion
 *
 * @return true if conversion succeeded, false otherwise
 */
bool rc_hexstrtoint32(const char *str, size_t len, int32_t *value)
{
  unsigned long l = 0;

  if (!rc_hexstrtoul(str, len, &l))
    return false;
  *value = (int32_t) l;
  return true;
}


bool rincmd_dummyFunction(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  return false;
}

#define DATEFORMAT_DDMMYY   0
#define DATEFORMAT_DDMMYYYY 1
#define DATEFORMAT_MMDDYY   2
#define DATEFORMAT_MMDDYYYY 3
#define DATEFORMAT_YYMMDD   4
#define DATEFORMAT_YYYYMMDD 5

t_datetime rincmd_dt;
uint8_t rincmd_dateformat = 0;

bool rincmd_FormatDate(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  if (opt == RC_FUNC_READ_POST)
  {
    size_t minSize = (len > RC_SIZE_BUFFER) ? RC_SIZE_BUFFER : len;

    /* Get current date and time */
    if (!GetDateTime(&rincmd_dt))
    {
      return false;
    }

    if (rincmd_dateformat == DATEFORMAT_DDMMYY)
      snprintf(val, minSize, "%02d/%02d/%02d %02d-%02d",
          rincmd_dt.date, rincmd_dt.month, rincmd_dt.year % 100,
          rincmd_dt.hour, rincmd_dt.minute);
    else if (rincmd_dateformat == DATEFORMAT_DDMMYYYY)
      snprintf(val, minSize, "%02d/%02d/%04d %02d-%02d",
          rincmd_dt.date, rincmd_dt.month, rincmd_dt.year,
          rincmd_dt.hour, rincmd_dt.minute);
    else if (rincmd_dateformat == DATEFORMAT_MMDDYY)
      snprintf(val, minSize, "%02d/%02d/%02d %02d-%02d",
          rincmd_dt.month, rincmd_dt.date, rincmd_dt.year % 100,
          rincmd_dt.hour, rincmd_dt.minute);
    else if (rincmd_dateformat == DATEFORMAT_MMDDYYYY)
      snprintf(val, minSize, "%02d/%02d/%04d %02d-%02d",
          rincmd_dt.month, rincmd_dt.date, rincmd_dt.year,
          rincmd_dt.hour, rincmd_dt.minute);
    else if (rincmd_dateformat == DATEFORMAT_YYMMDD)
      snprintf(val, minSize, "%02d/%02d/%02d %02d-%02d",
          rincmd_dt.year % 100, rincmd_dt.month, rincmd_dt.date,
          rincmd_dt.hour, rincmd_dt.minute);
    else /* If rincmd_dateformat == DATEFORMAT_YYYYMMDD */
      snprintf(val, minSize, "%04d/%02d/%02d %02d-%02d",
          rincmd_dt.year, rincmd_dt.month, rincmd_dt.date,
          rincmd_dt.hour, rincmd_dt.minute);
  }

  return false;
}


/** Default indicator settings */
bool rincmd_0007_default_settings(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  int32_t default_what_int;

  if (opt == RC_FUNC_EXEC)
  {
    if (!rc_hexstrtoint32(val, len, &default_what_int))
      return false;
    if (!comms_default_settings((enum e_comms_default_settings) default_what_int))
      return false;
  }
  return false;
}


/** Simulate a keypress for registers 0x0008 & 0x0324 */
static void rincmd_simulate_keypress(uint8_t regvalue, bool from_lua)
{
  bool longpress;
  uint8_t key;

  longpress = (regvalue & 0x80) != 0;
  key = regvalue & ~0x80;
  set_rincmd_comms_key(key, longpress, from_lua);
}


uint8_t rincmd_0008_key_buffer = KEY_PHYSICAL_NOKEY;
/** Write a key back to the indicator (original version, see also register 0324) */
bool rincmd_0008_key_buffer_fn(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  if (RC_FUNC_WRITE_POST == opt)
  {
    rincmd_simulate_keypress(rincmd_0008_key_buffer, false);
  }

  return false;
}


/** Read the LCD bitmap */
bool rincmd_0009_lcd_bitmap(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  if (opt == RC_FUNC_READ_POST)
  {
    size_t minSize = (len > RC_SIZE_BUFFER) ? RC_SIZE_BUFFER : len;
    (void) display_buffer_to_string(val, minSize, false);
  }
  return false;
}


/** Change the display mode (who has control of the LCD */
bool rincmd_000D_change_display_mode(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  if (opt == RC_FUNC_EXEC)
  {
    if (CHECK_SINGLE_VAL('0') || CHECK_SINGLE_VAL('1'))
      register_control_display_and_annunciators(false);
    else if (CHECK_SINGLE_VAL('2'))
      register_control_display_and_annunciators(true);
  }

  return false;
}


/** Save changes to settings to the databases */
bool rincmd_0010_save_settings(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  if (opt == RC_FUNC_EXEC)
  {
    (void) app_save_databases_and_reinit(true, &rincmd_840x_net_config);
    rincmd_840x_saved_settings();
  }
  return false;
}


/** All C520s are suitable for trade. This was a flag in the R400 factory database. Really just implemented for compatibility. */
uint8_t rincmd_0015_trade_enabled = 1;


/** Reboot the system */
bool rincmd_0016_system_reboot(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  if (opt == RC_FUNC_EXEC)
  {
    /* Start system reboot */
    system_reboot();
  }
  return false;
}


bool rincmd_RTC(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  if (opt == RC_FUNC_READ_PRE)
  {
    /* Get current date and time */
    if (!GetDateTime(&rincmd_dt))
    {
      return false;
    }
  }
  else if (opt == RC_FUNC_WRITE_POST)
  {
    SetDateTime(&rincmd_dt);
  }

  return false;
}

uint32_t rincmd_iostate = 0;
uint32_t rincmd_iomask = 0;

bool rincmd_IOStatus(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  if (opt == RC_FUNC_READ_PRE)
  {
    rincmd_iostate = (uint32_t) access_get_io_states();
  }
  else if (opt == RC_FUNC_WRITE_POST)
  {
    uint8_t i;
    bool val;

    for (i = 0; i < NUM_SETPOINTS; i++)
    {
      // if we are controlling the IO manually, set value
      if ((rincmd_iomask & (1L << i)) != 0)
      {
        val = (rincmd_iostate  & (1L << i)) != 0;
        direct_drive_output(&setpoint_running_info, i, val);
      }
    }
  }

  return false;
}

bool rincmd_IOMaskUpdate(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  if (opt == RC_FUNC_WRITE_POST)
  {
    uint8_t i;
    bool val;

    for (i = 0; i < NUM_SETPOINTS; i++)
    {
      // make sure we aren't still driving setpoints that have just been disabled
      if ((rincmd_iomask & (1L << i)) == 0)
        clear_direct_drive_output(&setpoint_running_info, i);
      // otherwise make sure the value is correct
      else
      {
        val = (rincmd_iostate  & (1L << i)) != 0;
        direct_drive_output(&setpoint_running_info, i, val);
      }
    }
  }
  return false;
}

uint32_t rincmd_setpointstate = 0;

bool rincmd_SetpointStatus(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  if (opt == RC_FUNC_READ_PRE)
  {
    size_t i;
    uint8_t setpState = 0;

    for (i = 0; i < NUM_SETPOINTS; i++)
      if (is_setpoint_output_active(&setpoint_running_info, i))
        setpState |= (1 << i);

    rincmd_setpointstate = (uint32_t) (((~setpState) << 16) | setpState);
  }

  return false;
}

#define RC_LS2_HIRES      0
#define RC_LS2_DISPMODE   (RC_LS2_HIRES+1)
#define RC_LS2_RANGE      (RC_LS2_DISPMODE+2)
#define RC_LS2_MENUACTIVE (RC_LS2_RANGE+2)
#define RC_LS2_PRODLOAD   (RC_LS2_MENUACTIVE+1)
#define RC_LS2_PRODSAVE   (RC_LS2_PRODLOAD+1)
#define RC_LS2_PWROFF     (RC_LS2_PRODSAVE+1)
#define RC_LS2_USB_NOTIFY (RC_LS2_PWROFF+1)
#define RC_LS2_INIT       (RC_LS2_USB_NOTIFY+15)
#define RC_LS2_RTC        (RC_LS2_INIT+1)
#define RC_LS2_WEIGHT     (RC_LS2_RTC+1)
#define RC_LS2_IO         (RC_LS2_WEIGHT+1)

uint32_t rincmd_0305_lua_status2 = 0;

bool rincmd_0305_fillout_lua_status2(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  t_datetime new_time;
  static uint8_t old_io = 0;
  uint8_t io = 0;
  static bool first_run = true;
  uint32_t ls2;
  AdcRdg displayed_rdg;

  if (opt == RC_FUNC_READ_PRE)
  {
    get_displayed_adcrdg(&displayed_rdg);
    ls2 = 0;

    /* Set Hi-res bit */
    if (displayed_rdg.info.status.bit.hiRes != 0)
      ls2 |= (1 << RC_LS2_HIRES);

    /* Set display mode */
    ls2 |= (adcSuperStructure.adcDataWorking.dispMode & 0x03) << RC_LS2_DISPMODE;

    /* Set range */
    ls2 |= (displayed_rdg.info.status.bit.range & 0x03) << RC_LS2_RANGE;

    /* Are menus active */
    if (edit_functions)
      ls2 |= (1 << RC_LS2_MENUACTIVE);

    /* PRODLOAD AND PRODSAVE are always 0 */

    /* PWROFF not implemented in C500 */

    /* USB Notification */
    ls2 |= ((usb_partition_notification & 0x01) << RC_LS2_USB_NOTIFY);

    /* TODO Add this in: Set init */

    /* Set rtc */
    if (rincmd_032A_lua_status2_rtc != 0)
    {
      /* toggle value each second */
      memset(&new_time, 0, sizeof(new_time));
      GetDateTime(&new_time);
      if ((new_time.second % 2) == 0)
        ls2 |= (1 << RC_LS2_RTC);
    }

    /* Set weight */
    if (rincmd_032B_lua_status2_weight_rdg != 0)
    {
      if ((adcSuperStructure.adcDataWorking.rdgId % rincmd_032B_lua_status2_weight_rdg) == 0)
        ls2 |= (1 << RC_LS2_WEIGHT);
    }

    /* Set IO. This should only be set high on change */
    io = access_get_io_states() & rincmd_032C_lua_status2_io_mask;
    if (first_run)
    {
      first_run = false;
    }
    else
    {
      /* Set io */
      if (io != old_io)
        ls2 |= (1 << RC_LS2_IO);
    }
    old_io = io;

    rincmd_0305_lua_status2 = ls2;
  }

  return false;
}

#define RC_LS_NET         0
#define RC_LS_GROSS       1
#define RC_LS_ZEROBAND    2
#define RC_LS_NOTZEROBAND 3
#define RC_LS_COZ         4
#define RC_LS_NOTCOZ      5
#define RC_LS_MOTION      6
#define RC_LS_STABLE      7
#define RC_LS_LORANGE     8
#define RC_LS_HIRANGE     9
#define RC_LS_PTARE      10
#define RC_LS_NOTPTARE   11
#define RC_LS_ERROR      12
#define RC_LS_ULOAD      13
#define RC_LS_OLOAD      14
#define RC_LS_NOERROR    15
#define RC_LS_HELD       16
#define RC_LS_NOTHELD    17

uint32_t rincmd_0329_lua_status = 0;

bool rincmd_0329_fillout_lua_status(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  AdcRdg displayed_rdg;
  uint32_t ls;

  if (opt == RC_FUNC_READ_PRE)
  {
    get_displayed_adcrdg(&displayed_rdg);
    ls = 0;

    /* Set gross and net bits */
    if (displayed_rdg.info.status.bit.grossOrNet != 0)
    {
      if (displayed_rdg.info.status.bit.gross != 0)
        ls |= (1 << RC_LS_GROSS);
      else
        ls |= (1 << RC_LS_NET);
    }

    /* Set zero bits using zero band */
    if (displayed_rdg.info.status.bit.zeroBand != 0)
      ls |= (1 << RC_LS_ZEROBAND);
    else
      ls |= (1 << RC_LS_NOTZEROBAND);

    /* Set COZ bits */
    if (displayed_rdg.info.status.bit.coz != 0)
      ls |= (1 << RC_LS_COZ);
    else
      ls |= (1 << RC_LS_NOTCOZ);

    /* Set motion bits */
    if (displayed_rdg.info.status.bit.motion != 0)
      ls |= (1 << RC_LS_MOTION);
    else
      ls |= (1 << RC_LS_STABLE);

    /* Set range bits */
    if (displayed_rdg.info.status.bit.range == 0)
      ls |= (1 << RC_LS_LORANGE);

    if (displayed_rdg.info.status.bit.range == 1)
      ls |= (1 << RC_LS_HIRANGE);

    /* Set preset tare bits */
    if (0 != adcSuperStructure.adcDataWorking.presetTare.rdg)
      ls |= (1 << RC_LS_PTARE);
    else
      ls |= (1 << RC_LS_NOTPTARE);

    /* Set Error bits */
    if (get_current_errors() != APP_ERR_NONE)
      ls |= (1 << RC_LS_ERROR);
    else
      ls |= (1 << RC_LS_NOERROR);

    /* Set underload bit */
    if (displayed_rdg.info.status.bit.uload != 0)
      ls |= (1 << RC_LS_ULOAD);

    /* Set overload bit */
    if (displayed_rdg.info.status.bit.oload != 0)
      ls |= (1 << RC_LS_OLOAD);

    /* Set held bits */
    if (displayed_rdg.info.status.bit.held != 0)
      ls |= (1 << RC_LS_HELD);
    else
      ls |= (1 << RC_LS_NOTHELD);

    rincmd_0329_lua_status = ls;
  }

  return false;
}

uint8_t rincmd_032A_lua_status2_rtc = 0;
uint8_t rincmd_032B_lua_status2_weight_rdg = 0;
uint32_t rincmd_032C_lua_status2_io_mask = 0;

char rincmd_dispBuf[DISP_BUF_SIZE] = {0};
uint8_t rincmd_dispUnits = ADC_UNITS_NONE;
uint32_t rincmd_dispAnnuns = 0;
uint8_t rincmd_dispAutoAnnun = false;

bool rincmd_writeTopLeft(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  if (opt == RC_FUNC_WRITE_POST)
  {
    write_register_display_buf((const char *) rincmd_dispBuf);
  }

  return false;
}

bool rincmd_writeUnits(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  if (opt == RC_FUNC_WRITE_POST)
  {
    write_register_display_units((enum eAdcUnits)rincmd_dispUnits);
  }

  return false;
}

bool rincmd_writeAnnuns(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  if (opt == RC_FUNC_WRITE_POST)
  {
    write_register_display_annuns(rincmd_dispAnnuns, rincmd_dispAutoAnnun);
  }

  return false;
}

bool rincmd_regInstrumentBegin(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  if (opt == RC_FUNC_EXEC)
  {
    /* Disable streaming for the topLevel */
    rc_resetStreaming(topLevel);

    /* Clear the lua displayed string */
    write_register_display_buf("");
    write_register_display_units(ADC_UNITS_NONE);

    /* Flush the keys */
    rincmd_0322_flush_key_fn(NULL, 0, 0, topLevel, NULL, RC_FUNC_EXEC);

    /* Remove the C500 applications event and keyboard */
    iface_usb_set_notifications_status(false);
    we2111_device_mounter_dismount_lua_devices();
    we2111_device_mounter_remove_lua_devices();
  }

  return false;
}

uint32_t rincmd_fullscale = 0;

bool rincmd_getFullscale(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  if (opt == RC_FUNC_READ_PRE)
  {
    rincmd_fullscale = (uint32_t) adcSuperStructure.adcUserDatabase.fullScale[(adcSuperStructure.adcUserDatabase.scaleType+1)/2];
  }

  return false;
}

uint8_t rincmd_heartbeat = 0;

bool rincmd_updateHeartbeat(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  if (opt == RC_FUNC_EXEC)
  {
    register_control_display_and_annunciators(false);
    set_rincmd_key_handler(kh_application);
    rincmd_heartbeat = 0;
  }

  return false;
}

uint8_t rincmd_0321_get_key = KEY_PHYSICAL_NOKEY;
bool rincmd_0321_get_key_fn(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  if (opt == RC_FUNC_READ_PRE)
  {
    rincmd_0321_get_key = get_rincmd_r400_key();
  }

  return false;
}

bool rincmd_0322_flush_key_fn(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  if (opt == RC_FUNC_EXEC)
  {
    while (get_rincmd_r400_key() != KEY_PHYSICAL_NOKEY)
      ;
  }

  return false;
}

uint8_t rincmd_0324_write_key = KEY_PHYSICAL_NOKEY;
/** Write a key back to the indicator (Lua version, see also register 0008) */
bool rincmd_0324_write_key_fn(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  if (RC_FUNC_WRITE_POST == opt)
  {
    rincmd_simulate_keypress(rincmd_0324_write_key, true);
  }

  return false;
}

uint8_t rincmd_0325_key_handler = kh_application;
bool rincmd_0325_update_key_handler_fn(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  if (opt == RC_FUNC_READ_PRE)
  {
    rincmd_0325_key_handler = (uint8_t) get_rincmd_key_handler();
  }
  if (opt == RC_FUNC_WRITE_POST)
  {
    set_rincmd_key_handler((enum e_key_handler) rincmd_0325_key_handler);
  }

  return false;
}

uint32_t rincmd_primary_units_dp_resolutions = 0;
uint32_t rincmd_secondary_units_dp_resolutions = 0;
uint32_t rincmd_tertiary_units_dp_resolutions = 0;

#define RC_UNIT_DP_RES_SET_FIELD(s, f, mask, shifts) \
  s = (s & ~(mask)) | (((f) << (shifts)) & (mask))

/** Bits 0-3 are the decimal point position */
#define RC_UNIT_DP_RES_SET_DP(s, dp) \
  RC_UNIT_DP_RES_SET_FIELD(s, dp, 0x0000000F, 0)

/** Bits 4-7 are the units */
#define RC_UNIT_DP_RES_SET_UNITS(s, units) \
  RC_UNIT_DP_RES_SET_FIELD(s, units, 0x000000F0, 4)

/** Bits 8-15, 16-23, 24-31 are the res3, res2, res1 (yes they are backwards) */
#define RC_UNIT_DP_RES_SET_RES(s, idx, res) \
  RC_UNIT_DP_RES_SET_FIELD(s, res, 0x000000FF << ((ADC_MAX_NUM_INTERVAL - idx) * 8), ((ADC_MAX_NUM_INTERVAL - idx) * 8))


static void rincmd_calc_altunits_resolutions(uint32_t *units_dp_res_ptr, const int32s alt_res[ADC_MAX_NUM_INTERVAL])
{
  size_t interval_idx;
  enum eAdcResolutions res_idx;

  /* Search for value in the Res array */
  for (interval_idx = 0; interval_idx < ADC_MAX_NUM_INTERVAL; interval_idx++)
  {
    for (res_idx = 0; res_idx < ADC_NUM_RESOLUTION; res_idx++)
    {
      if (AdcResolutions[res_idx] == alt_res[interval_idx])
        break;
    }
    RC_UNIT_DP_RES_SET_RES(*units_dp_res_ptr, interval_idx, res_idx);
  }
}


bool rincmd_update_units_dp_resolutions(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  size_t i;
  uint32_t units_dp_res = 0;
  uint32_t *rc_units_dp_res_ptr;
  AdcInfo *info_ptr;

  if (RC_FUNC_READ_PRE == opt)
  {
    switch (reg->addr)
    {
      case RINREG_BASIC_PRIMARY_UNITS_DP_RESOLUTIONS:
        info_ptr = &adcSuperStructure.adcDataWorking.dispRdg.info;
        rc_units_dp_res_ptr = &rincmd_primary_units_dp_resolutions;

        /* Straight copy for primary units */
        for (i = 0; i < ADC_MAX_NUM_INTERVAL; i++)
        {
          RC_UNIT_DP_RES_SET_RES(units_dp_res, i, adcSuperStructure.adcUserDatabase.resolution[i]);
        }
        break;

      case RINREG_BASIC_SECONDARY_UNITS_DP_RESOLUTIONS:
        info_ptr = &adcSuperStructure.adcDataWorking.altDispRdg.info;
        rc_units_dp_res_ptr = &rincmd_secondary_units_dp_resolutions;

        /* Search for value in the resolutions array */
        rincmd_calc_altunits_resolutions(&units_dp_res, adcSuperStructure.adcInternalData.altRes);
        break;

      case RINREG_BASIC_TERTIARY_UNITS_DP_RESOLUTIONS:
        info_ptr = &adcSuperStructure.adcDataWorking.tertDispRdg.info;
        rc_units_dp_res_ptr = &rincmd_tertiary_units_dp_resolutions;

        /* Search for value in the resolutions array */
        rincmd_calc_altunits_resolutions(&units_dp_res, adcSuperStructure.adcInternalData.tertRes);
        break;

      default:
        return false;
        break;
    }

    /* Set the units and DP */
    RC_UNIT_DP_RES_SET_DP(units_dp_res, info_ptr->unitdp.bit.dp);
    RC_UNIT_DP_RES_SET_UNITS(units_dp_res, info_ptr->unitdp.bit.units);

    /* Set the appropriate rincmd value */
    *rc_units_dp_res_ptr = units_dp_res;
  }

  return false;
}

uint8_t rincmd_buzzLen = 0;
uint8_t rincmd_buzzNum = 0;

bool rincmd_Buzz(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  uint16_t beepTime;

  if (opt == RC_FUNC_WRITE_POST)
  {
    if (rincmd_buzzNum)
    {
      switch (rincmd_buzzLen)
      {
        case 1:
          beepTime = 250;
          break;

        case 2:
          beepTime = 500;
          break;

        case 0:
        default:
          beepTime = BEEP_TIME;
          break;
      }
      buzz_times(beepTime, BEEP_WAIT, BUZZ_NORMAL, rincmd_buzzNum);
      rincmd_buzzNum = 0;
    }
  }
  return false;
}

int32_t rincmd_analogue = 0;

bool rincmd_updateAnalogue(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  if (opt == RC_FUNC_WRITE_POST)
  {
    access_set_register_analogue(rincmd_analogue);
  }

  if (opt == RC_FUNC_READ_PRE)
  {
    rincmd_analogue = acces_get_register_analogue();
  }

  return false;
}

bool rincmd_returnLong(char *val, size_t maxLen, int32_t res)
{
  // send ASCII hex string if there is room in the buffer
  if (maxLen >= 9)
  {
    sprintf(val, "%08X", res);
    return true;
  }

  return false;
}

bool rincmd_adcZero(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  uint8_t res;

  if (opt == RC_FUNC_EXEC)
  {
    res = AdcSemiautoZero(&adcSuperStructure, true, true, false, false);
    // send result
    return rincmd_returnLong(val, maxLen, (int32_t)res);
  }

  return false;
}

bool rincmd_adcTare(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  uint8_t res;

  if (opt == RC_FUNC_EXEC)
  {
    res = AdcSemiautoTare(&adcSuperStructure, true, true, false, false);
    // send result
    return rincmd_returnLong(val, maxLen, (int32_t)res);
  }

  return false;
}

bool rincmd_adcPTare(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  int32_t pTare;
  int8_t res;

  // convert val string to pTare int32_t
  if (opt == RC_FUNC_EXEC)
  {
    if (!rc_hexstrtoint32(val, len, &pTare))
      return false;

    // check if value is greater than 0 and less than full scale
    if (pTare >= 0 && pTare <= adcSuperStructure.adcUserDatabase.fullScale[(adcSuperStructure.adcUserDatabase.scaleType + 1) / 2])
    {
      res = AdcSetPTare(&adcSuperStructure, true, true, false, pTare);
      // send result
      return rincmd_returnLong(val, maxLen, (int32_t)res);
    }
  }

  return false;
}

bool rincmd_adcGrossNet(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  int8_t res;

  if (opt == RC_FUNC_EXEC)
  {
    if ((len == 1 && val[0] == '1') || (len >= 2 && val[0] == '1' && val[1] == 0))       // set to gross
      res = AdcToGross(&adcSuperStructure, true, true);
    else if ((len == 1 && val[0] == '2') || (len >= 2 && val[0] == '2' && val[1] == 0))  // set to net
      res = AdcToNet(&adcSuperStructure, true, true);
    else                                                                                 // all other values just toggle
      res = AdcGrossNetToggle(&adcSuperStructure, true, true);

    // send result
    return rincmd_returnLong(val, maxLen, (int32_t)res);
  }

  return false;
}

bool rincmd_adcHiRes(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  if (opt == RC_FUNC_EXEC)
  {
    if ((len == 1 && val[0] == '1') || (len >= 2 && val[0] == '1' && val[1] == 0))       // set to high res
      adcSuperStructure.adcRuntimeDatabase.hiRes = true;
    else if ((len == 1 && val[0] == '2') || (len >= 2 && val[0] == '2' && val[1] == 0))  // set to normal res
      adcSuperStructure.adcRuntimeDatabase.hiRes = false;
    else if ((len == 1 && val[0] == '3') || (len >= 2 && val[0] == '3' && val[1] == 0))  // set to DB setting
      adcSuperStructure.adcRuntimeDatabase.hiRes = adcSuperStructure.adcUserDatabase.hiRes;
    else                                                                                 // all other values just toggle
      adcSuperStructure.adcRuntimeDatabase.hiRes = ! adcSuperStructure.adcRuntimeDatabase.hiRes;

    // send current state
    return rincmd_returnLong(val, maxLen, (int32_t)adcSuperStructure.adcRuntimeDatabase.hiRes);
  }

  return false;
}

int32_t rincmd_calWeight = 0;

bool rincmd_calZero(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  if (opt == RC_FUNC_EXEC)
  {
    // zero cal
    if (get_remote_cal_mode() == REMOTE_CAL_MODE_IDLE)
      setup_for_zero_cal();
  }

  return false;
}

bool rincmd_calSpan(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  if (opt == RC_FUNC_EXEC)
  {
    // span cal
    if ((get_remote_cal_mode() == REMOTE_CAL_MODE_IDLE) && (rincmd_calWeight >= 100) &&
        (rincmd_calWeight <= adcSuperStructure.adcUserDatabase.fullScale[(adcSuperStructure.adcUserDatabase.scaleType+1)/2]))
    {
      adcSuperStructure.adcUserDatabase.calWeight = rincmd_calWeight;
      setup_for_span_cal();
    }
  }

  return false;
}

static bool rincmd_hexstr_to_linpt(const char *val, size_t len, int32_t *linpt)
{
  return (rc_hexstrtoint32(val, len, linpt) && *linpt >= 0 && *linpt < ADC_MAX_NUM_LIN_PTS);
}

bool rincmd_edLin(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  int32_t pt;

  if (opt == RC_FUNC_EXEC)
  {
    if (!rincmd_hexstr_to_linpt(val, len, &pt))
      return false;
    // clear point if weight is zero
    if (rincmd_calWeight == 0)
    {
      AdcClearLinPnt(&adcSuperStructure, true, false, pt);
    }
    // edit lin point
    if (rincmd_calWeight >= 100 &&
        rincmd_calWeight <= adcSuperStructure.adcUserDatabase.fullScale[(adcSuperStructure.adcUserDatabase.scaleType + 1) / 2])
    {
      AdcAddLinPnt(&adcSuperStructure, true, true, pt, rincmd_calWeight);
    }
  }

  return false;
}

bool rincmd_clrLin(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  int32_t pt;

  if (opt == RC_FUNC_EXEC)
  {
    if (!rincmd_hexstr_to_linpt(val, len, &pt))
      return false;
    // clear lin point
    AdcClearLinPnt(&adcSuperStructure, true, false, pt);
  }

  return false;
}

bool rincmd_dirZero(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  int32_t mvv;

  if (opt == RC_FUNC_EXEC)
  {
    // get mV/V value
    if (!rc_hexstrtoint32(val, len, &mvv))
      return false;
    // direct zero cal
    if (get_remote_cal_mode() == REMOTE_CAL_MODE_IDLE &&
        mvv >= ADC_RAW_TO_DUVV(APP_DB_U_ADCZERORAW_MIN) &&
        mvv <= ADC_RAW_TO_DUVV(APP_DB_U_ADCZERORAW_MAX))
    {
      setup_for_zero_direct_cal(mvv);
    }
  }

  return false;
}

bool rincmd_dirSpan(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  int32_t mvv;

  if (opt == RC_FUNC_EXEC)
  {
    // get mV/V value
    if (!rc_hexstrtoint32(val, len, &mvv))
      return false;
    // direct span cal
    if (get_remote_cal_mode() == REMOTE_CAL_MODE_IDLE &&
        mvv >= ADC_RAW_TO_DUVV(APP_DB_U_ADCSPANRAW_MIN) &&
        mvv <= ADC_RAW_TO_DUVV(APP_DB_U_ADCSPANRAW_MAX))
    {
      setup_for_span_direct_cal(mvv);
    }
  }

  return false;
}

int32_t rincmd_zeroMvv = 0;
int32_t rincmd_spanMvv = 0;
int32_t rincmd_spanWeight = 0;

bool rincmd_updateCalValues(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  if (opt == RC_FUNC_READ_PRE)
  {
    // zero mV/V
    if (reg->addr == RINREG_BASIC_ZERO_MVV_VALUE)
    {
      rincmd_zeroMvv = ADC_RAW_TO_DUVV_ROUNDED(adcSuperStructure.adcUserDatabase.zeroRaw);
    }
    // span mV/V
    if (reg->addr == RINREG_BASIC_SPAN_MVV_VALUE)
    {
      rincmd_spanMvv = ADC_RAW_TO_DUVV_ROUNDED(adcSuperStructure.adcUserDatabase.spanRaw);
    }
    // span weight
    if (reg->addr == RINREG_BASIC_SPAN_WEIGHT)
    {
      if (adcSuperStructure.adcUserDatabase.spanCnt > 0)
        rincmd_spanWeight = (adcSuperStructure.adcUserDatabase.spanCnt + 8) / 16;
      else
        rincmd_spanWeight = (adcSuperStructure.adcUserDatabase.spanCnt - 8) / 16;
    }
  }

  return false;
}

bool rincmd_getLinWeight(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  int32_t pt;
  int32_t weight;

  if (opt == RC_FUNC_EXEC)
  {
    // get linearisation point
    if (!rincmd_hexstr_to_linpt(val, len, &pt))
      return false;
    weight = adcSuperStructure.adcUserDatabase.linCor[pt];
    weight = AdcCountsToFinal(&adcSuperStructure, false, weight, 0, 0);
    // send result
    return rincmd_returnLong(val, maxLen, weight);
  }

  return false;
}

bool rincmd_getLinPC(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  int32_t pt;
  uint8_t index;
  int32_t pc;

  if (opt == RC_FUNC_EXEC)
  {
    // get linearisation point
    if (!rincmd_hexstr_to_linpt(val, len, &pt))
      return false;
    // get point at fullscale (fullscale is always +ve so -128 will work)
    index = (int8u)(AdcGetLinPt(&adcSuperStructure, false, AdcFinalToCounts(&adcSuperStructure, false, adcSuperStructure.adcUserDatabase.fullScale[(adcSuperStructure.adcUserDatabase.scaleType+1)/2],FALSE)) - 128);

    // calculate pc of fullscale
    pc = ((200 * (adcSuperStructure.adcUserDatabase.linPts[pt] - 128) / index) + 1) / 2;

    // send result
    return rincmd_returnLong(val, maxLen, pc);
  }

  return false;
}

bool rincmd_ejectUSB(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  if (opt == RC_FUNC_EXEC)
    we2111_unmount_usbdisk();

  return false;
}

bool rincmd_USBNotification(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  if (opt == RC_FUNC_READ_POST)
  {
    strcpy(val, usb_partition_notification_text);

    /* Clear the status bit. Leave the notification text in case this has to be
     * read on startup. */
    /* memset(usb_partition_notification_text, 0, USB_PARTITION_NOTIFICATION_TEXT_SIZE); */
    usb_partition_notification = false;
  }

  return false;
}

/* bits 0 - 7 are for calibration error, bit 8 is reserved (unused) */
#define RC_SS_NET          9
#define RC_SS_ZEROBAND    10
#define RC_SS_COZ         11
#define RC_SS_MOTION      12
#define RC_SS_CAL_IN_PROG 13
#define RC_SS_MENU_ACTIVE 14
#define RC_SS_ERROR       15
#define RC_SS_ULOAD       16
#define RC_SS_OLOAD       17
#define RC_SS_ADC_SUCCESS 18
#define RC_SS_ADC_IN_PROG 19
#define RC_SS_TILT_HI_XY  20
#define RC_SS_TILT_HI_Y   21
#define RC_SS_TILT_HI_X   22

uint32_t rincmd_0021_system_status = 0;

bool rincmd_0021_get_system_status(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  AdcRdg displayed_rdg;
  uint32_t rss;
  uint8_t adcopstatus;

  if (opt == RC_FUNC_READ_PRE)
  {
    get_displayed_adcrdg(&displayed_rdg);
    rss = 0;

    /* Set calibration error bits */
    rss |= get_last_cal_output();

    /* Set net bit */
    if (displayed_rdg.info.status.bit.grossOrNet != 0 && displayed_rdg.info.status.bit.gross == 0)
      rss |= (1 << RC_SS_NET);

    /* Set zero band bit */
    if (displayed_rdg.info.status.bit.zeroBand != 0)
      rss |= (1 << RC_SS_ZEROBAND);

    /* Set COZ bit */
    if (displayed_rdg.info.status.bit.coz != 0)
      rss |= (1 << RC_SS_COZ);

    /* Set motion bit */
    if (displayed_rdg.info.status.bit.motion != 0)
      rss |= (1 << RC_SS_MOTION);

    /* Set calibration in progress bit */
    if (get_remote_cal_mode() != REMOTE_CAL_MODE_IDLE)
      rss |= (1 << RC_SS_CAL_IN_PROG);

    /* Set menu active bit */
    if (edit_functions)
      rss |= (1 << RC_SS_MENU_ACTIVE);

    /* Set Error bits */
    if (get_current_errors() != APP_ERR_NONE)
      rss |= (1 << RC_SS_ERROR);

    /* Set underload bit */
    if (displayed_rdg.info.status.bit.uload != 0)
      rss |= (1 << RC_SS_ULOAD);

    /* Set overload bit */
    if (displayed_rdg.info.status.bit.oload != 0)
      rss |= (1 << RC_SS_OLOAD);

    /* Set last ADC operation successful bit */
    adcopstatus = AdcGetCurrentOpStatus(&adcSuperStructure, false);
    if (adcopstatus == ADC_OP_STATUS_SUCCESS)
      rss |= (1 << RC_SS_ADC_SUCCESS);

    /* Set ADC operation in progress bit */
    if (adcopstatus == ADC_OP_STATUS_IN_PROGRESS)
      rss |= (1 << RC_SS_ADC_IN_PROG);

    /* TODO Set tilt high bits when tilt compensation implemented */

    rincmd_0021_system_status = rss;
  }

  return false;
}

uint32_t rincmd_0022_system_error = 0;

bool rincmd_0022_get_sys_error(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  if (opt == RC_FUNC_READ_PRE)
  {
    rincmd_0022_system_error = get_current_errors();
  }

  return false;
}

int32_t rincmd_pulseCount = 0;

bool rincmd_getPulseCount(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  enum ePulseCount pulse;

  if (opt == RC_FUNC_READ_PRE)
  {
    /* start at pulse 0 */
    pulse = (enum ePulseCount) reg->addr - RINREG_BASIC_PULSE_COUNT_1;
    rincmd_pulseCount = access_get_pulses(pulse);
  }

  return false;
}

bool rincmd_clearPulseCount(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  unsigned long pulse;

  if (opt == RC_FUNC_EXEC)
  {
    if (!rc_hexstrtoul(val, len, &pulse))
      return false;
    access_clear_pulses((enum ePulseCount) pulse);
  }

  return false;
}

#define INIT_IO 1

bool rincmd_init(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt)
{
  unsigned long mask;

  if (opt == RC_FUNC_EXEC)
  {
    if (!rc_hexstrtoul(val, len, &mask))
      return false;
    if (mask & INIT_IO)
    {
      init_setpoints(&setpoint_running_info, &adcSuperStructure);
      access_dbconfigure_allio();
    }
  }

  return false;
}
