/*
 * comms_rincmd_support.h
 *
 *  Created on: 7 Apr 2016
 *      Author: rnduser
 */

#ifndef COMMS_RINCMD_SUPPORT_H_
#define COMMS_RINCMD_SUPPORT_H_

#include <stdint.h>
#include <stdbool.h>
#include <rincmd/rc_public.h>

#include "display.h"
#include "sysclock.h"

/* Read from this if planning on overwriting the data with a post-read function */
extern char rincmd_emptystring[2];

/* Function that returns true and does nothing else */
bool rincmd_dummyFunction(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);

/* RTC Register variables and functions */
extern t_datetime rincmd_dt;
extern uint8_t rincmd_dateformat;
bool rincmd_FormatDate(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);
bool rincmd_RTC(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);

/** Default indicator settings */
bool rincmd_0007_default_settings(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);

extern uint8_t rincmd_0008_key_buffer;
/** Write a key back to the indicator (original version, see also register 0324) */
bool rincmd_0008_key_buffer_fn(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);

/** Read the LCD bitmap */
bool rincmd_0009_lcd_bitmap(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);

/** Change the display mode (who has control of the LCD */
bool rincmd_000D_change_display_mode(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);

/** Save changes to settings to the databases */
bool rincmd_0010_save_settings(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);

/** All C520s are suitable for trade. This was a flag in the R400 factory database. Really just implemented for compatibility. */
extern uint8_t rincmd_0015_trade_enabled;

/** Reboot the system */
bool rincmd_0016_system_reboot(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);

/* IO Status variable */
extern uint32_t rincmd_iostate;
extern uint32_t rincmd_iomask;
bool rincmd_IOStatus(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);
bool rincmd_IOMaskUpdate(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);

/* Setpoint status variables and functions */
extern uint32_t rincmd_setpointstate;
bool rincmd_SetpointStatus(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);

/* Second lua status register */
extern uint32_t rincmd_0305_lua_status2;
bool rincmd_0305_fillout_lua_status2(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);

/* First lua status register */
extern uint32_t rincmd_0329_lua_status;
bool rincmd_0329_fillout_lua_status(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);

/* Enable bits within second lua status register */
extern uint8_t rincmd_032A_lua_status2_rtc;
extern uint8_t rincmd_032B_lua_status2_weight_rdg;
extern uint32_t rincmd_032C_lua_status2_io_mask;

/* Write to the screen */
extern char rincmd_dispBuf[DISP_BUF_SIZE];
extern uint8_t rincmd_dispUnits;
extern uint32_t rincmd_dispAnnuns;
extern uint8_t rincmd_dispAutoAnnun;
bool rincmd_writeTopLeft(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);
bool rincmd_writeUnits(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);
bool rincmd_writeAnnuns(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);

/* Set up the instrument for use with lua */
bool rincmd_regInstrumentBegin(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);

/* Scale type variable */
extern uint32_t rincmd_fullscale;
bool rincmd_getFullscale(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);

/* Handle the heartbeat */
extern uint8_t rincmd_heartbeat;
bool rincmd_updateHeartbeat(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);

/* Key handling variables and functions */
extern uint8_t rincmd_0321_get_key;
bool rincmd_0321_get_key_fn(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);
bool rincmd_0322_flush_key_fn(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);
extern uint8_t rincmd_0324_write_key;
/** Write a key back to the indicator (Lua version, see also register 0008) */
bool rincmd_0324_write_key_fn(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);
extern uint8_t rincmd_0325_key_handler;
bool rincmd_0325_update_key_handler_fn(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);

/*
 * Units, decimal point and resolutions:
 * Bits are as follows:
 *    0-3:   Decimal point
 *    4-7:   Units
 *    8-15:  Resolution 3
 *    16-23: Resolution 2
 *    23-31: Resolution 1
 */
extern uint32_t rincmd_primary_units_dp_resolutions;
extern uint32_t rincmd_secondary_units_dp_resolutions;
extern uint32_t rincmd_tertiary_units_dp_resolutions;
bool rincmd_update_units_dp_resolutions(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);

/* Buzzer support */
extern uint8_t rincmd_buzzLen;
extern uint8_t rincmd_buzzNum;
bool rincmd_Buzz(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);

/* Analogue support */
extern int32_t rincmd_analogue;

bool rincmd_updateAnalogue(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);

/* ADC functions */
bool rincmd_adcZero(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);
bool rincmd_adcTare(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);
bool rincmd_adcPTare(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);
bool rincmd_adcGrossNet(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);
bool rincmd_adcHiRes(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);

/* Calibration functions and variables */
extern int32_t rincmd_calWeight;
extern int32_t rincmd_zeroMvv;
extern int32_t rincmd_spanMvv;
extern int32_t rincmd_spanWeight;
bool rincmd_calZero(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);
bool rincmd_calSpan(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);
bool rincmd_edLin(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);
bool rincmd_clrLin(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);
bool rincmd_dirZero(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);
bool rincmd_dirSpan(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);
bool rincmd_updateCalValues(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);
bool rincmd_getLinWeight(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);
bool rincmd_getLinPC(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);

/* USB functions */
bool rincmd_ejectUSB(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);
bool rincmd_USBNotification(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);

/* system status */
extern uint32_t rincmd_0021_system_status;
bool rincmd_0021_get_system_status(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);

/* system errors */
extern uint32_t rincmd_0022_system_error;
bool rincmd_0022_get_sys_error(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);

/* Pulse counting */
extern int32_t rincmd_pulseCount;
bool rincmd_getPulseCount(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);
bool rincmd_clearPulseCount(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);

/* Re-init function */
bool rincmd_init(char *val, size_t len, size_t maxLen, struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);

#endif /* COMMS_RINCMD_SUPPORT_H_ */
