/*
 * comms_send_buffer.h
 *
 *  Created on: Feb 28, 2017
 *      Author: ian
 */

#ifndef COMMS_SEND_BUFFER_H_
#define COMMS_SEND_BUFFER_H_

#include <stddef.h>
#include <stdbool.h>
#include <stdarg.h>


/**
 * Buffer for holding comms output data.
 *
 * This is so we can send it in a single write for
 * performance improvements over the rinblock bus.
 */
struct s_comms_send_buffer
{
  char *buffer;             /*!< The character buffer */
  size_t alloc_increments;  /*!< Whenever we resize, we allocate  */
  size_t allocated_size;    /*!< The currently allocated size of the buffer */
  size_t used_size;         /*!< The number of used bytes in the buffer */
};


/** Create a comms send buffer */
bool comms_send_buffer_create(struct s_comms_send_buffer **csb, size_t alloc_increments);


/** Destroy a comms send buffer */
bool comms_send_buffer_destroy(struct s_comms_send_buffer *csb);


/** Clear the data from the comms send buffer (but doesn't free anything) */
bool comms_send_buffer_clear(struct s_comms_send_buffer *csb);


/** Get the comms send buffer pointer */
bool comms_send_buffer_get_buffer(struct s_comms_send_buffer *csb, const void **buffer, size_t *length);


/** Write a buffer to the comms send buffer */
bool comms_send_buffer_addbuf(struct s_comms_send_buffer *csb, const void *buf, size_t length);


/** Write a character to the comms send buffer */
bool comms_send_buffer_addch(struct s_comms_send_buffer *csb, char ch);


/** vprintf into a comms send buffer */
int comms_send_buffer_vprintf(struct s_comms_send_buffer *csb, const char *format, va_list args);


/* printf into a comms send buffer */
int comms_send_buffer_printf(struct s_comms_send_buffer *csb, const char *format, ...);


#endif /* COMMS_SEND_BUFFER_H_ */
