/*
 * comms_task.h
 *
 *  Created on: 21/10/2010
 *      Author: ian
 */

#ifndef COMMS_TASK_H_
#define COMMS_TASK_H_

#include <stdbool.h>
#include <netinet/in.h>
#include <rinhw/rio.h>
#include "comms_print.h"
#include "comms_udp.h"
#include "serial_ports.h"


/** Get serial port name as a string */
const char *get_serial_port_as_string(enum eSerialPort serialport);


/** Determine if port is a TCP connection */
bool comms_is_tcp_connection(enum eSerialPort serialport);


/** Determine if port is a UDS (unix domain socket) connection */
bool comms_is_uds_connection(enum eSerialPort serialport);


/** Determine if port is a stream connection (TCP or UDS) */
bool comms_is_stream_connection(enum eSerialPort serialport);


/** Determine if port is an incoming UDP connection */
bool comms_is_udp_in_connection(enum eSerialPort serialport);


/** Determine if port is an outgoing UDP connection */
bool comms_is_udp_out_connection(enum eSerialPort serialport);


/** Determine if port is a UDP connection */
bool comms_is_udp_connection(enum eSerialPort serialport);


/** Create a new comms handler thread for a serial port. udp info in host endian. */
int comms_create_thread_for_serial_port(struct t_io_dev *port, enum eSerialPort serialport, const struct s_comms_udp_info *udpinfo);


/** Kill a specific serial port task */
void comms_destroy_task(struct t_io_dev *port);


/** Kill all serial port tasks */
void comms_tasks_destroy(void);


/**
 * Force comms port(s) to reconfigure from database settings
 *
 * @param all set true if you want all ports to be re-configured, false if a specific port
 * @param port is the specific port to configure if all is false
 */
void comms_reconfigure(bool all, enum eSerialPort port);


/** Force comms port to trigger an automatic output, if it is configured as one */
void comms_auto_trigger(enum eSerialPort port);


/** Get the s_comms_print structure for a port (locks it first - must call comms_print_release_cp() when done) */
struct s_comms_print *comms_print_get_cp(struct t_io_dev *port);


/** Release a s_comms_print structure acquired by comms_print_get_cp() */
void comms_print_release_cp(struct s_comms_print *cp);


/**
 * Create any tasks for the serial ports. Call only once.
 *
 * @param ports is a list of serial ports to create tasks for. Last entry must be NULL
 * @param types is a list of the correspond serial port types.
 */
void comms_tasks_create(struct t_io_dev **ports, const enum eSerialPort *types);


/**
 * Run through the list of comms threads, and clean any terminated network threads.
 * If we don't do this, we'll end up with lots of zombie threads hanging around.
 */
void comms_task_list_clean(void);


/** Initialise the comms task list. Call before any other comms_ functions. Returns false if failed */
bool comms_task_list_init(const char *uds_net_file, const char *uds_auto_file);


/** Shutdown the comms task list. Call after everything else is shut down */
void comms_task_list_destroy(void);


/** Get the serial port type for a running serial port task */
enum eSerialPortType comms_task_list_get_port_type(enum eSerialPort port);


/** Return the serial port of the specified serial port device */
enum eSerialPort comms_task_list_get_port_from_device(const struct t_io_dev *const dev);


#endif /* COMMS_TASK_H_ */
