/*
 * comms_udp.c
 *
 *  Created on: Oct 12, 2017
 *      Author: ian
 */
#include <unistd.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <rinhw/rio.h>
#include <rinhw/rioctl.h>
#include <rinhw/sio.h>

#include "comms_udp.h"


/**
 * Do special read handling for UDP incoming connections
 * that only do transmits (autoout or print)
 *
 * We do a non-blocking recvfrom() here, so we can
 * get the sender ip:port and set this as the reply-to address
 */
void comms_udp_read_for_txonly(struct t_io_dev *serdev)
{
  int fd;
  ssize_t retval;
  socklen_t sockaddr_size;
  char dummy;
  struct sockaddr_in sender;

  /* Try to get linux file descriptor */
  fd = -1;
  if (rioctl(serdev, IOCTL_GET_LINUX_FD, &fd) < 0 || fd < 0)
  {
    return;
  }

  /* Read any available to get the source without waiting */
  sockaddr_size = sizeof(sender);
  retval = recvfrom(fd, &dummy, sizeof(dummy), MSG_DONTWAIT,
                    (struct sockaddr *) &sender, &sockaddr_size);
  if (retval < 0)
  {
    return;
  }

  /* Need to let rindevice know where to send the data back to */
  sender.sin_addr.s_addr = ntohl(sender.sin_addr.s_addr);
  sender.sin_port = ntohs(sender.sin_port);
  rioctl(serdev, IOCTL_SER_UDP_SET_DEST_SOCKADDR, &sender);
}
