#ifndef DATABASE_DOT_H
#define DATABASE_DOT_H 1

#include <stddef.h>
#include <stdint.h>
#include <limits.h>
#include "db_serial.h"
#include "db_remote.h"
#include "db_setpoint.h"
#include "db_build.h"
#include "db_option.h"
#include "db_special.h"


/** Total number of graduations permitted by OIML approval in single range mode */
#define OIML_SINGLE 10000L


/** Total number of graduations permitted by OIML approval in multi range mode */
#define OIML_MULTI  5000L


/** Possible OIML check errors */
enum eOIMLError { OIML_ERROR_NONE = 0,
                  OIML_ERROR_VERIFICATION_INTERVAL,
                  OIML_ERROR_GRADUATIONS,
                  OIML_ERROR_NO_UNITS,
                  OIML_ERROR_MOTION_NONE,
                  OIML_ERROR_ZERO_TRACK,
                  OIML_ERROR_ZERO_RANGE,
                  OIML_ERROR_ZERO_BAND,
                  OIML_ERROR_KEY_IMMED,
                  OIML_ERROR_DIRECT_MVV,
                  OIML_ERROR_NUM
};


/** Check for any OIML errors, run on exiting setup */
enum eOIMLError check_oiml(void);


/* the following routines provide derived values from the database.
**************************************************************************/
long true_full_scale(void);
/* returns the fullscale weight depending on multi-range mode setting.
**************************************************************************/
long min_span_value(void);
/* returns 2% of true fullscale.
**************************************************************************/
bool is_single_range(void);
/* returns true if scale type is DB_SERIAL_TYPE_SINGLE range
**************************************************************************/


bool is_weight_fmt(const char *s);
/* returns true if there is anything in the weight format string that involves
** weight.
**************************************************************************/


#endif /* #ifndef DATABASE_DOT_H */
