/*
 * db_build.c
 *
 *  Created on: 15/04/2011
 *      Author: ian
 */
#include <rinweigh/adcdatatypes.h>

#include "db_build.h"


/** Units strings used in the menus */
const char *const build_units_menus[NUM_UNITS] = BUILD_UNITS_MENU_STRS;


/** Units strings used for printouts */
const char *const build_units_print[NUM_UNITS] = BUILD_UNITS_PRINT_STRS;


/** Decimal point strings */
const char *const build_decimal_point[NUM_DP] = DECIMAL_POINT_STRS;


/** ADC scale type strings */
const char *const build_adc_scale_type_strs[ADC_NUM_SCALE_TYPE] = BUILD_ADC_SCALE_TYPE_STRS;


/** Sync cal settings strings */
const char *const sync_cal_setting_strs[ADC_NUM_SYNC_FREQ] = SYNC_CAL_SETTING_STRS;


/** Convert ADC_SCALE_TYPE_* into WE2111 WMD command scale type setting */
enum eWMDBuildType xlat_adc_library_scale_type_to_wmd(enum eAdcScaleType adc_scale_type)
{
  enum eWMDBuildType wmd_build_type;

  switch (adc_scale_type)
  {
    case ADC_SCALE_TYPE_SINGLE:
    default:
      wmd_build_type = SINGLE_RANGE;
      break;
    case ADC_SCALE_TYPE_DUAL_INTERVAL:
      wmd_build_type = DUAL_INTERVAL;
      break;
    case ADC_SCALE_TYPE_DUAL_RANGE:
      wmd_build_type = DUAL_RANGE;
      break;
  }
  return wmd_build_type;
}


/** Convert WE2111 WMD scale type into ADC library ADC_SCALE_TYPE_* setting */
enum eAdcScaleType xlat_wmd_scale_type_to_adc_library(enum eWMDBuildType wmd_scale_type)
{
  enum eAdcScaleType adc_scale_type;

  switch (wmd_scale_type)
  {
    case SINGLE_RANGE:
    default:
      adc_scale_type = ADC_SCALE_TYPE_SINGLE;
      break;
    case DUAL_INTERVAL:
      adc_scale_type = ADC_SCALE_TYPE_DUAL_INTERVAL;
      break;
    case DUAL_RANGE:
      adc_scale_type = ADC_SCALE_TYPE_DUAL_RANGE;
      break;
  }
  return adc_scale_type;
}


/** Convert ADC_UNITS_* into WE2111 UNITS_* setting */
enum eBuildUnits xlat_adc_library_units_to_we2111(enum eAdcUnits adc_units)
{
  enum eBuildUnits we_units;

  switch (adc_units)
  {
    default:
    case ADC_UNITS_NONE:
      we_units = UNITS_NONE;
      break;
    case ADC_UNITS_GRAMS:
      we_units = UNITS_GRAMS;
      break;
    case ADC_UNITS_KILOGRAMS:
      we_units = UNITS_KILOGRAMS;
      break;
    case ADC_UNITS_POUNDS:
      we_units = UNITS_POUNDS;
      break;
    case ADC_UNITS_TONNES:
      we_units = UNITS_TONNES;
      break;
    case ADC_UNITS_OUNCES:
      we_units = UNITS_OUNCES;
      break;
    case ADC_UNITS_USER0:
      we_units = UNITS_USER;
      break;
  }
  return we_units;
}


/** Convert WE2111 UNITS_* into ADC library ADC_UNITS_* setting */
enum eAdcUnits xlat_we2111_units_to_adc_library(enum eBuildUnits we_units)
{
  enum eAdcUnits adc_units;

  switch (we_units)
  {
    default:
    case UNITS_NONE:
      adc_units = ADC_UNITS_NONE;
      break;
    case UNITS_GRAMS:
      adc_units = ADC_UNITS_GRAMS;
      break;
    case UNITS_KILOGRAMS:
      adc_units = ADC_UNITS_KILOGRAMS;
      break;
    case UNITS_POUNDS:
      adc_units = ADC_UNITS_POUNDS;
      break;
    case UNITS_TONNES:
      adc_units = ADC_UNITS_TONNES;
      break;
    case UNITS_OUNCES:
      adc_units = ADC_UNITS_OUNCES;
      break;
    case UNITS_USER:
      adc_units = ADC_UNITS_USER0;
      break;
  }
  return adc_units;
}


/** Convert a sync frequency into the (nearest) ADC library sync frequency index. sync_freq in 0.1Hz (125=12.5Hz) */
void xlat_sync_freq_to_adc_library_sync_freq_idx(uint16_t sync_freq, int8u *adc_sync_idx)
{
  int i;

  if (sync_freq >= AdcSyncFrequencies[ADC_NUM_SYNC_FREQ - 1])
  {
    *adc_sync_idx = ADC_NUM_SYNC_FREQ - 1;
  }
  else
  {
    *adc_sync_idx = 0;
    for (i = 0; i < ADC_NUM_SYNC_FREQ - 1; i++)
    {
      if (sync_freq <= AdcSyncFrequencies[i])
      {
        *adc_sync_idx = i;
        break;
      }
    }
  }
}
