/*
 * db_build.h
 *
 *  Created on: 15/04/2011
 *      Author: ian
 */
#ifndef DB_BUILD_H_
#define DB_BUILD_H_

#include <rinweigh/adcdatatypes.h>


/** Resolution options */
/* #define NUM_RES_OPTS 7 Not required because RESOLUTION_MAX is used */


/** Minimum possible resolution */
#define RESOLUTION_MIN 1

/** Maximum possible resolution */
#define RESOLUTION_MAX 100


/** Build types maintained for backward compatibility of WMD command */
enum eWMDBuildType { SINGLE_RANGE = 0,
                     DUAL_RANGE,
                     DUAL_INTERVAL,
                     NUM_BUILD_TYPE
                   };


/** Build units */
enum eBuildUnits { UNITS_NONE = 0,
                   UNITS_GRAMS,
                   UNITS_KILOGRAMS,
                   UNITS_POUNDS,
                   UNITS_TONNES,
                   UNITS_OUNCES,
                   UNITS_USER,
                   NUM_UNITS
                 };


/** Build unit strings, must match @ref enum eBuildUnits */
#define BUILD_UNITS_PRINT_STRS { "__", \
                                 " g", \
                                 "kg", \
                                 "lb", \
                                 " t", \
                                 "oz", \
                                 "  "  \
                               }


/** Menu build unit strings, must match @ref enum eBuildUnits */
#define BUILD_UNITS_MENU_STRS { "NONE", \
                                "G",    \
                                "KG",   \
                                "LB",   \
                                "T",    \
                                "OZ",   \
                                "USER"  \
                               }


/** Units strings used in the menus */
extern const char *const build_units_menus[NUM_UNITS];


/** Units strings used for printouts */
extern const char *const build_units_print[NUM_UNITS];


/** Decimal point settings */
enum eBuildDecimalPoint { DP_NONE = 0,
                          DP_ONE,
                          DP_TWO,
                          DP_THREE,
                          DP_FOUR,
                          DP_FIVE,
                          NUM_DP
                        };


/** Decimal point strings, must match @ref enum eBuildDecimalPoint */
#define DECIMAL_POINT_STRS { "000000", \
                             "00000.0", \
                             "0000.00", \
                             "000.000", \
                             "00.0000", \
                             "0.00000" \
                           }


/** Decimal point strings */
extern const char *const build_decimal_point[NUM_DP];


/** Application number of scale types, we don't support triple range */
#define APP_NUM_SCALE_TYPE (ADC_SCALE_TYPE_DUAL_RANGE + 1)


/** ADC scale type strings, must match @ref enum eAdcScaleType */
#define BUILD_ADC_SCALE_TYPE_STRS { "SINGLE", \
                                    "DUAL I", \
                                    "DUAL R", \
                                    "TRIP I", \
                                    "TRIP R" \
                                  }


/** ADC scale type strings */
extern const char *const build_adc_scale_type_strs[ADC_NUM_SCALE_TYPE];


/** Sync cal settings strings, must match @ref enum eAdcSyncFrequency */
#define SYNC_CAL_SETTING_STRS { "10", \
                                "12.5", \
                                "15", \
                                "20", \
                                "25", \
                                "30", \
                                "50", \
                                "60", \
                                "100" \
                              }


/** Sync cal settings strings */
extern const char *const sync_cal_setting_strs[ADC_NUM_SYNC_FREQ];


/** Convert WE2111 WMD scale type into ADC library ADC_SCALE_TYPE_* setting */
enum eAdcScaleType xlat_wmd_scale_type_to_adc_library(enum eWMDBuildType wmd_scale_type);


/** Convert ADC_SCALE_TYPE_* into WE2111 WMD command scale type setting */
enum eWMDBuildType xlat_adc_library_scale_type_to_wmd(enum eAdcScaleType adc_scale_type);


/** Convert ADC_UNITS_* into WE2111 UNITS_* setting */
enum eBuildUnits xlat_adc_library_units_to_we2111(enum eAdcUnits adc_units);


/** Convert WE2111 UNITS_* into ADC library ADC_UNITS_* setting */
enum eAdcUnits xlat_we2111_units_to_adc_library(enum eBuildUnits we_units);


/** Convert a sync frequency into the (nearest) ADC library sync frequency index. sync_freq in 0.1Hz (125=12.5Hz) */
void xlat_sync_freq_to_adc_library_sync_freq_idx(uint16_t sync_freq, int8u *adc_sync_idx);


#endif /* DB_BUILD_H_ */
