/*
 * db_option.c
 *
 *  Created on: 10/05/2011
 *      Author: ian
 */
#include "db_option.h"


/** WE2111 approval mode strings */
const char *const we_approval_mode_strings[WE_APPROVAL_MODE_NUM] = WE_APPROVAL_MODE_STRS;


/** ADC library approval mode strings */
const char *const adc_approval_mode_strings[ADC_NUM_APPROVAL_MODE] = ADC_APPROVAL_MODE_STRS;


/** Averaging options lookup table */
const uint8_t average_options[NUM_AVERAGE_OPT] = AVERAGE_OPT_VALUES;


/** Averaging option strings */
const char *const average_setting[NUM_AVERAGE_OPT] = AVERAGE_OPT_STRS;


/** Jitter setting strings */
const char *const jitter_setting[ADC_NUM_JITTER] = JITTER_OPT_STRS;


/** Motion detection settings strings */
const char *const motion_setting[ADC_NUM_MOTION_DETECT] = MOTION_DETECT_STRS;


/** Zero range settings strings */
const char *const we21_zero_range_setting[WE21_NUM_ZERO_RANGE] = WE21_ZERO_RANGE_STRS;


/** Zero tracking settings strings */
const char *const zero_tracking_setting[ADC_NUM_ZERO_TRACKING] = ZERO_TRACKING_STRS;


/** Convert from we21 zero ranges to ADC library zero ranges */
static const uint8_t we21_adc_zr_xlat[WE21_NUM_ZERO_RANGE] =
{
  ADC_ZERO_RANGE_NEG20_TO_PLUS20,
  ADC_ZERO_RANGE_NEG100_TO_PLUS100,
  ADC_ZERO_RANGE_NEG2_TO_PLUS2,
  ADC_ZERO_RANGE_NEG1_TO_PLUS3
};


/** Convert from ADC library zero ranges to we21 zero ranges */
static const uint8_t adc_we21_zr_xlat[ADC_NUM_ZERO_RANGE] =
{
  WE21_ZERO_RANGE_NEG2_TO_PLUS2,
  WE21_ZERO_RANGE_NEG1_TO_PLUS3,
  DEFAULT_ZERO_RANGE_OPT,
  WE21_ZERO_RANGE_NEG20_TO_PLUS20,
  WE21_ZERO_RANGE_NEG100_TO_PLUS100
};


/** Translate WE2111 averaging setting into adcUserDatabase.average */
int16s xlat_we2111_average_to_adc_library(enum eAverageOptions av_idx)
{
  if (av_idx >= NUM_AVERAGE_OPT)
  {
    av_idx = DEFAULT_AVERAGE_OPT;
  }
  return average_options[av_idx];
}


/** Translate adcUserDatabase.average into WE2111 averaging setting */
enum eAverageOptions xlat_adc_library_average_to_we2111(int16s averaging)
{
  enum eAverageOptions i;
  enum eAverageOptions av_idx = DEFAULT_AVERAGE_OPT;

  for (i = 0; i < NUM_AVERAGE_OPT; i++)
  {
    if (averaging <= average_options[i])
    {
      av_idx = i;
      break;
    }
  }
  return av_idx;
}


/** Translate WE2111 zero range setting into adcUserDatabase.rangeOfZero */
uint8_t xlat_we2111_zero_range_to_adc_library(uint8_t we21_zr_idx)
{
  if (we21_zr_idx >= WE21_NUM_ZERO_RANGE)
  {
    we21_zr_idx = DEFAULT_ZERO_RANGE_OPT;
  }

  return we21_adc_zr_xlat[we21_zr_idx];
}


/** Translate adcUserDatabase.rangeOfZero into WE2111 zero range setting */
uint8_t xlat_adc_library_zero_range_to_we2111(uint8_t adc_zr_idx)
{
  if (adc_zr_idx >= ADC_NUM_ZERO_RANGE)
  {
    return DEFAULT_ZERO_RANGE_OPT;
  }
  else
  {
    return adc_we21_zr_xlat[adc_zr_idx];
  }
}


/** Convert WE2111 WE_APPROVAL_MODE_* into ADC library ADC_APPROVAL_MODE_* setting */
enum eAdcApprovalMode xlat_we2111_approval_mode_to_adc_library(enum eWeApprovalMode we_approval_mode)
{
  enum eAdcApprovalMode adc_approval_mode;

  switch (we_approval_mode)
  {
    default:
    case WE_APPROVAL_MODE_TRADE:
      adc_approval_mode = ADC_APPROVAL_MODE_OIML;
      break;
    case WE_APPROVAL_MODE_INDUST:
      adc_approval_mode = ADC_APPROVAL_MODE_NONE;
      break;
    case WE_APPROVAL_MODE_NTEP:
      adc_approval_mode = ADC_APPROVAL_MODE_NTEP;
      break;
  }
  return adc_approval_mode;
}


/** Convert ADC library ADC_APPROVAL_MODE_* to WE2111 WE_APPROVAL_MODE_* setting */
enum eWeApprovalMode xlat_adc_library_approval_mode_to_we2111(enum eAdcApprovalMode adc_approval_mode)
{
  enum eWeApprovalMode we_approval_mode;

  switch (adc_approval_mode)
  {
    default:
    case ADC_APPROVAL_MODE_OIML:
      we_approval_mode = WE_APPROVAL_MODE_TRADE;
      break;
    case ADC_APPROVAL_MODE_NONE:
      we_approval_mode = WE_APPROVAL_MODE_INDUST;
      break;
    case ADC_APPROVAL_MODE_NTEP:
      we_approval_mode = WE_APPROVAL_MODE_NTEP;
      break;
  }
  return we_approval_mode;
}
