/*
 * db_option.h
 *
 *  Created on: 10/05/2011
 *      Author: ian
 */

#ifndef DB_OPTION_H_
#define DB_OPTION_H_

#include <stdint.h>
#include <rinweigh/adcdatatypes.h>


/** WE weighing approval modes, a subset of @ref enum eAdcApprovalMode, and also in a different order */
enum eWeApprovalMode { WE_APPROVAL_MODE_TRADE = 0,
                       WE_APPROVAL_MODE_INDUST,
                       WE_APPROVAL_MODE_NTEP,
                       WE_APPROVAL_MODE_NUM
};


/** WE weighing approval mode strings, must match WE_APPROVAL_MODE_* constants */
#define WE_APPROVAL_MODE_STRS { "OIML",        /* WE_APPROVAL_MODE_TRADE */ \
                                "INDUST",      /* WE_APPROVAL_MODE_INDUST */ \
                                "NTEP"         /* WE_APPROVAL_MODE_NTEP */ \
                              }

/** WE2111 approval mode strings */
extern const char *const we_approval_mode_strings[WE_APPROVAL_MODE_NUM];


/** ADC approval mode strings, must match @ref enum eAdcApprovalMode */
#define ADC_APPROVAL_MODE_STRS { "INDUST", \
                                 "OIML", \
                                 "NTEP", \
                                 "NSC" \
                                  }


/** ADC library approval mode strings */
extern const char *const adc_approval_mode_strings[ADC_NUM_APPROVAL_MODE];


/** Averaging options */
enum eAverageOptions { AVERAGE_OPT_1 = 0,
                       AVERAGE_OPT_2,
                       AVERAGE_OPT_3,
                       AVERAGE_OPT_4,
                       AVERAGE_OPT_5,
                       AVERAGE_OPT_6,
                       AVERAGE_OPT_7,
                       AVERAGE_OPT_8,
                       AVERAGE_OPT_9,
                       AVERAGE_OPT_10,
                       AVERAGE_OPT_25,
                       AVERAGE_OPT_50,
                       AVERAGE_OPT_75,
                       AVERAGE_OPT_100,
                       AVERAGE_OPT_200,
                       NUM_AVERAGE_OPT
                     };


/** Default setting for averaging */
#define DEFAULT_AVERAGE_OPT AVERAGE_OPT_10


/** Lookup table for averaging options. Must match @ref enum eAverageOptions */
#define AVERAGE_OPT_VALUES { 1, \
                             2, \
                             3, \
                             4, \
                             5, \
                             6, \
                             7, \
                             8, \
                             9, \
                             10, \
                             25, \
                             50, \
                             75, \
                             100, \
                             200 \
                           }


/** Averaging options strings, must match @ref enum eAverageOptions */
#define AVERAGE_OPT_STRS { "1", \
                           "2", \
                           "3", \
                           "4", \
                           "5", \
                           "6", \
                           "7", \
                           "8", \
                           "9", \
                           "10", \
                           "25", \
                           "50", \
                           "75", \
                           "100", \
                           "200" \
                         }


/** Averaging options lookup table */
extern const uint8_t average_options[NUM_AVERAGE_OPT];


/** Averaging option strings */
extern const char *const average_setting[NUM_AVERAGE_OPT];


/** Jitter settings strings, must match ADC_JITTER_* constants */
#define JITTER_OPT_STRS { "OFF", \
                          "FINE", \
                          "COARSE" \
                        }


/** Jitter setting strings */
extern const char *const jitter_setting[ADC_NUM_JITTER];


/** Motion detection settings strings, must match ADC_MOTION_DETECT_* constants */
#define MOTION_DETECT_STRS { "NONE", \
                             "0.5-1.0", \
                             "1.0-1.0", \
                             "2.0-1.0", \
                             "5.0-1.0", \
                             "0.5-0.5", \
                             "1.0-0.5", \
                             "2.0-0.5", \
                             "5.0-0.5", \
                             "0.5-0.2", \
                             "1.0-0.2", \
                             "2.0-0.2", \
                             "5.0-0.2", \
                             "3.0-1.0", \
                             "3.0-0.5", \
                             "3.0-0.2" \
                           }


/** Motion detection settings strings */
extern const char *const motion_setting[ADC_NUM_MOTION_DETECT];


/** Zero ranges on the WE2111, these differ from the adc library, so translation is required */
enum eWE21ZeroRanges { WE21_ZERO_RANGE_NEG20_TO_PLUS20 = 0,
                       WE21_ZERO_RANGE_NEG100_TO_PLUS100,
                       WE21_ZERO_RANGE_NEG2_TO_PLUS2,
                       WE21_ZERO_RANGE_NEG1_TO_PLUS3,
                       WE21_NUM_ZERO_RANGE
                     };


/** Zero range option strings, must match @ref enum eWE21ZeroRanges */
#define WE21_ZERO_RANGE_STRS { "20-20", \
                               "100.100", \
                               "02-02", \
                               "01-03" \
                             }


/** Zero range settings strings */
extern const char *const we21_zero_range_setting[WE21_NUM_ZERO_RANGE];


/** Zero tracking option strings, must match @ref enum eAdcZeroTracking */
#define ZERO_TRACKING_STRS { "NONE", \
                             "0.5-1.0", \
                             "1.0-1.0", \
                             "2.0-1.0", \
                             "5.0-1.0", \
                             "0.5-0.5", \
                             "1.0-0.5", \
                             "2.0-0.5", \
                             "5.0-0.5", \
                             "0.5-0.2", \
                             "1.0-0.2", \
                             "2.0-0.2", \
                             "5.0-0.2", \
                             "3.0-1.0", \
                             "3.0-0.5", \
                             "3.0-0.2" \
                           }


/** Zero tracking settings strings */
extern const char *const zero_tracking_setting[ADC_NUM_ZERO_TRACKING];


/** Default setting for zero range */
#define DEFAULT_ZERO_RANGE_OPT WE21_ZERO_RANGE_NEG2_TO_PLUS2


/** Translate WE2111 averaging setting into adcUserDatabase.average */
int16s xlat_we2111_average_to_adc_library(enum eAverageOptions av_idx);


/** Translate adcUserDatabase.average into WE2111 averaging setting */
enum eAverageOptions xlat_adc_library_average_to_we2111(int16s averaging);


/** Translate WE2111 zero range setting into adcUserDatabase.rangeOfZero */
uint8_t xlat_we2111_zero_range_to_adc_library(uint8_t we21_zr_idx);


/** Translate adcUserDatabase.rangeOfZero into WE2111 zero range setting */
uint8_t xlat_adc_library_zero_range_to_we2111(uint8_t adc_zr_idx);


/** Convert WE2111 WE_APPROVAL_MODE_* into ADC library ADC_APPROVAL_MODE_* setting */
enum eAdcApprovalMode xlat_we2111_approval_mode_to_adc_library(enum eWeApprovalMode we_approval_mode);


/** Convert ADC library ADC_APPROVAL_MODE_* to WE2111 WE_APPROVAL_MODE_* setting */
enum eWeApprovalMode xlat_adc_library_approval_mode_to_we2111(enum eAdcApprovalMode adc_approval_mode);


#endif /* DB_OPTION_H_ */
