/*
 * db_serial_ip.c
 *
 *  Created on: Sep 27, 2017
 *      Author: ian
 */
#include "app_db_user.h"

#include "db_serial_ip.h"


/** Get destination IP address for fixed UDP ports. Always host endian. */
void db_get_serial_dest_ipaddress(enum eSerialPort port, struct in_addr *ip_address)
{
  switch (port)
  {
    case SERPORT_UDPOUT1:
      ip_address->s_addr = db_app_user.serial_udpo1_dstaddr;
      break;
    case SERPORT_UDPOUT2:
      ip_address->s_addr = db_app_user.serial_udpo2_dstaddr;
      break;
    default:
      ip_address->s_addr = 0;
      break;
  }
}


/** Set destination IP addresses for fixed UDP ports. Always host endian. */
void db_set_serial_dest_ipaddress(enum eSerialPort port, const struct in_addr *ip_address)
{
  switch (port)
  {
    case SERPORT_UDPOUT1:
      db_app_user.serial_udpo1_dstaddr = ip_address->s_addr;
      break;
    case SERPORT_UDPOUT2:
      db_app_user.serial_udpo2_dstaddr = ip_address->s_addr;
      break;
    default:
      break;
  }
}


/**
 * Get TCP/UDP port numbers for a serial port from the database
 * Always host endian
 *
 * For outgoing UDP port ports, this is the destination port
 * For TCP and incoming UDP ports, this is our listening port
 *
 * Any other ports return 0
 */
int db_get_serial_tcpport(enum eSerialPort port)
{
  int tcpudpport;

  switch (port)
  {
    case SERPORT_UDPOUT1:
      tcpudpport = db_app_user.serial_udpo1_port;
      break;
    case SERPORT_UDPOUT2:
      tcpudpport = db_app_user.serial_udpo2_port;
      break;
    case SERPORT_UDPIN1:
      tcpudpport = db_app_user.serial_udpi1_port;
      break;
    case SERPORT_UDPIN2:
      tcpudpport = db_app_user.serial_udpi2_port;
      break;
    case SERPORT_TCPNET:
      tcpudpport = db_app_user.serial_tcpi1_port;
      break;
    case SERPORT_TCPAUTO:
      tcpudpport = db_app_user.serial_tcpi2_port;
      break;
    default:
      tcpudpport = 0;
      break;
  }
  return tcpudpport;
}


/**
 * Set TCP/UDP port numbers for a serial port in the database
 * Always host endian
 *
 * For outgoing UDP port ports, this is the destination port
 * For TCP and incoming UDP ports, this is our listening port
 *
 * Any other ports return 0
 */
void db_set_serial_tcpport(enum eSerialPort port, int tcpudpport)
{
  switch (port)
  {
    case SERPORT_UDPOUT1:
      db_app_user.serial_udpo1_port = tcpudpport;
      break;
    case SERPORT_UDPOUT2:
      db_app_user.serial_udpo2_port = tcpudpport;
      break;
    case SERPORT_UDPIN1:
      db_app_user.serial_udpi1_port = tcpudpport;
      break;
    case SERPORT_UDPIN2:
      db_app_user.serial_udpi2_port = tcpudpport;
      break;
    case SERPORT_TCPNET:
      db_app_user.serial_tcpi1_port = tcpudpport;
      break;
    case SERPORT_TCPAUTO:
      db_app_user.serial_tcpi2_port = tcpudpport;
      break;
    default:
      break;
  }
}


/** Get TCP port timeouts from the database (in seconds) */
uint32_t db_get_serial_tcp_timeout(enum eSerialPort port)
{
  uint32_t timeout;

  switch (port)
  {
    case SERPORT_TCPNET:
      timeout = db_app_user.serial_tcpi1_timeout;
      break;
    case SERPORT_TCPAUTO:
      timeout = db_app_user.serial_tcpi2_timeout;
      break;
    default:
      timeout = 0;
      break;
  }
  return timeout;
}


/** Set TCP port timeouts for a serial port database */
void db_set_serial_tcp_timeout(enum eSerialPort port, uint32_t timeout)
{
  switch (port)
  {
    case SERPORT_TCPNET:
      db_app_user.serial_tcpi1_timeout = timeout;
      break;
    case SERPORT_TCPAUTO:
      db_app_user.serial_tcpi2_timeout = timeout;
      break;
    default:
      break;
  }
}


/** Function to get the configuration data for rinmodbus */
void db_rinmodbus_getconfiguration(in_port_t *mb_tcp_port, enum eRinModbusEndian *mb_endian)
{
  *mb_tcp_port = db_app_user.modbus_tcpport;
  *mb_endian = (enum eRinModbusEndian) db_app_user.modbus_endian;
}

