/*
 * db_serial_ip.h
 *
 *  Created on: Sep 27, 2017
 *      Author: ian
 */

#ifndef DB_SERIAL_IP_H_
#define DB_SERIAL_IP_H_

#include <stdint.h>
#include <netinet/in.h>
#include <setnetconfig.h>
#include "ipinfo.h"
#include "serial_ports.h"
#include "modbus_config.h"


/* Serial TCP port limits */
#define DB_SERIAL_TCPPORT_LOW_LIMIT 1024
#define DB_SERIAL_TCPPORT_HIGH_LIMIT 65535


/* Serial UDP input and output port limits */
#define DB_SERIAL_UDP_PORT_LOW_LIMIT 0
#define DB_SERIAL_UDP_PORT_HIGH_LIMIT 65535
#define DB_SERIAL_DEFAULT_UDPO_PORT 0 /* Disabled */
#define DB_SERIAL_DEFAULT_UDPI_PORT 0 /* Disabled */


/* Serial TCP default ports */
#define DB_SERIAL_DEFAULT_TCPI1_PORT 2222
#define DB_SERIAL_DEFAULT_TCPI2_PORT 2223


/* Serial TCP and UDP timeout limits */
#define DB_SERIAL_TCPTIMEOUT_LOW_LIMIT 0
#define DB_SERIAL_TCPTIMEOUT_HIGH_LIMIT (60 * 60 * 24)
#define DB_SERIAL_DEFAULT_TCPTIMEOUT 0


/* Serial UDP output destination IP */
#define DB_SERIAL_DEFAULT_UDPO_DEST SNC_IPV4_OCTETS_TO_INADDR_HOST(0, 0, 0, 0)


/** Get destination IP address for fixed UDP ports. Always host endian. */
void db_get_serial_dest_ipaddress(enum eSerialPort port, struct in_addr *ip_address);


/** Set destination IP addresses for fixed UDP ports. Always host endian. */
void db_set_serial_dest_ipaddress(enum eSerialPort port, const struct in_addr *ip_address);


/**
 * Get TCP/UDP port numbers for a serial port from the database
 * Always host endian
 *
 * For outgoing UDP port ports, this is the destination port
 * For TCP and incoming UDP ports, this is our listening port
 *
 * Any other ports return 0
 */
int db_get_serial_tcpport(enum eSerialPort port);


/**
 * Set TCP/UDP port numbers for a serial port in the database
 * Always host endian
 *
 * For outgoing UDP port ports, this is the destination port
 * For TCP and incoming UDP ports, this is our listening port
 *
 * Any other ports return 0
 */
void db_set_serial_tcpport(enum eSerialPort port, int tcpudpport);


/** Get TCP port timeouts from the database (in seconds) */
uint32_t db_get_serial_tcp_timeout(enum eSerialPort port);


/** Set TCP port timeouts for a serial port database */
void db_set_serial_tcp_timeout(enum eSerialPort port, uint32_t timeout);


/** Function to get the configuration data for rinmodbus */
void db_rinmodbus_getconfiguration(in_port_t *mb_tcp_port, enum eRinModbusEndian *mb_endian);


#endif /* DB_SERIAL_IP_H_ */
