/*
 * db_setpoint.c
 *
 *  Created on: 04/03/2011
 *      Author: ian
 */

#include "database.h"
#include "access.h"
#include "app_db_user.h"

#include "db_setpoint.h"


/** Setpoint active name strings */
const char *const setpoint_type_strings[SETP_TYPE_NUM] = SETPOINT_ACTIVE_STRS;


/** Setpoint lock options */
const char *const setp_lock_strings[SETP_LOCK_NUM] = SETP_LOCK_OPTION_STRS;


/** Setpoint source options */
const char *const setp_src_strings[SETP_SOURCE_NUM] = SETP_SOURCE_OPTION_STRS;


/** Setpoint logic options */
const char *const setp_logic_strings[SETP_LOGIC_NUM] = SETP_LOGIC_OPTION_STRS;


/** Setpoint alarm options */
const char *const setp_alarm_strings[SETP_ALARM_NUM] = SETP_ALARM_OPTION_STRS;


/** Setpoint timing options */
const char *const setp_timing_strings[SETP_TIMING_NUM] = SETP_TIMING_OPTION_STRS;


/** Setpoint reset options */
const char *const setp_reset_strings[SETP_RESET_NUM] = SETP_RESET_OPTION_STRS;


/** Setpoint feeder options */
const char *const setp_feeder_strings[SETP_FEEDER_NUM] = SETP_FEEDER_OPTION_STRS;


/** Convert setpoint type and remote key function to IO pin type */
enum eIOType access_sptype_remkeytype_to_io_type(enum eSetpointType sptype, enum eRemoteKeyFn rkftype)
{
  enum eIOType iotype;

  if (SETP_INACTIVE != sptype)
  {
    iotype = IOTypeOutput;
  }
  else if (REMOTE_DISABLED != rkftype)
  {
    iotype = IOTypeInput;
  }
  else
  {
    iotype = IOTypeUnused;
  }
  return iotype;
}


/** Determine the type of an IO pin, based on database settings */
enum eIOType access_get_io_type(uint8_t iobit)
{
  return access_sptype_remkeytype_to_io_type(db_app_user.setp_active[iobit], db_app_user.remote_f[iobit]);
}


/** Determine if an IO bit is an input, and it's remote function is implemented as a key */
bool access_get_io_is_key(uint8_t iobit)
{
  bool is_key;

  is_key = (SETP_INACTIVE == db_app_user.setp_active[iobit] &&
            remote_key_is_key[db_app_user.remote_f[iobit]]);
  return is_key;
}


/** Determine the states of all IO pins, based on database settings, and pin levels */
uint8_t access_get_io_states(void)
{
  uint8_t in;
  uint8_t out;
  uint8_t temp;
  uint8_t mask;
  int i;

  in = access_get_inputs();
  out = access_get_outputs();

  for (i = 0, mask = 0x01, temp = 0x00; i < NUM_SETPOINTS; i++, mask <<= 1)
  {
    if (IOTypeOutput == access_get_io_type(i))
    {
      temp |= out & mask;
    }
    else
    {
      temp |= in & mask;
    }
  }

  return temp;
}

