/*
 * db_setpoint.h
 *
 *  Created on: 04/03/2011
 *      Author: ian
 */

#ifndef DB_SETPOINT_H_
#define DB_SETPOINT_H_

#include "db_remote.h"


/** Number of setpoints supported by the system */
#define NUM_SETPOINTS 8


/** Setpoint types */
enum eSetpointType { SETP_INACTIVE = 0,
                     SETP_ON,
                     SETP_OVER,
                     SETP_UNDER,
                     SETP_COZ,
                     SETP_ZERO,
                     SETP_NET,
                     SETP_MOTION,
                     SETP_ERROR,
                     SETP_BUZZ,
                     SETP_W_OUT,
                     SETP_W_IN,
                     SETP_TYPE_NUM
                   };


/** Setpoint active strings, must match @ref enum eSetpointType */
#define SETPOINT_ACTIVE_STRS { "OFF", \
                               "ON", \
                               "OVER", \
                               "UNDER", \
                               "COZ", \
                               "ZERO", \
                               "NET", \
                               "MOTION", \
                               "ERROR", \
                               "BUZZER", \
                               "W. OUT", \
                               "W. IN" \
                             }


/** Setpoint active name strings */
extern const char *const setpoint_type_strings[SETP_TYPE_NUM];


/** Setpoint lock settings */
enum eSetpointLock { SETP_LOCK_DISABLED = 0,
                     SETP_LOCK_ENABLED,
                     SETP_LOCK_NUM
                   };


/** Setpoint lock option strings, must match @ref enum eSetpointLock */
#define SETP_LOCK_OPTION_STRS { "OFF", \
                                "ON" \
                              }


/** Setpoint lock options */
extern const char *const setp_lock_strings[SETP_LOCK_NUM];


/** Setpoint source settings */
enum eSetpointSource { SETP_SOURCE_GROSS = 0,
                       SETP_SOURCE_NET,
                       SETP_SOURCE_NUM
                     };


/** Setpoint source option strings, must match @ref enum eSetpointSource */
#define SETP_SOURCE_OPTION_STRS { "GROSS", \
                                  "NET" \
                                }


/** Setpoint source options */
extern const char *const setp_src_strings[SETP_SOURCE_NUM];


/** Setpoint logic settings */
enum eSetpointLogic { SETP_LOGIC_HIGH = 0,
                      SETP_LOGIC_LOW,
                      SETP_LOGIC_NUM
                    };


/** Setpoint logic option strings, must match @ref enum eSetpointLogic */
#define SETP_LOGIC_OPTION_STRS { "HIGH", \
                                 "LOW" \
                               }


/** Setpoint logic options */
extern const char *const setp_logic_strings[SETP_LOGIC_NUM];


/** Setpoint alarm settings */
enum eSetpointAlarm { SETP_ALARM_NONE = 0,
                      SETP_ALARM_SINGLE,
                      SETP_ALARM_DOUBLE,
                      SETP_ALARM_CONTINUOUS,
                      SETP_ALARM_FLASH,
                      SETP_ALARM_NUM
                    };



/** Setpoint alarm option strings, must match @ref enum eSetpointAlarm */
#define SETP_ALARM_OPTION_STRS { "OFF", \
                                 "SINGLE", \
                                 "DOUBLE", \
                                 "CONT.", \
                                 "FLASH" \
                               }


/** Setpoint alarm options */
extern const char *const setp_alarm_strings[SETP_ALARM_NUM];


/** Setpoint timing settings */
enum eSetpointTiming { SETP_TIMING_LEVEL = 0,
                       SETP_TIMING_EDGE,
                       SETP_TIMING_LATCH,
                       SETP_TIMING_PULSE,
                       SETP_TIMING_NUM
                     };


/** Setpoint timing option strings, must match @ref enum eSetpointTiming */
#define SETP_TIMING_OPTION_STRS { "LEVEL", \
                                  "EDGE", \
                                  "LATCH", \
                                  "PULSE" \
                                }


/** Setpoint timing options */
extern const char *const setp_timing_strings[SETP_TIMING_NUM];


/** Setpoint reset settings */
enum eSetpointReset { SETP_RESET_NONE = 0,
                      SETP_RESET_A,
                      SETP_RESET_B,
                      SETP_RESET_NUM
                    };


/** Setpoint reset option strings, must match @ref enum eSetpointReset */
#define SETP_RESET_OPTION_STRS { "NONE", \
                                 "SP.RST.A", \
                                 "SP.RST.B" \
                               }


/** Setpoint reset options */
extern const char *const setp_reset_strings[SETP_RESET_NUM];


/** IO types */
enum eIOType { IOTypeUnused = 0,
               IOTypeInput,
               IOTypeOutput
             };


/** Setpoint feeder options */
enum eSetpointFeeder { SETP_FEEDER_MULTIPLE = 0,
                       SETP_FEEDER_SINGLE,
                       SETP_FEEDER_NUM
                     };

/** Setpoint feeder strings, must match @ref enum eSetpointFeeder */
#define SETP_FEEDER_OPTION_STRS { "MULT.", \
                                  "SINGLE" \
                                }


/** Setpoint feeder options */
extern const char *const setp_feeder_strings[SETP_FEEDER_NUM];


#define NUM_SETP_OPT    5
#define SETP_SEQ        0
#define SETP_BATCH      1
#define SETP_DELAY      2
#define SETP_ERROR_CHK  3
#define SETP_AUTO_START 4


/* Defaults */
#define DB_SETP_DEFAULT_TYPE SETP_INACTIVE
#define DB_SETP_DEFAULT_ALARM SETP_ALARM_NONE
#define DB_SETP_DEFAULT_LOCK SETP_LOCK_DISABLED
#define DB_SETP_DEFAULT_SOURCE SETP_SOURCE_GROSS
#define DB_SETP_DEFAULT_DIRECTION SETP_DIRECTION_OVER
#define DB_SETP_DEFAULT_LOGIC SETP_LOGIC_HIGH
#define DB_SETP_DEFAULT_TIMING SETP_TIMING_LEVEL
#define DB_SETP_DEFAULT_RESET SETP_RESET_NONE


/** Convert setpoint type and remote key function to IO pin type */
enum eIOType access_sptype_remkeytype_to_io_type(enum eSetpointType sptype, enum eRemoteKeyFn rkftype);


/** Determine the type of an IO pin, based on database settings */
enum eIOType access_get_io_type(uint8_t iobit);


/** Determine if an IO bit is an input, and it's remote function is implemented as a key */
bool access_get_io_is_key(uint8_t iobit);


/** Determine the states of all IO pins, based on database settings, and pin levels */
uint8_t access_get_io_states(void);


#endif /* DB_SETPOINT_H_ */
