/*
 * db_special.h
 *
 *  Created on: 01/08/2011
 *      Author: ian
 */

#ifndef DB_SPECIAL_H_
#define DB_SPECIAL_H_

#include <stdint.h>
#include <rinweigh/adcdatatypes.h>
#include "kbd_5000.h"


/** Enum for the front keys */
enum eFrontKeys { FRONTKEY_ZERO = 0,
                  FRONTKEY_TARE,
                  FRONTKEY_GROSSNET,
                  FRONTKEY_FUNC1,
                  FRONTKEY_FUNC2,
                  FRONTKEY_FUNC3,
                  NUM_FRONT_KEYS
                };


/** Options for front key permission settings */
enum eFrontKeyPermissions { FRONTKEY_PERMIS_DISABLED = 0,
                            FRONTKEY_PERMIS_ENABLED,
                            FRONTKEY_PERMIS_IMMEDIATE,     /*!< Don't wait for no motion */
                            NUM_FRONTKEY_PERMIS
                          };


/** Enum for alternate units */
enum eAltUnits { ALT_UNITS_OFF = 0,
                 ALT_UNITS_GRAMS,
                 ALT_UNITS_KILOGRAMS,
                 ALT_UNITS_POUNDS,
                 ALT_UNITS_OUNCES,
                 ALT_UNITS_CUSTOM,
                 NUM_ALT_UNITS
               };


/** Menu alternate unit strings, must match @ref enum eAltUnits */
#define ALT_UNITS_MENU_STRS { "OFF",   \
                              "G",     \
                              "KG",    \
                              "LB",    \
                              "OZ",    \
                              "CUSTOM" \
                            }


/** Alternate units strings used in the menus */
extern const char *const alt_units_menus[NUM_ALT_UNITS];


/** Permission options for each of the front keys (N=No, Y=Yes, I=Immediate) */
#define FRONT_KEY_PERMISSION_STRS { "NYI",  /* Zero */ \
                                    "NYI",  /* Tare */ \
                                    "NY",   /* Gross/net */ \
                                    "NYI",  /* Func1 */ \
                                    "NYI",  /* Func2 */ \
                                    "NYI"   /* Func3 */ \
                                  }


/** Permission options for each of the front keys */
extern const char *const front_key_permission_settings[NUM_FRONT_KEYS];


/** Permission limits for each of the front keys (N=No, Y=Yes, I=Immediate) */
#define FRONT_KEY_PERMISSION_LIMITS { FRONTKEY_PERMIS_IMMEDIATE, /* Zero */ \
                                      FRONTKEY_PERMIS_IMMEDIATE, /* Tare */ \
                                      FRONTKEY_PERMIS_ENABLED,   /* Gross/net */ \
                                      FRONTKEY_PERMIS_IMMEDIATE, /* Func1 */ \
                                      FRONTKEY_PERMIS_IMMEDIATE, /* Func2 */ \
                                      FRONTKEY_PERMIS_IMMEDIATE, /* Func3 */ \
                                    }


/** Permission setting upper limit for each of the front keys */
extern const enum eFrontKeyPermissions front_key_permission_limits[NUM_FRONT_KEYS];


/** Convert an indicator key into a front key */
enum eFrontKeys xlat_indicator_key_to_front_key(enum eIndicatorKey key);


#endif /* DB_SPECIAL_H_ */
