/*
 * device_finder_we2111.c
 *
 *  Created on: 28/07/2011
 *      Author: ian
 */
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "ipinfo.h"
#include "device_finder.h"
#include "iface_ethnet.h"
#include "version.h"
#include "database.h"
#include "app_db_factory.h"
#include "app_db_user.h"
#include "we_debug.h"

#include "device_finder_we2111.h"


static void get_mac_address_and_hostname(char macstr[MAC_ADDRESS_STR_LENGTH], char *hostname, size_t hostname_maxlen)
{
  struct s_snc_net_config net_config;

  /* Get results */
  if (!net_ip_get_config(false, &net_config))
  {
    /* Defaults for failure */
    snprintf(hostname, hostname_maxlen, "unknown");
    snprintf(macstr, MAC_ADDRESS_STR_LENGTH, "00:00:00:00:00:00");
  }
  else
  {
    /* Copy over hostname */
    snprintf(hostname, hostname_maxlen, "%s", net_config.hostname);

    /* Format mac address */
    net_ip_macaddress_to_string(&net_config.mac_address, ':', macstr);
  }
}


/** Update the device finder information for the WE2111 */
void update_device_finder_we2111(void)
{
  char *xml_str;
  const char *class;
  #define SER_NO_STR_LEN 20
  char serial_no_str[SER_NO_STR_LEN];
  char macaddr_str[MAC_ADDRESS_STR_LENGTH];
  const char *customid;
  #define TCP_PORT_STR_LEN 20
  char tcp_port_str[TCP_PORT_STR_LEN];
  char hostname_str[HOST_NAME_MAX];

  WE_DEVICE_FINDER_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "update_device_finder called\n");

  /* Collect relevant system information */
  class = DEVICE_FINDER_XML_CLASS_INDICATOR;
  get_mac_address_and_hostname(macaddr_str, hostname_str, HOST_NAME_MAX);
  snprintf(serial_no_str, SER_NO_STR_LEN, "%lu", (unsigned long) db_app_factory.serial_no);
  customid = db_app_user.id;
  snprintf(tcp_port_str, TCP_PORT_STR_LEN, "%u", db_app_user.serial_tcpi1_port);

  /* Create XML string with relevant information */
  xml_str = create_device_finder_xml(class, db_app_factory.model_str, serial_no_str,
                                     app_version_str, customid, macaddr_str,
                                     tcp_port_str, hostname_str);
  if (NULL != xml_str)
  {
    WE_DEVICE_FINDER_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "update_device_finder writing to file\n");

    /* Write XML string to device finder update file */
    (void) write_device_finder_xml(DEVICE_FINDER_UPDATE_FILE, xml_str);

    /* Free up XML string */
    free(xml_str);
  }

  WE_DEVICE_FINDER_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "update_device_finder finished\n");
}
