/*
 * dm_partition.h
 *
 *  Created on: Nov 21, 2011
 *      Author: ian
 */

#ifndef DM_PARTITION_H_
#define DM_PARTITION_H_

#include <stdbool.h>
#include "devicemounter.h"


/** Typedef of function to call when a partition is mounted/unmounted */
typedef void (*t_dm_partition_callback_fn)(void *client_callback_data, enum eDeviceMounterAction action, const char *mount_directory, bool result);


/** Command to unmount a partition, command_data = NULL */
#define DM_PARTITION_CMD_UNMOUNT 1


/**
 * Initialise handling of partition devices. Currently only handles one device mounted at a time.
 *
 * @param dm is a device mounter
 * @param mount_location is the location to mount the partition to (eg /mnt/rwfs)
 * @param filesystem_type is the file system type to mount. NOTE: this only allows one type of file system to be mounted. Can be NULL if you want mount to try and guess. (eg vfat)
 * @param mount_options are the options passed to mount with -o (eg sync, nosuid), can be NULL if no options
 * @param partition_callback_fn is the function to call when a partition is mounted/unmounted
 * @param client_callback_data is a pointer to data to pass to callback function
 *
 * @return dm_handle >=0 for success, < 0 otherwise
 */
int dm_partition_init(struct s_devicemounter *dm, const char *mount_location,
                      const char *filesystem_type, const char *mount_options,
                      t_dm_partition_callback_fn partition_callback_fn, void *client_callback_data);


#endif /* DM_PARTITION_H_ */
