/*
 * display.h
 *
 *  Created on: 19/07/2010
 *      Author: ian
 */
#ifndef DISPLAY_H_
#define DISPLAY_H_

#include <stdint.h>
#include <rinhw/stdrinstrum.h>
#include <rinweigh/adcdatatypes.h>


#define NUM_DIGITS   6
#define DISP_BUF_SIZE (2*NUM_DIGITS + 1)


/** Special constant used to display all units when units == ADC_UNITS_NONE && weight held */
#define APP_UNITS_ALL ADC_NUM_UNITS


/* Annunciators */
#define ANNUN_NONE         0x00000000
#define ANNUN_ZERO         0x00000001
#define ANNUN_MOTION       0x00000002
#define ANNUN_GROSS        0x00000004
#define ANNUN_NET          0x00000008
#define ANNUN_ZEROBAND     0x00000010
#define ANNUN_HELD         0x00000020
#define ANNUN_SPARE        0x00000040
#define ANNUN_RANGE1       0x00000080
#define ANNUN_RANGE2       0x00000100
#define ANNUN_RANGE3       0x00000200
#define ANNUN_CHECK_UNDER2 0x00000400
#define ANNUN_CHECK_UNDER1 0x00000800
#define ANNUN_CHECK_PASS   0x00001000
#define ANNUN_CHECK_OVER1  0x00002000
#define ANNUN_CHECK_OVER2  0x00004000
#define ANNUN_IO1          0x00008000
#define ANNUN_IO2          0x00010000
#define ANNUN_IO3          0x00020000
#define ANNUN_IO4          0x00040000
#define ANNUN_IO5          0x00080000
#define ANNUN_IO6          0x00100000
#define ANNUN_IO7          0x00200000
#define ANNUN_IO8          0x00400000
#define ANNUN_SPARE2       0x00800000
#define ANNUN_SPARE3       0x01000000


/** Display a string on the display, with a decimal point */
void Dputstr_dp(const char *s, uint8_t dp);


/**
 * Display a string on the display,
 * without a decimal point, though the string may
 * contain 1 or more, which will be displayed
 */
void Dputstr(const char *s);


/**
 * Display a string on the display,
 * without a decimal point, though the string may
 * contain 1 or more, which will be displayed
 *
 * Any flashing will be stopped
 */
void Dputstr_nofl(const char *s);


/** Sets the units, but hides them if there is motion if the hide_if_motion flag is set */
void Dset_units(enum eAdcUnits adcunits, bool motion, bool hide_if_motion);


/** Set the units regardless of motion */
void Dset_units_ignore_motion(enum eAdcUnits adcunits);


/** Set the annunciators on the display. seg_bits contains ANNUN_* masks */
void Dset_annunciators(uint32_t seg_bits);


/** Set annunciators on the display to flash. seg_bits contains ANNUN_* masks */
void Dflash_annunciators(uint32_t seg_bits);


/** Convert the io bits into an annunciator bitmap */
uint32_t map_io_to_annunciators(uint8_t io);


/** Convert info & io_states into a bitmask of annunciators for display */
uint32_t get_annunciators(const AdcInfo *info, uint8_t io_states);


/** Display a string will all associate information (annunciators, dp, units, io states) */
void Dshow_str_info(const char *str, const AdcInfo *info, uint8_t io_status, bool hide_units_if_motion);


/** Display a number on the display */
void Dshow_number(int32s weight, size_t dp, size_t min_digits);


/** Display an oz weight as lb:oz on the display */
void Dshow_lb_oz_weight(int32s weight, int8u dp, uint8_t min_digits);


/** Display a reading will all associate information (annunciators, dp, units, io states) */
void Dshow_rdg_info(const AdcRdg *rdg, uint8_t io_status, bool hide_units_if_motion, uint8_t min_digits);


/** Set all segments in digit to flash */
void Dflash_segments(size_t digit, bool flashon);


/** Enable/disable units flashing */
void Dflash_units(bool on);


#define DFLASHMASK_NONE 0x00        /*!< No digits flashing */
#define DFLASHMASK_D0 0x01          /*!< Digit 0 flashing */
#define DFLASHMASK_D1 0x02          /*!< Digit 1 flashing */
#define DFLASHMASK_D2 0x04          /*!< Digit 2 flashing */
#define DFLASHMASK_D3 0x08          /*!< Digit 3 flashing */
#define DFLASHMASK_D4 0x10          /*!< Digit 4 flashing */
#define DFLASHMASK_D5 0x20          /*!< Digit 5 flashing */

/** All digits flashing */
#define DFLASHMASK_ALL (DFLASHMASK_D0 | DFLASHMASK_D1 | DFLASHMASK_D2 | DFLASHMASK_D3 | DFLASHMASK_D4 | DFLASHMASK_D5)


/** Set digits specified in digit_mask to flash, and those not set to clear flashing */
void Dflash_digits(uint8_t digit_mask);


/** Turn on all segments on the display */
void Dshow_all(void);


/** Turn off all segments on the display */
void Dclear_all(void);


/** Turn off all flashing on the display */
void Dclear_flashing(void);


/** Set the display brightness from the database setting */
void Dset_brightness(void);


/** Setup the display */
void init_display(void);


#endif /* DISPLAY_H_ */
