/*
 * edit_string.h
 *
 *  Created on: Apr 15, 2013
 *      Author: ian
 */

#ifndef EDIT_STRING_H_
#define EDIT_STRING_H_

#include <stdint.h>
#include "editor.h"


/** Character set of all printable characters */
extern const char charset_printable[];


/** Character set of all alpha numeric characters */
extern const char charset_alpha_numeric[];


/** Character set of valid characters for a hostname */
extern const char charset_hostname[];


/** Character set of valid characters for a domain name */
extern const char charset_domainname[];


/**
 * A menu editor that allows printable strings to be edited, only allowing characters from character_set
 *
 * @param str is the string to be edited or viewed
 * @param max_len is the maximum length of str (including a null termination character).
 *        ie max_len = 10 permits up to 9 printable characters and 1 null termination
 * @param character_set is the set of valid characters that can be used in the string
 *
 * @return enum eEditResult
 */
enum eEditResult edit_printable_string(char *str, size_t max_len, const char *character_set);


/**
 * A menu editor that allows a printable string to be viewed but not edited
 *
 * @param str is the string to be edited or viewed
 * @param max_len is the maximum length of str (including a null termination character).
 *        ie max_len = 10 permits up to 9 printable characters and 1 null termination
 * @param character_set is the set of valid characters that can be used in the string
 *
 * @return enum eEditResult
 */
enum eEditResult view_printable_string(const char *str);


#endif /* EDIT_STRING_H_ */
