#ifndef EDITOR_DOT_H
#define EDITOR_DOT_H 1

#include <stdbool.h>
#include <stdint.h>


/** Result of edit function */
enum eEditResult
{
  EDIT_ABORT = 0,    /*!< Editing of value aborted */
  EDIT_OK,           /*!< Value accepted */
  EDIT_CHG,          /*!< Value changed and accepted */
  EDIT_MENU_QUIT,    /*!< Quit menus */
};


/** Display dashes on the display for WECFG_DELAY_FULL_OPERATING_MILLISECONDS when an incorrect value is entered */
void block_entry(void);


/**
 * Edit a list of items
 *
 * @param data is a pointer to the setting value. Not NULL
 * @param min_setting is the minimum value of the setting
 * @param max_setting is the maximum value of the setting
 * @param opts is an array of pointers to strings containing the names of each setting. Must have at least max_settings entries.
 * @param allow_edit true if the setting can be edited, false if the setting can be viewed only
 *
 * @return EDIT_CHG on value changed, EDIT_OK on no change, EDIT_ABORT on cancel, EDIT_MENU_QUIT to move up a menu level
 */
enum eEditResult edit_register_data(uint8_t *data, uint8_t min_setting, uint8_t max_setting, const char *const *opts, bool allow_edit);


/** Bit list editor */
enum eEditResult edit_bits(uint8_t *data, uint8_t sets, const char *const opts[], bool full);


/** Edit a 32 bit signed number */
enum eEditResult edit_long(long *val, long max_val, long min_val, uint8_t dp, uint8_t num_digits, uint8_t dp2, bool full);


/** Edit a 8 bit unsigned number */
enum eEditResult edit_byte(uint8_t *val, uint8_t max_val, uint8_t min_val, uint8_t dp, uint8_t num_digits, bool full);


/***************************************************************************/
char *clear_leading_zeros(char *s, uint8_t dp);

/***************************************************************************/
char *pad_str(char *s, char pad, uint8_t len);
/* pads s with leading pad chars to length len. returns s. */
/***************************************************************************/


#endif /* #ifndef EDITOR_DOT_H */
