/*
 * facdb_endian.c
 *
 *  Created on: Mar 15, 2017
 *      Author: ian
 */
#define _BSD_SOURCE
#include <endian.h>
#include <rinweigh/adcdatatypes.h>
#include <rinweigh/adcdatabase.h>

#include "facdb_endian.h"


/* m68k-uclinux-gcc uclibc doesn't have htobe32(), htobe32() */
#ifndef htobe32
# include <byteswap.h>
# if __BYTE_ORDER == __LITTLE_ENDIAN
#  define htobe32(x) __bswap_32 (x)
#  define htole32(x) (x)
# else
#  define htobe32(x) (x)
#  define htole32(x) __bswap_32 (x)
# endif
#endif


static uint32_t htobe32_func(uint32_t n)
{
  return htobe32(n);
}


static uint32_t htole32_func(uint32_t n)
{
  return htole32(n);
}


/** Convert a factory database to a specific endian */
void facdb_to_endian(AdcFactoryDatabase *facdb, int endian)
{
  size_t i;
  uint32_t (*bsfunc)(uint32_t value);

  if (__BIG_ENDIAN == endian)
    bsfunc = htobe32_func;
  else
    bsfunc = htole32_func;

  for (i = 0; i < ADC_NUM_SYNC_FREQ; i++)
  {
    facdb->zeroOffset[i] = bsfunc(facdb->zeroOffset[i]);
    facdb->zeroDriftPos[i] = bsfunc(facdb->zeroDriftPos[i]);
    facdb->zeroDriftNeg[i] = bsfunc(facdb->zeroDriftNeg[i]);
    facdb->spanCal[i] = bsfunc(facdb->spanCal[i]);
    facdb->spanDriftPos[i] = bsfunc(facdb->spanDriftPos[i]);
    facdb->spanDriftNeg[i] = bsfunc(facdb->spanDriftNeg[i]);
  }
  facdb->calTemp = bsfunc(facdb->calTemp);
  facdb->chksum = bsfunc(facdb->chksum);
}


/** Calculate a checksum for a factory database given a specific endian */
uint16_t facdb_calc_checksum(const AdcFactoryDatabase *facdb, int endian)
{
  AdcFactoryDatabase tempfdb;

  tempfdb = *facdb;
  facdb_to_endian(&tempfdb, endian);
  return DB_CHECKSUM(AdcFactoryDatabase, tempfdb, ADC_DB_FACTORY_CHECKSUM_SEED);
}


/** Try to guess the endian of a factory database. Relies on a valid checksum */
int facdb_guess_endian(const AdcFactoryDatabase *facdb)
{
  uint16_t chksum;

  /* Check big endian */
  chksum = facdb_calc_checksum(facdb, __BIG_ENDIAN);
  if (facdb->chksum == chksum)
    return __BIG_ENDIAN;

  /* Check little endian */
  chksum = facdb_calc_checksum(facdb, __LITTLE_ENDIAN);
  if (facdb->chksum == chksum)
    return __LITTLE_ENDIAN;

  /* Dunno */
  return 0;
}
