/*
 * filebackup.c
 *
 *  Created on: 29/05/2012
 *      Author: ian
 */
#include <stdio.h>
#include <string.h>
#include <rinhw/rtc.h>
#include "sysclock.h"
#include "version.h"
#include "app_db_factory.h"

#include "filebackup.h"


/**
 * Create backup file name in the form "folder/model_serialno_date_time_postfix.extension
 *
 * @param filename is the resultant file name, must be filename_len characters in length. Not NULL
 * @param filename_len is the maximum length of filename. Must be > 0
 * @param folder is the folder to prefix the filename with, will separate filename with '/' where appropriate. Can be NULL, in which case not used
 * @param postfix is appended to the filename before the extension. Can be NULL, in which case nothing is appended
 * @param extension is appended to the end of the filename, must contain '.' if you need one. Can be NULL in which case nothing is appended
 */
void create_backup_filename(char *filename, size_t filename_len, const char *folder, const char *postfix, const char *extension)
{
  t_datetime dt;
  const char *sep_str;
  const char *postfix_sep_str;
  const char *nn_folder;
  const char *nn_postfix;

  (void) GetDateTime(&dt);

  /* Tack a slash on the end of the folder if required */
  sep_str = (NULL == folder || 0 == strlen(folder) || '/' == folder[strlen(folder) - 1]) ? "" : "/";
  nn_folder = (NULL == folder) ? "" : folder;

  /* Put a _ between time and postfix if we have a postfix */
  postfix_sep_str = (NULL == postfix || 0 == strlen(postfix)) ? "" : "_";
  nn_postfix = (NULL == postfix) ? "" : postfix;

  /* Assemble filename */
  snprintf(filename, filename_len - 1,
           "%s%s%s_%lu_%04u%02u%02u_%02u%02u%02u%s%s%s",
           nn_folder,
           sep_str,
           db_app_factory.model_str,
           (unsigned long) db_app_factory.serial_no,
           dt.year,
           dt.month,
           dt.date,
           dt.hour,
           dt.minute,
           dt.second,
           postfix_sep_str,
           nn_postfix,
           extension
          );
}
