#ifndef IFACE_DOT_H
#define IFACE_DOT_H 1

#include <stdint.h>
#include <rinweigh/adcdatatypes.h>
#include "kbd_5000.h"
#include "comms_print_interlock.h"
#include "iface_display_total.h"

/*
 * Random global variables
 */

extern bool edit_functions;     /* non-zero if calibration in progress via keyboard */
extern bool rear_key_entry;     /* true if the rear setup key has been used to access setup */
extern bool setpt_edit;         /* true if local setpoint edit mode entered, false otherwise */
extern bool blank_display;      /* true if display blanked due to tilt inputs */


extern struct s_display_total display_total_struct;


/** Display hold types */
enum eDisplayHold { dhNone, dhManual, dhPeak, dhLivestock };


/** What kind of hold the display is currently showing */
extern enum eDisplayHold show_hold;


/** Current ADC operation */
extern int8u adc_operation_counter;


/** Print interlock */
extern struct s_comms_print_interlock comms_print_interlock;


/** Get whatever weight reading is on the display, based on the hold state */
void get_displayed_adcrdg(AdcRdg *rdg);


/** Update operation counter so errors with previous operation will get displayed */
void update_adc_operation_counter(void);


/** Return true if the application has been asked to shutdown */
bool application_quit_forced(void);


/** Force the application to shutdown */
void application_quit_force(void);


/** Return true if the setup menus should be exited */
bool quit_setup_menus(void);


/** Display "PRE.TAR" and delay */
void display_pretar_and_delay(void);


/* Display PT value if non-zero */
void AdcGrossNetErrorDisplay(void);


/***************************************************************************/
void do_user_bgnd (void);
/* main user backgrnd routine.
** must be called during the display of user error messages,
** to keep autoprinting etc going.
** the routines called from do_user_bgnd themselves call idle()
** so they cannot be called by idle as with other backgrnd routines.
***********************************************************************/


/** Handle a keypress within the main user interface */
void user_iface_key_dispatcher(enum eIndicatorKey kbd, bool immediate);


/** Main user interface loop */
void do_user_iface(bool *quit);


void do_disp (void);
/* called every 100 msec to update display data
**********************************************************************/


/** Freeze the display (no display updates performed) */
void freeze_display(bool freeze);


/** Freeze the annunciators (no annunciator updates performed) */
void freeze_annunciators(bool freeze);


/** Freeze both the display and annunciators (no display or annunciator updates performed) */
void freeze_display_and_annunciators(bool freeze);


/** Give registers control of display and annunciators */
void register_control_display_and_annunciators(bool control);


/** Update the register display buffer */
void write_register_display_buf(const char *buf);


/** Update the register display units */
void write_register_display_units(enum eAdcUnits units);


/** Update the register display annuns */
void write_register_display_annuns(uint32_t annun, bool auto_annun);


/** Show the version and calibration count, delaying WECFG_DELAY_FULL_OPERATING_MILLISECONDS after each */
void show_version_and_cal_cnt(void);


/** Initialise the user interface */
void init_iface(void);


/** Shutdown the user interface */
void destroy_iface(void);


#endif /* #ifndef IFACE_DOT_H */
