/*
 * iface_alibi.c
 *
 *  Created on: Feb 21, 2012
 *      Author: ian
 */
#include <string.h>
#include <stdlib.h>
#include <rinweigh/adcdatabase.h>
#include <rinweigh/adcalibi.h>
#include <rinweigh/adcdsd.h>
#include <rinweigh/adcchecksum.h>
#include <rinweigh/adcversion.h>
#include "editor.h"
#include "display.h"
#include "kbd_5000.h"
#include "iface.h"
#include "comms_print_trigger.h"
#include "db_build.h"
#include "iface_sd.h"
#include "dsd.h"
#include "adcthread.h"
#include "app_db_changelog.h"
#include "system.h"

#include "iface_alibi.h"


/* These should remain unchanged so our OIML approval remains valid */
#if (0x45d6 != ADC_LIBRARY_CHECKSUM)
  #error "ADC library checksum has changed"
#endif
#if (101 != ADC_LIB_VERSION_INT)
  #error "ADC library version has changed"
#endif


static const char *const alibi_english_strs[aaisiNumStrings] = ADC_ALIBI_INTERNATIONALISATION_ENGLISH_STRS;


void we_alibi_force_quit(void *implementer_data, bool *force_quit)
{
  if (quit_setup_menus())
  {
    *force_quit = true;
  }
}


static void we_alibi_display_allsegs(void *implementer_data, enum eAdcAlibiDisplayType alibi_display_type, bool on_not_off)
{
  if (on_not_off)
  {
    Dshow_all();
  }
  else
  {
    Dclear_all();
  }
}


static void we_alibi_display(void *implementer_data, enum eAdcAlibiDisplayType alibi_display_type, const char **display_strs, const AdcInfo *info)
{
  Dputstr(*display_strs);
  Dset_annunciators(get_annunciators(info, 0));
  Dset_units(info->unitdp.bit.units, info->status.bit.motion, true);
}


static enum eAdcAlibiKeystroke we_alibi_getchar_weigh(uint8_t ch)
{
  enum eAdcAlibiKeystroke keystroke;
  switch (ch)
  {
    case ZERO_KEY:
      keystroke = aakZero;
      break;
    case TARE_KEY:
      keystroke = aakTare;
      break;
    case GROSS_KEY:
      keystroke = aakGrossNet;
      break;
    case FUNC1_KEY:
      keystroke = aakPrint;
      break;
    case DETARE_KEY:
      keystroke = aakEnterPresetTare;
      break;
    case DEFUNC2_KEY:
      keystroke = aakMode;
      break;
    default:
      keystroke = aakNone;
      break;
  }
  return keystroke;
}


static enum eAdcAlibiKeystroke we_alibi_getchar_dsd_changelog(uint8_t ch)
{
  enum eAdcAlibiKeystroke keystroke;
  switch (ch)
  {
    case KEY_EDIT_LEFT + DEKEY:
      keystroke = aakDSDRecordFirst;
      break;
    case KEY_EDIT_LEFT:
      keystroke = aakDSDRecordPrevious;
      break;
    case KEY_EDIT_RIGHT + DEKEY:
      keystroke = aakDSDRecordLast;
      break;
    case KEY_EDIT_RIGHT:
      keystroke = aakDSDRecordNext;
      break;
    case DEFUNC2_KEY:
      keystroke = aakMode;
      break;
    case FUNC2_KEY:
      keystroke = aakDSDRecordSelect;
      break;
    case KEY_EDIT_DOWN + DEKEY:
      keystroke = aakDSDItemFirst;
      break;
    case KEY_EDIT_DOWN:
      keystroke = aakDSDItemPrevious;
      break;
    case KEY_EDIT_UP + DEKEY:
      keystroke = aakDSDItemLast;
      break;
    case KEY_EDIT_UP:
      keystroke = aakDSDItemNext;
      break;
    default:
      keystroke = aakNone;
      break;
  }
  return keystroke;
}


static enum eAdcAlibiKeystroke we_alibi_getchar_counting(uint8_t ch)
{
  enum eAdcAlibiKeystroke keystroke;
  switch (ch)
  {
    case ZERO_KEY:
      keystroke = aakZero;
      break;
    case TARE_KEY:
      keystroke = aakTare;
      break;
    case GROSS_KEY:
      keystroke = aakGrossNet;
      break;
    case FUNC3_KEY:
      keystroke = aakCountEnterQty;
      break;
    case DEFUNC3_KEY:
      keystroke = aakCountCancel;
      break;
    case DEFUNC2_KEY:
      keystroke = aakMode;
      break;
    default:
      keystroke = aakNone;
      break;
  }
  return keystroke;
}


static void we_alibi_getchar(void *implementer_data, enum eAdcAlibiMode mode, enum eAdcAlibiKeystroke *keystroke, int16u timeout)
{
  uint8_t ch;

  ch = get_key_timeout(timeout);

  switch (mode)
  {
    case aamWeighing:
      *keystroke = we_alibi_getchar_weigh(ch);
      break;

    case aamDSDView:
    case aamChangelogView:
      *keystroke = we_alibi_getchar_dsd_changelog(ch);
      break;

    case aamCounting:
      *keystroke = we_alibi_getchar_counting(ch);
      break;

    default:
      *keystroke = (DEFUNC2_KEY == ch) ? aakQuit : aakNone;
      break;
  }
}


static void we_alibi_get_internationalisation_string(void *implementer_data, enum eAdcAlibiInternationisationStringIdx stridx, const char **display_str)
{
  *display_str = alibi_english_strs[stridx];
}


static bool we_alibi_edit_number(void *implementer_data, AdcRdg *rdg, int32s minimum, int32s maximium, bool is_signed)
{
  long lnum;
  bool result;
  enum eEditResult edit_result;

  lnum = rdg->rdg;
  Dset_units_ignore_motion(rdg->info.unitdp.bit.units);
  edit_result = edit_long(&lnum, maximium, minimum, rdg->info.unitdp.bit.dp, NUM_DIGITS, 0, true);
  result = (EDIT_OK == edit_result || EDIT_CHG == edit_result);
  if (result)
  {
    rdg->rdg = lnum;
  }
  return result;
}


static bool we_alibi_edit_list(void *implementer_data, int8u *idx, const char *const list_strings[], int8u list_length)
{
  uint8_t data;
  bool result;
  enum eEditResult edit_result;

  data = *idx;
  edit_result = edit_register_data(&data, 0, list_length - 1, list_strings, true);
  result = (EDIT_OK == edit_result || EDIT_CHG == edit_result);
  if (result)
  {
    *idx = data;
  }
  return result;
}


static void we_alibi_print_string(void *implementer_data, const char *print_str)
{
  comms_print_trigger_nontraceable_raw_data(print_str);
}


/** Adc library unit strings, must match @ref enum eAdcUnits */
#define ADC_UNITS_PRINT_STRS { "__", \
                               " g", \
                               "kg", \
                               " t", \
                               "lb", \
                               "oz", \
                               " N", \
                               " p", \
                               "u0", \
                               "u1", \
                               "u2", \
                               "u3", \
                               "u4", \
                               "u5", \
                               "u6", \
                               "u7" \
                             }


/** Units strings used for printouts */
static const char *const adc_units_print[ADC_NUM_UNITS] = ADC_UNITS_PRINT_STRS;


static void we_alibi_get_abbreviated_units_string(void *implementer_data, int8u adcunitsidx, const char **units_str)
{
  if (adcunitsidx >= ADC_NUM_UNITS)
  {
    *units_str = adc_units_print[ADC_UNITS_NONE];
  }
  else
  {
    *units_str = adc_units_print[adcunitsidx];
  }
}


bool we_alibi_dsd_convert_to_traceable(void *implementer_data, const int8u *data, size_t length, AdcTraceableWeight *atw)
{
  const struct s_we_dsd_record *dsdrec = (const struct s_we_dsd_record *) data;
  bool result = false;

  if (length >= sizeof(struct s_we_dsd_record))
  {
    AdcTraceableUnpackFromNetwork(&dsdrec->atwp, atw);
    result = true;
  }
  return result;
}


bool we_alibi_dsd_convert_from_traceable(void *implementer_data, const AdcTraceableWeight *atw, int8u *data, size_t length)
{
  struct s_we_dsd_record *dsdrec = (struct s_we_dsd_record *) data;
  bool result = false;

  if (length >= sizeof(struct s_we_dsd_record))
  {
    AdcTraceablePackToNetwork(atw, &dsdrec->atwp);
    result = true;
  }
  return result;
}


void we_alibi_convert_adcerrors_to_apperrors(void *implementer_data, AdcErrors adcerrors, int32u *apperrors)
{
  *apperrors = get_current_errors();
}


/** Run the alibi menus */
enum eMenuLevel do_alibi(bool full_edit, void *user_data, size_t index)
{
  struct s_adc_alibi_info adc_alibi_info;
  int32u recordsize;
  char display_work_buf1[ADC_ALIBI_DISPLAY_WORK_BUF_LEN(NUM_DIGITS)];

  /* Setup for long press available on all keys */
  setup_kbd(NONE_KBD, (ZERO_KBD | TARE_KBD | GROSS_KBD | FUNC1_KBD | FUNC2_KBD));

  /* Fill in implementation data */
  memset(&adc_alibi_info, 0, sizeof(adc_alibi_info));
  adc_alibi_info.implementer_data = NULL;
  adc_alibi_info.adc_alibi_display_type = aadtSingleDisplay;
  adc_alibi_info.num_digits[aadtSingleDisplay] = NUM_DIGITS;
  adc_alibi_info.num_digits[aadtDoubleDisplay] = 0;
  adc_alibi_info.display_work_buffers[aadtSingleDisplay] = display_work_buf1;
  adc_alibi_info.display_work_buffers[aadtDoubleDisplay] = NULL;
  adc_alibi_info.alibi_mode = aamWeighing;  /* Start in weighing mode */
  adc_alibi_info.status_can_show_pieces = false; /* Can't display pieces */

  adc_alibi_info.ass = &adcSuperStructure;

  /* Assume no DSD */
  adc_alibi_info.dsd = wedsd;
  adc_alibi_info.dsd_record_buffer = NULL;
  adc_alibi_info.dsd_record_buffer_length = 0;
  if (NULL != wedsd && ADC_DSD_SUCCESS == AdcDSDIsRunning(wedsd, true))
  {
    /* DSD working */
    if (ADC_DSD_SUCCESS == AdcDSDGetRecordSize(wedsd, true, &recordsize))
    {
      adc_alibi_info.dsd_record_buffer = malloc(recordsize);
      if (NULL != adc_alibi_info.dsd_record_buffer)
      {
        adc_alibi_info.dsd = wedsd;
        adc_alibi_info.dsd_record_buffer_length = recordsize;
      }
    }
  }

  /* Setup change log */
  adc_alibi_info.changelog_reader = app_changelog_reader;

  adc_alibi_info.no_piece_counting = false;
  adc_alibi_info.no_alternate_units = false;
  adc_alibi_info.adc_lib_checksum = ADC_LIBRARY_CHECKSUM;

  /* Function pointers */
  adc_alibi_info.adc_alibi_display_allsegs = we_alibi_display_allsegs;
  adc_alibi_info.adc_alibi_display = we_alibi_display;
  adc_alibi_info.adc_alibi_getchar = we_alibi_getchar;
  adc_alibi_info.adc_alibi_get_internationalisation_string = we_alibi_get_internationalisation_string;
  adc_alibi_info.adc_alibi_force_quit = we_alibi_force_quit;
  adc_alibi_info.adc_alibi_edit_number = we_alibi_edit_number;
  adc_alibi_info.adc_alibi_edit_list = we_alibi_edit_list;
  adc_alibi_info.adc_alibi_get_abbreviated_units_string = we_alibi_get_abbreviated_units_string;
  adc_alibi_info.adc_alibi_print_string = we_alibi_print_string;
  adc_alibi_info.adc_alibi_dsd_convert_to_traceable = we_alibi_dsd_convert_to_traceable;
  adc_alibi_info.adc_alibi_dsd_convert_from_traceable = we_alibi_dsd_convert_from_traceable;
  adc_alibi_info.adc_alibi_convert_adcerrors_to_apperrors = we_alibi_convert_adcerrors_to_apperrors;

  /* Run the alibi application */
  if (!AdcAlibiRun(&adc_alibi_info))
  {
    /* Alibi failed to run */
    error_strings("ALIBI", "FAIL");
  }

  if (NULL != adc_alibi_info.dsd_record_buffer)
  {
    free(adc_alibi_info.dsd_record_buffer);
  }

  /* Setup for normal keys */
  setup_kbd(KBD_DEFAULT_REPEAT, KBD_DEFAULT_HOLD);

  if (!quit_setup_menus())
  {
    /* If the system isn't shutting down, show the software version and cal count like we do when the application starts */
    show_version_and_cal_cnt();
  }
  return mlQuit;
}
