/*
 * iface_build.c
 *
 *  Created on: 15/04/2011
 *      Author: ian
 */
#include <stdio.h>
#include <string.h>
#include <rinweigh/adcdatabase.h>
#include <rinweigh/adcinterface.h>
#include "db_build.h"
#include "database.h"
#include "editor.h"
#include "iface.h"
#include "meas.h"
#include "display.h"
#include "kbd_5000.h"
#include "resolution.h"
#include "iface_sd.h"
#include "adcthread.h"
#include "app_db_user.h"
#include "commonstrings.h"
#include "dbprint.h"

#include "iface_build.h"


/** Edit the scale type */
static enum eMenuLevel edit_scale_type(bool full_edit, void *user_data, size_t index)
{
  enum eEditResult result;

  /* Edit scale type */
  result = edit_register_data(&adcSuperStructure.adcUserDatabase.scaleType, 0, APP_NUM_SCALE_TYPE - 1, build_adc_scale_type_strs, full_edit);
  return edit_result_to_menu_level(result);
}


/** Print the scale type */
static void to_string_scale_type(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  snprintf(buffer, buflen, "%s", build_adc_scale_type_strs[adcSuperStructure.adcUserDatabase.scaleType]);
}


/** Edit the decimal point position */
static enum eMenuLevel edit_decimal_point(bool full_edit, void *user_data, size_t index)
{
  return edit_result_to_menu_level(edit_register_data(&adcSuperStructure.adcUserDatabase.dpPos, 0, 5, build_decimal_point, full_edit));
}


/** Print the decimal point position */
static void to_string_decimal_point(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  snprintf(buffer, buflen, "%s", build_decimal_point[adcSuperStructure.adcUserDatabase.dpPos]);
}


/** Edit capacity */
static enum eMenuLevel edit_capacity(bool full_edit, void *user_data, size_t index)
{
  long capacity_long;
  int32s *capacity = (int32s *) user_data;
  enum eEditResult result;

  capacity_long = *capacity;
  Dset_units_ignore_motion(adcSuperStructure.adcUserDatabase.units);
  result = edit_long(&capacity_long, 999999L, 100L, adcSuperStructure.adcUserDatabase.dpPos, NUM_DIGITS, 0, full_edit);
  if (EDIT_CHG == result)
  {
    *capacity = capacity_long;

    /* Reign in additive tare */
    if (adcSuperStructure.adcUserDatabase.addTare > true_full_scale())
    {
      adcSuperStructure.adcUserDatabase.addTare = true_full_scale();
    }
  }
  return edit_result_to_menu_level(result);
}


/** Print the scale capacity */
static void to_string_capacity(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  AdcRdg rdg;
  int32s *capacity = (int32s *) user_data;

  memset(&rdg, 0, sizeof(rdg));
  rdg.rdg = *capacity;
  rdg.info.unitdp.bit.dp = adcSuperStructure.adcUserDatabase.dpPos;
  rdg.info.unitdp.bit.units = adcSuperStructure.adcUserDatabase.units;
  to_string_weight(&rdg, true, buffer, buflen);
}


/** Edit count by */
static enum eMenuLevel edit_resolution(bool full_edit, void *user_data, size_t index)
{
  uint8_t resolution;
  uint8_t i;
  uint8_t *res_idx = (uint8_t *) user_data;
  bool quit;
  enum eEditResult result = EDIT_ABORT;

  Dset_units_ignore_motion(adcSuperStructure.adcUserDatabase.units);

  /* Lookup current resolution */
  resolution = AdcResolutions[*res_idx];

  /* Edit until valid resolution entered */
  quit = false;
  while (!quit_setup_menus() && !quit)
  {
    result = edit_byte(&resolution, RESOLUTION_MAX, RESOLUTION_MIN, adcSuperStructure.adcUserDatabase.dpPos, 3, full_edit);
    if (EDIT_OK == result || EDIT_CHG == result)
    {
      for (i = 0; i < ADC_NUM_RESOLUTION; i++)
      {
        if (resolution == AdcResolutions[i])
        {
          if (i != *res_idx)
          {
            *res_idx = i;
          }
          quit = true;
        }
      }
      if (i >= ADC_NUM_RESOLUTION)
      {
        /* No match */
        block_entry();
      }
    }
    else if (EDIT_ABORT == result)
    {
      quit = true;
    }
  }
  return edit_result_to_menu_level(result);
}


/** Print the scale resolution */
static void to_string_resolution(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  AdcRdg rdg;
  uint8_t *res_idx = (uint8_t *) user_data;

  memset(&rdg, 0, sizeof(rdg));
  rdg.rdg = AdcResolutions[*res_idx];
  rdg.info.unitdp.bit.dp = adcSuperStructure.adcUserDatabase.dpPos;
  rdg.info.unitdp.bit.units = adcSuperStructure.adcUserDatabase.units;
  to_string_weight(&rdg, true, buffer, buflen);
}


/** Edit additive tare */
static enum eMenuLevel edit_additive_tare(bool full_edit, void *user_data, size_t index)
{
  long l;
  enum eEditResult result;

  l = adcSuperStructure.adcUserDatabase.addTare;
  result = edit_long(&l, true_full_scale(), 0L, adcSuperStructure.adcUserDatabase.dpPos, NUM_DIGITS, 0, full_edit);
  if (EDIT_CHG == result)
  {
    adcSuperStructure.adcUserDatabase.addTare = l;
  }
  return edit_result_to_menu_level(result);
}


/** Print the additive tare */
static void to_string_additive_tare(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  AdcRdg rdg;

  memset(&rdg, 0, sizeof(rdg));
  rdg.rdg = adcSuperStructure.adcUserDatabase.addTare;
  rdg.info.unitdp.bit.dp = adcSuperStructure.adcUserDatabase.dpPos;
  rdg.info.unitdp.bit.units = adcSuperStructure.adcUserDatabase.units;
  to_string_weight(&rdg, true, buffer, buflen);
}


/** Edit system units */
static enum eMenuLevel edit_units(bool full_edit, void *user_data, size_t index)
{
  enum eBuildUnits oldweunits;
  enum eBuildUnits weunits;
  uint8_t kbd;
  bool quit;
  enum eMenuLevel result = mlNum;

  oldweunits = xlat_adc_library_units_to_we2111(adcSuperStructure.adcUserDatabase.units);
  weunits = oldweunits;
  quit = false;
  while (!quit_setup_menus() && !quit)
  {
    Dputstr(build_units_menus[weunits]);
    Dset_units_ignore_motion(xlat_we2111_units_to_adc_library(weunits));

    kbd = get_key();
    switch (kbd)
    {
      case KEY_MENU_EXIT1:
      case KEY_MENU_EXIT2:
        result = mlQuit;
        quit = true;
        break;

      case KEY_EDIT_OK:
        if (full_edit && oldweunits != weunits)
        {
          adcSuperStructure.adcUserDatabase.units = xlat_we2111_units_to_adc_library(weunits);
        }
        quit = true;
        break;

      case KEY_EDIT_CANCEL1:
      case KEY_EDIT_CANCEL2:
        quit = true;
        break;

      case KEY_EDIT_UP:
        if (full_edit)
        {
          if (++weunits > NUM_UNITS - 1)
          {
            weunits = UNITS_NONE;
          }
        }
        else
        {
          access_denied();
        }
        break;

      case KEY_EDIT_DOWN:
        if (full_edit)
        {
          if (UNITS_NONE == weunits)
            weunits = NUM_UNITS - 1;
          else
            weunits--;
        }
        else
        {
          access_denied();
        }
        break;
    }
  }
  return result;
}


/** Print the units */
static void to_string_units(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  enum eBuildUnits weunits;

  weunits = xlat_adc_library_units_to_we2111(adcSuperStructure.adcUserDatabase.units);
  snprintf(buffer, buflen, "%s", build_units_menus[weunits]);
}


/** Edit sync cal (frequency) */
static enum eMenuLevel edit_sync_cal(bool full_edit, void *user_data, size_t index)
{
  /* Edit */
  return edit_result_to_menu_level(edit_register_data(&adcSuperStructure.adcUserDatabase.syncFreqIdx, 0, ADC_NUM_SYNC_FREQ - 1, sync_cal_setting_strs, full_edit));
}


/** Print the sync cal (frequency) */
static void to_string_sync_cal(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  snprintf(buffer, buflen, "%s Hz", sync_cal_setting_strs[adcSuperStructure.adcUserDatabase.syncFreqIdx]);
}


static enum eEditResult hi_res_mode(void)
{
  uint8_t kbd;
  uint32_t finish_time;
  bool trade_mode;
  bool timeout;
  bool old_hires;
  enum eEditResult result;

  /* Trade mode times out in 5 seconds */
  finish_time = OSTimeGet() + 5 * OS_TICKS_PER_SEC;
  trade_mode = (ADC_APPROVAL_MODE_NONE != adcSuperStructure.adcUserDatabase.approvalMode);

  /* Switch to hi res */
  old_hires = adcSuperStructure.adcUserDatabase.hiRes;
  if (!adcSuperStructure.adcUserDatabase.hiRes)
  {
    adcSuperStructure.adcUserDatabase.hiRes = true;
    AdcReInit(&adcSuperStructure, true, NULL, NULL, NULL);
  }

  /* Display weight temporarily in hi-res mode */
  freeze_display_and_annunciators(false);
  do
  {
    kbd = get_key();  /* get_kbd() yields */
    timeout = (trade_mode && OSTimeGet() >= finish_time);
  }
  while (!timeout && !quit_setup_menus() && (kbd == IDLE_CHG || kbd == NO_KEY));
  freeze_display_and_annunciators(true);

  /* No more hi-res */
  if (adcSuperStructure.adcUserDatabase.hiRes != old_hires)
  {
    adcSuperStructure.adcUserDatabase.hiRes = old_hires;
    AdcReInit(&adcSuperStructure, true, NULL, NULL, NULL);
  }

  /* Exit menus ?*/
  switch (kbd)
  {
    case KEY_MENU_EXIT1:
    case KEY_MENU_EXIT2:
      result = EDIT_MENU_QUIT;
      break;
    default:
      result = EDIT_OK;
      break;
  }
  return result;
}


static enum eMenuLevel test_edit_hires(bool full_edit, void *user_data, size_t index)
{
  uint8_t data;
  enum eEditResult result;

  if (!full_edit)
  {
    /* Safe setup will display hi-res here */
    result = hi_res_mode();
  }
  else
  {
    /* Full setup allow hi-res to be turned on and off */
    data = adcSuperStructure.adcUserDatabase.hiRes;
    result = edit_register_data(&data, 0, 1, off_on_setting, full_edit);
    if (EDIT_CHG == result)
    {
      adcSuperStructure.adcUserDatabase.hiRes = data;
    }
  }
  return edit_result_to_menu_level(result);
}


/** Convert the hires setting to a string */
static void to_string_hires(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  snprintf(buffer, buflen, "%s", off_on_setting[adcSuperStructure.adcUserDatabase.hiRes]);
}



/** Run checks after build menu run */
static bool build_menu_enter_exit(bool entry_not_exit, bool full_edit, void *user_data, bool edit_not_tostring)
{
  int32_t total_grads;

  /* On exit, run the total graduations check */
  if (edit_not_tostring && !entry_not_exit)
  {
    /* Display res hi/lo error if outside scale limits */
    total_grads = calculate_scale_total_graduations();
    if (total_grads > MAX_GRADS)
    {
      error_strings(res_str, hi_str);
    }
    else if (total_grads < MIN_GRADS)
    {
      error_strings(res_str, lo_str);
    }
  }
  return true;
}


static const struct s_menu_item sub_menu_build_type =    { .title = "TYPE",    .sub_menu = NULL, .edit_fn = edit_scale_type,    .to_string_fn = to_string_scale_type };
static const struct s_menu_item sub_menu_build_dp =      { .title = "DP",      .sub_menu = NULL, .edit_fn = edit_decimal_point, .to_string_fn = to_string_decimal_point };
static const struct s_menu_item sub_menu_build_cap1 =    { .title = "CAP1",    .sub_menu = NULL, .edit_fn = edit_capacity,      .to_string_fn = to_string_capacity,     .user_data = &adcSuperStructure.adcUserDatabase.fullScale[0]  };
static const struct s_menu_item sub_menu_build_res1 =    { .title = "E1",      .sub_menu = NULL, .edit_fn = edit_resolution,    .to_string_fn = to_string_resolution,   .user_data = &adcSuperStructure.adcUserDatabase.resolution[0] };
static const struct s_menu_item sub_menu_build_cap2 =    { .title = "CAP2",    .sub_menu = NULL, .edit_fn = edit_capacity,      .to_string_fn = to_string_capacity,     .user_data = &adcSuperStructure.adcUserDatabase.fullScale[1]  };
static const struct s_menu_item sub_menu_build_res2 =    { .title = "E2",      .sub_menu = NULL, .edit_fn = edit_resolution,    .to_string_fn = to_string_resolution,   .user_data = &adcSuperStructure.adcUserDatabase.resolution[1] };
static const struct s_menu_item sub_menu_build_addtare = { .title = "AD.TARE", .sub_menu = NULL, .edit_fn = edit_additive_tare, .to_string_fn = to_string_additive_tare };
static const struct s_menu_item sub_menu_build_units =   { .title = "UNITS",   .sub_menu = NULL, .edit_fn = edit_units,         .to_string_fn = to_string_units };
static const struct s_menu_item sub_menu_special_sync =  { .title = "SYNC",    .sub_menu = NULL, .edit_fn = edit_sync_cal,      .to_string_fn = to_string_sync_cal };
static const struct s_menu_item sub_menu_test_hires =    { .title = "HI.RES",  .sub_menu = NULL, .edit_fn = test_edit_hires,    .to_string_fn = to_string_hires };


static const struct s_menu_item *const menu_build_list[] =
{
  &sub_menu_build_type,
  &sub_menu_build_dp,
  &sub_menu_build_cap1,
  &sub_menu_build_res1,
  &sub_menu_build_cap2,
  &sub_menu_build_res2,
  &sub_menu_build_addtare,
  &sub_menu_build_units,
  &sub_menu_special_sync,
  &sub_menu_test_hires,
  NULL
};


/** Build menu item */
const struct s_menu_item menu_build =
{
  .title = "BUILD",
  .sub_menu = menu_build_list,
  .menu_enter_exit = build_menu_enter_exit
};
