/*
 * iface_changelog.c
 *
 *  Created on: May 14, 2012
 *      Author: ian
 */
#include <stdio.h>
#include <string.h>
#include <rindb/db_changelog.h>
#include <rinweigh/adcdatatypes.h>
#include "editor.h"
#include "iface_sd.h"
#include "iface.h"
#include "display.h"
#include "system.h"
#include "app_db_changelog.h"
#include "iface_cal.h"
#include "app_db_user.h"
#include "app_db.h"
#include "commonstrings.h"
#include "dbprint.h"

#include "iface_changelog.h"


/** Display change log status (% full) */
static enum eMenuLevel changelog_status_menu(bool full_edit, void *user_data, size_t index)
{
  size_t cl_size;
  size_t cl_maxsize;
  size_t percent;
  AdcRdg rdg;

  if (NULL == app_changelog)
  {
    error_strings(no_str, chg_log_str);
    return mlNum;
  }

  /* Get sizes */
  if (dbeSuccess != db_changelog_get_maximum_size(app_changelog, &cl_maxsize) ||
      db_changelog_get_size(app_changelog, &cl_size))
  {
    error_strings(chg_log_str, err_str);
    return mlNum;
  }

  /* Work out percent full * 10 */
  percent = (cl_size * 1000) / cl_maxsize;
  memset(&rdg, 0, sizeof(rdg));
  rdg.rdg = percent;
  rdg.info.unitdp.bit.dp = 1;
  Dshow_rdg_info(&rdg, 0, false, 0);
  delay(WECFG_DELAY_FULL_OPERATING_MILLISECONDS);

  Dputstr("PCT.FUL");
  delay(WECFG_DELAY_FULL_OPERATING_MILLISECONDS);
  return mlNum;
}


/** Convert the change log status (% full) to a string */
static void to_string_changelog_status(bool full_edit, void *user_data, size_t index, char *buffer, size_t buflen)
{
  size_t cl_size;
  size_t cl_maxsize;
  size_t percent;

  if (NULL == app_changelog)
  {
    snprintf(buffer, buflen, "No changelog");
    return;
  }

  /* Get sizes */
  if (dbeSuccess != db_changelog_get_maximum_size(app_changelog, &cl_maxsize) ||
      db_changelog_get_size(app_changelog, &cl_size))
  {
    snprintf(buffer, buflen, "Changelog error");
    return;
  }

  /* Work out percent full * 10 */
  percent = (cl_size * 1000) / cl_maxsize;
  snprintf(buffer, buflen, "%u.%1u%% full", (unsigned) percent / 10, (unsigned) percent % 10);
}


/** Clear the change log */
static enum eMenuLevel changelog_clear_menu(bool full_edit, void *user_data, size_t index)
{
  enum eDBError res;

  if (NULL == app_changelog)
  {
    error_strings(no_str, chg_log_str);
  }
  else if (!full_edit)
  {
    access_denied();
  }
  else if (ask(cont_str, false))
  {
    res = db_changelog_clear(app_changelog, time(NULL), APP_DB_U_NAME, APP_DB_CLMASK_DATABASE);
    if (dbeSuccess != res)
    {
      error_strings(chg_log_str, "CLR.ERR");
    }
    else
    {
      Dputstr(done_str);
      delay(WECFG_DELAY_FULL_OPERATING_MILLISECONDS);
    }
  }
  return mlNum;
}


static const struct s_menu_item sub_menu_changelog_status = { .title = "STATUS", .sub_menu = NULL, .edit_fn = changelog_status_menu, .to_string_fn = to_string_changelog_status };
static const struct s_menu_item sub_menu_changelog_clear =  { .title = "CLEAR",  .sub_menu = NULL, .edit_fn = changelog_clear_menu,  .to_string_fn = to_string_no_setting };


static const struct s_menu_item *const sub_menu_changelog[] =
{
  &sub_menu_changelog_status,
  &sub_menu_changelog_clear,
  NULL
};


/** Changelog menu item */
const struct s_menu_item menu_changelog =
{
  .title = "CHG.LOG",
  .sub_menu = sub_menu_changelog
};
