/*
 * iface_clock.c
 *
 *  Created on: 01/04/2011
 *      Author: ian
 */
#include <stdio.h>
#include <rinweigh/adcdatabase.h>
#include "sysclock.h"
#include "editor.h"
#include "database.h"
#include "iface_sd.h"
#include "iface.h"
#include "adcthread.h"
#include "commonstrings.h"
#include "db_clock.h"
#include "app_db_user.h"

#include "iface_clock.h"


/** Set the time */
static enum eMenuLevel do_time_cal(bool full_edit, void *user_data, size_t index)
{
  long tmpl;
  t_datetime dt;
  enum eEditResult result = EDIT_ABORT;

  for ( ; !application_quit_forced(); )
  {
    if (!GetDateTime(&dt))
    {
      break;
    }

    /* Make hours and minutes into a number to edit */
    tmpl = dt.hour * 100 + dt.minute;

    /* Edit hours and minutes */
    result = edit_long(&tmpl, 2359L, 0L, 2, 4, 0, ALLOW_SAFE);
    if (EDIT_CHG == result || EDIT_OK == result)
    {
      /* Read date and time again in case of change */
      if (!GetDateTime(&dt))
      {
        break;
      }

      /* Set hour and minute from what was edited */
      dt.hour = (uint8_t) (tmpl / 100);
      dt.minute = (uint8_t) (tmpl % 100);
      dt.second = 0;

      /* Some checks */
      if (dt.minute > 59 || dt.hour > 23)
      {
        block_entry();
      }
      else
      {
        /* Save it */
        (void) SetDateTime(&dt);
        break;
      }
    }
    else
    {
      break;
    }
  }
  return edit_result_to_menu_level(result);
}


/** Convert the time to a string based on time format from DB TODO should it take an argument for format? */
void to_string_time(char *buffer, size_t buflen, const t_datetime *datetime)
{
  unsigned hour;
  const char *am_pm;

  hour = datetime->hour;

  switch (db_app_user.time_fmt)
  {
    case TIME_FMT_12H:
      am_pm = (hour >= 12) ? "PM" : "AM";
      hour %= 12;
      if (hour == 0)
        hour = 12;
      break;

    case TIME_FMT_24H:
    default:
      am_pm = "";
      break;
  }

  snprintf(buffer, buflen, "%02u:%02u:%02u%s", hour, (unsigned) datetime->minute, (unsigned) datetime->second, am_pm);
}


/** Convert the time to a string */
static void to_string_time_internal(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  t_datetime dt;

  GetDateTime(&dt);
  to_string_time (buffer, buflen, &dt);
}


/** Set the date */
static enum eMenuLevel do_date_cal(bool full_edit, void *user_data, size_t index)
{
  long daymonth;
  long year;
  t_datetime dt;
  enum eEditResult result = EDIT_ABORT;

  for ( ; ; )
  {
    /* Get current date */
    if (!GetDateTime(&dt))
    {
      break;
    }

    /* Make days and months into number to edit */
    daymonth = dt.date * 100 + dt.month;

    /* Edit days & months */
    result = edit_long(&daymonth, 3112L, 101L, 2, 4, 0, ALLOW_SAFE);
    if (EDIT_CHG == result || EDIT_OK == result)
    {
      /* Now edit year */
      year = dt.year;
      result = edit_long(&year, 2097L, 1998L, 0, 4, 0, ALLOW_SAFE);
      if (EDIT_CHG == result || EDIT_OK == result)
      {
        /* Get current date */
        if (!GetDateTime(&dt))
        {
          break;
        }
        dt.date = (uint8_t) (daymonth / 100);
        dt.month = (uint8_t) (daymonth % 100);
        dt.year = (uint16_t) year;

        /* Check date */
        if (!check_date(dt.date, dt.month, dt.year))
        {
          block_entry();
        }
        else
        {
          (void) SetDateTime(&dt);
          break;
        }
      }
      else
      {
        break;
      }
    }
    else
    {
      break;
    }
  }
  return edit_result_to_menu_level(result);
}


/** Convert the date to a string based on date format from DB TODO should it take an argument for format? */
void to_string_date(char *buffer, size_t buflen, const t_datetime *datetime)
{
  unsigned num[3];
  unsigned len[3];
  unsigned year_digits;
  unsigned year;

  year = datetime->year;
  year_digits = 4;

  /* Check and correct year data. This is easiest done before order is assigned. */
  if (db_app_user.date_fmt <= DATE_FMT_YYYY_MM_DD)
  {
    if (year < 100)
      year += 2000;
  }
  else
  {
    year_digits = 2;
    year %= 100;
  }

  switch (db_app_user.date_fmt)
  {
    default:
    case DATE_FMT_DD_MM_YYYY:
    case DATE_FMT_DD_MM_YY:
      num[0] = datetime->date;
      len[0] = 2;
      num[1] = datetime->month;
      len[1] = 2;
      num[2] = year;
      len[2] = year_digits;
      break;

    case DATE_FMT_MM_DD_YYYY:
    case DATE_FMT_MM_DD_YY:
      num[0] = datetime->month;
      len[0] = 2;
      num[1] = datetime->date;
      len[1] = 2;
      num[2] = year;
      len[2] = year_digits;
      break;

    case DATE_FMT_YYYY_MM_DD:
    case DATE_FMT_YY_MM_DD:
      num[0] = year;
      len[0] = year_digits;
      num[1] = datetime->month;
      len[1] = 2;
      num[2] = datetime->date;
      len[2] = 2;
      break;
  }

  snprintf(buffer, buflen, "%0*u/%0*u/%0*u", len[0], num[0], len[1], num[1], len[2], num[2]);
}


/** Convert the date to a string */
static void to_string_date_internal(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  t_datetime dt;

  GetDateTime(&dt);
  to_string_date(buffer, buflen, &dt);
}


/** Enable QA setting */
static enum eMenuLevel edit_qa_enable(bool full_edit, void *user_data, size_t index)
{
  return edit_result_to_menu_level(edit_register_data(&adcSuperStructure.adcUserDatabase.qaEnable, 0, 1, off_on_setting, full_edit));
}


/** Convert the date to a string */
static void to_string_qa_enable(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  t_datetime dt;

  GetDateTime(&dt);
  snprintf(buffer, buflen, "%s", off_on_setting[adcSuperStructure.adcUserDatabase.qaEnable]);
}


/** Set the QA date */
static enum eMenuLevel set_qa_date(bool full_edit, void *user_data, size_t index)
{
  long ldaymonth;
  long lyear;
  uint8_t date;
  uint8_t month;
  enum eEditResult result;

  for ( ; ; )
  {
    date = adcSuperStructure.adcUserDatabase.qaDay;
    month = adcSuperStructure.adcUserDatabase.qaMonth;
    lyear = adcSuperStructure.adcUserDatabase.qaYear;

    ldaymonth = date * 100 + month;
    result = edit_long(&ldaymonth, 3112L, 101L, 2, 4, 0, full_edit);
    if (EDIT_CHG == result || EDIT_OK == result ||
        (!full_edit && EDIT_MENU_QUIT != result))
    {
      result = edit_long(&lyear, 2097L, 1998L, 0, 4, 0, full_edit);
      if (EDIT_CHG == result || EDIT_OK == result)
      {
        date = (uint8_t) (ldaymonth / 100);
        month = (uint8_t) (ldaymonth % 100);
        if (!check_date(date, month, lyear))
        {
          block_entry();
        }
        else
        {
          adcSuperStructure.adcUserDatabase.qaDay = date;
          adcSuperStructure.adcUserDatabase.qaMonth = month;
          adcSuperStructure.adcUserDatabase.qaYear = lyear;
          break;
        }
      }
      else
      {
        break;
      }
    }
    else
    {
      break;
    }
  }
  return edit_result_to_menu_level(result);
}


/** Convert the qa date to a string */
static void to_string_qa_date(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  snprintf(buffer, buflen, "%02u/%02u/%04lu",
           (unsigned) adcSuperStructure.adcUserDatabase.qaDay,
           (unsigned) adcSuperStructure.adcUserDatabase.qaMonth,
           (unsigned long) adcSuperStructure.adcUserDatabase.qaYear);
}


/** Edit time format setting */
static enum eMenuLevel edit_time_format(bool full_edit, void *user_data, size_t index)
{
  return edit_result_to_menu_level(edit_register_data(&db_app_user.time_fmt, 0, NUM_TIME_FMTS - 1, time_format_settings, full_edit));
}


/** Convert the time format to a string */
static void to_string_time_format(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  snprintf(buffer, buflen, "%s", time_format_settings[db_app_user.time_fmt]);
}


/** Edit date format setting */
static enum eMenuLevel edit_date_format(bool full_edit, void *user_data, size_t index)
{
  return edit_result_to_menu_level(edit_register_data(&db_app_user.date_fmt, 0, NUM_DATE_FMTS - 1, date_format_settings, full_edit));
}


/** Convert the date format to a string */
static void to_string_date_format(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  snprintf(buffer, buflen, "%s", date_format_settings[db_app_user.date_fmt]);
}


static const struct s_menu_item sub_menu_clock_time =     { .title = "TIME",    .sub_menu = NULL, .edit_fn = do_time_cal,      .to_string_fn = to_string_time_internal };
static const struct s_menu_item sub_menu_clock_date =     { .title = "DATE",    .sub_menu = NULL, .edit_fn = do_date_cal,      .to_string_fn = to_string_date_internal };
static const struct s_menu_item sub_menu_clock_qaopt =    { .title = "QA.OPT",  .sub_menu = NULL, .edit_fn = edit_qa_enable,   .to_string_fn = to_string_qa_enable };
static const struct s_menu_item sub_menu_clock_qadate =   { .title = "QA.DATE", .sub_menu = NULL, .edit_fn = set_qa_date,      .to_string_fn = to_string_qa_date };
static const struct s_menu_item sub_menu_clock_time_fmt = { .title = "T.FMT",   .sub_menu = NULL, .edit_fn = edit_time_format, .to_string_fn = to_string_time_format };
static const struct s_menu_item sub_menu_clock_date_fmt = { .title = "D.FMT",   .sub_menu = NULL, .edit_fn = edit_date_format, .to_string_fn = to_string_date_format };


static const struct s_menu_item *const sub_menu_clock[] =
{
  &sub_menu_clock_time,
  &sub_menu_clock_date,
  &sub_menu_clock_qaopt,
  &sub_menu_clock_qadate,
  &sub_menu_clock_time_fmt,
  &sub_menu_clock_date_fmt,
  NULL
};


/** Clock menu item */
const struct s_menu_item menu_clock =
{
  .title = "CLOCK",
  .sub_menu = sub_menu_clock
};
