/*
 * iface_dsd.c
 *
 *  Created on: 14/03/2012
 *      Author: ian
 */
#include <stdio.h>
#include <string.h>
#include <rinweigh/adcdsd.h>
#include "editor.h"
#include "iface_sd.h"
#include "iface.h"
#include "database.h"
#include "dsd.h"
#include "display.h"
#include "system.h"
#include "iface_cal.h"
#include "app_db_user.h"
#include "commonstrings.h"
#include "dbprint.h"

#include "iface_dsd.h"


/** Enable or disable the DSD */
static enum eMenuLevel dsd_enable_menu(bool full_edit, void *user_data, size_t index)
{
  return edit_result_to_menu_level(edit_register_data(&db_app_user.dsd_enable, 0, 1, off_on_setting, full_edit));
}


/** Convert DSD enable to string */
static void to_string_dsd_enable(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  snprintf(buffer, buflen, "%s", off_on_setting[db_app_user.dsd_enable]);
}


/** Display DSD status */
static enum eMenuLevel dsd_status_menu(bool full_edit, void *user_data, size_t index)
{
  int32u num_records;
  int32u capacity;
  AdcRdg rdg;

  if (!dsd_is_running())
  {
    error_strings(no_str, dsd_str);
  }
  else
  {
    if (ADC_DSD_SUCCESS != AdcDSDGetNumberRecords(wedsd, true, &num_records) ||
        ADC_DSD_SUCCESS != AdcDSDGetCapacity(wedsd, true, &capacity))
    {
      error_strings(dsd_str, err_str);
    }
    else
    {
      Dputstr("USED");
      delay(WECFG_DELAY_FULL_OPERATING_MILLISECONDS);
      memset(&rdg, 0, sizeof(rdg));
      rdg.rdg = num_records;
      Dshow_rdg_info(&rdg, 0, false, 0);
      delay(WECFG_DELAY_FULL_OPERATING_MILLISECONDS);

      Dputstr("TOTAL");
      delay(WECFG_DELAY_FULL_OPERATING_MILLISECONDS);
      memset(&rdg, 0, sizeof(rdg));
      rdg.rdg = capacity;
      Dshow_rdg_info(&rdg, 0, false, 0);
      delay(WECFG_DELAY_FULL_OPERATING_MILLISECONDS);
    }
  }
  return mlNum;
}


/** Convert DSD status to string */
static void to_string_dsd_status(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  int32u num_records;
  int32u capacity;

  if (!dsd_is_running())
  {
    snprintf(buffer, buflen, "No DSD");
  }
  else
  {
    if (ADC_DSD_SUCCESS != AdcDSDGetNumberRecords(wedsd, true, &num_records) ||
        ADC_DSD_SUCCESS != AdcDSDGetCapacity(wedsd, true, &capacity))
    {
      snprintf(buffer, buflen, "DSD Error");
    }
    else
    {
      snprintf(buffer, buflen, "%lu used of %lu total", (unsigned long) num_records, (unsigned long) capacity);
    }
  }
}


/** Enable or disable auto purge of the DSD */
static enum eMenuLevel dsd_autopurge_menu(bool full_edit, void *user_data, size_t index)
{
  return edit_result_to_menu_level(edit_register_data(&db_app_user.dsd_auto_purge, 0, 1, off_on_setting, full_edit));
}


/** Convert DSD auto purge to string */
static void to_string_dsd_autopurge(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  snprintf(buffer, buflen, "%s", off_on_setting[db_app_user.dsd_auto_purge]);
}


/** Purge the DSD now */
static enum eMenuLevel dsd_purge_menu(bool full_edit, void *user_data, size_t index)
{
  if (ask(cont_str, false))
  {
    if (!dsd_purge())
    {
      error_strings(dsd_err_str, "PURGE");
    }
    else
    {
      Dputstr(done_str);
      delay(WECFG_DELAY_FULL_OPERATING_MILLISECONDS);
    }
  }
  return mlNum;
}


/** Initialise the DSD */
static enum eMenuLevel dsd_initialise_menu(bool full_edit, void *user_data, size_t index)
{
  if (!db_app_user.dsd_enable)
  {
    error_strings(no_str, dsd_str);
  }
  else if (!full_edit)
  {
    access_denied();
  }
  else if (ask(cont_str, false))
  {
    if (!dsd_recreate_file())
    {
      error_strings(dsd_err_str, "INIT");
    }
    else
    {
      Dputstr(done_str);
      delay(WECFG_DELAY_FULL_OPERATING_MILLISECONDS);
    }
  }
  return mlNum;
}


static const struct s_menu_item sub_menu_dsd_enable =    { .title = "ENABLE",  .sub_menu = NULL, .edit_fn = dsd_enable_menu,     .to_string_fn = to_string_dsd_enable };
static const struct s_menu_item sub_menu_dsd_init =      { .title = "INIT",    .sub_menu = NULL, .edit_fn = dsd_initialise_menu, .to_string_fn = to_string_no_setting };
static const struct s_menu_item sub_menu_dsd_status =    { .title = "STATUS",  .sub_menu = NULL, .edit_fn = dsd_status_menu,     .to_string_fn = to_string_dsd_status };
static const struct s_menu_item sub_menu_dsd_autopurge = { .title = "A.PURGE", .sub_menu = NULL, .edit_fn = dsd_autopurge_menu,  .to_string_fn = to_string_dsd_autopurge };
static const struct s_menu_item sub_menu_dsd_purge =     { .title = "PURGE",   .sub_menu = NULL, .edit_fn = dsd_purge_menu,      .to_string_fn = to_string_no_setting };


static const struct s_menu_item *const sub_menu_dsd[] =
{
  &sub_menu_dsd_enable,
  &sub_menu_dsd_init,
  &sub_menu_dsd_status,
  &sub_menu_dsd_autopurge,
  &sub_menu_dsd_purge,
  NULL
};


/** DSD menu item */
const struct s_menu_item menu_dsd =
{
  .title = "DSD",
  .sub_menu = sub_menu_dsd
};
