/*
 * iface_option.c
 *
 *  Created on: 05/05/2011
 *      Author: ian
 */
#include <stdio.h>
#include <string.h>
#include <rinweigh/adcdatabase.h>
#include "database.h"
#include "display.h"
#include "editor.h"
#include "iface.h"
#include "db_option.h"
#include "iface_sd.h"
#include "adcthread.h"
#include "app_db_user.h"
#include "commonstrings.h"
#include "dbprint.h"

#include "iface_option.h"


/** Edit the operation mode/approval mode/use setting */
static enum eMenuLevel edit_approval_mode(bool full_edit, void *user_data, size_t index)
{
  uint8_t am_idx;
  enum eEditResult result;

  /* Translate adc library setting to we2111 approval mode index */
  am_idx = (uint8_t) xlat_adc_library_approval_mode_to_we2111(adcSuperStructure.adcUserDatabase.approvalMode);

  /* Edit the setting */
  result = edit_register_data(&am_idx, 0, WE_APPROVAL_MODE_NUM - 1, we_approval_mode_strings, full_edit);
  if (EDIT_CHG == result)
  {
    /* Changed, so translate back and save */
    adcSuperStructure.adcUserDatabase.approvalMode = xlat_we2111_approval_mode_to_adc_library(am_idx);
  }
  return edit_result_to_menu_level(result);
}


/** Convert the approval mode to a string */
static void to_string_approval_mode(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  enum eWeApprovalMode am;

  am = xlat_adc_library_approval_mode_to_we2111(adcSuperStructure.adcUserDatabase.approvalMode);
  snprintf(buffer, buflen, "%s", we_approval_mode_strings[am]);
}


/** Edit the filter/averaging setting */
static enum eMenuLevel edit_filter_setting(bool full_edit, void *user_data, size_t index)
{
  uint8_t av_idx;
  enum eEditResult result;

  /* Translate adc library setting to we2111 averaging index */
  av_idx = (uint8_t) xlat_adc_library_average_to_we2111(adcSuperStructure.adcUserDatabase.average);

  /* Edit the setting */
  result = edit_register_data(&av_idx, 0, NUM_AVERAGE_OPT - 1, average_setting, full_edit);
  if (EDIT_CHG == result)
  {
    /* Changed, so translate back and save */
    adcSuperStructure.adcUserDatabase.average = xlat_we2111_average_to_adc_library(av_idx);
  }
  return edit_result_to_menu_level(result);
}


/** Convert the filter setting to a string */
static void to_string_filter_setting(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  enum eAverageOptions ao;

  ao = xlat_adc_library_average_to_we2111(adcSuperStructure.adcUserDatabase.average);
  snprintf(buffer, buflen, "%s", average_setting[ao]);
}


/** Edit the jitter setting */
static enum eMenuLevel edit_jitter_setting(bool full_edit, void *user_data, size_t index)
{
  /* Edit the setting */
  return edit_result_to_menu_level(edit_register_data(&adcSuperStructure.adcUserDatabase.jitter, 0, ADC_NUM_JITTER - 1, jitter_setting, full_edit));
}


/** Convert the jitter setting to a string */
static void to_string_jitter_setting(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  snprintf(buffer, buflen, "%s", jitter_setting[adcSuperStructure.adcUserDatabase.jitter]);
}


/* Edit the divisions in time period of movement that indicate motion */
static enum eMenuLevel edit_motion_detection(bool full_edit, void *user_data, size_t index)
{
  /* Edit the setting */
  return edit_result_to_menu_level(edit_register_data(&adcSuperStructure.adcUserDatabase.motion, 0, ADC_NUM_MOTION_DETECT - 1, motion_setting, full_edit));
}


/** Convert the motion detection setting to a string */
static void to_string_motion_detection(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  snprintf(buffer, buflen, "%s d", motion_setting[adcSuperStructure.adcUserDatabase.motion]);
}


/** Edit the auto zero/zero on startup setting */
static enum eMenuLevel edit_auto_zero(bool full_edit, void *user_data, size_t index)
{
  /* Edit the setting */
  return edit_result_to_menu_level(edit_register_data(&adcSuperStructure.adcUserDatabase.zeroOnStart, 0, APP_DB_U_ADCZEROONSTART_MAX, off_on_setting, full_edit));
}


/** Convert the auto zero setting to a string */
static void to_string_auto_zero(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  snprintf(buffer, buflen, "%s", off_on_setting[adcSuperStructure.adcUserDatabase.zeroOnStart]);
}


/** Edit the zero tracking setting */
static enum eMenuLevel edit_zero_tracking(bool full_edit, void *user_data, size_t index)
{
  /* Edit the setting */
  return edit_result_to_menu_level(edit_register_data(&adcSuperStructure.adcUserDatabase.zeroTracking, 0, APP_DB_U_ADCZEROTRACK_MAX, zero_tracking_setting, full_edit));
}


/** Convert the zero tracking setting to a string */
static void to_string_zero_tracking(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  snprintf(buffer, buflen, "%s d", zero_tracking_setting[adcSuperStructure.adcUserDatabase.zeroTracking]);
}


/** Edit the zero range setting */
static enum eMenuLevel edit_zero_range(bool full_edit, void *user_data, size_t index)
{
  uint8_t zr;
  enum eEditResult result;

  /* Convert from ADC library setting */
  zr = xlat_adc_library_zero_range_to_we2111(adcSuperStructure.adcUserDatabase.rangeOfZero);

  /* Edit the setting */
  result = edit_register_data(&zr, 0, WE21_NUM_ZERO_RANGE - 1, we21_zero_range_setting, full_edit);
  if (EDIT_CHG == result)
  {
    /* Save the setting back to the adc library */
    adcSuperStructure.adcUserDatabase.rangeOfZero = xlat_we2111_zero_range_to_adc_library(zr);
  }
  return edit_result_to_menu_level(result);
}


/** Convert the zero range setting to a string */
static void to_string_zero_range(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  enum eAdcZeroRanges zr;

  zr = xlat_adc_library_zero_range_to_we2111(adcSuperStructure.adcUserDatabase.rangeOfZero);
  snprintf(buffer, buflen, "%s %%", we21_zero_range_setting[zr]);
}


/** Edit the zero range setting */
static enum eMenuLevel edit_zero_band(bool full_edit, void *user_data, size_t index)
{
  long l;
  enum eEditResult result;

  Dset_units_ignore_motion(adcSuperStructure.adcUserDatabase.units);
  l = adcSuperStructure.adcUserDatabase.zeroBand;
  result = edit_long(&l, APP_DB_U_ADCZEROBAND_MAX, APP_DB_U_ADCZEROBAND_MIN, adcSuperStructure.adcUserDatabase.dpPos, NUM_DIGITS, 0, full_edit);
  if (EDIT_CHG == result)
  {
    adcSuperStructure.adcUserDatabase.zeroBand = l;
  }
  return edit_result_to_menu_level(result);
}


/** Convert the zero band to a string */
static void to_string_zero_band(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  AdcRdg rdg;

  memset(&rdg, 0, sizeof(rdg));
  rdg.rdg = adcSuperStructure.adcUserDatabase.zeroBand;
  rdg.info.unitdp.bit.dp = adcSuperStructure.adcUserDatabase.dpPos;
  rdg.info.unitdp.bit.units = adcSuperStructure.adcUserDatabase.units;
  to_string_weight(&rdg, true, buffer, buflen);
}


/** Edit the sense check setting */
static enum eMenuLevel edit_sense_check(bool full_edit, void *user_data, size_t index)
{
  /* Edit the setting */
  return edit_result_to_menu_level(edit_register_data(&db_app_user.sense_check, 0, 1, off_on_setting, true));
}


/** Convert the sense check setting to a string */
static void to_string_sense_check(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  snprintf(buffer, buflen, "%s", off_on_setting[db_app_user.sense_check]);
}


static const struct s_menu_item sub_menu_option_use =         { .title = "USE",     .sub_menu = NULL, .edit_fn = edit_approval_mode,    .to_string_fn = to_string_approval_mode };
static const struct s_menu_item sub_menu_option_filter =      { .title = "FILTER",  .sub_menu = NULL, .edit_fn = edit_filter_setting,   .to_string_fn = to_string_filter_setting };
static const struct s_menu_item sub_menu_option_jitter =      { .title = "JITTER",  .sub_menu = NULL, .edit_fn = edit_jitter_setting,   .to_string_fn = to_string_jitter_setting };
static const struct s_menu_item sub_menu_option_motion =      { .title = "MOTION",  .sub_menu = NULL, .edit_fn = edit_motion_detection, .to_string_fn = to_string_motion_detection };
static const struct s_menu_item sub_menu_option_autozero =    { .title = "AUTO.Z",  .sub_menu = NULL, .edit_fn = edit_auto_zero,        .to_string_fn = to_string_auto_zero };
static const struct s_menu_item sub_menu_option_zerotrack =   { .title = "Z.TRACK", .sub_menu = NULL, .edit_fn = edit_zero_tracking,    .to_string_fn = to_string_zero_tracking };
static const struct s_menu_item sub_menu_option_zerorange =   { .title = "Z.RANGE", .sub_menu = NULL, .edit_fn = edit_zero_range,       .to_string_fn = to_string_zero_range };
static const struct s_menu_item sub_menu_option_zeroband =    { .title = "Z.BAND",  .sub_menu = NULL, .edit_fn = edit_zero_band,        .to_string_fn = to_string_zero_band };
static const struct s_menu_item sub_menu_option_sense_check = { .title = "SENS.CH", .sub_menu = NULL, .edit_fn = edit_sense_check,      .to_string_fn = to_string_sense_check };


static const struct s_menu_item *const sub_menu_option[] =
{
  &sub_menu_option_use,
  &sub_menu_option_filter,
  &sub_menu_option_jitter,
  &sub_menu_option_motion,
  &sub_menu_option_autozero,
  &sub_menu_option_zerotrack,
  &sub_menu_option_zerorange,
  &sub_menu_option_zeroband,
  &sub_menu_option_sense_check,
  NULL
};


/** Option menu item */
const struct s_menu_item menu_option =
{
  .title = "OPTION",
  .sub_menu = sub_menu_option
};
