/*
 * iface_rinblocks_modules.c
 *
 *  Created on: 25 Jan. 2019
 *      Author: ian
 */
#include <stdio.h>
#include <rinblock/rinblock.h>
#include "edit_string.h"
#include "editor.h"
#include "iface_sd.h"

#include "iface_rinblocks.h"


/** Convert the module type to a string */
static void to_string_module_type(bool full_edit, void *user_data, size_t index, char *buffer, size_t buflen)
{
  struct t_rinblock *rinblock = user_data;
  unsigned m5_model_number;

  /*
   * Because we use the module code from the M4xxx on the M5xxx modules, but
   * label the module as M5xxx, we translate here to prevent user confusion
   */
  switch (rinblock->tag.model_number)
  {
    case RB_MODEL_NUMBER_8DIO:
    case RB_MODEL_NUMBER_4BUTTON:
    case RB_MODEL_NUMBER_AOUT:
    case RB_MODEL_NUMBER_232_232:
    case RB_MODEL_NUMBER_232_485:
    case RB_MODEL_NUMBER_485_485:
    case RB_MODEL_NUMBER_DSD: /* Not supported, but anyway... */
    case RB_MODEL_NUMBER_16DIO:
    case RB_MODEL_NUMBER_24DIO:
    case RB_MODEL_NUMBER_32DIO:
    case RB_MODEL_NUMBER_TILT: /* Not supported, but anyway... */
    case RB_MODEL_NUMBER_ETHERNET: /* Not supported, but anyway... */
    case RB_MODEL_NUMBER_WIFI: /* Doesn't exist, not supported */
    case RB_MODEL_NUMBER_ADVANCED_ETHERNET: /* Not supported, but anyway... */
    case RB_MODEL_NUMBER_AIN: /* Doesn't technically exist */
    case RB_MODEL_NUMBER_2AOUT:
    case RB_MODEL_NUMBER_3AOUT:
    case RB_MODEL_NUMBER_4AOUT:
      m5_model_number = rinblock->tag.model_number + 1000;
      break;
    default:
      m5_model_number = rinblock->tag.model_number;
      break;
  }

  snprintf(buffer, buflen, "M%u", m5_model_number);
}


/** View the module type */
static enum eMenuLevel view_module_type(bool full_edit, void *user_data, size_t index)
{
  enum eEditResult result;
  char module_type_string[20];

  to_string_module_type(full_edit, user_data, index, module_type_string, 20);
  result = view_printable_string(module_type_string);
  return edit_result_to_menu_level(result);
}


/** Convert the module serial number to a string */
static void to_string_module_serialno(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  struct t_rinblock *rinblock = user_data;

  snprintf(buffer, buflen, "%lu", (unsigned long) rinblock->tag.serial_number);
}


/** View the module serial number */
static enum eMenuLevel view_module_serialno(bool full_edit, void *user_data, size_t index)
{
  enum eEditResult result;
  char module_type_string[20];

  to_string_module_serialno(full_edit, user_data, index, module_type_string, 20);
  result = view_printable_string(module_type_string);
  return edit_result_to_menu_level(result);
}


/** Convert the module software version to a string */
static void to_string_module_sw_version(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  struct t_rinblock *rinblock = user_data;

  snprintf(buffer, buflen, "%u.%02u", (unsigned) rinblock->tag.software_version / 100, (unsigned) rinblock->tag.software_version % 100);
}


/** View the module serial number */
static enum eMenuLevel view_module_sw_version(bool full_edit, void *user_data, size_t index)
{
  enum eEditResult result;
  struct t_rinblock *rinblock = user_data;
  long sw_version = rinblock->tag.software_version;

  result = edit_long(&sw_version, 0, 999999, 2, 3, 0, false);
  return edit_result_to_menu_level(result);
}


/** Check if we want to run the module slot menu */
static bool sub_menu_operator_module_slot_enter_exit(bool entry_not_exit, bool full_edit, void *user_data, bool edit_not_tostring)
{
  struct t_rinblock *rinblock = user_data;
  bool can_run;

  if (entry_not_exit)
  {
    can_run = false;
    if (!rinblock->present)
    {
      error_strings("NOT", "PRESNT");
    }
    else if (!rinblock->programmed)
    {
      error_strings("NOT", "PROG");
    }
    else if (!rinblock->running)
    {
      error_strings("NOT", "RUN");
    }
    else
    {
      can_run = true;
    }
    return can_run;
  }
  else
  {
    /* Can exit */
    return true;
  }
}


static const struct s_menu_item sub_menu_operator_module_type =          { .title = "TYPE",   .sub_menu = NULL, .edit_fn = view_module_type,       .to_string_fn = to_string_module_type,       .user_data_from_menu_above = true };
static const struct s_menu_item sub_menu_operator_module_serial_number = { .title = "SER.NO", .sub_menu = NULL, .edit_fn = view_module_serialno,   .to_string_fn = to_string_module_serialno,   .user_data_from_menu_above = true };
static const struct s_menu_item sub_menu_operator_module_version =       { .title = "SW.VER", .sub_menu = NULL, .edit_fn = view_module_sw_version, .to_string_fn = to_string_module_sw_version, .user_data_from_menu_above = true };
/* static const struct s_menu_item sub_menu_operator_module_status =        { .title = "STATUS", .sub_menu = NULL, .edit_fn = view_module_status,     .to_string_fn = to_string_module_status,     .user_data_from_menu_above = true }; */


static const struct s_menu_item *const menu_operator_module_slot[] =
{
  &sub_menu_operator_module_type,
  &sub_menu_operator_module_serial_number,
  &sub_menu_operator_module_version,
/*  &sub_menu_operator_module_status, */
  NULL
};


/* Rinblock 2 is C520 slot 1, Rinblock 3 is C520 slot 2 */
static const struct s_menu_item sub_menu_operator_module_slot_1 = { .title = "SLOT 1", .sub_menu = menu_operator_module_slot, .menu_enter_exit = sub_menu_operator_module_slot_enter_exit, .user_data = &rinblocks[2] };
static const struct s_menu_item sub_menu_operator_module_slot_2 = { .title = "SLOT 2", .sub_menu = menu_operator_module_slot, .menu_enter_exit = sub_menu_operator_module_slot_enter_exit, .user_data = &rinblocks[3] };


static const struct s_menu_item *const sub_menu_operator_modules[] =
{
  &sub_menu_operator_module_slot_1,
  &sub_menu_operator_module_slot_2,
  NULL
};


/** Operator rinblock modules menu item */
const struct s_menu_item menu_operator_modules =
{
  .title = "MODULE",
  .sub_menu = sub_menu_operator_modules
};
