#ifndef IFACE_SD_DOT_H
#define IFACE_SD_DOT_H 1

#include <stdint.h>
#include <stdbool.h>
#include "db_serial.h"
#include "db_special.h"


/** Allow modifications to items when in safe setup */
#define ALLOW_SAFE 1


/** Get permissions for a front key */
enum eFrontKeyPermissions get_frontkey_permissions(enum eFrontKeys key);


/**
 * Waits while there is an error or motion for a max of 15 seconds.
 *
 * @return true if the reading settles, or displays an error and returns false
 */
bool wait_till_no_motion(void);


/** Check for error with initial zero, and if so display error */
void check_and_display_startup_zero_error(void);


/**
 * Prompt user for 'Y'/'n' response to prompt
 *
 * @param s is the prompt
 * @param def is the default response
 *
 * @return true for Y, false for n
 *
 */
bool ask(const char *s, bool def);


/**
 * Wait timeout_ms milliseconds for a key press.
 *
 * @param timeout_ms is the timeout in milliseconds, or 0 for indefinite timeout
 * @param quit is a bool pointer to check to signal the application should quit. Can be NULL
 *
 * @return true if key pressed.
 */
bool wait_for_press(uint32_t timeout_ms, bool *quit);


/**
 * Prompt user with multiple strings.
 *
 * @param rate_ms is the delay between alternating strings in milliseconds.
 * @param timeout_ms is the timeout in milliseconds, or 0 for indefinite timeout
 * @param quit is a bool pointer to check to signal the application should quit. Can be NULL
 * @param ... is a number of strings terminated by NULL: "hello", "there", "bob", NULL
 *
 * @return true if key pressed, false if timed out or quit
 */
bool prompt_multi(uint32_t rate_ms, uint32_t timeout_ms, bool *quit, ...);


/**
 * Prompt user with 2 strings. Second string can be "".
 *
 * @param s1 is the first prompt string. Not NULL
 * @param s2 is the second prompt string. Can be NULL or "" if not required.
 * @param rate_ms is the delay between alternating strings in milliseconds.
 * @param timeout_ms is the timeout in milliseconds, or 0 for indefinite timeout
 * @param quit is a bool pointer to check to signal the application should quit. Can be NULL
 *
 * @return true if key pressed, false if timed out or quit
 */
bool prompt_timeout(const char *s1, const char *s2, uint32_t rate_ms, uint32_t timeout_ms, bool *quit);


/** Prompt user with 2 strings. Second string can be "". */
bool prompt(const char *s1, const char *s2);


/** Display an error message on the display */
void error_strings(const char *s1, const char *s2);


/** "NO" "ENTRY" error message on display */
void access_denied(void);


#endif /* #ifndef IFACE_SD_DOT_H */
