/*
 * iface_serial_config.h
 *
 * Serial configuration menus (auto opts, printer opts, net opts, modbus opts)
 *
 *  Created on: 03/09/2016
 *      Author: ian
 */

#ifndef IFACE_SERIAL_CONFIG_H_
#define IFACE_SERIAL_CONFIG_H_

#include "menus.h"
#include "modbus_config.h"


/** Automatic output source options */
enum eAutoOutputSrc { AUTO_OUT_SRC_DISP = 0, AUTO_OUT_SRC_GROSS, AUTO_OUT_SRC_NET, AUTO_OUT_SRC_TOTAL, AUTO_OUT_SRC_NUM };


/** This is for auto-outing the tare weight, but is not a menu option */
#define AUTO_OUT_SRC_TARE 5


/** Automatic output source strings, must match @ref enum eAutoOutputSrc */
#define AUTO_OUTPUT_SRC_STRS { "DISP", "GROSS", "NET", "TOTAL" }


/** Automatic output sources */
extern const char *const auto_output_src[AUTO_OUT_SRC_NUM];


/** Automatic output format settings */
enum eAutoOutFmt { AUTO_OUT_FMT_RINAUTO = 0, AUTO_OUT_FMT_HBMAUTO, AUTO_OUT_FMT_RINEXTEND,
                   AUTO_OUT_FMT_RINSIMPLE, AUTO_OUT_FMT_CUSTOM, AUTO_OUT_FMT_CONDEC,
                   AUTO_OUT_FMT_NUM
                 };


/** Automatic output format strings, must match @ref enum eAutoOutFmt */
#define AUTO_OUT_FMT_STRS { "AUTO.A", "AUTO.B", "AUTO.C", "AUTO.D", "CUSTOM", "AUTO.F" }


/** Automatic output formats */
extern const char *const auto_setting[AUTO_OUT_FMT_NUM];


/* Printer Output type definitions */
enum ePrinterModes
{
  P_SINGLE = 0,
  P_DOUBLE,
  P_TIC,
  PA_SINGLE,
  PA_DOUBLE,
  PA_TIC,
  P_TOTAL,
  PA_TOTAL,
  NUM_PRINTER_MODES
};


/** Printer mode strings, must match @ref enum ePrinterModes */
#define PRINTER_MODE_STRS { "SINGLE", "DOUBLE", "TICKET", "A.SING", "A.DOUB", "A.TICK", "TOTAL", "A.TOTAL" }


/** Printer mode setting strings */
extern const char *const printer_mode_setting[];


/** Modbus endian setting strings, must match @ref enum eRinModbusEndian */
#define RINMODBUS_ENDIAN_STRS { "BIG", "LITTLE" }


/** Modbus endian setting strings */
extern const char *const rinmodbus_endian_strs[RINMODBUS_ENDIAN_NUM];


/* Printer Output type definitions */
enum ePrintInterlock
{
  PRINT_INTERLOCK_NONE = 0,
  PRINT_INTERLOCK_MOTION,
  PRINT_INTERLOCK_RETZ,
  NUM_PRINT_INTERLOCK
};


/** Print interlock strings, must match @ref enum ePrintInterlock */
#define PRINT_INTERLOCK_STRS { "NONE", "MOTION", "RET.Z" }


/** Print interlock setting strings */
extern const char *const print_interlock_setting[];


extern const struct s_menu_item sub_menu_serial_netopt;
extern const struct s_menu_item sub_menu_serial_autoopt;
extern const struct s_menu_item sub_menu_serial_printopt;
extern const struct s_menu_item sub_menu_serial_modbusopt;


#endif /* IFACE_SERIAL_CONFIG_H_ */
