/*
 * iface_serial_ip.c
 *
 *  Created on: Sep 27, 2017
 *      Author: ian
 */
#include <stdbool.h>
#include <stdio.h>
#include <arpa/inet.h>
#include "menus.h"
#include "editor.h"
#include "edit_ip.h"
#include "serial_ports.h"
#include "db_serial.h"
#include "db_serial_ip.h"
#include "iface_serial_ports.h"

#include "iface_serial_ip.h"


/** Edit TCP/UDP socket port */
static enum eMenuLevel edit_tcpudp_port(bool full_edit, void *user_data, size_t index)
{
  enum eSerialPort port = (enum eSerialPort) user_data;
  enum eEditResult result;
  long tcpport;

  /* Figure out which port we're talking about based on menu position */
  switch (port)
  {
    case SERPORT_UDPOUT1:
    case SERPORT_UDPOUT2:
    case SERPORT_UDPIN1:
    case SERPORT_UDPIN2:
    case SERPORT_TCPNET:
    case SERPORT_TCPAUTO:
      tcpport = db_get_serial_tcpport(port);
      result = edit_long(&tcpport, DB_SERIAL_TCPPORT_HIGH_LIMIT, DB_SERIAL_TCPPORT_LOW_LIMIT, 0, 5, 0, true);
      if (EDIT_CHG == result)
      {
        db_set_serial_tcpport(port, tcpport);
      }
      break;

    default:
      comms_error_setting_not_avail();
      result = EDIT_ABORT;
      break;
  }
  return edit_result_to_menu_level(result);
}


/** Convert the TCP/UDP port to a string */
static void to_string_tcpudp_port(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  enum eSerialPort port = (enum eSerialPort) user_data;
  int tcpport;

  /* Figure out which port we're talking about based on menu position */
  switch (port)
  {
    case SERPORT_UDPOUT1:
    case SERPORT_UDPOUT2:
    case SERPORT_UDPIN1:
    case SERPORT_UDPIN2:
    case SERPORT_TCPNET:
    case SERPORT_TCPAUTO:
      tcpport = db_get_serial_tcpport(port);
      snprintf(buffer, buflen, "%d", tcpport);
      break;
    default:
      snprintf(buffer, buflen, "Unknown port");
      break;
  }
}


/** Edit TCP/UDP port timeout */
static enum eMenuLevel edit_tcpudp_timeout(bool full_edit, void *user_data, size_t index)
{
  enum eSerialPort port = (enum eSerialPort) user_data;
  enum eEditResult result;
  long timeout;

  /* Figure out which port we're talking about based on menu position */
  switch (port)
  {
    case SERPORT_UDPOUT1:
    case SERPORT_UDPOUT2:
    case SERPORT_UDPIN1:
    case SERPORT_UDPIN2:
    case SERPORT_TCPNET:
    case SERPORT_TCPAUTO:
      timeout = db_get_serial_tcp_timeout(port);
      result = edit_long(&timeout, DB_SERIAL_TCPTIMEOUT_HIGH_LIMIT, DB_SERIAL_TCPTIMEOUT_LOW_LIMIT, 0, 5, 0, true);
      if (EDIT_CHG == result)
      {
        db_set_serial_tcp_timeout(port, timeout);
      }
      break;

    default:
      comms_error_setting_not_avail();
      result = EDIT_ABORT;
      break;
  }
  return edit_result_to_menu_level(result);
}


/** Convert the TCP/UDP port timeout to a string */
static void to_string_tcpudp_timeout(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  enum eSerialPort port = (enum eSerialPort) user_data;
  unsigned long timeout;

  /* Figure out which port we're talking about based on menu position */
  switch (port)
  {
    case SERPORT_UDPOUT1:
    case SERPORT_UDPOUT2:
    case SERPORT_UDPIN1:
    case SERPORT_UDPIN2:
    case SERPORT_TCPNET:
    case SERPORT_TCPAUTO:
      timeout = db_get_serial_tcp_timeout(port);
      snprintf(buffer, buflen, "%lu", timeout);
      break;
    default:
      snprintf(buffer, buflen, "Unknown port");
      break;
  }
}


/** Edit UDP/TCP socket type */
static enum eMenuLevel edit_tcpudp_type(bool full_edit, void *user_data, size_t index)
{
  enum eSerialPort port = (enum eSerialPort) user_data;
  enum eEditResult result;
  uint8_t porttype;
  uint8_t porttypemin;
  uint8_t porttypemax;

  /* Figure out which port we're talking about based on menu position */
  switch (port)
  {
    case SERPORT_UDPOUT1:
    case SERPORT_UDPOUT2:
    case SERPORT_UDPIN1:
    case SERPORT_UDPIN2:
    case SERPORT_TCPAUTO:
      porttype = db_get_serial_type(port);
      porttypemin = db_get_serial_type_min(port);
      porttypemax = db_get_serial_type_max(port);
      result = edit_register_data(&porttype, porttypemin, porttypemax, ser_type_setting, true);
      if (EDIT_CHG == result)
      {
        db_set_serial_type(port, (enum eSerialPortType) porttype);
      }
      break;

    default:
      comms_error_setting_not_avail();
      result = EDIT_ABORT;
      break;
  }
  return edit_result_to_menu_level(result);
}


/** Convert the TCP port timeout to a string */
static void to_string_tcpudp_type(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  enum eSerialPort port = (enum eSerialPort) user_data;
  enum eSerialPortType porttype;

  /* Figure out which port we're talking about based on menu position */
  switch (port)
  {
    case SERPORT_UDPOUT1:
    case SERPORT_UDPOUT2:
    case SERPORT_UDPIN1:
    case SERPORT_UDPIN2:
    case SERPORT_TCPAUTO:
      porttype = db_get_serial_type(port);
      snprintf(buffer, buflen, "%s", ser_type_setting[porttype]);
      break;
    default:
      snprintf(buffer, buflen, "Unknown port");
      break;
  }
}


/** Edit a UDP destination IP address */
static enum eMenuLevel edit_tcpudp_ip_address(bool full_edit, void *user_data, size_t index)
{
  enum eSerialPort port = (enum eSerialPort) user_data;
  struct in_addr ip_address;
  enum eEditResult result;

  /* Figure out which port we're talking about based on menu position */
  switch (port)
  {
    case SERPORT_UDPOUT1:
    case SERPORT_UDPOUT2:
      db_get_serial_dest_ipaddress(port, &ip_address);
      result = edit_ipv4_address(&ip_address, eie_host);
      if (EDIT_CHG == result)
      {
        db_set_serial_dest_ipaddress(port, &ip_address);
      }
      break;

    default:
      comms_error_setting_not_avail();
      result = EDIT_ABORT;
      break;
  }
  return edit_result_to_menu_level(result);
}


/** Convert a UDP destination IP address to a string */
static void to_string_tcpudp_ip_address(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  enum eSerialPort port = (enum eSerialPort) user_data;
  struct in_addr ip_address;
  char ip_str[INET_ADDRSTRLEN];

  /* Figure out which port we're talking about based on menu position */
  switch (port)
  {
    case SERPORT_UDPOUT1:
    case SERPORT_UDPOUT2:
      db_get_serial_dest_ipaddress(port, &ip_address);
      (void) net_ip_inet_ntop_host_afinet(&ip_address, ip_str, INET_ADDRSTRLEN);
      snprintf(buffer, buflen, "%s", ip_str);
      break;
    default:
      snprintf(buffer, buflen, "Unknown port");
      break;
  }
}


/*
 * Menu items used by both TCP and UDP ports
 */

static const struct s_menu_item sub_menu_serial_tcpudp_port =        { .title = "PORT",    .sub_menu = NULL, .edit_fn = edit_tcpudp_port,       .to_string_fn = to_string_tcpudp_port,       .user_data_from_menu_above = true };
static const struct s_menu_item sub_menu_serial_tcpudp_timeout =     { .title = "T.OUT",   .sub_menu = NULL, .edit_fn = edit_tcpudp_timeout,    .to_string_fn = to_string_tcpudp_timeout,    .user_data_from_menu_above = true };
static const struct s_menu_item sub_menu_serial_tcpudp_type =        { .title = "TYPE",    .sub_menu = NULL, .edit_fn = edit_tcpudp_type,       .to_string_fn = to_string_tcpudp_type,       .user_data_from_menu_above = true };
static const struct s_menu_item sub_menu_serial_udpfixed_dest_ip =   { .title = "DST.IP",  .sub_menu = NULL, .edit_fn = edit_tcpudp_ip_address, .to_string_fn = to_string_tcpudp_ip_address, .user_data_from_menu_above = true };
static const struct s_menu_item sub_menu_serial_udpfixed_dest_port = { .title = "DST.PRT", .sub_menu = NULL, .edit_fn = edit_tcpudp_port,       .to_string_fn = to_string_tcpudp_port,       .user_data_from_menu_above = true };


/*
 * TCP ports menu
 */


static const struct s_menu_item *const menu_serial_tcpnet_options_list[] =
{
  &sub_menu_serial_tcpudp_port,
  &sub_menu_serial_tcpudp_timeout,
  NULL
};


static const struct s_menu_item *const menu_serial_tcpauto_options_list[] =
{
  &sub_menu_serial_tcpudp_type,
  &sub_menu_serial_tcpudp_port,
  &sub_menu_serial_tcpudp_timeout,
  NULL
};


/*
 * UDP ports menu
 */


static const struct s_menu_item *const menu_serial_udp_out_options_list[] =
{
  &sub_menu_serial_tcpudp_type,
  &sub_menu_serial_udpfixed_dest_ip,
  &sub_menu_serial_udpfixed_dest_port,
  NULL
};


static const struct s_menu_item *const menu_serial_udp_in_options_list[] =
{
  &sub_menu_serial_tcpudp_type,
  &sub_menu_serial_tcpudp_port,
  NULL
};


const struct s_menu_item sub_menu_serial_tcp1 =      { .title = "TCP.IN.1", .sub_menu = menu_serial_tcpnet_options_list,  .user_data = (void *) SERPORT_TCPNET };
const struct s_menu_item sub_menu_serial_tcp2 =      { .title = "TCP.IN.2", .sub_menu = menu_serial_tcpauto_options_list, .user_data = (void *) SERPORT_TCPAUTO };

const struct s_menu_item sub_menu_serial_udpfixed1 = { .title = "UDP.O.1",  .sub_menu = menu_serial_udp_out_options_list, .user_data = (void *) SERPORT_UDPOUT1 };
const struct s_menu_item sub_menu_serial_udpfixed2 = { .title = "UDP.O.2",  .sub_menu = menu_serial_udp_out_options_list, .user_data = (void *) SERPORT_UDPOUT2 };

const struct s_menu_item sub_menu_serial_udpdyn1 =   { .title = "UDP.IN.1", .sub_menu = menu_serial_udp_in_options_list,  .user_data = (void *) SERPORT_UDPIN1 };
const struct s_menu_item sub_menu_serial_udpdyn2 =   { .title = "UDP.IN.2", .sub_menu = menu_serial_udp_in_options_list,  .user_data = (void *) SERPORT_UDPIN2 };
