/*
 * iface_setpoint.c
 *
 *  Created on: 04/03/2011
 *      Author: ian
 */
#include <stdio.h>
#include <string.h>
#include <rinweigh/adcdatabase.h>
#include "database.h"
#include "db_setpoint.h"
#include "iface.h"
#include "display.h"
#include "iface_sd.h"
#include "editor.h"
#include "adcthread.h"
#include "app_db_user.h"
#include "app_db_runtime_vars.h"
#include "dbprint.h"
#include "app_db_user.h"
#include "app_db_runtime.h"

#include "iface_setpoint.h"


/* Check that setpoint numbers match */
/*
#if (SETP_MENU_NUM != NUM_SETPOINTS)
  #error "Setpoint number mismatch"
#endif
*/


static enum eMenuLevel edit_setp_type(bool full_edit, void *user_data, size_t index)
{
  intptr_t setpoint = (intptr_t) user_data;
  enum eEditResult result;

  /* Edit the setpoint type */
  result = edit_register_data(&db_app_user.setp_active[setpoint], 0, SETP_TYPE_NUM - 1, setpoint_type_strings, ALLOW_SAFE);
  if (EDIT_ABORT == result || EDIT_MENU_QUIT == result)
  {
    return edit_result_to_menu_level(result);
  }
  return mlNum;
}


/** Convert the setpoint  */
static void to_string_setp_type(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  intptr_t setpoint = (intptr_t) user_data;

  snprintf(buffer, buflen, "%s", setpoint_type_strings[db_app_user.setp_active[setpoint]]);
}


static enum eMenuLevel edit_setp_params(size_t setpoint, enum eSetpointParameter param)
{
  long l;
  bool ok_to_edit = false;
  enum eEditResult result;

  if (!setpt_edit || db_app_user.setp_lock[setpoint] == 0)
  {
    if (db_app_user.setp_active[setpoint] == SETP_OVER || db_app_user.setp_active[setpoint] == SETP_UNDER ||
        db_app_user.setp_active[setpoint] == SETP_W_OUT || db_app_user.setp_active[setpoint] == SETP_W_IN)
    {
      ok_to_edit = true;
    }
  }

  if (!ok_to_edit)
  {
    access_denied();
    return mlNum;
  }

  switch (param)
  {
    /* Edit targets */
    case spTarget:
      l = setpoint_target_get(setpoint);
      Dset_units_ignore_motion(adcSuperStructure.adcUserDatabase.units);
      result = edit_long(&l, APP_DB_R_TARGETS_MAX, APP_DB_R_TARGETS_MIN, adcSuperStructure.adcUserDatabase.dpPos,
                         NUM_DIGITS, 0, ALLOW_SAFE);
      if (EDIT_ABORT != result && EDIT_MENU_QUIT != result)
      {
        setpoint_target_set(setpoint, l);
      }
      break;

    /* Edit inflight */
    case spInFlight:
      l = setpoint_preflight_get(setpoint);
      Dset_units_ignore_motion(adcSuperStructure.adcUserDatabase.units);
      result = edit_long(&l, APP_DB_R_PREFLIGHT_MAX, APP_DB_R_PREFLIGHT_MIN, adcSuperStructure.adcUserDatabase.dpPos,
                         NUM_DIGITS, 0, ALLOW_SAFE);
      if (EDIT_ABORT != result && EDIT_MENU_QUIT != result)
      {
        setpoint_preflight_set(setpoint, l);
      }
      break;

    /* Edit hysteresis */
    case spHysteresis:
      l = setpoint_hysteresis_get(setpoint);
      Dset_units_ignore_motion(adcSuperStructure.adcUserDatabase.units);
      result = edit_long(&l, APP_DB_R_HYSTERESIS_MAX, APP_DB_R_HYSTERESIS_MIN, adcSuperStructure.adcUserDatabase.dpPos,
                         NUM_DIGITS, 0, ALLOW_SAFE);
      if (EDIT_ABORT != result && EDIT_MENU_QUIT != result)
      {
        setpoint_hysteresis_set(setpoint, l);
      }
      break;

    /* Fix compiler warning */
    default:
      result = mlNum;
      break;
  }
  return edit_result_to_menu_level(result);
}


static enum eMenuLevel edit_setp_target(bool full_edit, void *user_data, size_t index)
{
  intptr_t setpoint = (intptr_t) user_data;

  return edit_setp_params(setpoint, spTarget);
}


/** Convert the setpoint target to a string */
static void to_string_setp_target(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  intptr_t setpoint = (intptr_t) user_data;
  AdcRdg rdg;

  memset(&rdg, 0, sizeof(rdg));
  rdg.rdg = setpoint_target_get(setpoint);
  rdg.info.unitdp.bit.dp = adcSuperStructure.adcUserDatabase.dpPos;
  rdg.info.unitdp.bit.units = adcSuperStructure.adcUserDatabase.units;
  to_string_weight(&rdg, true, buffer, buflen);
}


static enum eMenuLevel edit_setp_inflight(bool full_edit, void *user_data, size_t index)
{
  intptr_t setpoint = (intptr_t) user_data;

  return edit_setp_params(setpoint, spInFlight);
}


/** Convert the setpoint inflight to a string */
static void to_string_setp_inflight(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  intptr_t setpoint = (intptr_t) user_data;
  AdcRdg rdg;

  memset(&rdg, 0, sizeof(rdg));
  rdg.rdg = setpoint_preflight_get(setpoint);
  rdg.info.unitdp.bit.dp = adcSuperStructure.adcUserDatabase.dpPos;
  rdg.info.unitdp.bit.units = adcSuperStructure.adcUserDatabase.units;
  to_string_weight(&rdg, true, buffer, buflen);
}


static enum eMenuLevel edit_setp_hysteresis(bool full_edit, void *user_data, size_t index)
{
  intptr_t setpoint = (intptr_t) user_data;

  return edit_setp_params(setpoint, spHysteresis);
}


/** Convert the setpoint hysteresis to a string */
static void to_string_setp_hysteresis(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  intptr_t setpoint = (intptr_t) user_data;
  AdcRdg rdg;

  memset(&rdg, 0, sizeof(rdg));
  rdg.rdg = setpoint_hysteresis_get(setpoint);
  rdg.info.unitdp.bit.dp = adcSuperStructure.adcUserDatabase.dpPos;
  rdg.info.unitdp.bit.units = adcSuperStructure.adcUserDatabase.units;
  to_string_weight(&rdg, true, buffer, buflen);
}


static enum eMenuLevel edit_setp_lock(bool full_edit, void *user_data, size_t index)
{
  intptr_t setpoint = (intptr_t) user_data;
  enum eEditResult result;

  /* Edit the setpoint lock */
  result = edit_register_data(&db_app_user.setp_lock[setpoint], 0, SETP_LOCK_NUM - 1, setp_lock_strings, ALLOW_SAFE);
  if (EDIT_ABORT == result || EDIT_MENU_QUIT == result)
  {
    return edit_result_to_menu_level(result);
  }
  return mlNum;
}


/** Convert the setpoint lock */
static void to_string_setp_lock(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  intptr_t setpoint = (intptr_t) user_data;

  snprintf(buffer, buflen, "%s", setp_lock_strings[db_app_user.setp_lock[setpoint]]);
}


static enum eMenuLevel edit_setp_source(bool full_edit, void *user_data, size_t index)
{
  intptr_t setpoint = (intptr_t) user_data;
  enum eEditResult result;

  /* Edit the setpoint source */
  result = edit_register_data(&db_app_user.setp_source[setpoint], 0, SETP_SOURCE_NUM - 1, setp_src_strings, ALLOW_SAFE);
  if (EDIT_ABORT == result || EDIT_MENU_QUIT == result)
  {
    return edit_result_to_menu_level(result);
  }
  return mlNum;
}


/** Convert the setpoint source */
static void to_string_setp_source(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  intptr_t setpoint = (intptr_t) user_data;

  snprintf(buffer, buflen, "%s", setp_src_strings[db_app_user.setp_source[setpoint]]);
}


static enum eMenuLevel edit_setp_logic(bool full_edit, void *user_data, size_t index)
{
  intptr_t setpoint = (intptr_t) user_data;
  enum eEditResult result;

  /* Edit the setpoint logic */
  result = edit_register_data(&db_app_user.setp_logic[setpoint], 0, SETP_LOGIC_NUM - 1, setp_logic_strings, ALLOW_SAFE);
  if (EDIT_ABORT == result || EDIT_MENU_QUIT == result)
  {
    return edit_result_to_menu_level(result);
  }
  return mlNum;
}


/** Convert the setpoint logic */
static void to_string_setp_logic(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  intptr_t setpoint = (intptr_t) user_data;

  snprintf(buffer, buflen, "%s", setp_logic_strings[db_app_user.setp_logic[setpoint]]);
}


static enum eMenuLevel edit_setp_alarm(bool full_edit, void *user_data, size_t index)
{
  intptr_t setpoint = (intptr_t) user_data;
  enum eEditResult result;

  /* Edit the setpoint alarm */
  result = edit_register_data(&db_app_user.setp_alarm[setpoint], 0, SETP_ALARM_NUM - 1, setp_alarm_strings, ALLOW_SAFE);
  if (EDIT_ABORT == result || EDIT_MENU_QUIT == result)
  {
    return edit_result_to_menu_level(result);
  }
  return mlNum;
}


/** Convert the setpoint alarm */
static void to_string_setp_alarm(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  intptr_t setpoint = (intptr_t) user_data;

  snprintf(buffer, buflen, "%s", setp_alarm_strings[db_app_user.setp_alarm[setpoint]]);
}


static enum eMenuLevel edit_setp_timing(bool full_edit, void *user_data, size_t index)
{
  intptr_t setpoint = (intptr_t) user_data;
  enum eEditResult result;

  /* Edit the setpoint timing */
  result = edit_register_data(&db_app_user.setp_timing[setpoint], 0, SETP_TIMING_NUM - 1, setp_timing_strings, ALLOW_SAFE);
  if (EDIT_ABORT == result || EDIT_MENU_QUIT == result)
  {
    return edit_result_to_menu_level(result);
  }
  return mlNum;
}


/** Convert the setpoint timing */
static void to_string_setp_timing(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  intptr_t setpoint = (intptr_t) user_data;

  snprintf(buffer, buflen, "%s", setp_timing_strings[db_app_user.setp_logic[setpoint]]);
}


static enum eMenuLevel edit_setp_reset(bool full_edit, void *user_data, size_t index)
{
  intptr_t setpoint = (intptr_t) user_data;
  enum eEditResult result;

  /* Edit the setpoint reset */
  result = edit_register_data(&db_app_user.setp_reset[setpoint], 0, SETP_RESET_NUM - 1, setp_reset_strings, ALLOW_SAFE);
  if (EDIT_ABORT == result || EDIT_MENU_QUIT == result)
  {
    return edit_result_to_menu_level(result);
  }
  return mlNum;
}


/** Convert the setpoint reset */
static void to_string_setp_reset(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  intptr_t setpoint = (intptr_t) user_data;

  snprintf(buffer, buflen, "%s", setp_reset_strings[db_app_user.setp_alarm[setpoint]]);
}


#define SETP_MS_DP 3


static enum eMenuLevel edit_setp_jog_on(bool full_edit, void *user_data, size_t index)
{
  long l;
  enum eEditResult result;

  l = db_app_user.setp_jog_on;

  /* Edit the setpoint jog on */
  result = edit_long(&l, APP_DB_U_SETPJOGON_MAX, APP_DB_U_SETPJOGON_MIN, SETP_MS_DP, NUM_DIGITS, 0, ALLOW_SAFE);
  if (EDIT_ABORT != result && EDIT_MENU_QUIT != result)
  {
    db_app_user.setp_jog_on = l;
  }

  return edit_result_to_menu_level(result);
}


/** Convert the setpoint jog on */
static void to_string_setp_jog_on(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  snprintf(buffer, buflen, "%1.3d", db_app_user.setp_jog_on);
}


static enum eMenuLevel edit_setp_jog_off(bool full_edit, void *user_data, size_t index)
{
  long l;
  enum eEditResult result;

  l = db_app_user.setp_jog_off;

  /* Edit the setpoint jog off */
  //Dset_units_ignore_motion(ADC_UNITS_NONE);
  result = edit_long(&l, APP_DB_U_SETPJOGOFF_MAX, APP_DB_U_SETPJOGOFF_MIN, SETP_MS_DP, NUM_DIGITS, 0, ALLOW_SAFE);
  if (EDIT_ABORT != result && EDIT_MENU_QUIT != result)
  {
    db_app_user.setp_jog_off = l;
  }

  return edit_result_to_menu_level(result);
}


/** Convert the setpoint jog off */
static void to_string_setp_jog_off(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  snprintf(buffer, buflen, "%1.3d", db_app_user.setp_jog_off);
}


static enum eMenuLevel edit_setp_jog_set(bool full_edit, void *user_data, size_t index)
{
  long l;
  enum eEditResult result;

  l = (long) db_app_user.setp_jog_set;

  /* Edit the setpoint jog set */
  result = edit_long(&l, APP_DB_U_SETPJOGSET_MAX, APP_DB_U_SETPJOGSET_MIN, 0, NUM_DIGITS, 0, ALLOW_SAFE);
  if (EDIT_ABORT != result && EDIT_MENU_QUIT != result)
  {
    db_app_user.setp_jog_set = (uint8_t) l;
  }

  return edit_result_to_menu_level(result);
}


/** Convert the setpoint jog set */
static void to_string_setp_jog_set(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  snprintf(buffer, buflen, "%d", db_app_user.setp_jog_set);
}


static enum eMenuLevel edit_setp_max_set(bool full_edit, void *user_data, size_t index)
{
  long l;
  enum eEditResult result;

  l = (long) db_app_user.setp_max_sets;

  /* Edit the setpoint max sets */
  result = edit_long(&l, APP_DB_U_SETPMAXSET_MAX, APP_DB_U_SETPMAXSET_MIN, 0, NUM_DIGITS, 0, ALLOW_SAFE);
  if (EDIT_ABORT != result && EDIT_MENU_QUIT != result)
  {
    db_app_user.setp_max_sets = (uint8_t) l;
  }

  return edit_result_to_menu_level(result);
}


/** Convert the setpoint max set */
static void to_string_setp_max_set(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  snprintf(buffer, buflen, "%d", db_app_user.setp_max_sets);
}


static enum eMenuLevel edit_setp_feeder(bool full_edit, void *user_data, size_t index)
{
  enum eEditResult result;

  /* Edit the setpoint feeder */
  result = edit_register_data(&db_app_user.setp_feeder, 0, SETP_FEEDER_NUM - 1, setp_feeder_strings, ALLOW_SAFE);
  if (EDIT_ABORT == result || EDIT_MENU_QUIT == result)
  {
    return edit_result_to_menu_level(result);
  }
  return mlNum;
}


/** Convert the setpoint feeder */
static void to_string_setp_feeder(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  snprintf(buffer, buflen, "%s", setp_feeder_strings[db_app_user.setp_feeder]);
}


static enum eMenuLevel edit_setp_delay_check(bool full_edit, void *user_data, size_t index)
{
  long l;
  enum eEditResult result;

  l = db_app_user.setp_delay_check;

  /* Edit the setpoint delay check */
  result = edit_long(&l, APP_DB_U_SETPDELAYCHECK_MAX, APP_DB_U_SETPDELAYCHECK_MIN, SETP_MS_DP, NUM_DIGITS, 0, ALLOW_SAFE);
  if (EDIT_ABORT != result && EDIT_MENU_QUIT != result)
  {
    db_app_user.setp_delay_check = l;
  }

  return edit_result_to_menu_level(result);
}


/** Convert the setpoint delay check */
static void to_string_setp_delay_check(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  snprintf(buffer, buflen, "%1.3d", db_app_user.setp_delay_check);
}


static const struct s_menu_item sub_menu_setpoint_type =       { .title = "TYPE",   .sub_menu = NULL, .edit_fn = edit_setp_type,       .to_string_fn = to_string_setp_type,       .user_data_from_menu_above = true };
static const struct s_menu_item sub_menu_setpoint_target =     { .title = "TARGET", .sub_menu = NULL, .edit_fn = edit_setp_target,     .to_string_fn = to_string_setp_target,     .user_data_from_menu_above = true };
static const struct s_menu_item sub_menu_setpoint_hysteresis = { .title = "HYS",    .sub_menu = NULL, .edit_fn = edit_setp_hysteresis, .to_string_fn = to_string_setp_hysteresis, .user_data_from_menu_above = true };
static const struct s_menu_item sub_menu_setpoint_inflight =   { .title = "FLIGHT", .sub_menu = NULL, .edit_fn = edit_setp_inflight,   .to_string_fn = to_string_setp_inflight,   .user_data_from_menu_above = true };
static const struct s_menu_item sub_menu_setpoint_lock =       { .title = "LOCK",   .sub_menu = NULL, .edit_fn = edit_setp_lock,       .to_string_fn = to_string_setp_lock,       .user_data_from_menu_above = true };
static const struct s_menu_item sub_menu_setpoint_source =     { .title = "SOURCE", .sub_menu = NULL, .edit_fn = edit_setp_source,     .to_string_fn = to_string_setp_source,     .user_data_from_menu_above = true };
static const struct s_menu_item sub_menu_setpoint_logic =      { .title = "LOGIC",  .sub_menu = NULL, .edit_fn = edit_setp_logic,      .to_string_fn = to_string_setp_logic,      .user_data_from_menu_above = true };
static const struct s_menu_item sub_menu_setpoint_alarm =      { .title = "ALARM",  .sub_menu = NULL, .edit_fn = edit_setp_alarm,      .to_string_fn = to_string_setp_alarm,      .user_data_from_menu_above = true };
static const struct s_menu_item sub_menu_setpoint_timing =     { .title = "TIMING", .sub_menu = NULL, .edit_fn = edit_setp_timing,     .to_string_fn = to_string_setp_timing,     .user_data_from_menu_above = true };
static const struct s_menu_item sub_menu_setpoint_reset =      { .title = "RESET",  .sub_menu = NULL, .edit_fn = edit_setp_reset,      .to_string_fn = to_string_setp_reset,      .user_data_from_menu_above = true };


static const struct s_menu_item *const sub_menu_setpoint_list[] =
{
  &sub_menu_setpoint_type,
  &sub_menu_setpoint_target,
  &sub_menu_setpoint_hysteresis,
  &sub_menu_setpoint_inflight,
  &sub_menu_setpoint_lock,
  &sub_menu_setpoint_source,
  &sub_menu_setpoint_logic,
  &sub_menu_setpoint_alarm,
  &sub_menu_setpoint_timing,
  &sub_menu_setpoint_reset,
  NULL
};


static const struct s_menu_item sub_menu_setpoint_jog_on =     { .title = "JOG ON",  .sub_menu = NULL, .edit_fn = edit_setp_jog_on,      .to_string_fn = to_string_setp_jog_on,       .user_data_from_menu_above = false };
static const struct s_menu_item sub_menu_setpoint_jog_off =    { .title = "JOG.OFF", .sub_menu = NULL, .edit_fn = edit_setp_jog_off,     .to_string_fn = to_string_setp_jog_off,      .user_data_from_menu_above = false };
static const struct s_menu_item sub_menu_setpoint_jog_set =    { .title = "JOG.SET", .sub_menu = NULL, .edit_fn = edit_setp_jog_set,     .to_string_fn = to_string_setp_jog_set,      .user_data_from_menu_above = false };
static const struct s_menu_item sub_menu_setpoint_max_set =    { .title = "MAX.SET", .sub_menu = NULL, .edit_fn = edit_setp_max_set,     .to_string_fn = to_string_setp_max_set,      .user_data_from_menu_above = false };
static const struct s_menu_item sub_menu_setpoint_feeder =     { .title = "FEEDER",  .sub_menu = NULL, .edit_fn = edit_setp_feeder,      .to_string_fn = to_string_setp_feeder,       .user_data_from_menu_above = false };
static const struct s_menu_item sub_menu_setpoint_delay_check ={ .title = "DLY.CHK", .sub_menu = NULL, .edit_fn = edit_setp_delay_check, .to_string_fn = to_string_setp_delay_check,  .user_data_from_menu_above = false };


static const struct s_menu_item *const sub_menu_setpoint_gen_list[] =
{
  &sub_menu_setpoint_jog_on,
  &sub_menu_setpoint_jog_off,
  &sub_menu_setpoint_jog_set,
  &sub_menu_setpoint_max_set,
  &sub_menu_setpoint_feeder,
  &sub_menu_setpoint_delay_check,
  NULL
};


static const struct s_menu_item sub_menu_setpoint_gen = { .title = "GEN",      .sub_menu = sub_menu_setpoint_gen_list };
static const struct s_menu_item sub_menu_setpoint1 =    { .title = "SET.PT.1", .sub_menu = sub_menu_setpoint_list,     .user_data = (void *) 0 };
static const struct s_menu_item sub_menu_setpoint2 =    { .title = "SET.PT.2", .sub_menu = sub_menu_setpoint_list,     .user_data = (void *) 1 };
static const struct s_menu_item sub_menu_setpoint3 =    { .title = "SET.PT.3", .sub_menu = sub_menu_setpoint_list,     .user_data = (void *) 2 };
static const struct s_menu_item sub_menu_setpoint4 =    { .title = "SET.PT.4", .sub_menu = sub_menu_setpoint_list,     .user_data = (void *) 3 };
static const struct s_menu_item sub_menu_setpoint5 =    { .title = "SET.PT.5", .sub_menu = sub_menu_setpoint_list,     .user_data = (void *) 4 };
static const struct s_menu_item sub_menu_setpoint6 =    { .title = "SET.PT.6", .sub_menu = sub_menu_setpoint_list,     .user_data = (void *) 5 };
static const struct s_menu_item sub_menu_setpoint7 =    { .title = "SET.PT.7", .sub_menu = sub_menu_setpoint_list,     .user_data = (void *) 6 };
static const struct s_menu_item sub_menu_setpoint8 =    { .title = "SET.PT.8", .sub_menu = sub_menu_setpoint_list,     .user_data = (void *) 7 };


static const struct s_menu_item *const menu_setpoint_list[] =
{
  &sub_menu_setpoint_gen,
  &sub_menu_setpoint1,
  &sub_menu_setpoint2,
  &sub_menu_setpoint3,
  &sub_menu_setpoint4,
  &sub_menu_setpoint5,
  &sub_menu_setpoint6,
  &sub_menu_setpoint7,
  &sub_menu_setpoint8,
  NULL
};


/** Setpoint menu item */
const struct s_menu_item menu_setpoint =
{
  .title = "SET.PTS",
  .sub_menu = menu_setpoint_list
};
