/*
 * iface_special.c
 *
 *  Created on: 03/03/2011
 *      Author: ian
 */
#include <stdio.h>
#include <string.h>
#include <rinweigh/adcdatabase.h>
#include "database.h"
#include "iface.h"
#include "iface_sd.h"
#include "editor.h"
#include "display.h"
#include "db_remote.h"
#include "kbd_5000.h"
#include "db_special.h"
#include "adcthread.h"
#include "app_db_user.h"
#include "commonstrings.h"

#include "iface_special.h"


/** Number of remote keys that can be edited at once */
#define MAX_REMOTE_EDIT 4


static enum eMenuLevel change_passcode(bool full_edit, void *user_data, size_t index)
{
  uint32_t *passcode = (uint32_t *) user_data;
  long l;
  enum eEditResult result;

  if (!full_edit)
  {
    access_denied();
    return mlNum;
  }
  l = (long) *passcode;
  result = edit_long(&l, 999999L, 0L, 0, NUM_DIGITS, 0, full_edit);
  if (EDIT_CHG == result)
  {
    *passcode = l;
  }
  return edit_result_to_menu_level(result);
}


/** Convert a passcode to a string */
static void to_string_passcode(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  uint32_t *passcode = (uint32_t *) user_data;

  if (full)
    snprintf(buffer, buflen, "%lu", (unsigned long) *passcode);
  else
    snprintf(buffer, buflen, "Not accessible - use Full setup");
}


static enum eMenuLevel edit_rear_entry_setting(bool full_edit, void *user_data, size_t index)
{
  bool can_edit;

  /* Can only edit this setting if the menus have been accessed via the rear key */
  can_edit = rear_key_entry && full_edit;

  /* Edit the setting */
  return edit_result_to_menu_level(edit_register_data(&db_app_user.rear_entry, 0, 1, off_on_setting, can_edit));
}


/** Convert the rear entry setting to a string */
static void to_string_rear_entry(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  snprintf(buffer, buflen, "%s", off_on_setting[db_app_user.rear_entry]);
}


static enum eMenuLevel edit_front_key_permissions(bool full_edit, void *user_data, size_t index)
{
  uint8_t tmp[NUM_FRONT_KEYS];
  int i;
  const char *strs[NUM_FRONT_KEYS];
  enum eEditResult result;

  /** Need to reverse settings so what appears on the screen matches the keys below it */
  for (i = 0; i < NUM_FRONT_KEYS; i++)
  {
    tmp[i] = db_app_user.front_key_permissions[NUM_FRONT_KEYS - 1 - i];
    strs[i] = front_key_permission_settings[NUM_FRONT_KEYS - 1 - i];
  }

  /* Edit the settings */
  result = edit_bits(tmp, NUM_FRONT_KEYS, strs, full_edit);
  if (EDIT_CHG == result)
  {
    /** Need to reverse settings back into the database array */
    for (i = 0; i < NUM_FRONT_KEYS; i++)
    {
      db_app_user.front_key_permissions[NUM_FRONT_KEYS - 1 - i] = tmp[i];
    }
  }
  return edit_result_to_menu_level(result);
}


/** Convert the front key permissions to a string */
static void to_string_front_key_permissions(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  size_t i;

  /** Need to reverse settings so what appears on the screen matches the keys below it */
  for (i = 0; i < NUM_FRONT_KEYS && i < buflen - 1; i++)
  {
    buffer[i] = front_key_permission_settings[i][db_app_user.front_key_permissions[i]];
  }
  buffer[i] = '\0';
}


/** Edit front function key function */
static enum eMenuLevel edit_key_functions(bool full_edit, void *user_data, size_t index)
{
  enum eEditResult result;

  result = edit_register_data(&db_app_user.front_key_function[index], 0, NUM_FRONTFUNC - 1, frontfunc_keyfn_strings, full_edit);
  return edit_result_to_menu_level(result);
}


/** Convert the key functions setting to a string */
static void to_string_key_functions(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  snprintf(buffer, buflen, "%s", frontfunc_keyfn_strings[db_app_user.front_key_function[index]]);
}


/** Edit remote functions */
static enum eMenuLevel edit_remote_functions(bool full_edit, void *user_data, size_t index)
{
  enum eEditResult result;

  result = edit_register_data(&db_app_user.remote_f[index], 0, NUM_REMOTE_KEY_FUNCTIONS - 1, remote_keyfn_strings, full_edit);
  return edit_result_to_menu_level(result);
}


/** Convert the remote functions setting to a string */
static void to_string_remote_functions(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  snprintf(buffer, buflen, "%s", remote_keyfn_strings[db_app_user.remote_f[index]]);
}


/** Edit buzzer enable */
static enum eMenuLevel edit_buzzer_enable(bool full_edit, void *user_data, size_t index)
{
  enum eEditResult result;

  result = edit_register_data(&db_app_user.buzzer_enable, 0, 1, off_on_setting, true);
  setup_kbd_buzz();
  return edit_result_to_menu_level(result);
}


/** Convert the buzzer enable setting to a string */
static void to_string_buzzer_enable(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  snprintf(buffer, buflen, "%s", off_on_setting[db_app_user.buzzer_enable]);
}


/** Edit display brightness */
static enum eMenuLevel edit_brightness(bool full_edit, void *user_data, size_t index)
{
  enum eEditResult result;

  result = edit_byte(&db_app_user.brightness, 100, 0, 0, 3, true);
  Dset_brightness();
  return edit_result_to_menu_level(result);
}


/** Convert the brightness setting to a string */
static void to_string_brightness(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  snprintf(buffer, buflen, "%u %%", (unsigned) db_app_user.brightness);
}


/** Edit secondary units */
static enum eMenuLevel edit_units_sec(bool full_edit, void *user_data, size_t index)
{
  enum eEditResult result;

  result = edit_register_data(&db_app_user.units_sec, 0, APP_DB_U_UNITS_SEC_MAX, alt_units_menus, true);
  return edit_result_to_menu_level(result);
}


/** Convert the secondary units setting to a string */
static void to_string_units_sec(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  snprintf(buffer, buflen, "%s", alt_units_menus[db_app_user.units_sec]);
}


/** Edit tertiary units */
static enum eMenuLevel edit_units_ter(bool full_edit, void *user_data, size_t index)
{
  enum eEditResult result;

  result = edit_register_data(&db_app_user.units_ter, 0, APP_DB_U_UNITS_TER_MAX, alt_units_menus, true);
  return edit_result_to_menu_level(result);
}


/** Convert the tertiary units setting to a string */
static void to_string_units_ter(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  snprintf(buffer, buflen, "%s", alt_units_menus[db_app_user.units_ter]);
}


static enum eMenuLevel edit_conv_factor(bool full_edit, void *user_data, size_t index)
{
  long l;
  enum eEditResult result;

  l = (long) db_app_user.unit_conv_factor;
  result = edit_long(&l, APP_DB_U_UNITS_CONV_MAX, APP_DB_U_UNITS_CONV_MIN, 3, NUM_DIGITS, 0, full_edit);
  if (EDIT_CHG == result)
  {
    db_app_user.unit_conv_factor = l;
  }
  return edit_result_to_menu_level(result);
}


/** Convert unit conversion factor to a string */
static void to_string_conv_factor(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  snprintf(buffer, buflen, "%lu", (unsigned long) db_app_user.unit_conv_factor);
}


/** Edit lb:oz enable */
static enum eMenuLevel edit_show_lboz(bool full_edit, void *user_data, size_t index)
{
  enum eEditResult result;

  result = edit_register_data(&db_app_user.show_lboz, 0, 1, off_on_setting, true);
  return edit_result_to_menu_level(result);
}


/** Convert the lb:oz enable setting to a string */
static void to_string_show_lboz(bool full, void *user_data, size_t index, char *buffer, size_t buflen)
{
  snprintf(buffer, buflen, "%s", off_on_setting[db_app_user.show_lboz]);
}


/* Front key functions */
static const struct s_menu_item sub_menu_special_keyfn1 = { .title = "KEY.FN.1", .sub_menu = NULL, .edit_fn = edit_key_functions, .to_string_fn = to_string_key_functions };
static const struct s_menu_item sub_menu_special_keyfn2 = { .title = "KEY.FN.2", .sub_menu = NULL, .edit_fn = edit_key_functions, .to_string_fn = to_string_key_functions };
static const struct s_menu_item sub_menu_special_keyfn3 = { .title = "KEY.FN.3", .sub_menu = NULL, .edit_fn = edit_key_functions, .to_string_fn = to_string_key_functions };


static const struct s_menu_item *const menu_special_keyfn[] =
{
  &sub_menu_special_keyfn1,
  &sub_menu_special_keyfn2,
  &sub_menu_special_keyfn3,
  NULL
};


/* Remote input functions */
static const struct s_menu_item sub_menu_special_inputfn1 =  { .title = "IN.FN.1", .sub_menu = NULL, .edit_fn = edit_remote_functions, .to_string_fn = to_string_remote_functions };
static const struct s_menu_item sub_menu_special_inputfn2 =  { .title = "IN.FN.2", .sub_menu = NULL, .edit_fn = edit_remote_functions, .to_string_fn = to_string_remote_functions };
static const struct s_menu_item sub_menu_special_inputfn3 =  { .title = "IN.FN.3", .sub_menu = NULL, .edit_fn = edit_remote_functions, .to_string_fn = to_string_remote_functions };
static const struct s_menu_item sub_menu_special_inputfn4 =  { .title = "IN.FN.4", .sub_menu = NULL, .edit_fn = edit_remote_functions, .to_string_fn = to_string_remote_functions };
static const struct s_menu_item sub_menu_special_inputfn5 =  { .title = "IN.FN.5", .sub_menu = NULL, .edit_fn = edit_remote_functions, .to_string_fn = to_string_remote_functions };
static const struct s_menu_item sub_menu_special_inputfn6 =  { .title = "IN.FN.6", .sub_menu = NULL, .edit_fn = edit_remote_functions, .to_string_fn = to_string_remote_functions };
static const struct s_menu_item sub_menu_special_inputfn7 =  { .title = "IN.FN.7", .sub_menu = NULL, .edit_fn = edit_remote_functions, .to_string_fn = to_string_remote_functions };
static const struct s_menu_item sub_menu_special_inputfn8 =  { .title = "IN.FN.8", .sub_menu = NULL, .edit_fn = edit_remote_functions, .to_string_fn = to_string_remote_functions };


static const struct s_menu_item *const menu_special_inputfn[] =
{
  &sub_menu_special_inputfn1,
  &sub_menu_special_inputfn2,
  &sub_menu_special_inputfn3,
  &sub_menu_special_inputfn4,
  &sub_menu_special_inputfn5,
  &sub_menu_special_inputfn6,
  &sub_menu_special_inputfn7,
  &sub_menu_special_inputfn8,
  NULL
};


/* Unit switching settings */
static const struct s_menu_item sub_menu_special_units_sec = { .title = "UNIT.2", .sub_menu = NULL, .edit_fn = edit_units_sec, .to_string_fn = to_string_units_sec };
static const struct s_menu_item sub_menu_special_units_ter = { .title = "UNIT.3", .sub_menu = NULL, .edit_fn = edit_units_ter, .to_string_fn = to_string_units_ter };
static const struct s_menu_item sub_menu_special_conv_fact = { .title = "FACTOR", .sub_menu = NULL, .edit_fn = edit_conv_factor, .to_string_fn = to_string_conv_factor };
static const struct s_menu_item sub_menu_special_show_lboz = { .title = "LB.OZ",  .sub_menu = NULL, .edit_fn = edit_show_lboz, .to_string_fn = to_string_show_lboz };


static const struct s_menu_item *const menu_special_units[] =
{
  &sub_menu_special_units_sec,
  &sub_menu_special_units_ter,
  &sub_menu_special_conv_fact,
  &sub_menu_special_show_lboz,
  NULL
};


static const struct s_menu_item sub_menu_special_safepc =     { .title = "SAFE.PC",   .sub_menu = NULL, .edit_fn = change_passcode,            .to_string_fn = to_string_passcode,             .user_data = &db_app_user.safe_passcode };
static const struct s_menu_item sub_menu_special_fullpc =     { .title = "FULL.PC",   .sub_menu = NULL, .edit_fn = change_passcode,            .to_string_fn = to_string_passcode,             .user_data = &db_app_user.full_passcode };
static const struct s_menu_item sub_menu_special_rearentry =  { .title = "R.ENTRY",   .sub_menu = NULL, .edit_fn = edit_rear_entry_setting,    .to_string_fn = to_string_rear_entry };
static const struct s_menu_item sub_menu_special_button =     { .title = "BUTTON",    .sub_menu = NULL, .edit_fn = edit_front_key_permissions, .to_string_fn = to_string_front_key_permissions };
static const struct s_menu_item sub_menu_special_keyfn =      { .title = "KEY.FN",    .sub_menu = menu_special_keyfn };
static const struct s_menu_item sub_menu_special_inputfn =    { .title = "IN.FN",     .sub_menu = menu_special_inputfn };
static const struct s_menu_item sub_menu_special_buzzer =     { .title = "BUZZER",    .sub_menu = NULL, .edit_fn = edit_buzzer_enable,         .to_string_fn = to_string_buzzer_enable };
static const struct s_menu_item sub_menu_special_brightness = { .title = "BRIGHT",    .sub_menu = NULL, .edit_fn = edit_brightness,            .to_string_fn = to_string_brightness };
static const struct s_menu_item sub_menu_special_units =      { .title = "UNITS",     .sub_menu = menu_special_units };


static const struct s_menu_item *const sub_menu_special[] =
{
  &sub_menu_special_safepc,
  &sub_menu_special_fullpc,
  &sub_menu_special_rearentry,
  &sub_menu_special_button,
  &sub_menu_special_keyfn,
  &sub_menu_special_inputfn,
  &sub_menu_special_buzzer,
  &sub_menu_special_brightness,
  &sub_menu_special_units,
  NULL
};


/** Special menu item */
const struct s_menu_item menu_special =
{
  .title = "SPEC",
  .sub_menu = sub_menu_special
};
