/*
 * ipinfo.h
 *
 *  Created on: 07/06/2010
 *      Author: ian
 */

#ifndef IPINFO_H_
#define IPINFO_H_

#include <arpa/inet.h>
#include <sys/socket.h>
#include <bits/local_lim.h>
#include <stdbool.h>
#include <setnetconfig.h>


/** Address of a MAC address when formatted as a string (xx:xx:xx:xx:xx:xx) */
#define MAC_ADDRESS_STR_LENGTH (SNC_ETH_ALEN * 3)


/** Get an octet from an IPv4 address */
#define GET_IN_ADDR_OCTET(in, octet) ((((in).s_addr) >> ((octet) * 8)) & 0x000000FF)


/** Set an octet in an IPv4 address */
#define SET_IN_ADDR_OCTET(in, octet, data) ((in).s_addr &= ~(0x000000FF << ((octet) * 8)), (in).s_addr |= ((in_addr_t) (data) & 0x000000FF) << ((octet) * 8))


/** Generate the broadcast address from the IP address and netmask */
#define BROADCAST_FROM_IP_NETMASK(ip, netmask) ((ip) | ~(netmask))


/* Defaults for the Ethernet settings */
#define DEFAULT_ETH_IPADDRESS   ( SNC_IPV4_OCTETS_TO_INADDR_NETWORK( 192, 168, 1, 254 ) )
#define DEFAULT_ETH_NETMASK     ( SNC_IPV4_OCTETS_TO_INADDR_NETWORK( 255, 255, 255, 0 ) )
#define DEFAULT_ETH_GATEWAY     ( SNC_IPV4_OCTETS_TO_INADDR_NETWORK( 192, 168, 1, 1 ) )
#define DEFAULT_ETH_BROADCAST   BROADCAST_FROM_IP_NETMASK(DEFAULT_ETH_IPADDRESS, DEFAULT_ETH_NETMASK)
#define DEFAULT_ETH_DNS1        ( SNC_IPV4_OCTETS_TO_INADDR_NETWORK( 192, 168, 1, 1 ) )
#define DEFAULT_ETH_DNS2        ( SNC_IPV4_OCTETS_TO_INADDR_NETWORK( 0, 0, 0, 0 ) )
#define DEFAULT_ETH_DNS3        ( SNC_IPV4_OCTETS_TO_INADDR_NETWORK( 0, 0, 0, 0 ) )
#define DEFAULT_ETH_USEDHCP     ( true )
/* Default hostname is model-serialno eg WE2111-3382127 */
#define DEFAULT_ETH_DOMAIN      ""
#define DEFAULT_ETH_SEARCH_DOMAIN ""


/** Result returned from @ref net_ip_save_config() */
enum e_net_ip_save_result
{
  nesr_success = 0,
  nesr_failure,
  nesr_about_to_save,
  nesr_bad_hostname,
  nesr_bad_searchdomain,
  nesr_bad_ipaddress,
  nesr_bad_netmask,
  nesr_bad_gateway,
  nesr_bad_broadcast,
  nesr_bad_dns0,
  nesr_bad_dns1,
  nesr_bad_dns2
};


/** Function callback called during net_ip_save_config() to allow UI error messages to be shown */
typedef void (*t_net_ip_save_ui_callback_fn)(enum e_net_ip_save_result);


/** Setnetconfig library configuration */
extern struct s_snc_configuration *libsnc_config;


/** Initialise libsetnetconfig to enable us to get/set the network configuration */
bool net_ip_setnetconfig_init(void);


/** Shutdown libsetnetconfig */
void net_ip_setnetconfig_destroy(void);


/** Convert an AF_INET (IPV4) address in network endian into a string */
const char *net_ip_inet_ntop_network_afinet(const struct in_addr *ipv4_net_endian, char *buf, socklen_t len);


/** Convert an AF_INET (IPV4) address in host endian into a string */
const char *net_ip_inet_ntop_host_afinet(const struct in_addr *ipv4_host_endian, char *buf, socklen_t len);


/** Convert string into an AF_INET (IPV4) address in network endian */
bool net_ip_inet_pton_network_afinet(const char *str, struct in_addr *ipv4_net_endian);


/** Convert string into an AF_INET (IPV4) address in host endian */
bool net_ip_inet_pton_host_afinet(const char *str, struct in_addr *ipv4_host_endian);


/** Convert a MAC address to a string (xx:xx:xx:xx:xx:xx) */
void net_ip_macaddress_to_string(const struct sockaddr *mac_address, char separator, char str[MAC_ADDRESS_STR_LENGTH]);


/** Set the C500 preferred network defaults in a s_snc_net_config */
void net_ip_set_defaults(struct s_snc_net_config *net_config);


/** Get the network configuration or network current settings */
bool net_ip_get_config(bool config_not_current, struct s_snc_net_config *net_config);


/** Saves any network configuration changes to the system */
enum e_net_ip_save_result net_ip_save_config(struct s_snc_net_config *net_config, t_net_ip_save_ui_callback_fn callback);


#endif /* IPINFO_H_ */
