/*
 * licensing.h
 *
 *  Created on: Nov 21, 2016
 *      Author: ian
 */

#ifndef LICENSING_H_
#define LICENSING_H_

#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>
#include <rinlicence/rl_licence.h>
#include "display.h"


/** Initalise licensing for the application. Requires configuration and databases to be initialised */
void app_licensing_init(void);


/** Shut down licensing for the application */
void app_licensing_destroy(void);


/** Call when a package is unlicensed to force the UI to re-check the licensing */
void app_licensing_force_recheck(void);


/** Split a package name at the "-" into two halves for putting on the display. Ie "C500" "K501" */
void app_licensing_split_package_name_for_display(const char *packagename, char split_pn[2][NUM_DIGITS + 1]);


/** Check for unlicensed packages, and prompt for passcodes for them */
void app_licensing_check(bool *quit);


/**
 * Reset the licensing check time after exiting the menus so
 * you get another 5 minutes before prompting for unlicensed packages again
 */
void app_licensing_reset_check_time(void);


/** Is a package licensed? */
bool app_licensing_is_installed(const char *package_name);


/**
 * Are there any addon packages installed?
 *
 * package_name is set to the first addon package found, otherwise not set
 */
bool app_licensing_are_addon_packages_installed(const char **package_name);


/** Used by LRP comms command to get the data it needs */
bool app_licensing_comms_query(size_t package_index, size_t *num_packages,
                               const char **package_name, bool *is_licensed,
                               char licence_code[LICENCECODE_LEN_C500 + 1]);


/** Used by LRP comms command to license/unlicense a package */
bool app_licensing_comms_cmd(const char *package_name,
                             bool licence_not_unlicence,
                             const char *licence_code);


/** Generate the setup bypass code for those fools who have forgotten their full setup passcode */
bool app_licensing_generate_setup_bypass_code(uint32_t *bypass_code);


#endif /* LICENSING_H_ */
