#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <time.h>
#include <rinhw/hwlib_init.h>
#include <rinhw/buzzer.h>
#include <rinhw/sio.h>
#include <rinhw/iic.h>
#include <rinhw/lcd.h>
#include <rinhw/adc_up.h>
#include <rinhw/adc_ads1216.h>
#include <rinhw/rtc.h>
#include <rinhw/sio_local_comport.h>
#include <rinhw/kbd_linux_event.h>
#include <rinhw/we21hw.h>
#include <rinhw/iic_local_linux.h>
#include <rinhw/adc_ads1216_local.h>
#include <rinhw/rtc_local.h>
#include <rinweigh/adcinterface.h>
#include <rinweigh/adcdatabase.h>
#include <rinhw/hw_vers.h>
#include <rinweigh/adcversion.h>
#include "system.h"
#include "version.h"
#include "iface.h"
#include "iface_sd.h"
#include "backgrnd.h"
#include "meas.h"
#include "kbd_5000.h"
#include "database.h"
#include "buzz5000.h"
#include "access.h"
#include "setpoint.h"
#include "comms_print.h"
#include "editor.h"
#include "modbus_old.h"
#include "nv_ram.h"
#include "asciidef.h"
#include "display.h"
#include "cmdline.h"
#include "serial_usb.h"
#include "comms_setup.h"
#include "comms_util.h"
#include "comms_task.h"
#include "comms_rincmd_support.h"
#include "serial_ports.h"
#include "comms_auto_trigger.h"
#include "we_debug.h"
#include "adcthread.h"
#include "device_finder_we2111.h"
#include "we_config.h"
#include "devices.h"
#include "dsd.h"
#include "app_db_factory.h"
#include "app_db_user.h"
#include "app_db_runtime.h"
#include "app_db.h"
#include "app_db_changelog.h"
#include "signals.h"
#include "daemonise.h"
#include "verify_tested.h"
#include "app_devicemount.h"
#include "watchdog.h"
#include "indicator_model.h"
#include "indicator_model_features.h"
#include "modbus_task.h"
#include "key_drvr.h"
#include "status_led.h"
#include "serial_usb_host.h"
#include "root_passwd.h"
#include "licensing.h"
#include "unitswitch.h"
#include "ipinfo.h"


#define APPLICATION_APPNAME "c500-500"
#define APPLICATION_DAEMONISE_ENVVAR "c500_daemonise"


/** Flag set by signal handler to force application to quit */
static bool application_force_quit = false;


void my_bf1000(void)
{
  /* Decrement the heartbeat for rincmd. When it hits zero, execute the
   * heartbeat function to clean up the lua application. */
  if (rincmd_heartbeat > 0)
  {
    rincmd_heartbeat--;
    if (rincmd_heartbeat == 0)
      rincmd_updateHeartbeat(NULL, 0, 0, NULL, NULL, RC_FUNC_EXEC);
  }

  do_background();
}


void my_bf100(void)
{
//  struct timespec tsStart = {0}, tsFin = {0}, temp = {0};
//  static struct timespec tsWait;
//  clock_gettime(CLOCK_MONOTONIC, &tsStart);

  do_disp();
/*  check_printing_error(); */
/*  update_setp_timers(); */
  /* do_background100(); */
  if (!edit_functions)
  {
    /* Trigger all serial ports configured as automatic outputs */
    auto_lo_output_trigger();
  }

//  clock_gettime(CLOCK_MONOTONIC, &tsFin);
//  temp = diffTime(tsWait, tsStart);
//  rdebug_printf(LOG_ERR, "my_bf100: %02d.%09d %09ld\n", temp.tv_sec, temp.tv_nsec, diffTime(tsStart, tsFin).tv_nsec);
//  clock_gettime(CLOCK_MONOTONIC, &tsWait);
}


void set_standard_scale(void)
{
  edit_functions = true;        /* block comms etc until power up sequence over */
}


/** Work out what serial ports are fitted, and create tasks for each one */
void init_tasks_for_serial_ports(void)
{
  int portcount = 0;
  struct t_io_dev *ports[SERPORT_NUM + 1]; /* Extra one for NULL */
  enum eSerialPort types[SERPORT_NUM + 1];

  if (!comms_task_list_init(we_configuration.uds_network_file, we_configuration.uds_autoout_file))
    return;

  /* Rinlink on front display board */
  if (0 != (dev_ser_front_rl.status & O_INITIALISED))
  {
    types[portcount]   = SERPORT_RL_FRONT;
    ports[portcount++] = &dev_ser_front_rl;
  }

  /* Fixed onboard port serial 1 */
  if (0 != (dev_ser1_local.status & O_INITIALISED))
  {
    types[portcount]   = SERPORT_SER1_RL_REAR;
    ports[portcount++] = &dev_ser1_local;
  }

  /* Fixed onboard port serial 2 */
  if (0 != (dev_ser2_local.status & O_INITIALISED))
  {
    types[portcount]   = SERPORT_SER2;
    ports[portcount++] = &dev_ser2_local;
  }

  /* SER3 - Rinblock RS422 port */
  if (0 != (dev_ser3_rb_rs422.status & O_INITIALISED))
  {
    types[portcount]   = SERPORT_RB1A;
    ports[portcount++] = &dev_ser3_rb_rs422;
  }
  if (0 != (dev_ser3_rb_rs422b.status & O_INITIALISED))
  {
    types[portcount]   = SERPORT_RB1B;
    ports[portcount++] = &dev_ser3_rb_rs422b;
  }

  /* SER4 - Rinblock RS232 port */
  if (0 != (dev_ser4_rb_rs232.status & O_INITIALISED))
  {
    types[portcount]   = SERPORT_RB2A;
    ports[portcount++] = &dev_ser4_rb_rs232;
  }
  if (0 != (dev_ser4_rb_rs232b.status & O_INITIALISED))
  {
    types[portcount]   = SERPORT_RB2B;
    ports[portcount++] = &dev_ser4_rb_rs232b;
  }

  /* SER 7 - USB serial gadget port */
  if (0 != (dev_ser7_usb.status & O_INITIALISED))
  {
    types[portcount]   = SERPORT_USBGADGET;
    ports[portcount++] = &dev_ser7_usb;
  }

  /*
   * SER 8 - USB host serial port and
   * SER 9 - USB line printer
   *
   * We don't add these devices here,
   * because the device mounter adds and removes them
   */

  /* Last one is NULL to signify end of list */
  ports[portcount] = NULL;

  /* Initialise serial ports */
  comms_tasks_create(ports, types);
}


/** Halt on ROM error - if invalid software in this indicator, return false if need to quit */
static bool rom_error_halt(void)
{
  char str[10];

  /* Invalid software in this indicator, so halt with error displayed */
  if (0 != (get_current_errors() & APP_ERR_ROM))
  {
    freeze_display_and_annunciators(true);
    while (!application_force_quit)
    {
      get_error_str(str, 10);
      Dputstr(str);
      delay(100);
    }
    return false;
  }
  return true;
}


/** Initialise pretty much everything. Returns false if the application can't run. */
static bool init_all(void)
{
  t_datetime dt;
  enum e_indicator_model_sub_type imst;

  WE_STARTUP_DEBUG_PRINTF(RDEBUG_LVL_FULL, LOG_INFO, "init_all called\n");

  /* Initialise libsetnetconfig */
  if (!net_ip_setnetconfig_init())
    rdebug_printf(LOG_ERR, "init_all: libsetnetconfig initialisation failed\n");

  /* Create application devices that don't require database */
  if (!init_app_devices_pre_db())
    return false;

  /* Initialise the NVRAM early, so it's available to other parts of the program */
  if (!nvram_init())
  {
    set_error(ADC_ERR_RUNTIME_DB, "init_all: setting error ADC_ERR_RUNTIME_DB because nvram initialisation failed\n");
  }

  /* Need change log before databases */
  if (!app_changelog_init())
  {
    rdebug_printf(LOG_ERR, "init_all: change log initialisation failed\n");
  }

  /* Initialise ADC data structures */
  WE_STARTUP_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "init_all: initialise adc super structure\n");
  if (!adc_super_structure_init())
  {
    rdebug_printf(LOG_ERR, "init_all: adc super structure init fail\n");
  }

  /* Initialise and read databases */
  WE_STARTUP_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "init_all: initialise and read databases\n");
  app_init_and_read_databases();

  /* Check if the user and root passwords have been set, and set if not */
  check_set_user_password(db_app_factory.serial_no);
  check_set_root_password(&db_app_factory);

  /* Setup indicator model */
  init_indicator_model_and_subtype(&db_app_factory);
  imst = get_indicator_model_subtype();
  WE_STARTUP_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "init_all: detected indicator model %s, subtype %s\n",
                          get_indicator_model_string(indicator_model_subtype_to_indicator_model(imst), imdt_long),
                          get_indicator_model_subtype_string(imst, imdt_long));

  /* Create application devices that do require database */
  if (!init_app_devices_post_db(imst))
    return false;

  /* Force rinlink serial settings */
  db_force_rinlink_serial_settings();

  /* Initialise the status LED */
  init_status_led(indicator_model_subtype_has_status_led(imst));

  /* Initialise licensing */
  app_licensing_init();
  WE_STARTUP_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "init_all: licensing initialised\n");

  /* Check that the clock is working */
  if (!GetDateTime(&dt))
  {
    set_error(APP_ERR_RTC, "init_all: setting error APP_ERR_RTC because rtc read failed\n");
  }

  /* Setup buzzer on keypresses */
  setup_kbd_buzz();

  /* Initialise serial port tasks */
  init_tasks_for_serial_ports();

  /* Initialise modbus with maximum number of connections */
  rinmodbus_init(WECFG_RINMODBUS_MAX_TCP_CONNECTIONS);
  rinmodbus_configure_from_db();

  /* Initialise the device mounter */
  WE_STARTUP_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "init_all: initialise device mounter\n");
  we2111_device_mounter_init(we_configuration.partition_mount_location, we_configuration.device_mounter_notification_pipe);

  /* Set other stuff up */
  init_iface();
  init_key_driver();
  init_display();
  init_access(!indicator_model_subtype_has_display(get_indicator_model_subtype()));
  init_background();
  init_kbd();
  enable_remote_keys(true);

  /* Invalid software in this indicator, so halt with error displayed */
  if (!rom_error_halt())
  {
    return false;
  }
  set_standard_scale();
  init_system();

  /* Start watchdog */
  WE_STARTUP_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "init_all: initialise watchdog\n");
  watchdog_init(we_configuration.watchdog_device);

  WE_STARTUP_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "init_all: initialise adc thread\n");
  if (!adc_thread_init())
  {
    rdebug_printf(LOG_ERR, "init_all: adc thread initialisation failed\n");
  }

  /* Re-intialise analog output and setpoints because these rely on the sync freq initialised by adc_thread_init() */
  access_dbconfigure_analog();
  init_setpoints(&setpoint_running_info, &adcSuperStructure);

  /* Init alternate units */
  init_unit_switch();

  /* Check if software version has changed, and bump calcounters, and add changelog entry */
  app_db_check_software_version();

  WE_STARTUP_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "init_all: initialise dsd\n");
  if (!dsd_init())
  {
    rdebug_printf(LOG_ERR, "init_all: dsd initialisation failed\n");
  }

  /* Update the device finder information with specific stuff from our application */
  WE_STARTUP_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "init_all: initialise device finder\n");
  update_device_finder_we2111();

  WE_STARTUP_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "init_all: verify that hardware has been tested\n");
  VerifyHardwareTested();

  /* All initialised successfully */
  return true;
}


/** Shutdown pretty much everything */
static void destroy_all(void)
{
  dsd_destroy();
  WE_SHUTDOWN_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "destroy_all: dsd destroyed\n");

  /* Shut down system timers */
  destroy_system();
  destroy_key_driver();
  destroy_iface();
  WE_SHUTDOWN_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "destroy_all: system timers destroyed\n");

  rinmodbus_destroy();
  WE_SHUTDOWN_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "destroy_all: modbus task destroyed\n");

  /* Force tasks to quit */
  comms_tasks_destroy();
  WE_SHUTDOWN_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "destroy_all: comms tasks destroyed\n");

  app_licensing_destroy();
  WE_SHUTDOWN_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "destroy_all: licensing destroyed\n");

  /* Shutdown the status LED */
  destroy_status_led();

  adc_thread_destroy();
  WE_SHUTDOWN_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "destroy_all: adc thread destroyed\n");

  watchdog_destroy();
  WE_SHUTDOWN_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "destroy_all: watchdog destroyed\n");

  app_destroy_databases();
  WE_SHUTDOWN_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "destroy_all: databases destroyed\n");

  adc_super_structure_destroy();
  WE_SHUTDOWN_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "destroy_all: adc super structure destroyed\n");

  app_changelog_destroy();
  WE_SHUTDOWN_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "destroy_all: change log destroyed\n");

  nvram_destroy();
  WE_SHUTDOWN_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "destroy_all: nvram destroyed\n");

  /* Shut down device mounter */
  we2111_device_mounter_destroy();
  WE_SHUTDOWN_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "destroy_all: device mounter destroyed\n");

  /* Shut down comms task list, needs to be done after device mounter destroy, as the printer mounter may call comms stuff */
  comms_task_list_destroy();
  WE_SHUTDOWN_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "destroy_all: comms task list destroyed\n");

  /* Clean up devices */
  destroy_app_devices();
  WE_SHUTDOWN_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "destroy_all: devices destroyed\n");

  /* Shut down libsetnetconfig */
  net_ip_setnetconfig_destroy();
}


/** Entry point */
int main(int argc, char *argv[])
{
  int result;
  int interim_result;

  WE_STARTUP_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "main: we2111 application started\n");

  /* Check the command line */
  if (parse_commandline(argc, argv))
  {
    rdebug_printf(LOG_ERR, "Invalid command line option, try --help\n");
    result = 1;
    goto main_done;
  }
  if (cmdline_showhelp)
  {
    print_commandlineopts(argv[0]);
    result = 0;
    goto main_done;
  }

  /* Setup syslog */
  if (cmdline_logtosyslog)
  {
    openlog(APPLICATION_APPNAME, LOG_PID, LOG_USER);
  }

  /* Run as a daemon if necessary */
  if (cmdline_daemonise)
  {
    if (NULL == getenv(APPLICATION_DAEMONISE_ENVVAR))
    {
      /* Initial instance, run vfork()/exec(), then quit */
      result = daemoniseStart(argc, argv, APPLICATION_DAEMONISE_ENVVAR);
      goto main_done;
    }
    else
    {
      /* Second child instance, run as daemon */
      interim_result = daemoniseContinue(argc, argv);
      if (interim_result != 0)
      {
        result = interim_result;
        goto main_done;
      }
    }
  }
  else
  {
    /* Run as non-daemon */
    rdebug_printf(LOG_INFO, "main: running in non-daemon mode\n");
  }

  rdebug_printf(LOG_INFO, "main: Application built at " __DATE__ " " __TIME__ "\n");
  rdebug_printf(LOG_INFO, "main: Hardware library version: %s\n", HwLibDrawingNumberStr);
  rdebug_printf(LOG_INFO, "main: ADC library version: " ADC_LIB_DRAWING_NUMBER_STR "\n");
  rdebug_printf(LOG_INFO, "main: Application version: " APP_DRAWING_NUMBER_STR "\n");
  rdebug_printf(LOG_INFO, "main: Application git version: " APP_GIT_VERSION "\n");

  /* Install signal handers */
  WE_STARTUP_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "main: install signal handlers\n");
  if (!init_signals(&application_force_quit, cmdline_daemonise))
  {
    rdebug_printf(LOG_ERR, "Initialising signals failed\n");
    result = 1;
    goto main_done;
  }

  /* Load configuration from file */
  if (!load_configuration(cmdline_config_file, &we_configuration))
  {
    rdebug_printf(LOG_ERR, "Configuration file %s invalid\n", cmdline_config_file);
    result = 1;
    goto main_done;
  }

  /* Initialise the system */
  WE_STARTUP_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "main: initialise everything\n");
  if (init_all())
  {
    if (rom_error_halt())
    {
      /* Main loop */
      WE_STARTUP_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "main: all up, running main loop\n");
      do_user_iface(&application_force_quit);
    }
  }

  /* Clean up everything */
  destroy_all();

  /* Free up the configuration */
  free_configuration(&we_configuration);

  if (cmdline_logtosyslog)
  {
    closelog();
  }
  result = 0;

main_done:
  WE_SHUTDOWN_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "main: shutdown complete, result %d\n", result);
  return result;
}
