/*
 * menus.h
 *
 *  Created on: Feb 13, 2015
 *      Author: ian
 */

#ifndef MENUS_H_
#define MENUS_H_

#include <stdbool.h>
#include <stddef.h>
#include "editor.h"


enum eMenuLevel { mlQuit = 0, mlZero, mlOne, mlTwo, mlThree, mlFour, mlFive, mlNum };


/** A menu item entry */
struct s_menu_item
{
  const char *title;                                                                                     /*!< Title displayed in the menus */
  const struct s_menu_item *const *sub_menu;                                                             /*!< Sub menu for this item, NULL if not */
  enum eMenuLevel (*edit_fn)(bool full_edit, void *user_data, size_t index);                             /*!< Edit function for this item, NULL if not required */
  bool (*menu_enter_exit)(bool entry_not_exit, bool full_edit, void *user_data, bool edit_not_tostring); /*!< Function to run on entry or exit to this sub-menu/edit item. For entry, returning false means the edit function won't be run. For exit, returning false means the menu level won't be left */
  bool user_data_from_menu_above;                                                                        /*!< If set, use the user specified data from the above menu level rather than user_data */
  void *user_data;                                                                                       /*!< Pointer to user specified data */
  void (*to_string_fn)(bool full, void *user_data, size_t index, char *buffer, size_t buflen);           /*!< Function to convert setting to a printable string */
};


void run_menu_editor(const struct s_menu_item *item, bool full_edit, enum eMenuLevel start_level);


/** Convert an edit result into menu level */
enum eMenuLevel edit_result_to_menu_level(enum eEditResult edit_result);


#endif /* MENUS_H_ */
