/*
 * modbus_regs.h
 *
 *  Created on: Sep 9, 2014
 *      Author: ian
 */

#ifndef MODBUS_REGS_H_
#define MODBUS_REGS_H_

#include <stdint.h>


enum e_rinmodbus_type
{
  rmsSINT16 = 0,
  rmsUINT16,
  rmsSINT32,
  rmsUINT32,
  rmsSTRING,
  rmsUNKNOWN
};


/** Rinmodbus read permission mask. Applies to @ref s_rinmodbus_translate.permis */
#define RMPM_READ 0x01


/** Rinmodbus write permission mask. Applies to @ref s_rinmodbus_translate.permis */
#define RMPM_WRITE 0x02


/** Rinmodbus translation table entry to translate Modbus register into HBM comms command */
struct s_rinmodbus_translate
{
  enum e_rinmodbus_type regtype;      /*!< Type of modbus register */
  uint16_t regnum;                    /*!< Modbus register number */
  const char *cmd;                    /*!< Translates to this HBM command */
  int8_t index_param;                 /*!< Which parameter is the index parameter (used for commands like BDX param 0) */
  int8_t index;                       /*!< The index which we are setting (ie value of BDX param 0) */
  int8_t param;                       /*!< Which parameter are we setting */
  int8_t readparam;                   /*!< This is used when the read parameter is different to the write parameter (eg LBT). Normally -1 */
  uint8_t permis;                     /*!< Read / write permissions */
  int stringlen;                      /*!< Length of string in bytes for string parameters (rmsSTRING) */
};


#endif /* MODBUS_REGS_H_ */
