/*
 * modbus_xlat.h
 *
 *  Created on: Sep 8, 2014
 *      Author: ian
 */

#ifndef MODBUS_XLAT_H_
#define MODBUS_XLAT_H_

#include <stdint.h>
#include <rinhw/rio.h>
#include <modbus.h>
#include "comms_net.h"
#include "iface_serial.h"
#include "modbus_regs.h"
#include "modbus_config.h"


#if (MODBUS_MAX_READ_REGISTERS > MODBUS_MAX_WRITE_REGISTERS)
  #define RINMODBUS_MAX_REGISTERS (MODBUS_MAX_READ_REGISTERS)
#else
  #define RINMODBUS_MAX_REGISTERS (MODBUS_MAX_WRITE_REGISTERS)
#endif


/**
 * Buffer used for translating Modbus registers into HBM comms commands.
 * Worst case:
 *  String of length RINMODBUS_MAX_REGISTERS * 2
 *  3 characters of command (IDN)
 *  1 character of ?
 *  2 character of ""
 *  2 characters of \r\n
 *  A number of commas
 *  An index parameter
 */
#define RINMODBUS_COMMS_BUFFER_LEN (RINMODBUS_MAX_REGISTERS * sizeof(uint16_t) + 100)


struct s_rinmodbus_interp
{
  struct t_io_dev *cn_pipe;                       /*!< IO pipe used to write to the comms network interpreter */
  struct s_comms_net *cn;                         /*!< Comms network interpreter */
  char buffer[RINMODBUS_COMMS_BUFFER_LEN + 1];    /*!< Buffer for constructing HBM commands from the Modbus registers */
  struct s_comms_params comms_params;             /*!< Comms parameters, used for breaking up cn response */

  uint8_t query[MODBUS_TCP_MAX_ADU_LENGTH];       /*!< Modbus query */
  uint8_t response[MODBUS_TCP_MAX_ADU_LENGTH];    /*!< Modbus response */
};


/** Run a HBM command, returns a MODBUS_EXCEPTION_*, MODBUS_EXCEPTION_MAX on success */
int rinmodbus_run_hbm_cmd(struct s_comms_net *cn, struct t_io_dev *cn_write_pipe, const char *cmd);


/** Implement the Modbus read registers command */
void rinmodbus_read_registers(const struct s_rinmodbus_translate *xlat_table, modbus_t *mb_ctx, struct s_rinmodbus_interp *interp, size_t query_length, enum eRinModbusEndian mb_endian);


/** Implement the Modbus read single register command */
void rinmodbus_write_single_register(const struct s_rinmodbus_translate *xlat_table, modbus_t *mb_ctx, struct s_rinmodbus_interp *interp, size_t query_length, enum eRinModbusEndian mb_endian);


/** Implement the Modbus write multiple registers command */
void rinmodbus_write_multiple_registers(const struct s_rinmodbus_translate *xlat_table, modbus_t *mb_ctx, struct s_rinmodbus_interp *interp, size_t query_length, enum eRinModbusEndian mb_endian);


#endif /* MODBUS_XLAT_H_ */
