#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <rinweigh/adcdatabase.h>
#include <rinweigh/adcinterface.h>
#include "comms_util.h"
#include "asciidef.h"
#include "meas.h"
#include "database.h"
#include "apperror.h"
#include "version.h"
#include "system.h"
#include "sysclock.h"
#include "setpoint.h"
#include "access.h"
#include "kbd_5000.h"
#include "adcthread.h"
#include "app_db_user.h"
#include "app_db_runtime.h"
#include "app_db_runtime_vars.h"

#include "modbus_old.h"


#define DUMMY1         0
#define RDG_STATUS     1

#define RDG_ERROR_HI   2
#define RDG_ERROR_LO   3

#define RDG_HI         4
#define RDG_LOW        5

#define GROSS_HI       6
#define GROSS_LOW      7

#define NET_HI         8
#define NET_LOW        9

#define NUM_BATCHES_HI  10
#define NUM_BATCHES_LOW 11

#define TOTAL1_HI      12
#define TOTAL1_LOW     13

#define TOTAL2_HI      14
#define TOTAL2_LOW     15

#define PEAK_HI        16
#define PEAK_LOW       17


/*
 * Take a 32 bit number and output two 16 bit registers in the supplied endian
 */
static void legacymodbus_int32_to_regs(uint16_t *data, int32_t data32, enum eRinModbusEndian mb_endian)
{
  union
  {
    int32_t l;
    uint16_t w[2];
  } U;

  U.l = data32;
  if (RINMODBUS_ENDIAN_BIG == mb_endian)
  {
    data[0] = U.w[0];
    data[1] = U.w[1];
  }
  else
  {
    data[0] = U.w[1];
    data[1] = U.w[0];
  }
}

/*
 * Take two 16 bit registers and make a 32 bit number in the supplied endian
 */
static void legacymodbus_regs_to_int32(int32_t *data32, uint16_t *data, enum eRinModbusEndian mb_endian)
{
  union
  {
    int32_t l;
    uint16_t w[2];
  } U;

  if (RINMODBUS_ENDIAN_BIG == mb_endian)
  {
    U.w[0] = data[0];
    U.w[1] = data[1];
  }
  else
  {
    U.w[0] = data[1];
    U.w[1] = data[0];
  }

  *data32 = U.l;
}


static void update_input_regs(struct s_comms_control *comms_ctrl, enum eRinModbusEndian mb_endian)
{
  AdcRdg disprdg;
  uint16_t w;

  memset(comms_ctrl->modbus_input_data, 0, sizeof(comms_ctrl->modbus_input_data));

  disprdg = adcSuperStructure.adcDataWorking.dispRdg;

  comms_ctrl->modbus_input_data[DUMMY1] = 0;

  legacymodbus_int32_to_regs(&comms_ctrl->modbus_input_data[RDG_HI], disprdg.rdg, mb_endian);

  w = (uint16_t) (access_get_outputs()) << 8;
  if (APP_ERR_NONE != get_current_errors())
    w |= 0x0001;
  if (disprdg.info.status.bit.oload)
    w |= 0x0002;
  if (disprdg.info.status.bit.uload)
    w |= 0x0004;
  if (disprdg.info.status.bit.coz)
    w |= 0x0008;
  if (disprdg.info.status.bit.motion)
    w |= 0x0010;
  if (disprdg.info.status.bit.gross)
    w |= 0x0020;
  if (disprdg.info.status.bit.range > 0)
    w |= 0x0040;
  comms_ctrl->modbus_input_data[RDG_STATUS] = w;

  legacymodbus_int32_to_regs(&comms_ctrl->modbus_input_data[RDG_ERROR_HI], get_current_errors(), mb_endian);

  legacymodbus_int32_to_regs(&comms_ctrl->modbus_input_data[GROSS_HI], adcSuperStructure.adcDataWorking.grossRdg.rdg, mb_endian);

  legacymodbus_int32_to_regs(&comms_ctrl->modbus_input_data[NET_HI], adcSuperStructure.adcDataWorking.netRdg.rdg, mb_endian);

  legacymodbus_int32_to_regs(&comms_ctrl->modbus_input_data[NUM_BATCHES_HI], statistics_get_num_items(), mb_endian);

  legacymodbus_int32_to_regs(&comms_ctrl->modbus_input_data[TOTAL1_HI], statistics_get_total(), mb_endian);

  legacymodbus_int32_to_regs(&comms_ctrl->modbus_input_data[TOTAL2_HI], statistics_get_total(), mb_endian);

  legacymodbus_int32_to_regs(&comms_ctrl->modbus_input_data[PEAK_HI], get_peak_user_rdg(), mb_endian);
}


/* Make a macro function that returns the address, rather than separately
 * defining each register.
 */
#define TARGET1_HI         0
#define TARGET1_LOW        1
#define PRE1_HI            2
#define PRE1_LOW           3

#define TARGET2_HI         4
#define TARGET2_LOW        5
#define PRE2_HI            6
#define PRE2_LOW           7

#define TARGET3_HI         8
#define TARGET3_LOW        9
#define PRE3_HI           10
#define PRE3_LOW          11

#define TARGET4_HI        12
#define TARGET4_LOW       13
#define PRE4_HI           14
#define PRE4_LOW          15

#define TARGET5_HI        16
#define TARGET5_LOW       17
#define PRE5_HI           18
#define PRE5_LOW          19

#define TARGET6_HI        20
#define TARGET6_LOW       21
#define PRE6_HI           22
#define PRE6_LOW          23

#define TARGET7_HI        24
#define TARGET7_LOW       25
#define PRE7_HI           26
#define PRE7_LOW          27

#define TARGET8_HI        28
#define TARGET8_LOW       29
#define PRE8_HI           30
#define PRE8_LOW          31


static void update_holding_regs(struct s_comms_control *comms_ctrl, enum eRinModbusEndian mb_endian)
{
  size_t i;
  size_t addr;

  memset(comms_ctrl->modbus_holding_data, 0, sizeof(comms_ctrl->modbus_holding_data));

  for (i = 0, addr = 0; i < NUM_SETPOINTS; i++)
  {
    legacymodbus_int32_to_regs(&comms_ctrl->modbus_holding_data[addr], setpoint_target_get(i), mb_endian);
    addr += 2;

    legacymodbus_int32_to_regs(&comms_ctrl->modbus_holding_data[addr], setpoint_preflight_get(i), mb_endian);
    addr += 2;
  }
}


static bool save_holding_regs(struct s_comms_control *comms_ctrl, enum eRinModbusEndian mb_endian)
{
  int32_t l;
  uint8_t i, addr;

  for (i = 0, addr = 0; i < NUM_SETPOINTS; i++)
  {
    legacymodbus_regs_to_int32(&l, &comms_ctrl->modbus_holding_data[addr], mb_endian);
    addr += 2;
    if ((l <= APP_DB_R_TARGETS_MAX) && (l >= APP_DB_R_TARGETS_MIN))
      setpoint_target_set(i, l);
    else
      return false;

    legacymodbus_regs_to_int32(&l, &comms_ctrl->modbus_holding_data[addr], mb_endian);
    addr += 2;
    if ((l <= APP_DB_R_PREFLIGHT_MAX) && (l >= APP_DB_R_PREFLIGHT_MIN))
      setpoint_preflight_set(i, l);
    else
      return false;
  }
  return true;
}


static char *byte_to_hstr(char *s, uint8_t b)
{
  s[0] = to_hex(b >> 4);
  s[1] = to_hex(b & 0x0F);
  s[2] = '\0';
  return s;
}


static char *word_to_hstr(char *s, uint16_t w)
{
  s[0] = to_hex((w >> 12) & 0x000F);
  s[1] = to_hex((w >> 8) & 0x000F);
  s[2] = to_hex((w >> 4) & 0x000F);
  s[3] = to_hex(w & 0x000F);
  s[4] = '\0';
  return s;
}


static uint8_t hstr_to_byte(const char *s)
{
  uint8_t b;

  b = (uint8_t) (to_bin(*s++) << 4);
  b += to_bin(*s);
  return b;
}


static uint16_t hstr_to_word(const char *s)
{
  uint16_t w;

  w = (uint16_t) (to_bin(*s++)) << 12;
  w += (uint16_t) (to_bin(*s++)) << 8;
  w += (uint16_t) (to_bin(*s++)) << 4;
  w += (uint16_t) (to_bin(*s));
  return w;
}


static uint8_t calc_lrc(const char *s, size_t len)
{
  uint8_t lrc;

  lrc = 0;
  while (len)
  {
    lrc += hstr_to_byte(s);
    s += 2;
    if (len < 2)
      break;
    len -= 2;
  }
  lrc = (uint8_t) (~lrc + 1);
  return lrc;
}


static uint8_t modbus_key;         /* used to hold kbd equivalent of a modbus command */


static void send_mod_str(struct s_comms_control *comms_ctrl, const char *s)
{
  while (*s)
  {
    comms_net_rfprintf(comms_ctrl, "%c%c", s[0], s[1]);
    comms_ctrl->lrc_out += hstr_to_byte(s);
    s++;
    if (*s == '\0')
      break;
    s++;
  }
}


#define ILLEGAL_FUNCTION   1
#define ILLEGAL_DATA_ADDR  2
#define ILLEGAL_DATA_VALUE 3


static void modbus_send_exception(struct s_comms_control *comms_ctrl, uint8_t fn, uint8_t ex)
{
  char s[10];

  comms_ctrl->lrc_out = 0;
  comms_net_rfprintf(comms_ctrl, ":");
  send_mod_str(comms_ctrl, byte_to_hstr(s, db_app_user.address));
  send_mod_str(comms_ctrl, byte_to_hstr(s, (uint8_t) (fn | 0x80)));
  send_mod_str(comms_ctrl, byte_to_hstr(s, ex));

  comms_ctrl->lrc_out = (uint8_t) (~comms_ctrl->lrc_out + 1);
  comms_net_rfprintf(comms_ctrl, "%s" COMMS_TERMINATOR, byte_to_hstr(s, comms_ctrl->lrc_out));
}


static void modbus_send_input_regs(struct s_comms_control *comms_ctrl, uint16_t addr, uint16_t num, enum eRinModbusEndian mb_endian)
{
  uint8_t i;
  char s[10];

  update_input_regs(comms_ctrl, mb_endian);
  comms_ctrl->reply_active = true;

  if (addr + num > NUM_INPUT_REGS)
  {
    modbus_send_exception(comms_ctrl, 4, ILLEGAL_DATA_ADDR);
    return;
  }

  comms_ctrl->lrc_out = 0;
  comms_net_rfprintf(comms_ctrl, ":");
  send_mod_str(comms_ctrl, byte_to_hstr(s, db_app_user.address));
  send_mod_str(comms_ctrl, "04");
  send_mod_str(comms_ctrl, byte_to_hstr(s, (uint8_t) (num * 2)));
  for (i = (uint8_t) addr; i < (uint8_t) (addr + num); i++)
    send_mod_str(comms_ctrl, word_to_hstr(s, comms_ctrl->modbus_input_data[i]));

  comms_ctrl->lrc_out = (uint8_t) (~comms_ctrl->lrc_out + 1);
  comms_net_rfprintf(comms_ctrl, "%s" COMMS_TERMINATOR, byte_to_hstr(s, comms_ctrl->lrc_out));
}


static void modbus_send_coils(struct s_comms_control *comms_ctrl, uint16_t addr, uint16_t num)
{
  char s[10];

  comms_ctrl->reply_active = true;
  if (addr + num > 22)
  {
    modbus_send_exception(comms_ctrl, 1, ILLEGAL_DATA_ADDR);
    return;
  }

  comms_ctrl->lrc_out = 0;
  /* Start modbus */
  comms_net_rfprintf(comms_ctrl, ":");
  /* Send address */
  send_mod_str(comms_ctrl, byte_to_hstr(s, db_app_user.address));
  /* Send Function */
  send_mod_str(comms_ctrl, "01");
  /* Send byte count + empty bytes */
  if (num <= 8)
  {
    send_mod_str(comms_ctrl, "01");
    send_mod_str(comms_ctrl, "00");
  }
  else if (num <= 16)
  {

    send_mod_str(comms_ctrl, "02");
    send_mod_str(comms_ctrl, "00");
    send_mod_str(comms_ctrl, "00");
  }
  else if (num <= 24)
  {
    send_mod_str(comms_ctrl, "03");
    send_mod_str(comms_ctrl, "00");
    send_mod_str(comms_ctrl, "00");
    send_mod_str(comms_ctrl, "00");
  }
  /* send LRC + trailer */
  comms_ctrl->lrc_out = (uint8_t) (~comms_ctrl->lrc_out + 1);
  comms_net_rfprintf(comms_ctrl, "%s" COMMS_TERMINATOR, byte_to_hstr(s, comms_ctrl->lrc_out));
}


static void modbus_send_holding_regs(struct s_comms_control *comms_ctrl, uint16_t addr, uint16_t num, enum eRinModbusEndian mb_endian)
{
  size_t i;
  char s[10];

  update_holding_regs(comms_ctrl, mb_endian);
  comms_ctrl->reply_active = true;
  if (addr + num > NUM_HOLDING_REGS)
  {
    modbus_send_exception(comms_ctrl, 3, ILLEGAL_DATA_ADDR);
    return;
  }

  comms_ctrl->lrc_out = 0;
  comms_net_rfprintf(comms_ctrl, ":");
  send_mod_str(comms_ctrl, byte_to_hstr(s, db_app_user.address));
  send_mod_str(comms_ctrl, "03");
  send_mod_str(comms_ctrl, byte_to_hstr(s, (uint8_t) (num * 2)));
  for (i = addr; i < addr + num; i++)
    send_mod_str(comms_ctrl, word_to_hstr(s, comms_ctrl->modbus_holding_data[i]));
  comms_ctrl->lrc_out = (uint8_t) (~comms_ctrl->lrc_out + 1);
  comms_net_rfprintf(comms_ctrl, "%s" COMMS_TERMINATOR, byte_to_hstr(s, comms_ctrl->lrc_out));
}


static bool modbus_handle_write_holding(struct s_comms_control *comms_ctrl, uint16_t addr, uint16_t data, enum eRinModbusEndian mb_endian)
{
  update_holding_regs(comms_ctrl, mb_endian);
  comms_ctrl->modbus_holding_data[addr] = data;
  return save_holding_regs(comms_ctrl, mb_endian);
}


static void echo_modbus(struct s_comms_control *comms_ctrl)
{
  char s[10];

  comms_ctrl->lrc_out = 0;
  comms_net_rfprintf(comms_ctrl, ":");
  send_mod_str(comms_ctrl, byte_to_hstr(s, db_app_user.address));
  send_mod_str(comms_ctrl, (char *) (&comms_ctrl->mod_mess[2]));
  comms_ctrl->lrc_out = (uint8_t) (~comms_ctrl->lrc_out + 1);
  comms_net_rfprintf(comms_ctrl, "%s" COMMS_TERMINATOR, byte_to_hstr(s, comms_ctrl->lrc_out));
}


static void modbus_handle_write_coil(struct s_comms_control *comms_ctrl, uint16_t addr, uint16_t num)
{
  /* FF00 is force coil on, 0000 is force coil off */
  if (num != 0xFF00)
    return;

  switch (addr)
  {
    case 0:
      modbus_key = ZERO_KEY;
      break;
    case 1:
      modbus_key = TARE_KEY;
      break;
    case 2:
      modbus_key = GROSS_KEY;
      break;
    case 3:
      modbus_key = FUNC1_KEY;
      break;

    /* Remote input 0-3 short press */
    case 4:
    case 5:
    case 6:
    case 7:
      modbus_key = remote_key_lookup[db_app_user.remote_f[addr - 4]];
      break;

    /* Remote input 0-3 long press */
    case 8:
    case 9:
    case 10:
    case 11:
      modbus_key = remote_key_lookup[db_app_user.remote_f[addr - 8]];
      if (modbus_key != NO_KEY)
        modbus_key += DEKEY;
      break;

    case 12:
      modbus_key = FUNC2_KEY;
      break;
    case 13:
      modbus_key = FUNC3_KEY;
      break;

    /* Remote input 4-7 short press */
    case 14:
    case 15:
    case 16:
    case 17:
      modbus_key = remote_key_lookup[db_app_user.remote_f[addr - 10]];
      break;

    /* Remote input 4-7 long press */
    case 18:
    case 19:
    case 20:
    case 21:
      modbus_key = remote_key_lookup[db_app_user.remote_f[addr - 14]];
      if (modbus_key != NO_KEY)
        modbus_key += DEKEY;
      break;

  }
}


static void echo_modbus_partial(struct s_comms_control *comms_ctrl)
{
  char s[10];
  uint8_t tmp;

  comms_ctrl->lrc_out = 0;
  comms_net_rfprintf(comms_ctrl, ":");
  send_mod_str(comms_ctrl, byte_to_hstr(s, db_app_user.address));
  /* send first section of command as reply */
  tmp = comms_ctrl->mod_mess[12];
  comms_ctrl->mod_mess[12] = '\0';
  send_mod_str(comms_ctrl, (char *) (&comms_ctrl->mod_mess[2]));
  comms_ctrl->mod_mess[12] = tmp;

  comms_ctrl->lrc_out = (uint8_t) (~comms_ctrl->lrc_out + 1);
  comms_net_rfprintf(comms_ctrl, "%s" COMMS_TERMINATOR, byte_to_hstr(s, comms_ctrl->lrc_out));
}


static void modbus_handle_write_multiple_coils(struct s_comms_control *comms_ctrl, const char *s, bool reply)
{
  uint16_t addr, num;
  uint8_t i, cnt, data, mask, num_bytes;

  addr = hstr_to_word(s);
  s += 4;
  num = hstr_to_word(s);
  s += 4;
  if (addr + num > 22)
  {
    if (reply)
      modbus_send_exception(comms_ctrl, 15, ILLEGAL_DATA_ADDR);
    return;
  }

  cnt = hstr_to_byte(s);
  s += 2;
  num_bytes = 1;
  if (num > 8)
    num_bytes++;
  if (num > 16)
    num_bytes++;

  if (cnt != num_bytes)
  {
    if (reply)
      modbus_send_exception(comms_ctrl, 15, ILLEGAL_DATA_VALUE);
    return;
  }

  for (data = 0, mask = 0, i = 0; i < num; i++, mask <<= 1)
  {
    if (mask == 0)
    {
      mask = 0x01;
      data = hstr_to_byte(s);
      s += 2;
    }
    if (data & mask)
    {
      modbus_handle_write_coil(comms_ctrl, (uint16_t) (addr + i), 0xFF00);
      break;
    }
  }
  if (reply)
    echo_modbus_partial(comms_ctrl);
}


static void modbus_handle_write_multiple_holding_regs(struct s_comms_control *comms_ctrl, const char *s, bool reply, enum eRinModbusEndian mb_endian)
{
  uint16_t addr, num, data;
  uint8_t i, cnt;

  addr = hstr_to_word(s);
  s += 4;
  num = hstr_to_word(s);
  s += 4;
  if (addr + num > NUM_HOLDING_REGS)
  {
    if (reply)
      modbus_send_exception(comms_ctrl, 15, ILLEGAL_DATA_ADDR);
    return;
  }

  cnt = hstr_to_byte(s);
  s += 2;
  if (cnt != num * 2)
  {
    if (reply)
      modbus_send_exception(comms_ctrl, 15, ILLEGAL_DATA_VALUE);
    return;
  }
  update_holding_regs(comms_ctrl, mb_endian);
  for (i = 0; i < num; i++)
  {
    data = hstr_to_word(s);
    s += 4;
    comms_ctrl->modbus_holding_data[addr + i] = data;
  }
  if (save_holding_regs(comms_ctrl, mb_endian))
  {
    if (reply)
      echo_modbus_partial(comms_ctrl);
  }
  else
  {
    if (reply)
      modbus_send_exception(comms_ctrl, 15, ILLEGAL_DATA_VALUE);
  }
}


/** Called to get keypresses simulated by legacy Modbus ASCII coil write */
uint8_t get_modbus_key(void)
{
  uint8_t tmp;

  tmp = modbus_key;
  if (tmp != NO_KEY && tmp != IDLE_CHG)
    modbus_key = IDLE_CHG;
  else
    modbus_key = NO_KEY;
  return tmp;
}


static void handle_modbus_cmd(struct s_comms_control *comms_ctrl, const char *s, bool reply, enum eRinModbusEndian mb_endian)
{
  uint8_t func;
  size_t len;
  uint16_t addr, num, data;

  len = strlen(s);
  func = hstr_to_byte(s);
  s += 2;

  switch (func)
  {
    /* Read coil status */
    case 1:
      if (len != 10)
        return;
      addr = hstr_to_word(s);
      s += 4;
      num = hstr_to_word(s);
      if (reply)
        modbus_send_coils(comms_ctrl, addr, num);
      break;

    /* Read holding registers */
    case 3:
      if (len != 10)
        return;
      addr = hstr_to_word(s);
      s += 4;
      num = hstr_to_word(s);
      if (reply)
        modbus_send_holding_regs(comms_ctrl, addr, num, mb_endian);
      break;

    /* Read input registers */
    case 4:
      if (len != 10)
        return;
      addr = hstr_to_word(s);
      s += 4;
      num = hstr_to_word(s);
      if (reply)
        modbus_send_input_regs(comms_ctrl, addr, num, mb_endian);
      break;

    /* Write single coil */
    case 5:
      if (len != 10)
        return;
      addr = hstr_to_word(s);
      s += 4;
      num = hstr_to_word(s);
      modbus_handle_write_coil(comms_ctrl, addr, num);
      if (reply)
        echo_modbus(comms_ctrl);
      break;

    /* Write single holding register */
    case 6:
      if (len != 10)
        return;
      addr = hstr_to_word(s);
      s += 4;
      data = hstr_to_word(s);
      if (modbus_handle_write_holding(comms_ctrl, addr, data, mb_endian))
      {
        if (reply)
          echo_modbus(comms_ctrl);
      }
      else
      {
        if (reply)
          modbus_send_exception(comms_ctrl, 6, ILLEGAL_DATA_VALUE);
      }
      break;

    /* Write multiple coils */
    case 15:
      if (len < 14)
        return;
      modbus_handle_write_multiple_coils(comms_ctrl, s, reply);
      break;

    /* Write multiple holding registers */
    case 16:
      if (len < 14)
        return;
      modbus_handle_write_multiple_holding_regs(comms_ctrl, s, reply, mb_endian);
      break;

    default:
      if (reply)
      {
        modbus_send_exception(comms_ctrl, func, ILLEGAL_FUNCTION);
      }
      break;
  }
}


static void process_modbus(struct s_comms_control *comms_ctrl, enum eRinModbusEndian mb_endian)
{
  const char *p;
  uint8_t address;

  /* Check minimum length */
  if (comms_ctrl->mod_len < 6)
    return;

  /* Check data looks ok */
  p = comms_ctrl->mod_mess;
  while (*p)
  {
    if (!is_hex(*p))
      return;
    p++;
  }

  /* Check checksum */
  if (calc_lrc(comms_ctrl->mod_mess, comms_ctrl->mod_len - 2) != hstr_to_byte(&comms_ctrl->mod_mess[comms_ctrl->mod_len - 2]))
    return;

  /* Check address */
  address = hstr_to_byte(comms_ctrl->mod_mess);
  if (address != 0 && address != db_app_user.address)
    return;

  /* Remove lrc from end of message */
  comms_ctrl->mod_len -= 2;
  comms_ctrl->mod_mess[comms_ctrl->mod_len] = '\0';

  /* Handle validated modbus command, no reply if broadcast address (=0) */
  handle_modbus_cmd(comms_ctrl, &comms_ctrl->mod_mess[2], (address != 0), mb_endian);
}


/** Legacy Modbus ASCII comms handler */
void handle_modbus(struct s_comms_control *comms_ctrl, char *mess, size_t len, enum eRinModbusEndian mb_endian)
{

  /* Check the start ':' */
  if (*mess != ':' || len <= 1)
    return;

  /*
   * Drop off the start ':'
   * make sure its terminated
   * and process the data
   */
  comms_ctrl->mod_mess = &mess[1];
  comms_ctrl->mod_len = len - 1;
  comms_ctrl->mod_mess[comms_ctrl->mod_len] = '\0';
  process_modbus(comms_ctrl, mb_endian);
}
