/*
 * model_licensing.c
 *
 *  Created on: 31 Aug. 2018
 *      Author: ian
 */
#include <stdbool.h>
#include "licensing.h"
#include "indicator_model.h"
#include "display.h"
#include "iface.h"
#include "iface_sd.h"

#include "model_licensing.h"


/**
 * Check that what the factory database says we are matches the installed package,
 * and refuse to run if not
 */
void model_licensing_check(bool *quit)
{
  enum e_indicator_model model;
  bool can_run;
  char split_pn[2][NUM_DIGITS + 1];
  const char *mainappname;
  const char *prompts[4];
  const char *package_name;

  model = get_indicator_model();

  switch (model)
  {
    case im_c510:
      /*
       * If factory database reports we are a C510, then:
       *  - Don't run if C510-K501 is not installed (opkg conflicts prevents other main apps being installed)
       *  - Don't run if any L9xx-5xx packages are installed
       */
      mainappname = "C510-K501";
      can_run = app_licensing_is_installed(mainappname);
      if (!can_run)
      {
        app_licensing_split_package_name_for_display(mainappname, split_pn);
        prompts[0] = split_pn[0], prompts[1] = split_pn[1], prompts[2] = "NOT", prompts[3] = "INSTLD";
      }
      else
      {
        can_run = !app_licensing_are_addon_packages_installed(&package_name);
        if (!can_run)
        {
          /* At least one addon package is installed, so don't run */
          app_licensing_split_package_name_for_display(package_name, split_pn);
          prompts[0] = split_pn[0], prompts[1] = split_pn[1], prompts[2] = "CANNOT", prompts[3] = "RUN";
        }
      }
      break;

    case im_c520:
    case im_c530:
      /*
       * If factory database reports we are a C520/C530, then:
       *  - Don't run if C500-K501 is not installed (opkg conflicts prevents other main apps being installed)
       */
      mainappname = "C500-K501";
      can_run = app_licensing_is_installed(mainappname);
      app_licensing_split_package_name_for_display(mainappname, split_pn);
      prompts[0] = split_pn[0], prompts[1] = split_pn[1], prompts[2] = "NOT", prompts[3] = "INSTLD";
      break;

    default:
      /* Unknown model, don't run */
      can_run = false;
      prompts[0] = "H.WARE", prompts[1] = "MODEL", prompts[2] = "NOT", prompts[3] = "KNOWN";
      break;
  }

  if (!can_run)
  {
    /* Freeze the display */
    freeze_display_and_annunciators(true);
    Dclear_flashing();

    while (!*quit)
    {
      (void) prompt_multi(1250, 0, quit, prompts[0], prompts[1], prompts[2], prompts[3], NULL);
    }

    /* Unfreeze the display */
    freeze_display_and_annunciators(false);
  }
}
