#ifndef NV_RAM_DOT_H
#define NV_RAM_DOT_H 1

#include <sys/types.h>
#include "database.h"


/** Size in bytes of the hardware NVRAM on a original version processor PCB */
#define NVRAM_HARDWARE_SIZE_VER0 32768


/** Size in bytes of the hardware NVRAM on a new version processor PCB (RTC registers steal 16 bytes) */
#define NVRAM_HARDWARE_SIZE_VER1 (32768 - 16)


/** Offset of the first copy of the runtime database in the NVRAM */
#define NVRAM_RUNTIME_DB0_OFFSET 0


/** Offset of the second copy of the runtime database in the NVRAM. Always located half way through the original NVRAM size. */
#define NVRAM_RUNTIME_DB1_OFFSET (NVRAM_HARDWARE_SIZE_VER0 / 2)


/*
 * NVRAM read/write functions
 */


/** Read data from the NVRAM hardware device */
bool nvram_hw_read(void *buffer, off_t offset, size_t length);


/** Write data to the NVRAM hardware device */
bool nvram_hw_write(const void *buffer, off_t offset, size_t length);


/*
 * NVRAM initialisation functions
 */


/**
 * Initialise access to the NVRAM, and check the contents for valid data
 * Returns false if any part of the NVRAM had to be defaulted
 */
bool nvram_init(void);


/** Clean up the nvram */
void nvram_destroy(void);


#endif /* #ifndef NV_RAM_DOT_H */
