/*
 * remote_cal.h
 *
 *  Created on: 19/05/2011
 *      Author: ian
 */

#ifndef REMOTE_CAL_H_
#define REMOTE_CAL_H_


enum eRemoteCalMode
{
  REMOTE_CAL_MODE_IDLE = 0,
  REMOTE_CAL_MODE_ZERO_NORMAL,
  REMOTE_CAL_MODE_SPAN_NORMAL,
  REMOTE_CAL_MODE_ZERO_DIRECT,
  REMOTE_CAL_MODE_SPAN_DIRECT
};


enum eRemoteCalResult
{
  REMOTE_CAL_RESULT_NO_ERROR = 0,
  REMOTE_CAL_RESULT_BUSY = 1,
  REMOTE_CAL_RESULT_ZERO = 101,
  REMOTE_CAL_RESULT_SPAN = 104,
  REMOTE_CAL_RESULT_ERROR = 106
};


/** Sets the result of the last calibration, see REMOTE_CAL_* constants */
void set_last_cal_result(enum eRemoteCalResult res);


/** Gets the result of the last calibration, see REMOTE_CAL_* constants  */
enum eRemoteCalResult get_last_cal_result(void);


/** Returns the current calibration mode */
enum eRemoteCalMode get_remote_cal_mode(void);


/** Sets the current calibration mode */
void set_remote_cal_mode(enum eRemoteCalMode mode);


/** Call when calibration is complete */
void finish_calibration(void);


/** Called by the comms thread to kick off a zero calibration */
void setup_for_zero_cal(void);


/** Called by the comms thread to initiate a direct zero calibration. mvv is milli-volts per volt * 10000 (1mvv = 10000) */
void setup_for_zero_direct_cal(int32s mvv);


/** Called by the comms thread to kick off a span calibration */
void setup_for_span_cal(void);


/** Called by the comms thread to initiate a direct span calibration. mvv is milli-volts per volt * 10000 (1mvv = 10000) */
void setup_for_span_direct_cal(int32s mvv);


/**
 * Poll to check if a remote calibration has been requested
 * by the comms thread, and perform the calibration
 */
void check_remote_cal(void);


/** Gets the result of the last calibration, see ADC_OP_STATUS* constants  */
uint8_t get_last_cal_output(void);


#endif /* REMOTE_CAL_H_ */
