#include <rinhw/sio.h>
#include "serial_ports.h"

#include "serial_levels.h"


/** Work out how much room there is left in the serial transmit buffer */
uint16_t get_serial_tx_free(struct t_io_dev *dev)
{
  unsigned short level = 0;
  unsigned short max = 0;

  rioctl(dev, IOCTL_SER_GET_TX_BUF_LEVEL, &level);
  rioctl(dev, IOCTL_SER_GET_TX_BUF_SIZE, &max);

  return (uint16_t) (max - level);
}


/** Work out the size of the serial transmit buffer */
uint16_t get_serial_tx_bufsize(struct t_io_dev *dev)
{
  unsigned short bufsize = 0;

  rioctl(dev, IOCTL_SER_GET_TX_BUF_SIZE, &bufsize);

  return bufsize;
}


/**
 * Return if CTS is blocking transmission on a port.
 * This is simply the pin state, not whether the port
 * is configured to ignore CTS (or not)
 */
bool get_serial_cts_blocking(struct t_io_dev *dev)
{
  unsigned char cts_free = 0;

  rioctl(dev, IOCTL_SER_GET_CTS_STATE, &cts_free);

  return (0 == cts_free);
}
