/*
 * serial_ports.h
 *
 *  Created on: 17/12/2010
 *      Author: ian
 */

#ifndef SERIAL_PORTS_H_
#define SERIAL_PORTS_H_

#include <rinhw/rstdio.h>
#include "we_config.h"


/** Serial ports on the WE2111 */
enum eSerialPort {
  SERPORT_RL_FRONT = 0,   /*!< Front rinlink, settings in database */
  SERPORT_SER1_RL_REAR,   /*!< Rear rinlink/Serial 1 DB9, settings in database */
  SERPORT_RB1A,           /*!< Module 1A, settings in database */
  SERPORT_RB1B,           /*!< Module 1B, settings in database */
  SERPORT_RB2A,           /*!< Module 2A, settings in database */
  SERPORT_RB2B,           /*!< Module 2B, settings in database */
  SERPORT_USBGADGET,      /*!< USB gadget serial port, type setting in database */
  SERPORT_USBHOST,        /*!< USB host serial port, settings in database */
  SERPORT_SER2,           /*!< Serial 2 DB9, settings in database */
  SERPORT_USBLP,          /*!< USB line printer, no settings */
  SERPORT_UDPOUT1,        /*!< Outgoing UDP port 1 (you specify the destination) */
  SERPORT_UDPOUT2,        /*!< Outgoing UDP port 2 (you specify the destination) */
  SERPORT_UDPIN1,         /*!< Incoming UDP port 1 (indicator will reply to latest incoming source ip:port) */
  SERPORT_UDPIN2,         /*!< Incoming UDP port 2 (indicator will reply to latest incoming source ip:port) */

  /* After the fixed ports, we have a range of dynamically allocated network ports */
  SERPORT_NETWORK_START,
  SERPORT_NUM = SERPORT_NETWORK_START + WECFG_NET_MAX_TCP_CONNECTIONS,

  /* Network ports get created using these pseudo port numbers */
  SERPORT_TCPNET = 100,
  SERPORT_TCPAUTO,
  SERPORT_TCP_FINAL,
  SERPORT_TCP_FIRST = 100,

  /* Unix domain socket ports get created using these pseudo port numbers */
  SERPORT_UDSNET = 200,
  SERPORT_UDSAUTO,
  SERPORT_UDS_FINAL,
  SERPORT_UDS_FIRST = 200,

  /* UDP ports */
  SERPORT_UDP_FIRST = SERPORT_UDPOUT1,
  SERPORT_UDP_FINAL = SERPORT_UDPIN2 + 1,

  /* A sentinel to indicate unknown type */
  SERPORT_UNKNOWN = 9999
};


/** Serial port names, must match @ref enum eSerialPort upto SERPORT_NETWORK_START - 1 */
#define SERIAL_PORT_STRINGS { "SER.RL.F", "SER.RL.R", "SER.M.1A", "SER.M.1B", "SER.M.2A", "SER.M.2B", "SER.SLV", "SER.HST", "SER.2", "SER.LP", "UDP.O1", "UDP.O2", "UDP.IN1", "UDP.IN2" }


/** TCP serial port names, must match @ref enum eSerialPort SERPORT_TCP_FIRST..SERPORT_TCP_FINAL */
#define SERIAL_TCP_PORT_STRINGS { "TCP.NET", "TCP.AUT" }


/** Unix domain socket serial port names, must match @ref enum eSerialPort SERPORT_UDS_FIRST..SERPORT_UDS_FINAL */
#define SERIAL_UDS_PORT_STRINGS { "UDS.NET", "UDS.AUT" }


/**
 * Number of serial port settings stored in the database.
 * (9 = Front rinlink, rear rinlink/serial1, 2 rinblock with 2 channels, 1 USB gadget, 1 USB host, serial 2)
 * The remaining "serial ports" (sockets, usb serial) don't have normal serial port settings
 */
#define NUM_DB_SERIAL_SETTINGS (SERPORT_SER2 + 1)


/** Number of listening TCP sockets */
#define NUM_TCP_LISTENER_PORTS (SERPORT_TCP_FINAL - SERPORT_TCP_FIRST)


/** Number of listening UDS sockets */
#define NUM_UDS_LISTENER_PORTS (SERPORT_UDS_FINAL - SERPORT_UDS_FIRST)


/** SER 1 - Onboard rear rinlink port or DB9 RS232/RS422 serial port */
extern struct t_io_dev dev_ser1_local;


/** SER 2 - Onboard RS232 secondary serial port */
extern struct t_io_dev dev_ser2_local;


/** Front rinlink via display rinblock */
extern struct t_io_dev dev_ser_front_rl;


/** Main (bi-directional) RS422 rinblock serial port */
extern struct t_io_dev dev_ser3_rb_rs422;


/** Secondary (transmit only) RS422 rinblock serial port, not used */
 extern struct t_io_dev dev_ser3_rb_rs422b;


/** Main (bi-directional) RS232 rinblock serial port */
extern struct t_io_dev dev_ser4_rb_rs232;


/** Secondary (transmit only) RS232 rinblock serial port, not used */
extern struct t_io_dev dev_ser4_rb_rs232b;


/** USB serial gadget serial port */
extern struct t_io_dev dev_ser7_usb;


/** USB host serial port */
extern struct t_io_dev dev_ser8_usbhostserial;


/** USB line printer serial port */
extern struct t_io_dev dev_ser9_usblp;


#endif /* SERIAL_PORTS_H_ */
