/*
 * serial_usb.c
 *
 *  Created on: 11/10/2010
 *      Author: ian
 */
#include <stdio.h>
#include <wait.h>
#include <rinexec/el_runcmd.h>
#include <rinhw/rio.h>
#include <rinhw/sio_local_comport.h>
#include "we_config.h"
#include "fileutils.h"
#include "we_debug.h"

#include "serial_usb.h"


/** Insert the necessary modules, make a device node, and create a rio serial device to access it */
int setup_linux_serial_gadget(struct t_io_dev *dev)
{
  int ret_code;
  int i;
  #define LSG_PARAM_STR_LEN 30
  char vid_str[LSG_PARAM_STR_LEN];
  char pid_str[LSG_PARAM_STR_LEN];
  char major[LSG_PARAM_STR_LEN];
  char minor[LSG_PARAM_STR_LEN];
  #define MODPROBE_NUM_ARGS 6
  const char *modprobe_args[MODPROBE_NUM_ARGS];
  #define MKNOD_NUM_ARGS 6
  const char *mknod_args[MKNOD_NUM_ARGS];

  /* Run modprobe and check the return code */
  const char *const modprobe_ftdi_args[] = { "/sbin/modprobe", "ftdi_sio", NULL };
  const char *const modprobe_cp2101_args[] = { "/sbin/modprobe", "cp2101", NULL };
  const char *const modprobe_pl2303_args[] = { "/sbin/modprobe", "pl2303", NULL };

  /* Construct g_serial modprobe command */
  i = 0;
  modprobe_args[i++] = "/sbin/modprobe";
  modprobe_args[i++] = "g_serial";
  modprobe_args[i++] = "use_acm=1";
  if (we_configuration.usb_gadget_vid > 0 && we_configuration.usb_gadget_pid > 0)
  {
    snprintf(vid_str, LSG_PARAM_STR_LEN, "idVendor=0x%04X", we_configuration.usb_gadget_vid);
    modprobe_args[i++] = vid_str;
    snprintf(pid_str, LSG_PARAM_STR_LEN, "idProduct=0x%04X", we_configuration.usb_gadget_pid);
    modprobe_args[i++] = pid_str;
  }
  modprobe_args[i++] = NULL;

  /* Run g_serial modprobe and check the return code */
  if (el_run_command(modprobe_args[0], modprobe_args, &ret_code, NULL, true, false, true) != elr_success ||
      !WIFEXITED(ret_code) || 0 != WEXITSTATUS(ret_code))
  {
    rdebug_printf(LOG_ERR, "setup_linux_serial_gadget: modprobe g_serial failed\n");
    return RIO_ERROR;
  }

  /* Check if device node exists, and if not create it */
  if (!file_exists(we_configuration.usb_gadget_device_name))
  {
    /* Construct mknod command */
    mknod_args[0] = "/bin/mknod";
    mknod_args[1] = we_configuration.usb_gadget_device_name;
    mknod_args[2] = "c";
    snprintf(major, LSG_PARAM_STR_LEN, "%d", we_configuration.usb_gadget_device_major);
    mknod_args[3] = major;
    snprintf(minor, LSG_PARAM_STR_LEN, "%d", we_configuration.usb_gadget_device_minor);
    mknod_args[4] = minor;
    mknod_args[5] = NULL;

    /* Run mknod and check the return code */
    if (el_run_command(mknod_args[0], mknod_args, &ret_code, NULL, true, false, true) != elr_success ||
        !WIFEXITED(ret_code) || 0 != WEXITSTATUS(ret_code))
    {
      rdebug_printf(LOG_ERR, "setup_linux_serial_gadget: \"mknod %s c %d %d\" failed\n",
                    we_configuration.usb_gadget_device_name, we_configuration.usb_gadget_device_major, we_configuration.usb_gadget_device_minor);
      return RIO_ERROR;
    }
  }

  /* Run modprobe ftdi serial drivers and check the return code */
  if (el_run_command(modprobe_ftdi_args[0], modprobe_ftdi_args, &ret_code, NULL, true, false, true) != elr_success ||
      !WIFEXITED(ret_code) || 0 != WEXITSTATUS(ret_code))
  {
    rdebug_printf(LOG_ERR, "setup_linux_serial_gadget: \"modprobe ftdi\" failed\n");
    return RIO_ERROR;
  }

  /* Run modprobe cp2102 serial drivers and check the return code */
  if (el_run_command(modprobe_cp2101_args[0], modprobe_cp2101_args, &ret_code, NULL, true, false, true) != elr_success ||
      !WIFEXITED(ret_code) || 0 != WEXITSTATUS(ret_code))
  {
    rdebug_printf(LOG_ERR, "setup_linux_serial_gadget: \"modprobe cp2101\" failed\n");
    return RIO_ERROR;
  }

  /* Run modprobe pl2303 serial drivers and check the return code */
  if (el_run_command(modprobe_pl2303_args[0], modprobe_pl2303_args, &ret_code, NULL, true, false, true) != elr_success ||
      !WIFEXITED(ret_code) || 0 != WEXITSTATUS(ret_code))
  {
    rdebug_printf(LOG_ERR, "setup_linux_serial_gadget: \"modprobe pl2303\" failed\n");
    return RIO_ERROR;
  }

  /* Now create a rinstrum serial device connected to the USB serial gadget port */
  return rio_create_device_sio_localcom(dev, we_configuration.usb_gadget_device_name, NULL, true);
}
