/*
 * serial_usb_host.c
 *
 *  Created on: Sep 2, 2016
 *      Author: ian
 */
#include <stdbool.h>
#include <stdio.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <rinhw/rio.h>
#include <rinhw/sio_local_comport.h>
#include "we_debug.h"
#include "app_db_user.h"
#include "serial_ports.h"
#include "db_serial.h"
#include "comms_task.h"
#include "iface_usb.h"

#include "serial_usb_host.h"


/** Structure with usb serial port info */
struct s_serial_usb_host_info
{
  char *devicename;
  enum eSerialPortType last_serial_port_type;
};


static struct s_serial_usb_host_info serial_usb_host_info =
{
  .devicename = NULL,
  .last_serial_port_type = SERPORT_UNKNOWN,
};


#define COMMS_IS_RINSTRUM(serial_type) (serial_type <= DB_SERIAL_TYPE_MODBUS_LEGACY)
#define COMMS_IS_MODBUS(serial_type) (!(COMMS_IS_RINSTRUM(serial_type)))


static bool add_usb_host_serial_as_rindevice(const char *device)
{
  bool result = false;

  if (0 == (dev_ser8_usbhostserial.status & O_INITIALISED))
  {
    /* Doesn't already exist, so create rinstrum device */
    if (rio_create_device_sio_localcom(&dev_ser8_usbhostserial, device, NULL, false) == RIO_SUCCESS)
    {
      /* Add new printer comms task to handle it */
      if (comms_create_thread_for_serial_port(&dev_ser8_usbhostserial, SERPORT_USBHOST, NULL) == RIO_SUCCESS)
      {
        WE_DEVICEMOUNTER_SERIAL_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "add_usb_host_serial_as_rindevice: successfully created rinstrum serial device and task for device %s\n", device);
        result = true;
      }
      else
      {
        /* Fail */
        rdebug_printf(LOG_ERR, "add_usb_host_serial_as_rindevice: failed create thread for serial device %s\n", device);
      }
    }
    else
    {
      rdebug_printf(LOG_ERR, "add_usb_host_serial_as_rindevice: failed to create rinstrum serial port for device %s\n", device);
    }
  }
  else
  {
    rdebug_printf(LOG_ERR, "add_usb_host_serial_as_rindevice: a serial port is already mounted, not mounting %s\n", device);
  }
  return result;
}


static bool remove_usb_host_serial_as_rindevice(const char *device)
{
  bool result = false;

  if (0 != (dev_ser8_usbhostserial.status & O_INITIALISED))
  {
    /* Already exists, so we can destroy it */
    WE_DEVICEMOUNTER_SERIAL_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "remove_usb_host_serial_as_rindevice: cleaning up registered serial port %s\n", device);

    /* Tell USB host serial port comms task to quit, this blocks until it quits */
    comms_destroy_task(&dev_ser8_usbhostserial);

    /* Destroy device */
    result = (rdestroy_io(&dev_ser8_usbhostserial) == RIO_SUCCESS);
  }
  else
  {
    rdebug_printf(LOG_ERR, "remove_usb_host_serial_as_rindevice: no serial port registered, so can't clean up %s\n", device);
  }
  return result;
}


/** Configure rinmodbus with database settings */
void rinmodbus_configure_from_db(void)
{
  t_ioctl_siocfg siocfg;
  bool use_rs232_dummy;
  enum eSerialPortType spt;

  spt = db_app_user.serial_type[SERPORT_USBHOST];

  /* If serial type changed to modbus, we need to shutdown the rinserial task first */
  if (COMMS_IS_MODBUS(spt) && 0 != (dev_ser8_usbhostserial.status & O_INITIALISED) && NULL != serial_usb_host_info.devicename)
  {
    (void) remove_usb_host_serial_as_rindevice(serial_usb_host_info.devicename);
  }

  /* Re-configure modbus */
  db_get_serialconfig(SERPORT_USBHOST, &siocfg, &use_rs232_dummy);
  rinmodbus_reinit(db_app_user.modbus_endian,
                   db_app_user.modbus_tcpport,
                   (DB_SERIAL_TYPE_MODBUS_ASCII == spt) ? RINMODBUS_SERIAL_MODE_ASCII : RINMODBUS_SERIAL_MODE_RTU,
                   db_app_user.address,
                   siocfg.baud,
                   (DB_SERIAL_PARITY_NONE == siocfg.parity_bits) ? 'N' : (DB_SERIAL_PARITY_EVEN == siocfg.parity_bits) ? 'E' : 'O',
                   siocfg.data_bits,
                   siocfg.stop_bits);

  /* Set serial port to device if enabled, else NULL */
  rinmodbus_reinit_serial_port((COMMS_IS_MODBUS(spt)) ? serial_usb_host_info.devicename : NULL);

  /* If serial type changed to rinserial, we need to start the rinserial task first */
  if (COMMS_IS_RINSTRUM(spt) && 0 == (dev_ser8_usbhostserial.status & O_INITIALISED) && NULL != serial_usb_host_info.devicename)
  {
    (void) add_usb_host_serial_as_rindevice(serial_usb_host_info.devicename);
  }
}


static bool add_usb_host_serial_as_modbus(const char *device)
{
  rinmodbus_configure_from_db();
  /* rinmodbus_reinit_serial_port(device); - rinmodbus_configure_from_db() does this */
  return true;
}


static bool remove_usb_host_serial_as_modbus(const char *device)
{
  rinmodbus_reinit_serial_port(NULL);
  return true;
}


/** Callback from devicemounter to announce arrive of serial port */
void usb_host_serial_device_callback(void *client_callback_data, enum eDeviceMounterAction action, bool result, const char *device)
{
  bool our_result = false;

  if (result && NULL != device)
  {
    if (dmaAdd == action)
    {
      /* Add new serial port */
      if (NULL == serial_usb_host_info.devicename)
      {
        serial_usb_host_info.devicename = strdup(device);
        serial_usb_host_info.last_serial_port_type = db_app_user.serial_type[SERPORT_USBHOST];
        if (NULL == serial_usb_host_info.devicename)
        {
          rdebug_printf(LOG_ERR, "usb_host_serial_device_callback: failed to allocate memory for device name %s\n", device);
          goto uhsdc_fail;
        }
        if (COMMS_IS_RINSTRUM(db_app_user.serial_type[SERPORT_USBHOST]))
        {
          /* Rinstrum comms */
          our_result = add_usb_host_serial_as_rindevice(serial_usb_host_info.devicename);
        }
        else
        {
          /* Modbus comms */
          our_result = add_usb_host_serial_as_modbus(serial_usb_host_info.devicename);
        }
      }
      else
      {
        rdebug_printf(LOG_ERR, "usb_host_serial_device_callback: a serial port is already mounted as %s, not mounting %s\n", serial_usb_host_info.devicename, device);
      }
    }
    else
    {
      /* Remove serial port */
      if (NULL != serial_usb_host_info.devicename)
      {
        if (0 == strcmp(serial_usb_host_info.devicename, device))
        {
          if (COMMS_IS_RINSTRUM(db_app_user.serial_type[SERPORT_USBHOST]))
          {
            our_result = remove_usb_host_serial_as_rindevice(device);
          }
          else
          {
            our_result = remove_usb_host_serial_as_modbus(device);
          }
        }
        else
        {
          rdebug_printf(LOG_ERR, "usb_host_serial_device_callback: %s has been removed, but it's not our serial port (%s), doing nothing\n", device, serial_usb_host_info.devicename);
        }

        /* Clean up */
        free(serial_usb_host_info.devicename);
        serial_usb_host_info.devicename = NULL;
        serial_usb_host_info.last_serial_port_type = SERPORT_UNKNOWN;
      }
      else
      {
        rdebug_printf(LOG_ERR, "usb_host_serial_device_callback: no serial port registered, so can't clean up %s\n", device);
      }
    }
  }

uhsdc_fail:
  iface_usb_serial_callback(client_callback_data, action, result && our_result);
}


/** Reconfigure usb host serial when serial port type changes */
void usb_host_serial_device_reconfigure(void)
{
  if (NULL != serial_usb_host_info.devicename)
  {
    /* There is a USB host serial port mounted */
    if (COMMS_IS_RINSTRUM(db_app_user.serial_type[SERPORT_USBHOST]) && COMMS_IS_MODBUS(serial_usb_host_info.last_serial_port_type))
    {
      /* And the port type has changed from modbus->rinstrum, so destroy modbus, create rinstrum */
      (void) remove_usb_host_serial_as_modbus(serial_usb_host_info.devicename);
      (void) add_usb_host_serial_as_rindevice(serial_usb_host_info.devicename);
      WE_DEVICEMOUNTER_SERIAL_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "usb_host_serial_device_reconfigure: changed from modbus comms to rinstrum comms for device %s\n", serial_usb_host_info.devicename);
    }
    else if (COMMS_IS_MODBUS(db_app_user.serial_type[SERPORT_USBHOST]) && COMMS_IS_RINSTRUM(serial_usb_host_info.last_serial_port_type))
    {
      (void) remove_usb_host_serial_as_rindevice(serial_usb_host_info.devicename);
      (void) add_usb_host_serial_as_modbus(serial_usb_host_info.devicename);
      WE_DEVICEMOUNTER_SERIAL_DEBUG_PRINTF(RDEBUG_LVL_BASIC, LOG_INFO, "usb_host_serial_device_reconfigure: changed from rinstrum comms to modbus comms for device %s\n", serial_usb_host_info.devicename);
    }
    else
    {
      /* No change */
    }
  }
}
