#ifndef SETPOINT_DOT_H
#define SETPOINT_DOT_H 1

#include <stdbool.h>
#include <stdint.h>
#include <rinweigh/adcinterface.h>
#include "db_remote.h"
#include "db_setpoint.h"


/** Setpoint direction settings */
enum eSetpointOperationMode
{
  somOverHigh = 0,
  somUnderHigh,
  somOverLow,
  somUnderLow,

  somNum
};


/** Per setpoint state */
struct s_setpoint_info
{
  int32_t splimit_high;
  int32_t splimit_low;
  enum eSetpointType sptype;
  enum eSetpointLogic splogic;
  enum eSetpointSource spsource;
  enum eRemoteKeyFn inputtype;
  enum eSetpointTiming spTiming;
  enum eSetpointReset spReset;
  bool onLogical;
  bool lastLogical;                 /*!< Previous onLogical value after setpoint timing was processed */
  uint8_t changeTime;
  bool reset;
  bool latchReset;
  bool latchLogical;
  uint8_t plsNum;
  struct timespec endTime;
  bool waitMotion;
  uint8_t setNum;
  bool prevLogical;                 /*!< Previous onLogical value before setpoint timing is processed */
};


/** Mask for setpoint states */
typedef uint8_t setpoint_data_t;


/** Setpoint running information */
struct s_setpoint_running_info
{
  uint32_t last_reading_id;                               /*!< Last reading ID */
  struct s_setpoint_info setpoint_info[NUM_SETPOINTS];    /*!< Running information for each setpoint */

  /* Masks */
  setpoint_data_t direct_is_driven;                       /*!< I/O to be configured as outputs by direct drive commands (FOP/POR) */
  setpoint_data_t direct_set;                             /*!< Outputs driven high by direct drive commands (FOP/POR) */
  setpoint_data_t direct_clear;                           /*!< Outputs driven low by direct drive commands (FOP/POR) */

  setpoint_data_t setp_active;                            /*!< Both these flags keep track of which outputs are on */
  setpoint_data_t outp_active;                            /*!< Output active allows for an active setpoint to not drive the relay */
  int32_t rdg_1[SETP_SOURCE_NUM];
  int32_t rdg_2[SETP_SOURCE_NUM];

  uint16_t rdg_period;

  uint32_t jog_on;
  uint32_t jog_off;
  uint8_t  jog_set;
  uint8_t  max_sets;
  enum eSetpointFeeder feeder;
  uint32_t delay_check;
  bool feed_active;
  struct timespec active_delay;
};


/** Drive an output directly (for use with comms commands like FOP/POR) */
bool direct_drive_output(struct s_setpoint_running_info *spri, size_t setpoint, bool on);


/** Is setpoint a direct drive output */
bool is_direct_drive_output(const struct s_setpoint_running_info *spri, size_t setpoint);


/** Is setpoint output currently driven */
bool is_setpoint_output_active(const struct s_setpoint_running_info *spri, size_t setpoint);


/** Poll this regularly to update setpoints */
void update_setpoints(struct s_setpoint_running_info *spri, const AdcSuperStructure *ass);


/** (Re)initialise setpoints with database settings */
void init_setpoints(struct s_setpoint_running_info *spri, const AdcSuperStructure *ass);


/** Stop direct drive of setpoint */
void clear_direct_drive_output(struct s_setpoint_running_info *spri, size_t setpoint);


#endif /* #ifndef SETPOINT_DOT_H */
