/*
 * status_led.c
 *
 *  Created on: Dec 17, 2014
 *      Author: ian
 */
#include <stdbool.h>
#include <stdio.h>
#include <pthread.h>
#include <time.h>
#include <rinhw/rinthread.h>
#include <rinhw/we21hw.h>
#include "we_debug.h"

#include "status_led.h"


static bool indicator_has_status_led = false;
static bool status_led_quit = false;
static pthread_t status_led_pthread;


/** State of the status LED */
static bool status_led_state = false;
static int status_led_frequency = STATUS_LED_NORMAL_FREQ;


/** Toggle the status LED */
static void status_led_toggle(void)
{
  (void) we21hw_select_rs232_not_rs422(NULL, (status_led_state) ? 1 : 0);
  status_led_state = !status_led_state;
}


/** Set the status LED frequency */
void status_led_set_frequency(int frequency)
{
  if (indicator_has_status_led)
  {
    status_led_frequency = frequency;
    rdebug_printf(LOG_ERR, "status_led_set_frequency: freq %d\n", frequency);
  }
}


/** Worker thread for the status LED */
static void *status_led_thread(void *data)
{
  struct timespec ts;
  unsigned int ms;
  int old_status_led_frequency;

  /* Thread loop */
  old_status_led_frequency = ~status_led_frequency;
  while (!status_led_quit)
  {
    status_led_toggle();

    if (old_status_led_frequency != status_led_frequency)
    {
      ms = 1000 / status_led_frequency;

      ts.tv_sec = ms / 1000;
      ts.tv_nsec = (ms % 1000) * 1000 * 1000;
    }
    nanosleep(&ts, NULL);
  }

  rin_thread_exit(NULL);
}


/** Initialise the status LED, if has_status_led */
void init_status_led(bool has_status_led)
{
  indicator_has_status_led = has_status_led;

  if (indicator_has_status_led)
  {
    if (!rin_thread_create(&status_led_pthread, status_led_thread, NULL, true))
    {
      rdebug_printf(LOG_ERR, "init_status_led: pthread task create failed\n");
    }
  }
}


/** Shutdown the status LED */
void destroy_status_led(void)
{
  if (indicator_has_status_led)
  {
    status_led_quit = true;

    /* Wait for thread to quit */
    pthread_join(status_led_pthread, NULL);
  }
}
