#include <stdlib.h>
#include <rinhw/ucos_ii.h>
#include <rinhw/bkgrnd.h>
#include "we_debug.h"
#include "debug_loop.h"

#include "system.h"


static uint8_t tenths;                    /* counts tenths of seconds */


/***************************************************************************/
uint8_t get_seconds(void)
{
  uint32_t ticks;
  uint32_t seconds;

  ticks = OSTimeGet();
  seconds = ticks / OS_TICKS_PER_SEC;
  return (uint8_t) seconds;
}


/***************************************************************************/
uint8_t get_tenths(void)
{
  return (tenths);
}


/** Yield the current task for a number of milliseconds */
void delay(uint16_t millis)
{
  long ticks = OS_TICKS_PER_SEC;

  ticks *= millis;
  ticks /= 1000;
  OSTimeDly((uint16_t) ticks);
}


#if RDEBUG_ENABLED(WE_SYSTEM_DEBUG_LVL, RDEBUG_LVL_FULL)
/* static struct s_loop_timing lt_10msec; */
static struct s_loop_timing lt_100msec;
static struct s_loop_timing lt_1000msec;
#endif

#if 0
/** Called every 10msec / 100 times per second */
static void bg_poll_10msec_fn(void *data)
{
#if 0
  if (delay_cnt)
    delay_cnt--;                /* update counter for delay routines */
  if (delay_cnt2)
    delay_cnt2--;               /* update counter for delay routines */
#endif

  my_idle();
/*   set_dmt_count(); */

#if WE_DEBUG(WE_SYSTEM_DEBUG_LVL, WE_DEBUG_FULL)
  /* Debug readings per second */
  loop_timing_check(&lt_10msec, "bg_10msec");
#endif
}
#endif


/** Called every 100msec / 10 times per second */
static void bg_poll_100msec_fn(void *data)
{
  if (++tenths > 9)
  {
    tenths = 0;
  }
  my_bf100();

#if RDEBUG_ENABLED(WE_SYSTEM_DEBUG_LVL, RDEBUG_LVL_FULL)
  /* Debug readings per second */
  loop_timing_check(&lt_100msec, "bg_100msec");
#endif
}


/** Called every 1000msec / 1 times per second */
static void bg_poll_1000msec_fn(void *data)
{
  my_bf1000();

#if RDEBUG_ENABLED(WE_SYSTEM_DEBUG_LVL, RDEBUG_LVL_FULL)
  /* Debug readings per second */
  loop_timing_check(&lt_1000msec, "bg_1000msec");
#endif
}


#if 0
static struct t_tick_fn bg_poll_10msec =
{
  OS_TICKS_PER_SEC / 100,
  0,
  bg_poll_10msec_fn,
  0,
  0
};
#endif


static struct t_tick_fn bg_poll_100msec =
{
  OS_TICKS_PER_SEC / 10,
  0,
  bg_poll_100msec_fn,
  0,
  0
};


static struct t_tick_fn bg_poll_1000msec =
{
  OS_TICKS_PER_SEC,
  0,
  bg_poll_1000msec_fn,
  0,
  0
};


/** Initialise the system poll functions */
void init_system(void)
{
#if RDEBUG_ENABLED(WE_SYSTEM_DEBUG_LVL, RDEBUG_LVL_FULL)
/*  loop_timing_init(&lt_10msec, 1); */
  loop_timing_init(&lt_100msec, 1);
  loop_timing_init(&lt_1000msec, 1);
#endif

/*  BackgroundAddFunction(&bg_poll_10msec); */
  BackgroundAddFunction(&bg_poll_100msec);
  BackgroundAddFunction(&bg_poll_1000msec);
}


/** Shutdown the system poll functions */
void destroy_system(void)
{
  BackgroundRemoveFunction(&bg_poll_1000msec);
  BackgroundRemoveFunction(&bg_poll_100msec);
/*  BackgroundRemoveFunction(&bg_poll_10msec); */
}
