/*
 * test_comms_send_buffer.c
 *
 *  Created on: Feb 28, 2017
 *      Author: ian
 */
#include <string.h>
#include <cutest/CuTest.h>
#include "comms_send_buffer.h"

#include "test_comms_send_buffer.h"


#define ALLOC_INCREMENTS 10


/** Check that we can create a comms send buffer */
static void test_comms_send_buffer_create(CuTest* tc)
{
  struct s_comms_send_buffer *csb;
  bool res;

  /* Create buffer */
  res = comms_send_buffer_create(&csb, ALLOC_INCREMENTS);
  CuAssertTrue(tc, res);
  CuAssertPtrNotNull(tc, csb);

  /* Check contents */
  CuAssertIntEquals(tc, ALLOC_INCREMENTS, csb->alloc_increments);
  CuAssertIntEquals(tc, 0, csb->allocated_size);
  CuAssertIntEquals(tc, 0, csb->used_size);
  CuAssertPtrEquals(tc, NULL, csb->buffer);

  /* Destroy it, can't really test this */
  comms_send_buffer_destroy(csb);
}


/** Check that we can add to a comms send buffer */
static void test_comms_send_buffer_add(CuTest* tc)
{
  bool res;
  struct s_comms_send_buffer *csb;
  char data;
  size_t i;

  /* Create buffer */
  res = comms_send_buffer_create(&csb, ALLOC_INCREMENTS);
  CuAssertTrue(tc, res);
  CuAssertPtrNotNull(tc, csb);

  /* Add a character to the buffer, to force realloc */
  for (i = 0; i < ALLOC_INCREMENTS; i++)
  {
    data = 'a' + i;
    res = comms_send_buffer_addbuf(csb, &data, sizeof(data));
    CuAssertTrue(tc, res);
    CuAssertIntEquals(tc, ALLOC_INCREMENTS, csb->allocated_size);
    CuAssertIntEquals(tc, i + 1, csb->used_size);
    CuAssertIntEquals(tc, data, csb->buffer[i]);
  }

  /* Next add will cause realloc */
  data = '0';
  res = comms_send_buffer_addbuf(csb, &data, sizeof(data));
  CuAssertTrue(tc, res);
  CuAssertIntEquals(tc, ALLOC_INCREMENTS * 2, csb->allocated_size);
  CuAssertIntEquals(tc, ALLOC_INCREMENTS + 1, csb->used_size);
  CuAssertIntEquals(tc, data, csb->buffer[ALLOC_INCREMENTS]);

  /* Test add char */
  data = '1';
  res = comms_send_buffer_addch(csb, data);
  CuAssertTrue(tc, res);
  CuAssertIntEquals(tc, ALLOC_INCREMENTS * 2, csb->allocated_size);
  CuAssertIntEquals(tc, ALLOC_INCREMENTS + 2, csb->used_size);
  CuAssertIntEquals(tc, data, csb->buffer[ALLOC_INCREMENTS + 1]);

  /* Destroy it */
  comms_send_buffer_destroy(csb);
}


/** Check that the comms send buffer reallocates correctly */
static void test_comms_send_buffer_realloc(CuTest* tc)
{
  bool res;
  struct s_comms_send_buffer *csb;
  char buf[ALLOC_INCREMENTS * 2];

  memset(buf, '\0', ALLOC_INCREMENTS * 2);

  /* Create buffer */
  res = comms_send_buffer_create(&csb, ALLOC_INCREMENTS);
  CuAssertTrue(tc, res);
  CuAssertPtrNotNull(tc, csb);

  /* Half fill first allocation */
  res = comms_send_buffer_addbuf(csb, buf, ALLOC_INCREMENTS / 2);
  CuAssertTrue(tc, res);
  CuAssertIntEquals(tc, ALLOC_INCREMENTS, csb->allocated_size);
  CuAssertIntEquals(tc, ALLOC_INCREMENTS / 2, csb->used_size);

  /* Next allocation */
  res = comms_send_buffer_addbuf(csb, buf, ALLOC_INCREMENTS + ALLOC_INCREMENTS / 2);
  CuAssertTrue(tc, res);
  CuAssertIntEquals(tc, ALLOC_INCREMENTS * 2, csb->allocated_size);
  CuAssertIntEquals(tc, ALLOC_INCREMENTS * 2, csb->used_size);
}


CuSuite *CuGetSuite_comms_send_buffer(void)
{
  CuSuite *suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_comms_send_buffer_create);
  SUITE_ADD_TEST(suite, test_comms_send_buffer_add);
  SUITE_ADD_TEST(suite, test_comms_send_buffer_realloc);

  return suite;
}
