/*
 * traceprint.h
 *
 *  Created on: Mar 16, 2012
 *      Author: ian
 */

#ifndef TRACEPRINT_H_
#define TRACEPRINT_H_

#include <semaphore.h>
#include <rinweigh/adcdatatypes.h>


/**
 * Initiate a (traceable) standard printout.
 *
 * For use with the comms, will wait for a traceable weight
 * and then print using it.
 */
void traceable_printout_initiate_standard_comms(sem_t *print_finished_sem);


/**
 * Initiate a (traceable) special remote printout.
 *
 * For use with the comms, will wait for a traceable weight
 * and then print using it.
 */
void traceable_printout_initiate_special_remote_comms(sem_t *print_finished_sem, const char *s);


/**
 * Finish up (traceable) standard printout, now that we have the traceable weight. For use with the UI.
 *
 * Must be called after a call to @ref traceable_printout_initiate_standard_ui(), regardless of the
 * success of the ADC library request traceable weight operation.
 */
void traceable_printout_finish_ui(const AdcTraceableWeight *atw, bool success);


/**
 * Initiate a (traceable) standard printout.
 *
 * Will display errors if requested. For use with the UI, will return immediately.
 *
 * Must be matched with a call to @ref traceable_printout_finish_ui() after the ADC
 * library has returned with success and traceable weight, or failure
 */
void traceable_printout_initiate_standard_ui(bool display_errors, bool immediate);


#endif /* TRACEPRINT_H_ */
