/*
 * unitswitch.h
 *
 *  Created on: Apr 10, 2018
 *      Author: ian
 */

#ifndef UNITSWITCH_H_
#define UNITSWITCH_H_

#include <stdbool.h>
#include <rinweigh/adcdatatypes.h>


/** Different weight types (displayed, gross, etc) */
enum e_weight_type { wt_displayed, wt_gross, wt_net, wt_tare, wt_num };


/** Get a weight from the adcSuperStructure in an AdcRdg */
bool unitsw_get_adcrdg(AdcRdg *rdgout, enum eAdcUnitSwitch usw, enum e_weight_type wt);


/** Get the currently configured units for primary/secondary/tertiary units */
enum eAdcUnits unitsw_get_units(enum eAdcUnitSwitch usw);


/** Get the currently configured decimal point position for primary/secondary/tertiary units */
uint8_t unitsw_get_decimal_point(enum eAdcUnitSwitch usw);


/** Get the resolution length for primary/secondary/tertiary units */
uint8_t unitsw_get_resolution_length(enum eAdcUnitSwitch usw);


/** Get the current preset tare value for primary/secondary/tertiary units */
void unitsw_get_preset_tare(AdcRdg *preset_tare, enum eAdcUnitSwitch usw);


/** Convert ADC display mode into ADC unit switch */
enum eAdcUnitSwitch unitsw_displaymode_to_unitswitch(enum eAdcDisplayMode dispmode);


/** Convert ADC unit switch into ADC display mode */
enum eAdcDisplayMode unitsw_unitswitch_to_displaymode(enum eAdcUnitSwitch usw);


/** Convert a final weight to alternate units */
long app_final_to_final_alt(long rdg);


/** Convert a final weight to tertiary units */
long app_final_to_final_tert(long rdg);


/** Convert final units of usw_from to usw_to units */
long unitsw_convert_final(long rdg, enum eAdcUnitSwitch usw_from, enum eAdcUnitSwitch usw_to, bool hires_from, bool hires_to);


/** Convert final units of usw_from to usw_to units in an AdcRdg */
void unitsw_convert_final_adcrdg(AdcRdg *rdgout, const AdcRdg *rdgin, enum eAdcUnitSwitch usw_from, enum eAdcUnitSwitch usw_to);


/** Initialise unit switching */
void init_unit_switch(void);


#endif /* UNITSWITCH_H_ */
