/*
 * weconfig.h
 *
 *  Created on: 17/04/2011
 *      Author: ian
 */

#ifndef WECONFIG_H_
#define WECONFIG_H_

#include <rinhw/stdrinstrum.h>


/** Location of the WE2111 configuration file */
#define WECFG_CONFIG_FILE "/etc/rinstrum/weconfig.xml"


/** Percent * 100 of analog output DAC fullscale that analog output calibration can be changed by (500 = 5.00%) */
#define WECFG_ANALOG_OUTPUT_CAL_LIMIT 500L


/** Percent * 100 of analog output DAC fullscale that analog output calibration is incremented/decremented by (5 = 0.05%) */
#define WECFG_ANALOG_OUTPUT_CAL_INCREMENT 5L


/** TOP SECRET Factory override code for FAC.RST menu */
#define WECFG_FACTORY_ACCESS_CODE 221927L


/** Percent of the DSD to purge when it gets full */
#define WECFG_DSD_PURGE_PERCENT (10)


/**
 * Upper limit of the calibration count. Unit will stop working beyond this.
 * Limited to largest 5 digit number (since it is displayed as C99999)
 */
#define WECFG_CAL_COUNT_MAXIMUM 99999


/** Live weigh averaging time */
#define WECFG_LIVE_WEIGH_AVERAGING_MILLISECONDS 10000


/** If true, will run the elongated startup which displays 000000-999999 and takes a long time */
#define WECFG_ELONGATED_STARTUP 0


/** Number of milliseconds to delay for a full delay when starting up */
#define WECFG_DELAY_FULL_STARTUP_MILLISECONDS 3000


/** Number of milliseconds to delay for a full delay once up and running */
#define WECFG_DELAY_FULL_OPERATING_MILLISECONDS 1500


/** Number of milliseconds to delay for each part of the display test (on, off, 0-9) */
#define WECFG_DELAY_DISPLAY_TEST_MILLISECONDS 250


/** Size of the print buffer for recording printouts for later retrieval by PRT */
#define WECFG_PRINT_BUFFER_SIZE 2048


/** Number of bytes that must be free in the transmit buffer to permit a auto output transmit */
#define WECFG_AUTO_BUF_REQUIRED_FREE 50


/** Number of bytes that must be free in the transmit buffer to permit a network command stream (MSV?,0) */
#define WECFG_NET_BUF_REQUIRED_FREE 50


/** Maximum number of Modbus TCP connections */
#define WECFG_RINMODBUS_MAX_TCP_CONNECTIONS 10


/** Maximum number of TCP network connections (2222 & 2223) */
#define WECFG_NET_MAX_TCP_CONNECTIONS 20


/**/
#define WECFG_CHGLOG_MAX_FILESIZE (512 * 1024)
#define WECFG_CHGLOG_EVENT_NAME "Changelog"
#define WECFG_CHGLOG_INITIAL_ENTRY "Changelog Created"
#define WECFG_CHGLOG_CLEARED_ENTRY "Changelog Cleared"


/** Number of times a licensing check should be performed before rebooting */
#define WECFG_LICENSING_CHECK_LIMIT 2


/** Number of ticks before the next licensing check should be performed (= 5 minutes) */
#define WECFG_LICENSING_CHECK_INTERVAL (5 * 60 * OS_TICKS_PER_SEC)


/** Number of attempts trying to license a package via LRP command before locking out and requiring a reboot */
#define WECFG_LICENSING_COMMS_TRIES 3


/** Temperature error upper limit in Kelvin +80 degrees Celcius */
#define WECFG_TEMPERATURE_MAX_KELVIN (273 + 80)


/** Temperature error lower limit in Kelvin -18 degrees Celcius */
#define WECFG_TEMPERATURE_MIN_KELVIN (273 - 18)


/** The maximum number invalid passcode attempts. Reboot is only way to reset this. */
#define WECFG_MAX_PASSCODE_INVALID_ATTEMPTS 5


/** Configuration structure */
struct s_hw_config
{
  char *we2111_hardware_device;
  char *rinblock_iic_device;
  char *adc_spi_device;
  char *serial1_device;
  char *serial2_device;
  char *nvram_device1;
  char *nvram_device2;
  char *factorydb1_file;
  char *factorydb2_file;
  char *userdb1_file;
  char *userdb2_file;
  char *licencedb1_file;
  char *licencedb2_file;
  char *dsd_file;
  char *changelog_file;
  char *partition_mount_location;
  char *watchdog_device;
  char *opkg_executable;
  char *ethernet_interface;
  char *device_mounter_notification_pipe;
  char *uds_network_file;
  char *uds_autoout_file;

  /* USB gadget stuff */
  char *usb_gadget_device_name;
  int usb_gadget_device_major;
  int usb_gadget_device_minor;
  int usb_gadget_vid;
  int usb_gadget_pid;
};


/** Struct of configuration read from the config file */
extern struct s_hw_config we_configuration;


/** Load the configuration from the configuration file */
bool load_configuration(const char *filename, struct s_hw_config *config);


/** Free up the contents of the configuration structure */
void free_configuration(struct s_hw_config *config);


#endif /* WECONFIG_H_ */
