#ifndef WE_DEBUG_DOT_H
#define WE_DEBUG_DOT_H 1

#include <syslog.h>
#include <stdarg.h>


#define RDEBUG_LVL_OFF 0
#define RDEBUG_LVL_BASIC 1
#define RDEBUG_LVL_FULL 2


#define RDEBUG_ENABLED(mod, lvl) (mod >= lvl)


#define RDEBUG_OFF(modlvl, sllvl, format, args...)
#define RDEBUG_BASIC(modlvl, sllvl, format, args...) if (modlvl <= RDEBUG_LVL_BASIC) rdebug_printf(sllvl, format, ##args)
#define RDEBUG_FULL(modlvl, sllvl, format, args...) if (modlvl <= RDEBUG_LVL_FULL) rdebug_printf(sllvl, format, ##args)


/** WE2111 ADC thread debug */
#define WE_ADC_THREAD_DEBUG_PRINTF RDEBUG_BASIC
#define WE_ADC_THREAD_DEBUG_LVL RDEBUG_LVL_BASIC

/**
 * Setting this debug will pulse the UART1 RS422 termination line
 * every time an ADC sample is received, and can be monitored with an oscilloscope
 *
 * WARNING: Never leave enabled in production code as it will stop the RS422 terminations working correctly
 */
#define WE_ADC_THREAD_DEBUG_PERFORMANCE_SYNCPULSE RDEBUG_LVL_OFF

/**
 * Setting this debug will drive the UART1 RS422 termination line
 * from the time that a ADC sample has been returned by the ADC library, until the IO
 * has been written
 * WARNING: Never leave enabled in production code as it will stop the RS422 terminations working correctly
 */
#define WE_ADC_THREAD_DEBUG_PERFORMANCE_IO RDEBUG_LVL_OFF

/**
 * Setting this debug will drive the UART1 RS422 termination line
 * for the time that the ADC thread is in AdcUpdate() function. This includes (some part of) the wait for data ready
 *
 * WARNING: Never leave enabled in production code as it will stop the RS422 terminations working correctly
 */
#define WE_ADC_THREAD_DEBUG_PERFORMANCE_ADCUPDATE RDEBUG_LVL_OFF

/** WE2111 comms task debug */
#define WE_COMMS_TASK_DEBUG_PRINTF RDEBUG_BASIC

/** WE2111 TCP port listen task debug */
#define WE_COMMS_LISTEN_TASK_DEBUG_PRINTF RDEBUG_BASIC

/** WE2111 UDP port task debug */
#define WE_COMMS_UDP_TASK_DEBUG_PRINTF RDEBUG_BASIC

/** WE2111 IP configuration debug */
#define WE_IPCONF_DEBUG_PRINTF RDEBUG_OFF
#define WE_IPCONF_DEBUG_LVL RDEBUG_LVL_OFF

/** WE2111 system debug */
#define WE_SYSTEM_DEBUG_PRINTF RDEBUG_OFF
#define WE_SYSTEM_DEBUG_LVL RDEBUG_LVL_OFF

/** WE2111 daemonise debug */
#define WE_DAEMONISE_DEBUG_PRINTF RDEBUG_FULL

/** WE2111 startup debug */
#define WE_STARTUP_DEBUG_PRINTF RDEBUG_FULL

/** WE2111 shutdown debug */
#define WE_SHUTDOWN_DEBUG_PRINTF RDEBUG_FULL

/** WE2111 device mounter thread debug */
#define WE_DEVICEMOUNTER_DEBUG_PRINTF RDEBUG_OFF

/* WE2111 partition device mounter debug */
#define WE_DEVICEMOUNTER_PARTITION_DEBUG_PRINTF RDEBUG_OFF

/* WE2111 printer device mounter debug */
#define WE_DEVICEMOUNTER_PRINTER_DEBUG_PRINTF RDEBUG_OFF

/* WE2111 event (keyboard) device mounter debug */
#define WE_DEVICEMOUNTER_EVENT_DEBUG_PRINTF RDEBUG_OFF

/* WE2111 serial port device mounter debug */
#define WE_DEVICEMOUNTER_SERIAL_DEBUG_PRINTF RDEBUG_FULL

/** WE2111 Analog Output Menu debug */
#define WE_IFACE_ANALOG_DEBUG_PRINTF RDEBUG_OFF

/** WE2111 User Interface Timing debug */
#define WE_IFACE_TIMING_DEBUG_PRINTF RDEBUG_OFF
#define WE_IFACE_TIMING_DEBUG_LVL RDEBUG_LVL_OFF

/** WE2111 Config debug */
#define WE_CONFIG_DEBUG_PRINTF RDEBUG_OFF

/** WE2111 signals debug */
#define WE_SIGNALS_DEBUG_PRINTF RDEBUG_FULL

/** WE2111 device finder debug */
#define WE_DEVICE_FINDER_DEBUG_PRINTF RDEBUG_OFF

/** WE2111 modbus task debug */
#define WE_MODBUS_TASK_DEBUG_PRINTF RDEBUG_OFF

/** WE2111 rinblock debug */
#define WE_RINBLOCK_DEBUG_PRINTF RDEBUG_OFF

/** Root password setting debug */
#define WE_ROOT_PASSWORD_DEBUG_PRINTF RDEBUG_OFF


/** Debug vprintf */
void rdebug_vprintf(int level, const char *format, va_list arglist);


/** Debug printf */
void rdebug_printf(int level, const char *format, ...);


#endif /* #ifndef WE_DEBUG_DOT_H */
