/*
 * _adc_ads1216_local.h
 *
 *  Created on: May 21, 2013
 *      Author: ian
 */

#ifndef _ADC_ADS1216_LOCAL_H_
#define _ADC_ADS1216_LOCAL_H_

#include <rinhw/stdrinstrum.h>


/* ADC Crystal Time/Frequency Dependent Constants */

/** Default operating frequency in 0.1Hz (ie 125 = 12.5Hz) */
#define DEFAULT_FREQ 500

/** ADC oscillator frequency */
#define ADS1216_FOSC 4915200
#define ADS1216_FMOD_SPEED128 (ADS1216_FOSC / 128)
#define ADS1216_FMOD_SPEED256 (ADS1216_FOSC / 256)

/** Time for one oscillation in nanoseconds */
#define ADS1216_TOSC ( 1000000000 / ADS1216_FOSC )

/** Convert a number of TOSCs into nanoseconds */
#define ADS1216_TOSCS_TO_NANOSEC(x) ( x * ADS1216_TOSC )

/** Convert a number of TOSCs into nanoseconds */
#define ADS1216_TOSCS_TO_MICROSEC(x) ( ( ADS1216_TOSCS_TO_NANOSEC( x ) ) / 1000 )

/** Serial clock maximum frequency */
/* #define ADS1216_SCLK_T1_FREQ_HZ ( ADS1216_FOSC / 4 ) */
#define ADS1216_SCLK_T1_FREQ_HZ ( 350000 )

/** Register read last SCK Din to first SCK Dout time t6 in microseconds (see datasheet) */
#define ADS1216_RREG_T6_MICROSEC ( ADS1216_TOSCS_TO_MICROSEC( 50 ) )

/** Read data last SCK Din to first SCK Dout time t6 in microseconds (see datasheet) */
#define ADS1216_RDATA_T6_MICROSEC ( ADS1216_TOSCS_TO_MICROSEC( 50 ) )

/** Reset sequence time t11 in microseconds (see datasheet) */
#define ADS1216_RESET_T11_MICROSEC ( ADS1216_TOSCS_TO_MICROSEC( 16 ) )

/** Read register time t11 in microseconds (see datasheet) */
#define ADS1216_RREG_T11_MICROSEC ( ADS1216_TOSCS_TO_MICROSEC( 4 ) )

/** Write register time t11 in microseconds (see datasheet) */
#define ADS1216_WREG_T11_MICROSEC ( ADS1216_TOSCS_TO_MICROSEC( 4 ) )

/** Read RAM time t11 in microseconds (see datasheet) */
#define ADS1216_RRAM_T11_MICROSEC ( ADS1216_TOSCS_TO_MICROSEC( 4 ) )

/** Write RAM time t11 in microseconds (see datasheet) */
#define ADS1216_WRAM_T11_MICROSEC ( ADS1216_TOSCS_TO_MICROSEC( 4 ) )

/** Reset sequence time t12 in microseconds (see datasheet) */
#define ADS1216_RESET_T12_MICROSEC ( ADS1216_TOSCS_TO_MICROSEC( 400 ) )

/** Reset sequence time t13 in microseconds (see datasheet) */
#define ADS1216_RESET_T13_MICROSEC ( ADS1216_TOSCS_TO_MICROSEC( 5 ) )

/** Reset sequence time t14 in microseconds (see datasheet) */
#define ADS1216_RESET_T14_MICROSEC ( ADS1216_TOSCS_TO_MICROSEC( 650 ) )

/** Reset sequence time t15 in microseconds (see datasheet) */
#define ADS1216_RESET_T15_MICROSEC ( ADS1216_TOSCS_TO_MICROSEC( 1150 ) )


#if 0

#define ADC_RESET_ON_DELAY1   ((int32u)(400 * ADC_T_OSC)) /* uSecs */
#define ADC_RESET_ON_DELAY2   ((int32u)(650 * ADC_T_OSC)) /* uSecs */
#define ADC_RESET_ON_DELAY3   ((int32u)(1150 * ADC_T_OSC))/* uSecs */
#define ADC_SPI_DELAY (int16u)(70 * ADC_T_OSC)
#endif
#if 0
#if SPEED == 0                              /* calculate modulator speed */
   #define ADC_F_MOD ADC_CRYSTAL_FREQ/128   /* based on the speed bit in adc */
#else
   #define ADC_F_MOD ADC_CRYSTAL_FREQ/256
#endif

#define ADC_SPI_RETRIES 4

//#define DEC0_DEFAULT     (int8u)(ADC_F_MOD/DEFAULT_SYNC)      /* calc 7 LSB's */
//#define DEC1_DEFAULT     (int8u)((ADC_F_MOD/DEFAULT_SYNC)/256)/* calc 3 MSB's */
//#define DEC0_ALT         (int8u)(ADC_F_MOD/ALT_SYNC)          /* calc 7 LSB's */
//#define DEC1_ALT         (int8u)((ADC_F_MOD/ALT_SYNC)/256)    /* calc 3 MSB's */

#endif


/* ADC SPI Commands */
#define ADS1216_CMD_RDATA              0x01    /* Reads the latest ADC value */
#define ADS1216_CMD_RDATAC             0x03    /* Read data continuously */
#define ADS1216_CMD_STOPC              0x0F    /* Stop read data continuously */
#define ADS1216_CMD_RREG               0x10    /* Read register in ADC */
#define ADS1216_CMD_RRAM               0x20    /* Read RAM bank */
#define ADS1216_CMD_CREG               0x40    /* Copy registers to RAM bank 'aaa' */
#define ADS1216_CMD_CREGA              0x48    /* Copy registers to all RAM banks */
#define ADS1216_CMD_WREG               0x50    /* Write register in ADC */
#define ADS1216_CMD_WRAM               0x60    /* Write RAM bank */
#define ADS1216_CMD_CRAM               0xC0    /* Copy RAM bank 'aaa' to register */
#define ADS1216_CMD_CSRAMX             0xD0    /* Calculate RAM bank 'aaa' checksum */
#define ADS1216_CMD_CSARAMX            0xD8    /* Calculate all RAM bank checksums */
#define ADS1216_CMD_CSREG              0xDF    /* Calculate register checksum */
#define ADS1216_CMD_CSRAM              0xE0    /* Calculate RAM bank 'aaa' checksum */
#define ADS1216_CMD_CSARAM             0xE8    /* Calculate all RAM bank checksums */
#define ADS1216_CMD_SELFCAL            0xF0    /* Self Calibrate Offset and Gain */
#define ADS1216_CMD_SELFOCAL           0xF1    /* Self calibrate offset */
#define ADS1216_CMD_SELFGCAL           0xF2    /* Self calibrate gain */
#define ADS1216_CMD_SYSOCAL            0xF3    /* System calibrate offset */
#define ADS1216_CMD_SYSGCAL            0xF4    /* System calibrate gain */
#define ADS1216_CMD_DYSNC              0xFC    /* Resets the conversion process */
#define ADS1216_CMD_SLEEP              0xFD    /* Put into sleep mode */
#define ADS1216_CMD_RESET              0xFE    /* Reset to power-up values */


/* ADC register Indexing */
#define ADS1216_REG_SETUP              0x00    /* Setup Register */
#define ADS1216_REG_MUX                0x01    /* Multiplex Control Register */
#define ADS1216_REG_ACR                0x02    /* Analog Control Register */
#define ADS1216_REG_IDAC1              0x03    /* Current DAC 1 */
#define ADS1216_REG_IDAC2              0x04    /* Current DAC 2 */
#define ADS1216_REG_ODAC               0x05    /* Offset DAC Setting */
#define ADS1216_REG_DIO                0x06    /* Digital Port Data Register */
#define ADS1216_REG_DIR                0x07    /* Direction Control Register */
#define ADS1216_REG_DEC0               0x08    /* Decimation Register (lower 8 bits) */
#define ADS1216_REG_MDEC1              0x09    /* Mode and Decimation Register */
#define ADS1216_REG_OCR0               0x0A    /* Offset Calibration Coefficient LSB */
#define ADS1216_REG_OCR1               0x0B    /* Offset Calibration Coefficient Middle Byte */
#define ADS1216_REG_OCR2               0x0C    /* Offset Calibration Coefficient MSB */
#define ADS1216_REG_FSR0               0x0D    /* Full-Scale Register LSB */
#define ADS1216_REG_FSR1               0x0E    /* Full-Scale Register Middle Byte */
#define ADS1216_REG_FSR2               0x0F    /* Full-Scale Register MSB */


/* ADS1216 Setup register constants */
#define REG_SETUP_BIT_ORDER_MSBFIRST   0x00    /* B0, 0=MSB first, 1=LSB first*/
#define REG_SETUP_BIT_ORDER_LSBFIRST   0x01    /* B0, 0=MSB first, 1=LSB first*/
#define REG_SETUP_BUF_EN_OFF           0x00    /* B1, 0=Buffer Off, 1=Buffer On */
#define REG_SETUP_BUF_EN_ON            0x02    /* B1, 0=Buffer Off, 1=Buffer On */
#define REG_SETUP_REF_HI_1V25          0x00    /* B2, 0=Int ref 1.25V 1=Int Reg 2.5V */
#define REG_SETUP_REF_HI_2V50          0x04    /* B2, 0=Int ref 1.25V 1=Int Reg 2.5V */
#define REG_SETUP_REF_EN_OFF           0x00    /* B3, 0=Int Ref off, 1=Int ref on */
#define REG_SETUP_REF_EN_ON            0x08    /* B3, 0=Int Ref off, 1=Int ref on */
#define REG_SETUP_SPEED_MASK           0x10    /* B4, Mask for speed bit */
#define REG_SETUP_SPEED_DIV128         0x00    /* B4, 0=Osc/128, 1=Osc/256 */
#define REG_SETUP_SPEED_DIV256         0x10    /* B4, 0=Osc/128, 1=Osc/256 */
#define REG_SETUP_ID_MASK              0xE0    /* B7-B5 Mask for the three factory programmed ID bits */
#define REG_SETUP_ID_VALUE1            0x80    /* B7-B5 Expected value for the three factory programmed ID bits */
#define REG_SETUP_ID_VALUE2            0x40    /* B7-B5 Expected value for the three factory programmed ID bits */


/* ADS1216 Multiplex register constants */
#define REG_MUX_NSEL_MASK              0x0F
#define REG_MUX_PSEL_MASK              0xF0
#define REG_MUX_CH_AIN0                0x00
#define REG_MUX_CH_AIN1                0x01
#define REG_MUX_CH_AIN2                0x02
#define REG_MUX_CH_AIN3                0x03
#define REG_MUX_CH_AIN4                0x04
#define REG_MUX_CH_AIN5                0x05
#define REG_MUX_CH_AIN6                0x06
#define REG_MUX_CH_AIN7                0x07
#define REG_MUX_CH_AINCOM              0x08
#define REG_MUX_CH_DIODE               0x0F
#define REG_MUX_NSEL_AIN(x)            ( x & REG_MUX_NSEL_MASK ) /* Set negative channel */
#define REG_MUX_PSEL_AIN(x)            ( ( x << 4 ) & REG_MUX_PSEL_MASK ) /* Set positive channel */


/* Analog control register constants */
#define REG_ACR_PGA_MASK               0x07
#define REG_ACR_PGA1                   0x00
#define REG_ACR_PGA2                   0x01
#define REG_ACR_PGA4                   0x02
#define REG_ACR_PGA8                   0x03
#define REG_ACR_PGA16                  0x04
#define REG_ACR_PGA32                  0x05
#define REG_ACR_PGA64                  0x06
#define REG_ACR_PGA128                 0x07
#define REG_ACR_PGA(x)                 ( x & REG_ACR_PGA_MASK ) /* Set PGA */
#define REG_ACR_IDAC1_RANGE_MASK       0x18
#define REG_ACR_IDAC2_RANGE_MASK       0x60
#define REG_ACR_IDAC_OFF               0x00
#define REG_ACR_IDAC_RANGE1            0x01
#define REG_ACR_IDAC_RANGE2            0x02
#define REG_ACR_IDAC_RANGE3            0x03
#define REG_ACR_IDAC1_RANGE(x)         ( ( x << 3 ) & REG_ACR_IDAC1_RANGE_MASK ) /* Set IDAC1 Range */
#define REG_ACR_IDAC2_RANGE(x)         ( ( x << 5 ) & REG_ACR_IDAC2_RANGE_MASK ) /* Set IDAC2 Range */
#define REG_ACR_BOCS_MASK              0x80
#define REG_ACR_BOCS_OFF               0x00   /* Burn Out Current Source Off */
#define REG_ACR_BOCS_ON                0x80   /* Burn Out Current Source On */


/* ADS1216 Mode and decimation register constants */
#define REG_MDEC_DEC_MASK              0x07
#define REG_MDEC_DEC(x)                ( x & REG_MDEC_DEC_MASK )
#define REG_MDEC_RESERVED_MASK         0x08
#define REG_MDEC_SMODE_MASK            0x30
#define REG_MDEC_SMODE_AUTO            0x00
#define REG_MDEC_SMODE_FAST            0x01
#define REG_MDEC_SMODE_SINC2           0x02
#define REG_MDEC_SMODE_SINC3           0x03
#define REG_MDEC_SMODE(x)              ( ( x << 4 ) & REG_MDEC_SMODE_MASK ) /* Set SMODE Range */
#define REG_MDEC_UB_BIPOLAR            0x00
#define REG_MDEC_UB_UNIPOLAR           0x40
#define REG_MDEC_DRDY_MASK             0x80    /* Data Ready Flag - low when data available */


/* Setup register */
#define REG_SETUP_SPEED_DEFAULT REG_SETUP_SPEED_DIV128
#define SETUP_VALUE ( REG_SETUP_BIT_ORDER_MSBFIRST | REG_SETUP_BUF_EN_OFF | \
                      REG_SETUP_REF_HI_1V25 | REG_SETUP_REF_EN_OFF | \
                      REG_SETUP_SPEED_DIV128 )

/* Multiplex Register: Select +ve channel as 4 and -ve channel as 5 */
#define MUX_VALUE ( REG_MUX_NSEL_AIN( REG_MUX_CH_AIN5 ) | REG_MUX_PSEL_AIN( REG_MUX_CH_AIN4 ) )


/* Analog Control Register: */
#define ACR_VALUE_WITH_PGA(x) ( REG_ACR_PGA( x ) | \
                                REG_ACR_IDAC1_RANGE( REG_ACR_IDAC_OFF ) | \
                                REG_ACR_IDAC2_RANGE( REG_ACR_IDAC_OFF ) | \
                                REG_ACR_BOCS_OFF )
#define ACR_VALUE_DEFAULT ( ACR_VALUE_WITH_PGA( REG_ACR_PGA64 ) )


/* Mode and decimation register */
#define MDEC_VALUE_WITH_DEC(x) ( REG_MDEC_DEC(x) | \
                                 REG_MDEC_SMODE( REG_MDEC_SMODE_AUTO ) | \
                                 REG_MDEC_UB_BIPOLAR )


/* The default value of offset calibration register value to use */
#define OCR_VALUE_DEFAULT 0


/* The default value of full scale register value to use */
#define FSR_VALUE_DEFAULT 0x008111BB


/** Check the ADC is responding by reading the SETUP register and checking the ID bits */
bool adc24bit_check_adc_present(int fd);


int adc24bit_read_register8(int fd, int8u ui8Reg, int8u *pui8Data);


int adc24bit_write_register8(int fd, int8u ui8Reg, int8u ui8Data);


#endif /* _ADC_ADS1216_LOCAL_H_ */
