/*
 * adc_ads1216_r400.c
 *
 * R400 specific code for the ADS1216
 *
 *  Created on: May 21, 2013
 *      Author: ian
 */
#include <rinhw/stdrinstrum.h>
#include <rinhw/adc_ads1216.h>
#include "_adc_ads1216_local.h"
#include <rinhw/adc_ads1216_local.h>

#include <rinhw/adc_ads1216_r400.h>


/* Mask for the IO pin used to control 4/6 wire mode */
#define LCWIRE_PIN_MASK 0x01
#define LCWIRE_PIN_6WIRE 0x00
#define LCWIRE_PIN_4WIRE 0x01


/* The default value for the DIO direction register: 4/6 wire pin as output (0=output, 1=input) */
#define DIR_VALUE_DEFAULT ( ~LCWIRE_PIN_MASK )


/** User hardware init for ADC on R400 */
int adc24bit_user_hw_init_r400(struct sAdcSpiDevice *adcdev)
{
  adc24bit_write_register8(adcdev->fd, ADS1216_REG_DIR, (int8u) DIR_VALUE_DEFAULT);
  adc24bit_set_loadcell_wire_mode_r400(adcdev, IOCTL_ADC24BIT_LCWIRE_4WIRE);
  return RIO_SUCCESS;
}


/* Use the special sequence on the SPI to reset the ADC */
static void adc24bit_reset_via_spi(void)
{
#if 0 /* TODO implement this for R400 somehow??? */
  int8u ui8Ctr;
  OS_DECLARE_CPU_SR();

  /* Make sure clock low for at least t13 5 * TOSC before starting */
  adc_spi_set_clock_output( false );
  DELAY_MICROSECOND( ADS1216_RESET_T13_MICROSEC );

  /* Min and max timing is critical here, so no interrupts */
  OS_ENTER_CRITICAL();

  /* Clock high for t12 300-500 * TOSC */
  adc_spi_set_clock_output( true );
  DELAY_MICROSECOND( ADS1216_RESET_T12_MICROSEC );

  /* Clock low for t13 5 * TOSC */
  adc_spi_set_clock_output( false );
  DELAY_MICROSECOND( ADS1216_RESET_T13_MICROSEC );

  /* Clock high for t14 550-750 * TOSC */
  adc_spi_set_clock_output( true );
  DELAY_MICROSECOND( ADS1216_RESET_T14_MICROSEC );

  /* Clock low for t13 5 * TOSC */
  adc_spi_set_clock_output( false );
  DELAY_MICROSECOND( ADS1216_RESET_T13_MICROSEC );

  /* Clock high for t15 1050-1250 * TOSC */
  adc_spi_set_clock_output( true );
  DELAY_MICROSECOND( ADS1216_RESET_T15_MICROSEC );

  /* Clock low for t11 (RESET) of 16 * TOSC before next command */
  adc_spi_set_clock_output( false );
  OS_EXIT_CRITICAL();
  DELAY_MICROSECOND( ADS1216_RESET_T11_MICROSEC );
#endif
}


/** Reset the ADC using the SPI reset sequence, and check that it's responding */
int adc24bit_sync_spi_r400(struct sAdcSpiDevice *adcdev)
{

  /*
   * On the R400 this requires the reset sequence because there is no chip select line
   */
  do
  {
    /* Reset the ADC */
    adc24bit_reset_via_spi();

    /* Do a time delay, it seems to need this to get it's shit together */
    OSTimeDly(OS_TICKS_PER_SEC / 2);
    adc24bit_check_adc_present(adcdev->fd);
  }
  while (!adc24bit_check_adc_present(adcdev->fd));
  return RIO_SUCCESS;
}


/** Set the loadcell 4/6 wire mode on a R400 */
int adc24bit_set_loadcell_wire_mode_r400(struct sAdcSpiDevice *adcdev, int level)
{
  int8u data;

  data = (IOCTL_ADC24BIT_LCWIRE_4WIRE == level) ? LCWIRE_PIN_4WIRE : LCWIRE_PIN_6WIRE;
  return adc24bit_write_register8(adcdev->fd, ADS1216_REG_DIO, data);
}


/**
 * Wait for a reading to become available.
 *
 * This implementation has no access to the /DRDY signal
 * and continually polls the ADS1216 until the DRDY bit
 * gets cleared
 */
int adc24bit_wait_for_reading_r400(struct sAdcSpiDevice *adcdev, unsigned short timeout)
{
  const int32u ui32StartTime = OSTimeGet();
  int8u ui8Data;

  while (OSTimeGet() < ui32StartTime + timeout)
  {
    adc24bit_read_register8(adcdev->fd, ADS1216_REG_MDEC1, &ui8Data);
    if (0 == (ui8Data & REG_MDEC_DRDY_MASK))
    {
      return RIO_SUCCESS;
    }
    /* Yield to let other stuff run */
    OSTimeDly(1);
  }
  return RIO_ERROR;
}
