/*
 * buzzer_local.c
 *
 *  Created on: 10/02/2009
 *      Author: ianb
 */
#if defined( __linux__ )
  #include <sys/ioctl.h>
  #include <unistd.h>
  #include <linux/kd.h>
  #include <fcntl.h>
  #include <stdio.h>
  #include <errno.h>
  #include <time.h>
#endif /* #if defined( __linux__ ) */
#include <rinhw/timemath.h>
#include <rinhw/buzzer_generic.h>
#include <rinhw/debug.h>

#include <rinhw/buzzer_local.h>


#if defined( WIN32 )


static int buz_local_run_seq(struct s_generic_buzzer *gb)
{
  for (gb->current_buz_seq = 0; gb->current_buz_seq < gb->num_buz_seq && !gb->buz_task_quit; gb->current_buz_seq++)
  {
    Beep(gb->buz_seq[gb->current_buz_seq].freq, gb->buz_seq[gb->current_buz_seq].time * 1000 / OS_TICKS_PER_SEC);
  }
}


#elif defined( __linux__ )


static int buz_local_run_seq(struct s_generic_buzzer *gb)
{
  struct timespec ts_req, ts_rem;
  int console_fd;
#if HW_DEBUG(BUZ_DEBUG, HW_DEBUG_FULL)
  int result;
#endif

  /* Try to open the console, need to root, or permissions on /dev/console changed */
  console_fd = open("/dev/console", O_WRONLY);
#if HW_DEBUG(BUZ_DEBUG, HW_DEBUG_BASIC)
  if (console_fd < 0)
  {
    DEBUG_PRINTF("run_buz_seq: console open failed: result %d, errno %d\n", console_fd, errno);
  }
#endif
  if (console_fd >= 0)
  {
    if (fcntl(console_fd, F_SETFD, FD_CLOEXEC) < 0)
    {
#if HW_DEBUG(BUZ_DEBUG, HW_DEBUG_BASIC)
      DEBUG_PRINTF("run_buz_seq: error setting FD_CLOEXEC on console, errno %d\n", errno);
#endif
    }
  }

  for (gb->current_buz_seq = 0; gb->current_buz_seq < gb->num_buz_seq && !gb->buz_task_quit; gb->current_buz_seq++)
  {
#if HW_DEBUG(BUZ_DEBUG, HW_DEBUG_FULL)
    if (console_fd < 0)
    {
      DEBUG_PRINTF("run_buz_seq: buz %d of %d\n", gb->current_buz_seq, gb->num_buz_seq);
    }
#endif

    if (console_fd < 0)
    {
      /* Couldn't open console device, so just make a beep via stdout */
      printf("\a");
    }
    else
    {
      /* Make the beep via /dev/console via the IOCTL function */
      if (0 == gb->buz_seq[gb->current_buz_seq].freq)
      {
#if !HW_DEBUG(BUZ_DEBUG, HW_DEBUG_FULL)
        (void) ioctl(console_fd, KIOCSOUND, 0);
#else
        result = ioctl(console_fd, KIOCSOUND, 0);
        if (result < 0)
        {
          DEBUG_PRINTF("run_buz_seq: ioctl KIOCSOUND failed, result %d, errno %d\n", result, errno);
        }
#endif
      }
      else
      {
        /* ioctl( console_fd, KIOCSOUND, CLOCK_TICK_RATE / buz_seq[ current_buz_seq ].freq ); TODO BUZZER use CLOCK_TICK_RATE */
#if !HW_DEBUG(BUZ_DEBUG, HW_DEBUG_FULL)
        (void) ioctl(console_fd, KIOCSOUND, 1193180 / gb->buz_seq[gb->current_buz_seq].freq);
#else
        result = ioctl(console_fd, KIOCSOUND, 1193180 / gb->buz_seq[gb->current_buz_seq].freq);
        if (result < 0)
        {
          DEBUG_PRINTF("run_buz_seq: ioctl KIOCSOUND failed, result %d, errno %d\n", result, errno);
        }
#endif
      }
    }

    /* Delay for the desired time */
    OSTicksToTimespec(&ts_req, gb->buz_seq[gb->current_buz_seq].time);
    nanosleep(&ts_req, &ts_rem);
  }

  /* make sure current_buz_seq is larger than num_buz_seq for the buz_read func */
  gb->current_buz_seq = (unsigned char) (gb->num_buz_seq+1);

  if (console_fd >= 0)
  {
    /* Shut things up and close the file */
    ioctl(console_fd, KIOCSOUND, 0);
    close(console_fd);
  }
  return 0;
}


#else
  #error "No buzzer defined for this platform"
#endif


/** Function to create and initialise a local buzzer device */
int rio_create_device_buzzer_local(struct t_io_dev *io_dev)
{
  int res;

  /* Create generic buzzer device */
  res = rio_create_device_buzzer_generic(io_dev,
                                         NULL,
                                         NULL,
                                         NULL,
                                         buz_local_run_seq);
  return res;
}
