#include <stdio.h>

#include <rinhw/byteq.h>


/**
 * Create a new byte queue
 *
 * Parameters:
 *   q is a pointer to an existing un-initialised byte queue struct
 *   pend_dirn is the direction in which the queue should pend (either pend for room to write (bqPendOnWrite), or pend for
 *     data to read (bqPendOnRead))
 *   buf is a pointer to a existing unused buffer to put the queue data into
 *   size is the size in bytes of the buffer and hence the queue
 *
 * Returns:
 *   bqrSuccess on success, bqrError otherwise
 */
enum eBQResult ByteQCreate(struct s_byteq *q, enum eBQPendDirn pend_dirn, uint8_t *buf, size_t size)
{
  size_t sem_value = (pend_dirn == bqPendOnRead) ? 0 : size;

  /* Fill out parameters */
  q->PendDirn = pend_dirn;
  q->Size = size;
  q->Level = 0;

  /*
   * Initialise pointers to the start of the buffer
   * and zero out the buffer byte count
   */
  q->pBuffer = buf;
  q->pRead = q->pBuffer;
  q->pWrite = q->pBuffer;
  q->Count = 0;

  /*
   * Create the semaphore to keep track of amount of data or space in the queue
   * (depends on direction)
   */
  if (sem_init(&q->Sem, 0, sem_value) != 0)
  {
    return bqrError;
  }

  /* Create the exclusive access mutex */
  if (pthread_mutex_init(&q->lock_mutex, NULL) != 0)
  {
    return bqrError;
  }
  return bqrSuccess;
}
