#include <pthread.h>

#include <rinhw/byteq.h>


/**
 * Get the amount of space left in the queue
 *
 * Parameters:
 *   q is a pointer to an existing initialised byte queue struct
 *
 * Returns:
 *   The amount of space (in bytes) left in the queue
 */
size_t ByteQGetSpace(struct s_byteq *q)
{
  size_t space;

  /* Lock this section */
  if (pthread_mutex_lock(&q->lock_mutex) != 0)
  {
    /* Not sure what else to return */
    return 0;
  }

  /* Calculate the space left and store into a temporary variable */
  space = q->Size - q->Level;

  /* Unlock and return the space left */
  pthread_mutex_unlock(&q->lock_mutex);
  return space;
}
