#ifndef DEVICE_DOT_H
#define DEVICE_DOT_H 1


#define NUM_FLASH_BLOCKS 12

#define FLASH_BLOCK0_START 0x000000
#define FLASH_BLOCK0_END 0x001000
#define FLASH_BLOCK0_EBR 0x0001

#define FLASH_BLOCK1_START 0x001000
#define FLASH_BLOCK1_END 0x002000
#define FLASH_BLOCK1_EBR 0x0002

#define FLASH_BLOCK2_START 0x002000
#define FLASH_BLOCK2_END 0x003000
#define FLASH_BLOCK2_EBR 0x0004

#define FLASH_BLOCK3_START 0x003000
#define FLASH_BLOCK3_END 0x004000
#define FLASH_BLOCK3_EBR 0x0008

#define FLASH_BLOCK4_START 0x004000
#define FLASH_BLOCK4_END 0x005000
#define FLASH_BLOCK4_EBR 0x0010

#define FLASH_BLOCK5_START 0x005000
#define FLASH_BLOCK5_END 0x006000
#define FLASH_BLOCK5_EBR 0x0020

#define FLASH_BLOCK6_START 0x006000
#define FLASH_BLOCK6_END 0x007000
#define FLASH_BLOCK6_EBR 0x0040

#define FLASH_BLOCK7_START 0x007000
#define FLASH_BLOCK7_END 0x008000
#define FLASH_BLOCK7_EBR 0x0080

#define FLASH_BLOCK8_START 0x008000
#define FLASH_BLOCK8_END 0x010000
#define FLASH_BLOCK8_EBR 0x0100

#define FLASH_BLOCK9_START 0x010000
#define FLASH_BLOCK9_END 0x020000
#define FLASH_BLOCK9_EBR 0x0200

#define FLASH_BLOCK10_START 0x020000
#define FLASH_BLOCK10_END 0x030000
#define FLASH_BLOCK10_EBR 0x0400

#define FLASH_BLOCK11_START 0x030000
#define FLASH_BLOCK11_END 0x040000
#define FLASH_BLOCK11_EBR 0x0800

/*
 * List of all flash block starts and EBRs to be used in an array
 */
#define FLASH_BLOCK_STARTS \
    FLASH_BLOCK0_START, FLASH_BLOCK1_START, FLASH_BLOCK2_START, \
    FLASH_BLOCK3_START, FLASH_BLOCK4_START, FLASH_BLOCK5_START, \
    FLASH_BLOCK6_START, FLASH_BLOCK7_START, FLASH_BLOCK8_START, \
    FLASH_BLOCK9_START, FLASH_BLOCK10_START, FLASH_BLOCK11_START, \
    FLASH_BLOCK11_END

#define FLASH_BLOCK_EBRS \
    FLASH_BLOCK0_EBR, FLASH_BLOCK1_EBR, FLASH_BLOCK2_EBR, \
    FLASH_BLOCK3_EBR, FLASH_BLOCK4_EBR, FLASH_BLOCK5_EBR, \
    FLASH_BLOCK6_EBR, FLASH_BLOCK7_EBR, FLASH_BLOCK8_EBR, \
    FLASH_BLOCK9_EBR, FLASH_BLOCK10_EBR, FLASH_BLOCK11_EBR \

/*
 * Size defines
 */
#define FLASH_LINE_SIZE         128
#define EBR_COUNT               2
#define ADDRESS_HOLDER unsigned long
#define EBR_HOLDER unsigned short

/*
 * Programming & Erasing FLASH register defines
 */
#define SET_FLMCR_PSU(val) (P_FLASH.FLMCR1.BIT.PSU1 = val)     /* FLASH Control Register controlling the PSU bit */
#define SET_FLMCR_ESU(val) (P_FLASH.FLMCR1.BIT.ESU1 = val)     /* FLASH Control Register controlling the ESU bit */
#define SET_FLMCR_P(val) (P_FLASH.FLMCR1.BIT.P1 = val)         /* FLASH Control Register controlling the P bit */
#define SET_FLMCR_E(val) (P_FLASH.FLMCR1.BIT.E1 = val)         /* FLASH Control Register controlling the E bit */
#define SET_FLMCR_SWE(val) (P_FLASH.FLMCR1.BIT.SWE1 = val)     /* FLASH Control Register controlling the SWE bit */
#define SET_FLMCR_PV(val) (P_FLASH.FLMCR1.BIT.PV1 = val)       /* FLASH Control Register controlling the PV bit */
#define SET_FLMCR_EV(val) (P_FLASH.FLMCR1.BIT.EV1 = val)       /* FLASH Control Register controlling the EV bit */

#define SET_FLMCR_BYTE(val) (P_FLASH.FLMCR1.BYTE = val)
#define SET_FLASH_ENABLE(val) (P_SCRX.BIT.FLSHE = val)         /* SCRX Flash Enable bit */

/*
 * FLMCR_FWE is monitored until it goes high on the 2238
 */
#define FLMCR_FWE (P_FLASH.FLMCR1.BIT.FWE)
#define INIT_FWE_DRIVE_PIN() (P_PF.DDR = 0x04)
#define SET_FWE_DRIVE_PIN(val) (P_PF.DR.BIT.B2 = val)

/*
 * Program & Erase verification defines
 */
#define VERIFY_SIZE 2
#define VERIFY_CAST unsigned short
#define VERIFY_VALUE 0xFFFF
#define VERIFY_DUMMY 0xFF

#endif
