/*
 * framdev_local.c
 *
 *  Created on: 03/02/2009
 *      Author: ianb
 */
#include <fcntl.h>
#include <stddef.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <rinhw/rio.h>
#include <rinhw/debug.h>
#include <rinhw/memorydev_local_file.h>

#include <rinhw/framdev_local.h>


/** FRAM driver init entry point, setup the hardware and device */
static int fram_local_init( struct t_io_dev *io_dev )
{
  /* struct sMemoryDeviceFileInfo *dev_info = (struct sMemoryDeviceFileInfo *) io_dev->dev_data; */
  int result;

#if HW_DEBUG(FRAM_DEBUG, HW_DEBUG_BASIC)
  DEBUG_PRINTF("fram_local_init called\n");
#endif

  /* Call the memory device initialiser, dev_info already filled out by rio_create_device_fram_local() */
  result = memorydev_local_file_init( io_dev );

#if HW_DEBUG(FRAM_DEBUG, HW_DEBUG_BASIC)
  DEBUG_PRINTF("fram_local_init finished\n");
#endif
  return result;
}


/** Destroy fram device */
static int fram_local_destroy(struct t_io_dev *io_dev)
{
  struct sMemoryDeviceFileInfo *dev_info = (struct sMemoryDeviceFileInfo *) io_dev->dev_data;

#if HW_DEBUG(FRAM_DEBUG, HW_DEBUG_BASIC)
  DEBUG_PRINTF("fram_local_destroy called\n");
#endif

  /* Close file */
  memorydev_local_file_close(io_dev);

  /* Free memory */
  io_dev->dev_data = NULL;
  if (NULL != dev_info)
  {
    if (NULL != dev_info->filename)
    {
      free(dev_info->filename);
      dev_info->filename = NULL;
    }
    free(dev_info);
    dev_info = NULL;
  }

#if HW_DEBUG(FRAM_DEBUG, HW_DEBUG_BASIC)
  DEBUG_PRINTF("fram_local_destroy finished\n");
#endif
  return RIO_SUCCESS;
}


/** Function to create and initialise a local fram device */
int rio_create_device_fram_local(struct t_io_dev *io_dev, const char *filename, size_t length, uint8_t blankfillchar)
{
  struct sMemoryDeviceFileInfo *dev_fram_info = NULL;

  /* Allocate memory device information struct */
  dev_fram_info = malloc(sizeof(struct sMemoryDeviceFileInfo));
  if (NULL == dev_fram_info)
  {
    goto rcdfl_fail;
  }

  /* Fill out struct */
  dev_fram_info->fd = -1;
  dev_fram_info->maxsize = length;
  dev_fram_info->blankfillchar = blankfillchar;

  dev_fram_info->filename = strdup(filename);
  if (NULL == dev_fram_info->filename)
  {
    goto rcdfl_fail;
  }

  /*
   * Setup device data pointer to point to the memory device struct
   * and initialise the status and position
   */
  io_dev->dev_data = (void *) dev_fram_info;
  io_dev->status = io_dev->posn = 0UL;
  io_dev->sem_open_excl = NULL;

  /* Setup function pointers */
  io_dev->dev_init = fram_local_init;
  io_dev->dev_open = memorydev_local_file_open;
  io_dev->dev_close = memorydev_local_file_close;
  io_dev->dev_read = memorydev_local_file_read;
  io_dev->dev_write = memorydev_local_file_write;
  io_dev->dev_filelength = memorydev_local_file_filelength;
  io_dev->dev_ioctl = default_ioctl;
  io_dev->dev_destroy = fram_local_destroy;
  io_dev->dev_isatty = default_isatty;
  io_dev->dev_tell = default_tell;
  io_dev->dev_lseek = default_lseek;

  /* Register and initialise the device */
  if (rregister_io(io_dev) < 0)
  {
    goto rcdfl_fail;
  }

  /* Success */
  return RIO_SUCCESS;

rcdfl_fail:
  io_dev->dev_data = NULL;
  if (NULL != dev_fram_info)
  {
    if (dev_fram_info->fd >= 0)
    {
      close(dev_fram_info->fd);
      dev_fram_info->fd = -1;
    }
    if (NULL != dev_fram_info->filename)
    {
      free(dev_fram_info->filename);
      dev_fram_info->filename = NULL;
    }
    free(dev_fram_info);
    dev_fram_info = NULL;
  }
  return RIO_ERROR;
}
