#include <stdio.h>
#include <rinhw/ucos_ii.h>
#include "winsocket.h"
#include <rinhw/we21hw.h>
#include <rinhw/tick.h>
#include <rinhw/wdt.h>
#include "_bkgrnd.h"
#include "_rio.h"

#include <rinhw/hwlib_init.h>


/**
 * Bring up the basic functionality of the hardware library:
 *    - uC/OS
 *    - Watchdog
 *    - Background function
 *    - RIO
 *
 * No devices are created here, and it is left up to the application
 * to create/initialise these using rregister_io() before they are used
 */
void hwlib_init(void)
{
  /* Bring up uC/OS operating system (or what remains of it) */
  OSInit();

#ifdef __WIN32__
  /* Initialise windows sockets only on windows platform */
  WindowsSocketsInit(NULL);
#endif /* #ifdef __WIN32__ */

  /* Watchdog */
  WatchdogInit(NULL);

  /* Background task */
  BackgroundInit(NULL);

  /* IO subsystem */
  init_io(NULL);
}


/** Shut down the hardware library */
void hwlib_destroy(void)
{
  /* Shut down IO subsystem */
  destroy_io();

  /* Shut down the background task */
  BackgroundDestroy();

  /* Shut down the watchdog */
  WatchdogDestroy();

#ifdef __WIN32__
  /* Shutdown windows sockets only on windows platform */
  WindowsSocketsDestroy();
#endif /* #ifdef __WIN32__ */

  /* Shut down OS */
  extern void OSDestroy(void);
  OSDestroy();
}
