/*
 * kbd_buzzer.c
 *
 * Handle key press buzzing
 *
 *  Created on: Dec 8, 2011
 *      Author: ian
 */
#include <stdio.h>
#include <rinhw/buzzer.h>

#include "kbd_buzzer.h"


/*
 * Key beep timing constants
 */
#define KEY_BEEP_FREQ (BUZZER_RESONANT_FREQ)
#define KEY_PRESS_BEEP_TIME (OS_TICKS_PER_SEC / 20)
#define KEY_HOLD_BEEP_TIME (OS_TICKS_PER_SEC / 20)
#define KEY_DISABLED_BEEP_TIME (OS_TICKS_PER_SEC / 2)


/*
 * Beep struct for short press (single beep)
 */
static const tBuzzer buz_press[] =
{
  { KEY_PRESS_BEEP_TIME, KEY_BEEP_FREQ }
};


/*
 * Beep struct for held key (double beep)
 */
static const tBuzzer buz_hold[] =
{
  { KEY_HOLD_BEEP_TIME, KEY_BEEP_FREQ },
  { KEY_HOLD_BEEP_TIME, 0 },
  { KEY_HOLD_BEEP_TIME, KEY_BEEP_FREQ }
};


/*
 * Beep struct for disabled key (long beep)
 */
static const tBuzzer buz_disabled[] =
{
  { KEY_DISABLED_BEEP_TIME, KEY_BEEP_FREQ }
};


/** Generate the appropriate buzz based on the key event */
void kbd_buzzer(struct t_io_dev *dev_buzzer, int buz_enabled, enum tKeyEvent event)
{
  const tBuzzer *buz;
  unsigned short buz_len;

  if (NULL != dev_buzzer && buz_enabled)
  {
    switch (event)
    {
      case keDown:
        buz = buz_press;
        buz_len = sizeof(buz_press);
        break;
      case keHeld:
        buz = buz_hold;
        buz_len = sizeof(buz_hold);
        break;
      case keDisabled:
        buz = buz_disabled;
        buz_len = sizeof(buz_disabled);
        break;
      default:
        return;
    }

    /*
     * Check that the buzzer has been initialised, and if so
     * open the buzzer, write the buzz data, with a quick timeout so
     * things don't get held up if something else is going on with the
     * buzzer.
     * Then close the buzzer.
     */
    if (0 != (dev_buzzer->status & O_INITIALISED))
    {
      if (RIO_SUCCESS == ropen(dev_buzzer, 0))
      {
        rwrite(dev_buzzer, buz, buz_len, 1);
        rclose(dev_buzzer);
      }
    }
  }
}
