#include <rinhw/kbd.h>
#include <rinhw/gpio.h>

#include "_kbd_hw.h"


/*
 * key_lookup -
 *
 * Translation table for internal keys
 */
static const unsigned char key_lookup[NUM_INTERNAL_KEYS] =
{
    KEY_PHYSICAL_NUM1, KEY_PHYSICAL_NUM2, KEY_PHYSICAL_NUM3, KEY_PHYSICAL_NUM4,
    KEY_PHYSICAL_NUM5, KEY_PHYSICAL_NUM6, KEY_PHYSICAL_NUM7, KEY_PHYSICAL_NUM8,
    KEY_PHYSICAL_NUM9, KEY_PHYSICAL_SIGN, KEY_PHYSICAL_NUM0, KEY_PHYSICAL_DP,
    KEY_PHYSICAL_CLEAR, KEY_PHYSICAL_UP, KEY_PHYSICAL_DOWN, KEY_PHYSICAL_ENTER,
    KEY_PHYSICAL_FUNC0, KEY_PHYSICAL_FUNC1, KEY_PHYSICAL_FUNC2, KEY_PHYSICAL_FUNC3,
    KEY_PHYSICAL_FUNC4, KEY_PHYSICAL_FUNC5, KEY_PHYSICAL_POWER, KEY_PHYSICAL_SETUP,
    KEY_PHYSICAL_POWER_AND_FUNC0, KEY_PHYSICAL_POWER_AND_FUNC1, KEY_PHYSICAL_POWER_AND_FUNC2, KEY_PHYSICAL_POWER_AND_FUNC3,
    KEY_PHYSICAL_POWER_AND_FUNC4, KEY_PHYSICAL_POWER_AND_FUNC5
};


/*
 * KeyToIndex -
 *
 * Convert a key code into a bit index
 * as per a reverse lookup of the key_lookup table, for internal keys
 * External keys have no translation
 */
unsigned char KeyToIndex(unsigned char key)
{
    unsigned char i;

    /*
     * External keys are require no translation
     */
    if ((key >= FIRST_EXTERNAL_KEY) && (key <= LAST_EXTERNAL_KEY))
        return key;

    /*
     * Internal keys require a reverse lookup of the translation table
     */
    for (i = 0; i < NUM_INTERNAL_KEYS; i++)
    {
        if (key == key_lookup[i])
            return i;
    }

    /*
     * No match, return error
     */
    return KEY_LOOKUP_FAIL;
}


/*
 * IndexToKey -
 *
 * Convert a bit index into a key code
 * as per the key_lookup table, for internal keys
 * External keys have no translation
 */
unsigned char IndexToKey(unsigned char idx)
{
    return (unsigned char) ((idx < NUM_INTERNAL_KEYS) ? (unsigned char) key_lookup[idx] : idx);
}
