#ifndef _LCDTBL_DOT_H
#define _LCDTBL_DOT_H 1


/*
 * The base mapping (the top left row digits), is as follows:
 *
 *    ---SA--
 *    |  |  |
 *   SF  SI SB
 *    |  |  |
 *    ---SG--
 *    |  |  |
 *   SE  SJ SC
 *    |  |  |  __
 *    ---SD-- |SH|
 *
 * Segment defines are the bit position offsets from 0 for a digit
 * Note that SK, SL are not used
 */
#define SA 0
#define SB 1
#define SC 2
#define SD 3
#define SE 4
#define SF 5
#define SG 6
#define SH 7
#define SI 8
#define SJ 9
#define SK 10
#define SL 11


/*
 *  Convert segment number to mask
 */
#define MASK(n) (1 << n)


/*
 * Segment bit masks
 */
#define MS_NONE 0x0000
#define MSA MASK(SA)
#define MSB MASK(SB)
#define MSC MASK(SC)
#define MSD MASK(SD)
#define MSE MASK(SE)
#define MSF MASK(SF)
#define MSG MASK(SG)
#define MSH MASK(SH)
#define MSI MASK(SI)
#define MSJ MASK(SJ)
#define MS_ALL (MSA | MSB | MSC | MSD | MSE | MSF | MSG | MSH | MSI | MSJ)


/*
 * The size of the character segment table (ASCII characters 0..127)
 */
#define NUM_SEG_TABLE 128

/*
 * The character segment table
 */
extern const unsigned short seg_table[NUM_SEG_TABLE];


#endif
