#include "_lcdtbl.h"
#include "_lcdxlat.h"

/*
 * LCD Character translation tables for the EWireless LCD
 */


/*
 * The segment translation table for each different type of digit
 */
const signed short digit_xlat[NUM_DIGIT_XLATS][SEGS_PER_DIGIT] =
{
/* 0      1      2      3      4      5      6      7      8      9     */
 { 15-0,  1-1,   0-2,   14-3,  8-4,   9-5,   13-6,  2-7,   11-8,  12-9  },       /* Top row digit 1 */
 { 3-0,   5-1,   4-2,   2-3,   12-4,  13-5,  1-6,   6-7,   15-8,  0-9   },       /* Top row digit 2 */
 { 1-0,   15-1,  14-2,  0-3,   6-4,   7-5,   3-6,   12-7,  5-8,   2-9   },       /* Bottom row digit 1 */
 { 12-0,  10-1,  9-2,   11-3,  1-4,   2-5,   14-6,  7-7,   0-8,   13-9  },       /* Bottom row digit 2 */
 { 8-0,   6-1,   5-2,   7-3,   1-4,   2-5,   10-6,  3-7,   0-8,   9-9   },       /* Bottom row digit 7 */
 { 13-0,  3-1,   2-2,   12-3,  10-4,  11-5,  15-6,  0-7,   9-8,   14-9  },       /* Bottom row right digit 1 */
 { 1-0,   7-1,   6-2,   0-3,   14-4,  15-5,  3-6,   4-7,   13-8,  2-9   },       /* Bottom row right digit 4 */
 { 0-0,   2-1,   20-2,  8-3,   12-4,  14-5,  10-6,  0,     0,     0     }        /* Loadcell indicator */
};


/*
 * The digit list - defines each digit on the LCD
 */
const tLcdDigit lcd_digits[NUM_LCD_DIGITS] =
{
    { 120, MS_ALL, digit_xlat[0], 1 },          /* Top row digit 1 */
    { 112, MS_ALL, digit_xlat[1], 1 },          /* Top row digit 2 */
    { 96,  MS_ALL, digit_xlat[0], 1 },          /* Top row digit 3 */
    { 88,  MS_ALL, digit_xlat[1], 1 },          /* Top row digit 4 */
    { 72,  MS_ALL, digit_xlat[1], 1 },          /* Top row digit 5 */
    { 56,  (MS_ALL & ~MSH), digit_xlat[0], 0 }, /* Top row digit 6 */

    { 48, MS_ALL, digit_xlat[1], 1 },           /* Top right digit 1 */
    { 32, MS_ALL, digit_xlat[0], 1 },           /* Top right digit 2 NOTE '-' appears for '.'*/
    { 24, MS_ALL, digit_xlat[1], 1 },           /* Top right digit 3 */
    { 8, (MS_ALL & ~MSH), digit_xlat[0], 0 },   /* Top right digit 4 */

    { 216, MS_ALL, digit_xlat[2], 1 },          /* Bottom row digit 1 */
    { 225, MS_ALL, digit_xlat[3], 1 },          /* Bottom row digit 2 */
    { 240, MS_ALL, digit_xlat[2], 1 },          /* Bottom row digit 3 */
    { 249, MS_ALL, digit_xlat[3], 1 },          /* Bottom row digit 4 */
    { 264, MS_ALL, digit_xlat[2], 1 },          /* Bottom row digit 5 */
    { 273, MS_ALL, digit_xlat[3], 1 },          /* Bottom row digit 6 */
    { 293, MS_ALL, digit_xlat[4], 1 },          /* Bottom row digit 7 */
    { 304, MS_ALL, digit_xlat[2], 1 },          /* Bottom row digit 8 */
    { 313, (MS_ALL & ~MSH), digit_xlat[3], 0 }, /* Bottom row digit 9 */

    { 200, MS_ALL, digit_xlat[5], 1 },          /* Bottom right row digit 0 */
    { 192, MS_ALL, digit_xlat[6], 1 },          /* Bottom right row digit 1 */
    { 176, MS_ALL, digit_xlat[5], 1 },          /* Bottom right row digit 2 */
    { 168, MS_ALL, digit_xlat[6], 1 },          /* Bottom right row digit 3 */
    { 400, MS_ALL, digit_xlat[5], 1 },          /* Bottom right row digit 4 */
    { 392, MS_ALL, digit_xlat[6], 1 },          /* Bottom right row digit 5 */
    { 376, MS_ALL, digit_xlat[5], 1 },          /* Bottom right row digit 6, with +6 bits offset on all segs, except dp */
    { 368, (MS_ALL & ~MSH), digit_xlat[6], 0 }, /* Bottom right row digit 7 */

    { 137, (MS_ALL & ~(MASK(SH) | MASK(SI) | MASK(SJ)) ), digit_xlat[7], 0 }    /* Loadcell digit */
};
