#include <rinhw/lcd.h>
#include "_lcdtbl.h"
#include "_lcdxlat.h"

/*
 * LCD Character translation tables for the Shenzhen LCD
 */


/*
 * The segment translation table for each different type of digit
 */
const signed short digit_xlat[NUM_DIGIT_XLATS][SEGS_PER_DIGIT] =
{
/* 0      1      2      3      4      5      6      7      8      9     */
 { 3-0,   9-1,   8-2,   0-3,   2-4,   1-5,   13-6,  12-7,  15-8,  14-9  },       /* Top row digit 1, 5 */
 { 9-0,   7-1,   8-2,   10-3,  0-4,   1-5,   11-6,  6-7,   13-8,  12-9  },       /* Top row digit 2 */
 { 7-0,   13-1,  12-2,  4-3,   6-4,   5-5,   1-6,   0-7,   3-8,   2-9   },       /* Top row digit 3 */
 { 15-0,  13-1,  14-2,  0-3,   6-4,   7-5,   1-6,   12-7,  3-8,   2-9   },       /* Top row digit 4 */
 { 9-0,   7-1,   8-2,   10-3,  0-4,   1-5,   11-6,  3-7,   13-8,  12-9  },       /* Top row digit 6 */
 { 11-0,  1-1,   0-2,   8-3,   10-4,  9-5,   13-6,  12-7,  15-8,  14-9  },       /* Bottom row right digit 1,5,9 */
 { 3-0,   1-1,   2-2,   12-3,  10-4,  11-5,  13-6,  0-7,   15-8,  14-9  },       /* Bottom row right digit 2,6 */
 { 15-0,  5-1,   4-2,   12-3,  14-4,  13-5,  1-6,   0-7,   3-8,   2-9   },       /* Bottom row right digit 3,7 */
 { 7-0,   5-1,   6-2,   0-3,   14-4,  15-5,  1-6,   4-7,   3-8,   2-9   },       /* Bottom row right digit 4,8 */
 { 15-0,  13-1,  14-2,  0-3,   2-4,   3-5,   1-6,   7-7,   8-8,   9-9   }       /* Loadcell Digit */
};


/*
 * The digit list - defines each digit on the LCD
 */
static const tLcdDigit lcd_digits[NUM_LCD_DIGITS] =
{
    { 168, MS_ALL, digit_xlat[0], 1 },          /* Top row digit 1 */
    { 178, MS_ALL, digit_xlat[1], 1 },          /* Top row digit 2 */
    { 192, MS_ALL, digit_xlat[2], 1 },          /* Top row digit 3 */
    { 200, MS_ALL, digit_xlat[3], 1 },          /* Top row digit 4 */
    { 208, MS_ALL, digit_xlat[0], 1 },          /* Top row digit 5 */
    { 218, (MS_ALL & ~MSH), digit_xlat[4], 0 }, /* Top row digit 6 */

    { 232, MS_ALL, digit_xlat[2], 1 },          /* Top right digit 1 */
    { 240, MS_ALL, digit_xlat[3], 1 },          /* Top right digit 2 NOTE '-' appears for '.'*/
    { 248, MS_ALL, digit_xlat[0], 1 },          /* Top right digit 3 */
    { 258, (MS_ALL & ~MSH), digit_xlat[4], 0 }, /* Top right digit 4 */

    { 136, MS_ALL, digit_xlat[5], 1 },          /* Bottom row digit 1 */
    { 128, MS_ALL, digit_xlat[6], 1 },          /* Bottom row digit 2 */
    { 120, MS_ALL, digit_xlat[7], 1 },          /* Bottom row digit 3 */
    { 112, MS_ALL, digit_xlat[8], 1 },          /* Bottom row digit 4 */
    { 96,  MS_ALL, digit_xlat[5], 1 },          /* Bottom row digit 5 */
    { 88,  MS_ALL, digit_xlat[6], 1 },          /* Bottom row digit 6 */
    { 80,  MS_ALL, digit_xlat[7], 1 },          /* Bottom row digit 7 */
    { 72,  MS_ALL, digit_xlat[8], 1 },          /* Bottom row digit 8 */
    { 56,  (MS_ALL & ~MSH), digit_xlat[5], 0 }, /* Bottom row digit 9 */

    { 48,  MS_ALL, digit_xlat[7], 1 },          /* Bottom right row digit 0 */
    { 40,  MS_ALL, digit_xlat[8], 1 },          /* Bottom right row digit 1 */
    { 24,  MS_ALL, digit_xlat[5], 1 },          /* Bottom right row digit 2 */
    { 16,  MS_ALL, digit_xlat[6], 1 },          /* Bottom right row digit 3 */
    { 280, MS_ALL, digit_xlat[2], 1 },          /* Bottom right row digit 4 */
    { 288, MS_ALL, digit_xlat[3], 1 },          /* Bottom right row digit 5 */
    { 296, MS_ALL, digit_xlat[0], 1 },          /* Bottom right row digit 6 */
    { 306, (MS_ALL & ~MSH), digit_xlat[4], 0 }, /* Bottom right row digit 7 */

    { 160, (MS_ALL & ~(MASK(SH) | MASK(SI) | MASK(SJ)) ), digit_xlat[9], 0 }    /* Loadcell digit */
};
