/*
 * adc_ads1216.h
 *
 *  Created on: 16/08/2009
 *      Author: ian
 */

#ifndef ADC_ADS1216_H_
#define ADC_ADS1216_H_

#include <rinhw/rio.h>


/*
 * IO Control functions for the ADC device -
 */

/**
 * rioctl(&dev_adc, IOCTL_ADC24BIT_SET_ZERO_REGISTER, unsigned long level);
 *  Set zero register on the ADC
 */
#define IOCTL_ADC24BIT_SET_ZERO_REGISTER 600

/**
 * rioctl(&dev_lcd, IOCTL_ADC24BIT_GET_ZERO_REGISTER, unsigned long *level);
 *  Get zero register on the ADC
 */
#define IOCTL_ADC24BIT_GET_ZERO_REGISTER 601

/**
 * rioctl(&dev_adc, IOCTL_ADC24BIT_SET_SPAN_REGISTER, unsigned long level);
 *  Set span register on the ADC
 */
#define IOCTL_ADC24BIT_SET_SPAN_REGISTER 602

/**
 * rioctl(&dev_lcd, IOCTL_ADC24BIT_GET_SPAN_REGISTER, unsigned long *level);
 *  Get span register on the ADC
 */
#define IOCTL_ADC24BIT_GET_SPAN_REGISTER 603

/**
 * rioctl(&dev_adc, IOCTL_ADC24BIT_SET_PGA_GAIN, int level);
 *  Set the programmable gain amplifier gain level in the ADC
 */
#define IOCTL_ADC24BIT_PGA_GAIN1 1
#define IOCTL_ADC24BIT_PGA_GAIN2 2
#define IOCTL_ADC24BIT_PGA_GAIN4 4
#define IOCTL_ADC24BIT_PGA_GAIN8 8
#define IOCTL_ADC24BIT_PGA_GAIN16 16
#define IOCTL_ADC24BIT_PGA_GAIN32 32
#define IOCTL_ADC24BIT_PGA_GAIN64 64
#define IOCTL_ADC24BIT_PGA_GAIN128 128
#define IOCTL_ADC24BIT_SET_PGA_GAIN 604

/**
 * rioctl(&dev_adc, IOCTL_ADC24BIT_SET_DECIMATION_RATIO, int freq);
 *  Set decimation ratio in the ADC. freq is in 0.1Hz (ie 125 = 12.5Hz)
 */
#define IOCTL_ADC24BIT_SET_DECIMATION_RATIO 605

/**
 * rioctl(&dev_adc, IOCTL_ADC24BIT_SET_LCWIRE_MODE, int mode);
 *  Set the loadcell connection as either 4 or 6 wire
 */
#define IOCTL_ADC24BIT_LCWIRE_6WIRE 0
#define IOCTL_ADC24BIT_LCWIRE_4WIRE 1
#define IOCTL_ADC24BIT_SET_LCWIRE_MODE 606

/**
 * rioctl(&dev_adc, IOCTL_ADC24BIT_FORCE_RESET);
 *  Force reset of ADC
 */
#define IOCTL_ADC24BIT_FORCE_RESET 607


/**
 * rioctl(&dev_adc, IOCTL_ADC24BIT_IS_RESPONDING, int *isresponding);
 *  Determine if the ADC is responding
 */
#define IOCTL_ADC24BIT_IS_RESPONDING 608


/**
 * rioctl(&dev_adc, IOCTL_ADC24BIT_SET_BURNOUTCURRENTSOURCE, int enable);
 *  Turn the burn out current source on or off. This can be used to detect
 *  if a loadcell is connected
 */
#define IOCTL_ADC24BIT_BURNOUT_OFF 0
#define IOCTL_ADC24BIT_BURNOUT_ON 1
#define IOCTL_ADC24BIT_SET_BURNOUTCURRENTSOURCE 609


/**
 * rioctl(&dev_adc, IOCTL_ADC24BIT_PERFORM_SELF_ZERO_CALIBRATION);
 *  Perform a self zero calibration. Returns <0 for error, >0 is the number
 *  of samples to discard
 */
#define IOCTL_ADC24BIT_PERFORM_SELF_ZERO_CALIBRATION 610


/**
 * rioctl(&dev_adc, IOCTL_ADC24BIT_GET_ID, int *id);
 *  Get the ADC ID, this is the 2 bits in the setup register, shifted right into the LSBs
 */
#define IOCTL_ADC24BIT_GET_ID 611


/**
 * rioctl(&dev_adc, IOCTL_ADC24BIT_WAIT_FOR_DATA_READY, unsigned long timeout);
 *  Wait for the data ready interrupt to occur. Returns RIO_SUCCESS if this happens, RIO_ERROR otherwise (timeout or error).
 *  Timeout is in OS_TICKS_PER_SEC units.
 */
#define IOCTL_ADC24BIT_WAIT_FOR_DATA_READY 612


#endif /* ADC_ADS1216_H_ */
