/*
 * buzzer_generic.h
 *
 *  Created on: May 1, 2012
 *      Author: ian
 */
#ifndef BUZZER_GENERIC_H_
#define BUZZER_GENERIC_H_

#include <pthread.h>
#include <semaphore.h>
#include <rinhw/stdrinstrum.h>
#include <rinhw/buzzer.h>


/** Maximum number of buzzer events */
#define MAX_BUZ_SEQ 10


/* Forward declaration */
struct s_generic_buzzer;


/** Typedef of function to run on buzzer initialisation */
typedef int (*t_buz_hw_init_fn)(struct s_generic_buzzer *buz);


/** Typedef of function to run on buzzer shutdown */
typedef int (*t_buz_hw_destroy_fn)(struct s_generic_buzzer *buz);


/** Typedef of function to run on buzzer beep sequence */
typedef int (*t_buz_hw_run_seq_fn)(struct s_generic_buzzer *buz);


/** Generic buzzer definition */
struct s_generic_buzzer
{
  void *buz_hw_data;
  t_buz_hw_init_fn buz_hw_init_fn;
  t_buz_hw_destroy_fn buz_hw_destroy_fn;
  t_buz_hw_run_seq_fn buz_hw_run_seq_fn;

  tBuzzer buz_seq[MAX_BUZ_SEQ];         /*!< Sequence of buzzer events */
  unsigned char num_buz_seq;            /*!< Number of items in buz_seq */
  unsigned char current_buz_seq;        /*!< Current buz_seq element */

  sem_t buz_busy_sem;
  sem_t buz_signal_sem;
  bool buz_task_quit;
  pthread_t buz_thread;
};


/** Function to create and initialise a local buzzer device */
int rio_create_device_buzzer_generic(struct t_io_dev *io_dev,
                                     void *buz_hw_data,
                                     t_buz_hw_init_fn buz_hw_init_fn,
                                     t_buz_hw_destroy_fn buz_hw_destroy_fn,
                                     t_buz_hw_run_seq_fn buz_hw_run_seq_fn);


#endif /* BUZZER_GENERIC_H_ */
