#ifndef DEBUG_DOT_H
#define DEBUG_DOT_H 1

#include <rinhw/rio.h>


/*
 * Debug levels
 */
#define HW_DEBUG_OFF 0
#define HW_DEBUG_BASIC 1
#define HW_DEBUG_FULL 2


/*
 * Device initialisation debugging
 */
#define INIT_DEBUG HW_DEBUG_OFF


/*
 * Background task debugging
 */
#define BKGRND_DEBUG HW_DEBUG_OFF


/*
 * Shutdown debugging
 */
#define SHUTDOWN_DEBUG HW_DEBUG_OFF


/*
 * 8 Bit Timer Debugging
 */
#define TMR8_DEBUG HW_DEBUG_OFF


/*
 * Backlight Debugging
 */
#define BKLIGHT_DEBUG HW_DEBUG_OFF


/*
 * LCD debugging
 */
#define LCD_DEBUG HW_DEBUG_OFF
#define LCD_CHAR_DEBUG HW_DEBUG_OFF


/*
 * Serial Debugging
 */
#define SIO_DEBUG HW_DEBUG_OFF
#define SIO_PRINTER_DEBUG HW_DEBUG_OFF
#define SIO_SOCKET_DEBUG HW_DEBUG_OFF
#define SIO_DATAGRAM_DEBUG HW_DEBUG_OFF
#define SIO_PIPE_DEBUG HW_DEBUG_OFF


/*
 * IIC Debugging
 */
#define IIC_DEBUG HW_DEBUG_OFF


/*
 * RTC Debugging
 */
#define RTC_DEBUG HW_DEBUG_OFF


/*
 * Keyboard Debugging
 */
#define KBD_DEBUG HW_DEBUG_OFF

/** Linux event keyboard driver */
#define KBD_EVENT_DEBUG HW_DEBUG_OFF


/*
 * Memory File Device Debugging
 */
#define MEMORYDEV_DEBUG HW_DEBUG_OFF


/*
 * Flash Programming Debugging
 */
#define FLASH_DEBUG HW_DEBUG_OFF


/*
 * IO Device Debugging
 */
#define RIO_DEBUG HW_DEBUG_OFF


/*
 * Buzzer Debugging
 */
#define BUZ_DEBUG HW_DEBUG_OFF


/*
 * Remote IO Debugging
 */
#define REMOTE_IO_DEBUG HW_DEBUG_OFF


/*
 * FRAM Debugging
 */
#define FRAM_DEBUG HW_DEBUG_OFF


/*
 * Microprocessor ADC Debugging
 */
#define ADC_UP_DEBUG HW_DEBUG_OFF


/*
 * ADS1216 24bit ADC Debugging
 */
#define ADC_ADS1216_DEBUG HW_DEBUG_OFF


/*
 * WE2111 Hardware Access Device Debugging
 */
#define WE21HW_DEBUG HW_DEBUG_OFF


#if 1


/*
 * Macro to test if debug level high enough
 */
#define HW_DEBUG(mod, lvl) (mod >= lvl)


/** Setup the debugging device (must be open already) */
void InitDebug(struct t_io_dev *new_dev);


/**
 * Function typedef for calling back to your application for debug printfs
 *
 * @param priority is one of the LOG_xxx constants from syslog.h
 * @param format is a printf format string
 * @param args is an variable argument list to match format
 */
typedef void (*t_rinhw_debug_printf_fn)(int priority, const char *format, va_list args);


/** Set the debug printf function for the hardware library, call from your application */
void rinhw_set_debug_printf_fn(t_rinhw_debug_printf_fn fn);


/** Hardware library debug printf function */
void rinhw_debug_printf(const char *format, ...);


#define DEBUG_PRINTF rinhw_debug_printf


#else /* #ifdef DEBUG */

/*
 * When debugging is not enabled
 */

/*
 * Macro to test if debug level high enough
 */
#define HW_DEBUG(mod, lvl) (0)


#define DEBUG_PRINTF() (0)

#endif /* #else -> #ifdef DEBUG */




/** Hardware library error printf function - only use when stuff has gone badly wrong */
void rinhw_error_printf(const char *format, ...);


/** Macro to print error messages */
#define ERROR_PRINTF rinhw_error_printf


#endif /* #ifndef DEBUG_DOT_H */
