#ifndef FLASH2238_DOT_H
#define FLASH2238_DOT_H 1

#include <rinhw/rio.h>


/*
 * Notes on the flash programming device
 *
 * The flash device provides access to ALL flash on the H8S2238,
 * hence you must be careful when using it that you don't erase/program
 * over you application code, or the system will crash
 *
 * To use this device you need to do the following:
 *
 * As part of your startup code and before first use of the flash device register it:
 *        result = rregister_io(&dev_flash2238);
 * Add the following things to the linker directive:
 *     In Flash PREMAP_E and PREMAP_W which contain the erase and program algorithms respectively
 *     In RAM PRAM_E and PRAM_W which will have the erase and program code copied to them
 *        PRAM_E and PRAM_W can be overlayed. Make PRAM_W the main one, and PRAM_E the overlay
 *     Make the following sections ROM->RAM mapped:
 *        PREMAP_E -> PRAM_E
 *        PREMAP_W -> PRAM_W
 */

/*
 * The device IO position is actually
 * relative to the start of flash, not
 * the start of memory, hence the need for
 * the FLASH_START_OFFSET constant, which in this
 * case is 0, but could be otherwise.
 *
 * So physical address to program = FLASH_START_OFFSET + io device position
 */
#define FLASH_START_OFFSET (0)


/*
 * IO Control functions for the flash device -
 *
 * rioctl(&dev_flash2238, IOCTL_FLASH_ERASE_BLOCK);
 *  Erase the flash block that the io device position currently resides in
 */
#define IOCTL_FLASH_ERASE_BLOCK 200


#endif /* #ifndef FLASH2238_DOT_H */
