#ifndef GPIO_DOT_H
#define GPIO_DOT_H 1

/*
 * gpio.h -
 *
 * Stupid hitachi make the DDR registers write only, so you can't do any ORs or ANDs, so we
 * have to keep track of the value in the DDR registers in a seperate table
 *
 * Rather than directly writing a DDR, you must use these functions
 *
 *
 * Copyright (C) Rinstrum 2003
 * by Ian Burns 2/9/2003
 */


/*
 * eDdrOper - the type of operation that gpio_set_ddr is to perform (set, logical OR, logical AND)
 */
enum eDdrOper { doSET = 0, doOR = 1, doAND = 2 };

/*
 * gpio_set_ddr -
 *
 * Set the data direction register port_ddr via operation oper (set, and, or) with data.
 * The function keeps a copy of the DDR, so that the current setting of the DDR is available
 * via gpio_get_ddr(), and AND and OR operations can be made to work
 */
void gpio_set_ddr(volatile unsigned char *port_ddr, unsigned char data, enum eDdrOper oper);

/*
 * gpio_get_ddr -
 *
 * Get the current value of the data direction register. This is read from the DDR table, and not
 * the DDR register, since they are write only
 */
unsigned char gpio_get_ddr(volatile unsigned char *port_ddr);

#endif
