/**
 * hwlib_init.h -
 *
 * This file provides an alternate way to initialise the hardware library.
 *
 * In it's original usage, the hardware library on the R400 started up
 * the operating system and all devices, before calling the application main
 * function (appmain()).
 *
 * Now we are using linux/windows, our application runs as an application
 * under the operating system. So it makes sense for main() to be in the
 * application, and the application then chooses what hardware devices to initialise.
 *
 * hwlib_init() provides a basic initialisation of the hardware library, and leaves
 * the device initialisation to the application.
 */
#ifndef HWLIB_INIT_H_
#define HWLIB_INIT_H_


/**
 * Bring up the basic functionality of the hardware library:
 *    - uC/OS
 *    - Tick
 *    - Watchdog
 *    - Background function
 *    - RIO
 *
 * No devices are created here, and it is left up to the application
 * to create/initialise these using rregister_io() before they are used
 */
void hwlib_init(void);


/** Shut down the hardware library */
void hwlib_destroy(void);


#endif /* HWLIB_INIT_H_ */
