#ifndef IIC_LOCAL_LINUX_DOT_H
#define IIC_LOCAL_LINUX_DOT_H 1

#include <rinhw/rio.h>


/** Function to select an rinblock i2c slot. slot = 0..#slots-1, or -1 to de-select all slots */
typedef int (*t_iic_set_slot_select_fn)(void *hw_data, int slot);


/**
 * Function to create and initialise a local iic device. Driver takes a local copy of device_name.
 *
 * @param io_dev is the IO device to be initialised
 * @param is the linux i2c device to map the IO device to (eg /dev/i2c-0)
 * @param iic_set_slot_select_fn is a function pointer to a function that sets the slot mask (like we21hw_set_slot_select())
 * @param iic_set_slot_select_fn_data is a pointer to data used by iic_set_slot_select_fn(). Untouched/unused by the i2c driver itself
 */
int rio_create_device_iic_local(struct t_io_dev *io_dev,
                                const char *device_name,
                                t_iic_set_slot_select_fn iic_set_slot_select_fn,
                                void *iic_set_slot_select_fn_data);


#endif /* IIC_LOCAL_LINUX_DOT_H */
