#ifndef KBD_DOT_H
#define KBD_DOT_H 1

#include <rinhw/rinendian.h>
#include <rinhw/rio.h>


/* Keyboard hardware definition */
struct s_kbd_hw_definition
{
  void *hw_data;                                            /* Pointer to user hardware data, passed to init and readkeybits functions */
  void (*kbd_hw_init)(void *hw_data);                       /* Function to initialise keyboard hardware, if NULL not called */
  unsigned long (*kbd_hw_readkeybits)(void *hw_data);       /* Function to read keybits from hardware, if NULL not called */
  unsigned char already_debounced;                          /* Flag to indicate that the keys are already debounced */
  void (*kbd_hw_destroy)(void *hw_data);                    /* Function to shutdown the keyboard hardware, if NULL not called */
  unsigned long (*kbd_hw_readhikeybits)(void *hw_data);     /* Function to read the high keybits, if NULL not called, and IOCTL_KBD_WRITE_HI_KEY_BITS should be used */
};


/*
 * IO Control functions for the keyboard -
 *
 * rioctl(&dev_kbd, IOCTL_KBD_SET_BASIC, unsigned char numkeys, unsigned char *keys);
 * rioctl(&dev_kbd, IOCTL_KBD_SET_REPEAT, unsigned char numkeys, unsigned char *keys);
 * rioctl(&dev_kbd, IOCTL_KBD_SET_HOLD, unsigned char numkeys, unsigned char *keys);
 * rioctl(&dev_kbd, IOCTL_KBD_SET_DISABLED, unsigned char numkeys, unsigned char *keys);
 * rioctl(&dev_kbd, IOCTL_KBD_WRITE_HI_KEY_BITS, unsigned long keybits);
 * rioctl(&dev_kbd, IOCTL_KBD_IS_KEY_PRESSED, unsigned char *key_and_answer);
 * rioctl(&dev_kbd, IOCTL_KBD_INTERNAL_BITS, unsigned long *key_bits);
 * rioctl(&dev_kbd, IOCTL_KBD_ENABLE_BUZZER, int enable);
 */
#define IOCTL_KBD_SET_BASIC         400
#define IOCTL_KBD_SET_REPEAT        401
#define IOCTL_KBD_SET_HOLD          402
#define IOCTL_KBD_SET_DISABLED      403
#define IOCTL_KBD_WRITE_HI_KEY_BITS 404
#define IOCTL_KBD_IS_KEY_PRESSED    405
#define IOCTL_KBD_INTERNAL_BITS     406
#define IOCTL_KBD_ENABLE_BUZZER     407


/*
 * Physical key definitions
 */
#define KEY_PHYSICAL_NUM0 0
#define KEY_PHYSICAL_NUM1 1
#define KEY_PHYSICAL_NUM2 2
#define KEY_PHYSICAL_NUM3 3
#define KEY_PHYSICAL_NUM4 4
#define KEY_PHYSICAL_NUM5 5
#define KEY_PHYSICAL_NUM6 6
#define KEY_PHYSICAL_NUM7 7
#define KEY_PHYSICAL_NUM8 8
#define KEY_PHYSICAL_NUM9 9

#define KEY_PHYSICAL_POWER 10
#define KEY_PHYSICAL_FUNC0 11
#define KEY_PHYSICAL_FUNC1 12
#define KEY_PHYSICAL_FUNC2 13
#define KEY_PHYSICAL_FUNC3 14
#define KEY_PHYSICAL_FUNC4 15
#define KEY_PHYSICAL_FUNC5 16

#define KEY_PHYSICAL_SIGN 17
#define KEY_PHYSICAL_DP 18
#define KEY_PHYSICAL_CLEAR 19
#define KEY_PHYSICAL_UP 20
#define KEY_PHYSICAL_DOWN 21
#define KEY_PHYSICAL_ENTER 22
#define KEY_PHYSICAL_SETUP 23

#define KEY_PHYSICAL_POWER_AND_FUNC0 24
#define KEY_PHYSICAL_POWER_AND_FUNC1 25
#define KEY_PHYSICAL_POWER_AND_FUNC2 26
#define KEY_PHYSICAL_POWER_AND_FUNC3 27
#define KEY_PHYSICAL_POWER_AND_FUNC4 28
#define KEY_PHYSICAL_POWER_AND_FUNC5 29
#define KEY_PHYSICAL_POWER_AND_CLEAR 30

#define KEY_PHYSICAL_NOKEY 31



#define KEY_PHYSICAL_EXTERNAL0 32
#define KEY_PHYSICAL_EXTERNAL1 33
#define KEY_PHYSICAL_EXTERNAL2 34
#define KEY_PHYSICAL_EXTERNAL3 35
#define KEY_PHYSICAL_EXTERNAL4 36
#define KEY_PHYSICAL_EXTERNAL5 37
#define KEY_PHYSICAL_EXTERNAL6 38
#define KEY_PHYSICAL_EXTERNAL7 39
#define KEY_PHYSICAL_EXTERNAL8 40
#define KEY_PHYSICAL_EXTERNAL9 41
#define KEY_PHYSICAL_EXTERNAL10 42
#define KEY_PHYSICAL_EXTERNAL11 43
#define KEY_PHYSICAL_EXTERNAL12 44
#define KEY_PHYSICAL_EXTERNAL13 45
#define KEY_PHYSICAL_EXTERNAL14 46
#define KEY_PHYSICAL_EXTERNAL15 47
#define KEY_PHYSICAL_EXTERNAL16 48
#define KEY_PHYSICAL_EXTERNAL17 49
#define KEY_PHYSICAL_EXTERNAL18 50
#define KEY_PHYSICAL_EXTERNAL19 51
#define KEY_PHYSICAL_EXTERNAL20 52
#define KEY_PHYSICAL_EXTERNAL21 53
#define KEY_PHYSICAL_EXTERNAL22 54
#define KEY_PHYSICAL_EXTERNAL23 55
#define KEY_PHYSICAL_EXTERNAL24 56
#define KEY_PHYSICAL_EXTERNAL25 57
#define KEY_PHYSICAL_EXTERNAL26 58
#define KEY_PHYSICAL_EXTERNAL27 59
#define KEY_PHYSICAL_EXTERNAL28 60
#define KEY_PHYSICAL_EXTERNAL29 61
#define KEY_PHYSICAL_EXTERNAL30 62
#define KEY_PHYSICAL_EXTERNAL31 63


/*
 * Key counts
 */
#define NUM_INTERNAL_KEYS 31
#define NUM_EXTERNAL_KEYS 32
#define FIRST_EXTERNAL_KEY KEY_PHYSICAL_EXTERNAL0
#define LAST_EXTERNAL_KEY KEY_PHYSICAL_EXTERNAL31
#define NUM_FUNC_KEYS 6


/*
 * The struct read using rread, each byte is structured like this
 */
#if (BYTE_ORDER == BIG_ENDIAN) 
/* Hitachi - big endian */
union un_key
{
    unsigned char BYTE;
    struct
    {
        unsigned char EVENT:2;
        unsigned char KEY  :6;
    } BIT;
};
#else
/* i386 - Little Endian */
union un_key
{
    unsigned char BYTE;
    struct
    {
        unsigned char KEY  :6;
        unsigned char EVENT:2;
    } BIT;
};
#endif


/*
 * Determines what type of key event has occurred
 */
enum tKeyEvent { keDown = 0, keUp = 1, keHeld = 2, keDisabled = 3 };


/*
 * converts a short press key code to a bit position for use as a mask
 */
unsigned long KeyToMask(unsigned char key);


#endif /* #ifndef KBD_DOT_H */
