/*
 * kbd_linux_event.h
 *
 *  Created on: Dec 7, 2011
 *      Author: ian
 */

#ifndef KBD_LINUX_EVENT_H_
#define KBD_LINUX_EVENT_H_

#include <rinhw/rio.h>


/** Invalid parameter passed to function */
#define RIO_ERROR_EVKBD_INVALID_PARAM (-2)


/** The event device is already being handled by the driver */
#define RIO_ERROR_EVKBD_ALREADY_HANDLED (-3)


/** The event device is not being handled by the driver */
#define RIO_ERROR_EVKBD_NOT_HANDLED (-4)


/** There's no room for handling more event devices */
#define RIO_ERROR_EVKBD_FULL (-5)


/** There was an error reported from Linux when opening/accessing event device */
#define RIO_ERROR_EVKBD_IO (-6)


/** The keys we want to use are not supported by this event device */
#define RIO_ERROR_EVKBD_KEYS_UNSUPPORTED (-7)


/** Memory could not be allocated */
#define RIO_ERROR_EVKBD_OUT_OF_MEMORY (-8)


/** Device is not an event device */
#define RIO_ERROR_EVKBD_NOT_EVENT (-9)


/** Miscellaneous errors */
#define RIO_ERROR_EVKBD_MISC (-10)


/** Function to create and initialise a linux event keyboard device */
int rio_create_device_kbd_linux_event(struct t_io_dev *io_dev, struct t_io_dev *buzzer_dev);


/** Add a new event device (eg /dev/event0) to monitor to a rinstrum linux event keyboard driver (created by rio_create_device_kbd_linux_event())  */
int rio_kbd_linux_event_add_event_device(struct t_io_dev *io_dev, const char *new_device_name);


/** Remove an event device (eg /dev/event0) from a rinstrum linux event keyboard driver (created by rio_create_device_kbd_linux_event())  */
int rio_kbd_linux_event_remove_event_device(struct t_io_dev *io_dev, const char *device_name);


#endif /* KBD_LINUX_EVENT_H_ */
