#ifndef	LCD_EW_DOT_H
#define	LCD_EW_DOT_H 1


/*
 * The number of bytes/segments in the LCD buffer
 */
#define NUM_LCD_BYTES 52
#define NUM_LCD_SEGMENTS (NUM_LCD_BYTES * 8)


/*
 * NUM_DIGIT_XLATS - Number of different digit configurations in the LCD
 */
#define NUM_DIGIT_XLATS 8


/*
 * A bitfield struct to contain
 * the raw segments in the correct order
 *
 * It's not advisable to use these segments to access the LCD
 * since the naming is LCD dependent, use the Named segments instead
 * as these don't change between LCDs
 */
typedef struct
{
    // Top IC
    unsigned char F3  : 1;   // B0.7
    unsigned char F2  : 1;
    unsigned char uu1 : 1;
    unsigned char F1  : 1;
    unsigned char F8  : 1;
    unsigned char F4  : 1;
    unsigned char F7  : 1;
    unsigned char F6  : 1;   // B0.0

    unsigned char F11 : 1;   // B1.7
    unsigned char F5  : 1;
    unsigned char F10 : 1;
    unsigned char F9  : 1;
    unsigned char uu2 : 1;
    unsigned char uu3 : 1;
    unsigned char D4B : 1;
    unsigned char D4C : 1;   // B1.0

    unsigned char D4A : 1;   // B2.7
    unsigned char D4D : 1;
    unsigned char D4G : 1;
    unsigned char D4J : 1;
    unsigned char D4I : 1;
    unsigned char uu4 : 1;
    unsigned char D4F : 1;
    unsigned char D4E : 1;   // B2.0

    unsigned char uu5 : 1;   // B3.7
    unsigned char D3H : 1;
    unsigned char D3B : 1;
    unsigned char D3C : 1;
    unsigned char D3A : 1;
    unsigned char D3D : 1;
    unsigned char D3G : 1;
    unsigned char D3J : 1;   // B3.0

    unsigned char D3I : 1;   // B4.7
    unsigned char uu6 : 1;
    unsigned char D3F : 1;
    unsigned char D3E : 1;
    unsigned char uu7 : 1;
    unsigned char D2H : 1;
    unsigned char D2B : 1;
    unsigned char D2C : 1;   // B4.0

    unsigned char D2A : 1;   // B5.7
    unsigned char D2D : 1;
    unsigned char D2G : 1;
    unsigned char D2J : 1;
    unsigned char D2I : 1;
    unsigned char uu8 : 1;
    unsigned char D2F : 1;
    unsigned char D2E : 1;   // B5.0

    unsigned char uu9 : 1;   // B6.7
    unsigned char D1H : 1;
    unsigned char D1B : 1;
    unsigned char D1C : 1;
    unsigned char D1A : 1;
    unsigned char D1D : 1;
    unsigned char D1G : 1;
    unsigned char D1J : 1;   // B6.0

    unsigned char D1I : 1;   // B7.7
    unsigned char uu10 : 1;
    unsigned char D1F : 1;
    unsigned char D1E : 1;
    unsigned char uu11 : 1;
    unsigned char uu12 : 1;
    unsigned char A6B : 1;
    unsigned char A6C : 1;   // B7.0

    unsigned char A6A : 1;   // B8.7
    unsigned char A6D : 1;
    unsigned char A6G : 1;
    unsigned char A6J : 1;
    unsigned char A6I : 1;
    unsigned char uu13 : 1;
    unsigned char A6F : 1;
    unsigned char A6E : 1;   // B8.0

    unsigned char uu14 : 1;   // B9.7
    unsigned char A5H : 1;
    unsigned char A5B : 1;
    unsigned char A5C : 1;
    unsigned char A5A : 1;
    unsigned char A5D : 1;
    unsigned char A5G : 1;
    unsigned char A5J : 1;   // B9.0

    unsigned char A5I : 1;   // B10.7
    unsigned char uu15 : 1;
    unsigned char A5F : 1;
    unsigned char A5E : 1;
    unsigned char uu16 : 4;   // B10.0

    unsigned char uu17 : 1;   // B11.7
    unsigned char A4H : 1;
    unsigned char A4B : 1;
    unsigned char A4C : 1;
    unsigned char A4A : 1;
    unsigned char A4D : 1;
    unsigned char A4G : 1;
    unsigned char A4J : 1;   // B11.0

    unsigned char A4I : 1;   // B12.7
    unsigned char uu18 : 1;
    unsigned char A4F : 1;
    unsigned char A4E : 1;
    unsigned char uu19 : 1;
    unsigned char A3H : 1;
    unsigned char A3B : 1;
    unsigned char A3C : 1;   // B12.0

    unsigned char A3A : 1;   // B13.7
    unsigned char A3D : 1;
    unsigned char A3G : 1;
    unsigned char A3J : 1;
    unsigned char A3I : 1;
    unsigned char uu20 : 1;
    unsigned char A3F : 1;
    unsigned char A3E : 1;   // B13.0

    unsigned char uu21 : 1;   // B14.7
    unsigned char A2H : 1;
    unsigned char A2B : 1;
    unsigned char A2C : 1;
    unsigned char A2A : 1;
    unsigned char A2D : 1;
    unsigned char A2G : 1;
    unsigned char A2J : 1;   // B14.0

    unsigned char A2I : 1;   // B15.7
    unsigned char uu22 : 1;
    unsigned char A2F : 1;
    unsigned char A2E : 1;
    unsigned char uu23 : 1;
    unsigned char A1H : 1;
    unsigned char A1B : 1;
    unsigned char A1C : 1;   // B15.0

    unsigned char A1A : 1;   // B16.7
    unsigned char A1D : 1;
    unsigned char A1G : 1;
    unsigned char A1J : 1;
    unsigned char A1I : 1;
    unsigned char E18 : 1;
    unsigned char A1F : 1;
    unsigned char A1E : 1;   // B16.0

    unsigned char E1 : 1;    // B17.7
    unsigned char uu24 : 1;
    unsigned char E2 : 1;
    unsigned char uu25 : 1;
    unsigned char E4 : 1;
    unsigned char uu26 : 1;
    unsigned char E3 : 1;
    unsigned char uu27 : 1;  // B17.0

    unsigned char E8 : 1;    // B18.7
    unsigned char uu28 : 1;
    unsigned char E7 : 1;
    unsigned char uu29 : 1;
    unsigned char E9 : 1;
    unsigned char uu30 : 1;
    unsigned char E6 : 1;
    unsigned char uu31 : 1;  // B18.0

    unsigned char uu32 : 2;    // B19.7
    unsigned char E5 : 1;
    unsigned char uu33 : 1;
    unsigned char E12 : 1;
    unsigned char uu34 : 1;
    unsigned char E10 : 1;
    unsigned char uu35 : 1;  // B19.0

    unsigned char E19 : 1;    // B20.7
    unsigned char uu36 : 1;
    unsigned char E11 : 1;
    unsigned char uu37 : 1;
    unsigned char E14 : 1;
    unsigned char E17 : 1;
    unsigned char E15 : 1;
    unsigned char E16 : 1;  // B20.0

    unsigned char C4B : 1;  // B21.7
    unsigned char C4C : 1;
    unsigned char uu38 : 1;
    unsigned char C4H : 1;
    unsigned char C4G : 1;
    unsigned char C4J : 1;
    unsigned char C4A : 1;
    unsigned char C4D : 1;  // B21.0

    unsigned char C4F : 1;  // B22.7
    unsigned char C4E : 1;
    unsigned char C4I : 1;
    unsigned char uu39 : 1;
    unsigned char C3B : 1;
    unsigned char C3C : 1;
    unsigned char uu40 : 1;
    unsigned char C3H : 1;  // B22.0

    unsigned char C3G : 1;  // B23.7
    unsigned char C3J : 1;
    unsigned char C3A : 1;
    unsigned char C3D : 1;
    unsigned char C3F : 1;
    unsigned char C3E : 1;
    unsigned char C3I : 1;
    unsigned char uu41 : 1;  // B23.0

    unsigned char C2B : 1;  // B24.7
    unsigned char C2C : 1;
    unsigned char uu42 : 1;
    unsigned char C2H : 1;
    unsigned char C2G : 1;
    unsigned char C2J : 1;
    unsigned char C2A : 1;
    unsigned char C2D : 1;  // B24.0

    unsigned char C2F : 1;  // B25.7
    unsigned char C2E : 1;
    unsigned char C2I : 1;
    unsigned char uu43 : 1;
    unsigned char C1B : 1;
    unsigned char C1C : 1;
    unsigned char uu44 : 1;
    unsigned char C1H : 1;  // B25.0

    unsigned char C1G : 1;  // B26.7
    unsigned char C1J : 1;
    unsigned char C1A : 1;
    unsigned char C1D : 1;
    unsigned char C1F : 1;
    unsigned char C1E : 1;
    unsigned char C1I : 1;
    unsigned char uu45 : 1;  // B26.0

    unsigned char B1F : 1;  // B27.7
    unsigned char B1E : 1;
    unsigned char B1I : 1;
    unsigned char uu46 : 1;
    unsigned char B1G : 1;
    unsigned char B1J : 1;
    unsigned char B1A : 1;
    unsigned char B1D : 1;  // B27.0

    unsigned char B1B : 1;  // B28.7
    unsigned char B1C : 1;
    unsigned char uu47 : 1;
    unsigned char B1H : 1;
    unsigned char B2F : 1;
    unsigned char B2E : 1;
    unsigned char B2I : 1;
    unsigned char uu48 : 1;  // B28.0

    unsigned char B2G : 1;  // B29.7
    unsigned char B2J : 1;
    unsigned char B2A : 1;
    unsigned char B2D : 1;
    unsigned char B2B : 1;
    unsigned char B2C : 1;
    unsigned char uu49 : 1;
    unsigned char B2H : 1;  // B29.0

    unsigned char B3F : 1;  // B30.7
    unsigned char B3E : 1;
    unsigned char B3I : 1;
    unsigned char uu50 : 1;
    unsigned char B3G : 1;
    unsigned char B3J : 1;
    unsigned char B3A : 1;
    unsigned char B3D : 1;  // B30.0

    unsigned char B3B : 1;  // B31.7
    unsigned char B3C : 1;
    unsigned char uu51 : 1;
    unsigned char B3H : 1;
    unsigned char B4F : 1;
    unsigned char B4E : 1;
    unsigned char B4I : 1;
    unsigned char uu52 : 1;  // B31.0

    unsigned char B4G : 1;  // B32.7
    unsigned char B4J : 1;
    unsigned char B4A : 1;
    unsigned char B4D : 1;
    unsigned char B4B : 1;
    unsigned char B4C : 1;
    unsigned char uu53 : 1;
    unsigned char B4H : 1;  // B32.0

    unsigned char B5F : 1;  // B33.7
    unsigned char B5E : 1;
    unsigned char B5I : 1;
    unsigned char uu54 : 1;
    unsigned char B5G : 1;
    unsigned char B5J : 1;
    unsigned char B5A : 1;
    unsigned char B5D : 1;  // B33.0

    unsigned char B5B : 1;  // B34.7
    unsigned char B5C : 1;
    unsigned char uu55 : 1;
    unsigned char B5H : 1;
    unsigned char B6F : 1;
    unsigned char B6E : 1;
    unsigned char B6I : 1;
    unsigned char uu56 : 1;  // B34.0

    unsigned char B6G : 1;  // B35.7
    unsigned char B6J : 1;
    unsigned char B6A : 1;
    unsigned char B6D : 1;
    unsigned char B6B : 1;
    unsigned char B6C : 1;
    unsigned char uu57 : 1;
    unsigned char B6H : 1;  // B35.0

    unsigned char B7F : 1;  // B36.7
    unsigned char B7E : 1;
    unsigned char B7I : 1;
    unsigned char uu58 : 5; // B36.0

    unsigned char B7G : 1;  // B37.7
    unsigned char B7J : 1;
    unsigned char B7A : 1;
    unsigned char B7D : 1;
    unsigned char B7B : 1;
    unsigned char B7C : 1;
    unsigned char uu59 : 1;
    unsigned char B7H : 1;  // B37.0

    unsigned char B8F : 1;  // B38.7
    unsigned char B8E : 1;
    unsigned char B8I : 1;
    unsigned char uu60 : 1;
    unsigned char B8G : 1;
    unsigned char B8J : 1;
    unsigned char B8A : 1;
    unsigned char B8D : 1;  // B38.0

    unsigned char B8B : 1;  // B39.7
    unsigned char B8C : 1;
    unsigned char uu61 : 1;
    unsigned char B8H : 1;
    unsigned char B9F : 1;
    unsigned char B9E : 1;
    unsigned char B9I : 1;
    unsigned char uu62 : 1;  // B39.0

    unsigned char B9G : 1;  // B40.7
    unsigned char B9J : 1;
    unsigned char B9A : 1;
    unsigned char B9D : 1;
    unsigned char B9B : 1;
    unsigned char B9C : 1;
    unsigned char uu63 : 1;
    unsigned char B9H : 1;  // B40.0

    unsigned char G2 : 1;  // B41.7
    unsigned char G1 : 1;
    unsigned char uu64 : 2;
    unsigned char G3 : 1;
    unsigned char G4 : 1;
    unsigned char G7 : 1;
    unsigned char uu65 : 1;  // B41.0

    unsigned char G6  : 1;  // B42.7
    unsigned char G5  : 1;
    unsigned char G8  : 1;
    unsigned char uu66 : 1;
    unsigned char G11 : 1;
    unsigned char G9  : 1;
    unsigned char G10 : 1;
    unsigned char G20 : 1;  // B42.0

    unsigned char G14 : 1;  // B43.7
    unsigned char G13 : 1;
    unsigned char G15 : 1;
    unsigned char G12 : 1;
    unsigned char uu67 : 3;
    unsigned char G18 : 1;  // B43.0

    unsigned char uu68 : 3;  // B44.7
    unsigned char G17 : 1;
    unsigned char uu69 : 3;
    unsigned char G16 : 1;  // B44.0

    unsigned char uu70 : 3;  // B45.7
    unsigned char G19 : 1;
    unsigned char G24 : 1;
    unsigned char G23 : 1;
    unsigned char G25 : 1;
    unsigned char G22 : 1;  // B45.0

    unsigned char C8B : 1;  // B46.7
    unsigned char C8C : 1;
    unsigned char uu71 : 1;
    unsigned char G21 : 1;
    unsigned char C8G : 1;
    unsigned char C8J : 1;
    unsigned char C8A : 1;
    unsigned char C8D : 1;  // B46.0

    unsigned char C8F : 1;  // B47.7
    unsigned char C8E : 1;
    unsigned char C8I : 1;
    unsigned char uu72 : 1;
    unsigned char C7B : 1;
    unsigned char C7C : 1;
    unsigned char uu73 : 1;
    unsigned char C7H : 1;  // B47.0

    unsigned char C7G : 1;  // B48.7
    unsigned char C7J : 1;
    unsigned char C7A : 1;
    unsigned char C7D : 1;
    unsigned char C7F : 1;
    unsigned char C7E : 1;
    unsigned char C7I : 1;
    unsigned char uu74 : 1;  // B48.0

    unsigned char C6B : 1;  // B49.7
    unsigned char C6C : 1;
    unsigned char uu75 : 1;
    unsigned char C6H : 1;
    unsigned char C6G : 1;
    unsigned char C6J : 1;
    unsigned char C6A : 1;
    unsigned char C6D : 1;  // B49.0

    unsigned char C6F : 1;  // B50.7
    unsigned char C6E : 1;
    unsigned char C6I : 1;
    unsigned char uu76 : 1;
    unsigned char C5B : 1;
    unsigned char C5C : 1;
    unsigned char uu77 : 1;
    unsigned char C5H : 1;  // B50.0

    unsigned char C5G : 1;  // B51.7
    unsigned char C5J : 1;
    unsigned char C5A : 1;
    unsigned char C5D : 1;
    unsigned char C5F : 1;
    unsigned char C5E : 1;
    unsigned char C5I : 1;
    unsigned char uu78 : 1;  // B51.0

} tRawSegments;


/*
 * All the segments in the digit structs below here are labelled as follows:
 * Accessing digit segments should be done using these structs
 * as the naming won't change between different LCDs, hence no changes
 * to app code will be required.
 *
 *    ---A--
 *    |  |  |
 *    F  I  B
 *    |  |  |
 *    ---G---
 *    |  |  |
 *    E  J  C
 *    |  |  |  _
 *    ---D--- |H|
 *
 */


/*
 * Annunciators -
 */
typedef struct
{
    unsigned char unused1[16];

    unsigned char uu2 : 5;
    unsigned char Zero : 1;         // E18
    unsigned char uu3 : 2;

    unsigned char Sigma : 1;        // E1
    unsigned char uu4 : 1;
    unsigned char Balance : 1;      // E2
    unsigned char uu5 : 1;
    unsigned char BalSegB : 1;      // E4
    unsigned char uu6 : 1;
    unsigned char BalSegA : 1;      // E3
    unsigned char uu7 : 1;

    unsigned char BalSegF : 1;      // E8
    unsigned char uu8 : 1;
    unsigned char BalSegE : 1;      // E7
    unsigned char uu9 : 1;
    unsigned char BalSegG : 1;      // E9
    unsigned char uu10 : 1;
    unsigned char BalSegD : 1;      // E6
    unsigned char uu11 : 1;

    unsigned char uu12 : 1;
    unsigned char uu13 : 1;
    unsigned char BalSegC : 1;      // E5
    unsigned char uu14 : 1;
    unsigned char RngSegC : 1;      // E12
    unsigned char uu15 : 1;
    unsigned char Range : 1;        // E10
    unsigned char uu16 : 1;

    unsigned char RngSegE : 1;      // E19
    unsigned char uu17 : 1;
    unsigned char RngSegADG : 1;    // E11
    unsigned char uu18 : 1;
    unsigned char Hold : 1;         // E14
    unsigned char Motion : 1;       // E17
    unsigned char Net : 1;          // E15
    unsigned char Coz : 1;          // E16
} tAnnunciators;


/*
 * Top Row Digits
 */
typedef struct
{
    unsigned char uu1[7];

    /*
     * Byte 7
     */
    unsigned char uu2 : 6;
    unsigned char D_6B : 1;
    unsigned char D_6C : 1;

    /*
     * Byte 8
     */
    unsigned char D_6A : 1;
    unsigned char D_6D : 1;
    unsigned char D_6G : 1;
    unsigned char D_6J : 1;
    unsigned char D_6I : 1;
    unsigned char uu3 : 1;
    unsigned char D_6F : 1;
    unsigned char D_6E : 1;

    /*
     * Byte 9
     */
    unsigned char uu4 : 1;
    unsigned char D_5H : 1;
    unsigned char D_5B : 1;
    unsigned char D_5C : 1;
    unsigned char D_5A : 1;
    unsigned char D_5D : 1;
    unsigned char D_5G : 1;
    unsigned char D_5J : 1;

    /*
     * Byte 10
     */
    unsigned char D_5I : 1;
    unsigned char uu5 : 1;
    unsigned char D_5F : 1;
    unsigned char D_5E : 1;
    unsigned char uu6 : 4;

    /*
     * Byte 11
     */
    unsigned char uu7 : 1;
    unsigned char D_4H : 1;
    unsigned char D_4B : 1;
    unsigned char D_4C : 1;
    unsigned char D_4A : 1;
    unsigned char D_4D : 1;
    unsigned char D_4G : 1;
    unsigned char D_4J : 1;

    /*
     * Byte 12
     */
    unsigned char D_4I : 1;
    unsigned char uu8 : 1;
    unsigned char D_4F : 1;
    unsigned char D_4E : 1;
    unsigned char uu9 : 1;
    unsigned char D_3H : 1;
    unsigned char D_3B : 1;
    unsigned char D_3C : 1;

    /*
     * Byte 13
     */
    unsigned char D_3A : 1;
    unsigned char D_3D : 1;
    unsigned char D_3G : 1;
    unsigned char D_3J : 1;
    unsigned char D_3I : 1;
    unsigned char uu10 : 1;
    unsigned char D_3F : 1;
    unsigned char D_3E : 1;

    /*
     * Byte 14
     */
    unsigned char uu11 : 1;
    unsigned char D_2H : 1;
    unsigned char D_2B : 1;
    unsigned char D_2C : 1;
    unsigned char D_2A : 1;
    unsigned char D_2D : 1;
    unsigned char D_2G : 1;
    unsigned char D_2J : 1;

    /*
     * Byte 15
     */
    unsigned char D_2I : 1;
    unsigned char uu13 : 1;
    unsigned char D_2F : 1;
    unsigned char D_2E : 1;
    unsigned char uu14 : 1;
    unsigned char D_1H : 1;
    unsigned char D_1B : 1;
    unsigned char D_1C : 1;

    /*
     * Byte 16
     */
    unsigned char D_1A : 1;
    unsigned char D_1D : 1;
    unsigned char D_1G : 1;
    unsigned char D_1J : 1;
    unsigned char D_1I : 1;
    unsigned char uu15 : 1;
    unsigned char D_1F : 1;
    unsigned char D_1E : 1;

} tTopRowDigits;


/*
 * Top row right digits
 */
typedef struct
{
    unsigned char uu1[1];

    /*
     * Byte 1
     */
    unsigned char uu2 : 6;
    unsigned char D_4B : 1;
    unsigned char D_4C : 1;

    /*
     * Byte 2
     */
    unsigned char D_4A : 1;
    unsigned char D_4D : 1;
    unsigned char D_4G : 1;
    unsigned char D_4J : 1;
    unsigned char D_4I : 1;
    unsigned char uu3 : 1;
    unsigned char D_4F : 1;
    unsigned char D_4E : 1;

    /*
     * Byte 3
     */
    unsigned char uu4 : 1;
    unsigned char D_3H : 1;
    unsigned char D_3B : 1;
    unsigned char D_3C : 1;
    unsigned char D_3A : 1;
    unsigned char D_3D : 1;
    unsigned char D_3G : 1;
    unsigned char D_3J : 1;

    /*
     * Byte 4
     */
    unsigned char D_3I : 1;
    unsigned char uu5 : 1;
    unsigned char D_3F : 1;
    unsigned char D_3E : 1;
    unsigned char uu6 : 1;
    unsigned char D_2H : 1;
    unsigned char D_2B : 1;
    unsigned char D_2C : 1;

    /*
     * Byte 5
     */
    unsigned char D_2A : 1;
    unsigned char D_2D : 1;
    unsigned char D_2G : 1;
    unsigned char D_2J : 1;
    unsigned char D_2I : 1;
    unsigned char uu7 : 1;
    unsigned char D_2F : 1;
    unsigned char D_2E : 1;

    /*
     * Byte 6
     */
    unsigned char uu8 : 1;
    unsigned char D_1H : 1;
    unsigned char D_1B : 1;
    unsigned char D_1C : 1;
    unsigned char D_1A : 1;
    unsigned char D_1D : 1;
    unsigned char D_1G : 1;
    unsigned char D_1J : 1;

    /*
     * Byte 7
     */
    unsigned char D_1I : 1;
    unsigned char uu9 : 1;
    unsigned char D_1F : 1;
    unsigned char D_1E : 1;
    unsigned char uu10 : 4;

} tTopRightRowDigits;


/*
 * Top Row Units
 */
typedef struct
{
    unsigned char K : 1;
    unsigned char T : 1;
    unsigned char uu1 : 1;
    unsigned char L : 1;
    unsigned char G : 1;
    unsigned char B : 1;
    unsigned char P : 1;
    unsigned char O : 1;

    unsigned char ArrowBot : 1;
    unsigned char N : 1;
    unsigned char Z : 1;
    unsigned char ArrowTop : 1;
    unsigned char uu2 : 4;
} tTopRowUnits;


/*
 * Bottom row digits
 */
typedef struct
{
    unsigned char uu1[27];

    /*
     * Byte 27
     */
    unsigned char D_1F : 1;
    unsigned char D_1E : 1;
    unsigned char D_1I : 1;
    unsigned char uu2 : 1;
    unsigned char D_1G : 1;
    unsigned char D_1J : 1;
    unsigned char D_1A : 1;
    unsigned char D_1D : 1;

    /*
     * Byte 28
     */
    unsigned char D_1B : 1;
    unsigned char D_1C : 1;
    unsigned char uu3 : 1;
    unsigned char D_1H : 1;
    unsigned char D_2F : 1;
    unsigned char D_2E : 1;
    unsigned char D_2I : 1;
    unsigned char uu4 : 1;

    /*
     * Byte 29
     */
    unsigned char D_2G : 1;
    unsigned char D_2J : 1;
    unsigned char D_2A : 1;
    unsigned char D_2D : 1;
    unsigned char D_2B : 1;
    unsigned char D_2C : 1;
    unsigned char uu5 : 1;
    unsigned char D_2H : 1;

    /*
     * Byte 30
     */
    unsigned char D_3F : 1;
    unsigned char D_3E : 1;
    unsigned char D_3I : 1;
    unsigned char uu6 : 1;
    unsigned char D_3G : 1;
    unsigned char D_3J : 1;
    unsigned char D_3A : 1;
    unsigned char D_3D : 1;

    /*
     * Byte 31
     */
    unsigned char D_3B : 1;
    unsigned char D_3C : 1;
    unsigned char uu7 : 1;
    unsigned char D_3H : 1;
    unsigned char D_4F : 1;
    unsigned char D_4E : 1;
    unsigned char D_4I : 1;
    unsigned char uu8 : 1;

    /*
     * Byte 32
     */
    unsigned char D_4G : 1;
    unsigned char D_4J : 1;
    unsigned char D_4A : 1;
    unsigned char D_4D : 1;
    unsigned char D_4B : 1;
    unsigned char D_4C : 1;
    unsigned char uu9 : 1;
    unsigned char D_4H : 1;

    /*
     * Byte 33
     */
    unsigned char D_5F : 1;
    unsigned char D_5E : 1;
    unsigned char D_5I : 1;
    unsigned char uu10 : 1;
    unsigned char D_5G : 1;
    unsigned char D_5J : 1;
    unsigned char D_5A : 1;
    unsigned char D_5D : 1;

    /*
     * Byte 34
     */
    unsigned char D_5B : 1;
    unsigned char D_5C : 1;
    unsigned char uu11 : 1;
    unsigned char D_5H : 1;
    unsigned char D_6F : 1;
    unsigned char D_6E : 1;
    unsigned char D_6I : 1;
    unsigned char uu12 : 1;

    /*
     * Byte 35
     */
    unsigned char D_6G : 1;
    unsigned char D_6J : 1;
    unsigned char D_6A : 1;
    unsigned char D_6D : 1;
    unsigned char D_6B : 1;
    unsigned char D_6C : 1;
    unsigned char uu13 : 1;
    unsigned char D_6H : 1;

    /*
     * Byte 36
     */
    unsigned char D_7F : 1;
    unsigned char D_7E : 1;
    unsigned char D_7I : 1;
    unsigned char uu14 : 5;

    /*
     * Byte 37
     */
    unsigned char D_7G : 1;
    unsigned char D_7J : 1;
    unsigned char D_7A : 1;
    unsigned char D_7D : 1;
    unsigned char D_7B : 1;
    unsigned char D_7C : 1;
    unsigned char uu15 : 1;
    unsigned char D_7H : 1;

    /*
     * Byte 38
     */
    unsigned char D_8F : 1;
    unsigned char D_8E : 1;
    unsigned char D_8I : 1;
    unsigned char uu16 : 1;
    unsigned char D_8G : 1;
    unsigned char D_8J : 1;
    unsigned char D_8A : 1;
    unsigned char D_8D : 1;

    /*
     * Byte 39
     */
    unsigned char D_8B : 1;
    unsigned char D_8C : 1;
    unsigned char uu17 : 1;
    unsigned char D_8H : 1;
    unsigned char D_9F : 1;
    unsigned char D_9E : 1;
    unsigned char D_9I : 1;
    unsigned char uu18 : 1;

    /*
     * Byte 40
     */
    unsigned char D_9G : 1;
    unsigned char D_9J : 1;
    unsigned char D_9A : 1;
    unsigned char D_9D : 1;
    unsigned char D_9B : 1;
    unsigned char D_9C : 1;
    unsigned char uu19 : 1;
    unsigned char D_9H : 1;

} tBottomRowDigits;


/*
 * Bottom right row digits
 */
typedef struct
{
    unsigned char uu1[21];

    /*
     * Byte 21
     */
    unsigned char D_4B : 1;
    unsigned char D_4C : 1;
    unsigned char uu2 : 1;
    unsigned char D_4H : 1;
    unsigned char D_4G : 1;
    unsigned char D_4J : 1;
    unsigned char D_4A : 1;
    unsigned char D_4D : 1;

    /*
     * Byte 22
     */
    unsigned char D_4F : 1;
    unsigned char D_4E : 1;
    unsigned char D_4I : 1;
    unsigned char uu3 : 1;
    unsigned char D_3B : 1;
    unsigned char D_3C : 1;
    unsigned char uu4 : 1;
    unsigned char D_3H : 1;

    /*
     * Byte 23
     */
    unsigned char D_3G : 1;
    unsigned char D_3J : 1;
    unsigned char D_3A : 1;
    unsigned char D_3D : 1;
    unsigned char D_3F : 1;
    unsigned char D_3E : 1;
    unsigned char D_3I : 1;
    unsigned char uu5 : 1;

    /*
     * Byte 24
     */
    unsigned char D_2B : 1;
    unsigned char D_2C : 1;
    unsigned char uu6 : 1;
    unsigned char D_2H : 1;
    unsigned char D_2G : 1;
    unsigned char D_2J : 1;
    unsigned char D_2A : 1;
    unsigned char D_2D : 1;

    /*
     * Byte 25
     */
    unsigned char D_2F : 1;
    unsigned char D_2E : 1;
    unsigned char D_2I : 1;
    unsigned char uu7 : 1;
    unsigned char D_1B : 1;
    unsigned char D_1C : 1;
    unsigned char uu8 : 1;
    unsigned char D_1H : 1;

    /*
     * Byte 26
     */
    unsigned char D_1G : 1;
    unsigned char D_1J : 1;
    unsigned char D_1A : 1;
    unsigned char D_1D : 1;
    unsigned char D_1F : 1;
    unsigned char D_1E : 1;
    unsigned char D_1I : 1;
    unsigned char uu9 : 1;

    /*
     * Bytes 27-45
     */
    unsigned char uu10[19];

    /*
     * Byte 46
     */
    unsigned char D_8B : 1;
    unsigned char D_8C : 1;
    unsigned char uu11 : 2;
    unsigned char D_8G : 1;
    unsigned char D_8J : 1;
    unsigned char D_8A : 1;
    unsigned char D_8D : 1;

    /*
     * Byte 47
     */
    unsigned char D_8F : 1;
    unsigned char D_8E : 1;
    unsigned char D_8I : 1;
    unsigned char uu12 : 1;
    unsigned char D_7B : 1;
    unsigned char D_7C : 1;
    unsigned char uu13 : 1;
    unsigned char D_7H : 1;

    /*
     * Byte 48
     */
    unsigned char D_7G : 1;
    unsigned char D_7J : 1;
    unsigned char D_7A : 1;
    unsigned char D_7D : 1;
    unsigned char D_7F : 1;
    unsigned char D_7E : 1;
    unsigned char D_7I : 1;
    unsigned char uu14 : 1;

    /*
     * Byte 49
     */
    unsigned char D_6B : 1;
    unsigned char D_6C : 1;
    unsigned char uu15 : 1;
    unsigned char D_6H : 1;
    unsigned char D_6G : 1;
    unsigned char D_6J : 1;
    unsigned char D_6A : 1;
    unsigned char D_6D : 1;

    /*
     * Byte 50
     */
    unsigned char D_6F : 1;
    unsigned char D_6E : 1;
    unsigned char D_6I : 1;
    unsigned char uu16 : 1;
    unsigned char D_5B : 1;
    unsigned char D_5C : 1;
    unsigned char uu17 : 1;
    unsigned char D_5H : 1;

    /*
     * Byte 51
     */
    unsigned char D_5G : 1;
    unsigned char D_5J : 1;
    unsigned char D_5A : 1;
    unsigned char D_5D : 1;
    unsigned char D_5F : 1;
    unsigned char D_5E : 1;
    unsigned char D_5I : 1;
    unsigned char uu18 : 1;

} tBottomRightRowDigits;


/*
 * Bottom Row Units
 */
typedef struct
{
    unsigned char unused1[41];

    unsigned char L : 1;        // G2 (G1-G2 swapped, mistake in table)
    unsigned char T : 1;        // G1 (G1-G2 swapped, mistake in table)
    unsigned char uu2 : 1;
    unsigned char uu3 : 1;
    unsigned char K : 1;        // G3
    unsigned char B : 1;        // G4
    unsigned char P : 1;        // G7
    unsigned char uu4 : 1;

    unsigned char O : 1;        // G6
    unsigned char N : 1;        // G5
    unsigned char G : 1;        // G8
    unsigned char uu5 : 1;
    unsigned char PctSlash : 1;    // G11
    unsigned char PctCircles : 1;  // G9
    unsigned char Z : 1;        // G10
    unsigned char uu6 : 1;      // G20 (not an annunciator)

    unsigned char H : 1;        // G14
    unsigned char S : 1;        // G13
    unsigned char TOT : 1;      // G15
    unsigned char M : 1;      // G12
    unsigned char uu7 : 4;        // G18 (not an annunciator)
} tBottomRowUnits;


/*
 * Misc Segments
 */
typedef struct
{
    unsigned char unused1[42];

    unsigned char uu2 : 7;
    unsigned char Clock : 1;      // G20

    unsigned char uu3 : 7;
    unsigned char Wait45 : 1;        // G18

    unsigned char uu4 : 3;
    unsigned char Wait90 : 1;      // G17
    unsigned char uu5 : 3;
    unsigned char Wait135 : 1;     // G16

    unsigned char uu6 : 3;
    unsigned char Wait0 : 1;         // G19
    unsigned char BatMidBot : 1;     // G24
    unsigned char BatMidTop : 1;     // G23
    unsigned char BatBot : 1;         // G25
    unsigned char BatTop : 1;         // G22

    unsigned char uu7 : 3;
    unsigned char Battery : 1;         // G21
    unsigned char uu8 : 4;
} tMiscSegments;


#endif
