/*
 * lcd_local_skt.h
 *
 * Create a LCD available over a socket, so that you can use a viewer to access it.
 * For debugging/testing purposes
 *
 *  Created on: 15/02/2009
 *      Author: ianb
 */

#ifndef LCD_LOCAL_SKT_H_
#define LCD_LOCAL_SKT_H_

#include <pthread.h>
#include <rinhw/ucos_ii.h>
#include <rinhw/rio.h>
#include <rinhw/lcd_generic.h>
#include <rinhw/kbd.h>


/**
 * Function typedef to parse data incoming from the socket
 *
 * @param outsockfd is the socket file descriptor to write any replies to
 * @param sock_data is the data from the socket
 * @param sock_data_length is the number of bytes received for this message so far
 * @param lcd_data is the current LCD data
 * @param lcd_data_length is the length of the LCD data
 *
 * @return true if message has been handled
 */
typedef bool (*t_lcd_local_skt_data_parser)(int outsockfd, const unsigned char *sock_data, size_t sock_data_length, const unsigned char *lcd_data, size_t lcd_data_length, bool *gotkey, union un_key *key);


/** Information about a socket lcd */
struct s_lcd_hw_socket
{
  int port;                     /* Port to listen on, <=0 for driverconfig.xml setting, >0 for actual port */
  int shutdown_pipe_write_fd;   /* Write anything to this fd to shutdown the thread */
  int shutdown_pipe_read_fd;    /* Thread reads from this fd to see if it needs to shutdown */
  pthread_t thread;             /* The thread */
  OS_EVENT *sem_exclusive;      /* Must point to device's exclusive access semaphore (t_io_dev.sem_open_excl) */
  struct t_io_dev *kbddev;      /* Keyboard device to write keystrokes to */
  t_lcd_local_skt_data_parser data_parser;  /*!< Function to parse the incoming data from the socket */
};


/** LCD hardware is a local socket */
int lcd_socket_hardware_init(struct s_lcd_definition *lcd_definition);


/** LCD hardware is a local socket */
void lcd_socket_hardware_destroy(struct s_lcd_definition *lcd_definition);


#endif /* LCD_LOCAL_SKT_H_ */
