/*
 * memorydev_local_file.h
 *
 * Functions for accessing a memory device (like flash or FRAM), emulated as a file.
 */
#ifndef MEMORYDEV_LOCAL_FILE_H_
#define MEMORYDEV_LOCAL_FILE_H_

#include <stdint.h>


/** Memory device mapped to a file device info. Your struct t_io_dev->dev_data must point to this. */
struct sMemoryDeviceFileInfo
{
  char *filename;
  int fd;
  size_t maxsize;
  uint8_t blankfillchar;
};


/** Memory device driver init entry point, setup the hardware and device */
int memorydev_local_file_init( struct t_io_dev *io_dev );


/** Memory device open device call, opens a local file */
int memorydev_local_file_open( struct t_io_dev *io_dev, unsigned short timeout );


/** Memory device close device call, closes a local file */
int memorydev_local_file_close( struct t_io_dev *io_dev );


/** Memory device read device call, reads data from a local file */
int memorydev_local_file_read( struct t_io_dev *io_dev, void *buf, unsigned short len, unsigned short timeout );


/** Memory device write device call, writes data to a local file */
int memorydev_local_file_write( struct t_io_dev *io_dev, const void *buf, unsigned short len, unsigned short timeout );


/** Memory device filelength device call, returns size of memory device */
long memorydev_local_file_filelength( struct t_io_dev *io_dev );


#endif /* MEMORYDEV_LOCAL_FILE_H_ */
