/*
 * endian.h
 *
 *  Created on: 4/11/2008
 *      Author: ianb
 */
#ifndef RINENDIAN_H_
#define RINENDIAN_H_

#if defined(__linux__)
  /* Use the linux definitions */
  #include <endian.h>
  #include <arpa/inet.h>
#else /* if defined(__linux__) */

#if defined(__WIN32__)
/* Winsock  */
  #include <windows.h>
#endif /* #if defined(__WIN32__) */


/** Little endian byte ordering */
#define LITTLE_ENDIAN 1234

/** Big endian byte ordering */
#define BIG_ENDIAN  4321

/** Networks (like TCP/IP) use this endian */
#define NETWORK_ENDIAN BIG_ENDIAN

/** Intel i386 byte ordering */
#if defined(__i386__)
#define BYTE_ORDER LITTLE_ENDIAN
#endif

/** Hitachi H8 byte ordering */
#if defined(__HITACHI__)
#define BYTE_ORDER BIG_ENDIAN
#endif

/** Atmel AVR byte ordering */
#if defined(__AVR__)
#define BYTE_ORDER LITTLE_ENDIAN
#endif

/** TI MSP430 byte ordering */
#if defined(__MSP430__)
#define BYTE_ORDER LITTLE_ENDIAN
#endif

/* Error if no endian defined */
#ifndef BYTE_ORDER
#error "Endian not defined"
#endif


#if ( BYTE_ORDER == LITTLE_ENDIAN )

#ifndef htonl
  /** Byte swapping - host to network long */
  #define htonl(x) ( BYTE_SWAP_LONG( x ) )
#endif /* #ifndef htonl */

#ifndef htons
  /** Byte swapping - host to network short */
  #define htons(x) ( BYTE_SWAP_SHORT( x ) )
#endif /* #ifndef htons */

#ifndef ntohl
  /** Byte swapping - network to host long */
  #define ntohl(x) ( BYTE_SWAP_LONG( x ) )
#endif /* #ifndef ntohl */

#ifndef ntohs
  /** Byte swapping - network to host long */
  #define ntohs(x) ( BYTE_SWAP_SHORT( x ) )
#endif /* #ifndef ntohs */


#elif ( BYTE_ORDER == BIG_ENDIAN )


#ifndef htonl
  /** Byte swapping - host to network long */
  #define htonl(x) (x)
#endif /* #ifndef htonl */

#ifndef htons
  /** Byte swapping - host to network short */
  #define htons(x) (x)
#endif /* #ifndef htons */

#ifndef ntohl
  /** Byte swapping - network to host long */
  #define ntohl(x) (x)
#endif /* #ifndef ntohl */

#ifndef ntohs
  /** Byte swapping - network to host long */
  #define ntohs(x) (x)
#endif /* #ifndef ntohs */


#endif /* #elif ( BYTE_ORDER == BIG_ENDIAN ) */

#endif /* #else -> if defined(__linux__) */


/** Swap bytes in short */
#define BYTE_SWAP_SHORT(A)      ((((unsigned short)(A) & 0xff00) >> 8) | \
                                 (((unsigned short)(A) & 0x00ff) << 8))

/** Swap bytes in long */
#define BYTE_SWAP_LONG(A)       ((((unsigned long)(A) & 0xff000000) >> 24) | \
                                 (((unsigned long)(A) & 0x00ff0000) >> 8)  | \
                                 (((unsigned long)(A) & 0x0000ff00) << 8)  | \
                                 (((unsigned long)(A) & 0x000000ff) << 24))

/** Swap bytes in short */
#define BYTE_SWAP_WORD(A)      BYTE_SWAP_SHORT(A)


/*
 * Macros for network/external number representation conversion.
 */
#define NTOHL(x)  (x) = ntohl((unsigned long)(x))
#define NTOHS(x)  (x) = ntohs((unsigned short)(x))
#define HTONL(x)  (x) = htonl((unsigned long)(x))
#define HTONS(x)  (x) = htons((unsigned short)(x))


#endif /* RINENDIAN_H_ */
