#ifndef SIO_DOT_H
#define SIO_DOT_H 1

#include <rinhw/rio.h>


/*
 * Serial parity settings
 */
#define SIO_PARITY_NONE 0
#define SIO_PARITY_ODD 1
#define SIO_PARITY_EVEN 2

/*
 * Serial data bit settings
 */
#define SIO_DATABITS_EIGHT 8
#define SIO_DATABITS_SEVEN 7

/*
 * Serial stop bit settings
 */
#define SIO_STOPBITS_ONE 1
#define SIO_STOPBITS_TWO 2


/*
 * t_ioctl_siocfg -
 *
 * Serial configuration struct, used with
 * IOCTL_SER_SET_CONFIG, IOCTL_SER_GET_CONFIG
 */
typedef struct
{
    unsigned long baud;
    unsigned char data_bits;
    unsigned char parity_bits;
    unsigned char stop_bits;
    unsigned char use_cts;
    unsigned char use_terminations;
} t_ioctl_siocfg;


/*
 * Serial IOCTL functions
 *
 * int rioctl(struct t_io_dev *io_dev, IOCTL_SER_SET_CONFIG, t_ioctl_siocfg* siocfg);
 * int rioctl(struct t_io_dev *io_dev, IOCTL_SER_GET_CONFIG, t_ioctl_siocfg* siocfg);
 * int rioctl(struct t_io_dev *io_dev, IOCTL_SER_GET_RX_BUF_LEVEL, unsigned short *level);
 * int rioctl(struct t_io_dev *io_dev, IOCTL_SER_GET_TX_BUF_LEVEL, unsigned short *level);
 * int rioctl(struct t_io_dev *io_dev, IOCTL_SER_GET_RX_BUF_SIZE, unsigned short *level);
 * int rioctl(struct t_io_dev *io_dev, IOCTL_SER_GET_TX_BUF_SIZE, unsigned short *level);
 *
 * int rioctl(struct t_io_dev *io_dev, IOCTL_SER_GET_CTS_STATE, unsigned char *cts_state);
 *   CTS state is Clear To Send state (therefore the port is clear to send, cts_state = 1,
 *   otherwise blocking transmission, cts_state = 0). This result is independent of the port
 *   configuration use_cts setting (ie this is the state of the physical pin). If the port
 *   does not support CTS, then the result will be 1, indicating that the port is not blocked.
 *
 * int rioctl(struct t_io_dev *io_dev, IOCTL_SER_GET_RX_ERR_COUNT, unsigned long *rx_err_count);
 *
 * int rioctl(struct t_io_dev *io_dev, IOCTL_SER_SET_RING_NETWORK, unsigned char enable_ring);
 *   This function is only available on the rinblocks. It will return error on the main UARTs
 *   since we don't have the RAM to implement it.
 *
 * int rioctl(struct t_io_dev *io_dev, IOCTL_SER_SET_RS232_NOT_RS422, int rs232_not_rs422);
 *   Switch between RS232 & RS422 on serial ports which support both, otherwise will return error.
 *
 * int rioctl(struct t_io_dev *io_dev, IOCTL_SER_SOCKET_SET_PORT, int port);
 *   Only applies to serial ports bound to TCP sockets. Changes the listen port the socket is bound to.
 *
 * int rioctl(struct t_io_dev *io_dev, IOCTL_SER_SOCKET_GET_PORT, int *port);
 *   Only applies to serial ports bound to TCP sockets. Returns the listen port the socket is bound to.
 *
 * int rioctl(struct t_io_dev *io_dev, IOCTL_SER_FLUSH_TX_BUF);
 *   Flush all data in the transmit buffer.
 *
 * int rioctl(struct t_io_dev *io_dev, IOCTL_SER_FLUSH_RX_BUF);
 *   Flush all data in the receive buffer.
 *
 * int rioctl(struct t_io_dev *io_dev, IOCTL_SER_GET_RTS_STATE, int *state);
 *   Get the state of the RTS pin (state = 1 means asserted (RS232 +ve voltage, TTL 0v))
 *
 * int rioctl(struct t_io_dev *io_dev, IOCTL_SER_SET_RTS_STATE, int state);
 *   Set the state of the RTS pin (state = 1 means asserted (RS232 +ve voltage, TTL 0v))
 *
 * int rioctl(struct t_io_dev *io_dev, IOCTL_SER_GET_DTR_STATE, int *state);
 *   Get the state of the DTR pin (state = 1 means asserted (RS232 +ve voltage, TTL 0v))
 *
 * int rioctl(struct t_io_dev *io_dev, IOCTL_SER_SET_DTR_STATE, int state);
 *   Set the state of the DTR pin (state = 1 means asserted (RS232 +ve voltage, TTL 0v))
 *
 * int rioctl(struct t_io_dev *io_dev, IOCTL_SER_UDP_GET_DEST_SOCKADDR, struct sockaddr_in *);
 *   Get the destination socket address (ip:port) for a udp serial in host endian
 *
 * int rioctl(struct t_io_dev *io_dev, IOCTL_SER_UDP_SET_DEST_SOCKADDR, struct sockaddr_in *);
 *   Set the destination socket address (ip:port) for a udp fixed serial in host endian
 */
#define IOCTL_SER_SET_CONFIG 600
#define IOCTL_SER_GET_CONFIG 601
#define IOCTL_SER_GET_RX_BUF_LEVEL 602
#define IOCTL_SER_GET_TX_BUF_LEVEL 603
#define IOCTL_SER_GET_RX_BUF_SIZE 604
#define IOCTL_SER_GET_TX_BUF_SIZE 605
#define IOCTL_SER_GET_CTS_STATE 606
#define IOCTL_SER_GET_RX_ERR_COUNT 607
#define IOCTL_SER_SET_RING_NETWORK 608
#define IOCTL_SER_SET_RS232_NOT_RS422 609
#define IOCTL_SER_SET_SOCKET_PORT 610
#define IOCTL_SER_GET_SOCKET_PORT 611
#define IOCTL_SER_FLUSH_TX_BUF 612
#define IOCTL_SER_FLUSH_RX_BUF 613
#define IOCTL_SER_GET_RTS_STATE 614
#define IOCTL_SER_SET_RTS_STATE 615
#define IOCTL_SER_GET_DTR_STATE 616
#define IOCTL_SER_SET_DTR_STATE 617
#define IOCTL_SER_UDP_GET_DEST_SOCKADDR 618
#define IOCTL_SER_UDP_SET_DEST_SOCKADDR 619


/*
 * Serial port defaults
 */
#define SIO_DEF_MODE SIO_FDPLX_MODE
#define SIO_DEF_BAUD 57600UL
#define SIO_DEF_DATA_BITS SIO_DATABITS_EIGHT
#define SIO_DEF_PARITY_BITS SIO_PARITY_NONE
#define SIO_DEF_STOP_BITS SIO_STOPBITS_ONE
#define SIO_DEF_USECTS false
#define SIO_DEF_TERMINATIONS false
#define SIO_DEF_RS232_NOT_RS422 true


#endif /* #ifndef SIO_DOT_H */
