/*
 * sio_local_comport.h
 *
 *  Created on: 04/02/2009
 *      Author: ianb
 */

#ifndef SIO_LOCAL_COMPORT_H_
#define SIO_LOCAL_COMPORT_H_

#include <stdbool.h>
#include <rinhw/rio.h>


/** RS422/RS485 modes supported by driver */
enum e_sio_local_rs422_mode
{
  slrm_no_rs422,                /*!< No RS422/RS485 on this serial port */
  slrm_rs422_gpio,              /*!< RS422 (transmitter always enabled) controlled by GPIO via callback (sio_set_rs422_transmitenable) */
  slrm_rs422_rts_asserted,      /*!< RS422 (transmitter always enabled) via RTS asserted */
  slrm_rs422_rts_deasserted,    /*!< RS422 (transmitter always enabled) via RTS de-asserted */
  slrm_rs485_rts_asserted,      /*!< RS485 (transmitter enabled to transmit) via RTS asserted. Requires TIOCSRS485 support in the kernel. */
  slrm_rs485_rts_deasserted,    /*!< RS485 (transmitter enabled to transmit) via RTS de-asserted. Requires TIOCSRS485 support in the kernel. */
};


/**
 * Struct containing information for controlling a RS422 serial port
 *
 * Any of the function pointers can be NULL, in which case they won't be called
 */
struct s_sio_local_rs422
{
  void *hw_data;                                                               /*!< Pointer to hardware specific data, passed to the functions in this struct by the driver */
  enum e_sio_local_rs422_mode rs422_mode;                                      /*!< RS422/RS485 operating mode */
  int (*sio_select_rs232_not_rs422)(void *hw_data, int rs232_not_rs422);       /*!< Function to switch from RS232 to RS422 */
  int (*sio_set_rs422_termination)(void *hw_data, int enable);                 /*!< Enable/disable RS422 terminations */
  int (*sio_set_rs422_transmitenable)(void *hw_data, int enable);              /*!< If rts_controls_txenable == 0, then use this function to enable the RS422 transmitter */
};


/**
 * Create a serial port connected to local serial port hardware.
 *
 * @param io_dev is an uninitialised io_dev structure to be fill out as a serial port
 * @param port_name is the serial port device name (eg /dev/ttyS0, COM1)
 * @param rs422_info is control information if this is a RS422 port. Driver takes a local copy of this struct. Can be NULL if not RS422 port
 * @param is_gadget_serial should be set to true if this is a gadget serial port, because we have to do some funny things to deal with these ports
 */
int rio_create_device_sio_localcom(struct t_io_dev *io_dev, const char *port_name, struct s_sio_local_rs422 *rs422_info, bool is_gadget_serial);


#endif /* SIO_LOCAL_COMPORT_H_ */
