/*
 * sio_local_dgram.h
 *
 */

#ifndef SIO_LOCAL_DGRAM_H_
#define SIO_LOCAL_DGRAM_H_

#include <rinhw/rio.h>


/**
 * Create a local datagram serial port, listening on port.
 *
 * This port works by buffering writes until an outgoing_termination_char is found in the write stream or
 * the maximum buffer size is reached, and then all data since the last write (including the termination character)
 * is packaged up into a UDP datagram and sent to the last host that contacted the port
 *
 * @param io_dev is the IO device to be initialised as a datagram serial port
 * @param port is the UDP port to listen for data on
 * @param outgoing_termination_chars is a list of characters that will terminate a write and force the data to be transmitted (not '\0' terminated). Can be NULL, in which case outgoing_buffer_size is used to determine when to send the data.
 * @param num_outgoing_termination_chars is the number of characters in outgoing_termination_chars. Can be 0, in which case outgoing_buffer_size is used to determine when to send the data.
 * @param outgoing_buffer_size is the maximum number of characters to buffer before sending a UDP datagram. Must be > 0.
 */
int rio_create_device_sio_localdgram(struct t_io_dev *io_dev, int port,
                                     const char *outgoing_termination_chars,
                                     int num_outgoing_termination_chars,
                                     int outgoing_buffer_size);


#endif /* SIO_LOCAL_DGRAM_H_ */
