/*
 * sio_local_established_dgram.h
 *
 *  Created on: Sep 25, 2017
 *      Author: ian
 */

#ifndef RINHW_SIO_LOCAL_ESTABLISHED_DGRAM_H_
#define RINHW_SIO_LOCAL_ESTABLISHED_DGRAM_H_

#include <netinet/in.h>
#include <rinhw/rio.h>


/**
 * Create a unbuffered established datagram (UDP) serial port, already bound to sockfd
 *
 * This port works by writing data passed by rwrite() as a UDP datagram and sent to dest
 *
 * @param io_dev is the IO device to be initialised as a datagram serial port
 * @param sockfd is the already bound datagram socket
 * @param dest is the UDP IP/port that data from this device will be read/written from/to.
 *          In host endian.
 *          If NULL, then the device will reply to the last IP/port it received data from.
 *
 */
int rio_create_device_sio_established_dgram_unbuffered(struct t_io_dev *io_dev,
                                                       int sockfd,
                                                       const struct sockaddr_in *dest);


/**
 * Create a buffered established datagram serial port, already bound to sockfd
 *
 * This port works by buffering writes until an outgoing_termination_char is found in the write stream or
 * the maximum buffer size is reached, and then all data since the last write (including the termination character)
 * is packaged up into a UDP datagram and sent to dest
 *
 * @param io_dev is the IO device to be initialised as a datagram serial port
 * @param sockfd is the already bound datagram socket
 * @param dest is the UDP IP/port that data from this device will be read/written from/to.
 *          In host endian.
 *          If NULL, then the device will reply to the last IP/port it received data from.
 * @param outgoing_termination_chars is a list of characters that will terminate a write and force the data to be transmitted (not '\0' terminated). Can be NULL, in which case outgoing_buffer_size is used to determine when to send the data.
 * @param num_outgoing_termination_chars is the number of characters in outgoing_termination_chars. Can be 0, in which case outgoing_buffer_size is used to determine when to send the data.
 * @param outgoing_buffer_size is the maximum number of characters to buffer before sending a UDP datagram. Must be > 0.
 */
int rio_create_device_sio_established_dgram_buffered(struct t_io_dev *io_dev,
                                                     int sockfd,
                                                     const struct sockaddr_in *dest,
                                                     const char *outgoing_termination_chars,
                                                     int num_outgoing_termination_chars,
                                                     int outgoing_buffer_size);


#endif /* RINHW_SIO_LOCAL_ESTABLISHED_DGRAM_H_ */
