/**
 * @file stdrinstrum.h
 *
 * Standard Rinstrum data types
 */
#ifndef STD_RINSTRUM_DOT_H
#define STD_RINSTRUM_DOT_H 1

#if !defined(__HITACHI__)
#include <stdint.h>
#endif


/*
 * Types
 */

#ifndef __cplusplus /* bool is a standard type in c++ */
  /** Boolean data type */
  #if (!defined(__HITACHI__) && !defined(PROD_MATCHPORT_AR) && ! defined(PROD_MATCHPORT_BG_PRO))
    /* Linux, windows, etc */
    #include <stdbool.h>
  #elif (defined(PROD_MATCHPORT_AR) || defined(PROD_MATCHPORT_BG_PRO))
    /* matchport AR */
    #include "evolution_types.h"
  #else
    /* Hitachi */

    /* Define bool type */
    typedef unsigned char bool;

    /*
     * Boolean values
     */

    #ifndef true
    /** Boolean true */
    #define true 1
    #endif

    #ifndef false
    /** Boolean false */
    #define false 0
    #endif

    #ifndef TRUE
    /** Boolean true */
    #define TRUE true
    #endif

    #ifndef FALSE
    /** Boolean false */
    #define FALSE false
    #endif

  #endif
#endif /* #ifndef __cplusplus */


#if defined(__HITACHI__)

/** Unsigned 8-bit data type */
typedef unsigned char int8u;

/** Signed 8-bit data type */
typedef signed char int8s;

/** Unsigned 16-bit data type */
typedef unsigned short int16u;

/** Signed 16-bit data type */
typedef signed short int16s;

/** Unsigned 32-bit data type */
typedef unsigned long int32u;

/** Signed 32-bit data type */
typedef signed long int32s;

/** 32-bit floating point data type */
typedef float fp32;


#else /* #if defined(__HITACHI__) */


/*
 * It's better to let the compiler tell us what size these are
 */


/** Unsigned 8-bit data type */
typedef uint8_t int8u;

/** Signed 8-bit data type */
typedef int8_t int8s;

/** Unsigned 16-bit data type */
typedef uint16_t int16u;

/** Signed 16-bit data type */
typedef int16_t int16s;

/** Unsigned 32-bit data type */
typedef uint32_t int32u;

/** Signed 32-bit data type */
typedef int32_t int32s;

/** 32-bit floating point data type */
typedef float fp32;


#endif /* #else -> #if defined(__HITACHI__) */


/*
 * Limits
 */

/** Upper limit of @ref int8s */
#define INT8S_MAX  127

/** Lower limit of @ref int8s */
#define INT8S_MIN  -128

/** Upper limit of @ref int8u */
#define INT8U_MAX  255

/** Lower limit of @ref int8u */
#define INT8U_MIN  0

/** Upper limit of @ref int16s */
#define INT16S_MAX 32767

/** Lower limit of @ref int16s */
#define INT16S_MIN -32768

/** Upper limit of @ref int16u */
#define INT16U_MAX 65535

/** Lower limit of @ref int16u */
#define INT16U_MIN 0

/** Upper limit of @ref int32s */
#define INT32S_MAX 2147483647

/** Lower limit of @ref int32s */
#define INT32S_MIN -2147483648

/** Upper limit of @ref int32u */
#define INT32U_MAX 4294967295

/** Lower limit of @ref int32u */
#define INT32U_MIN 0


/** Method for stopping warnings for a parameter passed to a function but not used */
#ifndef UNREFERENCED_PARAMETER
#if defined(__HITACHI__)
  #define UNREFERENCED_PARAMETER(nonUsedParameter)
#elif defined(__GNUC__)
  #define UNREFERENCED_PARAMETER(nonUsedParameter) nonUsedParameter = nonUsedParameter
#endif
#endif /* #ifndef UNREFERENCED_PARAMETER */


#endif /* #ifndef STD_RINSTRUM_DOT_H */
