#ifndef WDT_DOT_H
#define WDT_DOT_H 1


/*
 * Struct for a watchdog
 * You should define one of these for each task you want a watchdog for, and
 * pass it to the watch dog functions WatchdogAdd, and WatchdogPreset
 */
struct t_wdt
{
    unsigned short count;
    struct t_wdt *next;
};


/*
 * WatchdogClear -
 *
 * Internal function to physically clear the wdt hardware. System specific, so there
 * are conditional defines for the R400 and R500. Do not call, unless you know what
 * you are doing, since doing so will prevent a system reset in the event of a crash
 */
void WatchdogClear(void);


/*
 * WatchdogAdd -
 *
 * Add a new watchdog to the system. The struct new_wdt must be defined by the caller
 * and should not ever be released, as there is no way to delete a watchdog once it has been added
 * Once added, WatchdogPreset must be called at intevals less that ticks to stop the system
 * resetting
 */
void WatchdogAdd(struct t_wdt *new_wdt);

/*
 * WatchdogPreset -
 *
 * Preset a watchdog with a number of system ticks to keep running for.
 * Should be called in a task, so that if the task stops responding, then the
 * system will reset.
 */
void WatchdogPreset(struct t_wdt *wdt, unsigned short ticks);


/*
 * WatchdogInit -
 *
 * Initialise the watchdog proper. Sets up a timer at regular intevals
 * to toggle the watchdog as required.
 * Parameter data is unused, and just for compatibility with init
 */
void WatchdogInit(void *data);


/** Shut down the watchdog */
void WatchdogDestroy(void);


#endif
