/*
 * rio_default.c
 *
 *  Created on: 04/02/2009
 *      Author: ianb
 */
#include <rinhw/debug.h>
#include <rinhw/rio.h>


/** Standard do nothing open device */
int default_open(struct t_io_dev *io_dev, unsigned short timeout)
{
#if HW_DEBUG(RIO_DEBUG, HW_DEBUG_BASIC)
    DEBUG_PRINTF("default_open called\n");
#endif
    return RIO_SUCCESS;
}


/*
 * default_close -
 *
 * Standard do nothing close device
 */
int default_close(struct t_io_dev *io_dev)
{
#if HW_DEBUG(RIO_DEBUG, HW_DEBUG_BASIC)
    DEBUG_PRINTF("default_close called\n");
#endif
    return RIO_SUCCESS;
}


/*
 * default_ioctl -
 *
 * IOCTL function that returns RIO_ERROR for devices with
 * nothing to configure
 */
int default_ioctl(struct t_io_dev *io_dev, int func, va_list *arglist)
{
#if HW_DEBUG(RIO_DEBUG, HW_DEBUG_BASIC)
  DEBUG_PRINTF("default_ioctl called\n");
#endif
  return RIO_ERROR;
}


/** Returns whether the device is a character device (serialport, etc) */
int default_isatty(struct t_io_dev *io_dev)
{
  int result;

#if HW_DEBUG(RIO_DEBUG, HW_DEBUG_BASIC)
  DEBUG_PRINTF("default_isatty called\n");
#endif
  result = ( ( io_dev->status & O_DEVCHAR ) != 0 ) ? RIO_SUCCESS : 0;
  return result;
}


/** Current position in device, returns 0 for character devices */
long default_tell(struct t_io_dev *io_dev)
{
  long result;

  result = ( (risatty( io_dev ) ) ? 0L : io_dev->posn );
  return result;
}


/*
 * rlseek -
 *
 * Seek to a position within a device.
 *    fromwhere is one of SEEK_SET, SEEK_CUR, SEEK_END
 *    offset is a relative offset (in bytes) from the fromwhere position
 *
 * Result is -1 on error, new position in file otherwise
 *
 * Only available on block devices
 */
long default_lseek(struct t_io_dev *io_dev, long offset, int fromwhere)
{
  long new_posn;
  long result;

  if (risatty(io_dev))
  {
    /* Error if trying to seek in a character device */
    result = (long) RIO_ERROR;
  }
  else
  {
    /* Block device, figure out where to seek from */
    switch (fromwhere)
    {
      case SEEK_CUR:
        new_posn = io_dev->posn;
        break;
      case SEEK_SET:
        new_posn = 0;
        break;
      case SEEK_END:
        new_posn = rfilelength(io_dev);
        break;
      default:
        return (long) RIO_ERROR;
    }

    /* Calculate the new position, check that its in range */
    new_posn += offset;
    if ((new_posn < 0) || (new_posn > rfilelength(io_dev)))
      return (long) RIO_ERROR;

    /* Update the IOB with the new position */
    result = io_dev->posn = new_posn;
  }

  return result;
}
