#include <stdio.h>
#include <stdlib.h>
#include <rinhw/rio.h>
#include <rinhw/rprintf.h>
#include <rinhw/rstdio.h>


/*
 * rvfprintf -
 *
 * Rinstrum version of variable argument file printf. Actually takes
 * out the whole fxxx (eg fwrite) function layer of normal C, and just
 * calls write functions directly, due to code/ram constraints
 *
 * Uses internal_printf for doing the work
 */
int rvfprintf(struct t_io_dev *stream, const char *format, va_list args)
{
  int nLength;
  int nResult = RIO_ERROR;
  char *buffer;
  va_list argcopy;

  /* Work out length*/
  va_copy(argcopy, args);
  nLength = vsnprintf(NULL, 0, format, args);
  if (nLength >= 0)
  {
    /* Allocate buffer of required length */
    buffer = malloc(nLength + 1);
    if (NULL != buffer)
    {
      /* Format string and write out */
      nLength = vsnprintf(buffer, nLength + 1, format, argcopy);
      nResult = rwrite(stream, buffer, nLength, 0);
      free(buffer);
    }
  }
  va_end(argcopy);
  return nResult;
}


/*
 * rfprintf -
 *
 * Rinstrum version of file printf. Actually takes
 * out the whole fxxx (eg fwrite) function layer of normal C, and just
 * calls write functions directly, due to code/ram constraints
 *
 * Uses internal_printf for doing the work
 */
int rfprintf(struct t_io_dev *stream, const char *format, ...)
{
    va_list args;
    int length;

    va_start(args, format);
    length = rvfprintf(stream, format, args);
    va_end(args);
    return length;
}
