/*
 * abstractserial.c
 *
 *  Created on: 05/08/2009
 *      Author: ian
 */
#include "abstractserial.h"


/** Initialise the serial port structure with the port name. Doesn't open port. */
bool AbstractSerialInit( struct sAbstractSerialPort *pSerialPort, const char *pszPortName )
{
#if defined(__linux__)
  return LinuxSerialInit( pSerialPort, pszPortName );
#elif defined(WIN32)
  return WinSerialInit( pSerialPort, pszPortName );
#endif
}


/** Close port if open and clean up allocated memory in struct */
void AbstractSerialShutdown( struct sAbstractSerialPort *pSerialPort )
{
#if defined(__linux__)
  LinuxSerialShutdown( pSerialPort );
#elif defined(WIN32)
  WinSerialShutdown( pSerialPort );
#endif
}


/** Open the serial port */
bool AbstractSerialOpen( struct sAbstractSerialPort *pSerialPort )
{
#if defined(__linux__)
  return LinuxSerialOpen( pSerialPort );
#elif defined(WIN32)
  return WinSerialOpen( pSerialPort );
#endif
}


/** Close the serial port */
bool AbstractSerialClose( struct sAbstractSerialPort *pSerialPort )
{
#if defined(__linux__)
  return LinuxSerialClose( pSerialPort );
#elif defined(WIN32)
  return WinSerialClose( pSerialPort );
#endif
}


/** Check if serial port is open */
bool AbstractSerialIsOpen(struct sAbstractSerialPort *pSerialPort, bool *pbIsOpen)
{
#if defined(__linux__)
  return LinuxSerialIsOpen( pSerialPort, pbIsOpen );
#elif defined(WIN32)
  return WinSerialIsOpen( pSerialPort, pbIsOpen );
#endif
}


/** Set the write timeout, 0 for indefinate */
bool AbstractSerialSetWriteTimeout( struct sAbstractSerialPort *pSerialPort, int32u ui32MilliSeconds )
{
#if defined(__linux__)
  return LinuxSerialSetWriteTimeout( pSerialPort, ui32MilliSeconds );
#elif defined(WIN32)
  return WinSerialSetWriteTimeout( pSerialPort, ui32MilliSeconds );
#endif
}


/** Set the read timeout, 0 for indefinate */
bool AbstractSerialSetReadTimeout( struct sAbstractSerialPort *pSerialPort, int32u ui32MilliSeconds )
{
#if defined(__linux__)
  return LinuxSerialSetReadTimeout( pSerialPort, ui32MilliSeconds );
#elif defined(WIN32)
  return WinSerialSetReadTimeout( pSerialPort, ui32MilliSeconds );
#endif
}


/** Get the configuration for the serial port */
bool AbstractSerialGetConfig( struct sAbstractSerialPort *pSerialPort, int32u *pui32BaudRate,
                              int8u *pDataBits, int8u *pParity, int8u *pStopBits, bool *pUseCts )
{
#if defined(__linux__)
  return LinuxSerialGetConfig( pSerialPort, pui32BaudRate, pDataBits, pParity, pStopBits, pUseCts );
#elif defined(WIN32)
  return WinSerialGetConfig( pSerialPort, pui32BaudRate, pDataBits, pParity, pStopBits, pUseCts );
#endif
}


/** Set the configuration for the serial port */
bool AbstractSerialSetConfig( struct sAbstractSerialPort *pSerialPort, int32u ui32BaudRate,
                              int8u DataBits, int8u Parity, int8u StopBits, bool UseCts )
{
#if defined(__linux__)
  return LinuxSerialSetConfig( pSerialPort, ui32BaudRate, DataBits, Parity, StopBits, UseCts );
#elif defined(WIN32)
  return WinSerialSetConfig( pSerialPort, ui32BaudRate, DataBits, Parity, StopBits, UseCts );
#endif
}


/** Read data from the serial port */
enum eAbstractSerialResult AbstractSerialRead( struct sAbstractSerialPort *pSerialPort, int8u *pui8Buf, int32u ui32Size )
{
#if defined(__linux__)
  return LinuxSerialRead( pSerialPort, pui8Buf, ui32Size );
#elif defined(WIN32)
  return WinSerialRead( pSerialPort, pui8Buf, ui32Size );
#endif
}


/** Write data to the serial port */
enum eAbstractSerialResult AbstractSerialWrite( struct sAbstractSerialPort *pSerialPort, const int8u *pui8Buf, int32u ui32Size )
{
#if defined(__linux__)
  return LinuxSerialWrite( pSerialPort, pui8Buf, ui32Size );
#elif defined(WIN32)
  return WinSerialWrite( pSerialPort, pui8Buf, ui32Size );
#endif
}


/** Flush the serial port buffers. Can flush input, output or both buffers */
bool AbstractSerialFlush( struct sAbstractSerialPort *pSerialPort, enum eAbstractSerialFlush flush )
{
#if defined(__linux__)
  return LinuxSerialFlush( pSerialPort, flush );
#elif defined(WIN32)
  int nFlushWhat = 0;
  switch ( flush )
  {
    case FlushInput:
      nFlushWhat = WINSERIAL_FLUSH_INPUT;
      break;
    case FlushOutput:
      nFlushWhat = WINSERIAL_FLUSH_OUTPUT;
      break;
    case FlushInOut:
      nFlushWhat = WINSERIAL_FLUSH_INOUT;
      break;
  }
  return WinSerialFlush( pSerialPort, nFlushWhat );
#endif
}


/** Get the number of bytes available in the transmit buffer */
bool AbstractSerialGetTxBufferLevel( struct sAbstractSerialPort *pSerialPort, int32u *pui32Level )
{
#if defined(__linux__)
  return LinuxSerialGetTxBufferLevel( pSerialPort, pui32Level );
#elif defined(WIN32)
  return WinSerialGetTxBufferLevel( pSerialPort, pui32Level );
#endif
}


/** Get the size in bytes of the transmit buffer */
bool AbstractSerialGetTxBufferSize( struct sAbstractSerialPort *pSerialPort, int32u *pui32Level )
{
#if defined(__linux__)
  return LinuxSerialGetTxBufferSize( pSerialPort, pui32Level );
#elif defined(WIN32)
  return WinSerialGetTxBufferSize( pSerialPort, pui32Level );
#endif
}


/** Get the number of bytes available in the receive buffer */
bool AbstractSerialGetRxBufferLevel( struct sAbstractSerialPort *pSerialPort, int32u *pui32Level )
{
#if defined(__linux__)
  return LinuxSerialGetRxBufferLevel( pSerialPort, pui32Level );
#elif defined(WIN32)
  return WinSerialGetRxBufferLevel( pSerialPort, pui32Level );
#endif
}


/** Get the size in bytes of the receive buffer */
bool AbstractSerialGetRxBufferSize( struct sAbstractSerialPort *pSerialPort, int32u *pui32Level )
{
#if defined(__linux__)
  return LinuxSerialGetRxBufferSize( pSerialPort, pui32Level );
#elif defined(WIN32)
  return WinSerialGetRxBufferSize( pSerialPort, pui32Level );
#endif
}


/**
 * Get the state of the CTS pin (regardless of whether CTS blocking is enabled).
 * Returns 1 == Clear to send data, 0 == blocking
 */
bool AbstractSerialGetCtsState( struct sAbstractSerialPort *pSerialPort, bool *pbCts )
{
#if defined(__linux__)
  return LinuxSerialGetCtsState( pSerialPort, pbCts );
#elif defined(WIN32)
  return WinSerialGetCtsState( pSerialPort, pbCts );
#endif
}


/**
 * Get the state of the RTS pin
 *
 * @param pbRts set to true if RTS asserted (RS232 +ve, TTL 0v), false otherwise
 */
bool AbstractSerialGetRtsState( struct sAbstractSerialPort *pSerialPort, bool *pbRts )
{
#if defined(__linux__)
  return LinuxSerialGetRtsState( pSerialPort, pbRts );
#elif defined(WIN32)
  return WinSerialGetRtsState( pSerialPort, pbRts ); /* TODO */
#endif
}


/**
 * Get the state of the DTR pin
 *
 * @param pbDtr set to true if DTR asserted (RS232 +ve, TTL 0v), false otherwise
 */
bool AbstractSerialGetDtrState( struct sAbstractSerialPort *pSerialPort, bool *pbDtr )
{
#if defined(__linux__)
  return LinuxSerialGetDtrState( pSerialPort, pbDtr );
#elif defined(WIN32)
  return WinSerialGetDtrState( pSerialPort, pbDtr ); /* TODO */
#endif
}


/**
 * Set the state of the RTS pin
 *
 * @param bRts true to assert RTS (RS232 +ve, TTL 0v), false otherwise
 */
bool AbstractSerialSetRtsState( struct sAbstractSerialPort *pSerialPort, bool bRts )
{
#if defined(__linux__)
  return LinuxSerialSetRtsState( pSerialPort, bRts );
#elif defined(WIN32)
  return WinSerialSetRtsState( pSerialPort, bRts ); /* TODO */
#endif
}


/**
 * Set the state of the DTR pin
 *
 * @param bDts true to assert DTR (RS232 +ve, TTL 0v), false otherwise
 */
bool AbstractSerialSetDtrState( struct sAbstractSerialPort *pSerialPort, bool bDtr )
{
#if defined(__linux__)
  return LinuxSerialSetDtrState( pSerialPort, bDtr );
#elif defined(WIN32)
  return WinSerialSetDtrState( pSerialPort, bDtr ); /* TODO */
#endif
}


/**
 * Enable the use of the kernel RS485 transmitter enable using RTS support via TIOCSRS485
 *
 * @param pSerialPort is the serial port
 * @param enable is used to enable or disable RS485 transmitter enable using RTS
 * @param assert_on_transmit will assert RTS while transmitting if set, otherwise will de-assert RTS while transmitting
 */
bool AbstractSerialSetRS485Rts(struct sLinuxSerialPort *pSerialPort, bool enable, bool assert_on_transmit)
{
#if defined(__linux__)
  return LinuxSerialSetRS485Rts(pSerialPort, enable, assert_on_transmit);
#elif defined(WIN32)
  return WinSerialSetRS485Rts(pSerialPort, enable, assert_on_transmit); /* TODO */
#endif
}
