/*
 * abstractserial.h
 *
 *  Created on: 05/08/2009
 *      Author: ian
 */
#ifndef ABSTRACTSERIAL_H_
#define ABSTRACTSERIAL_H_

#include <rinhw/stdrinstrum.h>
#if defined(__linux__)
  #include "linuxserial.h"
  #define sAbstractSerialPort sLinuxSerialPort
#elif defined(WIN32)
  #include "winserial.h"
  #define sAbstractSerialPort sWinSerialPort
#endif
#include <rinhw/sio.h>


/** Options for flushing buffers */
enum eAbstractSerialFlush
{
  FlushInput,
  FlushOutput,
  FlushInOut
};


/** Initialise the serial port structure with the port name. Doesn't open port. */
bool AbstractSerialInit( struct sAbstractSerialPort *pSerialPort, const char *pszPortName );


/** Close port if open and clean up allocated memory in struct */
void AbstractSerialShutdown( struct sAbstractSerialPort *pSerialPort );


/** Open the serial port */
bool AbstractSerialOpen( struct sAbstractSerialPort *pSerialPort );


/** Close the serial port */
bool AbstractSerialClose( struct sAbstractSerialPort *pSerialPort );


/** Check if serial port is open */
bool AbstractSerialIsOpen(struct sAbstractSerialPort *pSerialPort, bool *pbIsOpen);


/** Set the write timeout, 0 for indefinate */
bool AbstractSerialSetWriteTimeout( struct sAbstractSerialPort *pSerialPort, int32u ui32MilliSeconds );


/** Set the read timeout, 0 for indefinate */
bool AbstractSerialSetReadTimeout( struct sAbstractSerialPort *pSerialPort, int32u ui32MilliSeconds );


/** Get the configuration for the serial port. Settings as per SIO_ constants in sio.h */
bool AbstractSerialGetConfig( struct sAbstractSerialPort *pSerialPort, int32u *pui32BaudRate,
                              int8u *pDataBits, int8u *pParity, int8u *pStopBits, bool *pUseCts );


/** Set the configuration for the serial port. Settings as per SIO_ constants in sio.h */
bool AbstractSerialSetConfig( struct sAbstractSerialPort *pSerialPort, int32u ui32BaudRate,
                              int8u DataBits, int8u Parity, int8u StopBits, bool UseCts );


/** Read data from the serial port */
enum eAbstractSerialResult AbstractSerialRead( struct sAbstractSerialPort *pSerialPort, int8u *pui8Buf, int32u ui32Size );


/** Write data to the serial port */
enum eAbstractSerialResult AbstractSerialWrite( struct sAbstractSerialPort *pSerialPort, const int8u *pui8Buf, int32u ui32Size );


/** Flush the serial port buffers. Can flush input, output or both buffers */
bool AbstractSerialFlush( struct sAbstractSerialPort *pSerialPort, enum eAbstractSerialFlush flush );


/** Get the number of bytes available in the transmit buffer */
bool AbstractSerialGetTxBufferLevel( struct sAbstractSerialPort *pSerialPort, int32u *pui32Level );


/** Get the size in bytes of the transmit buffer */
bool AbstractSerialGetTxBufferSize( struct sAbstractSerialPort *pSerialPort, int32u *pui32Level );


/** Get the number of bytes available in the receive buffer */
bool AbstractSerialGetRxBufferLevel( struct sAbstractSerialPort *pSerialPort, int32u *pui32Level );


/** Get the size in bytes of the receive buffer */
bool AbstractSerialGetRxBufferSize( struct sAbstractSerialPort *pSerialPort, int32u *pui32Level );


/**
 * Get the state of the CTS pin (regardless of whether CTS blocking is enabled).
 * Returns 1 == Clear to send data, 0 == blocking
 */
bool AbstractSerialGetCtsState( struct sAbstractSerialPort *pSerialPort, bool *pbCts );


/**
 * Get the state of the RTS pin
 *
 * @param pbRts set to true if RTS asserted (RS232 +ve, TTL 0v), false otherwise
 */
bool AbstractSerialGetRtsState( struct sAbstractSerialPort *pSerialPort, bool *pbRts );


/**
 * Get the state of the DTR pin
 *
 * @param pbDtr set to true if DTR asserted (RS232 +ve, TTL 0v), false otherwise
 */
bool AbstractSerialGetDtrState( struct sAbstractSerialPort *pSerialPort, bool *pbDtr );


/**
 * Set the state of the RTS pin
 *
 * @param bRts true to assert RTS (RS232 +ve, TTL 0v), false otherwise
 */
bool AbstractSerialSetRtsState( struct sAbstractSerialPort *pSerialPort, bool bRts );


/**
 * Set the state of the DTR pin
 *
 * @param bDts true to assert DTR (RS232 +ve, TTL 0v), false otherwise
 */
bool AbstractSerialSetDtrState( struct sAbstractSerialPort *pSerialPort, bool bDtr );


/**
 * Enable the use of the kernel RS485 transmitter enable using RTS support via TIOCSRS485
 *
 * @param pSerialPort is the serial port
 * @param enable is used to enable or disable RS485 transmitter enable using RTS
 * @param assert_on_transmit will assert RTS while transmitting if set, otherwise will de-assert RTS while transmitting
 */
bool AbstractSerialSetRS485Rts(struct sLinuxSerialPort *pSerialPort, bool enable, bool assert_on_transmit);


#endif /* ABSTRACTSERIAL_H_ */
