/*
 * winserial.h
 *
 *  Created on: 23/10/2008
 *      Author: ianb
 */
#ifndef WINSERIAL_H_
#define WINSERIAL_H_

#include <windows.h>
#include <rinhw/stdrinstrum.h>


/** Result of a read operation */
enum eAbstractSerialResult { asrSuccess = 0, asrTimeout, asrError, asrEndOfFile };


struct sWinSerialPort
{
  char *m_pszPortName;
  HANDLE m_hSerialComm;
};


/** Flush the input buffer */
#define WINSERIAL_FLUSH_INPUT 0

/** Flush the output buffer */
#define WINSERIAL_FLUSH_OUTPUT 1

/** Flush both input and output buffers */
#define WINSERIAL_FLUSH_INOUT 2


/** Initialise the serial port structure with the port name. Doesn't open port. */
bool WinSerialInit( struct sWinSerialPort *pWinSerialPort, const char *pszPortName );


/** Close port if open and clean up allocated memory in struct */
void WinSerialShutdown( struct sWinSerialPort *pWinSerialPort );


/** Open the serial port */
bool WinSerialOpen( struct sWinSerialPort *pWinSerialPort );


/** Close the serial port */
bool WinSerialClose( struct sWinSerialPort *pWinSerialPort );


/** Set the write timeout, 0 for indefinite */
bool WinSerialSetWriteTimeout( struct sWinSerialPort *pWinSerialPort, int32u ui32MilliSeconds );


/** Set the read timeout, 0 for indefinite */
bool WinSerialSetReadTimeout( struct sWinSerialPort *pWinSerialPort, int32u ui32MilliSeconds );


/** Get the configuration for the serial port */
bool WinSerialGetConfig( struct sWinSerialPort *pWinSerialPort, int32u *pui32BaudRate,
                           int8u *pDataBits, int8u *pParity, int8u *pStopBits, bool *pUseCts );


/** Set the configuration for the serial port */
bool WinSerialSetConfig( struct sWinSerialPort *pWinSerialPort, int32u ui32BaudRate,
                           int8u DataBits, int8u Parity, int8u StopBits, bool UseCts );


/** Read data from the serial port */
enum eAbstractSerialResult WinSerialRead( struct sWinSerialPort *pWinSerialPort, int8u *pui8Buf, int32u ui32Size );


/** Write data to the serial port */
enum eAbstractSerialResult WinSerialWrite( struct sWinSerialPort *pWinSerialPort, const int8u *pui8Buf, int32u ui32Size );


/** Flush the serial port buffers. queue_selector is one of WINSERIAL_FLUSH_INPUT, WINSERIAL_FLUSH_OUTPUT or WINSERIAL_FLUSH_INOUT */
bool WinSerialFlush( struct sWinSerialPort *pWinSerialPort, int nWhichBuffers );


/** Get the number of bytes available in the transmit buffer */
bool WinSerialGetTxBufferLevel( struct sWinSerialPort *pWinSerialPort, int32u *pui32Level );


/** Get the size in bytes of the transmit buffer */
bool WinSerialGetTxBufferSize( struct sWinSerialPort *pWinSerialPort, int32u *pui32Level );


/** Get the number of bytes available in the receive buffer */
bool WinSerialGetRxBufferLevel( struct sWinSerialPort *pWinSerialPort, int32u *pdwLevel );


/** Get the size in bytes of the receive buffer */
bool WinSerialGetRxBufferSize( struct sWinSerialPort *pWinSerialPort, int32u *pui32Level );


/**
 * Get the state of the CTS pin (regardless of whether CTS blocking is enabled).
 * Returns 1 == Clear to send data, 0 == blocking
 */
bool WinSerialGetCtsState( struct sWinSerialPort *pWinSerialPort, bool *pbCts );


#endif /* WINSERIAL_H_ */
