#include <rinhw/rstdio.h>
#include <rinhw/shutdown.h>
#include "_shutdown.h"
#include <rinhw/debug.h>

/*
 * This is a pointer that will maintain the linked list of functions to call
 */
struct t_shutdown_fn *shutdown_fn_list = (struct t_shutdown_fn *) NULL;


/*
 * ShutdownAddFunction -
 *
 * Add a new shutdown function to call to the list of functions
 * Parameter new_shutdown_fn must have the function filled out
 */
void ShutdownAddFunction(struct t_shutdown_fn *new_shutdown_fn)
{
    struct t_shutdown_fn *shutdown_fn;
//    OS_DECLARE_CPU_SR();        /* Must be last variable */

    /*
     * Don't want to get interrupted while stuffing round with the list
     */
//    OS_ENTER_CRITICAL();

    /*
     * Its the first one ever, just point the list pointer to it
     */
    if (shutdown_fn_list == NULL)
    {
        shutdown_fn_list = new_shutdown_fn;
    }
    else
    {

        /*
         * There is 1 or more already, navigate the list to find the end
         */
        shutdown_fn = shutdown_fn_list;
        while (shutdown_fn->next != NULL)
        {
            shutdown_fn = shutdown_fn->next;
        }

        /*
         * Attach it to the end
         */
        shutdown_fn->next = new_shutdown_fn;
    }

    /*
     * Make sure the new functions next pointer is null to mark the new end of the list
     * Out of critical section
     */
    new_shutdown_fn->next = (struct t_shutdown_fn *) NULL;
//    OS_EXIT_CRITICAL();
}


/*
 * rshutdown -
 *
 * Run through all the system shutdown functions and call them
 */
void rshutdown(void)
{
    struct t_shutdown_fn *shutdown_fn = shutdown_fn_list;

#if HW_DEBUG(SHUTDOWN_DEBUG, HW_DEBUG_BASIC)
    DEBUG_PRINTF("system shutdown: calling shutdown functions\n");
#endif

    /*
     * For each element in the list
     */
    while (shutdown_fn != NULL)
    {

        /*
         * Call the shutdown function if it is not NULL
         */
#if HW_DEBUG(SHUTDOWN_DEBUG, HW_DEBUG_BASIC)
        DEBUG_PRINTF("system shutdown: shutdown function @ %p\n", shutdown_fn->fn);
#endif
        if (shutdown_fn->fn != NULL)
            shutdown_fn->fn();

        /*
         * Onto the next one in the list
         */
        shutdown_fn = shutdown_fn->next;
    }
}
