#include <rinhw/rstdio.h>
#include <rinhw/ucos_ii.h>
#include <rinhw/tick.h>

#include <rinhw/wdt.h>


/*
 * A pointer to the first element in the WDT list
 */
static struct t_wdt *wdt_list = (struct t_wdt *) NULL;


/*
 * WatchdogClear -
 *
 * Internal function to physically clear the wdt hardware. System specific, so there
 * are conditional defines for the R400 and R500. Do not call, unless you know what
 * you are doing, since doing so will prevent a system reset in the event of a crash
 */
void WatchdogClear(void)
{
  /* TODO */
}


/*
 * WatchdogAdd -
 *
 * Add a new watchdog to the system. The struct new_wdt must be defined by the caller
 * and should not ever be released, as there is no way to delete a watchdog once it has been added
 * Once added, WatchdogPreset must be called at intevals less that ticks to stop the system
 * resetting
 */
void WatchdogAdd(struct t_wdt *new_wdt)
{
    struct t_wdt *wdt, *wdt_prev;
//    OS_CPU_SR cpu_sr;      /* Must be last variable */

//    OS_ENTER_CRITICAL();
    if (wdt_list == NULL)
    {
        wdt_list = new_wdt;
    } else
    {
        wdt_prev = wdt = wdt_list;
        while (wdt != NULL)
        {
            wdt_prev = wdt;
            wdt = wdt->next;
        }
        wdt_prev->next = new_wdt;
    }
    new_wdt->next = (struct t_wdt *) NULL;
//    OS_EXIT_CRITICAL();
}


/*
 * WatchdogPreset -
 *
 * Preset a watchdog with a number of system ticks to keep running for.
 * Should be called in a task, so that if the task stops responding, then the
 * system will reset.
 */
void WatchdogPreset(struct t_wdt *wdt, unsigned short ticks)
{
    wdt->count = ticks;
}

#if 0
/*
 * WatchdogToggle -
 *
 * Called at the system tick frequency to check each of the watchdogs in the list.
 * If they all have a count greater than 0 then the watchdog gets cleared, otherwise
 * the watchdog does not get cleared, and the system will eventually reset.
 * If the count is greater than 0, then the count for each watchdog in the list
 * is decremented by 1 for each call to this function.
 *
 * If there are no watchdogs in the list, then the system clears the hardware watchdog, so
 * the system will never reset.
 */
static void WatchdogToggle(void *data)
{
    struct t_wdt *wdt = wdt_list;
    BOOLEAN timed_out = FALSE;

    if (wdt == NULL)
    {
        WatchdogClear();
    } else
    {
        while (wdt != NULL)
        {
            if (wdt->count == 0)
                timed_out = TRUE;
            else
                wdt->count--;
            wdt = wdt->next;
        }
        if (!timed_out)
            WatchdogClear();
    }
}
#endif


/*
 * early_init_wdt -
 *
 * Specific initialisation of the WDT
 * This is called in the PowerON_Reset function, and requires the stack
 * to be located in the internal RAM, since the external RAM may not be up and running yet (depending on whether
 * early_init_dmt() is called before or after early_init_ram()
 */
void early_init_wdt(void)
{
    /* TODO */
    WatchdogClear();
}

#if 0
static struct t_tick_fn wdt_tick =
{
    1,                  /* unsigned short preset; (at os tick freq) */
    0,                  /* unsigned short count; */
    WatchdogToggle,     /* void (*fn)(void *); */
    (void *) 0,         /* void *data */
    NULL                /* struct t_tick_fn *next; */
};
#endif

/*
 * WatchdogInit -
 *
 * Initialise the watchdog proper. Sets up a timer at regular intevals
 * to toggle the watchdog as required.
 * Parameter data is unused, and just for compatibility with init
 */
void WatchdogInit(void *data)
{
/* TODO watchdog     TickAddFunction(&wdt_tick); */
}


/** Shut down the watchdog */
void WatchdogDestroy(void)
{
  /* TODO watchdog shutdown */
}
