#include <stdio.h>
#include <rinhw/hwlib_init.h>
#include <rinhw/we21hw.h>
#include <rinhw/rio.h>
#include <rinhw/sio_local_console.h>
#include <rinhw/iic_local_linux.h>
#include <rinhw/adc_ads1216_local.h>
#include <rinhw/adc_ads1216_we21.h>
#include <rinhw/rtc_local.h>
#include <rinhw/sio.h>
#include <rinhw/sio_local_comport.h>
#include <rinhw/sio_local_dgram.h>
#include <rinhw/sio_local_socket.h>
#include <rinhw/framdev_local.h>
#include <rinhw/sio_local_pipe.h>

#include "devices.h"

struct t_io_dev dev_console;
struct t_io_dev dev_iic0;
struct t_io_dev dev_adc24bit;
struct t_io_dev dev_rtc;
struct t_io_dev dev_fram;
struct t_io_dev dev_ser1;
struct t_io_dev dev_ser2;
struct t_io_dev dev_udp_datagram;
struct t_io_dev dev_tcp_stream;
struct t_io_dev dev_pipe1;
struct t_io_dev dev_pipe2;

struct t_io_dev dev_usb_serial[MAX_DEV_USB_SERIAL];


static void init_usb_serials(void)
{
  #define DEVNAMELEN 40
  char devname[DEVNAMELEN];
  int i;

  for (i = 0; i < MAX_DEV_USB_SERIAL; i++)
  {
    snprintf(devname, DEVNAMELEN, "/dev/ttyUSB%u", i);
    if (rio_create_device_sio_localcom(&dev_usb_serial[i], devname, NULL, false) < 0)
    {
      /* fprintf(stderr, "Failed create device for %s\n", devname); */
      dev_usb_serial[i].status &= ~O_INITIALISED;
    }
  }
}


static void destroy_usb_serials(void)
{
  int i;

  for (i = 0; i < MAX_DEV_USB_SERIAL; i++)
  {
    if (0 != (dev_usb_serial[i].status & O_INITIALISED))
      rdestroy_io(&dev_usb_serial[i]);
  }
}


/** Start up the system */
int init_test_app_devices(void)
{
  struct s_sio_local_rs422 sio_ser1_rs422;

  /* Start the hardware library */
  hwlib_init();
  if (rio_init_we21hw("/dev/we21hw") < 0)
  {
    fprintf(stderr, "WE21 hardware initialisation failed\n");
  }

  /* Create the devices */
  printf("main: create devices\n");

  /* Make a console for IO */
  if (rio_create_device_sio_localconsole(&dev_console) < 0)
  {
    fprintf(stderr, "Local console serial device creation failed\n");
    /* Can't live without this */
    return -1;
  }

/* Not required, because rinblock creates it
  if (rio_create_device_buzzer_local(&dev_buz_we) < 0)
    WE_DEBUG_PRINTF(WE_DEBUG_DEV, "Buzzer device creation failed\n");
*/
  if (rio_create_device_iic_local(&dev_iic0, "/dev/i2c-0", we21hw_set_slot_select, NULL) < 0)
  {
    fprintf(stderr, "I2C device creation failed\n");
  }

/* Not required, because rinblock creates it
  if (rregister_io(&dev_kbd) < 0)
    WE_DEBUG_PRINTF(WE_DEBUG_DEV, "Keyboard device creation failed\n");
*/

/* Not required, because rinblock creates it
  if (rregister_io(&dev_lcd) < 0)
    WE_DEBUG_PRINTF(WE_DEBUG_DEV, "LCD device creation failed\n");
*/
  if (rio_create_device_adc24bit_local(&dev_adc24bit, "/dev/spidev0.0",
                                       adc24bit_user_hw_init_we21,
                                       adc24bit_sync_spi_we21,
                                       NULL, /* No 4/6 wire switch on WE2111 */
                                       adc24bit_wait_for_reading_we21) < 0)
  {
    fprintf(stderr, "ADC 24bit device creation failed\n");
  }
  if (rio_create_device_rtc_local(&dev_rtc) < 0)
  {
    fprintf(stderr, "RTC device creation failed\n");
  }
  if (rio_create_device_fram_local(&dev_fram, "/etc/rinstrum/fram.bin", 65536, 0xFF) < 0)
  {
    fprintf(stderr, "FRAM device creation failed\n");
  }

  /* Setup serial port devices */
  sio_ser1_rs422.hw_data = NULL;
  sio_ser1_rs422.rs422_mode = slrm_rs485_rts_deasserted;
  sio_ser1_rs422.sio_select_rs232_not_rs422 = we21hw_select_rs232_not_rs422;
  sio_ser1_rs422.sio_set_rs422_termination = we21hw_set_rs422_termination;
  sio_ser1_rs422.sio_set_rs422_transmitenable = NULL;
  if (rio_create_device_sio_localcom(&dev_ser1, "/dev/ttyS1", &sio_ser1_rs422, false) < 0)
  {
    fprintf(stderr, "Local serial 1 setup fail\n");
  }
  if (rio_create_device_sio_localcom(&dev_ser2, "/dev/ttyS2", NULL, false) < 0)
  {
    fprintf(stderr, "Local serial 2 setup fail\n");
  }

  if (rio_create_device_sio_localdgram(&dev_udp_datagram, 2222, "\r\n", 2, 1024) < 0)
  {
    fprintf(stderr, "UDP datagram serial setup on port 2222 failed\n");
  }
  if (rio_create_device_sio_localskt(&dev_tcp_stream, 2222) < 0)
  {
    fprintf(stderr, "TCP stream serial setup on port 2222 failed\n");
  }
  init_usb_serials();

  if (rio_create_device_sio_pipe(&dev_pipe1, &dev_pipe2) < 0)
  {
    fprintf(stderr, "Pipe serial setup failed\n");
  }

  return 0;
}


/** Shut down the system */
void destroy_test_app_devices(void)
{
  printf("main: shutdown initiated\n");

  /* Destroy devices */
  rdestroy_io(&dev_pipe2);
  rdestroy_io(&dev_pipe1);
  destroy_usb_serials();
  rdestroy_io(&dev_tcp_stream);
  rdestroy_io(&dev_udp_datagram);
  rdestroy_io(&dev_ser2);
  rdestroy_io(&dev_ser1);
  rdestroy_io(&dev_fram);
  rdestroy_io(&dev_rtc);
  rdestroy_io(&dev_adc24bit);
/*  rdestroy_io(&dev_lcd_we); */
/*  rdestroy_io(&dev_kbd_we); */
  rdestroy_io(&dev_iic0);
/*  rdestroy_io(&dev_buz_we); */

  rdestroy_io(&dev_console);

  /* Shutdown the hardware library */
  rio_destroy_we21hw();
  hwlib_destroy();
}
