/*
 * adc24bittest.c
 *
 *  Created on: 18/08/2009
 *      Author: ianb
 */
#include <rinhw/rio.h>
#include <rinhw/rstdio.h>
#include <rinhw/ucos_ii.h>
#include <rinhw/adc_ads1216.h>
#include <rinhw/stdrinstrum.h>

#include "adc24bittest.h"


static void SetLoadCellWireMode( struct t_io_dev *iodev, struct t_io_dev *adcdev, bool bFourNotSixWire )
{
  int mode = ( bFourNotSixWire ) ? IOCTL_ADC24BIT_LCWIRE_4WIRE : IOCTL_ADC24BIT_LCWIRE_6WIRE;
  int result;

  result = rioctl( adcdev, IOCTL_ADC24BIT_SET_LCWIRE_MODE, mode );
  rfprintf( iodev, "Result of IOCTL_ADC24BIT_SET_LCWIRE_MODE to %d was %d\r\n", mode, result );
}


static void ResetAdc( struct t_io_dev *iodev, struct t_io_dev *adcdev )
{
  int result;

  result = rioctl( adcdev, IOCTL_ADC24BIT_FORCE_RESET );
  rfprintf( iodev, "Result of IOCTL_ADC24BIT_FORCE_RESET was %d\r\n", result );
}


static void CheckAdcIsResponding( struct t_io_dev *iodev, struct t_io_dev *adcdev )
{
  int result;
  int isresponding = 0;

  result = rioctl( adcdev, IOCTL_ADC24BIT_IS_RESPONDING, &isresponding );
  rfprintf( iodev, "Result of IOCTL_ADC24BIT_IS_RESPONDING was %d, responding %d\r\n", result, isresponding );
}


static void GetAdcReading( struct t_io_dev *iodev, struct t_io_dev *adcdev )
{
  int result;
  int32s i32Data = 0;

  rioctl(adcdev, IOCTL_ADC24BIT_WAIT_FOR_DATA_READY, (unsigned long) OS_TICKS_PER_SEC);
  rlseek( adcdev, 0L, SEEK_SET );
  result = rread( adcdev, &i32Data, sizeof( i32Data ), OS_TICKS_PER_SEC );
  rfprintf( iodev, "Result of rread was %d, data %ld\r\n", result, i32Data );
}


static void GetAdcReadings( struct t_io_dev *iodev, struct t_io_dev *adcdev )
{
  char ch;

  while ( reof( iodev ) )
  {
    GetAdcReading( iodev, adcdev );
//        OSTimeDly(OS_TICKS_PER_SEC / 10);
  }
  rread( iodev, &ch, 1, 0 );
}


static unsigned long GetLong(struct t_io_dev *dev)
{
    unsigned char ch = 0;
    unsigned long num = 0;

    while (ch != '\n' && ch != '\r')
    {
        rread(dev, &ch, 1, 0);
        rwrite(dev, &ch, 1, 0);
        if ((ch >= '0') && (ch <= '9'))
            num = num * 10 + (ch - '0');
    }
    return num;
}


static void SetFullscale( struct t_io_dev *iodev, struct t_io_dev *adcdev )
{
  int result;
  int32u ui32Data;

  rfprintf( iodev, "Enter fullscale value: " );
  ui32Data = GetLong( iodev );
  result = rioctl( adcdev, IOCTL_ADC24BIT_SET_SPAN_REGISTER, ui32Data );
  rfprintf( iodev, "Result of IOCTL_ADC24BIT_SET_SPAN_REGISTER to 0x%lx was %d\r\n", ui32Data, result );
}


static void GetFullscale( struct t_io_dev *iodev, struct t_io_dev *adcdev )
{
  int result;
  int32u ui32Data;

  result = rioctl( adcdev, IOCTL_ADC24BIT_GET_SPAN_REGISTER, &ui32Data );
  rfprintf( iodev, "Result of IOCTL_ADC24BIT_GET_SPAN_REGISTER was %d, data 0x%lx\r\n", result, ui32Data );
}


static void SetOffsetCalibration( struct t_io_dev *iodev, struct t_io_dev *adcdev )
{
  int result;
  int32u ui32Data;

  rfprintf( iodev, "Enter offset calibration value: " );
  ui32Data = GetLong( iodev );
  result = rioctl( adcdev, IOCTL_ADC24BIT_SET_ZERO_REGISTER, ui32Data );
  rfprintf( iodev, "Result of IOCTL_ADC24BIT_SET_ZERO_REGISTER to 0x%lx was %d\r\n", ui32Data, result );
}


static void GetOffsetCalibration( struct t_io_dev *iodev, struct t_io_dev *adcdev )
{
  int result;
  int32u ui32Data;

  result = rioctl( adcdev, IOCTL_ADC24BIT_GET_ZERO_REGISTER, &ui32Data );
  rfprintf( iodev, "Result of IOCTL_ADC24BIT_GET_ZERO_REGISTER was %d, data 0x%lx\r\n", result, ui32Data );
}


static void SetPGAGain( struct t_io_dev *iodev, struct t_io_dev *adcdev )
{
  int result;
  int32u ui32Data;

  rfprintf( iodev, "Enter PGA gain (1, 2, 4, 8, 16, 32, 64, 128): " );
  ui32Data = GetLong( iodev );
  result = rioctl( adcdev, IOCTL_ADC24BIT_SET_PGA_GAIN, (int) ui32Data );
  rfprintf( iodev, "Result of IOCTL_ADC24BIT_SET_PGA_GAIN to %ld was %d\r\n", ui32Data, result );
}


static void SetDecimationRatio( struct t_io_dev *iodev, struct t_io_dev *adcdev )
{
  int result;
  int32u ui32Data;

  rfprintf( iodev, "Enter decimation frequency in 0.1Hz (eg 125=12.5Hz): ");
  ui32Data = GetLong( iodev );
  result = rioctl( adcdev, IOCTL_ADC24BIT_SET_DECIMATION_RATIO, (int) ui32Data );
  rfprintf( iodev, "Result of IOCTL_ADC24BIT_SET_DECIMATION_RATIO to %ld was %d\r\n", ui32Data, result );
}


/** Cycle through 10Hz to 100Hz in 0.5Hz increments, and dump readings */
static void DecimationRatioReadingDump( struct t_io_dev *iodev, struct t_io_dev *adcdev )
{
  int i, j;
  int result;
  int32s i32Data = 0;
  int32u ui32Data = 0;
  char ch;

  result = rioctl( adcdev, IOCTL_ADC24BIT_GET_ZERO_REGISTER, &ui32Data );
  rfprintf( iodev, "Zero: %lu\r\n", ui32Data );

  result = rioctl( adcdev, IOCTL_ADC24BIT_GET_SPAN_REGISTER, &ui32Data );
  rfprintf( iodev, "Span: %lu\r\n", ui32Data );

  rfprintf(iodev, "Freq (Hz),Reading\r\n");
  for (i = 100; i <= 1000; i += 5)
  {
    result = rioctl( adcdev, IOCTL_ADC24BIT_SET_DECIMATION_RATIO, i );
    if (result < 0)
    {
      rfprintf( iodev, "Set Decimation Ratio Failed\r\n" );
      break;
    }

    /* Wait for it to take affect, ditch 10 readings */
    for (j = 0; j < 10; j++)
    {
      rioctl(adcdev, IOCTL_ADC24BIT_WAIT_FOR_DATA_READY, (unsigned long) OS_TICKS_PER_SEC);
      rlseek( adcdev, 0L, SEEK_SET );
      result = rread( adcdev, &i32Data, sizeof( i32Data ), 0 );
      if (result < 0)
      {
        rfprintf( iodev, "Read ADC data failed\r\n" );
        break;
      }
    }

    OSTimeDly(OS_TICKS_PER_SEC);

    /* Print a reading */
    rioctl(adcdev, IOCTL_ADC24BIT_WAIT_FOR_DATA_READY, (unsigned long) OS_TICKS_PER_SEC);
    rlseek( adcdev, 0L, SEEK_SET );
    result = rread( adcdev, &i32Data, sizeof( i32Data ), OS_TICKS_PER_SEC );
    rfprintf( iodev, "%d.%01d,%ld\r\n", i / 10, i % 10, i32Data );

    /* Check for keypress to stop */
    if (!reof(iodev))
    {
      rread( iodev, &ch, 1, 0 );
      rfprintf(iodev, "User interrupt\r\n");
      break;
    }
  }

  /* Put back to default */
  (void) rioctl( adcdev, IOCTL_ADC24BIT_SET_DECIMATION_RATIO, 500 );
}


static void CheckReadingRate(struct t_io_dev *iodev, struct t_io_dev *adcdev)
{
  char ch;
  int result;
  int32s i32Data = 0;
  int32u last, now;
  int32u samples;
  int32u readrate;

  last = OSTimeGet();
  samples = 0;
  while (reof(iodev))
  {
    rioctl(adcdev, IOCTL_ADC24BIT_WAIT_FOR_DATA_READY, (unsigned long) OS_TICKS_PER_SEC);
    rlseek(adcdev, 0L, SEEK_SET);
    result = rread(adcdev, &i32Data, sizeof(i32Data), OS_TICKS_PER_SEC);
    if (result != sizeof(i32Data))
    {
      rfprintf(iodev, "Result of rread was %d\r\n", result);
      break;
    }
    samples++;

    now = OSTimeGet();
    if (now - last > OS_TICKS_PER_SEC)
    {
      //readrate = samples;
      readrate = samples * 10;
      readrate *= OS_TICKS_PER_SEC;
      readrate /= (now - last);

      rfprintf(iodev, "Read rate is %lu.%lu\r\n", (unsigned long) readrate / 10, (unsigned long) readrate % 10);
      //rfprintf(iodev, "Read rate is %lu\r\n", (unsigned long) readrate);
      last = now;
      samples = 0;
    }
  }
  rread(iodev, &ch, 1, 0);
}


void TestAdc24Bit(struct t_io_dev *dev, struct t_io_dev *adcdev)
{
  unsigned char ch = 0;

  ropen( adcdev, 0 );

  while (ch != 'Q')
  {
    rfprintf(dev, "24Bit ADC Test\r\n"
                  "  4 - Select 4 wire mode\r\n"
                  "  6 - Select 6 wire mode\r\n"
                  "  R - Reset ADC\r\n"
                  "  r - Check ADC is responding\r\n"
                  "  g - Get ADC reading\r\n"
                  "  G - Get ADC readings\r\n"
                  "  P - Set PGA gain\r\n"
                  "  D - Set decimation ratio\r\n"
                  "  d - Dump readings for decimation ratios\r\n"
                  "  F - Set full scale setting\r\n"
                  "  f - Get full scale setting\r\n"
                  "  O - Set offset setting\r\n"
                  "  o - Get offset setting\r\n"
                  "  s - Check sample rate\r\n"
                  "  Q - Quit\r\n\r\n" );

    rread(dev, &ch, 1, 0);
    switch (ch)
    {
      case '4':
        rfprintf( dev, "Set 4 Wire Loadcell Mode\r\n" );
        SetLoadCellWireMode( dev, adcdev, true );
        break;
      case '6':
        rfprintf( dev, "Set 6 Wire Loadcell Mode\r\n" );
        SetLoadCellWireMode( dev, adcdev, false );
        break;
      case 'R':
        rfprintf( dev, "Reset ADC\r\n" );
        ResetAdc( dev, adcdev );
        break;
      case 'r':
        rfprintf( dev, "Check ADC is responding\r\n" );
        CheckAdcIsResponding( dev, adcdev );
        break;
      case 'g':
        rfprintf( dev, "Get ADC reading\r\n" );
        GetAdcReading( dev, adcdev );
        break;
      case 'G':
        rfprintf( dev, "Get ADC readings\r\n" );
        GetAdcReadings( dev, adcdev );
        break;
      case 'P':
        rfprintf( dev, "Set PGA setting\r\n" );
        SetPGAGain( dev, adcdev );
        break;
      case 'D':
        rfprintf( dev, "Set decimation ratio\r\n" );
        SetDecimationRatio( dev, adcdev );
        break;
      case 'd':
        rfprintf( dev, "Dump readings for decimation ratios\r\n" );
        DecimationRatioReadingDump( dev, adcdev );
        break;
      case 'F':
        rfprintf( dev, "Set full scale reading\r\n" );
        SetFullscale( dev, adcdev );
        break;
      case 'f':
        rfprintf( dev, "Get full scale reading\r\n" );
        GetFullscale( dev, adcdev );
        break;
      case 'O':
        rfprintf( dev, "Set offset reading\n" );
        SetOffsetCalibration( dev, adcdev );
        break;
      case 'o':
        rfprintf( dev, "Get offset reading\r\n" );
        GetOffsetCalibration( dev, adcdev );
        break;
      case 's':
        rfprintf(dev, "Check sample rate\r\n");
        CheckReadingRate(dev, adcdev);
        break;
    }
  }
  rclose( adcdev );
}


#if 0
void InitAdc24Bit(struct t_io_dev *dev)
{
    int result;

    result = rregister_io( &dev_adc24bit );
    rfprintf( dev, "Result of registering 24bit ADC device %d\r\n", result );
}
#endif
