/*
 * bkgrndtest.c
 *
 *  Created on: Mar 20, 2015
 *      Author: ian
 */
#include <limits.h>
#include <rinhw/tick.h>
#include <rinhw/rstdio.h>
#include <rinhw/ucos_ii.h>
#include <rinhw/bkgrnd.h>
#include "dumpdev.h"

#include "bkgrndtest.h"


#define NUM_BG_FNS 5


struct s_fn_data
{
  int number;
  struct t_io_dev *console;
};


static struct t_tick_fn bg_fn_list[NUM_BG_FNS];
static struct s_fn_data bg_fn_data[NUM_BG_FNS];


static void timer_function(void *data)
{
  struct s_fn_data *fn_data = (struct s_fn_data *) data;

  rfprintf(fn_data->console, "Timer %d fired\n", fn_data->number);
}


static void AddTimer(struct t_io_dev *console, int *numtimers)
{
  long preset;

  if (*numtimers < NUM_BG_FNS)
  {
    rfprintf(console, "Enter timer preset in ticks (%d = 1 second): ", OS_TICKS_PER_SEC);
    preset = GetLong(console);
    if (preset < 0 || preset > UINT16_MAX)
    {
      rfprintf(console, "\nPreset out of limits\n");
      return;
    }

    bg_fn_data[*numtimers].console = console;
    bg_fn_data[*numtimers].number = *numtimers;

    bg_fn_list[*numtimers].preset = preset;
    bg_fn_list[*numtimers].count = 0;
    bg_fn_list[*numtimers].fn = timer_function;
    bg_fn_list[*numtimers].data = (void *) &bg_fn_data[*numtimers];
    bg_fn_list[*numtimers].next = NULL;
    BackgroundAddFunction(&bg_fn_list[*numtimers]);
    (*numtimers)++;

    rfprintf(console, "\nAdded timer %d\n", *numtimers);
  }
  else
  {
    rfprintf(console, "Exceeded number of timers\n");
  }
}


static void RemoveTimer(struct t_io_dev *console, int *numtimers)
{
  if (*numtimers > 0)
  {
    (*numtimers)--;
    BackgroundRemoveFunction(&bg_fn_list[*numtimers]);
    rfprintf(console, "Removed timer %d\n", *numtimers);
  }
  else
  {
    rfprintf(console, "No timers to remove\n");
  }
}


static void ChangeTimer(struct t_io_dev *console, int numtimers)
{
  long timer;
  long preset;

  rfprintf(console, "Enter timer to change (0..%d): ", numtimers - 1);
  timer = GetLong(console);
  if (timer < 0 || timer >= numtimers)
  {
    rfprintf(console, "\nTimer out of limits\n");
    return;
  }

  rfprintf(console, "Enter timer preset in ticks (%d = 1 second): ", OS_TICKS_PER_SEC);
  preset = GetLong(console);
  if (preset < 0 || preset > UINT16_MAX)
  {
    rfprintf(console, "\nPreset out of limits\n");
    return;
  }

  BackgroundChangePreset(&bg_fn_list[timer], preset);
}


void TestBackgroundTimer(struct t_io_dev *console, struct t_io_dev *dummy)
{
  int numtimers = 0;
  int timer;
  unsigned char ch;

  for (ch = '\0'; ch != 'Q'; )
  {
    rfprintf(console, "Background Timer Test\n"
                      "  A - Add timer\n"
                      "  C - Change timer\n"
                      "  R - Remove timer\n"
                      "  Q - Quit\r\n"
            );
    rread(console, &ch, 1, 0);

    switch (ch)
    {
      case 'A':
        AddTimer(console, &numtimers);
        break;
      case 'C':
        ChangeTimer(console, numtimers);
        break;
      case 'R':
        RemoveTimer(console, &numtimers);
        break;
    }
  }

  /* Clean up */
  for (timer = 0; timer < numtimers; timer++)
  {
    BackgroundRemoveFunction(&bg_fn_list[timer]);
  }
}
