/*
 * dumpdev.c
 *
 *  Created on: 13/08/2009
 *      Author: ian
 */
#include <ctype.h>
#include <rinhw/rstdio.h>
#include <rinhw/stdrinstrum.h>

#include "dumpdev.h"


unsigned long GetLong(struct t_io_dev *dev)
{
    unsigned char ch = 0;
    unsigned long num = 0;

    while ( '\n' != ch && '\r' != ch )
    {
        rread(dev, &ch, 1, 0);
        rwrite(dev, &ch, 1, 0);
        if ((ch >= '0') && (ch <= '9'))
            num = num * 10 + (ch - '0');
    }
    return num;
}


#define DUMP_WIDTH 16


void DumpMemory(struct t_io_dev *dev, const void *buf, long length, bool absolute_addr)
{
    long this_width, i;
    unsigned long address;
    const unsigned char *buffer = buf;

    address = 0;
    while (length > 0)
    {
        this_width = (length < DUMP_WIDTH) ? length : DUMP_WIDTH;

        if (absolute_addr)
          rfprintf(dev, "%08lx: ", (unsigned long) buffer + address);
        else
          rfprintf(dev, "%08lx: ", address);

        for (i = 0; i < this_width; i++)
        {
            rfprintf(dev, "%02x ", buffer[address + i]);
        }

        rfprintf(dev, " ");
        for (i = 0; i < this_width; i++)
        {
            rfprintf(dev, "%c", isprint(buffer[address + i]) ? buffer[address + i] : '.');
        }
        rfprintf(dev, "\r\n");

        address += this_width;
        length -= this_width;
    }
}



void DumpBlockDevice(struct t_io_dev *dev, struct t_io_dev *flash_dev)
{
    long address, length, this_width, i;
    unsigned char buffer[DUMP_WIDTH];
    int res;

    rfprintf(dev, "Enter dump start address: ");
    address = (unsigned short) GetLong(dev);
    rfprintf(dev, "Enter dump length: ");
    length = (unsigned short) GetLong(dev);

    if (rlseek(flash_dev, address, SEEK_SET) < 0)
    {
        rfprintf(dev, "Seek error, abort\r\n");
        return;
    }

    rfprintf(dev, "Device Dump\r\n");
    while (length > 0)
    {
        this_width = (length < DUMP_WIDTH) ? length : DUMP_WIDTH;

        res = rread(flash_dev, buffer, (unsigned short) this_width, 0);
        if (res != this_width)
        {
            rfprintf(dev, "Read error, abort\r\n");
            break;
        }

        rfprintf(dev, "%08lx: ", address);
        for (i = 0; i < this_width; i++)
        {
            rfprintf(dev, "%02x ", buffer[i]);
        }

        rfprintf(dev, " ");
        for (i = 0; i < this_width; i++)
        {
            rfprintf(dev, "%c", isprint(buffer[i]) ? buffer[i] : '.');
        }
        rfprintf(dev, "\r\n");

        address += this_width;
        length -= this_width;
    }
}


void S2DumpBlockDevice(struct t_io_dev *dev, struct t_io_dev *flash_dev)
{
    long address, length, this_width, i;
    unsigned char buffer[DUMP_WIDTH];
    unsigned char chksum;
    int res;

    rfprintf(dev, "Enter s dump start address: ");
    address = (unsigned short) GetLong(dev);
    rfprintf(dev, "Enter s dump length: ");
    length = (unsigned short) GetLong(dev);

    if (rlseek(flash_dev, address, SEEK_SET) < 0)
    {
        rfprintf(dev, "Seek error, abort\r\n");
        return;
    }

    rfprintf(dev, "S-Record Dump\r\n");
    while (length > 0)
    {
        this_width = (length < DUMP_WIDTH) ? length : DUMP_WIDTH;

        res = rread(flash_dev, buffer, (unsigned short) this_width, 0);
        if (res != this_width)
        {
            rfprintf(dev, "Read error, abort\r\n");
            break;
        }

        rfprintf(dev, "S2%02lx%06lx", this_width + 4UL, address);

        chksum = (unsigned char) (this_width + 4);
        chksum += (unsigned char) (address & 0x000000FF);
        chksum += (unsigned char) ((address >> 8) & 0x000000FF);
        chksum += (unsigned char) ((address >> 16) & 0x000000FF);

        for (i = 0; i < this_width; i++)
        {
            rfprintf(dev, "%02x", buffer[i]);
            chksum += buffer[i];
        }
        chksum = (unsigned char) (0xFF - chksum);
        rfprintf(dev, "%02x\r\n", chksum);

        address += this_width;
        length -= this_width;
    }
    rfprintf(dev, "S804000000FB\r\n\r\n");
}
