#include <rinhw/ucos_ii.h>
#include <rinhw/rio.h>
#include <rinhw/framdev.h>
#include <rinhw/rstdio.h>
#include <rinhw/power.h>
#include "dumpdev.h"

#include "framtest.h"


#define addr1 0L        // start of chip one
#define addr2 65534L    // last two bytes of chip two
#define addr3 32766L    // last two bytes of chip one, first two bytes of chip two


#define FRAM_TEST_DATA_SIZE 4

const unsigned char data1[ FRAM_TEST_DATA_SIZE ] = { 'T', 'E', 'S', 'T' };
const unsigned char data2[ FRAM_TEST_DATA_SIZE ] = { 'C', 'H', 'I', 'P' };
const unsigned char data3[ FRAM_TEST_DATA_SIZE ] = { 'F', 'R', 'A', 'M' };

unsigned char read_data[ FRAM_TEST_DATA_SIZE ];

#if 0
void TestFram(struct t_io_dev *dev)
{
    unsigned char ch = 0x00;
	unsigned char i;

    while (ch != 'Q')
    {

        rfprintf(dev, "\r\nFRAM Test\r\n 'W' to write data, 'R' to read data, 'C' to check size, 'S' to system reset, 'Q' to quit.\r\n");
        rread(dev, &ch, 1, 0);

        switch (ch)
        {
            case 'W':
				fram_write((unsigned char*)data1, 4, addr1);
				fram_write((unsigned char*)data2, 4, addr2);
				fram_write((unsigned char*)data3, 4, addr3);

                break;
            case 'S':
                SystemReset();
                break;
            case 'R':
                fram_read((unsigned char*)read_data, 4, addr1);
				for (i = 0; i < 4; i++)
				{
				   if (read_data[i] != data1[i])
                      rfprintf(dev, "data %c from FRAM addr %ld does not match %c\r\n", read_data[i], addr1+i, data1[i]);
				}
				for (i = 0; i < 4; i++)
				{
					read_data[i] = 0;           //blank read_data for next test.
				}
				fram_read((unsigned char*)read_data, 4, addr2);
				for (i = 0; i < 4; i++)
				{
				   if (read_data[i] != data2[i])
                      rfprintf(dev, "data %c from FRAM addr %ld does not match %c\r\n", read_data[i], addr2+i, data2[i]);
				}
				for (i = 0; i < 4; i++)
				{
					read_data[i] = 0;           //blank read_data for next test.
				}
				fram_read((unsigned char*)read_data, 4, addr3);
				for (i = 0; i < 4; i++)
				{
				   if (read_data[i] != data3[i])
                      rfprintf(dev, "data %c from FRAM addr %ld does not match %c\r\n", read_data[i], addr3+i, data3[i]);
				}
                break;
			case 'C':
			    rfprintf(dev, "there are %ld bytes of FRAM\r\n",fram_get_size());
				break;


        }
    }
}
#endif


static void vWriteFramData( struct t_io_dev *dev, struct t_io_dev *framdev, const unsigned char *data, int size, long posn )
{
  long lResult;

  /* Seek */
  lResult = rlseek( framdev, posn, SEEK_SET );
  if ( posn != lResult )
  {
    rfprintf( dev, "Seek to %ld on FRAM failed with result %d\r\n", posn, lResult );
  }

  /* Write */
  lResult = rwrite( framdev, data, (unsigned short) size, 0 );
  if ( size != lResult )
  {
    rfprintf( dev, "Write of %d bytes to %ld on FRAM failed with result\r\n", size, posn, lResult );
  }
}


static void vCheckFramData( struct t_io_dev *dev, struct t_io_dev *framdev, const unsigned char *data, int size, long posn )
{
  unsigned char read_data[ FRAM_TEST_DATA_SIZE ];
  long lResult;
  int i;

  /* Seek */
  lResult = rlseek( framdev, posn, SEEK_SET );
  if ( posn != lResult )
  {
    rfprintf( dev, "Seek to %ld on FRAM failed with result %d\r\n", posn, lResult );
  }

  /* Read */
  lResult = rread( framdev, read_data, FRAM_TEST_DATA_SIZE, 0 );
  if ( size != lResult )
  {
    rfprintf( dev, "Read of %d bytes from %ld on FRAM failed with result\r\n", size, posn, lResult );
  }

  /* Check */
  for ( i = 0; i < size; i++ )
  {
     if ( read_data[ i ] != data[ i ] )
     {
       rfprintf(dev, "Check of FRAM failed, data 0x%02X from addr %ld does not match 0x%02X\r\n", read_data[i], posn+i, data[i]);
     }
  }
}


static void vFillDevice( struct t_io_dev *iodev, struct t_io_dev *framdev )
{
  int nResult;
  unsigned char data = 0xA5;

  rfprintf( iodev, "Filling with 0x%02x\r\n", data );

  /* Seek */
  rlseek( framdev, 0, SEEK_SET );

  /* Fill */
  while ( rtell( framdev ) < rfilelength( framdev ) )
  {
    nResult = rwrite( framdev, &data, sizeof( data ), 0 );
    if ( sizeof( data ) != nResult )
    {
      rfprintf( iodev, "Fill failed at %ld\r\n", rtell( framdev ) );
      return;
    }
  }
  rfprintf( iodev, "Filling complete\r\n" );
}


static void vWriteAddress0( struct t_io_dev *iodev, struct t_io_dev *framdev )
{
  unsigned char data = 0x12;

  rfprintf( iodev, "Writing 0x%02x to address 0\r\n", data );

  /* Seek */
  rlseek( framdev, 0, SEEK_SET );
  rwrite( framdev, &data, sizeof( data ), 0 );

  rlseek( framdev, 0, SEEK_SET );
  rwrite( framdev, &data, sizeof( data ), 0 );

  rfprintf( iodev, "Writing complete\r\n" );
}


/** Test the FRAM using the device interface */
void TestFramDev(struct t_io_dev *dev, struct t_io_dev *framdev)
{
    unsigned char ch = 0x00;
    int nResult;

    nResult = ropen( framdev, 0 );
    if ( RIO_SUCCESS != nResult )
    {
      rfprintf( dev, "Open of FRAM device failed result %d\r\n", nResult );
    }

    while (ch != 'Q')
    {
        rfprintf(dev, "\r\nFRAM Test\r\n 'W' to write data, 'V' to verify data, 'C' to check size, 'R' to system reset, 'D' to dump FRAM, 'S' to S2 dump FRAM, 'F' to fill, 'Q' to quit.\r\n");
        rread(dev, &ch, 1, 0);

        switch (ch)
        {
            case 'W':
                vWriteFramData( dev, framdev, data1, sizeof( data1 ), addr1 );
                vWriteFramData( dev, framdev, data2, sizeof( data2 ), addr2 );
                vWriteFramData( dev, framdev, data3, sizeof( data3 ), addr3 );
                break;
            case 'R':
                SystemReset();
                break;
            case 'V':
                vCheckFramData( dev, framdev, data1, sizeof( data1 ), addr1 );
                vCheckFramData( dev, framdev, data2, sizeof( data2 ), addr2 );
                vCheckFramData( dev, framdev, data3, sizeof( data3 ), addr3 );
            case 'C':
                rfprintf(dev, "There are %ld bytes of FRAM\r\n", rfilelength( framdev ) );
                break;
            case 'D':
                DumpBlockDevice(dev, framdev);
                break;
            case 'S':
                S2DumpBlockDevice(dev, framdev);
                break;
            case 'F':
                vFillDevice( dev, framdev );
                break;
            case '0':
                vWriteAddress0( dev, framdev );
                break;
        }
    }
    rclose( framdev );
}


#if 0
void InitFRAM(struct t_io_dev *dev)
{
    int result;

    result = rregister_io(&dev_fram);

    rfprintf(dev, "Result of registering FRAM device %d\r\n", result);
}
#endif
