#include <rinhw/rio.h>
#include <rinhw/rstdio.h>
#include <rinhw/ucos_ii.h>
#include <rinhw/kbd.h>

#include "kbdtest.h"



unsigned char GetChar(struct t_io_dev *dev, unsigned char *dat)
{
    if (!reof(dev))
    {
        rread(dev, dat, 1, 0);
        return 1;
    }
    return 0;
}


//#define NUM_FUNC_KEYS 6
const unsigned char func_keys[NUM_FUNC_KEYS] =
{
    KEY_PHYSICAL_FUNC0, KEY_PHYSICAL_FUNC1, KEY_PHYSICAL_FUNC2, KEY_PHYSICAL_FUNC3,
    KEY_PHYSICAL_FUNC4, KEY_PHYSICAL_FUNC5
};


void TestKeyboard(struct t_io_dev *dev, struct t_io_dev *kbd_dev)
{
    union un_key key;
    unsigned char ch;
    char *event;
    int res;

    rfprintf(dev, "Keyboard Test\r\nOpening keyboard\r\n");
    if (ropen(kbd_dev, 0) < 0)
        rfprintf(dev, "Could not open keyboard\r\n");

    ch = KEY_PHYSICAL_FUNC0;
    res = rioctl(kbd_dev, IOCTL_KBD_SET_REPEAT, 1, &ch);
    rfprintf(dev, "Result of ioctl %d\r\n", res);

    ch = KEY_PHYSICAL_NUM2;
    res = rioctl(kbd_dev, IOCTL_KBD_SET_HOLD, 1, &ch);
    rfprintf(dev, "Result of ioctl %d\r\n", res);

//  res = rioctl(kbd_dev, IOCTL_KBD_SET_DISABLED, NUM_FUNC_KEYS, &func_keys[0]);
//    rfprintf(dev, "Result of ioctl %d\r\n", res);

    res = rioctl(kbd_dev, IOCTL_KBD_SET_DISABLED, NUM_FUNC_KEYS - 1, &func_keys[1]);
    rfprintf(dev, "Result of ioctl %d\r\n", res);

    ch = KEY_PHYSICAL_POWER_AND_FUNC0;
    res = rioctl(kbd_dev, IOCTL_KBD_SET_HOLD, 1, &ch);
    rfprintf(dev, "Result of ioctl %d\r\n", res);

    rfprintf(dev, "Press keys on keyboard, any key to quit\r\n");
    while (!GetChar(dev, &ch))
    {
        if (GetChar(kbd_dev, &key.BYTE))
        {
            switch ((enum tKeyEvent) key.BIT.EVENT)
            {
                case keDown:
                    event = "press";
                    break;
                case keUp:
                    event = "release";
                    break;
                case keHeld:
                    event = "long press";
                    break;
                case keDisabled:
                    event = "disabled";
                    break;
                default:
                    event = "error";
                    break;
            }
            rfprintf(dev, "Got key %s keyboard (0x%02x)\r\n", event, key.BIT.KEY);
        }
        OSTimeDly(OS_TICKS_PER_SEC / 40);
    }
    rclose(kbd_dev);
}


void TestKeyboardWrite(struct t_io_dev *dev, struct t_io_dev *kbd_dev)
{
    union un_key key;
    unsigned char ch;
    char *event;
    int res;

    rfprintf(dev, "Keyboard Buffer Write Test\r\nOpening keyboard\r\n");
    if (ropen(kbd_dev, 0) < 0)
    {
        rfprintf(dev, "Could not open keyboard\r\n");
        return;
    }

    rfprintf(dev, "KEY_PHYSICAL_NUM0 = normal, KEY_PHYSICAL_NUM1 = repeat, KEY_PHYSICAL_NUM2 = hold, KEY_PHYSICAL_NUM3 = disabled\r\n");
    ch = KEY_PHYSICAL_NUM0;
    rioctl(kbd_dev, IOCTL_KBD_SET_BASIC, 1, &ch);

    ch = KEY_PHYSICAL_NUM1;
    rioctl(kbd_dev, IOCTL_KBD_SET_REPEAT, 1, &ch);

    ch = KEY_PHYSICAL_NUM2;
    rioctl(kbd_dev, IOCTL_KBD_SET_HOLD, 1, &ch);

    ch = KEY_PHYSICAL_NUM3;
    rioctl(kbd_dev, IOCTL_KBD_SET_DISABLED, 1, &ch);

    rfprintf(dev, "Press keys on keyboard, or send serial characters, ESC to quit\r\n");
    rfprintf(dev, "a..j = KEY_PHYSICAL_NUM0..KEY_PHYSICAL_NUM9 short press\r\n");
    rfprintf(dev, "A..J = KEY_PHYSICAL_NUM0..KEY_PHYSICAL_NUM9 long press\r\n");

    while (ch != 0x1B)
    {
        if (GetChar(dev, &ch))
        {
            if ((ch >= 'a') && (ch <= 'j'))
            {
                key.BIT.KEY = ch - 'a' + KEY_PHYSICAL_NUM0;
                key.BIT.EVENT = keDown;
                res = rwrite(kbd_dev, &key.BYTE, 1, 0);
                rfprintf(dev, "Writing key %d short press to keyboard with result %d.\r\n", key.BIT.KEY, res);
            }
            else if ((ch >= 'A') && (ch <= 'J'))
            {
                key.BIT.KEY = ch - 'A' + KEY_PHYSICAL_NUM0;
                key.BIT.EVENT = keHeld;
                res = rwrite(kbd_dev, &key.BYTE, 1, 0);
                rfprintf(dev, "Writing key %d long press to keyboard with result %d.\r\n", key.BIT.KEY, res);
            }
        }

        if (GetChar(kbd_dev, &key.BYTE))
        {
            switch ((enum tKeyEvent) key.BIT.EVENT)
            {
                case keDown:
                    event = "press";
                    break;
                case keUp:
                    event = "release";
                    break;
                case keHeld:
                    event = "long press";
                    break;
                case keDisabled:
                    event = "disabled";
                    break;
                default:
                    event = "error";
                    break;
            }
            rfprintf(dev, "Got key %s keyboard (0x%X)\r\n", event, key.BIT.KEY);
        }
        OSTimeDly(OS_TICKS_PER_SEC / 40);
    }
    rclose(kbd_dev);
}
