#include <string.h>
#include <rinhw/rio.h>
#include <rinhw/rstdio.h>
#include <rinhw/ucos_ii.h>
#include <rinhw/lcd.h>

#include "lcdtest.h"


static unsigned long get_num(struct t_io_dev *debug_dev)
{
    unsigned char ch;
    unsigned long num = 0UL;

    do
    {
        rread(debug_dev, &ch, 1, 0);
        rwrite(debug_dev, &ch, 1, 0);
        if ((ch >= '0') && (ch <= '9'))
        {
            num *= 10;
            num += (unsigned char) (ch - '0');
        }
    }
    while ((ch >= '0') && (ch <= '9'));
    return num;
}


static void SetAllSegs(struct t_io_dev *dev, BOOLEAN state)
{
    int i, l;
    unsigned char ch;

    ropen(dev, 0);

    rlseek(dev, 0, SEEK_SET);
    ch = (state) ? 0xFF : 0;

    l = (int) rfilelength(dev);
    for (i = 0; i < l; i++)
        rwrite(dev, &ch, 1, 0);
    rclose(dev);
}


#if 0
static unsigned char GetChar(struct t_io_dev *dev, unsigned char *dat)
{
    if (!reof(dev))
    {
        rread(dev, dat, 1, 0);
        return 1;
    }
    return 0;
}
#endif


unsigned short GetHexWord(struct t_io_dev *dev)
{
    unsigned char ch, i;
    unsigned short res;

    res = 0;
    rfprintf(dev, "Enter a number in hex (4 digits): ");
    for (i = 0; i < 4; i++)
    {
        res <<= 4;
        rread(dev, &ch, 1,  0);
        rwrite(dev, &ch, 1, 0);
        if ((ch >= '0') && (ch <= '9'))
            res += (ch - '0');
        else if ((ch >= 'A') && (ch <= 'F'))
            res += (ch - 'A' + 10);
        else if ((ch >= 'a') && (ch <= 'f'))
            res += (ch - 'a' + 10);
        else
        {
            rwrite(dev, "?", 1, 0);
            res = 0;
            break;
        }
    }
    rfprintf(dev, "\r\n");
    return res;
}


void TestBacklight(struct t_io_dev *dev, struct t_io_dev *lcd_dev)
{
    unsigned char bklight;

    ropen(lcd_dev, 0);

    while (1)
    {
//        rfprintf(dev, "TGRA=%04X TGRB=%04x\r\n", P_TPU0.TGRA, P_TPU0.TGRB);
        bklight = (unsigned char) GetHexWord(dev);
        if (bklight > 100)
            break;
        rioctl(lcd_dev, IOCTL_LCD_SET_BACKLIGHT, bklight);
    }
    rclose(lcd_dev);
}


void LcdSegsTest(struct t_io_dev *dev, struct t_io_dev *lcd_dev)
{
    int devpos, prevpos;
    int num_segments;
    unsigned char data, temp;

    ropen(lcd_dev, 0);
    rioctl(lcd_dev, IOCTL_LCD_ACCESS_DATA);
    num_segments = (int) rfilelength(lcd_dev) * 8;
    rlseek(lcd_dev, 0L, SEEK_SET);
    rclose(lcd_dev);

    prevpos = devpos = 0;
    while (reof(dev))
    {
        rfprintf(dev, "Segment %d\r\n", devpos);

        ropen(lcd_dev, 0);
        if (prevpos != devpos)
        {
            rlseek(lcd_dev, (long) (prevpos >> 3), SEEK_SET);
            temp = 0;
            rwrite(lcd_dev, &temp, 1, 0);
        }

        rlseek(lcd_dev, (long) (devpos >> 3), SEEK_SET);
        data = (unsigned char) (0x01 << (devpos & 0x07));
        rwrite(lcd_dev, &data, 1, 0);

        rclose(lcd_dev);

        prevpos = devpos;
        if (++devpos >= num_segments)
            devpos = 0;

        OSTimeDly(OS_TICKS_PER_SEC / 4);
    }

    rread(dev, &data, 1, 0);
}



void LcdSegsTestManual(struct t_io_dev *dev, struct t_io_dev *lcd_dev)
{
    int devpos, prevpos, num_segments;
    unsigned char data, temp = 0, fwd = 1, pause = 0;

    ropen(lcd_dev, 0);
    rioctl(lcd_dev, IOCTL_LCD_ACCESS_DATA);
    num_segments = (int) rfilelength(lcd_dev) * 8;
    rlseek(lcd_dev, 0L, SEEK_SET);
    rclose(lcd_dev);

    rfprintf(dev, "Manual Test Segments\r\n1 - Change Direction, 2 - Pause, Q - Quit, Anything else next seg\r\n");
    prevpos = devpos = 0;
    while (1)
    {
        rfprintf(dev, "Now at segment %d\r\n", devpos);

        ropen(lcd_dev, 0);
        if (prevpos != devpos)
        {
            rlseek(lcd_dev, (long) (prevpos >> 3), SEEK_SET);
            temp = 0;
            rwrite(lcd_dev, &temp, 1, 0);
        }
        rlseek(lcd_dev, (long) (devpos >> 3), SEEK_SET);
        data = (unsigned char) (0x01 << (devpos & 0x07));
        rwrite(lcd_dev, &data, 1, 0);
        rclose(lcd_dev);

        rread(dev, &temp, 1, 0);
        switch (temp)
        {
            case '1':
                fwd = !fwd;
                break;
            case '2':
                pause = !pause;
                break;
            case 'Q':
                return;

            default:
                prevpos = devpos;
                if (!pause)
                {
                    if (fwd)
                    {
                        if (devpos >= num_segments - 1)
                            devpos = 0;
                        else
                            devpos++;
                    }
                    else
                    {
                        if (devpos == 0)
                            devpos = num_segments - 1;
                        else
                            devpos--;
                    }
                }
                break;
        }
    }
}


void LcdData(struct t_io_dev *dev, struct t_io_dev *lcd_dev)
{
/*
    int res;
    tLcdSegments *lcd_segs = (tLcdSegments *) 0xA5A5A5A5;
*/
    unsigned char ch;

    /*
     * Open the LCD device with exclusive access, and wait until it becomes available, then
     */
    ropen(lcd_dev, 0);

/* This is R400 badness
    rfprintf(dev, "&lcd_segs=%p, lcd_segs=%p\r\n", &lcd_segs, lcd_segs);
    res = rioctl(lcd_dev, IOCTL_LCD_GET_DATAPTR, &lcd_segs);
    rfprintf(dev, "rioctl result: %d, The pointer is %p.\r\n", res, lcd_segs);
*/

    rfprintf(dev, "LCD Buffer Dump:\r\n");
    rlseek(lcd_dev, 0, SEEK_SET);
    while (reof(lcd_dev) == 0)
    {
        if ((rtell(lcd_dev) & 0x07) == 0x07)
            rfprintf(dev, "\r\n");
        rread(lcd_dev, &ch, 1, 0);
        rfprintf(dev, "0x%02x, ", ch);
    }
    rfprintf(dev, "\r\n");
    rclose(lcd_dev);
}


static void ShowSeg(struct t_io_dev *dev, struct t_io_dev *lcd_dev, void *s, int len, char *sctname, char *segname)
{
    unsigned char ch;

    rfprintf(dev, "Testing Seg %s.%s, press a key\r\n", sctname, segname);

    ropen(lcd_dev, 0);
    rlseek(lcd_dev, 0L, SEEK_SET);
    rwrite(lcd_dev, s, (unsigned short) len, 0);
    rclose(lcd_dev);

    rread(dev, &ch, 1, 0);
}


tLcdSegments segs;


#define N_SEG(sct, nam) s->Named.sct.nam = 1; ShowSeg(dev, lcd_dev, s, sizeof(*s), # sct, # nam); s->Named.sct.nam = 0


void TestLcdNamedSegments(struct t_io_dev *dev, struct t_io_dev *lcd_dev)
{
    tLcdSegments *s = &segs;

    /*
     * Zero out all segs
     */
    memset(s, 0, sizeof(*s));
    rfprintf(dev, "LCD Segment Bit Struct Testing\r\n\r\n");

    N_SEG(Annunciators, Zero);
    N_SEG(Annunciators, Sigma);
    N_SEG(Annunciators, Balance);
    N_SEG(Annunciators, BalSegB);
    N_SEG(Annunciators, BalSegA);
    N_SEG(Annunciators, BalSegF);
    N_SEG(Annunciators, BalSegE);
    N_SEG(Annunciators, BalSegG);
    N_SEG(Annunciators, BalSegD);
    N_SEG(Annunciators, BalSegC);
    N_SEG(Annunciators, RngSegC);
    N_SEG(Annunciators, Range);
    N_SEG(Annunciators, RngSegE);
    N_SEG(Annunciators, RngSegADG);
    N_SEG(Annunciators, Hold);
    N_SEG(Annunciators, Motion);
    N_SEG(Annunciators, Net);
    N_SEG(Annunciators, Coz);

    N_SEG(TopRowUnits, K);
    N_SEG(TopRowUnits, T);
    N_SEG(TopRowUnits, L);
    N_SEG(TopRowUnits, G);
    N_SEG(TopRowUnits, B);
    N_SEG(TopRowUnits, P);
    N_SEG(TopRowUnits, O);
    N_SEG(TopRowUnits, ArrowBot);
    N_SEG(TopRowUnits, N);
    N_SEG(TopRowUnits, Z);
    N_SEG(TopRowUnits, ArrowTop);

    N_SEG(BottomRowUnits, T);
    N_SEG(BottomRowUnits, L);
    N_SEG(BottomRowUnits, P);
    N_SEG(BottomRowUnits, B);
    N_SEG(BottomRowUnits, K);
    N_SEG(BottomRowUnits, G);
    N_SEG(BottomRowUnits, N);
    N_SEG(BottomRowUnits, O);
    N_SEG(BottomRowUnits, Z);
    N_SEG(BottomRowUnits, PctCircles);
    N_SEG(BottomRowUnits, PctSlash);
    N_SEG(BottomRowUnits, TOT);
    N_SEG(BottomRowUnits, S);
    N_SEG(BottomRowUnits, H);
    N_SEG(BottomRowUnits, M);

    N_SEG(MiscSegments, Clock);
    N_SEG(MiscSegments, Wait45);
    N_SEG(MiscSegments, Wait90);
    N_SEG(MiscSegments, Wait135);
    N_SEG(MiscSegments, Wait0);
    N_SEG(MiscSegments, BatBot);
    N_SEG(MiscSegments, BatMidTop);
    N_SEG(MiscSegments, BatMidBot);
    N_SEG(MiscSegments, BatTop);
    N_SEG(MiscSegments, Battery);

//    N_SEG(TopRowDigits);
//    s->Named.TopRightRowDigits;
}


#define D_SEG(sct, d, n) \
    s->Named.sct.D_ ## d ## n = 1; \
    ShowSeg(dev, lcd_dev, s, sizeof(*s), # sct, "D_" # d # n); \
    s->Named.sct.D_ ## d ## n = 0

#define DALL_NODP_SEG(sct, d) \
    D_SEG(sct, d, A); D_SEG(sct, d, B); D_SEG(sct, d, C); D_SEG(sct, d, D); \
    D_SEG(sct, d, E); D_SEG(sct, d, F); D_SEG(sct, d, G); \
    D_SEG(sct, d, I); D_SEG(sct, d, J)


#define DALL_SEG(sct, d) \
    D_SEG(sct, d, A); D_SEG(sct, d, B); D_SEG(sct, d, C); D_SEG(sct, d, D); \
    D_SEG(sct, d, E); D_SEG(sct, d, F); D_SEG(sct, d, G); D_SEG(sct, d, H); \
    D_SEG(sct, d, I); D_SEG(sct, d, J)


void TestLcdNamedDigitSegments(struct t_io_dev *dev, struct t_io_dev *lcd_dev)
{
    tLcdSegments *s = &segs;

    /*
     * Zero out all segs
     */
    memset(s, 0, sizeof(*s));
    rfprintf(dev, "LCD Segment Digit Bit Struct Testing\r\n\r\n");

    DALL_SEG(TopRowDigits, 1);
    DALL_SEG(TopRowDigits, 2);
    DALL_SEG(TopRowDigits, 3);
    DALL_SEG(TopRowDigits, 4);
    DALL_SEG(TopRowDigits, 5);
    DALL_NODP_SEG(TopRowDigits, 6);

    DALL_SEG(TopRightRowDigits, 1);
    DALL_SEG(TopRightRowDigits, 2);
    DALL_SEG(TopRightRowDigits, 3);
    DALL_NODP_SEG(TopRightRowDigits, 4);

    DALL_SEG(BottomRowDigits, 1);
    DALL_SEG(BottomRowDigits, 2);
    DALL_SEG(BottomRowDigits, 3);
    DALL_SEG(BottomRowDigits, 4);
    DALL_SEG(BottomRowDigits, 5);
    DALL_SEG(BottomRowDigits, 6);
    DALL_SEG(BottomRowDigits, 7);
    DALL_SEG(BottomRowDigits, 8);
    DALL_NODP_SEG(BottomRowDigits, 9);

    DALL_SEG(BottomRightRowDigits, 1);
    DALL_SEG(BottomRightRowDigits, 2);
    DALL_SEG(BottomRightRowDigits, 3);
    DALL_SEG(BottomRightRowDigits, 4);
    DALL_SEG(BottomRightRowDigits, 5);
    DALL_SEG(BottomRightRowDigits, 6);
    DALL_SEG(BottomRightRowDigits, 7);
    DALL_NODP_SEG(BottomRightRowDigits, 8);
}


static void LcdWriteChars(struct t_io_dev *dev, struct t_io_dev *lcd_dev)
{
    unsigned char ch, pos;

    rfprintf(dev, "LCD Character Test. ESC to quit.\r\n");
    SetAllSegs(lcd_dev, FALSE);

    ch = 0;
    pos = 0;
    while (ch != 0x1B)
    {
        ropen(lcd_dev, 0);
        rioctl(lcd_dev, IOCTL_LCD_ACCESS_CHAR);
        rlseek(lcd_dev, (long) pos, SEEK_SET);
        rread(dev, &ch, 1, 0);
        rwrite(lcd_dev, &ch, 1, 0);
        pos = (unsigned char) rtell(lcd_dev);
        if (pos >= rfilelength(lcd_dev))
            pos = 0;
        rclose(lcd_dev);
    }
}


#define MAX_BUF_LEN 20

static void LcdWriteBlockChars(struct t_io_dev *dev, struct t_io_dev *lcd_dev)
{
    unsigned char ch, len, pos;
    unsigned char buf[MAX_BUF_LEN];

    rfprintf(dev, "LCD Character Block Write Test. Enter posn followed by LF\r\n");

    pos = (unsigned char) get_num(dev);

    rfprintf(dev, "Send String (upto 20 chars) followed by LF\r\n");

    len = 0;
    do
    {
        rread(dev, &ch, 1, 0);
        rwrite(dev, &ch, 1, 0);
        if (ch != 0x0A)
            buf[len++] = ch;
    }
    while ((len < MAX_BUF_LEN) && (ch != 0x0A));
    buf[len] = '\0';

    SetAllSegs(lcd_dev, FALSE);

    ropen(lcd_dev, 0);
    rioctl(lcd_dev, IOCTL_LCD_ACCESS_CHAR);
    rlseek(lcd_dev, (long) pos, SEEK_SET);
    rwrite(lcd_dev, buf, (unsigned short) strlen((char *) buf), 0);
    rclose(lcd_dev);
}


static void LcdDumpChars(struct t_io_dev *dev, struct t_io_dev *lcd_dev)
{
    unsigned char ch;

    ropen(lcd_dev, 0);
    rioctl(lcd_dev, IOCTL_LCD_ACCESS_CHAR);
    rlseek(lcd_dev, 0L, SEEK_SET);

    rfprintf(dev, "Dump LCD Character Buffer (size %ld):\r\n", rfilelength(lcd_dev));

    while (rtell(lcd_dev) < rfilelength(lcd_dev))
    {
        rread(lcd_dev, &ch, 1, 0);
        rfprintf(dev, "%c", ch);
    }

    rclose(lcd_dev);
    rfprintf(dev, "\r\n");
}


#ifdef IOCTL_LCD_ACCESS_FLASH
/** Flash segments, 0 = none, 1 = all, 2 = digits, 3 = annunciators */
static void FlashSegments(struct t_io_dev *dev, struct t_io_dev *lcd_dev, int which)
{
    long size, i;
    unsigned char ch;

    ropen(lcd_dev, 0);
    rioctl(lcd_dev, IOCTL_LCD_ACCESS_FLASH);
    size = rfilelength(lcd_dev);

    switch (which)
    {
      /* No flash */
      case 0:
        ch = 0x00;
        for (i = 0; i < size; i++)
        {
          rwrite(lcd_dev, &ch, 1, 0);
        }
        break;

      /* All flash */
      case 1:
        ch = 0xFF;
        for (i = 0; i < size; i++)
        {
          rwrite(lcd_dev, &ch, 1, 0);
        }
        break;

#if 0
      /* Digits */
      case 2:
        i = offsetof(tWE21LcdDigits, Digits);
        size = i + sizeof(tWE21LcdSegments_Digits);
        rlseek(lcd_dev, i, SEEK_SET);
        ch = 0xFF;
        for (i = 0; i < size; i++)
        {
          rwrite(lcd_dev, &ch, 1, 0);
        }
        break;
#endif
#if 0
      /* Annunciators */
      case 3:
        i = offsetof(tWE21LcdDigits, Digits);
        rlseek(lcd_dev, i);
        ch = 0xFF;
        for (i = 0; i < size; i++)
        {
          rwrite(lcd_dev, &ch, 1, 0);
        }
        break;
#endif
    }
    rclose(lcd_dev);
}
#endif /* #ifdef IOCTL_LCD_ACCESS_FLASH */


void lcd_test(struct t_io_dev *dev, struct t_io_dev *lcd_dev)
{
    unsigned char temp;

    while (1)
    {
        rfprintf(dev, "LCD Test\r\n"
                      " Segments: 0 - All Segs Off, 1 - All Segs On, L - Test Segments, M - Manual Test Segments,\r\n"
                      "           D - Print LCD Data, B - Backlight, N - Test Named Segments, S - Test Named Digit\r\n"
#ifdef IOCTL_LCD_ACCESS_FLASH
                      "           6 - Flash off all segs, 7 - Flash all segs, 8 - Flash digits\r\n"
#endif /* #ifdef IOCTL_LCD_ACCESS_FLASH */
                      " Characters: C - Write LCD Characters, c - Write Block Chars, P - Print LCD Character buffer\r\n"
                      " Q - Quit\r\n");

        rread(dev, &temp, 1, 0);
        switch (temp)
        {
            case '0':
                SetAllSegs(lcd_dev, FALSE);
                break;
            case '1':
                SetAllSegs(lcd_dev, TRUE);
                break;
#ifdef IOCTL_LCD_ACCESS_FLASH
            case '6':
            case '7':
            case '8':
                FlashSegments(dev, lcd_dev, temp - '6');
                break;
#endif /* #ifdef IOCTL_LCD_ACCESS_FLASH */
            case 'L':
                LcdSegsTest(dev, lcd_dev);
                break;
            case 'M':
                LcdSegsTestManual(dev, lcd_dev);
                break;
            case 'D':
                LcdData(dev, lcd_dev);
                break;
            case 'B':
                TestBacklight(dev, lcd_dev);
                break;
            case 'N':
                TestLcdNamedSegments(dev, lcd_dev);
                break;
            case 'S':
                TestLcdNamedDigitSegments(dev, lcd_dev);
                break;
            case 'C':
                LcdWriteChars(dev, lcd_dev);
                break;
            case 'c':
                LcdWriteBlockChars(dev, lcd_dev);
                break;
            case 'P':
                LcdDumpChars(dev, lcd_dev);
                break;
            case 'Q':
                return;
        }
    }
}


#if 0
void LcdStuff(void)
{
    SetAllSegs(&dev_lcd, 1);
}
#endif
