#include <string.h>
#include <rinhw/rio.h>
#include <rinhw/rstdio.h>
#include <rinhw/ucos_ii.h>
#include <rinhw/rtc.h>

#include "rtctest.h"


static unsigned short GetNum(struct t_io_dev *dev, char *msg)
{
    unsigned char ch = 0;
    unsigned short num = 0;

    rwrite(dev, msg, (unsigned short) strlen(msg), 0);

    while (ch != '\n')
    {
        rread(dev, &ch, 1, 0);
        rwrite(dev, &ch, 1, 0);
        if ((ch >= '0') && (ch <= '9'))
            num = num * 10 + (ch - '0');
    }
    return num;
}


static void SetTime(struct t_io_dev *dev, struct t_io_dev *devrtc)
{
    t_datetime dt;
    int res;

    dt.hour = (unsigned char) GetNum(dev, "Enter Hours (0..23): ");
    dt.minute = (unsigned char) GetNum(dev, "Enter Minutes (0..59): ");
    dt.second = (unsigned char) GetNum(dev, "Enter Seconds (0..59): ");

    dt.day = 3;
    dt.date = (unsigned char) GetNum(dev, "Enter Day (0..31): ");
    dt.month = (unsigned char) GetNum(dev, "Enter Month (0..12): ");
    dt.year = GetNum(dev, "Enter Years (2000..2199): ");

    res = rwrite(devrtc, &dt, sizeof(dt), 0);
    rfprintf(dev, "Result of write to RTC0: %d\r\n", res);
}


void TestRTC(struct t_io_dev *dev, struct t_io_dev *rtcdev)
{
    int res;
    t_datetime dt;
    unsigned char ch;

    rfprintf(dev, "RTC Test on channel 0.\r\n Press 'S' to set time, ESC to exit.\r\n");

    ropen(rtcdev, 0);

    while (1)
    {
        res = rread(rtcdev, &dt, sizeof(dt), 0);
        rfprintf(dev, "Read Result: %d. ", res);
        rfprintf(dev, "Time: %02d:%02d:%02d, Date: %02d/%02d/%04d\r\n", dt.hour, dt.minute, dt.second, dt.date, dt.month, dt.year);

        if (!reof(dev))
        {
            rread(dev, &ch, 1, 0);
            if (ch == 'S')
                SetTime(dev, rtcdev);
            else if (ch == 0x1B)
                break;
        }

        OSTimeDly(OS_TICKS_PER_SEC / 2);
    }
    rclose(rtcdev);
}
