#include <rinhw/rio.h>
#include <rinhw/rstdio.h>
#include <rinhw/ucos_ii.h>
#include <rinhw/lcd.h>
#include <rinhw/kbd.h>

#include "shenzhenlcdtest.h"


#define SHENZHEN_TICK_TIME (OS_TICKS_PER_SEC / 4)
#define SHENZHEN_OSC_TIME (OS_TICKS_PER_SEC * 2)

#define SHENZHEN_OSC_COUNT (SHENZHEN_OSC_TIME / SHENZHEN_TICK_TIME)


static void SetAllSegs(struct t_io_dev *dev, BOOLEAN state)
{
    int i, l;
    unsigned char ch;

    ropen(dev, 0);
    
    rlseek(dev, 0, SEEK_SET);
    ch = (state) ? 0xFF : 0;

    l = (int) rfilelength(dev);
    for (i = 0; i < l; i++)
        rwrite(dev, &ch, 1, 0);
    rclose(dev);
}


static void ShenzhenLCDSegs(void)
{
    static int devpos = 0, prevpos = NUM_LCD_SEGMENTS;
    unsigned char data, temp;

//    rfprintf(dev, "Segment %d\n", devpos);

    ropen(&dev_lcd, 0);
    if (prevpos != devpos)
    {
        rlseek(&dev_lcd, (long) (prevpos >> 3), SEEK_SET);
        temp = 0;
        rwrite(&dev_lcd, &temp, 1, 0);
    }

    rlseek(&dev_lcd, (long) (devpos >> 3), SEEK_SET);
    data = (unsigned char) (0x01 << (devpos & 0x07));
    rwrite(&dev_lcd, &data, 1, 0);

    rclose(&dev_lcd);

    prevpos = devpos;
    if (++devpos >= NUM_LCD_SEGMENTS)
        devpos = 0;
}


static void ShenzhenLCDFlash(void)
{
    static unsigned char ctr = SHENZHEN_OSC_COUNT * 2;

    if (ctr >= SHENZHEN_OSC_COUNT * 2)
    {
        SetAllSegs(&dev_lcd, 0);
        ctr = 0;
    }
    else if (ctr == SHENZHEN_OSC_COUNT)
    {
        SetAllSegs(&dev_lcd, 1);       
        ctr++;
    }
    else
    {
        ctr++;
    }
}


static void ShenzhenLCDDigits(void)
{
    static unsigned char digit = '0';
    static unsigned char count = SHENZHEN_OSC_COUNT;
    int i, l;

    if (count++ < SHENZHEN_OSC_COUNT)
        return;

    ropen(&dev_lcd, 0);
    rioctl(&dev_lcd, IOCTL_LCD_ACCESS_CHAR);

    rlseek(&dev_lcd, 0, SEEK_SET);
    l = (int) rfilelength(&dev_lcd);
    for (i = 0; i < l; i++)
        rwrite(&dev_lcd, &digit, 1, 0);

    rclose(&dev_lcd);

    if (++digit > '9')
        digit = '0';
}


enum eSLcdTestType { slttOnOff, slttSegBySeg, slttDigits};

#define NUM_SHENZHEN_KEYS 3
static const unsigned char key_cfg[NUM_SHENZHEN_KEYS] = { KEY_PHYSICAL_FUNC0, KEY_PHYSICAL_FUNC1, KEY_PHYSICAL_FUNC2 };


void ShenzhenLCDTest(struct t_io_dev *dev, struct t_io_dev *lcd_dev)
{
    union un_key key;
    enum eSLcdTestType test_type = slttOnOff;

    ropen(&dev_kbd, 0);
    rioctl(&dev_kbd, IOCTL_KBD_SET_BASIC, NUM_SHENZHEN_KEYS, &key_cfg);

    while (1)
    {
        switch (test_type)
        {
            case slttOnOff:
                ShenzhenLCDFlash();
                break;
            case slttSegBySeg:
                ShenzhenLCDSegs();
                break;
            case slttDigits:
                ShenzhenLCDDigits();
                break;
        }
        if (rread(&dev_kbd, &key, sizeof(key), SHENZHEN_TICK_TIME) == sizeof(key))
        {
            if (key.BIT.EVENT == keDown)
            {
                switch (key.BIT.KEY)
                {
                    case KEY_PHYSICAL_FUNC0:
                        test_type = slttOnOff;
                        break;
                    case KEY_PHYSICAL_FUNC1:
                        test_type = slttSegBySeg;
                        SetAllSegs(&dev_lcd, 0);
                        break;
                    case KEY_PHYSICAL_FUNC2:
                        test_type = slttDigits;
                        SetAllSegs(&dev_lcd, 0);
                        break;
                }
            }
        }
    }
}

