#include <string.h>
#include <rinhw/rio.h>
#include <rinhw/rstdio.h>
#include <rinhw/ucos_ii.h>
#include <rinhw/sio.h>
#include "dumpdev.h"

#include "splitsiotest.h"


#if 0 /* Was R400 */


#define TASK_SPLIT_STK_SIZE 200
OS_STK SplitSioTaskStk[TASK_SPLIT_STK_SIZE];


struct t_io_dev *da_dev;

static void SplitSioTask(void *data)
{
    unsigned char master = (unsigned char) (int) data;
    struct t_io_dev *dev = (master) ? &dev_com3 : &dev_com4;

    while (1)
    {
        if (master)
        {
            rfprintf(da_dev, "Write master\r\n");
            rfprintf(dev, "MASTERSTRING\r\n");
        }
        else
        {
            rfprintf(da_dev, "Write slave\r\n");
            rfprintf(dev, "slavestring\r\n");
        }
        OSTimeDly(1);
    }
}


static void FloodSplitSio(struct t_io_dev *dev)
{
    t_ioctl_siocfg siocfg;

    da_dev = dev;
    rfprintf(da_dev, "Split Serial IO Test.\r\n Opening COM3s and COM3m\r\n");

    ropen(&dev_com3, 1);
    ropen(&dev_com4, 1);

    siocfg.baud = 57600UL;
    siocfg.data_bits = 8;
    siocfg.parity_bits = 0;
    siocfg.stop_bits = 1;
    siocfg.use_cts = 1;
    rioctl(&dev_com3, IOCTL_SER_SET_CONFIG, &siocfg);

    rfprintf(da_dev, "Starting task\r\n");
    OSTaskCreate(SplitSioTask, (void *) 0, &SplitSioTaskStk[TASK_SPLIT_STK_SIZE - 1], 8);

    rfprintf(da_dev, "Running task\r\n");
    SplitSioTask((void *) 1);
}


static void WriteSerialPort(struct t_io_dev *dev, struct t_io_dev *sio_dev)
{
    unsigned char ch = 0;
    static const char test_msg[] = "This is a serial test message\r\n";
    unsigned short len;

    rfprintf(dev, "Write Serial Port, ESC to stop\r\n");

    ropen(sio_dev, 1);
    len = (unsigned short) strlen(test_msg);

    while ( 1 )
    {
        rwrite( sio_dev, test_msg, len, 0 );
        if ( reof(dev) == 0 )
        {
            rread( dev, &ch, 1, 0 );
            if ( 0x1B == ch )
            {
                break;
            }
        }
    }
}


static void EchoSerialPort(struct t_io_dev *dev, struct t_io_dev *sio_dev)
{
    unsigned char ch;

    rfprintf(dev, "Echo Serial Port, ESC to stop\r\n");

    while ( 1 )
    {
        rread(sio_dev, &ch, 1, 0);
        rwrite(sio_dev, &ch, 1, 0);
        if ( sio_dev == dev )
        {
            if ( 0x1B == ch )
            {
                break;
            }
        }
        else if (reof(dev) == 0)
        {
            rread( dev, &ch, 1, 0 );
            if ( 0x1B == ch )
            {
                break;
            }
        }
    }
}

#endif


void ConfigureSerial(struct t_io_dev *dev, struct t_io_dev *sio_dev)
{
    t_ioctl_siocfg siocfg;
    unsigned char ch;

    rfprintf(dev, "Configure Serial Port\r\n");

    /*
     * Baud
     */
    rfprintf(dev, "Enter Baud followed by LF: ");
    siocfg.baud = GetLong(dev);

    /*
     * Parity
     */
    rfprintf(dev, "Enter Parity (NOE): ");
    rread(dev, &ch, 1, 0);
    rfprintf(dev, "%c\r\n", ch);
    switch (ch)
    {
        case 'N':
            siocfg.parity_bits = SIO_PARITY_NONE;
            break;
        case 'O':
            siocfg.parity_bits = SIO_PARITY_ODD;
            break;
        case 'E':
            siocfg.parity_bits = SIO_PARITY_EVEN;
            break;
        default:
            rfprintf(dev, "Bad Parity\r\n");
            return;
    }

    /*
     * Data Bits
     */
    rfprintf(dev, "Enter Data Bits (78): ");
    rread(dev, &ch, 1, 0);
    rfprintf(dev, "%c\r\n", ch);
    switch (ch)
    {
        case '7':
            siocfg.data_bits = 7;
            break;
        case '8':
            siocfg.data_bits = 8;
            break;
        default:
            rfprintf(dev, "Bad Data Bits\r\n");
            return;
    }

    /*
     * Stop Bits
     */
    rfprintf(dev, "Enter Data Bits (12): ");
    rread(dev, &ch, 1, 0);
    rfprintf(dev, "%c\r\n", ch);
    switch (ch)
    {
        case '1':
            siocfg.stop_bits = SIO_STOPBITS_ONE;
            break;
        case '2':
            siocfg.stop_bits = SIO_STOPBITS_TWO;
            break;
        default:
            rfprintf(dev, "Bad Stop Bits\r\n");
            return;
    }

    /*
     * Use CTS
     */
    rfprintf(dev, "Enter Use CTS (01): ");
    rread(dev, &ch, 1, 0);
    rfprintf(dev, "%c\r\n", ch);
    switch (ch)
    {
        case '0':
            siocfg.use_cts = 0;
            break;
        case '1':
            siocfg.use_cts = 1;
            break;
        default:
            rfprintf(dev, "Bad Use CTS\r\n");
            return;
    }

    /*
     * Use Terminations
     */
    rfprintf(dev, "Enter Use Terminations (01): ");
    rread(dev, &ch, 1, 0);
    rfprintf(dev, "%c\r\n", ch);
    switch (ch)
    {
        case '0':
            siocfg.use_terminations = 0;
            break;
        case '1':
            siocfg.use_terminations = 1;
            break;
        default:
            rfprintf(dev, "Bad Use Terminations\r\n");
            return;
    }

    /*
     * Do the configuration
     */
    rioctl(sio_dev, IOCTL_SER_SET_CONFIG, &siocfg);
}


void SerialDumpConfiguration(struct t_io_dev *dev, struct t_io_dev *sio_dev)
{
    t_ioctl_siocfg siocfg;
    int res;

    res = rioctl(sio_dev, IOCTL_SER_GET_CONFIG, &siocfg);
    if (res < 0)
    {
      rfprintf(dev, "Error getting serial configuration\r\n");
      return;
    }

    rfprintf(dev, "Serial configuration:\r\n");
    rfprintf(dev, "  Baud: %lu\r\n", siocfg.baud);
    rfprintf(dev, "  Parity: %c\r\n", (siocfg.parity_bits == SIO_PARITY_NONE) ? 'N' : (siocfg.parity_bits == SIO_PARITY_ODD) ? 'O'  : 'E');
    rfprintf(dev, "  Data Bits: %u\r\n", siocfg.data_bits);
    rfprintf(dev, "  Stop Bits: %u\r\n", siocfg.stop_bits);
    rfprintf(dev, "  Use CTS: %u\r\n", siocfg.use_cts);
    rfprintf(dev, "  Terminations: %u\r\n", siocfg.use_terminations);
}


#if 0 /* Was R400 */


struct t_io_dev *SelectPort(struct t_io_dev *dev)
{
    struct t_io_dev *p_sio_dev = (struct t_io_dev *) 0;
    unsigned char ch;

    rfprintf(dev, "Select port (2, 3, 4):");
    rread(dev, &ch, 1, 0);
    rfprintf(dev, "%c\r\n", ch);

    switch (ch)
    {
      case '2':
        p_sio_dev = &dev_com2;
        rfprintf(dev, "Selected dev_com2\r\n");
        break;
      case '3':
        p_sio_dev = &dev_com3;
        rfprintf(dev, "Selected dev_com3\r\n");
        break;
      case '4':
        p_sio_dev = &dev_com4;
        rfprintf(dev, "Selected dev_com4\r\n");
        break;
      default:
        p_sio_dev = &dev_com3;
        rfprintf(dev, "Bad port, selecting dev_com3 by default\r\n");
        break;
    }

    return p_sio_dev;
}


static const char *pszGetSerialPortName( struct t_io_dev *dev )
{
  const char *pszCurrentPort;
  if ( &dev_com2 == dev )
  {
    pszCurrentPort = "dev_com2";
  }
  else if ( &dev_com3 == dev )
  {
    pszCurrentPort = "dev_com3";
  }
  else if ( &dev_com4 == dev )
  {
    pszCurrentPort = "dev_com4";
  }
  else
  {
    pszCurrentPort = "unknown";
  }
  return pszCurrentPort;
}


void TestSplitSio(struct t_io_dev *dev)
{
    unsigned char ch = '\0';
    struct t_io_dev *p_sio_dev = dev;
    const char *pszCurrentPort;

    while (ch != 'Q')
    {
        pszCurrentPort = pszGetSerialPortName( p_sio_dev );
        rfprintf(dev, "Split Serial Test. Selected port is %s\r\n"
                      "S - Select port, F - Flood Split Serial, C - Configure port\r\n"
                      "E - Echo on port, D - Dump port config\r\n"
                      "W - Write port, Q - Quit\r\n",
                 pszCurrentPort);

        rread(dev, &ch, 1, 0);
        switch (ch)
        {
            case 'S':
                if (p_sio_dev != dev)
                {
                  rclose( p_sio_dev );
                }
                p_sio_dev = SelectPort(dev);
                if (p_sio_dev != dev)
                {
                  ropen( p_sio_dev, 0 );
                }
                break;
            case 'F':
                FloodSplitSio(dev);
                break;
            case 'C':
                ConfigureSerial(dev, p_sio_dev);
                break;
            case 'D':
                SerialDumpConfiguration(dev, p_sio_dev);
                break;
            case 'E':
                EchoSerialPort(dev, p_sio_dev);
                break;
            case 'W':
                WriteSerialPort(dev, p_sio_dev);
                break;
        }
    }
}


#endif


void TestSio3Cts(struct t_io_dev *dev, struct t_io_dev *sio_dev)
{
    t_ioctl_siocfg siocfg;
    unsigned char ch;

    rfprintf(dev, "Test SIO3 CTS Pin.\r\nOpening COM3, and configuring to use CTS.\r\n");

    ropen(sio_dev, 1);

    rfprintf(dev, "Transmitting on COM3, with CTS disabled, press any key to stop.\r\n");
    while (1)
    {
        rfprintf(dev, "Sending on COM3\r\n");
        rfprintf(sio_dev, "Data on COM3!!\r\n");
        if (!reof(dev))
            break;
        OSTimeDly(10);
    }
    rread(dev, &ch, 1, 0);

    siocfg.baud = 57600UL;
    siocfg.data_bits = 8;
    siocfg.parity_bits = 0;
    siocfg.stop_bits = 1;
    siocfg.use_cts = 1;
    rioctl(sio_dev, IOCTL_SER_SET_CONFIG, &siocfg);

    rfprintf(dev, "Testing CTS. Transmitting on COM3, press any key to stop.\r\n");
    while (1)
    {
        rfprintf(dev, "Sending on COM3.\r\n");
//        rfprintf(dev, "CTS=%d\r\n", (P_PA.PORT.BIT.B3 == 1));
        rfprintf(sio_dev, "Data on COM3!!\r\n");
        if (!reof(dev))
            break;
        OSTimeDly(10);
    }
    rread(dev, &ch, 1, 0);

    rfprintf(dev, "Clearing CTS setting on COM3 and closing port\r\n");
    siocfg.use_cts = 0;
    rioctl(sio_dev, IOCTL_SER_SET_CONFIG, &siocfg);
    rclose(sio_dev);
}
