#include <rinhw/rstdio.h>
#include <rinhw/ucos_ii.h>
#include <rinhw/rio.h>
#include <rinhw/sio.h>
#include <rinhw/hw_vers.h>
#include <rinhw/flash2238.h>
#include <rinhw/debug.h>
#include "adctest.h"
#include "adc24bittest.h"
#include "buztest.h"
#include "flashtest.h"
#include "framtest.h"
#include "iictest.h"
#include "kbdtest.h"
#include "lcdtest.h"
#include "rtctest.h"
#include "printftest.h"
#include "irtest.h"
#include "pwrtest.h"
#include "splitsiotest.h"
#include "shenzhenlcdtest.h"
#include "stkchktest.h"
#include "remotetest.h"
#include "ostest.h"
#include "devices.h"
#include "siotest.h"
#include "bkgrndtest.h"
/*#include "AllTests.h"*/


#define TEST_DEV dev_console


void test_debug_printf_fn(int priority, const char *format, va_list args)
{
  rvfprintf(&TEST_DEV, format, args);
}


typedef void (*t_testfn_ptr)(struct t_io_dev *iodev, struct t_io_dev *testdev);


struct t_test
{
    char shortcut;
    char *name;
    t_testfn_ptr testfn;
    struct t_io_dev *testdev;
};

struct t_test tests[] =
{
    { 'B', "Buzzer", TestBuzzer, NULL },
    { 'b', "Background Timer Test", TestBackgroundTimer, NULL },
    { 'I', "IIC0 (Bus)", TestIIC, &dev_iic0 },
    { 'K', "Keyboard", TestKeyboard, NULL },
    { 'k', "Keyboard Buffer Write", TestKeyboardWrite, NULL },
    { 'L', "LCD Tests", lcd_test, NULL },
    { 'R', "Real Time Clock", TestRTC, &dev_rtc },
    { 'O', "Opto Serial Link", TestIRHead, NULL },
    { 'o', "Operating System Tests", TestOS, NULL },
    { 'A', "CPU ADC", TestuPAdc, NULL },
    { 'P', "Printf", TestPrintf, NULL },
    { 'W', "Power Test", TestPower, NULL },
    { 'c', "COM3 CTS Test", TestSio3Cts, &dev_ser2 },
    /* { 'T', "Split Serial Com3 Test", TestSplitSio, NULL }, R400 only */
    { 'M', "IIC Module Test", TestIICModule, NULL },
    { 'f', "2238 Flash Memory Program/Erase Test", TestFlash2238, NULL },
    /* { 's', "Stack Check Test", TestStackCheck, NULL }, */
//    { 'r', "FRAM Test", TestFram },
    { 'm', "FRAM Test", TestFramDev, &dev_fram },
    { 'a', "24bit ADC Test", TestAdc24Bit, &dev_adc24bit },
    { 'z', "Concurrent Remote IO Test", TestRemote, NULL },
    { 'u', "UDP Datagram Serial Test", TestSio, &dev_udp_datagram },
    { 't', "TCP Stream Serial Test", TestSio, &dev_tcp_stream },
    { '1', "Serial 1 Test", TestSio, &dev_ser1 },
    { '2', "Serial 2 Test", TestSio, &dev_ser2 },
    { 'U', "USB Serial Test", TestUsbSio, NULL },
    { '3', "Pipe 1 Serial Test", TestSio, &dev_pipe1 },
    { '4', "Pipe 2 Serial Test", TestSio, &dev_pipe2 },
//    { 'U', "R400 HW Unit Tests", RunAllUnitTests },
    { '\0', "", NULL, NULL }
};


void PrintTests(struct t_io_dev *f, struct t_test tests[])
{
    unsigned char i = 0;

    while (tests[i].shortcut != '\0')
    {
        rfprintf(f, "  %c - %s\r\n", tests[i].shortcut, tests[i].name);
        i++;
    }
    rfprintf(f, "  Q - Quit\r\n");
}


t_testfn_ptr GetTestFn(struct t_test tests[], unsigned char ch, struct t_io_dev **devptr)
{
    unsigned char i = 0;

    while (tests[i].shortcut != '\0')
    {
        if (tests[i].shortcut == ch)
        {
            *devptr = tests[i].testdev;
            return tests[i].testfn;
        }
        i++;
    }
    return (t_testfn_ptr) NULL;
}



unsigned char GetBlockChar(struct t_io_dev *dev)
{
    unsigned char ch;

    rread(dev, &ch, 1, 0);
    return ch;
}


/*
 * main -
 *
 * This is where your user program starts.
 */
int main(int argc, char *argv[])
{
  unsigned char ch;
  t_testfn_ptr testfn;
  t_ioctl_siocfg sercfg;
  unsigned short level;
  struct t_io_dev *testdev;

  if (init_test_app_devices() < 0)
  {
    return 1;
  }

  /* Open debug device if not already open */
  if (TEST_DEV.status & O_OPEN)
  {
    ropen(&TEST_DEV, 0);
  }
  rinhw_set_debug_printf_fn(test_debug_printf_fn);

  /* Wait for data to flush from TX buffer before re-configuring */
  do
  {
    level = 0;
    rioctl(&TEST_DEV, IOCTL_SER_GET_TX_BUF_LEVEL, &level);
    if (level > 0)
    {
      OSTimeDly(OS_TICKS_PER_SEC / 10);
    }
  }
  while (level > 0);

  /* Reconfigure serial port */
  sercfg.baud = 57600;
  sercfg.data_bits = 8;
  sercfg.parity_bits = 0;
  sercfg.stop_bits = 1;
  sercfg.use_cts = 0;
  sercfg.use_terminations = 0;
  rioctl(&TEST_DEV, IOCTL_SER_SET_CONFIG, &sercfg);

  rfprintf(&TEST_DEV, "Hardware Library Version : %s\n", HwLibDrawingNumberStr);
/*    rfprintf(&TEST_DEV, "Power On Key : %d\n", key); */

  while (1)
  {
    rfprintf(&TEST_DEV, "\r\n\r\nR400 Device Testing\r\n\r\nTest options are:\r\n");
    PrintTests(&TEST_DEV, tests);
    rfprintf(&TEST_DEV, ">");
    ch = GetBlockChar(&TEST_DEV);
    if ('Q' == ch)
    {
      break;
    }
    testfn = GetTestFn(tests, ch, &testdev);
    if (testfn != NULL)
      testfn(&TEST_DEV, testdev);
    else
      rfprintf(&TEST_DEV, "Unknown command %c.\r\n", ch);
  }

  rclose(&TEST_DEV);

  destroy_test_app_devices();
  return 0;
}
