#include <rinhw/rstdio.h>

#include "printftest.h"


void TestPrintf(struct t_io_dev *dev, struct t_io_dev *dummy)
{
    unsigned short i = 1234;
    signed short si = -5678;
    unsigned long l = 12345678;
    signed long sl = -87654321;
    unsigned char c = 'C';
    unsigned short h = 0x12A5;
    unsigned long lh = 0x1234ABCD;
    unsigned char *ptr = (unsigned char *) 0;
    char str[20];
    int count;
    struct t_io_dev *f = dev;

    rfprintf(f, "\r\nTestPrintf Test\r\nBasic Test (no parameters)\r\n");
    rfprintf(f, "Char and String Parameters: %%, %%c C=%c, %%s Hello=%s\r\n", c, "Hello");

    rfprintf(f, "Positive 16bit Integers: %%d 1234=%d, %%i 1234=%i, %%u 1234=%u\r\n", i, i, i);
    rfprintf(f, "Negative 16bit Integers: %%d -5678=%d, %%i -5678=%i, %%u 59858=%u\r\n", si, si, si);
    rfprintf(f, "Hexadecimal, Octal and Binary Parameters: %%x 12A5=%x, %%o 11245=%o, %%b 1001010100101=%b\r\n", h, h, h);

    rfprintf(f, "Positive Long Parameters: %%ld 12345678=%ld, %%li 12345678=%li, %%lu 12345678=%lu\r\n", l, l, l);
    rfprintf(f, "Negative Long Parameters: %%ld -87654321=%ld, %%li -87654321=%li, %%lu 4207312975=%lu\r\n", sl, sl, sl);
    rfprintf(f, "Long Hexadecimal, Octal and Binary Parameters: %%lx 1234ABCD=%lx, %%lo 2215125715=%lo, %%lb 10010001101001010101111001101=%lb\r\n", lh, lh, lh);

    rfprintf(f, "Width and Zero Padded Integer: %%6d '  1234'='%6d', %%06d 001234=%06d\r\n", i, i);
    rfprintf(f, "Width and Precision Specified Integer: %%8.6d '  001234'='%8.6d'\r\n", i);

    rfprintf(f, "Width and Precision Argument Specified Integer: %%*.*d '  001234'='%*.*d'\r\n", 8, 6, i);
    rfprintf(f, "Precision Larger than Width: %%6.8d '00001234'='%6.8d'\r\n", 1234);

    rfprintf(f, "With Positive as '+', ' ': %%+d +1234=%+d, %% d ' 1234'='% d'\r\n", 1234, 1234);
    rfprintf(f, "With Positive as '+', but not long enough: %%+4d +1234=%+4d\r\n", 1234);
    rfprintf(f, "With Positive as '+', and zero padding: %%+06d +01234=%+06d\r\n", 1234);
    rfprintf(f, "With Positive as '+', and space padding: %%+6d ' +1234'='%+6d'\r\n", 1234);

    rfprintf(f, "Hexadecimal with Positive as '+': %%+x 1234=%+x\r\n", 0x1234);

    rfprintf(f, "Hexadecimal, Octal, Binary with Alternate form: %%#x 0x12A5=%#x, %%#o 011245=%#o, %%#b 0b1001=%#b\r\n", 0x12A5, 0x12A5, 9);
    rfprintf(f, "Hexadecimal with Alternate form and Width: %%#08x 0x0012A5=%#08x, %%#8x '  0x12A5'='%#8x'\r\n", 0x12A5, 0x12A5);

    rfprintf(f, "Precision unspecified: %%06.d '  1234'='%06.d'\r\n", 1234);
    rfprintf(f, "Width unspecified: %%.6d 001234=%.6d\r\n", 1234);

    count = rfprintf(f, "Test Length");
    rfprintf(f, ": count 11 = %d\r\n", count);

    rfprintf(f, "Oddball width/precision: %%.d 1234=%.d, %%.0d 1234=%.0d\r\n", 1234, 1234);
    rfprintf(f, "Normal width/precision with args = 0: %%d '0'='%d', %%6d '     0'='%6d', %%6.3d '   000'='%6.3d'\r\n", 0, 0, 0);
    rfprintf(f, "Oddball width/precision with args = 0: %%.d ''='%.d', %%6.d '      '='%6.d', %%06.d '      '='%06.d'\r\n", 0, 0, 0);

    rfprintf(f, "Normal width/precision with args = 1: %%d '1'='%d', %%6d '     1'='%6d', %%6.3d '   001'='%6.3d'\r\n", 1, 1, 1);
    rfprintf(f, "Oddball width/precision with args = 1: %%.d '1'='%.d', %%6.d '     1'='%6.d', %%06.d '     1'='%06.d'\r\n", 1, 1, 1);

    rfprintf(f, "Pointer: %%p 0=%p\r\n", ptr);

    rfprintf(f, "Precision and Width on String Parameters: %%6.3s '   ABC'='%6.3s'\r\n", "ABCDEF");

    count = rsprintf(str, "Hello %s %d", "There", 1234);
    rfprintf(f, "sprintf: count 16=%d 'Hello %%s %%d'='Hello There 1234'='%s'\r\n", count, str);

#if 0
    rfprintf(f, "Weight Printf %6.2lw\r\n", 123456L);
    rfprintf(f, "Weight Printf %6.4d\r\n", 12);
    rfprintf(f, "Weight Printf %6.4w\r\n", 12);
    rfprintf(f, "Weight Printf %06.4w\r\n", 12);
    rfprintf(f, "Weight Printf %6.04w\r\n", 12);
#endif

    rfprintf(f, "TestPrintf Test Finished\r\n");
}
