/*
 * pthread_test.c
 *
 *  Created on: Jun 12, 2015
 *      Author: ian
 */
#define _GNU_SOURCE
#include <semaphore.h>
#include <time.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>

#define NANO_SEC_PER_SEC 1000000000

sem_t semaphore;


/** pts1' = pts1 + pts2 */
static void AddTimespec(struct timespec *pts1, const struct timespec *pts2)
{
  const unsigned long int ns = pts1->tv_nsec + (unsigned long int)pts2->tv_nsec;
  pts1->tv_sec += ns / NANO_SEC_PER_SEC;
  pts1->tv_nsec = ns % NANO_SEC_PER_SEC;
  pts1->tv_sec += pts2->tv_sec;
}


double TimespecToDouble(const struct timespec *a)
{
  return a->tv_sec + a->tv_nsec / 1e9;
}


void OSSemPend2(sem_t *sem)
{
  int res;
  struct timespec ts_now, ts_delay, ts_end;
  struct timespec ts_meas_start, ts_meas_end;
  double time_delta;

  printf("Starting sem_timedwait test\n");
  while (1)
  {
    clock_gettime(CLOCK_MONOTONIC, &ts_meas_start);

    /* Pend on semaphore with timeout */
    if (clock_gettime(CLOCK_REALTIME, &ts_now) < 0)
    {
      printf("clock_gettime() failed\n");
      return;
    }
    ts_delay.tv_sec = 0;
    ts_delay.tv_nsec = 50000000;
    ts_end = ts_now;
    AddTimespec(&ts_end, &ts_delay);

    res = sem_timedwait(sem, &ts_end);

    clock_gettime(CLOCK_MONOTONIC, &ts_meas_end);
    time_delta = TimespecToDouble(&ts_meas_end) - TimespecToDouble(&ts_meas_start);
    if (time_delta > 5.0)
    {
      printf("OSSemPend2(): sem_timedwait(sem=%p, to=%ld.%09ld) took %f seconds\n", sem, (long) ts_end.tv_sec, (long) ts_end.tv_nsec, time_delta);
      printf("OSSemPend2(): ts_start=%ld.%09ld ts_delay=%ld.%09ld ts_end=%ld.%09ld\n",
             (long) ts_now.tv_sec, (long) ts_now.tv_nsec, (long) ts_delay.tv_sec, (long) ts_delay.tv_nsec, (long) ts_end.tv_sec, (long) ts_end.tv_nsec);
      printf("OSSemPend2(): ts_meas_start=%ld.%09ld ts_meas_end=%ld.%09ld\n",
             (long) ts_meas_start.tv_sec, (long) ts_meas_start.tv_nsec, (long) ts_meas_end.tv_sec, (long) ts_meas_end.tv_nsec);
    }

    /* Sort out errors */
    if (res < 0)
    {
      if (ETIMEDOUT != errno)
      {
        printf("Error other than ETIMEDOUT: %d %s\n", errno, strerror(errno));
        break;
      }
    }
    else
    {
      printf("Sempahore was signaled\n");
      break;
    }
  }
}


int main(int argc, char *argv[])
{
  if (sem_init(&semaphore, 0, 0) != 0)
  {
    printf("sem_init failed\n");
    return -1;
  }

  OSSemPend2(&semaphore);

  sem_destroy(&semaphore);
  return 0;
}
