/*
 * rin_thread_test.c
 *
 *  Created on: Jun 29, 2016
 *      Author: ian
 */
#include <pthread.h>
#include <errno.h>
#include <cutest/CuTest.h>
#include <rinhw/rinthread.h>

#include "rin_thread_test.h"


static pthread_t test_thread;


typedef void *(*pthread_start_routine_t)(void *);


static void *test_thread_function_simple(void *arg)
{
  rin_thread_exit(arg);
}


static void test_rin_thread_create_bad_param(CuTest *tc)
{
  /* Should fail, thread = NULL */
  CuAssertTrue(tc, !rin_thread_create(NULL, test_thread_function_simple, NULL, true));

  /* Should fail, start_routine = NULL */
  CuAssertTrue(tc, !rin_thread_create(&test_thread, NULL, NULL, true));
}


static void test_rin_thread_create_thread(CuTest *tc)
{
  void *retval = NULL;
  char data;
  void *arg;

  /* Create thread */
  arg = &data;
  CuAssertTrue(tc, rin_thread_create(&test_thread, test_thread_function_simple, arg, true));

  /* Join thread */
  CuAssertIntEquals(tc, 0, pthread_join(test_thread, &retval));

  /* retval should equal arg */
  CuAssertPtrEquals(tc, arg, retval);
}


/** Tests for the rin thread pthread wrappers */
CuSuite *CuGetSuite_rin_thread(void)
{
  CuSuite *suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_rin_thread_create_bad_param);
  SUITE_ADD_TEST(suite, test_rin_thread_create_thread);
  return suite;
}
