/*
 * driverconfig.c
 *
 *  Created on: 19/09/2009
 *      Author: ian
 */
#include <string.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include "ezxml.h"
#include "debug.h"
#include "power.h"

#include "driverconfig.h"


static ezxml_t ezxml_hwlib_config = (ezxml_t) 0;


static const ezxml_t xmlHwLibXMLConfig_GetDriver( const char *pszDriverName )
{
  ezxml_t drivers = ezxml_get( ezxml_hwlib_config, "drivers", -1 );
  ezxml_t driver;
  const char *pszCurrentDriverName;

  for ( driver = ezxml_child( drivers, "driver" ); ( (ezxml_t) 0 != driver ); driver = driver->next )
  {
    pszCurrentDriverName = ezxml_attr( driver, "name" );
    if ( 0 == strcmp( pszCurrentDriverName, pszDriverName) )
    {
      return driver;
    }
  }
  return (ezxml_t) 0;
}


bool bHwLibXMLConfig_GetRemoteIoSerialPort( char *pszSerialPort, int nMaxLength )
{
  ezxml_t xml_serport = ezxml_get( ezxml_hwlib_config, "remoteio", 0, "serialport", -1 );
  bool bResult = false;

  if ((ezxml_t) 0 != xml_serport)
  {
    strncpy( pszSerialPort, xml_serport->txt, nMaxLength );
    pszSerialPort[ nMaxLength - 1 ] = '\0';
    bResult = true;
  }
  return bResult;
}


bool bHwLibXMLConfig_GetRemoteIoSocket( char *pszServer, int nMaxLength, int *pnPort )
{
  ezxml_t xml_serport = ezxml_get( ezxml_hwlib_config, "remoteio", 0, "socket", -1 );
  bool bResult = false;
  const char *colonposn;
  int len;
  int i;

  if ((ezxml_t) 0 != xml_serport)
  {
    /* Of the form server:port eg blah.com.au:7020 or 172.17.9.1:7020 */
    colonposn = strchr(xml_serport->txt, ':');
    if (0 != colonposn)
    {
      len = (int) (colonposn - xml_serport->txt);
      for (i = 0 ; i < len && i < nMaxLength - 1; i++)
      {
        pszServer[i] = xml_serport->txt[i];
      }
      pszServer[i] = '\0';
      *pnPort = atoi(&colonposn[1]);
      bResult = true;
    }
  }
  return bResult;
}


bool bHwLibXMLConfig_GetRemoteIoEnabled( void )
{
  bool bResult = false;
  ezxml_t xml_serport = ezxml_get( ezxml_hwlib_config, "remoteio", 0, "enabled", -1 );

  if ( (ezxml_t) 0 != xml_serport )
  {
    if ( 0 == strcmp( xml_serport->txt, "true") )
    {
      bResult = true;
    }
  }
  return bResult;
}


void vHwLibXMLConfig_GetWe21HardwareDevice( char *pszDevice, int nMaxLength )
{
  ezxml_t xml_device = ezxml_get( ezxml_hwlib_config, "we21hw", 0, "device", -1 );
  const char *pszData = ( (ezxml_t) 0 != xml_device ) ? xml_device->txt : "";

  strncpy( pszDevice, pszData, nMaxLength );
  pszDevice[ nMaxLength - 1 ] = '\0';
}


bool bHwLibXMLConfig_GetDriverIsRemote( const char *pszDriverName )
{
  bool bResult = false;
  ezxml_t driver;
  ezxml_t remote;

  driver = xmlHwLibXMLConfig_GetDriver( pszDriverName );
  if ( (ezxml_t) 0 != driver )
  {
    remote = ezxml_get( driver, "remote", -1 );
    if ( (ezxml_t) 0 != remote )
    {
      if ( 0 == strcmp( remote->txt, "true") )
      {
        bResult = true;
      }
    }
  }
  return bResult;
}


bool bHwLibXMLConfig_GetDriverSettingString( const char *pszDriverName, const char *pszSetting, char *pszValue, int nMaxLength )
{
  bool bResult = false;
  ezxml_t driver;
  ezxml_t setting;

  /* Get driver XML entry for the driver named pszDriverName */
  driver = xmlHwLibXMLConfig_GetDriver( pszDriverName );
  if ( (ezxml_t) 0 != driver )
  {
    /* Get the setting under driver we are after */
    setting = ezxml_get( driver, pszSetting, -1 );
    if ( (ezxml_t) 0 != setting )
    {
      /* Copy it's text into the return string */
      if ( (char *) 0 != setting->txt )
      {
        strncpy( pszValue, setting->txt, nMaxLength );
        pszValue[ nMaxLength - 1 ] = '\0';
        bResult = true;
      }
    }
  }
  return bResult;
}


bool bHwLibXMLConfig_GetDriverSettingInteger( const char *pszDriverName, const char *pszSetting, int *pnValue )
{
  #define MAX_INT_LEN 30
  char szValue[ MAX_INT_LEN ];
  bool bResult = false;

  if (bHwLibXMLConfig_GetDriverSettingString( pszDriverName, pszSetting, szValue, MAX_INT_LEN ) )
  {
    bResult = true;
    *pnValue = atoi( szValue );
  }
  return bResult;
}


bool bHwLibXMLConfig_GetSerialDriverLocalEmulation( const char *pszDriverName, enum eXMLConfigSerialDriverLocalEmulation *pLocalEmulation,
                                                    char *pszSetting, int nMaxLength )
{
  bool bResult = false;
  ezxml_t driver;
  ezxml_t setting;
  ezxml_t locemulsetting;
  const char *pszData;
  const char *pszLocEmulSetting;

  /* Get driver XML entry for the driver named pszDriverName */
  driver = xmlHwLibXMLConfig_GetDriver( pszDriverName );
  if ( (ezxml_t) 0 != driver )
  {
    /* Get the local emulation setting under driver we are after */
    setting = ezxml_get( driver, "localemulation", -1 );
    if ( (ezxml_t) 0 != setting )
    {
      pszData = ezxml_attr( setting, "type" );
      if ( 0 == strcmp( "serial", pszData ) )
      {
        *pLocalEmulation = leSerial;
        pszLocEmulSetting = "serialport";
        bResult = true;
      }
      else if ( 0 == strcmp( "socket", pszData ) )
      {
        *pLocalEmulation = leSocket;
        pszLocEmulSetting = "socket";
        bResult = true;
      }
      else if ( 0 == strcmp( "file", pszData ) )
      {
        *pLocalEmulation = leFile;
        pszLocEmulSetting = "file";
        bResult = true;
      }

      /* Now get the local emulation device setting (<serialport>, <socket> or <file>) */
      if ( bResult )
      {
        locemulsetting = ezxml_get( setting, pszLocEmulSetting, -1 );
        if ( (ezxml_t) 0 != locemulsetting )
        {
          /* Copy it's text into the return string */
          if ( (char *) 0 != locemulsetting->txt )
          {
            strncpy( pszSetting, locemulsetting->txt, nMaxLength );
            pszSetting[ nMaxLength - 1 ] = '\0';
          }
          else
          {
            bResult = false;
          }
        }
        else
        {
          bResult = false;
        }
      }
    }
  }
  return bResult;
}


/** Initialise the XML configuration, pData should point to a pointer to a string containing the configuration file name (const char **filename) */
void vHwLibXMLConfig_Init( void *pData )
{
  const char **ppszfile = pData;
  const char *pszfile = *ppszfile;

#if HW_DEBUG(REMOTE_IO_DEBUG, HW_DEBUG_BASIC)
  DEBUG_PRINTF("vHwLibXMLConfig_Init() called, getting config from %s\n", pszfile);
#endif
  ezxml_hwlib_config = ezxml_parse_file( pszfile );
  if ( (ezxml_t) 0 == ezxml_hwlib_config )
  {
    ERROR_PRINTF("Error with xml configuration file %s\n", pszfile);
    PowerTurnOff();
  }

#if HW_DEBUG(REMOTE_IO_DEBUG, HW_DEBUG_BASIC)
  DEBUG_PRINTF("vHwLibXMLConfig_Init: Result of parsing XML config file %s: %d\n", pszfile, (0 != ezxml_hwlib_config) );
#endif
}


void vHwLibXMLConfig_Destroy( void )
{
  ezxml_free( ezxml_hwlib_config );
}
