#include <string.h>
#include "rio.h"
#include "remote.h"
#include "remoteexec.h"
#include "debug.h"
#include "stdrinstrum.h"
#include "rinendian.h"
#include "debug.h"
#include "remote_adc24bit.h"
#include "adc_ads1216_local.h"
#include "adc_ads1216_remote.h"
#include "driverconfig.h"

#include "adc_ads1216.h"


/*
 * Device function prototypes
 */

/** ADC driver init entry point, setup the hardware and device */
static int adc24bit_init( struct t_io_dev *io_dev );


/*
 * Device struct
 */
struct t_io_dev dev_adc24bit =
{
    (void *) NULL,           /* dev_data */
    0UL,                     /* status */
    0UL,                     /* posn */
    NULL,                    /* sem_open_excl */
    adc24bit_init,           /* dev_init fn */
    NULL,                    /* dev_open fn */
    NULL,                    /* dev_close fn */
    NULL,                    /* dev_read fn */
    NULL,                    /* dev_write fn */
    NULL,                    /* dev_filelength fn */
    NULL,                    /* dev_ioctl fn */
    NULL,                    /* dev_destroy */
    NULL,                    /* dev_isatty */
    NULL,                    /* dev_tell */
    NULL                     /* dev_lseek */
};


/** ADC driver init entry point, setup the hardware and device */
static int adc24bit_init( struct t_io_dev *io_dev )
{
  int nResult;
  #define FILENAME_MAX_LEN 256
  char filename[FILENAME_MAX_LEN];

#if HW_DEBUG(ADC_ADS1216_DEBUG, HW_DEBUG_BASIC)
  DEBUG_PRINTF("adc24bit_init called\n");
#endif

  /*
   * The option of a local ADC driver is only available under Linux,
   * since it has SPI support in the kernel
   */
#if defined(__linux__) /* TODO should we use TARGET == WE2111? */
  /* Check XML config to see if we are using remote IO or file */
  if ( bHwLibXMLConfig_GetDriverIsRemote( DRIVER_NAME_STR( dev_adc24bit ) ) )
  {
    /* Using remote device, so initialise that */
    nResult = rio_create_device_adc24bit_remote(io_dev);
  }
  else
  {
    /* Using local device, fill in device structure appropriately, and initialise */
    /* Get filename from XML config */
    if ( !bHwLibXMLConfig_GetDriverSettingString( "dev_adc24bit", "file", filename, FILENAME_MAX_LEN ) )
    {
  #if HW_DEBUG(FLASH_DEBUG, HW_DEBUG_BASIC)
      DEBUG_PRINTF("flash2238_local_init: failed getting filename from XML config\n");
  #endif
      return RIO_ERROR;
    }
    nResult = rio_create_device_adc24bit_local(io_dev, filename);
  }
#else
  /* Other platforms must use remote device, so initialise that */
  nResult = rio_create_device_adc24bit_remote(io_dev);
#endif

  /* Its a block device */
  io_dev->status &= ~O_DEVCHAR;

#if HW_DEBUG(ADC_ADS1216_DEBUG, HW_DEBUG_BASIC)
  DEBUG_PRINTF("adc24bit_init finished\n");
#endif
  return nResult;
}
