#include <stddef.h>
#include "rio.h"
#include "remote.h"
#include "rinendian.h"
#include "debug.h"
#include "adc_up_local.h"
#include "adc_up_remote.h"
#include "driverconfig.h"

#include "adc_up.h"


/** uP ADC driver init entry point, setup the hardware and device */
static int adc_init( struct t_io_dev *io_dev );


/*
 * Device struct
 */
struct t_io_dev dev_adc =
{
    (void *) NULL,      /* dev_data */
    0UL,                /* status */
    0UL,                /* posn */
    NULL,               /* sem_open_excl */
    adc_init,           /* dev_init */
    NULL,               /* dev_open */
    NULL,               /* dev_close */
    NULL,               /* dev_read */
    NULL,               /* dev_write */
    NULL,               /* dev_filelength */
    NULL,               /* dev_ioctl fn */
    NULL,               /* dev_destroy */
    NULL,               /* dev_isatty */
    NULL,               /* dev_tell */
    NULL                /* dev_lseek */
};


/** uP ADC driver init entry point, setup the hardware and device */
static int adc_init( struct t_io_dev *io_dev )
{
  int nResult;

#if HW_DEBUG(ADC_DEBUG, HW_DEBUG_BASIC)
    DEBUG_PRINTF("adc_init called\n");
#endif

  /* Check XML config to see if we are using remote IO or file */
  if ( bHwLibXMLConfig_GetDriverIsRemote( DRIVER_NAME_STR( dev_adc ) ) )
  {
    /* Using remote device, so initialise that */
    nResult = rio_create_device_adc_remote(io_dev);
  }
  else
  {
    /* Using local device, fill in device structure appropriately, and initialise */
    nResult = rio_create_device_adc_local(io_dev);
  }

  /* Its a block device */
  io_dev->status &= ~O_DEVCHAR;

#if HW_DEBUG(ADC_DEBUG, HW_DEBUG_BASIC)
  DEBUG_PRINTF("adc_init finished\n");
#endif
  return nResult;
}
