#include <stddef.h>
#include "rio.h"
#include "remote.h"
#include "buzzer_local.h"
#include "buzzer_remote.h"
#include "rinendian.h"
#include "debug.h"
#include "driverconfig.h"

#include "buzzer.h"


/** Buzzer driver init entry point, setup the hardware and device */
static int buzzer_init( struct t_io_dev *io_dev );


/*
 * Buzzer device struct
 */
struct t_io_dev dev_buz =
{
    (void *) NULL,      /* dev_data */
    0UL,                /* status */
    0UL,                /* posn */
    NULL,               /* sem_open_excl */
    buzzer_init,        /* dev_init */
    NULL,               /* dev_open */
    NULL,               /* dev_close */
    NULL,               /* dev_read */
    NULL,               /* dev_write */
    NULL,               /* dev_filelength */
    NULL,               /* dev_ioctl fn */
    NULL,               /* dev_destroy */
    NULL,               /* dev_isatty */
    NULL,               /* dev_tell */
    NULL                /* dev_lseek */
};


/** Buzzer driver init entry point, setup the hardware and device */
static int buzzer_init( struct t_io_dev *io_dev )
{
  int nResult;

#if HW_DEBUG(BUZ_DEBUG, HW_DEBUG_BASIC)
    DEBUG_PRINTF("buzzer_init called\n");
#endif

  /* Check XML config to see if we are using remote IO or file */
  if ( bHwLibXMLConfig_GetDriverIsRemote( DRIVER_NAME_STR( dev_buz ) ) )
  {
    /* Using remote device, so initialise that */
    nResult = rio_create_device_buzzer_remote(io_dev);
  }
  else
  {
    /* Using local device, fill in device structure appropriately, and initialise */
    nResult = rio_create_device_buzzer_local(io_dev);
  }

  /* Its a block device */
  io_dev->status &= ~O_DEVCHAR;

#if HW_DEBUG(BUZ_DEBUG, HW_DEBUG_BASIC)
  DEBUG_PRINTF("buzzer_init finished\n");
#endif
  return nResult;
}
