#include <string.h>
#include <stdarg.h>
#include "stdrinstrum.h"
#include "rstdio.h"
#include "rio.h"
#include "ucos_ii.h"
#include "debug.h"
#include "remote.h"
#include "driverconfig.h"
#include "flash2238_local.h"
#include "flash2238_remote.h"

#include "flash2238.h"


/*
 * Function prototypes for device driver calls
 */
static int flash2238_init(struct t_io_dev *io_dev);


/** Declare the flash device */
struct t_io_dev dev_flash2238 =
{
    (void *) 0,                       /* dev_data */
    0UL,                              /* status */
    0UL,                              /* posn */
    NULL,                             /* sem_open_excl */
    flash2238_init,                   /* dev_init */
    NULL,                             /* dev_open */
    NULL,                             /* dev_close */
    NULL,                             /* dev_read */
    NULL,                             /* dev_write */
    NULL,                             /* dev_filelength */
    NULL,                             /* dev_ioctl fn */
    NULL,                             /* dev_destroy */
    NULL,                             /* dev_isatty */
    NULL,                             /* dev_tell */
    NULL                              /* dev_lseek */
};


/** Flash driver init entry point, setup the hardware and device */
static int flash2238_init( struct t_io_dev *io_dev )
{
  int nResult;
  #define FILENAME_MAX_LEN 256
  char filename[FILENAME_MAX_LEN];

#if HW_DEBUG(FLASH_DEBUG, HW_DEBUG_BASIC)
  DEBUG_PRINTF("flash2238_init called\n");
#endif

  /* Check XML config to see if we are using remote IO or local */
  if ( bHwLibXMLConfig_GetDriverIsRemote(DRIVER_NAME_STR(dev_flash2238)))
  {
    /* Using remote device, so initialise that */
    nResult = rio_create_device_flash2238_remote(io_dev);
  }
  else
  {
    /* Get filename from XML config */
    if ( !bHwLibXMLConfig_GetDriverSettingString(DRIVER_NAME_STR(dev_flash2238), "file", filename, FILENAME_MAX_LEN))
    {
  #if HW_DEBUG(FLASH_DEBUG, HW_DEBUG_BASIC)
      DEBUG_PRINTF("flash2238_init: failed getting filename from XML config\n");
  #endif
      return RIO_ERROR;
    }

    /* Using local device, fill in device structure appropriately, and initialise */
    nResult = rio_create_device_flash2238_local(io_dev, filename, FLASH_LENGTH, 0xFF);
  }

  /* Its a block device */
  io_dev->status &= ~O_DEVCHAR;

#if HW_DEBUG(FLASH_DEBUG, HW_DEBUG_BASIC)
  DEBUG_PRINTF("flash2238_init finished\n");
#endif
  return nResult;
}
