#include <stddef.h>
#include "rio.h"
#include "remote.h"
#include "framdev_local.h"
#include "framdev_remote.h"
#include "debug.h"
#include "driverconfig.h"

#include "framdev.h"


/** Size of the FRAM as per R400 */
#define FRAM_MAX_SIZE 65536L


/** FRAM driver init entry point, setup the hardware and device */
static int framdev_init( struct t_io_dev *io_dev );


/** Declare the fram device */
struct t_io_dev dev_fram =
{
    (void *) NULL,      /* dev_data */
    0UL,                /* status */
    0UL,                /* posn */
    NULL,               /* sem_open_excl */
    framdev_init,       /* dev_init */
    NULL,               /* dev_open */
    NULL,               /* dev_close */
    NULL,               /* dev_read */
    NULL,               /* dev_write */
    NULL,               /* dev_filelength */
    NULL,               /* dev_ioctl fn */
    NULL,               /* dev_destroy */
    NULL,               /* dev_isatty */
    NULL,               /* dev_tell */
    NULL                /* dev_lseek */
};


/** FRAM driver init entry point, setup the hardware and device */
static int framdev_init( struct t_io_dev *io_dev )
{
  int nResult;
  #define FILENAME_MAX_LEN 256
  char filename[FILENAME_MAX_LEN];

#if HW_DEBUG(FRAM_DEBUG, HW_DEBUG_BASIC)
    DEBUG_PRINTF("framdev_init called\n");
#endif

  /* Check XML config to see if we are using remote IO or file */
  if ( bHwLibXMLConfig_GetDriverIsRemote( DRIVER_NAME_STR( dev_fram ) ) )
  {
    /* Using remote device, so initialise that */
    nResult = rio_create_device_fram_remote(io_dev);
  }
  else
  {
    /* Local - Get filename from XML config */
    if ( !bHwLibXMLConfig_GetDriverSettingString( "dev_fram", "file", filename, FILENAME_MAX_LEN ) )
    {
  #if HW_DEBUG(FRAM_DEBUG, HW_DEBUG_BASIC)
      DEBUG_PRINTF("fram_local_open: failed getting filename from XML config\n");
  #endif
      return RIO_ERROR;
    }

    /* Using local device, fill in device structure appropriately, and initialise */
    nResult = rio_create_device_fram_local(io_dev, filename, FRAM_MAX_SIZE, 0x00);
  }

  /* Its a block device */
  io_dev->status &= ~O_DEVCHAR;

#if HW_DEBUG(FRAM_DEBUG, HW_DEBUG_BASIC)
  DEBUG_PRINTF("framdev_init finished, result %d\n", nResult);
#endif
  return nResult;
}
