#include <stdarg.h>
#include "rstdio.h"
#include "ucos_ii.h"
#include "rio.h"
#include "remote.h"
#include "debug.h"
#include "remote_iic.h"
#include "remoteexec.h"
#include "rinendian.h"
#include "iic_local_linux.h"
#include "iic_remote.h"
#include "driverconfig.h"
#include "we21hw.h"

#include "iic.h"


/*
 * Function prototypes for device driver calls
 */
static int iic_init( struct t_io_dev *io_dev );


/*
 * Define to create IIC device for each port
 */
#define DEFINE_IICIO(port)                                             \
static struct s_remote_device_info iic_remote_device_info ## port =    \
{                                                                      \
  REMOTE_DEV_IIC0 + port,                      /* device_id */         \
};                                                                     \
struct t_io_dev dev_iic ## port =                                      \
{                                                                      \
    (void *) &iic_remote_device_info ## port,  /* dev_data */          \
    0UL,                                       /* status */            \
    0UL,                                       /* posn */              \
    NULL,                                      /* sem_open_excl */     \
    iic_init,                                  /* dev_open fn */       \
    NULL,                                      /* dev_open fn */       \
    NULL,                                      /* dev_close fn */      \
    NULL,                                      /* dev_read fn */       \
    NULL,                                      /* dev_write fn */      \
    NULL,                                      /* dev_filelength fn */ \
    NULL,                                      /* dev_ioctl fn */      \
    NULL,                                      /* dev_destroy */       \
    NULL,                                      /* dev_isatty */        \
    NULL,                                      /* dev_tell */          \
    NULL                                       /* dev_lseek */         \
}


/*
 * Declare each port if included in build
 */
#if (INCLUDE_IIC0 != 0)
  DEFINE_IICIO(0);
#endif
#if (INCLUDE_IIC1 != 0)
  DEFINE_IICIO(1);
#endif


static int setup_local_iic_device(struct t_io_dev *io_dev, const char *device_name)
{
  #define FILENAME_MAX_LEN 256
  char filename[ FILENAME_MAX_LEN ];

#if HW_DEBUG(IIC_DEBUG, HW_DEBUG_BASIC)
  DEBUG_PRINTF("iic_local_init called\n");
#endif

  /* Get filename from XML config */
  if ( !bHwLibXMLConfig_GetDriverSettingString( device_name, "file", filename, FILENAME_MAX_LEN ) )
  {
  #if HW_DEBUG(FRAM_DEBUG, HW_DEBUG_BASIC)
    DEBUG_PRINTF("iic_local_init: failed getting filename from XML config\n");
  #endif
    return RIO_ERROR;
  }

  return rio_create_device_iic_local(io_dev, filename, we21hw_set_slot_select, NULL);
}


/** IIC driver init entry point, setup the hardware and device */
static int iic_init( struct t_io_dev *io_dev )
{
#if 0 /* TODO iic_init */
  int nResult;
  #define DEVNAME_MAX_LEN 20
  char dev_name[ DEVNAME_MAX_LEN ];
  struct s_remote_device_info *remote_info = (struct s_remote_device_info *) io_dev->dev_data;

#if HW_DEBUG(IIC_DEBUG, HW_DEBUG_BASIC)
  DEBUG_PRINTF("iic_init called\n");
#endif

  /* Work out device name for XML config */
  rsprintf( dev_name, "dev_iic%d", remote_info->device_id - REMOTE_DEV_IIC0 );

  /* Check XML config to see if we are using remote IO or local */
  if ( bHwLibXMLConfig_GetDriverIsRemote( dev_name ) )
  {
    /* Using remote device, so initialise that */
    nResult = rio_create_device_iic_remote(io_dev, (struct s_remote_device_info *) io_dev->dev_data);
  }
  else
  {
    /* Using local device, fill in device structure appropriately, and initialise */
    nResult = setup_local_iic_device( io_dev, dev_name );
  }

  /* Its a block device */
  io_dev->status &= ~O_DEVCHAR;

#if HW_DEBUG(IIC_DEBUG, HW_DEBUG_BASIC)
  DEBUG_PRINTF("iic_init finished\n");
#endif
  return nResult;
#endif
  return RIO_ERROR;
}
