#include <stdarg.h>
#include <string.h>
#include "rstdio.h"
#include "ucos_ii.h"
#include "rio.h"
#include "remote.h"
#include "remoteexec.h"
#include "debug.h"
#include "rinendian.h"
#include "kbd_local.h"
#include "kbd_remote.h"
#include "driverconfig.h"
#include "buzzer.h"

#include "kbd.h"
#include "_kbd.h"


/*
 * Device function prototypes
 */
static int kbd_init(struct t_io_dev *io_dev);


/*
 * Keyboard device struct
 */
struct t_io_dev dev_kbd =
{
    (void *) NULL,      /* dev_data */
    0UL,                /* status */
    0UL,                /* posn */
    NULL,               /* sem_open_excl */
    kbd_init,           /* dev_init */
    NULL,               /* dev_open */
    NULL,               /* dev_close */
    NULL,               /* dev_read */
    NULL,               /* dev_write */
    NULL,               /* dev_filelength */
    NULL,               /* dev_ioctl fn */
    NULL,               /* dev_destroy */
    NULL,               /* dev_isatty */
    NULL,               /* dev_tell */
    NULL                /* dev_lseek */
};


/*
 * kbd_init -
 *
 * Keyboard device initialisation
 */
static int kbd_init(struct t_io_dev *io_dev)
{
    int nResult;

#if HW_DEBUG(KBD_DEBUG, HW_DEBUG_BASIC)
    DEBUG_PRINTF("kbd_init called\n");
#endif

    /* Check XML config to see if we are using remote IO or local keyboard */
    if ( bHwLibXMLConfig_GetDriverIsRemote( DRIVER_NAME_STR( dev_kbd ) ) )
    {
      /* Using remote device, so initialise that */
      nResult = rio_create_device_kbd_remote(io_dev);
    }
    else
    {
#if (INCLUDE_BUZZER != 0)
      /* Using local device, fill in device structure appropriately, and initialise */
      nResult = rio_create_device_kbd_local(io_dev, NULL, &dev_buz);
#else
      /* Using local device, fill in device structure appropriately, and initialise */
      nResult = rio_create_device_kbd_local(io_dev, NULL, NULL);
#endif
    }

    /*
     * Its a character device
     */
    io_dev->status |= O_DEVCHAR;

    /*
     * All done
     */
#if HW_DEBUG(KBD_DEBUG, HW_DEBUG_BASIC)
    DEBUG_PRINTF("kbd_init done\n");
#endif
    return nResult;
}


/*
 * BootKeyTranslate -
 *
 * Translate the key bitmap of keys pressed at powerup into
 * a keycode
 */
unsigned char BootKeyTranslate(unsigned long power_on_keys)
{
    union un_key key;

    key.BIT.KEY = (power_on_keys != 0) ? KEY_PHYSICAL_POWER : KEY_PHYSICAL_NOKEY;
    key.BIT.EVENT = (unsigned char) keDown;

    return key.BYTE;
}

unsigned long KeyToMask(unsigned char key)
{
//  return(1UL << KeyToIndex(key));
  return 1;
}
