#include <string.h>
#include "ucos_ii.h"
#include "rio.h"
#include "rstdio.h"
#include "debug.h"
#include "remote.h"
#include "remoteexec.h"
#include "rinendian.h"
#include "driverconfig.h"
#include "lcd_local_skt_r400.h"
#include "lcd_remote.h"

#include "lcd.h"


/** LCD driver init entry point, setup the hardware and device */
static int lcd_init( struct t_io_dev *io_dev );


/*
 * Device structure
 */
struct t_io_dev dev_lcd =
{
    (void *) NULL,      /* dev_data */
    0UL,                /* status */
    0UL,                /* posn */
    NULL,               /* sem_open_excl */
    lcd_init,           /* dev_init */
    NULL,               /* dev_open */
    NULL,               /* dev_close */
    NULL,               /* dev_read */
    NULL,               /* dev_write */
    NULL,               /* dev_filelength */
    NULL,               /* dev_ioctl fn */
    NULL,               /* dev_destroy */
    NULL,               /* dev_isatty */
    NULL,               /* dev_tell */
    NULL                /* dev_lseek */
};


/** LCD driver init entry point, setup the hardware and device */
static int lcd_init(struct t_io_dev *io_dev)
{
  int nResult;
  int sockport;

#if HW_DEBUG(LCD_DEBUG, HW_DEBUG_BASIC)
  DEBUG_PRINTF("lcd_init called\n");
#endif

  /* Check XML config to see if we are using remote IO or file */
  if (bHwLibXMLConfig_GetDriverIsRemote(DRIVER_NAME_STR(dev_lcd)))
  {
    /* Using remote device, so initialise that */
    nResult = rio_create_device_lcd_remote(io_dev);
  }
  else
  {
    if (!bHwLibXMLConfig_GetDriverSettingInteger(DRIVER_NAME_STR(dev_lcd), "socket", &sockport))
    {
#if HW_DEBUG(LCD_DEBUG, HW_DEBUG_BASIC)
      DEBUG_PRINTF("lcd_socket_hardware_init: error getting socket from config XML\n");
#endif
      return RIO_ERROR;
    }

    /* Using local device, fill in device structure appropriately, and initialise */
#if (INCLUDE_KBD != 0)
    nResult = rio_create_device_lcd_r400_local(&dev_lcd_r400, sockport, &dev_kbd);
#else
/*    nResult = rio_create_device_lcd_r400_local(&dev_lcd_r400, sockport, NULL); TODO */
#endif
    if (nResult > 0)
    {
      /* *io_dev = dev_lcd_r400;  TODO */
    }
  }

  /* Its a block device */
  io_dev->status &= ~O_DEVCHAR;

#if HW_DEBUG(LCD_DEBUG, HW_DEBUG_BASIC)
  DEBUG_PRINTF("lcd_init finished\n");
#endif
  return nResult;
}
