#include <stdarg.h>
#include <stdlib.h>
#include "rstdio.h"
#include "config.h"
#include "ucos_ii.h"
#include "rio.h"
#include "debug.h"
#include "remote.h"
#include "remoteexec.h"
#include "rinendian.h"
#include "sio_local_comport.h"
#include "sio_local_file.h"
#include "sio_local_socket.h"
#include "sio_remote.h"
#include "driverconfig.h"

#include "_sio.h"
#include "sio.h"

#if 0 /* TODO sio.c */

/** Serial port driver init entry point, setup the hardware and device */
static int sio_init( struct t_io_dev *io_dev );


/*
 * Struct to define a serial port device
 */
#define DEFINE_COMIO(port)                                    \
struct s_remote_device_info remote_ser_dev_info ## port =     \
{                                                             \
  REMOTE_DEV_SER0 + port,                /* device_id */      \
};                                                            \
struct t_io_dev dev_com ## port =                             \
{                                                             \
  (void *) &remote_ser_dev_info ## port, /* dev_data */       \
  0UL,                                   /* status */         \
  0UL,                                   /* posn */           \
  NULL,                                  /* sem_open_excl */  \
  sio_init,                              /* dev_init */       \
  NULL,                                  /* dev_open */       \
  NULL,                                  /* dev_close */      \
  NULL,                                  /* dev_read */       \
  NULL,                                  /* dev_write */      \
  NULL,                                  /* dev_filelength */ \
  NULL,                                  /* dev_ioctl fn */   \
  NULL,                                  /* dev_destroy */    \
  NULL,                                  /* dev_isatty */     \
  NULL,                                  /* dev_tell */       \
  NULL                                   /* dev_lseek */      \
}


/*
 * Define the serial port device structs
 */
#if (INCLUDE_SERIAL0 != 0)
    DEFINE_COMIO(0);
#endif
#if (INCLUDE_SERIAL1 != 0)
    DEFINE_COMIO(1);
#endif
#if (INCLUDE_SERIAL2 != 0)
    DEFINE_COMIO(2);
#endif
#if (INCLUDE_SERIAL3 != 0)
    DEFINE_COMIO(3);
#endif
#if (INCLUDE_SERIAL4 != 0)
    DEFINE_COMIO(4);
#endif


/** Fill out the io device structure with the appropriate device based on the XML config */
static int sio_create_local_device_from_xml( struct t_io_dev *io_dev, const char *pszDevName )
{
  enum eXMLConfigSerialDriverLocalEmulation LocalEmulation;
  #define LOCALEMULATIONSETTING_MAX_LEN 256
  char szLocEmulSetting[ LOCALEMULATIONSETTING_MAX_LEN ];
  int result;

  /* Figure out local emulation type and setting from XML */
  if ( !bHwLibXMLConfig_GetSerialDriverLocalEmulation( pszDevName, &LocalEmulation, szLocEmulSetting, LOCALEMULATIONSETTING_MAX_LEN ) )
  {
    return RIO_ERROR;
  }

  /* Create appropriate device */
  switch ( LocalEmulation )
  {
    case leSerial:
      result = rio_create_device_sio_localcom( io_dev, szLocEmulSetting, NULL );
      break;
    case leSocket:
      result = rio_create_device_sio_localskt( io_dev, atoi(szLocEmulSetting) );
      break;
    case leFile:
      result = rio_create_device_sio_localfile( io_dev, szLocEmulSetting );
      break;
  }

  return result;
}


/** Serial port driver init entry point, setup the hardware and device */
static int sio_init( struct t_io_dev *io_dev )
{
  int nResult;
  #define DEVNAME_MAX_LEN 20
  char dev_name[ DEVNAME_MAX_LEN ];
  struct s_remote_device_info *remote_info = (struct s_remote_device_info *) io_dev->dev_data;

#if HW_DEBUG(SIO_DEBUG, HW_DEBUG_BASIC)
    DEBUG_PRINTF("sio_init called\n");
#endif

  /* Work out device name for XML config */
  rsprintf( dev_name, "dev_com%d", remote_info->device_id - REMOTE_DEV_SER0 );

  /* Check XML config to see if we are using remote IO or local */
  if ( bHwLibXMLConfig_GetDriverIsRemote( dev_name ) )
  {
    /* Using remote device, so initialise that */
    nResult = rio_create_device_sio_remote(io_dev, remote_info);
  }
  else
  {
    /* Using local device, fill in device structure appropriately */
    nResult = sio_create_local_device_from_xml( io_dev, dev_name );
  }

#if HW_DEBUG(SIO_DEBUG, HW_DEBUG_BASIC)
  DEBUG_PRINTF("sio_init finished\n");
#endif
  return nResult;
}

#endif
