#include "config.h"
#include "_wdt.h"
#include "_power.h"
#include "cmdline.h"

/*
 * The definition of the system main function
 */
extern void sys_main(unsigned long PowerOn_Keys, void (*UserFlashFn)(unsigned char sci));


/*
 * LibIntReset -
 *
 * This is the entry point from the bootloader. The #pragma entry causes
 * the c compiler to set the stack pointer on entry to this function
 * to the top of segment S.
 *
 * PowerOn_Keys is passed through from the bootloader to indicate which
 * keys were pressed at startup.
 */
void LibIntReset(unsigned long PowerOn_Keys, void (*UserFlashFn)(unsigned char sci))
{

    /*
     * Disable interrupts
     */
//    set_imask_ccr(1);

    /*
     * Initialise the DDR table (it must be stored in internal RAM, since external RAM is running yet)
     * Initialise the external address/data bus
     * Initialise the external RAM
     * Initialise the external FLASH
     * Initialise the DMT
     */
//    early_gpio_init();
//    EARLY_INIT_BUS();
//    EARLY_INIT_RAM();
//    EARLY_INIT_FLASH();
    EARLY_INIT_POWER();
    EARLY_INIT_WDT();

    /*
     * Copy initialised data from ROM to RAM,
     * and zero out non-initialised data, uses
     * structs in dbsct.c
     */
//    _INITSCT();

    /*
     * Turn interrupts on
     */
//    set_imask_ccr(0);

    /*
     * Call main to start it all,
     * passing through the keys that were pressed
     */
    sys_main(PowerOn_Keys, UserFlashFn);

    /*
     * If it returns, stop everything
     */
//    sleep();
}


/** This is where it all starts */
int main(int argc, char *argv[])
{
  /* Parse the command line options */
  if (hwlib_parse_commandline(argc, argv) < 0)
  {
    return 1;
  }

  /* Run the system */
  LibIntReset( 0, 0 );
  return 0;
}
