#ifndef RB_DEBUG_DOT_H
#define RB_DEBUG_DOT_H 1


/** Define this to enable debug */
/* #define RINBLOCK_DEBUG 1 */


/** Define this if you want to use the debug printf callback to your application */
/* #define RB_DEBUG_APP_CALLBACK 1 */


#define RB_DEBUG_OFF 0
#define RB_DEBUG_BASIC 1
#define RB_DEBUG_FULL 2


/** Rinblock IIC driver access debugging */
#define RB_IIC_DEBUG_LVL RB_DEBUG_OFF

/** Rinblock messaging debugging */
#define RB_MESSAGE_DEBUG_LVL RB_DEBUG_OFF

/** Rinblock initialisation debugging */
#define RB_STARTUP_DEBUG_LVL RB_DEBUG_OFF

/** Rinblock initialisation debugging  */
#define RB_SHUTDOWN_DEBUG_LVL RB_DEBUG_OFF

/** Rinblock re-programming debugging */
#define RB_FLASH_DEBUG_LVL RB_DEBUG_OFF

/** Rinblock digital IO debugging */
#define RB_DIO_DEBUG_LVL RB_DEBUG_OFF

/** Rinblock Analog Output debugging */
#define RB_AOUT_DEBUG_LVL RB_DEBUG_OFF

/** Rinblock Tilt debugging */
#define RB_TILT_DEBUG_LVL RB_DEBUG_OFF

/** Rinblock serial debugging */
#define RB_SERIAL_DEBUG_LVL RB_DEBUG_OFF

/** Rinblock scratch pad debugging */
#define RB_SCRATCHPAD_DEBUG_LVL RB_DEBUG_OFF

/** Rinblock status debugging */
#define RB_STATUS_DEBUG_LVL RB_DEBUG_OFF

/** Rinblock DSD module debugging */
#define RB_DSD_DEBUG_LVL RB_DEBUG_OFF

/** Rinblock Ethernet debugging */
#define RB_ETHERNET_DEBUG_LVL RB_DEBUG_OFF

/** Rinblock LCD debugging (for WE2111/C500 display) */
#define RB_LCD_DEBUG_LVL RB_DEBUG_OFF

/** LCD local socket debugging (for WE2111/C500 display running as a local socket) */
#define RB_LCD_SKT_DEBUG_LVL RB_DEBUG_OFF

/** Rinblock ADC debugging (for WE2111/C500 connector PCB) */
#define RB_ADC_DEBUG_LVL RB_DEBUG_OFF

/** Rinblock buzzer debugging (for WE2111/C500 display) */
#define RB_BUZ_DEBUG_LVL RB_DEBUG_OFF

/** Rinblock keyboard debugging (for WE2111/C500 display) */
#define RB_KBD_DEBUG_LVL RB_DEBUG_OFF


/*
 * Overall debug flag for Rinblocks
 */
#ifdef RINBLOCK_DEBUG

/*
 * Bring in the debug output device
 */
#include <rinhw/sio.h>
#include <rinhw/rstdio.h>

#include <rinblock/rb_command.h>


void RBDump(unsigned char *ptr, unsigned short length);


#define RB_DEBUG(mod, lvl) (mod >= lvl)


#if RB_DEBUG_APP_CALLBACK


  /** Debugging via callback to application, no need for rindevice */
  #define RB_DEBUG_DEV NULL


  /**
   * Function typedef for calling back to your application for debug printfs
   *
   * @param priority is one of the LOG_xxx constants from syslog.h
   * @param format is a printf format string
   * @param args is an variable argument list to match format
   */
  typedef void (*t_rb_debug_printf_fn)(int priority, const char *format, va_list args);


  /** Set the debug printf function for the rinblock library, call from you application */
  void rb_set_debug_printf_fn(t_rb_debug_printf_fn fn);


  /** Debug printf function for the rinblock library, calls the application implemented handler */
  int rb_debug_printf(struct t_io_dev *dummy, const char *format, ...);
  #define RB_DEBUG_PRINTF rb_debug_printf

#else /* #if RB_DEBUG_APP_CALLBACK */

  /** Debugging on COM3 */
  #define RB_DEBUG_DEV &dev_com3

  #define RB_DEBUG_PRINTF rfprintf
#endif /* #else -> #if RB_DEBUG_APP_CALLBACK */


#else /* #ifdef RINBLOCK_DEBUG */

#define RB_DEBUG(mod, lvl) (0)

#define RB_DEBUG_PRINTF() (0)


#endif /* #else -> #ifdef RINBLOCK_DEBUG */


#endif /* #ifndef RB_DEBUG_DOT_H */
