#ifndef RB_DIO_DOT_H
#define RB_DIO_DOT_H 1

#include <rinblock/rinblock.h>


/*
 * Enum types for configuring IO
 */
enum eDIOType { ioInput = 0, ioOutput };
enum eDIOInversion { ioNormal, ioInverted };


/*
 * RinBlockConfigureIO -
 *
 * Configure the IO on a rinblock, as input/output, inversion,
 * and debounce time (for inputs only). Will work for all rinblocks
 * with IO (input only included).
 *
 * Returns 1 for success, 0 otherwise
 */
unsigned char RinBlockConfigureIO(struct t_rinblock *rb, unsigned char io_bit, enum eDIOType io_type, enum eDIOInversion io_invert, unsigned char debounce_time);


/*
 * RinBlockWriteOutputs -
 *
 * Writes the output data to the rinblock rb. As part of this process
 * it locks the rinblock, and must be unlocked with a call to RinBlockReadInputs()
 * otherwise the system will lock up.
 *
 * Returns 1 for success, 0 otherwise.
 * NOTE only call RinBlockReadInputs() if RinBlockWriteOutputs() has returned 1.
 *
 * It is ok to access other rinblocks while another is locked
 */
unsigned char RinBlockWriteOutputs(struct t_rinblock *rb, union u_rb_output_write *o_data);


/*
 * RinBlockPrimeForRead -
 *
 * Writes the read inputs command to the rinblock rb. As part of this process
 * it locks the rinblock, and must be unlocked with a call to RinBlockReadInputs()
 * otherwise the system will lock up.
 *
 * Returns 1 for success, 0 otherwise.
 * NOTE only call RinBlockReadInputs() if RinBlockPrimeForRead() has returned 1.
 *
 * It is ok to access other rinblocks while another is locked
 */
unsigned char RinBlockPrimeForRead(struct t_rinblock *rb);


/*
 * RinBlockReadInputs -
 *
 * Reads the input data from the rinblock rb. As part of this process
 * it locks the rinblock, and must be unlocked with a call to RinBlockReadInputs()
 * otherwise the system will lock up.
 *
 * Returns 1 for success, 0 otherwise.
 * NOTE only call RinBlockReadInputs() if RinBlockWriteOutputs() has returned 1.
 *
 * It is ok to access other rinblocks while another is locked
 */
unsigned char RinBlockReadInputs(struct t_rinblock *rb, struct s_rb_dio_read *data);

#endif
