#ifndef RB_PROGRAM_DOT_H
#define RB_PROGRAM_DOT_H 1

#include <rinblock/rinblock.h>

/*
 * For Rin block programming errors see rb_flash.h
 */
#include <rinblock/rb_flash.h>


/** Flash start offset */
#define RB_FLASH_START 0UL


/** ATMega 8 Rinblock Firmware Image Length 8k(total)-2k(bootloader) = 6k */
#define RB_FLASH_MEGA8_LENGTH 6144UL


/** ATMega 168 Rinblock Firmware Image Length 16k(total)-2k(bootloader) = 14k */
#define RB_FLASH_MEGA168_LENGTH 14336UL


/** Size of flash blocks passed to the rinblock to be programmed */
#define RB_FLASH_SERIAL_BLOCK_SIZE 0x0080


/** Length of the bootloader identification string */
#define RB_BL_IDEN_NAME_LEN 20


/**
 * Rinblock bootloader identification
 *
 * This whole structure of chk_seed, user_start, user_length, program_name
 * should be a total of 30 bytes long, and is spat out as part of the IDEN
 * command.
 *
 * NOTE: name is not null terminated
 */
struct s_rb_bootloader_identification
{
  unsigned short check_seed;
  unsigned long user_start;
  unsigned long user_length;
  unsigned char name[RB_BL_IDEN_NAME_LEN];
};


/**
 * Get the rinblock bootloader configuration
 *
 * @param rb is a pointer to rinblock that is present (and may be running)
 * @param err is set to an RB_ERR_ code to provide more information on a failure
 * @param iden is a pointer to a bootloader identification struction filled out on
 */
unsigned char RinBlockBootloaderGetConfig(struct t_rinblock *rb, unsigned char *err, struct s_rb_bootloader_identification *iden);


/**
 * Program rinblock rb from buffer buf.
 *
 * NOTE that the work_buffer must be in RAM, otherwise the the buffer can't be encrypted. Will
 * return RB_ERR_FLASH_LOCALBUF_READONLY if this is the case.
 * NOTE also that the contents of the work_buffer will be modified after calling this function
 *
 * @param rb is a pointer to rinblock that is present (and may be running)
 * @param buf is a pointer to a buffer containing the image to be programmed
 * @param err is set to an RB_ERR_ code to provide more information on a failure
 * @param start is the start address within the rinblock
 * @param length is the length of the buffer
 * @param work_buffer is buffer used by the programming routine, and will be modified
 */
unsigned char RinBlockTestProgram(struct t_rinblock *rb, const unsigned char *buf, unsigned char *err,
                                  unsigned long start, unsigned long length, unsigned char work_buffer[RB_FLASH_SERIAL_BLOCK_SIZE]);


#endif /* #ifndef RB_PROGRAM_DOT_H */
