/*
 * rb_we21_lcd.h
 *
 *  Created on: 17/05/2010
 *      Author: ianb
 */

#ifndef RB_WE21_LCD_H_
#define RB_WE21_LCD_H_

#include <rinhw/lcd.h>
#include <rinblock/rinblock.h>


/** HBM WE2111 Digits */
typedef struct
{
  unsigned char Digits[WE21_DISPLAY_NUM_DIGITS];
} tWE21LcdSegments_Digits;


/** HBM WE2111 Units */
typedef struct
{
#if (BYTE_ORDER == BIG_ENDIAN)
  /* Coldfire - big endian */
  unsigned char uu1 : 4;

  unsigned char Grams : 1;
  unsigned char Kilograms : 1;
  unsigned char Pounds : 1;
  unsigned char Tonnes : 1;
#else
  /* i386/AVR - little endian */
  unsigned char Tonnes : 1;
  unsigned char Pounds : 1;
  unsigned char Kilograms : 1;
  unsigned char Grams : 1;

  unsigned char uu1 : 4;
#endif
} tWE21LcdSegments_Units;


/** HBM WE2111 Annunciators */
typedef struct
{
#if (BYTE_ORDER == BIG_ENDIAN)
  /* Coldfire - big endian */
  unsigned char uu1 : 1;
  unsigned char Gross : 1;
  unsigned char Net : 1;
  unsigned char Coz : 1;
  unsigned char uu2 : 4;

  unsigned char Setpoint0 : 1;
  unsigned char Setpoint1 : 1;
  unsigned char Setpoint2 : 1;
  unsigned char Setpoint3 : 1;
  unsigned char Setpoint7 : 1;
  unsigned char Setpoint6 : 1;
  unsigned char Setpoint5 : 1;
  unsigned char Setpoint4 : 1;

  unsigned char uu3 : 5;
  unsigned char Range1 : 1;
  unsigned char uu4 : 1;
  unsigned char Range2 : 1;

  unsigned char uu5 : 8;
#else
  /* i386/AVR - little endian */
  unsigned char uu1 : 4;
  unsigned char Coz : 1;
  unsigned char Net : 1;
  unsigned char Gross : 1;
  unsigned char uu2 : 1;

  unsigned char Setpoint4 : 1;
  unsigned char Setpoint5 : 1;
  unsigned char Setpoint6 : 1;
  unsigned char Setpoint7 : 1;
  unsigned char Setpoint3 : 1;
  unsigned char Setpoint2 : 1;
  unsigned char Setpoint1 : 1;
  unsigned char Setpoint0 : 1;

  unsigned char Range2 : 1;
  unsigned char uu3 : 1;
  unsigned char Range1 : 1;
  unsigned char uu4 : 5;

  unsigned char uu5 : 8;
#endif
} tWE21LcdSegments_Annunciators;


typedef struct
{
  tWE21LcdSegments_Digits Digits;
} tWE21LcdDigits;


typedef struct
{
  unsigned char unused[sizeof(tWE21LcdDigits)];
} tWE21LcdSegments_UnusedAnnunciator;


typedef struct
{
  tWE21LcdSegments_UnusedAnnunciator unused;
  tWE21LcdSegments_Annunciators Segments;
} tWE21LcdAnnunciators;


typedef struct
{
  tWE21LcdSegments_UnusedAnnunciator unused;
  tWE21LcdSegments_Units Segments;
} tWE21LcdUnits;


typedef union
{
  tWE21LcdDigits Digits;
  tWE21LcdUnits Units;
  tWE21LcdAnnunciators Annunciators;
} tWE21LcdNamedSegments;


typedef union
{
  unsigned char Bytes[10];
  tWE21LcdNamedSegments Named;
} tWE21LcdSegments;


/** Create a LCD device for a rinblock with segment flash */
int RinBlockCreateWE21LcdDeviceSegFlash(struct t_io_dev *io_dev, struct t_rinblock *rb);


/** Create a LCD device for a rinblock without segment flashing */
int RinBlockCreateWE21LcdDeviceNoSegFlash(struct t_io_dev *io_dev, struct t_rinblock *rb);


#endif /* RB_WE21_LCD_H_ */
