/***********************************************************************
MODULE:    	CRC
VERSION:   	0.01
CONTAINS:  	Cyclic redundancy checksum calculation
COPYRIGHT: 	Rinstrum Pty Ltd 2004
DATE: 		9 Mar 2004
NOTES:      Many thanks to David Chambers
***********************************************************************/

#include <rinhw/debug.h>
#include <rinblock/crc.h>

/*
 * NAME: CRC16_Calc
 *
 * PARAMETERS: int8u  *pucByte - a pointer to the data to be CRC'ed
 *             int16u uiSize - the number of bytes to calculate for
 *             int16u uiCRC - the initial value for CRC
 *
 * RETURNS:    int16u - the CRC value calculated
 *
 * NOTES:      returns the CRC value (16 bits) for a block of data
 */
int16u CRC16_Calc(int8u *pucByte, int16u uiSize, int16u uiCRC)
{
  /* Calculate CRC-16 value; uses The CCITT-16 Polynomial,
     expressed as X^16 + X^12 + X^5 + 1 */
  int16u uiIndex;
  int8u ucByte;

  #ifdef DEBUG_CRC_FUNC_CALLS
  DebugPutStr("CRC16_Calc\n");
  #endif

  #ifdef DEBUG_CRC_CALC
  DebugPutStr("CRC on:\n");
  DebugPut(pucByte, uiSize);
  #endif

  for(uiIndex = 0; uiIndex < uiSize; ++uiIndex)
  {
    uiCRC ^= (((int16u) pucByte[uiIndex]) << 8);
    for(ucByte = 0; ucByte < 8; ++ucByte)
    {
      if(uiCRC & (int16u) 0x8000)
        uiCRC = (uiCRC << 1) ^ (int16u) 0x1021;
      else
        uiCRC = (uiCRC << 1);
    }
  }

  #ifdef DEBUG_CRC_CALC
  DebugPutStr("\nis=");
  DebugPut((int8u *)&uiCRC, sizeof(uiCRC));
  DebugPutChar('\n');
  #endif

  return uiCRC;
}

