#include <stdarg.h>
#include <string.h>
#include <rinblock/rinblock.h>
#include <rinblock/rb_debug.h>
#include <rinblock/rb_command.h>

#include <rinblock/rb_aout.h>


static unsigned char write_rb_aout_config(struct t_rinblock *rb, const struct s_rb_aout_configure *rb_aout_configure)
{
    RB_DECLARE_CMD_BUFFER( rb_msg, struct s_rb_aout_configure );

    /*
     * Write the config
     */
#if (BYTE_ORDER == NETWORK_ENDIAN)
    rb_msg->data.rb_command.params.rb_aout_configure = *rb_aout_configure;
#else /* #if (BYTE_ORDER == NETWORK_ENDIAN) */
    rb_msg->data.rb_command.params.rb_aout_configure.ucAnalogOutputType = rb_aout_configure->ucAnalogOutputType;
    rb_msg->data.rb_command.params.rb_aout_configure.iUserZero = htons( rb_aout_configure->iUserZero );
    rb_msg->data.rb_command.params.rb_aout_configure.iUserSpan = htons( rb_aout_configure->iUserSpan );
    rb_msg->data.rb_command.params.rb_aout_configure.cMaxLimit = rb_aout_configure->cMaxLimit;
    rb_msg->data.rb_command.params.rb_aout_configure.cMinLimit = rb_aout_configure->cMinLimit;
    rb_msg->data.rb_command.params.rb_aout_configure.uiUpdateRate = htons( rb_aout_configure->uiUpdateRate );
#endif /* #else -> #if (BYTE_ORDER == NETWORK_ENDIAN) */

    return RinBlockExecuteCommand( rb, rb_msg,
                                   RB_GET_SIZE_COMMAND( struct s_rb_aout_configure ),
                                   RB_GET_SIZE_CMD_HDR(),
                                   RB_TIMEOUT_DEFAULT, RB_CMD_WRITE_AO_CONFIG );
}


unsigned char RinBlockAOutReadConfig(struct t_rinblock *rb, struct s_rb_aout_configure *aout_config)
{
  RB_DECLARE_CMD_BUFFER( rb_msg, struct s_rb_aout_configure );
  unsigned char result;

#if RB_DEBUG(RB_AOUT_DEBUG_LVL, RB_DEBUG_BASIC)
  RB_DEBUG_PRINTF(RB_DEBUG_DEV, "RinBlockAOutReadConfig called: ");
#endif

  /*
   * Read the config
   */
  result = RinBlockExecuteCommand( rb, rb_msg,
                                    RB_GET_SIZE_CMD_HDR(),
                                    RB_GET_SIZE_COMMAND( struct s_rb_aout_configure ),
                                    RB_TIMEOUT_DEFAULT, RB_CMD_READ_AO_CONFIG );
  if (result)
  {
#if RB_DEBUG(RB_AOUT_DEBUG_LVL, RB_DEBUG_BASIC)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "success\n");
#endif

#if (BYTE_ORDER == NETWORK_ENDIAN)
    *aout_config = rb_msg->data.rb_response.params.rb_aout_configure;
#else /* #if (BYTE_ORDER == NETWORK_ENDIAN) */
    aout_config->ucAnalogOutputType = rb_msg->data.rb_response.params.rb_aout_configure.ucAnalogOutputType;
    aout_config->iUserZero = ntohs( rb_msg->data.rb_response.params.rb_aout_configure.iUserZero );
    aout_config->iUserSpan = ntohs( rb_msg->data.rb_response.params.rb_aout_configure.iUserSpan );
    aout_config->cMaxLimit = rb_msg->data.rb_response.params.rb_aout_configure.cMaxLimit;
    aout_config->cMinLimit = rb_msg->data.rb_response.params.rb_aout_configure.cMinLimit;
    aout_config->uiUpdateRate = ntohs( rb_msg->data.rb_response.params.rb_aout_configure.uiUpdateRate );
#endif /* #else -> #if (BYTE_ORDER == NETWORK_ENDIAN) */
  }
#if RB_DEBUG(RB_AOUT_DEBUG_LVL, RB_DEBUG_BASIC)
  else
  {
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "failure\n");
  }
#endif
  return result;
}


unsigned char RinBlockAOutWrite(struct t_rinblock *rb, signed long data)
{
    RB_DECLARE_CMD_BUFFER( rb_msg, struct s_rb_aout_access );

#if RB_DEBUG(RB_AOUT_DEBUG_LVL, RB_DEBUG_BASIC)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "RinBlockWriteAOut called\n");
#endif

    /*
     * Setup Aout struct
     */
    rb_msg->data.rb_command.params.rb_aout_access.data = htonl( data );

    /*
     * Write the rinblock
     */
    return RinBlockExecuteCommand( rb, rb_msg,
                                   RB_GET_SIZE_COMMAND( struct s_rb_aout_access ),
                                   RB_GET_SIZE_CMD_HDR(),
                                   RB_TIMEOUT_DEFAULT, RB_CMD_WRITE_ANALOG_OUT );
}


/*
 * CheckRinblockHasAOut -
 *
 * Check whether a rinblock has an analog output. At the moment
 * we only check that the module is an analog output module
 * however, if other modules come into existence, other
 * parameters in aout_config could be checked
 */
static unsigned char CheckRinblockHasAOut(struct t_rinblock *rb, const struct s_rb_aout_configure *aout_config)
{
    UNREFERENCED_PARAMETER( aout_config ); /* Stop compiler warnings on unused parameter */
    return ((rb->tag.interface_type == RB_IFACE_TYPE_AOUT) || (rb->tag.interface_type == RB_IFACE_TYPE_2AOUT) ||
            (rb->tag.interface_type == RB_IFACE_TYPE_3AOUT) || (rb->tag.interface_type == RB_IFACE_TYPE_4AOUT));
}


unsigned char RinBlockAOutWriteConfig(struct t_rinblock *rb, const struct s_rb_aout_configure *aout_config)
{
#if RB_DEBUG(RB_AOUT_DEBUG_LVL, RB_DEBUG_BASIC)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "configure_aout: ");
#endif

    /*
     * Check that the rinblock has an analog output
     * of the type requested
     */
    if (!CheckRinblockHasAOut(rb, aout_config))
    {
#if RB_DEBUG(RB_AOUT_DEBUG_LVL, RB_DEBUG_BASIC)
        RB_DEBUG_PRINTF(RB_DEBUG_DEV, "rinblock does not have an analog output of type requested\n");
#endif
        return 0;
    }

    /*
     * Its all ok, so configure it
     */
    if (!write_rb_aout_config(rb, aout_config))
    {
#if RB_DEBUG(RB_AOUT_DEBUG_LVL, RB_DEBUG_BASIC)
        RB_DEBUG_PRINTF(RB_DEBUG_DEV, "rinblock write analog out configuration failed\n");
#endif
        return 0;
    }

#if 0
    /*
     * Set output to zero
     */
    if (!write_rb_aout_hw(rb, 0, RB_TIMEOUT_DEFAULT))
    {
#if RB_DEBUG(RB_AOUT_DEBUG_LVL, RB_DEBUG_BASIC)
        RB_DEBUG_PRINTF(RB_DEBUG_DEV, "rinblock set analog out to zero failed\n");
#endif
        return 0;
    }
#endif

    /*
     * Configuration done, all good
     */
#if RB_DEBUG(RB_AOUT_DEBUG_LVL, RB_DEBUG_BASIC)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "success, cool!\n");
#endif
    return 1;
}


#if 0
static int aout_config_read(struct t_rb_aout *rb_aout, unsigned char aout_num, struct t_rinblock **rb, struct t_ioctl_aout_config *aout_config)
{
    unsigned char rb_num;

#if RB_DEBUG(RB_AOUT_DEBUG_LVL, RB_DEBUG_BASIC)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "aout_config_read: ");
#endif

    /*
     * Is the analog output to be configured in range
     */
    if (aout_num >= MAX_ANALOG_OUTPUTS)
    {
#if RB_DEBUG(RB_AOUT_DEBUG_LVL, RB_DEBUG_BASIC)
        RB_DEBUG_PRINTF(RB_DEBUG_DEV, "aout_num (%d) out of range (0..%d)\n", aout_num, MAX_ANALOG_OUTPUTS);
#endif
        return -1;
    }

    /*
     * rb == NULL to if analog output not present
     */
    if (rb_aout->aout_rinblocks[aout_num] == NULL)
    {
#if RB_DEBUG(RB_AOUT_DEBUG_LVL, RB_DEBUG_BASIC)
        RB_DEBUG_PRINTF(RB_DEBUG_DEV, "rinblock pointer NULL, no analog output configured\n");
#endif
        *rb = (struct t_rinblock *) NULL;
        memset((void *) aout_config, 0x00, sizeof(struct t_ioctl_aout_config));
        return 1;
    }

    /*
     * Read the config from the rinblock
     */
    *rb = rb_aout->aout_rinblocks[aout_num];
    if (!read_rb_aout_config(*rb, aout_config))
    {
#if RB_DEBUG(RB_AOUT_DEBUG_LVL, RB_DEBUG_BASIC)
        RB_DEBUG_PRINTF(RB_DEBUG_DEV, "rinblock read analog out configuration failed\n");
#endif
        return -1;
    }

    /*
     * Read configuration done, all good
     */
#if RB_DEBUG(RB_AOUT_DEBUG_LVL, RB_DEBUG_BASIC)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "success, cool!\n");
#endif
    return 1;
}
#endif
