#include <rinblock/rinblock.h>
#include <rinblock/rb_debug.h>
#include <rinblock/rb_command.h>

#include <rinblock/rb_aout_cal.h>


/*
 * RinBlockAOutReadFactoryCal -
 *
 * Read back the analog output factory calibration from an (analog) rinblock
 * Only used in the test code
 */
unsigned char RinBlockAOutReadFactoryCal(struct t_rinblock *rb, struct s_rb_aout_cal *aout_cal)
{
    RB_DECLARE_CMD_BUFFER(rb_msg, struct s_rb_aout_cal );
    unsigned char result;

#if RB_DEBUG(RB_AOUT_DEBUG_LVL, RB_DEBUG_BASIC)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "RinBlockAOutReadFactory called: ");
#endif

    /*
     * Read the factory calibration
     */
    result = RinBlockExecuteCommand( rb, rb_msg,
                                     RB_GET_SIZE_CMD_HDR(),
                                     RB_GET_SIZE_COMMAND( struct s_rb_aout_cal ),
                                     RB_TIMEOUT_DEFAULT, RB_CMD_READ_AO_FACTORY );
    if (result)
    {
#if RB_DEBUG(RB_AOUT_DEBUG_LVL, RB_DEBUG_BASIC)
      RB_DEBUG_PRINTF(RB_DEBUG_DEV, "success\n");
#endif

#if (BYTE_ORDER == NETWORK_ENDIAN)
      *aout_cal = rb_msg->data.rb_response.params.rb_aout_cal;
#else /* #if (BYTE_ORDER == NETWORK_ENDIAN) */
      aout_cal->uiFactoryCal0V = ntohs( rb_msg->data.rb_response.params.rb_aout_cal.uiFactoryCal0V );
      aout_cal->uiFactoryCal10V = ntohs( rb_msg->data.rb_response.params.rb_aout_cal.uiFactoryCal10V );
      aout_cal->uiFactoryCal4mA = ntohs( rb_msg->data.rb_response.params.rb_aout_cal.uiFactoryCal4mA );
      aout_cal->uiFactoryCal20mA = ntohs( rb_msg->data.rb_response.params.rb_aout_cal.uiFactoryCal20mA );
      aout_cal->uiCRCValue = ntohs( rb_msg->data.rb_response.params.rb_aout_cal.uiCRCValue );
#endif /* #else -> #if (BYTE_ORDER == NETWORK_ENDIAN) */
    }
#if RB_DEBUG(RB_AOUT_DEBUG_LVL, RB_DEBUG_BASIC)
    else
    {
      RB_DEBUG_PRINTF(RB_DEBUG_DEV, "failure\n");
    }
#endif
    return result;
}


/*
 * RinBlockAOutWriteFactoryCal -
 *
 * Write the analog output factory calibration to a (analog) rinblock
 * Only used in the test code
 */
unsigned char RinBlockAOutWriteFactoryCal(struct t_rinblock *rb, const struct s_rb_aout_cal *aout_cal)
{
    RB_DECLARE_CMD_BUFFER( rb_msg, struct s_rb_aout_cal );

#if RB_DEBUG(RB_AOUT_DEBUG_LVL, RB_DEBUG_BASIC)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "RinBlockAOutWriteFactoryCal called\n");
#endif

    /*
     * Write the factory calibration
     */
#if (BYTE_ORDER == NETWORK_ENDIAN)
    rb_msg->data.rb_command.params.rb_aout_cal = *aout_cal;
#else /* #if (BYTE_ORDER == NETWORK_ENDIAN) */
    rb_msg->data.rb_command.params.rb_aout_cal.uiFactoryCal0V = htons( aout_cal->uiFactoryCal0V );
    rb_msg->data.rb_command.params.rb_aout_cal.uiFactoryCal10V = htons( aout_cal->uiFactoryCal10V );
    rb_msg->data.rb_command.params.rb_aout_cal.uiFactoryCal4mA = htons( aout_cal->uiFactoryCal4mA );
    rb_msg->data.rb_command.params.rb_aout_cal.uiFactoryCal20mA = htons( aout_cal->uiFactoryCal20mA );
    rb_msg->data.rb_command.params.rb_aout_cal.uiCRCValue = htons( aout_cal->uiCRCValue );
#endif /* #else -> #if (BYTE_ORDER == NETWORK_ENDIAN) */

    return RinBlockExecuteCommand( rb, rb_msg,
                                   RB_GET_SIZE_COMMAND( struct s_rb_aout_cal ),
                                   RB_GET_SIZE_CMD_HDR(),
                                   RB_TIMEOUT_DEFAULT, RB_CMD_WRITE_AO_FACTORY );
}
