/*
 * rb_c500_lcd_segflash.c
 *
 *  Created on: 04/12/2014
 *      Author: ian
 */
#include <rinhw/rio.h>
#include <rinhw/lcd_generic.h>
#include <rinblock/rinblock.h>
#include <rinblock/rb_debug.h>
#include "_rb_c500_lcd_common.h"

#include <rinblock/rb_c500_lcd.h>


/** Create a LCD device for a C500 rinblock with segment flash */
int RinBlockCreateC500LcdDeviceSegFlash(struct t_io_dev *io_dev, struct t_rinblock *rb)
{
  /* Check that the mapped rinblock is actually a display one */
  if (RB_IFACE_TYPE_C500_DISP != rb->tag.interface_type)
  {
#if RB_DEBUG(RB_LCD_DEBUG_LVL, RB_DEBUG_BASIC)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "RinBlockCreateC500LcdDeviceSegFlash: attempting to map C500 lcd to non-C500 rinblock\n");
#endif
    goto rbcwld_fail;
  }

  /* Setup the new LCD device */
  if (lcd_generic_segflash_create_device(io_dev,
                                         rb,
                                         sizeof(struct s_rb_c500_disp_write_14seg),
                                         C500_DISPLAY_NUM_DIGITS,
                                         NULL,
                                         NULL,
                                         rb_c500_lcd_write_data,
                                         rb_c500_lcd_put_char,
                                         rb_c500_lcd_does_digit_have_dp,
                                         rb_c500_lcd_setbacklight) < 0)
  {
    goto rbcwld_fail;
  }

  /* Success */
  return RIO_SUCCESS;

rbcwld_fail:
  io_dev->dev_data = NULL;
  return RIO_ERROR;
}
