#include <rinhw/rinendian.h>

#include "_rb_c500_lcdsegs.h"


/*
 * The segment mapping is as follows:
 *
 *   AAAAAAAAA
 *  FG   H   JB
 *  F G  H  J B
 *  F  G H J  B
 *  F   GHJ   B
 *   PPPP KKKK
 *  E   NML   C
 *  E  N M L  C
 *  E N  M  L C
 *  EN   M   LC
 *   DDDDDDDDD   Q
 *
 * Segment defines are the bit position offsets from 0 for a digit
 */
#define SA 0
#define SB 1
#define SC 2
#define SD 3
#define SE 4
#define SF 5
#define SG 6
#define SH 7
#define SJ 8
#define SK 9
#define SL 10
#define SM 11
#define SN 12
#define SP 13
#define SQ 14


/*
 *  Convert segment number to mask
 */
#define MASK(n) (1 << n)


/*
 * Segment bit masks
 */
#define MS_NONE 0x0000
#define MSA MASK(SA)
#define MSB MASK(SB)
#define MSC MASK(SC)
#define MSD MASK(SD)
#define MSE MASK(SE)
#define MSF MASK(SF)
#define MSG MASK(SG)
#define MSH MASK(SH)
#define MSJ MASK(SJ)
#define MSK MASK(SK)
#define MSL MASK(SL)
#define MSM MASK(SM)
#define MSN MASK(SN)
#define MSP MASK(SP)
#define MSQ MASK(SQ)
#define MS_ALL (MSA | MSB | MSC | MSD | MSE | MSF | MSG | MSH | MSJ | MSK | MSL | MSM | MSN | MSP | MSQ)


/*
 * The LCD segment table
 */
const int16u rb_c500_seg_table[NUM_SEG_TABLE] LCDSEG_TABLE_MEMORY =
{
    MS_NONE,        /* 0 */
    MSA,            /* 1 */
    MSB,            /* 2 */
    MSC,            /* 3 */
    MSD,            /* 4 */
    MSE,            /* 5 */
    MSF,            /* 6 */
    MSG,            /* 7 */
    MSH,            /* 8 */
    MSJ,            /* 9 */
    MSK,            /* 10 */
    MS_ALL,         /* 11 */
    MSL,            /* 12 */
    MSM,            /* 13 */
    MSN,            /* 14 */
    MSP,            /* 15 */
    MSQ,            /* 16 */
    MS_NONE,        /* 17 */
    MS_NONE,        /* 18 */
    MS_NONE,        /* 19 */
    MS_NONE,        /* 20 */
    MS_NONE,        /* 21 */
    MS_NONE,        /* 22 */
    MS_NONE,        /* 23 */
    MS_NONE,        /* 24 */
    MS_NONE,        /* 25 */
    MS_NONE,        /* 26 */
    MS_NONE,        /* 27 */
    MS_NONE,        /* 28 */
    MS_NONE,        /* 29 */
    MS_NONE,        /* 30 */
    MS_NONE,        /* 31 */

    (MS_NONE),                                                    /* 32 Space */
    (MSB | MSQ),                                                  /* 33 ! */
    (MSB | MSF),                                                  /* 34 " */
    (MSB | MSF | MSE | MSC),                                      /* 35 # */
    (MSA | MSC | MSD | MSF | MSH | MSK | MSM | MSP),              /* 36 $ */
    (MSC | MSF | MSJ | MSN),                                      /* 37 % */
    (MSA | MSD | MSE | MSG | MSJ | MSL),                          /* 38 & */
    (MSH),                                                        /* 39 ' */
    (MSA | MSD | MSE | MSF),                                      /* 40 ( */
    (MSA | MSB | MSC | MSD),                                      /* 41 ) */
    (MSG | MSH | MSJ | MSL | MSM | MSN),                          /* 42 * */
    (MSH | MSK | MSM | MSP),                                      /* 43 + */
    (MSN),                                                        /* 44 , */
    (MSK | MSP),                                                  /* 45 - */
    (MSQ),                                                        /* 46 . */
    (MSJ | MSN),                                                  /* 47 / */

    (MSA | MSB | MSC | MSD | MSE | MSF),                          /* 48 0 */
    (MSB | MSC | MSJ),                                            /* 49 1 */
    (MSA | MSB | MSE | MSD | MSK | MSP),                          /* 50 2 */
    (MSA | MSB | MSC | MSD | MSK),                                /* 51 3 */
    (MSB | MSC | MSF | MSK | MSP),                                /* 52 4 */
    (MSA | MSC | MSD | MSF | MSK | MSP),                          /* 53 5 */
    (MSA | MSC | MSD | MSE | MSF | MSK | MSP),                    /* 54 6 */
    (MSA | MSJ | MSM),                                            /* 55 7 */
    (MSA | MSB | MSC | MSD | MSE | MSF | MSK | MSP),              /* 56 8 */
    (MSA | MSB | MSC | MSD | MSF | MSK | MSP),                    /* 57 9 */

    (MSH | MSM),                                                  /* 58 : */
    (MSH | MSN),                                                  /* 59 ; */
    (MSJ | MSL),                                                  /* 60 < */
    (MSD | MSK | MSP),                                            /* 61 = */
    (MSG | MSN),                                                  /* 62 > */
    (MSA | MSF | MSJ | MSM),                                      /* 63 ? */
    (MSA | MSB | MSC | MSD | MSE | MSF | MSG | MSJ | MSL | MSN),  /* 64 @ */

    (MSA | MSB | MSC | MSE | MSF | MSK | MSP),                    /* 65 A */
    (MSA | MSB | MSC | MSD | MSH | MSK | MSM),                    /* 66 B */
    (MSA | MSD | MSE | MSF),                                      /* 67 C */
    (MSA | MSB | MSC | MSD | MSH | MSM),                          /* 68 D */
    (MSA | MSD | MSE | MSF | MSK | MSP),                          /* 69 E */
    (MSA | MSE | MSF | MSK | MSP),                                /* 70 F */
    (MSA | MSC | MSD | MSE | MSF | MSK),                          /* 71 G */
    (MSB | MSC | MSE | MSF | MSK | MSP),                          /* 72 H */
    (MSA | MSD | MSH | MSM),                                      /* 73 I */
    (MSB | MSC | MSD | MSE),                                      /* 74 J */
    (MSE | MSF | MSJ | MSL | MSP),                                /* 75 K */
    (MSD | MSE | MSF),                                            /* 76 L */
    (MSB | MSC | MSE | MSF | MSG | MSJ),                          /* 77 M */
    (MSB | MSC | MSE | MSF | MSG | MSL),                          /* 78 N */
    (MSA | MSB | MSC | MSD | MSE | MSF),                          /* 79 O */
    (MSA | MSB | MSE | MSF | MSK | MSP),                          /* 80 P */
    (MSA | MSB | MSC | MSD | MSE | MSF | MSL),                    /* 81 Q */
    (MSA | MSB | MSE | MSF | MSK | MSL | MSP),                    /* 82 R */
    (MSA | MSC | MSD | MSF | MSK | MSP),                          /* 83 S */
    (MSA | MSH | MSM),                                            /* 84 T */
    (MSB | MSC | MSD | MSE | MSF),                                /* 85 U */
    (MSE | MSF | MSH | MSN),                                      /* 86 V */
    (MSB | MSC | MSE | MSF | MSL | MSN),                          /* 87 W */
    (MSG | MSJ | MSL | MSN),                                      /* 88 X */
    (MSG | MSJ | MSM),                                            /* 89 Y */
    (MSA | MSD | MSJ | MSN),                                      /* 90 Z */

    (MSA | MSD | MSE | MSF),                                      /* 91 [ */
    (MSG | MSL),                                                  /* 92 \ */
    (MSA | MSB | MSC | MSD),                                      /* 93 ] */
    (MSA | MSB | MSF),                                            /* 94 ^ */
    (MSD),                                                        /* 95 _ */
    (MSG),                                                        /* 96 ` */

    (MSA | MSB | MSC | MSE | MSF | MSK | MSP),                    /* 97 a */
    (MSC | MSD | MSE | MSF | MSK | MSP),                          /* 98 b */
    (MSD | MSE | MSK | MSP),                                      /* 99 c */
    (MSB | MSC | MSD | MSE | MSK | MSP),                          /* 100 d */
    (MSA | MSD | MSE | MSF | MSP),                                /* 101 e */
    (MSA | MSE | MSF | MSP),                                      /* 102 f */
    (MSA | MSB | MSC | MSD | MSF | MSK | MSP),                    /* 103 g */
    (MSC | MSE | MSF | MSK | MSP),                                /* 104 h */
    (MSM),                                                        /* 105 i */
    (MSB | MSC | MSD),                                            /* 106 j */
    (MSH | MSJ | MSL | MSM),                                      /* 107 k */
    (MSH | MSM),                                                  /* 108 l */
    (MSC | MSE | MSK | MSM | MSP),                                /* 109 m */
    (MSC | MSE | MSK | MSP),                                      /* 110 n */
    (MSC | MSD | MSE | MSK | MSP),                                /* 111 o */
    (MSA | MSB | MSE | MSF | MSK | MSP),                          /* 112 p */
    (MSA | MSB | MSC | MSF | MSK | MSP),                          /* 113 q */
    (MSE | MSP),                                                  /* 114 r */
    (MSA | MSC | MSD | MSF | MSK | MSP),                          /* 115 s */
    (MSD | MSE | MSF | MSP),                                      /* 116 t */
    (MSC | MSD | MSE),                                            /* 117 u */
    (MSE | MSN),                                                  /* 118 v */
    (MSC | MSE | MSL | MSN),                                      /* 119 w */
    (MSG | MSJ | MSL | MSN),                                      /* 120 x */
    (MSB | MSC | MSD | MSH | MSK),                                /* 121 y */
    (MSA | MSD | MSJ | MSN),                                      /* 122 z */

    (MSA | MSD | MSG | MSN | MSP),                                /* 123 { */
    (MSH | MSM),                                                  /* 124 | */
    (MSA | MSD | MSJ | MSK | MSL),                                /* 125 } */
    (MSA),                                                        /* 126 ~ */
    (MS_NONE),                                                    /* 127 Del */
};
