#include <stdio.h>
#include <stdarg.h>
#include <rinblock/rb_debug.h>

#ifdef RINBLOCK_DEBUG


#if RB_DEBUG_APP_CALLBACK
#include <syslog.h>


/** Rinblock debug printf function pointer */
static t_rb_debug_printf_fn rb_debug_printf_fn = NULL;


/** Set the debug printf function for the rinblock library */
void rb_set_debug_printf_fn(t_rb_debug_printf_fn fn)
{
  rb_debug_printf_fn = fn;
}


/** Debug printf function for the rinblock library, calls the application implemented handler */
int rb_debug_printf(struct t_io_dev *dummy, const char *format, ...)
{
  va_list args;

  if (NULL != rb_debug_printf_fn)
  {
    va_start(args, format);
    rb_debug_printf_fn(LOG_ERR, format, args);
    va_end(args);
  }
  return 0;
}

#endif /* #if RB_DEBUG_APP_CALLBACK */


unsigned char rbisprint(unsigned char ch)
{
    return ((ch >= ' ') && (ch <= '~'));
}


#define DUMP_WIDTH 8

void RBDump(unsigned char *ptr, unsigned short length)
{
    unsigned short i, num;

    while (length > 0)
    {
        num = (length > DUMP_WIDTH) ? DUMP_WIDTH : length;
        RB_DEBUG_PRINTF(RB_DEBUG_DEV, "%08lx: ", ptr);
        for (i = 0; i < num; i++)
        {
            RB_DEBUG_PRINTF(RB_DEBUG_DEV, "%02x", ptr[i]);
            if (i < (num - 1))
                RB_DEBUG_PRINTF(RB_DEBUG_DEV, ", ");
        }
        RB_DEBUG_PRINTF(RB_DEBUG_DEV, "  ");
        for (i = 0; i < num; i++)
        {
            RB_DEBUG_PRINTF(RB_DEBUG_DEV, "%c", (rbisprint(ptr[i])) ? ptr[i] : '.');
            if (i >= (num - 1))
                RB_DEBUG_PRINTF(RB_DEBUG_DEV, "\n");
        }
        length -= num;
        ptr += num;
    }
}

#endif /* #ifdef RINBLOCK_DEBUG */
