/*
 * rb_ethernet.c
 *
 *  Created on: 09/02/2009
 *      Author: ianb
 */
#include <string.h>
#include <rinblock/rinblock.h>
#include <rinblock/rb_debug.h>
#include <rinblock/rb_command.h>
#include <rinblock/rb_msg.h>

#include <rinblock/rb_ethernet.h>


/** Configure the Ethernet module with IP address and associated configuration. */
unsigned char RinBlockEthernetSetConfiguration( struct t_rinblock *rb, const struct s_rb_eth_ipconfigure *ethcfg_ptr )
{
    RB_DECLARE_CMD_BUFFER( rb_msg, struct s_rb_eth_ipconfigure );

#if RB_DEBUG(RB_ETHERNET_DEBUG_LVL, RB_DEBUG_FULL)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "RinBlockEthernetSetConfiguration called\n");
#endif

    /* Do endian swap */
#if (BYTE_ORDER == NETWORK_ENDIAN)
    rb_msg->data.rb_command.params.rb_eth_ipconfigure = *ethcfg_ptr;
#else /* #if (BYTE_ORDER == NETWORK_ENDIAN) */
    rb_msg->data.rb_command.params.rb_eth_ipconfigure.ui32IpAddress = htonl( ethcfg_ptr->ui32IpAddress );
    rb_msg->data.rb_command.params.rb_eth_ipconfigure.ui32IpNetMask = htonl( ethcfg_ptr->ui32IpNetMask );
    rb_msg->data.rb_command.params.rb_eth_ipconfigure.ui32IpGateway = htonl( ethcfg_ptr->ui32IpGateway );
    rb_msg->data.rb_command.params.rb_eth_ipconfigure.ui32IpDns1 = htonl( ethcfg_ptr->ui32IpDns1 );
    rb_msg->data.rb_command.params.rb_eth_ipconfigure.ui32IpDns2 = htonl( ethcfg_ptr->ui32IpDns2 );
    rb_msg->data.rb_command.params.rb_eth_ipconfigure.bUseDHCP = ethcfg_ptr->bUseDHCP;
#endif /* #else -> #if (BYTE_ORDER == NETWORK_ENDIAN) */

    return RinBlockExecuteCommand( rb, rb_msg,
                                   RB_GET_SIZE_COMMAND( struct s_rb_eth_ipconfigure ),
                                   RB_GET_SIZE_CMD_HDR(),
                                   RB_TIMEOUT_DEFAULT, RB_CMD_ETH_WRITE_IPCONFIG );
}


/** Read the IP address and associated configuration from the Ethernet module. */
unsigned char RinBlockEthernetGetConfiguration( struct t_rinblock *rb, struct s_rb_eth_ipconfigure *ethcfg_ptr )
{
    unsigned char result;
    RB_DECLARE_CMD_BUFFER( rb_msg, struct s_rb_eth_ipconfigure );

#if RB_DEBUG(RB_ETHERNET_DEBUG_LVL, RB_DEBUG_FULL)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "RinBlockEthernetGetConfiguration called\n");
#endif

    result = RinBlockExecuteCommand( rb, rb_msg,
                                     RB_GET_SIZE_CMD_HDR(),
                                     RB_GET_SIZE_COMMAND( struct s_rb_eth_ipconfigure ),
                                     RB_TIMEOUT_DEFAULT, RB_CMD_ETH_READ_IPCONFIG );
    if (result)
    {
      /* Do endian swap */
#if (BYTE_ORDER == NETWORK_ENDIAN)
      *ethcfg_ptr = rb_msg->data.rb_response.params.rb_eth_ipconfigure;
#else /* #if (BYTE_ORDER == NETWORK_ENDIAN) */
      ethcfg_ptr->ui32IpAddress = ntohl( rb_msg->data.rb_response.params.rb_eth_ipconfigure.ui32IpAddress );
      ethcfg_ptr->ui32IpNetMask = ntohl( rb_msg->data.rb_response.params.rb_eth_ipconfigure.ui32IpNetMask );
      ethcfg_ptr->ui32IpGateway = ntohl( rb_msg->data.rb_response.params.rb_eth_ipconfigure.ui32IpGateway );
      ethcfg_ptr->ui32IpDns1 = ntohl( rb_msg->data.rb_response.params.rb_eth_ipconfigure.ui32IpDns1 );
      ethcfg_ptr->ui32IpDns2 = ntohl( rb_msg->data.rb_response.params.rb_eth_ipconfigure.ui32IpDns2 );
      ethcfg_ptr->bUseDHCP = rb_msg->data.rb_response.params.rb_eth_ipconfigure.bUseDHCP;
#endif /* #else -> #if (BYTE_ORDER == NETWORK_ENDIAN) */
    }
    return result;
}


/** Restart the Matchport module (but not the Atmel) */
unsigned char RinBlockEthernetRestart( struct t_rinblock *rb )
{
    RB_DECLARE_CMD_BUFFER_NO_PARAMS( rb_msg );

#if RB_DEBUG(RB_ETHERNET_DEBUG_LVL, RB_DEBUG_FULL)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "RinBlockEthernetRestart called\n");
#endif

    return RinBlockExecuteCommand( rb, rb_msg,
                                   RB_GET_SIZE_CMD_HDR(),
                                   RB_GET_SIZE_CMD_HDR(),
                                   RB_TIMEOUT_DEFAULT, RB_CMD_ETH_MATCHPORT_RESET );
}


/** Force the Matchport module to Rinstrum defaults */
unsigned char RinBlockEthernetForceDefaults( struct t_rinblock *rb )
{
    RB_DECLARE_CMD_BUFFER_NO_PARAMS( rb_msg );

#if RB_DEBUG(RB_ETHERNET_DEBUG_LVL, RB_DEBUG_FULL)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "RinBlockEthernetForceDefaults called\n");
#endif

    return RinBlockExecuteCommand( rb, rb_msg,
                                   RB_GET_SIZE_CMD_HDR(),
                                   RB_GET_SIZE_CMD_HDR(),
                                   RB_TIMEOUT_DEFAULT, RB_CMD_ETH_MATCHPORT_DEFAULTS );
}


/** Disable the watchdog to allow the Matchport module to be reflashed */
unsigned char RinBlockEthernetDisableWatchdog( struct t_rinblock *rb, bool val )
{
	RB_DECLARE_CMD_BUFFER(rb_msg, struct s_rb_eth_watchdog);

#if RB_DEBUG(RB_ETHERNET_DEBUG_LVL, RB_DEBUG_FULL)
  RB_DEBUG_PRINTF(RB_DEBUG_DEV, "RinBlockEthernetDisableWatchdog: %u\n", val);
#endif

  rb_msg->data.rb_command.params.rb_eth_watchdog.enable = val ? 1 : 0;
  return RinBlockExecuteCommand(rb, rb_msg,
                                RB_GET_SIZE_COMMAND(struct s_rb_eth_watchdog),
                                RB_GET_SIZE_CMD_HDR(),
                                RB_TIMEOUT_DEFAULT, RB_CMD_ETH_WATCHDOG);
}


#if 0 /* TODO Include in later release */

unsigned char RinBlockConfigureWifi( struct t_rinblock *rinblock, const char *pszSsid, unsigned char ui8Encryption, const char *pszPsk )
{
  unsigned char buffer[sizeof(struct s_rb_wifi_configure) + 1];
  struct s_rb_message *rb_msg = (struct s_rb_message *) &buffer[0];

#if RB_DEBUG(RB_ETHERNET_DEBUG_LVL, RB_DEBUG_FULL)
  RB_DEBUG_PRINTF(RB_DEBUG_DEV, "RinBlockConfigureWifi called\n");
#endif

  /* Setup command */
  rb_msg->data.rb_command.cmd = RB_CMD_WIFI_WRITE_WIFICONFIG;

  /* Copy data to struct */
  strncpy( rb_msg->data.rb_command.params.rb_wifi_configure.szSSID, pszSsid, 32 );
  rb_msg->data.rb_command.params.rb_wifi_configure.szSSID[ 31 ] = '\0';

  /*
   * Write config to rinblock
   */
  if (!RinBlockExecuteCommand( rinblock, rb_msg, 1 + sizeof(struct s_rb_eth_ipconfigure), 1, 0))
  {
#if RB_DEBUG(RB_ETHERNET_DEBUG_LVL, RB_DEBUG_FULL)
      RB_DEBUG_PRINTF(RB_DEBUG_DEV, "rb_eth_config_set: write config failed\n");
#endif
      return 0;
  }
  if ((rb_msg->data.rb_response.resp & RB_RESP_ERROR_MASK) != 0)
  {
#if RB_DEBUG(RB_ETHERNET_DEBUG_LVL, RB_DEBUG_FULL)
      RB_DEBUG_PRINTF(RB_DEBUG_DEV, "rb_eth_config_set: module returned error\n");
#endif
      return 0;
  }

#if RB_DEBUG(RB_ETHERNET_DEBUG_LVL, RB_DEBUG_FULL)
  RB_DEBUG_PRINTF(RB_DEBUG_DEV, "rb_eth_config_set: write config success\n");
#endif
  return 1;
}
#endif
