#include <string.h>
#include <rinhw/rinendian.h>
#include <rinblock/rb_debug.h>
#include <rinblock/rb_startup.h>
#include <rinblock/rb_command.h>

#include <rinblock/rb_program.h>


#define IV_PRESET_LO 0x5E89CFB5UL
#define IV_PRESET_HI 0x9A31AE87UL

#define TEA_KEY_LO 0x5F713C19UL
#define TEA_KEY_MIDLO 0xEEBA917BUL
#define TEA_KEY_MIDHI 0x64B16401UL
#define TEA_KEY_HI 0x8BA93F71UL

#define RB_FLASH_CMD_GET_CONFIG 0x63DE3D96UL
#define RB_FLASH_CMD_ERASE 0xD768FEFAUL
#define RB_FLASH_CMD_PROGRAM 0xB7D6F53CUL
#define RB_FLASH_CMD_VERIFY 0x53E1A8D7UL
#define RB_FLASH_CMD_RESET 0xC33723C6UL

#define RB_FLASH_WRITE_REQUEST_MORE_DATA 0x00
#define RB_FLASH_WRITE_FLASH_VERIFY_OK 0x22
#define RB_FLASH_WRITE_FLASH_FAILURE 0x23
#define RB_FLASH_ERASE_COMPLETE 0x24
#define RB_FLASH_ERASE_FAILURE 0x25


static const unsigned long key[4] = { TEA_KEY_LO, TEA_KEY_MIDLO, TEA_KEY_MIDHI, TEA_KEY_HI };


static void TEAEncrypt(unsigned long *v, const unsigned long *const k)
{
    unsigned long y = v[0], z = v[1], sum = 0, delta = 0x9E3779B9;
    unsigned long a = k[0], b = k[1], c = k[2], d = k[3];

    unsigned char n = 32;

    while (n-- > 0)
    {
        sum += delta;
        y += (z << 4) + a ^ z + sum ^ (z >> 5) + b;
        z += (y << 4) + c ^ y + sum ^ (y >> 5) + d;
    }
    v[0] = y;
    v[1] = z;
}


static unsigned char CBCTeaEncrypt(unsigned long *v, unsigned long *iv)
{
    unsigned long temp_v[2];

    /*
     * Cipher Block Chain the data
     */
    temp_v[0] = v[0] ^ iv[0];
    temp_v[1] = v[1] ^ iv[1];

    /*
     * Encrypt it
     */
    TEAEncrypt(temp_v, key);

    /*
     * Write it back, checking that the buffer is writable
     */
    v[0] = temp_v[0];
    v[1] = temp_v[1];
    if ((v[0] != temp_v[0]) || (v[1] != temp_v[1]))
        return 0;

    /*
     * Update the initialisation vector to continue CBC
     */
    iv[0] = temp_v[0];
    iv[1] = temp_v[1];
    return 1;
}


static unsigned char EncryptBuffer(unsigned char *buf, unsigned long length, unsigned long *iv)
{
    unsigned long *v = (unsigned long *) buf;
    unsigned long i;

/*
    unsigned long iv[2];

    iv[0] = IV_PRESET_LO;
    iv[1] = IV_PRESET_HI;
*/

    for (i = 0; i < length; i += 2 * sizeof(unsigned long), v += 2)
    {
        if (!CBCTeaEncrypt(v, iv))
            return 0;
    }
    return 1;
}


/*
 * RinBlockProgrammingWrite -
 *
 * Write a byte to the rin block programming stream
 */
static unsigned char RinBlockProgrammingWrite(struct t_rinblock *rb, unsigned char dat)
{
    unsigned char buf[2], res;
    struct s_rb_message *rb_msg = (struct s_rb_message *) &buf;

    rb_msg->data.rb_command.header.cmd = RB_CMD_WRITE_BOOT_STREAM;
    rb_msg->data.rb_command.params.rb_general_boot_stream.data = dat;
    res = RinBlockInternalExecuteCommand(rb, rb_msg, 2, 1, 0, 0);

#if RB_DEBUG(RB_FLASH_DEBUG_LVL, RB_DEBUG_FULL)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "RB_CMD_WRITE_BOOT_STREAM (data 0x%02x) returned with res %d, response 0x%02x\n",
                    dat, res, rb_msg->data.rb_response.header.resp);
#endif
    return res;
}


static unsigned char RinBlockProgrammingBufWrite(struct t_rinblock *rb, unsigned char *buf, unsigned char length)
{
    unsigned char i;

    for (i = 0; i < length; i++)
    {
        if (!RinBlockProgrammingWrite(rb, *buf++))
            return 0;
    }
    return 1;
}


/*
 * RinBlockProgrammingRead -
 *
 * Read a byte from the rin block programming stream
 */
static unsigned char RinBlockProgrammingRead(struct t_rinblock *rb, unsigned char *dat)
{
    unsigned char buf[2];
    struct s_rb_message *rb_msg = (struct s_rb_message *) &buf;
    unsigned char result, res;

    rb_msg->data.rb_command.header.cmd = RB_CMD_READ_BOOT_STREAM;
    res = RinBlockInternalExecuteCommand(rb, rb_msg, 1, 2, 0, 0);

    result = ((res) && ((rb_msg->data.rb_response.header.resp & RB_RESP_ERROR_MASK) == 0));
    if (result)
        *dat = rb_msg->data.rb_command.params.rb_general_boot_stream.data;

#if RB_DEBUG(RB_FLASH_DEBUG_LVL, RB_DEBUG_FULL)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "RB_CMD_READ_BOOT_STREAM returned with result %d, response 0x%02x, data 0x%02x\n",
                    res, rb_msg->data.rb_response.header.resp, *dat);
#endif

    return result;
}


static unsigned char RinBlockProgrammingBufRead(struct t_rinblock *rb, unsigned char *buf, unsigned char length)
{
    unsigned char i;

    for (i = 0; i < length; i++)
    {
        if (!RinBlockProgrammingRead(rb, buf++))
            return 0;
    }
    return 1;
}


/*
 * RinBlockProgrammingFinished -
 *
 * Check whether the rin block has finished programming
 */
static unsigned char RinBlockProgrammingFinished(struct t_rinblock *rb)
{
    unsigned char mode;

    if (RinBlockGetMode(rb, &mode))
        return (mode == RB_MODE_PROG_DONE);
    return 0;
}

#if 0
static void byte_swap_word(unsigned short *w)
{
    unsigned char *b = (unsigned char *) w;
    unsigned char temp;

    temp = b[0];
    b[0] = b[1];
    b[1] = temp;
}
#endif

#if 0
static void byte_swap_long(unsigned long *l)
{
    unsigned char *b = (unsigned char *) l;
    unsigned char temp;

    temp = b[0];
    b[0] = b[3];
    b[3] = temp;
    temp = b[1];
    b[1] = b[2];
    b[2] = temp;
}
#endif


static void RinBlockByteSwapConfig(struct s_rb_bootloader_identification *iden)
{
#if 0 /* TODO rinblock programming does endian need to be taken into account here? */
    byte_swap_word(&iden->check_seed);
    byte_swap_long(&iden->user_start);
    byte_swap_long(&iden->user_length);
#else
    UNREFERENCED_PARAMETER(iden); /* Stop compiler warnings on unused parameter */
#endif
}


static unsigned char RinBlockFlashGetConfig(struct t_rinblock *rb, struct s_rb_bootloader_identification *iden)
{
    unsigned long cmd = RB_FLASH_CMD_GET_CONFIG;

    if (!RinBlockProgrammingBufWrite(rb, (unsigned char *) &cmd, 4))
    {
        return 0;
    }

    if (!RinBlockProgrammingBufRead(rb, (unsigned char *) iden, sizeof(*iden)))
    {
        return 0;
    }
    return 1;
}


static unsigned char RinBlockFlashVerifyCode(struct t_rinblock *rb,
                                             struct s_rb_bootloader_identification *iden,
                                             const unsigned char *buf,
                                             unsigned long start, unsigned long length)
{
    unsigned long i;
    unsigned short temp;
    unsigned short chksum;
    unsigned char res;

    UNREFERENCED_PARAMETER( rb ); /* Stop compiler warnings on unused parameter */

    /*
     * Check flash start
     */
    if (iden->user_start != start)
    {
#if RB_DEBUG(RB_FLASH_DEBUG_LVL, RB_DEBUG_FULL)
        RB_DEBUG_PRINTF(RB_DEBUG_DEV, "Flash start (%lu) doesn't match image (%lu)\n", iden->user_start, start);
#endif
        return 0;
    }

    /*
     * Check flash length
     */
    if (iden->user_length != length)
    {
#if RB_DEBUG(RB_FLASH_DEBUG_LVL, RB_DEBUG_FULL)
        RB_DEBUG_PRINTF(RB_DEBUG_DEV, "Flash length (%lu) doesn't match image (%lu)\n", iden->user_length, length);
#endif
        return 0;
    }

    /*
     * Work out checksum on flash image
     */
    chksum = iden->check_seed;
    for (i = 0; i < length - 2; i++)
    {
        chksum += *buf++;
    }

    /* Because buf may not be word aligned, do two byte reads and re-assemble */
    temp = (*buf << 8) & 0xFF00;
    buf++;
    temp |= *buf & 0x00FF;

    /* But on little endian machines we'll need to swap the byte order */
#if (BYTE_ORDER == LITTLE_ENDIAN)
    temp = BYTE_SWAP_SHORT(temp);
#endif /* #if (BYTE_ORDER == LITTLE_ENDIAN) */

    /* Now check the result */
    res = (unsigned char) (chksum == temp);
#if RB_DEBUG(RB_FLASH_DEBUG_LVL, RB_DEBUG_FULL)
    if (!res)
    {
        RB_DEBUG_PRINTF(RB_DEBUG_DEV, "Flash checksum @ 0x%p (0x%04x) doesn't match calculated checksum (0x%04x)\n", buf, temp, chksum);
    }
#endif
    return res;
}


static unsigned char RinBlockFlashSendCommand(struct t_rinblock *rb, unsigned long cmd, unsigned long start, unsigned long length)
{

    /*
     * Send command
     */
    if (!RinBlockProgrammingBufWrite(rb, (unsigned char *) &cmd, 4))
    {
        return 0;
    }

    /*
     * Send start address
     */
    if (!RinBlockProgrammingBufWrite(rb, (unsigned char *) &start, 4))
    {
        return 0;
    }

    /*
     * Send length
     */
    if (!RinBlockProgrammingBufWrite(rb, (unsigned char *) &length, 4))
    {
        return 0;
    }
    return 1;
}


static unsigned char RinBlockFlashErase(struct t_rinblock *rb, unsigned long start, unsigned long length)
{
    unsigned char resp;

    if (!RinBlockFlashSendCommand(rb, RB_FLASH_CMD_ERASE, start, length))
    {
        return 0;
    }

    if (!RinBlockProgrammingRead(rb, &resp))
    {
        return 0;
    }

    if (resp != RB_FLASH_ERASE_COMPLETE)
    {
        return 0;
    }

    return 1;
}


static unsigned char RinBlockFlashProgramOrVerify(struct t_rinblock *rb, const unsigned char *ptr, unsigned char prog_vrfy,
                                                  unsigned long start, unsigned long length, unsigned char work_buffer[RB_FLASH_SERIAL_BLOCK_SIZE],
                                                  unsigned char *err)
{
    unsigned long i;
    unsigned char temp;
    const unsigned long cmd = (prog_vrfy) ? RB_FLASH_CMD_PROGRAM : RB_FLASH_CMD_VERIFY;
    unsigned long iv[2];
    const unsigned char fail_err = (prog_vrfy) ? RB_ERR_FLASH_PROGRAM_FAIL : RB_ERR_FLASH_VERIFY_FAIL;

    /* Kick off programming/verifying on the rinblock */
    if (!RinBlockFlashSendCommand(rb, cmd, start, length))
    {
        *err = fail_err;
        return 0;
    }

    /* Encryption initialisation vectors */
    iv[0] = IV_PRESET_LO;
    iv[1] = IV_PRESET_HI;

    /* Send data in RB_FLASH_SERIAL_BLOCK_SIZE blocks */
    for (i = 0; i < length; i += RB_FLASH_SERIAL_BLOCK_SIZE, ptr += RB_FLASH_SERIAL_BLOCK_SIZE)
    {

        /* Copy data to work buffer, and encrypt */
        memcpy(work_buffer, ptr, RB_FLASH_SERIAL_BLOCK_SIZE);
        if (!EncryptBuffer(work_buffer, RB_FLASH_SERIAL_BLOCK_SIZE, iv))
        {
#if RB_DEBUG(RB_FLASH_DEBUG_LVL, RB_DEBUG_BASIC)
            RB_DEBUG_PRINTF(RB_DEBUG_DEV, "Rin Block Encrypt Buffer failed (work buffer is readonly, should be ram, not flash)\n");
#endif
            *err = RB_ERR_FLASH_LOCALBUF_READONLY;
            return 0;
        }

        /* Program encypted data */
        if (!RinBlockProgrammingBufWrite(rb, work_buffer, RB_FLASH_SERIAL_BLOCK_SIZE))
        {
            *err = fail_err;
            return 0;
        }
        if (!RinBlockProgrammingRead(rb, &temp))
        {
            *err = fail_err;
            return 0;
        }

        if (temp != RB_FLASH_WRITE_REQUEST_MORE_DATA)
        {
            *err = fail_err;
            return 0;
        }

        RinBlockProgrammingWrite(rb, RB_FLASH_WRITE_REQUEST_MORE_DATA);
    }

    if (!RinBlockProgrammingRead(rb, &temp))
    {
        *err = fail_err;
        return 0;
    }

    /* Return result */
    *err = (unsigned char) ((RB_FLASH_WRITE_FLASH_VERIFY_OK == temp) ? RB_ERR_NO_ERR : fail_err);
    return (RB_FLASH_WRITE_FLASH_VERIFY_OK == temp);
}


static unsigned char RinBlockDisconnect(struct t_rinblock *rb)
{
    unsigned long cmd = RB_FLASH_CMD_RESET;

    return (RinBlockProgrammingBufWrite(rb, (unsigned char *) &cmd, 4));
}


/**
 * Get the rinblock bootloader configuration
 *
 * @param rb is a pointer to rinblock that is present (and may be running)
 * @param err is set to an RB_ERR_ code to provide more information on a failure
 * @param iden is a pointer to a bootloader identification struction filled out on
 */
unsigned char RinBlockBootloaderGetConfig(struct t_rinblock *rb, unsigned char *err, struct s_rb_bootloader_identification *iden)
{
#if RB_DEBUG(RB_FLASH_DEBUG_LVL, RB_DEBUG_BASIC)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "Get rinblock bootloader info from rinblock at address %d\n", rb->iic_address);
#endif

    /* Check the rinblock is actually present on the bus */
    if (!rb->present)
    {
#if RB_DEBUG(RB_FLASH_DEBUG_LVL, RB_DEBUG_BASIC)
        RB_DEBUG_PRINTF(RB_DEBUG_DEV, "No rinblock present at address %d\n", rb->iic_address);
#endif
        *err = RB_ERR_NOT_PRESENT;
        return 0;
    }

    /* Set the mode of the rin block to programming */
#if RB_DEBUG(RB_FLASH_DEBUG_LVL, RB_DEBUG_BASIC)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "Setting rinblock mode for programming\n");
#endif
    if (!RinBlockSetMode(rb, RB_MODE_PROGRAM))
    {
#if RB_DEBUG(RB_FLASH_DEBUG_LVL, RB_DEBUG_BASIC)
        RB_DEBUG_PRINTF(RB_DEBUG_DEV, "Set mode failed\n");
#endif
        *err = RB_ERR_PROGRAM_MODE;
        return 0;
    }

    /* Get the flash configuration from the module */
    if (!RinBlockFlashGetConfig(rb, iden))
    {
#if RB_DEBUG(RB_FLASH_DEBUG_LVL, RB_DEBUG_BASIC)
        RB_DEBUG_PRINTF(RB_DEBUG_DEV, "Get flash config failed\n");
#endif
        *err = RB_ERR_READ_FLASH_CFG;
        return 0;
    }

    /* Sort out the byte orders */
    RinBlockByteSwapConfig(iden);

    /* Disconnect/reset the rinblock */
    if (!RinBlockDisconnect(rb))
    {
#if RB_DEBUG(RB_FLASH_DEBUG_LVL, RB_DEBUG_BASIC)
        RB_DEBUG_PRINTF(RB_DEBUG_DEV, "Disconnect/reset rinblock failed\n");
#endif
        *err = RB_ERR_PROGRAM_DISCONNECT;
        return 0;
    }

    /* Ask the module if programming is finished */
    while (!RinBlockProgrammingFinished(rb))
        OSTimeDly(OS_TICKS_PER_SEC / 10);

    /* Restart the rinblock */
#if RB_DEBUG(RB_FLASH_DEBUG_LVL, RB_DEBUG_BASIC)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "Restarting the rinblock");
#endif
    RinBlockRestart(rb);
    *err = RB_ERR_NO_ERR;
    return 1;
}


/**
 * Program rinblock rb from buffer buf.
 *
 * NOTE that the work_buffer must be in RAM, otherwise the the buffer can't be encrypted. Will
 * return RB_ERR_FLASH_LOCALBUF_READONLY if this is the case.
 * NOTE also that the contents of the work_buffer will be modified after calling this function
 *
 * @param rb is a pointer to rinblock that is present (and may be running)
 * @param buf is a pointer to a buffer containing the image to be programmed
 * @param err is set to an RB_ERR_ code to provide more information on a failure
 * @param start is the start address within the rinblock
 * @param length is the length of the buffer
 * @param work_buffer is buffer used by the programming routine, and will be modified
 */
unsigned char RinBlockTestProgram(struct t_rinblock *rb, const unsigned char *buf, unsigned char *err,
                                  unsigned long start, unsigned long length, unsigned char work_buffer[RB_FLASH_SERIAL_BLOCK_SIZE])
{
    struct s_rb_bootloader_identification iden;

#if RB_DEBUG(RB_FLASH_DEBUG_LVL, RB_DEBUG_BASIC)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "Program rinblock at address %d from local buffer\n", rb->iic_address);
#endif

    /*
     * Check the rinblock is actually present on the bus
     */
    if (!rb->present)
    {
#if RB_DEBUG(RB_FLASH_DEBUG_LVL, RB_DEBUG_BASIC)
        RB_DEBUG_PRINTF(RB_DEBUG_DEV, "No rinblock present at address %d\n", rb->iic_address);
#endif
        *err = RB_ERR_NOT_PRESENT;
        return 0;
    }

    /*
     * Set the mode of the rin block to programming
     */
#if RB_DEBUG(RB_FLASH_DEBUG_LVL, RB_DEBUG_BASIC)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "Setting rinblock mode for programming\n");
#endif
    if (!RinBlockSetMode(rb, RB_MODE_PROGRAM))
    {
#if RB_DEBUG(RB_FLASH_DEBUG_LVL, RB_DEBUG_BASIC)
        RB_DEBUG_PRINTF(RB_DEBUG_DEV, "Set mode failed\n");
#endif
        *err = RB_ERR_PROGRAM_MODE;
        return 0;
    }

    /*
     * Get the flash configuration from the module
     */
    if (!RinBlockFlashGetConfig(rb, &iden))
    {
#if RB_DEBUG(RB_FLASH_DEBUG_LVL, RB_DEBUG_BASIC)
        RB_DEBUG_PRINTF(RB_DEBUG_DEV, "Get flash config failed\n");
#endif
        *err = RB_ERR_READ_FLASH_CFG;
        return 0;
    }

    /*
     * Sort out the byte orders
     */
    RinBlockByteSwapConfig(&iden);

    /*
     * Check that the flash image is valid for this module
     */
    if (!RinBlockFlashVerifyCode(rb, &iden, buf, start, length))
    {
#if RB_DEBUG(RB_FLASH_DEBUG_LVL, RB_DEBUG_BASIC)
        RB_DEBUG_PRINTF(RB_DEBUG_DEV, "Flash image doesn't match the Rin Block\n");
#endif
        *err = RB_ERR_FLASH_CFG_INCOMPATIBLE;
        return 0;
    }

    /*
     * Erase the flash in the module
     */
    if (!RinBlockFlashErase(rb, start, length))
    {
#if RB_DEBUG(RB_FLASH_DEBUG_LVL, RB_DEBUG_BASIC)
        RB_DEBUG_PRINTF(RB_DEBUG_DEV, "Rin Block Flash Erase Failed\n");
#endif
        *err = RB_ERR_FLASH_ERASE_FAIL;
        return 0;
    }

    /*
     * Program the encrypted data into the module
     */
    if (!RinBlockFlashProgramOrVerify(rb, buf, 1, start, length, work_buffer, err))
    {
#if RB_DEBUG(RB_FLASH_DEBUG_LVL, RB_DEBUG_BASIC)
        RB_DEBUG_PRINTF(RB_DEBUG_DEV, "Rin Block Programming Failed\n");
#endif
        return 0;
    }

    /*
     * Verify that it programmed correctly
     */
    if (!RinBlockFlashProgramOrVerify(rb, buf, 0, start, length, work_buffer, err))
    {
#if RB_DEBUG(RB_FLASH_DEBUG_LVL, RB_DEBUG_BASIC)
        RB_DEBUG_PRINTF(RB_DEBUG_DEV, "Rin Block Verify Failed\n");
#endif
        return 0;
    }

    /*
     * Disconnect/reset the rinblock
     */
    if (!RinBlockDisconnect(rb))
    {
#if RB_DEBUG(RB_FLASH_DEBUG_LVL, RB_DEBUG_BASIC)
        RB_DEBUG_PRINTF(RB_DEBUG_DEV, "Disconnect/reset rinblock failed\n");
#endif
        *err = RB_ERR_PROGRAM_DISCONNECT;
        return 0;
    }

    /*
     * Ask the module if programming is finished
     */
    while (!RinBlockProgrammingFinished(rb))
        OSTimeDly(OS_TICKS_PER_SEC / 10);

    /*
     * Restart the rinblock
     */
#if RB_DEBUG(RB_FLASH_DEBUG_LVL, RB_DEBUG_BASIC)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "Restarting the rinblock");
#endif
    RinBlockRestart(rb);
    *err = RB_ERR_NO_ERR;
    return 1;
}
