#include <string.h>
#include <rinblock/rinblock.h>
#include <rinblock/rb_command.h>
#include <rinblock/rb_debug.h>

#include <rinblock/rb_scratch.h>


/*
 * RinBlockReadScratchPad -
 *
 * Read length bytes into buffer buf from the scratchpad
 * of rinblock rb, starting at address start
 */
unsigned char RinBlockReadScratchPad(struct t_rinblock *rb, unsigned char *buf, unsigned char start, unsigned length)
{
    RB_DECLARE_CMD_BUFFER( rb_msg, struct s_rb_general_scratchpad_read_reply );
    unsigned char posn, read_len;

#if RB_DEBUG(RB_SCRATCHPAD_DEBUG_LVL, RB_DEBUG_BASIC)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "RinBlockReadScratchPad() called, ");
#endif

    posn = start;
    while (length > 0)
    {

        /*
         * Work out how many bytes to read this time
         */
        read_len = (unsigned char) ((length > MAX_RB_SCRATCHPAD_DATA_LENGTH) ? MAX_RB_SCRATCHPAD_DATA_LENGTH : length);

        /*
         * Setup command structure
         */
        rb_msg->data.rb_command.params.rb_general_scratchpad_read_cmd.start = posn;
        rb_msg->data.rb_command.params.rb_general_scratchpad_read_cmd.length = read_len;

        /*
         * Do the scratchpad read, and deal with any errors
         */
        if ( !RinBlockExecuteCommand( rb, rb_msg,
                                      RB_GET_SIZE_COMMAND( struct s_rb_general_scratchpad_read_cmd ),
                                      (unsigned short) RB_GET_SIZE_CMD_SCRATCH_READ( read_len ),
                                      RB_TIMEOUT_INDEFINITE, RB_CMD_READ_SCRATCHPAD ) )
        {
#if RB_DEBUG(RB_SCRATCHPAD_DEBUG_LVL, RB_DEBUG_BASIC)
            RB_DEBUG_PRINTF(RB_DEBUG_DEV, "bad reply\n");
#endif
            return 0;
        }

        /*
         * Read was successful, copy data into output buffer
         */
        memcpy( buf, rb_msg->data.rb_response.params.rb_general_scratchpad_read_reply.data, (size_t) read_len );

        /*
         * Update counters
         */
        buf += read_len;
        posn += read_len;
        length -= read_len;
    }

    /*
     * All done successfully
     */
#if RB_DEBUG(RB_SCRATCHPAD_DEBUG_LVL, RB_DEBUG_BASIC)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "completed successfully\n");
#endif
    return 1;
}


/*
 * RinBlockWriteScratchPad -
 *
 * Write length bytes from buffer buf to the scratchpad
 * of rinblock rb, starting at address start in the scratchpad
 */
unsigned char RinBlockWriteScratchPad(struct t_rinblock *rb, unsigned char *buf, unsigned char start, unsigned length)
{
    RB_DECLARE_CMD_BUFFER( rb_msg, struct s_rb_general_scratchpad_write );
    unsigned char posn, write_len;

#if RB_DEBUG(RB_SCRATCHPAD_DEBUG_LVL, RB_DEBUG_BASIC)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "RinBlockWriteScratchPad() called, ");
#endif

    posn = start;
    while (length > 0)
    {

        /*
         * Work out how many bytes to write this time
         */
        write_len = (unsigned char) ((length > MAX_RB_SCRATCHPAD_DATA_LENGTH) ? MAX_RB_SCRATCHPAD_DATA_LENGTH : length);

        /*
         * Setup command structure, copy in write data
         */
        rb_msg->data.rb_command.params.rb_general_scratchpad_write.start = posn;
        rb_msg->data.rb_command.params.rb_general_scratchpad_write.length = write_len;
        memcpy( rb_msg->data.rb_command.params.rb_general_scratchpad_write.data, buf, (size_t) write_len );

        /*
         * Do the scratchpad write, and deal with any errors
         */
        if (!RinBlockExecuteCommand( rb, rb_msg,
                                     (unsigned short) RB_GET_SIZE_CMD_SCRATCH_WRITE( write_len ),
                                     RB_GET_SIZE_CMD_HDR(),
                                     RB_TIMEOUT_INDEFINITE, RB_CMD_WRITE_SCRATCHPAD ) )
        {
#if RB_DEBUG(RB_SCRATCHPAD_DEBUG_LVL, RB_DEBUG_BASIC)
            RB_DEBUG_PRINTF(RB_DEBUG_DEV, "bad reply\n");
#endif
            return 0;
        }

        /*
         * Write was successful, update counters
         */
        posn += write_len;
        buf += write_len;
        length -= write_len;
    }

    /*
     * All done successfully
     */
#if RB_DEBUG(RB_SCRATCHPAD_DEBUG_LVL, RB_DEBUG_BASIC)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "completed successfully\n");
#endif
    return 1;
}
