/*
 * rb_shutdown.c
 *
 *  Created on: Jun 7, 2012
 *      Author: ian
 */
#include <rinhw/rio.h>
#include <rinhw/rstdio.h>
#include <rinblock/rb_debug.h>
#include <rinblock/rinblock.h>

#include <rinblock/rb_shutdown.h>


/** Clean up all the elements within the rinblock structure */
static void RinBlockCleanupStruct(struct t_rinblock *rinblock)
{
    unsigned char err;

#if RB_DEBUG(RB_SHUTDOWN_DEBUG_LVL, RB_DEBUG_BASIC)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "RinBlockCleanupStruct() called\n");
#endif

    /* Close the IIC device if open */
    if (NULL != rinblock->iicdev && 0 == (rinblock->iicdev->status & O_OPEN))
    {
      rclose(rinblock->iicdev);
    }

    /* Clean general rinblock parameters */
    rinblock->iicdev = NULL;
    rinblock->iic_address = RB_DEFAULT_ADDRESS;
    rinblock->bus_slot = 0;
    rinblock->present = rinblock->programmed = rinblock->running = 0;

    /* Destroy rinblock message handling parameters */
    OSSemDel(rinblock->msg_handling.busy_sem, OS_DEL_ALWAYS, &err);
#if RB_DEBUG(RB_SHUTDOWN_DEBUG_LVL, RB_DEBUG_BASIC)
    if (OS_NO_ERR != err)
    {
      RB_DEBUG_PRINTF(RB_DEBUG_DEV, "RinBlockCleanupStruct: Error destroying rinblock semaphore\n");
    }
#endif
    rinblock->msg_handling.busy_sem = NULL;

    /* Setup rinblock DIO to have no IO */
    rinblock->RBDioHardware = (enum eRBDioHardware *) NULL;

#if RB_DEBUG(RB_SHUTDOWN_DEBUG_LVL, RB_DEBUG_BASIC)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "RinBlockCleanupStruct() finished\n");
#endif
}


/** Clean up the rinblock library & shut everything down */
void RinBlockDestroy(void)
{
    struct t_rinblock *rbptr;
    unsigned char i;

#if RB_DEBUG(RB_SHUTDOWN_DEBUG_LVL, RB_DEBUG_BASIC)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "RinBlockDestroy() called\n");
#endif

    /* Clean up all the rinblock structures */
#if RB_DEBUG(RB_SHUTDOWN_DEBUG_LVL, RB_DEBUG_FULL)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "RinBlockDestroy: Clean up rinblock structures\n");
#endif
    rbptr = &rinblocks[0];
    for (i = 0; i < MAX_RINBLOCKS; i++, rbptr++)
    {
        RinBlockCleanupStruct(rbptr);
    }

#if 0 /* Don't close the debug device in case something else is using it */
    /* Close the debugging device if debugging is enabled */
#if (defined(RINBLOCK_DEBUG) && !RB_DEBUG_APP_CALLBACK)
    if (0 == (RB_DEBUG_DEV)->status & O_OPEN)
        rclose(RB_DEBUG_DEV);
#endif
#endif

#if RB_DEBUG(RB_SHUTDOWN_DEBUG_LVL, RB_DEBUG_BASIC)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "RinBlockDestroy() finished\n");
#endif
}
