#include <rinhw/rio.h>
#include <rinhw/rstdio.h>
#include <rinblock/rb_debug.h>
#include <rinblock/rinblock.h>
#include <rinblock/rb_command.h>

#include <rinblock/rb_status.h>


/*
 * RinBlockReadStatusShort -
 *
 * Reads the status word from the rinblock
 */
unsigned char RinBlockReadStatusShort(struct t_rinblock *rb, unsigned short *status)
{
    RB_DECLARE_CMD_BUFFER( rb_msg, struct s_rb_status_short_read );

    /*
     * Setup command and execute, and deal with errors
     */
    if ( !RinBlockExecuteCommand( rb, rb_msg,
                                  RB_GET_SIZE_CMD_HDR(),
                                  RB_GET_SIZE_COMMAND( struct s_rb_status_short_read ),
                                  RB_TIMEOUT_DEFAULT, RB_CMD_READ_STATUS_SHORT ) )
    {
#if RB_DEBUG(RB_STATUS_DEBUG_LVL, RB_DEBUG_BASIC)
        RB_DEBUG_PRINTF(RB_DEBUG_DEV, "Module didn't reply to RB_CMD_READ_STATUS_SHORT command\n");
#endif
        return 0;
    }

    /*
     * Success, copy result, return success
     */
#if RB_DEBUG(RB_STATUS_DEBUG_LVL, RB_DEBUG_FULL)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "Reply success from module to RB_CMD_READ_STATUS_SHORT, status 0x%04x\n", rb_msg->data.rb_response.params.rb_status_short_read.status);
#endif
    *status = ntohs( rb_msg->data.rb_response.params.rb_status_short_read.status );
    return 1;
}


#if 0 /* This is not used anywhere, so I've removed it */
/*
 * RinBlockReadStatusLong -
 *
 * Reads the (long) status struct from the rinblock
 */
unsigned char RinBlockReadStatusLong(struct t_rinblock *rb, struct s_rb_status_long_read *status_long)
{
    RB_DECLARE_CMD_BUFFER( rb_msg, struct s_rb_status_long_read );

    /*
     * Setup command and execute, and deal with errors
     */
    if ( !RinBlockExecuteCommand( rb, rb_msg,
                                  RB_GET_SIZE_CMD_HDR(),
                                  RB_GET_SIZE_COMMAND( struct s_rb_status_long_read ),
                                  RB_TIMEOUT_DEFAULT, RB_CMD_READ_STATUS_LONG ) )
    {
#if RB_DEBUG(RB_STATUS_DEBUG_LVL, RB_DEBUG_BASIC)
        RB_DEBUG_PRINTF(RB_DEBUG_DEV, "Module didn't reply to RB_CMD_READ_STATUS_LONG command\n");
#endif
        return 0;
    }

    /*
     * Success, copy result, return success
     */
#if (BYTE_ORDER == NETWORK_ENDIAN)
    *status_long = rb_msg->data.rb_response.params.rb_status_long_read;
#else /* #if (BYTE_ORDER == NETWORK_ENDIAN) */
    status_long->long_status = ntohl( rb_msg->data.rb_response.params.rb_status_long_read.long_status );
    status_long->short_status = ntohs( rb_msg->data.rb_response.params.rb_status_long_read.short_status );
#endif /* #else -> #if (BYTE_ORDER == NETWORK_ENDIAN) */

#if RB_DEBUG(RB_STATUS_DEBUG_LVL, RB_DEBUG_FULL)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "Reply success from module to RB_CMD_READ_STATUS_LONG, long status 0x%08lx, short status 0x%04x\n",
             status_long->long_status,
             status_long->short_status);
#endif
    return 1;
}
#endif /* #if 0 */
