#include <rinhw/rio.h>
#include <rinhw/rstdio.h>
#include <rinblock/rb_debug.h>
#include <rinblock/rinblock.h>
#include <rinblock/rb_command.h>

#include "_rb_tag.h"
#include <rinblock/rb_tag.h>


#if RB_DEBUG(RB_STARTUP_DEBUG_LVL, RB_DEBUG_BASIC)

void RinBlockDumpTag(struct s_rb_general_tag *tag, unsigned char addr)
{
	RB_DEBUG_PRINTF(RB_DEBUG_DEV, "Module address %d, TAG: Model %04x, Serial number: %ld, Software Version: %04x, ",
    		 addr, tag->model_number, tag->serial_number, tag->software_version);
	RB_DEBUG_PRINTF(RB_DEBUG_DEV, "Interface Version: %04x, Interface Type: %04x, Hardware Version: %04x\n",
    		 tag->interface_version, tag->interface_type, tag->hardware_version);
	RB_DEBUG_PRINTF(RB_DEBUG_DEV, "Tag Dump:\n");
    RBDump((unsigned char *) tag, sizeof(struct s_rb_general_tag));
}

#endif


/*
 * RinBlockReadTag -
 *
 * Reads the tag from a rin block.
 */
unsigned char RinBlockReadTag(struct t_rinblock *rb, struct s_rb_general_tag *tag)
{
    RB_DECLARE_CMD_BUFFER( rb_msg, struct s_rb_general_tag );
    unsigned char res;

    /*
     * Setup command and execute
     */
    rb_msg->data.rb_command.header.cmd = RB_CMD_READ_TAG;
    res = RinBlockInternalExecuteCommand(rb, rb_msg,
                                         RB_GET_SIZE_CMD_HDR(),
                                         RB_GET_SIZE_COMMAND( struct s_rb_general_tag ),
                                         RB_TIMEOUT_TAG, 0);

#if RB_DEBUG(RB_STARTUP_DEBUG_LVL, RB_DEBUG_FULL)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "RB_CMD_READ_TAG returned with result %d, response 0x%02x\n", res, rb_msg->data.rb_response.header.resp);
#endif

    /*
     * Check for errors, and copy tag if success
     */
    res = ((res) && ((rb_msg->data.rb_response.header.resp & RB_RESP_ERROR_MASK) == 0));
    if (res)
    {
#if (BYTE_ORDER == NETWORK_ENDIAN)
      *tag = rb_msg->data.rb_response.params.rb_general_tag;
#else /* #if (BYTE_ORDER == NETWORK_ENDIAN) */
      tag->model_number = ntohs( rb_msg->data.rb_response.params.rb_general_tag.model_number );
      tag->serial_number = ntohl( rb_msg->data.rb_response.params.rb_general_tag.serial_number );
      tag->software_version = ntohs( rb_msg->data.rb_response.params.rb_general_tag.software_version );
      tag->interface_version = ntohs( rb_msg->data.rb_response.params.rb_general_tag.interface_version );
      tag->interface_type = ntohs( rb_msg->data.rb_response.params.rb_general_tag.interface_type );
      tag->hardware_version = ntohs( rb_msg->data.rb_response.params.rb_general_tag.hardware_version );
      /* tag->unused[2]; */
#endif /* #else -> #if (BYTE_ORDER == NETWORK_ENDIAN) */
    }
    return res;
}
