#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <stdbool.h>
#include <rinhw/lcd_generic.h>
#include <rinhw/rio.h>
#include <rinhw/lcd.h>
#include <rinblock/rb_debug.h>
#include <rinblock/rb_msg.h>
#include <rinblock/rinblock.h>
#include <rinblock/rb_command.h>
#include "_rb_we21_lcdsegs.h"

#include "_rb_we21_lcd_common.h"


static unsigned char RinBlockWe21WriteDisplay(struct t_rinblock *rb, const struct s_rb_we21_disp_write_7seg *we21_disp)
{
  RB_DECLARE_CMD_BUFFER( rb_msg, struct s_rb_we21_disp_write_7seg );

  /*
   * Write the config
   */
  rb_msg->data.rb_command.params.rb_we21_disp_7seg = *we21_disp;

  return RinBlockExecuteCommand( rb, rb_msg,
                                 RB_GET_SIZE_COMMAND( struct s_rb_we21_disp_write_7seg ),
                                 RB_GET_SIZE_CMD_HDR(),
                                 RB_TIMEOUT_DEFAULT, RB_CMD_WE21_DISP_WRITE_7SEG );
}


/** Write data to the LCD on the rinblock */
int rb_we21_lcd_write_data(struct s_lcd_definition *lcd_definition)
{
  struct t_rinblock *rb = lcd_definition->hw_data;
  unsigned char result;
  struct s_rb_we21_disp_write_7seg *segs;

#if RB_DEBUG(RB_LCD_DEBUG_LVL, RB_DEBUG_FULL)
  RB_DEBUG_PRINTF(RB_DEBUG_DEV, "rb_we21_lcd_write_data called\n");
#endif

  if (NULL != lcd_definition->lcd_output_segments)
    segs = (struct s_rb_we21_disp_write_7seg *) lcd_definition->lcd_output_segments;
  else
    segs = (struct s_rb_we21_disp_write_7seg *) lcd_definition->lcd_segments;
  result = RinBlockWe21WriteDisplay(rb, segs);

#if RB_DEBUG(RB_LCD_DEBUG_LVL, RB_DEBUG_FULL)
  if (0 == result)
  {
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "RinBlockWe21WriteDisplay failed\n");
  }
#endif
  return (result) ? RIO_SUCCESS : RIO_ERROR;
}


/** Write a character ch to digit position index on the LCD.
 *
 * cwm is the write mode - either set or or the data
 */
void rb_we21_lcd_put_char(struct s_lcd_definition *lcd_definition, unsigned char index, unsigned char ch, enum eCharWriteMode cwm)
{
  unsigned char segments;

#if RB_DEBUG(RB_LCD_DEBUG_LVL, RB_DEBUG_BASIC)
  RB_DEBUG_PRINTF(RB_DEBUG_DEV, "rb_we21_lcd_put_char called to write char 0x%02x to lcd digit %d\n", ch, index);
#endif

  /* Is the character in the table, if not display it with all segments on */
  if (ch >= NUM_SEG_TABLE)
  {
    ch = 11;
#if RB_DEBUG(RB_LCD_DEBUG_LVL, RB_DEBUG_FULL)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "Char modified to 0x%02x because outside table range (max 0x%02x)\n", ch, NUM_SEG_TABLE);
#endif
  }

  /* Convert the segments into a bitmap */
  segments = rb_we21_seg_table[ch];
#if RB_DEBUG(RB_LCD_DEBUG_LVL, RB_DEBUG_BASIC)
  RB_DEBUG_PRINTF(RB_DEBUG_DEV, "Character 0x%02x translates to bitmap 0x%02x\n", ch, segments);
#endif

  /* Modify the data, as dictated by the write mode */
  if (cwm == cwmOR)
  {
    /* Read the current data out of the LCD buffer */
    segments |= lcd_definition->lcd_segments[index];
  }
  lcd_definition->lcd_segments[index] = segments;
}


/** Work out if the digit index has a decimal point */
bool rb_we21_lcd_does_digit_have_dp(struct s_lcd_definition *lcd_definition, unsigned char index)
{
  /* Make sure its in range */
  if (index >= lcd_definition->lcd_num_characters)
    return false;

  /* All digits have decimal points on the WE2111 */
  return true;
}
