/*
 * rb_we21_lcd_segflash.c
 *
 *  Created on: 29/05/2012
 *      Author: ian
 */
#include <rinhw/rio.h>
#include <rinhw/lcd_generic.h>
#include <rinblock/rinblock.h>
#include <rinblock/rb_debug.h>
#include "_rb_we21_lcd_common.h"

#include <rinblock/rb_we21_lcd.h>


/** Create a LCD device for a rinblock with segment flash */
int RinBlockCreateWE21LcdDeviceSegFlash(struct t_io_dev *io_dev, struct t_rinblock *rb)
{
  /*
   * Check that the mapped rinblock is actually a display one.
   * Note that the C500 display supports RB_CMD_WE21_DISP_WRITE_7SEG, so we allow it here */
  if (RB_IFACE_TYPE_WE21_DISP != rb->tag.interface_type &&
      RB_IFACE_TYPE_C500_DISP != rb->tag.interface_type)
  {
#if RB_DEBUG(RB_LCD_DEBUG_LVL, RB_DEBUG_BASIC)
    RB_DEBUG_PRINTF(RB_DEBUG_DEV, "RinBlockCreateWE21LcdDeviceSegFlash: attempting to map lcd to non-lcd rinblock\n");
#endif
    goto rbcwld_fail;
  }

  /* Setup the new LCD device */
  if (lcd_generic_segflash_create_device(io_dev,
                                         rb,
                                         sizeof(struct s_rb_we21_disp_write_7seg),
                                         WE21_DISPLAY_NUM_DIGITS,
                                         NULL,
                                         NULL,
                                         rb_we21_lcd_write_data,
                                         rb_we21_lcd_put_char,
                                         rb_we21_lcd_does_digit_have_dp,
                                         NULL) < 0)
  {
    goto rbcwld_fail;
  }

  /* Success */
  return RIO_SUCCESS;

rbcwld_fail:
  io_dev->dev_data = NULL;
  return RIO_ERROR;
}
