#include <rinhw/rinendian.h>

#include "_rb_we21_lcdsegs.h"


/*
 * The segment mapping is as follows:
 *
 *    ---SA--
 *    |     |
 *   SF     SB
 *    |     |
 *    ---SG--
 *    |     |
 *   SE     SC
 *    |     |  __
 *    ---SD-- |SH|
 *
 * Segment defines are the bit position offsets from 0 for a digit
 */
/* #if (BYTE_ORDER == BIG_ENDIAN) */
  #define SA 7
  #define SB 6
  #define SC 5
  #define SD 4
  #define SE 3
  #define SF 2
  #define SG 1
  #define SH 0
/*
#elif (BYTE_ORDER == LITTLE_ENDIAN)
  #define SA 0
  #define SB 1
  #define SC 2
  #define SD 3
  #define SE 4
  #define SF 5
  #define SG 6
  #define SH 7
#else
  #error "Unknown endian while generating LCD segment masks"
#endif
*/

/*
 *  Convert segment number to mask
 */
#define MASK(n) (1 << n)


/*
 * Segment bit masks
 */
#define MS_NONE 0x0000
#define MSA MASK(SA)
#define MSB MASK(SB)
#define MSC MASK(SC)
#define MSD MASK(SD)
#define MSE MASK(SE)
#define MSF MASK(SF)
#define MSG MASK(SG)
#define MSH MASK(SH)
#define MS_ALL (MSA | MSB | MSC | MSD | MSE | MSF | MSG | MSH)


/*
 * The LCD segment table
 */
const unsigned char rb_we21_seg_table[NUM_SEG_TABLE] LCDSEG_TABLE_MEMORY =
{
    MS_NONE,        /* 0 */
    MSA,            /* 1 */
    MSB,            /* 2 */
    MSC,            /* 3 */
    MSD,            /* 4 */
    MSE,            /* 5 */
    MSF,            /* 6 */
    MSG,            /* 7 */
    MSH,            /* 8 */
    MS_NONE,        /* 9 */
    MS_NONE,        /* 10 */
    MS_ALL,         /* 11 */
    MS_NONE,        /* 12 */
    MS_NONE,        /* 13 */
    MS_NONE,        /* 14 */
    MS_NONE,        /* 15 */
    MS_NONE,        /* 16 */
    MS_NONE,        /* 17 */
    MS_NONE,        /* 18 */
    MS_NONE,        /* 19 */
    MS_NONE,        /* 20 */
    MS_NONE,        /* 21 */
    MS_NONE,        /* 22 */
    MS_NONE,        /* 23 */
    MS_NONE,        /* 24 */
    MS_NONE,        /* 25 */
    MS_NONE,        /* 26 */
    MS_NONE,        /* 27 */
    MS_NONE,        /* 28 */
    MS_NONE,        /* 29 */
    MS_NONE,        /* 30 */
    MS_NONE,        /* 31 */

    (MS_NONE),                                  /* 32 Space */
    (MSB | MSH),                                /* 33 ! */
    (MSB | MSF),                                /* 34 " */
    (MSB | MSF | MSE | MSC),                    /* 35 # */
    (MS_NONE),                                  /* 36 $ */
    (MSB | MSG | MSE | MSH),                    /* 37 % */
    (MS_NONE),                                  /* 38 & */
    (MSB),                                      /* 39 ' */
    (MSA | MSD | MSE | MSF),                    /* 40 ( */
    (MSA | MSB | MSC | MSD),                    /* 41 ) */
    (MS_NONE),                                  /* 42 * */
    (MS_NONE),                                  /* 43 + */
    (MSC),                                      /* 44 , */
    (MSG),                                      /* 45 - */
    (MSH),                                      /* 46 . */
    (MSB | MSE | MSG),                          /* 47 / */

    (MSA | MSB | MSC | MSD | MSE | MSF),        /* 48 0 */
    (MSB | MSC),                                /* 49 1 */
    (MSA | MSB | MSD | MSE | MSG),              /* 50 2 */
    (MSA | MSB | MSC | MSD | MSG),              /* 51 3 */
    (MSB | MSC | MSF | MSG),                    /* 52 4 */
    (MSA | MSC | MSD | MSF | MSG),              /* 53 5 */
    (MSA | MSC | MSD | MSE | MSF | MSG),        /* 54 6 */
    (MSA | MSB | MSC),                          /* 55 7 */
    (MSA | MSB | MSC | MSD | MSE | MSF | MSG),  /* 56 8 */
    (MSA | MSB | MSC | MSD | MSF | MSG),        /* 57 9 */

    (MS_NONE),                                  /* 58 : */
    (MSF | MSC),                                /* 59 ; */
    (MSB | MSC | MSG),                          /* 60 < */
    (MSD | MSG),                                /* 61 = */
    (MSE | MSF | MSG),                          /* 62 > */
    (MSA | MSB | MSE | MSG | MSH),              /* 63 ? */
    (MS_NONE),                                  /* 64 @ */

    (MSA | MSB | MSC | MSE | MSF | MSG),        /* 65 A */
    (MSC | MSD | MSE | MSF | MSG),              /* 66 B */
    (MSA | MSD | MSE | MSF),                    /* 67 C */
    (MSB | MSC | MSD | MSE | MSG),              /* 68 D */
    (MSA | MSD | MSE | MSF | MSG),              /* 69 E */
    (MSA | MSE | MSF | MSG),                    /* 70 F */
    (MSA | MSC | MSD | MSE | MSF),              /* 71 G */
    (MSB | MSC | MSE | MSF | MSG),              /* 72 H */
    (MSC),                                      /* 73 I */
    (MSB | MSC | MSD),                          /* 74 J */
    (MSC | MSE | MSF),                          /* 75 K */
    (MSD | MSE | MSF),                          /* 76 L */
    (MSA | MSC | MSE | MSG),                    /* 77 M */
    (MSA | MSB | MSC | MSE | MSF),              /* 78 N */
    (MSA | MSB | MSC | MSD | MSE | MSF),        /* 79 O */
    (MSA | MSB | MSE | MSF | MSG),              /* 80 P */
    (MSA | MSB | MSC | MSF | MSG),              /* 81 Q */
    (MSE | MSG),                                /* 82 R */
    (MSA | MSC | MSD | MSF | MSG),              /* 83 S */
    (MSD | MSE | MSF | MSG),                    /* 84 T */
    (MSB | MSC | MSD | MSE | MSF),              /* 85 U */
    (MSC | MSD | MSE),                          /* 86 V */
    (MSA | MSC | MSD | MSE),                    /* 87 W */
    (MSB | MSC | MSE | MSF | MSG),              /* 88 X, looks the same as H, but we need something */
    (MSB | MSC | MSD | MSF | MSG),              /* 89 Y */
    (MSA | MSB | MSD | MSE | MSG),              /* 90 Z */

    (MSA | MSD | MSE | MSF),                    /* 91 [ */
    (MSC | MSF | MSG),                          /* 92 \ */
    (MSA | MSB | MSC | MSD),                    /* 93 ] */
    (MSA | MSB | MSF),                          /* 94 ^ */
    (MSD),                                      /* 95 _ */
    (MSF),                                      /* 96 ` */

    (MSA | MSB | MSC | MSE | MSF | MSG),        /* 97 a */
    (MSC | MSD | MSE | MSF | MSG),              /* 98 b */
    (MSD | MSE | MSG),                          /* 99 c */
    (MSB | MSC | MSD | MSE | MSG),              /* 100 d */
    (MSA | MSD | MSE | MSF | MSG),              /* 101 e */
    (MSA | MSE | MSF | MSG),                    /* 102 f */
    (MSA | MSB | MSC | MSD | MSF | MSG),        /* 103 g */
    (MSC | MSE | MSF | MSG),                    /* 104 h */
    (MSC),                                      /* 105 i */
    (MSB | MSC | MSD),                          /* 106 j */
    (MSC | MSE | MSF),                          /* 107 k */
    (MSD | MSE | MSF),                          /* 108 l */
    (MSA | MSC | MSE | MSG),                    /* 109 m */
    (MSC | MSE | MSG),                          /* 110 n */
    (MSC | MSD | MSE | MSG),                    /* 111 o */
    (MSA | MSB | MSE | MSF | MSG),              /* 112 p */
    (MSA | MSB | MSC | MSF | MSG),              /* 113 q */
    (MSE | MSG),                                /* 114 r */
    (MSA | MSC | MSD | MSF | MSG),              /* 115 s */
    (MSD | MSE | MSF | MSG),                    /* 116 t */
    (MSC | MSD | MSE),                          /* 117 u */
    (MSC | MSD | MSE),                          /* 118 v */
    (MSA | MSC | MSD | MSE),                    /* 119 w */
    (MSB | MSC | MSE | MSF | MSG),              /* 120 x, looks the same as H, but we need something */
    (MSB | MSC | MSD | MSF | MSG),              /* 121 y */
    (MSA | MSB | MSD | MSE | MSG),              /* 122 z */

    (MSA | MSF | MSE | MSD),                    /* 123 { */
    (MSB | MSC),                                /* 124 | */
    (MSA | MSB | MSC | MSD),                    /* 125 } */
    (MSA),                                      /* 126 ~ */
    (MS_NONE),                                  /* 127 Del */
};
