#include <stdio.h>
#include <string.h>
#include <rinhw/hwlib_init.h>
#include <rinhw/debug.h>
#include <rinhw/we21hw.h>
#include <rinhw/rio.h>
#include <rinhw/sio_local_console.h>
#include <rinhw/iic_local_linux.h>
#include <rinhw/iic_ftdi.h>
#include <rinhw/adc_ads1216_local.h>
#include <rinhw/rtc_local.h>
#include <rinhw/sio.h>
#include <rinhw/sio_local_comport.h>
#include <rinhw/framdev_local.h>
#include <rinremote/remote.h>
#include <rinremote/remoteexec.h>
#include <rinremote/iic_remote.h>
#include "config.h"

#include "devices.h"

struct t_io_dev dev_console;
struct t_io_dev dev_iic0;
struct t_io_dev dev_kbd;
struct t_io_dev dev_adc;
struct t_io_dev dev_buz;
struct t_io_dev dev_lcd;
struct t_io_dev dev_lcd_local_skt;


static void test_debug_printf_fn(int priority, const char *format, va_list args)
{
  vfprintf(stdout, format, args);
}


/** Create devices for remote devices */
static bool init_test_app_devices_remote(const char *cmdline_remote_device)
{
  /* Start the hardware library */
  hwlib_init();

  /* Initialise remote device comms */
  if (vRemoteExecuteInit(cmdline_remote_device, NULL, 0) < 0)
  {
    fprintf(stderr, "init_test_app_devices_remote: failed to initialise remote comms\n");
    return false;
  }

  /* Make a console for IO */
  if (rio_create_device_sio_localconsole(&dev_console) < 0)
  {
    fprintf(stderr, "init_test_app_devices_remote: local console serial device creation failed\n");
    return false;
  }

  /* Remote I2C device for rinblocks */
  if (rio_create_device_iic_remote(&dev_iic0, REMOTE_DEV_IIC0) < 0)
  {
    fprintf(stderr, "init_test_app_devices_remote: remote i2c device creation failed\n");
    return false;
  }

  /* Buzzer, Keyboard, LCD devices are created as part of the rinblocks */
  return true;
}


/** Create devices for local devices */
static bool init_test_app_devices_normal(const char *i2c_device)
{
  /* Start the hardware library */
  hwlib_init();
  rio_init_we21hw("/dev/we21hw");

  /* Make a console for IO */
  if (rio_create_device_sio_localconsole(&dev_console) < 0)
  {
    fprintf(stderr, "init_test_app_devices_normal: local console serial device creation failed\n");
    return false;
  }

/* Not required, because rinblock creates it
  if (rio_create_device_buzzer_local(&dev_buz_we) < 0)
    WE_DEBUG_PRINTF(WE_DEBUG_DEV, "Buzzer device creation failed\n");
*/
  if (strcmp(i2c_device, LIBMPSSE_FTDI_DEVICE_NAME) != 0)
  {
	if (rio_create_device_iic_local(&dev_iic0, i2c_device, we21hw_set_slot_select, NULL) < 0)
	{
	  fprintf(stderr, "init_test_app_devices_normal: local i2c device creation failed for %s\n", i2c_device);
	  return false;
	}
  }
#if HAVE_LIBMPSSE
  else
  {
    if (rio_create_device_iic_ftdi(&dev_iic0) < 0)
    {
	  fprintf(stderr, "init_test_app_devices_normal: local ftdi i2c device creation failed\n");
	  return false;
    }
  }
#endif

  /* Buzzer, Keyboard, LCD devices are created as part of the rinblocks */
  return true;
}


/** Start up the system */
bool init_test_app_devices(const char *remote_device, const char *i2c_device)
{
  bool result;

  /* Setup debug printf */
  rinhw_set_debug_printf_fn(test_debug_printf_fn);

  /* Create the devices */
  printf("init_test_app_devices: create devices\n");

  if (NULL != remote_device)
  {
    result = init_test_app_devices_remote(remote_device);
  }
  else
  {
    result = init_test_app_devices_normal(i2c_device);
  }
  return result;
}


/** Shut down the system */
void destroy_test_app_devices(void)
{
  printf("main: shutdown initiated\n");

  /* Destroy devices */
  rdestroy_io(&dev_iic0);
  rdestroy_io(&dev_console);

  /* Shutdown the hardware library */
  rio_destroy_we21hw();
  hwlib_destroy();
}
