#include <rinhw/rio.h>
#include <rinhw/sio.h>
#include <rinhw/rstdio.h>
#include <rinhw/gpio.h>
#include <rinblock/rb_aout.h>
#include <rinblock/rb_dio.h>
#include <rinblock/rinblock.h>
#include <rinblock/rb_aout_cal.h>
#include <rinblock/crc.h>
#include "rb_dio_test.h"

#include "rb_aout_test.h"


static unsigned char get_char(struct t_io_dev *debug_dev)
{
    unsigned char ch;

    rread(debug_dev, &ch, 1, 0);
    return ch;
}


static unsigned char get_1_digit_num(struct t_io_dev *debug_dev, unsigned char *num)
{
    unsigned char ch;

    ch = get_char(debug_dev);
    if ((ch < '0') || (ch > '9'))
        return 0;
    *num = (unsigned char) (ch - '0');
    return 1;
}

#if 0
static unsigned char get_2_digit_num(struct t_io_dev *debug_dev, unsigned char *num)
{
    unsigned char ch;

    if (!get_1_digit_num(debug_dev, &ch))
        return 0;

    ch *= 10;

    if (!get_1_digit_num(debug_dev, num))
        return 0;

    *num += ch;
    return 1;
}
#endif

static unsigned char get_n_digit_u_num(struct t_io_dev *debug_dev, unsigned long *num, unsigned char len)
{
    unsigned char ch;

    *num = 0UL;
    while (len-- > 0)
    {
        if (!get_1_digit_num(debug_dev, &ch))
            return 0;
        *num *= 10;
        *num += ch;
    }
    return 1;
}


static unsigned char get_n_digit_s_num(struct t_io_dev *debug_dev, signed long *num, unsigned char len)
{
    unsigned char ch;

    *num = 0UL;
    while (len-- > 0)
    {
        ch = get_char(debug_dev);
        if ((ch >= '0') && (ch <= '9'))
        {
            *num *= 10;
            *num += (ch - '0');
        }
        else if (ch == '-')
        {
            *num = -*num;
        }
        else
        {
            return 0;
        }
    }
    return 1;
}


#if 0
static unsigned char get_n_digit_num(struct t_io_dev *debug_dev, signed long *num, unsigned char len)
{
    unsigned char ch;

    *num = 0UL;
    while (len-- > 0)
    {
        if (!get_1_digit_num(debug_dev, &ch))
            return 0;
        *num *= 10;
        *num += ch;
    }
    return 1;
}
#endif


static unsigned char get_long(struct t_io_dev *debug_dev, signed long *num)
{
    unsigned char ch;

    *num = 0;
    do
    {
        ch = get_char(debug_dev);
        if ((ch >= '0') && (ch <= '9'))
        {
            *num *= 10;
            *num += (unsigned char) (ch - '0');
            rwrite(debug_dev, &ch, 1, 0);
        }
        if (ch == '-')
        {
            *num = -*num;
            rwrite(debug_dev, &ch, 1, 0);
        }
    }
    while (ch != '\r' && ch != '\n');
    return 1;
}


static void ReadFactoryCal(struct t_io_dev *debug_dev)
{
    struct s_rb_aout_cal aout_cal;
    unsigned char rb_num;
    unsigned short calculated_crc;

    rfprintf(debug_dev, "Which rinblock (0..%d)?:", MAX_RINBLOCKS - 1);
    rread(debug_dev, &rb_num, 1, 0);
    rfprintf(debug_dev, "%c\r\n", rb_num);
    if ((rb_num < '0') || (rb_num >= MAX_RINBLOCKS + '0'))
    {
        rfprintf(debug_dev, "Out of range");
        return;
    }
    rb_num -= '0';

    rfprintf(debug_dev, "Read rinblock aout cal ");
    if (RinBlockAOutReadFactoryCal(&rinblocks[rb_num], &aout_cal) == 0)
    {
        rfprintf(debug_dev, "failed\r\n");
        return;
    }
    rfprintf(debug_dev, "Success. Results:\r\n");

    rfprintf(debug_dev, "Factory Cal 0V: %u\r\n", aout_cal.uiFactoryCal0V);
    rfprintf(debug_dev, "Factory Cal 10V: %u\r\n", aout_cal.uiFactoryCal10V);
    rfprintf(debug_dev, "Factory Cal 4mA: %u\r\n", aout_cal.uiFactoryCal4mA);
    rfprintf(debug_dev, "Factory Cal 20mA: %u\r\n", aout_cal.uiFactoryCal20mA);
    rfprintf(debug_dev, "Factory Cal CRC: 0x%04x\r\n", aout_cal.uiCRCValue);

    calculated_crc = CRC16_Calc((unsigned char *) &aout_cal, sizeof(aout_cal) - sizeof(aout_cal.uiCRCValue), 0);
    rfprintf(debug_dev, "Calculated Cal CRC (should match Factory Cal CRC): 0x%04x\r\n", calculated_crc);
}


static void WriteFactoryCal(struct t_io_dev *debug_dev)
{
    struct s_rb_aout_cal aout_cal;
    unsigned char rb_num;
    signed long num;

    rfprintf(debug_dev, "Which rinblock (0..%d)?:", MAX_RINBLOCKS - 1);
    rread(debug_dev, &rb_num, 1, 0);
    rfprintf(debug_dev, "%c\r\n", rb_num);
    if ((rb_num < '0') || (rb_num >= MAX_RINBLOCKS + '0'))
    {
        rfprintf(debug_dev, "Out of range");
        return;
    }
    rb_num -= '0';

    /*
     * Factory cal 0v
     */
    rfprintf(debug_dev, "Enter Factory Cal 0V and LF: ");
    if (!get_long(debug_dev, &num))
    {
        rfprintf(debug_dev, "\r\nBad number\r\n");
        return;
    }
    if ((num < 0) || (num > 65535))
    {
        rfprintf(debug_dev, "\r\nOut of range\r\n");
    }
    aout_cal.uiFactoryCal0V = (unsigned short) num;

    /*
     * Factory cal 10v
     */
    rfprintf(debug_dev, "\r\nEnter Factory Cal 10V and LF: ");
    if (!get_long(debug_dev, &num))
    {
        rfprintf(debug_dev, "\r\nBad number\r\n");
        return;
    }
    if ((num < 0) || (num > 65535))
    {
        rfprintf(debug_dev, "\r\nOut of range\r\n");
    }
    aout_cal.uiFactoryCal10V = (unsigned short) num;

    /*
     * Factory cal 4mA
     */
    rfprintf(debug_dev, "\r\nEnter Factory Cal 4mA and LF: ");
    if (!get_long(debug_dev, &num))
    {
        rfprintf(debug_dev, "\r\nBad number\r\n");
        return;
    }
    if ((num < 0) || (num > 65535))
    {
        rfprintf(debug_dev, "\r\nOut of range\r\n");
    }
    aout_cal.uiFactoryCal4mA = (unsigned short) num;

    /*
     * Factory cal 20mA
     */
    rfprintf(debug_dev, "\r\nEnter Factory Cal 20mA and LF: ");
    if (!get_long(debug_dev, &num))
    {
        rfprintf(debug_dev, "\r\nBad number\r\n");
        return;
    }
    if ((num < 0) || (num > 65535))
    {
        rfprintf(debug_dev, "\r\nOut of range\r\n");
    }
    aout_cal.uiFactoryCal20mA = (unsigned short) num;

    /*
     * Calculate crc
     */
    aout_cal.uiCRCValue = CRC16_Calc((unsigned char *) &aout_cal, sizeof(aout_cal) - sizeof(aout_cal.uiCRCValue), 0);

    /*
     * Print new values
     */
    rfprintf(debug_dev, "\r\nFactory Cal Values to be sent are:\r\n");
    rfprintf(debug_dev, "Factory Cal 0V: %u\r\n", aout_cal.uiFactoryCal0V);
    rfprintf(debug_dev, "Factory Cal 10V: %u\r\n", aout_cal.uiFactoryCal10V);
    rfprintf(debug_dev, "Factory Cal 4mA: %u\r\n", aout_cal.uiFactoryCal4mA);
    rfprintf(debug_dev, "Factory Cal 20mA: %u\r\n", aout_cal.uiFactoryCal20mA);
    rfprintf(debug_dev, "Factory Cal CRC: 0x%04x\r\n", aout_cal.uiCRCValue);

    rfprintf(debug_dev, "Write rinblock aout cal ");
    if (RinBlockAOutWriteFactoryCal(&rinblocks[rb_num], &aout_cal) == 0)
    {
        rfprintf(debug_dev, "failed\r\n");
        return;
    }
    rfprintf(debug_dev, "Success.\r\n");
}


static void aout_set_output(struct t_io_dev *debug_dev)
{
    unsigned char rb_num, res;
    signed long val;

    rfprintf(debug_dev, "Cycling outputs, any key to quit\r\n");
    do
    {
        rfprintf(debug_dev, "Which rinblock (0..%d), Q = quit?:", MAX_RINBLOCKS - 1);
        rread(debug_dev, &rb_num, 1, 0);
        rfprintf(debug_dev, "%c\r\n", rb_num);
        if (rb_num == 'Q')
            return;
        if ((rb_num < '0') || (rb_num >= MAX_RINBLOCKS + '0'))
        {
            rfprintf(debug_dev, "Out of range");
            continue;
        }
        rb_num -= '0';

        /*
         * Get a value to write
         */
        rfprintf(debug_dev, "Enter value: ");
        get_long(debug_dev, &val);
        rfprintf(debug_dev, "\r\n");

        /*
         * Write the value
         */
        res = RinBlockAOutWrite(&rinblocks[rb_num], val);
        rfprintf(debug_dev, "Write to AOUT rinblock %d, value %ld, returned with result %d\r\n", rb_num, val, res);

#if 0
        /*
         * Read it back
         */
        rlseek(rb_aout_dev, (long) (ao_num * AOUT_CHANNEL_SIZE), SEEK_SET);
        res = rread(rb_aout_dev, &val, 2, 0);
        rfprintf(debug_dev, "Read from AOUT channel %d returned with result %d, value %d\r\n", ao_num, res, val);
#endif
    }
    while (1);
}


static void aout_triangle(struct t_io_dev *debug_dev)
{
    signed long output, increment, low_lim, high_lim;
/*    struct t_rb_aout_configure aout_config; */
    unsigned char ch;

#if 0
    aout_config.ucAnalogOutputType = 1;
    aout_config.iUserZero = 0;
    aout_config.iUserSpan = 0;
    aout_config.cMaxLimit = 100;
    aout_config.cMinLimit = 0;
    aout_config.uiUpdateRate = 1000;  // 100hz
    RinBlockAOutWriteConfig(&rinblocks[0], &aout_config);
#endif

    rfprintf(debug_dev, "Analog Output Test\r\n\r\nEnter lower limit");
    if (!get_long(debug_dev, &low_lim))
    {
        rfprintf(debug_dev, "Bad value\r\n");
        return;
    }

    rfprintf(debug_dev, "\r\nEnter high limit");
    if (!get_long(debug_dev, &high_lim))
    {
        rfprintf(debug_dev, "Bad value\r\n");
        return;
    }
    rfprintf(debug_dev, "\r\nDriving analog now\r\nAny key to quit\r\n");

    increment = 5000;
    output = 0;
    while (reof(debug_dev) > 0)
    {
        RinBlockAOutWrite(&rinblocks[0], output);

        output += increment;
        if (output > high_lim)
        {
            output = high_lim;
            increment = -increment;
        }
        if (output < low_lim)
        {
            output = low_lim;
            increment = -increment;
        }

        OSTimeDly(1);
    }
    rread(debug_dev, &ch, 1, 0);
}


static void aout_dio_set_output(unsigned char rb, union u_rb_output_write *rb_odata, struct s_rb_dio_read *inputs)
{
    unsigned char res;

//    P_PC.DR.BIT.B0 = 1;
    res = RinBlockWriteOutputs(&rinblocks[rb], rb_odata);
//    P_PC.DR.BIT.B0 = 0;
//    P_PC.DR.BIT.B0 = 1;
    if (res)
        RinBlockReadInputs(&rinblocks[rb], inputs);
//    P_PC.DR.BIT.B0 = 0;
}


static void aout_dio_triangle(struct t_io_dev *debug_dev)
{
    signed long increment, low_lim, high_lim;
    union u_rb_output_write rb_odata;
    unsigned char ch, holdoff, ro, rb, count = 0;
    struct s_rb_dio_read inputs;

//    gpio_set_ddr(&P_PC.DDR, 0x01, doOR);

    /*
     * Get rinblock
     */
    rfprintf(debug_dev, "\r\nRinblock AOUT dio and aout triangle test.\r\nEnter a rinblock to play with (0..%d): ", MAX_RINBLOCKS - 1);
    if ((!get_1_digit_num(debug_dev, &rb)) || (rb >= MAX_RINBLOCKS))
    {
        rfprintf(debug_dev, "\r\nYou twit! That's not a valid rinblock.\r\n");
        return;
    }

    /*
     * Get analog output lower limit
     */
    rfprintf(debug_dev, "\r\nEnter lower limit: ");
    if (!get_long(debug_dev, &low_lim))
    {
        rfprintf(debug_dev, "Bad value\r\n");
        return;
    }

    /*
     * Get analog output high limit
     */
    rfprintf(debug_dev, "\r\nEnter high limit: ");
    if (!get_long(debug_dev, &high_lim))
    {
        rfprintf(debug_dev, "Bad value\r\n");
        return;
    }
    rfprintf(debug_dev, "\r\nDriving analog and digital outputs now\r\n+- to change holdoff, Q to quit\r\n");

    /*
     * Default stuff
     */
    increment = 5000;
    holdoff = count = 0;
    rb_odata.rb_aout_output_write.aout_value = 0;

    /*
     * Loopy
     */
    while (1)
    {

        /*
         * Stick DIO info into struct, and write to rinblock
         */
        for (ro = 0; ro < MAX_AOUT_IO; ro++)
        {
            rb_odata.rb_aout_output_write.rb_output[ro].output_bits.level = ((count & (1 << ro)) != 0);
            rb_odata.rb_aout_output_write.rb_output[ro].output_bits.holdoff = holdoff;
        }
        aout_dio_set_output(rb, &rb_odata, &inputs);

        /*
         * New dio output value
         */
        count++;

        /*
         * Triangle wave on analog output
         */
        rb_odata.rb_aout_output_write.aout_value += increment;
        if (rb_odata.rb_aout_output_write.aout_value > high_lim)
        {
            rb_odata.rb_aout_output_write.aout_value = high_lim;
            increment = -increment;
        }
        if (rb_odata.rb_aout_output_write.aout_value < low_lim)
        {
            rb_odata.rb_aout_output_write.aout_value = low_lim;
            increment = -increment;
        }

        /*
         * Allow user to adjust holdoff, quit
         */
        if (reof(debug_dev) == 0)
        {
            rread(debug_dev, &ch, 1, 0);
            switch (ch)
            {
                case '+':
                    holdoff++;
                    if (holdoff > 127)
                        holdoff = 0;
                    rfprintf(debug_dev, "Holdoff = %d\r\n", holdoff);
                    break;
                case '-':
                    holdoff--;
                    if (holdoff > 127)
                        holdoff = 127;
                    rfprintf(debug_dev, "Holdoff = %d\r\n", holdoff);
                    break;
                case 'Q':
                    return;
            }
        }
        OSTimeDly(1);
    }
}


static void aout_dump_setup(struct t_io_dev *debug_dev, struct s_rb_aout_configure *aout_config)
{
    rfprintf(debug_dev, "Analog Out Config Dump\r\nType: %d, UZero %d, USpan %d, MaxLim %d, MinLim %d, UpdateRate %d\r\n",
             aout_config->ucAnalogOutputType, aout_config->iUserZero, aout_config->iUserSpan,
             aout_config->cMaxLimit, aout_config->cMinLimit, aout_config->uiUpdateRate);
}


static void aout_cal_user(struct t_io_dev *debug_dev)
{
    signed short inc;
    struct s_rb_aout_configure aout_config;
    unsigned char cal_span;

    rfprintf(debug_dev, "User Calibrate Analog Out\r\nQ - Quit\r\nU - Up Fast\r\nu - Up Slow\r\nD - Down Fast\r\nd - Down Slow\r\nS - Span Cal\r\nZ - Zero Cal\r\n");

    aout_config.ucAnalogOutputType = 1;
    aout_config.iUserZero = 0;
    aout_config.iUserSpan = 0;
    aout_config.cMaxLimit = 100;
    aout_config.cMinLimit = 0;
    aout_config.uiUpdateRate = 1000;  // 100hz
    RinBlockAOutWriteConfig(&rinblocks[0], &aout_config);

    cal_span = 1;
    RinBlockAOutWrite(&rinblocks[0], 50000L);

    while (1)
    {
        inc = 0;
        switch (get_char(debug_dev))
        {
            case 'Q':
                return;
            case 'U':
                inc = 100;
                break;
            case 'u':
                inc = 1;
                break;
            case 'D':
                inc = -100;
                break;
            case 'd':
                inc = -1;
                break;
            case 'S':
                cal_span = 1;
                RinBlockAOutWrite(&rinblocks[0], 50000L);
                rfprintf(debug_dev, "Now span cal\r\n");
                break;
            case 'Z':
                cal_span = 0;
                RinBlockAOutWrite(&rinblocks[0], 0L);
                rfprintf(debug_dev, "Now zero cal\r\n");
                break;
        }

        if (inc != 0)
        {
            if (!RinBlockAOutReadConfig(&rinblocks[0], &aout_config))
            {
                rfprintf(debug_dev, "Error reading analog configuration\r\n");
                continue;
            }
            aout_dump_setup(debug_dev, &aout_config);

            if (cal_span)
            {
                aout_config.iUserSpan += inc;
                rfprintf(debug_dev, "Span Cal Value: %d\r\n", aout_config.iUserSpan);
            }
            else
            {
                aout_config.iUserZero += inc;
                rfprintf(debug_dev, "Zero Cal Value: %d\r\n", aout_config.iUserZero);
            }

            if (!RinBlockAOutWriteConfig(&rinblocks[0], &aout_config))
            {
                rfprintf(debug_dev, "Error writing analog configuration\r\n");
                continue;
            }

        }
    }
}


/*
 * A config line looks like -
 *
 * !rb_num typ(I/V) zero span max min
 * !0 I 00000 00000 000 000
 */
static unsigned char aout_load_config_line(struct t_io_dev *debug_dev)
{
    unsigned char ch, rb_num, res;
    struct s_rb_aout_configure aout_config;
    unsigned long unum;
    signed long snum;

    /*
     * Look for a ! to start a line
     */
    do
    {
        ch = get_char(debug_dev);
        if (ch == '#')
            return 2;
    }
    while (ch != '!');

    /*
     * Read the rin block number
     */
    if (!get_1_digit_num(debug_dev, &rb_num))
        return 0;

    /*
     * Read a space
     */
    if (get_char(debug_dev) != ' ')
        return 0;

    /*
     * Read the aout type
     */
    ch = get_char(debug_dev);
    switch (ch)
    {
        case 'I':
            aout_config.ucAnalogOutputType = 0;
            break;
        case 'V':
            aout_config.ucAnalogOutputType = 1;
            break;
        default:
            return 0;
    }

    /*
     * Read a space
     */
    if (get_char(debug_dev) != ' ')
        return 0;

    /*
     * Read the user zero (-4000..03000)
     */
    if (!get_n_digit_s_num(debug_dev, &snum, 5))
    {
        return 0;
    }
    aout_config.iUserZero = (signed short) (snum);

    /*
     * Read a space
     */
    if (get_char(debug_dev) != ' ')
        return 0;

    /*
     * Read the user span (-4000..03000)
     */
    if (!get_n_digit_s_num(debug_dev, &snum, 5))
    {
        return 0;
    }
    aout_config.iUserSpan = (signed short) (snum);

    /*
     * Read a space
     */
    if (get_char(debug_dev) != ' ')
        return 0;

    /*
     * Read the min limit (-128..0127)
     */
    if (!get_n_digit_s_num(debug_dev, &snum, 4))
    {
        return 0;
    }
    aout_config.cMinLimit = (signed char) (snum);

    /*
     * Read a space
     */
    if (get_char(debug_dev) != ' ')
        return 0;

    /*
     * Read the max limit (-128..0127)
     */
    if (!get_n_digit_s_num(debug_dev, &snum, 4))
    {
        return 0;
    }
    aout_config.cMaxLimit = (signed char) (snum);

    /*
     * Read a space
     */
    if (get_char(debug_dev) != ' ')
        return 0;

    /*
     * Read the update rate in .1Hz (10000 = 100Hz, 1 = 10sec)
     */
    if (!get_n_digit_u_num(debug_dev, &unum, 5))
    {
        return 0;
    }
    aout_config.uiUpdateRate = (int) unum;

    /*
     * Configure the aout
     */
    res = RinBlockAOutWriteConfig(&rinblocks[rb_num], &aout_config);
    rfprintf(debug_dev, "Configure aout - rinblock %d as type %c, user zero = %d, user span = %d, min limit = %d, max limit = %d, update rate %d, with result %d\r\n",
             rb_num, ch, aout_config.iUserZero, aout_config.iUserSpan, aout_config.cMinLimit, aout_config.cMaxLimit, aout_config.uiUpdateRate, res);
    return res;
}


static void aout_load_config(struct t_io_dev *debug_dev)
{
    unsigned char res;

    rfprintf(debug_dev, "Load aout config. Send as:\r\n");
    rfprintf(debug_dev, "!r t zzzzz sssss mmmm MMMM fffff\r\n");
    rfprintf(debug_dev, "where ! is start, r is the rinblock(0..3), and t is the aout type (IV), z is the user zero (-4000..03000), s is the user span (-4000.03000), m f is the update rate\r\n");
    rfprintf(debug_dev, "# on the last line to end\r\n");

    do
    {
        res = aout_load_config_line(debug_dev);
        if (res == 0)
            rfprintf(debug_dev, "Load config failed while parsing input\r\n");
    }
    while (res != 2);
}


/*
 * aout_test -
 *
 */
void aout_test(struct t_io_dev *debug_dev)
{
    unsigned char ch = 0;

    while (ch != 'Q')
    {
        rfprintf(debug_dev, "Rin Block AOut Test\r\n");
        rfprintf(debug_dev, "Commands:\r\nS - Set AOut To Value\r\nL - Load AOut config\r\nT - Triangle wave\r\nC - User Cal\r\nB - Aout/Dio Triangle\r\nD - Load DIO Config\r\n");
        rfprintf(debug_dev, "F - Print Factory Cal\r\nf - Write Factory Cal\r\nQ - quit\r\n");
        rread(debug_dev, &ch, 1, 0);
        switch (ch)
        {
            case 'S':
                aout_set_output(debug_dev);
                break;
            case 'L':
                aout_load_config(debug_dev);
                break;
            case 'T':
                aout_triangle(debug_dev);
                break;
            case 'C':
                aout_cal_user(debug_dev);
                break;
            case 'B':
                aout_dio_triangle(debug_dev);
                break;
            case 'D':
                dio_load_config(debug_dev);
                break;
            case 'F':
                ReadFactoryCal(debug_dev);
                break;
            case 'f':
                WriteFactoryCal(debug_dev);
                break;
        }
    }
}
