#include <rinhw/rio.h>
#include <rinhw/rstdio.h>
#include <rinhw/sio.h>
#include <rinblock/rinblock.h>
#include <rinblock/rb_scratch.h>
#include <rinblock/rb_debug.h>

#include "rb_database_test.h"


unsigned char temp_scratchpad[RB_SCRATCHPAD_SIZE];


#if 0


#pragma section

/*
 * Declare NV database
 */
struct t_rb_database rb_database[MAX_RINBLOCKS];


#pragma section


unsigned short crc16_calc(const unsigned char *byt, unsigned short size, unsigned short crc)
{

    /*
     * Calculate CRC-16 value; uses The CCITT-16 Polynomial,
     * expressed as X^16 + X^12 + X^5 + 1
     */
    unsigned short index;
    unsigned char b;

    for (index = 0; index < size; ++index)
    {
        crc ^= (((unsigned short) byt[index]) << 8);
        for (b = 0; b < 8; ++b)
        {
            if (crc & (unsigned short) 0x8000)
                crc = (crc << 1) ^ (unsigned short) 0x1021;
            else
                crc = (crc << 1);
        }
    }
    return crc;
}


static unsigned char RinBlockSaveDatabaseToRinblock(struct t_rinblock *rb)
{
    unsigned char res;

    /*
     *
     */
    res = RinBlockWriteScratchPad(rb, (unsigned char *) rb->database.settings, 0, sizeof(union t_rb_settings));
#if RB_DEBUG(RB_DATABASE_DEBUG_LVL, RB_DEBUG_BASIC)
    rfprintf(RB_DEBUG_DEV, "Write database to scratchpad %s.\n" ((res) ? "success" : "failed"));
#endif
    return res;
}


static unsigned char RinBlockLoadDatabaseFromRinblock(struct t_rinblock *rb)
{
    unsigned char res;

    /*
     *
     */
    res = RinBlockReadScratchPad(rb, (unsigned char *) rb->database.settings, 0, sizeof(union t_rb_settings));
#if RB_DEBUG(RB_DATABASE_DEBUG_LVL, RB_DEBUG_BASIC)
    rfprintf(RB_DEBUG_DEV, "Read database from scratchpad %s.\n" ((res) ? "success" : "failed"));
#endif
    return res;
}


/*
 * RinBlockCommitChanges -
 *
 * Save the setting changes to the rinblock, calculating checksums
 */
unsigned char RinBlockCommitChanges(struct t_rinblock *rb, unsigned char force)
{
    unsigned short crc;

    crc = crc16_calc((unsigned char *) rb->database.settings, sizeof(union t_rb_settings), 0xFFFF);
    if ((crc != rb->database.settings.crc) || (force))
    {
        rb->database.settings.crc = crc;
        RinBlockSaveDatabaseToRinblock(rb);


    }
}


void RinBlockDatabaseCheck(struct t_rinblock *rb)
{

    /*
     * First check if the local tag matches the rin block tag
     */
    TagCompare()
    {

        /*
         * Yes, nothing has changed
         */
    }
    else
    {

        /*
         * No, don't match
         */

    }
}


void RinBlockAlreadyKnown(struct t_rinblock *rb)
{

	/*
     * See if anything has changed in the tag
     */




}


void RinBlockNotKnown(struct t_rinblock *rb)
{

    /*
     * Copy the tag from the rinblock to the local database
     */



}


signed char RinBlockFindMyLocalDatabase(struct t_rinblock *rb)
{

	for (i = 0; i < MAX_RINBLOCKS; i++)
	{

    	/*
     	 * First check if this local tag serial number matches the rin block serial number
     	 */
    	if ((rb_database[i].tag.model_number == rb->tag.model_number) &&
            (rb_database[i].tag.serial_number == rb->tag.serial_number) &&
            (rb_database[i].tag.interface_version == rb->tag.interface_version))
    	{

            /*
             * Yes, found our local tag/database
             */
            return i;
        }
    }
    return -1;
}


void RinBlockDatabaseCheck(struct t_rinblock *rb)
{
    signed char db_num;

    /*
     * Find out if we know about this rinblock
     */
    db_num = RinBlockFindMyLocalDatabase(rb);

    if (db_num >= 0)
    {

        /*
         * Yes
         */
        RinBlockAlreadyKnown(rb, );


    }
    else
    {

        /*
         * No
         */

    }
}


static unsigned char RinBlockCheckDatabase(struct t_rb_database *db)
{
    unsigned short crc;
    crc = crc16_calc((unsigned char *) db, sizeof(struct t_rb_database) - sizeof(db->checksum), 0xFFFF);
    return (db->checksum == crc);
}


static void RinBlockCheckLocalDatabases(void)
{
    for (i = 0; i < MAX_RINBLOCKS; i++)
    {
        RinBlockCheckDatabase(

    }
}


void RinBlockDatabaseStartup(void)
{
    struct rb_db_status stats[MAX_RINBLOCKS];

    RinBlockCheckLocalDatabases();

    RinBlockMatchTag(struct t_rb_database *dbs, struct rb_db_status *stats)

    for (i = 0; i < MAX_RINBLOCKS; i++)
    {
        switch (stats[i].rbStatus)
        {
            case rbsUnassigned:


            case rbsNew:

            case rbsUnchanged:

            case rbsMoved:

            case rbsMissing:


        }
    }


}
#endif



/*
 Flags
 */


static void scratchpad_fill(struct t_io_dev *debug_dev, struct t_rinblock *rb)
{
    unsigned char ch, i;

    rfprintf(debug_dev, "Fill Scratchpad. Type a character to fill with: ");
    rread(debug_dev, &ch, 1, 0);
    rfprintf(debug_dev, "\nFilling with %c. ", ch);

    for (i = 0; i < RB_SCRATCHPAD_SIZE; i++)
    {
        if (!RinBlockWriteScratchPad(rb, &ch, i, 1))
        {
            rfprintf(debug_dev, "write address %d failed.\n", i);
            return;
        }
    }
    rfprintf(debug_dev, "done.\n");
}


#if 0
static void scratchpad_test(struct t_io_dev *debug_dev, struct t_rinblock *rb)
{
    rfprintf(debug_dev, "Fill Scratchpad. Type a character to fill with: ");
    rread(debug_dev, &ch, 1, 0);
    rfprintf(debug_dev, "\nFilling with %c. ", ch);

    for (i = 0; i <


    RB_SCRATCHPAD_SIZE

}
#endif


static void scratchpad_dump(struct t_io_dev *debug_dev, struct t_rinblock *rb)
{
    unsigned char i;

    rfprintf(debug_dev, "Dump Scratchpad.\n");
    if (!RinBlockReadScratchPad(rb, &temp_scratchpad[0], 0, RB_SCRATCHPAD_SIZE))
    {
        rfprintf(debug_dev, "read failed.\n");
        return;
    }

    for (i = 0; i < RB_SCRATCHPAD_SIZE; i++)
    {
        if ((i & 0x07) == 0)
        {
            rfprintf(debug_dev, "\n%02x: ", i);
        }
        rfprintf(debug_dev, "%02x ", temp_scratchpad[i]);
    }
    rfprintf(debug_dev, "\n");
}


static void scratchpad_type(struct t_io_dev *debug_dev, struct t_rinblock *rb)
{
    unsigned char ch, i;
    rfprintf(debug_dev, "Type to Scratchpad.\n Type characters, ESC when done\n");

    i = 0;
    while (ch != 0x1B)
    {
        rread(debug_dev, &ch, 1, 0);
        rfprintf(debug_dev, "Writing %c to address %d ", ch, i);
        if (!RinBlockWriteScratchPad(rb, &ch, i, 1))
        {
            rfprintf(debug_dev, "failed.\n", i);
            return;
        }
        rfprintf(debug_dev, "\n");
        i++;
        if (i >= RB_SCRATCHPAD_SIZE)
            i = 0;
    }
}


void scratchpad_test(struct t_io_dev *debug_dev, struct t_rinblock *rb)
{
    unsigned char ch = 0;

    while (ch != 'Q')
    {
        rfprintf(debug_dev, "Rin Block Scratchpad Test\n");
        rfprintf(debug_dev, "Commands:\nF - fill\nD - dump\nT - type\nQ - quit\n");
        rread(debug_dev, &ch, 1, 0);
        switch (ch)
        {
            case 'F':
                scratchpad_fill(debug_dev, rb);
                break;
            case 'D':
                scratchpad_dump(debug_dev, rb);
                break;
            case 'T':
                scratchpad_type(debug_dev, rb);
                break;
        }
    }
}

