#include <string.h>
#include <rinhw/rio.h>
#include <rinhw/rstdio.h>
#include <rinhw/sio.h>
#include <rinblock/rinblock.h>
#include <rinblock/rb_scratch.h>

#include "rb_scratch_test.h"


unsigned char temp_scratchpad[RB_SCRATCHPAD_SIZE];


static void scratchpad_fill(struct t_io_dev *debug_dev, struct t_rinblock *rb)
{
    unsigned char ch, i;

    rfprintf(debug_dev, "Fill Scratchpad. Type a character to fill with: ");
    rread(debug_dev, &ch, 1, 0);
    rfprintf(debug_dev, "\r\nFilling with %c. ", ch);

    for (i = 0; i < RB_SCRATCHPAD_SIZE; i++)
    {
        if (!RinBlockWriteScratchPad(rb, &ch, i, 1))
        {
            rfprintf(debug_dev, "write address %d failed.\r\n", i);
            return;
        }
    }
    rfprintf(debug_dev, "done.\r\n");
}


static void scratchpad_dump(struct t_io_dev *debug_dev, struct t_rinblock *rb)
{
    unsigned char i;

    rfprintf(debug_dev, "Dump Scratchpad.\r\n");
    if (!RinBlockReadScratchPad(rb, &temp_scratchpad[0], 0, RB_SCRATCHPAD_SIZE))
    {
        rfprintf(debug_dev, "read failed.\r\n");
        return;
    }

    for (i = 0; i < RB_SCRATCHPAD_SIZE; i++)
    {
        if ((i & 0x07) == 0)
        {
            rfprintf(debug_dev, "\r\n%02x: ", i);
        }
        rfprintf(debug_dev, "%02x ", temp_scratchpad[i]);
    }
    rfprintf(debug_dev, "\r\n");
}


static void scratchpad_type(struct t_io_dev *debug_dev, struct t_rinblock *rb)
{
    unsigned char ch, i;
    rfprintf(debug_dev, "Type to Scratchpad.\r\n Type characters, ESC when done\n");

    i = 0;
    while (ch != 0x1B)
    {
        rread(debug_dev, &ch, 1, 0);
        rfprintf(debug_dev, "Writing %c to address %d ", ch, i);
        if (!RinBlockWriteScratchPad(rb, &ch, i, 1))
        {
            rfprintf(debug_dev, "failed.\r\n", i);
            return;
        }
        rfprintf(debug_dev, "\r\n");
        i++;
        if (i >= RB_SCRATCHPAD_SIZE)
            i = 0;
    }
}

static const char sp_msg[] = 
"With your feet in the air and your head on the ground\r\nTry this trick and spin it, yeah\r\n";


static void scratchpad_blockwrite(struct t_io_dev *debug_dev, struct t_rinblock *rb)
{
    unsigned char len = (unsigned char) strlen(sp_msg);
    unsigned char res;

    rfprintf(debug_dev, "Block Write To Scratchpad\r\nWriting data (length %d) to address 0 in scratchpad ", len);
    res = RinBlockWriteScratchPad(rb, (unsigned char *) sp_msg, 0, (unsigned) len);
    rfprintf(debug_dev, (res) ? "success.\r\n" : "failed.\r\n");
}


void scratchpad_test(struct t_io_dev *debug_dev, struct t_rinblock *rb)
{
    unsigned char ch = 0;

    while (ch != 'Q')
    {
        rfprintf(debug_dev, "Rin Block Scratchpad Test\r\n");
        rfprintf(debug_dev, "Commands:\r\nF - fill\r\nD - dump\r\nT - type\r\nB - Block Write\r\nQ - quit\r\n");
        rread(debug_dev, &ch, 1, 0);
        switch (ch)
        {
            case 'F':
                scratchpad_fill(debug_dev, rb);
                break;
            case 'D':
                scratchpad_dump(debug_dev, rb);
                break;
            case 'T':
                scratchpad_type(debug_dev, rb);
                break;
            case 'B':
                scratchpad_blockwrite(debug_dev, rb);
                break;
        }
    }
}

