#include <string.h>
#include <ctype.h>
#include <stdint.h>
#include <rinhw/rio.h>
#include <rinhw/sio.h>
#include <rinhw/rstdio.h>
#include "rb_getnum.h"

#include "rb_serial_test.h"

#if 0
void SerialEcho(struct t_io_dev *dev)
{
    unsigned char ch = 0x00;
    int res;

    ropen(dev, 0);
    while (ch != 0x1B)
    {
        res = rread(dev, &ch, 1, 0);
        rfprintf(&TEST_DEV, "Read char %c, result %d\r\n", ch, res);
        res = rwrite(dev, &ch, 1, 0);
        rfprintf(&TEST_DEV, " result was %d\r\n", res);
    }
    rclose(dev);
}
#endif


static void serial_dump_config(struct t_io_dev *debug_dev, struct t_io_dev *rb_sio_dev)
{
    t_ioctl_siocfg siocfg;
    unsigned short rx_size, rx_lvl, tx_size, tx_lvl;
    int res, tx_size_res, rx_size_res, rx_lvl_res, tx_lvl_res;

    res = rioctl(rb_sio_dev, IOCTL_SER_GET_CONFIG, &siocfg);
    rfprintf(debug_dev, "IOCTL_SER_GET_CONFIG returned with result %d\r\n\r\n", res);
    if (res >= 0)
    {
        rfprintf(debug_dev, "baud %ld, parity bits %d, data bits %d, stop bits %d, use_cts %d, terminations %d\r\n",
                 siocfg.baud, siocfg.parity_bits, siocfg.data_bits, siocfg.stop_bits, siocfg.use_cts, siocfg.use_terminations);
    }

    tx_size_res = rioctl(rb_sio_dev, IOCTL_SER_GET_TX_BUF_SIZE, &tx_size);
    tx_lvl_res = rioctl(rb_sio_dev, IOCTL_SER_GET_TX_BUF_LEVEL, &tx_lvl);
    rx_size_res = rioctl(rb_sio_dev, IOCTL_SER_GET_RX_BUF_SIZE, &rx_size);
    rx_lvl_res = rioctl(rb_sio_dev, IOCTL_SER_GET_RX_BUF_LEVEL, &rx_lvl);
    rfprintf(debug_dev, "Serial buffer parameters (result, level):\r\n");
    rfprintf(debug_dev, "tx size: %d, %d, tx level: %d, %d, rx size: %d, %d, rx level: %d, %d\r\n",
             tx_size_res, tx_size, tx_lvl_res, tx_lvl, rx_size_res, rx_size, rx_lvl_res, rx_lvl);
}


static unsigned char get_char(struct t_io_dev *debug_dev)
{
    unsigned char ch;

    rread(debug_dev, &ch, 1, 0);
    return ch;
}


#if 0
static unsigned char get_1_digit_num(struct t_io_dev *debug_dev, unsigned char *num)
{
    unsigned char ch;

    ch = get_char(debug_dev);
    if ((ch < '0') || (ch > '9'))
        return 0;
    *num = (unsigned char) (ch - '0');
    return 1;
}
#endif


static void serial_set_config(struct t_io_dev *debug_dev, struct t_io_dev *rb_sio_dev)
{
    t_ioctl_siocfg siocfg;
    int res;

    /*
     * Get Baud Rate
     */
    rfprintf(debug_dev, "Set Serial Config\r\nEnter Baud Rate: ");
    if (!get_long(debug_dev, (signed long *) &siocfg.baud))
    {
        rfprintf(debug_dev, " Bad Baud\r\n");
        return;
    }

    /*
     * Get Parity Bits
     */
    rfprintf(debug_dev, "\r\nEnter parity bits (noe): ");
    switch (get_char(debug_dev))
    {
        case 'n':
        case 'N':
            siocfg.parity_bits = SIO_PARITY_NONE;
            break;

        case 'o':
        case 'O':
            siocfg.parity_bits = SIO_PARITY_ODD;
            break;

        case 'e':
        case 'E':
            siocfg.parity_bits = SIO_PARITY_EVEN;
            break;

        default:
            rfprintf(debug_dev, " Bad Parity\r\n");
            return;
    }

    /*
     * Get Data Bits
     */
    rfprintf(debug_dev, "\r\nEnter data bits (78): ");
    switch (get_char(debug_dev))
    {
        case '7':
            siocfg.data_bits = 7;
            break;

        case '8':
            siocfg.data_bits = 8;
            break;

        default:
            rfprintf(debug_dev, " Bad Data Bits\r\n");
            return;
    }

    /*
     * Get Stop Bits
     */
    rfprintf(debug_dev, "\r\nEnter stop bits (12): ");
    switch (get_char(debug_dev))
    {
        case '1':
            siocfg.stop_bits = 1;
            break;

        case '2':
            siocfg.stop_bits = 2;
            break;

        default:
            rfprintf(debug_dev, " Bad Stop Bits\r\n");
            return;
    }

    /*
     * Get Use Cts
     */
    rfprintf(debug_dev, "\r\nEnter Use CTS (01): ");
    switch (get_char(debug_dev))
    {
        case '0':
            siocfg.use_cts = 0;
            break;

        case '1':
            siocfg.use_cts = 1;
            break;

        default:
            rfprintf(debug_dev, " Bad Use CTS\r\n");
            return;
    }

    /*
     * Get Use Terminations
     */
    rfprintf(debug_dev, "\r\nEnter Use Terminations (01): ");
    switch (get_char(debug_dev))
    {
        case '0':
            siocfg.use_terminations = 0;
            break;

        case '1':
            siocfg.use_terminations = 1;
            break;

        default:
            rfprintf(debug_dev, " Bad Use Terminations\r\n");
            return;
    }

    res = rioctl(rb_sio_dev, IOCTL_SER_SET_CONFIG, &siocfg);
    rfprintf(debug_dev, "IOCTL_SER_SET_CONFIG returned with result %d\r\n\r\n", res);
}


static void serial_echo(struct t_io_dev *debug_dev, struct t_io_dev *rb_sio_dev)
{
    unsigned char buf[10];
    int res;

    rfprintf(debug_dev, "Serial Echo Test. Any key to quit.\r\n");
    while (rfilelength(debug_dev) == 0)
    {
        res = rread(rb_sio_dev, buf, 4, 0);
        rfprintf(debug_dev, "Read of 4 bytes from serial device returned %d\r\n", res);

        res = rwrite(rb_sio_dev, buf, 4, 0);
        rfprintf(debug_dev, "Write of 4 bytes to serial device returned %d\r\n", res);
//        rwrite(debug_dev, buf, 4, 0);
//        rfprintf(debug_dev, "\r\n");
    }
    rread(debug_dev, buf, 1, 0);
}


/** Dump any data received by rb_sio_dev */
static void serial_dump(struct t_io_dev *debug_dev, struct t_io_dev *rb_sio_dev)
{
  long len;
  unsigned char data;
  int count;

  rfprintf(debug_dev, "Serial Dump Test. Any key to quit.\r\n");
  count = 0;
  while ( 0 == rfilelength( debug_dev ) )
  {
    len = rfilelength( rb_sio_dev );
    if ( 0 < len )
    {
      if ( 0 == count % 16 )
      {
        rfprintf(debug_dev, "\r\n%04X: ", count);
      }
      (void) rread( rb_sio_dev, &data, 1, 0 );
      rfprintf( debug_dev, "%02x ", data );
      count++;
    }
    else
    {
      OSTimeDly( OS_TICKS_PER_SEC / 10 );
    }
  }
  rread( debug_dev, &data, 1, 0 );
}


static const char test_str[] = "This is a test string\r\n";


static void serial_printf(struct t_io_dev *rb_sio_dev)
{
    rfprintf(rb_sio_dev, test_str);
}


static void serial_write(struct t_io_dev *rb_sio_dev)
{
    rwrite(rb_sio_dev, test_str, (unsigned short) strlen(test_str), 0);
}


static unsigned short get_tx_free(struct t_io_dev *debug_dev, struct t_io_dev *rb_sio_dev)
{
    unsigned short buf_sz, buf_lvl;

    rioctl(rb_sio_dev, IOCTL_SER_GET_TX_BUF_LEVEL, &buf_lvl);
    rioctl(rb_sio_dev, IOCTL_SER_GET_TX_BUF_SIZE, &buf_sz);

//    rfprintf(debug_dev, "buf_sz: %d, buf_lvl: %d\r\n", buf_sz, buf_lvl);

    return (unsigned short) (buf_sz - buf_lvl);
}



static const char port1_str[] = "TESTING SPLIT SERIAL 1\r\n";
static const char port2_str[] = "testing_split_serial_2\r\n";


static void serial_split_test(struct t_io_dev *debug_dev, struct t_io_dev *rb_sio_dev1, struct t_io_dev *rb_sio_dev2)
{
    unsigned char ch;

    rfprintf(debug_dev, "Split Serial Test. Any key to quit.\r\n");
    while (rfilelength(debug_dev) == 0)
    {
        if (get_tx_free(debug_dev, rb_sio_dev1) >= strlen(port1_str))
        {
            rwrite(rb_sio_dev1, port1_str, (unsigned short) strlen(port1_str), 0);
        }
        if (get_tx_free(debug_dev, rb_sio_dev2) >= strlen(port2_str))
        {
            rwrite(rb_sio_dev2, port2_str, (unsigned short) strlen(port2_str), 0);
        }
//        OSTimeDly(1);
    }
    rread(debug_dev, &ch, 1, 0);
}



static void serial_typing(struct t_io_dev *debug_dev, struct t_io_dev *rb_sio_dev)
{
    unsigned char ch;

    rfprintf(debug_dev, "Serial Typing Test, ESC to quit\r\n");
    while (1)
    {
        rread(debug_dev, &ch, 1, 0);
        switch (ch)
        {
            case 0x1B:
                return;
            default:
                rfprintf(debug_dev, "Writing char %c (0x%02x)\r\n", ch, ch);
                rwrite(rb_sio_dev, &ch, 1, 0);
                break;
        }
    }
}


static void dump_buffer(struct t_io_dev *debug_dev, const unsigned char *buffer, size_t length)
{
  size_t i;
  size_t j;
  size_t left;
  #define DUMP_WIDTH 16

  for (i = 0; i < length; i += left)
  {
    rfprintf(debug_dev, "%04X: ", (unsigned) i);

    /* Print in hex */
    left = (length - i < DUMP_WIDTH) ? length - i : DUMP_WIDTH;
    for (j = 0; j < left; j++)
    {
      rfprintf(debug_dev, "%02X ", buffer[i + j]);
    }
    for (; j < DUMP_WIDTH; j++)
    {
      rfprintf(debug_dev, "   ");
    }

    /* Print characters */
    rfprintf(debug_dev, "  ");
    for (j = 0; j < left; j++)
    {
      rfprintf(debug_dev, "%c", isprint(buffer[i + j]) ? buffer[i + j] : '.');
    }
    for (; j < DUMP_WIDTH; j++)
    {
      rfprintf(debug_dev, " ");
    }
    rfprintf(debug_dev, "\n");
  }
}


static void serial_block_write(struct t_io_dev *debug_dev, struct t_io_dev *rb_sio_dev)
{
  #define BLOCK_WRITE_MAX_LEN 100
  unsigned char buffer[BLOCK_WRITE_MAX_LEN];
  long length;
  long timeout;
  int write_res;

  rfprintf(debug_dev, "Serial block write\nEnter block length (0..%d): ", BLOCK_WRITE_MAX_LEN);
  if (!get_long(debug_dev, &length) || length < 0 || length > BLOCK_WRITE_MAX_LEN)
  {
      rfprintf(debug_dev, " Bad length\n");
      return;
  }
  rfprintf(debug_dev, "\nEnter timeout (ticks 0..65535, 0 no timeout): ");
  if (!get_long(debug_dev, &timeout) || timeout < 0 || timeout > UINT16_MAX)
  {
      rfprintf(debug_dev, " Bad timeout\n");
      return;
  }
  rfprintf(debug_dev, "\nEnter %lu characters to block write: ", length);
  if (rread(debug_dev, buffer, length, 0) != length)
  {
    rfprintf(debug_dev, "\nError reading characters\n");
    return;
  }

  rfprintf(debug_dev, "\nWaiting %lu ticks to send %lu bytes\n", timeout, length);
  write_res = rwrite(rb_sio_dev, buffer, length, (unsigned short) timeout);
  if (write_res == length)
  {
    rfprintf(debug_dev, "rwrite() of %lu bytes with timeout %lu returned success\n", length, timeout);
  }
  else
  {
    rfprintf(debug_dev, "rwrite() of %lu bytes with timeout %lu returned with error %d\n", length, timeout, write_res);
  }
}


static void serial_block_read(struct t_io_dev *debug_dev, struct t_io_dev *rb_sio_dev)
{
  #define BLOCK_READ_MAX_LEN 100
  unsigned char buffer[BLOCK_READ_MAX_LEN];
  long length;
  long timeout;
  int read_res;

  rfprintf(debug_dev, "Serial block read\nEnter block length (0..%d): ", BLOCK_READ_MAX_LEN);
  if (!get_long(debug_dev, &length) || length < 0 || length > BLOCK_READ_MAX_LEN)
  {
      rfprintf(debug_dev, " Bad length\n");
      return;
  }
  rfprintf(debug_dev, "\nEnter timeout (ticks 0..65535, 0 no timeout): ");
  if (!get_long(debug_dev, &timeout) || timeout < 0 || timeout > UINT16_MAX)
  {
      rfprintf(debug_dev, " Bad timeout\n");
      return;
  }
  rfprintf(debug_dev, "\nWaiting %lu ticks for %lu bytes\n", timeout, length);

  read_res = rread(rb_sio_dev, buffer, length, (unsigned short) timeout);
  if (read_res == length)
  {
    rfprintf(debug_dev, "rread() of %lu bytes with timeout %lu returned success with data:\n", length, timeout);
    dump_buffer(debug_dev, buffer, length);
  }
  else
  {
    rfprintf(debug_dev, "rread() of %lu bytes with timeout %lu returned with error %d\n", length, timeout, read_res);
  }
}


void serial_test(struct t_io_dev *debug_dev, struct t_io_dev *rb_sio_dev1, struct t_io_dev *rb_sio_dev2)
{
    unsigned char ch = 0;

    ropen(rb_sio_dev1, 0);
    ropen(rb_sio_dev2, 0);

    while (ch != 'Q')
    {
        rfprintf(debug_dev, "Rin Block Serial Test\n"
                            "Commands:\n"
                            "  P - printf port 1\n"
                            "  W - rwrite port 1\n"
                            "  p - printf port 2\n"
                            "  w - rwrite port 2\n"
                            "  D - dump config port 1\n"
                            "  C - configure port 1\n"
                            "  d - dump config port 2\n"
                            "  c - configure port 2\n"
                            "  S - split serial test\n"
                            "  E - serial echo\n"
                            "  T - Serial Typing port 1\n"
                            "  t - Serial Typing port 2\n"
                            "  I - Dump Incoming Data port 1\n"
                            "  B - block write serial 1\n"
                            "  b - block write serial 2\n"
                            "  R - block read serial 1\n"
                            "  Q - quit\n");
        rread(debug_dev, &ch, 1, 0);
        switch (ch)
        {
            case 'P':
                serial_printf(rb_sio_dev1);
                break;
            case 'W':
                serial_write(rb_sio_dev1);
                break;
            case 'p':
                serial_printf(rb_sio_dev2);
                break;
            case 'w':
                serial_write(rb_sio_dev2);
                break;
            case 'D':
                serial_dump_config(debug_dev, rb_sio_dev1);
                break;
            case 'C':
                serial_set_config(debug_dev, rb_sio_dev1);
                break;
            case 'd':
                serial_dump_config(debug_dev, rb_sio_dev2);
                break;
            case 'c':
                serial_set_config(debug_dev, rb_sio_dev2);
                break;
            case 'S':
                serial_split_test(debug_dev, rb_sio_dev1, rb_sio_dev2);
                break;
            case 'E':
                serial_echo(debug_dev, rb_sio_dev1);
                break;
            case 'T':
                serial_typing(debug_dev, rb_sio_dev1);
                break;
            case 't':
                serial_typing(debug_dev, rb_sio_dev2);
                break;
            case 'I':
                serial_dump(debug_dev, rb_sio_dev1);
                break;
            case 'B':
                serial_block_write(debug_dev, rb_sio_dev1);
                break;
            case 'b':
              serial_block_write(debug_dev, rb_sio_dev2);
              break;
            case 'R':
                serial_block_read(debug_dev, rb_sio_dev1);
                break;
        }
    }

    rclose(rb_sio_dev1);
    rclose(rb_sio_dev2);
}
