#include <rinhw/rio.h>
#include <rinhw/rstdio.h>
#include <rinblock/rb_status.h>

#include "rb_status_test.h"


static const char *const status_strs[] =
{
    "RB_STATUS_UNDEFINED",
    "RB_STATUS_UNDEFINED",
    "RB_STATUS_UNDEFINED",
    "RB_STATUS_UNDEFINED",
    "RB_STATUS_UNDEFINED",
    "RB_STATUS_UNDEFINED",
    "RB_STATUS_RESET",
    "RB_STATUS_FACTORY_ERROR",
    "RB_STATUS_BUS_ERROR",
    "RB_STATUS_MISC_ERROR",
    "RB_STATUS_VOLTAGE_ERROR",
    "RB_STATUS_RUNNING",
    "RB_STATUS_TEMPERATURE_ERROR",
    "RB_STATUS_RAM_ERROR",
    "RB_STATUS_EEPROM_ERROR",
    "RB_STATUS_ROM_ERROR"
};



static void print_short_status(struct t_io_dev *debug_dev, unsigned short status)
{
    unsigned short i, mask;

    rfprintf(debug_dev, "Rin Block Short Status: 0x%04x, 0b%016b, Bit Meanings:\n", status, status);
    for (i = 0, mask = 0x0001; i < 16; i++, mask <<= 1)
    {
        if ((status & mask) != 0)
        {
            rfprintf(debug_dev, "Bit %d, %s\n", i, status_strs[i]);
        }
    }
}


void status_test(struct t_io_dev *debug_dev, struct t_rinblock *rb)
{
    unsigned short status;
    unsigned char res;
/*    struct s_rb_status_long_read status_long; */

    rfprintf(debug_dev, "Rin Block Status Test\n");
    res = RinBlockReadStatusShort(rb, &status);
    if (res)
    {
        print_short_status(debug_dev, status);
    }
    else
    {
        rfprintf(debug_dev, "Read Short Status Failed\n");
    }

#if 0 /* Removed, because this was not used anywhere */
    res = RinBlockReadStatusLong(rb, &status_long);
    if (res)
    {
        rfprintf(debug_dev, "Long status: 0x%08lx\n", status_long.long_status);
        print_short_status(debug_dev, status_long.short_status);
    }
    else
    {
        rfprintf(debug_dev, "Read Long Status Failed\n");
    }
#endif
}
