/*
 * rb_tilt_test.c
 *
 *  Created on: Sep 26, 2012
 *      Author: ian
 */
#include <rinhw/rstdio.h>
#include <rinblock/rb_tilt.h>
#include "rb_getnum.h"

#include "rb_tilt_test.h"


/** Perform a tilt zero calibration */
static void tilt_zero_calibration(struct t_io_dev *debug_dev, struct t_rinblock *rb)
{
  unsigned char res;

  res = RinBlockTiltZero(rb);
  rfprintf(debug_dev, "RinBlockTiltZero() %s\n", (0 == res) ? "failed" : "success");
}


/** Perform a tilt zero factory calibration */
static void tilt_zero_factory_calibration(struct t_io_dev *debug_dev, struct t_rinblock *rb)
{
  unsigned char res;

  res = RinBlockTiltFactoryZero(rb);
  rfprintf(debug_dev, "RinBlockTiltFactoryZero() %s\n", (0 == res) ? "failed" : "success");
}


/** Print td data */
#define PRINT_TILT_DATA(dev, td, le) \
  rfprintf(dev, "  xAngle: %6d" le \
                "  yAngle: %6d" le \
                "  cosX: %6u" le \
                "  cosY: %6u" le \
                "  comp: %6u" le \
                "  errFlag: 0x%02X (%s%s)" le, \
                (td).xAngle, \
                (td).yAngle, \
                (td).cosX, \
                (td).cosY, \
                (td).comp, \
                (td).errFlag, \
                (0 != ((td).errFlag & RB_TILT_ERR_TILT_SENSOR)) ? "RB_TILT_ERR_TILT_SENSOR " : "", \
                (0 != ((td).errFlag & RB_TILT_ERR_MAX_OVER)) ? "RB_TILT_ERR_MAX_OVER" : "" \
          )


#define PRINT_IMPROVED_FLAG(dev, val, mask) \
  if (0 != (val & mask)) rfprintf(dev, #mask " ")

#define PRINT_IMPROVED_FLAGS(dev, val) \
  if (RB_TILT_CAL_PASS == val) rfprintf(dev, "RB_TILT_CAL_PASS "); \
  PRINT_IMPROVED_FLAG(dev, val, RB_TILT_CAL_FAIL_AB_HIGH); \
  PRINT_IMPROVED_FLAG(dev, val, RB_TILT_CAL_FAIL_AB_LOW); \
  PRINT_IMPROVED_FLAG(dev, val, RB_TILT_CAL_FAIL_CD_HIGH); \
  PRINT_IMPROVED_FLAG(dev, val, RB_TILT_CAL_FAIL_CD_LOW); \
  PRINT_IMPROVED_FLAG(dev, val, RB_TILT_CAL_FAIL_NO_CONVERGE)


/** Print td improved data */
#define PRINT_TILT_DATA_IMPROVED(dev, td, le) \
  PRINT_TILT_DATA(dev, td, le); \
  rfprintf(dev, "  errFlagCalX: 0x%02X (", (td).errFlagCalX); \
  PRINT_IMPROVED_FLAGS(dev, (td).errFlagCalX); \
  rfprintf(dev, ")" le "  errFlagCalY: 0x%02X (", (td).errFlagCalY); \
  PRINT_IMPROVED_FLAGS(dev, (td).errFlagCalY); \
  rfprintf(dev, ")" le)


#define PRINT_TILT_DATA_BASIC_SINGLE_LINE(dev, td) PRINT_TILT_DATA(dev, td, "")
#define PRINT_TILT_DATA_IMPROVED_SINGLE_LINE(dev, td) PRINT_TILT_DATA_IMPROVED(dev, td, "")

#define PRINT_TILT_DATA_BASIC_MULTIPLE_LINE(dev, td) PRINT_TILT_DATA(dev, td, "\n")
#define PRINT_TILT_DATA_IMPROVED_MULTIPLE_LINE(dev, td) PRINT_TILT_DATA_IMPROVED(dev, td, "\n")


/** Read basic tilt data */
static void tilt_read_tilt_basic(struct t_io_dev *debug_dev, struct t_rinblock *rb)
{
  struct s_rb_tilt_data tilt_data;
  unsigned char res;

  res = RinBlockTiltReadTilt(rb, &tilt_data);
  if (0 == res)
  {
    rfprintf(debug_dev, "RinBlockTiltReadTilt() failed\n");
  }
  else
  {
    rfprintf(debug_dev, "RinBlockTiltReadTilt() returned:\n");
    PRINT_TILT_DATA_BASIC_MULTIPLE_LINE(debug_dev, tilt_data);
  }
}


/** Read basic tilt data repetitively */
static void tilt_read_tilt_basic_repeat(struct t_io_dev *debug_dev, struct t_rinblock *rb)
{
  struct s_rb_tilt_data tilt_data;
  unsigned char res;
  char ch;

  rfprintf(debug_dev, "Printing basic tilt data, any key to stop\n");

  for (res = 1; 0 != res && reof(debug_dev); )
  {
    res = RinBlockTiltReadTilt(rb, &tilt_data);
    if (0 == res)
    {
      rfprintf(debug_dev, "RinBlockTiltReadTilt() failed\n");
    }
    else
    {
      PRINT_TILT_DATA_BASIC_SINGLE_LINE(debug_dev, tilt_data);
      rfprintf(debug_dev, "\r");
    }
  }

  rread(debug_dev, &ch, 1, 0);
  rfprintf(debug_dev, "\n");
}


/** Read improved tilt data */
static void tilt_read_tilt_improved(struct t_io_dev *debug_dev, struct t_rinblock *rb)
{
  struct s_rb_tilt_data_improved tilt_data_improved;
  unsigned char res;

  res = RinBlockTiltReadTiltImproved(rb, &tilt_data_improved);
  if (0 == res)
  {
    rfprintf(debug_dev, "RinBlockTiltReadTiltImproved() failed\n");
  }
  else
  {
    rfprintf(debug_dev, "RinBlockTiltReadTiltImproved() returned:\n");
    PRINT_TILT_DATA_IMPROVED_MULTIPLE_LINE(debug_dev, tilt_data_improved);
  }
}


/** Read tilt data repetitively */
static void tilt_read_tilt_improved_repeat(struct t_io_dev *debug_dev, struct t_rinblock *rb)
{
  struct s_rb_tilt_data_improved tilt_data_improved;
  unsigned char res;
  char ch;

  rfprintf(debug_dev, "Printing improved tilt data, any key to stop\n");

  for (res = 1; 0 != res && reof(debug_dev); )
  {
    res = RinBlockTiltReadTiltImproved(rb, &tilt_data_improved);
    if (0 == res)
    {
      rfprintf(debug_dev, "RinBlockTiltReadTiltImproved() failed\n");
    }
    else
    {
      PRINT_TILT_DATA_IMPROVED_SINGLE_LINE(debug_dev, tilt_data_improved);
      rfprintf(debug_dev, "\r");
    }
  }

  rread(debug_dev, &ch, 1, 0);
  rfprintf(debug_dev, "\n");
}


/** Read configuration from tilt module */
static void tilt_read_config(struct t_io_dev *debug_dev, struct t_rinblock *rb)
{
  struct s_rb_tilt_factors tilt_factors;
  unsigned char res;
  int8u mt;

  res = RinBlockTiltReadConfig(rb, &tilt_factors);
  if (0 == res)
  {
    rfprintf(debug_dev, "RinBlockTiltReadConfig() failed\n");
  }
  else
  {
    mt = tilt_factors.maxTilt;
    rfprintf(debug_dev, "RinBlockTiltReadConfig() returned:\n"
                        "aFactor: %u\n"
                        "bFactor: %u\n"
                        "cFactor: %d\n"
                        "dFactor: %d\n"
                        "maxTilt (result): %u (%s)\n",
                        tilt_factors.aFactor,
                        tilt_factors.bFactor,
                        tilt_factors.cFactor,
                        tilt_factors.dFactor,
                        mt,
                        (0 == mt) ? "FAIL" : (1 == mt) ? "IN PROGRESS" : (2 == mt) ? "OK" : "UNKNOWN");
  }
}


#define GET_CAL_VALUE(dev, s, memb, min, max, onfail) \
  rfprintf(dev, "Enter " #memb " (%ld..%ld): ", (long) min, (long) max); \
  get_long(dev, &num); \
  if (num < min || num > max) \
  { \
    rfprintf(dev, "\nInvalid value for " #memb "\n"); \
    goto onfail; \
  } \
  s.memb = num; \
  rfprintf(dev, "\n")


/** Write configuration to tilt module */
static void tilt_write_config(struct t_io_dev *debug_dev, struct t_rinblock *rb)
{
  struct s_rb_tilt_factors tilt_factors;
  unsigned char res;
  long num;

  rfprintf(debug_dev, "Configuring tilt module:\n");

  /* Get values */
  GET_CAL_VALUE(debug_dev, tilt_factors, aFactor, INT16U_MIN, INT16U_MAX, wtc_fail);
  GET_CAL_VALUE(debug_dev, tilt_factors, bFactor, INT16U_MIN, INT16U_MAX, wtc_fail);
  GET_CAL_VALUE(debug_dev, tilt_factors, cFactor, INT16S_MIN, INT16S_MAX, wtc_fail);
  GET_CAL_VALUE(debug_dev, tilt_factors, dFactor, INT16S_MIN, INT16S_MAX, wtc_fail);
  GET_CAL_VALUE(debug_dev, tilt_factors, maxTilt, INT8U_MIN,  INT8U_MAX,  wtc_fail);

  /* Now write it */
  res = RinBlockTiltWriteConfig(rb, &tilt_factors);
  rfprintf(debug_dev, "RinBlockTiltWriteConfig() %s\n", (res) ? "success" : "failed");
  return;

wtc_fail:
  rfprintf(debug_dev, "No configuration written\n");
}


/** Perform basic calibration on tilt module */
static void tilt_perform_calibration_basic(struct t_io_dev *debug_dev, struct t_rinblock *rb)
{
  struct s_rb_tilt_calibrate tilt_calibrate;
  unsigned char res;
  long num;

  rfprintf(debug_dev, "Basic calibration of tilt module:\n");

  /* Get values */
  GET_CAL_VALUE(debug_dev, tilt_calibrate, xPosAngle, INT16S_MIN, INT16S_MAX, tpcb_fail);
  GET_CAL_VALUE(debug_dev, tilt_calibrate, xNegAngle, INT16S_MIN, INT16S_MAX, tpcb_fail);
  GET_CAL_VALUE(debug_dev, tilt_calibrate, xPosComp,  INT16U_MIN, INT16U_MAX, tpcb_fail);
  GET_CAL_VALUE(debug_dev, tilt_calibrate, xNegComp,  INT16U_MIN, INT16U_MAX, tpcb_fail);
  GET_CAL_VALUE(debug_dev, tilt_calibrate, yPosAngle, INT16S_MIN, INT16S_MAX, tpcb_fail);
  GET_CAL_VALUE(debug_dev, tilt_calibrate, yNegAngle, INT16S_MIN, INT16S_MAX, tpcb_fail);
  GET_CAL_VALUE(debug_dev, tilt_calibrate, yPosComp,  INT16U_MIN, INT16U_MAX, tpcb_fail);
  GET_CAL_VALUE(debug_dev, tilt_calibrate, yNegComp,  INT16U_MIN, INT16U_MAX, tpcb_fail);

  /* Now write it */
  res = RinBlockTiltCalibrate(rb, &tilt_calibrate);
  rfprintf(debug_dev, "RinBlockTiltCalibrate() %s\n", (res) ? "success" : "failed");
  return;

tpcb_fail:
  rfprintf(debug_dev, "No configuration performed\n");
}


/** Perform improved calibration on tilt module */
static void tilt_perform_calibration_improved(struct t_io_dev *debug_dev, struct t_rinblock *rb)
{
  struct s_rb_tilt_calibrate_improved tilt_calibrate_improved;
  unsigned char res;
  long num;

  rfprintf(debug_dev, "Improved calibration of tilt module:\n");

  /* Get values */
  GET_CAL_VALUE(debug_dev, tilt_calibrate_improved, xPosAngle, INT16S_MIN, INT16S_MAX, tpci_fail);
  GET_CAL_VALUE(debug_dev, tilt_calibrate_improved, xNegAngle, INT16S_MIN, INT16S_MAX, tpci_fail);
  GET_CAL_VALUE(debug_dev, tilt_calibrate_improved, xPosComp,  INT16U_MIN, INT16U_MAX, tpci_fail);
  GET_CAL_VALUE(debug_dev, tilt_calibrate_improved, xNegComp,  INT16U_MIN, INT16U_MAX, tpci_fail);
  GET_CAL_VALUE(debug_dev, tilt_calibrate_improved, yPosAngle, INT16S_MIN, INT16S_MAX, tpci_fail);
  GET_CAL_VALUE(debug_dev, tilt_calibrate_improved, yNegAngle, INT16S_MIN, INT16S_MAX, tpci_fail);
  GET_CAL_VALUE(debug_dev, tilt_calibrate_improved, yPosComp,  INT16U_MIN, INT16U_MAX, tpci_fail);
  GET_CAL_VALUE(debug_dev, tilt_calibrate_improved, yNegComp,  INT16U_MIN, INT16U_MAX, tpci_fail);
  GET_CAL_VALUE(debug_dev, tilt_calibrate_improved, maxTilt,   INT16U_MIN, INT16U_MAX, tpci_fail);
  GET_CAL_VALUE(debug_dev, tilt_calibrate_improved, numDiv,    INT16U_MIN, INT16U_MAX, tpci_fail);

  /* Now write it */
  res = RinBlockTiltCalibrateImproved(rb, &tilt_calibrate_improved);
  rfprintf(debug_dev, "RinBlockTiltCalibrateImproved() %s\n", (res) ? "success" : "failed");
  return;

tpci_fail:
  rfprintf(debug_dev, "No configuration performed\n");
}


/** Test the tilt module */
void tilt_test(struct t_io_dev *debug_dev, struct t_rinblock *rb)
{
  unsigned char ch = '\0';

  while ('Q' != ch)
  {
    rfprintf(debug_dev, "Rin Block Tilt Test\n"
                        "Commands:\n"
                        "  R - Read tilt configuration - RinBlockTiltReadConfig()\n"
                        "  W - Write tilt configuration - RinBlockTiltWriteConfig()\n"
                        "  B - Read basic tilt data - RinBlockTiltReadTilt()\n"
                        "  b - Read basic tilt data repetitively - RinBlockTiltReadTilt()\n"
                        "  I - Read improved tilt data - RinBlockTiltReadTiltImproved()\n"
                        "  i - Read improved tilt data repetitively - RinBlockTiltReadTiltImproved()\n"
                        "  z - Perform zero calibration - RinBlockTiltZero()\n"
                        "  F - Perform factory zero calibration - RinBlockTiltFactoryZero()\n"
                        "  c - Perform basic calibration - RinBlockTiltCalibrate()\n"
                        "  d - Perform improved calibration - RinBlockTiltCalibrateImproved()\n"
                        "  Q - Quit\n");
    rread(debug_dev, &ch, 1, 0);
    switch (ch)
    {
      case 'R':
        tilt_read_config(debug_dev, rb);
        break;
      case 'W':
        tilt_write_config(debug_dev, rb);
        break;
      case 'B':
        tilt_read_tilt_basic(debug_dev, rb);
        break;
      case 'b':
        tilt_read_tilt_basic_repeat(debug_dev, rb);
        break;
      case 'I':
        tilt_read_tilt_improved(debug_dev, rb);
        break;
      case 'i':
        tilt_read_tilt_improved_repeat(debug_dev, rb);
        break;
      case 'z':
        tilt_zero_calibration(debug_dev, rb);
        break;
      case 'F':
        tilt_zero_factory_calibration(debug_dev, rb);
        break;
      case 'c':
        tilt_perform_calibration_basic(debug_dev, rb);
        break;
      case 'd':
        tilt_perform_calibration_improved(debug_dev, rb);
        break;
    }
  }
}
