#include <rinhw/rio.h>
#include <rinhw/rstdio.h>
#include <rinhw/ucos_ii.h>
#include <rinblock/rb_we21_adc.h>

#include "rb_we21_adc_test.h"


#define TEST_MAX_CHANNELS 20


static void rb_adc_test_dump(struct t_io_dev *dev, struct t_io_dev *adcdev, int num_channels)
{
  int16u adc_results[TEST_MAX_CHANNELS];
  int i;

  if (rlseek(adcdev, 0L, SEEK_SET) < 0)
  {
    rfprintf(dev, "ADC rlseek() failed\n");
    return;
  }
  if ((int) (sizeof(int16u) * num_channels) != rread(adcdev, &adc_results[0], sizeof(int16u) * num_channels, 0))
  {
    rfprintf(dev, "ADC rread() failed\n");
    return;
  }

  rfprintf(dev, "ADC readings: ");
  for (i = 0; i < num_channels; i++)
  {
    rfprintf(dev, "%06d ", adc_results[i]);
  }
  rfprintf(dev, "\r");
  OSTimeDly(OS_TICKS_PER_SEC / 10);
}


static void rb_adc_test_dump_repeat(struct t_io_dev *dev, struct t_io_dev *adcdev, int num_channels)
{
  char ch;

  rfprintf(dev, "Dumping ADC readings, any key to stop\n");
  while (reof(dev))
  {
    rb_adc_test_dump(dev, adcdev, num_channels);
    OSTimeDly(OS_TICKS_PER_SEC / 10);
  }
  rread(dev, &ch, 1, 0);
  rfprintf(dev, "\n");
}


void rb_adc_test(struct t_io_dev *dev, struct t_io_dev *adcdev)
{
  unsigned char ch;
  int num_channels;

  rfprintf(dev, "uP Internal ADC Test\n");

  if (ropen(adcdev, 0) < 0)
  {
    rfprintf(dev, "Failed to open ADC device\n");
    return;
  }

  /* Work out number of channels */
  if (rioctl(adcdev, IOCTL_ADC_GET_NUM_CHANNELS, &num_channels) < 0)
  {
    rfprintf(dev, "ADC IOCTL_ADC_GET_NUM_CHANNELS failed, assuming old NUM_ADC_CHANNELS\n");
    num_channels = NUM_ADC_CHANNELS;
  }
  rfprintf(dev, "ADC has %d channels\n", num_channels);
  if (num_channels > TEST_MAX_CHANNELS)
  {
    num_channels = TEST_MAX_CHANNELS;
  }

  for (ch = '\0'; ch != 'Q'; )
  {
    rfprintf(dev, "ADC Test\n"
                  "  d - Dump 1 reading\n"
                  "  D - Dump readings continuously\n"
#if 0
                  "  c - Print calibration\n"
                  "  C - Write calibration\n"
#endif
                  "  Q - Quit\n"
            );

    rread(dev, &ch, 1, 0);
    switch (ch)
    {
      case 'd':
        rb_adc_test_dump(dev, adcdev, num_channels);
        break;
      case 'D':
        rb_adc_test_dump_repeat(dev, adcdev, num_channels);
        break;
    }
  }

  rclose(adcdev);
}
