#include <rinhw/ucos_ii.h>
#include <rinhw/rio.h>
#include <rinhw/buzzer.h>
#include <rinhw/rstdio.h>
#include "rb_getnum.h"

#include "rb_we21_buzzer_test.h"


static const tBuzzer beep_seq[6] =
{
    { OS_TICKS_PER_SEC / 10, BUZZER_RESONANT_FREQ },
    { OS_TICKS_PER_SEC / 20, 0 },
    { OS_TICKS_PER_SEC / 10, BUZZER_RESONANT_FREQ },
    { OS_TICKS_PER_SEC / 20, 0 },
    { OS_TICKS_PER_SEC / 10, BUZZER_RESONANT_FREQ },
    { OS_TICKS_PER_SEC / 20, 0 },
};



static void do_beep(struct t_io_dev *dev, unsigned char times)
{
    rwrite(dev, (void *) &beep_seq[0], (unsigned short) (sizeof(tBuzzer) * 2 * times), 0);
}


static void beep_once(struct t_io_dev *dev)
{
    do_beep(dev, 1);
}


static void beep_twice(struct t_io_dev *dev)
{
    do_beep(dev, 2);
}


static void beep_thrice(struct t_io_dev *dev)
{
    do_beep(dev, 3);
}


static void beep_manual(struct t_io_dev *dev, struct t_io_dev *buz_dev)
{
  long freq;
  long time;
  tBuzzer buzzer;

  rfprintf(dev, "Enter buzz frequency in Hz, resonant freq is %d: ", BUZZER_RESONANT_FREQ);
  (void) get_long(dev, &freq);
  buzzer.freq = (unsigned short) freq;

  rfprintf(dev, "\r\nEnter buzz time in milliseconds: ");
  (void) get_long(dev, &time);
  time *= OS_TICKS_PER_SEC;
  time /= 1000;
  buzzer.time = (unsigned short) time;
  rfprintf(dev, "\r\n");

  rwrite(buz_dev, (void *) &buzzer, (unsigned short) sizeof(buzzer), 0);
}


void rb_buzzer_test(struct t_io_dev *dev, struct t_io_dev *buzdev)
{
    unsigned char temp = 0;

    if (ropen(buzdev, 0) < 0)
    {
        rfprintf(dev, "Opening buzzer failed\r\n");
        return;
    }

    while (temp != 'Q')
    {
        rfprintf(dev, "Buzzer Test\r\n"
                      "  1 - Beep Once, 2 - Beep Twice, 3 - Beep Thrice\r\n"
                      "  M - Manual Beep\r\n"
                      "  Q - Quit\r\n");

        rread(dev, &temp, 1, 0);
        switch (temp)
        {
            case '1':
                beep_once(buzdev);
                break;
            case '2':
                beep_twice(buzdev);
                break;
            case '3':
                beep_thrice(buzdev);
                break;
            case 'M':
                beep_manual(dev, buzdev);
                break;
        }
    }
    rclose(buzdev);
}

