#include <rinhw/rio.h>
#include <rinhw/rstdio.h>
#include <rinhw/ucos_ii.h>
#include <rinhw/kbd.h>
#include <rinblock/rb_we21_kbd.h>
#include "rb_getnum.h"

#include "rb_we21_kbd_test.h"



static unsigned char GetChar(struct t_io_dev *dev, unsigned char *dat)
{
    if (!reof(dev))
    {
        rread(dev, dat, 1, 0);
        return 1;
    }
    return 0;
}


//#define NUM_FUNC_KEYS 6
const unsigned char func_keys[NUM_FUNC_KEYS] =
{
    KEY_PHYSICAL_FUNC0, KEY_PHYSICAL_FUNC1, KEY_PHYSICAL_FUNC2, KEY_PHYSICAL_FUNC3,
    KEY_PHYSICAL_FUNC4, KEY_PHYSICAL_FUNC5
};


static void FixedConfigureKeys(struct t_io_dev *dev, struct t_io_dev *kbd)
{
  unsigned char ch;

  rfprintf(dev, "KEY_PHYSICAL_FUNC0 = basic, KEY_PHYSICAL_FUNC1 = repeat, KEY_PHYSICAL_FUNC2 = hold, KEY_PHYSICAL_FUNC3 = disabled\r\n");
  ch = KEY_PHYSICAL_FUNC0;
  rioctl(kbd, IOCTL_KBD_SET_BASIC, 1, &ch);

  ch = KEY_PHYSICAL_FUNC1;
  rioctl(kbd, IOCTL_KBD_SET_REPEAT, 1, &ch);

  ch = KEY_PHYSICAL_FUNC2;
  rioctl(kbd, IOCTL_KBD_SET_HOLD, 1, &ch);

  ch = KEY_PHYSICAL_FUNC3;
  rioctl(kbd, IOCTL_KBD_SET_DISABLED, 1, &ch);
}


void ConfigureKey(struct t_io_dev *dev, struct t_io_dev *kbd)
{
  unsigned char ch;
  int ioctlfn;
  int res;
  long key;

  rfprintf(dev, "Configure Key\r\nFind key numbers from KEY_PHYSICAL_ defines in kbd.h\r\nEnter key :");
  get_long(dev, &key);
  if (key < KEY_PHYSICAL_NUM0 || key > LAST_EXTERNAL_KEY)
  {
    rfprintf(dev, "Invalid key\r\n");
    return;
  }

  rfprintf(dev, "Enter key configuration (B - Basic, R - Repeat, H - Hold, D - Disabled:");
  rread(dev, &ch, 1, 0);
  rfprintf(dev, "%c\r\n", ch);
  switch (ch)
  {
    case 'B':
      ioctlfn = IOCTL_KBD_SET_BASIC;
      break;
    case 'R':
      ioctlfn = IOCTL_KBD_SET_REPEAT;
      break;
    case 'H':
      ioctlfn = IOCTL_KBD_SET_HOLD;
      break;
    case 'D':
      ioctlfn = IOCTL_KBD_SET_DISABLED;
      break;
    default:
      rfprintf(dev, "Invalid key configuration\r\n");
      return;
  }

  ch = (unsigned char) key;
  res = rioctl(kbd, ioctlfn, 1, &ch);
  if (RIO_SUCCESS == res)
  {
    rfprintf(dev, "Configured key successfully\r\n");
  }
  else
  {
    rfprintf(dev, "rioctl() failed %d\r\n", res);
  }
}


#if 0
rfprintf(f, "Keyboard Test\r\nOpening keyboard\r\n");
if (ropen(kbd_dev, 0) < 0)
    rfprintf(f, "Could not open keyboard\r\n");

ch = KEY_PHYSICAL_FUNC0;
res = rioctl(kbd_dev, IOCTL_KBD_SET_REPEAT, 1, &ch);
rfprintf(f, "Result of ioctl %d\r\n", res);

ch = KEY_PHYSICAL_NUM2;
res = rioctl(kbd_dev, IOCTL_KBD_SET_HOLD, 1, &ch);
rfprintf(f, "Result of ioctl %d\r\n", res);

//  res = rioctl(kbd_dev, IOCTL_KBD_SET_DISABLED, NUM_FUNC_KEYS, &func_keys[0]);
//    rfprintf(f, "Result of ioctl %d\r\n", res);

res = rioctl(kbd_dev, IOCTL_KBD_SET_DISABLED, NUM_FUNC_KEYS - 1, &func_keys[1]);
rfprintf(f, "Result of ioctl %d\r\n", res);

ch = KEY_PHYSICAL_POWER_AND_FUNC0;
res = rioctl(kbd_dev, IOCTL_KBD_SET_HOLD, 1, &ch);
rfprintf(f, "Result of ioctl %d\r\n", res);
#endif


static void BasicKeyboardTest(struct t_io_dev *dev, struct t_io_dev *kbd)
{
    union un_key key;
    unsigned char ch;
    char *event;

    rfprintf(dev, "Press keys on keyboard, any key to quit\r\n");
    while (!GetChar(dev, &ch))
    {
        if (GetChar(kbd, &key.BYTE))
        {
            switch ((enum tKeyEvent) key.BIT.EVENT)
            {
                case keDown:
                    event = "press";
                    break;
                case keUp:
                    event = "release";
                    break;
                case keHeld:
                    event = "long press";
                    break;
                case keDisabled:
                    event = "disabled";
                    break;
                default:
                    event = "error";
                    break;
            }
            rfprintf(dev, "Got %s event of key %d (0x%02x)\r\n", event, key.BIT.KEY, key.BIT.KEY);
        }
        OSTimeDly(OS_TICKS_PER_SEC / 40);
    }
}


void TestKeyboardWrite(struct t_io_dev *dev, struct t_io_dev *kbd)
{
    union un_key key;
    unsigned char ch;
    char *event;
    int res;

    rfprintf(dev, "KEY_PHYSICAL_NUM0 = normal, KEY_PHYSICAL_NUM1 = repeat, KEY_PHYSICAL_NUM2 = hold, KEY_PHYSICAL_NUM3 = disabled\r\n");
    ch = KEY_PHYSICAL_NUM0;
    rioctl(kbd, IOCTL_KBD_SET_BASIC, 1, &ch);

    ch = KEY_PHYSICAL_NUM1;
    rioctl(kbd, IOCTL_KBD_SET_REPEAT, 1, &ch);

    ch = KEY_PHYSICAL_NUM2;
    rioctl(kbd, IOCTL_KBD_SET_HOLD, 1, &ch);

    ch = KEY_PHYSICAL_NUM3;
    rioctl(kbd, IOCTL_KBD_SET_DISABLED, 1, &ch);

    rfprintf(dev, "Press keys on keyboard, or send serial characters, ESC to quit\r\n");
    rfprintf(dev, "a..j = KEY_PHYSICAL_NUM0..KEY_PHYSICAL_NUM9 short press\r\n");
    rfprintf(dev, "A..J = KEY_PHYSICAL_NUM0..KEY_PHYSICAL_NUM9 long press\r\n");

    while (ch != 0x1B)
    {
        if (GetChar(dev, &ch))
        {
            if ((ch >= 'a') && (ch <= 'j'))
            {
                key.BIT.KEY = ch - 'a' + KEY_PHYSICAL_NUM0;
                key.BIT.EVENT = keDown;
                res = rwrite(kbd, &key.BYTE, 1, 0);
                rfprintf(dev, "Writing key %d short press to keyboard with result %d.\r\n", key.BIT.KEY, res);
            }
            else if ((ch >= 'A') && (ch <= 'J'))
            {
                key.BIT.KEY = ch - 'A' + KEY_PHYSICAL_NUM0;
                key.BIT.EVENT = keHeld;
                res = rwrite(kbd, &key.BYTE, 1, 0);
                rfprintf(dev, "Writing key %d long press to keyboard with result %d.\r\n", key.BIT.KEY, res);
            }
        }

        if (GetChar(kbd, &key.BYTE))
        {
            switch ((enum tKeyEvent) key.BIT.EVENT)
            {
                case keDown:
                    event = "press";
                    break;
                case keUp:
                    event = "release";
                    break;
                case keHeld:
                    event = "long press";
                    break;
                case keDisabled:
                    event = "disabled";
                    break;
                default:
                    event = "error";
                    break;
            }
            rfprintf(dev, "Got key %s keyboard (0x%X)\r\n", event, key.BIT.KEY);
        }
        OSTimeDly(OS_TICKS_PER_SEC / 40);
    }
}


void rb_keyboard_test(struct t_io_dev *dev, struct t_io_dev *kbddev)
{
    unsigned char ch = 0;

    if (ropen(kbddev, 0) < 0)
    {
        rfprintf(dev, "Opening keyboard failed\r\n");
        return;
    }

    while (ch != 'Q')
    {
        rfprintf(dev, "Keyboard Test\r\n"
                      "  F - Fixed key configuration\r\n"
                      "  M - Manual key configuration\r\n"
                      "  B - Basic keyboard test\r\n"
                      "  W - Keyboard buffer write test\r\n"
                      "  Q - Quit\r\n");

        rread(dev, &ch, 1, 0);
        switch (ch)
        {
            case 'F':
                FixedConfigureKeys(dev, kbddev);
                break;
            case 'M':
                ConfigureKey(dev, kbddev);
                break;
            case 'B':
                BasicKeyboardTest(dev, kbddev);
                break;
            case 'W':
                TestKeyboardWrite(dev, kbddev);
                break;
        }
    }
    rclose(kbddev);
}
