#include <string.h>
#include <rinhw/rio.h>
#include <rinhw/rstdio.h>
#include <rinhw/ucos_ii.h>
#include <rinhw/lcd.h>
#include <rinblock/rb_we21_lcd.h>
#include <rinblock/rb_c500_lcd.h>
#include <rinblock/rb_command.h>

#include "rb_we21_lcd_test.h"


static unsigned long get_num(struct t_io_dev *debug_dev)
{
    unsigned char ch;
    unsigned long num = 0UL;

    do
    {
        rread(debug_dev, &ch, 1, 0);
        rwrite(debug_dev, &ch, 1, 0);
        if ((ch >= '0') && (ch <= '9'))
        {
            num *= 10;
            num += (unsigned char) (ch - '0');
        }
    }
    while ((ch >= '0') && (ch <= '9'));
    return num;
}


static void SetAllSegs(struct t_io_dev *dev, BOOLEAN state)
{
    int i, l;
    unsigned char ch;

    ropen(dev, 0);

    rlseek(dev, 0, SEEK_SET);
    ch = (state) ? 0xFF : 0;

    l = (int) rfilelength(dev);
    for (i = 0; i < l; i++)
        rwrite(dev, &ch, 1, 0);
    rclose(dev);
}


#if 0
static unsigned char GetChar(struct t_io_dev *dev, unsigned char *dat)
{
    if (!reof(dev))
    {
        rread(dev, dat, 1, 0);
        return 1;
    }
    return 0;
}
#endif


unsigned short GetHexWord(struct t_io_dev *dev)
{
    unsigned char ch, i;
    unsigned short res;

    res = 0;
    rfprintf(dev, "Enter a number in hex (4 digits): ");
    for (i = 0; i < 4; i++)
    {
        res <<= 4;
        rread(dev, &ch, 1,  0);
        rwrite(dev, &ch, 1, 0);
        if ((ch >= '0') && (ch <= '9'))
            res += (ch - '0');
        else if ((ch >= 'A') && (ch <= 'F'))
            res += (ch - 'A' + 10);
        else if ((ch >= 'a') && (ch <= 'f'))
            res += (ch - 'a' + 10);
        else
        {
            rwrite(dev, "?", 1, 0);
            res = 0;
            break;
        }
    }
    rfprintf(dev, "\r\n");
    return res;
}


static void TestBacklight(struct t_io_dev *dev, struct t_io_dev *lcd_dev)
{
  unsigned char bklight;

  ropen(lcd_dev, 0);
  while (1)
  {
    rfprintf(dev, "\nEnter backlight level (0%%..100%%), or >100 to quit: ");
    bklight = get_num(dev);
    if (bklight > 100)
      break;
    rioctl(lcd_dev, IOCTL_LCD_SET_BACKLIGHT, bklight);
  }
  rclose(lcd_dev);
}


void LcdSegsTest(struct t_io_dev *dev, struct t_io_dev *lcd_dev)
{
    int devpos, prevpos;
    int num_segments;
    unsigned char data, temp;

    ropen(lcd_dev, 0);
    rioctl(lcd_dev, IOCTL_LCD_ACCESS_DATA);
    num_segments = (int) rfilelength(lcd_dev) * 8;
    rlseek(lcd_dev, 0L, SEEK_SET);
    rclose(lcd_dev);

    prevpos = devpos = 0;
    while (reof(dev))
    {
        rfprintf(dev, "Segment %d\r\n", devpos);

        ropen(lcd_dev, 0);
        if (prevpos != devpos)
        {
            rlseek(lcd_dev, (long) (prevpos >> 3), SEEK_SET);
            temp = 0;
            rwrite(lcd_dev, &temp, 1, 0);
        }

        rlseek(lcd_dev, (long) (devpos >> 3), SEEK_SET);
        data = (unsigned char) (0x01 << (devpos & 0x07));
        rwrite(lcd_dev, &data, 1, 0);

        rclose(lcd_dev);

        prevpos = devpos;
        if (++devpos >= num_segments)
            devpos = 0;

        OSTimeDly(OS_TICKS_PER_SEC / 4);
    }

    rread(dev, &data, 1, 0);
}



void LcdSegsTestManual(struct t_io_dev *dev, struct t_io_dev *lcd_dev)
{
    int devpos, prevpos, num_segments;
    unsigned char data, temp = 0, fwd = 1, pause = 0;

    ropen(lcd_dev, 0);
    rioctl(lcd_dev, IOCTL_LCD_ACCESS_DATA);
    num_segments = (int) rfilelength(lcd_dev) * 8;
    rlseek(lcd_dev, 0L, SEEK_SET);
    rclose(lcd_dev);

    rfprintf(dev, "Manual Test Segments\r\n1 - Change Direction, 2 - Pause, Q - Quit, Anything else next seg\r\n");
    prevpos = devpos = 0;
    while (1)
    {
        rfprintf(dev, "Now at segment %d\r\n", devpos);

        ropen(lcd_dev, 0);
        if (prevpos != devpos)
        {
            rlseek(lcd_dev, (long) (prevpos >> 3), SEEK_SET);
            temp = 0;
            rwrite(lcd_dev, &temp, 1, 0);
        }
        rlseek(lcd_dev, (long) (devpos >> 3), SEEK_SET);
        data = (unsigned char) (0x01 << (devpos & 0x07));
        rwrite(lcd_dev, &data, 1, 0);
        rclose(lcd_dev);

        rread(dev, &temp, 1, 0);
        switch (temp)
        {
            case '1':
                fwd = !fwd;
                break;
            case '2':
                pause = !pause;
                break;
            case 'Q':
                return;

            default:
                prevpos = devpos;
                if (!pause)
                {
                    if (fwd)
                    {
                        if (devpos >= num_segments - 1)
                            devpos = 0;
                        else
                            devpos++;
                    }
                    else
                    {
                        if (devpos == 0)
                            devpos = num_segments - 1;
                        else
                            devpos--;
                    }
                }
                break;
        }
    }
}


#if 0
void LcdData(struct t_io_dev *dev, struct t_io_dev *lcd_dev)
{
    int res;
    tLcdSegments *lcd_segs = (tLcdSegments *) 0xA5A5A5A5;
    unsigned char ch;

    /*
     * Open the LCD device with exclusive access, and wait until it becomes available, then
     */
    ropen(lcd_dev, 0);
    rfprintf(dev, "&lcd_segs=%p, lcd_segs=%p\r\n", &lcd_segs, lcd_segs);
    res = rioctl(lcd_dev, IOCTL_LCD_GET_DATAPTR, &lcd_segs);
    rfprintf(dev, "rioctl result: %d, The pointer is %p.\r\n", res, lcd_segs);

    rfprintf(dev, "LCD Buffer Dump:\r\n");
    rlseek(lcd_dev, 0, SEEK_SET);
    while (reof(lcd_dev) == 0)
    {
        if ((rtell(lcd_dev) & 0x07) == 0x07)
            rfprintf(dev, "\r\n");
        rread(lcd_dev, &ch, 1, 0);
        rfprintf(dev, "0x%02x, ", ch);
    }
    rfprintf(dev, "\r\n");
    rclose(lcd_dev);
}
#endif


static void ShowSeg(struct t_io_dev *dev, struct t_io_dev *lcd_dev, void *s, int len, char *sctname, char *segname)
{
    unsigned char ch;

    rfprintf(dev, "Testing Seg %s.%s, press a key\r\n", sctname, segname);

    ropen(lcd_dev, 0);
    rlseek(lcd_dev, 0L, SEEK_SET);
    rwrite(lcd_dev, s, (unsigned short) len, 0);
    rclose(lcd_dev);

    rread(dev, &ch, 1, 0);
}


#define N_SEG(sct, nam) s->Named.sct.Segments.nam = 1; ShowSeg(dev, lcd_dev, s, sizeof(*s), # sct, # nam); s->Named.sct.Segments.nam = 0


static void TestWe21LcdNamedSegments(struct t_io_dev *dev, struct t_io_dev *lcd_dev)
{
    tWE21LcdSegments segs_we21;
    tWE21LcdSegments *s = &segs_we21;

    /*
     * Zero out all segs
     */
    memset(s, 0, sizeof(*s));
    rfprintf(dev, "LCD Segment Bit Struct Testing For WE2111\r\n\r\n");

    N_SEG(Annunciators, Gross);
    N_SEG(Annunciators, Net);
    N_SEG(Annunciators, Coz);
    N_SEG(Annunciators, Setpoint0);
    N_SEG(Annunciators, Setpoint1);
    N_SEG(Annunciators, Setpoint2);
    N_SEG(Annunciators, Setpoint3);
    N_SEG(Annunciators, Setpoint4);
    N_SEG(Annunciators, Setpoint5);
    N_SEG(Annunciators, Setpoint6);
    N_SEG(Annunciators, Setpoint7);
    N_SEG(Annunciators, Range1);
    N_SEG(Annunciators, Range2);

    N_SEG(Units, Grams);
    N_SEG(Units, Kilograms);
    N_SEG(Units, Pounds);
    N_SEG(Units, Tonnes);
}


#define C5_N_SEG_VAL(sct, nam, val) s.sct.bits.nam = val; ShowSeg(dev, lcd_dev, &s, sizeof(s), # sct, # nam); s.annunciators.bits.nam = 0
#define C5_N_SEG(sct, nam) C5_N_SEG_VAL(sct, nam, 1)


static void TestC500LcdNamedSegments(struct t_io_dev *dev, struct t_io_dev *lcd_dev)
{
  struct s_rb_c500_disp_write_14seg s;

  /* Zero out all segs */
  memset(&s, 0, sizeof(s));
  rfprintf(dev, "LCD Segment Bit Struct Testing For C500\n\n");

  C5_N_SEG(annunciators, status_coz);
  C5_N_SEG(annunciators, status_motion);
  C5_N_SEG(annunciators, status_net);
  C5_N_SEG(annunciators, status_zeroband);
  C5_N_SEG(annunciators, status_held);
  C5_N_SEG(annunciators, status_spare);
  C5_N_SEG(annunciators, units_lb);
  C5_N_SEG(annunciators, units_kg);
  C5_N_SEG(annunciators, units_g);
  C5_N_SEG(annunciators, units_oz);
  C5_N_SEG(annunciators, units_t);
  C5_N_SEG(annunciators, units_spare);
  C5_N_SEG(annunciators, setpoint0);
  C5_N_SEG(annunciators, setpoint1);
  C5_N_SEG(annunciators, setpoint2);
  C5_N_SEG(annunciators, setpoint3);
  C5_N_SEG(annunciators, setpoint4);
  C5_N_SEG(annunciators, setpoint5);
  C5_N_SEG(annunciators, setpoint6);
  C5_N_SEG(annunciators, setpoint7);
  C5_N_SEG(annunciators, range1);
  C5_N_SEG(annunciators, range2);
  C5_N_SEG(annunciators, range3);
  C5_N_SEG(annunciators, check_over2);
  C5_N_SEG(annunciators, check_over1);
  C5_N_SEG(annunciators, check_pass);
  C5_N_SEG(annunciators, check_under1);
  C5_N_SEG(annunciators, check_under2);
  C5_N_SEG(annunciators, hobbin);
  C5_N_SEG(annunciators, nobbin);
}


#if 0
#define D_SEG(sct, d, n) \
    s->Named.sct.D_ ## d ## n = 1; \
    ShowSeg(dev, lcd_dev, s, # sct, "D_" # d # n); \
    s->Named.sct.D_ ## d ## n = 0

#define DALL_NODP_SEG(sct, d) \
    D_SEG(sct, d, A); D_SEG(sct, d, B); D_SEG(sct, d, C); D_SEG(sct, d, D); \
    D_SEG(sct, d, E); D_SEG(sct, d, F); D_SEG(sct, d, G); \
    D_SEG(sct, d, I); D_SEG(sct, d, J)


#define DALL_SEG(sct, d) \
    D_SEG(sct, d, A); D_SEG(sct, d, B); D_SEG(sct, d, C); D_SEG(sct, d, D); \
    D_SEG(sct, d, E); D_SEG(sct, d, F); D_SEG(sct, d, G); D_SEG(sct, d, H); \
    D_SEG(sct, d, I); D_SEG(sct, d, J)


void TestLcdNamedDigitSegments(struct t_io_dev *dev, struct t_io_dev *lcd_dev)
{
    tLcdSegments *s = &segs;

    /*
     * Zero out all segs
     */
    memset(s, 0, sizeof(*s));
    rfprintf(dev, "LCD Segment Digit Bit Struct Testing\r\n\r\n");

    DALL_SEG(TopRowDigits, 1);
    DALL_SEG(TopRowDigits, 2);
    DALL_SEG(TopRowDigits, 3);
    DALL_SEG(TopRowDigits, 4);
    DALL_SEG(TopRowDigits, 5);
    DALL_NODP_SEG(TopRowDigits, 6);

    DALL_SEG(TopRightRowDigits, 1);
    DALL_SEG(TopRightRowDigits, 2);
    DALL_SEG(TopRightRowDigits, 3);
    DALL_NODP_SEG(TopRightRowDigits, 4);

    DALL_SEG(BottomRowDigits, 1);
    DALL_SEG(BottomRowDigits, 2);
    DALL_SEG(BottomRowDigits, 3);
    DALL_SEG(BottomRowDigits, 4);
    DALL_SEG(BottomRowDigits, 5);
    DALL_SEG(BottomRowDigits, 6);
    DALL_SEG(BottomRowDigits, 7);
    DALL_SEG(BottomRowDigits, 8);
    DALL_NODP_SEG(BottomRowDigits, 9);

    DALL_SEG(BottomRightRowDigits, 1);
    DALL_SEG(BottomRightRowDigits, 2);
    DALL_SEG(BottomRightRowDigits, 3);
    DALL_SEG(BottomRightRowDigits, 4);
    DALL_SEG(BottomRightRowDigits, 5);
    DALL_SEG(BottomRightRowDigits, 6);
    DALL_SEG(BottomRightRowDigits, 7);
    DALL_NODP_SEG(BottomRightRowDigits, 8);
}
#endif


static void LcdWriteChars(struct t_io_dev *dev, struct t_io_dev *lcd_dev)
{
    unsigned char ch, pos;

    rfprintf(dev, "LCD Character Test. ESC to quit.\r\n");
    SetAllSegs(lcd_dev, FALSE);

    ch = 0;
    pos = 0;
    while (ch != 0x1B)
    {
        ropen(lcd_dev, 0);
        rioctl(lcd_dev, IOCTL_LCD_ACCESS_CHAR);
        rlseek(lcd_dev, (long) pos, SEEK_SET);
        rread(dev, &ch, 1, 0);
        rwrite(lcd_dev, &ch, 1, 0);
        pos = (unsigned char) rtell(lcd_dev);
        if (pos >= rfilelength(lcd_dev))
            pos = 0;
        rclose(lcd_dev);
    }
}


#define MAX_BUF_LEN 20

static void LcdWriteBlockChars(struct t_io_dev *dev, struct t_io_dev *lcd_dev)
{
    unsigned char ch, len, pos;
    unsigned char buf[MAX_BUF_LEN];

    rfprintf(dev, "LCD Character Block Write Test. Enter posn followed by LF\r\n");

    pos = (unsigned char) get_num(dev);

    rfprintf(dev, "Send String (upto 20 chars) followed by LF\r\n");

    len = 0;
    do
    {
        rread(dev, &ch, 1, 0);
        rwrite(dev, &ch, 1, 0);
        if (ch != 0x0A)
            buf[len++] = ch;
    }
    while ((len < MAX_BUF_LEN) && (ch != 0x0A));
    buf[len] = '\0';

    SetAllSegs(lcd_dev, FALSE);

    ropen(lcd_dev, 0);
    rioctl(lcd_dev, IOCTL_LCD_ACCESS_CHAR);
    rlseek(lcd_dev, (long) pos, SEEK_SET);
    rwrite(lcd_dev, buf, (unsigned short) strlen((char *) buf), 0);
    rclose(lcd_dev);
}


static void LcdDumpChars(struct t_io_dev *dev, struct t_io_dev *lcd_dev)
{
    unsigned char ch;

    ropen(lcd_dev, 0);
    rioctl(lcd_dev, IOCTL_LCD_ACCESS_CHAR);
    rlseek(lcd_dev, 0L, SEEK_SET);

    rfprintf(dev, "Dump LCD Character Buffer (size %ld):\r\n", rfilelength(lcd_dev));

    while (rtell(lcd_dev) < rfilelength(lcd_dev))
    {
        rread(lcd_dev, &ch, 1, 0);
        rfprintf(dev, "%c", ch);
    }

    rclose(lcd_dev);
    rfprintf(dev, "\r\n");
}


#ifdef IOCTL_LCD_ACCESS_FLASH
/** Flash segments, 0 = none, 1 = all, 2 = digits, 3 = annunciators */
static void FlashSegments(struct t_io_dev *dev, struct t_io_dev *lcd_dev, int which)
{
    long size, i;
    unsigned char ch;

    ropen(lcd_dev, 0);
    rioctl(lcd_dev, IOCTL_LCD_ACCESS_FLASH);
    size = rfilelength(lcd_dev);

    switch (which)
    {
      /* No flash */
      case 0:
        ch = 0x00;
        for (i = 0; i < size; i++)
        {
          rwrite(lcd_dev, &ch, 1, 0);
        }
        break;

      /* All flash */
      case 1:
        ch = 0xFF;
        for (i = 0; i < size; i++)
        {
          rwrite(lcd_dev, &ch, 1, 0);
        }
        break;

      /* Digits */
      case 2:
        i = offsetof(tWE21LcdDigits, Digits);
        size = i + sizeof(tWE21LcdSegments_Digits);
        rlseek(lcd_dev, i, SEEK_SET);
        ch = 0xFF;
        for (i = 0; i < size; i++)
        {
          rwrite(lcd_dev, &ch, 1, 0);
        }
        break;

#if 0
      /* Annunciators */
      case 3:
        i = offsetof(tWE21LcdDigits, Digits);
        rlseek(lcd_dev, i);
        ch = 0xFF;
        for (i = 0; i < size; i++)
        {
          rwrite(lcd_dev, &ch, 1, 0);
        }
        break;
#endif
    }
    rclose(lcd_dev);
}
#endif /* #ifdef IOCTL_LCD_ACCESS_FLASH */


void rb_lcd_test(struct t_io_dev *dev, struct t_io_dev *lcd_dev, int16u interface_type)
{
    unsigned char temp;

    while (1)
    {
        rfprintf(dev, "LCD Test\n"
                      " Segments:   0 - All Segs Off\n"
                      "             1 - All Segs On\n"
                      "             L - Test Segments\n"
                      "             M - Manual Test Segments\n"
                      "             D - Print LCD Data\n"
                      "             B - Backlight\n"
                      "             N - Test Named Segments\n"
                      "             S - Test Named Digit\n"
#ifdef IOCTL_LCD_ACCESS_FLASH
                      "             6 - Flash off all segs\n"
                      "             7 - Flash all segs\n"
                      "             8 - Flash digits\n"
#endif /* #ifdef IOCTL_LCD_ACCESS_FLASH */
                      " Characters: C - Write LCD Characters\n"
                      "             c - Write Block Chars\n"
                      "             P - Print LCD Character buffer\n"
                      " Other:\n"
                      "             Q - Quit\n");

        rread(dev, &temp, 1, 0);
        switch (temp)
        {
            case '0':
                SetAllSegs(lcd_dev, FALSE);
                break;
            case '1':
                SetAllSegs(lcd_dev, TRUE);
                break;
#ifdef IOCTL_LCD_ACCESS_FLASH
            case '6':
            case '7':
            case '8':
                FlashSegments(dev, lcd_dev, temp - '6');
                break;
#endif /* #ifdef IOCTL_LCD_ACCESS_FLASH */
            case 'L':
                LcdSegsTest(dev, lcd_dev);
                break;
            case 'M':
                LcdSegsTestManual(dev, lcd_dev);
                break;
#if 0
            case 'D':
                LcdData(dev, lcd_dev);
                break;
#endif
            case 'B':
                TestBacklight(dev, lcd_dev);
                break;
            case 'N':
                if (RB_IFACE_TYPE_WE21_DISP == interface_type)
                  TestWe21LcdNamedSegments(dev, lcd_dev);
                else
                  TestC500LcdNamedSegments(dev, lcd_dev);
                break;
#if 0
            case 'S':
                TestLcdNamedDigitSegments(dev, lcd_dev);
                break;
#endif
            case 'C':
                LcdWriteChars(dev, lcd_dev);
                break;
            case 'c':
                LcdWriteBlockChars(dev, lcd_dev);
                break;
            case 'P':
                LcdDumpChars(dev, lcd_dev);
                break;
            case 'Q':
                return;
        }
    }
}


#if 0
void LcdStuff(void)
{
    SetAllSegs(&dev_lcd, 1);
}
#endif
