#include <string.h>
#include <rinhw/rio.h>
#include <rinremote/remote.h>
#include <rinremote/remoteexec.h>
#include <rinhw/debug.h>
#include <rinhw/stdrinstrum.h>
#include <rinhw/rinendian.h>
#include <rinremote/remote_adc24bit.h>
#include <rinhw/adc_ads1216.h>
#include "remote_device.h"

#include <rinremote/adc_ads1216_remote.h>


static int remote_dev_ioctl_adc24bit_set_level( struct t_io_dev *io_dev, int16u ui16IoCtlFn, int32u ui32Level )
{
  int8u txbuf[ GET_SIZE_REMOTE_CMD_IOCTL_ADC24BIT_SET_LEVEL() ];
  int8u rxbuf[ GET_SIZE_REMOTE_RESP_IOCTL_DEFAULT() ];
  struct s_rem_device_command *prdCmd = (struct s_rem_device_command *) txbuf;
  struct s_rem_device_response *prdResponse = (struct s_rem_device_response *) rxbuf;
  int nResult;
  const struct s_remote_device_info *remote_info = (struct s_remote_device_info *) io_dev->dev_data;
  const int8u ui8Device = remote_info->device_id;

  vInitRemoteIoCtlCmdHdr( prdCmd, ui8Device, ui16IoCtlFn );
  prdCmd->rem_device_command.rem_dev_cmd_ioctl.remote_ioctl_command.rem_dev_cmd_ioctl_adc24bit_set_level.ui32Level = htonl( ui32Level );
  if ( !bRemoteExecuteCommand( prdCmd, GET_SIZE_REMOTE_CMD_IOCTL_ADC24BIT_SET_LEVEL(),
                               prdResponse, GET_SIZE_REMOTE_RESP_IOCTL_DEFAULT() ) )
  {
    nResult = RIO_ERROR;
  }
  else
  {
    nResult = nCheckRemoteIoCtlResp_Default( prdResponse, ui8Device, ui16IoCtlFn );
  }
  return nResult;
}


static int remote_dev_ioctl_adc24bit_get_level( struct t_io_dev *io_dev, int16u ui16IoCtlFn, int32u *pui32Level )
{
  int8u txbuf[ GET_SIZE_REMOTE_CMD_IOCTL_ADC24BIT_GET_LEVEL() ];
  int8u rxbuf[ GET_SIZE_REMOTE_RESP_IOCTL_ADC24BIT_GET_LEVEL() ];
  struct s_rem_device_command *prdCmd = (struct s_rem_device_command *) txbuf;
  struct s_rem_device_response *prdResponse = (struct s_rem_device_response *) rxbuf;
  int nResult;
  const struct s_remote_device_info *remote_info = (struct s_remote_device_info *) io_dev->dev_data;
  const int8u ui8Device = remote_info->device_id;

  vInitRemoteIoCtlCmdHdr( prdCmd, ui8Device, ui16IoCtlFn );
  prdCmd->rem_device_command.rem_dev_cmd_ioctl.remote_ioctl_command.rem_dev_cmd_ioctl_adc24bit_get_level.ui8Unused = 0;
  if ( !bRemoteExecuteCommand( prdCmd, GET_SIZE_REMOTE_CMD_IOCTL_ADC24BIT_GET_LEVEL(),
                               prdResponse, GET_SIZE_REMOTE_RESP_IOCTL_ADC24BIT_GET_LEVEL() ) )
  {
    nResult = RIO_ERROR;
  }
  else if ( !bCheckRemoteIoCtlRespHdr( prdResponse, ui8Device, ui16IoCtlFn ) )
  {
    nResult = RIO_ERROR;
  }
  else
  {
    nResult = ntohl( prdResponse->rem_device_response.rem_dev_resp_ioctl.remote_ioctl_response.rem_dev_resp_ioctl_adc24bit_get_level.i32Result );
    *pui32Level = ntohl( prdResponse->rem_device_response.rem_dev_resp_ioctl.remote_ioctl_response.rem_dev_resp_ioctl_adc24bit_get_level.ui32Level );
  }
  return nResult;
}


/** Swap byte order for adc data */
static int adc24bit_remote_read(struct t_io_dev *dev, void *buf, unsigned short len, unsigned short timeout)
{
  int read_result;
  int nResult = RIO_ERROR;
  int32s *pi32Data = (int32s *) buf;

#if HW_DEBUG(ADC_ADS1216_DEBUG, HW_DEBUG_BASIC)
  DEBUG_PRINTF("adc24bit_post_read called\n");
#endif

  /* Read from remote device */
  read_result = remote_dev_read(dev, buf, len, timeout);

  /* Do byte swap if read successful */
  if ( read_result == len && len == sizeof( int32s ) )
  {
    *pi32Data = ntohl( *pi32Data );
    nResult = read_result;
  }

#if HW_DEBUG(ADC_ADS1216_DEBUG, HW_DEBUG_FULL)
  DEBUG_PRINTF("adc24bit_post_read: result %d, data %ld\n", nResult, *pi32Data);
#endif
  return nResult;
}


/*
 * adc24bit_ioctl -
 *
 * ADC specific configuration
 */
static int adc24bit_remote_ioctl(struct t_io_dev *io_dev, int func, va_list *arglist)
{
  int result = RIO_SUCCESS;
  unsigned long ldata;
  int32u ui32data;
  int ndata;

#if HW_DEBUG(ADC_ADS1216_DEBUG, HW_DEBUG_BASIC)
  DEBUG_PRINTF("adc24bit_ioctl called\n");
#endif

  /* Function dispatch */
  switch (func)
  {

    /* Set zero register or span register */
    case IOCTL_ADC24BIT_SET_ZERO_REGISTER:
    case IOCTL_ADC24BIT_SET_SPAN_REGISTER:
      ldata = va_arg(*arglist, unsigned long);
      result = remote_dev_ioctl_adc24bit_set_level( io_dev, func, (int32u) ldata );
#if HW_DEBUG(ADC_ADS1216_DEBUG, HW_DEBUG_FULL)
      DEBUG_PRINTF("IOCTL_ADC24BIT_SET_ZERO_REGISTER/IOCTL_ADC24BIT_SET_SPAN_REGISTER: func %d, level %lu, result %d\n", func, ldata, result);
#endif
      break;

    /* Get zero register or span register */
    case IOCTL_ADC24BIT_GET_ZERO_REGISTER:
    case IOCTL_ADC24BIT_GET_SPAN_REGISTER:
      result = remote_dev_ioctl_adc24bit_get_level(io_dev, func, &ui32data);
      *(va_arg(*arglist, unsigned long *)) = ui32data;
#if HW_DEBUG(ADC_ADS1216_DEBUG, HW_DEBUG_FULL)
      DEBUG_PRINTF("IOCTL_ADC24BIT_GET_ZERO_REGISTER/IOCTL_ADC24BIT_GET_SPAN_REGISTER: func %d, level %lu, result %d\n", func, (unsigned long) ui32data, result);
#endif
      break;

    /* Set the PGA gain register */
    case IOCTL_ADC24BIT_SET_PGA_GAIN:
      ndata = va_arg(*arglist, int);
      result = remote_dev_ioctl_adc24bit_set_level( io_dev, IOCTL_ADC24BIT_SET_PGA_GAIN, (int32u) ndata );
#if HW_DEBUG(ADC_ADS1216_DEBUG, HW_DEBUG_FULL)
      DEBUG_PRINTF("IOCTL_ADC24BIT_SET_PGA_GAIN: level %d, result %d\n", ndata, result);
#endif
      break;

    /* Set the PGA gain register */
    case IOCTL_ADC24BIT_SET_DECIMATION_RATIO:
      ndata = va_arg(*arglist, int);
      result = remote_dev_ioctl_adc24bit_set_level( io_dev, IOCTL_ADC24BIT_SET_DECIMATION_RATIO, (int32u) ndata );
#if HW_DEBUG(ADC_ADS1216_DEBUG, HW_DEBUG_FULL)
      DEBUG_PRINTF("IOCTL_ADC24BIT_SET_DECIMATION_RATIO: level %d, result %d\n", ndata, result);
#endif
      break;

    /* Set 4 or 6 wire mode */
    case IOCTL_ADC24BIT_SET_LCWIRE_MODE:
      ndata = va_arg(*arglist, int);
      result = remote_dev_ioctl_adc24bit_set_level( io_dev, IOCTL_ADC24BIT_SET_LCWIRE_MODE, (int32u) ndata );
#if HW_DEBUG(ADC_ADS1216_DEBUG, HW_DEBUG_FULL)
      DEBUG_PRINTF("IOCTL_ADC24BIT_SET_LCWIRE_MODE: level %d, result %d\n", ndata, result);
#endif
      break;

    /* Force reset of ADC */
    case IOCTL_ADC24BIT_FORCE_RESET:
      result = remote_dev_ioctl_adc24bit_set_level( io_dev, IOCTL_ADC24BIT_FORCE_RESET, 0 );
#if HW_DEBUG(ADC_ADS1216_DEBUG, HW_DEBUG_FULL)
      DEBUG_PRINTF("IOCTL_ADC24BIT_FORCE_RESET: result %d\n", result);
#endif
      break;

    /* Check if the ADC is responding */
    case IOCTL_ADC24BIT_IS_RESPONDING:
      result = remote_dev_ioctl_adc24bit_get_level( io_dev, func, &ui32data );
      *(va_arg(*arglist, int *)) = (int) ui32data;
#if HW_DEBUG(ADC_ADS1216_DEBUG, HW_DEBUG_FULL)
      DEBUG_PRINTF("IOCTL_ADC24BIT_IS_RESPONDING: level %d, result %d\n", (int) ui32data, result);
#endif
      break;

    /* Turn the burnout current source off or on */
    case IOCTL_ADC24BIT_SET_BURNOUTCURRENTSOURCE:
      ndata = va_arg(*arglist, int);
      result = remote_dev_ioctl_adc24bit_set_level( io_dev, IOCTL_ADC24BIT_SET_BURNOUTCURRENTSOURCE, (int32u) ndata );
#if HW_DEBUG(ADC_ADS1216_DEBUG, HW_DEBUG_FULL)
      DEBUG_PRINTF("IOCTL_ADC24BIT_SET_BURNOUTCURRENTSOURCE: level %d, result %d\n", ndata, result);
#endif
      break;

    /* Perform a self zero calibration */
    case IOCTL_ADC24BIT_PERFORM_SELF_ZERO_CALIBRATION:
      result = remote_dev_ioctl_adc24bit_set_level( io_dev, IOCTL_ADC24BIT_PERFORM_SELF_ZERO_CALIBRATION, 0 );
#if HW_DEBUG(ADC_ADS1216_DEBUG, HW_DEBUG_FULL)
      DEBUG_PRINTF("IOCTL_ADC24BIT_PERFORM_SELF_ZERO_CALIBRATION: result %d\n", result);
#endif
      break;

    /* Get the ADC ID */
    case IOCTL_ADC24BIT_GET_ID:
      result = remote_dev_ioctl_adc24bit_get_level( io_dev, func, &ui32data );
      *(va_arg(*arglist, int *)) = (int) ui32data;
#if HW_DEBUG(ADC_ADS1216_DEBUG, HW_DEBUG_FULL)
      DEBUG_PRINTF("IOCTL_ADC24BIT_GET_ID: id %d, result %d\n", (int) ui32data, result);
#endif
      break;

    /* Wait for data ready interrupt to occur */
    case IOCTL_ADC24BIT_WAIT_FOR_DATA_READY:
      ldata = va_arg(*arglist, unsigned long);
      result = remote_dev_ioctl_adc24bit_set_level( io_dev, IOCTL_ADC24BIT_WAIT_FOR_DATA_READY, (int32u) ldata );
#if HW_DEBUG(ADC_ADS1216_DEBUG, HW_DEBUG_FULL)
      DEBUG_PRINTF("IOCTL_ADC24BIT_WAIT_FOR_DATA_READY: timeout %lu, result %d\n", ldata, result);
#endif
      break;

    /* Error */
    default:
      result = RIO_ERROR;
      break;
  }

  /* Done, return result */
#if HW_DEBUG(ADC_ADS1216_DEBUG, HW_DEBUG_BASIC)
  DEBUG_PRINTF("adc24bit_ioctl finished with result %d\n", result);
#endif
  return result;
}


/** Function to create and initialise a remote adc device */
int rio_create_device_adc24bit_remote(struct t_io_dev *io_dev)
{
  int result;

  /* Create and initialise a default remote device (standard IO functions, no IOCTL function) */
  result = rio_create_device_default_remote(io_dev, REMOTE_DEV_ADC_24BIT);
  io_dev->dev_read = adc24bit_remote_read;
  io_dev->dev_ioctl = adc24bit_remote_ioctl;

  return result;
}
