#include <stddef.h>
#include <rinhw/rio.h>
#include <rinhw/rinendian.h>
#include <rinhw/debug.h>
#include <rinhw/adc_up.h>
#include <rinremote/remote.h>
#include "remote_device.h"

#include <rinremote/adc_up_remote.h>


/** Swap byte order for adc data */
static int adc_up_remote_read(struct t_io_dev *dev, void *buf, unsigned short len, unsigned short timeout)
{
  int read_result;
  int nResult = RIO_ERROR;
  int i;
  unsigned short *ptr = (unsigned short *) buf;

  /* Read from remote device */
  read_result = remote_dev_read(dev, buf, len, timeout);

  if ( read_result == len &&
       ( len % sizeof( unsigned short ) ) == 0 )
  {
    for ( i = 0; i < len; i += sizeof( unsigned short ), ptr++ )
    {
      *ptr = ntohs( *ptr );
    }
    nResult = read_result;
  }
  return nResult;
}


/** Function to create and initialise a remote adc device */
int rio_create_device_adc_remote(struct t_io_dev *io_dev)
{
  int result;

  /* Create and initialise a default remote device (standard IO functions, no IOCTL function) */
  result = rio_create_device_default_remote(io_dev, REMOTE_DEV_CPU_ADC);
  io_dev->dev_read = adc_up_remote_read;
  return result;
}
