#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <rinhw/rio.h>
#include <rinhw/buzzer_local.h>
#include <rinhw/rinendian.h>
#include <rinhw/debug.h>
#include <rinhw/buzzer.h>
#include <rinremote/remote.h>
#include "remote_device.h"

#include <rinremote/buzzer_remote.h>


/* Swap byte order for buzzer data, then write */
static int buz_remote_write(struct t_io_dev *dev, const void *buf, unsigned short len, unsigned short timeout)
{
  int i, nResult;
  void *modify_buf = 0;
  tBuzzer *ptr;

  modify_buf = malloc( len );
  if ( 0 == modify_buf )
  {
    return RIO_ERROR;
  }

  memcpy( modify_buf, buf, len );
  ptr = (tBuzzer *) modify_buf;
  if ( ( len % sizeof( tBuzzer ) ) == 0 )
  {
    for ( i = 0; i < len; i += sizeof( tBuzzer ), ptr++ )
    {
      ptr->freq = htons( ptr->freq );
      ptr->time = htons( ptr->time );
    }
    nResult = remote_dev_write(dev, modify_buf, len, timeout);
  }
  else
  {
    nResult = RIO_ERROR;
  }

  free(modify_buf);
  return nResult;
}


/** Function to create and initialise a remote buzzer device */
int rio_create_device_buzzer_remote(struct t_io_dev *io_dev)
{
  int result;

  /* Create and initialise a default remote device (standard IO functions, no IOCTL function) */
  result = rio_create_device_default_remote(io_dev, REMOTE_DEV_BUZZER);
  io_dev->dev_write = buz_remote_write;
  return result;
}
