#include <stdarg.h>
#include <rinhw/rstdio.h>
#include <rinhw/ucos_ii.h>
#include <rinhw/rio.h>
#include <rinhw/debug.h>
#include <rinhw/rinendian.h>
#include <rinhw/iic_local_linux.h>
#include <rinhw/iic.h>
#include <rinremote/remote.h>
#include <rinremote/remote_iic.h>
#include <rinremote/remoteexec.h>
#include "remotehelper.h"
#include "remote_device.h"

#include <rinremote/iic_remote.h>


static int remote_dev_ioctl_iic_set_level( struct t_io_dev *io_dev, int16u ui16IoCtlFn, int8u ui8Level )
{
  int8u txbuf[ GET_SIZE_REMOTE_CMD_IOCTL_IIC_SET_LEVEL() ];
  int8u rxbuf[ GET_SIZE_REMOTE_RESP_IOCTL_DEFAULT() ];
  struct s_rem_device_command *prdCmd = (struct s_rem_device_command *) txbuf;
  struct s_rem_device_response *prdResponse = (struct s_rem_device_response *) rxbuf;
  int nResult;
  const struct s_remote_device_info *remote_info = (struct s_remote_device_info *) io_dev->dev_data;
  const int8u ui8Device = remote_info->device_id;

  vInitRemoteIoCtlCmdHdr( prdCmd, ui8Device, ui16IoCtlFn );
  prdCmd->rem_device_command.rem_dev_cmd_ioctl.remote_ioctl_command.rem_dev_cmd_ioctl_iic_set_level.ui8Level = ui8Level;
  if ( !bRemoteExecuteCommand( prdCmd, GET_SIZE_REMOTE_CMD_IOCTL_IIC_SET_LEVEL(),
                               prdResponse, GET_SIZE_REMOTE_RESP_IOCTL_DEFAULT() ) )
  {
    nResult = RIO_ERROR;
  }
  else
  {
    nResult = nCheckRemoteIoCtlResp_Default( prdResponse, ui8Device, ui16IoCtlFn );
  }
  return nResult;
}


static int remote_dev_ioctl_iic_get_level( struct t_io_dev *io_dev, int16u ui16IoCtlFn, int8u *pui8Level )
{
  int8u txbuf[ GET_SIZE_REMOTE_CMD_IOCTL_IIC_GET_LEVEL() ];
  int8u rxbuf[ GET_SIZE_REMOTE_RESP_IOCTL_IIC_GET_LEVEL() ];
  struct s_rem_device_command *prdCmd = (struct s_rem_device_command *) txbuf;
  struct s_rem_device_response *prdResponse = (struct s_rem_device_response *) rxbuf;
  int nResult;
  const struct s_remote_device_info *remote_info = (struct s_remote_device_info *) io_dev->dev_data;
  const int8u ui8Device = remote_info->device_id;

  vInitRemoteIoCtlCmdHdr( prdCmd, ui8Device, ui16IoCtlFn );
  if ( !bRemoteExecuteCommand( prdCmd, GET_SIZE_REMOTE_CMD_IOCTL_IIC_GET_LEVEL(),
                               prdResponse, GET_SIZE_REMOTE_RESP_IOCTL_IIC_GET_LEVEL() ) )
  {
    nResult = RIO_ERROR;
  }
  else if ( !bCheckRemoteIoCtlRespHdr( prdResponse, ui8Device, ui16IoCtlFn ) )
  {
    nResult = RIO_ERROR;
  }
  else
  {
    nResult = ntohl( prdResponse->rem_device_response.rem_dev_resp_ioctl.remote_ioctl_response.rem_dev_resp_ioctl_iic_get_level.i32Result );
    *pui8Level = prdResponse->rem_device_response.rem_dev_resp_ioctl.remote_ioctl_response.rem_dev_resp_ioctl_iic_get_level.ui8Level;
  }
  return nResult;
}


/*
 * iic_ioctl -
 *
 * Device specific configuration
 */
static int iic_remote_ioctl(struct t_io_dev *io_dev, int func, va_list *arglist)
{
    int result = RIO_SUCCESS;
    unsigned char **byte_ptr_param1;
    unsigned char temp;

#if HW_DEBUG(IIC_DEBUG, HW_DEBUG_BASIC)
    DEBUG_PRINTF("iic_ioctl called, function %d\n", func);
#endif

    /*
     * NOTE: Since character parameters are pushed as words,
     * we'll deal with them as words.
     *
     * Function dispatch
     */
    switch (func)
    {

        /*
         * Set the address of the slave we want to talk to
         */
        case IOCTL_IIC_SET_SLAVE_ADDR:
            temp = (unsigned char) va_arg(*arglist, int);
#if HW_DEBUG(IIC_DEBUG, HW_DEBUG_FULL)
            DEBUG_PRINTF("IOCTL_IIC_SET_SLAVE_ADDR: new slave address 0x%02x\n", temp);
#endif
            result = remote_dev_ioctl_iic_set_level( io_dev, IOCTL_IIC_SET_SLAVE_ADDR, temp );
            break;

        /*
         * Get the address of the slave we are talking to
         */
        case IOCTL_IIC_GET_SLAVE_ADDR:
#if HW_DEBUG(IIC_DEBUG, HW_DEBUG_FULL)
            DEBUG_PRINTF("IOCTL_IIC_GET_SLAVE_ADDR called\n");
#endif
            result = remote_dev_ioctl_iic_get_level( io_dev, IOCTL_IIC_GET_SLAVE_ADDR, &temp );
            byte_ptr_param1 = va_arg(*arglist, unsigned char **);
            **byte_ptr_param1 = temp;
            break;

        /*
         * Set the slot enable mask for the port
         */
        case IOCTL_IIC_SET_SLOT_MASK:
            temp = (unsigned char) va_arg(*arglist, int);
#if HW_DEBUG(IIC_DEBUG, HW_DEBUG_FULL)
            DEBUG_PRINTF("IOCTL_IIC_SET_SLOT_MASK called, mask 0x%02x\n", temp);
#endif
            result = remote_dev_ioctl_iic_set_level( io_dev, IOCTL_IIC_SET_SLOT_MASK, temp );
            break;

        /*
         * Get the slot enable mask for the port
         */
        case IOCTL_IIC_GET_SLOT_MASK:
#if HW_DEBUG(IIC_DEBUG, HW_DEBUG_FULL)
            DEBUG_PRINTF("IOCTL_IIC_GET_SLOT_MASK called\n");
#endif
            result = remote_dev_ioctl_iic_get_level( io_dev, IOCTL_IIC_GET_SLOT_MASK, &temp );
            byte_ptr_param1 = va_arg(*arglist, unsigned char **);
            **byte_ptr_param1 = temp;
            break;

        /*
         * Set the slot enable mask to defaults
         */
        case IOCTL_IIC_DEFAULT_SLOT_MASK:
#if HW_DEBUG(IIC_DEBUG, HW_DEBUG_FULL)
            DEBUG_PRINTF("IOCTL_IIC_DEFAULT_SLOT_MASK called\n");
#endif
            result = remote_dev_ioctl_iic_set_level( io_dev, IOCTL_IIC_DEFAULT_SLOT_MASK, 0 ); /* 0 is dummy param in this case */
            break;

        /*
         * Set the IIC bus speed
         */
        case IOCTL_IIC_SET_SPEED:
            temp = (unsigned char) va_arg(*arglist, int);
#if HW_DEBUG(IIC_DEBUG, HW_DEBUG_FULL)
            DEBUG_PRINTF("IOCTL_IIC_SET_SPEED called, speed %d\n", temp);
#endif
            result = remote_dev_ioctl_iic_set_level( io_dev, IOCTL_IIC_SET_SPEED, temp );
            break;

        /*
         * Unknown option
         */
        default:
#if HW_DEBUG(IIC_DEBUG, HW_DEBUG_FULL)
            DEBUG_PRINTF("Unknown IOCTL for IIC\n");
#endif
            result = RIO_ERROR;
            break;
    }
    return result;
}


/** Function to create and initialise a remote iic device */
int rio_create_device_iic_remote(struct t_io_dev *io_dev, int remote_iic_device_id)
{
  int result;

  /* Create and initialise a default remote device (standard IO functions, no IOCTL function) */
  result = rio_create_device_default_remote(io_dev, remote_iic_device_id);
  io_dev->dev_ioctl = iic_remote_ioctl;
  return result;
}
