#include <string.h>
#include <rinhw/ucos_ii.h>
#include <rinhw/rio.h>
#include <rinhw/rstdio.h>
#include <rinhw/debug.h>
#include <rinhw/rinendian.h>
#include <rinhw/lcd.h>
#include <rinremote/remote.h>
#include <rinremote/remoteexec.h>
#include "remotehelper.h"
#include "remote_device.h"

#include <rinremote/lcd_remote.h>


static int remote_dev_ioctl_lcd_set_backlight( struct t_io_dev *io_dev, int8u ui8Level )
{
  int8u txbuf[ GET_SIZE_REMOTE_CMD_IOCTL_LCD_SET_BACKLIGHT() ];
  int8u rxbuf[ GET_SIZE_REMOTE_RESP_IOCTL_DEFAULT() ];
  struct s_rem_device_command *prdCmd = (struct s_rem_device_command *) txbuf;
  struct s_rem_device_response *prdResponse = (struct s_rem_device_response *) rxbuf;
  int nResult;
  const struct s_remote_device_info *remote_info = (struct s_remote_device_info *) io_dev->dev_data;
  const int8u ui8Device = remote_info->device_id;

  vInitRemoteIoCtlCmdHdr( prdCmd, ui8Device, IOCTL_LCD_SET_BACKLIGHT );
  prdCmd->rem_device_command.rem_dev_cmd_ioctl.remote_ioctl_command.rem_dev_cmd_ioctl_lcd_set_backlight.ui8Level = ui8Level;
  if ( !bRemoteExecuteCommand( prdCmd, GET_SIZE_REMOTE_CMD_IOCTL_LCD_SET_BACKLIGHT(),
                               prdResponse, GET_SIZE_REMOTE_RESP_IOCTL_DEFAULT() ) )
  {
    nResult = RIO_ERROR;
  }
  else
  {
    nResult = nCheckRemoteIoCtlResp_Default( prdResponse, ui8Device, IOCTL_LCD_SET_BACKLIGHT );
  }
  return nResult;
}


static int remote_dev_ioctl_lcd_set_access( struct t_io_dev *io_dev, int16u ui16IoCtlFn )
{
  int8u txbuf[ GET_SIZE_REMOTE_CMD_IOCTL_LCD_SET_ACCESS() ];
  int8u rxbuf[ GET_SIZE_REMOTE_RESP_IOCTL_DEFAULT() ];
  struct s_rem_device_command *prdCmd = (struct s_rem_device_command *) txbuf;
  struct s_rem_device_response *prdResponse = (struct s_rem_device_response *) rxbuf;
  int nResult;
  const struct s_remote_device_info *remote_info = (struct s_remote_device_info *) io_dev->dev_data;
  const int8u ui8Device = remote_info->device_id;

  vInitRemoteIoCtlCmdHdr( prdCmd, ui8Device, ui16IoCtlFn );
  if ( !bRemoteExecuteCommand( prdCmd, GET_SIZE_REMOTE_CMD_IOCTL_LCD_SET_ACCESS(),
                               prdResponse, GET_SIZE_REMOTE_RESP_IOCTL_DEFAULT() ) )
  {
    nResult = RIO_ERROR;
  }
  else
  {
    nResult = nCheckRemoteIoCtlResp_Default( prdResponse, ui8Device, ui16IoCtlFn );
  }
  return nResult;
}


/** LCD Device specific configuration */
static int lcd_remote_ioctl(struct t_io_dev *io_dev, int func, va_list *arglist)
{
  unsigned char level;
  int result = RIO_SUCCESS;

#if HW_DEBUG(LCD_DEBUG, HW_DEBUG_BASIC)
  DEBUG_PRINTF("lcd_remote_ioctl called\n");
#endif

  /* Function dispatch */
  switch (func)
  {

    /* Set the backlight level */
    case IOCTL_LCD_SET_BACKLIGHT:
      level = (unsigned char) va_arg(*arglist, int);
#if HW_DEBUG(LCD_DEBUG, HW_DEBUG_FULL)
      DEBUG_PRINTF("IOCTL_LCD_SET_BACKLIGHT: level %d\n", level);
#endif
      result = remote_dev_ioctl_lcd_set_backlight(io_dev, level);
      break;

    /* Change the read and write buffer access */
    case IOCTL_LCD_ACCESS_DATA:
    case IOCTL_LCD_ACCESS_FLASH:
    case IOCTL_LCD_ACCESS_OUTPUT:
    case IOCTL_LCD_ACCESS_CHAR:
      result = remote_dev_ioctl_lcd_set_access(io_dev, func);
      break;

    /* Error */
    default:
      result = RIO_ERROR;
      break;
  }

  /* Done, return result */
  return result;
}


/** Function to create and initialise a remote LCD device */
int rio_create_device_lcd_remote(struct t_io_dev *io_dev)
{
  int result;

  /* Create and initialise a default remote device (standard IO functions, no IOCTL function) */
  result = rio_create_device_default_remote(io_dev, REMOTE_DEV_LCD);
  io_dev->dev_ioctl = lcd_remote_ioctl;
  return result;
}
