#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <rinhw/rio.h>
#include <rinhw/rinendian.h>
#include <rinhw/rtc_local.h>
#include <rinhw/debug.h>
#include <rinhw/rtc.h>
#include <rinremote/remote.h>
#include "remote_device.h"

#include <rinremote/rtc_remote.h>


/** Swap byte order for rtc data */
static int rtc_remote_read(struct t_io_dev *dev, void *buf, unsigned short len, unsigned short timeout)
{
  int read_result;
  int nResult = RIO_ERROR;
  int i;
  t_datetime *ptr = (t_datetime *) buf;

  /* Read from remote device */
  read_result = remote_dev_read(dev, buf, len, timeout);

  if ( read_result == len &&
       ( len % sizeof( t_datetime ) ) == 0 )
  {
    for ( i = 0; i < len; i += sizeof( t_datetime ), ptr++ )
    {
      ptr->year = ntohs( ptr->year );
    }
    nResult = read_result;
  }
  return nResult;
}


/** Swap byte order for rtc data */
static int rtc_remote_write(struct t_io_dev *dev, const void *buf, unsigned short len, unsigned short timeout)
{
  int nResult = RIO_ERROR;
  int i;
  void *modify_buf = 0;
  t_datetime *ptr;

  modify_buf = malloc( len );
  if ( 0 == modify_buf )
  {
    return RIO_ERROR;
  }

  memcpy( modify_buf, buf, len );
  ptr = (t_datetime *) modify_buf;
  if ( ( len % sizeof( t_datetime ) ) == 0 )
  {
    for ( i = 0; i < len; i += sizeof( t_datetime ), ptr++ )
    {
      ptr->year = ntohs( ptr->year );
    }
    nResult = remote_dev_write(dev, modify_buf, len, timeout);
  }

  free(modify_buf);
  return nResult;
}


/** Function to create and initialise a remote RTC device */
int rio_create_device_rtc_remote(struct t_io_dev *io_dev)
{
  int result;

  /* Create and initialise a default remote device (standard IO functions, no IOCTL function) */
  result = rio_create_device_default_remote(io_dev, REMOTE_DEV_RTC);
  io_dev->dev_write = rtc_remote_write;
  io_dev->dev_read = rtc_remote_read;
  return result;
}
