#include <stdarg.h>
#include <rinhw/rstdio.h>
#include <rinhw/ucos_ii.h>
#include <rinhw/rio.h>
#include <rinhw/debug.h>
#include <rinhw/rinendian.h>
#include <rinhw/sio_local_comport.h>
#include <rinhw/sio_local_file.h>
#include <rinhw/sio_local_socket.h>
#include <rinhw/sio.h>
#include "remote_device.h"
#include <rinremote/remote.h>
#include <rinremote/remoteexec.h>
#include "remotehelper.h"

#include <rinremote/sio_remote.h>


static int remote_dev_ioctl_sio_set_config( struct t_io_dev *io_dev, const t_ioctl_siocfg *siocfg_ptr )
{
  int8u txbuf[ GET_SIZE_REMOTE_CMD_IOCTL_SIO_SET_CONFIG() ];
  int8u rxbuf[ GET_SIZE_REMOTE_RESP_IOCTL_DEFAULT() ];
  struct s_rem_device_command *prdCmd = (struct s_rem_device_command *) txbuf;
  struct s_rem_device_response *prdResponse = (struct s_rem_device_response *) rxbuf;
  int nResult;
  const struct s_remote_device_info *remote_info = (struct s_remote_device_info *) io_dev->dev_data;
  const int8u ui8Device = remote_info->device_id;
  t_ioctl_siocfg *pOutgoingSiocfg = &prdCmd->rem_device_command.rem_dev_cmd_ioctl.remote_ioctl_command.rem_dev_cmd_ioctl_sio_set_config.sio_config;

  vInitRemoteIoCtlCmdHdr( prdCmd, ui8Device, IOCTL_SER_SET_CONFIG );
  *pOutgoingSiocfg = *siocfg_ptr;
  pOutgoingSiocfg->baud = htonl( pOutgoingSiocfg->baud );

  if ( !bRemoteExecuteCommand( prdCmd, GET_SIZE_REMOTE_CMD_IOCTL_SIO_SET_CONFIG(),
                               prdResponse, GET_SIZE_REMOTE_RESP_IOCTL_DEFAULT() ) )
  {
    nResult = RIO_ERROR;
  }
  else
  {
    nResult = nCheckRemoteIoCtlResp_Default( prdResponse, ui8Device, IOCTL_SER_SET_CONFIG );
  }
  return nResult;
}


static int remote_dev_ioctl_sio_get_config( struct t_io_dev *io_dev, t_ioctl_siocfg *siocfg_ptr )
{
  int8u txbuf[ GET_SIZE_REMOTE_CMD_IOCTL_SIO_GET_CONFIG() ];
  int8u rxbuf[ GET_SIZE_REMOTE_RESP_IOCTL_SIO_GET_CONFIG() ];
  struct s_rem_device_command *prdCmd = (struct s_rem_device_command *) txbuf;
  struct s_rem_device_response *prdResponse = (struct s_rem_device_response *) rxbuf;
  int nResult;
  const struct s_remote_device_info *remote_info = (struct s_remote_device_info *) io_dev->dev_data;
  const int8u ui8Device = remote_info->device_id;
  t_ioctl_siocfg *pIncomingSiocfg = &prdResponse->rem_device_response.rem_dev_resp_ioctl.remote_ioctl_response.rem_dev_resp_ioctl_sio_get_config.sio_config;

  vInitRemoteIoCtlCmdHdr( prdCmd, ui8Device, IOCTL_SER_GET_CONFIG );

  if ( !bRemoteExecuteCommand( prdCmd, GET_SIZE_REMOTE_CMD_IOCTL_SIO_GET_CONFIG(),
                               prdResponse, GET_SIZE_REMOTE_RESP_IOCTL_SIO_GET_CONFIG() ) )
  {
    nResult = RIO_ERROR;
  }
  else if ( !bCheckRemoteIoCtlRespHdr( prdResponse, ui8Device, IOCTL_SER_GET_CONFIG ) )
  {
    nResult = RIO_ERROR;
  }
  else
  {
    nResult = ntohl( prdResponse->rem_device_response.rem_dev_resp_ioctl.remote_ioctl_response.rem_dev_resp_ioctl_sio_get_config.i32Result );
    pIncomingSiocfg->baud = ntohl( pIncomingSiocfg->baud );
    *siocfg_ptr = *pIncomingSiocfg;
  }
  return nResult;
}


static int remote_dev_ioctl_sio_get_level( struct t_io_dev *io_dev, int16u ui16IoCtlFn, int32u *pui32Level )
{
  int8u txbuf[ GET_SIZE_REMOTE_CMD_IOCTL_SIO_GET_LEVEL() ];
  int8u rxbuf[ GET_SIZE_REMOTE_RESP_IOCTL_SIO_GET_LEVEL() ];
  struct s_rem_device_command *prdCmd = (struct s_rem_device_command *) txbuf;
  struct s_rem_device_response *prdResponse = (struct s_rem_device_response *) rxbuf;
  int nResult;
  const struct s_remote_device_info *remote_info = (struct s_remote_device_info *) io_dev->dev_data;
  const int8u ui8Device = remote_info->device_id;

  vInitRemoteIoCtlCmdHdr( prdCmd, ui8Device, ui16IoCtlFn );
  if ( !bRemoteExecuteCommand( prdCmd, GET_SIZE_REMOTE_CMD_IOCTL_SIO_GET_LEVEL(),
                               prdResponse, GET_SIZE_REMOTE_RESP_IOCTL_SIO_GET_LEVEL() ) )
  {
    nResult = RIO_ERROR;
  }
  else if ( !bCheckRemoteIoCtlRespHdr( prdResponse, ui8Device, ui16IoCtlFn ) )
  {
    nResult = RIO_ERROR;
  }
  else
  {
    nResult = ntohl( prdResponse->rem_device_response.rem_dev_resp_ioctl.remote_ioctl_response.rem_dev_resp_ioctl_sio_get_level.i32Result );
    *pui32Level = ntohl( prdResponse->rem_device_response.rem_dev_resp_ioctl.remote_ioctl_response.rem_dev_resp_ioctl_sio_get_level.ui32Level );
  }
  return nResult;
}


static int remote_dev_ioctl_sio_set_ring_network( struct t_io_dev *io_dev, int8u ui8Enable )
{
  int8u txbuf[ GET_SIZE_REMOTE_CMD_IOCTL_SIO_SET_RING_NETWORK() ];
  int8u rxbuf[ GET_SIZE_REMOTE_RESP_IOCTL_DEFAULT() ];
  struct s_rem_device_command *prdCmd = (struct s_rem_device_command *) txbuf;
  struct s_rem_device_response *prdResponse = (struct s_rem_device_response *) rxbuf;
  int nResult;
  const struct s_remote_device_info *remote_info = (struct s_remote_device_info *) io_dev->dev_data;
  const int8u ui8Device = remote_info->device_id;

  vInitRemoteIoCtlCmdHdr( prdCmd, ui8Device, IOCTL_SER_SET_RING_NETWORK );
  prdCmd->rem_device_command.rem_dev_cmd_ioctl.remote_ioctl_command.rem_dev_cmd_ioctl_sio_set_ring_network.ui8Enabled = ui8Enable;
  if ( !bRemoteExecuteCommand( prdCmd, GET_SIZE_REMOTE_CMD_IOCTL_SIO_SET_RING_NETWORK(),
                               prdResponse, GET_SIZE_REMOTE_RESP_IOCTL_DEFAULT() ) )
  {
    nResult = RIO_ERROR;
  }
  else
  {
    nResult = nCheckRemoteIoCtlResp_Default( prdResponse, ui8Device, IOCTL_SER_SET_RING_NETWORK );
  }
  return nResult;
}


/*
 * sio_ioctl -
 *
 * Configure etc the serial port
 */
static int sio_remote_ioctl(struct t_io_dev *io_dev,  int func, va_list *arglist)
{
    t_ioctl_siocfg *siocfg_ptr;
    int result = RIO_SUCCESS;
    int32u ui32Level;

#if HW_DEBUG(SIO_DEBUG, HW_DEBUG_BASIC)
    DEBUG_PRINTF("sio_ioctl called\n");
#endif

    /*
     * IOCTL function dispatch
     */
    switch (func)
    {
        case IOCTL_SER_SET_CONFIG:
            siocfg_ptr = va_arg(*arglist, t_ioctl_siocfg *);
#if HW_DEBUG(SIO_DEBUG, HW_DEBUG_BASIC)
            DEBUG_PRINTF("IOCTL_SER_SET_CONFIG, siocfg_ptr = %p\n", siocfg_ptr);
#endif
            result = remote_dev_ioctl_sio_set_config( io_dev, siocfg_ptr );
            break;

        case IOCTL_SER_GET_CONFIG:
            siocfg_ptr = va_arg(*arglist, t_ioctl_siocfg *);
#if HW_DEBUG(SIO_DEBUG, HW_DEBUG_BASIC)
            DEBUG_PRINTF("IOCTL_SER_GET_CONFIG, siocfg_ptr = %p\n", siocfg_ptr);
#endif
            result = remote_dev_ioctl_sio_get_config( io_dev, siocfg_ptr );
            break;

        /*
         * Get buffer levels and sizes
         */
        case IOCTL_SER_GET_RX_BUF_LEVEL:
        case IOCTL_SER_GET_TX_BUF_LEVEL:
        case IOCTL_SER_GET_RX_BUF_SIZE:
        case IOCTL_SER_GET_TX_BUF_SIZE:
            result = remote_dev_ioctl_sio_get_level( io_dev, func, &ui32Level );
            *va_arg(*arglist, unsigned short *) = (unsigned short) ui32Level;
            break;

        /*
         * Get the level of the cts pin
         */
        case IOCTL_SER_GET_CTS_STATE:
            result = remote_dev_ioctl_sio_get_level( io_dev, IOCTL_SER_GET_CTS_STATE, &ui32Level );
            *va_arg(*arglist, unsigned char *) = (unsigned char) ui32Level;
            break;

        /*
         * Get the number of receive errors on this port
         */
        case IOCTL_SER_GET_RX_ERR_COUNT:
            result = remote_dev_ioctl_sio_get_level( io_dev, IOCTL_SER_GET_RX_ERR_COUNT, &ui32Level );
            *va_arg(*arglist, unsigned long *) = ui32Level;
            break;

        /*
         * Enable/disable ring network
         */
        case IOCTL_SER_SET_RING_NETWORK:
            result = remote_dev_ioctl_sio_set_ring_network( io_dev, (unsigned char) va_arg(*arglist, int) );
            break;

        /*
         * Unknown function
         */
        default:
            result = RIO_ERROR;
            break;
    }
    return result;
}


/** Function to create and initialise a remote serial port device */
int rio_create_device_sio_remote(struct t_io_dev *io_dev, int remote_serial_device_id)
{
  int result;

  /* Create and initialise a default remote device (standard IO functions, no IOCTL function) */
  result = rio_create_device_default_remote(io_dev, remote_serial_device_id);
  io_dev->dev_ioctl = sio_remote_ioctl;
  return result;
}
