/*
 * remote.c
 *
 *  Created on: 22/10/2008
 *      Author: ianb
 */
#include <string.h>
#include <stdlib.h>
#include <rinhw/rio.h>
#include <rinhw/rinendian.h>
#include <rinhw/debug.h>
#include <rinremote/remoteexec.h>
#include "remotehelper.h"
#include "remote_device.h"

#include <rinremote/remote.h>


int remote_dev_init( struct t_io_dev *io_dev )
{
  int8u txbuf[ GET_SIZE_REMOTE_CMD_INIT() ];
  int8u rxbuf[ GET_SIZE_REMOTE_RESP_INIT() ];
  struct s_rem_device_command *prdCmd = (struct s_rem_device_command *) txbuf;
  struct s_rem_device_response *prdResponse = (struct s_rem_device_response *) rxbuf;
  int nResult;
  const struct s_remote_device_info *remote_info = (struct s_remote_device_info *) io_dev->dev_data;
  const int8u ui8Device = remote_info->device_id;

  vInitRemoteCmdHdr( prdCmd, REMOTE_CMD_INIT, ui8Device );
  prdCmd->rem_device_command.rem_dev_cmd_init.ui8Unused = 0;
  if ( !bRemoteExecuteCommand( prdCmd, GET_SIZE_REMOTE_CMD_INIT(), prdResponse, GET_SIZE_REMOTE_RESP_INIT() ) )
  {
    nResult = RIO_ERROR;
  }
  else
  {
    nResult = nCheckRemoteResp_Default( prdResponse, REMOTE_CMD_INIT, ui8Device );
  }
  return nResult;
}


int remote_dev_destroy( struct t_io_dev *io_dev )
{
  int8u txbuf[ GET_SIZE_REMOTE_CMD_DESTROY() ];
  int8u rxbuf[ GET_SIZE_REMOTE_RESP_DESTROY() ];
  struct s_rem_device_command *prdCmd = (struct s_rem_device_command *) txbuf;
  struct s_rem_device_response *prdResponse = (struct s_rem_device_response *) rxbuf;
  int nResult;
  struct s_remote_device_info *remote_info = (struct s_remote_device_info *) io_dev->dev_data;
  const int8u ui8Device = remote_info->device_id;

  vInitRemoteCmdHdr( prdCmd, REMOTE_CMD_DESTROY, ui8Device );
  prdCmd->rem_device_command.rem_dev_cmd_destroy.ui8Unused = 0;
  if ( !bRemoteExecuteCommand( prdCmd, GET_SIZE_REMOTE_CMD_DESTROY(), prdResponse, GET_SIZE_REMOTE_RESP_DESTROY() ) )
  {
    nResult = RIO_ERROR;
  }
  else
  {
    nResult = nCheckRemoteResp_Default( prdResponse, REMOTE_CMD_DESTROY, ui8Device );
  }

  /* Free stuff */
  io_dev->dev_data = NULL;
  free(remote_info);

  return nResult;
}


int remote_dev_open( struct t_io_dev *io_dev, unsigned short timeout )
{
  int8u txbuf[ GET_SIZE_REMOTE_CMD_OPEN() ];
  int8u rxbuf[ GET_SIZE_REMOTE_RESP_OPEN() ];
  struct s_rem_device_command *prdCmd = (struct s_rem_device_command *) txbuf;
  struct s_rem_device_response *prdResponse = (struct s_rem_device_response *) rxbuf;
  int nResult;
  const struct s_remote_device_info *remote_info = (struct s_remote_device_info *) io_dev->dev_data;
  const int8u ui8Device = remote_info->device_id;

  vInitRemoteCmdHdr( prdCmd, REMOTE_CMD_OPEN, ui8Device );
  prdCmd->rem_device_command.rem_dev_cmd_open.ui16Timeout = htons( timeout );
  if ( !bRemoteExecuteCommand( prdCmd, GET_SIZE_REMOTE_CMD_OPEN(), prdResponse, GET_SIZE_REMOTE_RESP_OPEN() ) )
  {
    nResult = RIO_ERROR;
  }
  else
  {
    nResult = nCheckRemoteResp_Default( prdResponse, REMOTE_CMD_OPEN, ui8Device );
  }
  return nResult;
}


int remote_dev_close( struct t_io_dev *io_dev )
{
  int8u txbuf[ GET_SIZE_REMOTE_CMD_CLOSE() ];
  int8u rxbuf[ GET_SIZE_REMOTE_RESP_CLOSE() ];
  struct s_rem_device_command *prdCmd = (struct s_rem_device_command *) txbuf;
  struct s_rem_device_response *prdResponse = (struct s_rem_device_response *) rxbuf;
  int nResult;
  const struct s_remote_device_info *remote_info = (struct s_remote_device_info *) io_dev->dev_data;
  const int8u ui8Device = remote_info->device_id;

  vInitRemoteCmdHdr( prdCmd, REMOTE_CMD_CLOSE, ui8Device );
  prdCmd->rem_device_command.rem_dev_cmd_close.ui8Unused = 0;
  if ( !bRemoteExecuteCommand( prdCmd, GET_SIZE_REMOTE_CMD_CLOSE(), prdResponse, GET_SIZE_REMOTE_RESP_CLOSE() ) )
  {
    nResult = RIO_ERROR;
  }
  else
  {
    nResult = nCheckRemoteResp_Default( prdResponse, REMOTE_CMD_CLOSE, ui8Device );
  }
  return nResult;
}


#ifndef min
#define min(a, b) ( (a < b) ? a : b )
#endif


int remote_dev_read(struct t_io_dev *io_dev, void *buf, unsigned short len, unsigned short timeout)
{
  char *pBuf = (char *) buf;
  int8u txbuf[ GET_SIZE_REMOTE_CMD_READ() ];
  int8u rxbuf[ GET_SIZE_REMOTE_RESP_READ( MAX_REMOTE_BUFFER ) ];
  struct s_rem_device_command *prdCmd = (struct s_rem_device_command *) txbuf;
  struct s_rem_device_response *prdResponse = (struct s_rem_device_response *) rxbuf;
  int nResult = 0;
  const struct s_remote_device_info *remote_info = (struct s_remote_device_info *) io_dev->dev_data;
  const int8u ui8Device = remote_info->device_id;
  int nCount = 0;
  int nReadThisTime;

  while ( nCount < len )
  {
    vInitRemoteCmdHdr( prdCmd, REMOTE_CMD_READ, ui8Device );
    prdCmd->rem_device_command.rem_dev_cmd_read.ui16Timeout = htons( timeout );
    nReadThisTime = min( len, MAX_REMOTE_BUFFER );
    prdCmd->rem_device_command.rem_dev_cmd_read.ui16Length = htons( nReadThisTime );

    if ( !bRemoteExecuteCommand( prdCmd, GET_SIZE_REMOTE_CMD_READ(), prdResponse, GET_SIZE_REMOTE_RESP_READ( nReadThisTime ) ) )
    {
      nResult = RIO_ERROR;
      break;
    }
    else if ( REMOTE_CMD_READ != prdResponse->rem_dev_cmd_header.ui8Command ||
              ui8Device != prdResponse->rem_dev_cmd_header.ui8Device ||
              RIO_ERROR == (int32s) ntohl( prdResponse->rem_device_response.rem_dev_resp_read.i32Result ) ||
              0 > (int32s) ntohl( prdResponse->rem_device_response.rem_dev_resp_read.i32Result ) )
    {
      nResult = RIO_ERROR;
      break;
    }
    else
    {
      nReadThisTime = ntohl( prdResponse->rem_device_response.rem_dev_resp_read.i32Result );
      memcpy( &pBuf[ nCount ], prdResponse->rem_device_response.rem_dev_resp_read.ui8Buffer, nReadThisTime );
      nCount += nReadThisTime;
      nResult += nReadThisTime;
    }
  }
  return nResult;
}


int remote_dev_write(struct t_io_dev *io_dev, const void *buf, unsigned short len, unsigned short timeout)
{
  const char *pBuf = (const char *) buf;
  int8u txbuf[ GET_SIZE_REMOTE_CMD_WRITE( MAX_REMOTE_BUFFER ) ];
  int8u rxbuf[ GET_SIZE_REMOTE_RESP_WRITE() ];
  struct s_rem_device_command *prdCmd = (struct s_rem_device_command *) txbuf;
  struct s_rem_device_response *prdResponse = (struct s_rem_device_response *) rxbuf;
  int nResult;
  const struct s_remote_device_info *remote_info = (struct s_remote_device_info *) io_dev->dev_data;
  const int8u ui8Device = remote_info->device_id;
  int nCount = 0;
  int nWriteThisTime;

#if HW_DEBUG(REMOTE_IO_DEBUG, HW_DEBUG_FULL)
  DEBUG_PRINTF("remote_dev_write() called, dev %d, buf %p, len %d, timeout %d\n", ui8Device, buf, len, timeout);
#endif

  while ( nCount < len )
  {
    vInitRemoteCmdHdr( prdCmd, REMOTE_CMD_WRITE, ui8Device );
    prdCmd->rem_device_command.rem_dev_cmd_write.ui16Timeout = htons( timeout );
    nWriteThisTime = min( len - nCount, MAX_REMOTE_BUFFER );
    prdCmd->rem_device_command.rem_dev_cmd_write.ui16Length = htons( nWriteThisTime );
    memcpy( prdCmd->rem_device_command.rem_dev_cmd_write.ui8Buffer, &pBuf[ nCount ], nWriteThisTime );

#if HW_DEBUG(REMOTE_IO_DEBUG, HW_DEBUG_FULL)
    DEBUG_PRINTF("remote_dev_write: bRemoteExecuteCommand() remote write of %d bytes\n", nWriteThisTime);
#endif
    if ( !bRemoteExecuteCommand( prdCmd, GET_SIZE_REMOTE_CMD_WRITE( nWriteThisTime ), prdResponse, GET_SIZE_REMOTE_RESP_WRITE() ) )
    {
#if HW_DEBUG(REMOTE_IO_DEBUG, HW_DEBUG_FULL)
      DEBUG_PRINTF("remote_dev_write: bRemoteExecuteCommand() failed\n");
#endif
      nResult = RIO_ERROR;
    }
    else
    {
      nResult = nCheckRemoteResp_Default( prdResponse, REMOTE_CMD_WRITE, ui8Device );
#if HW_DEBUG(REMOTE_IO_DEBUG, HW_DEBUG_FULL)
      DEBUG_PRINTF("remote_dev_write: Response check result %d\n", nResult);
#endif
    }

    if ( RIO_ERROR == nResult || 0 > nResult )
    {
#if HW_DEBUG(REMOTE_IO_DEBUG, HW_DEBUG_FULL)
      DEBUG_PRINTF("remote_dev_write: Fail, returning %d\n", nResult);
#endif
      return nResult;
    }
    else
    {
      nWriteThisTime = ntohl( prdResponse->rem_device_response.rem_dev_resp_read.i32Result );
      nCount += nWriteThisTime;
#if HW_DEBUG(REMOTE_IO_DEBUG, HW_DEBUG_FULL)
      DEBUG_PRINTF("remote_dev_write: bRemoteExecuteCommand() success, result %d, written so far %d\n", nWriteThisTime, nCount);
#endif
    }
  }
  nResult = nCount;
#if HW_DEBUG(REMOTE_IO_DEBUG, HW_DEBUG_FULL)
  DEBUG_PRINTF("remote_dev_write: Success, returning %d\n", nResult);
#endif
  return nResult;
}


int remote_dev_isatty(struct t_io_dev *io_dev)
{
  int8u txbuf[ GET_SIZE_REMOTE_CMD_ISATTY() ];
  int8u rxbuf[ GET_SIZE_REMOTE_RESP_ISATTY() ];
  struct s_rem_device_command *prdCmd = (struct s_rem_device_command *) txbuf;
  struct s_rem_device_response *prdResponse = (struct s_rem_device_response *) rxbuf;
  int nResult;
  const struct s_remote_device_info *remote_info = (struct s_remote_device_info *) io_dev->dev_data;
  const int8u ui8Device = remote_info->device_id;

  vInitRemoteCmdHdr( prdCmd, REMOTE_CMD_ISATTY, ui8Device );
  if ( !bRemoteExecuteCommand( prdCmd, GET_SIZE_REMOTE_CMD_ISATTY(), prdResponse, GET_SIZE_REMOTE_RESP_ISATTY() ) )
  {
    nResult = RIO_ERROR;
  }
  else
  {
    nResult = nCheckRemoteResp_Default( prdResponse, REMOTE_CMD_ISATTY, ui8Device );
  }
  return nResult;
}


long remote_dev_filelength(struct t_io_dev *io_dev)
{
  int8u txbuf[ GET_SIZE_REMOTE_CMD_FILELENGTH() ];
  int8u rxbuf[ GET_SIZE_REMOTE_RESP_FILELENGTH() ];
  struct s_rem_device_command *prdCmd = (struct s_rem_device_command *) txbuf;
  struct s_rem_device_response *prdResponse = (struct s_rem_device_response *) rxbuf;
  long nResult;
  const struct s_remote_device_info *remote_info = (struct s_remote_device_info *) io_dev->dev_data;
  const int8u ui8Device = remote_info->device_id;

  vInitRemoteCmdHdr( prdCmd, REMOTE_CMD_FILELENGTH, ui8Device );
  if ( !bRemoteExecuteCommand( prdCmd, GET_SIZE_REMOTE_CMD_FILELENGTH(), prdResponse, GET_SIZE_REMOTE_RESP_FILELENGTH() ) )
  {
    nResult = RIO_ERROR;
  }
  else
  {
    nResult = nCheckRemoteResp_Default( prdResponse, REMOTE_CMD_FILELENGTH, ui8Device );
  }
  return nResult;
}


long remote_dev_lseek(struct t_io_dev *io_dev, long offset, int fromwhere)
{
  int8u txbuf[ GET_SIZE_REMOTE_CMD_LSEEK() ];
  int8u rxbuf[ GET_SIZE_REMOTE_RESP_LSEEK() ];
  struct s_rem_device_command *prdCmd = (struct s_rem_device_command *) txbuf;
  struct s_rem_device_response *prdResponse = (struct s_rem_device_response *) rxbuf;
  long nResult;
  const struct s_remote_device_info *remote_info = (struct s_remote_device_info *) io_dev->dev_data;
  const int8u ui8Device = remote_info->device_id;

  vInitRemoteCmdHdr( prdCmd, REMOTE_CMD_LSEEK, ui8Device );
  prdCmd->rem_device_command.rem_dev_cmd_lseek.i32Offset = htonl( offset );
  prdCmd->rem_device_command.rem_dev_cmd_lseek.i32Fromwhere = htonl( fromwhere );

  if ( !bRemoteExecuteCommand( prdCmd, GET_SIZE_REMOTE_CMD_LSEEK(), prdResponse, GET_SIZE_REMOTE_RESP_LSEEK() ) )
  {
    nResult = RIO_ERROR;
  }
  else
  {
    nResult = nCheckRemoteResp_Default( prdResponse, REMOTE_CMD_LSEEK, ui8Device );
  }
  return nResult;
}


long remote_dev_tell(struct t_io_dev *io_dev)
{
  int8u txbuf[ GET_SIZE_REMOTE_CMD_TELL() ];
  int8u rxbuf[ GET_SIZE_REMOTE_RESP_TELL() ];
  struct s_rem_device_command *prdCmd = (struct s_rem_device_command *) txbuf;
  struct s_rem_device_response *prdResponse = (struct s_rem_device_response *) rxbuf;
  long nResult;
  const struct s_remote_device_info *remote_info = (struct s_remote_device_info *) io_dev->dev_data;
  const int8u ui8Device = remote_info->device_id;

  vInitRemoteCmdHdr( prdCmd, REMOTE_CMD_TELL, ui8Device );
  if ( !bRemoteExecuteCommand( prdCmd, GET_SIZE_REMOTE_CMD_TELL(), prdResponse, GET_SIZE_REMOTE_RESP_TELL() ) )
  {
    nResult = RIO_ERROR;
  }
  else
  {
    nResult = nCheckRemoteResp_Default( prdResponse, REMOTE_CMD_TELL, ui8Device );
  }
  return nResult;
}
