/*
 * remote_device.c
 *
 *  Created on: 24/09/2010
 *      Author: ian
 */
#include <stdio.h>
#include <stdlib.h>
#include <rinhw/rio.h>
#include <rinremote/remote.h>

#include "remote_device.h"


/** Function to create and initialise a default remote device (standard IO functions, no IOCTL function) */
int rio_create_device_default_remote(struct t_io_dev *io_dev, int remote_device_id)
{
  struct s_remote_device_info *remote_device = NULL;

  /* Allocate structure */
  remote_device = malloc(sizeof(struct s_remote_device_info));
  if (NULL == remote_device)
  {
    goto rcddr_fail;
  }

  /* Fill out members */
  remote_device->device_id = remote_device_id;

  /*
   * Setup device data pointer to point to the serial port
   * and initialise the status and position
   */
  io_dev->dev_data = (void *) remote_device;
  io_dev->status = io_dev->posn = 0UL;
  io_dev->sem_open_excl = NULL;

  /* Setup function pointers */
  io_dev->dev_init = remote_dev_init;
  io_dev->dev_open = remote_dev_open;
  io_dev->dev_close = remote_dev_close;
  io_dev->dev_read = remote_dev_read;
  io_dev->dev_write = remote_dev_write;
  io_dev->dev_filelength = remote_dev_filelength;
  io_dev->dev_ioctl = NULL;
  io_dev->dev_destroy = remote_dev_destroy; /* This frees io_dev->dev_data */
  io_dev->dev_isatty = remote_dev_isatty;
  io_dev->dev_tell = remote_dev_tell;
  io_dev->dev_lseek = remote_dev_lseek;

  /* Register and initialise the device */
  if (rregister_io(io_dev) < 0)
  {
    goto rcddr_fail;
  }

  /* Success */
  return RIO_SUCCESS;

rcddr_fail:
  io_dev->dev_data = NULL;
  if (NULL != remote_device)
  {
    free(remote_device);
  }
  return RIO_ERROR;
}
