/*
 * remoteexec.c
 *
 *  Created on: 23/10/2008
 *      Author: ianb
 */
#include <stddef.h>
#include <rinhw/ucos_ii.h>
#include <rinhw/debug.h>
#include <rinremote/remote.h>
#include <rinremote/slip.h>
#include <rinremote/slip_io.h>

#include <rinremote/remoteexec.h>


static OS_EVENT *semRemoteCommandLock = 0;
static bool bRemoteCommandsEnabled = false;


void vRemoteExecuteInit( void *data )
{
#if HW_DEBUG(REMOTE_IO_DEBUG, HW_DEBUG_BASIC)
  DEBUG_PRINTF("vRemoteExecuteInit() called\n");
#endif

  bRemoteCommandsEnabled = bHwLibXMLConfig_GetRemoteIoEnabled();
#if HW_DEBUG(REMOTE_IO_DEBUG, HW_DEBUG_FULL)
  DEBUG_PRINTF("Remote commands enabled: %d\n", bRemoteCommandsEnabled);
#endif
  if (!bRemoteCommandsEnabled)
  {
    return;
  }

#if HW_DEBUG(REMOTE_IO_DEBUG, HW_DEBUG_FULL)
  DEBUG_PRINTF("Open serial port\n");
#endif
  slip_serial_init();

#if HW_DEBUG(REMOTE_IO_DEBUG, HW_DEBUG_FULL)
  DEBUG_PRINTF("Create sem\n");
#endif

  /*
   * Create the open exclusive semaphore
   */
  semRemoteCommandLock = OSSemCreate(1);
  if ( semRemoteCommandLock == NULL )
  {
#if HW_DEBUG(REMOTE_IO_DEBUG, HW_DEBUG_BASIC)
      DEBUG_PRINTF("Can't create remote serial port excl sem\n");
#endif
  }

#if HW_DEBUG(REMOTE_IO_DEBUG, HW_DEBUG_BASIC)
  DEBUG_PRINTF("vRemoteExecuteInit() complete\n");
#endif
}


bool bRemoteExecuteCommand( const struct s_rem_device_command *pCommand, int nCmdLength,
                            struct s_rem_device_response *pResponse, int nRespLength )
{
  const char *pCmdBuf = (const char *) pCommand;
  char *pRespBuf = (char *) pResponse;
  int8u ui8Err;

  /* Check that remote commands are enabled */
  if ( !bRemoteCommandsEnabled )
  {
#if HW_DEBUG(REMOTE_IO_DEBUG, HW_DEBUG_FULL)
    DEBUG_PRINTF("bRemoteExecuteCommand: Remote commands not enabled, fail\n");
#endif
    return false;
  }

  OSSemPend( semRemoteCommandLock, 0, &ui8Err );
  if ( OS_NO_ERR == ui8Err )
  {
    slip_send_packet( (const int8u *) pCmdBuf, nCmdLength );
    slip_recv_packet( (int8u *) pRespBuf, nRespLength );
    OSSemPost( semRemoteCommandLock );
  }

  return ( OS_NO_ERR == ui8Err );
}
