/*
 * remotehelper.c
 *
 *  Created on: 24/10/2008
 *      Author: ianb
 */
#include <rinhw/rinendian.h>
#include "remotehelper.h"


void vInitRemoteCmdHdr( struct s_rem_device_command *prdCmd, int8u ui8Cmd, int8u ui8Device )
{
  prdCmd->rem_dev_cmd_header.ui8Command = ui8Cmd;
  prdCmd->rem_dev_cmd_header.ui8Device = ui8Device;
}


void vInitRemoteRespHdr( struct s_rem_device_response *prdResponse, int8u ui8Cmd, int8u ui8Device )
{
  prdResponse->rem_dev_cmd_header.ui8Command = ui8Cmd;
  prdResponse->rem_dev_cmd_header.ui8Device = ui8Device;
}


void vInitRemoteResp_Default( struct s_rem_device_response *prdResponse, int8u ui8Cmd, int8u ui8Device, int32s i32Result )
{
  vInitRemoteRespHdr( prdResponse, ui8Cmd, ui8Device );
  prdResponse->rem_device_response.rem_dev_resp_default.i32Result = htonl( i32Result );
}


void vInitRemoteIoCtlCmdHdr( struct s_rem_device_command *prdCmd, int8u ui8Device, int16u ui16IoCtlFn )
{
  vInitRemoteCmdHdr( prdCmd, REMOTE_CMD_IOCTL, ui8Device );
  prdCmd->rem_device_command.rem_dev_cmd_ioctl.rem_dev_cmd_ioctl_header.ui16IoCtlFunction = htons( ui16IoCtlFn );
}


void vInitRemoteIoCtlRespHdr( struct s_rem_device_response *prdResponse, int8u ui8Device, int16u ui16IoCtlFn )
{
  vInitRemoteRespHdr( prdResponse, REMOTE_CMD_IOCTL, ui8Device );
  prdResponse->rem_device_response.rem_dev_resp_ioctl.rem_dev_cmd_ioctl_header.ui16IoCtlFunction = htons( ui16IoCtlFn );
}


void vInitRemoteIoCtlResp_Default( struct s_rem_device_response *prdResponse, int8u ui8Device, int16u ui16IoCtlFn, int32s i32Result )
{
  vInitRemoteIoCtlRespHdr( prdResponse, ui8Device, ui16IoCtlFn );
  prdResponse->rem_device_response.rem_dev_resp_ioctl.remote_ioctl_response.s_rem_dev_resp_ioctl_default.i32Result = htonl( i32Result );
}


bool bCheckRemoteRespHdr( const struct s_rem_device_response *prdResponse, int8u ui8Cmd, int8u ui8Device )
{
  return ( prdResponse->rem_dev_cmd_header.ui8Command == ui8Cmd &&
           prdResponse->rem_dev_cmd_header.ui8Device == ui8Device );
}


int nCheckRemoteResp_Default( const struct s_rem_device_response *prdResponse, int8u ui8Cmd, int8u ui8Device )
{
  int nResult;
  if ( !bCheckRemoteRespHdr( prdResponse, ui8Cmd, ui8Device ) )
  {
    nResult = RIO_ERROR;
  }
  else
  {
    nResult = (int) ntohl( prdResponse->rem_device_response.rem_dev_resp_default.i32Result );
  }
  return nResult;
}


bool bCheckRemoteIoCtlRespHdr( const struct s_rem_device_response *prdResponse, int8u ui8Device, int16u ui16IoCtlFn )
{
  return ( bCheckRemoteRespHdr( prdResponse, REMOTE_CMD_IOCTL, ui8Device ) &&
           ntohs( prdResponse->rem_device_response.rem_dev_resp_ioctl.rem_dev_cmd_ioctl_header.ui16IoCtlFunction ) == ui16IoCtlFn );
}


int nCheckRemoteIoCtlResp_Default( const struct s_rem_device_response *prdResponse, int8u ui8Device, int16u ui16IoCtlFn )
{
  int nResult;
  if ( !bCheckRemoteIoCtlRespHdr( prdResponse, ui8Device, ui16IoCtlFn ) )
  {
    nResult = RIO_ERROR;
  }
  else
  {
    nResult = (int) ntohl( prdResponse->rem_device_response.rem_dev_resp_ioctl.remote_ioctl_response.s_rem_dev_resp_ioctl_default.i32Result );
  }
  return nResult;
}


/** Fill out a default response */
void vDefaultResponse( const struct s_rem_device_command *prdCmd,
                       struct s_rem_device_response *prdResponse, int *pnLength, int nResult )
{
  int8u ui8Cmd = prdCmd->rem_dev_cmd_header.ui8Command;
  int8u ui8Dev = prdCmd->rem_dev_cmd_header.ui8Device;

  vInitRemoteResp_Default( prdResponse, ui8Cmd, ui8Dev, (int32s) nResult );
  *pnLength = GET_SIZE_REMOTE_RESP_DEFAULT();
}


/** Fill out a response as an error */
void vDefaultErrorResponse( const struct s_rem_device_command *prdCmd,
                            struct s_rem_device_response *prdResponse, int *pnLength )
{
  vDefaultResponse( prdCmd, prdResponse, pnLength, RIO_ERROR );
}


/** Fill out a response as success */
void vDefaultSuccessResponse( const struct s_rem_device_command *prdCmd,
                              struct s_rem_device_response *prdResponse, int *pnLength )
{
  vDefaultResponse( prdCmd, prdResponse, pnLength, RIO_SUCCESS );
}
