/*
 * slipio.c
 *
 *  Created on: 23/10/2008
 *      Author: ianb
 */
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include "abstractserial.h"
#include <rinhw/debug.h>
#include <rinremote/slip.h>

#include <rinremote/slip_io.h>


static void slip_serial_send_char(void *data, uint8_t ch)
{
  struct sAbstractSerialPort *pSlipSerialPort = data;

  if (NULL != pSlipSerialPort)
    AbstractSerialWrite(pSlipSerialPort, &ch, 1);
}


static uint8_t slip_serial_recv_char(void *data)
{
  uint8_t ch = '\0';
  struct sAbstractSerialPort *pSlipSerialPort = data;

  if (NULL != pSlipSerialPort)
    AbstractSerialRead(pSlipSerialPort, &ch, 1);
  return ch;
}


/** Clean up the slip serial port */
static void slip_serial_destroy(void *data)
{
  struct sAbstractSerialPort *pSlipSerialPort = data;

  if (NULL != pSlipSerialPort)
  {
    AbstractSerialShutdown(pSlipSerialPort);
    free(pSlipSerialPort);
  }
}


/** Initialise slip using a serial port */
int slip_serial_init(struct s_slipio *slipio, const char *devicename)
{
  struct sAbstractSerialPort *slip_serial_port;

#if HW_DEBUG(REMOTE_IO_DEBUG, HW_DEBUG_BASIC)
  DEBUG_PRINTF("slip_serial_init() called\n");
#endif

  slip_serial_port = calloc(1, sizeof(struct sAbstractSerialPort));
  if (NULL == slip_serial_port)
  {
    rinhw_error_printf("slip_serial_init: malloc failed\n");
    goto ssi_fail1;
  }

  /* Configure and open the com port */
#if HW_DEBUG(REMOTE_IO_DEBUG, HW_DEBUG_FULL)
  DEBUG_PRINTF("slip_serial_init: Open and configure serial port %s\n", devicename);
#endif
  if (!AbstractSerialInit(slip_serial_port, devicename))
  {
    rinhw_error_printf("slip_serial_init: AbstractSerialInit failed\n");
    goto ssi_fail2;
  }

  if (!AbstractSerialOpen(slip_serial_port))
  {
    rinhw_error_printf("slip_serial_init: AbstractSerialOpen failed\n");
    goto ssi_fail3;
  }

  if (!AbstractSerialSetConfig(slip_serial_port, 57600, SIO_DATABITS_EIGHT, SIO_PARITY_NONE, SIO_STOPBITS_ONE, false))
  {
    rinhw_error_printf("slip_serial_init: AbstractSerialSetConfig failed\n");
    goto ssi_fail3;
  }

  /* Setup slipio struct */
  memset(slipio, 0, sizeof(struct s_slipio));
  slipio->data = slip_serial_port;
  slipio->slip_send_char = slip_serial_send_char;
  slipio->slip_pre_send = NULL;
  slipio->slip_post_send = NULL;
  slipio->slip_recv_char = slip_serial_recv_char;
  slipio->slip_destroy = slip_serial_destroy;

#if HW_DEBUG(REMOTE_IO_DEBUG, HW_DEBUG_BASIC)
  DEBUG_PRINTF("slip_serial_init() finished\n");
#endif
  return RIO_SUCCESS;

ssi_fail3:
  AbstractSerialShutdown(slip_serial_port);

ssi_fail2:
  free(slip_serial_port);

ssi_fail1:
  return RIO_ERROR;
}
