/*
 * remote.h
 *
 *  Created on: 23/10/2008
 *      Author: ianb
 */
#ifndef REMOTE_H_
#define REMOTE_H_

#include <rinhw/stdrinstrum.h>
#include <rinhw/rio.h>
#include <rinremote/remote_kbd.h>
#include <rinremote/remote_lcd.h>
#include <rinremote/remote_sio.h>
#include <rinremote/remote_iic.h>
#include <rinremote/remote_adc24bit.h>


//enum eRemoteCommand { rcInit, rcOpen, rcClose, rcRead, rcWrite };

#define REMOTE_CMD_INIT 0
#define REMOTE_CMD_OPEN 1
#define REMOTE_CMD_CLOSE 2
#define REMOTE_CMD_READ 3
#define REMOTE_CMD_WRITE 4
#define REMOTE_CMD_ISATTY 5
#define REMOTE_CMD_FILELENGTH 6
#define REMOTE_CMD_LSEEK 7
#define REMOTE_CMD_TELL 8
#define REMOTE_CMD_IOCTL 9
#define REMOTE_CMD_DESTROY 10
#define REMOTE_MAX_CMD 10


// enum eRemoteDevice { rdCpuAdc, rdBuzzer, rdFlash, rdFram, rdIIC1, rdIIC2 };

#define REMOTE_DEV_CPU_ADC 0
#define REMOTE_DEV_BUZZER 1
#define REMOTE_DEV_FLASH 2
#define REMOTE_DEV_FRAM 3
#define REMOTE_DEV_IIC0 4
#define REMOTE_DEV_IIC1 5
#define REMOTE_DEV_KBD 6
#define REMOTE_DEV_LCD 7
#define REMOTE_DEV_RTC 8
#define REMOTE_DEV_SER0 9
#define REMOTE_DEV_SER1 10
#define REMOTE_DEV_SER2 11
#define REMOTE_DEV_SER3 12
#define REMOTE_DEV_SER4 13
#define REMOTE_DEV_ADC_24BIT 14
#define REMOTE_DEV_RESET 15
#define REMOTE_MAX_DEV 15
#define REMOTE_DEV_INVALID (REMOTE_MAX_DEV + 1)


#ifdef __HITACHI__
#pragma pack 1
#elif defined(__GNUC__)
#pragma pack(1)
#else
#error "Struct packing not defined"
#endif


/*
 * Init structure
 */


struct s_rem_dev_cmd_init
{
  int8u ui8Unused;
};


/*
 * Destroy structure
 */


struct s_rem_dev_cmd_destroy
{
  int8u ui8Unused;
};


/*
 * Open structure
 */


struct s_rem_dev_cmd_open
{
  int16u ui16Timeout;
};


/*
 * Close structure
 */


struct s_rem_dev_cmd_close
{
  int8u ui8Unused;
};


/*
 * Read structure
 */


struct s_rem_dev_cmd_read
{
  int16u ui16Length;
  int16u ui16Timeout;
};


struct s_rem_dev_resp_read
{
  int32s i32Result;
  int8u ui8Buffer[ 1 ];
};


/*
 * Write structure
 */


struct s_rem_dev_cmd_write
{
  int16u ui16Length;
  int16u ui16Timeout;
  int8u ui8Buffer[ 1 ];
};


/*
 * IsATty structure
 */


struct s_rem_dev_cmd_isatty
{
  int8u ui8Unused;
};


/*
 * Filelength structure
 */


struct s_rem_dev_cmd_filelength
{
  int8u ui8Unused;
};


/*
 * LSeek Structure
 */


struct s_rem_dev_cmd_lseek
{
  int32s i32Offset;
  int32s i32Fromwhere;
};


/*
 * Tell Structure
 */


struct s_rem_dev_cmd_tell
{
  int8u ui8Unused;
};


/*
 * Default response
 */


struct s_rem_dev_resp_default
{
  int32s i32Result;
};


/*
 * IOCTL Structure
 */


struct s_rem_dev_cmd_ioctl_header
{
  int16u ui16IoCtlFunction;
};


struct s_rem_dev_resp_ioctl_default
{
  int32s i32Result;
};


struct s_rem_dev_cmd_ioctl
{
  struct s_rem_dev_cmd_ioctl_header rem_dev_cmd_ioctl_header;
  union
  {
    struct s_rem_dev_cmd_ioctl_kbd_configure rem_dev_cmd_ioctl_kbd_configure;
    struct s_rem_dev_cmd_ioctl_kbd_external_keybits rem_dev_cmd_ioctl_kbd_external_keybits;
    struct s_rem_dev_cmd_ioctl_kbd_internal_keybits rem_dev_cmd_ioctl_kbd_internal_keybits;
    struct s_rem_dev_cmd_ioctl_lcd_set_backlight rem_dev_cmd_ioctl_lcd_set_backlight;
    struct s_rem_dev_cmd_ioctl_lcd_set_access rem_dev_cmd_ioctl_lcd_set_access;
    struct s_rem_dev_cmd_ioctl_sio_set_config rem_dev_cmd_ioctl_sio_set_config;
    struct s_rem_dev_cmd_ioctl_sio_get_config rem_dev_cmd_ioctl_sio_get_config;
    struct s_rem_dev_cmd_ioctl_sio_get_level rem_dev_cmd_ioctl_sio_get_level;
    struct s_rem_dev_cmd_ioctl_sio_set_ring_network rem_dev_cmd_ioctl_sio_set_ring_network;
    struct s_rem_dev_cmd_ioctl_iic_set_level rem_dev_cmd_ioctl_iic_set_level;
    struct s_rem_dev_cmd_ioctl_iic_get_level rem_dev_cmd_ioctl_iic_get_level;
    struct s_rem_dev_cmd_ioctl_adc24bit_set_level rem_dev_cmd_ioctl_adc24bit_set_level;
    struct s_rem_dev_cmd_ioctl_adc24bit_get_level rem_dev_cmd_ioctl_adc24bit_get_level;
  } remote_ioctl_command;
};


struct s_rem_dev_resp_ioctl
{
  struct s_rem_dev_cmd_ioctl_header rem_dev_cmd_ioctl_header;
  union
  {
    struct s_rem_dev_resp_ioctl_default s_rem_dev_resp_ioctl_default;
    struct s_rem_dev_resp_ioctl_kbd_internal_keybits rem_dev_resp_ioctl_kbd_internal_keybits;
    struct s_rem_dev_resp_ioctl_sio_get_config rem_dev_resp_ioctl_sio_get_config;
    struct s_rem_dev_resp_ioctl_sio_get_level rem_dev_resp_ioctl_sio_get_level;
    struct s_rem_dev_resp_ioctl_iic_get_level rem_dev_resp_ioctl_iic_get_level;
    struct s_rem_dev_resp_ioctl_adc24bit_get_level rem_dev_resp_ioctl_adc24bit_get_level;
  } remote_ioctl_response;
};


/*
 * Command structure
 */


struct s_rem_dev_cmd_header
{
  int8u ui8Command;
  int8u ui8Device;
};


struct s_rem_device_command
{
  struct s_rem_dev_cmd_header rem_dev_cmd_header;
  union
  {
    struct s_rem_dev_cmd_init rem_dev_cmd_init;
    struct s_rem_dev_cmd_open rem_dev_cmd_open;
    struct s_rem_dev_cmd_close rem_dev_cmd_close;
    struct s_rem_dev_cmd_read rem_dev_cmd_read;
    struct s_rem_dev_cmd_write rem_dev_cmd_write;
    struct s_rem_dev_cmd_isatty rem_dev_cmd_isatty;
    struct s_rem_dev_cmd_filelength rem_dev_cmd_filelength;
    struct s_rem_dev_cmd_lseek rem_dev_cmd_lseek;
    struct s_rem_dev_cmd_tell rem_dev_cmd_tell;
    struct s_rem_dev_cmd_ioctl rem_dev_cmd_ioctl;
    struct s_rem_dev_cmd_destroy rem_dev_cmd_destroy;
  } rem_device_command;
};


/*
 * Command sizes
 */
#define GET_SIZE_REMOTE_CMD_INIT()          ( sizeof( struct s_rem_dev_cmd_header) + sizeof( struct s_rem_dev_cmd_init ) )
#define GET_SIZE_REMOTE_CMD_OPEN()          ( sizeof( struct s_rem_dev_cmd_header) + sizeof( struct s_rem_dev_cmd_open ) )
#define GET_SIZE_REMOTE_CMD_CLOSE()         ( sizeof( struct s_rem_dev_cmd_header) + sizeof( struct s_rem_dev_cmd_close ) )
#define GET_SIZE_REMOTE_CMD_READ()          ( sizeof( struct s_rem_dev_cmd_header) + sizeof( struct s_rem_dev_cmd_read ) )
#define GET_SIZE_REMOTE_CMD_WRITE(datalen)  ( sizeof( struct s_rem_dev_cmd_header) + sizeof( struct s_rem_dev_cmd_write ) + datalen - 1 ) /* Minus 1 for the overlap between buffer[1] and datalen */
#define GET_SIZE_REMOTE_CMD_ISATTY()        ( sizeof( struct s_rem_dev_cmd_header) + sizeof( struct s_rem_dev_cmd_isatty ) )
#define GET_SIZE_REMOTE_CMD_FILELENGTH()    ( sizeof( struct s_rem_dev_cmd_header) + sizeof( struct s_rem_dev_cmd_filelength ) )
#define GET_SIZE_REMOTE_CMD_LSEEK()         ( sizeof( struct s_rem_dev_cmd_header) + sizeof( struct s_rem_dev_cmd_lseek ) )
#define GET_SIZE_REMOTE_CMD_TELL()          ( sizeof( struct s_rem_dev_cmd_header) + sizeof( struct s_rem_dev_cmd_tell ) )
#define GET_SIZE_REMOTE_CMD_DESTROY()       ( sizeof( struct s_rem_dev_cmd_header) + sizeof( struct s_rem_dev_cmd_destroy ) )


/*
 * IOCTL Command sizes
 */
#define IOCTL_CMD_HDR_SIZE ( sizeof( struct s_rem_dev_cmd_header ) + sizeof( struct s_rem_dev_cmd_ioctl_header ) )
#define GET_SIZE_REMOTE_CMD_IOCTL_KBD_CONFIGURE(keys) ( IOCTL_CMD_HDR_SIZE + sizeof( struct s_rem_dev_cmd_ioctl_kbd_configure ) + keys - 1 ) /* Minus 1 for the overlap between buffer[1] and keys */
#define GET_SIZE_REMOTE_CMD_IOCTL_KBD_EXT_KEYBITS() ( IOCTL_CMD_HDR_SIZE + sizeof( struct s_rem_dev_cmd_ioctl_kbd_external_keybits ) )
#define GET_SIZE_REMOTE_CMD_IOCTL_KBD_INT_KEYBITS() ( IOCTL_CMD_HDR_SIZE + sizeof( struct s_rem_dev_cmd_ioctl_kbd_internal_keybits ) )
#define GET_SIZE_REMOTE_CMD_IOCTL_LCD_SET_BACKLIGHT() ( IOCTL_CMD_HDR_SIZE + sizeof( struct s_rem_dev_cmd_ioctl_lcd_set_backlight ) )
#define GET_SIZE_REMOTE_CMD_IOCTL_LCD_SET_ACCESS() ( IOCTL_CMD_HDR_SIZE + sizeof( struct s_rem_dev_cmd_ioctl_lcd_set_access ) )
#define GET_SIZE_REMOTE_CMD_IOCTL_SIO_SET_CONFIG() ( IOCTL_CMD_HDR_SIZE + sizeof( struct s_rem_dev_cmd_ioctl_sio_set_config ) )
#define GET_SIZE_REMOTE_CMD_IOCTL_SIO_GET_CONFIG() ( IOCTL_CMD_HDR_SIZE + sizeof( struct s_rem_dev_cmd_ioctl_sio_get_config ) )
#define GET_SIZE_REMOTE_CMD_IOCTL_SIO_GET_LEVEL() ( IOCTL_CMD_HDR_SIZE + sizeof( struct s_rem_dev_cmd_ioctl_sio_get_level ) )
#define GET_SIZE_REMOTE_CMD_IOCTL_SIO_SET_RING_NETWORK() ( IOCTL_CMD_HDR_SIZE + sizeof( struct s_rem_dev_cmd_ioctl_sio_set_ring_network ) )
#define GET_SIZE_REMOTE_CMD_IOCTL_IIC_SET_LEVEL()  ( IOCTL_CMD_HDR_SIZE + sizeof( struct s_rem_dev_cmd_ioctl_iic_set_level ) )
#define GET_SIZE_REMOTE_CMD_IOCTL_IIC_GET_LEVEL()  ( IOCTL_CMD_HDR_SIZE + sizeof( struct s_rem_dev_cmd_ioctl_iic_get_level ) )
#define GET_SIZE_REMOTE_CMD_IOCTL_ADC24BIT_SET_LEVEL()  ( IOCTL_CMD_HDR_SIZE + sizeof( struct s_rem_dev_cmd_ioctl_adc24bit_set_level ) )
#define GET_SIZE_REMOTE_CMD_IOCTL_ADC24BIT_GET_LEVEL()  ( IOCTL_CMD_HDR_SIZE + sizeof( struct s_rem_dev_cmd_ioctl_adc24bit_get_level ) )



struct s_rem_device_response
{
  struct s_rem_dev_cmd_header rem_dev_cmd_header;
  union
  {
    struct s_rem_dev_resp_default rem_dev_resp_default;
    struct s_rem_dev_resp_read rem_dev_resp_read;
    struct s_rem_dev_resp_ioctl rem_dev_resp_ioctl;
  } rem_device_response;
};


#ifdef __HITACHI__
#pragma pack 2
#elif defined(__GNUC__)
#pragma pack()
#endif


/*
 * Response sizes
 */
#define GET_SIZE_REMOTE_RESP_DEFAULT()       ( sizeof( struct s_rem_dev_cmd_header) + sizeof( struct s_rem_dev_resp_default ) )
#define GET_SIZE_REMOTE_RESP_INIT()          GET_SIZE_REMOTE_RESP_DEFAULT()
#define GET_SIZE_REMOTE_RESP_OPEN()          GET_SIZE_REMOTE_RESP_DEFAULT()
#define GET_SIZE_REMOTE_RESP_CLOSE()         GET_SIZE_REMOTE_RESP_DEFAULT()
#define GET_SIZE_REMOTE_RESP_READ(datalen)   ( sizeof( struct s_rem_dev_cmd_header) + sizeof( struct s_rem_dev_resp_read ) + datalen - 1 )  /* Minus 1 for the overlap between buffer[1] and datalen */
#define GET_SIZE_REMOTE_RESP_WRITE()         GET_SIZE_REMOTE_RESP_DEFAULT()
#define GET_SIZE_REMOTE_RESP_ISATTY()        GET_SIZE_REMOTE_RESP_DEFAULT()
#define GET_SIZE_REMOTE_RESP_FILELENGTH()    GET_SIZE_REMOTE_RESP_DEFAULT()
#define GET_SIZE_REMOTE_RESP_LSEEK()         GET_SIZE_REMOTE_RESP_DEFAULT()
#define GET_SIZE_REMOTE_RESP_TELL()          GET_SIZE_REMOTE_RESP_DEFAULT()
#define GET_SIZE_REMOTE_RESP_DESTROY()       GET_SIZE_REMOTE_RESP_DEFAULT()


/*
 * IOCTL Response sizes
 */
#define IOCTL_RESP_HDR_SIZE ( sizeof( struct s_rem_dev_cmd_header ) + sizeof( struct s_rem_dev_cmd_ioctl_header ) )
#define GET_SIZE_REMOTE_RESP_IOCTL_DEFAULT() ( IOCTL_RESP_HDR_SIZE + sizeof( struct s_rem_dev_resp_ioctl_default ) )
#define GET_SIZE_REMOTE_RESP_IOCTL_KBD_INT_KEYBITS() ( IOCTL_RESP_HDR_SIZE + sizeof( struct s_rem_dev_resp_ioctl_kbd_internal_keybits ) )
#define GET_SIZE_REMOTE_RESP_IOCTL_SIO_GET_CONFIG()  ( IOCTL_RESP_HDR_SIZE + sizeof( struct s_rem_dev_resp_ioctl_sio_get_config ) )
#define GET_SIZE_REMOTE_RESP_IOCTL_SIO_GET_LEVEL()   ( IOCTL_RESP_HDR_SIZE + sizeof( struct s_rem_dev_resp_ioctl_sio_get_level ) )
#define GET_SIZE_REMOTE_RESP_IOCTL_IIC_GET_LEVEL()   ( IOCTL_RESP_HDR_SIZE + sizeof( struct s_rem_dev_resp_ioctl_iic_get_level ) )
#define GET_SIZE_REMOTE_RESP_IOCTL_ADC24BIT_GET_LEVEL()   ( IOCTL_RESP_HDR_SIZE + sizeof( struct s_rem_dev_resp_ioctl_adc24bit_get_level ) )


#define MAX_REMOTE_BUFFER 100
#define MAX_COMMAND_BUFFER ( sizeof( struct s_rem_device_command ) + MAX_REMOTE_BUFFER )
#define MAX_RESPONSE_BUFFER ( sizeof( struct s_rem_device_response ) + MAX_REMOTE_BUFFER )


int remote_dev_init( struct t_io_dev *io_dev );
int remote_dev_destroy( struct t_io_dev *io_dev );
int remote_dev_open( struct t_io_dev *io_dev, unsigned short timeout );
int remote_dev_close( struct t_io_dev *io_dev );
int remote_dev_read(struct t_io_dev *io_dev, void *buf, unsigned short len, unsigned short timeout);
int remote_dev_write(struct t_io_dev *io_dev, const void *buf, unsigned short len, unsigned short timeout);
int remote_dev_isatty(struct t_io_dev *dev);
long remote_dev_filelength(struct t_io_dev *dev);
long remote_dev_lseek(struct t_io_dev *dev, long offset, int fromwhere);
long remote_dev_tell(struct t_io_dev *dev);
int remote_dev_rioctl(struct t_io_dev *dev, int func, ...);


void vInitRemoteCmdHdr( struct s_rem_device_command *prdCmd, int8u ui8Cmd, int8u ui8Device );
void vInitRemoteIoCtlCmdHdr( struct s_rem_device_command *prdCmd, int8u ui8Device, int16u ui16IoCtlFn );
bool bCheckRemoteRespHdr( const struct s_rem_device_response *prdResponse, int8u ui8Cmd, int8u ui8Device );
int nCheckRemoteResp_Default( const struct s_rem_device_response *prdResponse, int8u ui8Cmd, int8u ui8Device );
bool bCheckRemoteIoCtlRespHdr( const struct s_rem_device_response *prdResponse, int8u ui8Device, int16u ui16IoCtlFn );
int nCheckRemoteIoCtlResp_Default( const struct s_rem_device_response *prdResponse, int8u ui8Device, int16u ui16IoCtlFn );


#endif /* REMOTE_H_ */
