/*
 * slip.h
 *
 *  Created on: 23/10/2008
 *      Author: ianb
 */
#ifndef SLIP_DOT_H
#define SLIP_DOT_H 1

#include <stdint.h>


/** SLIP IO control structure */
struct s_slipio
{
  void *data;                                       /*!< User data */
  void (*slip_send_char)(void *data, uint8_t ch);   /*!< Called to send a character */
  void (*slip_pre_send)(void *data);                /*!< Called before a slip packet is sent */
  void (*slip_post_send)(void *data);               /*!< Called after a slip packet has been sent */
  uint8_t (*slip_recv_char)(void *data);            /*!< Called to receive a character */
  void (*slip_destroy)(void *data);                 /*!< Shut down the slip IO */
};


/* SEND_PACKET: sends a packet of length "len", starting at
 * location "p".
 */
void slip_send_packet(struct s_slipio *slipio, const uint8_t *p, int len);


/* RECV_PACKET: receives a packet into the buffer located at "p".
*      If more than len bytes are received, the packet will
*      be truncated.
*      Returns the number of bytes stored in the buffer.
*/
int slip_recv_packet(struct s_slipio *slipio, uint8_t *p, int len);


/** Destroy slip */
void slip_destroy(struct s_slipio *slipio);


#endif /* #ifndef SLIP_DOT_H */
