/*
 * cmdline.c
 *
 *  Created on: Jul 27, 2016
 *      Author: ian
 */
#include <stdbool.h>
#include <stdio.h>
#include <getopt.h>
#include <stdlib.h>

#include "cmdline.h"


/** Display command line help */
bool cmdline_showhelp = false;


/** Log error messages to stdout */
bool cmdline_logtostdout = false;


/** Log error messages to syslog */
bool cmdline_logtosyslog = false;


/** Serial port device to use for remote devices */
const char *cmdline_remote_device = NULL;


/** TCP port to listen on for remote devices */
int cmdline_remote_tcpport = -1;


/** Print out the command line options */
void print_commandlineopts(const char *program_name)
{
  fprintf(stdout, "Usage: %s [-options]\n"
                  "  -h           --help             Print help\n"
                  "  -r <serial>  --remote <serial>  Communicate with application over\n"
                  "                                  <serial> port (/dev/ttySx)\n"
                  "  -t <port>    --tcp <port>       Communicate with application over tcp port\n"
                  "                                  <port>\n"
                  "  -l           --syslog           Log error messages to syslog\n"
                  "  -s           --stdout           Log error messages to stdout\n",
          program_name);
}


/** Parse the command line options */
int parse_commandline(int argc, char *argv[])
{
  int c;
  int result = 0;
  static const char opts[] = "hr:t:ls";
  static const struct option lopts[] =
  {
    { "help",          0, 0, 'h'  },
    { "remote",        1, 0, 'r'  },
    { "tcp",           1, 0, 't'  },
    { "syslog",        0, 0, 'l'  },
    { "stdout",        0, 0, 's'  },
    { NULL,            0, 0, '\0' }
  };

  while (1)
  {
    c = getopt_long(argc, argv, opts, lopts, NULL);
    if (-1 == c)
    {
      break;
    }

    switch (c)
    {
      case 'h':
        cmdline_showhelp = true;
        break;
      case 'r':
        cmdline_remote_device = optarg;
        break;
      case 't':
        cmdline_remote_tcpport = atoi(optarg);
        break;
      case 'l':
        cmdline_logtosyslog = true;
        break;
      case 's':
        cmdline_logtostdout = true;
        break;
      default:
        result = -1;
        break;
    }
  }
  return result;
}
