/*
 * devices.c
 *
 *  Created on: 24/10/2008
 *      Author: ianb
 */
#include <stdio.h>
#include <rinhw/debug.h>
#include <rinhw/hwlib_init.h>
#include <rinhw/we21hw.h>
#include <rinhw/rio.h>
#include <rinhw/sio_local_console.h>
#include <rinhw/iic_local_linux.h>
#include <rinhw/adc_ads1216_local.h>
#include <rinhw/adc_ads1216_we21.h>
#include <rinhw/rtc_local.h>
#include <rinhw/sio.h>
#include <rinhw/sio_local_comport.h>
#include <rinhw/sio_local_dgram.h>
#include <rinhw/sio_local_socket.h>
#include <rinremote/remote.h>
#include "remote_debug.h"

#include "devices_we.h"


/** SER 1 - onboard RS232/RS422 secondary serial port */
/* struct t_io_dev dev_ser1_local; */


/** SER 2 - onboard RS232 secondary serial port */
/* struct t_io_dev dev_ser2_local; */


/** Buzzer device (located on display rinblock) */
/* struct t_io_dev dev_buz_we; */


/** Primary keyboard device, (located on display rinblock) */
/* struct t_io_dev dev_kbd_we; */


/** Real time clock device */
/* struct t_io_dev dev_rtc_we; */


/** I2C device */
struct t_io_dev dev_iic0_we;


/** 24bit ADC device */
struct t_io_dev dev_adc24bit_we;


/** LCD device */
/* struct t_io_dev dev_lcd_we; */


/** ADC device */
/* struct t_io_dev dev_adc_we; */


/** Console device */
struct t_io_dev dev_ser_console;


#if 0
static struct t_io_dev *system_devices[REMOTE_MAX_DEV + 1] =
{
    &dev_adc_we,        /* REMOTE_DEV_CPU_ADC */
    &dev_buz_we,        /* REMOTE_DEV_BUZZER */
    NULL,               /* REMOTE_DEV_FLASH */
    NULL,               /* REMOTE_DEV_FRAM */
    &dev_iic0_we,       /* REMOTE_DEV_IIC0 */
    NULL,               /* REMOTE_DEV_IIC1 */
    &dev_kbd_we,        /* REMOTE_DEV_KBD */
    &dev_lcd_we,        /* REMOTE_DEV_LCD */
    &dev_rtc_we,        /* REMOTE_DEV_RTC */
    NULL,               /* REMOTE_DEV_SER0 */
    &dev_ser1_local,    /* REMOTE_DEV_SER1 */
    &dev_ser2_local,    /* REMOTE_DEV_SER2 */
    NULL,               /* REMOTE_DEV_SER3 */
    NULL,               /* REMOTE_DEV_SER4 */
    &dev_adc24bit_we,   /* REMOTE_DEV_ADC_24BIT */
    NULL,               /* REMOTE_DEV_RESET */
};
#endif


/** Implement only the bare minimum of devices we need */
static struct t_io_dev *system_devices[REMOTE_MAX_DEV + 1] =
{
    NULL,               /* REMOTE_DEV_CPU_ADC */
    NULL,               /* REMOTE_DEV_BUZZER */
    NULL,               /* REMOTE_DEV_FLASH */
    NULL,               /* REMOTE_DEV_FRAM */
    &dev_iic0_we,       /* REMOTE_DEV_IIC0 */
    NULL,               /* REMOTE_DEV_IIC1 */
    NULL,               /* REMOTE_DEV_KBD */
    NULL,               /* REMOTE_DEV_LCD */
    NULL,               /* REMOTE_DEV_RTC */
    NULL,               /* REMOTE_DEV_SER0 */
    NULL,               /* REMOTE_DEV_SER1 */
    NULL,               /* REMOTE_DEV_SER2 */
    NULL,               /* REMOTE_DEV_SER3 */
    NULL,               /* REMOTE_DEV_SER4 */
    &dev_adc24bit_we,   /* REMOTE_DEV_ADC_24BIT */
    NULL,               /* REMOTE_DEV_RESET */
};


struct t_io_dev *GetDevice(int8u ui8Device)
{
  if (ui8Device > REMOTE_MAX_DEV)
  {
    return NULL;
  }
  return system_devices[ui8Device];
}


/** Start up the system */
int init_test_app_devices(void)
{
  /* struct s_sio_local_rs422 sio_ser1_rs422; */

  /* Start the hardware library */
  hwlib_init();
  rinhw_set_debug_printf_fn(rdebug_vprintf);
  if (rio_init_we21hw("/dev/we21hw") < 0)
  {
    rdebug_printf(LOG_ERR, "init_test_app_devices: we21 hardware initialisation failed\n");
    return -1;
  }

  /* Make a console for IO */
  if (rio_create_device_sio_localconsole(&dev_ser_console) < 0)
  {
    rdebug_printf(LOG_ERR, "init_test_app_devices: local console serial device creation failed\n");
    return -1;
  }

  if (rio_create_device_iic_local(&dev_iic0_we, "/dev/i2c-0", we21hw_set_slot_select, NULL) < 0)
  {
    rdebug_printf(LOG_ERR, "init_test_app_devices: i2c device creation failed\n");
    return -1;
  }

  if (rio_create_device_adc24bit_local(&dev_adc24bit_we, "/dev/spidev0.0",
                                       adc24bit_user_hw_init_we21,
                                       adc24bit_sync_spi_we21,
                                       NULL,
                                       adc24bit_wait_for_reading_we21) < 0)
  {
    rdebug_printf(LOG_ERR, "init_test_app_devices: adc 24bit device creation failed\n");
    return -1;
  }

#if 0
  if (rio_create_device_rtc_local(&dev_rtc_we) < 0)
  {
    rdebug_printf(LOG_ERR, "init_test_app_devices: rtc device creation failed\n");
    return -1;
  }

  /* Setup serial port devices */
  sio_ser1_rs422.hw_data = NULL;
  //sio_ser1_rs422.rts_controls_txenable = true;
  sio_ser1_rs422.sio_select_rs232_not_rs422 = we21hw_select_rs232_not_rs422;
  sio_ser1_rs422.sio_set_rs422_termination = we21hw_set_rs422_termination;
  sio_ser1_rs422.sio_set_rs422_transmitenable = NULL;
  if (rio_create_device_sio_localcom(&dev_ser1_local, "/dev/ttyS1", &sio_ser1_rs422, false) < 0)
  {
    rdebug_printf(LOG_ERR, "init_test_app_devices: local serial 1 setup fail\n");
    return -1;
  }

  if (rio_create_device_sio_localcom(&dev_ser2_local, "/dev/ttyS2", NULL, false) < 0)
  {
    rdebug_printf(LOG_ERR, "init_test_app_devices: local serial 2 setup fail\n");
    return -1;
  }
#endif

  return 0;
}


/** Shut down the system */
void destroy_test_app_devices(void)
{
  printf("main: shutdown initiated\n");

  /* Destroy devices */
  /* rdestroy_io(&dev_ser2_local); */
  /* rdestroy_io(&dev_ser1_local); */
  /* rdestroy_io(&dev_rtc_we); */
  rdestroy_io(&dev_adc24bit_we);
  rdestroy_io(&dev_iic0_we);
  rdestroy_io(&dev_ser_console);

  /* Shutdown the hardware library */
  rio_destroy_we21hw();
  hwlib_destroy();
}
