#include <signal.h>
#include <stdio.h>
#include <getopt.h>
#include <rinhw/rstdio.h>
#include <rinhw/ucos_ii.h>
#include <rinhw/rio.h>
#include <rinhw/sio.h>
#include <rinhw/framdev.h>
#include <rinhw/adc_ads1216.h>
#include <rinhw/hw_vers.h>
#include "remotedemuxhandler.h"
#include "devices.h"
#include "cmdline.h"
#include "remote_debug.h"


static void remote_menu(void)
{
  char ch;

  for (ch ='\0'; 'Q' != ch; )
  {
    rfprintf(&dev_ser_console, "Remote server menu\n");
    rfprintf(&dev_ser_console, " Q - Quit\n");
    rfprintf(&dev_ser_console, " S - Statistics\n");

    if (rread(&dev_ser_console, &ch, sizeof(ch), 0) == sizeof(char))
    {
      switch (ch)
      {
        case 'Q':
          /* Handled by for loop */
          break;
        case 'S':
          rfprintf(&dev_ser_console, "TODO\n");
          break;
      }
    }
  }
}


/** Entry point */
int main(int argc, char *argv[])
{
  int result;

  /* Parse command line */
  if (parse_commandline(argc, argv) < 0)
  {
    fprintf(stderr, "Invalid command line, try --help\n");
    result = 1;
    goto m_fail;
  }

  /* Show help if requested */
  if (cmdline_showhelp)
  {
    print_commandlineopts(argv[0]);
    result = 0;
    goto m_fail;
  }

  /* Setup syslog */
  if (cmdline_logtosyslog)
  {
    openlog("librinhw_remote_server", LOG_PID, LOG_USER);
  }

  /* Need either serial device or listening tcp port */
  if (NULL == cmdline_remote_device && cmdline_remote_tcpport < 0)
  {
    fprintf(stderr, "A serial device or TCP port must be specified, try --help\n");
    result = 1;
    goto m_fail;
  }

  /* Create remote devices */
  if (init_test_app_devices() < 0)
  {
    result = 1;
    goto m_fail;
  }

  /* Open debug device if not already open */
  if (dev_ser_console.status & O_OPEN)
  {
    ropen(&dev_ser_console, 0);
  }

  /* Start listening */
  rfprintf(&dev_ser_console, "librinhw Remote Hardware Server\n");
  rfprintf(&dev_ser_console, "Hardware Library Version: %s\n", HwLibDrawingNumberStr);
  if (NULL != cmdline_remote_device)
    rfprintf(&dev_ser_console, "Listening on serial port %s\n", cmdline_remote_device);
  else
    rfprintf(&dev_ser_console, "Listening on tcp port %d\n", cmdline_remote_tcpport);
  if (!remote_listen_init(cmdline_remote_device, cmdline_remote_tcpport))
  {
    result = 1;
    goto m_fail;
  }

  /* Run menu on console */
  remote_menu();

  /* Shut it down */
  rfprintf(&dev_ser_console, "Stopped listening\n");
  remote_listen_destroy();
  rclose(&dev_ser_console);
  destroy_test_app_devices();
  result = 0;

m_fail:
  return result;
}
