/*
 * remotedemuxioctl.c
 *
 *  Created on: 24/10/2008
 *      Author: ianb
 */
#include <rinhw/rstdio.h>
#include <rinhw/sio.h>
#include <rinremote/remote.h>
#include "remotedemuxioctl_adc24bit.h"
#include "remotedemuxioctl_iic.h"
#include "remotedemuxioctl_kbd.h"
#include "remotedemuxioctl_lcd.h"
#include "remotedemuxioctl_sio.h"
#include "remotehelper.h"

#include "remotedemuxioctl.h"



static bool bHandleCommandIoCtl_Unhandled( const struct s_rem_device_command *prdCmd,
                                           struct s_rem_device_response *prdResponse, int *pnLength )
{
  vInitRemoteIoCtlResp_Default( prdResponse, prdCmd->rem_dev_cmd_header.ui8Device,
                                prdCmd->rem_device_command.rem_dev_cmd_ioctl.rem_dev_cmd_ioctl_header.ui16IoCtlFunction,
                                RIO_ERROR );
  *pnLength = GET_SIZE_REMOTE_RESP_IOCTL_DEFAULT();
//  rfprintf( &dev_com2, "Unhandled result length %d\n", *pnLength );
  return true;
}


void vHandleCommandIoCtl( const struct s_rem_device_command *prdCmd,
                          struct s_rem_device_response *prdResponse, int *pnLength )
{
//  rfprintf( &dev_com2, "IOCTL for device %d\n", prdCmd->rem_dev_cmd_header.ui8Device );

  switch ( prdCmd->rem_dev_cmd_header.ui8Device )
  {
    case REMOTE_DEV_ADC_24BIT:
    {
      bHandleCommandIoCtl_Adc24Bit( prdCmd, prdResponse, pnLength );
    }
    break;

    case REMOTE_DEV_KBD:
    {
      bHandleCommandIoCtl_Kbd( prdCmd, prdResponse, pnLength );
    }
    break;

    case REMOTE_DEV_IIC0:
    case REMOTE_DEV_IIC1:
    {
      bHandleCommandIoCtl_Iic( prdCmd, prdResponse, pnLength );
    }
    break;

    case REMOTE_DEV_LCD:
    {
      bHandleCommandIoCtl_Lcd( prdCmd, prdResponse, pnLength );
    }
    break;

    case REMOTE_DEV_SER0:
    case REMOTE_DEV_SER1:
    case REMOTE_DEV_SER2:
    case REMOTE_DEV_SER3:
    case REMOTE_DEV_SER4:
    {
      bHandleCommandIoCtl_Sio( prdCmd, prdResponse, pnLength );
    }
    break;

    default:
    {
//      rfprintf( &dev_com2, "Unhandled IOCTL device\n" );
      bHandleCommandIoCtl_Unhandled( prdCmd, prdResponse, pnLength );
    }
    break;
  }
}


/** Get the packet size for an IOCTL, which has to inspect the packet in some instances */
int nGetCmdPacketSizeIoCtl( const struct s_rem_device_command *prdCmd )
{
  int nLength;

  switch ( prdCmd->rem_dev_cmd_header.ui8Device )
  {
    case REMOTE_DEV_ADC_24BIT:
    {
      nLength = nGetCmdPacketSizeIoCtl_Adc24Bit( prdCmd );
    }
    break;

    case REMOTE_DEV_KBD:
    {
      nLength = nGetCmdPacketSizeIoCtl_Kbd( prdCmd );
    }
    break;

    case REMOTE_DEV_IIC0:
    case REMOTE_DEV_IIC1:
    {
      nLength = nGetCmdPacketSizeIoCtl_Iic( prdCmd );
    }
    break;

    case REMOTE_DEV_LCD:
    {
      nLength = nGetCmdPacketSizeIoCtl_Lcd( prdCmd );
    }
    break;

    case REMOTE_DEV_SER0:
    case REMOTE_DEV_SER1:
    case REMOTE_DEV_SER2:
    case REMOTE_DEV_SER3:
    case REMOTE_DEV_SER4:
    {
      nLength = nGetCmdPacketSizeIoCtl_Sio( prdCmd );
    }
    break;

    default:
    {
      nLength = 0;
    }
    break;
  }
  return nLength;
}
