/*
 * remotedemuxioctl_adc24bit.c
 *
 *  Created on: 20/08/2009
 *      Author: ianb
 */
#include <rinhw/adc_ads1216.h>
#include <rinhw/rio.h>
#include <rinhw/rinendian.h>
#include <rinhw/rstdio.h>
#include "devices.h"
#include "remotehelper.h"
#include "remotedemux.h"

#include "remotedemuxioctl_adc24bit.h"


bool bHandleCommandIoCtl_Adc24Bit( const struct s_rem_device_command *prdCmd,
                                   struct s_rem_device_response *prdResponse, int *pnLength )
{
  int nResult = RIO_ERROR;
  struct t_io_dev *dev = GetDevice( prdCmd->rem_dev_cmd_header.ui8Device );
  int nFunc = ntohs( prdCmd->rem_device_command.rem_dev_cmd_ioctl.rem_dev_cmd_ioctl_header.ui16IoCtlFunction );
  bool bDefaultResponse = true;
  int32u ui32Level;
  int nLevel;

//  rfprintf( &dev_com2, "ADC24Bit IOCTL %d\n", nFunc );
  if ( 0 != dev && slip_device != dev )
  {
    switch ( nFunc )
    {
      case IOCTL_ADC24BIT_SET_ZERO_REGISTER:
      case IOCTL_ADC24BIT_SET_SPAN_REGISTER:
        ui32Level = ntohl( prdCmd->rem_device_command.rem_dev_cmd_ioctl.remote_ioctl_command.rem_dev_cmd_ioctl_adc24bit_set_level.ui32Level );
        nResult = rioctl( dev, nFunc, ui32Level );
        break;

      case IOCTL_ADC24BIT_GET_ZERO_REGISTER:
      case IOCTL_ADC24BIT_GET_SPAN_REGISTER:
        nResult = rioctl( dev, nFunc, &ui32Level );
        bDefaultResponse = false;
        vInitRemoteIoCtlRespHdr( prdResponse, prdCmd->rem_dev_cmd_header.ui8Device, (int16u) nFunc );
        prdResponse->rem_device_response.rem_dev_resp_ioctl.remote_ioctl_response.rem_dev_resp_ioctl_adc24bit_get_level.i32Result = htonl( nResult );
        prdResponse->rem_device_response.rem_dev_resp_ioctl.remote_ioctl_response.rem_dev_resp_ioctl_adc24bit_get_level.ui32Level = htonl( ui32Level );
        *pnLength = GET_SIZE_REMOTE_RESP_IOCTL_ADC24BIT_GET_LEVEL();
        break;

      case IOCTL_ADC24BIT_SET_PGA_GAIN:
      case IOCTL_ADC24BIT_SET_DECIMATION_RATIO:
      case IOCTL_ADC24BIT_SET_LCWIRE_MODE:
      case IOCTL_ADC24BIT_SET_BURNOUTCURRENTSOURCE:
        nLevel = (int) ntohl( prdCmd->rem_device_command.rem_dev_cmd_ioctl.remote_ioctl_command.rem_dev_cmd_ioctl_adc24bit_set_level.ui32Level );
        nResult = rioctl( dev, nFunc, nLevel );
        break;

      case IOCTL_ADC24BIT_FORCE_RESET:
      case IOCTL_ADC24BIT_PERFORM_SELF_ZERO_CALIBRATION:
        nResult = rioctl( dev, nFunc );
        break;

      case IOCTL_ADC24BIT_IS_RESPONDING:
        nResult = rioctl( dev, nFunc, &nLevel );
        bDefaultResponse = false;
        vInitRemoteIoCtlRespHdr( prdResponse, prdCmd->rem_dev_cmd_header.ui8Device, (int16u) nFunc );
        prdResponse->rem_device_response.rem_dev_resp_ioctl.remote_ioctl_response.rem_dev_resp_ioctl_adc24bit_get_level.i32Result = htonl( nResult );
        prdResponse->rem_device_response.rem_dev_resp_ioctl.remote_ioctl_response.rem_dev_resp_ioctl_adc24bit_get_level.ui32Level = htonl( nLevel );
        *pnLength = GET_SIZE_REMOTE_RESP_IOCTL_ADC24BIT_GET_LEVEL();
        break;

      case IOCTL_ADC24BIT_GET_ID:
        nResult = rioctl(dev, nFunc, &nLevel);
        bDefaultResponse = false;
        vInitRemoteIoCtlRespHdr(prdResponse, prdCmd->rem_dev_cmd_header.ui8Device, (int16u) nFunc);
        prdResponse->rem_device_response.rem_dev_resp_ioctl.remote_ioctl_response.rem_dev_resp_ioctl_adc24bit_get_level.i32Result = htonl( nResult );
        prdResponse->rem_device_response.rem_dev_resp_ioctl.remote_ioctl_response.rem_dev_resp_ioctl_adc24bit_get_level.ui32Level = htonl( nLevel );
        *pnLength = GET_SIZE_REMOTE_RESP_IOCTL_ADC24BIT_GET_LEVEL();
        break;

      case IOCTL_ADC24BIT_WAIT_FOR_DATA_READY:
        ui32Level = ntohl( prdCmd->rem_device_command.rem_dev_cmd_ioctl.remote_ioctl_command.rem_dev_cmd_ioctl_adc24bit_set_level.ui32Level );
        nResult = rioctl(dev, nFunc, &ui32Level);
        break;

      default:
        nResult = RIO_ERROR;
        break;
    }
  }
//  rfprintf( &dev_com2, "ADC24Bit IOCTL result %d\n", nResult );

  if ( bDefaultResponse )
  {
    vInitRemoteIoCtlResp_Default( prdResponse, prdCmd->rem_dev_cmd_header.ui8Device,
                                  (int16u) nFunc, (int32s) nResult );
    *pnLength = GET_SIZE_REMOTE_RESP_IOCTL_DEFAULT();
  }
//  rfprintf( &dev_com2, "ADC24Bit result length %d\n", *pnLength );
  return true;
}


/** Get the packet size of an ADC IOCTL */
int nGetCmdPacketSizeIoCtl_Adc24Bit( const struct s_rem_device_command *prdCmd )
{
  return GET_SIZE_REMOTE_CMD_IOCTL_ADC24BIT_SET_LEVEL();
}
