/*
 * remotedemuxioctl_kbd.c
 *
 *  Created on: 24/10/2008
 *      Author: ianb
 */
#include <rinhw/kbd.h>
#include <rinhw/rio.h>
#include <rinhw/rstdio.h>
#include <rinhw/sio.h>
#include <rinhw/rinendian.h>
#include "devices.h"
#include "remotehelper.h"
#include "remotedemux.h"

#include "remotedemuxioctl_kbd.h"


bool bHandleCommandIoCtl_Kbd( const struct s_rem_device_command *prdCmd,
                              struct s_rem_device_response *prdResponse, int *pnLength )
{
  int nResult = RIO_ERROR;
  struct t_io_dev *dev = GetDevice( prdCmd->rem_dev_cmd_header.ui8Device );
  int nFunc = ntohs( prdCmd->rem_device_command.rem_dev_cmd_ioctl.rem_dev_cmd_ioctl_header.ui16IoCtlFunction );
  int32u ui32IntKeybits;
  bool bDefaultResponse = true;

//  rfprintf( &dev_com2, "Keyboard IOCTL %d\n", nFunc );
  if ( 0 != dev && slip_device != dev )
  {
    switch ( nFunc )
    {
      case IOCTL_KBD_SET_BASIC:
      case IOCTL_KBD_SET_REPEAT:
      case IOCTL_KBD_SET_HOLD:
      case IOCTL_KBD_SET_DISABLED:
//        rfprintf( &dev_com2, "Keyboard configure IOCTL\n" );
        nResult = rioctl( dev, nFunc,
                          prdCmd->rem_device_command.rem_dev_cmd_ioctl.remote_ioctl_command.rem_dev_cmd_ioctl_kbd_configure.ui8NumKeys,
                          prdCmd->rem_device_command.rem_dev_cmd_ioctl.remote_ioctl_command.rem_dev_cmd_ioctl_kbd_configure.ui8Buffer );
        break;

      case IOCTL_KBD_WRITE_HI_KEY_BITS:
//        rfprintf( &dev_com2, "Keyboard external keybits IOCTL\n" );
        nResult = rioctl( dev, IOCTL_KBD_WRITE_HI_KEY_BITS,
                          ntohl( prdCmd->rem_device_command.rem_dev_cmd_ioctl.remote_ioctl_command.rem_dev_cmd_ioctl_kbd_external_keybits.ui32ExternalKeybits ) );
        break;

      case IOCTL_KBD_INTERNAL_BITS:
        nResult = rioctl( dev, IOCTL_KBD_INTERNAL_BITS, &ui32IntKeybits );
        bDefaultResponse = false;
        vInitRemoteIoCtlRespHdr( prdResponse, prdCmd->rem_dev_cmd_header.ui8Device, IOCTL_KBD_INTERNAL_BITS );
        prdResponse->rem_device_response.rem_dev_resp_ioctl.remote_ioctl_response.rem_dev_resp_ioctl_kbd_internal_keybits.i32Result = htonl( nResult );
        prdResponse->rem_device_response.rem_dev_resp_ioctl.remote_ioctl_response.rem_dev_resp_ioctl_kbd_internal_keybits.ui32InternalKeybits = htonl( ui32IntKeybits );
        *pnLength = GET_SIZE_REMOTE_RESP_IOCTL_KBD_INT_KEYBITS();
        break;

      case IOCTL_KBD_IS_KEY_PRESSED: /* TODO */
      default:
        nResult = RIO_ERROR;
        break;
    }
  }
//  rfprintf( &dev_com2, "Keyboard IOCTL result %d\n", nResult );

  if ( bDefaultResponse )
  {
    vInitRemoteIoCtlResp_Default( prdResponse, prdCmd->rem_dev_cmd_header.ui8Device,
                                  nFunc, nResult );
    *pnLength = GET_SIZE_REMOTE_RESP_IOCTL_DEFAULT();
  }
//  rfprintf( &dev_com2, "Keyboard result length %d\n", *pnLength );
  return true;
}


/** Get the packet size of an Kbd IOCTL */
int nGetCmdPacketSizeIoCtl_Kbd( const struct s_rem_device_command *prdCmd )
{
  int nFunc = ntohs( prdCmd->rem_device_command.rem_dev_cmd_ioctl.rem_dev_cmd_ioctl_header.ui16IoCtlFunction );
  int nLength;

  switch ( nFunc )
  {
    case IOCTL_KBD_SET_BASIC:
    case IOCTL_KBD_SET_REPEAT:
    case IOCTL_KBD_SET_HOLD:
    case IOCTL_KBD_SET_DISABLED:
      nLength = GET_SIZE_REMOTE_CMD_IOCTL_KBD_CONFIGURE( prdCmd->rem_device_command.rem_dev_cmd_ioctl.remote_ioctl_command.rem_dev_cmd_ioctl_kbd_configure.ui8NumKeys );
      break;

    default:
      nLength = GET_SIZE_REMOTE_CMD_IOCTL_KBD_EXT_KEYBITS();
      break;
  }
  return nLength;
}
