/*
 * remotedemuxioctl_sio.c
 *
 *  Created on: 26/10/2008
 *      Author: ianb
 */
#include <rinhw/sio.h>
#include <rinhw/rio.h>
#include <rinhw/rinendian.h>
#include <rinhw/rstdio.h>
#include "devices.h"
#include "remotehelper.h"
#include "remotedemux.h"

#include "remotedemuxioctl_sio.h"


bool bHandleCommandIoCtl_Sio( const struct s_rem_device_command *prdCmd,
                              struct s_rem_device_response *prdResponse, int *pnLength )
{
  int nResult = RIO_ERROR;
  struct t_io_dev *dev = GetDevice( prdCmd->rem_dev_cmd_header.ui8Device );
  int nFunc = ntohs( prdCmd->rem_device_command.rem_dev_cmd_ioctl.rem_dev_cmd_ioctl_header.ui16IoCtlFunction );
  bool bDefaultResponse = true;
  t_ioctl_siocfg siocfg;
  int32u ui32Level;
  int16u ui16Level;
  int8u ui8Level;

//  rfprintf( &dev_com2, "SIO IOCTL %d\n", nFunc );
  if ( 0 != dev && slip_device != dev )
  {
    switch ( nFunc )
    {
      case IOCTL_SER_SET_CONFIG:
        siocfg = prdCmd->rem_device_command.rem_dev_cmd_ioctl.remote_ioctl_command.rem_dev_cmd_ioctl_sio_set_config.sio_config;
        siocfg.baud = ntohl( siocfg.baud );
        nResult = rioctl( dev, IOCTL_SER_SET_CONFIG, &siocfg );
        break;

      case IOCTL_SER_GET_CONFIG:
        nResult = rioctl( dev, IOCTL_SER_GET_CONFIG, &siocfg );
        bDefaultResponse = false;
        vInitRemoteIoCtlRespHdr( prdResponse, prdCmd->rem_dev_cmd_header.ui8Device, IOCTL_SER_GET_CONFIG );
        siocfg.baud = ntohl( siocfg.baud );
        prdResponse->rem_device_response.rem_dev_resp_ioctl.remote_ioctl_response.rem_dev_resp_ioctl_sio_get_config.i32Result = htonl( nResult );
        prdResponse->rem_device_response.rem_dev_resp_ioctl.remote_ioctl_response.rem_dev_resp_ioctl_sio_get_config.sio_config = siocfg;
        *pnLength = GET_SIZE_REMOTE_RESP_IOCTL_SIO_GET_CONFIG();
        break;

      case IOCTL_SER_GET_RX_BUF_LEVEL:
      case IOCTL_SER_GET_TX_BUF_LEVEL:
      case IOCTL_SER_GET_RX_BUF_SIZE:
      case IOCTL_SER_GET_TX_BUF_SIZE:
        nResult = rioctl( dev, nFunc, &ui16Level );
        bDefaultResponse = false;
        vInitRemoteIoCtlRespHdr( prdResponse, prdCmd->rem_dev_cmd_header.ui8Device, nFunc );
        prdResponse->rem_device_response.rem_dev_resp_ioctl.remote_ioctl_response.rem_dev_resp_ioctl_sio_get_level.i32Result = htonl( nResult );
        prdResponse->rem_device_response.rem_dev_resp_ioctl.remote_ioctl_response.rem_dev_resp_ioctl_sio_get_level.ui32Level = htonl( ui16Level );
        *pnLength = GET_SIZE_REMOTE_RESP_IOCTL_SIO_GET_LEVEL();
        break;

      case IOCTL_SER_GET_CTS_STATE:
        nResult = rioctl( dev, IOCTL_SER_GET_CTS_STATE, &ui8Level );
        bDefaultResponse = false;
        vInitRemoteIoCtlRespHdr( prdResponse, prdCmd->rem_dev_cmd_header.ui8Device, IOCTL_SER_GET_CTS_STATE );
        prdResponse->rem_device_response.rem_dev_resp_ioctl.remote_ioctl_response.rem_dev_resp_ioctl_sio_get_level.i32Result = htonl( nResult );
        prdResponse->rem_device_response.rem_dev_resp_ioctl.remote_ioctl_response.rem_dev_resp_ioctl_sio_get_level.ui32Level = htonl( ui8Level );
        *pnLength = GET_SIZE_REMOTE_RESP_IOCTL_SIO_GET_LEVEL();
        break;

      case IOCTL_SER_GET_RX_ERR_COUNT:
        nResult = rioctl( dev, IOCTL_SER_GET_RX_ERR_COUNT, &ui32Level );
        bDefaultResponse = false;
        vInitRemoteIoCtlRespHdr( prdResponse, prdCmd->rem_dev_cmd_header.ui8Device, IOCTL_SER_GET_RX_ERR_COUNT );
        prdResponse->rem_device_response.rem_dev_resp_ioctl.remote_ioctl_response.rem_dev_resp_ioctl_sio_get_level.i32Result = htonl( nResult );
        prdResponse->rem_device_response.rem_dev_resp_ioctl.remote_ioctl_response.rem_dev_resp_ioctl_sio_get_level.ui32Level = htonl( ui32Level );
        *pnLength = GET_SIZE_REMOTE_RESP_IOCTL_SIO_GET_LEVEL();
        break;

      case IOCTL_SER_SET_RING_NETWORK:
        nResult = rioctl( dev, IOCTL_SER_SET_RING_NETWORK,
                          prdCmd->rem_device_command.rem_dev_cmd_ioctl.remote_ioctl_command.rem_dev_cmd_ioctl_sio_set_ring_network.ui8Enabled );
        break;

      default:
        nResult = RIO_ERROR;
        break;
    }
  }
//  rfprintf( &dev_com2, "SIO IOCTL result %d\n", nResult );

  if ( bDefaultResponse )
  {
    vInitRemoteIoCtlResp_Default( prdResponse, prdCmd->rem_dev_cmd_header.ui8Device,
                                  nFunc, nResult );
    *pnLength = GET_SIZE_REMOTE_RESP_IOCTL_DEFAULT();
  }
//  rfprintf( &dev_com2, "SIO result length %d\n", *pnLength );
  return true;
}


/** Get the packet size of an Sio IOCTL */
int nGetCmdPacketSizeIoCtl_Sio( const struct s_rem_device_command *prdCmd )
{
  int nMax = ( GET_SIZE_REMOTE_CMD_IOCTL_SIO_SET_CONFIG() > GET_SIZE_REMOTE_CMD_IOCTL_SIO_SET_RING_NETWORK() )
             ? GET_SIZE_REMOTE_CMD_IOCTL_SIO_SET_CONFIG()
             : GET_SIZE_REMOTE_CMD_IOCTL_SIO_SET_RING_NETWORK();
  return nMax;
}
