/*
 * remoteserial.c
 *
 *  Created on: 21/08/2009
 *      Author: ianb
 */
#include <stddef.h>
#include <pthread.h>
#include <rinhw/stdrinstrum.h>
#include <rinhw/ucos_ii.h>
#include <rinremote/slip.h>

#include "remoteserial.h"


static pthread_mutex_t serial_tx_lock;


/** Send the data in pui8TxBuffer encapsulated into a slip packet over the serial port. Gets exclusive access to write the serial port. */
void vExclusiveSlipSendPacket(struct s_slipio *slipio, const int8u *pui8TxBuffer, int nTxLength)
{
  /* Get exclusive access to write the serial port */
  pthread_mutex_lock(&serial_tx_lock);

  /* Send the packet */
  slip_send_packet(slipio, pui8TxBuffer, nTxLength);

  /* Release exclusive access */
  pthread_mutex_unlock(&serial_tx_lock);
}


/** Initialise the exclusive slip packet sending */
void vInitExclusiveSlipSendPacket(void)
{
  pthread_mutex_init(&serial_tx_lock, NULL);
}


/** Destroy the exclusive slip packet sending */
void vDestroyExclusiveSlipSendPacket(void)
{
  pthread_mutex_destroy(&serial_tx_lock);
}
