/*
 * alibi_changelog.c
 *
 *  Created on: Feb 20, 2012
 *      Author: ian
 */
#include <string.h>
#include <rinweigh/adcalibi.h>
#include <rindb/db_changelog_reader.h>
#include <rinalibi/alibi_kbd.h>
#include <rinalibi/alibi_config.h>
#include <rinalibi/alibi_error.h>
#include <rinalibi/alibi_mode.h>
#include <rinalibi/alibi_display.h>
#include <rinalibi/alibi_quit.h>
#include <rinalibi/alibi_datetime.h>
#include <rinalibi/alibi_weight.h>
#include <rinalibi/alibi_scroll.h>
#include <rinalibi/alibi_editnum.h>

#include <rinalibi/alibi_changelog.h>


/** Item within the change log to display */
enum eAlibiChangelogDisplayItem
{
  acdiName,
  acdiIndex,
  acdiOldValue,
  acdiNewValue,
  acdiDate,
  acdiTime,
  acdiCalCounter,
  acdiConfigCounter,

  acdiNumItems
};


/** Get the display item name (essentially the name of the item in the change log entry structure) */
static const char *AdcIntAlibiChangelogGetDisplayItemName(struct s_adc_alibi_info *adc_alibi_info, enum eAlibiChangelogDisplayItem display_item)
{
  const char *result = NULL;

  switch (display_item)
  {
    default:
    case acdiName:
      result = AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiChangeLogName);
      break;
    case acdiIndex:
      result = AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiChangeLogIndex);
      break;
    case acdiOldValue:
      result = AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiChangeLogOldValue);
      break;
    case acdiNewValue:
      result = AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiChangeLogNewValue);
      break;
    case acdiDate:
      result = AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiDate);
      break;
    case acdiTime:
      result = AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiTime);
      break;
    case acdiCalCounter:
      result = AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiCalCount);
      break;
    case acdiConfigCounter:
      result = AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiConfigCount);
      break;
  }

  if (NULL == result)
  {
    result = "??????";
  }

  return result;
}


/** Display the item value (essentially the value of the item in the change log entry structure) */
static void AdcIntAlibiChangelogGetSingleItemValue(struct s_adc_alibi_info *adc_alibi_info, const struct s_db_changelog_entry *cle, enum eAlibiChangelogDisplayItem display_item, const char **pstr, AdcInfo *info, bool *scroll)
{
  AdcRdg rdg;
  t_datetime dt;
  int nd;
  char *wb;

  memset(info, 0, sizeof(*info));

  switch (display_item)
  {
    default:
    case acdiName:
      /* Try menu name first, then use db name */
      *scroll = true;
      if (NULL != cle->menu_elem_name && 0 != strcmp(cle->menu_elem_name, ""))
        *pstr = cle->menu_elem_name;
      else
        *pstr = cle->db_elem_name;
      break;

    case acdiIndex:
      *scroll = false;
      if (cle->has_index)
      {
        memset(&rdg, 0, sizeof(rdg));
        rdg.rdg = cle->index;

        nd = adc_alibi_info->num_digits[aadtSingleDisplay];
        wb = adc_alibi_info->display_work_buffers[aadtSingleDisplay];
        AdcIntAlibiWeightToStr(adc_alibi_info, &rdg, nd, wb, ADC_ALIBI_DISPLAY_WORK_BUF_LEN(nd));
        *pstr = wb;
      }
      else
      {
        *pstr = "";
      }
      break;

    case acdiOldValue:
      *scroll = true;
      *pstr = cle->old_value;
      break;

    case acdiNewValue:
      *scroll = true;
      *pstr = cle->new_value;
      break;

    case acdiDate:
      *scroll = false;
      AdcIntAlibiTmToDateTime(&cle->timestamp, &dt);
      nd = adc_alibi_info->num_digits[aadtSingleDisplay];
      wb = adc_alibi_info->display_work_buffers[aadtSingleDisplay];
      AdcIntAlibiDateToStr(&dt, wb, ADC_ALIBI_DISPLAY_WORK_BUF_LEN(nd));
      *pstr = wb;
      break;

    case acdiTime:
      *scroll = false;
      AdcIntAlibiTmToDateTime(&cle->timestamp, &dt);
      nd = adc_alibi_info->num_digits[aadtSingleDisplay];
      wb = adc_alibi_info->display_work_buffers[aadtSingleDisplay];
      AdcIntAlibiTimeToStr(&dt, wb, ADC_ALIBI_DISPLAY_WORK_BUF_LEN(nd));
      *pstr = wb;
      break;

    case acdiCalCounter:
      *scroll = false;
      *pstr = cle->cal_counter;
      break;

    case acdiConfigCounter:
      *scroll = false;
      *pstr = cle->config_counter;
      break;
  }
}


/** Display the item value (essentially the value of the item in the change log entry structure) */
static void AdcIntAlibiChangelogDisplayDualItemValue(struct s_adc_alibi_info *adc_alibi_info, const struct s_db_changelog_entry *cle, enum eAlibiChangelogDisplayItem display_item, const char *msg, int32u scroll_start_time)
{
  t_datetime dt;
  AdcInfo i1;
  AdcInfo i2;
  char *d2;
  size_t nd2;
  AdcRdg rdg;
  bool scroll_s2;

  nd2 = adc_alibi_info->num_digits[aadtDoubleDisplay];
  scroll_s2 = true;

  /* Display 2 */
  switch (display_item)
  {
    default:
    case acdiName:
      /* Try menu name first, then use db name */
      if (NULL != cle->menu_elem_name && 0 != strcmp(cle->menu_elem_name, ""))
        d2 = cle->menu_elem_name;
      else
        d2 = cle->db_elem_name;
      break;

    case acdiIndex:
      if (cle->has_index)
      {
        memset(&rdg, 0, sizeof(rdg));
        rdg.rdg = cle->index;
        d2 = adc_alibi_info->display_work_buffers[aadtDoubleDisplay];
        AdcIntAlibiWeightToStr(adc_alibi_info, &rdg, nd2, d2, ADC_ALIBI_DISPLAY_WORK_BUF_LEN(nd2));
        scroll_s2 = false;
      }
      else
      {
        d2 = "";
      }
      break;

    case acdiOldValue:
      d2 = cle->old_value;
      break;

    case acdiNewValue:
      d2 = cle->new_value;
      break;

    case acdiDate:
      AdcIntAlibiTmToDateTime(&cle->timestamp, &dt);
      d2 = adc_alibi_info->display_work_buffers[aadtDoubleDisplay];
      AdcIntAlibiDateToStr(&dt, d2, ADC_ALIBI_DISPLAY_WORK_BUF_LEN(nd2));
      scroll_s2 = false;
      break;

    case acdiTime:
      AdcIntAlibiTmToDateTime(&cle->timestamp, &dt);
      d2 = adc_alibi_info->display_work_buffers[aadtDoubleDisplay];
      AdcIntAlibiTimeToStr(&dt, d2, ADC_ALIBI_DISPLAY_WORK_BUF_LEN(nd2));
      scroll_s2 = false;
      break;

    case acdiCalCounter:
      d2 = cle->cal_counter;
      break;

    case acdiConfigCounter:
      d2 = cle->config_counter;
      break;
  }

  /* Both displays have no annunciators or units */
  memset(&i1, 0, sizeof(i1));
  memset(&i2, 0, sizeof(i2));

  /* Display it */
  AdcIntAlibiDualDisplayStrScroll(adc_alibi_info, msg, &i1, false, 0, d2, &i2, scroll_s2, scroll_start_time);
}


/** Display a traceable weight on a single display */
static void AdcIntAlibiChangelogDisplayEntrySingle(struct s_adc_alibi_info *adc_alibi_info, const struct s_db_changelog_entry *cle,
                                                   enum eAlibiChangelogDisplayItem display_item, int32u dpy_alternate_start_time,
                                                   int32u *scroll_start_time)
{
  int32u divisor;
  int32u modulo;
  const char *s;
  const char *value_str;
  AdcInfo value_info;
  AdcInfo i;
  bool scroll;

  /* Get the value string because we need it's length */
  AdcIntAlibiChangelogGetSingleItemValue(adc_alibi_info, cle, display_item, &value_str, &value_info, &scroll);

  /* Display "Alibi" alternating display item name and value which can scroll */
  if (scroll)
    divisor = 2 + AdcIntAlibiScrollSingleDirnGetNumScrollSteps(value_str, adc_alibi_info->num_digits[aadtSingleDisplay]);
  else
    divisor = 2 + 1;

  modulo = ((OSTimeGet() - dpy_alternate_start_time) / ADC_ALIBI_CONFIG_SCROLL_RATE) % divisor;
  switch (modulo)
  {
    case 0:
      /* Display "alibi" */
      memset(&i, 0, sizeof(i));
      s = AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiAlibi);
      AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, s);
      *scroll_start_time = 0;
      break;

    case 1:
      /* Display item name */
      memset(&i, 0, sizeof(i));
      s = AdcIntAlibiChangelogGetDisplayItemName(adc_alibi_info, display_item);
      AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, s);
      *scroll_start_time = 0;
      break;

    default:
      /* Display item value (scrolling if longer than display width) */
      if (scroll)
      {
        if (0 == *scroll_start_time)
        {
          *scroll_start_time = OSTimeGet();
        }
        AdcIntAlibiSingleDisplayStrScroll(adc_alibi_info, value_str, &value_info, scroll, *scroll_start_time, false);
      }
      else
      {
        AdcIntAlibiSingleDisplayStr(adc_alibi_info, value_str, &value_info);
      }
      break;
  }
}


/** Display a change log entry on a double display */
static void AdcIntAlibiChangelogDisplayEntryDouble(struct s_adc_alibi_info *adc_alibi_info, const struct s_db_changelog_entry *cle, enum eAlibiChangelogDisplayItem display_item, int32u scroll_start_time)
{
  int32u divisor;
  int32u modulo;
  const char *msg;

  /* Display "Alibi" alternating display item name on one display, and the value on the other */
  divisor = 2;
  modulo = (OSTimeGet() / OS_TICKS_PER_SEC) % divisor;
  switch (modulo)
  {
    case 0:
      /* Display "alibi" */
      msg = AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiAlibi);
      break;

    case 1:
      /* Display item name */
      msg = AdcIntAlibiChangelogGetDisplayItemName(adc_alibi_info, display_item);
      break;
  }
  AdcIntAlibiChangelogDisplayDualItemValue(adc_alibi_info, cle, display_item, msg, scroll_start_time);
}


/** Display a change log entry */
static void AdcIntAlibiChangelogDisplayEntry(struct s_adc_alibi_info *adc_alibi_info, const struct s_db_changelog_entry *cle, enum eAlibiChangelogDisplayItem display_item, int32u scroll_start_time1, int32u *scroll_start_time2)
{
  if (aadtSingleDisplay == adc_alibi_info->adc_alibi_display_type)
  {
    AdcIntAlibiChangelogDisplayEntrySingle(adc_alibi_info, cle, display_item, scroll_start_time1, scroll_start_time2);
  }
  else
  {
    AdcIntAlibiChangelogDisplayEntryDouble(adc_alibi_info, cle, display_item, scroll_start_time1);
  }
}


/** Display change log errors on a single display */
static void AdcIntAlibiChangelogDisplayErrorSingle(struct s_adc_alibi_info *adc_alibi_info, enum eAdcAlibiInternationisationStringIdx err)
{
  int32u divisor;
  int32u modulo;

  /* Display "CL.ERR" alternating with the error message */
  divisor = 2;
  modulo = (OSTimeGet() / OS_TICKS_PER_SEC) % divisor;
  switch (modulo)
  {
    case 0:
      /* Display "CL.ERR" */
      AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiChangeLogError));
      break;

    case 1:
      /* Display error */
      AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, err));
      break;
  }
}


/** Display change log errors on a double display */
static void AdcIntAlibiChangelogDisplayErrorDouble(struct s_adc_alibi_info *adc_alibi_info, enum eAdcAlibiInternationisationStringIdx err)
{
  int32u divisor;
  int32u modulo;
  const char *s1;
  const char *s2;

  /* "Alibi" on first display */
  s1 = AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiAlibi);

  /* Display "CL.ERR" alternating with the error message */
  divisor = 2;
  modulo = (OSTimeGet() / OS_TICKS_PER_SEC) % divisor;
  switch (modulo)
  {
    case 0:
      /* Display "CL.ERR" */
      s2 = AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiChangeLogError);
      break;

    case 1:
      /* Display error */
      s2 = AdcIntAlibiGetInternationisation(adc_alibi_info, err);
      break;
  }

  AdcIntAlibiDualDisplayStrNoAnnun(adc_alibi_info, s1, s2);
}


/** Display change log errors */
static void AdcIntAlibiChangelogDisplayError(struct s_adc_alibi_info *adc_alibi_info, enum eAdcAlibiInternationisationStringIdx err)
{
  if (aadtSingleDisplay == adc_alibi_info->adc_alibi_display_type)
  {
    AdcIntAlibiChangelogDisplayErrorSingle(adc_alibi_info, err);
  }
  else
  {
    AdcIntAlibiChangelogDisplayErrorDouble(adc_alibi_info, err);
  }
}


static void AdcIntAlibiChangelogShowSeekMessage(struct s_adc_alibi_info *adc_alibi_info, enum eDBChangeLogReaderLocator locator)
{
  enum eAdcAlibiInternationisationStringIdx stridx;

  switch (locator)
  {
    case dbclrlFirst:
      stridx = aaisiFirst;
      break;

    case dbclrlLast:
      stridx = aaisiLast;
      break;

    case dbclrlNext:
      stridx = aaisiNext;
      break;

    case dbclrlPrevious:
      stridx = aaisiPrevious;
      break;

    default:
      stridx = aaisiChangeLogError;
      break;
  }

  (void) AdcIntAlibiShowMessageIndex(adc_alibi_info, stridx, ADC_ALIBI_CONFIG_MESSAGE_TIMEOUT);
}


static enum eDBError AdcIntAlibiChangelogSeek(struct s_adc_alibi_info *adc_alibi_info, struct s_db_changelog_entry *cle, enum eDBChangeLogReaderLocator locator, int32u *cal_counter_search)
{
  enum eDBError res;

  res = db_changelog_reader_entry_get(adc_alibi_info->changelog_reader, locator, cle);
  if (dbeSuccess != res)
  {
    (void) AdcIntAlibiShowErrorIndexes(adc_alibi_info, aaisiChangeLogError, aaisiDSDSearch, ADC_ALIBI_CONFIG_ERROR_TIMEOUT);
  }
  else
  {
    AdcIntAlibiChangelogShowSeekMessage(adc_alibi_info, locator);
    if (NULL != cle->cal_counter || 0 != strcmp("", cle->cal_counter))
    {
      *cal_counter_search = strtoul(cle->cal_counter, NULL, 10);
    }
  }
  return res;
}


/** Select a calibration counter value */
static void AdcIntAlibiChangelogSelect(struct s_adc_alibi_info *adc_alibi_info, struct s_db_changelog_entry *cle, enum eDBError *changelog_result, int32u *cal_counter_search)
{
  bool found;
  AdcRdg cal_cnt_rdg;
  int32u cal_cnt;
  int32u last_cal_cnt;

  /* Prompt */
  AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiEnter));
  OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);
  AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiCalCount));
  OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);

  /* Edit calibration counter value */
  memset(&cal_cnt_rdg, 0, sizeof(cal_cnt_rdg));
  cal_cnt_rdg.rdg = *cal_counter_search;
  if (!AdcIntAlibiEditNumber(adc_alibi_info, &cal_cnt_rdg, 0, 999999, false))
  {
    /* User cancelled */
    return;
  }

  /* Search change log for calibration counter value */
  for (found = false, last_cal_cnt = 0, *changelog_result = db_changelog_reader_entry_get(adc_alibi_info->changelog_reader, dbclrlFirst, cle);
       (dbeSuccess == *changelog_result);
       *changelog_result = db_changelog_reader_entry_get(adc_alibi_info->changelog_reader, dbclrlNext, cle))
  {
    cal_cnt = strtoul(cle->cal_counter, NULL, 10);
    if (cal_cnt == (int32u) cal_cnt_rdg.rdg ||
        (last_cal_cnt < (int32u) cal_cnt_rdg.rdg && cal_cnt > (int32u) cal_cnt_rdg.rdg))
    {
      found = true;
      break;
    }
  }

  *cal_counter_search = cal_cnt_rdg.rdg;

  if (!found)
  {
    (void) AdcIntAlibiShowErrorIndexes(adc_alibi_info, aaisiChangeLogError, aaisiDSDSearch, ADC_ALIBI_CONFIG_ERROR_TIMEOUT);
    return;
  }
}


/** Should this display item be displayed (we may not show index, calibration or config counter, depending on what is in the changelog entry) */
static bool AdcIntAlibiChangelogShouldDisplayItem(struct s_db_changelog_entry *cle, enum eAlibiChangelogDisplayItem display_item)
{
  bool result;

  switch (display_item)
  {
    case acdiIndex:
      /* Special check to skip over acdiIndex if item has no index */

      result = cle->has_index;
      break;
    case acdiCalCounter:
      /* Special check to skip over acdiCalCounter if item has no cal counter value */

      result = NULL != cle->cal_counter && 0 != strcmp("", cle->cal_counter);
      break;
    case acdiConfigCounter:
      /* Special check to skip over acdiConfigCounter if item has no config counter value */

      result = NULL != cle->config_counter && 0 != strcmp("", cle->config_counter);
      break;
    default:
      result = true;
      break;
  }
  return result;
}


/** Display item has changed, check and change to next/previous item if the new selection isn't shown */
static void AdcIntAlibiChangelogFirstLastDisplayItemChange(struct s_db_changelog_entry *cle, enum eAlibiChangelogDisplayItem *display_item, bool first_not_last)
{
  if (first_not_last)
  {
    /* Select first item, and check forward to find the next displayable item */
    *display_item = 0;
    while (!AdcIntAlibiChangelogShouldDisplayItem(cle, *display_item))
    {
      (*display_item)++;
      if (*display_item >= acdiNumItems)
        *display_item = 0;
    }
  }
  else
  {
    /* Select last item, and check backwards to find previous displayable item */
    *display_item = acdiNumItems - 1;
    while (!AdcIntAlibiChangelogShouldDisplayItem(cle, *display_item))
    {
      if (0 == *display_item || *display_item >= acdiNumItems)
        *display_item = acdiNumItems - 1;
      else
        (*display_item)--;
    }
  }
}


/** Advance to the next display item, or retreat to the previous display item, based on advance */
static void AdcIntAlibiChangelogNextPrevDisplayItem(struct s_db_changelog_entry *cle, enum eAlibiChangelogDisplayItem *display_item, int advance)
{
  if (advance > 0)
  {
    /* Advance forward, find next displayable item */
    do
    {
      (*display_item)++;
      if (*display_item >= acdiNumItems)
        *display_item = 0;
    }
    while (!AdcIntAlibiChangelogShouldDisplayItem(cle, *display_item));
  }
  else if (advance < 0)
  {
    /* Retreat backward, find previous displayable item */
    do
    {
      if (0 == *display_item || *display_item >= acdiNumItems)
        *display_item = acdiNumItems - 1;
      else
        (*display_item)--;
    }
    while (!AdcIntAlibiChangelogShouldDisplayItem(cle, *display_item));
  }
}


/** Process keystroke for the change log application */
static void AdcIntAlibiChangelogProcessKey(struct s_adc_alibi_info *adc_alibi_info, struct s_db_changelog_entry *cle, enum eAlibiChangelogDisplayItem *display_item, enum eDBError *changelog_result, int32u *scroll_start_time, int32u *cal_counter_search)
{
  enum eAdcAlibiKeystroke key;

  /* Process keys */
  AdcIntAlibiGetKey(adc_alibi_info, &key, ADC_ALIBI_CONFIG_DISPLAY_UPDATE_HZ);

  /* Dispatch */
  switch (key)
  {

    /* Quit alibi mode */
    case aakQuit:
      adc_alibi_info->alibi_mode = aamQuitAlibi;
      break;

    /* Jump to first (oldest) change log record */
    case aakDSDRecordFirst:
      *changelog_result = AdcIntAlibiChangelogSeek(adc_alibi_info, cle, dbclrlFirst, cal_counter_search);
      *scroll_start_time = OSTimeGet();
      break;

    /* Jump to last (newest) change log record */
    case aakDSDRecordLast:
      *changelog_result = AdcIntAlibiChangelogSeek(adc_alibi_info, cle, dbclrlLast, cal_counter_search);
      *scroll_start_time = OSTimeGet();
      break;

    /* Advance to next change log record */
    case aakDSDRecordNext:
      *changelog_result = AdcIntAlibiChangelogSeek(adc_alibi_info, cle, dbclrlNext, cal_counter_search);
      *scroll_start_time = OSTimeGet();
      break;

    /* Back to previous change log record */
    case aakDSDRecordPrevious:
      *changelog_result = AdcIntAlibiChangelogSeek(adc_alibi_info, cle, dbclrlPrevious, cal_counter_search);
      *scroll_start_time = OSTimeGet();
      break;

    /* Find the first entry with specific calibration counter */
    case aakDSDRecordSelect:
      AdcIntAlibiChangelogSelect(adc_alibi_info, cle, changelog_result, cal_counter_search);
      *scroll_start_time = OSTimeGet();
      break;

    /* First item within the change log entry */
    case aakDSDItemFirst:
      AdcIntAlibiChangelogFirstLastDisplayItemChange(cle, display_item, true);
      *scroll_start_time = OSTimeGet();
      break;

    /* Last item within the change log entry */
    case aakDSDItemLast:
      AdcIntAlibiChangelogFirstLastDisplayItemChange(cle, display_item, false);
      *scroll_start_time = OSTimeGet();
      break;

    /* Reverse through items within the change log entry */
    case aakDSDItemPrevious:
      AdcIntAlibiChangelogNextPrevDisplayItem(cle, display_item, -1);
      *scroll_start_time = OSTimeGet();
      break;

    /* Advance through items within the change log entry */
    case aakDSDItemNext:
      AdcIntAlibiChangelogNextPrevDisplayItem(cle, display_item, 1);
      *scroll_start_time = OSTimeGet();
      break;

    /* Change alibi mode */
    case aakMode:
      AdcIntAlibiSelectMode(adc_alibi_info);
      break;

    case aakNone:
    default:
      /* Do nothing with other keys */
      break;
  } /* switch */
}


/** Run the alibi change log viewer application */
void AdcIntAlibiChangeLogViewer(struct s_adc_alibi_info *adc_alibi_info)
{
  enum eAlibiChangelogDisplayItem display_item;
  enum eDBError changelog_result;
  struct s_db_changelog_entry *cle;
  int32u scroll_start_time1;
  int32u scroll_start_time2 = 0;
  int32u cal_counter_search;

  /* Check the change log is working/available */
  if (NULL == adc_alibi_info->changelog_reader ||
      dbeSuccess != db_changelog_reader_entry_create(&cle))
  {
    /* No change log so generate error, and back to weighing mode */
    AdcIntAlibiShowErrorIndexes(adc_alibi_info, aaisiNo, aaisiChangeLog, ADC_ALIBI_CONFIG_ERROR_TIMEOUT);
    adc_alibi_info->alibi_mode = aamWeighing;
    return;
  }

  /* Setup */
  display_item = 0;
  cal_counter_search = 0;

  /* Find last record */
  changelog_result = AdcIntAlibiChangelogSeek(adc_alibi_info, cle, dbclrlLast, &cal_counter_search);
  scroll_start_time1 = OSTimeGet();

  /* Run change log viewer */
  for (; aamChangelogView == adc_alibi_info->alibi_mode; )
  {
    /* Display the change log entry, or error if not valid */
    switch (changelog_result)
    {
      case dbeSuccess:
        AdcIntAlibiChangelogDisplayEntry(adc_alibi_info, cle, display_item, scroll_start_time1, &scroll_start_time2);
        break;

      case dbeChangelogNoMoreRecords:
        AdcIntAlibiChangelogDisplayError(adc_alibi_info, aaisiDSDEmpty);
        break;

      default:
        AdcIntAlibiChangelogDisplayError(adc_alibi_info, aaisiFailed);
        break;
    }

    /* Process key */
    AdcIntAlibiChangelogProcessKey(adc_alibi_info, cle, &display_item, &changelog_result, &scroll_start_time1, &cal_counter_search);

    /* Should we quit? */
    if (AdcIntAlibiForceQuit(adc_alibi_info))
    {
      adc_alibi_info->alibi_mode = aamQuitAlibi;
    }
  } /* for () */

  /* Clean up */
  db_changelog_reader_entry_destroy(cle);
}
