/*
 * alibi_counting.c
 *
 *  Created on: Feb 21, 2012
 *      Author: ian
 */
#include <stdio.h>
#include <string.h>
#include "alibi_config.h"
#include <rinweigh/adcalibi.h>
#include <rinweigh/adcinterface.h>
#include "alibi_display.h"
#include "alibi_editnum.h"
#include "alibi_kbd.h"
#include "alibi_mode.h"
#include "alibi_error.h"
#include "alibi_weighing.h"
#include "alibi_quit.h"
#include "alibi_weight.h"

#include "alibi_counting.h"


/** Display the pieces on a single display */
static void AdcIntAlibiCountingDisplayPiecesSingle(struct s_adc_alibi_info *adc_alibi_info)
{
  int32u error;
  int32u divisor;
  int32u modulo;
  AdcRdg rdg;
  char *s;
  int nd;

  AdcMutexLock(adc_alibi_info->ass);
  error = AdcIntConvertAdcErrorsToApplicationErrors(adc_alibi_info, adc_alibi_info->ass->adcDataWorking.error);
  rdg = adc_alibi_info->ass->adcDataWorking.dispPieceRdg;
  AdcMutexUnlock(adc_alibi_info->ass);

  /* Display "Alibi" alternating with errors (if present) and pieces, and "pieces" if no capability to display pieces units */
  divisor = 2;
  if (error)
  {
    divisor++;
  }
  if (!adc_alibi_info->status_can_show_pieces &&
      !rdg.info.status.bit.oload &&
      !rdg.info.status.bit.uload)
  {
    divisor++;
  }
  modulo = (OSTimeGet() / OS_TICKS_PER_SEC) % divisor;
  if (0 == modulo)
  {
    /* Display "alibi" */
    AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiAlibi));
  }
  else if (1 == modulo || (2 == modulo && !error))
  {
    /* Display over/under/weight */
    if (rdg.info.status.bit.oload)
    {
      AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiOverload));
    }
    else if (rdg.info.status.bit.uload)
    {
      AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiUnderload));
    }
    else if (1 == modulo)
    {
      AdcIntAlibiSingleDisplayWeight(adc_alibi_info, &rdg);
    }
    else
    {
      AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiCountPieces));
    }
  }
  else
  {
    /* Display error */
    s = adc_alibi_info->display_work_buffers[aadtSingleDisplay];
    nd = adc_alibi_info->num_digits[aadtSingleDisplay];
    AdcIntAlibiErrorToStr(adc_alibi_info, error, nd, s, ADC_ALIBI_DISPLAY_WORK_BUF_LEN(nd));
    AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, s);
  }
}


/** Display the pieces on a double display */
static void AdcIntAlibiCountingDisplayPiecesDouble(struct s_adc_alibi_info *adc_alibi_info)
{
  int32u error;
  int32u divisor;
  int32u modulo;
  AdcRdg rdg;
  const char *s1;
  const char *s2;
  AdcInfo i1;
  AdcInfo i2;
  /* int nd1; */
  int nd2;

  /* nd1 = adc_alibi_info->num_digits[aadtSingleDisplay]; */
  nd2 = adc_alibi_info->num_digits[aadtDoubleDisplay];

  AdcMutexLock(adc_alibi_info->ass);
  error = AdcIntConvertAdcErrorsToApplicationErrors(adc_alibi_info, adc_alibi_info->ass->adcDataWorking.error);
  rdg = adc_alibi_info->ass->adcDataWorking.dispPieceRdg;
  AdcMutexUnlock(adc_alibi_info->ass);

  /* Display "alibi" on first display */
  s1 = AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiAlibi);
  memset(&i1, 0, sizeof(i1));

  /* Display errors (if present) alternating with pieces, and "pieces" if no capability to display pieces units */
  divisor = 1;
  if (error)
  {
    divisor++;
  }
  if (!adc_alibi_info->status_can_show_pieces &&
      !rdg.info.status.bit.oload &&
      !rdg.info.status.bit.uload)
  {
    divisor++;
  }
  modulo = (OSTimeGet() / OS_TICKS_PER_SEC) % divisor;
  memset(&i2, 0, sizeof(i2));
  if (0 == modulo || (1 == modulo && !error))
  {
    /* Display over/under/pieces */
    if (rdg.info.status.bit.oload)
    {
      s2 = AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiOverload);
    }
    else if (rdg.info.status.bit.uload)
    {
      s2 = AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiUnderload);
    }
    else if (0 == modulo)
    {
      AdcIntAlibiWeightToStr(adc_alibi_info, &rdg, nd2, adc_alibi_info->display_work_buffers[aadtDoubleDisplay], ADC_ALIBI_DISPLAY_WORK_BUF_LEN(nd2));
      s2 = adc_alibi_info->display_work_buffers[aadtDoubleDisplay];
      i2 = rdg.info;
    }
    else
    {
      s2 = AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiCountPieces);
    }
  }
  else
  {
    /* Display error */
    AdcIntAlibiErrorToStr(adc_alibi_info, error, nd2, adc_alibi_info->display_work_buffers[aadtDoubleDisplay], ADC_ALIBI_DISPLAY_WORK_BUF_LEN(nd2));
    s2 = adc_alibi_info->display_work_buffers[aadtDoubleDisplay];
  }
  AdcIntAlibiDualDisplayStr(adc_alibi_info, s1, &i1, s2, &i2);
}


/** Display the weight */
static void AdcIntAlibiCountingDisplayPieces(struct s_adc_alibi_info *adc_alibi_info)
{
  if (aadtSingleDisplay == adc_alibi_info->adc_alibi_display_type)
  {
    AdcIntAlibiCountingDisplayPiecesSingle(adc_alibi_info);
  }
  else
  {
    AdcIntAlibiCountingDisplayPiecesDouble(adc_alibi_info);
  }
}


static void AdcIntAlibiCountingEnterQuantity(struct s_adc_alibi_info *adc_alibi_info, int8u *opcounter)
{
  int32s sampleSize;
  int32s cntWeight;
  AdcRdg qty;
  AdcRdg current_rdg;
  int8u res;

  /* Prompt */
  AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiEnter));
  OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);
  AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiCountQuantity));
  OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);

  /* Edit quantity */
  memset(&qty, 0, sizeof(qty));
  qty.info.unitdp.bit.dp = 0;
  qty.info.unitdp.bit.units = ADC_UNITS_PIECES;
  qty.rdg = 1;
  if (AdcIntAlibiEditNumber(adc_alibi_info, &qty, 1, 999999, false))
  {
    /* Enable counting */
    AdcMutexLock(adc_alibi_info->ass);
    current_rdg = adc_alibi_info->ass->adcDataWorking.dispRdg;

    if (current_rdg.info.status.bit.hiRes)
      sampleSize = qty.rdg * 10;
    else
      sampleSize = qty.rdg;
    cntWeight = AdcFinalToCounts(adc_alibi_info->ass, false, current_rdg.rdg, false) + current_rdg.info.status.bit.gradPos;

    /* Would it succeed? */
    res = AdcSetCounting(adc_alibi_info->ass, false, false, true, sampleSize, cntWeight);
    AdcMutexUnlock(adc_alibi_info->ass);

    switch (res)
    {
      /* Yep, start counting */
      case ADC_OP_STATUS_SUCCESS:
        AdcMutexLock(adc_alibi_info->ass);
        (void) AdcSetCounting(adc_alibi_info->ass, false, false, false, sampleSize, cntWeight);
        *opcounter = AdcGetCurrentCommandCnt(adc_alibi_info->ass, false);
        AdcMutexUnlock(adc_alibi_info->ass);
        break;

      /* Nope, error message */
      case ADC_OP_STATUS_FAILED_MOTION:
        AdcIntAlibiShowErrorIndexes(adc_alibi_info, aaisiError, aaisiMotion, ADC_ALIBI_CONFIG_ERROR_TIMEOUT);
        break;
      default:
        AdcIntAlibiShowErrorIndexes(adc_alibi_info, aaisiError, aaisiFailed, ADC_ALIBI_CONFIG_ERROR_TIMEOUT);
        break;
    }
  }
}


/** Process keystroke in for the counting application */
static void AdcIntAlibiCountingProcessKey(struct s_adc_alibi_info *adc_alibi_info, int8u *opcounter, int8u *opopt, bool *show_pieces)
{
  enum eAdcAlibiKeystroke key;

  /* Process keys */
  AdcIntAlibiGetKey(adc_alibi_info, &key, ADC_ALIBI_CONFIG_DISPLAY_UPDATE_HZ);

  /* Dispatch */
  switch (key)
  {
    case aakQuit:
      adc_alibi_info->alibi_mode = aamQuitAlibi;
      break;

    case aakZero:
      AdcMutexLock(adc_alibi_info->ass);
      (void) AdcSemiautoZero(adc_alibi_info->ass, false, false, false, false);
      *opcounter = AdcGetCurrentCommandCnt(adc_alibi_info->ass, false);
      AdcMutexUnlock(adc_alibi_info->ass);
      break;

    case aakTare:
      AdcMutexLock(adc_alibi_info->ass);
      (void) AdcSemiautoTare(adc_alibi_info->ass, false, false, false, false);
      *opcounter = AdcGetCurrentCommandCnt(adc_alibi_info->ass, false);
      AdcMutexUnlock(adc_alibi_info->ass);
      break;

    case aakGrossNet:
      AdcMutexLock(adc_alibi_info->ass);
      *opopt = adc_alibi_info->ass->adcDataWorking.dispRdg.info.status.bit.gross;
      (void) AdcGrossNetToggle(adc_alibi_info->ass, false, false);
      /* *opcounter = AdcGetCurrentCommandCnt(); */
      AdcMutexUnlock(adc_alibi_info->ass);
      break;

    case aakMode:
      AdcIntAlibiSelectMode(adc_alibi_info);
      break;

    case aakCountEnterQty:
      AdcIntAlibiCountingEnterQuantity(adc_alibi_info, opcounter);
      break;

    case aakCountCancel:
      *show_pieces = false;
      break;

    case aakNone:
    default:
      /* Do nothing with other keys */
      break;
  } /* switch */
}


/** Run a simple alibi counting application */
void AdcIntAlibiCounting(struct s_adc_alibi_info *adc_alibi_info)
{
  int8u opcounter;
  int8u opopt;
  bool show_pieces;
  int8u cmd_status;
  int8u cmd_cnt;

  /* Do alibi weighing application loop */
  opcounter = ~AdcGetCurrentCommandCnt(adc_alibi_info->ass, true);
  for (show_pieces = false; aamCounting == adc_alibi_info->alibi_mode; )
  {
    if (show_pieces)
    {
      /* Display pieces */
      AdcIntAlibiCountingDisplayPieces(adc_alibi_info);
    }
    else
    {
      /* Display the weight */
      AdcIntAlibiWeighingDisplayWeight(adc_alibi_info);
    }

    /* Process keys */
    AdcMutexLock(adc_alibi_info->ass);
    cmd_status = AdcGetCurrentOpStatus(adc_alibi_info->ass, false);
    cmd_cnt = AdcGetCurrentCommandCnt(adc_alibi_info->ass, false);
    AdcMutexUnlock(adc_alibi_info->ass);

    if (ADC_OP_STATUS_IN_PROGRESS != cmd_status && cmd_cnt != opcounter)
    {
      AdcIntAlibiCountingProcessKey(adc_alibi_info, &opcounter, &opopt, &show_pieces);
    }

    /* Operation error messages and post-operation processing */
    AdcIntAlibiWeighingProcessAdcResult(adc_alibi_info, &opcounter, &opopt, &show_pieces);

    /* Should we quit? */
    if (AdcIntAlibiForceQuit(adc_alibi_info))
    {
      adc_alibi_info->alibi_mode = aamQuitAlibi;
    }
  } /* for () */
}
