/*
 * alibi_display.c
 *
 *  Created on: Feb 21, 2012
 *      Author: ian
 */
#include <stdio.h>
#include <string.h>
#include <rinweigh/adcalibi.h>
#include "alibi_weight.h"

#include "alibi_display.h"


/** Display a weight reading on the display */
void AdcIntAlibiSingleDisplayWeight(struct s_adc_alibi_info *adc_alibi_info, const AdcRdg *rdg)
{
  const char *s_ptrs[1];
  int nd;

  nd = adc_alibi_info->num_digits[aadtSingleDisplay];
  AdcIntAlibiWeightToStr(adc_alibi_info, rdg, nd, adc_alibi_info->display_work_buffers[aadtSingleDisplay], ADC_ALIBI_DISPLAY_WORK_BUF_LEN(nd));

  /* Display the weight */
  s_ptrs[0] = adc_alibi_info->display_work_buffers[aadtSingleDisplay];
  adc_alibi_info->adc_alibi_display(adc_alibi_info->implementer_data, adc_alibi_info->adc_alibi_display_type, s_ptrs, &rdg->info);
}


/** Set or clear all segments on the display */
void AdcIntAlibiSetAllSegs(struct s_adc_alibi_info *adc_alibi_info, bool on_not_off)
{
  adc_alibi_info->adc_alibi_display_allsegs(adc_alibi_info->implementer_data, adc_alibi_info->adc_alibi_display_type, on_not_off);
}


/** Get an internationisation string */
const char *AdcIntAlibiGetInternationisation(struct s_adc_alibi_info *adc_alibi_info, enum eAdcAlibiInternationisationStringIdx stridx)
{
  const char *ptr = NULL;

  adc_alibi_info->adc_alibi_get_internationalisation_string(adc_alibi_info->implementer_data, stridx, &ptr);
  if (NULL == ptr)
  {
    return "??????";
  }
  return ptr;
}


/** Display a string on a single display indicator */
void AdcIntAlibiSingleDisplayStr(struct s_adc_alibi_info *adc_alibi_info, const char *s, const AdcInfo *info)
{
  const char *s_ptrs[1] = { s };

  adc_alibi_info->adc_alibi_display(adc_alibi_info->implementer_data, adc_alibi_info->adc_alibi_display_type, s_ptrs, info);
}


/** Display a string on a single display indicator, with no annunciators */
void AdcIntAlibiSingleDisplayStrNoAnnun(struct s_adc_alibi_info *adc_alibi_info, const char *s)
{
  AdcInfo info;

  memset(&info, 0, sizeof(info));
  info.status.bit.grossOrNet = 0; /* Weight is not gross & not net */
  AdcIntAlibiSingleDisplayStr(adc_alibi_info, s, &info);
}


/** Display both strings on a dual display indicator (with annunciators) */
void AdcIntAlibiDualDisplayStr(struct s_adc_alibi_info *adc_alibi_info, const char *s1, const AdcInfo *i1, const char *s2, const AdcInfo *i2)
{
  const char *s_ptrs[2] = { s1, s2 };
  AdcInfo infos[2];

  infos[0] = *i1;
  infos[1] = *i2;
  adc_alibi_info->adc_alibi_display(adc_alibi_info->implementer_data, adc_alibi_info->adc_alibi_display_type, s_ptrs, infos);
}


/** Display a string on display 1, and a weight on display 2 */
void AdcIntAlibiDualDisplayStrNoAnnunAndWeight(struct s_adc_alibi_info *adc_alibi_info, const char *s1, AdcRdg *rdg2)
{
  int nd2;
  char *s2;
  AdcInfo i1;

  memset(&i1, 0, sizeof(i1));

  nd2 = adc_alibi_info->num_digits[aadtDoubleDisplay];
  s2 = adc_alibi_info->display_work_buffers[aadtDoubleDisplay];
  AdcIntAlibiWeightToStr(adc_alibi_info, rdg2, nd2, s2, ADC_ALIBI_DISPLAY_WORK_BUF_LEN(nd2));

  AdcIntAlibiDualDisplayStr(adc_alibi_info, s1, &i1, s2, &rdg2->info);
}


/** Display both strings on a dual display indicator (with no annunciators) */
void AdcIntAlibiDualDisplayStrNoAnnun(struct s_adc_alibi_info *adc_alibi_info, const char *s1, const char *s2)
{
  AdcInfo infos[2];

  memset(&infos[0], 0, 2 * sizeof(AdcInfo));
  infos[0].status.bit.grossOrNet = 0; /* Weight is not gross & not net */
  infos[1].status.bit.grossOrNet = 0; /* Weight is not gross & not net */
  AdcIntAlibiDualDisplayStr(adc_alibi_info, s1, &infos[0], s2, &infos[1]);
}
