/*
 * alibi_dsd.c
 *
 *  Created on: Feb 20, 2012
 *      Author: ian
 */
#include <string.h>
#include <rinweigh/adcalibi.h>
#include <rinweigh/adcdsd.h>
#include "alibi_display.h"
#include "alibi_error.h"
#include "alibi_config.h"
#include "alibi_editnum.h"
#include "alibi_kbd.h"
#include "alibi_mode.h"
#include "alibi_weight.h"
#include "alibi_quit.h"
#include "alibi_datetime.h"

#include "alibi_dsd.h"


/** Item within the traceable weight structure to display */
enum eAlibiDsdDisplayItem
{
  addiPrintId,
  addiReading,
  addiAlternateReading,
  addiPieces,
  addiTareWeight,
  addiDate,
  addiTime,

  addiNumItems
};


/** Get the display item name (essentially the name of the item in the traceable weight structure) */
static const char *AdcIntAlibiDSDGetDisplayItemName(struct s_adc_alibi_info *adc_alibi_info, enum eAlibiDsdDisplayItem display_item)
{
  const char *result = NULL;

  switch (display_item)
  {
    default:
    case addiPrintId:
      result = AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiDSDPrintId);
      break;
    case addiReading:
      result = AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiDSDReading);
      break;
    case addiAlternateReading:
      result = AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiDSDAltReading);
      break;
    case addiPieces:
      result = AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiDSDPieces);
      break;
    case addiTareWeight:
      result = AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiDSDTareWeight);
      break;
    case addiDate:
      result = AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiDate);
      break;
    case addiTime:
      result = AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiTime);
      break;
  }

  if (NULL == result)
  {
    result = "??????";
  }

  return result;
}


/** Display the item value (essentially the value of the item in the traceable weight structure) */
static void AdcIntAlibiDSDDisplaySingleItemValue(struct s_adc_alibi_info *adc_alibi_info, const AdcTraceableWeight *atw, enum eAlibiDsdDisplayItem display_item)
{
  AdcRdg rdg;

  switch (display_item)
  {
    default:
    case addiPrintId:
      memset(&rdg, 0, sizeof(rdg));
      rdg.rdg = atw->printId;
      AdcIntAlibiSingleDisplayWeight(adc_alibi_info, &rdg);
      break;

    case addiReading:
      AdcIntAlibiSingleDisplayWeight(adc_alibi_info, &atw->rdg);
      break;

    case addiAlternateReading:
      AdcIntAlibiSingleDisplayWeight(adc_alibi_info, &atw->altRdg);
      break;

    case addiPieces:
      memset(&rdg, 0, sizeof(rdg));
      rdg.rdg = atw->pieces;
      rdg.info.unitdp.bit.units = ADC_UNITS_PIECES;
      AdcIntAlibiSingleDisplayWeight(adc_alibi_info, &rdg);
      break;

    case addiTareWeight:
      memset(&rdg, 0, sizeof(rdg));
      rdg.rdg = atw->tareWgt;
      rdg.info.unitdp = atw->rdg.info.unitdp;
      rdg.info.status.bit.preTare = atw->pTare;
      rdg.info.status.bit.tare = !atw->pTare;
      AdcIntAlibiSingleDisplayWeight(adc_alibi_info, &rdg);
      break;

    case addiDate:
      AdcIntAlibiSingleDisplayDate(adc_alibi_info, &atw->dt);
      break;

    case addiTime:
      AdcIntAlibiSingleDisplayTime(adc_alibi_info, &atw->dt);
      break;
  }
}


/** Display the item value (essentially the value of the item in the traceable weight structure) */
static void AdcIntAlibiDSDDisplayDualItemValue(struct s_adc_alibi_info *adc_alibi_info, const AdcTraceableWeight *atw, enum eAlibiDsdDisplayItem display_item, const char *msg)
{
  AdcRdg rdg;
  AdcInfo i1;
  AdcInfo i2;
  char *s2;
  size_t nd2;

  /* Second display buffers */
  s2 = adc_alibi_info->display_work_buffers[aadtDoubleDisplay];
  nd2 = adc_alibi_info->num_digits[aadtDoubleDisplay];

  /* Display 2 */
  switch (display_item)
  {
    default:
    case addiPrintId:
      memset(&rdg, 0, sizeof(rdg));
      rdg.rdg = atw->printId;
      AdcIntAlibiWeightToStr(adc_alibi_info, &rdg, nd2, s2, ADC_ALIBI_DISPLAY_WORK_BUF_LEN(nd2));
      i2 = rdg.info;
      break;

    case addiReading:
      AdcIntAlibiWeightToStr(adc_alibi_info, &atw->rdg, nd2, s2, ADC_ALIBI_DISPLAY_WORK_BUF_LEN(nd2));
      i2 = atw->rdg.info;
      break;

    case addiAlternateReading:
      AdcIntAlibiWeightToStr(adc_alibi_info, &atw->altRdg, nd2, s2, ADC_ALIBI_DISPLAY_WORK_BUF_LEN(nd2));
      i2 = atw->altRdg.info;
      break;

    case addiPieces:
      memset(&rdg, 0, sizeof(rdg));
      rdg.rdg = atw->pieces;
      rdg.info.unitdp.bit.units = ADC_UNITS_PIECES;
      AdcIntAlibiWeightToStr(adc_alibi_info, &rdg, nd2, s2, ADC_ALIBI_DISPLAY_WORK_BUF_LEN(nd2));
      i2 = rdg.info;
      break;

    case addiTareWeight:
      memset(&rdg, 0, sizeof(rdg));
      rdg.rdg = atw->tareWgt;
      rdg.info.unitdp = atw->rdg.info.unitdp;
      rdg.info.status.bit.preTare = atw->pTare;
      rdg.info.status.bit.tare = !atw->pTare;
      AdcIntAlibiWeightToStr(adc_alibi_info, &rdg, nd2, s2, ADC_ALIBI_DISPLAY_WORK_BUF_LEN(nd2));
      i2 = rdg.info;
      break;

    case addiDate:
      AdcIntAlibiDateToStr(&atw->dt, s2, ADC_ALIBI_DISPLAY_WORK_BUF_LEN(nd2));
      memset(&i2, 0, sizeof(i2));
      break;

    case addiTime:
      AdcIntAlibiTimeToStr(&atw->dt, s2, ADC_ALIBI_DISPLAY_WORK_BUF_LEN(nd2));
      memset(&i2, 0, sizeof(i2));
      break;
  }

  /* Display 1 */
  memset(&i1, 0, sizeof(i1));

  /* Display it */
  AdcIntAlibiDualDisplayStr(adc_alibi_info, msg, &i1, s2, &i2);
}


/** Display a traceable weight on a single display */
static void AdcIntAlibiDSDDisplayTraceableSingle(struct s_adc_alibi_info *adc_alibi_info, const AdcTraceableWeight *atw, enum eAlibiDsdDisplayItem display_item)
{
  int32u divisor;
  int32u modulo;

  /* Display "Alibi" alternating display item name and value */
  divisor = 3;
  modulo = (OSTimeGet() / OS_TICKS_PER_SEC) % divisor;
  switch (modulo)
  {
    case 0:
      /* Display "alibi" */
      AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiAlibi));
      break;

    case 1:
      /* Display item name */
      AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiDSDGetDisplayItemName(adc_alibi_info, display_item));
      break;

    case 2:
      /* Display item value */
      AdcIntAlibiDSDDisplaySingleItemValue(adc_alibi_info, atw, display_item);
      break;
  }
}


/** Display a traceable weight on a double display */
static void AdcIntAlibiDSDDisplayTraceableDouble(struct s_adc_alibi_info *adc_alibi_info, const AdcTraceableWeight *atw, enum eAlibiDsdDisplayItem display_item)
{
  int32u divisor;
  int32u modulo;
  const char *msg;

  /* Display "Alibi" alternating display item name on one display, and the value on the other */
  divisor = 2;
  modulo = (OSTimeGet() / OS_TICKS_PER_SEC) % divisor;
  switch (modulo)
  {
    case 0:
      /* Display "alibi" */
      msg = AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiAlibi);
      break;

    case 1:
      /* Display item name */
      msg = AdcIntAlibiDSDGetDisplayItemName(adc_alibi_info, display_item);
      break;
  }
  AdcIntAlibiDSDDisplayDualItemValue(adc_alibi_info, atw, display_item, msg);
}


/** Display a traceable weight */
static void AdcIntAlibiDSDDisplayTraceable(struct s_adc_alibi_info *adc_alibi_info, const AdcTraceableWeight *atw, enum eAlibiDsdDisplayItem display_item)
{
  if (aadtSingleDisplay == adc_alibi_info->adc_alibi_display_type)
  {
    AdcIntAlibiDSDDisplayTraceableSingle(adc_alibi_info, atw, display_item);
  }
  else
  {
    AdcIntAlibiDSDDisplayTraceableDouble(adc_alibi_info, atw, display_item);
  }
}


/** Display DSD errors on a single display */
static void AdcIntAlibiDSDDisplayErrorSingle(struct s_adc_alibi_info *adc_alibi_info, enum eAdcAlibiInternationisationStringIdx err)
{
  int32u divisor;
  int32u modulo;

  /* Display "DSD.ERR" alternating with the error message */
  divisor = 2;
  modulo = (OSTimeGet() / OS_TICKS_PER_SEC) % divisor;
  switch (modulo)
  {
    case 0:
      /* Display "DSD.ERR" */
      AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiDSDError));
      break;

    case 1:
      /* Display error */
      AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, err));
      break;
  }
}


/** Display DSD errors on a double display */
static void AdcIntAlibiDSDDisplayErrorDouble(struct s_adc_alibi_info *adc_alibi_info, enum eAdcAlibiInternationisationStringIdx err)
{
  int32u divisor;
  int32u modulo;
  const char *s1;
  const char *s2;

  /* "Alibi" on first display */
  s1 = AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiAlibi);

  /* Display "DSD.ERR" alternating with the error message */
  divisor = 2;
  modulo = (OSTimeGet() / OS_TICKS_PER_SEC) % divisor;
  switch (modulo)
  {
    case 0:
      /* Display "DSD.ERR" */
      s2 = AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiDSDError);
      break;

    case 1:
      /* Display error */
      s2 = AdcIntAlibiGetInternationisation(adc_alibi_info, err);
      break;
  }

  AdcIntAlibiDualDisplayStrNoAnnun(adc_alibi_info, s1, s2);
}


/** Display DSD errors */
static void AdcIntAlibiDSDDisplayError(struct s_adc_alibi_info *adc_alibi_info, enum eAdcAlibiInternationisationStringIdx err)
{
  if (aadtSingleDisplay == adc_alibi_info->adc_alibi_display_type)
  {
    AdcIntAlibiDSDDisplayErrorSingle(adc_alibi_info, err);
  }
  else
  {
    AdcIntAlibiDSDDisplayErrorDouble(adc_alibi_info, err);
  }
}


/** Get a record from the DSD */
enum eAdcDsdError AdcIntAlibiDSDGetRecord(struct s_adc_alibi_info *adc_alibi_info, bool lock, int32u record, AdcTraceableWeight *atw)
{
  enum eAdcDsdError res;

  /* Read the record from the DSD */
  res = AdcDSDReadRecord(adc_alibi_info->dsd, lock, record,
                         adc_alibi_info->dsd_record_buffer, adc_alibi_info->dsd_record_buffer_length);
  if (ADC_DSD_SUCCESS != res)
  {
    return res;
  }

  /* Now use the application to give us the traceable weight from the DSD record (we don't know how to extract it from the raw data) */
  if (!adc_alibi_info->adc_alibi_dsd_convert_to_traceable(adc_alibi_info->implementer_data,
                                                          adc_alibi_info->dsd_record_buffer, adc_alibi_info->dsd_record_buffer_length,
                                                          atw))
  {
    return ADC_DSD_ERROR_MISC;
  }

  /* Success */
  return ADC_DSD_SUCCESS;
}


/** Add a record to the DSD */
bool AdcIntAlibiDSDAddRecord(struct s_adc_alibi_info *adc_alibi_info, const AdcTraceableWeight *atw)
{
  enum eAdcDsdError res;

  /* Some checks */
  if (NULL == adc_alibi_info->adc_alibi_dsd_convert_from_traceable ||
      NULL == adc_alibi_info->dsd_record_buffer ||
      0 == adc_alibi_info->dsd_record_buffer_length)
  {
    return false;
  }

  /* Now use the application to give us a DSD record from the traceable weight (we don't know how to insert it into the raw data) */
  if (!adc_alibi_info->adc_alibi_dsd_convert_from_traceable(adc_alibi_info->implementer_data,
                                                            atw,
                                                            adc_alibi_info->dsd_record_buffer, adc_alibi_info->dsd_record_buffer_length))
  {
    return false;
  }


  /* Write the record to the DSD */
  res = AdcDSDWriteRecord(adc_alibi_info->dsd, true, adc_alibi_info->dsd_record_buffer, adc_alibi_info->dsd_record_buffer_length);
  if (ADC_DSD_SUCCESS != res)
  {
    return false;
  }

  /* Success */
  return true;
}


/** Select a print id */
static void AdcIntAlibiDSDSelect(struct s_adc_alibi_info *adc_alibi_info, int32u *print_id, int32u *record)
{
  int32u i;
  int32u numrecords;
  bool found;
  AdcTraceableWeight atw;
  AdcRdg prid;
  int32u last_prid;

  /* Prompt */
  AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiEnter));
  OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);
  AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiDSDPrintId));
  OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);

  /* Edit print id */
  memset(&prid, 0, sizeof(prid));
  prid.rdg = *print_id;
  if (!AdcIntAlibiEditNumber(adc_alibi_info, &prid, 0, 999999, false))
  {
    /* User cancelled */
    return;
  }

  /* Search DSD for print id */
  AdcDSDMutexLock(adc_alibi_info->dsd);

  if (ADC_DSD_SUCCESS != AdcDSDGetNumberRecords(adc_alibi_info->dsd, false, &numrecords))
  {
    (void) AdcIntAlibiShowErrorIndexes(adc_alibi_info, aaisiDSDError, aaisiDSDSearch, ADC_ALIBI_CONFIG_ERROR_TIMEOUT);
    AdcDSDMutexUnlock(adc_alibi_info->dsd);
    return;
  }
  for (i = 0, found = false, last_prid = 0; i < numrecords; i++)
  {
    if (ADC_DSD_SUCCESS == AdcIntAlibiDSDGetRecord(adc_alibi_info, false, i, &atw))
    {
      if (atw.printId == (int32u) prid.rdg ||
          (last_prid < (int32u) prid.rdg && atw.printId > (int32u) prid.rdg))
      {
        found = true;
        break;
      }
      last_prid = prid.rdg;
    }
  }
  AdcDSDMutexUnlock(adc_alibi_info->dsd);

  if (!found)
  {
    (void) AdcIntAlibiShowErrorIndexes(adc_alibi_info, aaisiDSDError, aaisiDSDSearch, ADC_ALIBI_CONFIG_ERROR_TIMEOUT);
    return;
  }

  *print_id = prid.rdg;
  *record = i;
}


/** Should this display item be displayed (we may not show alternate reading or pieces, depending on configuration) */
static bool AdcIntAlibiDSDShouldDisplayItem(struct s_adc_alibi_info *adc_alibi_info, enum eAlibiDsdDisplayItem display_item)
{
  bool result;

  switch (display_item)
  {
    case addiAlternateReading:
      result = !adc_alibi_info->no_alternate_units;
      break;
    case addiPieces:
      result = !adc_alibi_info->no_piece_counting;
      break;
    default:
      result = true;
      break;
  }
  return result;
}


/** Advance to the next display item, or retreat to the previous display item, based on advance */
static void AdcIntAlibiDSDNextPrevDisplayItem(struct s_adc_alibi_info *adc_alibi_info, enum eAlibiDsdDisplayItem *display_item, int advance)
{
  if (advance > 0)
  {
    /* Advance forward, find next displayable item */
    do
    {
      (*display_item)++;
      if (*display_item >= addiNumItems)
        *display_item = 0;
    }
    while (!AdcIntAlibiDSDShouldDisplayItem(adc_alibi_info, *display_item));
  }
  else if (advance < 0)
  {
    /* Retreat backward, find previous displayable item */
    do
    {
      if (0 == *display_item || *display_item >= addiNumItems)
        *display_item = addiNumItems - 1;
      else
        (*display_item)--;
    }
    while (!AdcIntAlibiDSDShouldDisplayItem(adc_alibi_info, *display_item));
  }
}


/** Process keystroke for the DSD application */
static void AdcIntAlibiDSDProcessKey(struct s_adc_alibi_info *adc_alibi_info, int32u *current_record, int32u *print_id, enum eAlibiDsdDisplayItem *display_item)
{
  enum eAdcAlibiKeystroke key;
  int32u numrecords;

  /* Process keys */
  AdcIntAlibiGetKey(adc_alibi_info, &key, ADC_ALIBI_CONFIG_DISPLAY_UPDATE_HZ);

  /* Dispatch */
  switch (key)
  {

    /* Quit alibi mode */
    case aakQuit:
      adc_alibi_info->alibi_mode = aamQuitAlibi;
      break;

    /* Jump to first (oldest) DSD record */
    case aakDSDRecordFirst:
      *current_record = 0;
      AdcIntAlibiShowMessageIndex(adc_alibi_info, aaisiFirst, ADC_ALIBI_CONFIG_MESSAGE_TIMEOUT);
      break;

    /* Jump to last (newest) DSD record */
    case aakDSDRecordLast:
      if (ADC_DSD_SUCCESS != AdcDSDGetNumberRecords(adc_alibi_info->dsd, true, &numrecords))
      {
        (void) AdcIntAlibiShowErrorIndexes(adc_alibi_info, aaisiDSDError, aaisiDSDSearch, ADC_ALIBI_CONFIG_ERROR_TIMEOUT);
      }
      else
      {
        if (numrecords > 0)
          *current_record = numrecords - 1;
        else
          *current_record = 0;
        AdcIntAlibiShowMessageIndex(adc_alibi_info, aaisiLast, ADC_ALIBI_CONFIG_MESSAGE_TIMEOUT);
      }
      break;

    /* Advance to next DSD record */
    case aakDSDRecordNext:
      if (ADC_DSD_SUCCESS != AdcDSDGetNumberRecords(adc_alibi_info->dsd, true, &numrecords))
      {
        (void) AdcIntAlibiShowErrorIndexes(adc_alibi_info, aaisiDSDError, aaisiDSDSearch, ADC_ALIBI_CONFIG_ERROR_TIMEOUT);
      }
      else
      {
        if (numrecords > 0)
        {
          (*current_record)++;
          if (*current_record >= numrecords)
          {
            *current_record = numrecords - 1;
          }
        }
        else
        {
          *current_record = 0;
        }
        AdcIntAlibiShowMessageIndex(adc_alibi_info, aaisiNext, ADC_ALIBI_CONFIG_MESSAGE_TIMEOUT);
      }
      break;

    /* Back to previous DSD record */
    case aakDSDRecordPrevious:
      if (*current_record > 0)
        (*current_record)--;
      AdcIntAlibiShowMessageIndex(adc_alibi_info, aaisiPrevious, ADC_ALIBI_CONFIG_MESSAGE_TIMEOUT);
      break;

    /* Select a specific print ID */
    case aakDSDRecordSelect:
      AdcIntAlibiDSDSelect(adc_alibi_info, print_id, current_record);
      break;

    /* First item within the current traceable weight record */
    case aakDSDItemFirst:
      *display_item = 0;
      break;

    /* Last item within the current traceable weight record */
    case aakDSDItemLast:
      *display_item = addiNumItems - 1;
      break;

    /* Reverse through items within the current traceable weight record */
    case aakDSDItemPrevious:
      AdcIntAlibiDSDNextPrevDisplayItem(adc_alibi_info, display_item, -1);
      break;

    /* Advance through items within the current traceable weight record */
    case aakDSDItemNext:
      AdcIntAlibiDSDNextPrevDisplayItem(adc_alibi_info, display_item, 1);
      break;

    /* Change alibi mode */
    case aakMode:
      AdcIntAlibiSelectMode(adc_alibi_info);
      break;

    case aakNone:
    default:
      /* Do nothing with other keys */
      break;
  } /* switch */
}


/** Run the alibi DSD viewer application */
void AdcIntAlibiDSDViewer(struct s_adc_alibi_info *adc_alibi_info)
{
  int32u current_record;
  int32u previous_record;
  AdcTraceableWeight atw;
  enum eAlibiDsdDisplayItem display_item;
  enum eAdcDsdError dsd_result;

  /* Check the DSD is working/available */
  if (NULL == adc_alibi_info->dsd ||
      ADC_DSD_SUCCESS != AdcDSDIsRunning(adc_alibi_info->dsd, true) ||
      NULL == adc_alibi_info->dsd_record_buffer ||
      0 == adc_alibi_info->dsd_record_buffer_length ||
      NULL == adc_alibi_info->adc_alibi_dsd_convert_to_traceable)
  {
    /* No DSD so generate error, and back to weighing mode */
    AdcIntAlibiShowErrorIndexes(adc_alibi_info, aaisiNo, aaisiDSD, ADC_ALIBI_CONFIG_ERROR_TIMEOUT);
    adc_alibi_info->alibi_mode = aamWeighing;
    return;
  }

  /* Do alibi DSD application loop */
  current_record = 0;
  previous_record = ~current_record;
  dsd_result = ADC_DSD_ERROR_MISC;
  display_item = 0;
  for (; aamDSDView == adc_alibi_info->alibi_mode; )
  {
    if (current_record != previous_record)
    {
      /* Get the record */
      dsd_result = AdcIntAlibiDSDGetRecord(adc_alibi_info, true, current_record, &atw);
      previous_record = current_record;
    }

    /* Display the traceable weight, or error if not valid */
    switch (dsd_result)
    {
      case ADC_DSD_SUCCESS:
        AdcIntAlibiDSDDisplayTraceable(adc_alibi_info, &atw, display_item);
        break;

      case ADC_DSD_ERROR_EMPTY:
        AdcIntAlibiDSDDisplayError(adc_alibi_info, aaisiDSDEmpty);
        break;

      default:
        AdcIntAlibiDSDDisplayError(adc_alibi_info, aaisiFailed);
        break;
    }

    /* Process key */
    AdcIntAlibiDSDProcessKey(adc_alibi_info, &current_record, &atw.printId, &display_item);

    /* Should we quit? */
    if (AdcIntAlibiForceQuit(adc_alibi_info))
    {
      adc_alibi_info->alibi_mode = aamQuitAlibi;
    }
  } /* for () */
}
