/*
 * alibi_error.c
 *
 *  Created on: Feb 21, 2012
 *      Author: ian
 */
#include <rinweigh/adcalibi.h>
#include "alibi_display.h"
#include "alibi_kbd.h"
#include "alibi_quit.h"

#include "alibi_error.h"


/** Used to display an error string */
static void AdcIntAlibiErrorDisplayStr(struct s_adc_alibi_info *adc_alibi_info, const char *s)
{
  if (aadtSingleDisplay == adc_alibi_info->adc_alibi_display_type)
  {
    AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, s);
  }
  else if (aadtDoubleDisplay == adc_alibi_info->adc_alibi_display_type)
  {
    AdcIntAlibiDualDisplayStrNoAnnun(adc_alibi_info,
                                     AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiAlibi),
                                     s);
  }
}


/** Show a message */
enum eAdcAlibiKeystroke AdcIntAlibiShowMessage(struct s_adc_alibi_info *adc_alibi_info, const char *s, int16u timeout)
{
  enum eAdcAlibiKeystroke key;
  int32u endtime;

  /* Clear display to start with */
  AdcIntAlibiSetAllSegs(adc_alibi_info, false);

  /* Loop until timeout */
  for (endtime = OSTimeGet() + timeout; OSTimeGet() < endtime; )
  {
    /* First string */
    AdcIntAlibiErrorDisplayStr(adc_alibi_info, s);
    AdcIntAlibiGetKey(adc_alibi_info, &key, OS_TICKS_PER_SEC / 2);
    if (aakNone != key || AdcIntAlibiForceQuit(adc_alibi_info))
    {
      break;
    }
  }
  return key;
}


/** Show a message using internationalisation indexes, rather than strings */
enum eAdcAlibiKeystroke AdcIntAlibiShowMessageIndex(struct s_adc_alibi_info *adc_alibi_info, enum eAdcAlibiInternationisationStringIdx idx, int16u timeout)
{
  return AdcIntAlibiShowMessage(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, idx), timeout);
}


/** Show an error */
enum eAdcAlibiKeystroke AdcIntAlibiShowError(struct s_adc_alibi_info *adc_alibi_info, const char *s1, const char *s2, int16u timeout)
{
  enum eAdcAlibiKeystroke key;
  int32u endtime;

  /* Clear display to start with */
  AdcIntAlibiSetAllSegs(adc_alibi_info, false);

  /* Loop until timeout */
  for (endtime = OSTimeGet() + timeout; OSTimeGet() < endtime; )
  {
    /* First string */
    AdcIntAlibiErrorDisplayStr(adc_alibi_info, s1);
    AdcIntAlibiGetKey(adc_alibi_info, &key, OS_TICKS_PER_SEC / 2);
    if (aakNone != key || AdcIntAlibiForceQuit(adc_alibi_info))
    {
      break;
    }

    /* Second string */
    AdcIntAlibiErrorDisplayStr(adc_alibi_info, s2);
    AdcIntAlibiGetKey(adc_alibi_info, &key, OS_TICKS_PER_SEC / 2);
    if (aakNone != key || AdcIntAlibiForceQuit(adc_alibi_info))
    {
      break;
    }

    key = aakNone;
  }

  return key;
}


/** Show an error using internationalisation indexes, rather than strings */
enum eAdcAlibiKeystroke AdcIntAlibiShowErrorIndexes(struct s_adc_alibi_info *adc_alibi_info, enum eAdcAlibiInternationisationStringIdx idx1, enum eAdcAlibiInternationisationStringIdx idx2, int16u timeout)
{
  return AdcIntAlibiShowError(adc_alibi_info,
                              AdcIntAlibiGetInternationisation(adc_alibi_info, idx1),
                              AdcIntAlibiGetInternationisation(adc_alibi_info, idx2),
                              timeout);
}


/** Convert ADC errors to application errors, which allows Alibi to display the same errors as the main application (for consistency) */
int32u AdcIntConvertAdcErrorsToApplicationErrors(struct s_adc_alibi_info *adc_alibi_info, AdcErrors adcerrors)
{
  int32u apperrors;

  if (NULL == adc_alibi_info->adc_alibi_convert_adcerrors_to_apperrors)
  {
    /* If no function, then no translation of errors */
    apperrors = adcerrors;
  }
  else
  {
    /* Convert ADC errors to application errors */
    adc_alibi_info->adc_alibi_convert_adcerrors_to_apperrors(adc_alibi_info->implementer_data, adcerrors, &apperrors);
  }
  return apperrors;
}
