/*
 * alibi_main.c
 *
 *  Created on: Feb 20, 2012
 *      Author: ian
 */
#include <stdio.h>
#include <rinhw/ucos_ii.h>
#include <rinhw/stdrinstrum.h>
#include <rinweigh/adcinterface.h>
#include <rinweigh/adcdatabase.h>
#include <rinweigh/adcversion.h>
#include "alibi_changelog.h"
#include "alibi_counting.h"
#include "alibi_display.h"
#include "alibi_dsd.h"
#include "alibi_weighing.h"
#include "alibi_config.h"

#include <rinweigh/adcalibi.h>


/** Convert the ADC library checksum into a string ready for display */
static void AdcIntAlibiChecksumToString(struct s_adc_alibi_info *adc_alibi_info, char *display, int num_display_chars)
{
  snprintf(display, ADC_ALIBI_DISPLAY_WORK_BUF_LEN(num_display_chars), "%0*lu", num_display_chars, (unsigned long) adc_alibi_info->adc_lib_checksum);
}


/** Convert a calibration counter into a string ready for display */
static void AdcIntAlibiCalCntToString(char *display, int num_display_chars, char prefix, uint32_t cal_count)
{
  snprintf(display, ADC_ALIBI_DISPLAY_WORK_BUF_LEN(num_display_chars), "%c%0*lu",
           prefix,
           num_display_chars - 1,
           (unsigned long) cal_count);
}


/** Display various elements on the display when starting */
static void AdcIntAlibiStartDisplay(struct s_adc_alibi_info *adc_alibi_info)
{
  char *d1;
  char *d2;
  int nd1;
  int nd2;

  /* Setup display temp strings */
  d1 = adc_alibi_info->display_work_buffers[aadtSingleDisplay];
  d2 = adc_alibi_info->display_work_buffers[aadtDoubleDisplay];
  nd1 = adc_alibi_info->num_digits[aadtSingleDisplay];
  nd2 = adc_alibi_info->num_digits[aadtDoubleDisplay];

  /* All segs off */
  AdcIntAlibiSetAllSegs(adc_alibi_info, false);
  OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);

  /* All segs on */
  AdcIntAlibiSetAllSegs(adc_alibi_info, true);
  OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);

  /* All segs off again, but no pause */
  AdcIntAlibiSetAllSegs(adc_alibi_info, false);

  /* Different handling based on number of displays */
  if (aadtSingleDisplay == adc_alibi_info->adc_alibi_display_type)
  {
    /* Single display */

    /* Alibi */
    AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiAlibi));
    OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);

    /* ADC library version */
    AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcLibVersionFormattedStr);
    OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);

    /* ADC library checksum */
    AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiChecksum));
    OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);
    AdcIntAlibiChecksumToString(adc_alibi_info, d1, nd1);
    AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, d1);
    OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);

    /* Cal count(s) */
    if (ADC_APPROVAL_MODE_NTEP == adc_alibi_info->ass->adcUserDatabase.approvalMode)
    {
      /* NTEP - 2 counters */
      AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiCalCount));
      OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);
      AdcIntAlibiCalCntToString(d1, nd1, 'C', adc_alibi_info->ass->adcUserDatabase.calCntNTEPCalibration);
      AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, d1);
      OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);

      AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiConfigCount));
      OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);
      AdcIntAlibiCalCntToString(d1, nd1, 'F', adc_alibi_info->ass->adcUserDatabase.calCntNTEPSettings);
      AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, d1);
      OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);
    }
    else
    {
      /* OIML - 1 counter */
      AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiCalCount));
      OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);
      AdcIntAlibiCalCntToString(d1, nd1, 'C', adc_alibi_info->ass->adcUserDatabase.calCntOIML);
      AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, d1);
      OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);
    }
  }
  else if (aadtDoubleDisplay == adc_alibi_info->adc_alibi_display_type)
  {
    /* Dual display */

    /* Alibi & ADC library version */
    AdcIntAlibiDualDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiAlibi), AdcLibVersionFormattedStr);
    OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);

    /* ADC library checksum */
    AdcIntAlibiChecksumToString(adc_alibi_info, d2, nd2);
    AdcIntAlibiDualDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiChecksum), d2);
    OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);

    /* Cal count(s) */
    if (ADC_APPROVAL_MODE_NTEP == adc_alibi_info->ass->adcUserDatabase.approvalMode)
    {
      /* NTEP - 2 counters */
      AdcIntAlibiCalCntToString(d2, nd2, 'C', adc_alibi_info->ass->adcUserDatabase.calCntNTEPCalibration);
      AdcIntAlibiDualDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiCalCount), d2);
      OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);

      AdcIntAlibiCalCntToString(d2, nd2, 'F', adc_alibi_info->ass->adcUserDatabase.calCntNTEPSettings);
      AdcIntAlibiDualDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiConfigCount), d2);
      OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);
    }
    else
    {
      /* OIML - 1 counter */
      AdcIntAlibiCalCntToString(d2, nd2, 'C', adc_alibi_info->ass->adcUserDatabase.calCntOIML);
      AdcIntAlibiDualDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiCalCount), d2);
      OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);
    }
  }
}


/** Display various elements on the display when finishing */
static void AdcIntAlibiEndDisplay(struct s_adc_alibi_info *adc_alibi_info)
{
  /* Different handling based on number of displays */
  if (aadtSingleDisplay == adc_alibi_info->adc_alibi_display_type)
  {
    /* Single display */

    /* "End" */
    AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiEnd));
    OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);

    /* "Alibi" */
    AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiAlibi));
    OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);
  }
  else if (aadtDoubleDisplay == adc_alibi_info->adc_alibi_display_type)
  {
    /* Dual display */

    /* "End" "Alibi" */
    AdcIntAlibiDualDisplayStrNoAnnun(adc_alibi_info,
                                     AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiEnd),
                                     AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiAlibi));
    OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);
  }
}


/** Run the alibi application */
bool AdcAlibiRun(struct s_adc_alibi_info *adc_alibi_info)
{
  /* Do some checks */
  if (NULL == adc_alibi_info ||
      NULL == adc_alibi_info->ass ||
      NULL == adc_alibi_info->adc_alibi_display_allsegs ||
      NULL == adc_alibi_info->adc_alibi_display ||
      NULL == adc_alibi_info->adc_alibi_getchar ||
      NULL == adc_alibi_info->adc_alibi_get_internationalisation_string ||
      NULL == adc_alibi_info->adc_alibi_force_quit ||
      NULL == adc_alibi_info->adc_alibi_edit_number ||
      NULL == adc_alibi_info->adc_alibi_edit_list ||
      NULL == adc_alibi_info->adc_alibi_get_abbreviated_units_string ||
      NULL == adc_alibi_info->adc_alibi_print_string ||
      (adc_alibi_info->adc_alibi_display_type >= aadtSingleDisplay && NULL == adc_alibi_info->display_work_buffers[aadtSingleDisplay]) ||
      (adc_alibi_info->adc_alibi_display_type >= aadtDoubleDisplay && NULL == adc_alibi_info->display_work_buffers[aadtDoubleDisplay]))
  {
    return false;
  }

  adc_alibi_info->force_quit = false;

  /* All off/on/version, etc */
  AdcIntAlibiStartDisplay(adc_alibi_info);

  /* Main loop */
  while (aamQuitAlibi != adc_alibi_info->alibi_mode)
  {
    switch (adc_alibi_info->alibi_mode)
    {
      default:
      case aamWeighing:
        AdcIntAlibiWeighing(adc_alibi_info);
        break;
      case aamCounting:
        AdcIntAlibiCounting(adc_alibi_info);
        break;
      case aamDSDView:
        AdcIntAlibiDSDViewer(adc_alibi_info);
        break;
      case aamChangelogView:
        AdcIntAlibiChangeLogViewer(adc_alibi_info);
        break;
      case aamQuitAlibi:
        break;
    }
  }

  /* End message on normal exit */
  if (!adc_alibi_info->force_quit)
  {
    AdcIntAlibiEndDisplay(adc_alibi_info);
  }
  return true;
}
