/*
 * alibi_scroll.c
 *
 *  Created on: May 24, 2012
 *      Author: ian
 */
#include <string.h>
#include "alibi_config.h"
#include "alibi_display.h"

#include "alibi_scroll.h"


/** Calculate the number of steps when scrolling in one direction */
size_t AdcIntAlibiScrollSingleDirnGetNumScrollSteps(const char *str, size_t display_length)
{
  size_t slen;
  size_t result;

  slen = strlen(str);
  if (slen <= display_length)
  {
    result = 1;
  }
  else
  {
    /* Scroll the display */
    result = slen - display_length + 1;
  }
  return result;
}


/** Calculate the number of steps when scrolling in both directions */
size_t AdcIntAlibiScrollDualDirnGetNumScrollSteps(const char *str, size_t display_length)
{
  size_t slen;
  size_t result;

  slen = strlen(str);
  if (slen <= display_length)
  {
    result = 1;
  }
  else
  {
    /* Scroll the display */
    result = (slen - display_length) * 2;
  }
  return result;
}


/** Scroll scroll_str within outstr_len characters by copying the appropriate part to out_str. */
static void AdcIntAlibiScrollSingleDirnStr(const char *scroll_str, char *out_str, size_t outstr_len, int32u start_ticks)
{
  size_t slen;
  int num_scroll_posns;
  int current_scroll_posn;

  slen = strlen(scroll_str);
  if (slen <= outstr_len)
  {
    /* If the string will fit on a single display, then don't scroll it */
    strncpy(out_str, scroll_str, outstr_len);
    out_str[outstr_len] = '\0';
  }
  else
  {
    /* Scroll the display */
    num_scroll_posns = slen - outstr_len + 1;
    current_scroll_posn = ((OSTimeGet() - start_ticks) / ADC_ALIBI_CONFIG_SCROLL_RATE) % (num_scroll_posns);

    /* Scrolling to the left */
    strncpy(out_str, &scroll_str[current_scroll_posn], outstr_len);
    out_str[outstr_len] = '\0';
  }
}


/** Scroll scroll_str within outstr_len characters by copying the appropriate part to out_str, in both directions */
static void AdcIntAlibiScrollDualDirnStr(const char *scroll_str, char *out_str, size_t outstr_len, int32u start_ticks)
{
  size_t slen;
  int num_scroll_posns;
  int current_scroll_posn;
  int divisor;

  slen = strlen(scroll_str);
  if (slen <= outstr_len)
  {
    /* If the string will fit on a single display, then don't scroll it */
    strncpy(out_str, scroll_str, outstr_len);
    out_str[outstr_len] = '\0';
  }
  else
  {
    /* Scroll the display */
    num_scroll_posns = (slen - outstr_len);
    divisor = num_scroll_posns * 2;

    current_scroll_posn = ((OSTimeGet() - start_ticks) / ADC_ALIBI_CONFIG_SCROLL_RATE) % (divisor);

    if (current_scroll_posn >= num_scroll_posns)
    {
      /* Scrolling to the right */
      current_scroll_posn = current_scroll_posn - num_scroll_posns;
      current_scroll_posn = num_scroll_posns - current_scroll_posn;

      strncpy(out_str, &scroll_str[current_scroll_posn], outstr_len);
      out_str[outstr_len] = '\0';
    }
    else
    {
      /* Scrolling to the left */
      strncpy(out_str, &scroll_str[current_scroll_posn], outstr_len);
      out_str[outstr_len] = '\0';
    }
  }
}


/** Display a scrolling string on a single display indicator */
void AdcIntAlibiSingleDisplayStrScroll(struct s_adc_alibi_info *adc_alibi_info,
                                       const char *s, const AdcInfo *info, bool scroll, int32u start_ticks, bool dual_direction)
{
  char *os1;
  const char *cos1;
  int nd1;

  if (scroll)
  {
    os1 = adc_alibi_info->display_work_buffers[aadtSingleDisplay];
    cos1 = os1;
    nd1 = adc_alibi_info->num_digits[aadtSingleDisplay];

    if (dual_direction)
      AdcIntAlibiScrollSingleDirnStr(s, os1, nd1, start_ticks);
    else
      AdcIntAlibiScrollDualDirnStr(s, os1, nd1, start_ticks);
  }
  else
  {
    cos1 = s;
  }

  AdcIntAlibiSingleDisplayStr(adc_alibi_info, cos1, info);
}


/** Display scrolling strings on a dual display indicator */
void AdcIntAlibiDualDisplayStrScroll(struct s_adc_alibi_info *adc_alibi_info,
                                     const char *s1, const AdcInfo *i1, bool scroll_s1, int32u start_ticks1,
                                     const char *s2, const AdcInfo *i2, bool scroll_s2, int32u start_ticks2)
{
  char *os1;
  char *os2;
  const char *cos1;
  const char *cos2;
  int nd1;
  int nd2;

  if (scroll_s1)
  {
    os1 = adc_alibi_info->display_work_buffers[aadtSingleDisplay];
    cos1 = os1;
    nd1 = adc_alibi_info->num_digits[aadtSingleDisplay];
    AdcIntAlibiScrollDualDirnStr(s1, os1, nd1, start_ticks1);
  }
  else
  {
    cos1 = s1;
  }

  if (scroll_s2)
  {
    os2 = adc_alibi_info->display_work_buffers[aadtDoubleDisplay];
    cos2 = os2;
    nd2 = adc_alibi_info->num_digits[aadtDoubleDisplay];
    AdcIntAlibiScrollDualDirnStr(s2, os2, nd2, start_ticks2);
  }
  else
  {
    cos2 = s2;
  }

  AdcIntAlibiDualDisplayStr(adc_alibi_info, cos1, i1, cos2, i2);
}
