/*
 * alibi_weighing.c
 *
 *  Created on: Feb 21, 2012
 *      Author: ian
 */
#include <stdio.h>
#include <string.h>
#include <rinweigh/adcinterface.h>
#include <rinweigh/adcdatabase.h>
#include <rinweigh/adcalibi.h>
#include "alibi_kbd.h"
#include "alibi_editnum.h"
#include "alibi_display.h"
#include "alibi_config.h"
#include "alibi_error.h"
#include "alibi_print.h"
#include "alibi_mode.h"
#include "alibi_dsd.h"
#include "alibi_weight.h"
#include "alibi_quit.h"

#include "alibi_weighing.h"


/** Display the weight on a single display */
static void AdcIntAlibiWeighingDisplayWeightSingle(struct s_adc_alibi_info *adc_alibi_info)
{
  int32u error;
  int32u divisor;
  int32u modulo;
  AdcRdg rdg;
  char *s;
  int nd;

  AdcMutexLock(adc_alibi_info->ass);
  error = AdcIntConvertAdcErrorsToApplicationErrors(adc_alibi_info, adc_alibi_info->ass->adcDataWorking.error);
  rdg = adc_alibi_info->ass->adcDataWorking.dispRdg;
  AdcMutexUnlock(adc_alibi_info->ass);

  /* Display "Alibi" alternating with errors (if present) and weight */
  divisor = (error) ? 3 : 2;
  modulo = (OSTimeGet() / OS_TICKS_PER_SEC) % divisor;
  if (0 == modulo)
  {
    /* Display "alibi" */
    AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiAlibi));
  }
  else if (1 == modulo)
  {
    /* Display over/under/weight */
    if (rdg.info.status.bit.oload)
    {
      AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiOverload));
    }
    else if (rdg.info.status.bit.uload)
    {
      AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiUnderload));
    }
    else
    {
      AdcIntAlibiSingleDisplayWeight(adc_alibi_info, &rdg);
    }
  }
  else
  {
    /* Display error */
    s = adc_alibi_info->display_work_buffers[aadtSingleDisplay];
    nd = adc_alibi_info->num_digits[aadtSingleDisplay];
    AdcIntAlibiErrorToStr(adc_alibi_info, error, nd, s, ADC_ALIBI_DISPLAY_WORK_BUF_LEN(nd));
    AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, s);
  }
}


/** Display the weight on a double display */
static void AdcIntAlibiWeighingDisplayWeightDouble(struct s_adc_alibi_info *adc_alibi_info)
{
  int32u error;
  int32u divisor;
  int32u modulo;
  AdcRdg rdg;
  const char *s1;
  const char *s2;
  AdcInfo i1;
  AdcInfo i2;
  /* int nd1; */
  int nd2;

  /* nd1 = adc_alibi_info->num_digits[aadtSingleDisplay]; */
  nd2 = adc_alibi_info->num_digits[aadtDoubleDisplay];

  AdcMutexLock(adc_alibi_info->ass);
  error = AdcIntConvertAdcErrorsToApplicationErrors(adc_alibi_info, adc_alibi_info->ass->adcDataWorking.error);
  rdg = adc_alibi_info->ass->adcDataWorking.dispRdg;
  AdcMutexUnlock(adc_alibi_info->ass);

  /* Display "alibi" on first display */
  s1 = AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiAlibi);
  memset(&i1, 0, sizeof(i1));

  /* Display errors (if present) alternating with the weight */
  divisor = (error) ? 2 : 1;
  modulo = (OSTimeGet() / OS_TICKS_PER_SEC) % divisor;
  memset(&i2, 0, sizeof(i2));
  if (0 == modulo)
  {
    /* Display over/under/weight */
    if (rdg.info.status.bit.oload)
    {
      s2 = AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiOverload);
    }
    else if (rdg.info.status.bit.uload)
    {
      s2 = AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiUnderload);
    }
    else
    {
      AdcIntAlibiWeightToStr(adc_alibi_info, &rdg, nd2, adc_alibi_info->display_work_buffers[aadtDoubleDisplay], ADC_ALIBI_DISPLAY_WORK_BUF_LEN(nd2));
      s2 = adc_alibi_info->display_work_buffers[aadtDoubleDisplay];
      i2 = rdg.info;
    }
  }
  else
  {
    /* Display error */
    AdcIntAlibiErrorToStr(adc_alibi_info, error, nd2, adc_alibi_info->display_work_buffers[aadtDoubleDisplay], ADC_ALIBI_DISPLAY_WORK_BUF_LEN(nd2));
    s2 = adc_alibi_info->display_work_buffers[aadtDoubleDisplay];
  }
  AdcIntAlibiDualDisplayStr(adc_alibi_info, s1, &i1, s2, &i2);
}


/** Display the weight */
void AdcIntAlibiWeighingDisplayWeight(struct s_adc_alibi_info *adc_alibi_info)
{
  if (aadtSingleDisplay == adc_alibi_info->adc_alibi_display_type)
  {
    AdcIntAlibiWeighingDisplayWeightSingle(adc_alibi_info);
  }
  else
  {
    AdcIntAlibiWeighingDisplayWeightDouble(adc_alibi_info);
  }
}


/** Get user to enter preset tare, and apply */
static void AdcIntAlibiWeighingSetPresetTare(struct s_adc_alibi_info *adc_alibi_info, int8u *opcounter)
{
  AdcRdg presettare;

  /* Prompt */
  AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiEnter));
  OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);
  AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiPresetTare));
  OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);

  /* Edit preset tare */
  memset(&presettare, 0, sizeof(presettare));
  presettare.info.unitdp.bit.dp = adc_alibi_info->ass->adcUserDatabase.dpPos;
  presettare.info.unitdp.bit.units = adc_alibi_info->ass->adcUserDatabase.units;
  presettare.info.status.bit.preTare = 1;
  presettare.rdg = 0;
  if (AdcIntAlibiEditNumber(adc_alibi_info, &presettare, -99999, 999999, true))
  {
    /* Apply preset tare */
    AdcMutexLock(adc_alibi_info->ass);
    (void) AdcSetPTare(adc_alibi_info->ass, false, false, false, presettare.rdg);
    *opcounter = AdcGetCurrentCommandCnt(adc_alibi_info->ass, false);
    AdcMutexUnlock(adc_alibi_info->ass);
  }
}


/** Process keystroke in for the weighing application */
static void AdcIntAlibiWeighingProcessKey(struct s_adc_alibi_info *adc_alibi_info, int8u *opcounter, int8u *opopt)
{
  enum eAdcAlibiKeystroke key;

  /* Process keys */
  AdcIntAlibiGetKey(adc_alibi_info, &key, ADC_ALIBI_CONFIG_DISPLAY_UPDATE_HZ);

  /* Dispatch */
  switch (key)
  {
    case aakQuit:
      adc_alibi_info->alibi_mode = aamQuitAlibi;
      break;

    case aakZero:
      AdcMutexLock(adc_alibi_info->ass);
      (void) AdcSemiautoZero(adc_alibi_info->ass, false, false, false, false);
      *opcounter = AdcGetCurrentCommandCnt(adc_alibi_info->ass, false);
      AdcMutexUnlock(adc_alibi_info->ass);
      break;

    case aakTare:
      AdcMutexLock(adc_alibi_info->ass);
      (void) AdcSemiautoTare(adc_alibi_info->ass, false, false, false, false);
      *opcounter = AdcGetCurrentCommandCnt(adc_alibi_info->ass, false);
      AdcMutexUnlock(adc_alibi_info->ass);
      break;

    case aakGrossNet:
      AdcMutexLock(adc_alibi_info->ass);
      *opopt = adc_alibi_info->ass->adcDataWorking.dispRdg.info.status.bit.gross;
      (void) AdcGrossNetToggle(adc_alibi_info->ass, false, false);
      /* opCounter = AdcGetCurrentCommandCnt(); */
      AdcMutexUnlock(adc_alibi_info->ass);
      break;

    case aakPrint:
      AdcMutexLock(adc_alibi_info->ass);
      (void) AdcGetTraceableRdg(adc_alibi_info->ass, false, false, false, false);
      *opcounter = AdcGetCurrentCommandCnt(adc_alibi_info->ass, false);
      AdcMutexUnlock(adc_alibi_info->ass);
      break;

    case aakMode:
      AdcIntAlibiSelectMode(adc_alibi_info);
      break;

    case aakEnterPresetTare:
      AdcIntAlibiWeighingSetPresetTare(adc_alibi_info, opcounter);
      break;

    case aakNone:
    default:
      /* Do nothing with other keys */
      break;
  } /* switch */
}


/** Display the preset tare value */
void AdcIntAlibiWeightingDisplayPresetTare(struct s_adc_alibi_info *adc_alibi_info)
{
  AdcRdg rdg;

  AdcMutexLock(adc_alibi_info->ass);
  rdg = adc_alibi_info->ass->adcDataWorking.presetTare;
  AdcMutexUnlock(adc_alibi_info->ass);

  if (aadtSingleDisplay == adc_alibi_info->adc_alibi_display_type)
  {
    /* Display preset tare message */
    AdcIntAlibiSingleDisplayStrNoAnnun(adc_alibi_info, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiPresetTare));
    OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);

    /* Display preset tare value */
    AdcIntAlibiSingleDisplayWeight(adc_alibi_info, &rdg);
    OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);
  }
  else if (aadtDoubleDisplay == adc_alibi_info->adc_alibi_display_type)
  {
    AdcIntAlibiDualDisplayStrNoAnnunAndWeight(adc_alibi_info,
                                              AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiPresetTare),
                                              &rdg);
    OSTimeDly(ADC_ALIBI_CONFIG_PROMPT_PAUSE);
  }
}


/** Process the ADC operation result */
void AdcIntAlibiWeighingProcessAdcResult(struct s_adc_alibi_info *adc_alibi_info, int8u *opcounter, int8u *opopt, bool *show_pieces)
{
  int8u cmd;
  int8u cmd_cnt;
  int8u cmd_status;

  AdcMutexLock(adc_alibi_info->ass);
  cmd = AdcGetCurrentCommand(adc_alibi_info->ass, false);
  cmd_cnt = AdcGetCurrentCommandCnt(adc_alibi_info->ass, false);
  cmd_status = AdcGetCurrentOpStatus(adc_alibi_info->ass, false);
  AdcMutexUnlock(adc_alibi_info->ass);

  if (cmd_cnt == *opcounter &&
      cmd_status != ADC_OP_STATUS_IN_PROGRESS)
  {
    /* Make sure that same error display cannot happen twice */
    (*opcounter)--;

    switch (cmd)
    {
      case ADC_OP_CMD_SEMIAUTO_ZERO:
      case ADC_OP_CMD_SEMIAUTO_TARE:
      case ADC_OP_CMD_SET_PRESET_TARE:
      case ADC_OP_CMD_TRACEABLE_RDG:
      case ADC_OP_CMD_SET_COUNTING:
        switch (cmd_status)
        {
          case ADC_OP_STATUS_FAILED_MOTION:
            AdcIntAlibiShowError(adc_alibi_info,
                                 AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiError),
                                 AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiMotion),
                                 ADC_ALIBI_CONFIG_ERROR_TIMEOUT);
            break;

          case ADC_OP_STATUS_FAILED_BAND:
            AdcIntAlibiShowError(adc_alibi_info,
                                 AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiError),
                                 AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiRange),
                                 ADC_ALIBI_CONFIG_ERROR_TIMEOUT);
            break;

          case ADC_OP_STATUS_FAILED_TRADE:
            AdcIntAlibiShowError(adc_alibi_info,
                                 AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiError),
                                 AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiTrade),
                                 ADC_ALIBI_CONFIG_ERROR_TIMEOUT);
            break;

          case ADC_OP_STATUS_FAILED_HI_RES:
            AdcIntAlibiShowError(adc_alibi_info,
                                 AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiError),
                                 AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiHiRes),
                                 ADC_ALIBI_CONFIG_ERROR_TIMEOUT);
            break;

          default:
          case ADC_OP_STATUS_FAILED_ERROR:
            AdcIntAlibiShowError(adc_alibi_info,
                                 AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiError),
                                 AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiADC),
                                 ADC_ALIBI_CONFIG_ERROR_TIMEOUT);
            break;

          case ADC_OP_STATUS_SUCCESS:
            switch (cmd)
            {
              case ADC_OP_CMD_TRACEABLE_RDG:
                (void) AdcIntAlibiDSDAddRecord(adc_alibi_info, &adc_alibi_info->ass->adcTraceableWeight);
                AdcIntAlibiTraceablePrint(adc_alibi_info);
                break;

              case ADC_OP_CMD_SET_PRESET_TARE:
                AdcMutexLock(adc_alibi_info->ass);
                if (0 == adc_alibi_info->ass->adcDataWorking.presetTare.rdg &&
                    ADC_APPROVAL_MODE_NONE == adc_alibi_info->ass->adcUserDatabase.approvalMode)
                {
                  AdcToGross(adc_alibi_info->ass, false, false);
                }
                else
                {
                  AdcToNet(adc_alibi_info->ass, false, false);
                }
                /* opCounter = AdcGetCurrentCommandCnt(); */
                AdcMutexUnlock(adc_alibi_info->ass);
                break;

              case ADC_OP_CMD_SET_COUNTING:
                if (NULL != show_pieces)
                {
                  *show_pieces = true;
                }
                break;
            }
            break;
        }
        break;

      case ADC_OP_CMD_GROSSNET_TOGGLE:
        if (ADC_OP_STATUS_SUCCESS == cmd_status &&
            0 != adc_alibi_info->ass->adcDataWorking.presetTare.rdg && *opopt) /* opopt = was gross */
        {
          /* Display preset tare */
          AdcIntAlibiWeightingDisplayPresetTare(adc_alibi_info);
        }
        break;
    }
  }
}


/** Run a simple alibi weighing application */
void AdcIntAlibiWeighing(struct s_adc_alibi_info *adc_alibi_info)
{
  int8u opcounter;
  int8u opopt;
  int8u cmd_status;
  int8u cmd_cnt;


  /* Do alibi weighing application loop */
  opcounter = ~AdcGetCurrentCommandCnt(adc_alibi_info->ass, true);
  for (; aamWeighing == adc_alibi_info->alibi_mode; )
  {
    /* Display the weight */
    AdcIntAlibiWeighingDisplayWeight(adc_alibi_info);

    /* Process keys */
    AdcMutexLock(adc_alibi_info->ass);
    cmd_status = AdcGetCurrentOpStatus(adc_alibi_info->ass, false);
    cmd_cnt = AdcGetCurrentCommandCnt(adc_alibi_info->ass, false);
    AdcMutexUnlock(adc_alibi_info->ass);
    if (ADC_OP_STATUS_IN_PROGRESS != cmd_status && cmd_cnt != opcounter)
    {
      AdcIntAlibiWeighingProcessKey(adc_alibi_info, &opcounter, &opopt);
    }

    /* Operation error messages and post-operation processing */
    AdcIntAlibiWeighingProcessAdcResult(adc_alibi_info, &opcounter, &opopt, NULL);

    /* Should we quit? */
    if (AdcIntAlibiForceQuit(adc_alibi_info))
    {
      adc_alibi_info->alibi_mode = aamQuitAlibi;
    }
  } /* for () */
}
