/*
 * alibi_weight.c
 *
 *  Created on: Mar 29, 2012
 *      Author: ian
 */
#include <stdio.h>
#include <string.h>
#include "alibi_display.h"

#include "alibi_weight.h"


/**
 * Convert a weight to a string
 * (number, sign and decimal point, right justified
 * ready for display or printing)
 *
 * @param adc_alibi_info is the alibi information structure
 * @param rdg is the reading to convert to a string (only the reading and decimal point are used)
 * @param num_digits is the number of digits on the display (not including decimal points)
 * @param str is a string to put the weight into
 * @param strlength is the length of str in bytes (including the null terminator)
 */
void AdcIntAlibiWeightToStr(struct s_adc_alibi_info *adc_alibi_info, const AdcRdg *rdg, int num_digits, char *str, size_t strlength)
{
  bool minus;
  int32s weight = rdg->rdg;
  int8u dp = rdg->info.unitdp.bit.dp;
  int i;

  /* Deal with minus */
  minus = (weight < 0);
  if (minus)
  {
    weight = -weight;
  }

  /* Weight to string */
  snprintf(str, strlength, "%*.*ld", num_digits, dp + 1, (long) weight);
  /* Put the decimal point into the string */
  if (0 != dp)
  {
    for (i = num_digits - 1; i > num_digits - 1 - dp; i--)
    {
      str[i + 1] = str[i];
    }
    str[num_digits + 1] = '\0';
    str[num_digits - dp] = '.';
  }

  /* Put sign back in */
  if (minus)
  {
    /* Check if number is too big to fit with minus sign */
    if ('0' != str[0] && ' ' != str[0])
    {
      strncpy(str, AdcIntAlibiGetInternationisation(adc_alibi_info, aaisiUnderload), num_digits);
      str[num_digits] = '\0';
    }
    else
    {
      str[0] = '-';
    }
  }
}


/**
 * Convert an ADC error to a string
 *
 * @param adc_alibi_info is the alibi information structure
 * @param error is the error bit mask (adcData.error)
 * @param num_digits is the number of digits on the display (not including decimal points)
 * @param str is a string to put the weight into
 * @param strlength is the length of str in bytes (including the null terminator)
 */
void AdcIntAlibiErrorToStr(struct s_adc_alibi_info *adc_alibi_info, int32u error, int num_digits, char *str, size_t strlength)
{
  snprintf(str, strlength, "E%0*lX", num_digits - 1, (unsigned long) error);
}
