'''
Created on Sep 28, 2017

@author: ian
'''
import unittest


def pyrincmd_crc16(data, value=0xFFFF):
    """Compute CRC16 for data, value being the checksum seed"""
    crc = value
    for ch in data:
        crc ^= ord(ch) << 8
        for _ in range(0, 8):
            if ((crc & 0x8000) != 0):
                crc = (crc << 1) ^ 0x1021
            else:
                crc <<= 1
    return crc & 0xFFFF


class TestPyRinCmdCrc16(unittest.TestCase):
    """Test the CRC16 algorithm"""

    def test_crc16(self):
        crc16 = pyrincmd_crc16("20110005:")
        self.assertEqual(crc16, 0x6FD0)

        crc16 = pyrincmd_crc16("81110005:00327F78")
        self.assertEqual(crc16, 0x5424)

        crc16 = pyrincmd_crc16("20110003:")
        self.assertEqual(crc16, 0xC576)

        crc16 = pyrincmd_crc16("81110009:009900000000000002E13E007C31E2300000000000000000620BCC2C00000000000000C8B4C03D0BC4B888A0000000000000000000")
        self.assertEqual(crc16, 0x0417)

        crc16 = pyrincmd_crc16("81110003:K401")
        self.assertEqual(crc16, 0xADA1)

        crc16 = pyrincmd_crc16("20110004:")
        self.assertEqual(crc16, 0x5CE1)

        crc16 = pyrincmd_crc16("81110004:V2.61")
        self.assertEqual(crc16, 0x393D)

if __name__ == '__main__':
    unittest.main()
