'''
Created on Sep 21, 2017

@author: ian
'''
import re
import unittest


def pyrincmd_escape(data):
    """Escape data suitable for sending a rincmd string"""
    outstr = ""
    for ch in data:
        if (ord(ch) < ord(' ') or ord(ch) > ord('~') or ch in [ ":", ";"]):
            outstr += "\\{:02X}".format(ord(ch))
        elif (ch == "\\"):
            outstr += "\\\\"
        else:
            outstr += ch
    return outstr


def pyrincmd_unescape(data):
    """Unescape rincmd string into a buffer"""
    outbuf = ""
    hexpattern = '{slash}[0-9a-fA-F]{{2}}'.format(slash=re.escape("\\"))
    slashpattern = '{slash}{slash}'.format(slash=re.escape("\\"))
    pattern = '({}|{})'.format(hexpattern, slashpattern)
    bits = re.split(pattern, data)
    for bit in bits:
        if (re.match(slashpattern, bit)):
            outbuf += "\\"
        elif (re.match(hexpattern, bit)):
            outbuf += chr(int(bit[1:], 16))
        else:
            outbuf += bit
    return outbuf


class TestPyRinCmdEscape(unittest.TestCase):
    """Test escaping/unescaping rincmd data"""

    def test_escape(self):
        """Test escaping rincmd data"""
        unescaped = "This is some test data. \\:;\x00\x1F\x7F\xFF"
        #           This is some test data. \   :   ;   \x00\x1F\x7F\xFF
        expected = "This is some test data. \\\\\\3A\\3B\\00\\1F\\7F\\FF"

        escaped = pyrincmd_escape(data=unescaped)
        self.assertEqual(expected, escaped)

    def test_unescape(self):
        """Test unescaping rincmd data"""

        #           This is some test data. \   :   ;   \x00\x1F\x7F\xFF
        escaped =  "This is some test data. \\\\\\3A\\3B\\00\\1F\\7F\\FF"
        expected = "This is some test data. \\:;\x00\x1F\x7F\xFF"

        unescaped = pyrincmd_unescape(data=escaped)
        self.assertEqual(expected, unescaped)

        # Test some poorly escaped data
        escaped  = "Test \\not hex";
        expected = "Test \\not hex";

        unescaped = pyrincmd_unescape(data=escaped)
        self.assertEqual(expected, unescaped)


if __name__ == '__main__':
    unittest.main()
