import rc_reg_to_c
import rc_reg_to_latex
try:
    import png
    import rc_reg_to_png
    has_pypng = True
except ImportError:
    has_pypng = False
import argparse


def gen_c(infile, outfolder):
    regxlatc = rc_reg_to_c.PyRinCmdRegTranslate_C(xmltree=infile)
    regxlatc.xml_register_to_ch_file(outputfolder=outfolder)

def gen_latex(infile, outfolder):
    regxlattex = rc_reg_to_latex.PyRinCmdRegTranslate_Latex(xmltree=infile)
    regxlattex.xml_register_to_latex_file(outputfolder=outfolder)

def gen_png(infile, outfolder):
    if (not has_pypng):
        print("You don't have pypng installed, so --generate-imagemap is not available. Try 'pip install pypng' first.")
    else:
        regxlatpng = rc_reg_to_png.PyRinCmdRegTranslate_PNG(xmltree=infile)
        regxlatpng.xml_registers_to_png_file(outputfolder=outfolder)

def main(argv=None):

    gen_options = \
    [
        { "name": "c",     "func":gen_c },
        { "name": "latex", "func":gen_latex },
        { "name": "png",   "func":gen_png }
    ]
    gen_choices = [ d["name"] for d in gen_options ]

    # Parse command line options
    parser = argparse.ArgumentParser(description='Generate C register definitions and Latex documentation from XML register definition file.')
    parser.add_argument('inputfile', action='store',
                        help='XML register definition file to process')
    parser.add_argument('--outputfolder', action='store', default='.',
                        help='Specify the output folder')
    parser.add_argument('--generate', action='append', choices=gen_choices, default=[],
                        help='What output to generate. Can be specified multiple times.')
    args = parser.parse_args(args=argv)

    if (len(args.generate) < 1):
        print("Nothing to do")
    else:
        for genname in args.generate:
            gendict = [d for d in gen_options if d['name'] == genname]
            gendict[0]["func"](infile=args.inputfile, outfolder=args.outputfolder)

if __name__ == '__main__':
    main()
