import xml.etree.ElementTree as ET
import datetime
import os
import rc_reg_translate


class PyRinCmdRegTranslate_Latex(rc_reg_translate.PyRinCmdRegTranslate):
    """Class for translating rincmd registers XML into Latex"""

    def register_to_latex_def(self, reg):
        """Convert a register into a latex definition"""
        rperm = reg.get_register_read_permissions_latex()
        wperm = reg.get_register_write_permissions_latex()

        tex = "\\raggedright {name} & \\texttt{{{id:04X}}}\\textsubscript{{H}} & {regtype} & {rperm} & {wperm} & \\raggedright {desc} \\\\ \\hline "\
              .format(name=reg.name, id=reg.register_id, regtype=reg.type, rperm=rperm, wperm=wperm, desc=reg.description)
        return tex

    def registers_to_latex_defs(self):

        reg_dict = {}

        for reg in self.registers:
            tex = self.register_to_latex_def(reg=reg)
            reg_dict[reg.register_id] = tex

        for reg in self.local_stream_registers:
            tex = self.register_to_latex_def(reg=reg)
            reg_dict[reg.register_id] = tex

        tex_reg_list = []
        for reg in sorted(reg_dict):
            tex_reg_list.append(reg_dict[reg])
        return tex_reg_list

    def xml_registers_to_latex(self, outtexfilename=None):
        """Generate a Latex file containing all global and local registers.
           Returns a tuple of (latexfilename, latexstr)"""

        tex_list = self.registers_to_latex_defs()

        if (outtexfilename is None):
            outtexfilename = self.get_output_filename(ext="tex")

        tex_template = \
"""%
% {outfile}
%
% Generated by pyrincmd.rc_reg_to_latex.py
%
% AUTOMATICALLY GENERATED FILE. DO NOT EDIT!
%
% Generated from: {infile}
% Generated at: {date}
%

\\begin{{longtabu}}{{ |p{{5cm}}|l|l|l|l|X| }}
    \\hline
    Name & Address & Type & R & W & Description \\\\ \\hline
    \\endhead
    {regData}
\\end{{longtabu}}
"""     .format(outfile=outtexfilename,
                infile=self.xmlfilename,
                date=str(datetime.datetime.now()),
                regData='\n    '.join(tex_list))
        return outtexfilename, tex_template

    def xml_register_to_latex_file(self, outputfolder=None, outputtexfilename=None):
        """Convert XML registers into a .tex file in outputfolder

           If outputfolder is not None, then it is prepended to outputtexfilename
           If outputtexfilename is None, then it will be 'rc_gen_<xmlname>.tex'
        """
        if (outputtexfilename is None):
            outputtexfilename = self.get_output_filename(ext="tex")
        if (outputfolder is not None):
            outputtexfilename = os.path.join(outputfolder, outputtexfilename)

        _, outtexstr = self.xml_registers_to_latex(outtexfilename=outputtexfilename)

        with open(outputtexfilename, 'w') as f:
            f.write(outtexstr)
            f.close()
