import xml.etree.ElementTree
import png
import os
import rc_reg_translate


class PyRinCmdRegTranslate_PNG(rc_reg_translate.PyRinCmdRegTranslate):
    """Class for translating rincmd registers XML into a PNG image
       to illustrate where registers are used in register address space"""

    def xml_registers_to_png(self, outfile):
        """Generate a black and white PNG to graphically illustrate where registers are used
           within the 16bit register address space
           """

        # Create a list of all used registers
        used_registers = []

        for reg in self.registers:
            reg_id = reg.register_id
            used_registers.append(reg_id)

        for reg in self.local_stream_registers:
            reg_id = reg.register_id
            used_registers.append(reg_id)

        for reg in self.local_registers:
            reg_id = reg.register_id
            used_registers.append(reg_id)

        # Write to file object as a png
        #used_registers.sort()
        bitmap = []
        for reghi in range(0, 256):
            row = []
            for reglo in range(0, 256):
                regidx = (reghi << 8) | reglo
                data = 1 if (regidx in used_registers) else 0
                row.append(data)
            bitmap.append(row)

        png_writer = png.Writer(256, 256, greyscale=True, bitdepth=1)
        png_writer.write(outfile, bitmap)

    def xml_registers_to_png_file(self, outputfolder=None, outputfilename=None):
        """Generate a black and white PNG to graphically illustrate where registers are used
           within the 16bit register address space

           If outputfilename is None, then it will be the register_group_name.png
           """
        if (outputfilename is None):
            outputfilename = self.get_output_filename(ext="png")
        outfname = outputfilename if (outputfolder is None) else os.path.join(outputfolder, outputfilename)

        outfile = open(outfname, 'wb')
        self.xml_registers_to_png(outfile=outfile)
        outfile.close()
