Pyrincmd
========

Pyrincmd is a pure python implementation of the Rinstrum rincmd protocol.

## The Client ##

The client is probably the most outwardly useful part.
It supports the rincmd protocol over TCP, UDP, UDS and serial.
In the examples below, the indicator is located at IP address 192.168.200.254.

Invoking from the command line:

	cd librinhw/rincmd
	python -m pyrincmd.rc_client --help

To read a register (in this case the serial number register):

	python -m pyrincmd.rc_client --output=pretty 192.168.200.254 CMD_READ_DECIMAL 0x5
	> Address: 0x81 addr:1 ADDR_FLAG_RESPONSE
	> Command: 0x16 CMD_READ_DECIMAL
	> Register: 0x0005
	> Channel: None
	> Data: 3477731

To write a register (in this case setpoint 8 target to value 5):

	python -m pyrincmd.rc_client --output=pretty 192.168.200.254 CMD_WRITE_DECIMAL 0xA4E8 5
	> Address: 0x81 addr:1 ADDR_FLAG_RESPONSE
	> Command: 0x17 CMD_WRITE_DECIMAL
	> Register: 0xA4E8
	> Channel: None
	> Data: (response) RC_HNDL_ERR_NONE:RC_NONE_ERR_NONE (0x00:0x00)

To perform multiple actions:

	python -m pyrincmd.rc_client --output=pretty 192.168.200.254 CMD_READ_TYPE 0xA4E8 CMD_READ_DECIMAL 0xA4E8 CMD_WRITE_DECIMAL 0xA4E8 10
	> 0xA4E8 CMD_WRITE_DECIMAL 0xA4E8 10
	> Address: 0x81 addr:1 ADDR_FLAG_RESPONSE
	> Command: 0x01 CMD_READ_TYPE
	> Register: 0xA4E8
	> Channel: None
	> Data: 4
	> Address: 0x81 addr:1 ADDR_FLAG_RESPONSE
	> Command: 0x16 CMD_READ_DECIMAL
	> Register: 0xA4E8
	> Channel: None
	> Data: 5
	> Address: 0x81 addr:1 ADDR_FLAG_RESPONSE
	> Command: 0x17 CMD_WRITE_DECIMAL
	> Register: 0xA4E8
	> Channel: None
	> Data: (response) RC_HNDL_ERR_NONE:RC_NONE_ERR_NONE (0x00:0x00)

Using from python:

	python
	import pyrincmd
	client = pyrincmd.PyRinCmdClient(tcphost='192.168.200.254')
	client.connect()
	reg = client.read_register(command=pyrincmd.PyRinCmdMessage.CMD_READ_DECIMAL, register=5, datatype='guess')
	print(reg.to_pretty_string())
	> Address: 0x81 addr:1 ADDR_FLAG_RESPONSE
	> Command: 0x16 CMD_READ_DECIMAL
	> Register: 0x0005
	> Channel: None
	> Data: 3477731
	reg = client.write_register(command=pyrincmd.PyRinCmdMessage.CMD_WRITE_DECIMAL, register=0xA4E8, data=5)
	print(reg.to_pretty_string())
	> Address: 0x81 addr:1 ADDR_FLAG_RESPONSE
	> Command: 0x17 CMD_WRITE_DECIMAL
	> Register: 0xA4E8
	> Channel: None
	> Data: (response) RC_HNDL_ERR_NONE:RC_NONE_ERR_NONE (0x00:0x00)

It also supports UDP, UDS and serial:

	client = pyrincmd.PyRinCmdClient(udphost='192.168.200.254', udpport=3000)
	...
	client = pyrincmd.PyRinCmdClient(udsfile='sock.uds')
	...
	client = pyrincmd.PyRinCmdClient(serialport='/dev/ttyUSB0', settings='9600n81')
	...
	python -m pyrincmd.rc_client --protocol=udp --port=3000 192.168.200.254 CMD_READ_DECIMAL 5
	...
	python -m pyrincmd.rc_client --protocol=uds sock.uds CMD_READ_DECIMAL 5
	...
	python -m pyrincmd.rc_client --protocol=serial --serial-settings=9600n81 /dev/ttyUSB0 CMD_READ_DECIMAL 5
	...


## Device Finder ##

The device finder locates Rinstrum indicators on the network.
In the examples below, the network broadcast address is 172.17.1.255.

Invoking from the command line:

	cd librinhw/rincmd
	python -m pyrincmd.rc_finder --help

Searching for indicators:

	python -m pyrincmd.rc_finder 172.17.1.255
	> Searching for Rinstrum indicators on 172.17.1.255
	> Search results (3 indicators found):
	> IP Address,    Class,                     Name,   Serial Number,  Version,  Custom ID,  MAC Address,        TCP Port,  Hostname
	> 172.17.1.73,   Digital Weight Indicator,  C520,   3510162,        1.0.3,    C500,       70:4A:E4:00:0D:EB,  2222,      c520-3510162
	> 172.17.1.27,   Ethernet Module,           M4223,  1234,           x.xx,     Unknown,    00:80:A3:95:C1:4A,  2222,      None
	> 172.17.1.143,  Digital Weight Indicator,  C520,   3477744,        1.0.3,    C500,       70:4A:E4:00:07:7C,  2222,      c520-3477744

Using from python:

	python
	import pyrincmd
	devfinder = pyrincmd.PyRinCmdDeviceFinder()
	indicators = devfinder.find_devices(sendtoaddr='172.17.1.255')
	print(indicators)
	> [{'macaddress': '00:80:A3:95:C1:4A', 'hostname': 'None', 'name': 'M4223', 'serialnumber': '1234', 'version': 'x.xx', 'tcpport': '2222', 'customid': 'Unknown', 'ipaddress': '172.17.1.27', 'class': 'Ethernet Module'},
	>  {'macaddress': '70:4A:E4:00:07:7C', 'hostname': 'c520-3477744', 'name': 'C520', 'serialnumber': '3477744', 'version': '1.0.3', 'tcpport': '2222', 'customid': 'C500', 'ipaddress': '172.17.1.143', 'class': 'Digital Weight Indicator'},
	> {'macaddress': '70:4A:E4:00:0D:EB', 'hostname': 'c520-3510162', 'name': 'C520', 'serialnumber': '3510162', 'version': '1.0.3', 'tcpport': '2222', 'customid': 'C500', 'ipaddress': '172.17.1.73', 'class': 'Digital Weight Indicator'}]


## The Automatic Output Client ##

The automatic output client allows reading of automatic output data from an indicator.
It supports TCP, UDP and serial.
In the examples below, the indicator is located at IP address 192.168.200.254.

Invoking from the command line:

	cd librinhw/rincmd
	python -m pyrincmd.rc_autoout --help

To read automatic output data:

	python -m pyrincmd.rc_autoout --show-framing 192.168.200.254
	> \x02    1868G\x03
	> \x02    1868G\x03
	> \x02    1868G\x03
	...


## Rincmd Generation and Parsing Tool ##

This tool allows the generation and parsing of rincmd messages from a command line tool.
There are many command line options for configuring various rincmd fields.

Invoking from the command line:

	cd librinhw/rincmd
	python -m pyrincmd.rc_generate --help
	python -m pyrincmd.rc_generate generate --help
	python -m pyrincmd.rc_generate parse --help

To generate a rincmd message:

	python -m pyrincmd.rc_generate generate CMD_WRITE_DECIMAL 100 1234
	> 20170064:1234;

To parse a rincmd message:

	python -m pyrincmd.rc_generate parse '\x0120120005:283DA\x04'
	> Deframed as crc framed response
	> Address: 0x20 addr:0 ADDR_FLAG_REPLY
	> Command: 0x12 CMD_WRITE_HEXADEC
	> Register: 0x0005
	> Channel: None
	> Data: 2


## The Server ##

The server runs a simple rincmd server from a register list to allow testing of client code.

Invoking from the command line:

	python -m pyrincmd.rc_server --help

To run the server listening on localhost TCP & UDP port 2222, serving up registers defined in registers.xml:

	python -m pyrincmd.rc_server registers.xml --log-level=INFO
	> INFO:__main__:PyRinCmdServer listening on TCP localhost:2222
	> INFO:__main__:PyRinCmdServer listening on UDP localhost:2222

Note the server is not a complete implementation at the moment.
It doesn't support streaming, and probably other things.
Some of the returned error codes may not exactly match the C implementation.


## Generation of source code from XML register definitions ##

TODO
