'''
Created on Sep 13, 2017

@author: ian
'''
import argparse
import unittest
import sys
import logging
sys.path.append("../..")
import pyrincmd
import test_client_server


def run_unit_tests(verbosity):
    """Run all the pyrincmd unit tests"""

    # Load tests
    testsuite = unittest.TestLoader().discover(start_dir='.', pattern='test*.py')
    testsuite.addTests(unittest.TestLoader().loadTestsFromTestCase(pyrincmd.TestPyRinCmdEscape))
    testsuite.addTests(unittest.TestLoader().loadTestsFromTestCase(pyrincmd.TestPyRinCmdCrc16))
    testsuite.addTests(unittest.TestLoader().loadTestsFromTestCase(pyrincmd.TestPyRinCmdFrameCrc))
    testsuite.addTests(unittest.TestLoader().loadTestsFromTestCase(pyrincmd.TestPyRinCmdRegister))

    # Run tests
    result = unittest.TextTestRunner(verbosity=verbosity).run(testsuite)

    # Set exit code
    exitcode = 0 if (result.wasSuccessful()) else 1
    return exitcode


def main(argv=None):
    # Parse command line options
    parser = argparse.ArgumentParser(description='Run unit tests against pyrincmd')
    parser.add_argument('--verbosity', action='store', type=int, default=1,
                        help='Verbosity of unit test results (0..2)')
    parser.add_argument('--log-level', action='store', default=logging.ERROR,
                        help='Debug log level, string or int (FATAL(50), ERROR(40), WARNING(30), INFO(20), DEBUG(10), NOTSET(0)')
    parser.add_argument('--log-file', action='store', default=None,
                        help='Store debug info into file')
    parser.add_argument('--client-comms-log-file', type=argparse.FileType('a'),
                        help='Store client communications log into file')
    args = parser.parse_args(args=argv)

    logging.basicConfig(level=args.log_level, filename=args.log_file)
    test_client_server.client_comms_logfile = args.client_comms_log_file

    # Start server
    test_client_server.start_server(loglevel=args.log_level)
    try:
        # Run tests
        retcode = run_unit_tests(verbosity=args.verbosity)
    finally:
        test_client_server.stop_server()

    return retcode


if __name__ == "__main__":
    retcode = main()
    sys.exit(retcode)
