'''
Created on Sep 12, 2017

@author: ian
'''
import unittest
import pyrincmd


xmlfilename = "registers_test.xml"


class TestRegistersConstants(object):
    """Constants to match those used in the XML file"""

    TEST_STRING_LENGTH = 100
    TEST_UINT8_MIN = 0
    TEST_UINT8_MAX = 0xFF
    TEST_UINT16_MIN = 0
    TEST_UINT16_MAX = 0xFFFF
    TEST_UINT32_MIN = 0
    TEST_UINT32_MAX = 0xFFFFFFFF


class TestPyRinCmdMessage(unittest.TestCase):
    """Test PyRinCmdMessage class"""

    def test_read_command(self):
        address = pyrincmd.PyRinCmdMessage.ADDR_BROADCAST | pyrincmd.PyRinCmdMessage.ADDR_FLAG_REPLY
        command = pyrincmd.PyRinCmdMessage.CMD_READ_HEXADEC
        msg = pyrincmd.PyRinCmdMessage(address=address, command=command, register=0x1234, channel=None, data=None)

        self.assertEqual(first="20111234:", second=msg.to_string())
        self.assertFalse(msg.is_response())
        self.assertFalse(msg.is_error())
        self.assertTrue(msg.is_addressed_to_me(my_addr=1))
        self.assertTrue(msg.is_reply_requested())

    def test_write_command_hex(self):
        address = pyrincmd.PyRinCmdMessage.ADDR_BROADCAST | pyrincmd.PyRinCmdMessage.ADDR_FLAG_REPLY
        command = pyrincmd.PyRinCmdMessage.CMD_WRITE_HEXADEC
        msg = pyrincmd.PyRinCmdMessage(address=address, command=command, register=0x1234, channel=None, data=0xABCD)

        self.assertEqual(first="20121234:ABCD", second=msg.to_string())
        self.assertFalse(msg.is_response())
        self.assertFalse(msg.is_error())
        self.assertTrue(msg.is_addressed_to_me(my_addr=1))
        self.assertTrue(msg.is_reply_requested())

    def test_write_command_dec(self):
        address = pyrincmd.PyRinCmdMessage.ADDR_BROADCAST | pyrincmd.PyRinCmdMessage.ADDR_FLAG_REPLY
        command = pyrincmd.PyRinCmdMessage.CMD_WRITE_DECIMAL
        msg = pyrincmd.PyRinCmdMessage(address=address, command=command, register=0x1234, channel=None, data=0xABCD)

        self.assertEqual(first="20171234:43981", second=msg.to_string())
        self.assertFalse(msg.is_response())
        self.assertFalse(msg.is_error())
        self.assertTrue(msg.is_addressed_to_me(my_addr=1))
        self.assertTrue(msg.is_reply_requested())

    def test_write_command_negative(self):
        address = pyrincmd.PyRinCmdMessage.ADDR_BROADCAST | pyrincmd.PyRinCmdMessage.ADDR_FLAG_REPLY
        command = pyrincmd.PyRinCmdMessage.CMD_WRITE_DECIMAL
        msg = pyrincmd.PyRinCmdMessage(address=address, command=command, register=0x1234, channel=None, data=-56789)

        self.assertEqual(first="20171234:-56789", second=msg.to_string())

        address = pyrincmd.PyRinCmdMessage.ADDR_BROADCAST | pyrincmd.PyRinCmdMessage.ADDR_FLAG_REPLY
        command = pyrincmd.PyRinCmdMessage.CMD_WRITE_HEXADEC
        msg = pyrincmd.PyRinCmdMessage(address=address, command=command, register=0x1234, channel=None, data=-56789)

        self.assertEqual(first="20121234:FFFF222B", second=msg.to_string())

    def test_write_command_string(self):
        address = pyrincmd.PyRinCmdMessage.ADDR_BROADCAST | pyrincmd.PyRinCmdMessage.ADDR_FLAG_REPLY
        command = pyrincmd.PyRinCmdMessage.CMD_WRITE_HEXADEC
        msg = pyrincmd.PyRinCmdMessage(address=address, command=command, register=0x1234, channel=None, data="hello there")

        self.assertEqual(first="20121234:hello there", second=msg.to_string())
        self.assertFalse(msg.is_response())
        self.assertFalse(msg.is_error())
        self.assertTrue(msg.is_addressed_to_me(my_addr=1))
        self.assertTrue(msg.is_reply_requested())

        # Write decimal should have the same effect
        msg.command = pyrincmd.PyRinCmdMessage.CMD_WRITE_DECIMAL

        self.assertEqual(first="20171234:hello there", second=msg.to_string())
        self.assertFalse(msg.is_response())
        self.assertFalse(msg.is_error())
        self.assertTrue(msg.is_addressed_to_me(my_addr=1))
        self.assertTrue(msg.is_reply_requested())


class TestPyRinCmdParser(unittest.TestCase):
    """Test PyRinCmdParser class"""

    def test_write_command_hex(self):
        msg = pyrincmd.PyRinCmdParser().str_to_message("20121234:ABCD", datatype=int)

        self.assertEqual(first=pyrincmd.PyRinCmdMessage.CMD_WRITE_HEXADEC, second=msg.command)
        self.assertEqual(first=0x1234, second=msg.register)
        self.assertEqual(first=0xABCD, second=msg.data)

    def test_write_command_dec(self):
        msg = pyrincmd.PyRinCmdParser().str_to_message("20171234:43981", datatype=int)

        self.assertEqual(first=pyrincmd.PyRinCmdMessage.CMD_WRITE_DECIMAL, second=msg.command)
        self.assertEqual(first=0x1234, second=msg.register)
        self.assertEqual(first=43981, second=msg.data)

    def test_write_command_negative(self):
        msg = pyrincmd.PyRinCmdParser().str_to_message("20171234:-56789", datatype=int)

        self.assertEqual(first=pyrincmd.PyRinCmdMessage.CMD_WRITE_DECIMAL, second=msg.command)
        self.assertEqual(first=0x1234, second=msg.register)
        self.assertEqual(first=-56789, second=msg.data)

        msg = pyrincmd.PyRinCmdParser().str_to_message("20121234:FFFF222B", datatype=int)

        self.assertEqual(first=pyrincmd.PyRinCmdMessage.CMD_WRITE_HEXADEC, second=msg.command)
        self.assertEqual(first=0x1234, second=msg.register)
        self.assertEqual(first=-56789, second=msg.data)

    def test_check_reply(self):
        cmd_address = pyrincmd.PyRinCmdMessage.ADDR_BROADCAST | pyrincmd.PyRinCmdMessage.ADDR_FLAG_REPLY
        command = pyrincmd.PyRinCmdMessage.CMD_WRITE_HEXADEC
        cmd = pyrincmd.PyRinCmdMessage(address=cmd_address, command=command, register=0x1234, channel=None, data=0xABCD)

        resp = pyrincmd.PyRinCmdMessage(address=0x01, command=command, register=0x1234, channel=None, data=0xABCD)
        parser = pyrincmd.PyRinCmdParser()

        # No ADDR_FLAG_RESPONSE should cause exception
        resp.address = 1
        with self.assertRaises(pyrincmd.PyRinCmdException):
            parser.check_reply(cmd=cmd, reply=resp)

        # Command doesn't match
        resp.address = 1 | pyrincmd.PyRinCmdMessage.ADDR_FLAG_RESPONSE
        resp.command = pyrincmd.PyRinCmdMessage.CMD_WRITE_DECIMAL
        with self.assertRaises(pyrincmd.PyRinCmdException):
            pyrincmd.PyRinCmdParser().check_reply(cmd=cmd, reply=resp)

        # Register doesn't match
        resp.command = cmd.command
        resp.register = cmd.register - 1
        with self.assertRaises(pyrincmd.PyRinCmdException):
            pyrincmd.PyRinCmdParser().check_reply(cmd=cmd, reply=resp)

        # All match, no exception
        resp.register = cmd.register
        pyrincmd.PyRinCmdException, pyrincmd.PyRinCmdParser().check_reply(cmd=cmd, reply=resp)
