'''
Created on Sep 12, 2017

@author: ian
'''
import unittest
import tempfile
import os
import subprocess
try:
    import png
    has_pypng = True
except ImportError:
    has_pypng = False
import pyrincmd
import regconst

hasgcc = (subprocess.call(["gcc", "--version"]) == 0)


class TestXMLRegisterToC(unittest.TestCase):

    def setUp(self):
        self.outfolder = tempfile.mkdtemp()
        print(self.outfolder)

    def tearDown(self):
        # TODO Delete tempfolder: shutil.rmtree() is so scary...
        pass

    def test_xml_to_c_and_h(self):
        regxlatc = pyrincmd.PyRinCmdRegTranslate_C(xmltree=regconst.xmlfilename)
        cfname, cstr, hfname, hstr = regxlatc.xml_registers_to_c()

        self.assertEqual(first="rc_gen_testtable.c", second=cfname)
        self.assertEqual(first="rc_gen_testtable.h", second=hfname)

#        print(cstr)
#        print(hstr)

    def test_xml_to_file(self):
        regxlatc = pyrincmd.PyRinCmdRegTranslate_C(xmltree=regconst.xmlfilename)
        regxlatc.xml_register_to_ch_file(outputfolder=self.outfolder)

        self.assertTrue(os.path.isfile(os.path.join(self.outfolder, "rc_gen_testtable.c")))
        self.assertTrue(os.path.isfile(os.path.join(self.outfolder, "rc_gen_testtable.h")))

    @unittest.skipUnless(hasgcc, "Requires gcc (apt-get install gcc)")
    def test_xml_to_c_and_compile(self):
        regxlatc = pyrincmd.PyRinCmdRegTranslate_C(xmltree=regconst.xmlfilename)
        regxlatc.xml_register_to_ch_file(outputfolder=self.outfolder)

        # Generate header file with constants to match those used in registers_test.xml
        hfile = os.path.join(self.outfolder, "test_local.h")
        rc = regconst.TestRegistersConstants()
        rc.to_header_file(filename=hfile, registers=regxlatc)

        # Setup GCC command line and run
        cpath = os.path.join(self.outfolder, "rc_gen_testtable.c")
        ipath1 = "-I{}".format(os.path.abspath("../../.."))
        ipath2 = "-I{}".format(os.path.abspath("../../../hw_lib"))
        compilecmdline = [ "gcc", "-c", ipath1, ipath2, cpath ]
        compile_result = subprocess.call(compilecmdline)

        # It should compile
        self.assertEqual(first=0, second=compile_result, msg="Failed to compile")


class TestXMLRegisterToPng(unittest.TestCase):

    @unittest.skipUnless(has_pypng, "Requires pypng (pip install pypng)")
    def test_xml_to_png(self):
        regxlatpng = pyrincmd.PyRinCmdRegTranslate_PNG(xmltree=regconst.xmlfilename)

        # Write PNG to automatically generated file name
        regxlatpng.xml_registers_to_png_file()
        self.assertTrue(os.path.isfile("rc_gen_testtable.png"))
        r = png.Reader(filename="rc_gen_testtable.png")
        r.read()

        # Write PNG to specified file name
        _, fname = tempfile.mkstemp(suffix=".png")
        regxlatpng.xml_registers_to_png_file(outputfilename=fname)

        r = png.Reader(filename=fname)
        r.read()


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()
