/*
 * rc_cli_async_bulk.h
 *
 *  Created on: 28 May 2018
 *      Author: ian
 */

#ifndef RC_CLI_ASYNC_BULK_H_
#define RC_CLI_ASYNC_BULK_H_

#include <time.h>
#include <stdint.h>
#include <stdbool.h>
#include <stddef.h>
#include <rincmd/rc_cli_error.h>
#include <rincmd/rc_parser.h>
#include <rincmd/rc_public.h>
#include <rincmd/rc_cli_async.h>


/** An async client request structure to be used with @ref rc_async_client_bulk_request() */
struct s_rc_async_client_request
{
  enum e_rc_reg_command command;            /*!< The rincmd command to execute */
  rc_RegId_t regid;                         /*!< The rincmd registers to read/write */
  const int64_t *regvalue_int;              /*!< Pointer to integer value if integer write */
  const void *regvalue_buf;                 /*!< Pointer to data buffer if buffer/string write */
  size_t regvalue_buf_len;                  /*!< Length of the data buffer if buffer/string write */
  t_rc_async_callback_fn callback;          /*!< The callback to call once a response or timeout has been received */
  void *userdata;                           /*!< User data to pass to the callback */
  bool has_timeout;                         /*!< Does this request have a timeout */
  struct timespec timeout;                  /*!< The timeout period */
};


/** Fill out a single bulk request structure for a register read */
enum e_rc_cli_result rc_async_client_read_request_fillout(struct s_rc_async_client_request *acr,
                                                          enum e_rc_reg_command read_cmd, rc_RegId_t regid,
                                                          t_rc_async_callback_fn callback, void *userdata,
                                                          const struct timespec *timeout);


/** Fill out a single bulk request structure for an integer register write */
enum e_rc_cli_result rc_async_client_int_write_request_fillout(struct s_rc_async_client_request *acr,
                                                               enum e_rc_reg_command write_cmd, rc_RegId_t regid,
                                                               const int64_t *regvalue,
                                                               t_rc_async_callback_fn callback, void *userdata,
                                                               const struct timespec *timeout);


/** Fill out a single bulk request structure for a buffer register write */
enum e_rc_cli_result rc_async_client_buf_write_request_fillout(struct s_rc_async_client_request *acr,
                                                               enum e_rc_reg_command write_cmd, rc_RegId_t regid,
                                                               const void *regvalue, size_t regvalue_len,
                                                               t_rc_async_callback_fn callback, void *userdata,
                                                               const struct timespec *timeout);


/** Fill out a single bulk request structure for a string register write */
enum e_rc_cli_result rc_async_client_str_write_request_fillout(struct s_rc_async_client_request *acr,
                                                               enum e_rc_reg_command write_cmd, rc_RegId_t regid,
                                                               const char *regvalue,
                                                               t_rc_async_callback_fn callback, void *userdata,
                                                               const struct timespec *timeout);


/**
 * Read or write a number of registers.
 *
 * The resulting rincmds are assembled into a buffer and written with a single write
 * to minimise network overheads.
 *
 * @param rac is the rincmd asynchronous client
 * @param requests is an array of filled out request structures
 * @param num_requests is the number of requests in the requests
 */
enum e_rc_cli_result rc_async_client_bulk_request(struct s_rc_async_client *rac,
                                                  const struct s_rc_async_client_request *requests,
                                                  size_t num_requests);



#endif /* RC_CLI_ASYNC_BULK_H_ */
