/*
 * rc_comms.c
 *
 *  Created on: Nov 16, 2017
 *      Author: ian
 */
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>

#include <rincmd/rc_cli_comms.h>


/** Put an entry in the comms log if one is configured */
void rc_cli_comms_log(struct s_rc_cli_comms *rc_cli_comms, bool in_not_out, const void *data, ssize_t len)
{
  struct timespec now;
  ssize_t i;
  const char *chptr = data;

  if (NULL == rc_cli_comms || NULL == rc_cli_comms->comms_log || NULL == data)
    return;

  clock_gettime(CLOCK_REALTIME, &now);
  fprintf(rc_cli_comms->comms_log, "%10ld.%.09ld %s \"",
          (long int) now.tv_sec, (long int) now.tv_nsec,
          (in_not_out) ? "<<<" : ">>>");
  for (i = 0; i < len; i++)
  {
    if (!isprint(chptr[i]))
    {
      fprintf(rc_cli_comms->comms_log, "\\x%02X", chptr[i]);
    }
    else if ('\\' == chptr[i])
    {
      fprintf(rc_cli_comms->comms_log, "\\\\");
    }
    else if ('"' == chptr[i])
    {
      fprintf(rc_cli_comms->comms_log, "\\\"");
    }
    else
    {
      fputc(chptr[i], rc_cli_comms->comms_log);
    }
  }
  fprintf(rc_cli_comms->comms_log, "\"\n");
}


/** Connect to the rincmd host to retrieve data from using synchronous connect */
enum e_rc_cli_result rc_cli_comms_connect_sync(struct s_rc_cli_comms *rc_cli_comms, const struct timespec *timeout)
{
  if (NULL == rc_cli_comms || NULL == rc_cli_comms->comms_connect_sync)
    return rcr_paramfail;
  return rc_cli_comms->comms_connect_sync(rc_cli_comms, timeout);
}


/** Connect to the rincmd host to retrieve data from using asynchronous connect */
enum e_rc_cli_result rc_cli_comms_connect_async(struct s_rc_cli_comms *rc_cli_comms, t_rc_async_connect_callback_fn connect_callback, void *userdata)
{
  if (NULL == rc_cli_comms || NULL == rc_cli_comms->comms_connect_async)
    return rcr_paramfail;
  return rc_cli_comms->comms_connect_async(rc_cli_comms, connect_callback, userdata);
}


/** Complete connection to the rincmd host to retrieve data from using asynchronous connect */
enum e_rc_cli_result rc_cli_comms_complete_connect_async(struct s_rc_cli_comms *rc_cli_comms, bool connected_not_timedout)
{
  if (NULL == rc_cli_comms || NULL == rc_cli_comms->comms_complete_connect_async)
    return rcr_paramfail;
  return rc_cli_comms->comms_complete_connect_async(rc_cli_comms, connected_not_timedout);
}


/** Disconnect from the rincmd host */
enum e_rc_cli_result rc_cli_comms_disconnect(struct s_rc_cli_comms *rc_cli_comms)
{
  if (NULL == rc_cli_comms || NULL == rc_cli_comms->comms_disconnect)
    return rcr_paramfail;
  return rc_cli_comms->comms_disconnect(rc_cli_comms);
}


/** Write data to the rincmd host */
enum e_rc_cli_result rc_cli_comms_write(struct s_rc_cli_comms *rc_cli_comms, const void *data, ssize_t *len)
{
  if (NULL == rc_cli_comms || NULL == rc_cli_comms->comms_write || NULL == data || NULL == len)
    return rcr_paramfail;
  rc_cli_comms_log(rc_cli_comms, false, data, *len);
  return rc_cli_comms->comms_write(rc_cli_comms, data, len);
}


/** Read data from the rincmd host */
enum e_rc_cli_result rc_cli_comms_read(struct s_rc_cli_comms *rc_cli_comms, void *data, ssize_t *len)
{
  if (NULL == rc_cli_comms || NULL == rc_cli_comms->comms_read || NULL == data || NULL == len)
    return rcr_paramfail;
  return rc_cli_comms->comms_read(rc_cli_comms, data, len);
}


/** Get the file descriptors for the communication channel */
enum e_rc_cli_result rc_cli_comms_getfds(struct s_rc_cli_comms *rc_cli_comms, int *readfd, int *connectfd)
{
  if (NULL == rc_cli_comms || NULL == rc_cli_comms->comms_getfds || NULL == readfd || NULL == connectfd)
    return rcr_paramfail;
  return rc_cli_comms->comms_getfds(rc_cli_comms, readfd, connectfd);
}


/** Create a rincmd client structure. Don't call directly, called by rc_comms_init_tcpudp() etc */
enum e_rc_cli_result rc_cli_comms_init(struct s_rc_cli_comms **rc_cli_comms, FILE *comms_log)
{
  struct s_rc_cli_comms *lrcc = NULL;

  if (NULL == rc_cli_comms)
  {
    return rcr_paramfail;
  }

  /* Allocate structure */
  lrcc = calloc(1, sizeof(*lrcc));
  if (NULL == lrcc)
    return rcr_allocfail;

  /* Set members */
  lrcc->comms_log = comms_log;

  /* Return new structure */
  *rc_cli_comms = lrcc;
  return rcr_success;
}


/** Clean up a rincmd client comms */
enum e_rc_cli_result rc_cli_comms_destroy(struct s_rc_cli_comms *cli_comms)
{
  enum e_rc_cli_result res;

  if (NULL == cli_comms || NULL == cli_comms->comms_destroy)
    return rcr_paramfail;

  (void) rc_cli_comms_disconnect(cli_comms);

  res = cli_comms->comms_destroy(cli_comms);

  free(cli_comms);
  cli_comms = NULL;
  return res;
}
