/*
 * rc_cli_error.h
 *
 *  Created on: 30 Apr. 2018
 *      Author: ian
 */

#ifndef RC_CLI_ERROR_H_
#define RC_CLI_ERROR_H_


/** Results of rincmd client functions */
enum e_rc_cli_result
{
  rcr_success = 0,          /*!< Function has completed successfully */
  rcr_paramfail,            /*!< Bad parameter passed to function */
  rcr_allocfail,            /*!< Memory allocation has failed */
  rcr_connectfail,          /*!< Connect to host has failed */
  rcr_framingfail,          /*!< Bad framing on a message */
  rcr_parsefail,            /*!< Parsing a response failed */
  rcr_reply_badaddress,     /*!< The address in the response didn't match */
  rcr_reply_notresponse,    /*!< The response bit was not set in a response */
  rcr_reply_badcommand,     /*!< The command didn't match the expected command */
  rcr_reply_badregister,    /*!< The register ID didn't match the expected register */
  rcr_reply_error,          /*!< The error bit was set in the response */
  rcr_msgassemblefail,      /*!< Assembling a message failed */
  rcr_bufferoverflow,       /*!< A buffer would have overflowed with the amount of data supplied */
  rcr_writefail,            /*!< A write to the underlying transport has failed */
  rcr_readfail,             /*!< A read from the underlying transport has failed */
  rcr_listfull,             /*!< A list is full */
  rcr_listempty,            /*!< A list is empty */
  rcr_timeout,              /*!< An operation has timeout without a response */
  rcr_streamsetupfail,      /*!< Setting up a stream has failed */
  rcr_streamsetupsuccess,   /*!< Setting up a stream has succeeded */

  rcr_failure,              /*!< General other failure */
  rcr_num                   /*!< Number of errors */
};


/** Rincmd client result strings, must match @ref enum e_rc_cli_result */
#define RC_CLI_RESULT_STRS \
{ \
  "success", \
  "parameter failure", \
  "memory allocation failure", \
  "connection failure", \
  "message framing failure", \
  "message parsing failure", \
  "reply address doesn't match", \
  "reply is not a response", \
  "reply command doesn't match", \
  "reply register doesn't match", \
  "reply error", \
  "message assembly failure", \
  "buffer overflow", \
  "write failure", \
  "read failure", \
  "list full", \
  "list empty", \
  "timeout", \
  "stream setup failure", \
  "stream setup success", \
\
  "general failure", \
}


/** Get an error/result string for a rincmd client result */
const char *rc_cli_error_strings(enum e_rc_cli_result result);


#endif /* RC_CLI_ERROR_H_ */
