/*
 * rc_cli_sync_stream.c
 *
 *  Created on: Dec 13, 2017
 *      Author: ian
 */
#include <stdio.h>
#include <rincmd/rc_cli_comms.h>
#include <rincmd/rc_streaming.h>

#include <rincmd/rc_cli_sync_stream.h>


/** Setup a streaming set for a synchronous client */
enum e_rc_cli_result rc_sync_client_streaming_setup(struct s_rc_sync_client *rsc,
                                                    rc_RegId_t stream_reg_base,
                                                    enum e_rc_stream_mode stream_mode,
                                                    const int regs[RC_STREAM_DATA_REGISTERS],
                                                    const struct timespec *timeout)
{
  enum e_rc_cli_result result;
  size_t i;

  if (NULL == rsc)
    return rcr_paramfail;

  /* Write each of the streaming data registers */
  for (i = 0; i < RC_STREAM_DATA_REGISTERS; i++)
  {
    result = rc_sync_client_int_register_write(rsc,
                                               RC_MESG_WRITE_HEXADEC, stream_reg_base + RC_STREAM_REG_DATA_OFFSET + i,
                                               regs[i], timeout);
    if (rcr_success != result)
      goto rscss_fail;
  }

  /* Write the mode register */
  result = rc_sync_client_int_register_write(rsc,
                                             RC_MESG_WRITE_HEXADEC, stream_reg_base + RC_STREAM_REG_MODE_OFFSET,
                                             (int64_t) stream_mode, timeout);
  if (rcr_success != result)
    goto rscss_fail;

  /* Write the data register to enable streaming */
  result = rc_sync_client_int_register_write(rsc,
                                             RC_MESG_EXECUTE, stream_reg_base,
                                             1, timeout);
  if (rcr_success != result)
    goto rscss_fail;

  /* Success */
  result = rcr_success;

rscss_fail:
  return result;
}


/** Read a streaming response from a synchronous client */
enum e_rc_cli_result rc_sync_client_streaming_read(struct s_rc_sync_client *rsc,
                                                   rc_RegId_t stream_reg_base,
                                                   int64_t regs[RC_STREAM_DATA_REGISTERS],
                                                   const struct timespec *timeout)
{
  enum e_rc_cli_result result;

  if (NULL == rsc)
    return rcr_paramfail;

  result = rc_sync_client_read_response(rsc, timeout);
  if (rcr_success != result)
    return result;

  return rc_cli_parse_stream_register_read(rsc->cli_parser, stream_reg_base, rsc->buffer, regs);
}


/** Teardown a streaming set for a synchronous client */
enum e_rc_cli_result rc_sync_client_streaming_teardown(struct s_rc_sync_client *rsc,
                                                       rc_RegId_t stream_reg_base,
                                                       const struct timespec *timeout)
{
  enum e_rc_cli_result result;
  size_t i;

  if (NULL == rsc)
    return rcr_paramfail;

  /* Write the data register to disable streaming */
  result = rc_sync_client_int_register_write(rsc,
                                             RC_MESG_EXECUTE, stream_reg_base,
                                             0, timeout);
  if (rcr_success != result)
    goto rscts_fail;

  /* Write the mode register */
  result = rc_sync_client_int_register_write(rsc,
                                             RC_MESG_WRITE_HEXADEC, stream_reg_base + RC_STREAM_REG_MODE_OFFSET,
                                             (int64_t) RC_STREAM_MODE_OFF, timeout);
  if (rcr_success != result)
    goto rscts_fail;

  /* Write each of the streaming data registers */
  for (i = 0; i < RC_STREAM_DATA_REGISTERS; i++)
  {
    result = rc_sync_client_int_register_write(rsc,
                                               RC_MESG_WRITE_HEXADEC, stream_reg_base + RC_STREAM_REG_DATA_OFFSET + i,
                                               0, timeout);
    if (rcr_success != result)
      goto rscts_fail;
  }

  /* Success */
  result = rcr_success;

rscts_fail:
  return result;
}
