/*
 * rc_frame.h
 *
 *  Created on: Nov 21, 2017
 *      Author: ian
 */

#ifndef RC_FRAME_H_
#define RC_FRAME_H_

#include <stdbool.h>
#include <rincmd/rc_cli_comms.h>


/** Structure for framing rincmd messages */
struct s_rc_frame
{
  char *cmd_prefix;     /*!< Prefix for commands */
  char *cmd_postfix;    /*!< Postfix for commands */
  char *resp_prefix;    /*!< Prefix for responses */
  char *resp_postfix;   /*!< Postfix for responses */
  bool has_crc;         /*!< Does this framing mean the message has a CRC */
};


/** Setup framing for CRC messages */
enum e_rc_cli_result rc_frame_init_crc(struct s_rc_frame **rc_frame);


/**
 * Setup framing for normal messages with command terminated by semicolon
 *
 * Command: No prefix, ; postfix
 * Response: No prefix, \r\n postfix
 */
enum e_rc_cli_result rc_frame_init_normal(struct s_rc_frame **rc_frame);


/**
 * Setup framing for normal messages with commands terminated by carriage return
 *
 * Command: No prefix, \r postfix
 * Response: No prefix, \r\n postfix
 */
enum e_rc_cli_result rc_frame_init_normal_cr(struct s_rc_frame **rc_frame);


/**
 * Setup framing for normal messages with commands terminated by line feed
 *
 * Command: No prefix, \r postfix
 * Response: No prefix, \r\n postfix
 */
enum e_rc_cli_result rc_frame_init_normal_lf(struct s_rc_frame **rc_frame);


/** Destroy the framing */
enum e_rc_cli_result rc_frame_destroy(struct s_rc_frame *rc_frame);


/** Insert the pre-message framing into buffer, setting endpos as the starting position for following message */
enum e_rc_cli_result rc_preframe_message(const struct s_rc_frame *rc_frame, bool cmd_not_resp, char *message, size_t maxlen, size_t *endpos);


/** Append the post message framing to the message */
enum e_rc_cli_result rc_postframe_message(const struct s_rc_frame *rc_frame, bool cmd_not_resp, char *message, size_t maxlen);


/**
 * Check the message pre and post framing against what we expect.
 *
 * @param rc_frame is the rincmd framer. Not NULL
 * @param cmd_not_resp is whether to expect command or response framing
 * @param message is the null terminated string to look for the framed message in. Not NULL
 * @param startpos is the position within message that the response begins at (after pre-framing). Can be NULL
 * @param endpos is the position within message that the response ends at (before post-framing). Can be NULL
 * @param msgend is the position within message that the post-framing ends. Can be NULL
 *
 * @return a rincmd client result code
 */
enum e_rc_cli_result rc_frame_response_check(const struct s_rc_frame *rc_frame, bool cmd_not_resp, const char *message, size_t *startpos, size_t *endpos, size_t *msgend);


#endif /* RC_FRAME_H_ */
