/*
 * rc_registers.h
 *
 *  Created on: 24 Mar 2015
 *      Author: merrick
 */

#ifndef RC_REGISTERS_H_
#define RC_REGISTERS_H_

#include <stdint.h>
#include <stdbool.h>
#include <stddef.h>
#include <rincmd/rc_public.h>
#include <rincmd/rc_parser.h>


#if 0
// #define SUBTYPE_NORMAL        0
// #define SUBTYPE_ARRAY         1

// regFunc opt parameter
#define RC_REG_FUNC_DATA_READ          0       ///< StdRegT.regFunc() called on register read with val = ptr to register data, opt = REG_FUNC_DATA_READ
#define RC_REG_FUNC_DATA_WRITE         1       ///< StdRegT.regFunc() called on register write with val = ptr to register data, opt = REG_FUNC_DATA_WRITE
#define RC_REG_FUNC_DATA_READ_MIN      2       ///< StdRegT.regFunc() called on register read minimum with val = (int32_t *) minimum, opt = REG_FUNC_DATA_READ_MIN
#define RC_REG_FUNC_DATA_READ_MAX      3       ///< StdRegT.regFunc() called on register read maximum with val = (int32_t *) maximum, opt = REG_FUNC_DATA_READ_MAX
#define RC_REG_FUNC_DATA_READ_DEF      4       ///< Not implemented
#define RC_REG_FUNC_ENABLE             5       ///< StdRegT.regFunc() called on register enable with val = (uint8_t *) enabled, opt = REG_FUNC_ENABLE
#define RC_REG_FUNC_DATA_READ_PRE      6       ///< StdRegT.regFunc() called on before register read with val = ptr to register data, opt = REG_FUNC_DATA_READ_PRE
#define RC_REG_FUNC_DATA_WRITE_POST    7       ///< StdRegT.regFunc() called on after register write with val = ptr to register data, opt = REG_FUNC_DATA_WRITE_POST
#define RC_REG_FUNC_READ_DISP_SPRT     8       ///< StdRegT.regFunc() called on get display support info with val = (DispSprtT *), opt = REG_FUNC_READ_DISP_SPRT

// enableOpt parameter
#define RC_REG_ENABLE_OPT_EQUAL         0
#define RC_REG_ENABLE_OPT_NOT_EQUAL     1
#define RC_REG_ENABLE_OPT_EQUAL_OR_LESS 2
#define RC_REG_ENABLE_OPT_EQUAL_OR_MORE 3
#define RC_REG_ENABLE_MASK_HIGH         4
#define RC_REG_ENABLE_MASK_LOW          5
#define RC_REG_ENABLE_MASK_EQUAL        6
#define RC_REG_ENABLE_MASK_NOT_EQUAL    7

//////////////////////////////////////////////////////////////////////////
// DP and Unit options for DispSprtT
//////////////////////////////////////////////////////////////////////////

#define DISPSPRT_DP_MASK           0x0F     ///< Mask of decimal point for DispSprtT.data (NOTE: Not portable). See @ref DispSprtT.
#define DISPSPRT_VAR_DP            0x0F     ///< Use variable decimal point (ie the system decimal point). See @ref DispSprtT.
#define DISPSPRT_RDG_DP            0x0E     ///< Use the reading decimal point (ie @ref AdcRdg).  This only applies to registers of type @ref TYPE_WEIGHT. See @ref DispSprtT.
#define DISPSPRT_UNITS_MASK        0xF0     ///< Mask of units for DispSprtT.data (NOTE: Not portable). See @ref DispSprtT.
#define DISPSPRT_UNITS_SHIFTL      4        ///< Left shifts required for the units within DispSprtT.data. See @ref DispSprtT.
#define DISPSPRT_VAR_UNITS         0x0F     ///< Use variable units (ie the system units). See @ref DispSprtT.
#define DISPSPRT_VAR_UNITS_OR      0xF0     ///< No idea???? Seems to be unused. See @ref DispSprtT.
#define DISPSPRT_VAR_UNITS_ALT     0x0E     ///< Use alternative variable units (ie the system alternative units). See @ref DispSprtT.
#define DISPSPRT_VAR_UNITS_ALT_OR  0xE0     ///< No idea???? Seems to be unused. See @ref DispSprtT.
#define DISPSPRT_RDG_UNITS         0x0D     ///< Use the reading units (ie @ref AdcRdg). This only applies to registers of type @ref TYPE_WEIGHT. See @ref DispSprtT.
#endif

#define RC_DISP_UNITS_OTHER_NONE    0x00   /*!< Other units: None */
#define RC_DISP_UNITS_OTHER_M       0x01   /*!< Other units: Minutes */
#define RC_DISP_UNITS_OTHER_S       0x02   /*!< Other units: Seconds */
#define RC_DISP_UNITS_OTHER_H       0x03   /*!< Other units: Hours */
#define RC_DISP_UNITS_OTHER_SLASH   0x04   /*!< Other units: Slash */
#define RC_DISP_UNITS_OTHER_SLASH_M 0x05   /*!< Other units: Slash */
#define RC_DISP_UNITS_OTHER_SLASH_S 0x06   /*!< Other units: Slash */
#define RC_DISP_UNITS_OTHER_SLASH_H 0x07   /*!< Other units: Slash */
#define RC_DISP_UNITS_OTHER_PERCENT 0x08   /*!< Other units: Circle */

#define RC_DISP_MASK_UNITS          0x0F00
#define RC_DISP_MASK_UNITS_OTHER    0x00FF

//////////////////////////////////////////////////////////////////////////
// Error codes
//////////////////////////////////////////////////////////////////////////

/* Major errors */
#define RC_HNDL_ERR_NONE         0
#define RC_HNDL_ERR_PARSE        1
#define RC_HNDL_ERR_CRC          2
#define RC_HNDL_ERR_REG          3
#define RC_HNDL_ERR_READ         4
#define RC_HNDL_ERR_WRITE        5
#define RC_HNDL_ERR_EXEC         6
#define RC_HNDL_ERR_NOTIMP       7

/* Success (RC_HNDL_ERR_NONE) minor errors */
#define RC_NONE_ERR_NONE         0

/* Parser minor errors */
#define RC_PARSE_ERR_NONE     0
#define RC_PARSE_ERR_ADDR     1
#define RC_PARSE_ERR_CMD      2
#define RC_PARSE_ERR_REG      3
#define RC_PARSE_ERR_CHAN     4
#define RC_PARSE_ERR_DELIM    5
#define RC_PARSE_ERR_DATA     6
#define RC_PARSE_ERR_FRAMING  8

/* CRC minor errors */
#define RC_CRC_ERR_NONE       0
#define RC_CRC_ERR_PARSE      1
#define RC_CRC_ERR_BAD        2

/* Register minor errors */
#define RC_REG_ERR_NONE       0

/* Read minor errors */
#define RC_READ_ERR_NONE         0
#define RC_READ_ERR_PERMISSION   1
#define RC_READ_ERR_UNKNOWNTYPE  2
#define RC_READ_ERR_NOTYPDAT     3
#define RC_READ_ERR_COMMAND      4
#define RC_READ_ERR_BADDATA      5
#define RC_READ_ERR_REGNULL      6
#define RC_READ_ERR_BUFSIZE      7
#define RC_READ_ERR_SNPRINTF     8

#define RC_WRITE_ERR_NONE        0
#define RC_WRITE_ERR_PERMISSION  1
#define RC_WRITE_ERR_UNKNOWNTYPE 2
#define RC_WRITE_ERR_NOTYPDAT    3
#define RC_WRITE_ERR_COMMAND     4
#define RC_WRITE_ERR_PARSE       5
#define RC_WRITE_ERR_RANGE_LOW   6
#define RC_WRITE_ERR_RANGE_HIGH  7
#define RC_WRITE_ERR_LENGTH_LOW  8
#define RC_WRITE_ERR_LENGTH_HIGH 9
#define RC_WRITE_ERR_REGNULL    10
#define RC_WRITE_ERR_BUFSIZE    11
#define RC_WRITE_ERR_SNPRINTF   12
#define RC_WRITE_ERR_STRTOLL    13
#define RC_WRITE_ERR_TYPE_LEN   14

#define RC_EXEC_ERR_NONE         0
#define RC_EXEC_ERR_PERMISSION   1
#define RC_EXEC_ERR_UNKNOWNTYPE  2
#define RC_EXEC_ERR_TYPDAT       3
#define RC_EXEC_ERR_COMMAND      4
#define RC_EXEC_ERR_REGNULL      5
#define RC_EXEC_ERR_WRITEBACK    6

#define RC_CNSTR_ERR_NONE     0
#define RC_CNSTR_ERR_BUFSIZE  1
#define RC_CNSTR_ERR_SNPRINTF 2


/** Construct an error code from major and minor codes */
#define RC_BUILD_ERR(major, minor) (major << 8 | minor)


/** Get the major part of the error */
#define RC_ERROR_MAJOR(err) ((err >> 8) & 0xFF)


/** Get the minor part of the error */
#define RC_ERROR_MINOR(err) (err 0xFF)


/** No error (major or minor) */
#define RC_NO_ERROR (RC_BUILD_ERR(RC_HNDL_ERR_NONE, RC_NONE_ERR_NONE))


#define RC_HNDL_RET_REPLY        0
#define RC_HNDL_RET_NOREPLY      1
#define RC_HNDL_RET_ERR          2


//////////////////////////////////////////////////////////////////////////
// Other defines
//////////////////////////////////////////////////////////////////////////


/** Read literal weights and numbers are 7 digits wide with space padding */
#define RC_LITERAL_WEIGHT_WIDTH 7


//////////////////////////////////////////////////////////////////////////
// Register Function Definitions
//////////////////////////////////////////////////////////////////////////

struct s_rc_registerTable *rc_initRegTable(void);

void rc_freeRegTable(struct s_rc_registerTable *regTable);

struct s_rc_localRegisterTable *rc_initLocalRegTable(struct s_rc_registerTable *global);

void rc_freeLocalRegTable(struct s_rc_localRegisterTable *topLevel);

bool rc_addRegs(struct s_rc_registerTable *regTable, struct s_rc_Register *regArray, size_t size);

bool rc_addLocalRegs(struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *regArray, size_t size);

struct s_rc_Register *rc_findReg(struct s_rc_registerTable *regTable, rc_RegId_t addr);

/**
 * Insert a decimal point into a number already in buf.
 * Note that numbers larger than 999999 will be mutilated by this function
 * Note that numbers smaller than -99999 will be mutilated by this function
 *
 * @param buf is a buffer containing a 7 digit zero padded number
 * @param dp is the decimal point position within the buffer (from the right). Zero means no decimal point
 * @param isNegative if set will insert a minus sign in the first character
 */
void rc_formatLit(char buf[RC_LITERAL_WEIGHT_WIDTH + 1], uint8_t dp, bool isNegative);

rc_errcode_t rc_readReg(struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, char *buf, size_t bufSize, enum e_rc_reg_command command);

rc_errcode_t rc_writeReg(struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, char *buf, enum e_rc_reg_command command);

rc_errcode_t rc_execReg(struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, char *buf, size_t bufLen, size_t maxLen, uint32_t command, const char *chan);


/**
 * Rincmd command handler
 *
 * @param inputBuf is a null terminated rincmd message string
 * @param crc if the message has a CRC checksum attached
 * @param outputBuf is the buffer into which the reply is placed
 * @param topLevel is the register table
 * @param indAddr is the address of the indicator
 *
 * Return RC_HNDL_RET_REPLY if outputBuf must be sent as a reply
 *        RC_HNDL_RET_NOREPLY if no action to be taken
 */
rc_errcode_t rc_commandHandler(const char *inputBuf, bool crc,
                               char *outputBuf, size_t outputBufSize,
                               struct s_rc_localRegisterTable *topLevel, uint32_t indAddr);


#endif /* RC_REGISTERS_H_ */
