/*
 * rc_registers.h
 *
 *  Created on: 21 Jan 2016
 *      Author: merrick
 */

#ifndef RC_STREAMING_H_
#define RC_STREAMING_H_

#include <rincmd/rc_public.h>
#include <rincmd/rc_registers.h>


// Offset between the mode (frequency) register and the stream register
#define RC_STREAM_REG_MODE_OFFSET 1
// Offset between the first data register and the stream register
#define RC_STREAM_REG_DATA_OFFSET 2

enum e_rc_stream_mode
{
  RC_STREAM_MODE_OFF = 0,
  RC_STREAM_MODE_AUTO,
  RC_STREAM_MODE_10HZ,
  RC_STREAM_MODE_5HZ,
  RC_STREAM_MODE_1HZ,
  RC_STREAM_MODE_CHANGE,

  RC_STREAM_MODE_NUM
};

#define RC_STREAM_MAKEMASK(freq) (1 << freq)

#define RC_STREAM_AUTO_MASK    RC_STREAM_MAKEMASK(RC_STREAM_MODE_AUTO)
#define RC_STREAM_10HZ_MASK    RC_STREAM_MAKEMASK(RC_STREAM_MODE_10HZ)
#define RC_STREAM_5HZ_MASK     RC_STREAM_MAKEMASK(RC_STREAM_MODE_5HZ)
#define RC_STREAM_1HZ_MASK     RC_STREAM_MAKEMASK(RC_STREAM_MODE_1HZ)
#define RC_STREAM_CHANGE_MASK  RC_STREAM_MAKEMASK(RC_STREAM_MODE_CHANGE)

#define RC_STREAM_ERR_NONE      0
#define RC_STREAM_ERR_BUFFER    1
#define RC_STREAM_ERR_REGISTER  2
#define RC_STREAM_ERR_DISABLED  3
#define RC_STREAM_ERR_NODATA    4 // This will be returned if there is no data to stream.
#define RC_STREAM_ERR_NULLREG   5


struct s_rc_stream *rc_initStreamData(void);

void rc_freeStreamData(struct s_rc_stream *data);

bool rc_addFreeableData(struct s_rc_localRegisterTable *topLevel, void *freeable);

void rc_setChan(struct s_rc_stream *data, const char *chan, size_t size);

uint32_t rc_getStreamFreqMask(const struct s_rc_localRegisterTable *topLevel);

bool rc_addStreams(struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *regPtrArray[]);

bool rc_setStreaming(char *buf, size_t bufSize, size_t maxLen,
                     struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);

bool rc_handleAddStreamDataReg(char *buf, size_t bufSize, size_t maxLen,
                               struct s_rc_localRegisterTable *topLevel, struct s_rc_Register *reg, uint8_t opt);

void rc_resetStreaming(struct s_rc_localRegisterTable *topLevel);

rc_errcode_t rc_streamDataFreq(struct s_rc_localRegisterTable *topLevel, uint8_t streamNo,
                               char *buf, size_t bufSize, int32_t addr);

rc_errcode_t rc_streamDataSync(struct s_rc_localRegisterTable *topLevel, uint8_t streamNo,
                               char *buf, size_t bufSize, int32_t addr);

#endif /* RC_STREAMING_H_ */
