#include <stdio.h>
#include <cutest/CuTest.h>
#include "test_rc_escape.h"
#include "test_rc_frame.h"
#include "test_rc_parser.h"
#include "test_rc_registers.h"
#include "test_rc_streaming.h"
#include "test_rc_strncpy.h"
#include "test_rc_timespec.h"
#include "test_rc_cli_parse.h"
#include "test_rc_cli_async_timer.h"
#include "test_rc_cli_comms.h"


int main(void)
{
  int result;

  CuString *output = CuStringNew();
  CuSuite *suite = CuSuiteNew();

  /* Add tests */
  CuSuiteConsume(suite, CuGetSuite_rc_escape());
  CuSuiteConsume(suite, CuGetSuite_rc_frame());
  CuSuiteConsume(suite, CuGetSuite_rc_parser());
  CuSuiteConsume(suite, CuGetSuite_rc_registers());
  CuSuiteConsume(suite, CuGetSuite_rc_streaming());
  CuSuiteConsume(suite, CuGetSuite_rc_strncpy());
  CuSuiteConsume(suite, CuGetSuite_rc_timespec());
  CuSuiteConsume(suite, CuGetSuite_rc_cli_parse());
  CuSuiteConsume(suite, CuGetSuite_rc_cli_async_timer());
  CuSuiteConsume(suite, CuGetSuite_rc_cli_comms());

  /* Run all tests */
  CuSuiteRun(suite);
  CuSuiteSummary(suite, output);
  CuSuiteDetails(suite, output);

  /* Print results */
  printf("%s\n", output->buffer);
  result = (suite->failCount > 0) ? 1 : 0;

  /* Clean up */
  CuSuiteDelete(suite);
  suite = NULL;
  CuStringDelete(output);
  output = NULL;

  /* Return result */
  return result;
}
