/*
 * rc_client_sync.c
 *
 *  Created on: Dec 15, 2017
 *      Author: ian
 */
#include <stdint.h>
#include <stddef.h>
#include <stdio.h>
#include <rincmd/rc_public.h>
#include <rincmd/rc_cli_comms.h>
#include <rincmd/rc_cli_sync.h>
#include <rincmd/rc_cli_sync_stream.h>
#include <rincmd/rc_cli_error.h>

#include "rc_client_sync.h"


/** Get a timeout pointer */
struct timespec *get_timeout_ptr(struct timespec *to, int timeoutms)
{
  struct timespec *toptr;

  if (timeoutms < 0)
  {
    toptr = NULL;
  }
  else
  {
    to->tv_nsec = (timeoutms % 1000) * 1000000;
    to->tv_sec = timeoutms / 1000;
    toptr = to;
  }
  return toptr;
}


static enum e_rc_cli_result sync_client_streaming(struct s_rc_sync_client *rsc, struct timespec *toptr, unsigned repeats, bool *quit)
{
  /*                                           Disp    Gross   Net     Status  Error */
  const int regs[RC_STREAM_DATA_REGISTERS] = { 0x0025, 0x0026, 0x0027, 0x0021, 0x0023 };
  size_t i;
  int64_t regvalues[RC_STREAM_DATA_REGISTERS] = { 0, 0, 0, 0, 0 };
  enum e_rc_cli_result result;

  /* Setup streaming */
  result = rc_sync_client_streaming_setup(rsc, 0x0040, RC_STREAM_MODE_AUTO, regs, toptr);
  if (rcr_success != result)
    return result;

  /* Read responses */
  for (i = 0; repeats == 0 || i < repeats; i++)
  {
    result = rc_sync_client_streaming_read(rsc, 0x0040, regvalues, toptr);
    if (rcr_success != result)
      return result;
    printf("Streaming result %u: %lld, %lld, %lld, %lld, %lld\n", (unsigned) i,
           (long long) regvalues[0], (long long) regvalues[1], (long long) regvalues[2], (long long) regvalues[3], (long long) regvalues[4]);

    if (NULL != quit && *quit)
    {
      printf("Shutdown requested\n");
      break;
    }
  }

  /* Teardown the streaming */
  return rc_sync_client_streaming_teardown(rsc, 0x0040, toptr);
}


/** Synchronous client example */
enum e_rc_cli_result sync_client_example(struct s_rc_cli_comms *rcc, rc_RegId_t regid,
                                         bool usecrc, bool streaming,
                                         unsigned repeats, int timeoutms, bool *quit)
{
  enum e_rc_cli_result result;
  struct s_rc_sync_client *rsc = NULL;
  int64_t regvalue;
  struct timespec to;
  struct timespec *toptr;
  size_t i;

  toptr = get_timeout_ptr(&to, timeoutms);

  /* Create client and connect */
  result = rc_sync_client_init(&rsc, rcc, RC_MESG_BROADCAST, usecrc);
  if (rcr_success != result)
  {
    printf("rc_sync_client_init() failed with %s\n", rc_cli_error_strings(result));
    goto sc_fail;
  }

  result = rc_sync_client_connect(rsc, toptr);
  if (rcr_success != result)
  {
    printf("rc_sync_client_connect() failed with %s\n", rc_cli_error_strings(result));
    goto sc_fail;
  }

  if (streaming)
  {
    result = sync_client_streaming(rsc, toptr, repeats, quit);
    if (rcr_success != result)
    {
      printf("sync_client_streaming() failed with %s\n", rc_cli_error_strings(result));
      goto sc_fail;
    }
  }
  else
  {
    for (i = 0; repeats == 0 || i < repeats; i++)
    {
      result = rc_sync_client_int_register_read(rsc, RC_MESG_READ_DECIMAL, regid, &regvalue, toptr);
      if (rcr_success != result)
      {
        printf("rc_sync_client_int_register_read() failed with %s\n", rc_cli_error_strings(result));
        goto sc_fail;
      }

      printf("Response %u: %lld\n", (unsigned) i, (long long) regvalue);

      if (NULL != quit && *quit)
      {
        printf("Shutdown requested\n");
        break;
      }
    }
  }

  /* Disconnect from the server */
  (void) rc_sync_client_disconnect(rsc);

  result = rcr_success;

sc_fail:
  if (NULL != rsc)
  {
    (void) rc_sync_client_destroy(rsc);
  }
  return result;
}
