/*
 * rc_timer_demo.c
 *
 *  Created on: 25 May 2018
 *      Author: ian
 */
#define _XOPEN_SOURCE 700
#include <stdio.h>
#include <sys/select.h>
#include <rincmd/rc_cli_error.h>
#include <rincmd/rc_cli_async_timer.h>
#include <rincmd/rc_timespec.h>


/** Callback from the rincmd client library to get the current time */
enum e_rc_cli_result rc_cli_external_gettime(struct timespec *ts)
{
  if (NULL == ts)
    return rcr_paramfail;
  else
    return (clock_gettime(CLOCK_MONOTONIC, ts) == 0) ? rcr_success : rcr_failure;
}


static void timer_oneshot_abs_callback(void *userdata, intptr_t timer_id)
{
  bool *quit = userdata;

  *quit = true;
  printf("timer_oneshot_abs_callback: called, quitting application\n");
}


static void timer_oneshot_rel_callback(void *userdata, intptr_t timer_id)
{
  printf("timer_oneshot_rel_callback: called\n");
}


static void timer_persistent_callback(void *userdata, intptr_t timer_id)
{
  printf("timer_persistent_callback: called\n");
}



static void run_timer_demo(void)
{
  enum e_rc_cli_result result;
  struct s_rc_async_timer *rat;
  bool hastimeout;
  struct timespec timeout;
  struct timespec delay;
  struct timespec now;
  bool quit;

  /* Create the asynchronous timer */
  result = rc_async_timer_init(&rat, 20);
  if (rcr_success != result)
  {
    fprintf(stderr, "reip_async_init_host: failed to create async timer because %s\n", rc_cli_error_strings(result));
    return;
  }

  /* Add some callbacks */
  rc_cli_external_gettime(&now);
  delay.tv_sec = 20;
  delay.tv_nsec = 0;
  rc_timespec_add(&timeout, &now, &delay);
  rc_async_timer_add_oneshot_absolute(rat, timer_oneshot_abs_callback, &quit, &timeout, NULL);

  timeout.tv_sec = 5;
  timeout.tv_nsec = 0;
  rc_async_timer_add_oneshot_relative(rat, timer_oneshot_rel_callback, NULL, &timeout, NULL);

  timeout.tv_sec = 2;
  timeout.tv_nsec = 0;
  rc_async_timer_add_persistent(rat, timer_persistent_callback, NULL, &timeout, NULL);

  /* Run the timer */
  for (quit = false; !quit; )
  {
    hastimeout = false;
    result = rc_async_timer_run(rat, &hastimeout, &timeout);
    if (rcr_success != result)
      break;
    if (!hastimeout)
      break;
    pselect(0, NULL, NULL, NULL, &timeout, NULL);
  }

  if (NULL != rat)
  {
    rc_async_timer_destroy(rat);
    rat = NULL;
  }
}


int main(int argc, char *argv[])
{
  run_timer_demo();

  return 0;
}
