/*
 * test_rc_cli_comms.c
 *
 *  Created on: 25 May 2018
 *      Author: ian
 */
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <sys/un.h>
#include <stdlib.h>
#include <cutest/CuTest.h>
#include <rincmd/rc_cli_comms.h>

#include "test_rc_cli_comms.h"

#define TEST_RC_CLI_COMMS_TCP_PORT 4567
#define TEST_RC_CLI_COMMS_UDP_PORT 4567


static int server_tcp_socket(CuTest *tc, int listen_port)
{
  int server_sockfd;
  struct sockaddr_in server_address;
  int enable;
  int res;

  server_sockfd = socket(AF_INET, SOCK_STREAM, 0);
  CuAssertTrue(tc, (server_sockfd >= 0));
#if 0
  if (server_sockfd < 0)
  {
    fprintf(stderr, "server tcp socket failed\n");
    goto sts_fail;
  }
#endif

  enable = 1;
  res = setsockopt(server_sockfd, SOL_SOCKET, SO_REUSEADDR, &enable, sizeof(enable));
  CuAssertIntEquals(tc, 0, res);
#if 0
  if (res < 0)
  {
    fprintf(stderr, "server setsockopt(SO_REUSEADDR) error\n");
    goto sts_fail;
  }
#endif

  server_address.sin_family = AF_INET;
  server_address.sin_addr.s_addr = htonl(INADDR_ANY);
  server_address.sin_port = htons(listen_port);
  res = bind(server_sockfd, (struct sockaddr *) &server_address, sizeof(server_address));
  CuAssertIntEquals(tc, 0, res);
#if 0
  if (res < 0)
  {
    fprintf(stderr, "server tcp socket bind failed\n");
    goto sts_fail;
  }
#endif

  res = listen(server_sockfd, 5);
  CuAssertIntEquals(tc, 0, res);
#if 0
  if (res < 0)
  {
    fprintf(stderr, "server tcp socket listen failed\n");
    goto sts_fail;
  }
  printf("server listening on tcp port %d\n", listen_port);
#endif
  return server_sockfd;

#if 0
sts_fail:
  if (server_sockfd >= 0)
  {
    close(server_sockfd);
    server_sockfd = -1;
  }
  return -1;
#endif
}


static int server_ud_socket(CuTest *tc, const char *sockfile)
{
  int server_sockfd = -1;
  struct sockaddr_un addr;
  int res;

  server_sockfd = socket(AF_UNIX, SOCK_STREAM, 0);
  CuAssertTrue(tc, (server_sockfd >= 0));
#if 0
  if (server_sockfd < 0)
  {
    fprintf(stderr, "server ud socket failed\n");
    goto sus_fail;
  }
#endif

  memset(&addr, 0, sizeof(addr));
  addr.sun_family = AF_UNIX;
  strncpy(addr.sun_path, sockfile, sizeof(addr.sun_path) - 1);
  addr.sun_path[sizeof(addr.sun_path) - 1] = '\0';
  unlink(sockfile);

  res = bind(server_sockfd, (struct sockaddr *) &addr, sizeof(addr));
  CuAssertIntEquals(tc, 0, res);
#if 0
  if (res < 0)
  {
    fprintf(stderr, "server ud socket bind failed\n");
    goto sus_fail;
  }
#endif

  res = listen(server_sockfd, 5);
  CuAssertIntEquals(tc, 0, res);
#if 0
  if (res < 0)
  {
    fprintf(stderr, "server ud socket listen failed\n");
    goto sus_fail;
  }
  printf("server listening on ud socket file %s\n", sockfile);
#endif
  return server_sockfd;

#if 0
sus_fail:
  if (server_sockfd >= 0)
  {
    close(server_sockfd);
    server_sockfd = -1;
  }
  return -1;
#endif
}


static void check_rc_cli_comms_write_read(CuTest *tc, int client_fd, struct s_rc_cli_comms *rcc)
{
  const char test_message1[] = "This is the first comms test message";
  const char test_message2[] = "This is a completely different comms test message";
  char read_buffer[100];
  ssize_t to_write_len;
  ssize_t written_len;
  ssize_t to_read_len;
  ssize_t read_len;
  enum e_rc_cli_result result;

  /* Write to client comms */
  to_write_len = strlen(test_message1);
  written_len = to_write_len;
  result = rc_cli_comms_write(rcc, test_message1, &written_len);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertIntEquals(tc, to_write_len, written_len);

  /* Read from server client socket */
  to_read_len = sizeof(read_buffer);
  read_len = read(client_fd, read_buffer, to_read_len);
  CuAssertIntEquals(tc, written_len, read_len);
  CuAssertIntEquals(tc, 0, memcmp(test_message1, read_buffer, read_len));

  /* Write to server client socket */
  to_write_len = strlen(test_message2);
  written_len = write(client_fd, test_message2, to_write_len);
  CuAssertIntEquals(tc, to_write_len, written_len);

  /* Read from server client socket */
  to_read_len = sizeof(read_buffer);
  read_len = to_read_len;
  result = rc_cli_comms_read(rcc, read_buffer, &read_len);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertIntEquals(tc, written_len, read_len);
  CuAssertIntEquals(tc, 0, memcmp(test_message2, read_buffer, read_len));
}


struct s_connected_data
{
  CuTest *tc;
  bool connecting_not_disconnecting;
  bool succeeded;
};


static void connected_callback(void *userdata, bool connect_not_disconnect, enum e_rc_cli_result result)
{
  struct s_connected_data *cd = userdata;

  if (cd->connecting_not_disconnecting)
    CuAssertTrue(cd->tc, connect_not_disconnect);
  else
    CuAssertTrue(cd->tc, !connect_not_disconnect);
  CuAssertIntEquals(cd->tc, rcr_success, result);

  cd->succeeded = true;
}


static void check_rc_cli_comms_stream(CuTest *tc, int server_fd, struct s_rc_cli_comms *rcc)
{
  enum e_rc_cli_result result;
  int client_fd;
  struct sockaddr client_address;
  socklen_t client_address_len;
  struct s_connected_data cd;

  /* Inititate the client asynchronous connect */
  cd.tc = tc;
  cd.connecting_not_disconnecting = true;
  cd.succeeded = false;
  result = rc_cli_comms_connect_async(rcc, connected_callback, &cd);
  CuAssertIntEquals(tc, rcr_success, result);

  /* Now on the server end, accept the connection */
  client_fd = accept(server_fd, &client_address, &client_address_len);
  CuAssertTrue(tc, (client_fd >= 0));

  /* Now complete the client connection, connected callback should have been called */
  result = rc_cli_comms_complete_connect_async(rcc, true);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertTrue(tc, cd.succeeded);

  /* Now read and write some data */
  check_rc_cli_comms_write_read(tc, client_fd, rcc);

  /* Now check disconnecting */
  cd.connecting_not_disconnecting = false;
  cd.succeeded = false;
  result = rc_cli_comms_disconnect(rcc);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertTrue(tc, cd.succeeded);

  close(client_fd);
}


static void test_rc_cli_comms_tcp(CuTest *tc)
{
  struct s_rc_cli_comms *rcc_tcp;
  enum e_rc_cli_result result;
  int server_fd;

  /* Create a rincmd client TCP connection for use with hostname:port */
  result = rc_comms_init_tcp(&rcc_tcp, "localhost", TEST_RC_CLI_COMMS_TCP_PORT, NULL);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertPtrNotNull(tc, rcc_tcp);

  /* Setup the server socket */
  server_fd = server_tcp_socket(tc, TEST_RC_CLI_COMMS_TCP_PORT);

  /* Run the test for a stream socket */
  check_rc_cli_comms_stream(tc, server_fd, rcc_tcp);

  /* Clean up */
  result = rc_cli_comms_destroy(rcc_tcp);
  CuAssertIntEquals(tc, rcr_success, result);

  close(server_fd);
}


static void test_rc_cli_comms_uds(CuTest *tc)
{
  struct s_rc_cli_comms *rcc_uds;
  enum e_rc_cli_result result;
  int server_fd;
  char sock_file[] = "/tmp/librincmdtest_uds_XXXXXX";
  int sock_file_fd;

  sock_file_fd = mkstemp(sock_file);
  CuAssertTrue(tc, (sock_file_fd >= 0));
  close(sock_file_fd);

  /* Create a rincmd client UDS connection for use with socket */
  result = rc_comms_init_uds(&rcc_uds, sock_file, NULL);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertPtrNotNull(tc, rcc_uds);

  /* Setup the server socket */
  server_fd = server_ud_socket(tc, sock_file);

  /* Run the test for a stream socket */
  check_rc_cli_comms_stream(tc, server_fd, rcc_uds);

  /* Clean up */
  result = rc_cli_comms_destroy(rcc_uds);
  CuAssertIntEquals(tc, rcr_success, result);

  close(server_fd);
  unlink(sock_file);
}


CuSuite *CuGetSuite_rc_cli_comms(void)
{
  CuSuite *suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_rc_cli_comms_tcp);
  SUITE_ADD_TEST(suite, test_rc_cli_comms_uds);

  return suite;
}
