/*
 * test_rc_streaming.c
 *
 *  Created on: 21 Jan 2016
 *      Author: merrick
 */

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <cutest/CuTest.h>
#include <rincmd/rc_private.h>
#include <rincmd/rc_streaming.h>

#include "test_rc_streaming.h"

static void rc_gen_streams(struct s_rc_localRegisterTable *topLevel)
{
  union u_rc_TypData *typ = calloc(7, sizeof(union u_rc_TypData));

  typ[0].streamData.data=rc_initStreamData();

  typ[1].uint8Data.min=0;
  typ[1].uint8Data.max=RC_STREAM_MODE_CHANGE;
  typ[1].uint8Data.value=&typ[0].streamData.data->freq;
  typ[1].uint8Data.dispSprt=NULL;

  typ[2].registerData.min=0;
  typ[2].registerData.max=0xFFFF;
  typ[2].registerData.value=&typ[0].streamData.data->regAddrs[0];

  typ[3].registerData.min=0;
  typ[3].registerData.max=0xFFFF;
  typ[3].registerData.value=&typ[0].streamData.data->regAddrs[1];

  typ[4].registerData.min=0;
  typ[4].registerData.max=0xFFFF;
  typ[4].registerData.value=&typ[0].streamData.data->regAddrs[2];

  typ[5].registerData.min=0;
  typ[5].registerData.max=0xFFFF;
  typ[5].registerData.value=&typ[0].streamData.data->regAddrs[3];

  typ[6].registerData.min=0;
  typ[6].registerData.max=0xFFFF;
  typ[6].registerData.value=&typ[0].streamData.data->regAddrs[4];

  struct s_rc_Register *arr = calloc(7, sizeof(struct s_rc_Register));

  arr[0].addr = 0x0040;
  arr[0].type=RC_REG_TYPE_STREAM;
  arr[0].perm.PByte=(RC_PERM_RD_A|RC_PERM_WR_A);
  arr[0].regFunc=&rc_setStreaming;
  arr[0].typDat=&typ[0];

  arr[1].addr = 0x0041;
  arr[1].type=RC_REG_TYPE_UINT8;
  arr[1].perm.PByte=(RC_PERM_RD_A|RC_PERM_WR_A);
  arr[1].regFunc=NULL;
  arr[1].typDat=&typ[1];

  arr[2].addr = 0x0042;
  arr[2].type=RC_REG_TYPE_REGISTER;
  arr[2].perm.PByte=(RC_PERM_RD_A|RC_PERM_WR_A);
  arr[2].regFunc=&rc_handleAddStreamDataReg;
  arr[2].typDat=&typ[2];

  arr[3].addr = 0x0043;
  arr[3].type=RC_REG_TYPE_REGISTER;
  arr[3].perm.PByte=(RC_PERM_RD_A|RC_PERM_WR_A);
  arr[3].regFunc=&rc_handleAddStreamDataReg;
  arr[3].typDat=&typ[3];

  arr[4].addr = 0x0044;
  arr[4].type=RC_REG_TYPE_REGISTER;
  arr[4].perm.PByte=(RC_PERM_RD_A|RC_PERM_WR_A);
  arr[4].regFunc=&rc_handleAddStreamDataReg;
  arr[4].typDat=&typ[4];

  arr[5].addr = 0x0045;
  arr[5].type=RC_REG_TYPE_REGISTER;
  arr[5].perm.PByte=(RC_PERM_RD_A|RC_PERM_WR_A);
  arr[5].regFunc=&rc_handleAddStreamDataReg;
  arr[5].typDat=&typ[5];

  arr[6].addr = 0x0046;
  arr[6].type=RC_REG_TYPE_REGISTER;
  arr[6].perm.PByte=(RC_PERM_RD_A|RC_PERM_WR_A);
  arr[6].regFunc=&rc_handleAddStreamDataReg;
  arr[6].typDat=&typ[6];

  struct s_rc_Register *stream[] = {
      &arr[0],
      NULL,
  };

  rc_addRegs(topLevel->local, arr, 7);
  rc_addStreams(topLevel, stream);
  rc_addFreeableData(topLevel, (void *) typ[0].streamData.data);
  rc_addFreeableData(topLevel, (void *) typ);
  rc_addFreeableData(topLevel, (void *) arr);
}

static void test_rc_streaming(CuTest *tc)
{
  struct s_rc_registerTable *globalTable;
  struct s_rc_localRegisterTable *localTable;
  struct s_rc_localRegisterTable *localTable2;
  uint16_t regData[] = {0x1001, 0x0202, 0x0033, 0x4040, 0x0550};
  uint32_t regData32 = 0x55000550;
  struct s_rc_Register *streamReg, *freqReg, *stReg1, *stReg2, *stReg3, *stReg4, *stReg5;
  char buf[RC_SIZE_BUFFER] = {0};
  rc_errcode_t ret;
  int32_t addr = 8;
  uint16_t i;

  union u_rc_TypData typ[] = {
      {.uint16Data = {.min=0, .max=0xFFFF, .value=&regData[0], .dispSprt=NULL}},
      {.uint16Data = {.min=0, .max=0xFFFF, .value=&regData[1], .dispSprt=NULL}},
      {.uint16Data = {.min=0, .max=0xFFFF, .value=&regData[2], .dispSprt=NULL}},
      {.uint16Data = {.min=0, .max=0xFFFF, .value=&regData[3], .dispSprt=NULL}},
      {.uint32Data = {.min=0, .max=0xFFFF, .value=&regData32, .dispSprt=NULL}},
  };

  // Initialise the registers
  struct s_rc_Register arr[5] = {
      {.addr = 0x1000, .type=RC_REG_TYPE_UINT16, .perm={.PByte=(RC_PERM_RD_A|RC_PERM_WR_A)}, .regFunc=NULL, .typDat=&typ[0]},
      {.addr = 0x1001, .type=RC_REG_TYPE_UINT16, .perm={.PByte=(RC_PERM_RD_A|RC_PERM_WR_A)}, .regFunc=NULL, .typDat=&typ[1]},
      {.addr = 0x1002, .type=RC_REG_TYPE_UINT16, .perm={.PByte=(RC_PERM_RD_A|RC_PERM_WR_A)}, .regFunc=NULL, .typDat=&typ[2]},
      {.addr = 0x1003, .type=RC_REG_TYPE_UINT16, .perm={.PByte=(RC_PERM_RD_A|RC_PERM_WR_A)}, .regFunc=NULL, .typDat=&typ[3]},
      {.addr = 0x1004, .type=RC_REG_TYPE_UINT32, .perm={.PByte=(RC_PERM_RD_A|RC_PERM_WR_A)}, .regFunc=NULL, .typDat=&typ[4]},
  };

  // Set up the global register table
  globalTable = rc_initRegTable();
  rc_addRegs(globalTable, arr, 5);

  // Set up the local register table
  localTable = rc_initLocalRegTable(globalTable);
  rc_gen_streams(localTable);
  localTable->priv = RC_PRIVILEGE_GENERAL;

  localTable2 = rc_initLocalRegTable(globalTable);
  rc_gen_streams(localTable2);

  // Find the registers we need
  streamReg = rc_findReg(localTable->local, 0x0040);
  CuAssertPtrNotNull(tc, streamReg);

  freqReg = rc_findReg(localTable->local, 0x0041);
  CuAssertPtrNotNull(tc, freqReg);

  stReg1 = rc_findReg(localTable->local, 0x0042);
  CuAssertPtrNotNull(tc, stReg1);

  stReg2 = rc_findReg(localTable->local, 0x0043);
  CuAssertPtrNotNull(tc, stReg2);

  stReg3 = rc_findReg(localTable->local, 0x0044);
  CuAssertPtrNotNull(tc, stReg3);

  stReg4 = rc_findReg(localTable->local, 0x0045);
  CuAssertPtrNotNull(tc, stReg4);

  stReg5 = rc_findReg(localTable->local, 0x0046);
  CuAssertPtrNotNull(tc, stReg5);

  // Call the stream function. By default this should be off.
  ret = rc_streamDataFreq(localTable, 0, buf, RC_SIZE_BUFFER, addr);
  CuAssertIntEquals(tc, RC_STREAM_ERR_DISABLED, ret);

  // Enable streaming, set channel to 01234.
  ret = rc_execReg(localTable, streamReg, "1", 2, 2, RC_MESG_EXECUTE, "01234");
  CuAssertIntEquals(tc, RC_EXEC_ERR_NONE, ret);

  // Call the stream function. This returns with an error as the frequency is
  // set to RC_STREAM_MODE_OFF by default.
  ret = rc_streamDataFreq(localTable, 0, buf, RC_SIZE_BUFFER, addr);
  CuAssertIntEquals(tc, RC_STREAM_ERR_NODATA, ret);

  // Stream at 10Hz
  ret = rc_writeReg(localTable, freqReg, "2", RC_MESG_WRITE_DECIMAL);
  CuAssertIntEquals(tc, RC_WRITE_ERR_NONE, ret);

  // Call the stream function.
  ret = rc_streamDataFreq(localTable, 0, buf, RC_SIZE_BUFFER, addr);
  CuAssertIntEquals(tc, RC_STREAM_ERR_NONE, ret);
  CuAssertStrEquals(tc, "8811004001234:", buf);

  // Add 1 data register into stream register 1.
  rc_writeReg(localTable, stReg1, "1000", RC_MESG_WRITE_HEXADEC);
  ret = rc_streamDataFreq(localTable, 0, buf, RC_SIZE_BUFFER, addr);
  CuAssertIntEquals(tc, RC_STREAM_ERR_NONE, ret);
  CuAssertStrEquals(tc, "8811004001234:00001001", buf);

  // Test manual stream read
  ret = rc_readReg(localTable, streamReg, buf, RC_SIZE_BUFFER, RC_MESG_READ_HEXADEC);
  CuAssertIntEquals(tc, RC_READ_ERR_NONE, ret);
  CuAssertStrEquals(tc, "00001001", buf);

  // Check nothing comes out of localTable2
  ret = rc_execReg(localTable, rc_findReg(localTable2->local, 0x0040), "1", 2, 2, RC_MESG_EXECUTE, "5678");
  ret = rc_writeReg(localTable, rc_findReg(localTable2->local, 0x0041), "2", RC_MESG_WRITE_DECIMAL);
  ret = rc_streamDataFreq(localTable2, 0, buf, RC_SIZE_BUFFER, addr);
  CuAssertIntEquals(tc, RC_STREAM_ERR_NONE, ret);
  CuAssertStrEquals(tc, "881100405678:", buf);

  // Change data register 1
  rc_writeReg(localTable, stReg1, "1001", RC_MESG_WRITE_HEXADEC);
  ret = rc_streamDataFreq(localTable, 0, buf, RC_SIZE_BUFFER, addr);
  CuAssertIntEquals(tc, RC_STREAM_ERR_NONE, ret);
  CuAssertStrEquals(tc, "8811004001234:00000202", buf);

  // Remove data from stream register 1 and place in register 2. Should give error.
  rc_writeReg(localTable, stReg1, "0000", RC_MESG_WRITE_HEXADEC);
  rc_writeReg(localTable, stReg2, "1000", RC_MESG_WRITE_HEXADEC);
  ret = rc_streamDataFreq(localTable, 0, buf, RC_SIZE_BUFFER, addr);
  CuAssertIntEquals(tc, RC_STREAM_ERR_NONE, ret);
  CuAssertStrEquals(tc, "C811004001234:0000000000001001", buf);

  // Test with a few different register setups.
  rc_writeReg(localTable, stReg1, "1000", RC_MESG_WRITE_HEXADEC);
  rc_writeReg(localTable, stReg2, "0000", RC_MESG_WRITE_HEXADEC);
  rc_writeReg(localTable, stReg5, "1004", RC_MESG_WRITE_HEXADEC);
  ret = rc_streamDataFreq(localTable, 0, buf, RC_SIZE_BUFFER, addr);
  CuAssertIntEquals(tc, RC_STREAM_ERR_NONE, ret);
  CuAssertStrEquals(tc, "C811004001234:0000100100000000000000000000000055000550", buf);

  rc_writeReg(localTable, stReg1, "1001", RC_MESG_WRITE_HEXADEC);
  rc_writeReg(localTable, stReg2, "0000", RC_MESG_WRITE_HEXADEC);
  rc_writeReg(localTable, stReg5, "0000", RC_MESG_WRITE_HEXADEC);
  ret = rc_streamDataFreq(localTable, 0, buf, RC_SIZE_BUFFER, addr);
  CuAssertIntEquals(tc, RC_STREAM_ERR_NONE, ret);
  CuAssertStrEquals(tc, "8811004001234:00000202", buf);

  // Test 5Hz streaming
  ret = rc_writeReg(localTable, freqReg, "3", RC_MESG_WRITE_DECIMAL);
  CuAssertIntEquals(tc, RC_WRITE_ERR_NONE, ret);
  streamReg->typDat->streamData.data->counter = 0;
  for (i = 1; i < 20; i++)
  {
    ret = rc_streamDataFreq(localTable, 0, buf, RC_SIZE_BUFFER, addr);
    if (i % 5 == 0)
    {
      CuAssertIntEquals(tc, RC_STREAM_ERR_NONE, ret);
      CuAssertStrEquals(tc, "8811004001234:00000202", buf);
    }
    else
    {
      CuAssertIntEquals(tc, RC_STREAM_ERR_NODATA, ret);
    }
  }

  // Test 1Hz streaming
  ret = rc_writeReg(localTable, freqReg, "4", RC_MESG_WRITE_DECIMAL);
  CuAssertIntEquals(tc, RC_WRITE_ERR_NONE, ret);
  streamReg->typDat->streamData.data->counter = 0;
  for (i = 1; i < 25; i++)
  {
    ret = rc_streamDataFreq(localTable, 0, buf, RC_SIZE_BUFFER, addr);
    if (i % 10 == 0)
    {
      CuAssertIntEquals(tc, RC_STREAM_ERR_NONE, ret);
      CuAssertStrEquals(tc, "8811004001234:00000202", buf);
    }
    else
    {
      CuAssertIntEquals(tc, RC_STREAM_ERR_NODATA, ret);
    }
  }

  // Test Sync streaming
  ret = rc_writeReg(localTable, freqReg, "1", RC_MESG_WRITE_DECIMAL);
  CuAssertIntEquals(tc, RC_WRITE_ERR_NONE, ret);
  ret = rc_streamDataSync(localTable, 0, buf, RC_SIZE_BUFFER, addr);
  CuAssertIntEquals(tc, RC_STREAM_ERR_NONE, ret);
  CuAssertStrEquals(tc, "8811004001234:00000202", buf);

  // Test On-Change streaming
  ret = rc_writeReg(localTable, freqReg, "5", RC_MESG_WRITE_DECIMAL);
  CuAssertIntEquals(tc, RC_WRITE_ERR_NONE, ret);
  ret = rc_streamDataSync(localTable, 0, buf, RC_SIZE_BUFFER, addr);
  CuAssertIntEquals(tc, RC_STREAM_ERR_NODATA, ret);
  regData[1] = 0x0203;
  ret = rc_streamDataSync(localTable, 0, buf, RC_SIZE_BUFFER, addr);
  CuAssertIntEquals(tc, RC_STREAM_ERR_NONE, ret);
  CuAssertStrEquals(tc, "8811004001234:00000203", buf);
  ret = rc_streamDataSync(localTable, 0, buf, RC_SIZE_BUFFER, addr);
  CuAssertIntEquals(tc, RC_STREAM_ERR_NODATA, ret);
  ret = rc_streamDataSync(localTable, 0, buf, RC_SIZE_BUFFER, addr);
  CuAssertIntEquals(tc, RC_STREAM_ERR_NODATA, ret);
  regData[1] = 0x2002;
  ret = rc_streamDataSync(localTable, 0, buf, RC_SIZE_BUFFER, addr);
  CuAssertIntEquals(tc, RC_STREAM_ERR_NONE, ret);
  CuAssertStrEquals(tc, "8811004001234:00002002", buf);
  regData[1] = 0x0404;
  ret = rc_streamDataSync(localTable, 0, buf, RC_SIZE_BUFFER, addr);
  CuAssertIntEquals(tc, RC_STREAM_ERR_NONE, ret);
  CuAssertStrEquals(tc, "8811004001234:00000404", buf);

  // Check mask
  ret = rc_getStreamFreqMask(localTable);
  CuAssertIntEquals(tc, RC_STREAM_CHANGE_MASK, ret);

  // Streaming should be disabled after we reset streaming
  rc_resetStreaming(localTable);
  ret = rc_streamDataSync(localTable, 0, buf, RC_SIZE_BUFFER, addr);
  CuAssertIntEquals(tc, RC_STREAM_ERR_DISABLED, ret);

  rc_freeLocalRegTable(localTable);
  rc_freeLocalRegTable(localTable2);
  rc_freeRegTable(globalTable);
}

static void test_rc_streambug(CuTest *tc)
{
  struct s_rc_registerTable *globalTable;
  struct s_rc_localRegisterTable *localTable;
  uint16_t regData = 0x1001;
  char buf[RC_SIZE_BUFFER] = {0};
  rc_errcode_t ret;
  int32_t addr = 5;

  union u_rc_TypData typ[] = {
      {.uint16Data = {.min=0, .max=0xFFFF, .value=&regData, .dispSprt=NULL}},
  };

  // Initialise the registers
  struct s_rc_Register arr[1] = {
      {.addr = 0x1000, .type=RC_REG_TYPE_UINT16, .perm={.PByte=(RC_PERM_RD_A|RC_PERM_WR_A)}, .regFunc=NULL, .typDat=&typ[0]},
  };

  // Set up the global register table
  globalTable = rc_initRegTable();
  rc_addRegs(globalTable, arr, 1);

  // Set up the local register table
  localTable = rc_initLocalRegTable(globalTable);
  rc_gen_streams(localTable);

  // Stream on change
//  ret = rc_commandHandler("20120041:5", 11, false, buf, RC_SIZE_BUFFER, localTable, 0x05);
//  /CuAssertIntEquals(tc, RC_HNDL_RET_REPLY, ret);
//  CuAssertStrEquals(tc, "85120041:0000", buf);
  ret = rc_commandHandler("00120041:5", false, buf, RC_SIZE_BUFFER, localTable, 0x05);
  CuAssertIntEquals(tc, RC_HNDL_RET_NOREPLY, ret);

  // Add data to stream
//  ret = rc_commandHandler("20120042:1000", 14, false, buf, RC_SIZE_BUFFER, localTable, 0x05);
//  CuAssertIntEquals(tc, RC_HNDL_RET_REPLY, ret);
//  CuAssertStrEquals(tc, "85120042:0000", buf);
  ret = rc_commandHandler("00120042:1000", false, buf, RC_SIZE_BUFFER, localTable, 0x05);
  CuAssertIntEquals(tc, RC_HNDL_RET_NOREPLY, ret);

  // Kick off streaming
//  ret = rc_commandHandler("20100040:1", 11, false, buf, RC_SIZE_BUFFER, localTable, 0x05);
//  CuAssertIntEquals(tc, RC_HNDL_RET_REPLY, ret);
//  CuAssertStrEquals(tc, "85100040:0000", buf);
  ret = rc_commandHandler("00100040:1", false, buf, RC_SIZE_BUFFER, localTable, 0x05);
  CuAssertIntEquals(tc, RC_HNDL_RET_NOREPLY, ret);

  // Stream data at sync
  regData = 0x2020;
  ret = rc_streamDataSync(localTable, 0, buf, RC_SIZE_BUFFER, addr);
  CuAssertIntEquals(tc, RC_STREAM_ERR_NONE, ret);
  CuAssertStrEquals(tc, "85110040:00002020", buf);

  // Test streaming a bit more
  ret = rc_streamDataSync(localTable, 0, buf, RC_SIZE_BUFFER, addr);
  CuAssertIntEquals(tc, RC_STREAM_ERR_NODATA, ret);
  ret = rc_streamDataSync(localTable, 0, buf, RC_SIZE_BUFFER, addr);
  CuAssertIntEquals(tc, RC_STREAM_ERR_NODATA, ret);
  regData = 0x0404;
  ret = rc_streamDataSync(localTable, 0, buf, RC_SIZE_BUFFER, addr);
  CuAssertIntEquals(tc, RC_STREAM_ERR_NONE, ret);
  CuAssertStrEquals(tc, "85110040:00000404", buf);

  rc_freeLocalRegTable(localTable);
  rc_freeRegTable(globalTable);
}

CuSuite *CuGetSuite_rc_streaming(void)
{
  CuSuite *suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_rc_streaming);
  SUITE_ADD_TEST(suite, test_rc_streambug);

  return suite;
}
