/*
 * db_buffer.h
 *
 *  Created on: 12/11/2010
 *      Author: ian
 */

#ifndef DB_BUFFER_H_
#define DB_BUFFER_H_

#include <stdint.h>
#include <stddef.h>
#include <stdio.h>
#include <stdbool.h>
#include <rindb/db_error.h>


/** Buffer structure */
struct s_db_buffer
{
  uint8_t *buffer;           /*!< The data buffer */
  size_t posn;               /*!< Current position within the buffer */
  size_t length;             /*!< Length of buffer */
  bool mallocd;              /*!< Is the data buffer malloc'd */
  bool can_resize;           /*!< Can the buffer be resized */
  size_t resize_increment;   /*!< Add this much to the buffer when re-sizing */
};


/** Create a buffer with static (supplied) buffer */
enum eDBError db_buffer_create_static(struct s_db_buffer *db_buffer, uint8_t *buf, size_t length);


/** Create a buffer by malloc'ing a buffer of specified length */
enum eDBError db_buffer_create_malloc(struct s_db_buffer *db_buffer, size_t length, size_t resize_increment);


/** Create a buffer from a file stream */
enum eDBError db_buffer_create_file(struct s_db_buffer *db_buffer, FILE *f, size_t resize_increment);


/** Resize a buffer, will round up newlength to next resize_increment */
enum eDBError db_buffer_resize(struct s_db_buffer *db_buffer, size_t newlength);


/** Destroy the buffer, free if originally malloc'd */
enum eDBError db_buffer_destroy(struct s_db_buffer *db_buffer);


/** Load as much data as possible (all if buffer resizing enabled), from current position in file into buffer */
enum eDBError db_buffer_load_file(struct s_db_buffer *db_buffer, FILE *f);


/** Move buffer position back to start of buffer */
enum eDBError db_buffer_rewind(struct s_db_buffer *db_buffer);


/** Add a byte to the buffer and advance the position */
enum eDBError db_buffer_addbyte(struct s_db_buffer *db_buffer, uint8_t data);


/** Add a array of bytes to the buffer and advance the position */
enum eDBError db_buffer_addbuffer(struct s_db_buffer *db_buffer, const void *data, size_t length);


/** Get a byte from the buffer and advance the position */
enum eDBError db_buffer_getbyte(struct s_db_buffer *db_buffer, uint8_t *data);


/** Get an array of bytes from the buffer and advance the position */
enum eDBError db_buffer_getbuffer(struct s_db_buffer *db_buffer, void *data, size_t length);


#endif /* DB_BUFFER_H_ */
