/*
 * db_changelog_reader.h
 *
 *  Created on: Apr 21, 2012
 *      Author: ian
 */

#ifndef DB_CHANGELOG_READER_H_
#define DB_CHANGELOG_READER_H_

#include <stddef.h>
#include <time.h>
#include <stdbool.h>
#include <libcsv/csv.h>
#include <rindb/db_error.h>
#include <rindb/db_changelog.h>


/** An entry in the change log */
struct s_db_changelog_entry
{
  char *cal_counter;          /*!< Calibration counter column */
  char *config_counter;       /*!< Configuration counter column */
  struct tm timestamp;        /*!< Entry time stamp */
  char *db_name;              /*!< Name of database */
  char *db_elem_name;         /*!< Database element name */
  char *menu_elem_name;       /*!< Menu element name */
  bool has_index;             /*!< Does this element have an index */
  size_t index;               /*!< Index, if array setting */
  char *old_value;            /*!< Previous value */
  char *new_value;            /*!< Current value */
  char *log_mask;             /*!< Log mask description (eg trade, user) */
};


/** Used for navigating the change log */
enum eDBChangeLogReaderLocator { dbclrlFirst, dbclrlLast, dbclrlNext, dbclrlPrevious };


/** Change log reader */
struct s_db_changelog_reader
{
  FILE *file;                                     /*!< Change log file */
  bool ourfile;                                   /*!< File was opened by library, therefore needs to be closed by library */
  enum eDBChangeLogElement current_elem;          /*!< Used by CSV parser callbacks to know where we're at */
  struct csv_parser csvparse;                     /*!< CSV file parser used to parse the file */

  enum eDBError csvparse_last_result;             /*!< Result of last CSV line parsing, used by CSV parser callbacks */
  struct s_db_changelog_entry *cl_entry;          /*!< Current entry in the change log, used by CSV parser callbacks */
  size_t max_csvlinesize;                         /*!< Maximum length of a line in the change log */
  char *csv_line;                                 /*!< Single line from the CSV file, used for parsing */
};


/** Create by mallocing a new change log entry structure. Members are set to NULL/0. */
enum eDBError db_changelog_reader_entry_create(struct s_db_changelog_entry **cl_entry);


/** Destroy a change log reader entry, frees all allocated memory */
enum eDBError db_changelog_reader_entry_destroy(struct s_db_changelog_entry *cl_entry);


/** Read an entry from the file */
enum eDBError db_changelog_reader_entry_get(struct s_db_changelog_reader *clr, enum eDBChangeLogReaderLocator locator, struct s_db_changelog_entry *entry);


/** Create a change log reader from a FILE. Caller is responsible for closing file after db_changelog_reader_destroy() call. */
enum eDBError db_changelog_reader_create_file(struct s_db_changelog_reader **clr, FILE *file, size_t max_csvlinesize);


/** Create a change log reader from a filename (file is opened read only). db_changelog_reader_destroy() will close the file. */
enum eDBError db_changelog_reader_create_filename(struct s_db_changelog_reader **clr, const char *filename, size_t max_csvlinesize);


/** Destroy a change log reader */
enum eDBError db_changelog_reader_destroy(struct s_db_changelog_reader *clr);


#endif /* DB_CHANGELOG_READER_H_ */
