/*
 * db_crc.c
 *
 *  Created on: 20/06/2011
 *      Author: ian
 */

#include <stdint.h>

#include <rindb/db_crc.h>


#define WIDTH (8 * sizeof(crc_t))
#define TOPBIT (1 << (WIDTH - 1))
#define POLYNOMIAL 0x04C11DB7


/** Calculate CRC of data */
crc_t db_crc32(const uint8_t *message, size_t message_length, crc_t crc_seed)
{
  crc_t remainder = crc_seed;
  size_t byte;
  size_t bit;

  /* Perform modulo-2 division, a byte at a time */
  for (byte = 0; byte < message_length; ++byte)
  {
    /* Bring the next byte into the remainder */
    remainder ^= (message[byte] << (WIDTH - 8));

    /* Perform modulo-2 division, a bit at a time */
    for (bit = 8; bit > 0; --bit)
    {
      /* Try to divide the current data bit */
      if (remainder & TOPBIT)
      {
        remainder = (remainder << 1) ^ POLYNOMIAL;
      }
      else
      {
        remainder = (remainder << 1);
      }
    }
  }

  /* The final remainder is the CRC result */
  return remainder;
}
