/*
 * db_database.h
 *
 *  Created on: 08/07/2011
 *      Author: ian
 */

#ifndef DB_DATABASE_H_
#define DB_DATABASE_H_

#include <stdint.h>
#include <stddef.h>
#include <stdbool.h>
#include <rindb/db_element.h>


/** Database top level structure, application code instantiates these */
struct s_db_database
{
  uint32_t db_database_version;            /*!< Database version */
  uint32_t db_database_chksum_seed;        /*!< Database checksum seed */
  struct s_db_element *db_elements;        /*!< Array of database elements in this section */
  size_t db_num_elements;                  /*!< Number of database elements in this section */
  const char *db_name;                     /*!< Database name, saved into some database file formats (eg XML) */
  struct s_db_buffer *db_checksum_buffer;  /*!< Pre-allocated buffer used for calculating checksum */
};


/* Forward declaration of database interpreter */
struct s_db_database_interp;


/** Calculate the number of elements in an array */
#define DB_NUM_ELEMENTS(array) (sizeof(array) / sizeof(array[0]))


/** Database start export function */
typedef enum eDBError (*t_db_database_export_start_fn)(struct s_db_database_interp *ddi, const struct s_db_database *db, uint32_t at_version, uint32_t checksum);


/** Database end export function */
typedef enum eDBError (*t_db_database_export_end_fn)(struct s_db_database_interp *ddi, const struct s_db_database *db, uint32_t at_version, uint32_t checksum);


/** Database start import function */
typedef enum eDBError (*t_db_database_import_start_fn)(struct s_db_database_interp *ddi, struct s_db_database *db, uint32_t *version, uint32_t *checksum);


/** Database end import function */
typedef enum eDBError (*t_db_database_import_end_fn)(struct s_db_database_interp *dsi, struct s_db_database *db, uint32_t *version, uint32_t *checksum);


/** Database interpreter structure */
struct s_db_database_interp
{
  /* Data */
  void *database_data;
  struct s_db_interp interp;

  /* Export functions */
  t_db_database_export_start_fn database_export_start_fn;
  t_db_database_export_end_fn database_export_end_fn;

  /* Import functions */
  t_db_database_import_start_fn database_import_start_fn;
  t_db_database_import_end_fn database_import_end_fn;
};


/** Write an entire database */
enum eDBError db_database_write(const struct s_db_database *database, struct s_db_database_interp *ddi, uint32_t at_version);


/** Read an entire database */
enum eDBError db_database_read(struct s_db_database *database, struct s_db_database_interp *ddi, uint32_t *version, bool ignore_checksum);


/** Set a database to defaults */
enum eDBError db_database_default(struct s_db_database *database);


/** Update the previous versions for all elements of the database */
enum eDBError db_database_update_previous(struct s_db_database *database);


/** Revert a database to the values stored in each element's previous value */
enum eDBError db_database_revert(struct s_db_database *database);


/** Initialise all elements in the database */
enum eDBError db_database_init(struct s_db_database *database);


/** Destroy all elements in the database */
enum eDBError db_database_destroy(struct s_db_database *database);


#endif /* DB_DATABASE_H_ */
