/*
 * db_error.h
 *
 *  Created on: 19/06/2011
 *      Author: ian
 */

#ifndef DB_ERROR_H_
#define DB_ERROR_H_


/** Errors returned by database library functions */
enum eDBError { dbeSuccess = 0,             /*!< Function completed successfully */
                dbeFailure,                 /*!< Unspecified failure */
                dbeMemAllocFail,            /*!< Memory allocation failed */
                dbeMemFreeFail,             /*!< Memory free failed */
                dbeBufferFull,              /*!< Buffer is full */
                dbeParseFail,               /*!< Parsing of value failed */
                dbeOutsideLimits,           /*!< Database value was outside limits */
                dbeChecksumFail,            /*!< Checksum failure */
                dbeFileFail,                /*!< A file operation failed */
                dbeParamFail,               /*!< A function parameter is outside acceptable limits */
                dbeChangelogFull,           /*!< The change log is full */
                dbeNullPointer,             /*!< A NULL pointer was encountered */
                dbeChangelogNoMoreRecords,  /*!< There are no more records in the change log */
                dbeNotFound,                /*!< A search didn't find anything */

                dbeNumErrors                /*!< Number of errors */
};


/** Strings that match the errors in @ref enum eDBError */
#define DB_ERROR_STRS { "success", \
                        "unspecified failure", \
                        "memory allocation failure", \
                        "memory free failure", \
                        "buffer full", \
                        "parsing failure", \
                        "value outside limits", \
                        "checksum verification failure", \
                        "file operation failure", \
                        "parameter failure", \
                        "change log full failure", \
                        "null pointer failure", \
                        "no more records in change log", \
                        "not found" \
                      }


/** Convert a database library error into a human readable error string */
const char *db_error_to_string(enum eDBError dberror);


#endif /* DB_ERROR_H_ */
