/*
 * db_interp.c
 *
 *  Created on: 12/11/2010
 *      Author: ian
 */

#include <rindb/db_interp.h>
#include <rindb/db_element.h>


/*
 * Export functions
 */


enum eDBError db_interp_uint8_export(const char *db_name, struct s_db_interp *db_interp, const char *name, const struct s_db_element_uint8 *elem)
{
  if (NULL != db_interp->uint8_export)
    return db_interp->uint8_export(db_name, name, db_interp->interpreter, *elem->data);
  else
    return dbeNullPointer;
}


enum eDBError db_interp_uint32_export(const char *db_name, struct s_db_interp *db_interp, const char *name, const struct s_db_element_uint32 *elem)
{
  if (NULL != db_interp->uint32_export)
    return db_interp->uint32_export(db_name, name, db_interp->interpreter, *elem->data);
  else
    return dbeNullPointer;
}


enum eDBError db_interp_int32_export(const char *db_name, struct s_db_interp *db_interp, const char *name, const struct s_db_element_int32 *elem)
{
  if (NULL != db_interp->int32_export)
    return db_interp->int32_export(db_name, name, db_interp->interpreter, *elem->data);
  else
    return dbeNullPointer;
}


enum eDBError db_interp_str_export(const char *db_name, struct s_db_interp *db_interp, const char *name, const struct s_db_element_string *elem)
{
  if (NULL != db_interp->str_export)
    return db_interp->str_export(db_name, name, db_interp->interpreter, elem->data);
  else
    return dbeNullPointer;
}


enum eDBError db_interp_buf_export(const char *db_name, struct s_db_interp *db_interp, const char *name, const struct s_db_element_buffer *elem)
{
  if (NULL != db_interp->buf_export)
    return db_interp->buf_export(db_name, name, db_interp->interpreter, elem->data, elem->length);
  else
    return dbeNullPointer;
}


enum eDBError db_interp_uint8_array_export(const char *db_name, struct s_db_interp *db_interp, const char *name, const struct s_db_element_uint8_array *elem)
{
  if (NULL != db_interp->uint8_array_export)
    return db_interp->uint8_array_export(db_name, name, db_interp->interpreter, elem->data, elem->num_elements);
  else
    return dbeNullPointer;
}


enum eDBError db_interp_uint32_array_export(const char *db_name, struct s_db_interp *db_interp, const char *name, const struct s_db_element_uint32_array *elem)
{
  if (NULL != db_interp->uint32_array_export)
    return db_interp->uint32_array_export(db_name, name, db_interp->interpreter, elem->data, elem->num_elements);
  else
    return dbeNullPointer;
}


enum eDBError db_interp_int32_array_export(const char *db_name, struct s_db_interp *db_interp, const char *name, const struct s_db_element_int32_array *elem)
{
  if (NULL != db_interp->int32_array_export)
    return db_interp->int32_array_export(db_name, name, db_interp->interpreter, elem->data, elem->num_elements);
  else
    return dbeNullPointer;
}


enum eDBError db_interp_string_array_export(const char *db_name, struct s_db_interp *db_interp, const char *name, const struct s_db_element_string_array *elem)
{
  const char *const *const_ptr;

  if (NULL != db_interp->string_array_export)
  {
    /* Not sure why casting from (char *const *) to (const char *const *) generates a warning, so cast around it */
    const_ptr = (const char *const *) elem->data;
    return db_interp->string_array_export(db_name, name, db_interp->interpreter, const_ptr, elem->num_elements);
  }
  else
    return dbeNullPointer;
}


/*
 * Import functions
 */


enum eDBError db_interp_uint8_import(const char *db_name, struct s_db_interp *db_interp, const char *name, struct s_db_element_uint8 *elem)
{
  if (NULL != db_interp->uint8_import)
    return db_interp->uint8_import(db_name, name, db_interp->interpreter, elem->data, elem->minimum, elem->maximum, elem->default_value);
  else
    return dbeNullPointer;
}


enum eDBError db_interp_uint32_import(const char *db_name, struct s_db_interp *db_interp, const char *name, struct s_db_element_uint32 *elem)
{
  if (NULL != db_interp->uint32_import)
    return db_interp->uint32_import(db_name, name, db_interp->interpreter, elem->data, elem->minimum, elem->maximum, elem->default_value);
  else
    return dbeNullPointer;
}


enum eDBError db_interp_int32_import(const char *db_name, struct s_db_interp *db_interp, const char *name, struct s_db_element_int32 *elem)
{
  if (NULL != db_interp->int32_import)
    return db_interp->int32_import(db_name, name, db_interp->interpreter, elem->data, elem->minimum, elem->maximum, elem->default_value);
  else
    return dbeNullPointer;
}


enum eDBError db_interp_str_import(const char *db_name, struct s_db_interp *db_interp, const char *name, struct s_db_element_string *elem)
{
  if (NULL != db_interp->str_import)
    return db_interp->str_import(db_name, name, db_interp->interpreter, elem->data, elem->max_length, elem->default_value);
  else
    return dbeNullPointer;
}


enum eDBError db_interp_buf_import(const char *db_name, struct s_db_interp *db_interp, const char *name, struct s_db_element_buffer *elem)
{
  if (NULL != db_interp->buf_import)
    return db_interp->buf_import(db_name, name, db_interp->interpreter, elem->data, elem->length, elem->default_value);
  else
    return dbeNullPointer;
}


enum eDBError db_interp_uint8_array_import(const char *db_name, struct s_db_interp *db_interp, const char *name, struct s_db_element_uint8_array *elem)
{
  if (NULL != db_interp->uint8_array_import && NULL != elem && NULL != elem->data)
    return db_interp->uint8_array_import(db_name, name, db_interp->interpreter, elem->data, elem->num_elements,
                                         elem->minimum, elem->maximum, elem->default_value,
                                         elem->single_minimum, elem->single_maximum, elem->single_default);
  else
    return dbeNullPointer;
}


enum eDBError db_interp_uint32_array_import(const char *db_name, struct s_db_interp *db_interp, const char *name, struct s_db_element_uint32_array *elem)
{
  if (NULL != db_interp->uint32_array_import && NULL != elem && NULL != elem->data)
    return db_interp->uint32_array_import(db_name, name, db_interp->interpreter, elem->data, elem->num_elements,
                                          elem->minimum, elem->maximum, elem->default_value,
                                          elem->single_minimum, elem->single_maximum, elem->single_default);
  else
    return dbeNullPointer;
}


enum eDBError db_interp_int32_array_import(const char *db_name, struct s_db_interp *db_interp, const char *name, struct s_db_element_int32_array *elem)
{
  if (NULL != db_interp->int32_array_import && NULL != elem && NULL != elem->data)
    return db_interp->int32_array_import(db_name, name, db_interp->interpreter, elem->data, elem->num_elements,
                                         elem->minimum, elem->maximum, elem->default_value,
                                         elem->single_minimum, elem->single_maximum, elem->single_default);
  else
    return dbeNullPointer;
}


enum eDBError db_interp_string_array_import(const char *db_name, struct s_db_interp *db_interp, const char *name, struct s_db_element_string_array *elem)
{
  if (NULL != db_interp->string_array_import && NULL != elem && NULL != elem->data)
    return db_interp->string_array_import(db_name, name, db_interp->interpreter, elem->data, elem->num_elements,
                                          elem->max_length, elem->default_value,
                                          elem->single_max_length, elem->single_default);
  else
    return dbeNullPointer;
}
