/*
 * db_interp.h
 *
 *  Created on: 12/11/2010
 *      Author: ian
 */

#ifndef DB_INTERP_H_
#define DB_INTERP_H_

#include <stdint.h>
#include <stddef.h>
#include <rindb/db_error.h>


/* Forward declarations */
struct s_db_element_uint8;
struct s_db_element_uint32;
struct s_db_element_int32;
struct s_db_element_string;
struct s_db_element_buffer;
struct s_db_element_uint8_array;
struct s_db_element_uint32_array;
struct s_db_element_int32_array;
struct s_db_element_string_array;


/*
 * Export function typedefs
 */


/** Function to export a unsigned 8bit integer setting */
typedef enum eDBError (*t_db_interp_uint8_export_fn)(const char *db_name, const char *name, void *interpreter, uint8_t data);


/** Function to export an unsigned 32bit integer setting */
typedef enum eDBError (*t_db_interp_uint32_export_fn)(const char *db_name, const char *name, void *interpreter, uint32_t data);


/** Function to export a signed 32bit integer setting */
typedef enum eDBError (*t_db_interp_int32_export_fn)(const char *db_name, const char *name, void *interpreter, int32_t data);


/** Function to export a string (null terminated) setting */
typedef enum eDBError (*t_db_interp_str_export_fn)(const char *db_name, const char *name, void *interpreter, const char *data);


/** Function to export a buffer setting */
typedef enum eDBError (*t_db_interp_buf_export_fn)(const char *db_name, const char *name, void *interpreter, const uint8_t *data, size_t datalen);


/** Function to export a unsigned 8bit integer array setting */
typedef enum eDBError (*t_db_interp_uint8_array_export_fn)(const char *db_name, const char *name, void *interpreter, const uint8_t *data, size_t num_elems);


/** Function to export an unsigned 32bit integer array setting */
typedef enum eDBError (*t_db_interp_uint32_array_export_fn)(const char *db_name, const char *name, void *interpreter, const uint32_t *data, size_t num_elems);


/** Function to export a signed 32bit integer array setting */
typedef enum eDBError (*t_db_interp_int32_array_export_fn)(const char *db_name, const char *name, void *interpreter, const int32_t *data, size_t num_elems);


/** Function to export a string array setting */
typedef enum eDBError (*t_db_interp_string_array_export_fn)(const char *db_name, const char *name, void *interpreter, const char *const *data, size_t num_elems);


/*
 * Import function typedefs
 */


/** Function to import a unsigned 8bit integer setting */
typedef enum eDBError (*t_db_interp_uint8_import_fn)(const char *db_name, const char *name, void *interpreter, uint8_t *data, uint8_t lowlim, uint8_t hilim, uint8_t default_value);


/** Function to import an unsigned 32bit integer setting */
typedef enum eDBError (*t_db_interp_uint32_import_fn)(const char *db_name, const char *name, void *interpreter, uint32_t *data, uint32_t lowlim, uint32_t hilim, uint32_t default_value);


/** Function to import a signed 32bit integer setting */
typedef enum eDBError (*t_db_interp_int32_import_fn)(const char *db_name, const char *name, void *interpreter, int32_t *data, int32_t lowlim, int32_t hilim, int32_t default_value);


/** Function to import a string (null terminated) setting */
typedef enum eDBError (*t_db_interp_str_import_fn)(const char *db_name, const char *name, void *interpreter, char *data, size_t maxlen, const char *default_value);


/** Function to import a buffer setting */
typedef enum eDBError (*t_db_interp_buf_import_fn)(const char *db_name, const char *name, void *interpreter, uint8_t *data, size_t maxlen, const uint8_t *default_value);


/** Function to import a unsigned 8bit integer array setting */
typedef enum eDBError (*t_db_interp_uint8_array_import_fn)(const char *db_name, const char *name, void *interpreter, uint8_t *data, size_t num_elems, const uint8_t *lowlims, const uint8_t *hilims, const uint8_t *default_values, uint8_t single_lowlim, uint8_t single_hilim, uint8_t single_default);


/** Function to import an unsigned 32bit integer array setting */
typedef enum eDBError (*t_db_interp_uint32_array_import_fn)(const char *db_name, const char *name, void *interpreter, uint32_t *data, size_t num_elems, const uint32_t *lowlims, const uint32_t *hilims, const uint32_t *default_values, uint32_t single_lowlim, uint32_t single_hilim, uint32_t single_default);


/** Function to import a signed 32bit integer array setting */
typedef enum eDBError (*t_db_interp_int32_array_import_fn)(const char *db_name, const char *name, void *interpreter, int32_t *data, size_t num_elems, const int32_t *lowlims, const int32_t *hilims, const int32_t *default_values, int32_t single_lowlim, int32_t single_hilim, int32_t single_default);


/** Function to import a string array setting */
typedef enum eDBError (*t_db_interp_string_array_import_fn)(const char *db_name, const char *name, void *interpreter, char *const *data, size_t num_elems, const size_t *max_lengths, const char *const *default_values, size_t single_max_length, const char *single_default);


/** Structure to describe a database interpreter */
struct s_db_interp
{
  void *interpreter;

  /* Export functions */
  t_db_interp_uint8_export_fn uint8_export;
  t_db_interp_uint32_export_fn uint32_export;
  t_db_interp_int32_export_fn int32_export;
  t_db_interp_str_export_fn str_export;
  t_db_interp_buf_export_fn buf_export;
  t_db_interp_uint8_array_export_fn uint8_array_export;
  t_db_interp_uint32_array_export_fn uint32_array_export;
  t_db_interp_int32_array_export_fn int32_array_export;
  t_db_interp_string_array_export_fn string_array_export;

  /* Import functions */
  t_db_interp_uint8_import_fn uint8_import;
  t_db_interp_uint32_import_fn uint32_import;
  t_db_interp_int32_import_fn int32_import;
  t_db_interp_str_import_fn str_import;
  t_db_interp_buf_import_fn buf_import;
  t_db_interp_uint8_array_import_fn uint8_array_import;
  t_db_interp_uint32_array_import_fn uint32_array_import;
  t_db_interp_int32_array_import_fn int32_array_import;
  t_db_interp_string_array_import_fn string_array_import;
};


/*
 * Export functions
 */


enum eDBError db_interp_uint8_export(const char *db_name, struct s_db_interp *db_interp, const char *name, const struct s_db_element_uint8 *elem);


enum eDBError db_interp_uint32_export(const char *db_name, struct s_db_interp *db_interp, const char *name, const struct s_db_element_uint32 *elem);


enum eDBError db_interp_int32_export(const char *db_name, struct s_db_interp *db_interp, const char *name, const struct s_db_element_int32 *elem);


enum eDBError db_interp_str_export(const char *db_name, struct s_db_interp *db_interp, const char *name, const struct s_db_element_string *elem);


enum eDBError db_interp_buf_export(const char *db_name, struct s_db_interp *db_interp, const char *name, const struct s_db_element_buffer *elem);


enum eDBError db_interp_uint8_array_export(const char *db_name, struct s_db_interp *db_interp, const char *name, const struct s_db_element_uint8_array *elem);


enum eDBError db_interp_uint32_array_export(const char *db_name, struct s_db_interp *db_interp, const char *name, const struct s_db_element_uint32_array *elem);


enum eDBError db_interp_int32_array_export(const char *db_name, struct s_db_interp *db_interp, const char *name, const struct s_db_element_int32_array *elem);


enum eDBError db_interp_string_array_export(const char *db_name, struct s_db_interp *db_interp, const char *name, const struct s_db_element_string_array *elem);


/*
 * Import functions
 */


enum eDBError db_interp_uint8_import(const char *db_name, struct s_db_interp *db_interp, const char *name, struct s_db_element_uint8 *elem);


enum eDBError db_interp_uint32_import(const char *db_name, struct s_db_interp *db_interp, const char *name, struct s_db_element_uint32 *elem);


enum eDBError db_interp_int32_import(const char *db_name, struct s_db_interp *db_interp, const char *name, struct s_db_element_int32 *elem);


enum eDBError db_interp_str_import(const char *db_name, struct s_db_interp *db_interp, const char *name, struct s_db_element_string *elem);


enum eDBError db_interp_buf_import(const char *db_name, struct s_db_interp *db_interp, const char *name, struct s_db_element_buffer *elem);


enum eDBError db_interp_uint8_array_import(const char *db_name, struct s_db_interp *db_interp, const char *name, struct s_db_element_uint8_array *elem);


enum eDBError db_interp_uint32_array_import(const char *db_name, struct s_db_interp *db_interp, const char *name, struct s_db_element_uint32_array *elem);


enum eDBError db_interp_int32_array_import(const char *db_name, struct s_db_interp *db_interp, const char *name, struct s_db_element_int32_array *elem);


enum eDBError db_interp_string_array_import(const char *db_name, struct s_db_interp *db_interp, const char *name, struct s_db_element_string_array *elem);


#endif /* DB_INTERP_H_ */
