/*
 * db_strescape.h
 *
 *  Created on: Jun 11, 2012
 *      Author: ian
 */

#ifndef DB_STRESCAPE_H_
#define DB_STRESCAPE_H_

#include <stdlib.h>
#include <rindb/db_error.h>


/**
 * Escape non-printable characters in a string in C string style. \ is used to escape characters
 * so \\ is a single \, and \xnn is a hex representation of a character. \0 is a special
 * case used to terminate the string
 *
 * @param src is the source string to be escaped. Not NULL
 * @param dst is the destination string that the escaped string will be written to. Not NULL. Must be 4 times longer than src + 1 (null terminator) for worst case.
 * @param dstlen is the maximum length of the destination string. See dst for info on recommended length
 */
enum eDBError db_string_escape_encode(const char *src, char *dst, size_t dstlen);


/**
 * Decode an C style escaped string. \ is used to escape characters
 * so \\ is a single \, and \xnn is a hex representation of a character. \0 is a special
 * case used to terminate the string
 *
 * @param src is the source string to be un-escaped. Not NULL
 * @param dst is the destination string that the un-escaped string will be written to. Not NULL. Must be the same length as src + 1 (null terminator) for worst case.
 * @param dstlen is the maximum length of the destination string. See dst for info on recommended length
 */
enum eDBError db_string_escape_decode(const char *src, char *dst, size_t dstlen);


#endif /* DB_STRESCAPE_H_ */
