/*
 * db_we2111.h
 *
 *  Created on: 26/08/2011
 *      Author: ian
 */

#ifndef DB_WE2111_H_
#define DB_WE2111_H_


#include <rindb/db_database.h>


/* Changelog masks */
#define WE21_DB_CLMASK_FACTORY 0x00000001
#define WE21_DB_CLMASK_TRADE   0x00000002
#define WE21_DB_CLMASK_USER    0x00000004

#define WE21_DB_CL_MASK_DESC_STRS { "Factory", "Trade", "User" }


/* Database versions */
#define WE_DB_U_VERSION1 1
#define WE_DB_U_VERSION2 2

#define WE_DB_F_VERSION1 1


#define APP_DB_MODEL_STR_LEN 7
#define SIX_DIGITS_MAX 999999
#define NUM_SETPOINTS 8
#define NUM_ADC_SYNC_FREQ 9


extern const int32_t db_we21_zoffs_default[NUM_ADC_SYNC_FREQ];
extern const int32_t db_we21_zoffs_minimum[NUM_ADC_SYNC_FREQ];
extern const int32_t db_we21_zoffs_maximum[NUM_ADC_SYNC_FREQ];


#if 0
extern const int32_t db_we21_spancal_default[NUM_ADC_SYNC_FREQ];
extern const int32_t db_we21_spancal_minimum[NUM_ADC_SYNC_FREQ];
extern const int32_t db_we21_spancal_maximum[NUM_ADC_SYNC_FREQ];
#endif


/** The ADC factory database. This is filled in by the Rinstrum oven factory calibration process, and stored in flash. */
typedef struct {
  int32_t zeroOffset[NUM_ADC_SYNC_FREQ];       /*!< Fixed zero offset */
  int32_t zeroDrift;                           /*!< Zero drift */
  int32_t spanCal[NUM_ADC_SYNC_FREQ];          /*!< Span calibration */
  int32_t spanDrift;                           /*!< span_cal drift per temperature count */
  uint32_t calTemp;                            /*!< Raw temperature at calibration */
  uint8_t calFlag;                             /*!< Set to 0xA5 if factory calibration valid */
  uint8_t gainFlag;                            /*!< Set to 0xA5 if pga gain 64 instead of 128 */

  uint16_t chksum;                             /*!< Checksum will be checked before database use */
} AdcFactoryDatabase;


/* Factory database default values */
#define ADC_DB_FACT_ZEROOFFSET_DEFAULT (0)
#define ADC_DB_FACT_ZERODRIFT_DEFAULT (0)
#define ADC_DB_FACT_SPANCAL_DEFAULT (0x4FEC5CL)
#define ADC_DB_FACT_SPANDRIFT_DEFAULT (0)
#define ADC_DB_FACT_CALTEMP_DEFAULT (600)
#define ADC_DB_FACT_CALFLAG_DEFAULT (0)
#define ADC_DB_FACT_GAINFLAG_DEFAULT (0xA5)



/** Application factory database definition */
struct s_db_app_factory
{
  AdcFactoryDatabase adc_factory_database;
  uint32_t serial_no;
  uint8_t cal_flag;            /* set to 0xA5 if factory calibration valid */
  uint8_t economy;             /* bits set for economy units */
  uint32_t application_id;
  uint8_t trade_enable;
  char model_str[APP_DB_MODEL_STR_LEN];
};


/** Application factory database */
extern struct s_db_app_factory db_app_factory;


#define WE21_DB_F_NAME "factorydb"
#define WE21_DB_F_CHECKSUM_SEED 0x12345678

#define WE21_DB_F_SERIALNO_STR "serialno"
#define WE21_DB_F_SERIALNO_MIN 0
#define WE21_DB_F_SERIALNO_MAX UINT_MAX
#define WE21_DB_F_SERIALNO_DEFAULT 0

#define WE21_DB_F_CALFLAG_STR "calflag"
#define WE21_DB_F_CALFLAG_MIN 0
#define WE21_DB_F_CALFLAG_MAX UCHAR_MAX
#define WE21_DB_F_CALFLAG_DEFAULT 0

#define WE21_DB_F_ECONOMY_STR "economy"
#define WE21_DB_F_ECONOMY_MIN 0
#define WE21_DB_F_ECONOMY_MAX UCHAR_MAX
#define WE21_DB_F_ECONOMY_DEFAULT 0

#define WE21_DB_F_APPLICID_STR "applicid"
#define WE21_DB_F_APPLICID_MIN 0
#define WE21_DB_F_APPLICID_MAX INT_MAX
#define WE21_DB_F_APPLICID_DEFAULT 0

#define WE21_DB_F_TRADEENABLE_STR "tradeenable"
#define WE21_DB_F_TRADEENABLE_MIN 0
#define WE21_DB_F_TRADEENABLE_MAX 1
#define WE21_DB_F_TRADEENABLE_DEFAULT 0

#define WE21_DB_F_MODEL_STR "model"
#define WE21_DB_F_MODEL_MAXLEN APP_DB_MODEL_STR_LEN
#define WE21_DB_F_MODEL_DEFAULT "WE2111"

#define WE21_DB_F_ADCZOFFS_STR "zerooffset"
#define WE21_DB_F_ADCZOFFS_MIN (-(1 << 23))
#define WE21_DB_F_ADCZOFFS_MAX ((1 << 23) - 1)
#define WE21_DB_F_ADCZOFFS_DEFAULT ADC_DB_FACT_ZEROOFFSET_DEFAULT

#define WE21_DB_F_ADCZDRIFT_STR "zerodrift"
#define WE21_DB_F_ADCZDRIFT_MIN (-(1 << 23))
#define WE21_DB_F_ADCZDRIFT_MAX ((1 << 23) - 1)
#define WE21_DB_F_ADCZDRIFT_DEFAULT ADC_DB_FACT_ZERODRIFT_DEFAULT

#define WE21_DB_F_ADCSCAL_STR "spancal"
#define WE21_DB_F_ADCSCAL_MIN (-(1 << 23))
#define WE21_DB_F_ADCSCAL_MAX ((1 << 23) - 1)
#define WE21_DB_F_ADCSCAL_DEFAULT ADC_DB_FACT_SPANCAL_DEFAULT

#define WE21_DB_F_ADCSDRIFT_STR "spandrift"
#define WE21_DB_F_ADCSDRIFT_MIN (-(1 << 23))
#define WE21_DB_F_ADCSDRIFT_MAX ((1 << 23) - 1)
#define WE21_DB_F_ADCSDRIFT_DEFAULT ADC_DB_FACT_SPANDRIFT_DEFAULT

#define WE21_DB_F_ADCCALTEMP_STR "caltemp"
#define WE21_DB_F_ADCCALTEMP_MIN 0
#define WE21_DB_F_ADCCALTEMP_MAX UINT16_MAX
#define WE21_DB_F_ADCCALTEMP_DEFAULT ADC_DB_FACT_CALTEMP_DEFAULT

#define WE21_DB_F_ADCCALFLAG_STR "calflag"
#define WE21_DB_F_ADCCALFLAG_MIN 0
#define WE21_DB_F_ADCCALFLAG_MAX UINT8_MAX
#define WE21_DB_F_ADCCALFLAG_DEFAULT ADC_DB_FACT_CALFLAG_DEFAULT

#define WE21_DB_F_ADCGAINFLAG_STR "gainflag"
#define WE21_DB_F_ADCGAINFLAG_MIN 0
#define WE21_DB_F_ADCGAINFLAG_MAX UINT8_MAX
#define WE21_DB_F_ADCGAINFLAG_DEFAULT ADC_DB_FACT_GAINFLAG_DEFAULT

/*
 * Section D1
 */


/** Defaults for setpoint types */
extern const uint8_t setpointtype_defaults[NUM_SETPOINTS];


#define WE21_DB_D1_NAME "userdb"
#define WE21_DB_D1_CHECKSUM_SEED 0x12345678

#define WE21_DB_D1_SAFEPASSCODE_STR "safepasscode"
#define WE21_DB_D1_SAFEPASSCODE_MIN 0
#define WE21_DB_D1_SAFEPASSCODE_MAX SIX_DIGITS_MAX
#define WE21_DB_D1_SAFEPASSCODE_DEFAULT 5678

#define WE21_DB_D1_FULLPASSCODE_STR "fullpasscode"
#define WE21_DB_D1_FULLPASSCODE_MIN 0
#define WE21_DB_D1_FULLPASSCODE_MAX SIX_DIGITS_MAX
#define WE21_DB_D1_FULLPASSCODE_DEFAULT 8765

#define WE21_DB_D1_BUTTONPERMS_STR "buttonperms"
#define WE21_DB_D1_BUTTONPERMS_MIN 0
#define WE21_DB_D1_BUTTONPERMS_MAX UCHAR_MAX
#define WE21_DB_D1_BUTTONPERMS_DEFAULT 0x0F

#define WE21_DB_D1_ZEROBAND_STR "zeroband"
#define WE21_DB_D1_ZEROBAND_MIN 0
#define WE21_DB_D1_ZEROBAND_MAX INT_MAX
#define WE21_DB_D1_ZEROBAND_DEFAULT 100

#define WE21_DB_D1_SETPTYPES_STR "setpointtypes"
#define WE21_DB_D1_SETPTYPES_LENGTH NUM_SETPOINTS
#define WE21_DB_D1_SETPTYPES_DEFAULT { 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08 }
#define WE21_DB_D1_SETPTYPES_MINIMUM { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 }
#define WE21_DB_D1_SETPTYPES_MAXIMUM { 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08 }

#define WE21_DB_D1_INFLIGHTADJ_STR "inflightadjustment"
#define WE21_DB_D1_INFLIGHTADJ_MIN 0
#define WE21_DB_D1_INFLIGHTADJ_MAX UCHAR_MAX
#define WE21_DB_D1_INFLIGHTADJ_DEFAULT 50

#define WE21_DB_D1_PULSETIME_STR "pulsetime"
#define WE21_DB_D1_PULSETIME_MIN 0
#define WE21_DB_D1_PULSETIME_MAX 200
#define WE21_DB_D1_PULSETIME_DEFAULT 20

#define WE21_DB_D1_QAOPTION_STR "qaoption"
#define WE21_DB_D1_QAOPTION_MIN 0
#define WE21_DB_D1_QAOPTION_MAX 1
#define WE21_DB_D1_QAOPTION_DEFAULT 1

#define WE21_DB_D1_ANALOGOUTSPAN_STR "analogoutspan"
#define WE21_DB_D1_ANALOGOUTSPAN_MIN -32767
#define WE21_DB_D1_ANALOGOUTSPAN_MAX 32767
#define WE21_DB_D1_ANALOGOUTSPAN_DEFAULT 1000

#define WE21_DB_D1_ANALOGOUTZERO_STR "analogoutzero"
#define WE21_DB_D1_ANALOGOUTZERO_MIN -32767
#define WE21_DB_D1_ANALOGOUTZERO_MAX 32767
#define WE21_DB_D1_ANALOGOUTZERO_DEFAULT 5

#define WE21_DB_D1_TESTVAR_STR "testvar"
#define WE21_DB_D1_TESTVAR_MIN 0
#define WE21_DB_D1_TESTVAR_MAX 9999
#define WE21_DB_D1_TESTVAR_DEFAULT 2000

#define WE21_DB_D1_TESTSTRARRAY_STR "teststringarray"
#define WE21_DB_D1_TESTSTRARRAY_NUM 10
#define WE21_DB_D1_TESTSTRARRAY_MAXLEN 20
#define WE21_DB_D1_TESTSTRARRAY_DEFAULT "0123456789012345678"

#define WE21_DB_D1_TESTSTRING_STR "teststring"
#define WE21_DB_D1_TESTSTRING_MAXLEN 100
#define WE21_DB_D1_TESTSTRING_DEFAULT "hello there"


struct s_db_we2111_d1
{
  uint32_t safe_passcode;
  uint32_t full_passcode;
  uint8_t button_permissions;
  int32_t zero_band;           /* zero band in display units */
  uint8_t setpoint_types[NUM_SETPOINTS];       /* setpoint product */
  uint8_t flt_adj;             /* percentage of auto inflight adjustment */
  uint8_t pulse_time;          /* in 0.1 second increments up to 20 seconds */
  uint8_t qaopt;               /* TRUE if qa date used */
  int32_t analog_span_cal;    /* analogue card span correction */
  int32_t analog_zero_cal;    /* analogue card zero correction */
  uint32_t test_var;
  int32_t zeroOffset[NUM_ADC_SYNC_FREQ];       /*!< Fixed zero offset */
  int32_t spanCal[NUM_ADC_SYNC_FREQ];          /*!< Span calibration */
  char model_str[APP_DB_MODEL_STR_LEN];
  char teststrarray[WE21_DB_D1_TESTSTRARRAY_NUM][WE21_DB_D1_TESTSTRARRAY_MAXLEN];
  char teststring[WE21_DB_D1_TESTSTRING_MAXLEN];
};


extern struct s_db_we2111_d1 db_we2111_d1;


/*
 * Database definition
 */


extern struct s_db_database we2111_factory_database;


extern struct s_db_database we2111_user_database_v1;


extern struct s_db_database we2111_user_database_v2;


#endif /* DB_WE2111_H_ */
