/*
 * test_db_changelog.c
 *
 *  Created on: Apr 21, 2012
 *      Author: ian
 */
#include <string.h>
#include <unistd.h>
#include <cutest/CuTest.h>
#include <rindb/db_database.h>
#include <rindb/db_database_binfile.h>
#include <rindb/db_database_xml.h>
#include <rindb/db_changelog.h>
#include <rindb/db_changelog_reader.h>
#include "db_we2111.h"

#include "test_db_changelog.h"

/** Change log */
struct s_db_changelog *changelog = NULL;
static const char *cl_mask_desc_strs[] = WE21_DB_CL_MASK_DESC_STRS;
static const char *cl_header_rows[dbcleNum] =
{
  "CalCounter",
  "ConfigCounter",
  "Time",
  "DB Name",
  "DB Element Name",
  "DB Menu Name",
  "Index",
  "Old Value",
  "New Value",
  "Log Mask"
};

/** Change log reader */
struct s_db_changelog_reader *cl_reader = NULL;
struct s_db_changelog_entry *cl_entry = NULL;


#if 0
/** Dump the contents of the changelog to stdout */
static void test_dblib_changelog_dump(struct s_db_changelog *changelog)
{
  (void) db_changelog_slave_copy_file(changelog, stdout);
}
#endif


/** Delete any existing change logs */
static void test_dblib_changelog_cleanup(CuTest *tc)
{
  /* Delete existing change log */
  unlink(TEST_CHANGELOG_FILENAME);
  unlink(TEST_CHANGELOG_SLAVECOPY_FILENAME);
}


/** Test creating a changelog reader. Changelog file must exist before calling this */
static void test_dblib_changelogreader_init(CuTest *tc)
{
  enum eDBError res;

  /* Create changelog reader */
  res = db_changelog_reader_create_filename(&cl_reader, TEST_CHANGELOG_FILENAME, TEST_CHANGELOG_MAXLINESIZE);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Create changelog entry */
  res = db_changelog_reader_entry_create(&cl_entry);
  CuAssertIntEquals(tc, dbeSuccess, res);
}


/** Test creating a changelog */
static void test_dblib_changelog_init_new(CuTest *tc)
{
  enum eDBError res;
  time_t time;
  time_t readtime;

  time = 1334988883;

  /* Create a changelog, should have 1 entry */
  res = db_changelog_create(&changelog,
                            TEST_CHANGELOG_FILENAME,
                            TEST_CHANGELOG_MAXSIZE,
                            WE21_DB_CLMASK_TRADE,
                            cl_mask_desc_strs,
                            TEST_CHANGELOG_EVENT_DB_NAME,
                            TEST_CHANGELOG_EVENT_MENU_NAME,
                            TEST_CHANGELOG_EVENT_INITIAL,
                            TEST_CHANGELOG_EVENT_CLEARED,
                            cl_header_rows,
                            NULL,
                            NULL,
                            time,
                            WE21_DB_D1_NAME,
                            WE21_DB_CLMASK_TRADE);
  CuAssertIntEquals(tc, dbeSuccess, res);

  test_dblib_changelogreader_init(tc);

  /* Test first entry */
  res = db_changelog_reader_entry_get(cl_reader, dbclrlFirst, cl_entry);
  CuAssertIntEquals(tc, dbeSuccess, res);
  readtime = mktime(&cl_entry->timestamp);
  CuAssertIntEquals(tc, time, readtime);
  CuAssertStrEquals(tc, WE21_DB_D1_NAME, cl_entry->db_name);
  CuAssertStrEquals(tc, TEST_CHANGELOG_EVENT_DB_NAME, cl_entry->db_elem_name);
  CuAssertStrEquals(tc, TEST_CHANGELOG_EVENT_MENU_NAME, cl_entry->menu_elem_name);
  CuAssertTrue(tc, !cl_entry->has_index);
  CuAssertStrEquals(tc, "", cl_entry->old_value);
  CuAssertStrEquals(tc, TEST_CHANGELOG_EVENT_INITIAL, cl_entry->new_value);

  /* Next read should fail as there's nothing more in the changelog */
  res = db_changelog_reader_entry_get(cl_reader, dbclrlNext, cl_entry);
  CuAssertIntEquals(tc, dbeChangelogNoMoreRecords, res);
}


/** Test creating an existing changelog (relies on test order) */
static void test_dblib_changelog_init_existing(CuTest *tc)
{
  enum eDBError res;
  time_t time;
  time_t readtime;

  time = 1334988883;

  /* Create a changelog, using pre-existing file */
  res = db_changelog_create(&changelog,
                            TEST_CHANGELOG_FILENAME,
                            TEST_CHANGELOG_MAXSIZE,
                            WE21_DB_CLMASK_TRADE,
                            cl_mask_desc_strs,
                            TEST_CHANGELOG_EVENT_DB_NAME,
                            TEST_CHANGELOG_EVENT_MENU_NAME,
                            TEST_CHANGELOG_EVENT_INITIAL,
                            TEST_CHANGELOG_EVENT_CLEARED,
                            cl_header_rows,
                            NULL,
                            NULL,
                            time,
                            WE21_DB_D1_NAME,
                            WE21_DB_CLMASK_TRADE);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Test first entry still exists */
  res = db_changelog_reader_entry_get(cl_reader, dbclrlFirst, cl_entry);
  CuAssertIntEquals(tc, dbeSuccess, res);
  readtime = mktime(&cl_entry->timestamp);
  CuAssertIntEquals(tc, time, readtime);
  CuAssertStrEquals(tc, WE21_DB_D1_NAME, cl_entry->db_name);
  CuAssertStrEquals(tc, TEST_CHANGELOG_EVENT_DB_NAME, cl_entry->db_elem_name);
  CuAssertStrEquals(tc, TEST_CHANGELOG_EVENT_MENU_NAME, cl_entry->menu_elem_name);
  CuAssertTrue(tc, !cl_entry->has_index);
  CuAssertStrEquals(tc, "", cl_entry->old_value);
  CuAssertStrEquals(tc, TEST_CHANGELOG_EVENT_INITIAL, cl_entry->new_value);

  /* Next read should fail as there's nothing more in the changelog */
  res = db_changelog_reader_entry_get(cl_reader, dbclrlNext, cl_entry);
  CuAssertIntEquals(tc, dbeChangelogNoMoreRecords, res);
}


/** Test creating an existing change log (relies on test order) */
static void test_dblib_changelog_add_entries(CuTest *tc)
{
  enum eDBError res;
  time_t time;
  time_t readtime;

  time = 1334988883;

  /* Set database to defaults */
  res = db_database_default(&we2111_user_database_v1);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Update the previous copy */
  res = db_database_update_previous(&we2111_user_database_v1);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Change some things, these should show up in the changelog */
  db_we2111_d1.safe_passcode = WE21_DB_D1_SAFEPASSCODE_DEFAULT + 1;       /* This one should show up in the changelog because it's WE21_DB_CLMASK_TRADE */
  db_we2111_d1.analog_span_cal = WE21_DB_D1_ANALOGOUTSPAN_DEFAULT + 100;  /* This one shouldn't show up in the changelog because it's WE21_DBCL_MASK_USER */

  res = db_changelog_update_database(changelog, time, &we2111_user_database_v1, we2111_user_database_v1.db_database_version);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Check what's in the change log, skip first entry, read next */
  res = db_changelog_reader_entry_get(cl_reader, dbclrlFirst, cl_entry);
  CuAssertIntEquals(tc, dbeSuccess, res);

  res = db_changelog_reader_entry_get(cl_reader, dbclrlNext, cl_entry);
  CuAssertIntEquals(tc, dbeSuccess, res);
  readtime = mktime(&cl_entry->timestamp);
  CuAssertIntEquals(tc, time, readtime);
  CuAssertStrEquals(tc, WE21_DB_D1_NAME, cl_entry->db_name);
  CuAssertStrEquals(tc, WE21_DB_D1_SAFEPASSCODE_STR, cl_entry->db_elem_name);
  CuAssertTrue(tc, !cl_entry->has_index);
  CuAssertIntEquals(tc, WE21_DB_D1_SAFEPASSCODE_DEFAULT, atol(cl_entry->old_value));
  CuAssertIntEquals(tc, WE21_DB_D1_SAFEPASSCODE_DEFAULT + 1, atol(cl_entry->new_value));

  /* Next read should fail as there's nothing more in the changelog */
  res = db_changelog_reader_entry_get(cl_reader, dbclrlNext, cl_entry);
  CuAssertIntEquals(tc, dbeChangelogNoMoreRecords, res);
}


/** Test seeking around in the change log (relies on test order) */
static void test_dblib_changelog_seek(CuTest *tc)
{
  enum eDBError res;

  /* There should be 2 entries in the change log after test_dblib_changelog_add_entries() */
  /* test_dblib_changelog_dump(changelog); */

  /* Seek last record */
  res = db_changelog_reader_entry_get(cl_reader, dbclrlLast, cl_entry);
  CuAssertIntEquals(tc, dbeSuccess, res);
  CuAssertStrEquals(tc, WE21_DB_D1_SAFEPASSCODE_STR, cl_entry->db_elem_name);

  /* Seek next record should fail */
  res = db_changelog_reader_entry_get(cl_reader, dbclrlNext, cl_entry);
  CuAssertIntEquals(tc, dbeChangelogNoMoreRecords, res);

  /* Seek back one record, should be first record */
  res = db_changelog_reader_entry_get(cl_reader, dbclrlPrevious, cl_entry);
  CuAssertIntEquals(tc, dbeSuccess, res);
  CuAssertStrEquals(tc, TEST_CHANGELOG_EVENT_DB_NAME, cl_entry->db_elem_name);
  CuAssertStrEquals(tc, TEST_CHANGELOG_EVENT_MENU_NAME, cl_entry->menu_elem_name);

  /* Seek back another record, should fail */
  res = db_changelog_reader_entry_get(cl_reader, dbclrlPrevious, cl_entry);
  CuAssertIntEquals(tc, dbeChangelogNoMoreRecords, res);

  /* Seek forward one record, should be first record */
  res = db_changelog_reader_entry_get(cl_reader, dbclrlNext, cl_entry);
  CuAssertIntEquals(tc, dbeSuccess, res);
  CuAssertStrEquals(tc, TEST_CHANGELOG_EVENT_DB_NAME, cl_entry->db_elem_name);
  CuAssertStrEquals(tc, TEST_CHANGELOG_EVENT_MENU_NAME, cl_entry->menu_elem_name);
}


/** Test creating a slave copy of the changelog */
static void test_dblib_changelog_slave_copy(CuTest *tc)
{
  enum eDBError res;
  FILE *slavecopy = NULL;
  long savepos;
  uint8_t masterbuf;
  uint8_t slavebuf;

  /* Create slave copy of changelog */
  res = db_changelog_slave_copy(changelog, TEST_CHANGELOG_SLAVECOPY_FILENAME);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Open file and check against master copy */
  slavecopy = fopen(TEST_CHANGELOG_SLAVECOPY_FILENAME, "r");
  CuAssertPtrNotNull(tc, slavecopy);

  /* Seek start of master changelog */
  savepos = ftell(changelog->file);
  CuAssertIntEquals(tc, 0, fseek(changelog->file, 0, SEEK_SET));

  /* Compare */
  while (1)
  {
    /* Read master */
    fread(&masterbuf, sizeof(uint8_t), 1, changelog->file);
    CuAssertIntEquals(tc, 0, ferror(changelog->file));

    /* Read slave */
    fread(&slavebuf, sizeof(uint8_t), 1, slavecopy);
    CuAssertIntEquals(tc, 0, ferror(slavecopy));

    /* Compare */
    CuAssertIntEquals(tc, masterbuf, slavebuf);

    /* Neither EOF, or both EOF, otherwise file sizes don't match */
    CuAssertTrue(tc, feof(changelog->file) == feof(slavecopy));

    /* Done? */
    if (feof(changelog->file) && feof(slavecopy))
      break;
  }

  /* Set back to old pos and close slave copy */
  CuAssertIntEquals(tc, 0, fseek(changelog->file, savepos, SEEK_SET));
  CuAssertIntEquals(tc, 0, fclose(slavecopy));
  slavecopy = NULL;
}


/** Test clearing the change log */
static void test_dblib_changelog_clear(CuTest *tc)
{
  enum eDBError res;
  time_t time;
  time_t readtime;

  time = 1334988883;

  /* Clear the change log */
  res = db_changelog_clear(changelog, time, WE21_DB_D1_NAME, WE21_DB_CLMASK_TRADE);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Test change log cleared entry exists */
  res = db_changelog_reader_entry_get(cl_reader, dbclrlFirst, cl_entry);
  CuAssertIntEquals(tc, dbeSuccess, res);
  readtime = mktime(&cl_entry->timestamp);
  CuAssertIntEquals(tc, time, readtime);
  CuAssertStrEquals(tc, WE21_DB_D1_NAME, cl_entry->db_name);
  CuAssertStrEquals(tc, TEST_CHANGELOG_EVENT_DB_NAME, cl_entry->db_elem_name);
  CuAssertStrEquals(tc, TEST_CHANGELOG_EVENT_MENU_NAME, cl_entry->menu_elem_name);
  CuAssertTrue(tc, !cl_entry->has_index);
  CuAssertStrEquals(tc, "", cl_entry->old_value);
  CuAssertStrEquals(tc, TEST_CHANGELOG_EVENT_CLEARED, cl_entry->new_value);

  /* Next read should fail as there's nothing more in the change log */
  res = db_changelog_reader_entry_get(cl_reader, dbclrlNext, cl_entry);
  CuAssertIntEquals(tc, dbeChangelogNoMoreRecords, res);
}


/** Test filling change log */
static void test_dblib_changelog_fill(CuTest *tc)
{
  enum eDBError res;
  time_t time;
  int count;

  time = 1334988883;

  /* Clear the change log */
  res = db_changelog_clear(changelog, time, WE21_DB_D1_NAME, WE21_DB_CLMASK_TRADE);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Does change log report not full */
  res = db_changelog_is_full(changelog);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Now fill it, count starts at 1 because the clear adds 1 record */
  for (count = 1, res = dbeSuccess; dbeSuccess == res ; count++)
  {
    res = db_changelog_add_misc_entry(changelog, time, WE21_DB_D1_NAME, "miscentry db", "miscentry menu", "oldvalue", "newvalue", WE21_DB_CLMASK_TRADE);
  }

  /* Should tell us that the change log is full */
  CuAssertIntEquals(tc, dbeChangelogFull, res);

  /* Does change log report full */
  res = db_changelog_is_full(changelog);
  CuAssertIntEquals(tc, dbeChangelogFull, res);
}


/** Test destroying a changelog */
static void test_dblib_changelog_destroy(CuTest *tc)
{
  enum eDBError res;

  /* Destroy the changelog */
  res = db_changelog_destroy(changelog);
  CuAssertIntEquals(tc, dbeSuccess, res);
}


/** Test destroying a changelog reader */
static void test_dblib_changelogreader_destroy(CuTest *tc)
{
  enum eDBError res;

  /* Destroy changelog entry */
  res = db_changelog_reader_entry_destroy(cl_entry);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Destroy changelog reader */
  res = db_changelog_reader_destroy(cl_reader);
  CuAssertIntEquals(tc, dbeSuccess, res);
}


CuSuite *CuGetSuite_DB_Changelog(void)
{
  CuSuite *suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_dblib_changelog_cleanup);
  SUITE_ADD_TEST(suite, test_dblib_changelog_init_new);
  SUITE_ADD_TEST(suite, test_dblib_changelog_destroy);
  SUITE_ADD_TEST(suite, test_dblib_changelog_init_existing);
  SUITE_ADD_TEST(suite, test_dblib_changelog_add_entries);
  SUITE_ADD_TEST(suite, test_dblib_changelog_seek);
  SUITE_ADD_TEST(suite, test_dblib_changelog_slave_copy);
  SUITE_ADD_TEST(suite, test_dblib_changelog_clear);
  SUITE_ADD_TEST(suite, test_dblib_changelog_fill);

  /* Final clean up */
  SUITE_ADD_TEST(suite, test_dblib_changelog_destroy);
  SUITE_ADD_TEST(suite, test_dblib_changelogreader_destroy);
/*  SUITE_ADD_TEST(suite, test_dblib_changelog_cleanup); Leave the change log file for debugging */

  return suite;
}
