/*
 * test_db_create.c
 *
 *  Created on: 22/07/2011
 *      Author: ian
 */
#include <string.h>
#include <stdbool.h>
#include <cutest/CuTest.h>
#include <rindb/db_database.h>
#include <rindb/db_database_binfile.h>
#include <rindb/db_database_xml.h>
#include "db_we2111.h"

#include "test_db_create.h"


struct s_db_database_interp ddi_bin;
struct s_db_database_interp ddi_xml;


/** Zero all elements in the database */
void db_zero(void)
{
  memset(&db_we2111_d1, 0, sizeof(db_we2111_d1));
}


/** Initialise the database */
static void test_dblib_init(CuTest* tc)
{
  enum eDBError res;

  /* Initialise the database (creates previous data for some elements) */
  res = db_database_init(&we2111_user_database_v1);
  CuAssertIntEquals(tc, dbeSuccess, res);
  res = db_database_init(&we2111_user_database_v2);
  CuAssertIntEquals(tc, dbeSuccess, res);
}


/** Destroy the database */
static void test_dblib_destroy(CuTest* tc)
{
  enum eDBError res;

  /* Destroy all elements in the database */
  res = db_database_destroy(&we2111_user_database_v1);
  CuAssertIntEquals(tc, dbeSuccess, res);
  res = db_database_destroy(&we2111_user_database_v2);
  CuAssertIntEquals(tc, dbeSuccess, res);
}


static void test_dblib_bin_create(CuTest* tc)
{
  enum eDBError res;

  /* Create and destroy one, to check there are no problems */
  res = db_create_database_interp_bin(&ddi_bin, TEST_BIN_FILENAME);
  CuAssertIntEquals(tc, dbeSuccess, res);
  res = db_destroy_database_interp_bin(&ddi_bin);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Create one for later tests to use */
  res = db_create_database_interp_bin(&ddi_bin, TEST_BIN_FILENAME);
  CuAssertIntEquals(tc, dbeSuccess, res);
}


static void test_dblib_xml_create(CuTest* tc)
{
  enum eDBError res;

  /* Create and destroy one, to check there are no problems */
  res = db_create_database_interp_xml(&ddi_xml, TEST_XML_FILENAME);
  CuAssertIntEquals(tc, dbeSuccess, res);
  res = db_destroy_database_interp_xml(&ddi_xml);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Create one for later tests to use */
  res = db_create_database_interp_xml(&ddi_xml, TEST_XML_FILENAME);
  CuAssertIntEquals(tc, dbeSuccess, res);
}


/** Check values set to defaults */
void test_dblib_default_check(CuTest* tc, uint32_t dbversion)
{
  size_t i;

  /* Cal section */
#if 0
  CuAssertIntEquals(tc, WE21_DB_CAL_SERIALNO_DEFAULT, we2111_cal.serial_no);
  CuAssertIntEquals(tc, WE21_DB_CAL_CALFLAG_DEFAULT, we2111_cal.cal_flag);
  CuAssertIntEquals(tc, WE21_DB_CAL_ECONOMY_DEFAULT, we2111_cal.economy);
  CuAssertIntEquals(tc, WE21_DB_CAL_APPLICID_DEFAULT, we2111_cal.application_id);
  CuAssertIntEquals(tc, WE21_DB_CAL_TRADEENABLE_DEFAULT, we2111_cal.trade_enable);
  CuAssertStrEquals(tc, WE21_DB_CAL_MODEL_DEFAULT, we2111_cal.model_str);
#endif

  /* D1 section */
  CuAssertIntEquals(tc, WE21_DB_D1_SAFEPASSCODE_DEFAULT, db_we2111_d1.safe_passcode);
  CuAssertIntEquals(tc, WE21_DB_D1_FULLPASSCODE_DEFAULT, db_we2111_d1.full_passcode);
  CuAssertIntEquals(tc, WE21_DB_D1_BUTTONPERMS_DEFAULT, db_we2111_d1.button_permissions);
  CuAssertIntEquals(tc, WE21_DB_D1_ZEROBAND_DEFAULT, db_we2111_d1.zero_band);
  CuAssertTrue(tc, (0 == memcmp(setpointtype_defaults, db_we2111_d1.setpoint_types, WE21_DB_D1_SETPTYPES_LENGTH)));
  CuAssertIntEquals(tc, WE21_DB_D1_INFLIGHTADJ_DEFAULT, db_we2111_d1.flt_adj);
  CuAssertIntEquals(tc, WE21_DB_D1_PULSETIME_DEFAULT, db_we2111_d1.pulse_time);
  if (dbversion <= WE_DB_U_VERSION1)
    CuAssertIntEquals(tc, WE21_DB_D1_QAOPTION_DEFAULT, db_we2111_d1.qaopt);
  CuAssertIntEquals(tc, WE21_DB_D1_ANALOGOUTSPAN_DEFAULT, db_we2111_d1.analog_span_cal);
  CuAssertIntEquals(tc, WE21_DB_D1_ANALOGOUTZERO_DEFAULT, db_we2111_d1.analog_zero_cal);

  for (i = 0; i < WE21_DB_D1_TESTSTRARRAY_NUM; i++)
  {
    CuAssertTrue(tc, (0 == strcmp(WE21_DB_D1_TESTSTRARRAY_DEFAULT, db_we2111_d1.teststrarray[i])));
  }
}


static void test_dblib_default(CuTest* tc)
{
  enum eDBError res;

  /* Default database */
  res = db_database_default(&we2111_user_database_v1);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Check values set to defaults */
  test_dblib_default_check(tc, we2111_user_database_v1.db_database_version);
}


/** Write the database to a binary file */
static void test_dblib_bin_write(CuTest* tc)
{
  enum eDBError res;

  res = db_database_write(&we2111_user_database_v1, &ddi_bin, we2111_user_database_v1.db_database_version);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Check file exists */
}


/** Read the database from a binary file */
static void test_dblib_bin_read(CuTest* tc)
{
  enum eDBError res;
  uint32_t version;

  /* Zero all elements in the database */
  db_zero();

  /* Read database */
  res = db_database_read(&we2111_user_database_v1, &ddi_bin, &version, false);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Check version */
  CuAssertIntEquals(tc, we2111_user_database_v1.db_database_version, version);

  /* Check each database element is at defaults (cause that's what we wrote) */
  test_dblib_default_check(tc, we2111_user_database_v1.db_database_version);
}


/** Write the database to an XML file */
static void test_dblib_xml_write(CuTest* tc)
{
  enum eDBError res;

  res = db_database_write(&we2111_user_database_v1, &ddi_xml, we2111_user_database_v1.db_database_version);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Check file exists */
}


/** Read the database from an XML file */
static void test_dblib_xml_read(CuTest* tc)
{
  enum eDBError res;
  uint32_t version;

  /* Zero all elements in the database */
  db_zero();

  /* Read database */
  res = db_database_read(&we2111_user_database_v1, &ddi_xml, &version, false);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Check version */
  CuAssertIntEquals(tc, we2111_user_database_v1.db_database_version, version);

  /* Check each database element is at defaults (cause that's what we wrote) */
  test_dblib_default_check(tc, we2111_user_database_v1.db_database_version);
}


static void test_dblib_bin_destroy(CuTest* tc)
{
  enum eDBError res;

  res = db_destroy_database_interp_bin(&ddi_bin);
  CuAssertIntEquals(tc, dbeSuccess, res);
}


static void test_dblib_xml_destroy(CuTest* tc)
{
  enum eDBError res;

  res = db_destroy_database_interp_xml(&ddi_xml);
  CuAssertIntEquals(tc, dbeSuccess, res);
}


/** Test reading a database using a freshly initialised interpreter */
static void test_dblib_fresh_read(CuTest* tc)
{
  enum eDBError res;
  struct s_db_database_interp ddi_xml_temp;
  uint32_t version;

  /* Write the database using one interpreter */
  res = db_database_write(&we2111_user_database_v1, &ddi_bin, we2111_user_database_v1.db_database_version);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Create a temporary interpreter */
  res = db_create_database_interp_xml(&ddi_xml_temp, TEST_XML_FILENAME);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Read database using fresh temporary interpreter */
  res = db_database_read(&we2111_user_database_v1, &ddi_xml_temp, &version, false);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Clean up temporary interpreter */
  res = db_destroy_database_interp_xml(&ddi_xml_temp);
  CuAssertIntEquals(tc, dbeSuccess, res);
}


CuSuite *CuGetSuite_DB_Create(void)
{
  CuSuite *suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_dblib_init);

  SUITE_ADD_TEST(suite, test_dblib_bin_create);
  SUITE_ADD_TEST(suite, test_dblib_xml_create);

  SUITE_ADD_TEST(suite, test_dblib_default);

  SUITE_ADD_TEST(suite, test_dblib_bin_write);
  SUITE_ADD_TEST(suite, test_dblib_xml_write);

  SUITE_ADD_TEST(suite, test_dblib_bin_read);
  SUITE_ADD_TEST(suite, test_dblib_xml_read);

  SUITE_ADD_TEST(suite, test_dblib_fresh_read);

  return suite;
}


CuSuite *CuGetSuite_DB_Destroy(void)
{
  CuSuite *suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_dblib_bin_destroy);
  SUITE_ADD_TEST(suite, test_dblib_xml_destroy);

  SUITE_ADD_TEST(suite, test_dblib_destroy);

  return suite;
}

