/*
 * test_db_strescape.c
 *
 *  Created on: Jun 11, 2012
 *      Author: ian
 */

#include <limits.h>
#include <string.h>
#include <cutest/CuTest.h>
#include <rindb/db_strescape.h>

#include "test_db_strescape.h"


/** Test escaping strings */
static void test_dblib_string_escape(CuTest *tc)
{
  enum eDBError res;
  char unencoded_str[1024];
  char encoded_str[1024];

  /* Empty string */
  #define EMPTY_STR ""
  res = db_string_escape_encode(EMPTY_STR, encoded_str, 1024);
  CuAssertIntEquals(tc, dbeSuccess, res);
  CuAssertStrEquals(tc, EMPTY_STR, encoded_str);

  res = db_string_escape_decode(encoded_str, unencoded_str, 1024);
  CuAssertIntEquals(tc, dbeSuccess, res);
  CuAssertStrEquals(tc, EMPTY_STR, unencoded_str);

  /* Simple string */
  #define SIMPLE_STR "hello there"
  res = db_string_escape_encode(SIMPLE_STR, encoded_str, 1024);
  CuAssertIntEquals(tc, dbeSuccess, res);
  CuAssertStrEquals(tc, SIMPLE_STR, encoded_str);

  res = db_string_escape_decode(encoded_str, unencoded_str, 1024);
  CuAssertIntEquals(tc, dbeSuccess, res);
  CuAssertStrEquals(tc, SIMPLE_STR, unencoded_str);

  /* More complicated string with escaped characters */
  #define UNESCAPED_STR "\\hello\x01\x02\x0a\xbc\xAF there"
  #define ESCAPED_STR "\\\\hello\\x01\\x02\\x0a\\xbc\\xaf there"
  res = db_string_escape_encode(UNESCAPED_STR, encoded_str, 1024);
  CuAssertIntEquals(tc, dbeSuccess, res);
  CuAssertStrEquals(tc, ESCAPED_STR, encoded_str);

  res = db_string_escape_decode(encoded_str, unencoded_str, 1024);
  CuAssertIntEquals(tc, dbeSuccess, res);
  CuAssertStrEquals(tc, UNESCAPED_STR, unencoded_str);
}


/** Test bad escaped strings */
static void test_dblib_bad_string_escape(CuTest *tc)
{
  enum eDBError res;
  char unencoded_str[1024];

  /* Bad string - single escape character */
  res = db_string_escape_decode("\\", unencoded_str, 1024);
  CuAssertIntEquals(tc, dbeParseFail, res);

  /* Bad string - not \x */
  res = db_string_escape_decode("\\a", unencoded_str, 1024);
  CuAssertIntEquals(tc, dbeParseFail, res);

  /* Bad string - not \xNN */
  res = db_string_escape_decode("\\x", unencoded_str, 1024);
  CuAssertIntEquals(tc, dbeParseFail, res);
  res = db_string_escape_decode("\\xQ", unencoded_str, 1024);
  CuAssertIntEquals(tc, dbeParseFail, res);
  res = db_string_escape_decode("\\xQW", unencoded_str, 1024);
  CuAssertIntEquals(tc, dbeParseFail, res);
  res = db_string_escape_decode("\\xaQ", unencoded_str, 1024);
  CuAssertIntEquals(tc, dbeParseFail, res);
}


CuSuite *CuGetSuite_DB_StringEscape(void)
{
  CuSuite *suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_dblib_string_escape);
  SUITE_ADD_TEST(suite, test_dblib_bad_string_escape);

  return suite;
}
