/*
 * test_db_xmlstr.c
 *
 *  Created on: Jun 8, 2012
 *      Author: ian
 */
#include <limits.h>
#include <string.h>
#include <cutest/CuTest.h>
#include <rindb/db_database.h>
#include <rindb/db_strncpy.h>
#include "db_we2111.h"
#include "test_db_create.h"

#include "test_db_strings.h"


/** Test reading and writing strange characters in strings */
static void test_dblib_string_strange_characters(CuTest *tc, struct s_db_database_interp *ddi)
{
  enum eDBError res;
  uint32_t version;
  uint8_t i;

  /* Try writing different characters into a string */
  for (i = 1; i < UINT8_MAX; i++)
  {
    /* Make a string with a (possibly) weird character */
    db_we2111_d1.teststring[0] = (char) i;
    db_we2111_d1.teststring[1] = '\0';

    /* Write the database to XML */
    res = db_database_write(&we2111_user_database_v1, ddi, we2111_user_database_v1.db_database_version);
    CuAssertIntEquals(tc, dbeSuccess, res);

    /* Clear the database */
    memset(&db_we2111_d1, 0, sizeof(db_we2111_d1));

    /* Read database back */
    res = db_database_read(&we2111_user_database_v1, ddi, &version, false);
    CuAssertIntEquals(tc, dbeSuccess, res);

    /* Check the string */
    CuAssertIntEquals(tc, i, (uint8_t) db_we2111_d1.teststring[0]);
    CuAssertIntEquals(tc, '\0', (uint8_t) db_we2111_d1.teststring[1]);
  }
}


/** Test reading and writing strange characters in strings to XML database */
static void test_dblib_xml_string_strange_characters(CuTest *tc)
{
  test_dblib_string_strange_characters(tc, &ddi_xml);
}


/** Test reading and writing strange characters in strings to binary database */
static void test_dblib_bin_string_strange_characters(CuTest *tc)
{
  test_dblib_string_strange_characters(tc, &ddi_bin);
}


/** Test reading and writing XML escaped characters to strings */
static void test_dblib_string_xml_escape_characters(CuTest *tc)
{
  enum eDBError res;
  uint32_t version;

  /*
   * These are the XML escape chars. Ezxml takes care of this for us
   *
   *  "   &quot;
   *  '   &apos;
   *  <   &lt;
   *  >   &gt;
   *  &   &amp;
   */
  #define XML_ESCAPE_STRING "\"'<>&"

  /* Make a string with a (possibly) weird character */
  db_safe_strncpy(db_we2111_d1.teststring, XML_ESCAPE_STRING, WE21_DB_D1_TESTSTRING_MAXLEN);

  /* Write the database to XML */
  res = db_database_write(&we2111_user_database_v1, &ddi_xml, we2111_user_database_v1.db_database_version);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Read database back */
  res = db_database_read(&we2111_user_database_v1, &ddi_xml, &version, false);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Check the string */
  CuAssertStrEquals(tc, XML_ESCAPE_STRING, db_we2111_d1.teststring);
}


CuSuite *CuGetSuite_DB_Strings(void)
{
  CuSuite *suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_dblib_xml_string_strange_characters);
  SUITE_ADD_TEST(suite, test_dblib_bin_string_strange_characters);
  SUITE_ADD_TEST(suite, test_dblib_string_xml_escape_characters);

  return suite;
}
