/*
 * test_db_uboot.c
 *
 *  Created on: Sep 24, 2012
 *      Author: ian
 */
#include <cutest/CuTest.h>
#include <rindb/db_database_uboot.h>
#include "db_we2111.h"
#include "test_db_create.h"

#include "test_db_uboot.h"


static struct s_db_buffer db_buffer_uboot;
static struct s_db_database_interp ddi_uboot;


static void test_dblib_uboot_create(CuTest* tc)
{
  enum eDBError res;

  /* Create buffer */
  res = db_buffer_create_malloc(&db_buffer_uboot, 1024, 1024);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Create uboot database interpreter */
  res = db_create_database_interp_uboot(&ddi_uboot, &db_buffer_uboot);
  CuAssertIntEquals(tc, dbeSuccess, res);
}


static void test_dblib_uboot_write(CuTest* tc)
{
  enum eDBError res;
  size_t i;

  /* Write database at defaults */
  res = db_database_default(&we2111_user_database_v2);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Write the database */
  res = db_database_write(&we2111_user_database_v2, &ddi_uboot, we2111_user_database_v2.db_database_version);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /*
   * Now translate the u-boot fw_setenv format (name value) to fw_printenv format (name=value)
   *
   * There should be no spaces in the buffer, apart from the one between name and value, because database strings are base64 encoded
   * so we can just replace all spaces with equals
   */
  for (i = 0; i < db_buffer_uboot.posn; i++)
  {
    if (' ' == db_buffer_uboot.buffer[i])
      db_buffer_uboot.buffer[i] = '=';
  }
}


static void test_dblib_uboot_read(CuTest* tc)
{
  enum eDBError res;
  uint32_t version;

  /* Zero all elements in the database */
  db_zero();

  /* Read the database */
  res = db_database_read(&we2111_user_database_v2, &ddi_uboot, &version, false);
  CuAssertIntEquals(tc, dbeSuccess, res);
  CuAssertIntEquals(tc, we2111_user_database_v2.db_database_version, version);

  /* Check that database is at defaults (we wrote database set to defaults) */
  test_dblib_default_check(tc, we2111_user_database_v2.db_database_version);
}


static void test_dblib_uboot_destroy(CuTest* tc)
{
  enum eDBError res;

  /* Destroy interpreter */
  res = db_destroy_database_interp_uboot(&ddi_uboot);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Destroy buffer */
  res = db_buffer_destroy(&db_buffer_uboot);
  CuAssertIntEquals(tc, dbeSuccess, res);
}


/** Test reading and writing database in u-boot environment format variables */
CuSuite *CuGetSuite_DB_uboot(void)
{
  CuSuite *suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_dblib_uboot_create);

  /* Add tests here */
  SUITE_ADD_TEST(suite, test_dblib_uboot_write);
  SUITE_ADD_TEST(suite, test_dblib_uboot_read);

  SUITE_ADD_TEST(suite, test_dblib_uboot_destroy);

  return suite;
}
