/*
 * test_db_version.c
 *
 *  Created on: Apr 10, 2012
 *      Author: ian
 */
#include <string.h>
#include <cutest/CuTest.h>
#include <rindb/db_database.h>
#include <rindb/db_error.h>
#include <rindb/db_strncpy.h>
#include "db_we2111.h"
#include "test_db_create.h"

#include "test_db_version.h"


static const char *const teststrarray_values[WE21_DB_D1_TESTSTRARRAY_NUM] =
{
  "Zero",
  "One",
  "Two",
  "Three",
  "Four",
  "Five",
  "Six",
  "Seven",
  "Eight",
  "Nine"
};


static void test_dblib_value_set(struct s_db_we2111_d1 *db)
{
  int i;

  /* Set values */
  db->safe_passcode = 5678;
  db->full_passcode = 9876;
  db->button_permissions = 0x55;
  db->zero_band = 99;
  memset(&db->setpoint_types, 2, sizeof(db->setpoint_types));
  db->flt_adj = 33;
  db->pulse_time = 25;
  db->qaopt = 0;
  db->analog_span_cal = 31854;
  db->analog_zero_cal = 538;
  db->test_var = 1234;
  for (i = 0; i < NUM_ADC_SYNC_FREQ; i++)
  {
    db->zeroOffset[i] = i * 10;
    db->spanCal[i] = i * 20;
  }
  db_safe_strncpy(db->model_str, "noob", APP_DB_MODEL_STR_LEN);

  for (i = 0; i < WE21_DB_D1_TESTSTRARRAY_NUM; i++)
  {
    db_safe_strncpy(db->teststrarray[i], teststrarray_values[i], WE21_DB_D1_TESTSTRARRAY_MAXLEN);
  }
}


static void test_dblib_value_check(CuTest *tc, struct s_db_we2111_d1 *db, uint32_t version)
{
  int i;

  /* Check values */
  CuAssertIntEquals(tc, 5678, db->safe_passcode);
  CuAssertIntEquals(tc, 9876, db->full_passcode);
  CuAssertIntEquals(tc, 0x55, db->button_permissions);
  CuAssertIntEquals(tc, 99, db->zero_band);
  for (i = 0; i < NUM_SETPOINTS; i++)
    CuAssertIntEquals(tc, 2, db->setpoint_types[i]);
  if (WE_DB_U_VERSION1 == version)
    CuAssertIntEquals(tc, 33, db->flt_adj);
  CuAssertIntEquals(tc, 25, db->pulse_time);
  if (WE_DB_U_VERSION1 == version)
    CuAssertIntEquals(tc, 0, db->qaopt);
  CuAssertIntEquals(tc, 31854, db->analog_span_cal);
  CuAssertIntEquals(tc, 538, db->analog_zero_cal);
  for (i = 0; i < WE21_DB_D1_TESTSTRARRAY_NUM; i++)
  {
    CuAssertIntEquals(tc, 0, strcmp(db->teststrarray[i], teststrarray_values[i]));
  }

  if (version > WE_DB_U_VERSION2)
  {
    CuAssertIntEquals(tc, 1234, db->test_var);
    for (i = 0; i < NUM_ADC_SYNC_FREQ; i++)
    {
      CuAssertIntEquals(tc, i * 10, db->zeroOffset[i]);
      CuAssertIntEquals(tc, i * 20, db->spanCal[i]);
    }
    CuAssertIntEquals(tc, 0, strcmp(db->model_str, "noob"));
  }
}


static void test_dblib_v2_default_check(CuTest *tc, struct s_db_we2111_d1 *db)
{
  int i;

  CuAssertIntEquals(tc, WE21_DB_D1_TESTVAR_DEFAULT, db->test_var);
  for (i = 0; i < NUM_ADC_SYNC_FREQ; i++)
  {
    CuAssertIntEquals(tc, db_we21_zoffs_default[i], db->zeroOffset[i]);
  }
  for (i = 0; i < NUM_ADC_SYNC_FREQ; i++)
  {
    CuAssertIntEquals(tc, WE21_DB_F_ADCSCAL_DEFAULT, db->spanCal[i]);
  }
  CuAssertIntEquals(tc, 0, strcmp(db->model_str, WE21_DB_F_MODEL_DEFAULT));
}


static void test_dblib_version1_write(CuTest *tc)
{
  enum eDBError res;
  uint32_t version;

  /* Set values */
  test_dblib_value_set(&db_we2111_d1);

  /* Write database at version 1 */
  res = db_database_write(&we2111_user_database_v1, &ddi_xml, we2111_user_database_v1.db_database_version);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Clear database */
  memset(&db_we2111_d1, 0, sizeof(db_we2111_d1));

  /* Read it back at version 2 */
  res = db_database_read(&we2111_user_database_v2, &ddi_xml, &version, false);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Check version */
  CuAssertIntEquals(tc, we2111_user_database_v1.db_database_version, version);

  /* Check values that are common */
  test_dblib_value_check(tc, &db_we2111_d1, we2111_user_database_v1.db_database_version);

  /* Check newly added values (should be default) */
  test_dblib_v2_default_check(tc, &db_we2111_d1);
}


static void test_dblib_version2_write(CuTest *tc)
{
  enum eDBError res;
  uint32_t version;

  /* Set values */
  test_dblib_value_set(&db_we2111_d1);

  /* Write version 2 database back at version 1 */
  res = db_database_write(&we2111_user_database_v2, &ddi_xml, we2111_user_database_v1.db_database_version);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /*
   * Read version 1
   */

  /* Clear database */
  memset(&db_we2111_d1, 0, sizeof(db_we2111_d1));

  /* Read it back at version 1 */
  res = db_database_read(&we2111_user_database_v1, &ddi_xml, &version, false);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Check version */
  CuAssertIntEquals(tc, we2111_user_database_v1.db_database_version, version);

  /* Check values that are common */
  test_dblib_value_check(tc, &db_we2111_d1, we2111_user_database_v1.db_database_version);

  /* Check newly added value (should be not set) */
  CuAssertIntEquals(tc, 0, db_we2111_d1.test_var);

  /* Clear database */
  memset(&db_we2111_d1, 0, sizeof(db_we2111_d1));

  /*
   * Read version 2
   */

  /* Write version 2 database back at version 2 */
  test_dblib_value_set(&db_we2111_d1);
  res = db_database_write(&we2111_user_database_v2, &ddi_xml, we2111_user_database_v2.db_database_version);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Read it back at version 2 */
  res = db_database_read(&we2111_user_database_v2, &ddi_xml, &version, false);
  CuAssertIntEquals(tc, dbeSuccess, res);

  /* Check version */
  CuAssertIntEquals(tc, we2111_user_database_v2.db_database_version, version);

  /* Check values that are common */
  test_dblib_value_check(tc, &db_we2111_d1, we2111_user_database_v2.db_database_version);
}


CuSuite *CuGetSuite_DB_Version(void)
{
  CuSuite *suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_dblib_version1_write);
  SUITE_ADD_TEST(suite, test_dblib_version2_write);
  return suite;
}
