/*
 * dsd_byteorder.c
 *
 *  Created on: Mar 7, 2012
 *      Author: ian
 */
#if defined(WIN32)
  #include <winsock.h>
#else
  #include <arpa/inet.h>
#endif
#include <rinweigh/adcdsd.h>

#include "dsd_byteorder.h"


/** Change the DSD record header from host to network byte order */
void AdcIntDSDRecHdrToNetworkByteOrder(struct s_adc_dsd_record_header *rec_hdr)
{
  rec_hdr->crc = htons(rec_hdr->crc);
}


/** Change the DSD record header from network to host byte order */
void AdcIntDSDRecHdrFromNetworkByteOrder(struct s_adc_dsd_record_header *rec_hdr)
{
  rec_hdr->crc = ntohs(rec_hdr->crc);
}


/** Change the DSD file index from host to network byte order */
void AdcIntDSDFileIndexToNetworkByteOrder(struct s_adc_dsd_file_index *fi)
{
  /* dsd_magic is a string, no byteswapping required */
  fi->dsd_version = htons(fi->dsd_version);
  fi->dsd_recordsize = htonl(fi->dsd_recordsize);
  fi->dsd_maxrecords = htonl(fi->dsd_maxrecords);
  fi->dsd_newestindex = htonl(fi->dsd_newestindex);
  fi->dsd_oldestindex = htonl(fi->dsd_oldestindex);
  fi->dsd_recordcount = htonl(fi->dsd_recordcount);
  fi->dsd_recordcrcseed = htons(fi->dsd_recordcrcseed);
  fi->dsd_serialnumber = htonl(fi->dsd_serialnumber);

  /* fi->chksum = htons(fi->chksum); Checksum done by itself elsewhere */
}


/** Change the DSD file index from network to host byte order */
void AdcIntDSDFileIndexFromNetworkByteOrder(struct s_adc_dsd_file_index *fi)
{
  /* dsd_magic is a string, no byteswapping required */
  fi->dsd_version = ntohs(fi->dsd_version);
  fi->dsd_recordsize = ntohl(fi->dsd_recordsize);
  fi->dsd_maxrecords = ntohl(fi->dsd_maxrecords);
  fi->dsd_newestindex = ntohl(fi->dsd_newestindex);
  fi->dsd_oldestindex = ntohl(fi->dsd_oldestindex);
  fi->dsd_recordcount = ntohl(fi->dsd_recordcount);
  fi->dsd_recordcrcseed = ntohs(fi->dsd_recordcrcseed);
  fi->dsd_serialnumber = ntohl(fi->dsd_serialnumber);

  /* fi->chksum = ntohs(fi->chksum); Checksum done by itself elsewhere */
}


/**
 * Calculate the checksum on a file index taking into account
 * the following:
 *
 * Checksum is special: so it will work irrespective of machine endianness, we
 * first put the file index contents into network byte order, then calculate the
 * checksum, then set the network order version in the header.
 *
 * The checksum returned will be in host order, it's up to you to htons() it
 *
 * This assumes the file index is already in network byte order
 */
void AdcIntDSDCalculateFileIndexChecksum(const struct s_adc_dsd_file_index *fi_netorder, int16u *checksum)
{
  *checksum = ADC_DSD_CHECKSUM(struct s_adc_dsd_file_index, *fi_netorder, ADC_DSD_FILE_HEADER_CHECKSUM_SEED);
}


/**
 * Calculate the checksum on a file index taking into account
 * the following:
 *
 * Checksum is special: so it will work irrespective of machine endianness, we
 * first put the file index contents into network byte order, then calculate the
 * checksum, then set the network order version in the header.
 *
 * The checksum returned will be in host order, it's up to you to htons() it
 *
 * This assumes the file index is in host order.
 */
void AdcDSDCalculateFileIndexChecksum(const struct s_adc_dsd_file_index *fi, int16u *checksum)
{
  struct s_adc_dsd_file_index fi_netorder;

  fi_netorder = *fi;
  AdcIntDSDFileIndexToNetworkByteOrder(&fi_netorder);
  AdcIntDSDCalculateFileIndexChecksum(&fi_netorder, checksum);
}
