/*
 * dsd_fileindex.h
 *
 *  Created on: Mar 14, 2012
 *      Author: ian
 */

#ifndef DSD_FILEINDEX_H_
#define DSD_FILEINDEX_H_

#include <sys/types.h>
#include <rinweigh/adcdsd.h>


/** Write the file indexes back to the DSD file */
enum eAdcDsdError AdcIntDSDWriteFileIndexes(struct s_adc_dsd *dsd);


/** Read the file indexes from the DSD file */
enum eAdcDsdError AdcIntDSDReadFileIndexes(struct s_adc_dsd *dsd);


/**
 * Read a file index from the DSD file (no locking).
 *
 * @param dsd is the DSD structure
 * @param which is 0 for the first index, or 1 for the second index
 * @param fi is the returned file index
 */
enum eAdcDsdError AdcIntDSDReadFileIndex(struct s_adc_dsd *dsd, int which, struct s_adc_dsd_file_index *fi);


/** Calculate the (theoretical) size of the DSD file */
off_t AdcIntDSDCalculateFileSize(struct s_adc_dsd *dsd);


/** Get the number of records currently held in the DSD */
enum eAdcDsdError AdcIntDSDGetNumberRecords(struct s_adc_dsd *dsd, int32u *records);


/** Get the number of records currently held in the DSD (no locking) */
enum eAdcDsdError AdcIntDSDGetCapacity(struct s_adc_dsd *dsd, int32u *records);


/** Get the number of records currently held in the DSD (no locking) */
enum eAdcDsdError AdcIntDSDGetCapacity(struct s_adc_dsd *dsd, int32u *records);


/** Get the record size for the DSD (no locking) */
enum eAdcDsdError AdcIntDSDGetRecordSize(struct s_adc_dsd *dsd, int32u *recordsize);


/**
 * Called prior to writing a record to the DSD to verify
 * that there is room, etc to write the record.
 *
 * Paired with @ref AdcIntDSDPostWrite, but no need to call AdcIntDSDPostWrite if something fails
 *
 * @param write_pos is the file position to write the next record to the DSD, only set on ADC_DSD_SUCCESS
 * @return ADC_DSD_SUCCESS if the write can proceed
 */
enum eAdcDsdError AdcIntDSDPreWrite(struct s_adc_dsd *dsd, off_t *write_pos);


/**
 * Called after to writing a record to the DSD to update file
 * indexes
 *
 * Paired with @ref AdcIntDSDPreWrite, never call by itself.
 *
 * @param write_pos is the file position to write the next record to the DSD, only set on ADC_DSD_SUCCESS
 * @return ADC_DSD_SUCCESS if the write can proceed
 */
enum eAdcDsdError AdcIntDSDPostWrite(struct s_adc_dsd *dsd);


/** Locate the file position of record */
enum eAdcDsdError AdcIntDSDGetRecordFilePos(struct s_adc_dsd *dsd, int32u record, off_t *filepos);


#endif /* DSD_FILEINDEX_H_ */
