/*
 * dsd_purge.c
 *
 *  Created on: Mar 7, 2012
 *      Author: ian
 */
#include <sys/types.h>
#include <unistd.h>
#include "dsd_mutex.h"
#include "dsd_fileindex.h"

#include <rinweigh/adcdsd.h>


/** Make room for more records, by purging num_records from the start of the DSD (no locking) */
enum eAdcDsdError AdcIntDSDPurge(struct s_adc_dsd *dsd, size_t num_records)
{
  enum eAdcDsdError result;
  int32u newest;
  int32u oldest;
  int32u recordcount;
  int32u maxrecords;

  /* Check pointers */
  if (NULL == dsd)
  {
    return ADC_DSD_ERROR_NULL;
  }

  /* Work with locals */
  newest = dsd->file_index.dsd_newestindex;
  oldest = dsd->file_index.dsd_oldestindex;
  recordcount = dsd->file_index.dsd_recordcount;
  maxrecords = dsd->file_index.dsd_maxrecords;

  /* Special check */
  if (0 == num_records)
  {
    /* Removing 0 records will always work */
    result = ADC_DSD_SUCCESS;
    goto aidp_fail;
  }

  /* If empty, report that */
  if (0 == recordcount)
  {
    /* No records in DSD */
    result = ADC_DSD_ERROR_EMPTY;
    goto aidp_fail;
  }

  /* Can we purge that many records */
  if (num_records > recordcount || num_records > maxrecords)
  {
    result = ADC_DSD_ERROR_OUTOFRANGE;
    goto aidp_fail;
  }

  /* Purge */
  if (newest == oldest)
  {
    /* Full or empty */
    if (0 == recordcount)
    {
      /* No records in DSD */
      result = ADC_DSD_ERROR_EMPTY;
      goto aidp_fail;
    }
    else
    {
      /* DSD full, advance oldest */
      if (oldest + num_records >= maxrecords)
      {
        /* Wraps */
        oldest = num_records - (maxrecords - oldest);
      }
      else
      {
        /* No wrap */
        oldest += num_records;
      }
    }
  }
  else if (newest > oldest)
  {
    /* Newest after oldest, so just advance oldest (can't wrap) */
    oldest += num_records;
  }
  else
  {
    /* Newest before oldest, file writing has wrapped */
    if (oldest + num_records >= maxrecords)
    {
      /* Wraps */
      oldest = num_records - (maxrecords - oldest);
    }
    else
    {
      /* No wrap */
      oldest += num_records;
    }
  }

  /* Adjust record count */
  recordcount -= num_records;

  /* Write back */
  dsd->file_index.dsd_newestindex = newest;
  dsd->file_index.dsd_oldestindex = oldest;
  dsd->file_index.dsd_recordcount = recordcount;
  dsd->file_index.dsd_maxrecords = maxrecords;

  /* Now write the file indexes back to the DSD file */
  result = AdcIntDSDWriteFileIndexes(dsd);

aidp_fail:
  return result;
}
