/*
 * dsd_read.c
 *
 *  Created on: Mar 14, 2012
 *      Author: ian
 */
#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include <rinhw/stdrinstrum.h>
#include "dsd_fileindex.h"
#include <rinweigh/adcdsd.h>
#include "dsd_byteorder.h"

#include "dsd_read.h"


/** Read a record from the DSD (no locking) */
enum eAdcDsdError AdcIntDSDReadRecord(struct s_adc_dsd *dsd, int32u record, void *buffer, int16u buflen)
{
  struct s_adc_dsd_record_header rec_hdr;
  enum eAdcDsdError result;
  enum eAdcDsdError interim_result;
  off_t read_pos;
  int32u recordsize;
  int8u *workbuf;

  /* Check pointers */
  if (NULL == dsd || NULL == buffer)
  {
    result = ADC_DSD_ERROR_NULL;
    goto adirr_fail;
  }

  /* Get file position of record to read */
  interim_result = AdcIntDSDGetRecordFilePos(dsd, record, &read_pos);
  if (ADC_DSD_SUCCESS != interim_result)
  {
    result = interim_result;
    goto adirr_fail;
  }

  /* Locate read position in DSD file */
  if (lseek(dsd->fd, read_pos, SEEK_SET) < 0)
  {
    result = ADC_DSD_ERROR_FILE;
    goto adirr_fail;
  }

  /* Read the record from the file */
  recordsize = dsd->file_index.dsd_recordsize;
  if ((ssize_t) recordsize != read(dsd->fd, dsd->work_buffer, recordsize))
  {
    result = ADC_DSD_ERROR_FILE;
    goto adirr_fail;
  }

  /* Get checksum */
  workbuf = dsd->work_buffer;
  memcpy(&rec_hdr, workbuf, sizeof(rec_hdr));
  AdcIntDSDRecHdrFromNetworkByteOrder(&rec_hdr);
  workbuf += sizeof(rec_hdr);

  /* Check data against checksum */
  if (rec_hdr.crc != AdcCalcCRC16(workbuf, recordsize - sizeof(rec_hdr),
                                  dsd->file_index.dsd_recordcrcseed))
  {
    result = ADC_DSD_ERROR_RECORDCHKSUM;
    goto adirr_fail;
  }

  /* TODO dsd read check lengths */

  /* Copy data back to user */
  memcpy(buffer, workbuf, buflen);
  result = ADC_DSD_SUCCESS;

adirr_fail:
  return result;
}
