/*
 * dsd_write.c
 *
 *  Created on: Mar 6, 2012
 *      Author: ian
 */
#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include <rinhw/stdrinstrum.h>
#include <rinweigh/adcdatabase.h>
#include "dsd_mutex.h"
#include "dsd_byteorder.h"
#include <rinweigh/adcdsd.h>
#include "dsd_fileindex.h"

#include "dsd_write.h"


/** Write a record into the DSD (no locking) */
enum eAdcDsdError AdcIntDSDWriteRecord(struct s_adc_dsd *dsd, const void *buffer, const int16u buflen)
{
  struct s_adc_dsd_record_header rec_hdr;
  enum eAdcDsdError result;
  enum eAdcDsdError interim_result;
  off_t write_pos;
  int8u *workbuf;
  size_t record_size;
  size_t data_size;

  /* Check pointers */
  if (NULL == dsd || NULL == buffer)
  {
    result = ADC_DSD_ERROR_NULL;
    goto adwr_fail;
  }

  /* Check size of data to be written will fit */
  data_size = buflen + sizeof(rec_hdr);
  record_size = dsd->file_index.dsd_recordsize;
  if (data_size > record_size)
  {
    result = ADC_DSD_ERROR_RECORDSIZE;
    goto adwr_fail;
  }

  /* Get the file position to write the next record to the DSD */
  interim_result = AdcIntDSDPreWrite(dsd, &write_pos);
  if (ADC_DSD_SUCCESS != interim_result)
  {
    result = interim_result;
    goto adwr_fail;
  }

  /* Assemble data in the work buffer */
  workbuf = dsd->work_buffer;
  memset(workbuf, 0, record_size);
  workbuf += sizeof(rec_hdr);
  memcpy(workbuf, buffer, buflen);

  /* Fill out header and put into work buffer */
  rec_hdr.crc = AdcCalcCRC16(workbuf, record_size - sizeof(rec_hdr), dsd->file_index.dsd_recordcrcseed);
  AdcIntDSDRecHdrToNetworkByteOrder(&rec_hdr);
  memcpy(dsd->work_buffer, &rec_hdr, sizeof(rec_hdr));

  /* Locate write position in DSD file */
  if (lseek(dsd->fd, write_pos, SEEK_SET) < 0)
  {
    result = ADC_DSD_ERROR_FILE;
    goto adwr_fail;
  }

  /* Write data */
  if ((ssize_t) record_size != write(dsd->fd, dsd->work_buffer, record_size))
  {
    result = ADC_DSD_ERROR_FILE;
    goto adwr_fail;
  }

  /* Update file indexes, only if successful this far */
  interim_result = AdcIntDSDPostWrite(dsd);
  if (ADC_DSD_SUCCESS != interim_result)
  {
    result = interim_result;
    goto adwr_fail;
  }

  /* Success */
  result = ADC_DSD_SUCCESS;

adwr_fail:
  return result;
}
