/*
 * el_pipe.c
 *
 *  Created on: Nov 6, 2017
 *      Author: ian
 */
#include <stdio.h>
#include <string.h>
#include <rinexec/el_runcmd.h>

#include <rinexec/el_pipe.h>


/** Initialise a s_runcmd_pipe_config structure (to empty, ie no data will be sent or received from the child process) */
void el_init_runcmd_pipe_config(struct s_runcmd_pipe_config *rcc)
{
  memset(rcc, 0, sizeof(*rcc));
  rcc->read_cmd_data_stdout = NULL;
  rcc->read_cmd_pipe_stdout_fn = NULL;
  rcc->read_cmd_data_stderr = NULL;
  rcc->read_cmd_pipe_stderr_fn = NULL;
  rcc->write_cmd_data_stdin = NULL;
  rcc->write_cmd_pipe_stdin_fn = NULL;
}


/** Create a callback for reading data from a process from stdout, stderr, or both */
void el_init_read_cmd_pipe_callback(struct s_runcmd_pipe_config *rcpc, enum e_el_read_pipe read_pipe, read_cmd_pipe_fn_t read_fn, void *read_cmd_data)
{
  switch (read_pipe)
  {
    case erp_stdout:
      rcpc->read_cmd_data_stdout = read_cmd_data;
      rcpc->read_cmd_pipe_stdout_fn = read_fn;
      break;
    case erp_stderr:
      rcpc->read_cmd_data_stderr = read_cmd_data;
      rcpc->read_cmd_pipe_stderr_fn = read_fn;
      break;
    case erp_stdout_and_stderr:
      rcpc->read_cmd_data_stdout = read_cmd_data;
      rcpc->read_cmd_pipe_stdout_fn = read_fn;
      rcpc->read_cmd_data_stderr = read_cmd_data;
      rcpc->read_cmd_pipe_stderr_fn = read_fn;
      break;
    default:
      break;
  }
}


/** Create a callback for writing data to a process to stdin */
void el_init_write_cmd_pipe_callback(struct s_runcmd_pipe_config *rcpc, write_cmd_pipe_fn_t write_fn, void *write_cmd_data)
{
  rcpc->write_cmd_data_stdin = write_cmd_data;
  rcpc->write_cmd_pipe_stdin_fn = write_fn;
}
