/*
 * el_readpipe_bufalloc.c
 *
 *  Created on: Sep 14, 2012
 *      Author: ian
 */
#include <stddef.h>
#include <string.h>
#include <stdlib.h>
#include <rinexec/el_runcmd.h>
#include <rinexec/el_pipe.h>

#include <rinexec/el_readpipe_bufalloc.h>


static enum e_el_result el_read_cmd_pipe_bufferalloc(void *user_data, const char *data, size_t bytes_read)
{
  struct s_runcmd_bufferalloc *rcba = (struct s_runcmd_bufferalloc *) user_data;
  size_t room_left;
  size_t bytes_to_write;
  size_t allocsize;
  char *newdata;
  enum e_el_result result;
  size_t total_bytes;

  /* Extra byte if null terminating */
  total_bytes = (rcba->null_terminate) ? bytes_read + sizeof(char) : bytes_read;

  room_left = rcba->data_max_len - rcba->data_posn;
  if (total_bytes < room_left || NULL == rcba->data)
  {
    /* Need to re-allocate more room */
    if (rcba->data_alloc_increments < total_bytes)
      allocsize = ((total_bytes / rcba->data_alloc_increments) + 1) * rcba->data_alloc_increments;
    else
      allocsize = rcba->data_alloc_increments;
    newdata = realloc(rcba->data, rcba->data_max_len + allocsize);
    if (NULL == newdata)
    {
      result = elr_allocfail;
      goto rcpb_fail;
    }
    rcba->data = newdata;
    rcba->data_max_len += allocsize;
  }

  /* Work out how many bytes to copy */
  room_left = rcba->data_max_len - rcba->data_posn;
  bytes_to_write = (bytes_read < room_left) ? bytes_read : room_left;

  /* Copy data */
  memcpy(&rcba->data[rcba->data_posn], data, bytes_to_write);
  rcba->data_posn += bytes_to_write;

  /* Null terminate if necessary */
  if (rcba->null_terminate)
    rcba->data[rcba->data_posn] = '\0';

  result = (room_left >= bytes_read) ? elr_success : elr_bufferfull;

rcpb_fail:
  return result;
}


/** Create a buffer allocator for data read from a process (length set by rcba->data_posn) */
void el_init_read_cmd_pipe_bufferalloc(struct s_runcmd_pipe_config *rcpc, struct s_runcmd_bufferalloc *rcba, size_t data_alloc_increments, enum e_el_read_pipe read_pipe)
{
  rcba->data = NULL;
  rcba->data_posn = 0;
  rcba->data_alloc_increments = data_alloc_increments;
  rcba->data_max_len = 0;
  rcba->null_terminate = false;

  el_init_read_cmd_pipe_callback(rcpc, read_pipe, el_read_cmd_pipe_bufferalloc, rcba);
}


/** Create a string allocator for data read from a process (length set by null termination) */
void el_init_read_cmd_pipe_stringalloc(struct s_runcmd_pipe_config *rcpc, struct s_runcmd_bufferalloc *rcba, size_t data_alloc_increments, enum e_el_read_pipe read_pipe)
{
  el_init_read_cmd_pipe_bufferalloc(rcpc, rcba, data_alloc_increments, read_pipe);
  rcba->null_terminate = true;
}


/** Free the buffers within a buffer allocator */
void el_free_read_cmd_pipe_bufferalloc(struct s_runcmd_bufferalloc *rcba)
{
  if (NULL != rcba->data)
  {
    free(rcba->data);
    rcba->data = NULL;
  }
  rcba->data_posn = 0;
  rcba->data_max_len = 0;
}

