/*
 * el_readpipe_buffer.c
 *
 *  Created on: Sep 14, 2012
 *      Author: ian
 */
#include <stddef.h>
#include <string.h>
#include <unistd.h>
#include <rinexec/el_runcmd.h>
#include <rinexec/el_pipe.h>

#include <rinexec/el_readpipe_buffer.h>


static enum e_el_result el_read_cmd_pipe_buffer(void *user_data, const char *data, size_t bytes_read)
{
  struct s_runcmd_buffer *rcb = (struct s_runcmd_buffer *) user_data;
  size_t room_left;
  size_t bytes_to_write;

  /* Work out how many bytes to copy */
  room_left = rcb->data_max_len - rcb->data_posn;
  if (rcb->null_terminate)
    room_left -= sizeof(char);
  bytes_to_write = (bytes_read < room_left) ? bytes_read : room_left;

  /* Copy data */
  memcpy(&rcb->data[rcb->data_posn], data, bytes_to_write);
  rcb->data_posn += bytes_to_write;

  /* Null terminate if necessary */
  if (rcb->null_terminate)
    rcb->data[rcb->data_posn] = '\0';

  return (room_left > bytes_read) ? elr_success : elr_bufferfull;
}


/** Create a buffer reader for reading data from a process (length set by rcba->data_posn) */
void el_init_read_cmd_pipe_buffer(struct s_runcmd_pipe_config *rcpc, struct s_runcmd_buffer *rcb, char *data, size_t data_max_len, enum e_el_read_pipe read_pipe)
{
  rcb->data = data;
  rcb->data_posn = 0;
  rcb->data_max_len = data_max_len;
  rcb->null_terminate = false;

  el_init_read_cmd_pipe_callback(rcpc, read_pipe, el_read_cmd_pipe_buffer, rcb);
}


/** Create a string reader for reading data from a process (length set by null termination) */
void el_init_read_cmd_pipe_string(struct s_runcmd_pipe_config *rcpc, struct s_runcmd_buffer *rcb, char *data, size_t data_max_len, enum e_el_read_pipe read_pipe)
{
  el_init_read_cmd_pipe_buffer(rcpc, rcb, data, data_max_len, read_pipe);
  rcb->null_terminate = true;
}
