/*
 * fileutils.c
 *
 *  Created on: 11/10/2010
 *      Author: ian
 */
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <sys/wait.h>
#include <errno.h>
#include <stdbool.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdbool.h>

#include <rinexec/el_internal.h>
#include <rinexec/el_runcmd.h>


/** Write any data to the process, and read and data back from the process as configured */
enum e_el_result read_write_cmd_pipes(int in_stdout_fd, int in_stderr_fd, int out_stdin_fd, bool *out_stdin_fd_isopen, struct s_runcmd_pipe_config *rcpc)
{
  fd_set read_fds;
  fd_set write_fds;
  fd_set except_fds;
  int bytes_read;
  bool read_done_stdout;
  bool read_done_stderr;
  bool write_done_stdin;
  bool do_read_stdout;
  bool do_read_stderr;
  bool do_write_stdin;
  size_t read_buf_len;
  char *read_buf = NULL;
  size_t write_stdin_buf_len;
  char *write_stdin_buf = NULL;
  enum e_el_result result;
  enum e_el_result readres;
  size_t bytes_to_write;
  ssize_t bytes_written;
  ssize_t write_res;
  int maxfd;

  do_read_stdout = (NULL != rcpc->read_cmd_pipe_stdout_fn);
  do_read_stderr = (NULL != rcpc->read_cmd_pipe_stderr_fn);
  do_write_stdin = (NULL != rcpc->write_cmd_pipe_stdin_fn);

  /*
   * Allocate buffers
   *
   * Because the read_cmd_pipe_stdout_fn() and read_cmd_pipe_stderr_fn() are expected
   * to take all data passed to them, we can re-use the read buffer for both reading stdin and stdout
   *
   * Writing stdin is a different case, because write() may not take all data, and so needs it's own buffer
   */
  if (do_read_stdout || do_read_stderr)
  {
    read_buf_len = BUFSIZ;
    read_buf = malloc(read_buf_len);
    if (NULL == read_buf)
    {
      result = elr_allocfail;
      goto rwcp_fail;
    }
  }
  if (do_write_stdin)
  {
    write_stdin_buf_len = BUFSIZ;
    write_stdin_buf = malloc(write_stdin_buf_len);
    if (NULL == write_stdin_buf)
    {
      result = elr_allocfail;
      goto rwcp_fail;
    }
  }

  /* Read and write data */
  for (read_done_stdout = false, read_done_stderr = false, write_done_stdin = false, bytes_to_write = 0, bytes_written = 0;
       ((do_read_stdout && !read_done_stdout) || (do_read_stderr && !read_done_stderr) || (do_write_stdin && !write_done_stdin)); )
  {
    /* Watch infd for data, outfd for room to send and problems */
    FD_ZERO(&read_fds);
    FD_ZERO(&write_fds);
    FD_ZERO(&except_fds);
    maxfd = -1;
    if (do_read_stdout && !read_done_stdout)
    {
      FD_SET(in_stdout_fd, &read_fds);
      FD_SET(in_stdout_fd, &except_fds);
      if (in_stdout_fd > maxfd)
        maxfd = in_stdout_fd;
    }
    if (do_read_stderr && !read_done_stderr)
    {
      FD_SET(in_stderr_fd, &read_fds);
      FD_SET(in_stderr_fd, &except_fds);
      if (in_stderr_fd > maxfd)
        maxfd = in_stderr_fd;
    }
    if (do_write_stdin && !write_done_stdin)
    {
      FD_SET(out_stdin_fd, &write_fds);
      FD_SET(out_stdin_fd, &except_fds);
      if (out_stdin_fd > maxfd)
        maxfd = out_stdin_fd;
    }

    /* Wait for stuff to happen */
    if (select(maxfd + 1, &read_fds, &write_fds, &except_fds, 0) < 0)
    {
      if (EINTR == errno)
      {
        /* select() was interrupted */
        continue;
      }
      else
      {
        /* Error - stop */
        result = elr_iofail;
        goto rwcp_fail;
      }
    }

    /* Check for read stdout data */
    if (do_read_stdout && !read_done_stdout)
    {

      /* Check for errors */
      if (FD_ISSET(in_stdout_fd, &except_fds))
      {
        result = elr_iofail;
        goto rwcp_fail;
      }

      /* Check for read data */
      if (FD_ISSET(in_stdout_fd, &read_fds))
      {
        bytes_read = read(in_stdout_fd, read_buf, read_buf_len);
        if (bytes_read < 0)
        {
          /* Error (<0) */
          result = elr_iofail;
          goto rwcp_fail;
        }
        else if (bytes_read > 0)
        {
          /* Read data (>0) */
          readres = rcpc->read_cmd_pipe_stdout_fn(rcpc->read_cmd_data_stdout, read_buf, bytes_read);
          if (elr_success != readres)
          {
            result = readres;
            goto rwcp_fail;
          }
        }
        else /* bytes_read == 0, means EOF */
        {
          read_done_stdout = true;
        }
      }
    }

    /* Check for read stderr data */
    if (do_read_stderr && !read_done_stderr)
    {

      /* Check for errors */
      if (FD_ISSET(in_stderr_fd, &except_fds))
      {
        result = elr_iofail;
        goto rwcp_fail;
      }

      /* Check for read data */
      if (FD_ISSET(in_stderr_fd, &read_fds))
      {
        bytes_read = read(in_stderr_fd, read_buf, read_buf_len);
        if (bytes_read < 0)
        {
          /* Error (<0) */
          result = elr_iofail;
          goto rwcp_fail;
        }
        else if (bytes_read > 0)
        {
          /* Read data (>0) */
          readres = rcpc->read_cmd_pipe_stderr_fn(rcpc->read_cmd_data_stderr, read_buf, bytes_read);
          if (elr_success != readres)
          {
            result = readres;
            goto rwcp_fail;
          }
        }
        else /* bytes_read == 0, means EOF */
        {
          read_done_stderr = true;
        }
      }
    }

    /* Get data to write */
    if (do_write_stdin && !write_done_stdin)
    {
      /* Check for errors */
      if (FD_ISSET(out_stdin_fd, &except_fds))
      {
        result = elr_iofail;
        goto rwcp_fail;
      }

      if (FD_ISSET(out_stdin_fd, &write_fds))
      {
        if (0 == bytes_to_write)
        {
          result = rcpc->write_cmd_pipe_stdin_fn(rcpc->write_cmd_data_stdin, write_stdin_buf, write_stdin_buf_len, &bytes_to_write);
          if (elr_success != result)
          {
            /* Error */
            result = elr_iofail;
            goto rwcp_fail;
          }
          else if (0 == bytes_to_write)
          {
            /* == 0 No more data. Close outfd to signal child process that we don't have any more data */
            close(out_stdin_fd);
            *out_stdin_fd_isopen = false;
            write_done_stdin = true;
          }
          bytes_written = 0;
        }
        if (bytes_to_write > 0)
        {
          write_res = write(out_stdin_fd, &write_stdin_buf[bytes_written], bytes_to_write);
          if (write_res < 0)
          {
            /* Error (<0) */
            result = elr_iofail;
            goto rwcp_fail;
          }
          bytes_written += write_res;
          bytes_to_write -= write_res;
        }
      }
    }
  }
  result = elr_success;

rwcp_fail:
  if (NULL != write_stdin_buf)
    free(write_stdin_buf);
  if (NULL != read_buf)
    free(read_buf);

  return result;
}


/** Duplicate the arguments */
static char **internal_dup_args(const char *const args[])
{
  size_t arg_cnt;
  size_t arg_num;
  char **dup_args;

  /* Count the arguments, extra one for NULL */
  for (arg_num = 0; NULL != args[arg_num]; arg_num++)
    ;
  arg_num++;

  /* Allocate array of pointers */
  dup_args = malloc(sizeof(char *) * arg_num);
  if (NULL == dup_args)
  {
    goto ida_fail;
  }
  memset(dup_args, 0, sizeof(char *) * arg_num);

  /* Duplicate arguments */
  for (arg_cnt = 0; arg_cnt < arg_num - 1; arg_cnt++)
  {
    dup_args[arg_cnt] = strdup(args[arg_cnt]);
    if (NULL == dup_args[arg_cnt])
    {
      goto ida_fail;
    }
  }
  dup_args[arg_cnt] = NULL;

  return dup_args;

ida_fail:
  /* Clean up everything and return NULL */
  if (NULL != dup_args)
  {
    for (arg_cnt = 0; NULL != dup_args[arg_cnt]; arg_cnt++)
    {
      free(dup_args[arg_cnt]);
      dup_args[arg_cnt] = NULL;
    }
    free(dup_args);
  }
  return NULL;
}


/** Free up the allocated arguments */
static void internal_free_args(char **dup_args)
{
  size_t arg_cnt;

  /* Duplicate arguments */
  for (arg_cnt = 0; NULL != dup_args[arg_cnt]; arg_cnt++)
  {
    free(dup_args[arg_cnt]);
    dup_args[arg_cnt] = NULL;
  }
  free(dup_args);
}


/** Close all open file descriptors, except stdin, stdout, stderr */
static void internal_close_fds(void)
{
  int num_handles;
  int fd;
  int fd_flags;

  num_handles = getdtablesize();
  for (fd = 0; fd < num_handles; fd++)
  {
    if (STDIN_FILENO != fd && STDOUT_FILENO != fd && STDERR_FILENO != fd)
    {
      fd_flags = fcntl(fd, F_GETFD);
      if (fd_flags >= 0)
      {
        close(fd);
      }
    }
  }
}


/**
 * Run cmd with arguments args, and return cmd_ret_code
 * Depending on the configuration of rcc, data can be sent and received from the child process
 *
 * @param cmd is the command to run, not NULL
 * @param args is the arguments to pass to the cmd (including arg[0] is the command), not NULL
 * @param cmd_ret_code is a pointer to an integer that is set to the result of the command. Can be NULL if !wait_complete, otherwise not NULL.
 * @param rcc is a pre-configured structure that defines how data should be sent to and received from the child process. Can be NULL if this is not required.
 * @param wait_complete if true, then will wait until the child process exits, otherwise returns immediately. Will still delay if rcc is being used to send/receive data.
 * @param search_path if true, then execvp() will be used (will search path for cmd), otherwise execv() will be used (must specify exact location of cmd)
 * @param close_all_fds if true, then all file descriptors except stdin, stdout, stderr will be closed in the child prior to the execv() call.
 *
 * @return one of the @ref enum e_el_result result codes
 */
enum e_el_result el_run_command(const char *cmd, const char *const args[], int *cmd_ret_code,
                                struct s_runcmd_pipe_config *rcc,
                                bool wait_complete,
                                bool search_path,
                                bool close_all_fds)
{
  int pid;
  char **dup_args = NULL;

  int child_to_parent_stdout_pipefd[2];
#define child_to_parent_stdout_read_fd child_to_parent_stdout_pipefd[0]
#define child_to_parent_stdout_write_fd child_to_parent_stdout_pipefd[1]
  int child_to_parent_stderr_pipefd[2];
#define child_to_parent_stderr_read_fd child_to_parent_stderr_pipefd[0]
#define child_to_parent_stderr_write_fd child_to_parent_stderr_pipefd[1]
  int parent_to_child_stdin_pipefd[2];
#define parent_to_child_stdin_read_fd parent_to_child_stdin_pipefd[0]
#define parent_to_child_stdin_write_fd parent_to_child_stdin_pipefd[1]

  /* Only connect up pipes if actually required */
  bool need_child_to_parent_stdout;
  bool need_child_to_parent_stderr;
  bool need_parent_to_child_stdin;

  /* We need to keep a track of which ones are open, and using -1 confuses the vfork parent/child process */
  bool child_to_parent_stdout_read_isopen = false;
  bool child_to_parent_stdout_write_isopen = false;
  bool child_to_parent_stderr_read_isopen = false;
  bool child_to_parent_stderr_write_isopen = false;
  bool parent_to_child_stdin_read_isopen = false;
  bool parent_to_child_stdin_write_isopen = false;

  enum e_el_result result;

  need_child_to_parent_stdout = (NULL != rcc && NULL != rcc->read_cmd_pipe_stdout_fn);
  need_child_to_parent_stderr = (NULL != rcc && NULL != rcc->read_cmd_pipe_stderr_fn);
  need_parent_to_child_stdin = (NULL != rcc && NULL != rcc->write_cmd_pipe_stdin_fn);

  /* Duplicate arguments because execv() takes char *const[] */
  dup_args = internal_dup_args(args);
  if (NULL == dup_args)
  {
    /* malloc failed */
    result = elr_allocfail;
    goto irc_fail;
  }

  if (need_child_to_parent_stdout)
  {
    /* Create pipe to capture stdout from exec'd command */
    if (pipe(child_to_parent_stdout_pipefd) < 0)
    {
      /* pipe() failed */
      result = elr_iofail;
      goto irc_fail;
    }
    child_to_parent_stdout_read_isopen = true;
    child_to_parent_stdout_write_isopen = true;
  }

  if (need_child_to_parent_stderr)
  {
    /* Create pipe to capture stderr from exec'd command */
    if (pipe(child_to_parent_stderr_pipefd) < 0)
    {
      /* pipe() failed */
      result = elr_iofail;
      goto irc_fail;
    }
    child_to_parent_stderr_read_isopen = true;
    child_to_parent_stderr_write_isopen = true;
  }

  if (need_parent_to_child_stdin)
  {
    /* Create pipe to write stdin of exec'd command */
    if (pipe(parent_to_child_stdin_pipefd) < 0)
    {
      /* pipe() failed */
      result = elr_iofail;
      goto irc_fail;
    }
    parent_to_child_stdin_read_isopen = true;
    parent_to_child_stdin_write_isopen = true;
  }

  /* vfork() to create child process to run the command */
  pid = vfork();
  if (pid < 0)
  {
    /* vfork() failed, cleanup is below */
    result = elr_execfail;
  }
  else if (0 == pid)
  {
    /* Child process, close read end of child to parent pipe, and write end of parent to child pipe */
    if (need_child_to_parent_stdout)
      close(child_to_parent_stdout_read_fd);
    if (need_child_to_parent_stderr)
      close(child_to_parent_stderr_read_fd);
    if (need_parent_to_child_stdin)
      close(parent_to_child_stdin_write_fd);

    /* Remap stdout to the write end of the child to parent pipe */
    close(STDOUT_FILENO);
    if (need_child_to_parent_stdout)
      dup2(child_to_parent_stdout_write_fd, STDOUT_FILENO);

    /* Remap stderr to the write end of the child to parent pipe */
    close(STDERR_FILENO);
    if (need_child_to_parent_stderr)
      dup2(child_to_parent_stderr_write_fd, STDERR_FILENO);

    /* Remap stdin to the read end of the parent to child pipe */
    close(STDIN_FILENO);
    if (need_parent_to_child_stdin)
      dup2(parent_to_child_stdin_read_fd, STDIN_FILENO);

    /* Close all open file descriptors */
    if (close_all_fds)
      internal_close_fds();

    /* Execute the command, should never return unless something is really broken */
    if (search_path)
      (void) execvp(cmd, dup_args);
    else
      (void) execv(cmd, dup_args);

    /* Should never reach here, but if it does, clean up and exit */
    if (need_child_to_parent_stdout)
      close(child_to_parent_stdout_write_fd);
    if (need_child_to_parent_stderr)
      close(child_to_parent_stderr_write_fd);
    if (need_parent_to_child_stdin)
      close(parent_to_child_stdin_read_fd);
    _exit(errno);
  }
  else
  {
    /* Parent process, close write end of child to parent pipe, and read end of the parent to child pipe */
    if (need_child_to_parent_stdout)
    {
      close(child_to_parent_stdout_write_fd);
      child_to_parent_stdout_write_isopen = false;
    }
    if (need_child_to_parent_stderr)
    {
      close(child_to_parent_stderr_write_fd);
      child_to_parent_stderr_write_isopen = false;
    }
    if (need_parent_to_child_stdin)
    {
      close(parent_to_child_stdin_read_fd);
      parent_to_child_stdin_read_isopen = false;
    }

    /* Read and write data to child process by configured means */
    if (NULL != rcc)
    {
      result = read_write_cmd_pipes(child_to_parent_stdout_read_fd, child_to_parent_stderr_read_fd, parent_to_child_stdin_write_fd, &parent_to_child_stdin_write_isopen, rcc);
      /* We don't test the result here, so we can do the waitpid() below, regardless of the result */
    }
    else
    {
      result = elr_success;
    }

    if (wait_complete)
    {
      /* Wait for child process to finish */
      if (waitpid(pid, cmd_ret_code, 0) < 0)
      {
        result = elr_execfail;
        goto irc_fail;
      }
    }
  }

irc_fail:
  /* Clean up fds */
  if (child_to_parent_stdout_read_isopen)
    close(child_to_parent_stdout_read_fd);
  if (child_to_parent_stdout_write_isopen)
    close(child_to_parent_stdout_write_fd);
  if (child_to_parent_stderr_read_isopen)
    close(child_to_parent_stderr_read_fd);
  if (child_to_parent_stderr_write_isopen)
    close(child_to_parent_stderr_write_fd);
  if (parent_to_child_stdin_read_isopen)
    close(parent_to_child_stdin_read_fd);
  if (parent_to_child_stdin_write_isopen)
    close(parent_to_child_stdin_write_fd);

  /* Clean up args */
  if (NULL != dup_args)
    internal_free_args(dup_args);

  return result;
}


/**
 * Convert cmd_ret_code of the child process from @ref el_run_command into the exited flag.
 * If you only want to compile for posix systems, you can use WIFEXITED, et el.
 * This function is for running on both posix and windows systems.
 *
 * @param cmd_ret_code is the value returned from el_run_command for the child process
 *
 * @return whether the child process has exited
 */
bool el_return_code_has_exited(int cmd_ret_code)
{
  return WIFEXITED(cmd_ret_code);
}


/**
 * Convert cmd_ret_code of the child process from @ref el_run_command into the exit code.
 * If you only want to compile for posix systems, you can use WEXITSTATUS, et el.
 * This function is for running on both posix and windows systems.
 *
 * @param cmd_ret_code is the value returned from el_run_command for the child process
 *
 * @return the return code from the child process
 */
unsigned el_convert_return_code(int cmd_ret_code)
{
  return WEXITSTATUS(cmd_ret_code);
}
