/*
 * el_test_fds.c
 *
 *  Created on: Aug 4, 2014
 *      Author: ian
 */
#include <sys/types.h>
#include <string.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <rinexec/el_runcmd.h>
#include <cutest/CuTest.h>

#include "el_test_fds.h"


/** Check if a file descriptor is open */
static bool fd_is_open(int fd)
{
  int fd_flags;

  fd_flags = fcntl(fd, F_GETFD);
  return (fd_flags >= 0);
}


/** Open a few file descriptors, execute "execlibtest_fdchild", and check that they have been closed */
static void test_runcmd_fds(CuTest* tc)
{
  const char *const execlibtest_fdchild_cmd[] = { "./librinexectest_fdchild", NULL };
  enum e_el_result result;
  int ret_code;
  int fd1;
  int fd2;

  /* Open some file descriptors */
  fd1 = open("/dev/null", O_RDWR);
  fd2 = open("/dev/null", O_RDWR);

  /* Check they are open */
  CuAssertTrue(tc, fd_is_open(fd1));
  CuAssertTrue(tc, fd_is_open(fd2));

  /* Run execlibtest_fdchild which will return 0 if all inherited fds are closed. el_run_command is set to close all fds */
  result = el_run_command(execlibtest_fdchild_cmd[0], execlibtest_fdchild_cmd, &ret_code, NULL, true, false, true);
  CuAssertIntEquals(tc, elr_success, result);
  CuAssertTrue(tc, el_return_code_has_exited(ret_code));
  CuAssertIntEquals_Msg(tc, "execlibtest_fdchild should return 0 if all file descriptors closed", 0, el_convert_return_code(ret_code));

  /* Run execlibtest_fdchild which will return 0 if all inherited fds are closed. el_run_command is set to leave open */
  result = el_run_command(execlibtest_fdchild_cmd[0], execlibtest_fdchild_cmd, &ret_code, NULL, true, false, false);
  CuAssertIntEquals(tc, elr_success, result);
  CuAssertTrue(tc, el_return_code_has_exited(ret_code));
  CuAssertTrue(tc, (0 != el_convert_return_code(ret_code)));

  /* Check they are still open */
  CuAssertTrue(tc, fd_is_open(fd1));
  CuAssertTrue(tc, fd_is_open(fd2));

  close(fd1);
  close(fd2);
}


CuSuite *CuGetSuite_runcmd_fds(void)
{
  CuSuite *suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_runcmd_fds);

  return suite;
}
