/*
 * el_test_runcmd.c
 *
 *  Created on: Sep 17, 2012
 *      Author: ian
 */
#include <sys/types.h>
#include <string.h>
#include <rinexec/el_runcmd.h>
#include <cutest/CuTest.h>

#include "el_test_runcmd.h"


/** Test simple running a command, no stdio, or stdout */
static void test_runcmd(CuTest* tc)
{
  const char *const test_retcode0_cmd[] = { "test", "0", "=", "0", NULL };
  const char *const test_retcode1_cmd[] = { "test", "0", "=", "1", NULL };
  enum e_el_result result;
  int ret_code;

  /* Run "test 0 == 0", and wait for the command to complete */
  result = el_run_command(test_retcode0_cmd[0], test_retcode0_cmd, &ret_code, NULL, true, true, true);
  CuAssertIntEquals(tc, elr_success, result);
  CuAssertTrue(tc, el_return_code_has_exited(ret_code));
  CuAssertIntEquals(tc, 0, el_convert_return_code(ret_code));

  /* Run "test 0 == 1", and wait for the command to complete */
  result = el_run_command(test_retcode1_cmd[0], test_retcode1_cmd, &ret_code, NULL, true, true, true);
  CuAssertIntEquals(tc, elr_success, result);
  CuAssertTrue(tc, el_return_code_has_exited(ret_code));
  CuAssertIntEquals(tc, 1, el_convert_return_code(ret_code));
}


/** Test with and without path search */
static void test_runcmd_path(CuTest* tc)
{
  const char *const test_bin_true_cmd[] = { "/bin/true", NULL };
  const char *const test_true_cmd[] = { "true", NULL };
  enum e_el_result result;
  int ret_code;

  /* Run "/bin/true", should work with and without path */
  result = el_run_command(test_bin_true_cmd[0], test_bin_true_cmd, &ret_code, NULL, true, false, true);
  CuAssertIntEquals(tc, elr_success, result);
  CuAssertTrue(tc, el_return_code_has_exited(ret_code));
  CuAssertIntEquals(tc, 0, el_convert_return_code(ret_code));
  result = el_run_command(test_bin_true_cmd[0], test_bin_true_cmd, &ret_code, NULL, true, true, true);
  CuAssertIntEquals(tc, elr_success, result);
  CuAssertTrue(tc, el_return_code_has_exited(ret_code));
  CuAssertIntEquals(tc, 0, el_convert_return_code(ret_code));

  /* Run "true", should work with path, but not without path */
  result = el_run_command(test_true_cmd[0], test_true_cmd, &ret_code, NULL, true, false, true);
  CuAssertIntEquals(tc, elr_success, result);
  CuAssertTrue(tc, el_return_code_has_exited(ret_code));
  CuAssertIntEquals(tc, 2, el_convert_return_code(ret_code));
  result = el_run_command(test_true_cmd[0], test_true_cmd, &ret_code, NULL, true, true, true);
  CuAssertIntEquals(tc, elr_success, result);
  CuAssertTrue(tc, el_return_code_has_exited(ret_code));
  CuAssertIntEquals(tc, 0, el_convert_return_code(ret_code));
}


/** Test init pipe config */
static void test_el_init_runcmd_pipe_config(CuTest* tc)
{
  struct s_runcmd_pipe_config rcpc;

  memset(&rcpc, 0xFF, sizeof(rcpc));
  el_init_runcmd_pipe_config(&rcpc);

  CuAssertPtrEquals(tc, NULL, rcpc.read_cmd_data_stdout);
  CuAssertPtrEquals(tc, NULL, rcpc.read_cmd_pipe_stdout_fn);
  CuAssertPtrEquals(tc, NULL, rcpc.read_cmd_data_stderr);
  CuAssertPtrEquals(tc, NULL, rcpc.read_cmd_pipe_stderr_fn);
  CuAssertPtrEquals(tc, NULL, rcpc.write_cmd_data_stdin);
  CuAssertPtrEquals(tc, NULL, rcpc.write_cmd_pipe_stdin_fn);
}


CuSuite *CuGetSuite_runcmd(void)
{
  CuSuite *suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_runcmd);
  SUITE_ADD_TEST(suite, test_runcmd_path);
  SUITE_ADD_TEST(suite, test_el_init_runcmd_pipe_config);

  return suite;
}
