/*
 * el_test_write.c
 *
 *  Created on: Sep 19, 2012
 *      Author: ian
 */
#include <sys/types.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <cutest/CuTest.h>
#include <rinexec/el_runcmd.h>
#include <rinexec/el_readpipe_buffer.h>
#include <rinexec/el_readpipe_bufalloc.h>
#include <rinexec/el_readpipe_file.h>
#include <rinexec/el_writepipe_buffer.h>
#include <rinexec/el_writepipe_file.h>

#include "el_test_write.h"


static void strupper(char *s)
{
  char *p;
  for (p = s; *p; ++p)
    *p = toupper(*p);
}


/** Test running a command and writing to it from a buffer */
static void test_writebuffer(CuTest* tc)
{
  #define HELLO_WORLD_STR "hello world"
  const char *const tr_cmd[] = { "tr", "[a-z]", "[A-Z]", NULL };
  enum e_el_result result;
  int ret_code;
  #define BUFFER_SIZE 100
  char write_buf[BUFFER_SIZE] = HELLO_WORLD_STR;
  char read_buf[BUFFER_SIZE];
  struct s_runcmd_pipe_config rcpc;
  struct s_runcmd_buffer wcb;
  struct s_runcmd_buffer rcb;

  el_init_runcmd_pipe_config(&rcpc);
  el_init_write_cmd_pipe_buffer(&rcpc, &wcb, write_buf, strlen(write_buf));
  el_init_read_cmd_pipe_buffer(&rcpc, &rcb, read_buf, BUFFER_SIZE, erp_stdout);

  /* Run "tr [a-z] [A-Z]", and wait for the command to complete */
  result = el_run_command(tr_cmd[0], tr_cmd, &ret_code, &rcpc, true, true, true);
  CuAssertIntEquals(tc, elr_success, result);
  CuAssertTrue(tc, el_return_code_has_exited(ret_code));
  CuAssertIntEquals(tc, 0, el_convert_return_code(ret_code));

  /* Check for captured "HELLO WORLD" in the buffer */
  strupper(write_buf);
  CuAssertIntEquals(tc, strlen(write_buf), rcb.data_posn);
  CuAssertTrue(tc, 0 == memcmp(rcb.data, write_buf, strlen(write_buf)));
}


/** Test running a command and writing to it from a FILE */
static void test_writefile(CuTest* tc)
{
  #define HELLO_WORLD_STR "hello world"
  const char *const tr_cmd[] = { "tr", "[a-z]", "[A-Z]", NULL };
  enum e_el_result result;
  int ret_code;
  #define BUFFER_SIZE 100
  char write_buf[BUFFER_SIZE] = HELLO_WORLD_STR;
  char read_buf[BUFFER_SIZE];
  struct s_runcmd_pipe_config rcpc;
  struct s_runcmd_file wcf;
  struct s_runcmd_buffer rcb;
  FILE *file;

  /* Create temporary file, write some data to it */
  file = tmpfile();
  CuAssertPtrNotNull(tc, file);
  CuAssertIntEquals(tc, strlen(write_buf), fwrite(write_buf, sizeof(char), strlen(write_buf), file));
  CuAssertIntEquals(tc, 0, fseek(file, 0, SEEK_SET));

  el_init_runcmd_pipe_config(&rcpc);
  el_init_write_cmd_pipe_file(&rcpc, &wcf, file);
  el_init_read_cmd_pipe_buffer(&rcpc, &rcb, read_buf, BUFFER_SIZE, erp_stdout);

  /* Run "tr [a-z] [A-Z]", and wait for the command to complete */
  result = el_run_command(tr_cmd[0], tr_cmd, &ret_code, &rcpc, true, true, true);
  CuAssertIntEquals(tc, elr_success, result);
  CuAssertTrue(tc, el_return_code_has_exited(ret_code));
  CuAssertIntEquals(tc, 0, el_convert_return_code(ret_code));

  /* Check for captured "HELLO WORLD" in the buffer */
  strupper(write_buf);
  CuAssertIntEquals(tc, strlen(write_buf), rcb.data_posn);
  CuAssertTrue(tc, 0 == memcmp(rcb.data, write_buf, strlen(write_buf)));

  fclose(file);
}


CuSuite *CuGetSuite_runcmd_write(void)
{
  CuSuite *suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_writebuffer);
  SUITE_ADD_TEST(suite, test_writefile);

  return suite;
}
