#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <cutest/CuTest.h>
#include "el_test_runcmd.h"
#include "el_test_read.h"
#include "el_test_write.h"
#include "el_test_fds.h"
#include "el_test_pipesize.h"


int main(int argc, char *argv[])
{
  int result;

  /* We need rand() in one of the tests, so seed the PRNG here */
  srand(time(NULL));

  CuString *output = CuStringNew();
  CuSuite *suite = CuSuiteNew();

  /* Add tests */
  CuSuiteConsume(suite, CuGetSuite_runcmd());
  CuSuiteConsume(suite, CuGetSuite_runcmd_read());
  CuSuiteConsume(suite, CuGetSuite_runcmd_write());
  CuSuiteConsume(suite, CuGetSuite_runcmd_fds());
  CuSuiteConsume(suite, CuGetSuite_runcmd_pipesize_bug());

  /* Run all tests */
  CuSuiteRun(suite);
  CuSuiteSummary(suite, output);
  CuSuiteDetails(suite, output);

  /* Print results */
  printf("%s\n", output->buffer);
  result = (suite->failCount > 0) ? 1 : 0;

  /* Clean up */
  CuSuiteDelete(suite);
  suite = NULL;
  CuStringDelete(output);
  output = NULL;

  /* Return result */
  return result;
}
