/*
 * rlc_assemble.h
 *
 *  Created on: 1 Jan. 2019
 *      Author: ian
 */

#ifndef RLC_ASSEMBLE_VA_H_
#define RLC_ASSEMBLE_VA_H_

#include <stddef.h>
#include <stdbool.h>
#include <stdarg.h>
#include <rincmd/rc_cli_error.h>
#include <rinlegacycmd/rlc_parameter.h>


/**
 * Convert a rinstrum legacy command or query request into a string suitable for sending to the indicator
 *
 * @param buffer is the buffer to output the message to
 * @param buffer_len is the length of buffer
 * @param cmd is the rinstrum legacy command (ie IDN)
 * @param cmd_not_query true if the message is a command, false if it is a query
 * @param param_format is a string containing format characters to match the variable argument list
 *         "n" is used when a parameter is not required
 *         "i" is a signed integer parameter (passed as int64_t)
 *         "u" is an unsigned integer parameter (passed as uint64_t)
 *         "s" is null terminated string parameter (passed as const char *)
 *         "b" is a buffer parameter (passed as const void *buf and size_t len)
 * @param args is a variable argument list corresponding to the param_format
 * @return e_rc_cli_result result code
 */
enum e_rc_cli_result rlc_format_request_va(char *buffer, size_t buffer_len,
                                           const char cmd[RLC_COMMAND_LENGTH], bool cmd_not_query,
                                           const char *param_format, va_list args);


/**
 * Convert a rinstrum legacy command or query request into a string suitable for sending to the indicator
 *
 * @param buffer is the buffer to output the message to
 * @param buffer_len is the length of buffer
 * @param cmd is the rinstrum legacy command (ie IDN)
 * @param cmd_not_query true if the message is a command, false if it is a query
 * @param param_format is a string containing format characters to match the variable argument list
 *         "n" is used when a parameter is not required
 *         "i" is a signed integer parameter (passed as int64_t)
 *         "u" is an unsigned integer parameter (passed as uint64_t)
 *         "s" is null terminated string parameter (passed as const char *)
 *         "b" is a buffer parameter (passed as const void *buf and size_t len)
 * @param ... are the arguments corresponding to the param_format
 * @return e_rc_cli_result result code
 */
enum e_rc_cli_result rlc_format_request(char *buffer, size_t buffer_len,
                                        const char cmd[RLC_COMMAND_LENGTH], bool cmd_not_query, const char *param_format, ...);


/**
 * Convert a rinstrum legacy response into a string suitable for the indicator to send as a response
 *
 * @param buffer is the buffer to output the message to
 * @param buffer_len is the length of buffer
 * @param cmd is the rinstrum legacy command (ie IDN)
 * @param param_format is a string containing format characters to match the variable argument list
 *         "n" is used when a parameter is not required
 *         "i" is a signed integer parameter (passed as int64_t)
 *         "u" is an unsigned integer parameter (passed as uint64_t)
 *         "s" is null terminated string parameter (passed as const char *)
 *         "b" is a buffer parameter (passed as const void *buf and size_t len)
 * @param ... are the arguments corresponding to the param_format
 * @return e_rc_cli_result result code
 */
enum e_rc_cli_result rlc_format_response(char *buffer, size_t buffer_len,
                                         const char cmd[RLC_COMMAND_LENGTH], const char *param_format, ...);


#endif /* RLC_ASSEMBLE_VA_H_ */
