/*
 * rlc_cli_parse.h
 *
 *  Created on: 1 Jan. 2019
 *      Author: ian
 */

#ifndef RLC_CLI_PARSE_H_
#define RLC_CLI_PARSE_H_

#include <stddef.h>
#include <stdarg.h>
#include <rincmd/rc_frame.h>
#include <rincmd/rc_cli_error.h>
#include <rinlegacycmd/rlc_parameter.h>


/** Command executer structure */
struct s_rlc_cli_parser
{
  struct s_rc_frame *framing;         /*!< Framing of messages */
};


/** Create a rinstrum legacy cmd client parser. The framer must not be destroyed before the parser. */
enum e_rc_cli_result rlc_cli_parse_init(struct s_rlc_cli_parser **rcp, struct s_rc_frame *framer);


/** Destroy a rinstrum legacy cmd client parser */
enum e_rc_cli_result rlc_cli_parse_destroy(struct s_rlc_cli_parser *rcp);


/** Assemble a command ready to be sent to the host device from a rlc_command structure */
enum e_rc_cli_result rlc_cli_assemble_command(const struct s_rlc_cli_parser *rcp, const struct s_rlc_command *rlc_cmd,
                                              char *buffer, size_t maxlen);


/** Assemble a command ready to be sent to the host device (with variable arguments) */
enum e_rc_cli_result rlc_cli_assemble_command_va(const struct s_rlc_cli_parser *rcp,
                                                 const char cmd[RLC_COMMAND_LENGTH], bool cmd_not_query,
                                                 const char *param_format, va_list args,
                                                 char *buffer, size_t maxlen);


/** Parse a response from the host device into rlc_cmd */
enum e_rc_cli_result rlc_cli_parse_response(const struct s_rlc_cli_parser *rcp, const char *response, struct s_rlc_command *rlc_cmd);


#endif /* RLC_CLI_PARSE_H_ */
