/*
 * rlc_parameter_print.c
 *
 *  Created on: 7 Jan. 2019
 *      Author: ian
 */
#include <stdio.h>
#include <stddef.h>
#include <rincmd/rc_cli_error.h>
#include <rinlegacycmd/rlc_parameter.h>

#include <rinlegacycmd/rlc_command_print.h>


/** Print out a rinstrum legacy comms parameter in a human readable form */
enum e_rc_cli_result rlc_parameter_fprintf(FILE *out, const char *prefix, const struct s_rlc_parameter *param)
{
  switch (param->param_type)
  {
    case rpt_unused:
      fprintf(out, "%sType:  Unset\n", prefix);
      break;
    case rpt_error:
      fprintf(out, "%sType:  Error\n", prefix);
      break;
    case rpt_integer:
      fprintf(out, "%sType:  Integer\n", prefix);
      fprintf(out, "%sValue: %lld\n", prefix, (long long) param->int_value);
      break;
    case rpt_buffer:
      fprintf(out, "%sType:  Buffer\n", prefix);
      fprintf(out, "%sValue: %.*s\n", prefix, (int) param->buffer_length, param->buffer);
      break;
  }
  return rcr_success;
}


/** Print out a rinstrum legacy comms command in a human readable form */
enum e_rc_cli_result rlc_command_fprintf(FILE *out, const struct s_rlc_command *rlc_cmd)
{
  size_t param;
  #define RLC_PREFIX_LEN 40
  char prefix[RLC_PREFIX_LEN];

  fprintf(out, "Command: %.3s\n", rlc_cmd->command);
  fprintf(out, "Command Or Query: %s\n", (rlc_cmd->command_not_query) ? "Command" : "Query");
  fprintf(out, "Maximum Parameters: %u\n", (unsigned) rlc_cmd->max_parameters);
  fprintf(out, "Number Parameters: %u\n", (unsigned) rlc_cmd->num_parameters);
  for (param = 0; param < rlc_cmd->num_parameters; param++)
  {
    snprintf(prefix, RLC_PREFIX_LEN, "Parameter %u ", (unsigned) param);
    (void) rlc_parameter_fprintf(out, prefix, &rlc_cmd->parameters[param]);
  }
  return rcr_success;
}
