/*
 * rlc_server_handle.h
 *
 *  Created on: 14 Jan. 2019
 *      Author: ian
 */

#ifndef RLC_SERVER_HANDLE_H_
#define RLC_SERVER_HANDLE_H_

#include <stdbool.h>
#include <stddef.h>
#include <rincmd/rc_cli_error.h>
#include <rinlegacycmd/rlc_parameter.h>


/** Function pointer for command handler */
typedef void (*t_rlc_server_cmd_handler_fn)(void *userdata,
                                            const struct s_rlc_command *command, struct s_rlc_command *response);


/** Function pointer for command matching function */
typedef bool (*t_rlc_server_cmd_match_fn)(void *userdata, const char command[RLC_COMMAND_LENGTH]);


/** Handler for a server command */
struct s_rlc_server_cmd_handler
{
  char command[RLC_COMMAND_LENGTH];             /*!< The command (eg IDN) */
  unsigned command_int;                         /*!< The command converted to an integer */
  t_rlc_server_cmd_match_fn cmd_match_fn;       /*!< Function pointer to check for a match. If NULL, use command/command_int */

  bool run_unselected;                          /*!< Command can run even if unit is unselected (eg STP, Sxx) */
  t_rlc_server_cmd_handler_fn cmd_handler_fn;   /*!< Function pointer to handle the command */
  void *userdata;                               /*!< User data pointer for the command handler function */
};


/** Rinstrum legacy comms server structure */
struct s_rlc_server
{
  struct s_rlc_server_cmd_handler *cmd_handlers;  /*!< List of command handlers */
  size_t num_cmd_handlers;                        /*!< Number of command handlers */
  size_t max_cmd_handlers;                        /*!< Number of command handlers */
  struct s_rlc_command *command;                  /*!< Command parsed from incoming data */
  struct s_rlc_command *response;                 /*!< Response filled out by the appropriate command handler */

  bool unit_selected;                             /*!< The unit has been selected by a Sxx command */
  bool reply_active;                              /*!< Replies should be sent */
  unsigned unit_address;

  bool streaming_enabled;
  unsigned streaming_count;
};


/** Handle a rinstrum legacy comms server */
enum e_rc_cli_result rlc_server_handler(struct s_rlc_server *server,
                                        const char *input, size_t input_length,
                                        char *output, size_t output_length);


/** Create a rinstrum legacy comms server */
enum e_rc_cli_result rlc_server_create(struct s_rlc_server **server, size_t max_cmd_handlers, size_t max_parameters, size_t param_buffer_length);


/** Destroy a rinstrum legacy comms server */
enum e_rc_cli_result rlc_server_destroy(struct s_rlc_server *server);


#endif /* RLC_SERVER_HANDLE_H_ */
