/*
 * rlc_server_register.c
 *
 *  Created on: 14 Jan. 2019
 *      Author: ian
 */
#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <rinlegacycmd/rlc_parameter.h>
#include <rinlegacycmd/rlc_server_handle.h>

#include <rinlegacycmd/rlc_server_register.h>


/** Convert a command into an integer */
enum e_rc_cli_result rlc_command_to_int(const char command[RLC_COMMAND_LENGTH], unsigned *command_int)
{
  size_t i;
  unsigned cmdint;

  for (i = 0, cmdint = 0; i < RLC_COMMAND_LENGTH; i++)
  {
    cmdint += toupper(command[i]) - 'A';
    cmdint *= 26;
  }
  *command_int = cmdint;
  return rcr_success;
}


static enum e_rc_cli_result rlc_internal_server_register_command(struct s_rlc_server *server,
                                                                 const char command[RLC_COMMAND_LENGTH], t_rlc_server_cmd_match_fn cmd_match_fn,
                                                                 t_rlc_server_cmd_handler_fn cmd_handler_fn, void *userdata,
                                                                 bool run_unselected)
{
  if (server->num_cmd_handlers >= server->max_cmd_handlers)
    return rcr_listfull;

  server->cmd_handlers[server->num_cmd_handlers].cmd_handler_fn = cmd_handler_fn;
  server->cmd_handlers[server->num_cmd_handlers].userdata = userdata;
  server->cmd_handlers[server->num_cmd_handlers].run_unselected = run_unselected;
  if (NULL == cmd_match_fn)
  {
    memcpy(server->cmd_handlers[server->num_cmd_handlers].command, command, RLC_COMMAND_LENGTH);
    rlc_command_to_int(command, &server->cmd_handlers[server->num_cmd_handlers].command_int);
  }
  else
  {
    server->cmd_handlers[server->num_cmd_handlers].cmd_match_fn = cmd_match_fn;
  }
  server->num_cmd_handlers++;
  return rcr_success;
}


/** Register a command handler using a match function */
enum e_rc_cli_result rlc_server_register_command_matchfn(struct s_rlc_server *server,
                                                         t_rlc_server_cmd_match_fn cmd_match_fn, t_rlc_server_cmd_handler_fn cmd_handler_fn, void *userdata,
                                                         bool run_unselected)
{
  return rlc_internal_server_register_command(server, "", cmd_match_fn, cmd_handler_fn, userdata, run_unselected);
}


/** Register a command handler using a command */
enum e_rc_cli_result rlc_server_register_command(struct s_rlc_server *server,
                                                 const char command[RLC_COMMAND_LENGTH], t_rlc_server_cmd_handler_fn cmd_handler_fn, void *userdata,
                                                 bool run_unselected)
{
  return rlc_internal_server_register_command(server, command, NULL, cmd_handler_fn, userdata, run_unselected);
}
