/*
 * rlc_server_simple.h
 *
 *  Created on: 21 Jan. 2019
 *      Author: ian
 */

#ifndef RLC_SERVER_SIMPLE_H_
#define RLC_SERVER_SIMPLE_H_

#include <stddef.h>
#include <rincmd/rc_frame.h>
#include <rinlegacycmd/rlc_server_handle.h>


/** A client for the simple rinstrum legacy command server */
struct s_rlc_server_simple_client
{
  int client_sock_fd;
  struct s_rlc_server *server;
  struct s_rc_frame **framers;
  size_t num_framers;

  char *inbuffer;
  size_t inbuffer_posn;
  size_t inbuffer_maxsize;

  char *outbuffer;
  size_t outbuffer_maxsize;
};


/** A simple rinstrum legacy command server over TCP implementation */
struct s_rlc_server_simple
{
  char *interface;                              /*!< The interface to bind to: 0.0.0.0 for all interfaces, non-ip for unix domain socket */
  int tcp_port;                                 /*!< The TCP port to bind to when using TCP socket */
  int listen_sock_fd;                           /*!< The listening socket file descriptor */

  struct s_rlc_server_simple_client *clients;   /*!< The list of clients */
  size_t max_clients;                           /*!< The number of allocated clients */
};


/** Run the the rinstrum legacy client simple server */
enum e_rc_cli_result rlc_server_simple_handler(struct s_rlc_server_simple *server, const struct timespec *timeout);


/**
 * Create and run the simple server
 *
 * @param server is set to point to an allocated server on success
 * @param interface is the interface to bind to: 0.0.0.0 for all interfaces, non-ip for unix domain socket
 * @param tcp_port is the port to listen on for incoming connections
 * @param max_clients is the maximum number of clients to handle simultaneously
 * @param client_data_buffer_size is the size of the incoming and outgoing data buffers. Must be able to accommodate the largest command/response
 * @param max_cmd_handlers is the maximum number of command handlers the server will support
 * @param max_parameters is the maximum number of parameters for a command/response
 * @param param_buffer_length is the maximum length of buffer parameters
 */
enum e_rc_cli_result rlc_server_simple_create(struct s_rlc_server_simple **server, const char *interface, int tcp_port, size_t max_clients,
                                              size_t client_data_buffer_size, size_t max_cmd_handlers, size_t max_parameters, size_t param_buffer_length);


/** Clean up the rinstrum legacy client simple server */
enum e_rc_cli_result rlc_server_simple_destroy(struct s_rlc_server_simple *server);


/** Register a command handler using a match function for a simple server */
enum e_rc_cli_result rlc_server_simple_register_command_matchfn(struct s_rlc_server_simple *server,
                                                                t_rlc_server_cmd_match_fn cmd_match_fn, t_rlc_server_cmd_handler_fn cmd_handler_fn, void *userdata,
                                                                bool run_unselected);


/** Register a command handler using a command for a simple server */
enum e_rc_cli_result rlc_server_simple_register_command(struct s_rlc_server_simple *server,
                                                        const char command[RLC_COMMAND_LENGTH], t_rlc_server_cmd_handler_fn cmd_handler_fn, void *userdata,
                                                        bool run_unselected);


#endif /* RLC_SERVER_SIMPLE_H_ */
