/*
 * rlc_server_sxx.c
 *
 *  Created on: 14 Jan. 2019
 *      Author: ian
 */
#include <ctype.h>
#include <rinlegacycmd/rlc_server_reply.h>
#include <rinlegacycmd/rlc_server_handle.h>

#include <rinlegacycmd/rlc_server_sxx.h>


/** Handle STP - stop MSV streaming command */
void rlc_server_handle_STP(void *userdata,
                           const struct s_rlc_command *command, struct s_rlc_command *response)
{
  struct s_rlc_server *server = userdata;

  /* Disable streaming */
  server->streaming_count = 1;

  /* No reply to this command */
  rlc_server_command_reply(response, false, RLC_REPLY_SUCCESS);
}


/** Match command for Snn - addressing commands */
bool rlc_server_match_Snn(void *userdata, const char command[RLC_COMMAND_LENGTH])
{
  return ('S' == command[0] && isdigit(command[1]) && isdigit(command[2]));
}


/** Handle Snn - addressing commands */
void rlc_server_handle_Snn(void *userdata,
                           const struct s_rlc_command *command, struct s_rlc_command *response)
{
  struct s_rlc_server *server = userdata;
  unsigned addr;

  /* If streaming, Sxx does nothing */
  if (server->streaming_enabled)
  {
    /* No reply */
    rlc_server_command_reply(response, false, RLC_REPLY_SUCCESS);
    return;
  }

  /* Check address in command, and convert to integer */
  if (!isdigit(command->command[1]) || !isdigit(command->command[2]))
  {
    rlc_server_command_reply(response, true, RLC_REPLY_ERROR);
    return;
  }
  addr = (10 * (command->command[1] - '0')) + (command->command[2] - '0');

#if 0
  /* Check passcode */
  if (0 == db_app_user.full_passcode)
  {
    comms_ctrl->passcode_ok = true;
  }
#endif

  /* Work out what to do based on address in Sxx command */
  switch (addr)
  {
    /* De-select all units */
    case 96:
      server->reply_active = false;
      server->unit_selected = false;
      /* comms_ctrl->passcode_ok = (0 == db_app_user.full_passcode); */
      break;

    /* Select all units, but don't reply to messages */
    case 97:
    case 98:
      server->reply_active = false;
      server->unit_selected = true;
      break;

    /* Select all units, and reply to messages */
    case 99:
      server->reply_active = true;
      server->unit_selected = true;
      break;

    /* Select if our address matches */
    default:
#if 0
      if (addr == db_app_user.address)
      {
        server->reply_active = true;
        server->unit_selected = true;
      }
      else
      {
        server->reply_active = false;
        server->unit_selected = false;
        server->passcode_ok = (0 == db_app_user.full_passcode);
      }
#endif
      break;
  }

  /* No reply to Sxx commands */
  rlc_server_command_reply(response, false, RLC_REPLY_SUCCESS);
}
