/*
 * rc_client_sync.c
 *
 *  Created on: Dec 15, 2017
 *      Author: ian
 */
#include <stdint.h>
#include <stddef.h>
#include <stdio.h>
#include <rincmd/rc_public.h>
#include <rincmd/rc_cli_comms.h>
#include <rincmd/rc_cli_sync.h>
#include <rincmd/rc_cli_sync_stream.h>
#include <rincmd/rc_cli_error.h>
#include <rinlegacycmd/rlc_cli_sync.h>
#include <rinlegacycmd/rlc_command_print.h>

#include "rlc_client_sync.h"


/** Get a timeout pointer */
struct timespec *get_timeout_ptr(struct timespec *to, int timeoutms)
{
  struct timespec *toptr;

  if (timeoutms < 0)
  {
    toptr = NULL;
  }
  else
  {
    to->tv_nsec = (timeoutms % 1000) * 1000000;
    to->tv_sec = timeoutms / 1000;
    toptr = to;
  }
  return toptr;
}


#if 0
static enum e_rc_cli_result sync_client_streaming(struct s_rc_sync_client *rsc, struct timespec *toptr, unsigned repeats, bool *quit)
{
  /*                                           Disp    Gross   Net     Status  Error */
  const int regs[RC_STREAM_DATA_REGISTERS] = { 0x0025, 0x0026, 0x0027, 0x0021, 0x0023 };
  size_t i;
  int64_t regvalues[RC_STREAM_DATA_REGISTERS] = { 0, 0, 0, 0, 0 };
  enum e_rc_cli_result result;

  /* Setup streaming */
  result = rc_sync_client_streaming_setup(rsc, 0x0040, RC_STREAM_MODE_AUTO, regs, toptr);
  if (rcr_success != result)
    return result;

  /* Read responses */
  for (i = 0; repeats == 0 || i < repeats; i++)
  {
    result = rc_sync_client_streaming_read(rsc, 0x0040, regvalues, toptr);
    if (rcr_success != result)
      return result;
    printf("Streaming result %u: %lld, %lld, %lld, %lld, %lld\n", (unsigned) i,
           (long long) regvalues[0], (long long) regvalues[1], (long long) regvalues[2], (long long) regvalues[3], (long long) regvalues[4]);

    if (NULL != quit && *quit)
    {
      printf("Shutdown requested\n");
      break;
    }
  }

  /* Teardown the streaming */
  return rc_sync_client_streaming_teardown(rsc, 0x0040, toptr);
}
#endif


/** Rinstrum legacy client synchronous client example */
enum e_rc_cli_result rlc_sync_client_example(struct s_rc_cli_comms *rcc,
                                             const char query_command[RLC_COMMAND_LENGTH],
                                             bool usecrc, bool streaming,
                                             unsigned repeats, int timeoutms, bool *quit)
{
  enum e_rc_cli_result result;
  struct s_rlc_command *rlc_response = NULL;
  struct s_rlc_sync_client *rsc = NULL;
  struct timespec to;
  struct timespec *toptr;
  size_t i;

  toptr = get_timeout_ptr(&to, timeoutms);

  result = rlc_command_create_prealloc(&rlc_response, "", false, 20, 100, false);
  if (rcr_success != result)
  {
    printf("rlc_command_create_prealloc() failed with %s\n", rc_cli_error_strings(result));
    goto sc_fail;
  }

  /* Create client and connect */
  result = rlc_sync_client_init(&rsc, rcc, usecrc);
  if (rcr_success != result)
  {
    printf("rlc_sync_client_init() failed with %s\n", rc_cli_error_strings(result));
    goto sc_fail;
  }

  result = rlc_sync_client_connect(rsc, toptr);
  if (rcr_success != result)
  {
    printf("rc_sync_client_connect() failed with %s\n", rc_cli_error_strings(result));
    goto sc_fail;
  }

  /* Select all units, no response expected */
  result = rlc_sync_client_command_request_va(rsc, toptr, NULL, "S99", true, "");
  if (rcr_success != result)
  {
    printf("rlc_sync_client_command_request_va() select unit S99 failed with %s\n", rc_cli_error_strings(result));
    goto sc_fail;
  }

  if (streaming)
  {
    printf("sync_client_streaming() not yet implemented\n");
    goto sc_fail;
/*
    result = sync_client_streaming(rsc, toptr, repeats, quit);
    if (rcr_success != result)
    {
      printf("sync_client_streaming() failed with %s\n", rc_cli_error_strings(result));
      goto sc_fail;
    }
*/
  }
  else
  {
    for (i = 0; repeats == 0 || i < repeats; i++)
    {
      result = rlc_sync_client_command_request_va(rsc, toptr, rlc_response, query_command, false, "");
      if (rcr_success != result)
      {
        printf("rlc_sync_client_command_request() failed with %s\n", rc_cli_error_strings(result));
        goto sc_fail;
      }

      rlc_command_fprintf(stdout, rlc_response);

      if (NULL != quit && *quit)
      {
        printf("Shutdown requested\n");
        break;
      }
    }
  }

  /* Disconnect from the server */
  (void) rlc_sync_client_disconnect(rsc);

  result = rcr_success;

sc_fail:
  if (NULL != rsc)
  {
    (void) rlc_sync_client_destroy(rsc);
  }
  if (NULL != rlc_response)
  {
    (void) rlc_command_destroy(rlc_response);
  }
  return result;
}
