/*
 * test_rlc_assemble.c
 *
 *  Created on: 1 Jan. 2019
 *      Author: ian
 */
#include <string.h>
#include <cutest/CuTest.h>
#include <rincmd/rc_cli_error.h>
#include <rinlegacycmd/rlc_assemble.h>

#include "test_rlc_assemble.h"


/** Test converting a rinstrum legacy command or query request into a string suitable for sending to the indicator */
static void test_rlc_command_request_to_string(CuTest *tc)
{
  #define TEST_NUM_PARAMS 3
  enum e_rc_cli_result result;
  struct s_rlc_command *rlc_cmd = NULL;
  const char buf_param_data[] = "some parameter test data";
  #define OUTBUFFER_LEN 100
  char outbuffer[OUTBUFFER_LEN];

  /* Create command */
  result = rlc_command_create_prealloc(&rlc_cmd, "MSV", false, TEST_NUM_PARAMS, 20, true);
  CuAssertIntEquals(tc, rcr_success, result);

  /* Setup parameters */
  result = rlc_command_parameter_buffer_set(rlc_cmd, 0, buf_param_data, strlen(buf_param_data));
  CuAssertIntEquals(tc, rcr_success, result);
  result = rlc_command_parameter_unused_set(rlc_cmd, 1);
  CuAssertIntEquals(tc, rcr_success, result);
  result = rlc_command_parameter_integer_set(rlc_cmd, 2, 1234567890);
  CuAssertIntEquals(tc, rcr_success, result);
  rlc_cmd->num_parameters = 3;

  /* Convert command request to string */
  result = rlc_command_request_to_string(outbuffer, OUTBUFFER_LEN, rlc_cmd);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertStrEquals(tc, "MSV?\"some parameter test data\",,1234567890", outbuffer);

  /* Clean up */
  result = rlc_command_destroy(rlc_cmd);
  CuAssertIntEquals(tc, rcr_success, result);
}


/** Test converting a rinstrum legacy response into a string suitable for the indicator to reply with */
static void test_rlc_command_response_to_string(CuTest *tc)
{
  #define TEST_NUM_PARAMS 3
  enum e_rc_cli_result result;
  struct s_rlc_command *rlc_cmd = NULL;
  const char buf_param_data[] = "some parameter test data";
  #define OUTBUFFER_LEN 100
  char outbuffer[OUTBUFFER_LEN];

  /* Create command */
  result = rlc_command_create_prealloc(&rlc_cmd, "MSV", false, TEST_NUM_PARAMS, 20, true);
  CuAssertIntEquals(tc, rcr_success, result);

  /* Setup parameters */
  result = rlc_command_parameter_buffer_set(rlc_cmd, 0, buf_param_data, strlen(buf_param_data));
  CuAssertIntEquals(tc, rcr_success, result);
  result = rlc_command_parameter_unused_set(rlc_cmd, 1);
  CuAssertIntEquals(tc, rcr_success, result);
  result = rlc_command_parameter_integer_set(rlc_cmd, 2, 1234567890);
  CuAssertIntEquals(tc, rcr_success, result);
  rlc_cmd->num_parameters = 3;

  /* Convert command response to string */
  result = rlc_command_response_to_string(outbuffer, OUTBUFFER_LEN, rlc_cmd);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertStrEquals(tc, "\"some parameter test data\",,1234567890", outbuffer);

  /* Clean up */
  result = rlc_command_destroy(rlc_cmd);
  CuAssertIntEquals(tc, rcr_success, result);
}


CuSuite *CuGetSuite_rlc_assemble(void)
{
  CuSuite *suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_rlc_command_request_to_string);
  SUITE_ADD_TEST(suite, test_rlc_command_response_to_string);

  return suite;
}
