/*
 * test_rlc_assemble_va.c
 *
 *  Created on: 4 Jan. 2019
 *      Author: ian
 */
#include <string.h>
#include <cutest/CuTest.h>
#include <rincmd/rc_cli_error.h>
#include <rinlegacycmd/rlc_assemble_va.h>

#include "test_rlc_assemble_va.h"


/** Test converting a rinstrum legacy command or query request into a string suitable for sending to the indicator */
static void test_rlc_format_request(CuTest *tc)
{
  enum e_rc_cli_result result;
  const char buf_param_data[] = "some parameter test data";
  #define OUTBUFFER_LEN 100
  char outbuffer[OUTBUFFER_LEN];

  /* Convert command request to string (variadic method) */
  result = rlc_format_request(outbuffer, OUTBUFFER_LEN, "MSV", false,
                              "snib",
                              buf_param_data,
                              (int64_t) 1234567890,
                              buf_param_data, (size_t) strlen(buf_param_data));
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertStrEquals(tc, "MSV?\"some parameter test data\",,1234567890,\"some parameter test data\"", outbuffer);
}


/** Test converting a rinstrum legacy response into a string suitable for the indicator to reply with */
static void test_rlc_format_response(CuTest *tc)
{
  enum e_rc_cli_result result;
  const char buf_param_data[] = "some parameter test data";
  #define OUTBUFFER_LEN 100
  char outbuffer[OUTBUFFER_LEN];

  /* Convert command response to string (variadic method) */
  result = rlc_format_response(outbuffer, OUTBUFFER_LEN, "MSV",
                               "snib",
                               buf_param_data,
                               (int64_t) 1234567890,
                               buf_param_data, (size_t) strlen(buf_param_data));
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertStrEquals(tc, "\"some parameter test data\",,1234567890,\"some parameter test data\"", outbuffer);
}


CuSuite *CuGetSuite_rlc_assemble_va(void)
{
  CuSuite *suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_rlc_format_request);
  SUITE_ADD_TEST(suite, test_rlc_format_response);

  return suite;
}
