/*
 * test_rlc_parameter.c
 *
 *  Created on: 4 Jan. 2019
 *      Author: ian
 */
#include <string.h>
#include <cutest/CuTest.h>
#include <rincmd/rc_cli_error.h>
#include <rinlegacycmd/rlc_parameter.h>

#include "test_rlc_parameter_escape.h"


/** Test creating a rinstrum legacy comms command, pre-allocating parameters and parameter strings */
static void test_rlc_command_create_prealloc(CuTest *tc)
{
  #define TEST_NUM_PARAMS_PREALLOC 3
  enum e_rc_cli_result result;
  struct s_rlc_command *rlc_cmd = NULL;
  size_t i;

  /* Create command */
  result = rlc_command_create_prealloc(&rlc_cmd, "MSV", false, TEST_NUM_PARAMS_PREALLOC, 20, true);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertPtrNotNull(tc, rlc_cmd);

  /* Check members */
  CuAssertTrue(tc, !rlc_cmd->command_not_query);
  CuAssertPtrNotNull(tc, rlc_cmd->parameters);
  CuAssertIntEquals(tc, 0, rlc_cmd->num_parameters);
  CuAssertIntEquals(tc, TEST_NUM_PARAMS_PREALLOC, rlc_cmd->max_parameters);
  for (i = 0; i < TEST_NUM_PARAMS_PREALLOC; i++)
  {
    CuAssertIntEquals(tc, rpt_unused, rlc_cmd->parameters[i].param_type);
    CuAssertTrue(tc, rlc_cmd->parameters[i].can_reallocate);
    CuAssertPtrNotNull(tc, rlc_cmd->parameters[i].buffer);
    CuAssertIntEquals(tc, 0, rlc_cmd->parameters[i].buffer_length);
    CuAssertIntEquals(tc, 20, rlc_cmd->parameters[i].buffer_maxlength);
    CuAssertIntEquals(tc, 0, rlc_cmd->parameters[i].int_value);
  }

  /* Clean up */
  result = rlc_command_destroy(rlc_cmd);
  CuAssertIntEquals(tc, rcr_success, result);
}


/** Test setting parameters within a rinstrum legacy comms command */
static void test_rlc_command_parameters(CuTest *tc)
{
  #define TEST_NUM_PARAMS 20
  #define TEST_PARAM_BUF_LEN 100
  enum e_rc_cli_result result;
  struct s_rlc_command *rlc_cmd = NULL;
  enum e_rlc_parameter_type param_type;
  int64_t int_value;
  const char *buffer;
  size_t buffer_length;
  size_t param;

  /* Create command */
  result = rlc_command_create_prealloc(&rlc_cmd, "MSV", false, TEST_NUM_PARAMS, TEST_PARAM_BUF_LEN, true);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertPtrNotNull(tc, rlc_cmd);

  /* Set members */
  param = 0;
  result = rlc_command_parameter_unused_set(rlc_cmd, param++);
  CuAssertIntEquals(tc, rcr_success, result);
  result = rlc_command_parameter_error_set(rlc_cmd, param++);
  CuAssertIntEquals(tc, rcr_success, result);
  result = rlc_command_parameter_integer_set(rlc_cmd, param++, 123456789);
  CuAssertIntEquals(tc, rcr_success, result);
  result = rlc_command_parameter_buffer_unescape_set(rlc_cmd, param++, "abcdef\\008", 10);
  CuAssertIntEquals(tc, rcr_success, result);
  result = rlc_command_parameter_buffer_set(rlc_cmd, param++, "abcdef\\008", 10);
  CuAssertIntEquals(tc, rcr_success, result);
  result = rlc_command_parameter_string_set(rlc_cmd, param++, "blahblah");
  CuAssertIntEquals(tc, rcr_success, result);
  rlc_cmd->num_parameters = param;

  /* Check members */
  param = 0;
  result = rlc_command_parameter_type_get(rlc_cmd, param++, &param_type);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertIntEquals(tc, rpt_unused, param_type);

  result = rlc_command_parameter_type_get(rlc_cmd, param++, &param_type);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertIntEquals(tc, rpt_error, param_type);

  result = rlc_command_parameter_type_get(rlc_cmd, param, &param_type);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertIntEquals(tc, rpt_integer, param_type);
  result = rlc_command_parameter_integer_get(rlc_cmd, param, &int_value);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertIntEquals(tc, 123456789, int_value);

  param++;
  result = rlc_command_parameter_type_get(rlc_cmd, param, &param_type);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertIntEquals(tc, rpt_buffer, param_type);
  result = rlc_command_parameter_buffer_get(rlc_cmd, param, &buffer, &buffer_length);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertIntEquals(tc, 7, buffer_length);
  CuAssertIntEquals(tc, 0, memcmp(buffer, "abcdef\x08", 7));

  param++;
  result = rlc_command_parameter_type_get(rlc_cmd, param, &param_type);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertIntEquals(tc, rpt_buffer, param_type);
  result = rlc_command_parameter_buffer_get(rlc_cmd, param, &buffer, &buffer_length);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertIntEquals(tc, 10, buffer_length);
  CuAssertIntEquals(tc, 0, memcmp(buffer, "abcdef\\008", 10));

  param++;
  result = rlc_command_parameter_type_get(rlc_cmd, param, &param_type);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertIntEquals(tc, rpt_buffer, param_type);
  result = rlc_command_parameter_buffer_get(rlc_cmd, param, &buffer, &buffer_length);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertIntEquals(tc, 8, buffer_length);
  CuAssertIntEquals(tc, 0, memcmp(buffer, "blahblah", 8));

  /* Clean up */
  result = rlc_command_destroy(rlc_cmd);
  CuAssertIntEquals(tc, rcr_success, result);
}


/** Test checking parameter limits within a rinstrum legacy comms command */
static void test_rlc_command_parameters_limits(CuTest *tc)
{
  #define TEST_NUM_PARAMS 20
  #define TEST_PARAM_BUF_LEN 100
  enum e_rc_cli_result result;
  struct s_rlc_command *rlc_cmd = NULL;
  enum e_rlc_parameter_type param_type;
  int64_t int_value;
  size_t param;
  const int64_t expected_int_value = 123456789;
  const char expected_string_value[] = "abcdefghijklmnopqrstuvwxyz";
  #define RESULT_STRING_LEN 100
  char result_string[RESULT_STRING_LEN];

  /* Create command */
  result = rlc_command_create_prealloc(&rlc_cmd, "MSV", false, TEST_NUM_PARAMS, TEST_PARAM_BUF_LEN, true);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertPtrNotNull(tc, rlc_cmd);

  /* Set members */
  param = 0;
  result = rlc_command_parameter_integer_set(rlc_cmd, param++, expected_int_value);
  CuAssertIntEquals(tc, rcr_success, result);
  result = rlc_command_parameter_string_set(rlc_cmd, param++, expected_string_value);
  CuAssertIntEquals(tc, rcr_success, result);
  rlc_cmd->num_parameters = param;

  /* Check integer parameter limits */
  param = 0;
  result = rlc_command_parameter_type_get(rlc_cmd, param, &param_type);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertIntEquals(tc, rpt_integer, param_type);
  result = rlc_command_parameter_integer_check_get(rlc_cmd, param, &int_value, expected_int_value, expected_int_value);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertIntEquals(tc, expected_int_value, int_value);
  result = rlc_command_parameter_integer_check_get(rlc_cmd, param, &int_value, expected_int_value + 1, expected_int_value);
  CuAssertIntEquals(tc, rcr_paramfail, result);
  result = rlc_command_parameter_integer_check_get(rlc_cmd, param, &int_value, expected_int_value, expected_int_value - 1);
  CuAssertIntEquals(tc, rcr_paramfail, result);

  /* Check string parameter limits */
  param = 1;
  result = rlc_command_parameter_type_get(rlc_cmd, param, &param_type);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertIntEquals(tc, rpt_buffer, param_type);
  result = rlc_command_parameter_string_check_copy(rlc_cmd, param, result_string, 0, RESULT_STRING_LEN);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertStrEquals(tc, expected_string_value, result_string);

  /* Too short */
  result = rlc_command_parameter_string_check_copy(rlc_cmd, param, result_string, 0, 25);
  CuAssertIntEquals(tc, rcr_bufferoverflow, result);

  /* Too long */
  result = rlc_command_parameter_string_check_copy(rlc_cmd, param, result_string, 27, RESULT_STRING_LEN);
  CuAssertIntEquals(tc, rcr_paramfail, result);

  /* Valid character set */
  result = rlc_command_parameter_string_check_charset_copy(rlc_cmd, param, result_string, 0, RESULT_STRING_LEN, expected_string_value);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertStrEquals(tc, expected_string_value, result_string);

  /* Invalid character set */
  result = rlc_command_parameter_string_check_charset_copy(rlc_cmd, param, result_string, 0, RESULT_STRING_LEN, "0123456789");
  CuAssertIntEquals(tc, rcr_paramfail, result);

  /* Clean up */
  result = rlc_command_destroy(rlc_cmd);
  CuAssertIntEquals(tc, rcr_success, result);
}


/** Test problem found when using for licensing LRP command */
static void test_rlc_command_parameters_licensing_failure(CuTest *tc)
{
  #define TEST_NUM_PARAMS 20
  #define TEST_PARAM_BUF_LEN 100
  enum e_rc_cli_result result;
  struct s_rlc_command *rlc_cmd = NULL;
  enum e_rlc_parameter_type param_type;

  /** Each C500 licence code is 6 characters in length (cccccc) */
  #define LICENCECODE_LEN_C500 6

  /** Each C500 licence code digit has 26 combinations (A-Z) */
  #define LICENCECODE_DIGIT_COMBINATIONS_C500 "BCDFGHJKLMNPQRSTVWXYZ25789"

  const char expected_string_value[] = "BLQFM5";
  char licence_code[LICENCECODE_LEN_C500 + 1];

  /* Create command */
  result = rlc_command_create_prealloc(&rlc_cmd, "LRP", false, TEST_NUM_PARAMS, TEST_PARAM_BUF_LEN, true);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertPtrNotNull(tc, rlc_cmd);

  /* Set members */
  result = rlc_command_parameter_string_set(rlc_cmd, 0, expected_string_value);
  CuAssertIntEquals(tc, rcr_success, result);
  rlc_cmd->num_parameters = 1;

  /* Check string parameter limits */
  result = rlc_command_parameter_type_get(rlc_cmd, 0, &param_type);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertIntEquals(tc, rpt_buffer, param_type);
  result = rlc_command_parameter_string_check_copy(rlc_cmd, 0, licence_code, 0, LICENCECODE_LEN_C500 + 1);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertStrEquals(tc, expected_string_value, licence_code);
  result = rlc_command_parameter_string_check_charset_copy(rlc_cmd, 0, licence_code, 0, LICENCECODE_LEN_C500 + 1, LICENCECODE_DIGIT_COMBINATIONS_C500);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertStrEquals(tc, expected_string_value, licence_code);

  /* Clean up */
  result = rlc_command_destroy(rlc_cmd);
  CuAssertIntEquals(tc, rcr_success, result);
}


CuSuite *CuGetSuite_rlc_parameter(void)
{
  CuSuite *suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_rlc_command_create_prealloc);
  SUITE_ADD_TEST(suite, test_rlc_command_parameters);
  SUITE_ADD_TEST(suite, test_rlc_command_parameters_limits);
  SUITE_ADD_TEST(suite, test_rlc_command_parameters_licensing_failure);

  return suite;
}
