/*
 * test_rlc_server_handle.c
 *
 *  Created on: 14 Jan. 2019
 *      Author: ian
 */
#include <string.h>
#include <cutest/CuTest.h>
#include <rincmd/rc_cli_error.h>
#include <rinlegacycmd/rlc_server_handle.h>
#include <rinlegacycmd/rlc_server_reply.h>
#include <rinlegacycmd/rlc_server_register.h>

#include "test_rlc_server_handle.h"


/** Test the rinstrum legacy comms server and select command Snn */
static void test_rlc_server_handler_Snn(CuTest *tc)
{
  struct s_rlc_server *server = NULL;
  enum e_rc_cli_result result;
  const char *cmd;
  char response[100];

  /* Create a rinstrum legacy comms server */
  result = rlc_server_create(&server, 10, 10, 100);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertPtrNotNull(tc, server);

  /* Parse an unknown command */
  cmd = "AAA";
  result = rlc_server_handler(server, cmd, strlen(cmd), response, 100);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertStrEquals(tc, "", response);

  /* Parse a known select command - S96 - de-select all */
  server->reply_active = true;
  server->unit_selected = true;
  cmd = "S96";
  result = rlc_server_handler(server, cmd, strlen(cmd), response, 100);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertTrue(tc, !server->reply_active);
  CuAssertTrue(tc, !server->unit_selected);
  CuAssertStrEquals(tc, "", response);

  /* Parse a known select command - S97 - select all, but no reply */
  server->reply_active = true;
  server->unit_selected = false;
  cmd = "S97";
  result = rlc_server_handler(server, cmd, strlen(cmd), response, 100);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertTrue(tc, !server->reply_active);
  CuAssertTrue(tc, server->unit_selected);
  CuAssertStrEquals(tc, "", response);

  /* Parse a known select command - S98 - select all, but no reply */
  server->reply_active = true;
  server->unit_selected = false;
  cmd = "S98";
  result = rlc_server_handler(server, cmd, strlen(cmd), response, 100);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertTrue(tc, !server->reply_active);
  CuAssertTrue(tc, server->unit_selected);
  CuAssertStrEquals(tc, "", response);

  /* Parse a known select command - S99 - select all, with reply */
  server->reply_active = false;
  server->unit_selected = false;
  cmd = "S99";
  result = rlc_server_handler(server, cmd, strlen(cmd), response, 100);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertTrue(tc, server->reply_active);
  CuAssertTrue(tc, server->unit_selected);
  CuAssertStrEquals(tc, "", response);

  /* Destroy a rinstrum legacy comms server */
  result = rlc_server_destroy(server);
  CuAssertIntEquals(tc, rcr_success, result);
}


/** Test the rinstrum legacy comms server STP command */
static void test_rlc_server_handler_STP(CuTest *tc)
{
  struct s_rlc_server *server = NULL;
  enum e_rc_cli_result result;
  const char *cmd;
  char response[100];

  /* Create a rinstrum legacy comms server */
  result = rlc_server_create(&server, 10, 10, 100);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertPtrNotNull(tc, server);

  /* Parse a known select command - S99 - select all, with reply */
  cmd = "S99";
  result = rlc_server_handler(server, cmd, strlen(cmd), response, 100);
  CuAssertIntEquals(tc, rcr_success, result);

  /* Now that the unit is selected, run the STP command */
  server->streaming_count = 100;
  cmd = "STP";
  result = rlc_server_handler(server, cmd, strlen(cmd), response, 100);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertStrEquals(tc, "", response);
  CuAssertIntEquals(tc, 1, server->streaming_count);

  /* Destroy a rinstrum legacy comms server */
  result = rlc_server_destroy(server);
  CuAssertIntEquals(tc, rcr_success, result);
}


/** Handle test ZZZ command */
void test_handler_ZZZ(void *userdata,
                      const struct s_rlc_command *command, struct s_rlc_command *response)
{
  CuTest *tc = userdata;
  int64_t value;
  enum e_rlc_parameter_type param_type;
  char str[100];

  if (command->command_not_query)
  {
    CuAssertIntEquals(tc, 2, command->num_parameters);
    rlc_command_parameter_integer_get(command, 0, &value);
    CuAssertIntEquals(tc, 1234, value);
    rlc_command_parameter_string_check_copy(command, 1, str, 0, 100);
    CuAssertStrEquals(tc, "asdf", str);
    rlc_server_command_reply(response, true, RLC_REPLY_SUCCESS);
  }
  else
  {
    CuAssertIntEquals(tc, 3, command->num_parameters);
    rlc_command_parameter_type_get(command, 0, &param_type);
    CuAssertIntEquals(tc, rpt_unused, param_type);
    rlc_command_parameter_integer_get(command, 1, &value);
    CuAssertIntEquals(tc, 5678, value);
    rlc_command_parameter_string_check_copy(command, 2, str, 0, 100);
    CuAssertStrEquals(tc, "fdsa", str);

    response->num_parameters = 3;
    rlc_command_parameter_integer_set(response, 0, 4321);
    rlc_command_parameter_string_set(response, 1, "blah");
    rlc_command_parameter_string_set(response, 2, "fdsa");
  }
}


/** Test the rinstrum legacy comms server parsing known and unknown commands */
static void test_rlc_server_handler_commands(CuTest *tc)
{
  struct s_rlc_server *server = NULL;
  enum e_rc_cli_result result;
  const char *cmd;
  char response[100];

  /* Create a rinstrum legacy comms server */
  result = rlc_server_create(&server, 10, 10, 100);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertPtrNotNull(tc, server);

  /* Register ZZZ commands */
  result = rlc_server_register_command(server, "ZZZ", test_handler_ZZZ, tc, false);
  CuAssertIntEquals(tc, rcr_success, result);

  /* Select unit */
  cmd = "S99";
  result = rlc_server_handler(server, cmd, strlen(cmd), response, 100);
  CuAssertIntEquals(tc, rcr_success, result);

  /* Now that the unit is selected, parse an unknown command */
  cmd = "AAA";
  result = rlc_server_handler(server, cmd, strlen(cmd), response, 100);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertStrEquals(tc, "?", response);

  /* Now that the unit is selected, parse an known command */
  cmd = "ZZZ1234,\"asdf\"";
  result = rlc_server_handler(server, cmd, strlen(cmd), response, 100);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertStrEquals(tc, "0", response);

  /* Now that the unit is selected, parse an known command */
  cmd = "ZZZ?,5678,\"fdsa\"";
  result = rlc_server_handler(server, cmd, strlen(cmd), response, 100);
  CuAssertIntEquals(tc, rcr_success, result);
  CuAssertStrEquals(tc, "4321,\"blah\",\"fdsa\"", response);

  /* Destroy a rinstrum legacy comms server */
  result = rlc_server_destroy(server);
  CuAssertIntEquals(tc, rcr_success, result);
}


CuSuite *CuGetSuite_rlc_server_handle(void)
{
  CuSuite *suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_rlc_server_handler_Snn);
  SUITE_ADD_TEST(suite, test_rlc_server_handler_STP);
  SUITE_ADD_TEST(suite, test_rlc_server_handler_commands);

  return suite;
}
