/*
 * rl_generate_c500.c
 *
 *  Created on: Nov 4, 2016
 *      Author: ian
 */
#include <stdio.h>
#include <stdint.h>
#include <stdbool.h>
#include <limits.h>
#include <string.h>
#define LTC_SHA256
#include <tomcrypt.h>
#include <tommath.h>

#include <rinlicence/rl_licence.h>
#include "rl_generate_c500.h"


#if 0
static void rl_print_bignum(const char *desc, mp_int *num)
{
  int size;
  char *buffer;

  if (mp_radix_size(num, 10, &size) != MP_OKAY)
    return;
  buffer = malloc(size);
  if (NULL == buffer)
    return;
  if (mp_toradix(num, buffer, 10) == MP_OKAY)
    printf("%s: %s\n", desc, buffer);
  free(buffer);
}
#endif


/** Licence code is hash % (LICENCECODE_DIGIT_COMBINATIONS ^ LICENCECODE_LEN) */
static enum e_rl_result rl_hash_to_passcode(const uint8_t *sha256_hash, size_t sha256_hash_length,
                                            size_t licence_code_length, size_t num_licence_code_digit_combinations,
                                            uint32_t *licence_code)
{
  mp_int hash_num;
  mp_int lc_combinations;
  mp_int lc_divisor;
  mp_int big_passcode;
  mp_digit lc_len;
  int res;
  enum e_rl_result result;

  if (NULL == sha256_hash || NULL == licence_code)
    return rlr_paramfail;

  /* Initialise some big integers */
  res = mp_init_multi(&hash_num, &lc_divisor, &big_passcode, &lc_combinations, NULL);
  if (MP_OKAY != res)
  {
    return rlr_mathfail;
  }

  /* Convert hash into a big integer */
  res = mp_read_unsigned_bin(&hash_num, sha256_hash, sha256_hash_length);
  if (MP_OKAY != res)
  {
    result = rlr_mathfail;
    goto rhtp_fail;
  }
  /* rl_print_bignum("hash", &hash_num); */

  /* Range of licence code is 0 .. (LICENCECODE_DIGIT_COMBINATIONS ^ LICENCECODE_LEN) - 1 */
  if (mp_set_int(&lc_combinations, num_licence_code_digit_combinations) != MP_OKAY)
  {
    result = rlr_mathfail;
    goto rhtp_fail;
  }
  lc_len = licence_code_length;
  /* rl_print_bignum("lc comb", &lc_combinations); */

  /* c = a**b */
  res = mp_expt_d(&lc_combinations, lc_len, &lc_divisor);
  if (MP_OKAY != res)
  {
    result = rlr_mathfail;
    goto rhtp_fail;
  }
  /* rl_print_bignum("lc divisor", &lc_divisor); */

  /* Licence code is hash % combinations */
  res = mp_div(&hash_num, &lc_divisor, NULL, &big_passcode);
  if (MP_OKAY != res)
  {
    result = rlr_mathfail;
    goto rhtp_fail;
  }

  /* Licence code, relies on (LICENCECODE_DIGIT_COMBINATIONS ^ LICENCECODE_LEN) being < 2^32 */
  *licence_code = mp_get_int(&big_passcode);
  result = rlr_success;

rhtp_fail:
  mp_clear_multi(&hash_num, &lc_divisor, &big_passcode, &lc_combinations, NULL);
  return result;
}


#define RL_NUM_BYTE_MUNGES 16

/** Generated with "shuf -i 0-7 -n 8" x RL_NUM_BYTE_MUNGES */
static const uint8_t rl_byte_munges[RL_NUM_BYTE_MUNGES][CHAR_BIT] =
{
  { 0, 6, 3, 2, 7, 4, 5, 1 },
  { 1, 4, 5, 2, 0, 6, 3, 7 },
  { 7, 6, 0, 4, 1, 5, 2, 3 },
  { 1, 3, 6, 5, 4, 7, 0, 2 },
  { 4, 3, 1, 7, 0, 6, 5, 2 },
  { 5, 3, 0, 6, 1, 7, 4, 2 },
  { 6, 7, 5, 2, 4, 3, 1, 0 },
  { 7, 2, 4, 3, 0, 5, 6, 1 },
  { 3, 2, 5, 7, 6, 4, 0, 1 },
  { 3, 6, 0, 4, 7, 2, 5, 1 },
  { 3, 1, 0, 5, 2, 7, 4, 6 },
  { 4, 1, 3, 0, 7, 2, 6, 5 },
  { 4, 2, 7, 6, 1, 3, 0, 5 },
  { 5, 0, 3, 6, 4, 7, 1, 2 },
  { 2, 1, 4, 6, 5, 7, 0, 3 },
  { 6, 1, 4, 2, 3, 5, 7, 0 }
};


/** Shuffle the bits around in a byte, based on algorithm */
static uint8_t rl_byte_munge(size_t algorithm, uint8_t data)
{
  uint8_t result;
  size_t i;
  uint8_t mask_in;
  const uint8_t *byte_munge;

  byte_munge = rl_byte_munges[algorithm % RL_NUM_BYTE_MUNGES];

  for (i = 0, result = 0, mask_in = 0x01; i < CHAR_BIT; i++, mask_in <<= 1)
  {
    if (mask_in & data)
      result |= 0x01 << byte_munge[i];
  }
  return result;
}


/** Extract a byte from the serial number */
static uint8_t rl_sn_to_byte(uint32_t serial_number, size_t idx)
{
  uint8_t data;
  size_t idx_mod;

  idx_mod = idx % sizeof(uint32_t);
  data = (serial_number >> (idx_mod * CHAR_BIT)) & 0x000000FF;
  return data;
}


/** Extract a a byte from the package name */
static uint8_t rl_pn_to_byte(const char *package_name, size_t idx)
{
  size_t idx_mod;

  idx_mod = idx % strlen(package_name);
  return (uint8_t) package_name[idx_mod];
}


/**
 * Munge up serial number and package name into a buffer.
 * This is the security by obscurity part of our licensing and should be kept secret.
 * Caller must free buffer
 *
 * Function not static so we can test it in the unit tests
 */
enum e_rl_result rl_munge_serial_and_package(const char *package_name, uint32_t serial_number, uint8_t **munge_buffer, size_t *munge_buffer_length)
{
  size_t package_name_length;
  size_t internal_munge_buffer_length;
  uint8_t *internal_munge_buffer;
  size_t pi;
  size_t si;
  size_t oi;
  size_t alt_pack;
  uint8_t data;

  /* Check parameters */
  if (NULL == package_name || NULL == munge_buffer || NULL == munge_buffer_length)
    return rlr_paramfail;

  /* Allocate buffer for munged serial number and package name */
  package_name_length = strlen(package_name);
  internal_munge_buffer_length = package_name_length + sizeof(uint32_t);
  internal_munge_buffer = calloc(1, internal_munge_buffer_length);
  if (NULL == internal_munge_buffer)
    return rlr_allocfail;

  /* Munge package name and serial number into buffer */
  alt_pack = (package_name_length < sizeof(uint32_t)) ? package_name_length * 2 : sizeof(uint32_t) * 2;

  /* Alternate package name characters and serial number bytes */
  pi = 0;
  si = 0;
  for (oi = 0; oi < alt_pack; oi++)
  {
    if ((oi % 2) == 0)
    {
      data = rl_pn_to_byte(package_name, pi);
      pi++;
    }
    else
    {
      data = rl_sn_to_byte(serial_number, si);
      si++;
    }
    internal_munge_buffer[oi] = rl_byte_munge(oi, data);
  }

  /* Pack in whatever is left */
  for (; pi < package_name_length; pi++, oi++)
  {
    internal_munge_buffer[oi] = rl_byte_munge(oi, rl_pn_to_byte(package_name, pi));
  }
  for (; si < sizeof(uint32_t); si++, oi++)
  {
    internal_munge_buffer[oi] = rl_byte_munge(oi, rl_sn_to_byte(serial_number, si));
  }

  /* Return buffer and length */
  *munge_buffer = internal_munge_buffer;
  *munge_buffer_length = internal_munge_buffer_length;
  return rlr_success;
}


/** Generate a C500 licence code for given package name and serial number */
enum e_rl_result rl_generate_licence_code_c500(struct s_rl_licence *rl_licence, const char *package_name, uint32_t *licence_code)
{
  uint8_t *lc_hash = NULL;
  hash_state md;
  enum e_rl_result result;
  size_t munge_buffer_length = 0;
  uint8_t *munge_buffer = NULL;
  size_t num_licence_code_digit_combinations;

  /* Check parameters */
  if (NULL == rl_licence || NULL == package_name || NULL == licence_code)
    return rlr_paramfail;

  /* Munge up the package name and serial number */
  munge_buffer_length = 0;
  result = rl_munge_serial_and_package(package_name, rl_licence->serial_number, &munge_buffer, &munge_buffer_length);
  if (rlr_success != result)
  {
    goto rgp_fail;
  }

  /* Use libtomcrypt to generate a SHA256 hash of the buffer */
  lc_hash = calloc(1, sha256_desc.hashsize);
  if (NULL == lc_hash)
  {
    result = rlr_allocfail;
    goto rgp_fail;
  }
  if (sha256_init(&md) != CRYPT_OK)
  {
    result = rlr_cryptofail;
    goto rgp_fail;
  }
  if (sha256_process(&md, munge_buffer, munge_buffer_length) != CRYPT_OK)
  {
    result = rlr_cryptofail;
    goto rgp_fail;
  }
  if (sha256_done(&md, lc_hash) != CRYPT_OK)
  {
    result = rlr_cryptofail;
    goto rgp_fail;
  }

#if 0
  {
    size_t i;
    for (i = 0; i < sha256_desc.hashsize; i++)
      printf("%02x", lc_hash[i]);
    printf("\n");
  }
#endif
  num_licence_code_digit_combinations = strlen(rl_licence->licence_code_digit_combinations);

  /* Convert the hash into a passcode we can get users to enter */
  result = rl_hash_to_passcode(lc_hash, sha256_desc.hashsize,
                               rl_licence->licence_code_length, num_licence_code_digit_combinations,
                               licence_code);
  if (rlr_success != result)
  {
    goto rgp_fail;
  }

  /* Success */
  result = rlr_success;

rgp_fail:
  if (NULL != munge_buffer)
  {
    memset(munge_buffer, 0, munge_buffer_length);
    free(munge_buffer);
    munge_buffer = NULL;
  }
  if (NULL != lc_hash)
  {
    memset(lc_hash, 0, sha256_desc.hashsize);
    free(lc_hash);
    lc_hash = NULL;
  }
  return result;
}
