/*
 * rl_generate_r300.c
 *
 *  Created on: Nov 4, 2016
 *      Author: ian
 */
#include "config.h"
#include <stdint.h>
#include <stdio.h>
#include <stdbool.h>
#include <stdlib.h>
#ifdef HAVE_BYTESWAP_H
#include <byteswap.h>
#else
/* On windows stdlib.h contains _byteswap_ulong(), which we can use */
#define bswap_32 _byteswap_ulong
#endif
#include <rinlicence/rl_licence.h>
#include "rl_rx00_list.h"

#include "rl_generate_r300.h"


#define NUM_LICENCE_PASSES_R300 14


static const struct s_rx00_licence_passes licence_passes_r300[NUM_LICENCE_PASSES_R300] =
{
  { 2,  "K342" },
  { 2,  "K381" },
  { 3,  "K344" },
  { 3,  "K383" },
  { 4,  "K346" },
  { 4,  "K385" },
  { 5,  "K354" },
  { 6,  "K356" },
  { 7,  "K376" },
  { 8,  "K378" },
  { 8,  "K364" },
  { 9,  "A320" },
  { 10, "A309" },
  { 11, "A308" },
};


/** Add all R300 known packages to known package list */
enum e_rl_result rl_init_r300_known_packages(UT_array *known_packages)
{
  size_t i;

  for (i = 0; i < NUM_LICENCE_PASSES_R300; i++)
    utarray_push_back(known_packages, &licence_passes_r300[i]);
  return rlr_success;
}


/** Lookup the R300 pass for a given package name */
static enum e_rl_result rl_lookup_r300_pass(struct s_rl_licence *rl_licence, const char *package_name, uint32_t *pass)
{
  return rl_rx00_lookup_pass(rl_licence->known_packages, package_name, pass);
}


/** Calculate the licence code for an R300 based on serial number and pass */
static enum e_rl_result rl_calculate_licence_code_r300(struct s_rl_licence *rl_licence, uint32_t pass, uint32_t *licence_code)
{
  uint32_t l;
  uint32_t divisor;
  bool lowbit;

  /* Start with inverted byte swapped serial number */
  l = bswap_32((~rl_licence->serial_number));

  /* Iterate pass number of times */
  while (pass--)
  {
    lowbit = ((l & 1) != 0);
    l >>= 1;
    l += rl_licence->serial_number;

    /* Pop low order bit back on the front */
    if (lowbit)
      l |= 0x80000000;
  }

  divisor = rl_rx00_licence_code_divisor(rl_licence);
  *licence_code = l % divisor;
  return rlr_success;
}


/** Generate an R300 licence code for given package name and serial number */
enum e_rl_result rl_generate_licence_code_r300(struct s_rl_licence *rl_licence, const char *package_name, uint32_t *licence_code)
{
  uint32_t pass;
  enum e_rl_result result;

  if (NULL == rl_licence || NULL == package_name || NULL == licence_code)
    return rlr_paramfail;

  /* Get the pass */
  result = rl_lookup_r300_pass(rl_licence, package_name, &pass);
  if (rlr_success != result)
  {
    goto rglcr_fail;
  }

  /* Calculate the licence code from the pass */
  result = rl_calculate_licence_code_r300(rl_licence, pass, licence_code);
  if (rlr_success != result)
  {
    goto rglcr_fail;
  }

  /* Success */
  result = rlr_success;

rglcr_fail:
  return result;
}
