/*
 * rl_generate_r400.c
 *
 *  Created on: Nov 4, 2016
 *      Author: ian
 */
#include "config.h"
#include <stdint.h>
#include <stdio.h>
#include <stdbool.h>
#include <stdlib.h>
#ifdef HAVE_BYTESWAP_H
#include <byteswap.h>
#else
/* On windows stdlib.h contains _byteswap_ulong(), which we can use */
#define bswap_32 _byteswap_ulong
#endif
#include <rinlicence/rl_licence.h>
#include "rl_rx00_list.h"

#include "rl_generate_r400.h"


#define NUM_LICENCE_PASSES_R400 17


static const struct s_rx00_licence_passes licence_passes_r400[NUM_LICENCE_PASSES_R400] =
{
  { 0,  "K401old" },    /*!< Pass 0 */
  { 2,  "K402" },       /*!< Pass 2 */
  { 3,  "GSE255" },     /*!< Pass 3 */
  { 4,  "K491" },       /*!< Pass 4 */
  { 5,  "K481" },       /*!< Pass 5 */
  { 6,  "K403" },       /*!< Pass 6 */
  { 7,  "A406" },       /*!< Pass 7 */
  { 8,  "K404" },       /*!< Pass 8 */
  { 9,  "K422" },       /*!< Pass 9 */
  { 10, "K410" },       /*!< Pass 10 */
  { 11, "K411" },       /*!< Pass 11 */
  { 12, "K412" },       /*!< Pass 12 */
  { 13, "K401" },       /*!< Pass 13 */
  { 14, "L401" },       /*!< Pass 14 */
  { 15, "K415" },       /*!< Pass 15 */
  { 16, "K405" },       /*!< Pass 16 */
  { 20, "A408" },       /*!< Pass 20 */

  /* "A403" = Pass 403501, different algorithm */
};


/** Add all R400 known packages to known package list */
enum e_rl_result rl_init_r400_known_packages(UT_array *known_packages)
{
  size_t i;

  for (i = 0; i < NUM_LICENCE_PASSES_R400; i++)
    utarray_push_back(known_packages, &licence_passes_r400[i]);
  return rlr_success;
}


/** Lookup the R400 pass for a given package name */
static enum e_rl_result rl_lookup_r400_pass(struct s_rl_licence *rl_licence, const char *package_name, uint32_t *pass)
{
  return rl_rx00_lookup_pass(rl_licence->known_packages, package_name, pass);
}


/** Calculate the licence code for an R400 based on serial number and pass */
static enum e_rl_result rl_calculate_licence_code_r400(struct s_rl_licence *rl_licence, uint32_t pass, uint32_t *licence_code)
{
  uint32_t l;
  uint32_t divisor;

  if (0 == pass)
  {
    /* Special case for pass 0 */
    l = rl_licence->serial_number;
  }
  else
  {
    /* Start with inverted byte swapped serial number */
    l = bswap_32((~rl_licence->serial_number));

    /* Iterate pass number of times */
    while (pass--)
    {
      l >>= 1;
      l += rl_licence->serial_number;
    }
  }

  divisor = rl_rx00_licence_code_divisor(rl_licence);
  *licence_code = l % divisor;
  return rlr_success;
}


/** Generate an R400 licence code for given package name and serial number */
enum e_rl_result rl_generate_licence_code_r400(struct s_rl_licence *rl_licence, const char *package_name, uint32_t *licence_code)
{
  uint32_t pass;
  enum e_rl_result result;

  if (NULL == rl_licence || NULL == package_name || NULL == licence_code)
    return rlr_paramfail;

  /* Get the pass */
  result = rl_lookup_r400_pass(rl_licence, package_name, &pass);
  if (rlr_success != result)
  {
    goto rglcr_fail;
  }

  /* Calculate the licence code from the pass */
  result = rl_calculate_licence_code_r400(rl_licence, pass, licence_code);
  if (rlr_success != result)
  {
    goto rglcr_fail;
  }

  /* Success */
  result = rlr_success;

rglcr_fail:
  return result;
}
