/*
 * rl_init_c500.c
 *
 *  Created on: Nov 15, 2016
 *      Author: ian
 */
#include <stdio.h>
#include <stdint.h>
#include <stddef.h>
#include <string.h>
#include "rl_generate_c500.h"
#include "rl_init_common.h"
#include "rl_opkg.h"
#include "rl_pkgtype_c500.h"

#include <rinlicence/rl_licence.h>


/** Allocate and initialise rin licence structure for a C500 & C500Rude */
static enum e_rl_result rl_internal_init_c500(struct s_rl_licence **rl_licence,
                                              uint32_t serial_number,
                                              uint32_t *package_licence_codes,
                                              uint32_t *num_package_licence_codes,
                                              size_t total_num_package_licence_codes,
                                              const char *licence_code_digit_combinations,
                                              const char *opkg_executable)
{
  enum e_rl_result result;
  struct s_rl_licence *rll = NULL;

  /* Check parameters */
  if (NULL == rl_licence || NULL == package_licence_codes ||
      NULL == num_package_licence_codes)
    return rlr_paramfail;

  /* Initialise common elements */
  result = rl_init_common(&rll, serial_number, package_licence_codes,
                          num_package_licence_codes, total_num_package_licence_codes,
                          LICENCECODE_LEN_C500, licence_code_digit_combinations,
                          rl_generate_licence_code_c500,
                          rl_get_package_type_c500);
  if (rlr_success != result)
  {
    goto ric5_fail;
  }

  /* Fill out C500 specific elements */
  rll->licensable_package_regex = strdup(RL_PACKAGE_NAME_REGEX_C500);
  if (NULL == rll->licensable_package_regex)
  {
    result = rlr_allocfail;
    goto ric5_fail;
  }
  rll->primary_package_regex = strdup(RL_PACKAGE_NAME_REGEX_C500_PRIMARY);
  if (NULL == rll->primary_package_regex)
  {
    result = rlr_allocfail;
    goto ric5_fail;
  }
  rll->addon_package_regex = strdup(RL_PACKAGE_NAME_REGEX_C500_ADDON);
  if (NULL == rll->addon_package_regex)
  {
    result = rlr_allocfail;
    goto ric5_fail;
  }
  if (NULL != opkg_executable)
  {
    rll->opkg_executable = strdup(opkg_executable);
    if (NULL == rll->opkg_executable)
    {
      result = rlr_allocfail;
      goto ric5_fail;
    }
  }
  else
  {
    rll->opkg_executable = NULL;
  }

  /* Get list of installed packages, if we have the tools to do so */
  if (NULL != rll->opkg_executable && NULL != rll->licensable_package_regex)
  {
    result = rl_opkg_get_installed_packages(rll);
    if (rlr_success != result)
    {
      goto ric5_fail;
    }
  }

  /* Success */
  *rl_licence = rll;
  return rlr_success;

ric5_fail:
  if (NULL != rll)
  {
    rl_destroy(rll);
    rll = NULL;
  }
  return result;
}


/** Allocate and initialise rin licence structure for a C500 */
enum e_rl_result rl_init_c500(struct s_rl_licence **rl_licence,
                              uint32_t serial_number,
                              uint32_t *package_licence_codes,
                              uint32_t *num_package_licence_codes,
                              size_t total_num_package_licence_codes,
                              const char *opkg_executable)
{
  return rl_internal_init_c500(rl_licence,
                               serial_number,
                               package_licence_codes,
                               num_package_licence_codes,
                               total_num_package_licence_codes,
                               LICENCECODE_DIGIT_COMBINATIONS_C500,
                               opkg_executable);
}


/**
 * Allocate and initialise rin licence structure for a C500Rude.
 *
 * This is the old C500 algorithm that would generate rude words
 * from time to time
 */
enum e_rl_result rl_init_c500rude(struct s_rl_licence **rl_licence,
                                  uint32_t serial_number,
                                  uint32_t *package_licence_codes,
                                  uint32_t *num_package_licence_codes,
                                  size_t total_num_package_licence_codes,
                                  const char *opkg_executable)
{
  return rl_internal_init_c500(rl_licence,
                               serial_number,
                               package_licence_codes,
                               num_package_licence_codes,
                               total_num_package_licence_codes,
                               LICENCECODE_DIGIT_COMBINATIONS_C500RUDE,
                               opkg_executable);
}
