/*
 * rl_init_common.c
 *
 *  Created on: Nov 15, 2016
 *      Author: ian
 */
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <utarray.h>
#include <rinlicence/rl_licence.h>
#include "rl_rx00_list.h"

#include "rl_init_common.h"


/** Allocate and initialise rin licence structure */
enum e_rl_result rl_init_common(struct s_rl_licence **rl_licence,
                                uint32_t serial_number,
                                uint32_t *package_licence_codes,
                                uint32_t *num_package_licence_codes,
                                size_t total_num_package_licence_codes,
                                size_t licence_code_length,
                                const char *licence_code_digit_combinations,
                                t_rl_generate_licence_code_fn rl_generate_licence_code_fn,
                                t_rl_get_package_type_fn rl_get_package_type_fn)
{
  enum e_rl_result result;
  struct s_rl_licence *rll = NULL;

  /* Check parameters */
  if (NULL == rl_licence || NULL == package_licence_codes || NULL == num_package_licence_codes)
    return rlr_paramfail;

  /* Allocate structure */
  rll = calloc(1, sizeof(struct s_rl_licence));
  if (NULL == rll)
  {
    result = rlr_allocfail;
    goto rli_fail;
  }

  /* Fill out members */
  rll->serial_number = serial_number;
  rll->package_licence_codes = package_licence_codes;
  rll->num_package_licence_codes = num_package_licence_codes;
  rll->total_num_package_licence_codes = total_num_package_licence_codes;
  rll->licence_code_length = licence_code_length;
  rll->licence_code_digit_combinations = licence_code_digit_combinations;
  rll->rl_generate_licence_code_fn = rl_generate_licence_code_fn;
  rll->rl_get_package_type_fn = rl_get_package_type_fn;

  /* Create list for installed licensable packages */
  utarray_new(rll->installed_licensable_packages, &ut_str_icd);
  if (NULL == rll->installed_licensable_packages)
  {
    result = rlr_allocfail;
    goto rli_fail;
  }

  /* Create list for known packages (applies to R300/R400 only, empty for C500) */
  result = rl_rx00_licence_passes_list_init(&rll->known_packages);
  if (rlr_success != result)
  {
    goto rli_fail;
  }

  /* C500 specific filled in later */
  rll->licensable_package_regex = NULL;
  rll->opkg_executable = NULL;

  /* Success */
  *rl_licence = rll;
  return rlr_success;

rli_fail:
  if (NULL != rll)
  {
    if (NULL != rll->installed_licensable_packages)
    {
      utarray_free(rll->installed_licensable_packages);
      rll->installed_licensable_packages = NULL;
    }
    if (NULL != rll->known_packages)
    {
      utarray_free(rll->known_packages);
      rll->known_packages = NULL;
    }
    free(rll);
    rll = NULL;
  }
  return result;
}


/** Clean up rin licence structure */
enum e_rl_result rl_destroy(struct s_rl_licence *rl_licence)
{
  if (NULL == rl_licence)
    return rlr_paramfail;

  if (NULL != rl_licence->installed_licensable_packages)
  {
    utarray_free(rl_licence->installed_licensable_packages);
    rl_licence->installed_licensable_packages = NULL;
  }
  if (NULL != rl_licence->known_packages)
  {
    utarray_free(rl_licence->known_packages);
    rl_licence->known_packages = NULL;
  }
  if (NULL != rl_licence->licensable_package_regex)
  {
    free(rl_licence->licensable_package_regex);
    rl_licence->licensable_package_regex = NULL;
  }
  if (NULL != rl_licence->primary_package_regex)
  {
    free(rl_licence->primary_package_regex);
    rl_licence->primary_package_regex = NULL;
  }
  if (NULL != rl_licence->addon_package_regex)
  {
    free(rl_licence->addon_package_regex);
    rl_licence->addon_package_regex = NULL;
  }
  if (NULL != rl_licence->opkg_executable)
  {
    free(rl_licence->opkg_executable);
    rl_licence->opkg_executable = NULL;
  }
  free(rl_licence);

  return rlr_success;
}
