/*
 * rl_init_r300.c
 *
 *  Created on: Nov 15, 2016
 *      Author: ian
 */
#include <stdio.h>
#include <stdint.h>
#include <stddef.h>
#include "rl_generate_r300.h"
#include "rl_init_common.h"
#include "rl_pkgtype_r300.h"

#include <rinlicence/rl_licence.h>


/** Allocate and initialise rin licence structure for an R300 */
enum e_rl_result rl_init_r300(struct s_rl_licence **rl_licence,
                              uint32_t serial_number,
                              uint32_t *package_licence_codes,
                              uint32_t *num_package_licence_codes,
                              size_t total_num_package_licence_codes,
                              const char *installed_package)
{
  enum e_rl_result result;
  struct s_rl_licence *rll = NULL;

  /* Check parameters */
  if (NULL == rl_licence || NULL == package_licence_codes || NULL == num_package_licence_codes)
    return rlr_paramfail;

  /* Initialise common elements */
  result = rl_init_common(&rll, serial_number, package_licence_codes,
                          num_package_licence_codes, total_num_package_licence_codes,
                          LICENCECODE_LEN_RX00, LICENCECODE_DIGIT_COMBINATIONS_RX00,
                          rl_generate_licence_code_r300,
                          rl_get_package_type_r300);
  if (rlr_success != result)
  {
    goto rir3_fail;
  }

  /* Fill out R300 known packages */
  result = rl_init_r300_known_packages(rll->known_packages);
  if (rlr_success != result)
  {
    goto rir3_fail;
  }

  /* Only one installed licensable package on R300 (eg K342), reported by application */
  if (NULL != installed_package)
    utarray_push_back(rll->installed_licensable_packages, &installed_package);

  /* Success */
  *rl_licence = rll;
  return rlr_success;

rir3_fail:
  if (NULL != rll)
  {
    rl_destroy(rll);
    rll = NULL;
  }
  return result;
}
