/*
 * rl_package.c
 *
 *  Created on: Nov 21, 2016
 *      Author: ian
 */
#include <stdio.h>
#include <stddef.h>
#include <stdbool.h>
#include <utarray.h>
#include <string.h>
#include "rl_rx00_list.h"

#include <rinlicence/rl_licence.h>


/** Get the number of known packages */
enum e_rl_result rl_get_num_known_packages(struct s_rl_licence *rl_licence, size_t *num_packages)
{
  if (NULL == rl_licence || NULL == num_packages)
    return rlr_paramfail;

  *num_packages = utarray_len(rl_licence->known_packages);
  return rlr_success;
}


/** Get a known package for index */
enum e_rl_result rl_get_known_package(struct s_rl_licence *rl_licence, size_t index, const char **package_name, uint32_t *pass)
{
  struct s_rx00_licence_passes *rlp;

  if (NULL == rl_licence)
    return rlr_paramfail;

  /* Get element */
  rlp = (struct s_rx00_licence_passes *) utarray_eltptr(rl_licence->known_packages, index);
  if (NULL == rlp)
  {
    return rlr_unknown_package;
  }

  if (NULL != package_name)
    *package_name = rlp->package;
  if (NULL != pass)
    *pass = rlp->passes;
  return rlr_success;
}


/** Get the number of installed packages */
enum e_rl_result rl_get_num_installed_packages(struct s_rl_licence *rl_licence, size_t *num_packages)
{
  if (NULL == rl_licence || NULL == num_packages)
    return rlr_paramfail;

  *num_packages = utarray_len(rl_licence->installed_licensable_packages);
  return rlr_success;
}


/** Get the installed package name for index */
enum e_rl_result rl_get_installed_package(struct s_rl_licence *rl_licence, size_t index, const char **package_name)
{
  char **package;

  if (NULL == rl_licence || NULL == package_name)
    return rlr_paramfail;

  /* Get element */
  package = (char **) utarray_eltptr(rl_licence->installed_licensable_packages, index);
  if (NULL == package)
  {
    return rlr_unknown_package;
  }

  *package_name = *package;
  return rlr_success;
}


/** Check if a package is installed */
enum e_rl_result rl_package_is_installed(struct s_rl_licence *rl_licence, const char *package_name, bool *result)
{
  char **package;

  if (NULL == rl_licence || NULL == package_name || NULL == result)
    return rlr_paramfail;

  /* Iterate the list looking for package name */
  *result = false;
  for (package = (char **) utarray_front(rl_licence->installed_licensable_packages);
       package != NULL;
       package = (char **) utarray_next(rl_licence->installed_licensable_packages, package))
  {
    if (strcmp(package_name, *package) == 0)
    {
      *result = true;
      break;
    }
  }
  return rlr_success;
}
