/*
 * rl_remove.c
 *
 *  Created on: Nov 28, 2016
 *      Author: ian
 */
#include <stdio.h>
#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>

#include <rinlicence/rl_licence.h>


static enum e_rl_result rl_internal_remove_licence_code(struct s_rl_licence *rl_licence, size_t index)
{
  size_t i;

  /* Found licence code for package, shuffle the rest of the codes down */
  for (i = index; i < *rl_licence->num_package_licence_codes - 1; i++)
  {
    rl_licence->package_licence_codes[i] = rl_licence->package_licence_codes[i + 1];
  }
  rl_licence->package_licence_codes[*rl_licence->num_package_licence_codes] = 0;
  (*rl_licence->num_package_licence_codes)--;
  return rlr_success;
}


/** Unlicense an installed package. Returns rlr_licence_not_removed is no licence code removed */
enum e_rl_result rl_unlicense_package(struct s_rl_licence *rl_licence, const char *package_name)
{
  enum e_rl_result result;
  //bool installed;
  size_t i;
  uint32_t generated_licence_code;

  /* Check parameters */
  if (NULL == rl_licence || NULL == package_name)
    return rlr_paramfail;

  /* Package installed, generate licence code */
  result = rl_generate_licence_code(rl_licence, package_name, &generated_licence_code);
  if (rlr_success != result)
  {
    return result;
  }

  /* Is this licence code in the list (already licensed) */
  for (i = 0; i < *rl_licence->num_package_licence_codes; i++)
  {
    if (rl_licence->package_licence_codes[i] == generated_licence_code)
    {
      result = rl_internal_remove_licence_code(rl_licence, i);
      return rlr_success;
    }
  }

  /* Package was not licensed */
  return rlr_licence_not_removed;
}


/** Remove any licence codes that don't have installed packages. Returns rlr_licence_not_removed is no licence code removed */
enum e_rl_result rl_flush_licence_codes_for_uninstalled_packages(struct s_rl_licence *rl_licence)
{
  size_t codeidx;
  size_t pkgidx;
  size_t num_packages;
  enum e_rl_result result;
  const char *package_name;
  uint32_t licence_code;
  bool pkgfound;
  size_t removed;

  /* Check parameter */
  if (NULL == rl_licence)
    return rlr_paramfail;

  /** Get the number of installed packages */
  result = rl_get_num_installed_packages(rl_licence, &num_packages);
  if (rlr_success != result)
    return result;

  /* For each licence code in the list */
  removed = 0;
rflcfup_restart:
  for (codeidx = 0; codeidx < *rl_licence->num_package_licence_codes; codeidx++)
  {
    /* Is there an installed package with matching licence code? */
    for (pkgidx = 0, pkgfound = false; pkgidx < num_packages; pkgidx++)
    {
      result = rl_get_installed_package(rl_licence, pkgidx, &package_name);
      if (rlr_success != result)
        return result;
      result = rl_generate_licence_code(rl_licence, package_name, &licence_code);
      if (rlr_success != result)
        return result;
      if (rl_licence->package_licence_codes[codeidx] == licence_code)
      {
        pkgfound = true;
        break;
      }
    }
    if (!pkgfound)
    {
      /* No matching installled package, so remove this licence code from the list */
      result = rl_internal_remove_licence_code(rl_licence, codeidx);
      if (rlr_success != result)
        return result;
      removed++;
      goto rflcfup_restart;
    }
  }

  return (removed > 0) ? rlr_success : rlr_licence_not_removed;
}
