/*
 * rl_rx00_list.c
 *
 *  Created on: Nov 8, 2016
 *      Author: ian
 */
#include <stdbool.h>
#include <stdio.h>
#include <string.h>
#include <utarray.h>
#include <rinlicence/rl_licence.h>

#include "rl_rx00_list.h"


static void rx00_licence_passes_copy(void *_dst, const void *_src)
{
  struct s_rx00_licence_passes *dst = (struct s_rx00_licence_passes *) _dst;
  const struct s_rx00_licence_passes *src = (const struct s_rx00_licence_passes *) _src;

  if (NULL == dst || NULL == src)
    return;

  dst->passes = src->passes;
  dst->package = (NULL != src->package) ? strdup(src->package) : NULL;
}


static void rx00_licence_passes_dtor(void *_elt)
{
  struct s_rx00_licence_passes *elt = (struct s_rx00_licence_passes *) _elt;

  if (NULL != elt && NULL != elt->package)
  {
    free(elt->package);
    elt->package = NULL;
  }
}


static UT_icd rx00_licence_passes_icd =
{
  .sz = sizeof(struct s_rx00_licence_passes),
  .init = NULL,
  .copy = rx00_licence_passes_copy,
  .dtor = rx00_licence_passes_dtor,
};


/** Create a list of licences and passes */
enum e_rl_result rl_rx00_licence_passes_list_init(UT_array **list)
{
  utarray_new(*list, &rx00_licence_passes_icd);
  return rlr_success;
}


/** Lookup the pass for a given package name */
enum e_rl_result rl_rx00_lookup_pass(UT_array *list, const char *package_name, uint32_t *pass)
{
  size_t i;
  bool isnum;
  struct s_rx00_licence_passes *rlp;

  if (NULL == list || NULL == package_name || NULL == pass)
    return rlr_paramfail;

  /* Special case -> if package name is a number, then that is the number of passes */
  for (i = 0, isnum = true; i < strlen(package_name); i++)
  {
    if (package_name[i] < '0' || package_name[i] > '9')
    {
      isnum = false;
      break;
    }
  }
  if (isnum)
  {
    *pass = strtoul(package_name, NULL, 10);
    return rlr_success;
  }

  /* Iterate the list looking for package name */
  for (rlp = (struct s_rx00_licence_passes *) utarray_front(list);
       rlp != NULL;
       rlp = (struct s_rx00_licence_passes *) utarray_next(list, rlp))
  {
    if (strcmp(package_name, rlp->package) == 0)
    {
      *pass = rlp->passes;
      return rlr_success;
    }
  }
  return rlr_unknown_package;
}


/** Calculate the licence code divisor */
uint32_t rl_rx00_licence_code_divisor(struct s_rl_licence *rl_licence)
{
  size_t combinations;
  size_t i;
  uint32_t divisor;

  combinations = strlen(rl_licence->licence_code_digit_combinations);

  /* divisor = combinations**length */
  for (i = 0, divisor = 1; i < rl_licence->licence_code_length; i++)
  {
    divisor *= combinations;
  }

  return divisor;
}
