/*
 * rl_test_generate_c500.c
 *
 *  Created on: Oct 31, 2016
 *      Author: ian
 */

#include <string.h>
#include <rinlicence/rl_licence.h>
#include <rinlicence/rl_generate_c500.h>
#include <cutest/CuTest.h>

#include "rl_test_generate_c500.h"


/** Check munging up serial number an package name - see rinlicense.py for generated strings */
static void test_rinlicence_munge_c500(CuTest* tc)
{
  uint8_t *munge_buffer;
  size_t munge_buffer_length;
  enum e_rl_result result;

  /* Test bad parameters */
  result = rl_munge_serial_and_package(NULL, 0x00000000, &munge_buffer, &munge_buffer_length);
  CuAssertIntEquals(tc, rlr_paramfail, result);
  result = rl_munge_serial_and_package("", 0x00000000, NULL, &munge_buffer_length);
  CuAssertIntEquals(tc, rlr_paramfail, result);
  result = rl_munge_serial_and_package("", 0x00000000, &munge_buffer, NULL);
  CuAssertIntEquals(tc, rlr_paramfail, result);

  /* Test zero length package name and serial number = 0 */
  result = rl_munge_serial_and_package("", 0x00000000, &munge_buffer, &munge_buffer_length);
  CuAssertIntEquals(tc, rlr_success, result);
  CuAssertIntEquals(tc, 4, munge_buffer_length);
  CuAssertIntEquals(tc, 0, memcmp("\x00\x00\x00\x00", munge_buffer, 4));
  free(munge_buffer);
  munge_buffer = NULL;

  /* Test package name and serial number = 0 */
  result = rl_munge_serial_and_package("test", 0x00000000, &munge_buffer, &munge_buffer_length);
  CuAssertIntEquals(tc, rlr_success, result);
  CuAssertIntEquals(tc, 8, munge_buffer_length);
  CuAssertIntEquals(tc, 0, memcmp("\xB8\x00\xA5\x00\x79\x00\x3A\x00", munge_buffer, 8));
  free(munge_buffer);
  munge_buffer = NULL;

  /* Test zero length package name and serial number <> 0 */
  result = rl_munge_serial_and_package("", 0x12345678, &munge_buffer, &munge_buffer_length);
  CuAssertIntEquals(tc, rlr_success, result);
  CuAssertIntEquals(tc, 4, munge_buffer_length);
  CuAssertIntEquals(tc, 0, memcmp("\xB4\x39\x23\x18", munge_buffer, 4));
  free(munge_buffer);
  munge_buffer = NULL;

  /* Test a real example - serial number 3382159=0x00339B8F */
  result = rl_munge_serial_and_package("L900-500", 3382159, &munge_buffer, &munge_buffer_length);
  CuAssertIntEquals(tc, rlr_success, result);
  CuAssertIntEquals(tc, 12, munge_buffer_length);
  CuAssertIntEquals(tc, 0, memcmp("\x2C\xB6\xB2\x3E\x41\xAA\x18\x00\xB8\x8D\x84\x84", munge_buffer, 12));
  free(munge_buffer);
  munge_buffer = NULL;
}


#define NUM_PACKAGE_LICENCE_CODES 10


/** Check generating a licence code code from serial number an package name - see rinlicense.py for generated strings */
static void test_rinlicence_generate_c500(CuTest* tc)
{
  enum e_rl_result result;
  uint32_t licence_code;
  struct s_rl_licence *rl_licence;
  uint32_t package_licence_codes[NUM_PACKAGE_LICENCE_CODES];
  uint32_t num_package_licence_codes = 0;

  /* Create rin licence structure */
  result = rl_init_c500(&rl_licence, 0, package_licence_codes, &num_package_licence_codes, NUM_PACKAGE_LICENCE_CODES, NULL);
  CuAssertIntEquals(tc, rlr_success, result);

  /* Test bad parameters */
  result = rl_generate_licence_code_c500(NULL, "", &licence_code);
  CuAssertIntEquals(tc, rlr_paramfail, result);
  result = rl_generate_licence_code_c500(rl_licence, NULL, &licence_code);
  CuAssertIntEquals(tc, rlr_paramfail, result);
  result = rl_generate_licence_code_c500(rl_licence, "", NULL);
  CuAssertIntEquals(tc, rlr_paramfail, result);

  /* Test zero length package name and serial number = 0 */
  result = rl_set_serial_number(rl_licence, 0x00000000);
  CuAssertIntEquals(tc, rlr_success, result);
  result = rl_generate_licence_code_c500(rl_licence, "", &licence_code);
  CuAssertIntEquals(tc, rlr_success, result);
  CuAssertIntEquals(tc, 0x1258C019, licence_code);

  /* Test package name and serial number = 0 */
  result = rl_set_serial_number(rl_licence, 0x00000000);
  CuAssertIntEquals(tc, rlr_success, result);
  result = rl_generate_licence_code_c500(rl_licence, "test", &licence_code);
  CuAssertIntEquals(tc, rlr_success, result);
  CuAssertIntEquals(tc, 0x06ECF056, licence_code);

  /* Test zero length package name and serial number <> 0 */
  result = rl_set_serial_number(rl_licence, 0x12345678);
  CuAssertIntEquals(tc, rlr_success, result);
  result = rl_generate_licence_code_c500(rl_licence, "", &licence_code);
  CuAssertIntEquals(tc, rlr_success, result);
  CuAssertIntEquals(tc, 0x0A00F407, licence_code);

  /* Test a real example - serial number 3382159=0x00339B8F */
  result = rl_set_serial_number(rl_licence, 3382159);
  CuAssertIntEquals(tc, rlr_success, result);
  result = rl_generate_licence_code_c500(rl_licence, "L900-500", &licence_code);
  CuAssertIntEquals(tc, rlr_success, result);
  CuAssertIntEquals(tc, 0x083C6873, licence_code);

  /* Destroy rin licence structure */
  result = rl_destroy(rl_licence);
  CuAssertIntEquals(tc, rlr_success, result);
}


CuSuite *CuGetSuite_rinlicence_generate_c500(void)
{
  CuSuite *suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_rinlicence_munge_c500);
  SUITE_ADD_TEST(suite, test_rinlicence_generate_c500);

  return suite;
}
