/*
 * rl_test_generate_r400.c
 *
 *  Created on: Oct 31, 2016
 *      Author: ian
 */

#include <string.h>
#include <stdio.h>
#include <rinlicence/rl_licence.h>
#include <rinlicence/rl_generate_r400.h>
#include <cutest/CuTest.h>

#include "rl_test_generate_r400.h"


#define NUM_R400_LICENCE_CODES 29

static const struct s_rx00_licence_code_tests r400_licence_codes[NUM_R400_LICENCE_CODES] =
{
  { 1234567, 0,  234567, "K401old" },
  { 1234567, 2,  855306, "K402" },
  { 1234567, 4,  315677, "K491" },
  { 1234567, 5,  392405, "K481" },
  { 1234567, 6,  930769, "K403" },
  { 1234567, 7,  199951, "A406" },
  { 1234567, 8,  334542, "K404" },
  { 1234567, 10, 435486, "K410" },
  { 1234567, 11, 452310, "K411" },
  { 3172711, 2,  764186, "K402" },
  { 3172711, 3,  554804, "GSE255" },
  { 3172711, 4,  950113, "K491" },
  { 3172711, 5,  147767, "K481" },
  { 3172711, 6,  246594, "K403" },
  { 3172711, 7,  296008, "A406" },
  { 3172711, 8,  320715, "K404" },
  { 3172711, 9,  333068, "K422" },
  { 3172711, 10, 839245, "K410" },
  { 3172711, 11, 592333, "K411" },
  { 3172711, 12, 968877, "K412" },
  { 9876543, 0,  876543, "K401old" },
  { 9876543, 2,  356766, "K402" },
  { 9876543, 4,  154006, "K491" },
  { 9876543, 5,  953546, "K481" },
  { 9876543, 6,  853316, "K403" },
  { 9876543, 7,  803201, "A406" },
  { 9876543, 8,  278143, "K404" },
  { 9876543, 10, 884350, "K410" },
  { 9876543, 11, 318718, "K411" },
};


#define NUM_PACKAGE_LICENCE_CODES 10

/** Check bad parameters to R400 licence code */
static void test_rinlicence_generate_r400_badparam(CuTest* tc)
{
  enum e_rl_result result;
  uint32_t licence_code;
  struct s_rl_licence *rl_licence;
  uint32_t package_licence_codes[NUM_PACKAGE_LICENCE_CODES];
  uint32_t num_package_licence_codes = 0;

  result = rl_init_r400(&rl_licence, 0, package_licence_codes, &num_package_licence_codes, NUM_PACKAGE_LICENCE_CODES, NULL);
  CuAssertIntEquals(tc, rlr_success, result);

  /* Test bad parameters */
  result = rl_generate_licence_code_r400(NULL, "", &licence_code);
  CuAssertIntEquals(tc, rlr_paramfail, result);
  result = rl_generate_licence_code_r400(rl_licence, NULL, &licence_code);
  CuAssertIntEquals(tc, rlr_paramfail, result);
  result = rl_generate_licence_code_r400(rl_licence, "", NULL);
  CuAssertIntEquals(tc, rlr_paramfail, result);

  /* Destroy rin licence structure */
  result = rl_destroy(rl_licence);
  CuAssertIntEquals(tc, rlr_success, result);
}


/** Check generating a R400 licence code from serial number and number of passes */
static void test_rinlicence_generate_r400_passes(CuTest* tc)
{
  enum e_rl_result result;
  uint32_t licence_code;
  size_t i;
  char package[40];
  struct s_rl_licence *rl_licence;
  uint32_t package_licence_codes[NUM_PACKAGE_LICENCE_CODES];
  uint32_t num_package_licence_codes = 0;

  result = rl_init_r400(&rl_licence, 0, package_licence_codes, &num_package_licence_codes, NUM_PACKAGE_LICENCE_CODES, NULL);
  CuAssertIntEquals(tc, rlr_success, result);

  /* Check package name as number (pass) -> licence code */
  for (i = 0; i < NUM_R400_LICENCE_CODES; i++)
  {
    result = rl_set_serial_number(rl_licence, r400_licence_codes[i].serialno);
    CuAssertIntEquals(tc, rlr_success, result);

    snprintf(package, 40, "%lu", (unsigned long) r400_licence_codes[i].passes);
    result = rl_generate_licence_code_r400(rl_licence, package, &licence_code);
    CuAssertIntEquals(tc, rlr_success, result);
    CuAssertIntEquals(tc, r400_licence_codes[i].expected_licence_code, licence_code);
  }

  /* Destroy rin licence structure */
  result = rl_destroy(rl_licence);
  CuAssertIntEquals(tc, rlr_success, result);
}


/** Check generating a R400 licence code from serial number and package name */
static void test_rinlicence_generate_r400_package(CuTest* tc)
{
  enum e_rl_result result;
  uint32_t licence_code;
  size_t i;
  struct s_rl_licence *rl_licence;
  uint32_t package_licence_codes[NUM_PACKAGE_LICENCE_CODES];
  uint32_t num_package_licence_codes = 0;

  result = rl_init_r400(&rl_licence, 0, package_licence_codes, &num_package_licence_codes, NUM_PACKAGE_LICENCE_CODES, NULL);
  CuAssertIntEquals(tc, rlr_success, result);

  /* Check package name (K4xx) -> licence code */
  for (i = 0; i < NUM_R400_LICENCE_CODES; i++)
  {
    if (NULL != r400_licence_codes[i].package)
    {
      result = rl_set_serial_number(rl_licence, r400_licence_codes[i].serialno);
      CuAssertIntEquals(tc, rlr_success, result);

      result = rl_generate_licence_code_r400(rl_licence, r400_licence_codes[i].package, &licence_code);
      CuAssertIntEquals(tc, rlr_success, result);
      CuAssertIntEquals(tc, r400_licence_codes[i].expected_licence_code, licence_code);
    }
  }

  /* Destroy rin licence structure */
  result = rl_destroy(rl_licence);
  CuAssertIntEquals(tc, rlr_success, result);
}


CuSuite *CuGetSuite_rinlicence_generate_r400(void)
{
  CuSuite *suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_rinlicence_generate_r400_badparam);
  SUITE_ADD_TEST(suite, test_rinlicence_generate_r400_passes);
  SUITE_ADD_TEST(suite, test_rinlicence_generate_r400_package);

  return suite;
}
