/*
 * rl_test_init.c
 *
 *  Created on: Aug 4, 2014
 *      Author: ian
 */
#include <sys/types.h>
#include <string.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <rinlicence/rl_licence.h>
#include <cutest/CuTest.h>

#include "rl_test_init.h"


static const char opkg_executable_0_licensed[] = "./opkg-0-licensed";
static const char opkg_executable_1_licensed[] = "./opkg-1-licensed";


#define NUM_PACKAGE_LICENCE_CODES 10


#if 0
static void list_installed_packages(struct s_rl_licence *rl_licence)
{
  size_t num_packages;
  size_t i;
  enum e_rl_result result;
  const char *package_name;

  /** Get the number of installed packages */
  result = rl_get_num_installed_packages(rl_licence, &num_packages);
  if (rlr_success != result)
    return;

  for (i = 0; i < num_packages; i++)
  {
    result = rl_get_installed_package(rl_licence, i, &package_name);
    if (rlr_success != result)
      return;
    printf("%s\n", package_name);
  }
}
#endif


/** Check creating rinlicence structure for R300 */
static void test_rinlicence_init_r300(CuTest *tc)
{
  enum e_rl_result result;
  struct s_rl_licence *rl_licence;
  uint32_t package_licence_codes[NUM_PACKAGE_LICENCE_CODES];
  uint32_t num_package_licence_codes = 0;

  /* Check dodgy parameters */
  result = rl_init_r300(NULL,        0, package_licence_codes, &num_package_licence_codes, NUM_PACKAGE_LICENCE_CODES, NULL);
  CuAssertIntEquals(tc, rlr_paramfail, result);
  result = rl_init_r300(&rl_licence, 0, NULL,                  &num_package_licence_codes, NUM_PACKAGE_LICENCE_CODES, NULL);
  CuAssertIntEquals(tc, rlr_paramfail, result);
  result = rl_init_r300(&rl_licence, 0, package_licence_codes, NULL,                       NUM_PACKAGE_LICENCE_CODES, NULL);
  CuAssertIntEquals(tc, rlr_paramfail, result);

  /* Create a real one with no installed licensable packages */
  result = rl_init_r300(&rl_licence, 0, package_licence_codes, &num_package_licence_codes, NUM_PACKAGE_LICENCE_CODES, NULL);
  CuAssertIntEquals(tc, rlr_success, result);
  CuAssertIntEquals(tc, 0, utarray_len(rl_licence->installed_licensable_packages));
  (void) rl_destroy(rl_licence);

  /* Create a real one with one installed licensable package */
  result = rl_init_r300(&rl_licence, 0, package_licence_codes, &num_package_licence_codes, NUM_PACKAGE_LICENCE_CODES, "K342");
  CuAssertIntEquals(tc, rlr_success, result);
  CuAssertIntEquals(tc, 1, utarray_len(rl_licence->installed_licensable_packages));
  (void) rl_destroy(rl_licence);
}


/** Check creating rinlicence structure for R400 */
static void test_rinlicence_init_r400(CuTest *tc)
{
  enum e_rl_result result;
  struct s_rl_licence *rl_licence;
  uint32_t package_licence_codes[NUM_PACKAGE_LICENCE_CODES];
  uint32_t num_package_licence_codes = 0;

  /* Check dodgy parameters */
  result = rl_init_r400(NULL,        0, package_licence_codes, &num_package_licence_codes, NUM_PACKAGE_LICENCE_CODES, NULL);
  CuAssertIntEquals(tc, rlr_paramfail, result);
  result = rl_init_r400(&rl_licence, 0, NULL,                  &num_package_licence_codes, NUM_PACKAGE_LICENCE_CODES, NULL);
  CuAssertIntEquals(tc, rlr_paramfail, result);
  result = rl_init_r400(&rl_licence, 0, package_licence_codes, NULL,                       NUM_PACKAGE_LICENCE_CODES, NULL);
  CuAssertIntEquals(tc, rlr_paramfail, result);

  /* Create a real one with no installed licensable packages */
  result = rl_init_r400(&rl_licence, 0, package_licence_codes, &num_package_licence_codes, NUM_PACKAGE_LICENCE_CODES, NULL);
  CuAssertIntEquals(tc, rlr_success, result);
  CuAssertIntEquals(tc, 0, utarray_len(rl_licence->installed_licensable_packages));
  (void) rl_destroy(rl_licence);

  /* Create a real one with one installed licensable package */
  result = rl_init_r400(&rl_licence, 0, package_licence_codes, &num_package_licence_codes, NUM_PACKAGE_LICENCE_CODES, "K402");
  CuAssertIntEquals(tc, rlr_success, result);
  CuAssertIntEquals(tc, 1, utarray_len(rl_licence->installed_licensable_packages));
  (void) rl_destroy(rl_licence);
}


/** Check creating rinlicence structure for C500 */
static void test_rinlicence_init_c500(CuTest *tc)
{
  enum e_rl_result result;
  struct s_rl_licence *rl_licence;
  uint32_t package_licence_codes[NUM_PACKAGE_LICENCE_CODES];
  uint32_t num_package_licence_codes = 0;

  /* Check dodgy parameters */
  result = rl_init_c500(NULL,        0, package_licence_codes, &num_package_licence_codes, NUM_PACKAGE_LICENCE_CODES, opkg_executable_0_licensed);
  CuAssertIntEquals(tc, rlr_paramfail, result);
  result = rl_init_c500(&rl_licence, 0, NULL,                  &num_package_licence_codes, NUM_PACKAGE_LICENCE_CODES, opkg_executable_0_licensed);
  CuAssertIntEquals(tc, rlr_paramfail, result);
  result = rl_init_c500(&rl_licence, 0, package_licence_codes, NULL,                       NUM_PACKAGE_LICENCE_CODES, opkg_executable_0_licensed);
  CuAssertIntEquals(tc, rlr_paramfail, result);

  /* Create a real one with no licensed packages (no opkg) */
  result = rl_init_c500(&rl_licence, 0, package_licence_codes, &num_package_licence_codes, NUM_PACKAGE_LICENCE_CODES, NULL);
  CuAssertIntEquals(tc, rlr_success, result);
  CuAssertIntEquals(tc, 0, utarray_len(rl_licence->installed_licensable_packages));
  (void) rl_destroy(rl_licence);

  /* Create a real one with no licensed packages (with opkg) */
  result = rl_init_c500(&rl_licence, 0, package_licence_codes, &num_package_licence_codes, NUM_PACKAGE_LICENCE_CODES, opkg_executable_0_licensed);
  CuAssertIntEquals(tc, rlr_success, result);
  /* list_installed_packages(rl_licence); */
  CuAssertIntEquals(tc, 0, utarray_len(rl_licence->installed_licensable_packages));
  (void) rl_destroy(rl_licence);

  /* Create a real one with one licensed package */
  result = rl_init_c500(&rl_licence, 0, package_licence_codes, &num_package_licence_codes, NUM_PACKAGE_LICENCE_CODES, opkg_executable_1_licensed);
  CuAssertIntEquals(tc, rlr_success, result);
  /* list_installed_packages(rl_licence); */
  CuAssertIntEquals(tc, 1, utarray_len(rl_licence->installed_licensable_packages));
  (void) rl_destroy(rl_licence);
}


/** Check cleaning up rinlicence structure */
static void test_rinlicence_destroy(CuTest *tc)
{
  enum e_rl_result result;
  struct s_rl_licence *rl_licence;
  uint32_t package_licence_codes[NUM_PACKAGE_LICENCE_CODES];
  uint32_t num_package_licence_codes = 0;

  /* Allocate needs to work so we can test destroy */
  result = rl_init_c500(&rl_licence, 0, package_licence_codes, &num_package_licence_codes, NUM_PACKAGE_LICENCE_CODES, opkg_executable_0_licensed);
  CuAssertIntEquals(tc, rlr_success, result);

  /* Check dodgy parameters */
  result = rl_destroy(NULL);
  CuAssertIntEquals(tc, rlr_paramfail, result);

  /* Try a real destroy */
  result = rl_destroy(rl_licence);
  CuAssertIntEquals(tc, rlr_success, result);
}


CuSuite *CuGetSuite_rinlicence_init(void)
{
  CuSuite *suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_rinlicence_init_r300);
  SUITE_ADD_TEST(suite, test_rinlicence_init_r400);
  SUITE_ADD_TEST(suite, test_rinlicence_init_c500);
  SUITE_ADD_TEST(suite, test_rinlicence_destroy);

  return suite;
}
