/*
 * rl_test_pkgtype.c
 *
 *  Created on: 4 Sep. 2018
 *      Author: ian
 */
#include <rinlicence/rl_licence.h>
#include <cutest/CuTest.h>

#include "rl_test_pkgtype.h"


#define NUM_PACKAGE_LICENCE_CODES 10


static const char opkg_executable_4_licensed[] = "./opkg-4-licensed";


/** Check package type for the R300 */
static void test_package_type_r300(CuTest *tc)
{
  enum e_rl_result result;
  struct s_rl_licence *rl_licence;
  uint32_t num_package_licence_codes = 0;
  uint32_t package_licence_codes[NUM_PACKAGE_LICENCE_CODES];
  enum e_rl_package_type package_type;

  /* Create a R300 rin licence */
  result = rl_init_r300(&rl_licence, 0, package_licence_codes, &num_package_licence_codes, NUM_PACKAGE_LICENCE_CODES, NULL);
  CuAssertIntEquals(tc, rlr_success, result);

  result = rl_get_package_type(NULL, "C500-K501", &package_type);
  CuAssertIntEquals(tc, rlr_paramfail, result);
  result = rl_get_package_type(rl_licence, NULL, &package_type);
  CuAssertIntEquals(tc, rlr_paramfail, result);
  result = rl_get_package_type(rl_licence, "C500-K501", NULL);
  CuAssertIntEquals(tc, rlr_paramfail, result);

  /* All packages are primary on R300 */
  result = rl_get_package_type(rl_licence, "K342", &package_type);
  CuAssertIntEquals(tc, rlr_success, result);
  CuAssertIntEquals(tc, rlpt_primary, package_type);

  /* Clean up */
  (void) rl_destroy(rl_licence);
}


/** Check package type for the R400 */
static void test_package_type_r400(CuTest *tc)
{
  enum e_rl_result result;
  struct s_rl_licence *rl_licence;
  uint32_t num_package_licence_codes = 0;
  uint32_t package_licence_codes[NUM_PACKAGE_LICENCE_CODES];
  enum e_rl_package_type package_type;

  /* Create a R400 rin licence */
  result = rl_init_r400(&rl_licence, 0, package_licence_codes, &num_package_licence_codes, NUM_PACKAGE_LICENCE_CODES, NULL);
  CuAssertIntEquals(tc, rlr_success, result);

  result = rl_get_package_type(NULL, "C500-K501", &package_type);
  CuAssertIntEquals(tc, rlr_paramfail, result);
  result = rl_get_package_type(rl_licence, NULL, &package_type);
  CuAssertIntEquals(tc, rlr_paramfail, result);
  result = rl_get_package_type(rl_licence, "C500-K501", NULL);
  CuAssertIntEquals(tc, rlr_paramfail, result);

  /* All packages are primary on R400 */
  result = rl_get_package_type(rl_licence, "K481", &package_type);
  CuAssertIntEquals(tc, rlr_success, result);
  CuAssertIntEquals(tc, rlpt_primary, package_type);

  /* Clean up */
  (void) rl_destroy(rl_licence);
}


/** Check package type for the C500 */
static void test_package_type_c500(CuTest *tc)
{
  enum e_rl_result result;
  struct s_rl_licence *rl_licence;
  uint32_t num_package_licence_codes = 0;
  uint32_t package_licence_codes[NUM_PACKAGE_LICENCE_CODES];
  enum e_rl_package_type package_type;

  /* Create a C500 licensing structure */
  result = rl_init_c500(&rl_licence, 0, package_licence_codes, &num_package_licence_codes, NUM_PACKAGE_LICENCE_CODES, opkg_executable_4_licensed);
  CuAssertIntEquals(tc, rlr_success, result);

  result = rl_get_package_type(NULL, "C500-K501", &package_type);
  CuAssertIntEquals(tc, rlr_paramfail, result);
  result = rl_get_package_type(rl_licence, NULL, &package_type);
  CuAssertIntEquals(tc, rlr_paramfail, result);
  result = rl_get_package_type(rl_licence, "C500-K501", NULL);
  CuAssertIntEquals(tc, rlr_paramfail, result);

  result = rl_get_package_type(rl_licence, "C500-K501", &package_type);
  CuAssertIntEquals(tc, rlr_success, result);
  CuAssertIntEquals(tc, rlpt_primary, package_type);

  result = rl_get_package_type(rl_licence, "L900-502", &package_type);
  CuAssertIntEquals(tc, rlr_success, result);
  CuAssertIntEquals(tc, rlpt_addon, package_type);

  result = rl_get_package_type(rl_licence, "we21-503-m01", &package_type);
  CuAssertIntEquals(tc, rlr_success, result);
  CuAssertIntEquals(tc, rlpt_other, package_type);

  /* Clean up */
  (void) rl_destroy(rl_licence);
}


CuSuite *CuGetSuite_rinlicence_package_type(void)
{
  CuSuite *suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_package_type_r300);
  SUITE_ADD_TEST(suite, test_package_type_r400);
  SUITE_ADD_TEST(suite, test_package_type_c500);

  return suite;
}
