/*
 * rl_test_regex.c
 *
 *  Created on: Nov 28, 2016
 *      Author: ian
 */
#include <stdbool.h>
#include <rinlicence/rl_licence.h>

#include "rl_test_regex.h"


#define ARRAY_SIZE(x) ((sizeof x) / (sizeof *x))


#define NUM_PACKAGE_LICENCE_CODES 2


static const char opkg_executable_regex_test[] = "./opkg-regex-test";


struct s_pkg_install
{
  const char *package_name;
  bool is_installed;
};


/** These are the packages listed by opkg-regex-test, and whether they are licensable packages or not */
static struct s_pkg_install pkg_install[] =
{
  { "L000-502", false },
  { "L000-514", false },
  { "L000-520", false },
  { "L000-521", false },
  { "c500-500", false },
  { "C500-K500", true },
  { "C500-K599", true },
  { "C511-K501", true },
  { "C599-K501", true },
  { "C50a-K501", false },
  { "C500-X500", false },
  { "C500-K401", false },
  { "we21-503-m01", false },
  { "we21-503-m05", false },
  { "we21-517", false },
  { "L900-500", true  },
  { "L900-501", true  },
  { "L900-520", true  },
  { "L901-500", true  },
  { "L920-500", true  },
  { "L800-500", false }
};


/** Check the regex for the C500 licensable packages */
static void test_rinlicence_regex_c500(CuTest *tc)
{
  enum e_rl_result result;
  struct s_rl_licence *rl_licence;
  uint32_t package_licence_codes[NUM_PACKAGE_LICENCE_CODES];
  uint32_t num_package_licence_codes = 0;
  bool is_installed;
  size_t i;

  /* Create C500 licensing */
  result = rl_init_c500(&rl_licence, 0, package_licence_codes, &num_package_licence_codes, NUM_PACKAGE_LICENCE_CODES, opkg_executable_regex_test);
  CuAssertIntEquals(tc, rlr_success, result);

  /* Check installed packages against regex */
  for (i = 0; i < ARRAY_SIZE(pkg_install); i++)
  {
    result = rl_package_is_installed(rl_licence, pkg_install[i].package_name, &is_installed);
    CuAssertIntEquals(tc, rlr_success, result);
    CuAssertIntEquals(tc, (int) pkg_install[i].is_installed, (int) is_installed);
  }

  /* Clean up */
  (void) rl_destroy(rl_licence);
}


CuSuite *CuGetSuite_rinlicence_regex(void)
{
  CuSuite *suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_rinlicence_regex_c500);

  return suite;
}
