/*
 * rl_test_remove.c
 *
 *  Created on: Nov 28, 2016
 *      Author: ian
 */
#include <rinlicence/rl_licence.h>
#include <cutest/CuTest.h>

#include "rl_test_remove.h"


static const char opkg_executable_1_licensed[] = "./opkg-1-licensed";


/** Check removing licence codes when the package is no longer installed */
static void test_rinlicence_remove(CuTest *tc)
{
  enum e_rl_result result;
  struct s_rl_licence *rl_licence;
  #define NUM_PLC_ALL_LICENSED 4
  /*                                                       L900-520,  L900-500,  L940-500, c500-500 */
  uint32_t package_licence_codes[NUM_PLC_ALL_LICENSED] = { 280668447, 138093037, 33255967, 216341909 };
  uint32_t num_package_licence_codes = 4;
  uint32_t serial_number = 3382549;

  /* Create a C500 licensing structure */
  result = rl_init_c500(&rl_licence, serial_number, package_licence_codes, &num_package_licence_codes, NUM_PLC_ALL_LICENSED, opkg_executable_1_licensed);
  CuAssertIntEquals(tc, rlr_success, result);

  /* Check dodgy parameters to rl_unlicense_package() */
  result = rl_unlicense_package(NULL, "");
  CuAssertIntEquals(tc, rlr_paramfail, result);
  result = rl_unlicense_package(rl_licence, NULL);
  CuAssertIntEquals(tc, rlr_paramfail, result);

  /* L900-500 is the only package installed */

  /* Remove L900-520 package */
  result = rl_unlicense_package(rl_licence, "L900-520");
  CuAssertIntEquals(tc, rlr_success, result);
  CuAssertIntEquals(tc, 3, num_package_licence_codes);
  CuAssertIntEquals(tc, 138093037, package_licence_codes[0]);
  CuAssertIntEquals(tc, 33255967, package_licence_codes[1]);
  CuAssertIntEquals(tc, 216341909, package_licence_codes[2]);

  /* Try to remove L900-520 again */
  result = rl_unlicense_package(rl_licence, "L900-520");
  CuAssertIntEquals(tc, rlr_licence_not_removed, result);

  /* Remove L940-500 package */
  result = rl_unlicense_package(rl_licence, "L940-500");
  CuAssertIntEquals(tc, rlr_success, result);
  CuAssertIntEquals(tc, 2, num_package_licence_codes);
  CuAssertIntEquals(tc, 138093037, package_licence_codes[0]);
  CuAssertIntEquals(tc, 216341909, package_licence_codes[1]);

  /* Remove c500-500 package */
  result = rl_unlicense_package(rl_licence, "c500-500");
  CuAssertIntEquals(tc, rlr_success, result);
  CuAssertIntEquals(tc, 1, num_package_licence_codes);
  CuAssertIntEquals(tc, 138093037, package_licence_codes[0]);

  /* Remove L900-500 package */
  result = rl_unlicense_package(rl_licence, "L900-500");
  CuAssertIntEquals(tc, rlr_success, result);
  CuAssertIntEquals(tc, 0, num_package_licence_codes);

  /* Clean up */
  result = rl_destroy(rl_licence);
  CuAssertIntEquals(tc, rlr_success, result);
}


/** Check flushing all licence codes for packages no longer installed */
static void test_rinlicence_flush(CuTest *tc)
{
  enum e_rl_result result;
  struct s_rl_licence *rl_licence;
  #define NUM_PLC_ALL_LICENSED 4
  /*                                                       L900-520,  L900-500,  L940-500, c500-500 */
  uint32_t package_licence_codes[NUM_PLC_ALL_LICENSED] = { 280668447, 138093037, 33255967, 216341909 };
  uint32_t num_package_licence_codes = 4;
  uint32_t serial_number = 3382549;

  /* Create a C500 licensing structure */
  result = rl_init_c500(&rl_licence, serial_number, package_licence_codes, &num_package_licence_codes, NUM_PLC_ALL_LICENSED, opkg_executable_1_licensed);
  CuAssertIntEquals(tc, rlr_success, result);

  /* Check dodgy parameters to rl_flush_licence_codes_for_uninstalled_packages() */
  result = rl_flush_licence_codes_for_uninstalled_packages(NULL);
  CuAssertIntEquals(tc, rlr_paramfail, result);

  /* L900-500 is the only package installed */

  /* Flush all licence codes for packages that are no longer installed */
  result = rl_flush_licence_codes_for_uninstalled_packages(rl_licence);
  CuAssertIntEquals(tc, rlr_success, result);
  CuAssertIntEquals(tc, 1, num_package_licence_codes);
  CuAssertIntEquals(tc, 138093037, package_licence_codes[0]);

  /* Flush again shouldn't remove anything */
  result = rl_flush_licence_codes_for_uninstalled_packages(rl_licence);
  CuAssertIntEquals(tc, rlr_licence_not_removed, result);
  CuAssertIntEquals(tc, 1, num_package_licence_codes);
  CuAssertIntEquals(tc, 138093037, package_licence_codes[0]);

  /* Clean up */
  result = rl_destroy(rl_licence);
  CuAssertIntEquals(tc, rlr_success, result);
}


CuSuite *CuGetSuite_rinlicence_remove(void)
{
  CuSuite *suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_rinlicence_remove);
  SUITE_ADD_TEST(suite, test_rinlicence_flush);

  return suite;
}
