/*
 * rl_test_string.c
 *
 *  Created on: Nov 15, 2016
 *      Author: ian
 */
#include <rinlicence/rl_licence.h>
#include <cutest/CuTest.h>

#include "rl_test_init.h"


#define LICENCE_CODE_STR_LEN 100


/** Integer power function */
unsigned int int_power(unsigned int x, unsigned int n)
{
  unsigned int i;
  unsigned int number = 1;

  for (i = 0; i < n; i++)
    number *= x;

  return number;
}


/** Check dodgy parameters converting licence code to a string */
static void check_rinlicence_to_string_dodgy_param(CuTest *tc, struct s_rl_licence *rl_licence)
{
  enum e_rl_result result;
  char licence_code_str[LICENCE_CODE_STR_LEN];

  /* NULL params */
  result = rl_licence_code_to_string(NULL, 0, licence_code_str, LICENCE_CODE_STR_LEN);
  CuAssertIntEquals(tc, rlr_paramfail, result);
  result = rl_licence_code_to_string(rl_licence, 0, NULL, LICENCE_CODE_STR_LEN);
  CuAssertIntEquals(tc, rlr_paramfail, result);

  /* Bad length */
  result = rl_licence_code_to_string(rl_licence, 0, licence_code_str, 0);
  CuAssertIntEquals(tc, rlr_paramfail, result);
}


/** Check dodgy parameters converting string to licence code */
static void check_rinlicence_from_string_dodgy_param(CuTest *tc, struct s_rl_licence *rl_licence)
{
  enum e_rl_result result;
  uint32_t licence_code;

  /* NULL params */
  result = rl_string_to_licence_code(NULL, "000000", &licence_code);
  CuAssertIntEquals(tc, rlr_paramfail, result);
  result = rl_string_to_licence_code(rl_licence, NULL, &licence_code);
  CuAssertIntEquals(tc, rlr_paramfail, result);
  result = rl_string_to_licence_code(rl_licence, "000000", NULL);
  CuAssertIntEquals(tc, rlr_paramfail, result);
}


/** Check converting licence code to a string */
static void check_rinlicence_to_string(CuTest *tc, struct s_rl_licence *rl_licence, uint32_t licence_code, const char *expected_response)
{
  enum e_rl_result result;
  char licence_code_str[LICENCE_CODE_STR_LEN];

  result = rl_licence_code_to_string(rl_licence, licence_code, licence_code_str, LICENCE_CODE_STR_LEN);
  CuAssertIntEquals(tc, rlr_success, result);
  CuAssertStrEquals(tc, expected_response, licence_code_str);
}


/** Check converting string to licence code */
static void check_rinlicence_from_string(CuTest *tc, struct s_rl_licence *rl_licence, const char *licence_code_str, uint32_t expected_licence_code)
{
  enum e_rl_result result;
  uint32_t licence_code;

  result = rl_string_to_licence_code(rl_licence, licence_code_str, &licence_code);
  CuAssertIntEquals(tc, rlr_success, result);
  CuAssertIntEquals(tc, expected_licence_code, licence_code);
}


/** Check converting R300 licence code to a string */
static void test_rinlicence_to_string_r300(CuTest *tc)
{
  enum e_rl_result result;
  struct s_rl_licence *rl_licence;
  uint32_t package_licence_codes[1];
  uint32_t num_package_licence_codes = 0;

  /* Create a R300 rin licence */
  result = rl_init_r300(&rl_licence, 0, package_licence_codes, &num_package_licence_codes, 1, NULL);
  CuAssertIntEquals(tc, rlr_success, result);

  /* Bad params */
  check_rinlicence_to_string_dodgy_param(tc, rl_licence);
  check_rinlicence_from_string_dodgy_param(tc, rl_licence);

  /* Run some convert to string checks */
  check_rinlicence_to_string(tc, rl_licence, 0, "000000");
  check_rinlicence_to_string(tc, rl_licence, 1, "000001");
  check_rinlicence_to_string(tc, rl_licence, 999999, "999999");

  /* Run some convert from string checks */
  check_rinlicence_from_string(tc, rl_licence, "000000", 0);
  check_rinlicence_from_string(tc, rl_licence, "000001", 1);
  check_rinlicence_from_string(tc, rl_licence, "999999", 999999);

  /* Clean up */
  (void) rl_destroy(rl_licence);
}


/** Check converting R400 licence code to a string */
static void test_rinlicence_to_string_r400(CuTest *tc)
{
  enum e_rl_result result;
  struct s_rl_licence *rl_licence;
  uint32_t package_licence_codes[1];
  uint32_t num_package_licence_codes = 0;

  /* Create a R400 rin licence */
  result = rl_init_r400(&rl_licence, 0, package_licence_codes, &num_package_licence_codes, 1, NULL);
  CuAssertIntEquals(tc, rlr_success, result);

  /* Bad params */
  check_rinlicence_to_string_dodgy_param(tc, rl_licence);
  check_rinlicence_from_string_dodgy_param(tc, rl_licence);

  /* Run some convert to string checks */
  check_rinlicence_to_string(tc, rl_licence, 0, "000000");
  check_rinlicence_to_string(tc, rl_licence, 1, "000001");
  check_rinlicence_to_string(tc, rl_licence, 999999, "999999");

  /* Run some convert from string checks */
  check_rinlicence_from_string(tc, rl_licence, "000000", 0);
  check_rinlicence_from_string(tc, rl_licence, "000001", 1);
  check_rinlicence_from_string(tc, rl_licence, "999999", 999999);

  /* Clean up */
  (void) rl_destroy(rl_licence);
}


/** Check converting C500 licence code to a string */
static void test_rinlicence_to_string_c500(CuTest *tc)
{
  enum e_rl_result result;
  struct s_rl_licence *rl_licence;
  uint32_t package_licence_codes[1];
  uint32_t num_package_licence_codes = 0;

  /* Create a R400 rin licence */
  result = rl_init_c500(&rl_licence, 0, package_licence_codes, &num_package_licence_codes, 1, NULL);
  CuAssertIntEquals(tc, rlr_success, result);

  /* Bad params */
  check_rinlicence_to_string_dodgy_param(tc, rl_licence);
  check_rinlicence_from_string_dodgy_param(tc, rl_licence);

  /* Run some convert to string checks */
  check_rinlicence_to_string(tc, rl_licence, 0, "BBBBBB");
  check_rinlicence_to_string(tc, rl_licence, 1, "BBBBBC");
  check_rinlicence_to_string(tc, rl_licence, int_power(26, 6) - 1, "999999");

  /* Run some convert from string checks */
  check_rinlicence_from_string(tc, rl_licence, "BBBBBB", 0);
  check_rinlicence_from_string(tc, rl_licence, "BBBBBC", 1);
  check_rinlicence_from_string(tc, rl_licence, "999999", int_power(26, 6) - 1);

  /* Clean up */
  (void) rl_destroy(rl_licence);
}


CuSuite *CuGetSuite_rinlicence_string(void)
{
  CuSuite *suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_rinlicence_to_string_r300);
  SUITE_ADD_TEST(suite, test_rinlicence_to_string_r400);
  SUITE_ADD_TEST(suite, test_rinlicence_to_string_c500);

  return suite;
}
