/*
 * rp_checksum.c
 *
 *  Created on: Aug 16, 2017
 *      Author: ian
 */
#include <stdio.h>
#include <stdint.h>
#include <rinprog/rinprog.h>
#include "rp_util.h"

#include "rp_checksum.h"


/** Verify stream in has a valid checksum at one of blocksize intervals */
enum e_rp_result rp_verify_buffer_block_checksum(FILE *in, uint16_t seed, uint32_t start_offs, uint32_t blocksize)
{
  uint32_t i;
  uint32_t size;
  uint8_t data;
  uint8_t chk1;
  uint8_t chk2;
  uint16_t sum;
  uint16_t bufsum;
  enum e_rp_result result;

  /* Find size of stream */
  result = rp_get_file_length(in, &size);
  if (rpr_success != result)
    return result;

  /* Seek start offset */
  if (fseek(in, start_offs, SEEK_SET) != 0)
    return rpr_iofail;

  /* Sum all data, looking for checksum at block boundaries */
  for (i = start_offs, sum = seed; i < size - 2; )
  {
    if (fread(&data, sizeof(data), 1, in) != 1)
      return rpr_iofail;
    sum += data;
    i++;

    if (((i + 2) % blocksize) == 0)
    {
      /* At block boundary, read checksum, and compare */
      if (fread(&chk1, sizeof(chk1), 1, in) != 1)
        return rpr_iofail;
      bufsum = chk1;
      bufsum <<= 8;
      if (fread(&chk2, sizeof(chk2), 1, in) != 1)
        return rpr_iofail;
      bufsum |= chk2;
      if (bufsum == sum)
      {
        return rpr_success;
      }
      else
      {
        sum += chk1;
        sum += chk2;
        i += 2;
      }
    }
  }

  return rpr_checksumfail;
}


/** Verify there is a valid checksum at the end of stream in */
enum e_rp_result rp_verify_buffer_all_checksum(FILE *in, uint16_t seed, uint32_t start_offs)
{
  uint32_t size;
  enum e_rp_result result;

  /* Find size of stream */
  result = rp_get_file_length(in, &size);
  if (rpr_success != result)
    return result;

  return rp_verify_buffer_block_checksum(in, seed, start_offs, size);
}
