/*
 * rp_srecord.c
 *
 *  Created on: Aug 17, 2017
 *      Author: ian
 */
#include <stdio.h>
#include <libGIS/srecord.h>
#include <rinprog/rinprog.h>
#include "rp_util.h"

#include "rp_srecord.h"


/** Write a single s-record out to file */
static enum e_rp_result rp_srecord_write_out(FILE *bin_file_out, const SRecord *const srec, const uint8_t pad, long *end)
{
  /* Pad from previous end to where we want to write now */
  if (fseek(bin_file_out, *end, SEEK_SET) < 0)
    return rpr_iofail;
  while (*end < srec->address)
  {
    if (fwrite(&pad, sizeof(pad), 1, bin_file_out) != 1)
      return rpr_iofail;
    (*end) += sizeof(pad);
  }

  /* Write the srecord data into the file at specified address */
  if (fseek(bin_file_out, srec->address, SEEK_SET) < 0)
    return rpr_iofail;
  if (fwrite(srec->data, srec->dataLen, 1, bin_file_out) != 1)
    return rpr_iofail;

  /* Update end position */
  if (srec->address + srec->dataLen > *end)
    *end = srec->address + srec->dataLen;
  return rpr_success;
}


/** Read an s-record file from s_file_in, and write its binary equivalent to bin_file_out, padding unused space with pad */
enum e_rp_result rp_read_srecord_file(FILE *s_file_in, FILE *bin_file_out, const uint8_t pad)
{
  SRecord srec;
  int sres;
  enum e_rp_result result;
  long end = 0;

  if (NULL == s_file_in || NULL == bin_file_out)
    return rpr_paramfail;

  do
  {
    /* Read the record from the file */
    sres = Read_SRecord(&srec, s_file_in);
    if (SRECORD_ERROR_EOF == sres)
      break;
    else if (SRECORD_OK != sres)
      return rpr_iofail;

    /* Write data records to output file */
    switch (srec.type)
    {
      case SRECORD_TYPE_S1:
      case SRECORD_TYPE_S2:
      case SRECORD_TYPE_S3:
        result = rp_srecord_write_out(bin_file_out, &srec, pad, &end);
        if (rpr_success != result)
          return result;
        break;

      default:
        break;
    }
  }
  while (!feof(s_file_in));

  return rpr_success;
}
