/*
 * rp_teacrypt.c
 *
 *  Created on: Aug 10, 2017
 *      Author: ian
 */

#include <stdint.h>
#include <stdio.h>

#include "rp_teacrypt.h"
#include <rinprog/rinprog.h>


typedef void (*rp_cbc_tea_enc_dec_t)(struct s_rp_tea_data *const tea_v, struct s_rp_tea_data *const tea_iv, const struct s_rp_tea_key *const tea_k);


void rp_tea_encrypt(struct s_rp_tea_data *const tea_v, const struct s_rp_tea_key *const tea_k)
{
  uint32_t y, z, sum, delta;
  size_t n;

  y = tea_v->data[0];
  z = tea_v->data[1];
  sum = 0;
  delta = 0x9E3779B9;

  for (n = 0; n < 32; n++)
  {
    sum += delta;
    y += (((z << 4) + tea_k->key[0]) ^ (z + sum) ^ ((z >> 5) + tea_k->key[1]));
    z += (((y << 4) + tea_k->key[2]) ^ (y + sum) ^ ((y >> 5) + tea_k->key[3]));
  }
  tea_v->data[0] = y;
  tea_v->data[1] = z;
}


void rp_tea_decrypt(struct s_rp_tea_data *const tea_v, const struct s_rp_tea_key *const tea_k)
{
  uint32_t y, z, sum, delta;
  size_t n;

  y = tea_v->data[0];
  z = tea_v->data[1];
  delta = 0x9E3779B9;
  sum = delta << 5;

  for (n = 0; n < 32; n++)
  {
    z -= (((y << 4) + tea_k->key[2]) ^ (y + sum) ^ ((y >> 5) + tea_k->key[3]));
    y -= (((z << 4) + tea_k->key[0]) ^ (z + sum) ^ ((z >> 5) + tea_k->key[1]));
    sum -= delta;
  }
  tea_v->data[0] = y;
  tea_v->data[1] = z;
}


void rp_cbc_tea_encrypt(struct s_rp_tea_data *const tea_v, struct s_rp_tea_data *const tea_iv, const struct s_rp_tea_key *const tea_k)
{
  tea_v->data[0] ^= tea_iv->data[0];
  tea_v->data[1] ^= tea_iv->data[1];
  rp_tea_encrypt(tea_v, tea_k);
  *tea_iv = *tea_v;
}


void rp_cbc_tea_decrypt(struct s_rp_tea_data *const tea_v, struct s_rp_tea_data *const tea_iv, const struct s_rp_tea_key *const tea_k)
{
  struct s_rp_tea_data tea_temp_v;

  tea_temp_v = *tea_v;
  rp_tea_decrypt(tea_v, tea_k);
  tea_v->data[0] ^= tea_iv->data[0];
  tea_v->data[1] ^= tea_iv->data[1];
  *tea_iv = tea_temp_v;
}


#if 0
static enum e_rp_result rp_tea_encrypt_decrypt_stream(FILE *in, FILE *out, const struct s_rp_tea_data *const tea_iv, const struct s_rp_tea_key *const tea_k, rp_cbc_tea_enc_dec_t cbc_tea_enc_dec)
{
  long int size;
  struct s_rp_tea_data tea_iiv = *tea_iv;
  struct s_rp_tea_data tea_v;

  /* Find size of stream */
  if (fseek(in, 0, SEEK_END) != 0)
    return rpr_iofail;
  size = ftell(in);
  if (size < 0)
    return rpr_iofail;
  if (fseek(in, 0, SEEK_SET) != 0)
    return rpr_iofail;

  /* Size must be a multiple of tea_v */
  if ((size % sizeof(tea_v)) != 0)
  {
    return rpr_streamsizefail;
  }

  /* Read from in stream, encrypt/decrypt and write to out stream */
  do
  {
    if (fread(&tea_v, sizeof(tea_v), 1, in) != 1 && ferror(in))
      return rpr_iofail;
    cbc_tea_enc_dec(&tea_v, &tea_iiv, tea_k);
    if (fwrite(&tea_v, sizeof(tea_v), 1, out) != 1 && ferror(out))
      return rpr_iofail;
  }
  while (!feof(in));
  return rpr_success;
}


enum e_rp_result rp_tea_encrypt_stream(FILE *in, FILE *out, const struct s_rp_tea_data *const tea_iv, const struct s_rp_tea_key *const tea_k)
{
  return rp_tea_encrypt_decrypt_stream(in, out, tea_iv, tea_k, rp_cbc_tea_encrypt);
}


enum e_rp_result rp_tea_decrypt_stream(FILE *in, FILE *out, const struct s_rp_tea_data *const tea_iv, const struct s_rp_tea_key *const tea_k)
{
  return rp_tea_encrypt_decrypt_stream(in, out, tea_iv, tea_k, rp_cbc_tea_decrypt);
}
#endif

