/*
 * rp_test_checksum.c
 *
 *  Created on: Aug 22, 2017
 *      Author: ian
 */
#include <string.h>
#include <stdio.h>
#include <rinprog/rp_devini.h>
#include <rinprog/rp_checksum.h>
#include <rinprog/rp_srecord.h>
#include <cutest/CuTest.h>

#include "rp_test_checksum.h"


/** Read device and s-record file */
static void read_device_and_srecords(CuTest* tc, const char *dev_filename, const char *srec_filename, struct s_rp_device_settings **settings, FILE **bin_out)
{
  enum e_rp_result result;
  FILE *dev_in;
  FILE *srec_in;

  /* Open device file */
  dev_in = fopen(dev_filename, "r");
  CuAssertPtrNotNull(tc, dev_in);

  /* Read settings from device file */
  result = rp_device_settings_read(dev_in, settings);
  CuAssertIntEquals(tc, rpr_success, result);

  fclose(dev_in);

  /* Open s-record file */
  srec_in = fopen(srec_filename, "r");
  CuAssertPtrNotNull(tc, srec_in);

  /* Create binary file */
  *bin_out = tmpfile();
  CuAssertPtrNotNull(tc, *bin_out);

  /* Read s-records into binary file */
  result = rp_read_srecord_file(srec_in, *bin_out, 0xFF);
  CuAssertIntEquals(tc, rpr_success, result);

  fclose(srec_in);
}


/** Verify block checksum */
static void test_rinprog_block_checksum(CuTest* tc)
{
  enum e_rp_result result;
  struct s_rp_device_settings *settings;
  FILE *bin_out;

  /* Get settings and s-records from file */
  read_device_and_srecords(tc, "R400.dev", "R400-507-301-M01.mot", &settings, &bin_out);

  /* Verify there is a valid checksum somewhere in the file */
  result = rp_verify_buffer_block_checksum(bin_out, settings->flash_checksum_seed, settings->flash_start, settings->flash_block_size);
  CuAssertIntEquals(tc, rpr_success, result);

  /* Free up */
  result = rp_device_settings_free(settings);
  CuAssertIntEquals(tc, rpr_success, result);
  fclose(bin_out);
}


/** Verify all checksum */
static void test_rinprog_all_checksum(CuTest* tc)
{
  enum e_rp_result result;
  struct s_rp_device_settings *settings;
  FILE *bin_out;

  /* Get settings and s-records from file */
  read_device_and_srecords(tc, "R320.dev", "R300-501-394-M01.mot", &settings, &bin_out);

  /* Verify there is a valid checksum at the end of the file */
  result = rp_verify_buffer_all_checksum(bin_out, settings->flash_checksum_seed, settings->flash_start);
  CuAssertIntEquals(tc, rpr_success, result);

  /* Free up */
  result = rp_device_settings_free(settings);
  CuAssertIntEquals(tc, rpr_success, result);
  fclose(bin_out);
}


CuSuite *CuGetSuite_rinprog_checksum(void)
{
  CuSuite *suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_rinprog_block_checksum);
  SUITE_ADD_TEST(suite, test_rinprog_all_checksum);

  return suite;
}
