/*
 * rp_test_devini.c
 *
 *  Created on: Aug 17, 2017
 *      Author: ian
 */
#include <string.h>
#include <rinprog/rp_devini.h>
#include <cutest/CuTest.h>

#include "rp_test_devini.h"


/** Test reading settings from device file */
static void test_rinprog_device_settings_read(CuTest* tc)
{
  enum e_rp_result result;
  struct s_rp_device_settings *settings;
  FILE *in;

  /* Open device file */
  in = fopen("R320.dev", "rb");
  CuAssertPtrNotNull(tc, in);

  /* Check bad parameters */
  result = rp_device_settings_read(NULL, &settings);
  CuAssertIntEquals(tc, rpr_paramfail, result);
  result = rp_device_settings_read(in, NULL);
  CuAssertIntEquals(tc, rpr_paramfail, result);

  /* Read settings from R320.dev */
  result = rp_device_settings_read(in, &settings);
  CuAssertIntEquals(tc, rpr_success, result);

  /* Check values */
  CuAssertIntEquals(tc, 9600, settings->connect_settings.baud_rate);
  CuAssertIntEquals(tc, 1, settings->connect_settings.stop_bits);
  CuAssertIntEquals(tc, 8, settings->connect_settings.data_bits);
  CuAssertIntEquals(tc, 'n', settings->connect_settings.parity_bits);

  CuAssertIntEquals(tc, 9600, settings->program_settings.baud_rate);
  CuAssertIntEquals(tc, 1, settings->program_settings.stop_bits);
  CuAssertIntEquals(tc, 8, settings->program_settings.data_bits);
  CuAssertIntEquals(tc, 'n', settings->program_settings.parity_bits);

  CuAssertStrEquals(tc, "FlAsHRuN", settings->boot_connect_string);
  CuAssertStrEquals(tc, "\x02PF0000\x03", settings->user_connect_string);
  CuAssertStrEquals(tc, "FlAsHRuN", settings->connect_response);

  CuAssertIntEquals(tc, 2048, settings->flash_start);
  CuAssertIntEquals(tc, 55296, settings->flash_length);
  CuAssertIntEquals(tc, 12800, settings->flash_checksum_seed);
  CuAssertIntEquals(tc, 4096, settings->flash_block_size);
  CuAssertIntEquals(tc, rfv_simple, settings->flash_verification);
  CuAssertTrue(tc, settings->flash_encrypted);

  /* Free up */
  result = rp_device_settings_free(settings);
  CuAssertIntEquals(tc, rpr_success, result);

  fclose(in);
}


CuSuite *CuGetSuite_rinprog_devini(void)
{
  CuSuite *suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_rinprog_device_settings_read);

  return suite;
}
