/*
 * rp_test_srecord.c
 *
 *  Created on: Aug 17, 2017
 *      Author: ian
 */
#define _GNU_SOURCE
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <rinprog/rp_srecord.h>
#include <rinprog/rp_util.h>
#include <cutest/CuTest.h>

#include "rp_test_srecord.h"


char test_sdata[] =
"S00E000032313030202020206D6F745E\n"
"S11308005A00946A5A0008845A0008845A000884DA\n"
"S11308705A0008845A0008845A0008845A000884DC\n"
"S11308805A00088456707A01000007FC6818A85AB8\n"
"S11308C0E8985470F8016A88F78140E8F8016A886A\n"
"S11308D0F78240E06A08F781470255166A08F782F2\n"
"S11308E04702553055CE18886A88F7816A88F7829E\n"
"S9030400F8\n";

/*
 * Convert data above into binary with:
 *   objcopy -I srec -O binary --gap-fill=0xFF test.srec test.bin && hexdump -v -e ' 16/1 "0x%02X, " "\n"' test.bin
 * This data starts at the lowest address in the srecord data. In this case 0x0800.
 */
const uint8_t test_bindata[] =
{
  0x5A, 0x00, 0x94, 0x6A, 0x5A, 0x00, 0x08, 0x84, 0x5A, 0x00, 0x08, 0x84, 0x5A, 0x00, 0x08, 0x84,
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
  0x5A, 0x00, 0x08, 0x84, 0x5A, 0x00, 0x08, 0x84, 0x5A, 0x00, 0x08, 0x84, 0x5A, 0x00, 0x08, 0x84,
  0x5A, 0x00, 0x08, 0x84, 0x56, 0x70, 0x7A, 0x01, 0x00, 0x00, 0x07, 0xFC, 0x68, 0x18, 0xA8, 0x5A,
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
  0xE8, 0x98, 0x54, 0x70, 0xF8, 0x01, 0x6A, 0x88, 0xF7, 0x81, 0x40, 0xE8, 0xF8, 0x01, 0x6A, 0x88,
  0xF7, 0x82, 0x40, 0xE0, 0x6A, 0x08, 0xF7, 0x81, 0x47, 0x02, 0x55, 0x16, 0x6A, 0x08, 0xF7, 0x82,
  0x47, 0x02, 0x55, 0x30, 0x55, 0xCE, 0x18, 0x88, 0x6A, 0x88, 0xF7, 0x81, 0x6A, 0x88, 0xF7, 0x82,
};


#define SRECORD_OFFSET 0x800
#define PAD_CHAR 0xFF

/** Test reading s-record from file */
static void test_rinprog_srecord_read(CuTest* tc)
{
  enum e_rp_result result;
  FILE *in;
  FILE *out;
  char *outbuf;
  size_t outbuf_len;
  size_t i;
  char errstr[100];
  uint8_t data;
  /* long outflen; */

  /* Create s-record data as a stream */
  in = fmemopen(test_sdata, strlen(test_sdata), "r");
  CuAssertPtrNotNull(tc, in);

  /* Create buffer to write data to */
  outbuf = NULL;
  outbuf_len = 0;
  out = open_memstream(&outbuf, &outbuf_len);
  fflush(out);
  CuAssertPtrNotNull(tc, in);
  CuAssertPtrNotNull(tc, outbuf);

  /* Check bad parameters */
  result = rp_read_srecord_file(NULL, out, 0x00);
  CuAssertIntEquals(tc, rpr_paramfail, result);
  result = rp_read_srecord_file(in, NULL, 0x00);
  CuAssertIntEquals(tc, rpr_paramfail, result);

  /* Read data in */
  result = rp_read_srecord_file(in, out, PAD_CHAR);
  fflush(out);
  CuAssertIntEquals(tc, rpr_success, result);

  /* Compare size with expected */
  CuAssertIntEquals(tc, sizeof(test_bindata) + SRECORD_OFFSET, outbuf_len);
/* With open_memstream() this returns larger, I assume because it allocates in chunks
  rp_get_file_length(out, &outflen);
  CuAssertIntEquals(tc, sizeof(test_bindata) + SRECORD_OFFSET, outflen);
*/
  /* Compare data with expected */
  fseek(out, 0, SEEK_SET);
  for (i = 0; i < SRECORD_OFFSET; i++)
  {
    snprintf(errstr, 100, "expected pad char 0x%02x at buffer offset 0x%x, data was 0x%02x", (uint8_t) PAD_CHAR, (unsigned) i, (uint8_t) outbuf[i]);
    CuAssertIntEquals_Msg(tc, errstr, (uint8_t) PAD_CHAR, (uint8_t) outbuf[i]);

    fread(&data, sizeof(data), 1, out);
    snprintf(errstr, 100, "expected pad char 0x%02x at file offset 0x%x, data was 0x%02x", (uint8_t) PAD_CHAR, (unsigned) i, data);
    CuAssertIntEquals_Msg(tc, errstr, (uint8_t) PAD_CHAR, data);
  }
  for (; i < outbuf_len; i++)
  {
    snprintf(errstr, 100, "expected 0x%02x at buffer offset 0x%x, data was 0x%02x", test_bindata[i - SRECORD_OFFSET], (unsigned) i, (uint8_t) outbuf[i]);
    CuAssertIntEquals_Msg(tc, errstr, test_bindata[i - SRECORD_OFFSET], (uint8_t) outbuf[i]);

    fread(&data, sizeof(data), 1, out);
    snprintf(errstr, 100, "expected 0x%02x at file offset 0x%x, data was 0x%02x", test_bindata[i - SRECORD_OFFSET], (unsigned) i, data);
    CuAssertIntEquals_Msg(tc, errstr, test_bindata[i - SRECORD_OFFSET], data);
  }
  CuAssertTrue(tc, memcmp(test_bindata, &outbuf[SRECORD_OFFSET], sizeof(test_bindata)) == 0);

  /* Free up */
  fclose(in);
  fclose(out);
  free(outbuf);
}


CuSuite *CuGetSuite_rinprog_srecord(void)
{
  CuSuite *suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_rinprog_srecord_read);

  return suite;
}
