/******************************************************************************

HEADER FILE

DESCRIPTION:    This module is a header file.  The file name and revision
                status of this file are given below.


COMPILATION ENVIRONMENT:

                 - IDE:         Hitachi Embedded Workshop, Ver 1.3
                 - Toolchain:   Hitachi H8S, H8/300 Standard Toolchain Ver 4.0

$Source: \\\\marvin\\users\\alan\\RCS\\C\\Alan-McFadyen\\Projects\\R300\\Design\\Product\\Software\\R310-5xx\\110\\adc1216.h,v $

$Log: adc1216.h,v $
Revision 1.0  2003-04-07 13:13:20+10  alan
Initial revision


Revision 1.0  2003-04-07 12:34:21+10  alan
Initial revision

---Released--- Revisions Below are for development


*******************************************************************************/
#ifndef ADC_ADS1216_DOT_H
#define ADC_ADS1216_DOT_H  1

#include <rinhw/stdrinstrum.h>
#include <rinweigh/adcdatatypes.h>


/** Calculate a new zero value for the current temperature */
int32s AdcIntTempComp_CalculateZero(const AdcSuperStructure *ass);


/** Calculate a new span value for the current temperature */
int32s AdcIntTempComp_CalculateSpan(const AdcSuperStructure *ass);


/**
 * Get a reading from the ADC hardware and run the temperature compensation
 *
 * The read will block until the ADC signals data is ready, hence this function
 * controls the rate at which the ADC thread runs.
 *
 * @param ass is a pointer to the ADC super structure containing all information on the ADC, not NULL
 * @param rdg is a pointer to a reading which is set to the new reading value, not NULL
 * @param data_wait_failed is a pointer to a bool that is set if there was a read problem,
 *        and you should implement a delay elsewhere, to prevent the ADC thread going to 100% CPU, not NULL
 * @return true if new data acquired (not a skipped reading)
 */
bool AdcIntHwRead(AdcSuperStructure *ass, int32s *rdg, bool *data_wait_failed);


/** Reconfigure the ADC hardware with the configuration passed in cfg, can be called multiple times */
void AdcIntHwReconfigure(AdcSuperStructure *ass, int32s zero, int32s span, int16u sync_freq, bool four_wire);


/** Initialise the ADC hardware, call only once */
bool AdcIntHwInit(AdcHwInfo *hwinfo, struct t_io_dev *adcdev);


/** Shutdown ADC hardware */
void AdcIntHwDestroy(AdcHwInfo *hwinfo);


#endif /* #ifndef ADC_ADS1216_DOT_H */
