/*
 * adcalibi.h
 *
 *  Created on: Feb 20, 2012
 *      Author: ian
 */

#ifndef ADCALIBI_H_
#define ADCALIBI_H_

#include <rinhw/stdrinstrum.h>
#include <rindb/db_changelog_reader.h>
#include <rinweigh/adcdatatypes.h>


enum eAdcAlibiDisplayType
{
  aadtSingleDisplay,    /*!< Single display, will alternate between "alibi" and weight */
  aadtDoubleDisplay,    /*!< Double display, first display will be weight, second display will be "alibi" */
  aadtNum
};


/** Keystrokes the alibi application knows how to handle */
enum eAdcAlibiKeystroke
{
  aakNone,
  aakZero,              /*!< Usually the zero key */
  aakTare,              /*!< Usually the tare key */
  aakGrossNet,          /*!< Usually the gross/net key */
  aakPrint,             /*!< Usually the print key */
  aakMode,              /*!< Usually the function key, used to change modes */
  aakEnterPresetTare,   /*!< Long press tare is a reasonable choice */
  aakDSDRecordFirst,    /*!< Jump to first (oldest) DSD or change log record */
  aakDSDRecordLast,     /*!< Jump to last (newest) DSD or change log record */
  aakDSDRecordNext,     /*!< Advance to next DSD or change log record */
  aakDSDRecordPrevious, /*!< Back to previous DSD or change log record */
  aakDSDRecordSelect,   /*!< Select a specific DSD or change log record */
  aakDSDItemFirst,      /*!< Jump to the first item within the current DSD or change log record (printid->reading->etc) */
  aakDSDItemLast,       /*!< Jump to the last item within the current DSD or change log record (etc->reading->printid) */
  aakDSDItemNext,       /*!< Advance to next item within the current DSD or change log record (printid->reading->etc) */
  aakDSDItemPrevious,   /*!< Reverse to previous item within the current DSD or change log record (etc->reading->printid) */
  aakCountEnterQty,     /*!< Counting enter quantity key */
  aakCountCancel,       /*!< Cancel piece counting */
  aakQuit               /*!< Quit the alibi mode */
};


/** Modes that the alibi application can run in */
enum eAdcAlibiMode
{
  aamWeighing = 0,    /*!< Normal simple weighing application */
  aamCounting,        /*!< Piece counting application */
  aamDSDView,         /*!< DSD viewer application */
  aamChangelogView,   /*!< Changelog viewer application */
  aamQuitAlibi,       /*!< Quit the alibi application */
  aamNumAlibiModes
};


/** Allows for internationalisation */
enum eAdcAlibiInternationisationStringIdx
{
  /* Weighing/general */
  aaisiAlibi,         /*!< "Alibi" */
  aaisiChecksum,      /*!< "Checksum" */
  aaisiCalCount,      /*!< "Cal.Cnt" - NTEP/OIML Calibration counter */
  aaisiConfigCount,   /*!< "Config" - NTEP Configuration counter */
  aaisiVersion,       /*!< "Version" */
  aaisiPresetTare,    /*!< "Pre.Tar" */
  aaisiEnter,         /*!< "Enter" */
  aaisiError,         /*!< "Error" */
  aaisiMotion,        /*!< "Motion" */
  aaisiRange,         /*!< "Range" */
  aaisiADC,           /*!< "ADC" */
  aaisiFailed,        /*!< "Failed" */
  aaisiTrade,         /*!< "Trade" */
  aaisiHiRes,         /*!< "Hi.Res" */
  aaisiOverload,      /*!< "Overload" */
  aaisiUnderload,     /*!< "Underload" */
  aaisiEnd,           /*!< "End" */
  aaisiNo,            /*!< "No" */
  aaisiDate,          /*!< "Date" */
  aaisiTime,          /*!< "Time" */
  aaisiFirst,         /*!< "First" - Jump to first record (DSD/change log) */
  aaisiLast,          /*!< "Last" - Jump to last record (DSD/change log) */
  aaisiPrevious,      /*!< "Prev" - Jump to previous record (DSD/change log) */
  aaisiNext,          /*!< "Next" - Jump to next record (DSD/change log) */

  /* DSD */
  aaisiDSDPrintId,    /*!< "Prn.Id" - Traceable weight print id */
  aaisiDSDReading,    /*!< "Weight" - Traceable weight reading */
  aaisiDSDAltReading, /*!< "Alt.Wgt" - Traceable alternate units weight reading */
  aaisiDSDPieces,     /*!< "Pieces" - Traceable pieces */
  aaisiDSDTareWeight, /*!< "Tar.Wgt" - Traceable tare weight */
  aaisiDSDError,      /*!< "DSD.ERR" */
  aaisiDSDSearch,     /*!< "SRCH" */
  aaisiDSDEmpty,      /*!< "Empty" */
  aaisiDSD,           /*!< "DSD" */

  /* Modes */
  aaisiModeWeighing,  /*!< "Weigh" */
  aaisiModeCounting,  /*!< "Count" */
  aaisiModeDSDView,   /*!< "DSD" */
  aaisiModeChangeLog, /*!< "Chg.Log" */
  aaisiModeQuitAlibi, /*!< "Quit" */

  /* Counting */
  aaisiCountQuantity, /*!< "Qty" */
  aaisiCountPieces,   /*!< "Pieces" */

  /* Change log */
  aaisiChangeLog,           /*!< "CHG.LOG" */
  aaisiChangeLogError,      /*!< "CL.ERR" */
  aaisiChangeLogName,       /*!< "Name" - change log element name */
  aaisiChangeLogIndex,      /*!< "Index" - change log element index */
  aaisiChangeLogOldValue,   /*!< "Old.Val" - change log old value */
  aaisiChangeLogNewValue,   /*!< "New.Val" - change log new value */

  aaisiNumStrings
};


/**
 * English strings for a 6 character display. Must match @ref enum eAdcAlibiInternationisationStringIdx.
 * Feel free to not use these and create your own
 */
#define ADC_ALIBI_INTERNATIONALISATION_ENGLISH_STRS \
{ \
  "ALIBI",            /* aaisiAlibi */ \
  "CHK.SUM",          /* aaisiChecksum */ \
  "CAL.CNT",          /* aaisiCalCount */ \
  "CFG.CNT",          /* aaisiConfigCount */ \
  "VERS",             /* aaisiVersion */ \
  "PRE.TAR",          /* aaisiPresetTare */ \
  "ENTER",            /* aaisiEnter */ \
  "ERROR",            /* aaisiError */ \
  "MOTION",           /* aaisiMotion */ \
  "RANGE",            /* aaisiRange */ \
  "ADC",              /* aaisiADC */ \
  "FAILED",           /* aaisiFailed */ \
  "TRADE",            /* aaisiTrade */ \
  "HI.RES",           /* aaisiHiRes */ \
  "O-----",           /* aaisiOverload */ \
  "U-----",           /* aaisiUnderload */ \
  "END",              /* aaisiEnd */ \
  "NO",               /* aaisiNo */ \
  "DATE",             /* aaisiDate */ \
  "TIME",             /* aaisiTime */ \
  "FIRST",            /* aaisiFirst */ \
  "LAST",             /* aaisiLast */ \
  "PREV",             /* aaisiPrevious */ \
  "NEXT",             /* aaisiNext */ \
\
  "PRN.ID",           /* aaisiDSDPrintId */ \
  "WEIGHT",           /* aaisiDSDReading */ \
  "ALT.WGT",          /* aaisiDSDAltReading */ \
  "PIECES",           /* aaisiDSDPieces */ \
  "TAR.WGT",          /* aaisiDSDTareWeight */ \
  "DSD.ERR",          /* aaisiDSDError */ \
  "SRCH",             /* aaisiDSDSearch */ \
  "EMPTY",            /* aaisiDSDEmpty */ \
  "DSD",              /* aaisiDSD */ \
\
  "WEIGH",            /* aaisiModeWeighing */ \
  "COUNT",            /* aaisiModeCounting */ \
  "DSD",              /* aaisiModeDSDView */ \
  "CHG.LOG",          /* aaisiModeChangeLog */ \
  "-END-",            /* aaisiModeQuitAlibi */ \
\
  "QTY",              /* aaisiCountQuantity */ \
  "PIECES",           /* aaisiCountPieces */ \
\
  "CHG.LOG",          /* aaisiChangeLog */ \
  "CL.ERR",           /* aaisiChangeLogError */ \
  "NAME",             /* aaisiChangeLogName */ \
  "INDEX",            /* aaisiChangeLogIndex */ \
  "OLD.VAL",          /* aaisiChangeLogOldValue */ \
  "NEW.VAL",          /* aaisiChangeLogNewValue */ \
}


/**
 * Function typedef to notify the the alibi application to quit (used when the main application is shutting down)
 *
 * @param implementer_data is a pointer to some implementation specific data
 * @param force_quit is the pointer to a boolean that can be set to force the application to quit
 */
typedef void (*t_adc_alibi_force_quit_fn)(void *implementer_data, bool *force_quit);


/**
 * Function typedef to turn all segments on or off
 *
 * @param implementer_data is a pointer to some implementation specific data
 * @param alibi_display_type is the display type (single, dual)
 * @param on_not_off is true to turn all segments on, false otherwise
 */
typedef void (*t_adc_alibi_display_allsegs_fn)(void *implementer_data, enum eAdcAlibiDisplayType alibi_display_type, bool on_not_off);


/**
 * Function typedef to display text & annunciators
 *
 * @param implementer_data is a pointer to some implementation specific data
 * @param alibi_display_type is the display type (single, dual)
 * @param display_strs is an array of pointers to strings, one for each display
 * @param info is a pointer to an array of AdcInfos, one for each display
 */
typedef void (*t_adc_alibi_display_fn)(void *implementer_data, enum eAdcAlibiDisplayType alibi_display_type, const char **display_strs, const AdcInfo *info);


/**
 * Function typedef to get a keystroke
 *
 * @param implementer_data is a pointer to some implementation specific data
 * @param mode is the current alibi mode, this may effect what key is returned (same key, different mode, different function)
 * @param keystroke is a pointer to a keystroke to be set. Never NULL. Set to aakNone if no keystroke, timeout, error, etc
 * @param timeout is a uCOS time out, 0 for indefinate, anything else in ticks (see @ref OS_TICKS_PER_SEC)
 */
typedef void (*t_adc_alibi_getchar_fn)(void *implementer_data, enum eAdcAlibiMode mode, enum eAdcAlibiKeystroke *keystroke, int16u timeout);


/**
 * Function typedef to get an internationalised display string
 *
 * @param implementer_data is a pointer to some implementation specific data
 * @param stridx is the index of the string being requested.
 * @param display_str is a pointer to a pointer to a string. You should set *display_str to point to your string. This needs to be a non-stack declared string.
 */
typedef void (*t_adc_alibi_get_internationalisation_string_fn)(void *implementer_data, enum eAdcAlibiInternationisationStringIdx stridx, const char **display_str);


/**
 * Function typedef to edit a number
 *
 * @param implementer_data is a pointer to some implementation specific data
 * @param rdg is the number to be edited, along with units, dp and status to be displayed while editing
 * @param minimum is the minimum value
 * @param maximum is the maximum value
 * @param is_signed indicates whether the number is signed.
 */
typedef bool (*t_adc_alibi_edit_number_fn)(void *implementer_data, AdcRdg *rdg, int32s minimum, int32s maximium, bool is_signed);


/**
 * Function typedef to edit a list
 *
 * @param implementer_data is a pointer to some implementation specific data
 * @param idx is a pointer to the selected index, that can be edited
 * @param list_strings is a list of pointers to strings of list elements
 * @param list_length is the number of elements in the list
 */
typedef bool (*t_adc_alibi_edit_list_fn)(void *implementer_data, int8u *idx, const char *const list_strings[], int8u list_length);


/**
 * Function typedef to get a abbreviated units string (for printing, eg kg, lb)
 *
 * @param implementer_data is a pointer to some implementation specific data
 * @param adcunitsidx is the units index
 * @param units_str is a pointer to a pointer to a string. You should set *units_str to point to your string. This needs to be a non-stack declared string (ie should still be in scope when your function returns)
 */
typedef void (*t_adc_alibi_get_abbreviated_units_string_fn)(void *implementer_data, int8u adcunitsidx, const char **units_str);


/**
 * Function typedef to print some text
 *
 * @param implementer_data is a pointer to some implementation specific data
 * @param print_str is the null terminated string to be printed
 */
typedef void (*t_adc_alibi_print_string_fn)(void *implementer_data, const char *print_str);


/**
 * Function typedef to convert the binary data read from the DSD into a traceable weight
 *
 * Your function should extract the traceable weight from however it stored it in the DSD record
 * and return it in atw
 *
 * @param implementer_data is a pointer to some implementation specific data
 * @param data is a pointer to the raw data from the DSD record
 * @param length is the length of the data
 * @param atw is the traceable weight that should be set by the function
 *
 * @return true if the traceable weight was successfully extracted
 */
typedef bool (*t_adc_alibi_dsd_convert_to_traceable_fn)(void *implementer_data, const int8u *data, size_t length, AdcTraceableWeight *atw);


/**
 * Function typedef to convert a traceable weight into the binary data to write to the DSD
 *
 * Your function should write the traceable weight however it is stored it in the DSD record
 * and return it data
 *
 * @param implementer_data is a pointer to some implementation specific data
 * @param atw is the traceable weight that should be inserted into data
 * @param data is a pointer to the raw data to be written as a DSD record to the DSD
 * @param length is the length of the data
 *
 * @return true if the traceable weight was successfully written
 */
typedef bool (*t_adc_alibi_dsd_convert_from_traceable_fn)(void *implementer_data, const AdcTraceableWeight *atw, int8u *data, size_t length);


/**
 * Function typedef to convert ADC errors into application errors
 *
 * Your function should convert adcerrors into application errors. This allows the
 * alibi application to display the same errors as your application, in the case
 * where application errors don't match the ADC library errors
 *
 * @param implementer_data is a pointer to some implementation specific data
 * @param adcerrors are the current ADC errors
 * @param apperrors is a pointer to the application errors to be returned
 */
typedef void (*t_adc_alibi_convert_adcerrors_to_apperrors_fn)(void *implementer_data, AdcErrors adcerrors, int32u *apperrors);


/**
 * Calculates the length for a display work buffer from the number of digits:
 * x2 for 1 decimal place for each digit
 * +1 for the null terminator
 */
#define ADC_ALIBI_DISPLAY_WORK_BUF_LEN(numdigits) (numdigits * 2 + 1)


/** Alibi information provided to @ref AdcAlibiRun to implement platform specific stuff */
struct s_adc_alibi_info
{
  void *implementer_data;                                     /*!< Implementer specific data, passed to callbacks */
  enum eAdcAlibiDisplayType adc_alibi_display_type;           /*!< Type of display to drive (single row, dual row) */
  int num_digits[aadtNum];                                    /*!< The number of digits on each display (excluding decimal points) */
  char *display_work_buffers[aadtNum];                        /*!< Array of pointers to a work buffer for each display, of length calculated by @ref ADC_ALIBI_DISPLAY_WORK_BUF_LEN() */
  enum eAdcAlibiMode alibi_mode;                              /*!< Current alibi mode */
  bool force_quit;                                            /*!< Used to ensure a fast exit when quit is force by application shutting down */
  bool status_can_show_pieces;                                /*!< The annunciators can show pieces. If not the word "pieces" will be displayed with pieces */

  AdcSuperStructure *ass;                                     /*!< ADC structure, passed to all ADC library calls. Not NULL. */
  struct s_adc_dsd *dsd;                                      /*!< DSD structure, passed to all DSD library calls. Can be NULL if no DSD is present */
  int8u *dsd_record_buffer;                                   /*!< Set to buffer of length AdcDSDGetRecordSize(dsd). Used internally */
  int32u dsd_record_buffer_length;                            /*!< Set to AdcDSDGetRecordSize(dsd) */
  struct s_db_changelog_reader *changelog_reader;             /*!< Change log reader structure, passed to all DB library calls. Can be NULL if no changelog present */

  bool no_piece_counting;                                     /*!< Application doesn't support piece counting, so hide all piece counting in the alibi */
  bool no_alternate_units;                                    /*!< Application doesn't support alternate units, so hide all alternate units in the alibi */
  int32u adc_lib_checksum;                                    /*!< ADC library checksum, displayed as the alibi mode starts up */

  t_adc_alibi_display_allsegs_fn adc_alibi_display_allsegs;
  t_adc_alibi_display_fn adc_alibi_display;
  t_adc_alibi_getchar_fn adc_alibi_getchar;
  t_adc_alibi_get_internationalisation_string_fn adc_alibi_get_internationalisation_string;
  t_adc_alibi_force_quit_fn adc_alibi_force_quit;
  t_adc_alibi_edit_number_fn adc_alibi_edit_number;
  t_adc_alibi_edit_list_fn adc_alibi_edit_list;
  t_adc_alibi_get_abbreviated_units_string_fn adc_alibi_get_abbreviated_units_string;
  t_adc_alibi_print_string_fn adc_alibi_print_string;
  t_adc_alibi_dsd_convert_to_traceable_fn adc_alibi_dsd_convert_to_traceable;
  t_adc_alibi_dsd_convert_from_traceable_fn adc_alibi_dsd_convert_from_traceable;
  t_adc_alibi_convert_adcerrors_to_apperrors_fn adc_alibi_convert_adcerrors_to_apperrors;    /*!< Function ptr to convert ADC errors to application errors for display in alibi. Can be NULL, in which case ADC errors are displayed without translation */
};


/** Run the alibi application */
bool AdcAlibiRun(struct s_adc_alibi_info *adc_alibi_info);


#endif /* ADCALIBI_H_ */
