/*
 * adccalcountcheck_std.c
 *
 *  Created on: 25/06/2012
 *      Author: ian
 */
#include "adcdatatypes.h"

#include "adccalcountcheck_std.h"


/** Determine if a calibration relevant setting has changed by comparing 2 databases */
static bool bCalibrationRelevantSettingChanged(const AdcUserDatabase *pDb1, const AdcUserDatabase *pDb2)
{
  bool bSettingChanged = false;
  int8u i;

  /* Check to see if cal counter needs to be bumped */
  bSettingChanged = ( ( pDb1->zeroRaw != pDb2->zeroRaw ) ||
                      ( pDb1->spanRaw != pDb2->spanRaw ) ||
                      ( pDb1->spanCnt != pDb2->spanCnt ) ||
                      ( pDb1->calFlag != pDb2->calFlag ) );

  for ( i = 0; i <= ADC_MAX_NUM_LIN_PTS; i++ )
  {
    if ( ( pDb1->linPts[ i ] != pDb2->linPts[ i ] ) ||
         ( pDb1->linCor[ i ] != pDb2->linCor[ i ] ) )
    {
      bSettingChanged = true;
    }
  }
  return bSettingChanged;
}


/** Determine if a configuration relevant setting has changed by comparing 2 databases */
static bool bConfigurationRelevantSettingChanged(const AdcUserDatabase *pDb1, const AdcUserDatabase *pDb2)
{
  /* Check to see if config counter needs to be bumped */

  return ( ( pDb1->scaleType     != pDb2->scaleType ) ||
           ( pDb1->syncFreqIdx   != pDb2->syncFreqIdx ) ||
           ( pDb1->jitter        != pDb2->jitter ) ||
           ( pDb1->dpPos         != pDb2->dpPos ) ||
           ( pDb1->fullScale[0]  != pDb2->fullScale[0] ) ||
           ( pDb1->fullScale[1]  != pDb2->fullScale[1] ) ||
           ( pDb1->fullScale[2]  != pDb2->fullScale[2] ) ||
           ( pDb1->resolution[0] != pDb2->resolution[0] ) ||
           ( pDb1->resolution[1] != pDb2->resolution[1] ) ||
           ( pDb1->resolution[2] != pDb2->resolution[2] ) ||
           ( pDb1->units         != pDb2->units ) ||
           ( pDb1->calWeight     != pDb2->calWeight ) ||
           ( pDb1->addTare       != pDb2->addTare ) ||
           ( pDb1->zeroBand      != pDb2->zeroBand ) ||
           ( pDb1->motion        != pDb2->motion ) ||
           ( pDb1->zeroTracking  != pDb2->zeroTracking ) ||
           /* ( pDb1->average       != pDb2->average ) || */
           ( pDb1->rangeOfZero   != pDb2->rangeOfZero ) ||
           ( pDb1->approvalMode  != pDb2->approvalMode ) ||
           /* ( pDb1->zeroOnStart   != pDb2->zeroOnStart ) || */
           ( pDb1->wire4Mode     != pDb2->wire4Mode ) ||
           ( pDb1->tempCalZero   != pDb2->tempCalZero ) ||
           ( pDb1->hiRes         != pDb2->hiRes ) );
}


/** Standard implementation for checking for ADC user database changes that affect calibration and configuration counters */
void AdcStandardDatabaseCalCountCheckImplementer(AdcSuperStructure *ass, const AdcUserDatabase *changed_aud, const AdcUserDatabase *reference_aud,
                                                 bool *calibration_changed, bool *configuration_changed)
{
  /* Check to see if cal counter needs to be bumped */
  *calibration_changed = bCalibrationRelevantSettingChanged(changed_aud, reference_aud);

  /* Check to see if config counter needs to be bumped */
  *configuration_changed = bConfigurationRelevantSettingChanged(changed_aud, reference_aud);
}
