#ifndef ADC_CONVERT_DOT_H
#define ADC_CONVERT_DOT_H 1
                 
#include <rinhw/stdrinstrum.h>
#include <rinweigh/adcdatatypes.h>


/**
 * Convert internal ADC counts into final (displayable) numbers,
 * taking into account the resolution of the specified range,
 * and whether an extra digit of resolution is required.
 *
 * @param ass is the ADC data super structure
 * @param rdg is the reading in ADC counts
 * @param range is the current range (0, 1, 2)
 * @param hiRes is a flag to indicate that the final number should have an extra digit of resolution
 * @param unit is the library units to set (secondary, tertiary etc)
 * @return the rdg converted to final (displayable) numbers
 */
int32s AdcIntCountsToFinal(const AdcSuperStructure *ass, int32s rdg, int8u range, bool hiRes, enum eAdcUnitSwitch unit);


/**
 * Convert final (displayable) numbers into internal ADC counts,
 * taking into account whether an extra digit of resolution is present.
 *
 * @param ass is the ADC data super structure
 * @param rdg is the final (displayable) number
 * @param hiRes is a flag to indicate that rdg has an extra digit of resolution
 * @param unit is the library units to set (secondary, tertiary etc)
 * @return the rdg converted to internal ADC counts
 */
int32s AdcIntFinalToCounts(const AdcSuperStructure *ass, int32s rdg, bool hiRes, enum eAdcUnitSwitch unit);


/**
 * Convert a final (displayable) number with (possibly) extra resolution
 * into a final (displayable) number with no extra resolution
 *
 * @param ass is the ADC data super structure
 * @param rdg is the final number with (possibly) extra resolution
 * @param range is the current range of the final number
 * @param hiRes is a flag to indicate that rdg has extra resolution
 * @return rdg with any extra resolution removed
 */
int32s AdcIntHiResToNormal(AdcSuperStructure *ass, int32s rdg, int8u range, bool hiRes);


#endif /* #ifndef ADC_CONVERT_DOT_H */
