/*
 * AdcCrc.c
 *
 *  Created on: 31/03/2011
 *      Author: ian
 */
#include "adcdatabase.h"


/**
 * Calculate a 16bit CCITT CRC checksum for a block of data
 *
 * @param byt is an array of bytes to calculate the checksum for
 * @param size is the number of bytes to calculate the checksum over
 * @param crc is the checksum seed
 * @return the checksum of the given data
 */
int16u AdcCalcCRC16(const int8u *byt, size_t size, int16u crc)
{
  /*
   * Calculate CRC-16 value; uses The CCITT-16 Polynomial,
   * expressed as X^16 + X^12 + X^5 + 1
   */
  size_t index;
  int8u b;

  for (index = 0; index < size; ++index)
  {
    crc ^= (((int16u) byt[index]) << 8);
    for (b = 0; b < 8; ++b)
    {
      if (crc & 0x8000)
        crc = (crc << 1) ^ 0x1021;
      else
        crc = (crc << 1);
    }
  }
  return crc;
}
