/*
 * adcdatafiller.h
 *
 *  Created on: Mar 31, 2012
 *      Author: ian
 */

#ifndef ADCDATAFILLER_H_
#define ADCDATAFILLER_H_

#include <rinhw/stdrinstrum.h>
#include <rinweigh/adcdatatypes.h>


/** Work out whether the alternate units conversion factors are valid */
bool AdcIntAlternateUnitsValid(const AdcInternalData *iad, enum eAdcUnitSwitch unit);


/** Fill out heldrdg from rdg, and set appropriate status bits */
void AdcIntFillOutHeldReading(AdcRdg *heldrdg, const AdcRdg *rdg);


/** Fill out AdcUnitDp structure */
void AdcIntFillOutUnitDp(const AdcSuperStructure *ass, AdcUnitDp *unitdp, enum eAdcUnitSwitch unit);


/** Fill out a tare reading */
void AdcIntFillOutTareReading(const AdcSuperStructure *ass, AdcRdg *tarerdg, int32s cntTare, bool tare_not_presettare, enum eAdcUnitSwitch unit);


/**
 * Fill out the ADC data structure
 *
 * @param ass is a pointer to the ADC super structure containing all information on the ADC, not NULL
 * @param ad is a pointer to the ADC data structure to be filled out, not NULL
 * @param ai is a pointer to an ADC info structure to be filled out, not NULL
 * @param rawRdg is the raw ADC reading (post averaging)
 * @param cntSysRdg is the rawRdg in counts minus the initial zero. Calculate with @ref AdcIntCalculateCounts
 * @param cntGross is the gross weight in counts. Calculate with @ref AdcIntCalculateCounts
 * @param cntNet is the net weight in counts. Calculate with @ref AdcIntCalculateCounts
 * @param cntOload is the overload weight. (In NTEP this changes with range)
 *
 */
void AdcIntFillOutAdcData(const AdcSuperStructure *ass,
                          AdcData *ad,
                          AdcInfo *ai,
                          int32s rawRdg,
                          int32s cntSysRdg,
                          int32s cntGross,
                          int32s cntNet,
                          int32s cntOload,
                          int8u previousRange);


/**
 * Calculate various counts from raw reading
 *
 * @param ass is a pointer to the ADC super structure containing all information on the ADC, not NULL
 * @param rawRdg is the raw ADC reading (post averaging)
 * @param cntSysRdg is the rawRdg in counts minus the initial zero
 * @param cntGross is the gross weight in counts
 * @param cntNet is the net weight in counts
 */
void AdcIntCalculateCounts(const AdcSuperStructure *ass, int32s rawRdg,
                           int32s *cntSysRdg, int32s *cntGross, int32s *cntNet);


/** Set the held bit in all AdcInfo structures within AdcData */
void AdcIntAdcDataSetHeld(AdcData *adcData, bool held);


#endif /* ADCDATAFILLER_H_ */
