/*
 * adcdbdefault.c
 *
 *  Created on: Apr 17, 2012
 *      Author: ian
 */
#include <rinweigh/adcdatatypes.h>
#include <rinweigh/adcinterface.h>
#include <rinweigh/adcdbfactory.h>
#include <rinweigh/adcdbuser.h>
#include <rinweigh/adcdbruntime.h>
#include <rinweigh/adcmutex.h>

#include <rinweigh/adcdbdefault.h>


/**
 * Internal defaulting of databases without locking
 *
 * @param ass is the ADC super structure.
 * @param afd is a pointer to a factory database to be defaulted. Can be NULL in which case shouldn't be defaulted.
 * @param aud is a pointer to a user database to be defaulted. Can be NULL in which case shouldn't be defaulted.
 * @param ard is a pointer to a runtime database to be defaulted. Can be NULL in which case shouldn't be defaulted.
 * @param preserve_cal_counters means the calibration counters should be preserved and incremented over the default.
 * @param cal_only means only the calibration settings should be defaulted (this applies to aud only) (settings to be defaulted are zeroRaw, spanRaw, spanCnt, linearisation).
 */
void AdcIntDefaultDatabases(AdcSuperStructure *ass,
                            AdcFactoryDatabase *afd,
                            AdcUserDatabase *aud,
                            AdcRuntimeDatabase *ard,
                            bool preserve_cal_counters,
                            bool cal_only)
{
  /* Default the databases */
  if (NULL != ass->adcMiscellaneous.adcDatabaseDefaultImplementerFn)
  {
    ass->adcMiscellaneous.adcDatabaseDefaultImplementerFn(ass, afd, aud, ard, preserve_cal_counters, cal_only);
  }

  /* Set checksums on databases */
  if (NULL != afd)
  {
    AdcIntFactoryDatabaseSetChecksum(afd);
  }
  if (NULL != aud)
  {
    AdcIntUserDatabaseSetChecksum(aud);
  }
  if (NULL != ard)
  {
    AdcIntRuntimeDatabaseSetChecksum(ard);
  }
}


/**
 * Defaulting of databases with locking
 *
 * @param ass is the ADC super structure.
 * @param lock specifies if the function should lock ass before performing it's function. This allows the caller to have locked the structure prior to the call
 * @param afd is a pointer to a factory database to be defaulted. Can be NULL in which case shouldn't be defaulted.
 * @param aud is a pointer to a user database to be defaulted. Can be NULL in which case shouldn't be defaulted.
 * @param ard is a pointer to a runtime database to be defaulted. Can be NULL in which case shouldn't be defaulted.
 * @param preserve_cal_counters means the calibration counters should be preserved and incremented over the default.
 * @param cal_only means only the calibration settings should be defaulted (this applies to aud only) (settings to be defaulted are zeroRaw, spanRaw, spanCnt, linearisation).
 */
void AdcDefaultDatabases(AdcSuperStructure *ass,
                         bool lock,
                         AdcFactoryDatabase *afd,
                         AdcUserDatabase *aud,
                         AdcRuntimeDatabase *ard,
                         bool preserve_cal_counters,
                         bool cal_only)
{
  ADC_INT_IFACE_LOCK();
  AdcIntDefaultDatabases(ass, afd, aud, ard, preserve_cal_counters, cal_only);
  ADC_INT_IFACE_UNLOCK();
}
