/*
 * AdcDbFactory.c
 *
 *  Created on: 31/03/2011
 *      Author: ian
 */
#include <stdlib.h>
#include <string.h>
#include <rinhw/stdrinstrum.h>
#include <rinhw/rstdio.h>
#include <rinweigh/adcoperations.h>
#include <rinweigh/adcreadings.h>
#include <rinweigh/adcdatatypes.h>
#include <rinweigh/adcmotion.h>
#include <rinweigh/adcinterface.h>
#include <rinweigh/adcdatabase.h>

#include <rinweigh/adcdbfactory.h>


/** Default values for the ADC factory database */
static const AdcFactoryDatabase s_C_DefaultAdcFactoryDatabase =
{
  .zeroOffset =                                           /* int32s zeroOffset; */
  {
    ADC_DB_FACT_ZEROOFFSET_DEFAULT, ADC_DB_FACT_ZEROOFFSET_DEFAULT, ADC_DB_FACT_ZEROOFFSET_DEFAULT, ADC_DB_FACT_ZEROOFFSET_DEFAULT,
    ADC_DB_FACT_ZEROOFFSET_DEFAULT, ADC_DB_FACT_ZEROOFFSET_DEFAULT, ADC_DB_FACT_ZEROOFFSET_DEFAULT, ADC_DB_FACT_ZEROOFFSET_DEFAULT,
    ADC_DB_FACT_ZEROOFFSET_DEFAULT
  },
  .zeroDriftPos =                                         /* int32s zeroDriftPos[ADC_NUM_SYNC_FREQ]; */
  {
    ADC_DB_FACT_ZERODRIFT_DEFAULT, ADC_DB_FACT_ZERODRIFT_DEFAULT, ADC_DB_FACT_ZERODRIFT_DEFAULT, ADC_DB_FACT_ZERODRIFT_DEFAULT,
    ADC_DB_FACT_ZERODRIFT_DEFAULT, ADC_DB_FACT_ZERODRIFT_DEFAULT, ADC_DB_FACT_ZERODRIFT_DEFAULT, ADC_DB_FACT_ZERODRIFT_DEFAULT,
    ADC_DB_FACT_ZERODRIFT_DEFAULT
  },
  .zeroDriftNeg =                                         /* int32s zeroDriftNeg[ADC_NUM_SYNC_FREQ]; */
  {
    ADC_DB_FACT_ZERODRIFT_DEFAULT, ADC_DB_FACT_ZERODRIFT_DEFAULT, ADC_DB_FACT_ZERODRIFT_DEFAULT, ADC_DB_FACT_ZERODRIFT_DEFAULT,
    ADC_DB_FACT_ZERODRIFT_DEFAULT, ADC_DB_FACT_ZERODRIFT_DEFAULT, ADC_DB_FACT_ZERODRIFT_DEFAULT, ADC_DB_FACT_ZERODRIFT_DEFAULT,
    ADC_DB_FACT_ZERODRIFT_DEFAULT
  },
  .spanCal =                                              /* int32s spanCal[ADC_NUM_SYNC_FREQ]; */
  {
    ADC_DB_FACT_SPANCAL_DEFAULT0, ADC_DB_FACT_SPANCAL_DEFAULT1, ADC_DB_FACT_SPANCAL_DEFAULT2, ADC_DB_FACT_SPANCAL_DEFAULT3,
    ADC_DB_FACT_SPANCAL_DEFAULT4, ADC_DB_FACT_SPANCAL_DEFAULT5, ADC_DB_FACT_SPANCAL_DEFAULT6, ADC_DB_FACT_SPANCAL_DEFAULT7,
    ADC_DB_FACT_SPANCAL_DEFAULT8
  },
  .spanDriftPos =                                         /* int32s spanDrift[ADC_NUM_SYNC_FREQ]; */
  {
    ADC_DB_FACT_SPANDRIFT_DEFAULT, ADC_DB_FACT_SPANDRIFT_DEFAULT, ADC_DB_FACT_SPANDRIFT_DEFAULT, ADC_DB_FACT_SPANDRIFT_DEFAULT,
    ADC_DB_FACT_SPANDRIFT_DEFAULT, ADC_DB_FACT_SPANDRIFT_DEFAULT, ADC_DB_FACT_SPANDRIFT_DEFAULT, ADC_DB_FACT_SPANDRIFT_DEFAULT,
    ADC_DB_FACT_SPANDRIFT_DEFAULT
  },
  .spanDriftNeg =                                         /* int32s spanDrift[ADC_NUM_SYNC_FREQ]; */
  {
    ADC_DB_FACT_SPANDRIFT_DEFAULT, ADC_DB_FACT_SPANDRIFT_DEFAULT, ADC_DB_FACT_SPANDRIFT_DEFAULT, ADC_DB_FACT_SPANDRIFT_DEFAULT,
    ADC_DB_FACT_SPANDRIFT_DEFAULT, ADC_DB_FACT_SPANDRIFT_DEFAULT, ADC_DB_FACT_SPANDRIFT_DEFAULT, ADC_DB_FACT_SPANDRIFT_DEFAULT,
    ADC_DB_FACT_SPANDRIFT_DEFAULT
  },
  .calTemp = ADC_DB_FACT_CALTEMP_DEFAULT,                 /* int32u calTemp; */
  .calFlag = ADC_DB_FACT_CALFLAG_DEFAULT,                 /* int8u  calFlag; */

  .chksum = ADC_DB_FACT_CHKSUM_DEFAULT                    /* int32u chksum; */
};


/** Set a factory database to defaults */
void AdcIntFactoryDatabaseDefault(AdcFactoryDatabase *afd)
{
  memset(afd, 0, sizeof(AdcFactoryDatabase));
  *afd = s_C_DefaultAdcFactoryDatabase;
}


/** Check the factory database checksum */
bool AdcIntFactoryDatabaseCheckChecksum(const AdcFactoryDatabase *afd)
{
  return DB_CHECKSUM_GOOD(AdcFactoryDatabase, *afd, ADC_DB_FACTORY_CHECKSUM_SEED);
}


/** Set the factory database checksum */
void AdcIntFactoryDatabaseSetChecksum(AdcFactoryDatabase *afd)
{
  DB_SET_CHECKSUM(AdcFactoryDatabase, *afd, ADC_DB_FACTORY_CHECKSUM_SEED);
}


/** Compare factory databases, return true if different */
bool AdcIntFactoryDatabaseDiff(const AdcFactoryDatabase *afd1, const AdcFactoryDatabase *afd2)
{
  int i;
  bool different;

  /* Check arrays: zeroOffset, zeroDrift, spanDrift & spanCal */
  for (i = 0, different = false; i < ADC_NUM_SYNC_FREQ; i++)
  {
    if (afd1->zeroOffset[i] != afd2->zeroOffset[i] ||
        afd1->zeroDriftPos[i] != afd2->zeroDriftPos[i] ||
        afd1->zeroDriftNeg[i] != afd2->zeroDriftNeg[i] ||
        afd1->spanCal[i] != afd2->spanCal[i] ||
        afd1->spanDriftPos[i] != afd2->spanDriftPos[i] ||
        afd1->spanDriftNeg[i] != afd2->spanDriftNeg[i])
    {
      different = true;
      break;
    }
  }
  if (!different)
  {
    /* Check the others */
    different = (afd1->calTemp != afd2->calTemp ||
                 afd1->calFlag != afd2->calFlag);
  }
  return different;
}
