/*
 * AdcDbRuntime.c
 *
 *  Created on: 31/03/2011
 *      Author: ian
 */
#include <stdlib.h>
#include <string.h>
#include <rinhw/stdrinstrum.h>
#include <rinhw/rstdio.h>
#include <rinweigh/adcoperations.h>
#include <rinweigh/adcreadings.h>
#include <rinweigh/adcdatatypes.h>
#include <rinweigh/adcmotion.h>
#include <rinweigh/adcinterface.h>
#include <rinweigh/adcdebug.h>

#include <rinweigh/adcdbruntime.h>


/** Default values for the ADC runtime database */
static const AdcRuntimeDatabase s_C_DefaultAdcRTDb =
{
  .cntZero = ADC_DB_RT_CNTZERO_DEFAULT,
  .cntTare = ADC_DB_RT_CNTTARE_DEFAULT,
  .cntPTare = ADC_DB_RT_CNTPTARE_DEFAULT,
  .hiRes = ADC_DB_RT_HIRES_DEFAULT,
  .netMode = ADC_DB_RT_NETMODE_DEFAULT,
  .printId = ADC_DB_RT_PRINTID_DEFAULT,
  .dispMode = ADC_DB_RT_DISPMODE_DEFAULT,
  .tempCalOn = ADC_DB_RT_TEMPCALON_DEFAULT,
  .overloadCnt = ADC_DB_RT_OVERLOADCNT_DEFAULT,

  .chksum = ADC_DB_RT_CHKSUM_DEFAULT
};


/** Set a runtime database to defaults */
void AdcIntRuntimeDatabaseDefault(AdcRuntimeDatabase *ard)
{
  /* Both checksums are bad, or overwrite forced, so blow away database and re-initialise */
  memset(ard, 0, sizeof(AdcRuntimeDatabase));
  *ard = s_C_DefaultAdcRTDb;
}


/** Check the runtime database checksum */
bool AdcIntRuntimeDatabaseCheckChecksum(const AdcRuntimeDatabase *ard)
{
  return DB_CHECKSUM_GOOD(AdcRuntimeDatabase, *ard, ADC_DB_RUNTIME_CHECKSUM_SEED);
}


/** Set the runtime database checksum */
void AdcIntRuntimeDatabaseSetChecksum(AdcRuntimeDatabase *ard)
{
  DB_SET_CHECKSUM(AdcRuntimeDatabase, *ard, ADC_DB_RUNTIME_CHECKSUM_SEED);
}


/** Compare runtime databases, return true if different */
bool AdcIntRuntimeDatabaseDiff(const AdcRuntimeDatabase *ard1, const AdcRuntimeDatabase *ard2)
{
#if ADC_DEBUG(ADC_DB_RUNTIME_DEBUG_LVL, ADC_DEBUG_BASIC)

  #define DEBUG_RT_CHECK_DIFF(mbr) \
    if (ard1->mbr != ard2->mbr) \
      ADC_DEBUG_PRINTF(LOG_DEBUG, "AdcIntRuntimeDatabaseDiff: " #mbr " has changed from %d to %d\n", ard2->mbr, ard1->mbr)

  DEBUG_RT_CHECK_DIFF(cntZero);
  DEBUG_RT_CHECK_DIFF(cntTare);
  DEBUG_RT_CHECK_DIFF(cntPTare);
  DEBUG_RT_CHECK_DIFF(hiRes);
  DEBUG_RT_CHECK_DIFF(netMode);
  DEBUG_RT_CHECK_DIFF(printId);
  DEBUG_RT_CHECK_DIFF(dispMode);
  DEBUG_RT_CHECK_DIFF(tempCalOn);
  DEBUG_RT_CHECK_DIFF(overloadCnt);
#endif

  return (ard1->cntZero != ard2->cntZero ||
          ard1->cntTare != ard2->cntTare ||
          ard1->cntPTare != ard2->cntPTare ||
          ard1->hiRes != ard2->hiRes ||
          ard1->netMode != ard2->netMode ||
          ard1->printId != ard2->printId ||
          ard1->dispMode != ard2->dispMode ||
          ard1->tempCalOn != ard2->tempCalOn ||
          ard1->overloadCnt != ard2->overloadCnt);
}
