/*
 * adcdbsave.c
 *
 *  Created on: May 8, 2012
 *      Author: ian
 */
#include <rinweigh/adcdatatypes.h>
#include <rinweigh/adcdbfactory.h>
#include <rinweigh/adcdbuser.h>
#include <rinweigh/adcdbruntime.h>
#include <rinweigh/adcmutex.h>

#include <rinweigh/adcdbsave.h>
#include <rinweigh/adcinterface.h>


/**
 * Internal saving of ADC databases without locking
 *
 * @param ass is the ADC super structure, provided for your reference. Don't save the databases included in this.
 * @param afd is a pointer to a factory database to be saved. Can be NULL in which case shouldn't be saved.
 * @param aud is a pointer to a user database to be saved. Can be NULL in which case shouldn't be saved.
 * @param ard is a pointer to a runtime database to be saved. Can be NULL in which case shouldn't be saved.
 * @param aud_reference is a pointer to a user database used as a reference to determine whether to increment the calibration counters. Can be NULL only if aud is NULL
 *
 * @return result of the save
 */
bool AdcIntSaveDatabases(AdcSuperStructure *ass,
                         AdcFactoryDatabase *afd,
                         AdcUserDatabase *aud,
                         AdcRuntimeDatabase *ard,
                         const AdcUserDatabase *aud_reference)
{
  bool result = false;

  /* Set checksums */
  if (NULL != afd)
  {
    AdcIntFactoryDatabaseSetChecksum(afd);
  }
  if (NULL != aud && NULL != aud_reference)
  {
    /* Increment calibration counters, then save */
    AdcIntUserDatabaseIncrementCalCountersOnChanges(ass, aud, aud_reference);
    AdcIntUserDatabaseSetChecksum(aud);
  }
  if (NULL != ard)
  {
    AdcIntRuntimeDatabaseSetChecksum(ard);
  }

  /* Call the saver in your application */
  if (NULL != ass->adcMiscellaneous.adcDatabaseSaveImplementerFn)
  {
    result = ass->adcMiscellaneous.adcDatabaseSaveImplementerFn(ass, afd, aud, ard);
  }
  return result;
}


/**
 * Saving of ADC databases with locking
 *
 * @param ass is the ADC super structure.
 * @param lock specifies if the function should lock ass before performing it's function. This allows the caller to have locked the structure prior to the call
 * @param afd is a pointer to a factory database to be defaulted. Can be NULL in which case shouldn't be defaulted.
 * @param aud is a pointer to a user database to be defaulted. Can be NULL in which case shouldn't be defaulted.
 * @param ard is a pointer to a runtime database to be defaulted. Can be NULL in which case shouldn't be defaulted.
 * @param aud_reference is a pointer to a user database used as a reference to determine whether to increment the calibration counters. Can be NULL only if aud is NULL
 *
 * @return result of the save
 */
bool AdcSaveDatabases(AdcSuperStructure *ass,
                      bool lock,
                      AdcFactoryDatabase *afd,
                      AdcUserDatabase *aud,
                      AdcRuntimeDatabase *ard,
                      const AdcUserDatabase *aud_reference)
{
  bool result;

  ADC_INT_IFACE_LOCK();
  result = AdcIntSaveDatabases(ass, afd, aud, ard, aud_reference);
  ADC_INT_IFACE_UNLOCK();
  return result;
}
