#ifndef ADCDEBUG_DOT_H
#define ADCDEBUG_DOT_H 1

#include <syslog.h>
#include "adcdatatypes.h"


#define ADC_DEBUG_OFF 0
#define ADC_DEBUG_BASIC 1
#define ADC_DEBUG_FULL 2


/** Temperature compensation debug */
#define ADC_TEMP_COMP_DEBUG_LVL ADC_DEBUG_OFF


/** ADC hardware access debug */
#define ADC_HARDWARE_DEBUG_LVL ADC_DEBUG_OFF


/** ADC reading update debug (when a reading is taken) */
#define ADC_UPDATE_DEBUG_LVL ADC_DEBUG_OFF


/** ADC calculation debug */
#define ADC_CALCULATION_DEBUG_LVL ADC_DEBUG_OFF


/** ADC initialisation debug */
#define ADC_INITIALISATION_DEBUG_LVL ADC_DEBUG_OFF


/*
 * Debug for averaging
 */


/** ADC averaging initialisation debug */
#define ADC_AVERAGE_INIT_DEBUG_LVL ADC_DEBUG_OFF


/** ADC averaging debug (debug of the averaging function @ref AdcIntAverageReading) */
#define ADC_AVERAGE_AVERAGING_DEBUG_LVL ADC_DEBUG_OFF


/** ADC reading primary averaging debug */
#define ADC_AVERAGE_READING_DEBUG_LVL ADC_DEBUG_OFF


/** ADC jitter averaging debug */
#define ADC_AVERAGE_JITTER_DEBUG_LVL ADC_DEBUG_OFF


/*
 * Debug for zero tracking
 */


/** ADC zero tracking initialisation debug */
#define ADC_ZERO_TRACKING_INIT_DEBUG_LVL ADC_DEBUG_OFF


/*
 * Database debug
 */


/** ADC runtime database */
#define ADC_DB_RUNTIME_DEBUG_LVL ADC_DEBUG_OFF


/*
 * Live weigh debug
 */


/** ADC improved live weigh debug */
#define ADC_LIVE_IMPROVED_DEBUG_LVL ADC_DEBUG_OFF


#define ADC_DEBUG(mod, lvl) (mod >= lvl)
#define ADC_DEBUG_PRINTF AdcIntDebugPrintf


/* Include the ADC averaging dump function if any of the averaging debug is turned on */
#if (ADC_DEBUG(ADC_AVERAGE_INIT_DEBUG_LVL, ADC_DEBUG_FULL) || \
     ADC_DEBUG(ADC_AVERAGE_AVERAGING_DEBUG_LVL, ADC_DEBUG_FULL) || \
     ADC_DEBUG(ADC_AVERAGE_READING_DEBUG_LVL, ADC_DEBUG_FULL) || \
     ADC_DEBUG(ADC_AVERAGE_JITTER_DEBUG_LVL, ADC_DEBUG_FULL))
  /** ADC averaging dump function debug */
  #define ADC_AVERAGE_DUMP_DEBUG_LVL ADC_DEBUG_FULL
#else
  /** ADC averaging dump function debug */
  #define ADC_AVERAGE_DUMP_DEBUG_LVL ADC_DEBUG_OFF
#endif


/** Set the debug printf function */
void AdcIntDebugSetVPrintfImplementerFn(AdcDebugVPrintfImplementerFn fn);


/** Debug printf function for the ADC library, calls the application implemented handler */
void AdcIntDebugPrintf(int priority, const char *format, ...);


#endif /* #ifndef ADCDEBUG_DOT_H */
