#include <stdlib.h>
#include <limits.h>
#include <rinhw/stdrinstrum.h>

#include <rinweigh/adcdlong.h>
#include <rinweigh/adcinterface.h>


#ifdef __HITACHI__

/**
 * Perform a multiply followed by and divide on base. Uses 64 bit internals.
 *
 * @param base is the number to be multiplied and then divided
 * @param mult is the multiplier
 * @param div is the divisor
 * @return is the the result of base * mult / div
 */
int32s AdcMultDiv(int32s base, int32s mult, int32s div)
{

  /*
   * The following code is endian dependant. For non-hitachi support, we
   * rely on the "long long" data type
   */
  int8u minus;
  ulor2w tmp, mid;
  dlong dl;
  ulor2w b, m;

  /* Do some quick checks */
  if ( 0L == div )
  {
    return INT32S_MAX;
  }
  if ( 0L == base || 0L == mult )
  {
    return 0L;
  }

  /* Work out what sign result will be */
  minus = 0;
  if (base < 0)
  {
    base = -base;
    minus++;
  }
  if (mult < 0)
  {
    mult = -mult;
    minus++;
  }
  if (div < 0)
  {
    div = -div;
    minus++;
  }

  b.l = base;
  m.l = mult;

  /* Do multiply */
  dl.lw[0] = (int32u)(b.w[0]) * (int32u)(m.w[0]);
  dl.lw[1] = (int32u)(b.w[1]) * (int32u)(m.w[1]);
  mid.w[0] = 0;
  mid.w[1] = dl.w[2];
  tmp.l = (int32u)(b.w[0]) * (int32u)(m.w[1]);
  dl.lw[0] += (int32u)(tmp.w[0]);
  mid.l += (int32u)(tmp.w[1]);
  tmp.l = (int32u)(b.w[1]) * (int32u)(m.w[0]);
  dl.lw[0] += (int32u)(tmp.w[0]);
  mid.l += (int32u)(tmp.w[1]);
  dl.w[2] = mid.w[1];
  dl.lw[0] += (int32u)(mid.w[0]);

  //b.l = dl.lw[1];
  //if (minus & 0x01) b.l = -b.l;
  //return(b.l);

  /* Divide algorithm cannot handle a div > 0x00FFFFFF, so shift everything down */
  while ( div > 0x00FFFFFF )
  {
    div >>= 1;
    dl.lw[1] >>= 1;
    if ( dl.b[3] & 0x01 )
    {
      dl.b[4] |= 0x80;
    }
    dl.lw[0] >>= 1;
  }

  /* Now do divide */
  m.b[0] = dl.b[1];
  m.b[1] = dl.b[2];
  m.b[2] = dl.b[3];
  m.b[3] = dl.b[4];
  b.b[0] = (int8u)(m.l/div);

  m.l = (m.l % div)<<8;
  m.b[3] = dl.b[5];
  b.b[1] = (int8u)(m.l/div);

  m.l = (m.l % div)<<8;
  m.b[3] = dl.b[6];
  b.b[2] = (int8u)(m.l/div);

  m.l = (m.l % div)<<8;
  m.b[3] = dl.b[7];
  b.b[3] = (int8u)(m.l/div);

  /* Put sign back into result */
  if ( minus & 0x01 )
  {
    b.l = -b.l;
  }

  return (int32s) b.l;
}


#else /* #ifdef __HITACHI__ */


/**
 * Perform a multiply followed by and divide on base. Uses 64 bit internals.
 *
 * @param base is the number to be multiplied and then divided
 * @param mult is the multiplier
 * @param div is the divisor
 * @return is the the result of base * mult / div
 */
int32s AdcMultDiv(int32s base, int32s mult, int32s div)
{
  long long intermediate;

  intermediate = base;
  intermediate *= mult;
  intermediate /= div;
  return (int32s) intermediate;
}

#endif /* #else -> #ifdef __HITACHI__ */
