#include <rinhw/stdrinstrum.h>
#include <rinweigh/adcdatatypes.h>
#include <rinweigh/adcdatabase.h>
#include <rinweigh/adcreadings.h>
#include <rinweigh/adcinterface.h>

#include <rinweigh/adclin.h>


void AdcIntLinearisationInit(const AdcUserDatabase *aud, AdcLinearisation *linear, int32s cntFullscale)
{
  int16s inc, len, first, second;
  int16s pt[ADC_MAX_NUM_LIN_PTS + 2];
  int8u j;
  int32s tmpl, tmpl2;
  int i;

  // calculate
  tmpl = cntFullscale;
  linear->fsShr = 0;
  while (tmpl >= 0x80L)
  {
    tmpl >>=1;
    linear->fsShr++;
  }

  // initialise linearity table
  for (i = 0; i < ADC_LINEAR_POINTS; i++)
    linear->linTable[i] = INT16S_MAX;

  // insert linearity table points
  for (j = 0; j < ADC_MAX_NUM_LIN_PTS + 1; j++)
    linear->linTable[aud->linPts[j]] = aud->linCor[j];
  linear->linTable[128] = 0;   // zero

  for (j = 0; j < ADC_MAX_NUM_LIN_PTS + 2; j++)
    pt[j] = -1;
  for (i = 0, j = 0; i < ADC_LINEAR_POINTS; i++)
  {
    if (linear->linTable[i] < INT16S_MAX)
      pt[j++] = i;
  }
  inc = 0; len = 1;
  first = pt[0]; second = pt[1]; j = 2;
  i = 0;
  while (i < ADC_LINEAR_POINTS)
  {
    if (second != -1)
    {
      inc = (linear->linTable[second] - linear->linTable[first]);
      len = second - first;
    }
    /* was below but it would not compile
    tmpl += ((int32s)(inc) * (int32s)(i - first) / (int32s)(len));
    */
    tmpl = linear->linTable[first];
    tmpl2 = ((int32s)i) - first;
    tmpl2 *= inc;
    tmpl += (tmpl2 / len);
    if (tmpl > 2000L) tmpl = 2000L;
    if (tmpl < -2000L) tmpl = -2000L;
    linear->linTable[i] = (int16s)(tmpl);

/*      send_int32s(i); out_comma();
      send_int32s(tmpl);
      send_end();
      delay_milli(20); set_dmt_count();
*/
    i++;
    if (i == second)
    {
      first = second;
      if (j < ADC_MAX_NUM_LIN_PTS + 2)
      {
        second = pt[j];
        j++;
      }
      else
        second = -1;
    }
  }
}


int32s AdcIntLinearisationCorrection(const AdcLinearisation *linear, int32s rdg)
{
  return (int32s) (linear->linTable[AdcIntLinearisationGetLinPt(linear, rdg)]);
}


/**
 * Get linearisation point at a weight
 *
 * @param rdg is the weight to get the linearisation point for
 * @return the linearisation point at rdg
 */
int8u AdcIntLinearisationGetLinPt(const AdcLinearisation *linear, int32s rdg)
{
  int8u pt;

  if (rdg < 0)
  {
    rdg = -rdg;
    rdg >>= linear->fsShr;
    if (rdg > 128) pt = 0;
    else pt = (int8u)(128 - (int8u)(rdg));
  }
  else
  {
    rdg >>= linear->fsShr;
    if (rdg > 127) pt = 255;
    else pt = (int8u)(128 + (int8u)(rdg));
  }

  return pt;
}


/**
 * Get the weight at a linearisation point
 *
 * @param pnt is the point to return the weight for
 * @return the weight at linearisation point pnt
 */
int32s AdcIntLinearisationGetLinWgt(const AdcLinearisation *linear, int8u pnt)
{
  int32s rdg;

  if (pnt < 128)
  {
    rdg = 128 - ((int32s)pnt);
    rdg <<= linear->fsShr;
    rdg = -rdg;
  }
  else
  {
    rdg = ((int32s) pnt) - 128;
    rdg <<= linear->fsShr;
  }

  return rdg;
}


/** Clear a particular linearisation point in the database */
void AdcIntClearDbLinear(AdcSuperStructure *ass, int8u pnt)
{
  ass->adcUserDatabase.linPts[pnt] = 128;
  ass->adcUserDatabase.linCor[pnt] = 0;
}


/** Clear all linearisation points in the database */
void AdcIntClearAllDbLinear(AdcSuperStructure *ass)
{
  int i;

  for (i = 0; i < ADC_MAX_NUM_LIN_PTS; i++)
  {
    AdcIntClearDbLinear(ass, i);
  }
}
