/*
 * adcliveimpl.h
 *
 *  Created on: Mar 31, 2012
 *      Author: ian
 */

#ifndef ADCLIVEIMPL_H_
#define ADCLIVEIMPL_H_

#include <rinhw/stdrinstrum.h>


enum eAdcLiveWeighResult
{
  alwrNotImplemented,
  alwrDisabled,
  alwrRunning,
  alwrComplete,
  alwrFailed
};


/* Forward declaration */
struct sAdcLiveWeighImplementer;


/** Structure used to implement live weigh for the ADC library */
typedef struct sAdcLiveWeighImplementer AdcLiveWeighImplementer;


/**
 * Run the simple live weigh
 * Needs to be called at the sample frequency
 *
 * @param alwi is the live weigh implementer structure
 * @param rdg is the reading to be averaged
 * @param resultrdg is a pointer to a value set with the live weigh result on success
 *
 * @return alwrComplete if a live weight has been obtained
 */
typedef enum eAdcLiveWeighResult (*AdcLiveWeighRunFn)(AdcLiveWeighImplementer *alwi, int32s rdg, int32s *resultrdg);


/** Initiate live weighing function */
typedef enum eAdcLiveWeighResult (*AdcLiveWeighInitiateFn)(AdcLiveWeighImplementer *alwi);


/** Disable simple live weighing */
typedef enum eAdcLiveWeighResult (*AdcLiveWeighDisableFn)(AdcLiveWeighImplementer *alwi);


/**
 * Function to determine if the live weigh is currently running
 *
 * @param alwi is the live weigh implementer structure
 * @param success is a pointer to a bool set to true if the live weigh has completed successfully
 */
typedef enum eAdcLiveWeighResult (*AdcLiveWeighRunningFn)(AdcLiveWeighImplementer *alwi);


/** Destroy live weigh function */
typedef void (*AdcLiveWeighDestroyFn)(AdcLiveWeighImplementer *alwi);


/** Live weigh implementer */
struct sAdcLiveWeighImplementer
{
  void *implementer_data;

  AdcLiveWeighRunFn AdcLiveWeighRun;
  AdcLiveWeighInitiateFn AdcLiveWeighInitiate;
  AdcLiveWeighDisableFn AdcLiveWeighDisable;
  AdcLiveWeighRunningFn AdcLiveWeighRunning;
  AdcLiveWeighDestroyFn AdcLiveWeighDestroy;
};


/** Destroy a live weigh implementer */
void AdcLiveWeighDestroy(AdcLiveWeighImplementer *alwi);


#endif /* ADCLIVEIMPL_H_ */
