/*
 * adcliveimpl_simple.h
 *
 *  Created on: Mar 31, 2012
 *      Author: ian
 */

#ifndef ADCLIVEIMPL_SIMPLE_H_
#define ADCLIVEIMPL_SIMPLE_H_

#include <stdint.h>
#include "adcliveimpl.h"
#include "adcdatatypes.h"


/** Struct to implement simple live weighing */
typedef struct
{
  bool live_enabled;
  int64_t live_total;               /*!< Total */
  int32u live_averaged;             /*!< How many readings collected so far */
  int32s live_result;               /*!< What we worked out the live weight to be */
  int32u live_main_average_samples;

  /* Filled out by configure function */
  int32u live_toaverage;            /*!< How many readings to average. If 0, then live weigh is disabled */
  int32u live_main_average_size;
} AdcLiveSimple;


/** Create a simple live weigh implementer */
AdcLiveWeighImplementer *AdcLiveWeighImplementerSimpleCreate(void);


/**
 * Configure a simple live weigh implementer
 *
 * This live weigh works by averaging a large number of samples
 *
 * @param ass is the ADC super structure running this live weigh. ass mutex should be locked before calling this function
 * @param alwi is an initialised improved live weigh implementer
 * @param averagingMilliseconds is the number of milliseconds to average data for
 *
 * @return one of the ADC_OP_STATUS_* codes
 */
int8u AdcLiveWeighImplementerSimpleConfigure(AdcSuperStructure *ass,
                                             AdcLiveWeighImplementer *alwi,
                                             int32u averagingMilliseconds);


#endif /* ADCLIVEIMPL_SIMPLE_H_ */
