#include <rinhw/stdrinstrum.h>
#include <rinweigh/adcdatatypes.h>
#include <rinweigh/adcaverage.h>

#include <rinweigh/adcmotion.h>


/** Disable motion detection, Motion() will always return false */
void AdcIntMotionDetectionDisable(AdcMotionDetection *motion)
{
  motion->motion_enabled = false;
  motion->motOnCnt = 0;
}


/** Initialise motion detection
 *
 * Must be called with correct values of rawDelta (in counts) and
 * window length in samples and onTime in samples.
 * Note: OnTime should be greater than averaging length.
 */
void AdcIntMotionDetectionInit(AdcMotionDetection *motion, int16u rawDelta, int8u window, int8u onTime)
{
  motion->motRawDelta = rawDelta;
  motion->motDeltaWindow = window;
  motion->motWindow = onTime;

  motion->motSubNum = 1;
  if (motion->motDeltaWindow > ADC_MOTION_MAX_DELTA_RDGS)
  {
    while (motion->motDeltaWindow > ADC_MOTION_MAX_DELTA_RDGS)
    {
      motion->motDeltaWindow -= ADC_MOTION_MAX_DELTA_RDGS;
      motion->motSubNum++;
    }
    motion->motDeltaWindow = ADC_MOTION_MAX_DELTA_RDGS;
  }

  motion->motDeltaTotal = 0;
  motion->motDeltaSubtotal = 0;
  motion->motDeltaPos = 0;
  motion->motDeltaCnt = 0;
  motion->motSubCnt = 0;

  if (motion->motRawDelta > 0)
    motion->motOnCnt = motion->motWindow;
  else
    motion->motOnCnt = 0;

  motion->motion_enabled = true;
}


/** Update motion status */
void AdcIntMotionDetectionUpdate(AdcMotionDetection *motion, int32s delta)
{
  int32s tmp;

  /* Don't run if motion detection not enabled */
  if (!motion->motion_enabled)
  {
    return;
  }

  if (motion->motOnCnt > 0)
    motion->motOnCnt--;   //decrement motion counter

  motion->motLastDelta = delta;

  if (delta < 0)
    tmp = -delta;
  else
    tmp = delta;

  if (tmp < motion->motRawDelta)
  {
    motion->motDeltaSubtotal += delta;
    if (++motion->motSubCnt >= motion->motSubNum)
    {
      motion->motDeltaTotal = motion->motDeltaTotal + motion->motDeltaSubtotal;
      if (motion->motDeltaCnt == motion->motDeltaWindow)
        motion->motDeltaTotal = motion->motDeltaTotal - motion->motDeltaData[motion->motDeltaPos];
      motion->motDeltaData[motion->motDeltaPos] = motion->motDeltaSubtotal;
      motion->motDeltaSubtotal = 0;
      motion->motSubCnt = 0;
      if (++motion->motDeltaPos >= motion->motDeltaWindow)
        motion->motDeltaPos = 0;
      if (motion->motDeltaCnt < motion->motDeltaWindow)
        motion->motDeltaCnt++;
      if (motion->motDeltaTotal < 0)
        motion->motAbsDelta = -motion->motDeltaTotal;
      else
        motion->motAbsDelta = motion->motDeltaTotal;
    }
  }
  else
  {
    motion->motDeltaTotal = 0;
    motion->motDeltaSubtotal = 0;
    motion->motDeltaPos = 0;
    motion->motDeltaCnt = 0;
    motion->motSubCnt = 0;
    motion->motAbsDelta = tmp;
  }

  if (motion->motRawDelta > 0 && motion->motAbsDelta >= motion->motRawDelta)
    motion->motOnCnt = motion->motWindow;
}


/** Return true if motion detected */
bool AdcIntMotionDetectionGetMotion(const AdcMotionDetection *motion)
{
  return (motion->motOnCnt != 0);
}
