/*
 * adcmutex_pthreads.c
 *
 *  Created on: Mar 19, 2012
 *      Author: ian
 */
#include <pthread.h>
#include <stdlib.h>
#include "adcmuteximpl.h"

#include "adcmuteximpl_pthreads.h"


/** Lock the pthreads mutex implementer */
static bool AdcMutexImplementerPthreadsLock(AdcMutexImplementer *ami)
{
  pthread_mutex_t *mutex = (pthread_mutex_t *) ami->implementer_data;

  return (0 == pthread_mutex_lock(mutex));
}


/** Unlock the pthreads mutex implementer */
static bool AdcMutexImplementerPthreadsUnlock(AdcMutexImplementer *ami)
{
  pthread_mutex_t *mutex = (pthread_mutex_t *) ami->implementer_data;

  return (0 == pthread_mutex_unlock(mutex));
}


/** Create a mutex implementer using pthreads */
AdcMutexImplementer *AdcMutexImplementerPthreadsCreate(void)
{
  AdcMutexImplementer *ami = NULL;
  pthread_mutex_t *mutex = NULL;

  /* Make structure */
  ami = malloc(sizeof(AdcMutexImplementer));
  if (NULL == ami)
  {
    goto amifc_fail;
  }

  /* Make and initialise mutex */
  mutex = malloc(sizeof(pthread_mutex_t));
  if (NULL == mutex)
  {
    goto amifc_fail;
  }
  if (0 != pthread_mutex_init(mutex, NULL))
  {
    goto amifc_fail;
  }

  /* Fill out members */
  ami->implementer_data = mutex;
  ami->mutex_lock_fn = AdcMutexImplementerPthreadsLock;
  ami->mutex_unlock_fn = AdcMutexImplementerPthreadsUnlock;

  /* Success */
  return ami;

amifc_fail:
  if (NULL != ami)
  {
    free(ami);
  }
  if (NULL != mutex)
  {
    free(mutex);
  }
  return NULL;
}


/** Create a mutex implementer using pthreads */
bool AdcMutexImplementerPthreadsDestroy(AdcMutexImplementer *ami)
{
  pthread_mutex_t *mutex;
  bool result = false;

  if (NULL != ami)
  {
    if (NULL != ami->implementer_data)
    {
      mutex = (pthread_mutex_t *) ami->implementer_data;
      result = (0 == pthread_mutex_destroy(mutex));
      free(ami->implementer_data);
    }
    free(ami);
    ami = NULL;
  }
  return result;
}
