#ifndef ADC_READINGS_DOT_H
#define ADC_READINGS_DOT_H 1

#include <rinhw/stdrinstrum.h>
#include <rinweigh/adcdatatypes.h>
#include <rinweigh/adcdatabase.h>
#include <rinweigh/adcaverage.h>


/** Calculate which range a number of counts would appear in */
int8u AdcIntGetRangeForCounts(const AdcSuperStructure *ass, int32s cnt);


/** Calculate number of samples @ the current sampling frequency for required averaging (in 0.01s increments), as configured in ass->adcUserDatabase.average */
int32u AdcIntCalculateAverageNumSamples(const AdcSuperStructure *ass);


/** Initialise the readings engine. Can be called more than once. */
void AdcIntReInitReadings(AdcSuperStructure *ass);


/**
 * Update readings. Will rate limit to the sync frequency. Does the hard work.
 *
 * @param ass is a pointer to the ADC super structure containing all information on the ADC, not NULL
 * @param data_wait_failed is a pointer to a bool that is set if there was a read problem,
 *        and you should implement a delay elsewhere, to prevent the ADC thread going to 100% CPU, not NULL
 * @return true if new data acquired (not a skipped reading, or blocked)
 */
bool AdcIntUpdateReadings(AdcSuperStructure *ass, bool *data_wait_failed);


/** Generate the alternate units resolutions */
void AdcIntGenerateAltRes(AdcSuperStructure *ass, enum eAdcUnitSwitch unit);


#endif /* #ifndef ADC_READINGS_DOT_H */
