/*
 * adctemperature.c
 *
 *  Created on: 10/08/2011
 *      Author: ian
 */
#include <string.h>
#include "adcreadings.h"

#include "adctemperature.h"


/** Initialise the temperature calibration structure */
void AdcIntTemperatureInit(AdcSuperStructure *ass, t_adc_get_temperature_fn adc_get_temp_fn)
{
  memset(&ass->adcTempCal, 0, sizeof(AdcTempCal));
  ass->adcTempCal.stage = 2010;
  ass->adcTempCal.start = ADC_TEMP_CAL_DEF_NUM_CALS;
  ass->adcTempCal.zeroCal = 0;
  ass->adcTempCal.spanReg = 0;
  ass->adcTempCal.currentTemp = 0x0258;  /* About 20deg.C */

  /* These get initialise to something better elsewhere */
  ass->adcTempCal.adcTempCalStagePeriod = 0;
  ass->adcTempCal.adcTempCalStagePerMax = 0;

  ass->adcTempCal.adc_get_temperature_fn = adc_get_temp_fn;
}


/** Update the current temperature by calling the user supplied temperature update function */
bool AdcIntUpdateTemperature(AdcSuperStructure *ass)
{
  bool result = false;

  /* Get temperature if we can */
  if (NULL != ass && NULL != ass->adcTempCal.adc_get_temperature_fn)
  {
    if (ass->adcTempCal.adc_get_temperature_fn(ass, &ass->adcTempCal.currentTemp))
    {
      result = true;
    }
  }

  /* Set/clear error based on result */
  if (result)
  {
    ass->adcMiscellaneous.adcErrors &= ~ADC_ERR_TEMPERATURE;
  }
  else
  {
    ass->adcMiscellaneous.adcErrors |= ADC_ERR_TEMPERATURE;
  }
  return result;
}


/**
 * AdcIntStartZeroCalCycle will return immediately. It starts a series of zero muxes
 * and span calibrations. This is the same as the muxes that occur on instrument
 * startup. This process takes about 31s. Valid readings are available during
 * this time but the zero will be changing slightly.
 */
void AdcIntStartZeroCalCycle(AdcTempCal *tempcal)
{
  tempcal->start = ADC_TEMP_CAL_DEF_NUM_CALS;
}

/**
 * AdcIntZeroCalCycleFinished returns true if the cal cycle has finished. This works
 * with startup muxes as well as those started by @ref AdcIntStartZeroCalCycle or
 * @ref AdcDoTemperatureCal (TRUE). Note that muxes will continue to occur while
 * there are ADC errors. In this case, AdcIntZeroCalCycleFinished will always be
 * false. Therefore, some sort of timeout failure should be used.
 */
bool AdcIntZeroCalCycleFinished(const AdcTempCal *tempcal)
{
  return (tempcal->start == 0 && tempcal->stage >= ADC_TEMP_CAL_STAGE_FINISHED);
}
