/*
 * adctemperature.h
 *
 *  Created on: 10/08/2011
 *      Author: ian
 */

#ifndef ADCTEMPERATURE_H_
#define ADCTEMPERATURE_H_

#include <rinhw/stdrinstrum.h>
#include <rinweigh/adcdatatypes.h>
#include <rinweigh/adcinterface.h>


// structure for temperature compensation
#define ADC_TEMP_CAL_STAGE_END_ZERO 1
#define ADC_TEMP_CAL_STAGE_FINISHED (5+ADC_TEMP_CAL_STAGE_END_ZERO)
#define ADC_TEMP_CAL_DEF_NUM_CALS   (ADC_ZERO_MUX_AVERAGE_RDGS*2)


/** Initialise the temperature calibration structure */
void AdcIntTemperatureInit(AdcSuperStructure *ass, t_adc_get_temperature_fn adc_get_temp_fn);


/** Update the current temperature by calling the user supplied temperature update function */
bool AdcIntUpdateTemperature(AdcSuperStructure *ass);


/**
 * AdcIntStartZeroCalCycle will return immediately. It starts a series of zero muxes
 * and span calibrations. This is the same as the muxes that occur on instrument
 * startup. This process takes about 31s. Valid readings are available during
 * this time but the zero will be changing slightly.
 */
void AdcIntStartZeroCalCycle(AdcTempCal *tempcal);


/**
 * AdcIntZeroCalCycleFinished returns true if the cal cycle has finished. This works
 * with startup muxes as well as those started by @ref AdcIntStartZeroCalCycle or
 * @ref AdcDoTemperatureCal (TRUE). Note that muxes will continue to occur while
 * there are ADC errors. In this case, AdcIntZeroCalCycleFinished will always be
 * false. Therefore, some sort of timeout failure should be used.
 */
bool AdcIntZeroCalCycleFinished(const AdcTempCal *tempcal);


#endif /* ADCTEMPERATURE_H_ */
