/*
 * alibi_test.c
 *
 *  Created on: Mar 28, 2012
 *      Author: ian
 */


/*
 * iface_alibi.c
 *
 *  Created on: Feb 21, 2012
 *      Author: ian
 */
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <rinweigh/adcdatabase.h>
#include <rinweigh/adcalibi.h>
#include <rinweigh/adcdsd.h>
#include <rinweigh/adcpacked.h>
#include <rinweigh/adcchecksum.h>
#include "getchar.h"
#include "adcthread.h"

#include "alibi_test.h"



static const char *const alibi_english_strs[aaisiNumStrings] = ADC_ALIBI_INTERNATIONALISATION_ENGLISH_STRS;


/** Build unit strings, must match @ref enum eBuildUnits */
#define BUILD_UNITS_PRINT_STRS { " ", \
                                 "g", \
                                 "kg", \
                                 "t",  \
                                 "lb", \
                                 "oz", \
                                 "N", \
                                 "p", \
                                 "u0", \
                                 "u1", \
                                 "u2", \
                                 "u3", \
                                 "u4", \
                                 "u5", \
                                 "u6", \
                                 "u7" \
                               }


/** Units strings used for printouts */
const char *const build_units_print[ADC_NUM_UNITS] = BUILD_UNITS_PRINT_STRS;


/** Number of digits */
#define NUM_DIGITS (6)


/** NUM_DIGITS_WITH_DPS is number of digits + number of decimal points */
#define NUM_DIGITS_WITH_DPS (NUM_DIGITS * 2)


void test_alibi_force_quit(void *implementer_data, bool *force_quit)
{
  *force_quit = false;
#if 0
  if (quit_setup_menus())
  {
    *force_quit = true;
  }
#endif
}


static void test_alibi_display_strings_single(const char *str, const char *units, const char *status)
{
  printf("|%*s %2s %6s|\r", NUM_DIGITS_WITH_DPS, str, units, status);
}


static void test_alibi_display_strings_double(const char *str1, const char *units1, const char *status1,
                                              const char *str2, const char *units2, const char *status2)
{
  printf("|%*s %2s %6s||%*s %2s %6s|\r",
         NUM_DIGITS_WITH_DPS, str1, units1, status1,
         NUM_DIGITS_WITH_DPS, str2, units2, status2);
}


static void test_alibi_display_allsegs_single(bool on_not_off)
{
  if (on_not_off)
  {
    test_alibi_display_strings_single("******", "**", "******");
  }
  else
  {
    test_alibi_display_strings_single("      ", "  ", "      ");
  }
}


static void test_alibi_display_allsegs_double(bool on_not_off)
{
  if (on_not_off)
  {
    test_alibi_display_strings_double("******", "**", "******", "******", "**", "******");
  }
  else
  {
    test_alibi_display_strings_double("      ", "  ", "      ", "      ", "  ", "      ");
  }
}


static void test_alibi_display_allsegs(void *implementer_data, enum eAdcAlibiDisplayType alibi_display_type, bool on_not_off)
{
  switch (alibi_display_type)
  {
    case aadtSingleDisplay:
      test_alibi_display_allsegs_single(on_not_off);
      break;
    case aadtDoubleDisplay:
      test_alibi_display_allsegs_double(on_not_off);
      break;
    default:
      break;
  }
}


static void test_alibi_annun(const AdcStatus *status, char status_str[5])
{
  /* Gross/net annunciator */
  if (status->bit.grossOrNet)
  {
    status_str[0] = (status->bit.gross) ? 'G' : 'N';
  }
  else
  {
    if (status->bit.preTare)
      status_str[0] = 'P';
    else if (status->bit.tare)
      status_str[0] = 'T';
    else
      status_str[0] = ' ';
  }

  /* Motion annunciator */
  status_str[1] = (status->bit.motion) ? 'M' : ' ';

  /* Centre of zero annunciator */
  status_str[2] = (status->bit.coz) ? 'Z' : ' ';

  /* Range, if multi range/interval */
  if (status->bit.multiRange)
  {
    status_str[3] = status->bit.range + 1 + '0';
  }
  else
  {
    status_str[3] = ' ';
  }
  status_str[4] = '\0';
}


static void test_alibi_get_abbreviated_units_string(void *implementer_data, int8u unitsidx, const char **units_str)
{
  if (unitsidx >= ADC_NUM_UNITS)
  {
    *units_str = build_units_print[ADC_UNITS_NONE];
  }
  else
  {
    *units_str = build_units_print[unitsidx];
  }
}


static void test_alibi_display(void *implementer_data, enum eAdcAlibiDisplayType alibi_display_type, const char **display_strs, const AdcInfo *info)
{
  char status_str1[5];
  char status_str2[5];
  const char *units1;
  const char *units2;

  switch (alibi_display_type)
  {
    case aadtSingleDisplay:
      test_alibi_annun(&info->status, status_str1);
      test_alibi_get_abbreviated_units_string(implementer_data, info->unitdp.bit.units, &units1);
      test_alibi_display_strings_single(*display_strs, units1, status_str1);
      break;
    case aadtDoubleDisplay:
      test_alibi_annun(&info[0].status, status_str1);
      test_alibi_annun(&info[1].status, status_str2);
      test_alibi_get_abbreviated_units_string(implementer_data, info[0].unitdp.bit.units, &units1);
      test_alibi_get_abbreviated_units_string(implementer_data, info[1].unitdp.bit.units, &units2);
      test_alibi_display_strings_double(display_strs[0], units1, status_str1,
                                        display_strs[1], units2, status_str2);
      break;
    default:
      break;
  }
}


static enum eAdcAlibiKeystroke test_alibi_getchar_weigh(char ch)
{
  enum eAdcAlibiKeystroke keystroke;
  switch (ch)
  {
    case 'z':
      keystroke = aakZero;
      break;
    case 't':
      keystroke = aakTare;
      break;
    case 'g':
      keystroke = aakGrossNet;
      break;
    case 'p':
      keystroke = aakPrint;
      break;
    case 'm':
      keystroke = aakMode;
      break;
    case 'T':
      keystroke = aakEnterPresetTare;
      break;
    case 'q':
      keystroke = aakQuit;
      break;
    default:
      keystroke = aakNone;
      break;
  }
  return keystroke;
}


static enum eAdcAlibiKeystroke test_alibi_getchar_dsd_changelog(char ch)
{
  enum eAdcAlibiKeystroke keystroke;
  switch (ch)
  {
    case 'f':
      keystroke = aakDSDRecordFirst;
      break;
    case 'l':
      keystroke = aakDSDRecordLast;
      break;
    case 'p':
      keystroke = aakDSDRecordPrevious;
      break;
    case 's':
      keystroke = aakDSDRecordSelect;
      break;
    case 'n':
      keystroke = aakDSDRecordNext;
      break;
    case 'q':
      keystroke = aakQuit;
      break;
    case 'a':
      keystroke = aakDSDItemNext;
      break;
    case 'b':
      keystroke = aakDSDItemPrevious;
      break;
    case 'm':
      keystroke = aakMode;
      break;
    default:
      keystroke = aakNone;
      break;
  }
  return keystroke;
}


static enum eAdcAlibiKeystroke test_alibi_getchar_counting(char ch)
{
  enum eAdcAlibiKeystroke keystroke;
  switch (ch)
  {
    case 'z':
      keystroke = aakZero;
      break;
    case 't':
      keystroke = aakTare;
      break;
    case 'g':
      keystroke = aakGrossNet;
      break;
    case 'e':
      keystroke = aakCountEnterQty;
      break;
    case 'c':
      keystroke = aakCountCancel;
      break;
    case 'm':
      keystroke = aakMode;
      break;
    case 'q':
      keystroke = aakQuit;
      break;
    default:
      keystroke = aakNone;
      break;
  }
  return keystroke;
}


static void test_alibi_getchar(void *implementer_data, enum eAdcAlibiMode mode, enum eAdcAlibiKeystroke *keystroke, int16u timeout)
{
  char ch;

  if (get_stdin_char_timeout(&ch, timeout))
  {
    switch (mode)
    {
      case aamWeighing:
        *keystroke = test_alibi_getchar_weigh(ch);
        break;

      case aamDSDView:
      case aamChangelogView:
        *keystroke = test_alibi_getchar_dsd_changelog(ch);
        break;

      case aamCounting:
        *keystroke = test_alibi_getchar_counting(ch);
        break;

      default:
        *keystroke = ('q' == ch) ? aakQuit : aakNone;
        break;
    }
  }
  else
  {
    *keystroke = aakNone;
  }
}


static void test_alibi_get_internationalisation_string(void *implementer_data, enum eAdcAlibiInternationisationStringIdx stridx, const char **display_str)
{
  *display_str = alibi_english_strs[stridx];
}


static void test_alibi_num_display(int32s rdg, const AdcInfo *info)
{
  char status_str[5];
  char rdg_str[NUM_DIGITS_WITH_DPS + 1];
  size_t end;
  int i;
  int dp;
  const char *units1;

  dp = info->unitdp.bit.dp;
  test_alibi_annun(&info->status, status_str);
  snprintf(rdg_str, NUM_DIGITS_WITH_DPS, "%*.*ld", NUM_DIGITS, dp + 1, (long) rdg);

  if (0 != dp && dp < NUM_DIGITS)
  {
    end = strlen(rdg_str);

    /* Insert DP, first make room by moving '\0' and characters back to the dp to the right by 1 */
    for (i = 0; i <= dp; i++)
    {
      rdg_str[end - i + 1] = rdg_str[end - i];
    }
    rdg_str[end - dp ] = '.';
  }
  test_alibi_get_abbreviated_units_string(NULL, info->unitdp.bit.units, &units1);
  test_alibi_display_strings_single(rdg_str, units1, status_str);
}


static bool test_alibi_edit_number(void *implementer_data, AdcRdg *rdg, int32s minimum, int32s maximium, bool is_signed)
{
  bool result;
  bool finished;
  char ch;
  int32s editnum;

  editnum = rdg->rdg;
  for (result = false, finished = false; !finished; )
  {
    test_alibi_num_display(editnum, &rdg->info);
    if (!get_stdin_char_block(&ch))
    {
      break;
    }
    if (ch >= '0' && ch <= '9')
    {
      editnum *= 10;
      editnum += ch - '0';
      if (editnum < minimum || editnum > maximium)
      {
        editnum = 0;
      }
    }
    else switch (ch)
    {
      case 0x7F:
        editnum /= 10;
        break;
      case '-':
        if (is_signed)
          editnum = -editnum;
        break;
      case '\n':
      case '\r':
        if (editnum >= minimum && editnum <= maximium)
        {
          result = true;
          finished = true;
          rdg->rdg = editnum;
        }
        break;
    }
  }
  return result;
}


static bool test_alibi_edit_list(void *implementer_data, int8u *idx, const char *const list_strings[], int8u list_length)
{
  int8u index;
  bool result;
  bool finished;
  char ch;

  if (0 == list_length)
  {
    return false;
  }

  index = *idx;
  for (result = false, finished = false; !finished; )
  {
    test_alibi_display_strings_single(list_strings[index], "", "");
    if (!get_stdin_char_block(&ch))
    {
      break;
    }
    switch (ch)
    {
      case '+':
        index++;
        if (index >= list_length)
          index = 0;
        break;
      case '-':
        if (index == 0)
          index = list_length - 1;
        else
          index--;
        break;
      case '\n':
      case '\r':
        result = true;
        finished = true;
        *idx = index;
        break;
    }
  }
  return result;
}


static void test_alibi_print_string(void *implementer_data, const char *print_str)
{
  printf("\nPrintout:\n%s\n", print_str);
}


bool test_alibi_dsd_convert_to_traceable(void *implementer_data, const int8u *data, size_t length, AdcTraceableWeight *atw)
{
  AdcTraceableWeightPacked *atwp = (AdcTraceableWeightPacked *) data;

  if (length < sizeof(AdcTraceableWeightPacked))
  {
    return false;
  }
  else
  {
    AdcTraceableUnpackFromNetwork(atwp, atw);
    return true;
  }
}


bool test_alibi_dsd_convert_from_traceable(void *implementer_data, const AdcTraceableWeight *atw, int8u *data, size_t length)
{
#if 0
  struct s_we_dsd_record *dsdrec = (struct s_we_dsd_record *) data;

  AdcTraceablePackToNetwork(atw, &dsdrec->atwp);
  return true; /* TODO more checks? */
#endif
  return false;
}


/** Run the alibi test */
void adc_alibi_test(enum eAdcAlibiDisplayType alibi_display_type, struct s_adc_dsd *testdsd, struct s_db_changelog_reader *test_changelog_reader, bool dsd_has_altrdg, bool dsd_has_pieces)
{
  struct s_adc_alibi_info adc_alibi_info;
  int32u recordsize;
  char display_work_buf1[ADC_ALIBI_DISPLAY_WORK_BUF_LEN(NUM_DIGITS)];
  char display_work_buf2[ADC_ALIBI_DISPLAY_WORK_BUF_LEN(NUM_DIGITS)];

  /* Fill in implementation data */
  memset(&adc_alibi_info, 0, sizeof(adc_alibi_info));
  adc_alibi_info.implementer_data = NULL;
  adc_alibi_info.adc_alibi_display_type = alibi_display_type;
  adc_alibi_info.num_digits[aadtSingleDisplay] = NUM_DIGITS;
  adc_alibi_info.num_digits[aadtDoubleDisplay] = NUM_DIGITS;
  adc_alibi_info.display_work_buffers[aadtSingleDisplay] = display_work_buf1;
  adc_alibi_info.display_work_buffers[aadtDoubleDisplay] = display_work_buf2;
  adc_alibi_info.alibi_mode = aamWeighing;  /* Start in weighing mode */
  adc_alibi_info.status_can_show_pieces = false; /* Can't display pieces */

  adc_alibi_info.ass = &adcSuperStructure;

  /* Assume no DSD */
  adc_alibi_info.dsd = NULL;
  adc_alibi_info.dsd_record_buffer = NULL;
  adc_alibi_info.dsd_record_buffer_length = 0;
  if (NULL != testdsd && ADC_DSD_SUCCESS == AdcDSDIsRunning(testdsd, true))
  {
    /* DSD working */
    if (ADC_DSD_SUCCESS == AdcDSDGetRecordSize(testdsd, true, &recordsize))
    {
      adc_alibi_info.dsd_record_buffer = malloc(recordsize);
      if (NULL != adc_alibi_info.dsd_record_buffer)
      {
        adc_alibi_info.dsd = testdsd;
        adc_alibi_info.dsd_record_buffer_length = recordsize;
      }
    }
  }

  /* Change log */
  adc_alibi_info.changelog_reader = test_changelog_reader;

  adc_alibi_info.no_piece_counting = !dsd_has_pieces;
  adc_alibi_info.no_alternate_units = !dsd_has_altrdg;
  adc_alibi_info.adc_lib_checksum = ADC_LIBRARY_CHECKSUM;

  /* Function pointers */
  adc_alibi_info.adc_alibi_display_allsegs = test_alibi_display_allsegs;
  adc_alibi_info.adc_alibi_display = test_alibi_display;
  adc_alibi_info.adc_alibi_getchar = test_alibi_getchar;
  adc_alibi_info.adc_alibi_get_internationalisation_string = test_alibi_get_internationalisation_string;
  adc_alibi_info.adc_alibi_force_quit = test_alibi_force_quit;
  adc_alibi_info.adc_alibi_edit_number = test_alibi_edit_number;
  adc_alibi_info.adc_alibi_edit_list = test_alibi_edit_list;
  adc_alibi_info.adc_alibi_get_abbreviated_units_string = test_alibi_get_abbreviated_units_string;
  adc_alibi_info.adc_alibi_print_string = test_alibi_print_string;
  adc_alibi_info.adc_alibi_dsd_convert_to_traceable = test_alibi_dsd_convert_to_traceable;
  adc_alibi_info.adc_alibi_dsd_convert_from_traceable = test_alibi_dsd_convert_from_traceable;
  adc_alibi_info.adc_alibi_convert_adcerrors_to_apperrors = NULL; /* No translation from ADC errors */

  printf("\n"
         "This runs the alibi application\n"
         "In weigh mode: z:Zero, t:Tare, T:Enter PT, g:Gross/Net, p:Print, m:Mode, q:quit\n"
         "In DSD mode: f:First, l:Last, n:Next, p:Prev, a:Advance, b:Reverse, m:Mode, q:quit\n"
         "In Change Log mode: f:First, l:Last, n:Next, p:Prev, a:Advance, b:Reverse, m:Mode, q:quit\n"
         "In counting mode: z:Zero, t:Tare, g:Gross/Net, e:Enter Qty, c:Cancel Counting m:Mode, q:quit\n"
         "In a list: +:Next, -:Prev, Enter:Accept\n"
         "In a number edit: 0-9:Number, -:Change sign, Backspace:Delete digit, Enter:Accept\n"
         "\n"
         "Weight       Units Status\n"
        );

  /* Run the alibi application */
  if (!AdcAlibiRun(&adc_alibi_info))
  {
    /* Alibi failed to run */
    printf("Alibi failed to run\n");
  }

  if (NULL != adc_alibi_info.dsd_record_buffer)
  {
    free(adc_alibi_info.dsd_record_buffer);
  }
}
