/*
 * app_db.c
 *
 *  Created on: Apr 17, 2012
 *      Author: ian
 */
#include <stdio.h>
#include <rinhw/stdrinstrum.h>
#include <rinweigh/adcinterface.h>
#include "app_db_factory.h"
#include "app_db_user.h"
#include "app_db_runtime.h"
#include "adcthread.h"

#include "app_db.h"


/** Initialise and read all the databases. Call only once on startup. */
void app_init_and_read_databases(const char *factorydb_filename, const char *userdb_filename, const char *runtimedb_filename)
{
  bool fdb_needs_default;
  bool udb_needs_default;
  bool rdb_needs_default;

  /* Initialise and read factory database */
  fdb_needs_default = true;
  if (app_init_factory_database(factorydb_filename))
  {
    if (app_read_factory_database())
    {
      fdb_needs_default = false;
    }
  }
  if (fdb_needs_default)
  {
    /* Something went wrong, set to defaults */
    app_default_factory_database();
  }

  /* Initialise and read user database */
  udb_needs_default = true;
  if (app_init_user_database(userdb_filename))
  {
    if (app_read_user_database())
    {
      udb_needs_default = false;
    }
  }
  if (udb_needs_default)
  {
    /* Something went wrong, set to defaults */
    app_default_user_database();
  }

  /* Initialise and read runtime database */
  rdb_needs_default = true;
  if (app_init_runtime_database(runtimedb_filename))
  {
    if (app_read_runtime_database())
    {
      rdb_needs_default = false;
    }
  }
  if (rdb_needs_default)
  {
    /* Something went wrong, set to defaults */
    app_default_runtime_database();
  }
}


/** Destroy all the databases */
void app_destroy_databases(void)
{
  app_destroy_runtime_database();
  app_destroy_user_database();
  app_destroy_factory_database();
}


/** Save the databases */
bool app_save_databases(AdcSuperStructure *ass, const AdcFactoryDatabase *afd, const AdcUserDatabase *aud, const AdcRuntimeDatabase *ard)
{
  bool resf = true;
  bool resu = true;
  bool resr = true;

  if (NULL != afd)
  {
    printf("Writing factory database\n");
    resf = app_write_factory_database(true, false);
  }
  if (NULL != aud)
  {
    printf("Writing user database\n");
    resu = app_write_user_database(true, false);
  }
  if (NULL != ard)
  {
    printf("Writing runtime database\n");
    resr = app_write_runtime_database(true, false);
  }

  return (resf && resu && resr);
}
