/*
 * app_db_factory.c
 *
 *  Created on: Apr 10, 2012
 *      Author: ian
 */
#include <string.h>
#include <stdio.h>
#include <rindb/db_element.h>
#include <rindb/db_database.h>
#include <rindb/db_database_xml.h>
#include <rinweigh/adcinterface.h>
#include "adcthread.h"
#include "app_db.h"
#include "app_db_changelog.h"

#include "app_db_factory.h"


/** Factory span calibration minimums */
static int32s app_db_f_adcscal_min[ADC_NUM_SYNC_FREQ] =
{
  APP_DB_F_ADCSCAL_MIN, APP_DB_F_ADCSCAL_MIN, APP_DB_F_ADCSCAL_MIN, APP_DB_F_ADCSCAL_MIN,
  APP_DB_F_ADCSCAL_MIN, APP_DB_F_ADCSCAL_MIN, APP_DB_F_ADCSCAL_MIN, APP_DB_F_ADCSCAL_MIN,
  APP_DB_F_ADCSCAL_MIN
};


/** Factory span calibration maximums */
static int32s app_db_f_adcscal_max[ADC_NUM_SYNC_FREQ] =
{
  APP_DB_F_ADCSCAL_MAX, APP_DB_F_ADCSCAL_MAX, APP_DB_F_ADCSCAL_MAX, APP_DB_F_ADCSCAL_MAX,
  APP_DB_F_ADCSCAL_MAX, APP_DB_F_ADCSCAL_MAX, APP_DB_F_ADCSCAL_MAX, APP_DB_F_ADCSCAL_MAX,
  APP_DB_F_ADCSCAL_MAX
};


/** Factory span calibration defaults */
static int32s app_db_f_adcscal_defaults[ADC_NUM_SYNC_FREQ] =
{
  APP_DB_F_ADCSCAL_DEFAULT0, APP_DB_F_ADCSCAL_DEFAULT1, APP_DB_F_ADCSCAL_DEFAULT2, APP_DB_F_ADCSCAL_DEFAULT3,
  APP_DB_F_ADCSCAL_DEFAULT4, APP_DB_F_ADCSCAL_DEFAULT5, APP_DB_F_ADCSCAL_DEFAULT6, APP_DB_F_ADCSCAL_DEFAULT7,
  APP_DB_F_ADCSCAL_DEFAULT8
};


/* Forward declaration */
struct s_db_app_factory db_app_factory;


struct s_db_element app_factory_elements[] =
{
  DB_DECLARE_INT32_ARRAY_ELEM_SINGLE(        APP_DB_F_ADCZOFFS_STR,       NULL,        db_app_factory.adc_factory_database.zeroOffset,     ADC_NUM_SYNC_FREQ,        APP_DB_F_ADCZOFFS_MIN,      APP_DB_F_ADCZOFFS_MAX,       APP_DB_F_ADCZOFFS_DEFAULT,     DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_FACTORY),
  DB_DECLARE_INT32_ARRAY_ELEM_SINGLE(        APP_DB_F_ADCZDRIFTPOS_STR,   NULL,        db_app_factory.adc_factory_database.zeroDriftPos,   ADC_NUM_SYNC_FREQ,        APP_DB_F_ADCZDRIFT_MIN,     APP_DB_F_ADCZDRIFT_MAX,      APP_DB_F_ADCZDRIFT_DEFAULT,    DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_FACTORY),
  DB_DECLARE_INT32_ARRAY_ELEM_SINGLE(        APP_DB_F_ADCZDRIFTNEG_STR,   NULL,        db_app_factory.adc_factory_database.zeroDriftNeg,   ADC_NUM_SYNC_FREQ,        APP_DB_F_ADCZDRIFT_MIN,     APP_DB_F_ADCZDRIFT_MAX,      APP_DB_F_ADCZDRIFT_DEFAULT,    DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_FACTORY),
  DB_DECLARE_INT32_ARRAY_ELEM_INDIVIDUAL(    APP_DB_F_ADCSCAL_STR,        NULL,        db_app_factory.adc_factory_database.spanCal,        ADC_NUM_SYNC_FREQ,        app_db_f_adcscal_min,       app_db_f_adcscal_max,        app_db_f_adcscal_defaults,     DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_FACTORY),
  DB_DECLARE_INT32_ARRAY_ELEM_SINGLE(        APP_DB_F_ADCSDRIFTPOS_STR,   NULL,        db_app_factory.adc_factory_database.spanDriftPos,   ADC_NUM_SYNC_FREQ,        APP_DB_F_ADCSDRIFT_MIN,     APP_DB_F_ADCSDRIFT_MAX,      APP_DB_F_ADCSDRIFT_DEFAULT,    DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_FACTORY),
  DB_DECLARE_INT32_ARRAY_ELEM_SINGLE(        APP_DB_F_ADCSDRIFTNEG_STR,   NULL,        db_app_factory.adc_factory_database.spanDriftNeg,   ADC_NUM_SYNC_FREQ,        APP_DB_F_ADCSDRIFT_MIN,     APP_DB_F_ADCSDRIFT_MAX,      APP_DB_F_ADCSDRIFT_DEFAULT,    DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_FACTORY),
  DB_DECLARE_UINT32_ELEM(                    APP_DB_F_ADCCALTEMP_STR,     NULL,        &db_app_factory.adc_factory_database.calTemp,                                 APP_DB_F_ADCCALTEMP_MIN,    APP_DB_F_ADCCALTEMP_MAX,     APP_DB_F_ADCCALTEMP_DEFAULT,   DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_FACTORY),
  DB_DECLARE_UINT8_ELEM(                     APP_DB_F_ADCCALFLAG_STR,     NULL,        &db_app_factory.adc_factory_database.calFlag,                                 APP_DB_F_ADCCALFLAG_MIN,    APP_DB_F_ADCCALFLAG_MAX,     APP_DB_F_ADCCALFLAG_DEFAULT,   DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_FACTORY),
  DB_DECLARE_UINT32_ELEM(                    APP_DB_F_ADCCHKSUM_STR,      NULL,        &db_app_factory.adc_factory_database.chksum,                                  APP_DB_F_ADCCHKSUM_MIN,     APP_DB_F_ADCCHKSUM_MAX,      APP_DB_F_ADCCHKSUM_DEFAULT,    DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_FACTORY),
};


static struct s_db_database app_factory_database =
{
  .db_database_version = APP_DB_F_VERSION1,
  .db_database_chksum_seed = APP_DB_F_CHECKSUM_SEED,
  .db_elements = app_factory_elements,
  .db_num_elements = DB_NUM_ELEMENTS(app_factory_elements),
  .db_name = APP_DB_F_NAME
};


static struct s_db_database_interp app_factory_ddi_xml;


/**
 * Write the factory database to file
 *
 * @param write_ass_copy specifies whether the adcSuperStructure copy of the adcFactoryDatabase should be written
 * @param lock specifies whether adcSuperStructure should be locked before writing
 */
bool app_write_factory_database(bool write_ass_copy, bool lock)
{
  enum eDBError res;

  if (write_ass_copy)
  {
    /* Write copy from adcSuperStructure */
    if (lock)
      AdcMutexLock(&adcSuperStructure);
    db_app_factory.adc_factory_database = adcSuperStructure.adcFactoryDatabase;
    if (lock)
      AdcMutexUnlock(&adcSuperStructure);
  }

  /* Write the database */
  res = db_database_write(&app_factory_database, &app_factory_ddi_xml, app_factory_database.db_database_version);
  if (dbeSuccess != res)
  {
    printf("app_write_factory_database: db_database_write() returned %d\n", res);
  }

  /* Update the change log */
  (void) app_changelog_update(&app_factory_database);

  return (dbeSuccess == res);
}


/** Read the factory database from file */
bool app_read_factory_database(void)
{
  enum eDBError res;
  uint32_t version;

  /* Read database from file */
  res = db_database_read(&app_factory_database, &app_factory_ddi_xml, &version, false);
  if (dbeSuccess != res)
  {
    printf("app_read_factory_database: db_database_read() returned %d\n", res);
  }

  /* Update previous data, so change log will work */
  db_database_update_previous(&app_factory_database);

  return (dbeSuccess == res);
}


/** Default the factory database */
bool app_default_factory_database(void)
{
  enum eDBError res;

  res = db_database_default(&app_factory_database);
  if (dbeSuccess != res)
  {
    printf("app_default_factory_database: db_database_default() returned %d\n", res);
  }

  /* Use the ADC database defaulter for ADC values */
  AdcDefaultDatabases(&adcSuperStructure, true, &db_app_factory.adc_factory_database, NULL, NULL, false, false);

  return (dbeSuccess == res);
}


/** Initialise the factory database */
bool app_init_factory_database(const char *factorydb_filename)
{
  enum eDBError res;

  /* Initialise the database (creates previous data for some elements) */
  res = db_database_init(&app_factory_database);
  if (dbeSuccess == res)
  {
    res = db_create_database_interp_xml(&app_factory_ddi_xml, factorydb_filename);
    if (dbeSuccess != res)
    {
      printf("app_init_factory_database: db_create_database_interp_xml() returned %d\n", res);
    }
  }
  else
  {
    printf("app_init_factory_database: db_database_init() returned %d\n", res);
  }
  return (dbeSuccess == res);
}


/** Destroy the factory database */
bool app_destroy_factory_database(void)
{
  (void) db_database_destroy(&app_factory_database);
  (void) db_destroy_database_interp_xml(&app_factory_ddi_xml);
  return true;
}
