/*
 * db_factory.h
 *
 *  Created on: Apr 10, 2012
 *      Author: ian
 */

#ifndef APP_DB_FACTORY_H_
#define APP_DB_FACTORY_H_

#include <stddef.h>
#include <stdint.h>
#include <limits.h>
#include <rinweigh/adcdatatypes.h>


/** Application factory database definition */
struct s_db_app_factory
{
  AdcFactoryDatabase adc_factory_database;
};


/** Application factory database */
extern struct s_db_app_factory db_app_factory;


#define APP_DB_F_NAME "factorydb"
#define APP_DB_F_CHECKSUM_SEED 0x12345678


/** Initial factory database version */
#define APP_DB_F_VERSION1 1


/*
 * ADC database settings
 */


#define APP_DB_F_ADCZOFFS_STR "adczerooffset"
#define APP_DB_F_ADCZOFFS_MIN ADC_DB_FACT_ZEROOFFSET_MINIMUM
#define APP_DB_F_ADCZOFFS_MAX ADC_DB_FACT_ZEROOFFSET_MAXIMUM
#define APP_DB_F_ADCZOFFS_DEFAULT ADC_DB_FACT_ZEROOFFSET_DEFAULT

#define APP_DB_F_ADCZDRIFTPOS_STR "adczerodriftpos"
#define APP_DB_F_ADCZDRIFTNEG_STR "adczerodriftneg"
#define APP_DB_F_ADCZDRIFT_MIN ADC_DB_FACT_ZERODRIFT_MINIMUM
#define APP_DB_F_ADCZDRIFT_MAX ADC_DB_FACT_ZERODRIFT_MAXIMUM
#define APP_DB_F_ADCZDRIFT_DEFAULT ADC_DB_FACT_ZERODRIFT_DEFAULT

#define APP_DB_F_ADCSCAL_STR "adcspancal"
#define APP_DB_F_ADCSCAL_MIN ADC_DB_FACT_SPANCAL_MINIMUM
#define APP_DB_F_ADCSCAL_MAX ADC_DB_FACT_SPANCAL_MAXIMUM
#define APP_DB_F_ADCSCAL_DEFAULT0 ADC_DB_FACT_SPANCAL_DEFAULT0
#define APP_DB_F_ADCSCAL_DEFAULT1 ADC_DB_FACT_SPANCAL_DEFAULT1
#define APP_DB_F_ADCSCAL_DEFAULT2 ADC_DB_FACT_SPANCAL_DEFAULT2
#define APP_DB_F_ADCSCAL_DEFAULT3 ADC_DB_FACT_SPANCAL_DEFAULT3
#define APP_DB_F_ADCSCAL_DEFAULT4 ADC_DB_FACT_SPANCAL_DEFAULT4
#define APP_DB_F_ADCSCAL_DEFAULT5 ADC_DB_FACT_SPANCAL_DEFAULT5
#define APP_DB_F_ADCSCAL_DEFAULT6 ADC_DB_FACT_SPANCAL_DEFAULT6
#define APP_DB_F_ADCSCAL_DEFAULT7 ADC_DB_FACT_SPANCAL_DEFAULT7
#define APP_DB_F_ADCSCAL_DEFAULT8 ADC_DB_FACT_SPANCAL_DEFAULT8

#define APP_DB_F_ADCSDRIFTPOS_STR "adcspandriftpos"
#define APP_DB_F_ADCSDRIFTNEG_STR "adcspandriftneg"
#define APP_DB_F_ADCSDRIFT_MIN ADC_DB_FACT_SPANDRIFT_MINIMUM
#define APP_DB_F_ADCSDRIFT_MAX ADC_DB_FACT_SPANDRIFT_MAXIMUM
#define APP_DB_F_ADCSDRIFT_DEFAULT ADC_DB_FACT_SPANDRIFT_DEFAULT

#define APP_DB_F_ADCCALTEMP_STR "adccaltemp"
#define APP_DB_F_ADCCALTEMP_MIN ADC_DB_FACT_CALTEMP_MINIMUM
#define APP_DB_F_ADCCALTEMP_MAX ADC_DB_FACT_CALTEMP_MAXIMUM
#define APP_DB_F_ADCCALTEMP_DEFAULT ADC_DB_FACT_CALTEMP_DEFAULT

#define APP_DB_F_ADCCALFLAG_STR "adcadccalflag"
#define APP_DB_F_ADCCALFLAG_MIN ADC_DB_FACT_CALFLAG_MINIMUM
#define APP_DB_F_ADCCALFLAG_MAX ADC_DB_FACT_CALFLAG_MAXIMUM
#define APP_DB_F_ADCCALFLAG_DEFAULT ADC_DB_FACT_CALFLAG_DEFAULT

#define APP_DB_F_ADCCHKSUM_STR "adcchksum"
#define APP_DB_F_ADCCHKSUM_MIN ADC_DB_FACT_CHKSUM_MINIMUM
#define APP_DB_F_ADCCHKSUM_MAX ADC_DB_FACT_CHKSUM_MAXIMUM
#define APP_DB_F_ADCCHKSUM_DEFAULT ADC_DB_FACT_CHKSUM_DEFAULT


/**
 * Write the factory database to file
 *
 * @param write_ass_copy specifies whether the adcSuperStructure copy of the adcFactoryDatabase should be written
 * @param lock specifies whether adcSuperStructure should be locked before writing
 */
bool app_write_factory_database(bool write_ass_copy, bool lock);


/** Read the factory database from file */
bool app_read_factory_database(void);


/** Default the factory database */
bool app_default_factory_database(void);


/** Initialise the factory database */
bool app_init_factory_database(const char *factorydb_filename);


/** Destroy the factory database */
bool app_destroy_factory_database(void);


#endif /* APP_DB_FACTORY_H_ */
