/*
 * app_db_runtime.c
 *
 *  Created on: Apr 18, 2012
 *      Author: ian
 */
#include <string.h>
#include <stdio.h>
#include <pthread.h>
#include <rindb/db_element.h>
#include <rindb/db_database.h>
#include <rindb/db_database_xml.h>
#include <rinweigh/adcinterface.h>
#include "adcthread.h"
#include "app_db.h"
#include "app_db_changelog.h"

#include "app_db_runtime.h"


/** User runtime database */
struct s_db_app_runtime db_app_runtime;


struct s_db_element app_runtime_elements[] =
{
  /* ADC Runtime Database */
  DB_DECLARE_INT32_ELEM(                     APP_DB_R_ADCCNTZERO_STR,       NULL,          &db_app_runtime.adc_runtime_database.cntZero,                              APP_DB_R_ADCCNTZERO_MIN,     APP_DB_R_ADCCNTZERO_MAX,      APP_DB_R_ADCCNTZERO_DEFAULT,    DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_RUNTIME),
  DB_DECLARE_INT32_ELEM(                     APP_DB_R_ADCCNTTARE_STR,       NULL,          &db_app_runtime.adc_runtime_database.cntTare,                              APP_DB_R_ADCCNTTARE_MIN,     APP_DB_R_ADCCNTTARE_MAX,      APP_DB_R_ADCCNTTARE_DEFAULT,    DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_RUNTIME),
  DB_DECLARE_INT32_ELEM(                     APP_DB_R_ADCCNTPTARE_STR,      NULL,          &db_app_runtime.adc_runtime_database.cntPTare,                             APP_DB_R_ADCCNTPTARE_MIN,    APP_DB_R_ADCCNTPTARE_MAX,     APP_DB_R_ADCCNTPTARE_DEFAULT,   DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_RUNTIME),
  DB_DECLARE_UINT8_ELEM(                     APP_DB_R_ADCHIRES_STR,         NULL,          &db_app_runtime.adc_runtime_database.hiRes,                                APP_DB_R_ADCHIRES_MIN,       APP_DB_R_ADCHIRES_MAX,        APP_DB_R_ADCHIRES_DEFAULT,      DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_RUNTIME),
  DB_DECLARE_UINT8_ELEM(                     APP_DB_R_ADCNETMODE_STR,       NULL,          &db_app_runtime.adc_runtime_database.netMode,                              APP_DB_R_ADCNETMODE_MIN,     APP_DB_R_ADCNETMODE_MAX,      APP_DB_R_ADCNETMODE_DEFAULT,    DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_RUNTIME),
  DB_DECLARE_UINT32_ELEM(                    APP_DB_R_ADCPRINTID_STR,       NULL,          &db_app_runtime.adc_runtime_database.printId,                              APP_DB_R_ADCPRINTID_MIN,     APP_DB_R_ADCPRINTID_MAX,      APP_DB_R_ADCPRINTID_DEFAULT,    DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_RUNTIME),
  DB_DECLARE_UINT8_ELEM(                     APP_DB_R_ADCDISPMODE_STR,      NULL,          &db_app_runtime.adc_runtime_database.dispMode,                             APP_DB_R_ADCDISPMODE_MIN,    APP_DB_R_ADCDISPMODE_MAX,     APP_DB_R_ADCDISPMODE_DEFAULT,   DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_RUNTIME),
  DB_DECLARE_UINT8_ELEM(                     APP_DB_R_ADCTEMPCALON_STR,     NULL,          &db_app_runtime.adc_runtime_database.tempCalOn,                            APP_DB_R_ADCTEMPCALON_MIN,   APP_DB_R_ADCTEMPCALON_MAX,    APP_DB_R_ADCTEMPCALON_DEFAULT,  DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_RUNTIME),
  DB_DECLARE_UINT32_ELEM(                    APP_DB_R_ADCOVERLOADCNT_STR,   NULL,          &db_app_runtime.adc_runtime_database.overloadCnt,                          APP_DB_R_ADCOVERLOADCNT_MIN, APP_DB_R_ADCOVERLOADCNT_MAX,  APP_DB_R_ADCOVERLOADCNT_DEFAULT,DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_RUNTIME),
  DB_DECLARE_UINT32_ELEM(                    APP_DB_R_ADCCHKSUM_STR,        NULL,          &db_app_runtime.adc_runtime_database.chksum,                               APP_DB_R_ADCCHKSUM_MIN,      APP_DB_R_ADCCHKSUM_MAX,       APP_DB_R_ADCCHKSUM_DEFAULT,     APP_DB_R_VERSION2,     DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_RUNTIME),
};


static struct s_db_database app_runtime_database =
{
  .db_database_version = APP_DB_R_VERSION2,
  .db_database_chksum_seed = APP_DB_R_CHECKSUM_SEED,
  .db_elements = app_runtime_elements,
  .db_num_elements = DB_NUM_ELEMENTS(app_runtime_elements),
  .db_name = APP_DB_R_NAME
};


/* Used for database writing */
static struct s_db_database_interp app_runtime_ddi_xml;


/**
 * Write the runtime database to NVRAM
 *
 * @param write_ass_copy specifies whether the adcSuperStructure copy of the adcRuntimeDatabase should be written
 * @param lock specifies whether adcSuperStructure should be locked before writing
 */
bool app_write_runtime_database(bool write_ass_copy, bool lock)
{
  enum eDBError res;

  if (write_ass_copy)
  {
    if (lock)
      AdcMutexLock(&adcSuperStructure);
    db_app_runtime.adc_runtime_database = adcSuperStructure.adcRuntimeDatabase;
    if (lock)
      AdcMutexUnlock(&adcSuperStructure);
  }

  /* Write database to binary buffer */
  res = db_database_write(&app_runtime_database, &app_runtime_ddi_xml, app_runtime_database.db_database_version);
  if (dbeSuccess != res)
  {
    printf("app_write_runtime_database: db_database_write() returned %d\n", res);
  }

  (void) app_changelog_update(&app_runtime_database);

  return (dbeSuccess == res);
}


/** Read the runtime database from NVRAM */
bool app_read_runtime_database(void)
{
  enum eDBError res = dbeFailure;
  uint32_t version;

  /* Read database settings from file */
  res = db_database_read(&app_runtime_database, &app_runtime_ddi_xml, &version, false);
  if (dbeSuccess != res)
  {
    printf("app_read_runtime_database: db_database_read() returned %d\n", res);
  }

  db_database_update_previous(&app_runtime_database);

  return (dbeSuccess == res);
}


/** Default the runtime database */
bool app_default_runtime_database(void)
{
  enum eDBError res;

  res = db_database_default(&app_runtime_database);
  if (dbeSuccess != res)
  {
    printf("app_default_runtime_database: db_database_default() returned %d\n", res);
  }

  /* Use the ADC database defaulter for ADC values */
  AdcDefaultDatabases(&adcSuperStructure, true, NULL, NULL, &db_app_runtime.adc_runtime_database, false, false);

  return (dbeSuccess == res);
}


/** Initialise the runtime database */
bool app_init_runtime_database(const char *runtimedb_filename)
{
  enum eDBError res;

  /* Initialise the database (creates previous data for some elements) */
  res = db_database_init(&app_runtime_database);
  if (dbeSuccess == res)
  {
    res = db_create_database_interp_xml(&app_runtime_ddi_xml, runtimedb_filename);
    if (dbeSuccess != res)
    {
      printf("app_init_runtime_database: db_create_database_interp_xml() returned %d\n", res);
    }
  }
  else
  {
    printf("app_init_runtime_database: db_database_init() returned %d\n", res);
  }
  return (dbeSuccess == res);
}


/** Destroy the runtime database */
bool app_destroy_runtime_database(void)
{
  (void) db_database_destroy(&app_runtime_database);
  (void) db_destroy_database_interp_xml(&app_runtime_ddi_xml);
  return true;
}
