/*
 * app_db_runtime.h
 *
 *  Created on: Apr 18, 2012
 *      Author: ian
 */

#ifndef APP_DB_RUNTIME_H_
#define APP_DB_RUNTIME_H_

#include <stddef.h>
#include <stdint.h>
#include <limits.h>
#include <rinweigh/adcdatatypes.h>


/** Application runtime database definition */
struct s_db_app_runtime
{
  /* ADC runtime database */
  AdcRuntimeDatabase adc_runtime_database;
};


/** User runtime database */
extern struct s_db_app_runtime db_app_runtime;


#define APP_DB_R_NAME "runtimedb"
#define APP_DB_R_CHECKSUM_SEED 0x12345678


/** Initial runtime database version */
#define APP_DB_R_VERSION1 1


/** Runtime database version which added checksum AdcRuntimeDatabase.chksum */
#define APP_DB_R_VERSION2 2


/*
 * ADC database settings
 */


#define APP_DB_R_ADCCNTZERO_STR "adccntzero"
#define APP_DB_R_ADCCNTZERO_MIN (ADC_DB_RT_CNTZERO_MINIMUM)
#define APP_DB_R_ADCCNTZERO_MAX (ADC_DB_RT_CNTZERO_MAXIMUM)
#define APP_DB_R_ADCCNTZERO_DEFAULT (ADC_DB_RT_CNTZERO_DEFAULT)

#define APP_DB_R_ADCCNTTARE_STR "adccnttare"
#define APP_DB_R_ADCCNTTARE_MIN (ADC_DB_RT_CNTTARE_MINIMUM)
#define APP_DB_R_ADCCNTTARE_MAX (ADC_DB_RT_CNTTARE_MAXIMUM)
#define APP_DB_R_ADCCNTTARE_DEFAULT (ADC_DB_RT_CNTTARE_DEFAULT)

#define APP_DB_R_ADCCNTPTARE_STR "adccntptare"
#define APP_DB_R_ADCCNTPTARE_MIN (ADC_DB_RT_CNTPTARE_MINIMUM)
#define APP_DB_R_ADCCNTPTARE_MAX (ADC_DB_RT_CNTPTARE_MAXIMUM)
#define APP_DB_R_ADCCNTPTARE_DEFAULT (ADC_DB_RT_CNTPTARE_DEFAULT)

#define APP_DB_R_ADCHIRES_STR "adchires"
#define APP_DB_R_ADCHIRES_MIN (ADC_DB_RT_HIRES_MINIMUM)
#define APP_DB_R_ADCHIRES_MAX (ADC_DB_RT_HIRES_MAXIMUM)
#define APP_DB_R_ADCHIRES_DEFAULT (ADC_DB_RT_HIRES_DEFAULT)

#define APP_DB_R_ADCNETMODE_STR "adcnetmode"
#define APP_DB_R_ADCNETMODE_MIN (ADC_DB_RT_NETMODE_MINIMUM)
#define APP_DB_R_ADCNETMODE_MAX (ADC_DB_RT_NETMODE_MAXIMUM)
#define APP_DB_R_ADCNETMODE_DEFAULT (ADC_DB_RT_NETMODE_DEFAULT)

#define APP_DB_R_ADCPRINTID_STR "adcprintid"
#define APP_DB_R_ADCPRINTID_MIN (ADC_DB_RT_PRINTID_MINIMUM)
#define APP_DB_R_ADCPRINTID_MAX (ADC_DB_RT_PRINTID_MAXIMUM)
#define APP_DB_R_ADCPRINTID_DEFAULT (ADC_DB_RT_PRINTID_DEFAULT)

#define APP_DB_R_ADCDISPMODE_STR "adcdispmode"
#define APP_DB_R_ADCDISPMODE_MIN (ADC_DB_RT_DISPMODE_MINIMUM)
#define APP_DB_R_ADCDISPMODE_MAX (ADC_DB_RT_DISPMODE_MAXIMUM)
#define APP_DB_R_ADCDISPMODE_DEFAULT (ADC_DB_RT_DISPMODE_DEFAULT)

#define APP_DB_R_ADCTEMPCALON_STR "adctempcalon"
#define APP_DB_R_ADCTEMPCALON_MIN (ADC_DB_RT_TEMPCALON_MINIMUM)
#define APP_DB_R_ADCTEMPCALON_MAX (ADC_DB_RT_TEMPCALON_MAXIMUM)
#define APP_DB_R_ADCTEMPCALON_DEFAULT (ADC_DB_RT_TEMPCALON_DEFAULT)

#define APP_DB_R_ADCOVERLOADCNT_STR "adcoverloadcnt"
#define APP_DB_R_ADCOVERLOADCNT_MIN (ADC_DB_RT_OVERLOADCNT_MINIMUM)
#define APP_DB_R_ADCOVERLOADCNT_MAX (ADC_DB_RT_OVERLOADCNT_MAXIMUM)
#define APP_DB_R_ADCOVERLOADCNT_DEFAULT (ADC_DB_RT_OVERLOADCNT_DEFAULT)

#define APP_DB_R_ADCCHKSUM_STR "adcchksum"
#define APP_DB_R_ADCCHKSUM_MIN ADC_DB_RT_CHKSUM_MINIMUM
#define APP_DB_R_ADCCHKSUM_MAX ADC_DB_RT_CHKSUM_MAXIMUM
#define APP_DB_R_ADCCHKSUM_DEFAULT ADC_DB_RT_CHKSUM_DEFAULT


/**
 * Write the runtime database to NVRAM
 *
 * @param write_ass_copy specifies whether the adcSuperStructure copy of the adcRuntimeDatabase should be written
 * @param lock specifies whether adcSuperStructure should be locked before writing
 */
bool app_write_runtime_database(bool write_ass_copy, bool lock);


/** Read the runtime database from NVRAM */
bool app_read_runtime_database(void);


/** Default the runtime database */
bool app_default_runtime_database(void);


/** Initialise the runtime database */
bool app_init_runtime_database(const char *runtimedb_filename);


/** Destroy the runtime database */
bool app_destroy_runtime_database(void);


#endif /* APP_DB_RUNTIME_H_ */
