/*
 * app_db_user.c
 *
 *  Created on: Apr 13, 2012
 *      Author: ian
 */
#include <string.h>
#include <stdio.h>
#include <rindb/db_element.h>
#include <rindb/db_database.h>
#include <rindb/db_database_xml.h>
#include <rinweigh/adcinterface.h>
#include "adcthread.h"
#include "app_db.h"
#include "app_db_changelog.h"

#include "app_db_user.h"


static const uint8_t adc_linpts_minimums[APP_DB_U_ADCLINPTS_NUM] =
{
  APP_DB_U_ADCLINPTS_MIN, APP_DB_U_ADCLINPTS_MIN, APP_DB_U_ADCLINPTS_MIN, APP_DB_U_ADCLINPTS_MIN, APP_DB_U_ADCLINPTS_MIN,
  APP_DB_U_ADCLINPTS_MIN, APP_DB_U_ADCLINPTS_MIN, APP_DB_U_ADCLINPTS_MIN, APP_DB_U_ADCLINPTS_MIN, APP_DB_U_ADCLINPTS_MIN,
  APP_DB_U_ADCLINPTS_MIN
};
static const uint8_t adc_linpts_maximums[APP_DB_U_ADCLINPTS_NUM] =
{
  APP_DB_U_ADCLINPTS_MAX, APP_DB_U_ADCLINPTS_MAX, APP_DB_U_ADCLINPTS_MAX, APP_DB_U_ADCLINPTS_MAX, APP_DB_U_ADCLINPTS_MAX,
  APP_DB_U_ADCLINPTS_MAX, APP_DB_U_ADCLINPTS_MAX, APP_DB_U_ADCLINPTS_MAX, APP_DB_U_ADCLINPTS_MAX, APP_DB_U_ADCLINPTS_MAX,
  APP_DB_U_ADCLINPTS_MAX
};
static const uint8_t adc_linpts_defaults[APP_DB_U_ADCLINPTS_NUM] =
{
  APP_DB_U_ADCLINPTS_DEFAULT, APP_DB_U_ADCLINPTS_DEFAULT, APP_DB_U_ADCLINPTS_DEFAULT, APP_DB_U_ADCLINPTS_DEFAULT, APP_DB_U_ADCLINPTS_DEFAULT,
  APP_DB_U_ADCLINPTS_DEFAULT, APP_DB_U_ADCLINPTS_DEFAULT, APP_DB_U_ADCLINPTS_DEFAULT, APP_DB_U_ADCLINPTS_DEFAULT, APP_DB_U_ADCLINPTS_DEFAULT,
  APP_DB_U_ADCLINPTSLAST_DEFAULT
};

static const int32_t adc_fs_minimums[APP_DB_U_ADCFS_NUM] = { APP_DB_U_ADCFS0_MIN, APP_DB_U_ADCFS1_MIN, APP_DB_U_ADCFS2_MIN };
static const int32_t adc_fs_maximums[APP_DB_U_ADCFS_NUM] = { APP_DB_U_ADCFS0_MAX, APP_DB_U_ADCFS1_MAX, APP_DB_U_ADCFS2_MAX };
static const int32_t adc_fs_defaults[APP_DB_U_ADCFS_NUM] = { APP_DB_U_ADCFS0_DEFAULT, APP_DB_U_ADCFS0_DEFAULT, APP_DB_U_ADCFS0_DEFAULT };
static const uint8_t adc_res_minimums[APP_DB_U_ADCRES_NUM] = { APP_DB_U_ADCRES0_MIN, APP_DB_U_ADCRES1_MIN, APP_DB_U_ADCRES2_MIN };
static const uint8_t adc_res_maximums[APP_DB_U_ADCRES_NUM] = { APP_DB_U_ADCRES0_MAX, APP_DB_U_ADCRES1_MAX, APP_DB_U_ADCRES2_MAX };
static const uint8_t adc_res_defaults[APP_DB_U_ADCRES_NUM] = { APP_DB_U_ADCRES0_DEFAULT, APP_DB_U_ADCRES1_DEFAULT, APP_DB_U_ADCRES2_DEFAULT };


/** User application database */
struct s_db_app_user db_app_user;


struct s_db_element app_user_elements[] =
{
  /* Element type                                                Element database name            Element menu name   Element data                                     Number of elements        Element minimum              Element maximum               Element default                 First appeared in      Last appeared in       Change log mask      Change log lookup */
  /* ADC User Database */
  DB_DECLARE_INT32_ELEM(                                         APP_DB_U_ADCZERORAW_STR,         NULL,               &db_app_user.adc_user_database.zeroRaw,                                    APP_DB_U_ADCZERORAW_MIN,     APP_DB_U_ADCZERORAW_MAX,      APP_DB_U_ADCZERORAW_DEFAULT,    DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_INT32_ELEM(                                         APP_DB_U_ADCSPANRAW_STR,         NULL,               &db_app_user.adc_user_database.spanRaw,                                    APP_DB_U_ADCSPANRAW_MIN,     APP_DB_U_ADCSPANRAW_MAX,      APP_DB_U_ADCSPANRAW_DEFAULT,    DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_INT32_ELEM(                                         APP_DB_U_ADCSPANCNT_STR,         NULL,               &db_app_user.adc_user_database.spanCnt,                                    APP_DB_U_ADCSPANCNT_MIN,     APP_DB_U_ADCSPANCNT_MAX,      APP_DB_U_ADCSPANCNT_DEFAULT,    DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_UINT8_ARRAY_ELEM_INDIVIDUAL(                        APP_DB_U_ADCLINPTS_STR,          NULL,               db_app_user.adc_user_database.linPts,            APP_DB_U_ADCLINPTS_NUM,   adc_linpts_minimums,         adc_linpts_maximums,          adc_linpts_defaults,            DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_INT32_ARRAY_ELEM_SINGLE(                            APP_DB_U_ADCLINCOR_STR,          NULL,               db_app_user.adc_user_database.linCor,            APP_DB_U_ADCLINCOR_NUM,   APP_DB_U_ADCLINCOR_MIN,      APP_DB_U_ADCLINCOR_MAX,       APP_DB_U_ADCLINCOR_DEFAULT,     DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_UINT8_ELEM(                                         APP_DB_U_ADCSCALETYPE_STR,       NULL,               &db_app_user.adc_user_database.scaleType,                                  APP_DB_U_ADCSCALETYPE_MIN,   APP_DB_U_ADCSCALETYPE_MAX,    APP_DB_U_ADCSCALETYPE_DEFAULT,  DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_UINT8_ELEM(                                         APP_DB_U_ADCSYNCFREQ_STR,        NULL,               &db_app_user.adc_user_database.syncFreqIdx,                                APP_DB_U_ADCSYNCFREQ_MIN,    APP_DB_U_ADCSYNCFREQ_MAX,     APP_DB_U_ADCSYNCFREQ_DEFAULT,   DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_UINT8_ELEM(                                         APP_DB_U_ADCJITTER_STR,          NULL,               &db_app_user.adc_user_database.jitter,                                     APP_DB_U_ADCJITTER_MIN,      APP_DB_U_ADCJITTER_MAX,       APP_DB_U_ADCJITTER_DEFAULT,     DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_UINT8_ELEM(                                         APP_DB_U_ADCDPPOS_STR,           NULL,               &db_app_user.adc_user_database.dpPos,                                      APP_DB_U_ADCDPPOS_MIN,       APP_DB_U_ADCDPPOS_MAX,        APP_DB_U_ADCDPPOS_DEFAULT,      DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_INT32_ARRAY_ELEM_INDIVIDUAL(                        APP_DB_U_ADCFS_STR,              NULL,               db_app_user.adc_user_database.fullScale,         APP_DB_U_ADCFS_NUM,       adc_fs_minimums,             adc_fs_maximums,              adc_fs_defaults,                DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_UINT8_ARRAY_ELEM_INDIVIDUAL(                        APP_DB_U_ADCRES_STR,             NULL,               db_app_user.adc_user_database.resolution,        APP_DB_U_ADCRES_NUM,      adc_res_minimums,            adc_res_maximums,             adc_res_defaults,               DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_UINT8_ELEM(                                         APP_DB_U_ADCUNITS_STR,           NULL,               &db_app_user.adc_user_database.units,                                      APP_DB_U_ADCUNITS_MIN,       APP_DB_U_ADCUNITS_MAX,        APP_DB_U_ADCUNITS_DEFAULT,      DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_INT32_ELEM(                                         APP_DB_U_ADCCALWEIGHT_STR,       NULL,               &db_app_user.adc_user_database.calWeight,                                  APP_DB_U_ADCCALWEIGHT_MIN,   APP_DB_U_ADCCALWEIGHT_MAX,    APP_DB_U_ADCCALWEIGHT_DEFAULT,  DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_INT32_ELEM(                                         APP_DB_U_ADCADDTARE_STR,         NULL,               &db_app_user.adc_user_database.addTare,                                    APP_DB_U_ADCADDTARE_MIN,     APP_DB_U_ADCADDTARE_MAX,      APP_DB_U_ADCADDTARE_DEFAULT,    DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_INT32_ELEM(                                         APP_DB_U_ADCZEROBAND_STR,        NULL,               &db_app_user.adc_user_database.zeroBand,                                   APP_DB_U_ADCZEROBAND_MIN,    APP_DB_U_ADCZEROBAND_MAX,     APP_DB_U_ADCZEROBAND_DEFAULT,   DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_UINT8_ELEM(                                         APP_DB_U_ADCMOTION_STR,          NULL,               &db_app_user.adc_user_database.motion,                                     APP_DB_U_ADCMOTION_MIN,      APP_DB_U_ADCMOTION_MAX,       APP_DB_U_ADCMOTION_DEFAULT,     DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_UINT8_ELEM(                                         APP_DB_U_ADCZEROTRACK_STR,       NULL,               &db_app_user.adc_user_database.zeroTracking,                               APP_DB_U_ADCZEROTRACK_MIN,   APP_DB_U_ADCZEROTRACK_MAX,    APP_DB_U_ADCZEROTRACK_DEFAULT,  DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_UINT32_ELEM(                                        APP_DB_U_ADCAVERAGE_STR,         NULL,               &db_app_user.adc_user_database.average,                                    APP_DB_U_ADCAVERAGE_MIN,     APP_DB_U_ADCAVERAGE_MAX,      APP_DB_U_ADCAVERAGE_DEFAULT,    DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_UINT8_ELEM(                                         APP_DB_U_ADCRANGEOFZERO_STR,     NULL,               &db_app_user.adc_user_database.rangeOfZero,                                APP_DB_U_ADCRANGEOFZERO_MIN, APP_DB_U_ADCRANGEOFZERO_MAX,  APP_DB_U_ADCRANGEOFZERO_DEFAULT,DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_UINT8_ELEM(                                         APP_DB_U_ADCAPPROVMODE_STR,      NULL,               &db_app_user.adc_user_database.approvalMode,                               APP_DB_U_ADCAPPROVMODE_MIN,  APP_DB_U_ADCAPPROVMODE_MAX,   APP_DB_U_ADCAPPROVMODE_DEFAULT, DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_UINT8_ELEM(                                         APP_DB_U_ADCZEROONSTART_STR,     NULL,               &db_app_user.adc_user_database.zeroOnStart,                                APP_DB_U_ADCZEROONSTART_MIN, APP_DB_U_ADCZEROONSTART_MAX,  APP_DB_U_ADCZEROONSTART_DEFAULT,DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_UINT8_ELEM(                                         APP_DB_U_ADC4WIRE_STR,           NULL,               &db_app_user.adc_user_database.wire4Mode,                                  APP_DB_U_ADC4WIRE_MIN,       APP_DB_U_ADC4WIRE_MAX,        APP_DB_U_ADC4WIRE_DEFAULT,      DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_UINT8_ELEM(                                         APP_DB_U_ADCTEMPCALZERO_STR,     NULL,               &db_app_user.adc_user_database.tempCalZero,                                APP_DB_U_ADCTEMPCALZERO_MIN, APP_DB_U_ADCTEMPCALZERO_MAX,  APP_DB_U_ADCTEMPCALZERO_DEFAULT,DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_UINT8_ELEM(                                         APP_DB_U_ADCHIRES_STR,           NULL,               &db_app_user.adc_user_database.hiRes,                                      APP_DB_U_ADCHIRES_MIN,       APP_DB_U_ADCHIRES_MAX,        APP_DB_U_ADCHIRES_DEFAULT,      DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_UINT8_ELEM(                                         APP_DB_U_ADCQAENABLE_STR,        NULL,               &db_app_user.adc_user_database.qaEnable,                                   APP_DB_U_ADCQAENABLE_MIN,    APP_DB_U_ADCQAENABLE_MAX,     APP_DB_U_ADCQAENABLE_DEFAULT,   DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_UINT8_ELEM(                                         APP_DB_U_ADCQADAY_STR,           NULL,               &db_app_user.adc_user_database.qaDay,                                      APP_DB_U_ADCQADAY_MIN,       APP_DB_U_ADCQADAY_MAX,        APP_DB_U_ADCQADAY_DEFAULT,      DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_UINT8_ELEM(                                         APP_DB_U_ADCQAMONTH_STR,         NULL,               &db_app_user.adc_user_database.qaMonth,                                    APP_DB_U_ADCQAMONTH_MIN,     APP_DB_U_ADCQAMONTH_MAX,      APP_DB_U_ADCQAMONTH_DEFAULT,    DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_UINT32_ELEM(                                        APP_DB_U_ADCQAYEAR_STR,          NULL,               &db_app_user.adc_user_database.qaYear,                                     APP_DB_U_ADCQAYEAR_MIN,      APP_DB_U_ADCQAYEAR_MAX,       APP_DB_U_ADCQAYEAR_DEFAULT,     DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_UINT32_ELEM(                                        APP_DB_U_ADCCCNTEPCAL_STR,       NULL,               &db_app_user.adc_user_database.calCntNTEPCalibration,                      APP_DB_U_ADCCCNTEPCAL_MIN,   APP_DB_U_ADCCCNTEPCAL_MAX,    APP_DB_U_ADCCCNTEPCAL_DEFAULT,  DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_UINT32_ELEM(                                        APP_DB_U_ADCCCNTEPSET_STR,       NULL,               &db_app_user.adc_user_database.calCntNTEPSettings,                         APP_DB_U_ADCCCNTEPSET_MIN,   APP_DB_U_ADCCCNTEPSET_MAX,    APP_DB_U_ADCCCNTEPSET_DEFAULT,  DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_UINT32_ELEM(                                        APP_DB_U_ADCCCOIML_STR,          NULL,               &db_app_user.adc_user_database.calCntOIML,                                 APP_DB_U_ADCCCOIML_MIN,      APP_DB_U_ADCCCOIML_MAX,       APP_DB_U_ADCCCOIML_DEFAULT,     DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_UINT8_ELEM(                                         APP_DB_U_ADCCALFLAG_STR,         NULL,               &db_app_user.adc_user_database.calFlag,                                    APP_DB_U_ADCCALFLAG_MIN,     APP_DB_U_ADCCALFLAG_MAX,      APP_DB_U_ADCCALFLAG_DEFAULT,    DB_NO_MINIMUM_VERSION, DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_TRADE),
  DB_DECLARE_UINT32_ELEM(                                        APP_DB_U_ADCCHKSUM_STR,          NULL,               &db_app_user.adc_user_database.chksum,                                     APP_DB_U_ADCCHKSUM_MIN,      APP_DB_U_ADCCHKSUM_MAX,       APP_DB_U_ADCCHKSUM_DEFAULT,     APP_DB_U_VERSION2,     DB_NO_MAXIMUM_VERSION, APP_DB_CLMASK_USER),
};


static struct s_db_database app_user_database =
{
  .db_database_version = APP_DB_U_VERSION2,
  .db_database_chksum_seed = APP_DB_U_CHECKSUM_SEED,
  .db_elements = app_user_elements,
  .db_num_elements = DB_NUM_ELEMENTS(app_user_elements),
  .db_name = APP_DB_U_NAME
};


static struct s_db_database_interp app_user_ddi_xml;


/**
 * Write the user database to file
 *
 * @param write_ass_copy specifies whether the adcSuperStructure copy of the adcUserDatabase should be written
 * @param lock specifies whether adcSuperStructure should be locked before writing
 */
bool app_write_user_database(bool write_ass_copy, bool lock)
{
  enum eDBError res;

  if (write_ass_copy)
  {
    /* Write the adcSuperStructure copy */
    if (lock)
      AdcMutexLock(&adcSuperStructure);
    db_app_user.adc_user_database = adcSuperStructure.adcUserDatabase;
    if (lock)
      AdcMutexUnlock(&adcSuperStructure);
  }

  /* Write user database to file */
  res = db_database_write(&app_user_database, &app_user_ddi_xml, app_user_database.db_database_version);
  if (dbeSuccess != res)
  {
    printf("app_write_user_database: db_database_write() returned %d\n", res);
  }

  /* Update the change log */
  (void) app_changelog_update(&app_user_database);

  return (dbeSuccess == res);
}


/** Read the user database from file */
bool app_read_user_database(void)
{
  enum eDBError res;
  uint32_t version;

  /* Read the database from file */
  res = db_database_read(&app_user_database, &app_user_ddi_xml, &version, false);
  if (dbeSuccess != res)
  {
    printf("app_read_user_database: db_database_read() returned %d\n", res);
  }

  /* Update previous data, so change log will work */
  db_database_update_previous(&app_user_database);

  return (dbeSuccess == res);
}


/** Default the user database */
bool app_default_user_database(void)
{
  enum eDBError res;

  res = db_database_default(&app_user_database);
  if (dbeSuccess != res)
  {
    printf("app_default_user_database: db_database_default() returned %d\n", res);
  }

  /* Use the ADC database defaulter for ADC values */
  AdcDefaultDatabases(&adcSuperStructure, true, NULL, &db_app_user.adc_user_database, NULL, true, false);

  return (dbeSuccess == res);
}


/** Initialise the user database */
bool app_init_user_database(const char *userdb_filename)
{
  enum eDBError res;

  /* Initialise the database (creates previous data for some elements) */
  res = db_database_init(&app_user_database);
  if (dbeSuccess == res)
  {
    res = db_create_database_interp_xml(&app_user_ddi_xml, userdb_filename);
    if (dbeSuccess != res)
    {
      printf("app_init_user_database: db_create_database_interp_xml() returned %d\n", res);
    }
  }
  else
  {
    printf("app_init_user_database: db_database_init() returned %d\n", res);
  }
  return (dbeSuccess == res);
}


/** Destroy the user database */
bool app_destroy_user_database(void)
{
  (void) db_database_destroy(&app_user_database);
  (void) db_destroy_database_interp_xml(&app_user_ddi_xml);
  return true;
}
