/*
 * app_db_user.h
 *
 *  Created on: Apr 13, 2012
 *      Author: ian
 */

#ifndef APP_DB_USER_H_
#define APP_DB_USER_H_

#include <stddef.h>
#include <stdint.h>
#include <limits.h>
#include <rinweigh/adcdatatypes.h>


/** Application user database definition */
struct s_db_app_user
{
  /* ADC User database */
  AdcUserDatabase adc_user_database;                    /*!< All settings for ADC user calibration */
};


#define APP_DB_U_NAME "userdb"
#define APP_DB_U_CHECKSUM_SEED 0x12345678


/** Initial user database version */
#define APP_DB_U_VERSION1 1


/** User database version which added checksum AdcUserDatabase.chksum */
#define APP_DB_U_VERSION2 2


/*
 * ADC database settings
 */


#define APP_DB_U_ADCZERORAW_STR "adczeroraw"
#define APP_DB_U_ADCZERORAW_MIN (ADC_DB_USER_ZERORAW_MINIMUM)
#define APP_DB_U_ADCZERORAW_MAX (ADC_DB_USER_ZERORAW_MAXIMUM)
#define APP_DB_U_ADCZERORAW_DEFAULT (ADC_DB_USER_ZERORAW_DEFAULT)

#define APP_DB_U_ADCSPANRAW_STR "adcspanraw"
#define APP_DB_U_ADCSPANRAW_MIN (ADC_DB_USER_SPANRAW_MINIMUM)
#define APP_DB_U_ADCSPANRAW_MAX (ADC_DB_USER_SPANRAW_MAXIMUM)
#define APP_DB_U_ADCSPANRAW_DEFAULT (ADC_DB_USER_SPANRAW_DEFAULT)

#define APP_DB_U_ADCSPANCNT_STR "adcspancnt"
#define APP_DB_U_ADCSPANCNT_MIN (ADC_DB_USER_SPANCNT_MINIMUM)
#define APP_DB_U_ADCSPANCNT_MAX (ADC_DB_USER_SPANCNT_MAXIMUM)
#define APP_DB_U_ADCSPANCNT_DEFAULT (ADC_DB_USER_SPANCNT_DEFAULT)

#define APP_DB_U_ADCLINPTS_STR "adclinpoints"
#define APP_DB_U_ADCLINPTS_NUM (ADC_MAX_NUM_LIN_PTS + 1)
#define APP_DB_U_ADCLINPTS_MIN (ADC_DB_USER_LINPTS_MINIMUM)
#define APP_DB_U_ADCLINPTS_MAX (ADC_DB_USER_LINPTS_MAXIMUM)
#define APP_DB_U_ADCLINPTS_DEFAULT (ADC_DB_USER_LINPTS_DEFAULT)
#define APP_DB_U_ADCLINPTSLAST_DEFAULT (ADC_DB_USER_LINPTSLAST_DEFAULT)

#define APP_DB_U_ADCLINCOR_STR "adclincorrection"
#define APP_DB_U_ADCLINCOR_NUM (ADC_MAX_NUM_LIN_PTS + 1)
#define APP_DB_U_ADCLINCOR_MIN (ADC_DB_USER_LINCOR_MINIMUM)
#define APP_DB_U_ADCLINCOR_MAX (ADC_DB_USER_LINCOR_MAXIMUM)
#define APP_DB_U_ADCLINCOR_DEFAULT (ADC_DB_USER_LINCOR_DEFAULT)

#define APP_DB_U_ADCSCALETYPE_STR "adcscaletype"
#define APP_DB_U_ADCSCALETYPE_MIN (ADC_DB_USER_SCALETYPE_MINIMUM)
#define APP_DB_U_ADCSCALETYPE_MAX (ADC_DB_USER_SCALETYPE_MAXIMUM)
#define APP_DB_U_ADCSCALETYPE_DEFAULT (ADC_DB_USER_SCALETYPE_DEFAULT)

#define APP_DB_U_ADCSYNCFREQ_STR "adcsyncfreq"
#define APP_DB_U_ADCSYNCFREQ_MIN (ADC_DB_USER_SYNCFREQIDX_MINIMUM)
#define APP_DB_U_ADCSYNCFREQ_MAX (ADC_DB_USER_SYNCFREQIDX_MAXIMUM)
#define APP_DB_U_ADCSYNCFREQ_DEFAULT (ADC_DB_USER_SYNCFREQIDX_DEFAULT)

#define APP_DB_U_ADCJITTER_STR "adcjitter"
#define APP_DB_U_ADCJITTER_MIN (ADC_DB_USER_JITTER_MINIMUM)
#define APP_DB_U_ADCJITTER_MAX (ADC_DB_USER_JITTER_MAXIMUM)
#define APP_DB_U_ADCJITTER_DEFAULT (ADC_DB_USER_JITTER_DEFAULT)

#define APP_DB_U_ADCDPPOS_STR "adcdppos"
#define APP_DB_U_ADCDPPOS_MIN (ADC_DB_USER_DPPOS_MINIMUM)
#define APP_DB_U_ADCDPPOS_MAX (ADC_DB_USER_DPPOS_MAXIMUM)
#define APP_DB_U_ADCDPPOS_DEFAULT (ADC_DB_USER_DPPOS_DEFAULT)

#define APP_DB_U_ADCFS_STR "adcfullscale"
#define APP_DB_U_ADCFS_NUM (ADC_MAX_NUM_INTERVAL)
#define APP_DB_U_ADCFS0_MIN (ADC_DB_USER_FULLSCALE0_MINIMUM)
#define APP_DB_U_ADCFS0_MAX (ADC_DB_USER_FULLSCALE0_MAXIMUM)
#define APP_DB_U_ADCFS0_DEFAULT (ADC_DB_USER_FULLSCALE0_DEFAULT)
#define APP_DB_U_ADCFS1_MIN (ADC_DB_USER_FULLSCALE1_MINIMUM)
#define APP_DB_U_ADCFS1_MAX (ADC_DB_USER_FULLSCALE1_MAXIMUM)
#define APP_DB_U_ADCFS1_DEFAULT (ADC_DB_USER_FULLSCALE1_DEFAULT)
#define APP_DB_U_ADCFS2_MIN (ADC_DB_USER_FULLSCALE2_MINIMUM)
#define APP_DB_U_ADCFS2_MAX (ADC_DB_USER_FULLSCALE2_MAXIMUM)
#define APP_DB_U_ADCFS2_DEFAULT (ADC_DB_USER_FULLSCALE2_DEFAULT)

#define APP_DB_U_ADCRES_STR "adcresolution"
#define APP_DB_U_ADCRES_NUM (ADC_MAX_NUM_INTERVAL)
#define APP_DB_U_ADCRES0_MIN (ADC_DB_USER_RESOLUTION0_MINIMUM)
#define APP_DB_U_ADCRES0_MAX (ADC_DB_USER_RESOLUTION0_MAXIMUM)
#define APP_DB_U_ADCRES0_DEFAULT (ADC_DB_USER_RESOLUTION0_DEFAULT)
#define APP_DB_U_ADCRES1_MIN (ADC_DB_USER_RESOLUTION1_MINIMUM)
#define APP_DB_U_ADCRES1_MAX (ADC_DB_USER_RESOLUTION1_MAXIMUM)
#define APP_DB_U_ADCRES1_DEFAULT (ADC_DB_USER_RESOLUTION1_DEFAULT)
#define APP_DB_U_ADCRES2_MIN (ADC_DB_USER_RESOLUTION2_MINIMUM)
#define APP_DB_U_ADCRES2_MAX (ADC_DB_USER_RESOLUTION2_MAXIMUM)
#define APP_DB_U_ADCRES2_DEFAULT (ADC_DB_USER_RESOLUTION2_DEFAULT)

#define APP_DB_U_ADCUNITS_STR "adcunits"
#define APP_DB_U_ADCUNITS_MIN (ADC_DB_USER_UNITS_MINIMUM)
#define APP_DB_U_ADCUNITS_MAX (ADC_DB_USER_UNITS_MAXIMUM)
#define APP_DB_U_ADCUNITS_DEFAULT (ADC_DB_USER_UNITS_DEFAULT)

#define APP_DB_U_ADCCALWEIGHT_STR "adccalweight"
#define APP_DB_U_ADCCALWEIGHT_MIN (ADC_DB_USER_CALWEIGHT_MINIMUM)
#define APP_DB_U_ADCCALWEIGHT_MAX (ADC_DB_USER_CALWEIGHT_MAXIMUM)
#define APP_DB_U_ADCCALWEIGHT_DEFAULT (ADC_DB_USER_CALWEIGHT_DEFAULT)

#define APP_DB_U_ADCADDTARE_STR "adcaddtare"
#define APP_DB_U_ADCADDTARE_MIN (ADC_DB_USER_ADDTARE_MINIMUM)
#define APP_DB_U_ADCADDTARE_MAX (ADC_DB_USER_ADDTARE_MAXIMUM)
#define APP_DB_U_ADCADDTARE_DEFAULT (ADC_DB_USER_ADDTARE_DEFAULT)

#define APP_DB_U_ADCZEROBAND_STR "adczeroband"
#define APP_DB_U_ADCZEROBAND_MIN (ADC_DB_USER_ZEROBAND_MINIMUM)
#define APP_DB_U_ADCZEROBAND_MAX (ADC_DB_USER_ZEROBAND_MAXIMUM)
#define APP_DB_U_ADCZEROBAND_DEFAULT (ADC_DB_USER_ZEROBAND_DEFAULT)

#define APP_DB_U_ADCMOTION_STR "adcmotion"
#define APP_DB_U_ADCMOTION_MIN (ADC_DB_USER_MOTION_MINIMUM)
#define APP_DB_U_ADCMOTION_MAX (ADC_DB_USER_MOTION_MAXIMUM)
#define APP_DB_U_ADCMOTION_DEFAULT (ADC_DB_USER_MOTION_DEFAULT)

#define APP_DB_U_ADCZEROTRACK_STR "adczerotracking"
#define APP_DB_U_ADCZEROTRACK_MIN (ADC_DB_USER_ZEROTRACKING_MINIMUM)
#define APP_DB_U_ADCZEROTRACK_MAX (ADC_DB_USER_ZEROTRACKING_MAXIMUM)
#define APP_DB_U_ADCZEROTRACK_DEFAULT (ADC_DB_USER_ZEROTRACKING_DEFAULT)

#define APP_DB_U_ADCAVERAGE_STR "adcaverage"
#define APP_DB_U_ADCAVERAGE_MIN (ADC_DB_USER_AVERAGE_MINIMUM)
#define APP_DB_U_ADCAVERAGE_MAX (ADC_DB_USER_AVERAGE_MAXIMUM)
#define APP_DB_U_ADCAVERAGE_DEFAULT (ADC_DB_USER_AVERAGE_DEFAULT)

#define APP_DB_U_ADCRANGEOFZERO_STR "adcrangeofzero"
#define APP_DB_U_ADCRANGEOFZERO_MIN (ADC_DB_USER_RANGEOFZERO_MINIMUM)
#define APP_DB_U_ADCRANGEOFZERO_MAX (ADC_DB_USER_RANGEOFZERO_MAXIMUM)
#define APP_DB_U_ADCRANGEOFZERO_DEFAULT (ADC_DB_USER_RANGEOFZERO_DEFAULT)

#define APP_DB_U_ADCAPPROVMODE_STR "adcapprovalmode"
#define APP_DB_U_ADCAPPROVMODE_MIN (ADC_DB_USER_APPROVALMODE_MINIMUM)
#define APP_DB_U_ADCAPPROVMODE_MAX (ADC_DB_USER_APPROVALMODE_MAXIMUM)
#define APP_DB_U_ADCAPPROVMODE_DEFAULT (ADC_DB_USER_APPROVALMODE_DEFAULT)

#define APP_DB_U_ADCZEROONSTART_STR "adczeroonstartup"
#define APP_DB_U_ADCZEROONSTART_MIN (ADC_DB_USER_ZEROONSTART_MINIMUM)
#define APP_DB_U_ADCZEROONSTART_MAX (ADC_DB_USER_ZEROONSTART_MAXIMUM)
#define APP_DB_U_ADCZEROONSTART_DEFAULT (ADC_DB_USER_ZEROONSTART_DEFAULT)

#define APP_DB_U_ADC4WIRE_STR "adc4wiremode"
#define APP_DB_U_ADC4WIRE_MIN (ADC_DB_USER_WIRE4MODE_MINIMUM)
#define APP_DB_U_ADC4WIRE_MAX (ADC_DB_USER_WIRE4MODE_MAXIMUM)
#define APP_DB_U_ADC4WIRE_DEFAULT (ADC_DB_USER_WIRE4MODE_DEFAULT)

#define APP_DB_U_ADCTEMPCALZERO_STR "adctempcalzero"
#define APP_DB_U_ADCTEMPCALZERO_MIN (ADC_DB_USER_TEMPCALZERO_MINIMUM)
#define APP_DB_U_ADCTEMPCALZERO_MAX (ADC_DB_USER_TEMPCALZERO_MAXIMUM)
#define APP_DB_U_ADCTEMPCALZERO_DEFAULT (ADC_DB_USER_TEMPCALZERO_DEFAULT)

#define APP_DB_U_ADCHIRES_STR "adchires"
#define APP_DB_U_ADCHIRES_MIN (ADC_DB_USER_HIRES_MINIMUM)
#define APP_DB_U_ADCHIRES_MAX (ADC_DB_USER_HIRES_MAXIMUM)
#define APP_DB_U_ADCHIRES_DEFAULT (ADC_DB_USER_HIRES_DEFAULT)

#define APP_DB_U_ADCQAENABLE_STR "adcqaenable"
#define APP_DB_U_ADCQAENABLE_MIN (ADC_DB_USER_QAENABLE_MINIMUM)
#define APP_DB_U_ADCQAENABLE_MAX (ADC_DB_USER_QAENABLE_MAXIMUM)
#define APP_DB_U_ADCQAENABLE_DEFAULT (ADC_DB_USER_QAENABLE_DEFAULT)

#define APP_DB_U_ADCQADAY_STR "adcqaday"
#define APP_DB_U_ADCQADAY_MIN (ADC_DB_USER_QADAY_MINIMUM)
#define APP_DB_U_ADCQADAY_MAX (ADC_DB_USER_QADAY_MAXIMUM)
#define APP_DB_U_ADCQADAY_DEFAULT (ADC_DB_USER_QADAY_DEFAULT)

#define APP_DB_U_ADCQAMONTH_STR "adcqamonth"
#define APP_DB_U_ADCQAMONTH_MIN (ADC_DB_USER_QAMONTH_MINIMUM)
#define APP_DB_U_ADCQAMONTH_MAX (ADC_DB_USER_QAMONTH_MAXIMUM)
#define APP_DB_U_ADCQAMONTH_DEFAULT (ADC_DB_USER_QAMONTH_DEFAULT)

#define APP_DB_U_ADCQAYEAR_STR "adcqayear"
#define APP_DB_U_ADCQAYEAR_MIN (ADC_DB_USER_QAYEAR_MINIMUM)
#define APP_DB_U_ADCQAYEAR_MAX (ADC_DB_USER_QAYEAR_MAXIMUM)
#define APP_DB_U_ADCQAYEAR_DEFAULT (ADC_DB_USER_QAYEAR_DEFAULT)

#define APP_DB_U_ADCCCNTEPCAL_STR "adcntepcalibrationcounter"
#define APP_DB_U_ADCCCNTEPCAL_MIN (ADC_DB_USER_CALCNTNTEPCALIBRATION_MINIMUM)
#define APP_DB_U_ADCCCNTEPCAL_MAX (ADC_DB_USER_CALCNTNTEPCALIBRATION_MAXIMUM)
#define APP_DB_U_ADCCCNTEPCAL_DEFAULT (ADC_DB_USER_CALCNTNTEPCALIBRATION_DEFAULT)

#define APP_DB_U_ADCCCNTEPSET_STR "adcntepsettingscounter"
#define APP_DB_U_ADCCCNTEPSET_MIN (ADC_DB_USER_CALCNTNTEPSETTINGS_MINIMUM)
#define APP_DB_U_ADCCCNTEPSET_MAX (ADC_DB_USER_CALCNTNTEPSETTINGS_MAXIMUM)
#define APP_DB_U_ADCCCNTEPSET_DEFAULT (ADC_DB_USER_CALCNTNTEPSETTINGS_DEFAULT)

#define APP_DB_U_ADCCCOIML_STR "adcoimlcalibrationcounter"
#define APP_DB_U_ADCCCOIML_MIN (ADC_DB_USER_CALCNTOIML_MINIMUM)
#define APP_DB_U_ADCCCOIML_MAX (ADC_DB_USER_CALCNTOIML_MAXIMUM)
#define APP_DB_U_ADCCCOIML_DEFAULT (ADC_DB_USER_CALCNTOIML_DEFAULT)

#define APP_DB_U_ADCCALFLAG_STR "adccalflag"
#define APP_DB_U_ADCCALFLAG_MIN (ADC_DB_USER_CALFLAG_MINIMUM)
#define APP_DB_U_ADCCALFLAG_MAX (ADC_DB_USER_CALFLAG_MAXIMUM)
#define APP_DB_U_ADCCALFLAG_DEFAULT (ADC_DB_USER_CALFLAG_DEFAULT)

#define APP_DB_U_ADCCHKSUM_STR "adcchksum"
#define APP_DB_U_ADCCHKSUM_MIN ADC_DB_USER_CHKSUM_MINIMUM
#define APP_DB_U_ADCCHKSUM_MAX ADC_DB_USER_CHKSUM_MAXIMUM
#define APP_DB_U_ADCCHKSUM_DEFAULT ADC_DB_USER_CHKSUM_DEFAULT


/** User application database */
extern struct s_db_app_user db_app_user;


/**
 * Write the user database to file
 *
 * @param write_ass_copy specifies whether the adcSuperStructure copy of the adcUserDatabase should be written
 * @param lock specifies whether adcSuperStructure should be locked before writing
 */
bool app_write_user_database(bool write_ass_copy, bool lock);


/** Read the user database from file */
bool app_read_user_database(void);


/** Default the user database */
bool app_default_user_database(void);


/** Initialise the user database */
bool app_init_user_database(const char *userdb_filename);


/** Destroy the user database */
bool app_destroy_user_database(void);


#endif /* APP_DB_USER_H_ */
