/*
 * cmdline.c
 *
 *  Created on: Mar 17, 2012
 *      Author: ian
 */
#include <stdio.h>
#include <getopt.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>

#include "cmdline.h"


#define DEFAULT_I2C_DEVICE "/dev/i2c-0"
#define DEFAULT_WE21HW_DEVICE "/dev/we21hw"
#define DEFAULT_ADCSPI_DEVICE "/dev/spidev0.0"


bool cmdline_display_help = false;
const char *cmdline_i2c_device = DEFAULT_I2C_DEVICE;
const char *cmdline_we21hw_device = DEFAULT_WE21HW_DEVICE;
const char *cmdline_adcspi_device = DEFAULT_ADCSPI_DEVICE;
const char *cmdline_factorydb_filename = NULL;
const char *cmdline_userdb_filename = NULL;
const char *cmdline_runtimedb_filename = NULL;
const char *cmdline_dsd_filename = NULL;
bool cmdline_dsd_has_alt_rdg = true;
bool cmdline_dsd_has_pieces = true;
const char *cmdline_changelog_filename = NULL;
int cmdline_liveweigh_algorithm = 0;


/** Serial port device to use for remote devices, if NULL remote devices are not used */
const char *cmdline_remote_device = NULL;



void cmdline_print_usage(const char *prog)
{
  printf("Usage: %s [options]\n", prog);
  printf("General Options:\n"
         "  -h --help              this help\n"
         "Hardware Options:\n"
         "  -i --i2cdev <dev>      set the i2c device, default " DEFAULT_I2C_DEVICE "\n"
         "  -w --wedev <dev>       set the we21hw device, default " DEFAULT_WE21HW_DEVICE "\n"
         "  -s --spidev <dev>      set the adc spi device, default " DEFAULT_ADCSPI_DEVICE "\n"
         " OR:\n"
         "  -e --remote <serial>   use remote devices over <serial> port (/dev/ttySx)\n"
         "Database Options:\n"
         "  -f --factory <file>    use file as factory database\n"
         "  -u --user <file>       use file as user database\n"
         "  -r --runtime <file>    use file as runtime database\n"
         "DSD Options:\n"
         "  -d --dsd <file>        use file as the DSD file\n"
         "  -a --no-alt-rdg        dsd entries don't have an alternate reading\n"
         "  -p --no-pieces         dsd entries don't have pieces\n"
         "Change Log Options:\n"
         "  -c --changelog <file>  use file as the change log file\n"
         "Liveweigh Options:\n"
         "  -l --liveweigh <type>  configure live weigh algorithm as type none (default), simple or improved\n"
         "\n"
        );
  printf("\nRinstrum ADC library testing\n");
}


int cmdline_parse_options(int argc, char *argv[])
{
  static const struct option lopts[] =
  {
    /* General */
    { "help",       0, 0, 'h' },

    /* Hardware */
    { "i2cdev",     1, 0, 'i' },
    { "wedev",      1, 0, 'w' },
    { "spidev",     1, 0, 's' },
    { "remote",     1, 0, 'e' },

    /* Databases */
    { "factory",    1, 0, 'f' },
    { "user",       1, 0, 'u' },
    { "runtime",    1, 0, 'r' },

    /* DSD & change log */
    { "dsd",        1, 0, 'd' },
    { "changelog",  1, 0, 'c' },
    { "no-alt-rdg", 0, 0, 'a' },
    { "no-pieces",  0, 0, 'p' },

    /* Live weigh */
    { "liveweigh",  1, 0, 'l' },

    { NULL,         0, 0, '\0' },
  };
  int c;

  while (1)
  {
    c = getopt_long(argc, argv, "hi:w:s:e:f:u:r:d:c:apl:", lopts, NULL);

    if (c == -1)
      break;

    switch (c)
    {
      /* General */
      case 'h':
        cmdline_display_help = true;
        return 0;
        break;

      /* Hardware */
      case 'i':
        cmdline_i2c_device = optarg;
        break;
      case 'w':
        cmdline_we21hw_device = optarg;
        break;
      case 's':
        cmdline_adcspi_device = optarg;
        break;
      case 'e':
        cmdline_remote_device = optarg;
        break;

      /* Databases */
      case 'f':
        cmdline_factorydb_filename = optarg;
        break;
      case 'u':
        cmdline_userdb_filename = optarg;
        break;
      case 'r':
        cmdline_runtimedb_filename = optarg;
        break;

      /* DSD */
      case 'd':
        cmdline_dsd_filename = optarg;
        break;
      case 'a':
        cmdline_dsd_has_alt_rdg = false;
        break;
      case 'p':
        cmdline_dsd_has_pieces = false;
        break;

      /* Change log */
      case 'c':
        cmdline_changelog_filename = optarg;
        break;

      /* Live weigh */
      case 'l':
        if (0 == strcmp(optarg, "none"))
        {
          cmdline_liveweigh_algorithm = 0;
        }
        else if (0 == strcmp(optarg, "simple"))
        {
          cmdline_liveweigh_algorithm = 1;
        }
        else if (0 == strcmp(optarg, "improved"))
        {
          cmdline_liveweigh_algorithm = 2;
        }
        else
        {
          fprintf(stderr, "ERROR: invalid live weigh algorithm, try --help\n");
          return -1;
        }
        break;

      default:
        fprintf(stderr, "ERROR: invalid command line option, try --help\n");
        return -1;
        break;
    }
  }

  return 0;
}

