#include <stdio.h>
#include <rinhw/stdrinstrum.h>
#include <rinhw/hwlib_init.h>
#include <rinblock/rinblock.h>
#include <rinhw/we21hw.h>
#include <rinhw/iic_local_linux.h>
#include <rinremote/remote.h>
#include <rinhw/adc_ads1216_we21.h>
#include <rinhw/sio_local_console.h>
#include <rinblock/rb_we21_adc.h>
#include <rinhw/rtc_local.h>
#include <rinremote/remoteexec.h>
#include <rinremote/iic_remote.h>
#include <rinremote/adc_ads1216_remote.h>
#include <rinblock/rb_startup.h>
#include "cmdline.h"

#include "devices.h"


struct t_io_dev dev_iic0;
struct t_io_dev dev_adc;
struct t_io_dev dev_adc24bit;
struct t_io_dev dev_rtc_test;
struct t_io_dev dev_console;


static void RinBlockAssignDevices(void)
{
  int i;
  int8u adcdone = 0;

  for (i = 0; i < MAX_RINBLOCKS; i++)
  {
    /* ADC Device */
    if (RB_IFACE_TYPE_WE21_CONN == rinblocks[i].tag.interface_type && !adcdone)
    {
      /* Replace dev_adc with the rinblock one */
      if (RIO_SUCCESS == RinBlockCreateWE21AdcDevice(&dev_adc, &rinblocks[i]))
      {
        /* printf("Created ADC device for WE2111 rinblock %d ADC\r\n", i); */
        adcdone = 1;
      }
    }
  }
}


int hwlib_local_start(void)
{
  /* Start the hardware library */
  hwlib_init();

  /* Initialise WE2111 hardware */
  if (rio_init_we21hw(cmdline_we21hw_device) < 0)
  {
    fprintf(stderr, "ERROR: WE2111 hardware device fail\n");
    return -1;
  }

  /* Create the devices */

  if (rio_create_device_sio_localconsole(&dev_console) < 0)
  {
    fprintf(stderr, "ERROR: Local console device creation failed\n");
    return -1;
  }
  if (rio_create_device_iic_local(&dev_iic0, cmdline_i2c_device, we21hw_set_slot_select, NULL) < 0)
  {
    fprintf(stderr, "ERROR: I2C device creation failed\n");
    return -1;
  }
  if (rio_create_device_adc24bit_local(&dev_adc24bit, cmdline_adcspi_device,
                                       adc24bit_user_hw_init_we21,
                                       adc24bit_sync_spi_we21,
                                       NULL,
                                       adc24bit_wait_for_reading_we21) < 0)
  {
    fprintf(stderr, "ERROR: ADC 24bit device creation failed\n");
    return -1;
  }
  if (rio_create_device_rtc_local(&dev_rtc_test) < 0)
  {
    fprintf(stderr, "ERROR: RTC device creation failed\n");
    return -1;
  }

  RinBlockInit(&dev_iic0);
  RinBlockEnumerate();
  RinBlockStart();
  RinBlockAssignDevices();

  return 0;
}


int hwlib_remote_start(void)
{
  /* Start the hardware library */
  hwlib_init();

  /* Initialise remote device comms */
  if (vRemoteExecuteInit(cmdline_remote_device, NULL, 0) < 0)
  {
    fprintf(stderr, "ERROR: Failed to initialise remote comms\n");
    return -1;
  }

  /* Create the devices */
  if (rio_create_device_sio_localconsole(&dev_console) < 0)
  {
    fprintf(stderr, "ERROR: Local console device creation failed\n");
    return -1;
  }
  if (rio_create_device_iic_remote(&dev_iic0, REMOTE_DEV_IIC0) < 0)
  {
    fprintf(stderr, "ERROR: I2C device creation failed\n");
    return -1;
  }
  if (rio_create_device_adc24bit_remote(&dev_adc24bit) < 0)
  {
    fprintf(stderr, "ERROR: ADC 24bit device creation failed\n");
    return -1;
  }
  if (rio_create_device_rtc_local(&dev_rtc_test) < 0)
  {
    fprintf(stderr, "ERROR: RTC device creation failed\n");
    return -1;
  }

  RinBlockInit(&dev_iic0);
  RinBlockEnumerate();
  RinBlockStart();
  RinBlockAssignDevices();

  return 0;
}


int hwlib_start(void)
{
  int result;

  if (NULL != cmdline_remote_device)
  {
    result = hwlib_remote_start();
  }
  else
  {
    result = hwlib_local_start();
  }
  return result;
}


void hwlib_stop(void)
{
  /* Destroy devices */
  rdestroy_io(&dev_console);
  rdestroy_io(&dev_rtc_test);
  rdestroy_io(&dev_adc24bit);
  rdestroy_io(&dev_adc);
  rdestroy_io(&dev_iic0);

  /* Destroy the WE2111 hardware */
  rio_destroy_we21hw();

  /* Shutdown the hardware library */
  hwlib_destroy();
}
