/*
 * dsdtest_purge.c
 *
 *  Created on: Mar 16, 2012
 *      Author: ian
 */
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include <cutest/CuTest.h>
#include <rinweigh/adcdsd.h>
#include "dsdtest.h"
#include "dsdtest_add.h"

#include "dsdtest_purge.h"


/** Purge records from the DSD */
static void test_dsd_purge(CuTest *tc)
{
  enum eAdcDsdError res;
  int8u buffer[DSD_TEST_RECORD_SIZE];
  int32u beforerecords;
  int32u afterrecords;

  /* Make sure DSD is full */
  test_dsd_fill(tc);

  /* Get before number of records */
  res = AdcDSDGetNumberRecords(test_dsd, true, &beforerecords);
  CuAssertIntEquals(tc, ADC_DSD_SUCCESS, res);

  /* Try to add a record, will fail */
  memset(buffer, 'a', DSD_TEST_RECORD_SIZE);
  res = AdcDSDWriteRecord(test_dsd, true, buffer, DSD_TEST_RECORD_SIZE);
  CuAssertIntEquals(tc, ADC_DSD_ERROR_FULL, res);

  /* printf("Purge Start time: %lu\n", (unsigned long) time(NULL)); */

  /* Now purge a record */
  res = AdcDSDPurge(test_dsd, true, 1);
  CuAssertIntEquals(tc, ADC_DSD_SUCCESS, res);

  /* printf("Purge End time: %lu\n", (unsigned long) time(NULL)); */

  /* Get after number of records, and check how many purged */
  res = AdcDSDGetNumberRecords(test_dsd, true, &afterrecords);
  CuAssertIntEquals(tc, ADC_DSD_SUCCESS, res);
  CuAssertIntEquals(tc, beforerecords - 1, afterrecords);

  /* Try to add a record, will succeed */
  memset(buffer, '1', DSD_TEST_RECORD_SIZE);
  res = AdcDSDWriteRecord(test_dsd, true, buffer, DSD_TEST_RECORD_SIZE);
  CuAssertIntEquals(tc, ADC_DSD_SUCCESS, res);

  /* Try to add a second record, will fail */
  memset(buffer, '2', DSD_TEST_RECORD_SIZE);
  res = AdcDSDWriteRecord(test_dsd, true, buffer, DSD_TEST_RECORD_SIZE);
  CuAssertIntEquals(tc, ADC_DSD_ERROR_FULL, res);
}


/** Purge outside DSD limits */
static void test_dsd_purge_limits(CuTest *tc)
{
  enum eAdcDsdError res;
  int32u beforerecords;
  int32u afterrecords;

  /* Make sure DSD is full */
  test_dsd_fill(tc);

  /* Get before number of records */
  res = AdcDSDGetNumberRecords(test_dsd, true, &beforerecords);
  CuAssertIntEquals(tc, ADC_DSD_SUCCESS, res);

  /* Try to purge more records than there are */
  res = AdcDSDPurge(test_dsd, true, beforerecords + 1);
  CuAssertIntEquals(tc, ADC_DSD_ERROR_OUTOFRANGE, res);

  /* Get after number of records, and check none purged */
  res = AdcDSDGetNumberRecords(test_dsd, true, &afterrecords);
  CuAssertIntEquals(tc, ADC_DSD_SUCCESS, res);
  CuAssertIntEquals(tc, beforerecords, afterrecords);

  /* Now purge all */
  res = AdcDSDPurge(test_dsd, true, beforerecords);
  CuAssertIntEquals(tc, ADC_DSD_SUCCESS, res);

  /* Get after number of records, and check all purged */
  res = AdcDSDGetNumberRecords(test_dsd, true, &afterrecords);
  CuAssertIntEquals(tc, ADC_DSD_SUCCESS, res);
  CuAssertIntEquals(tc, 0, afterrecords);

  /* Try to purge again, will fail because empty */
  res = AdcDSDPurge(test_dsd, true, 1);
  CuAssertIntEquals(tc, ADC_DSD_ERROR_EMPTY, res);

  /* Get after number of records, should still be 0 */
  res = AdcDSDGetNumberRecords(test_dsd, true, &afterrecords);
  CuAssertIntEquals(tc, ADC_DSD_SUCCESS, res);
  CuAssertIntEquals(tc, 0, afterrecords);
}


/** Test purging of records from the DSD */
CuSuite *CuGetSuite_DSD_Purge(void)
{
  CuSuite *suite = CuSuiteNew();

  /* Purge records from the DSD */
  SUITE_ADD_TEST(suite, test_dsd_purge);
  SUITE_ADD_TEST(suite, test_dsd_purge_limits);

  return suite;
}
