#include <stdio.h>
#include <rinhw/rio.h>
#include <rinweigh/adcdsd.h>
#include "devices.h"
#include "cmdline.h"
#include "app_db_changelog.h"
#include "menu_liveweigh.h"
#include "menu.h"
#include "adcthread.h"
#include "app_db.h"


int create_dsd(struct s_adc_dsd **testdsd)
{
  if (NULL != cmdline_dsd_filename)
  {
    if (ADC_DSD_SUCCESS != AdcDSDInit(testdsd, NULL))
    {
      fprintf(stderr, "ERROR: failed to create DSD\n");
      return -1;
    }
    if (ADC_DSD_SUCCESS != AdcDSDOpenExisting(*testdsd, true, cmdline_dsd_filename))
    {
      fprintf(stderr, "ERROR: failed to open DSD file\n");
      return -1;
    }
  }
  return 0;
}


/** Application entry point */
int main(int argc, char *argv[])
{
  struct s_adc_dsd *testdsd = NULL;
  AdcLiveWeighImplementer *liveweigh = NULL;

  /* Do command line */
  if (cmdline_parse_options(argc, argv) < 0)
  {
    return 1;
  }
  if (cmdline_display_help)
  {
    cmdline_print_usage(argv[0]);
    return 0;
  }

  setvbuf(stdout, NULL, _IONBF, 0); /* Unbuffered output */

  /* Start hardware library and create devices */
  if (hwlib_start() < 0)
  {
    return 0;
  }

  if (NULL != cmdline_changelog_filename)
  {
    if (!app_changelog_init(cmdline_changelog_filename))
    {
      fprintf(stderr, "ERROR: change log and/or change log reader failed\n");
      goto fail;
    }
  }

  /* Create databases */
  app_init_and_read_databases(cmdline_factorydb_filename, cmdline_userdb_filename, cmdline_runtimedb_filename);

  /* Create live weigh implementation */
  liveweigh = create_live_weigh_implementation(cmdline_liveweigh_algorithm);

  /* Create ADC thread */
  adc_thread_init(liveweigh);

  /* Create DSD & Change log */
  if (create_dsd(&testdsd) < 0)
  {
    goto fail;
  }

  /* Run application */
  ropen(&dev_console, 0);
  adc_test_menu(testdsd, app_changelog_reader, liveweigh, cmdline_dsd_has_alt_rdg, cmdline_dsd_has_pieces);
  rclose(&dev_console);

fail:
  if (NULL != testdsd)
  {
    AdcDSDDestroy(testdsd);
  }

  adc_thread_destroy();
  if (NULL != liveweigh)
  {
    AdcLiveWeighDestroy(liveweigh);
  }
  app_destroy_databases();
  app_changelog_destroy();
  hwlib_stop();
  return 0;
}
