﻿/*
 * menu.c
 *
 *  Created on: 29/03/2011
 *      Author: ian
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdlib.h>
#include <termios.h>
#include <unistd.h>
#include <rinhw/stdrinstrum.h>
#include <rinweigh/adcdatabase.h>
#include <rinweigh/adcinterface.h>
#include "getchar.h"
#include "print_rdg.h"
#include "print_settings.h"
#include "menu_database.h"
#include "menu_liveweigh.h"
#include "adcthread.h"
#include "alibi_test.h"
#include "app_db_user.h"

#include "menu.h"


bool get_int(int *val, int low, int high)
{
  char str[20];
  int pos;
  char ch;
  int tmp;

  for (pos = 0; pos < 20 && ch != '\r' && ch != '\n'; pos++)
  {
    get_stdin_char_block(&ch);
    write(STDOUT_FILENO, &ch, sizeof(ch));
    str[pos] = ch;
  }
  str[20 - 1] = '\0';

  tmp = strtol(str, 0, 0);

  if (tmp < low || tmp > high)
  {
    return false;
  }

  *val = tmp;
  return true;
}


struct s_set_var
{
  const char *name;
  const char *pre_text;
  int32s min;
  int32s max;
  int8u *val8;
  int32s *val32;
};


/** Scale type */
const struct s_set_var cal_var_scale_type =
{
  .name = "Scale Type",
  .pre_text = "0=Single, 1=DualInterval, 2=DualRange, 3=TripleInterval, 4=TripleRange",
  .min = ADC_SCALE_TYPE_SINGLE,
  .max = ADC_SCALE_TYPE_TRIPLE_RANGE,
  .val8 = &adcSuperStructure.adcUserDatabase.scaleType,
  .val32 = NULL,
};


/** Decimal place */
const struct s_set_var cal_var_decimal_places =
{
  .name = "Decimal Places",
  .pre_text = "0..5",
  .min = 0,
  .max = 5,
  .val8 = &adcSuperStructure.adcUserDatabase.dpPos,
  .val32 = NULL,
};


/** Units */
const struct s_set_var cal_var_units =
{
  .name = "Units",
  .pre_text = "0=none, 1=g, 2=kg, 3=t, 4=lb, 5=oz, 6=N, 7=P, 8-15=user defined",
  .min = 0,
  .max = 15,
  .val8 = &adcSuperStructure.adcUserDatabase.units,
  .val32 = NULL,
};


/** Full scale 1 */
const struct s_set_var cal_var_fullscale1 =
{
  .name = "Fullscale 1",
  .pre_text = "100..999999",
  .min = 100,
  .max = 999999,
  .val8 = NULL,
  .val32 = &adcSuperStructure.adcUserDatabase.fullScale[0],
};


/** Resolution 1 */
const struct s_set_var cal_var_resolution1 =
{
  .name = "Resolution 1",
  .pre_text = "0='1',1='2',2='5',3='10',4='20',5='50',6='100'",
  .min = 0,
  .max = 6,
  .val8 = &adcSuperStructure.adcUserDatabase.resolution[0],
  .val32 = NULL,
};

/** Sync Freq */
const struct s_set_var cal_var_syncfreq =
{
  .name = "Sync frequency",
  .pre_text = "0=10Hz, 1=12.5Hz, 2=15Hz, 3=20Hz, 4=25Hz, 5=30Hz, 6=50Hz, 7=60Hz, 8=100Hz",
  .min = ADC_SYNC_FREQ_10_HZ,
  .max = ADC_NUM_SYNC_FREQ - 1,
  .val8 = &adcSuperStructure.adcUserDatabase.syncFreqIdx,
  .val32 = NULL,
};

/** Jitter filter */
const struct s_set_var cal_var_jitter =
{
  .name = "Anti-jitter_filter",
  .pre_text = "0-2",
  .min = 0,
  .max = 2,
  .val8 = &adcSuperStructure.adcUserDatabase.jitter,
  .val32 = NULL,
};

/** Averaging filter */
const struct s_set_var cal_var_average =
{
  .name = "Averaging filter",
  .pre_text = "0-500",
  .min = 0,
  .max = 500,
  .val8 = NULL,
  .val32 = (int32s *) &adcSuperStructure.adcUserDatabase.average
};

/** Motion */
const struct s_set_var cal_var_motion =
{
  .name = "Motion",
  .pre_text = "0-12",
  .min = 0,
  .max = 12,
  .val8 = &adcSuperStructure.adcUserDatabase.motion,
  .val32 = NULL,
};

static void set_variable(const struct s_set_var *sv)
{
  int val;

  if (sv->val8 != NULL)
  {
    val = *(sv->val8);
  }
  else if (sv->val32 != NULL)
  {
    val = *(sv->val32);
  }
  else
  {
    val = 0;
  }

  printf("\nSet %s (current: %d):%s\n>", sv->name, val, sv->pre_text);

  if (get_int(&val, sv->min, sv->max))
  {
    if (sv->val8 != NULL)
    {
      *sv->val8 = val;
    }
    else if (sv->val32 != NULL)
    {
      *sv->val32 = val;
    }

    printf("\n%s Set to %d\n", sv->name, val);
  }
  else
  {
    printf("\nBad %s, not set\n", sv->name);
  }
}


static void adc_calibration_menu(void)
{
  char ch;
  bool quit = false;
  int32s temp;
  int i;

  while (!quit)
  {
    printf("\nADC Library Calibration\n\n"
           " p - Print settings\n"
           " t - Scale Type\n"
           " d - Decimal Places\n"
           " u - Units\n"
           " f - Full Scale 1\n"
           " r - Resolution 1\n"
           " h - Sync frequency\n"
           " a - Averaging [x 0.01s]\n"
           " j - Anti-jitter filter\n"
           " m - Motion\n"
           " z - Do Zero Calibration\n"
           " s - Do Span Calibration\n"
           " w - Save ADC Database\n"
           " i - Re-initialise ADC\n"
           " q - Quit\n"
          );

    get_stdin_char_block(&ch);
    switch (ch)
    {
      /* Scale settings */
      case 'p':
        print_build_settings  (stdout, &adcSuperStructure.adcUserDatabase);
        print_option_settings (stdout, &adcSuperStructure.adcUserDatabase);
        print_other_settings  (stdout, &adcSuperStructure.adcUserDatabase);
        print_cal_settings    (stdout, &adcSuperStructure.adcUserDatabase);
        print_factory_settings(stdout, &adcSuperStructure.adcFactoryDatabase);
        break;
      case 't':
        set_variable(&cal_var_scale_type);
        break;
      case 'd':
        set_variable(&cal_var_decimal_places);
        break;
      case 'u':
        set_variable(&cal_var_units);
        break;
      case 'f':
        set_variable(&cal_var_fullscale1);
        break;
      case 'r':
        set_variable(&cal_var_resolution1);
        break;
      case 'h':
        set_variable(&cal_var_syncfreq);
        break;
      case 'j':
        set_variable(&cal_var_jitter);
        break;
      case 'a':
        set_variable(&cal_var_average);
        break;
      case 'm':
        set_variable(&cal_var_motion);
        break;

      /* Save database */
      case 'w':
        printf("\nSaving ADC Database\n");
        AdcSaveDatabases(&adcSuperStructure, true, NULL, &adcSuperStructure.adcUserDatabase, NULL, &db_app_user.adc_user_database);
        break;

      /* Re-initialise */
      case 'i':
        AdcReInit(&adcSuperStructure, true, NULL, NULL, NULL);
        break;

      /* zero calibration */
      case 'z':
        printf("\nPerforming zero calibration .");

        // start zero calibration
        AdcZeroCalibration(&adcSuperStructure, true, false);

        // wait for cal to finish
        while ((temp=AdcGetCurrentOpStatus(&adcSuperStructure, true)) == ADC_OP_STATUS_IN_PROGRESS) {printf(".");usleep(100000);}

        // process result
        if (temp == ADC_OP_STATUS_SUCCESS)
          printf("done\n");
        else
          {print_adc_operation_error(stdout,temp); printf("\n");}
        break;

      /* zero calibration */
      case 's':
        printf("\nEnter span calibration weight (without dp, 100 .. %ld): ", (long) adcSuperStructure.adcUserDatabase.fullScale[adcSuperStructure.adcInternalData.rangeMax]);
        usleep(100000);

        if (get_int(&i, 100, adcSuperStructure.adcUserDatabase.fullScale[adcSuperStructure.adcInternalData.rangeMax]))
        {
          temp = i;
          printf("\nPerforming span calibration .");
          AdcSpanCalibration(&adcSuperStructure, true, false, temp);

          // wait for cal to finish
          while ((temp=AdcGetCurrentOpStatus(&adcSuperStructure, true)) == ADC_OP_STATUS_IN_PROGRESS) {printf(".");usleep(100000);}

          // process result
          if (temp == ADC_OP_STATUS_SUCCESS)
            printf("done\n");
          else
            {print_adc_operation_error(stdout,temp); printf("\n");}
          break;
        }
        else
          printf("\nCalibration weight error\n");

        break;

      /* Quit */
      case 'q':
        quit = true;
        break;
    }
  }
}


void wait_for_average(void)
{
  time_t start_time;
  time_t end_time;
  int8u temp;

  printf("Waiting for average buffer to fill, averaging is: %lu [*0.01s]\n", (unsigned long) adcSuperStructure.adcUserDatabase.average);
  start_time = time(NULL);
  AdcWaitForAverage(&adcSuperStructure, true, false);
  while ((temp = AdcGetCurrentOpStatus(&adcSuperStructure, true)) == ADC_OP_STATUS_IN_PROGRESS)
  {
    printf(".");
    usleep(100000);
  }
  end_time = time(NULL);
  printf("\nTook %u seconds (with second granularity), result: ", (unsigned) (end_time - start_time));
  print_adc_operation_error(stdout, temp);
  printf("\n");
}


void adc_test_menu(struct s_adc_dsd *testdsd,
                   struct s_db_changelog_reader *test_changelog_reader,
                   AdcLiveWeighImplementer *liveweigh,
                   bool dsd_has_altrdg,
                   bool dsd_has_pieces)
{
  char ch;
  bool quit;

  for (quit = false; !quit; )
  {
    printf("\nADC Library Test Application\n\n"
           " p - Print Reading\n"
           " d - Print ADC Data\n"
           " z - Zero Scale\n"
           " t - Tare Scale\n"
           " g - Switch to Gross Reading\n"
           " n - Switch to Net Reading\n"
           " s - Switch between Gross and Net\n"
           " h - Toggle high resolution mode\n"
           " m - Manual hold\n"
           " M - Livestock menu\n"
           " w - Wait for average\n"
           " P - Print reading at 10Hz.  Press any key to halt.\n"
           " c - Calibration menu\n"
           " a - Database menu\n"
           " l - Alibi test with single display\n"
           " L - Alibi test with dual display\n"
           " q - Quit\n"
          );

    get_stdin_char_block(&ch);
    switch (ch)
    {
      /* Display readings */
      case 'p':
        AdcMutexLock(&adcSuperStructure);
        print_adc_full_reading(stdout, &adcSuperStructure.adcDataWorking.dispRdg);
        AdcMutexUnlock(&adcSuperStructure);
        break;
      case 'd':
        AdcMutexLock(&adcSuperStructure);
        print_adc_data(&adcSuperStructure.adcDataWorking);
        AdcMutexUnlock(&adcSuperStructure);
        break;

      /* Zero/Gross/Net/Tare Operations */
      case 'z':
        printf("Semiautomatic Zero Scale\n");
        print_adc_operation_error(stdout, AdcSemiautoZero(&adcSuperStructure, true, true, false, false));
        break;
      case 't':
        printf("Semiautomatic Tare Scale\n");
        print_adc_operation_error(stdout, AdcSemiautoTare(&adcSuperStructure, true, true, false, false));
        break;
      case 'g':
        printf("Switch Scale To Gross\n");
        print_adc_operation_error(stdout, AdcToGross(&adcSuperStructure, true, true));
        break;
      case 'n':
        printf("Switch Scale To Net\n");
        print_adc_operation_error(stdout, AdcToNet(&adcSuperStructure, true, true));
        break;
      case 's':
        printf("Toggle Scale Gross/Net\n");
        print_adc_operation_error(stdout, AdcGrossNetToggle(&adcSuperStructure, true, true));
        break;
      case 'h':
        if (adcSuperStructure.adcInternalData.rawMult == 1000000)
        {
          printf("Hi-res on\n");
          AdcSetCompValues(&adcSuperStructure, true, 0, 100000);
        }
        else
        {
          printf("\nHi-res off\n");
          AdcSetCompValues(&adcSuperStructure, true, 0, 1000000);
        }
        break;

      /* Holds */
      case 'm':
        printf("Manual hold\n");
        print_adc_operation_error(stdout, AdcManualHold(&adcSuperStructure, true, true));
        AdcMutexLock(&adcSuperStructure);
        print_adc_data(&adcSuperStructure.adcHeldData);
        AdcMutexUnlock(&adcSuperStructure);
        break;
      case 'M':
        adc_liveweigh_menu(liveweigh);
        break;

      case 'w':
        wait_for_average();
        break;

      case 'P':
        while (!get_stdin_char(&ch))
        {
          AdcMutexLock(&adcSuperStructure);
          print_adc_full_reading(stdout, &adcSuperStructure.adcDataWorking.dispRdg);
          AdcMutexUnlock(&adcSuperStructure);
          usleep(100000);
        }
        break;

      /* Submenus */
      case 'c':
        adc_calibration_menu();
        break;
      case 'a':
        adc_database_menu();
        break;
      case 'l':
        adc_alibi_test(aadtSingleDisplay, testdsd, test_changelog_reader, dsd_has_altrdg, dsd_has_pieces);
        break;
      case 'L':
        adc_alibi_test(aadtDoubleDisplay, testdsd, test_changelog_reader, dsd_has_altrdg, dsd_has_pieces);
        break;

      /* Quit */
      case 'q':
        quit = true;
        break;
    }
  }
}
