/*
 * print_rdg.c
 *
 *  Created on: 29/03/2011
 *      Author: ian
 */
#include <math.h>
#include <stdio.h>
#include <rinweigh/adcdatabase.h>
#include <rinweigh/adcdatatypes.h>
#include <rinweigh/adcinterface.h>

#include "print_settings.h"

void print_group_start(FILE *f, const char *name)
{
  fprintf(f,"%s = (",name);
}

void print_setting(FILE *f, const char *name, long setting)
{
  fprintf(f,"%s:%ld, ",name, setting);
}


void print_hex_setting(FILE *f, const char *name, long setting)
{
  fprintf(f,"%s:0x%lX, ", name, setting);
}


void print_group_finish(FILE *f)
{
  fprintf(f,")\n");
}

void print_build_settings(FILE *f, const AdcUserDatabase *udb)
{
  print_group_start(f,"Build");
  print_setting(f, "Type",    udb->scaleType);
  print_setting(f, "Dp",      udb->dpPos);
  print_setting(f, "Cap1",    udb->fullScale[0]);
  print_setting(f, "E1",      udb->resolution[0]);
  print_setting(f, "Cap2",    udb->fullScale[1]);
  print_setting(f, "E2",      udb->resolution[1]);
  print_setting(f, "A.tare",udb->addTare);
  print_setting(f, "Units",   udb->units);
  print_group_finish(f);
}

void print_option_settings(FILE *f, const AdcUserDatabase *udb)
{
  print_group_start(f,"Option");
  print_setting(f, "Use",    udb->approvalMode);
  print_setting(f, "Av",     udb->average);
  print_setting(f, "Jit",    udb->jitter);
  print_setting(f, "Mot",    udb->motion);
  print_setting(f, "Auto.Z", udb->zeroOnStart);
  print_setting(f, "Z.trac", udb->zeroTracking);
  print_setting(f, "Z.ran",  udb->rangeOfZero);
  print_setting(f, "Z.band", udb->zeroBand);
  //print_setting(f, "Sense Error", ); //todo
  print_group_finish(f);
}

void print_other_settings(FILE *f, const AdcUserDatabase *udb)
{
  print_group_start(f,"Other");
  print_setting(f, "SyncFreqIdx",  udb->syncFreqIdx);
  print_setting(f, "Hires", udb->hiRes);
  print_group_finish(f);
}

void print_cal_settings(FILE *f, const AdcUserDatabase *udb)
{
  print_group_start(f,"Cal");
  print_setting(f, "Z.raw",    udb->zeroRaw);
  print_setting(f, "S.raw",    udb->spanRaw);
  print_setting(f, "S.cnts", udb->spanCnt);
  /*
  int8u i;
  char s[15];

  for (i=0;i<(ADC_MAX_NUM_LIN_PTS+1);i++)
  {
    fprintf(f,"\n        ");
    sprintf(s,"Lin.Pnt.%d",i);
    print_setting(f, s, udb->linPts[i]);
    sprintf(s,"Lin.Cor.%d",i);
    print_setting(f, s, udb->linCor[i]);
  }
  */
  print_setting(f, "OIML",     udb->calCntOIML);
  print_setting(f, "N.cal",    udb->calCntNTEPCalibration);
  print_setting(f, "N.conf",   udb->calCntNTEPSettings);
  print_hex_setting(f, "Cal.flag", udb->calFlag);
  print_group_finish(f);
}


void print_user_checksum(FILE *f, const AdcUserDatabase *udb)
{
  bool chksum_ok;
  int16u calc_chksum;

  calc_chksum = DB_CHECKSUM(AdcUserDatabase, *udb, ADC_DB_USER_CHECKSUM_SEED);
  chksum_ok = (udb->chksum == calc_chksum);

  print_group_start(f, "U.Chksum");
  print_hex_setting(f, "db.chk", udb->chksum);
  print_hex_setting(f, "calc.chk", calc_chksum);
  fprintf(f, "Checksum:%s", (chksum_ok) ? "Pass" : "Fail");
  print_group_finish(f);
}


// Constants for use with AdcUserDatabase.syncFreqIdx
static const int16u LocalAdcSyncFrequencies[ADC_NUM_SYNC_FREQ] = ADC_SYNC_FREQUENCIES;


void print_factory_settings(FILE *f, const AdcFactoryDatabase *fdb)
{
  int i;
  char str[40];

  print_group_start(f, "Factory");
  for (i = 0; i < ADC_NUM_SYNC_FREQ; i++)
  {
    snprintf(str, 40, "Z.ofs@%u.%01uHz", LocalAdcSyncFrequencies[i] / 10, LocalAdcSyncFrequencies[i] % 10);
    print_setting(f, str, fdb->zeroOffset[i]);
  }
  for (i = 0; i < ADC_NUM_SYNC_FREQ; i++)
  {
    snprintf(str, 40, "Z.drift.pos@%u.%01uHz", LocalAdcSyncFrequencies[i] / 10, LocalAdcSyncFrequencies[i] % 10);
    print_setting(f, str, fdb->zeroDriftPos[i]);
  }
  for (i = 0; i < ADC_NUM_SYNC_FREQ; i++)
  {
    snprintf(str, 40, "Z.drift.neg@%u.%01uHz", LocalAdcSyncFrequencies[i] / 10, LocalAdcSyncFrequencies[i] % 10);
    print_setting(f, str, fdb->zeroDriftNeg[i]);
  }
  for (i = 0; i < ADC_NUM_SYNC_FREQ; i++)
  {
    snprintf(str, 40, "S.cal@%u.%01uHz", LocalAdcSyncFrequencies[i] / 10, LocalAdcSyncFrequencies[i] % 10);
    print_setting(f, str, fdb->spanCal[i]);
  }
  for (i = 0; i < ADC_NUM_SYNC_FREQ; i++)
  {
    snprintf(str, 40, "S.drift.pos@%u.%01uHz", LocalAdcSyncFrequencies[i] / 10, LocalAdcSyncFrequencies[i] % 10);
    print_setting(f, str, fdb->spanDriftPos[i]);
  }
  for (i = 0; i < ADC_NUM_SYNC_FREQ; i++)
  {
    snprintf(str, 40, "S.drift.neg@%u.%01uHz", LocalAdcSyncFrequencies[i] / 10, LocalAdcSyncFrequencies[i] % 10);
    print_setting(f, str, fdb->spanDriftNeg[i]);
  }
  print_setting(f, "CalTemp", fdb->calTemp);
  print_hex_setting(f, "Cal.flg", fdb->calFlag);
  print_group_finish(f);
}


void print_factory_checksum(FILE *f, const AdcFactoryDatabase *fdb)
{
  bool chksum_ok;
  int16u calc_chksum;

  calc_chksum = DB_CHECKSUM(AdcFactoryDatabase, *fdb, ADC_DB_FACTORY_CHECKSUM_SEED);
  chksum_ok = (fdb->chksum == calc_chksum);

  print_group_start(f, "F.Chksum");
  print_hex_setting(f, "db.chk", fdb->chksum);
  print_hex_setting(f, "calc.chk", calc_chksum);
  fprintf(f, "Checksum:%s", (chksum_ok) ? "Pass" : "Fail");
  print_group_finish(f);
}

