/*
 * lsnc_dhcp.c
 *
 *  Created on: 22 Jun. 2018
 *      Author: ian
 */
#include "config.h"
#include <stdbool.h>
#include <string.h>
#include <setnetconfig.h>
#include "lsnc_rcconf.h"
#include "lsnc_network_manager.h"

#include "lsnc_dhcp.h"


/** Should method be used */
static bool snc_use_config_method(const struct s_snc_configuration *config, const char *this_method, const char *method)
{
  return (strcasecmp(this_method, method) == 0 ||
          (strcasecmp(SNC_METHOD_LIB_CONFIG, method) == 0 &&
           strcasecmp(this_method, config->config_method) == 0));
}


/** Get the network configuration DHCP setting by the specified config method (rcconf or nm) */
static enum e_snc_result snc_get_net_config_dhcp(const struct s_snc_configuration *config,
                                                 const char *method, const char *eth_interface,
                                                 bool *use_dhcp)
{
  enum e_snc_result result;

  if (snc_use_config_method(config, SNC_METHOD_RCCONF, method))
    result = snc_get_net_config_dhcp_rcconf(config->rcconf_file, eth_interface, use_dhcp);
#if defined(HAVE_LIBNM)
  else if (snc_use_config_method(config, SNC_METHOD_NETWORK_MANAGER, method))
    result = snc_get_net_config_dhcp_network_manager(eth_interface, use_dhcp);
#endif /* #if defined(HAVE_LIBNM) */
  else if (snc_use_config_method(config, SNC_METHOD_AUTO, method))
  {
    result = snc_get_net_config_dhcp_rcconf(config->rcconf_file, eth_interface, use_dhcp);
#if defined(HAVE_LIBNM)
    if (sncr_success != result)
      result = snc_get_net_config_dhcp_network_manager(eth_interface, use_dhcp);
#endif /* #if defined(HAVE_LIBNM) */
  }
  else
  {
    result = sncr_paramerror;
  }

  return result;
}


/**
 * Get the network configuration DHCP setting by trying to guess the best config method
 * based on the current method (ioctl, getifaddrs, etc)
 */
enum e_snc_result snc_get_net_current_dhcp(const struct s_snc_configuration *config,
                                           const char *current_method, const char *eth_interface,
                                           bool *use_dhcp)
{
  const char *config_method;
  enum e_snc_result result;

  /* Convert the current_method into config method */
  if (strcasecmp(SNC_METHOD_NETWORK_MANAGER, current_method) == 0 ||
      strcasecmp(SNC_METHOD_RCCONF, current_method) == 0 ||
      strcasecmp(SNC_METHOD_AUTO, current_method) == 0 ||
      strcasecmp(SNC_METHOD_LIB_CONFIG, current_method) == 0)
  {
    config_method = current_method;
  }
  else
  {
    config_method = SNC_METHOD_AUTO;
  }
  result = snc_get_net_config_dhcp(config, config_method, eth_interface, use_dhcp);
  return result;
}

