/*
 * lsnc_internal.h
 *
 *  Created on: 14 Jun. 2018
 *      Author: ian
 */

#ifndef LSNC_INTERNAL_H_
#define LSNC_INTERNAL_H_


/** When parsing lines from files/process output, this is the maximum length that can be handled. This is malloc'd */
#define SNC_PARSE_LINE_LENGTH 1024


/** When getting route information from PF_NETLINK socket, use a buffer this big. This is malloc'd */
#define SNC_PACKET_FILTER_BUFFER_SIZE 8192


/** Maximum length of a variable name in rc.conf (this is allocated on the stack) */
#define SNC_RCCONF_VARNAME_LENGTH 100


/** Maximum length of a command to execute (this is allocated on the stack) */
#define SNC_EXEC_COMMAND_LENGTH 100


/** "/etc/rc.d/init.d/network" supports 5 interfaces */
#define SNC_RCCONF_MAX_INTERFACES 5


/*
 * Variable names in rc.conf
 */
/** The interface name variable ie INTERFACE0=eth0 */
#define SNC_RCCONF_VARNAME_INTERFACE "INTERFACE"

/** Should the interface be configured y/n, ie SYSCFG_IFACE0=y */
#define SNC_RCCONF_VARNAME_SYSCFG_IFACE "SYSCFG_IFACE"
#define SNC_RCCONF_VARNAME_IPADDR "IPADDR"
#define SNC_RCCONF_VARNAME_NETMASK "NETMASK"
#define SNC_RCCONF_VARNAME_BROADCAST "BROADCAST"
#define SNC_RCCONF_VARNAME_GATEWAY "GATEWAY"
#define SNC_RCCONF_VARNAME_NAMESERVER "NAMESERVER"
#define SNC_RCCONF_VARNAME_SEARCHDOMAIN "SEARCHDOMAIN"
#define SNC_RCCONF_VARNAME_HOSTNAME "HOSTNAME"
#define SNC_RCCONF_VARNAME_DOMAINNAME "DOMAINNAME"


/** The IPADDRx setting if we are using DHCP for this interface */
#define SNC_RCCONF_VARVALUE_IPADDR_DHCP "dhcp"


/*
 * Library configuration
 */


#define SNC_CONFIG_VARNAME_RESOLV_FILE "snc_resolvconf"
#define SNC_CONFIG_VARNAME_ROUTE_CMD "snc_routecmd"
#define SNC_CONFIG_VARNAME_IFCONFIG_CMD "snc_ifconfigcmd"
#define SNC_CONFIG_VARNAME_IP_CMD "snc_ipcmd"
#define SNC_CONFIG_VARNAME_RCCONF_FILE "snc_rcconf"
#define SNC_CONFIG_VARNAME_CONFIG_METHOD "snc_config_method"
#define SNC_CONFIG_VARNAME_CURRENT_METHOD "snc_current_method"
#define SNC_CONFIG_VARNAME_NET_RECONFIGURE_CMD "snc_net_reconfigure_cmd"
#define SNC_CONFIG_VARNAME_HOSTNAME_RECONFIGURE_CMD "snc_hostname_reconfigure_cmd"


#define SNC_CONFIG_RESOLV_FILE_DEFAULT "/etc/resolv.conf"
#define SNC_CONFIG_ROUTE_CMD_DEFAULT "/sbin/route"
#define SNC_CONFIG_IFCONFIG_CMD_DEFAULT "/sbin/ifconfig"
#define SNC_CONFIG_IP_CMD_DEFAULT "/sbin/ip"
#define SNC_CONFIG_RCCONF_FILE_DEFAULT "/etc/rc.d/rc.conf"
#define SNC_CONFIG_CONFIG_METHOD_DEFAULT SNC_METHOD_AUTO
#define SNC_CONFIG_CURRENT_METHOD_DEFAULT SNC_METHOD_AUTO
#define SNC_CONFIG_NET_RECONFIGURE_CMD_DEFAULT "/etc/rc.d/init.d/network restart"
#define SNC_CONFIG_HOSTNAME_RECONFIGURE_CMD_DEFAULT "/etc/rc.d/init.d/hostname restart"


/** Dispose of whatever ptr points to with func, then set ptr to NULL */
#define SNC_DESTROY_AND_NULLIFY(func, ptr) \
  do \
  { \
    if (NULL != (ptr)) \
    { \
      func((ptr)); \
      (ptr) = NULL; \
    } \
  } while (0)


/** Free a dynamically allocated variable, and set its pointer to NULL */
#define SNC_FREE_AND_NULLIFY(var) SNC_DESTROY_AND_NULLIFY(free, var)


/** fclose() a FILE, and set its pointer to NULL */
#define SNC_FCLOSE_AND_NULLIFY(file) SNC_DESTROY_AND_NULLIFY(fclose, file)


/** pclose() a FILE from popen(), and set its pointer to NULL */
#define SNC_PCLOSE_AND_NULLIFY(file) SNC_DESTROY_AND_NULLIFY(pclose, file)


/** close() a file descriptor, and set it to -1 */
#define SNC_CLOSE_AND_INVALIDATE(fd) \
  do \
  { \
    if ((fd) >= 0) \
    { \
      close((fd)); \
      (fd) = -1; \
    } \
  } while (0)


#endif /* LSNC_INTERNAL_H_ */
