/*
 * lsnc_netconfig.c
 *
 *  Created on: 6 Jun. 2018
 *      Author: ian
 */
#include "config.h"
#include <stdbool.h>
#include <string.h>

#include <setnetconfig.h>


/** Should method be used */
static bool snc_use_config_method(const struct s_snc_configuration *config, const char *this_method, const char *method)
{
  return (strcasecmp(this_method, method) == 0 ||
          (strcasecmp(SNC_METHOD_LIB_CONFIG, method) == 0 &&
           strcasecmp(this_method, config->config_method) == 0));
}


/** Get the network configuration by the specified method */
enum e_snc_result snc_get_net_config(const struct s_snc_configuration *config,
                                     const char *method, const char *eth_interface,
                                     struct s_snc_net_config *net_config)
{
  enum e_snc_result result;

  if (snc_use_config_method(config, SNC_METHOD_RCCONF, method))
    result = snc_get_net_config_rcconf(config->rcconf_file, eth_interface, net_config);
#if defined(HAVE_LIBNM)
  else if (snc_use_config_method(config, SNC_METHOD_NETWORK_MANAGER, method))
    result = snc_get_net_config_network_manager(eth_interface, net_config);
#endif /* #if defined(HAVE_LIBNM) */
  else if (snc_use_config_method(config, SNC_METHOD_AUTO, method))
  {
    result = snc_get_net_config_rcconf(config->rcconf_file, eth_interface, net_config);
#if defined(HAVE_LIBNM)
    if (sncr_success != result)
      result = snc_get_net_config_network_manager(eth_interface, net_config);
#endif /* #if defined(HAVE_LIBNM) */
  }
  else
  {
    result = sncr_paramerror;
  }

  return result;
}


/**
 * Set the network configuration by the specified method
 *
 * @param config is the library configuration structure
 * @param method is one of the SNC_METHOD_* strings
 * @param eth_interface is the interface to set the configuration for. Can be NULL if setting hostname or domain only
 * @param net_config is the network configuration values to set
 * @param set_ifacecfg is a pointer to a bool that should be set to true if you want the interface configuration to be set.
 *          The function will set/clear this depending on if the interface configuration has actually changed.
 * @param set_hostname is a pointer to a bool that should be set to true if you want the hostname to be set.
 *          The function will set/clear this depending on if the hostname has actually changed.
 * @param set_domain is a pointer to a bool that should be set to true if you want the domain to be set.
 *          The function will set/clear this depending on if the domain has actually changed.
 */
enum e_snc_result snc_set_net_config(const struct s_snc_configuration *config,
                                     const char *method, const char *eth_interface,
                                     const struct s_snc_net_config *net_config,
                                     bool *set_ifacecfg, bool *set_hostname, bool *set_domain)
{
  enum e_snc_result result;

  if (snc_use_config_method(config, SNC_METHOD_RCCONF, method))
    result = snc_set_net_config_rcconf(config, eth_interface, net_config, set_ifacecfg, set_hostname, set_domain);
#if defined(HAVE_LIBNM)
  else if (snc_use_config_method(config, SNC_METHOD_NETWORK_MANAGER, method))
    result = snc_set_net_config_network_manager(eth_interface, net_config, set_ifacecfg, set_hostname, set_domain);
#endif /* #if defined(HAVE_LIBNM) */
  else if (snc_use_config_method(config, SNC_METHOD_AUTO, method))
  {
    result = snc_set_net_config_rcconf(config, eth_interface, net_config, set_ifacecfg, set_hostname, set_domain);
#if defined(HAVE_LIBNM)
    if (sncr_success != result)
      result = snc_set_net_config_network_manager(eth_interface, net_config, set_ifacecfg, set_hostname, set_domain);
#endif /* #if defined(HAVE_LIBNM) */
  }
  else
  {
    result = sncr_paramerror;
  }
  return result;
}
