/*
 * lsnc_netcurrent.c
 *
 *  Created on: 6 Jun. 2018
 *      Author: ian
 */
#include "config.h"
#include <stdbool.h>
#include <string.h>

#include <setnetconfig.h>


/** Should method be used */
static bool snc_use_current_config_method(const struct s_snc_configuration *config, const char *this_method, const char *method)
{
  return (strcasecmp(this_method, method) == 0 ||
          (strcasecmp(SNC_METHOD_LIB_CONFIG, method) == 0 &&
           strcasecmp(this_method, config->current_method) == 0));
}


/** Get the currently active network configuration using specified method (see SNC_METHOD_* constants) */
enum e_snc_result snc_get_net_current(const struct s_snc_configuration *config,
                                      const char *method, const char *eth_interface,
                                      struct s_snc_net_config *net_current)
{
  enum e_snc_result result;

  if (snc_use_current_config_method(config, SNC_METHOD_IOCTL, method))
    result = snc_get_net_current_ioctl(config, eth_interface, net_current);
#ifdef HAVE_IFADDRS_H
  else if (snc_use_current_config_method(config, SNC_METHOD_GETIFADDRS, method))
    result = snc_get_net_current_getifaddrs(config, eth_interface, net_current);
#endif /* #ifdef HAVE_IFADDRS_H */
  else if (snc_use_current_config_method(config, SNC_METHOD_IFCONFIG, method))
    result = snc_get_net_current_ifconfig(config, eth_interface, net_current);
  else if (snc_use_current_config_method(config, SNC_METHOD_IPCMD, method))
    result = snc_get_net_current_ip_command(config, eth_interface, net_current);
#if defined(HAVE_LIBNM)
  else if (snc_use_current_config_method(config, SNC_METHOD_NETWORK_MANAGER, method))
    result = snc_get_net_current_network_manager(eth_interface, net_current);
#endif /* #if defined(HAVE_LIBNM) */
  else if (snc_use_current_config_method(config, SNC_METHOD_AUTO, method))
  {
    /* Try all options, until success */
    result = snc_get_net_current_ioctl(config, eth_interface, net_current);
#ifdef HAVE_IFADDRS_H
    if (sncr_success != result)
      result = snc_get_net_current_getifaddrs(config, eth_interface, net_current);
#endif /* #ifdef HAVE_IFADDRS_H */
    if (sncr_success != result)
      result = snc_get_net_current_ifconfig(config, eth_interface, net_current);
    if (sncr_success != result)
      result = snc_get_net_current_ip_command(config, eth_interface, net_current);
#if defined(HAVE_LIBNM)
    if (sncr_success != result)
      result = snc_get_net_current_network_manager(eth_interface, net_current);
#endif /* #if defined(HAVE_LIBNM) */
  }
  else
  {
    result = sncr_paramerror;
  }

  return result;
}
