/*
 * lsnc_network_manager.c
 *
 *  Created on: 5 Jun. 2018
 *      Author: ian
 */
#include <arpa/inet.h>
#include <string.h>
#include <glib.h>
#include <NetworkManager.h>
#include "lsnc_parse_addr.h"

#include <setnetconfig.h>
#include "lsnc_network_manager.h"


/*
 * nmcli device show ens33
 * GENERAL.DEVICE:                         ens33
 * GENERAL.TYPE:                           ethernet
 * GENERAL.HWADDR:                         00:0C:29:48:A4:FA
 * GENERAL.MTU:                            1500
 * GENERAL.STATE:                          100 (connected)
 * GENERAL.CONNECTION:                     Wired connection 1
 * GENERAL.CON-PATH:                       /org/freedesktop/NetworkManager/ActiveConnection/1
 * WIRED-PROPERTIES.CARRIER:               on
 * IP4.ADDRESS[1]:                         172.17.1.60/24
 * IP4.GATEWAY:                            172.17.1.1
 * IP4.ROUTE[1]:                           dst = 0.0.0.0/0, nh = 172.17.1.1, mt = 100
 * IP4.ROUTE[2]:                           dst = 172.17.1.0/24, nh = 0.0.0.0, mt = 100
 * IP4.ROUTE[3]:                           dst = 169.254.0.0/16, nh = 0.0.0.0, mt = 1000
 * IP4.DNS[1]:                             172.17.1.2
 * IP4.DOMAIN[1]:                          rinstrumau.local
 * IP6.ADDRESS[1]:                         fe80::f13d:10a5:4902:3754/64
 * IP6.GATEWAY:                            --
 * IP6.ROUTE[1]:                           dst = ff00::/8, nh = ::, mt = 256, table=255
 * IP6.ROUTE[2]:                           dst = fe80::/64, nh = ::, mt = 256
 * IP6.ROUTE[3]:                           dst = fe80::/64, nh = ::, mt = 100
 *
 * nmcli general hostname
 * craptop1604
 *
 * nmcli connection show "Wired connection 1"
 * connection.id:                          Wired connection 1
 * connection.uuid:                        e65b844b-42c1-3bb2-a989-64fb03214234
 * connection.stable-id:                   --
 * connection.type:                        802-3-ethernet
 * connection.interface-name:              --
 * connection.autoconnect:                 yes
 * connection.autoconnect-priority:        -999
 * connection.autoconnect-retries:         -1 (default)
 * connection.auth-retries:                -1
 * connection.timestamp:                   1528945965
 * connection.read-only:                   no
 * connection.permissions:                 --
 * connection.zone:                        --
 * connection.master:                      --
 * connection.slave-type:                  --
 * connection.autoconnect-slaves:          -1 (default)
 * connection.secondaries:                 --
 * connection.gateway-ping-timeout:        0
 * connection.metered:                     unknown
 * connection.lldp:                        default
 * 802-3-ethernet.port:                    --
 * 802-3-ethernet.speed:                   0
 * 802-3-ethernet.duplex:                  --
 * 802-3-ethernet.auto-negotiate:          no
 * 802-3-ethernet.mac-address:             00:0C:29:48:A4:FA
 * 802-3-ethernet.cloned-mac-address:      --
 * 802-3-ethernet.generate-mac-address-mask:--
 * 802-3-ethernet.mac-address-blacklist:   --
 * 802-3-ethernet.mtu:                     auto
 * 802-3-ethernet.s390-subchannels:        --
 * 802-3-ethernet.s390-nettype:            --
 * 802-3-ethernet.s390-options:            --
 * 802-3-ethernet.wake-on-lan:             default
 * 802-3-ethernet.wake-on-lan-password:    --
 * ipv4.method:                            auto
 * ipv4.dns:                               --
 * ipv4.dns-search:                        --
 * ipv4.dns-options:                       ""
 * ipv4.dns-priority:                      0
 * ipv4.addresses:                         --
 * ipv4.gateway:                           --
 * ipv4.routes:                            --
 * ipv4.route-metric:                      -1
 * ipv4.route-table:                       0 (unspec)
 * ipv4.ignore-auto-routes:                no
 * ipv4.ignore-auto-dns:                   no
 * ipv4.dhcp-client-id:                    --
 * ipv4.dhcp-timeout:                      0 (default)
 * ipv4.dhcp-send-hostname:                yes
 * ipv4.dhcp-hostname:                     --
 * ipv4.dhcp-fqdn:                         --
 * ipv4.never-default:                     no
 * ipv4.may-fail:                          yes
 * ipv4.dad-timeout:                       -1 (default)
 * ipv6.method:                            auto
 * ipv6.dns:                               --
 * ipv6.dns-search:                        --
 * ipv6.dns-options:                       ""
 * ipv6.dns-priority:                      0
 * ipv6.addresses:                         --
 * ipv6.gateway:                           --
 * ipv6.routes:                            --
 * ipv6.route-metric:                      -1
 * ipv6.route-table:                       0 (unspec)
 * ipv6.ignore-auto-routes:                no
 * ipv6.ignore-auto-dns:                   no
 * ipv6.never-default:                     no
 * ipv6.may-fail:                          yes
 * ipv6.ip6-privacy:                       -1 (unknown)
 * ipv6.addr-gen-mode:                     stable-privacy
 * ipv6.dhcp-send-hostname:                yes
 * ipv6.dhcp-hostname:                     --
 * ipv6.token:                             --
 * proxy.method:                           none
 * proxy.browser-only:                     no
 * proxy.pac-url:                          --
 * proxy.pac-script:                       --
 * GENERAL.NAME:                           Wired connection 1
 * GENERAL.UUID:                           e65b844b-42c1-3bb2-a989-64fb03214234
 * GENERAL.DEVICES:                        ens33
 * GENERAL.STATE:                          activated
 * GENERAL.DEFAULT:                        yes
 * GENERAL.DEFAULT6:                       no
 * GENERAL.SPEC-OBJECT:                    --
 * GENERAL.VPN:                            no
 * GENERAL.DBUS-PATH:                      /org/freedesktop/NetworkManager/ActiveConnection/1
 * GENERAL.CON-PATH:                       /org/freedesktop/NetworkManager/Settings/3
 * GENERAL.ZONE:                           --
 * GENERAL.MASTER-PATH:                    --
 * IP4.ADDRESS[1]:                         172.17.1.60/24
 * IP4.GATEWAY:                            172.17.1.1
 * IP4.ROUTE[1]:                           dst = 0.0.0.0/0, nh = 172.17.1.1, mt = 100
 * IP4.ROUTE[2]:                           dst = 172.17.1.0/24, nh = 0.0.0.0, mt = 100
 * IP4.ROUTE[3]:                           dst = 169.254.0.0/16, nh = 0.0.0.0, mt = 1000
 * IP4.DNS[1]:                             172.17.1.2
 * IP4.DOMAIN[1]:                          rinstrumau.local
 * DHCP4.OPTION[1]:                        domain_name = rinstrumau.local
 * DHCP4.OPTION[2]:                        requested_domain_search = 1
 * DHCP4.OPTION[3]:                        requested_broadcast_address = 1
 * DHCP4.OPTION[4]:                        requested_time_offset = 1
 * DHCP4.OPTION[5]:                        dhcp_rebinding_time = 3150
 * DHCP4.OPTION[6]:                        requested_rfc3442_classless_static_routes = 1
 * DHCP4.OPTION[7]:                        requested_domain_name = 1
 * DHCP4.OPTION[8]:                        requested_netbios_scope = 1
 * DHCP4.OPTION[9]:                        expiry = 1528949229
 * DHCP4.OPTION[10]:                       next_server = 0.0.0.0
 * DHCP4.OPTION[11]:                       broadcast_address = 172.17.1.255
 * DHCP4.OPTION[12]:                       requested_interface_mtu = 1
 * DHCP4.OPTION[13]:                       dhcp_message_type = 5
 * DHCP4.OPTION[14]:                       dhcp_lease_time = 3600
 * DHCP4.OPTION[15]:                       ip_address = 172.17.1.60
 * DHCP4.OPTION[16]:                       requested_subnet_mask = 1
 * DHCP4.OPTION[17]:                       subnet_mask = 255.255.255.0
 * DHCP4.OPTION[18]:                       requested_ntp_servers = 1
 * DHCP4.OPTION[19]:                       dhcp_renewal_time = 1800
 * DHCP4.OPTION[20]:                       requested_domain_name_servers = 1
 * DHCP4.OPTION[21]:                       requested_wpad = 1
 * DHCP4.OPTION[22]:                       domain_name_servers = 172.17.1.2
 * DHCP4.OPTION[23]:                       requested_netbios_name_servers = 1
 * DHCP4.OPTION[24]:                       routers = 172.17.1.1
 * DHCP4.OPTION[25]:                       requested_ms_classless_static_routes = 1
 * DHCP4.OPTION[26]:                       requested_routers = 1
 * DHCP4.OPTION[27]:                       requested_static_routes = 1
 * DHCP4.OPTION[28]:                       network_number = 172.17.1.0
 * DHCP4.OPTION[29]:                       requested_host_name = 1
 * DHCP4.OPTION[30]:                       dhcp_server_identifier = 172.17.1.2
 * IP6.ADDRESS[1]:                         fe80::f13d:10a5:4902:3754/64
 * IP6.GATEWAY:                            --
 * IP6.ROUTE[1]:                           dst = ff00::/8, nh = ::, mt = 256, table=255
 * IP6.ROUTE[2]:                           dst = fe80::/64, nh = ::, mt = 256
 * IP6.ROUTE[3]:                           dst = fe80::/64, nh = ::, mt = 100
 *
 * https://github.com/NetworkManager/NetworkManager/blob/644aa42f68d9d6f30144dba243f95690226a777c/clients/cli/devices.c#L1557
 * https://cgit.freedesktop.org/NetworkManager/NetworkManager/tree/examples
 *
 * https://wiki.ubuntu.com/DebuggingDBus
 *
 * Restarting NetworkManager to apply network configuration is wrong.
 * Instead, reactive the connection profile via
 *   nmcli connection up
 *   nmcli device connect
 *   nmcli device reapply
 *   nmcli device modify
 * Also see activate_connection() in nmtui-connect.c?
 */


/** Get the hostname from network manager via libnm */
static enum e_snc_result snc_network_manager_get_hostname(NMClient *nmcli, struct s_snc_net_config *net_config)
{
  char *hostname = NULL;
  enum e_snc_result result;

  g_object_get(nmcli, NM_CLIENT_HOSTNAME, &hostname, NULL);
  if (NULL == hostname || snc_hostname_validate(hostname) != sncr_success)
  {
    hostname = g_strdup(SNC_HOSTNAME_DEFAULT);
  }

  snprintf(net_config->hostname, SNC_HOST_NAME_MAX, "%s", hostname);
  result = sncr_success;

  g_free(hostname);
  return result;
}


/** Get the domain name from network manager via libnm (we only support one, Network manager supports many) */
static enum e_snc_result snc_network_manager_get_domainname(NMClient *nmcli, struct s_snc_net_config *net_config)
{
  enum e_snc_result result;
  NMDnsEntry *nmdns;
  const GPtrArray *p;
  size_t i;
  const char *const *domains;

  p = nm_client_get_dns_configuration(nmcli);
  for (i = 0, result = sncr_parseerror; NULL != p && i < p->len; i++)
  {
    nmdns = p->pdata[i];
    domains = nm_dns_entry_get_domains(nmdns);
    if (NULL != domains && NULL != domains[0])
    {
      snprintf(net_config->domain, SNC_DOMAIN_NAME_MAX, "%s", domains[0]);
      result = sncr_success;
      break;
    }
  }

  if (sncr_success != result)
  {
    snprintf(net_config->domain, SNC_DOMAIN_NAME_MAX, "%s", SNC_DOMAINNAME_DEFAULT);
  }
  return result;
}


/** TODO Set the domain name from network manager via libnm (we only support one, Network manager supports many) */
static enum e_snc_result snc_network_manager_set_domainname(NMClient *nmcli, const struct s_snc_net_config *net_config)
{
  /* I'm not sure how to do this network manager.
   * We could make a new one using
     NMDnsEntry *nm_dns_entry_new (const char *interface,
                                   const char * const *nameservers,
                                   const char * const *domains,
                                   int priority,
                                   gboolean vpn)
   * with modified domains value.
   * But then I'm not sure what to do with it...
   */
  return sncr_success;
}


/** Get a network manager client and device for eth_interface */
static enum e_snc_result snc_network_manager_get_client_device(const char *eth_interface, NMClient **nmclient, NMDevice **nmdevice)
{
  /* Create a network manager client */
  *nmclient = nm_client_new(NULL, NULL);
  if (NULL == *nmclient)
    return sncr_syscallerror;

  /* Get the device for this interface */
  if (NULL != eth_interface)
  {
    *nmdevice = nm_client_get_device_by_iface(*nmclient, eth_interface);
    if (NULL == *nmdevice)
      return sncr_syscallerror;
  }
  else
  {
    *nmdevice = NULL;
  }

  return sncr_success;
}


/** Get a network manager client, device and connection */
static enum e_snc_result snc_network_manager_get_client_device_connection(const char *eth_interface, NMClient **nmclient, NMDevice **nmdevice, NMConnection **nmconnection)
{
  enum e_snc_result result;
  NMConnection *conn;
  const char *uuid;

  /* Get client and device */
  result = snc_network_manager_get_client_device(eth_interface, nmclient, nmdevice);
  if (sncr_success != result)
    return result;

  /* Get the connection applied to this device */
  if (NULL != *nmdevice)
  {
    conn = nm_device_get_applied_connection(*nmdevice, 0, NULL, NULL, NULL);
    if (NULL == conn)
      return sncr_syscallerror;

    /*
     * Use the UUID of the connection returned by the device its to get the connection from the client's list of connections
     *
     * For some reason, using the connection returned by nm_device_get_applied_connection() doesn't update when the settings change
     */
    uuid = nm_connection_get_uuid(conn);
    *nmconnection = NM_CONNECTION(nm_client_get_connection_by_uuid(*nmclient, uuid));
  }
  else
  {
    *nmconnection = NULL;
  }

  return sncr_success;
}


/** Get a network manager client, device and connection */
static enum e_snc_result snc_network_manager_is_connection_dhcp(NMConnection *nmconnection, bool *is_dhcp)
{
  NMSettingIPConfig *setting_ipv4_config;

  /* Get the settings for this connection */
  setting_ipv4_config = nm_connection_get_setting_ip4_config(nmconnection);
  if (NULL == setting_ipv4_config)
    return sncr_syscallerror;

  /* DHCP setting */
  *is_dhcp = (strcmp(nm_setting_ip_config_get_method(setting_ipv4_config), NM_SETTING_IP4_CONFIG_METHOD_AUTO) == 0);
  return sncr_success;
}


/** Get the current network settings from network manager via libnm */
enum e_snc_result snc_get_net_current_network_manager(const char *eth_interface, struct s_snc_net_config *net_current)
{
  NMClient *nmclient = NULL;
  NMDevice *nmdevice;
  NMConnection *nmconnection;
  NMIPConfig *nmipconfig4;
  GPtrArray *ptr_array;
  NMIPAddress *addr;
  int prefix;
  enum e_snc_result result;
  const char *const *nameservers;
  const char *const *domains;
  size_t i;
  const char *ipstr;

  if (NULL == eth_interface || NULL == net_current)
    return sncr_paramerror;

  /* Set some initial values */
  result = snc_net_config_struct_init(net_current);
  if (sncr_success != result)
    return result;

  /* Get client, device and connection (connection needed for use_dhcp) */
  result = snc_network_manager_get_client_device_connection(eth_interface, &nmclient, &nmdevice, &nmconnection);
  if (sncr_success != result)
    goto sgncnm_fail;

  /* DHCP used on this connection */
  result = snc_network_manager_is_connection_dhcp(nmconnection, &net_current->use_dhcp);
  if (sncr_success != result)
    goto sgncnm_fail;

  /* Hostname comes straight from the client */
  result = snc_network_manager_get_hostname(nmclient, net_current);
  if (sncr_success != result)
    goto sgncnm_fail;

  /* Get the IPv4 configuration for the device */
  nmipconfig4 = nm_device_get_ip4_config(nmdevice);
  if (NULL == nmipconfig4)
  {
    result = sncr_syscallerror;
    goto sgncnm_fail;
  }

  /* Get a list of addresses for this interface device */
  ptr_array = nm_ip_config_get_addresses(nmipconfig4);
  if (NULL == ptr_array || ptr_array->len <= 0)
  {
    /* No IP address configured for this interface, set it to 0 */
    net_current->ip_address.s_addr = SNC_IPV4_UNSET_INADDR_NETWORK;
    net_current->ip_netmask.s_addr = SNC_IPV4_UNSET_INADDR_NETWORK;
    net_current->ip_broadcast.s_addr = SNC_IPV4_UNSET_INADDR_NETWORK;
  }
  else
  {
    /* We'll just take the first one */
    addr = g_ptr_array_index(ptr_array, 0);

    /* Get net mask */
    prefix = nm_ip_address_get_prefix(addr);
    result = snc_ipv4_prefix_to_netmask(prefix, &net_current->ip_netmask);
    if (sncr_success != result)
      goto sgncnm_fail;

    /* Get IP address */
    nm_ip_address_get_address_binary(addr, &net_current->ip_address);

    /* Make up the broadcast address from IP and prefix */
    result = snc_ipv4_broadcast_from_ip_netmask(&net_current->ip_address, &net_current->ip_netmask, &net_current->ip_broadcast);
    if (sncr_success != result)
      goto sgncnm_fail;
  }

  /* Name servers */
  i = 0;
  nameservers = nm_ip_config_get_nameservers(nmipconfig4);
  if (NULL != nameservers)
  {
    for (i = 0; i < SNC_NUM_DNS_SERVERS && NULL != *nameservers; i++, nameservers++)
    {
      inet_pton(AF_INET, *nameservers, &net_current->ip_dns[i]);
    }
  }
  for ( ; i < SNC_NUM_DNS_SERVERS; i++)
    net_current->ip_dns[i].s_addr = SNC_IPV4_UNSET_INADDR_NETWORK;

  /* Domain */
  domains = nm_ip_config_get_domains(nmipconfig4);
  if (NULL != domains && NULL != domains[0])
    snprintf(net_current->domain, SNC_DOMAIN_NAME_MAX, "%s", domains[0]);
  else
    snprintf(net_current->domain, SNC_DOMAIN_NAME_MAX, "%s", SNC_DOMAINNAME_DEFAULT);

  /* Search domains */
  domains = nm_ip_config_get_searches(nmipconfig4);
  if (NULL != domains && NULL != domains[0])
    snprintf(net_current->search_domain, SNC_DOMAIN_NAME_MAX, "%s", domains[0]);
  else
    snprintf(net_current->search_domain, SNC_DOMAIN_NAME_MAX, "%s", SNC_DOMAINNAME_DEFAULT);

  /* Default gateway */
  ipstr = nm_ip_config_get_gateway(nmipconfig4);
  inet_pton(AF_INET, ipstr, &net_current->ip_gateway);

  /* MAC Address */
  ipstr = nm_device_get_hw_address(nmdevice);
  result = snc_parse_mac_from_line(NULL, false, ipstr, &net_current->mac_address);
  if (sncr_success != result)
    goto sgncnm_fail;

  /* Success */
  result = sncr_success;

sgncnm_fail:
  g_clear_object(&nmclient);
  return sncr_success;
}


/** Internal get the network configuration from network manager via libnm */
static enum e_snc_result snc_get_net_config_network_manager_internal(NMClient *nmcli, NMConnection *nmconnection, struct s_snc_net_config *net_config)
{
  enum e_snc_result result;
  NMIPAddress *ipaddr;
  int prefix;
  NMSettingIPConfig *setting_ipv4_config;
  size_t i;
  size_t num;
  const char *ipstr;

  /* Set some initial values */
  result = snc_net_config_struct_init(net_config);
  if (sncr_success != result)
    goto sgncnmi_fail;

  if (NULL != nmconnection)
  {
    /* Get the settings for this connection */
    setting_ipv4_config = nm_connection_get_setting_ip4_config(nmconnection);
    if (NULL == setting_ipv4_config)
    {
      result = sncr_syscallerror;
      goto sgncnmi_fail;
    }

    /* DHCP used on this connection */
    result = snc_network_manager_is_connection_dhcp(nmconnection, &net_config->use_dhcp);
    if (sncr_success != result)
      goto sgncnmi_fail;
    if (net_config->use_dhcp)
    {
      /* All IP values unset for DHCP, set use_dhcp after snc_net_config_struct_init() cleared it */
      (void) snc_net_config_struct_init(net_config);
      net_config->use_dhcp = true;
    }
    else
    {
      /* Static IP configuration, retrieve values from network manager */
      /* net_config->use_dhcp = false; */

      /* IP address and netmask */
      if (nm_setting_ip_config_get_num_addresses(setting_ipv4_config) < 1)
      {
        net_config->ip_address.s_addr = SNC_IPV4_UNSET_INADDR_NETWORK;
      }
      else
      {
        ipaddr = nm_setting_ip_config_get_address(setting_ipv4_config, 0);

        /* Get net mask */
        prefix = nm_ip_address_get_prefix(ipaddr);
        result = snc_ipv4_prefix_to_netmask(prefix, &net_config->ip_netmask);
        if (sncr_success != result)
          goto sgncnmi_fail;

        /* Get IP address */
        nm_ip_address_get_address_binary(ipaddr, &net_config->ip_address);

        /* Make up the broadcast address from IP and prefix */
        result = snc_ipv4_broadcast_from_ip_netmask(&net_config->ip_address, &net_config->ip_netmask, &net_config->ip_broadcast);
        if (sncr_success != result)
          goto sgncnmi_fail;
      }

      /* Default gateway */
      ipstr = nm_setting_ip_config_get_gateway(setting_ipv4_config);
      if (NULL != ipstr)
        inet_pton(AF_INET, ipstr, &net_config->ip_gateway);
      else
        net_config->ip_gateway.s_addr = SNC_IPV4_UNSET_INADDR_NETWORK;

      /* DNS servers */
      num = nm_setting_ip_config_get_num_dns(setting_ipv4_config);
      for (i = 0; i < num && i < SNC_NUM_DNS_SERVERS; i++)
      {
        ipstr = nm_setting_ip_config_get_dns(setting_ipv4_config, i);
        inet_pton(AF_INET, ipstr, &net_config->ip_dns[i]);
      }
      for ( ; i < SNC_NUM_DNS_SERVERS; i++)
        net_config->ip_dns[i].s_addr = SNC_IPV4_UNSET_INADDR_NETWORK;
    }

    /* Search domain */
    num = nm_setting_ip_config_get_num_dns_searches(setting_ipv4_config);
    if (num >= 1)
      snprintf(net_config->search_domain, SNC_DOMAIN_NAME_MAX, "%s",
               nm_setting_ip_config_get_dns_search(setting_ipv4_config, 0));
    else
      snprintf(net_config->search_domain, SNC_DOMAIN_NAME_MAX, "%s", SNC_DOMAINNAME_DEFAULT);
  } /* if (NULL != nmconnection) */

  /* Hostname */
  result = snc_network_manager_get_hostname(nmcli, net_config);
  if (sncr_success != result)
    goto sgncnmi_fail;

  /* Domain name */
  result = snc_network_manager_get_domainname(nmcli, net_config);
  if (sncr_success != result)
    goto sgncnmi_fail;

  /* MAC address. Set this to 0, as it's not a configuration parameter */
  result = snc_zero_mac_address(&net_config->mac_address);
  if (sncr_success != result)
    goto sgncnmi_fail;

  /* Success */
  result = sncr_success;

sgncnmi_fail:
  return result;
}


/** Get the network configuration from network manager via libnm */
enum e_snc_result snc_get_net_config_network_manager(const char *eth_interface, struct s_snc_net_config *net_config)
{
  NMClient *nmcli = NULL;
  NMDevice *nmdevice = NULL;
  NMConnection *nmconnection = NULL;
  enum e_snc_result result;

  if (NULL == eth_interface || NULL == net_config)
    return sncr_paramerror;

  /* Get a network manager client, device and connection */
  result = snc_network_manager_get_client_device_connection(eth_interface, &nmcli, &nmdevice, &nmconnection);
  if (sncr_success != result)
    goto sgncnm_fail;
  /* nm_connection_dump(nmconnection); */

  /* Get the network configuration */
  result = snc_get_net_config_network_manager_internal(nmcli, nmconnection, net_config);
  if (sncr_success != result)
    goto sgncnm_fail;

sgncnm_fail:
  g_clear_object(&nmcli);
  return result;
}


/** Get the network configuration DHCP setting from network manager via libnm */
enum e_snc_result snc_get_net_config_dhcp_network_manager(const char *eth_interface, bool *use_dhcp)
{
  NMClient *nmcli = NULL;
  NMDevice *nmdevice = NULL;
  NMConnection *nmconnection = NULL;
  enum e_snc_result result;

  if (NULL == eth_interface || NULL == use_dhcp)
    return sncr_paramerror;

  /* Get a network manager client, device and connection */
  result = snc_network_manager_get_client_device_connection(eth_interface, &nmcli, &nmdevice, &nmconnection);
  if (sncr_success != result)
    goto sgncdnm_fail;

  /* DHCP used on this connection */
  result = snc_network_manager_is_connection_dhcp(nmconnection, use_dhcp);
  if (sncr_success != result)
    goto sgncdnm_fail;

sgncdnm_fail:
  g_clear_object(&nmcli);
  return result;
}


/**
 * Set the network configuration in network manager via libnm
 *
 * @param eth_interface is the interface to set the configuration for. Can be NULL if setting hostname or domain only
 * @param net_config is the network configuration values to set
 * @param set_ifacecfg is a pointer to a bool that should be set to true if you want the interface configuration to be set.
 *          The function will set/clear this depending on if the interface configuration has actually changed.
 * @param set_hostname is a pointer to a bool that should be set to true if you want the hostname to be set.
 *          The function will set/clear this depending on if the hostname has actually changed.
 * @param set_domain is a pointer to a bool that should be set to true if you want the domain to be set.
 *          The function will set/clear this depending on if the domain has actually changed.
 */
enum e_snc_result snc_set_net_config_network_manager(const char *eth_interface,
                                                     const struct s_snc_net_config *net_config,
                                                     bool *set_ifacecfg, bool *set_hostname, bool *set_domain)
{
  struct s_snc_net_config current_net_config;
  enum e_snc_result result;
  NMClient *nmclient = NULL;
  NMDevice *nmdevice = NULL;
  NMConnection *nmconnection = NULL;
  bool changed_dhcp;
  bool hostname_differs = false;
  bool domain_differs = false;
  bool ifacecfg_differs = false;
  bool should_set_hostname = false;
  bool should_set_domain = false;
  bool should_set_ifacecfg = false;
  NMSettingIPConfig *s_ip4;
  size_t i;
  char ipstr[SNC_INET_ADDRSTRLEN + 1];
  NMIPAddress *ipv4_addr;
  unsigned ipv4_prefix;

  if (NULL == net_config || NULL == set_ifacecfg || NULL == set_hostname || NULL == set_domain)
    return sncr_paramerror;
  if (*set_ifacecfg && NULL == eth_interface)
    return sncr_paramerror;

  /* Get a network manager client, device and connection */
  result = snc_network_manager_get_client_device_connection(eth_interface, &nmclient, &nmdevice, &nmconnection);
  if (sncr_success != result)
    goto ssncnm_fail;
#if 0
  if (NULL != nmconnection)
    nm_connection_dump(nmconnection);
#endif

  /* Get the current network configuration, so we've got something to compare against */
  result = snc_get_net_config_network_manager_internal(nmclient, nmconnection, &current_net_config);
  if (sncr_success != result)
    goto ssncnm_fail;

#if 0
  fprintf(stdout, "Setting: %s%s%s\n",
          (*set_ifacecfg) ? "interface configuration " : "",
          (*set_hostname) ? "hostname " : "",
          (*set_domain) ? "domain " : "");
  fprintf(stdout, "Current network configuration\n");
  (void) snc_print_net_config(stdout, &current_net_config);
  fprintf(stdout, "New network configuration\n");
  (void) snc_print_net_config(stdout, net_config);
#endif

  /* Work out what's changed, and what we should set */
  result = snc_net_config_diff(net_config, &current_net_config, &ifacecfg_differs, &hostname_differs, &domain_differs, NULL);
  should_set_hostname = hostname_differs && *set_hostname;
  should_set_domain = domain_differs && *set_domain;
  should_set_ifacecfg = ifacecfg_differs && *set_ifacecfg;
  changed_dhcp = net_config->use_dhcp != current_net_config.use_dhcp;

  /* Set hostname if changed */
  if (should_set_hostname)
  {
    if (!nm_client_save_hostname(nmclient, net_config->hostname, NULL, NULL))
    {
      result = sncr_syscallerror;
      goto ssncnm_fail;
    }
  }

  /* Set domainname if changed */
  if (should_set_domain)
  {
    result = snc_network_manager_set_domainname(nmclient, net_config);
    if (sncr_success != result)
      goto ssncnm_fail;
  }

  /* Set interface configuration if something has changed */
  if (should_set_ifacecfg)
  {
    if (NULL == nmdevice || NULL == nmconnection)
    {
      result = sncr_paramerror;
      goto ssncnm_fail;
    }

    s_ip4 = nm_connection_get_setting_ip4_config(nmconnection);
    if (NULL == s_ip4)
    {
      s_ip4 = (NMSettingIPConfig *) nm_setting_ip4_config_new();
      nm_connection_add_setting(nmconnection, NM_SETTING(s_ip4));
    }

    /* Set the method (auto/manual) based on DHCP setting */
    if (changed_dhcp)
    {
      g_object_set(s_ip4, NM_SETTING_IP_CONFIG_METHOD,
                   (net_config->use_dhcp) ? NM_SETTING_IP4_CONFIG_METHOD_AUTO : NM_SETTING_IP4_CONFIG_METHOD_MANUAL,
                   NULL);
      if (net_config->use_dhcp)
      {
        /* Remove all IPs, DNS servers, search domains and the gateway */
        nm_setting_ip_config_clear_addresses(s_ip4);
        nm_setting_ip_config_clear_dns(s_ip4);
        nm_setting_ip_config_clear_dns_searches(s_ip4);
        g_object_set(s_ip4, NM_SETTING_IP_CONFIG_GATEWAY, NULL, NULL);
      }
    }

    /* If we've not changed the DHCP setting, then we must have changed one of the static settings */
    if (!changed_dhcp && !net_config->use_dhcp)
    {
      /* Remove all IPs and add ours */
      nm_setting_ip_config_clear_addresses(s_ip4);
      snc_ipv4_netmask_to_prefix(&net_config->ip_netmask, &ipv4_prefix);
      ipv4_addr = nm_ip_address_new_binary(AF_INET, &net_config->ip_address, ipv4_prefix, NULL);
      nm_setting_ip_config_add_address(s_ip4, ipv4_addr);

      /* Remove all DNSs and add ours */
      nm_setting_ip_config_clear_dns(s_ip4);
      for (i = 0; i < SNC_NUM_DNS_SERVERS; i++)
      {
        if (net_config->ip_dns[i].s_addr != SNC_IPV4_UNSET_INADDR_NETWORK)
        {
          inet_ntop(AF_INET, &net_config->ip_dns[i], ipstr, SNC_INET_ADDRSTRLEN);
          nm_setting_ip_config_add_dns(s_ip4, ipstr);
        }
      }

      /* Remove all search domains and add ours */
      nm_setting_ip_config_clear_dns_searches(s_ip4);
      nm_setting_ip_config_add_dns_search(s_ip4, net_config->search_domain);

      /* Set gateway address */
      inet_ntop(AF_INET, &net_config->ip_gateway, ipstr, SNC_INET_ADDRSTRLEN);
      g_object_set(s_ip4, NM_SETTING_IP_CONFIG_GATEWAY, ipstr, NULL);
    }

    /* Verify the connection */
    if (!nm_connection_verify(nmconnection, NULL))
    {
      result = sncr_paramerror;
      goto ssncnm_fail;
    }

    /* Reapply the updated connection settings to the device */
    if (!nm_device_reapply(nmdevice, nmconnection, 0, 0, NULL, NULL))
    {
      result = sncr_syscallerror;
      goto ssncnm_fail;
    }

#if 0
    if (!nm_client_reload_connections(nmclient, NULL, NULL))
    {
      result = sncr_syscallerror;
      goto ssncnm_fail;
    }
#endif
  }

  /* Let caller know what was actually different and hence has been set */
  *set_hostname = should_set_hostname;
  *set_domain = should_set_domain;
  *set_ifacecfg = should_set_ifacecfg;

  /* Success */
  result = sncr_success;

ssncnm_fail:
  g_clear_object(&nmclient);
  return result;
}
