/*
 * snc_validate.c
 *
 *  Created on: 25 Jul. 2018
 *      Author: ian
 */
#include <stdbool.h>
#include <stdio.h>
#include <setnetconfig.h>

#include "snc_validate.h"


/**
 * Validate net_config and print out any
 *
 * @param program_name is the name of the program to print with error messages
 * @param net_config is the network configuration to validate
 * @param set_ifacecfg if set, then interface configuration members are validated
 * @param set_hostname if set, then hostname is validated
 * @param set_domain if set, then domain is validated
 *
 * @return result of the validation
 */
enum e_snc_result validate_and_print_errors(const char *program_name, const struct s_snc_net_config *net_config,
                                            bool set_ifacecfg, bool set_hostname, bool set_domain)
{
  enum e_snc_result result;
  uint32_t valid_mask;
  size_t i;

  /* Verify the new configuration */
  (void) snc_net_config_validate(net_config, &valid_mask, false);
  result = sncr_success;
  if (set_hostname && (valid_mask & SNC_VALIDATE_HOSTNAME_INVALID) != 0)
  {
    fprintf(stderr, "%s: invalid hostname \"%s\"\n", program_name, net_config->hostname);
    result = sncr_parseerror;
  }
  if (set_domain && (valid_mask & SNC_VALIDATE_DOMAIN_INVALID) != 0)
  {
    fprintf(stderr, "%s: invalid domain name \"%s\"\n", program_name, net_config->domain);
    result = sncr_parseerror;
  }
  if (set_ifacecfg)
  {
    if ((valid_mask & SNC_VALIDATE_IPV4_ADDRESS_INVALID) != 0)
    {
      fprintf(stderr, "%s: invalid ip address\n", program_name);
      result = sncr_parseerror;
    }
    if ((valid_mask & SNC_VALIDATE_IPV4_NETMASK_INVALID) != 0)
    {
      fprintf(stderr, "%s: invalid netmask\n", program_name);
      result = sncr_parseerror;
    }
    if ((valid_mask & SNC_VALIDATE_IPV4_BROADCAST_INVALID) != 0)
    {
      fprintf(stderr, "%s: invalid broadcast address\n", program_name);
      result = sncr_parseerror;
    }
    if ((valid_mask & SNC_VALIDATE_IPV4_GATEWAY_INVALID) != 0)
    {
      fprintf(stderr, "%s: invalid gateway address\n", program_name);
      result = sncr_parseerror;
    }
    for (i = 0; i < SNC_NUM_DNS_SERVERS; i++)
    {
      if ((valid_mask & (SNC_VALIDATE_IPV4_DNS0_INVALID << i)) != 0)
      {
        fprintf(stderr, "%s: invalid dns server %u\n", program_name, (unsigned) i);
        result = sncr_parseerror;
      }
    }
    if ((valid_mask & SNC_VALIDATE_SEARCHDOMAIN_INVALID) != 0)
    {
      fprintf(stderr, "%s: invalid search domain \"%s\"\n", program_name, net_config->search_domain);
      result = sncr_parseerror;
    }
  }
  return result;
}
