/*
 * lsnc_test_ipv4_validate.c
 *
 *  Created on: 20 Jul. 2018
 *      Author: ian
 */
#include <CuTest.h>
#include <setnetconfig.h>

#include "lsnc_test_ipv4_validate.h"


/** Test snc_ipv4_netmask_validate() */
static void test_snc_ipv4_netmask_validate(CuTest *tc)
{
  enum e_snc_result result;
  struct in_addr ipv4_netmask;

  /* Test an everyday netmask */
  ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 0);
  result = snc_ipv4_netmask_validate(&ipv4_netmask);
  CuAssertIntEquals(tc, sncr_success, result);

  /* Test all 0s netmask */
  ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(0, 0, 0, 0);
  result = snc_ipv4_netmask_validate(&ipv4_netmask);
  CuAssertIntEquals(tc, sncr_success, result);

  /* Test all 1s netmask */
  ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 255);
  result = snc_ipv4_netmask_validate(&ipv4_netmask);
  CuAssertIntEquals(tc, sncr_success, result);

  /* Test first 1 only */
  ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(128, 0, 0, 0);
  result = snc_ipv4_netmask_validate(&ipv4_netmask);
  CuAssertIntEquals(tc, sncr_success, result);

  /* Test last 0 only */
  ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 254);
  result = snc_ipv4_netmask_validate(&ipv4_netmask);
  CuAssertIntEquals(tc, sncr_success, result);

  /* Test last 11110111 only */
  ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 0xF7);
  result = snc_ipv4_netmask_validate(&ipv4_netmask);
  CuAssertIntEquals(tc, sncr_parseerror, result);
}


/** Test snc_ipv4_single_destination_address_validate() */
static void test_snc_ipv4_single_destination_address_validate(CuTest *tc)
{
  enum e_snc_result result;
  struct in_addr ipv4_dst;
  struct in_addr ipv4_src;
  struct in_addr ipv4_src_nm;

  /* Test an everyday destination address */
  ipv4_dst.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(172, 17, 5, 100);
  result = snc_ipv4_single_destination_address_validate(&ipv4_dst, NULL, NULL);
  CuAssertIntEquals(tc, sncr_success, result);

  /* Test classes A-C */
  ipv4_dst.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(1, 0, 0, 1);
  result = snc_ipv4_single_destination_address_validate(&ipv4_dst, NULL, NULL);
  CuAssertIntEquals(tc, sncr_success, result);
  ipv4_dst.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(223, 255, 255, 254);
  result = snc_ipv4_single_destination_address_validate(&ipv4_dst, NULL, NULL);
  CuAssertIntEquals(tc, sncr_success, result);

  /* Test lower limits of class D */
  ipv4_dst.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(223, 255, 255, 254);
  result = snc_ipv4_single_destination_address_validate(&ipv4_dst, NULL, NULL);
  CuAssertIntEquals(tc, sncr_success, result);
  ipv4_dst.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(224, 0, 0, 1);
  result = snc_ipv4_single_destination_address_validate(&ipv4_dst, NULL, NULL);
  CuAssertIntEquals(tc, sncr_parseerror, result);

  /* Test upper limits of class D */
  ipv4_dst.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(239, 255, 255, 254);
  result = snc_ipv4_single_destination_address_validate(&ipv4_dst, NULL, NULL);
  CuAssertIntEquals(tc, sncr_parseerror, result);

  /* Test lower limits of class E */
  ipv4_dst.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(240, 0, 0, 1);
  result = snc_ipv4_single_destination_address_validate(&ipv4_dst, NULL, NULL);
  CuAssertIntEquals(tc, sncr_parseerror, result);

  /* Test upper limits of class E */
  ipv4_dst.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 254);
  result = snc_ipv4_single_destination_address_validate(&ipv4_dst, NULL, NULL);
  CuAssertIntEquals(tc, sncr_parseerror, result);

  /* Test broadcast on same subnet */
  ipv4_dst.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(172, 17, 5, 100);
  ipv4_src.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(172, 17, 5, 1);
  ipv4_src_nm.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 0);
  result = snc_ipv4_single_destination_address_validate(&ipv4_dst, &ipv4_src, &ipv4_src_nm);
  CuAssertIntEquals(tc, sncr_success, result);
  ipv4_dst.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(172, 17, 5, 255);
  ipv4_src.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(172, 17, 5, 1);
  ipv4_src_nm.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 0);
  result = snc_ipv4_single_destination_address_validate(&ipv4_dst, &ipv4_src, &ipv4_src_nm);
  CuAssertIntEquals(tc, sncr_parseerror, result);
  ipv4_dst.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(172, 17, 5, 0);
  ipv4_src.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(172, 17, 5, 1);
  ipv4_src_nm.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 0);
  result = snc_ipv4_single_destination_address_validate(&ipv4_dst, &ipv4_src, &ipv4_src_nm);
  CuAssertIntEquals(tc, sncr_parseerror, result);

  /* Test /31 special case */
  ipv4_dst.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(172, 17, 5, 0);
  ipv4_src.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(172, 17, 5, 1);
  ipv4_src_nm.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 254);
  result = snc_ipv4_single_destination_address_validate(&ipv4_dst, &ipv4_src, &ipv4_src_nm);
  CuAssertIntEquals(tc, sncr_success, result);
  ipv4_dst.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(172, 17, 5, 1);
  ipv4_src.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(172, 17, 5, 0);
  ipv4_src_nm.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 254);
  result = snc_ipv4_single_destination_address_validate(&ipv4_dst, &ipv4_src, &ipv4_src_nm);
  CuAssertIntEquals(tc, sncr_success, result);
}


struct s_snc_validate_data
{
  struct in_addr ipv4_address;
  struct in_addr ipv4_netmask;
  struct in_addr ipv4_gateway;
  enum e_snc_result expected_result;
};


typedef void (*t_run_check_fn)(CuTest *tc, const struct s_snc_validate_data *vd);


static void run_snc_ipv4_host_address_validate(CuTest *tc, const struct s_snc_validate_data *svd)
{
  enum e_snc_result result;

  result = snc_ipv4_host_address_validate(&svd->ipv4_address, &svd->ipv4_netmask);
  CuAssertIntEquals(tc, svd->expected_result, result);
}


static void run_snc_ipv4_gateway_address_validate(CuTest *tc, const struct s_snc_validate_data *svd)
{
  enum e_snc_result result;
  struct s_snc_validate_data svd_copy;

  /* Simulate host on the same subnet as gateway */
  svd_copy = *svd;
  svd_copy.ipv4_gateway = svd->ipv4_address;

  result = snc_ipv4_gateway_address_validate(&svd_copy.ipv4_gateway, &svd_copy.ipv4_address, &svd_copy.ipv4_netmask);
  CuAssertIntEquals(tc, svd->expected_result, result);
}


/** Run common checks for a host address (snc_ipv4_host_address_validate()) */
static void check_snc_ipv4_host_address_validate(CuTest *tc, t_run_check_fn run_check_fn)
{
  struct s_snc_validate_data svd;

  /* Test an everyday IP address */
  svd.ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 1, 20);
  svd.ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 0);
  svd.expected_result = sncr_success;
  run_check_fn(tc, &svd);

  /* Test classes A-C */
  svd.ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(1, 0, 0, 1);
  svd.ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 0, 0, 0);
  svd.expected_result = sncr_success;
  run_check_fn(tc, &svd);
  svd.ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(223, 255, 255, 254);
  svd.ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 0, 0, 0);
  svd.expected_result = sncr_success;
  run_check_fn(tc, &svd);

  /* Test lower limits of class D */
  svd.ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(223, 255, 255, 254);
  svd.ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 0, 0, 0);
  svd.expected_result = sncr_success;
  run_check_fn(tc, &svd);
  svd.ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(224, 0, 0, 1);
  svd.ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 0, 0, 0);
  svd.expected_result = sncr_parseerror;
  run_check_fn(tc, &svd);

  /* Test upper limits of class D */
  svd.ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(239, 255, 255, 254);
  svd.ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 0, 0, 0);
  svd.expected_result = sncr_parseerror;
  run_check_fn(tc, &svd);

  /* Test lower limits of class E */
  svd.ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(240, 0, 0, 1);
  svd.ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 0, 0, 0);
  svd.expected_result = sncr_parseerror;
  run_check_fn(tc, &svd);

  /* Test upper limits of class E */
  svd.ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 254);
  svd.ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 0, 0, 0);
  svd.expected_result = sncr_parseerror;
  run_check_fn(tc, &svd);

  /* Test lower limits of loopback */
  svd.ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(127, 0, 0, 1);
  svd.ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 0, 0, 0);
  svd.expected_result = sncr_parseerror;
  run_check_fn(tc, &svd);

  /* Test upper limits of loopback */
  svd.ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(127, 255, 255, 254);
  svd.ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 0, 0, 0);
  svd.expected_result = sncr_parseerror;
  run_check_fn(tc, &svd);

  /* Test subnet 0 (old skool broadcast) */
  svd.ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(172, 17, 1, 0);
  svd.ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 0);
  svd.expected_result = sncr_parseerror;
  run_check_fn(tc, &svd);
  svd.ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(172, 17, 1, 1);
  svd.ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 0);
  svd.expected_result = sncr_success;
  run_check_fn(tc, &svd);

  /* Test subnet broadcast */
  svd.ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(172, 17, 1, 255);
  svd.ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 0);
  svd.expected_result = sncr_parseerror;
  run_check_fn(tc, &svd);
  svd.ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(172, 17, 1, 254);
  svd.ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 0);
  svd.expected_result = sncr_success;
  run_check_fn(tc, &svd);

  /* Test /31 special case */
  svd.ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(172, 17, 5, 0);
  svd.ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 254);
  svd.expected_result = sncr_success;
  run_check_fn(tc, &svd);
  svd.ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(172, 17, 5, 1);
  svd.ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 254);
  svd.expected_result = sncr_success;
  run_check_fn(tc, &svd);
}


/** Test snc_ipv4_host_address_validate() */
static void test_snc_ipv4_host_address_validate(CuTest *tc)
{
  check_snc_ipv4_host_address_validate(tc, run_snc_ipv4_host_address_validate);
}


/** Test snc_ipv4_loopback_address_validate() */
static void test_snc_ipv4_loopback_address_validate(CuTest *tc)
{
  enum e_snc_result result;
  struct in_addr ipv4_netmask;
  struct in_addr ipv4_address;

  /* Test an everyday loopback address */
  ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(127, 0, 0, 1);
  ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 0, 0, 0);
  result = snc_ipv4_loopback_address_validate(&ipv4_address, &ipv4_netmask);
  CuAssertIntEquals(tc, sncr_success, result);

  /* Test lower limit of loopback address range */
  ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(126, 255, 255, 254);
  ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 0, 0, 0);
  result = snc_ipv4_loopback_address_validate(&ipv4_address, &ipv4_netmask);
  CuAssertIntEquals(tc, sncr_parseerror, result);
  ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(127, 0, 0, 1);
  ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 0, 0, 0);
  result = snc_ipv4_loopback_address_validate(&ipv4_address, &ipv4_netmask);
  CuAssertIntEquals(tc, sncr_success, result);

  /* Test upper limit of loopback address range */
  ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(128, 0, 0, 1);
  ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 0, 0, 0);
  result = snc_ipv4_loopback_address_validate(&ipv4_address, &ipv4_netmask);
  CuAssertIntEquals(tc, sncr_parseerror, result);
  ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(127, 255, 255, 254);
  ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 0, 0, 0);
  result = snc_ipv4_loopback_address_validate(&ipv4_address, &ipv4_netmask);
  CuAssertIntEquals(tc, sncr_success, result);

  /* Test loopback subnet 0 (old skool broadcast) */
  ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(127, 0, 0, 0);
  ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 0, 0, 0);
  result = snc_ipv4_loopback_address_validate(&ipv4_address, &ipv4_netmask);
  CuAssertIntEquals(tc, sncr_parseerror, result);
  ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(127, 0, 0, 1);
  ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 0, 0, 0);
  result = snc_ipv4_loopback_address_validate(&ipv4_address, &ipv4_netmask);
  CuAssertIntEquals(tc, sncr_success, result);

  /* Test loopback subnet broadcast */
  ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(127, 255, 255, 255);
  ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 0, 0, 0);
  result = snc_ipv4_loopback_address_validate(&ipv4_address, &ipv4_netmask);
  CuAssertIntEquals(tc, sncr_parseerror, result);
  ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(127, 255, 255, 254);
  ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 0, 0, 0);
  result = snc_ipv4_loopback_address_validate(&ipv4_address, &ipv4_netmask);
  CuAssertIntEquals(tc, sncr_success, result);

  /* Test bad loopback netmask */
  ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(127, 0, 0, 1);
  ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(254, 0, 0, 0);
  result = snc_ipv4_loopback_address_validate(&ipv4_address, &ipv4_netmask);
  CuAssertIntEquals(tc, sncr_parseerror, result);

  /* Test /31 special case */
  ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(127, 0, 0, 0);
  ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 254);
  result = snc_ipv4_loopback_address_validate(&ipv4_address, &ipv4_netmask);
  CuAssertIntEquals(tc, sncr_success, result);
  ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(127, 0, 0, 1);
  ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 254);
  result = snc_ipv4_loopback_address_validate(&ipv4_address, &ipv4_netmask);
  CuAssertIntEquals(tc, sncr_success, result);
}


/** Test snc_ipv4_gateway_address_validate() */
static void test_snc_ipv4_gateway_address_validate(CuTest *tc)
{
  enum e_snc_result result;
  struct in_addr ipv4_netmask;
  struct in_addr ipv4_address;
  struct in_addr ipv4_gateway;

  /* Test an everyday gateway address */
  ipv4_gateway.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 10, 1);
  ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 10, 10);
  ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 0);
  result = snc_ipv4_gateway_address_validate(&ipv4_gateway, &ipv4_address, &ipv4_netmask);
  CuAssertIntEquals(tc, sncr_success, result);

  /* A gateway address has the same criteria as a host address */
  check_snc_ipv4_host_address_validate(tc, run_snc_ipv4_gateway_address_validate);

  /* Test a gateway on a different subnet */
  ipv4_gateway.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 1, 1);
  ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 10, 10);
  ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 0);
  result = snc_ipv4_gateway_address_validate(&ipv4_gateway, &ipv4_address, &ipv4_netmask);
  CuAssertIntEquals(tc, sncr_parseerror, result);

  /* Test /31 special case */
  ipv4_gateway.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 1, 0);
  ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 1, 1);
  ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 254);
  result = snc_ipv4_gateway_address_validate(&ipv4_gateway, &ipv4_address, &ipv4_netmask);
  CuAssertIntEquals(tc, sncr_success, result);
  ipv4_gateway.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 1, 1);
  ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 1, 0);
  ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 254);
  result = snc_ipv4_gateway_address_validate(&ipv4_gateway, &ipv4_address, &ipv4_netmask);
  CuAssertIntEquals(tc, sncr_success, result);
}


/** Test snc_ipv4_broadcast_validate() */
static void test_snc_ipv4_broadcast_validate(CuTest *tc)
{
  enum e_snc_result result;
  struct in_addr ipv4_netmask;
  struct in_addr ipv4_address;
  struct in_addr ipv4_broadcast;

  /* Test an everyday broadcast address */
  ipv4_broadcast.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 10, 255);
  ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 10, 10);
  ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 0);
  result = snc_ipv4_broadcast_validate(&ipv4_broadcast, &ipv4_address, &ipv4_netmask);
  CuAssertIntEquals(tc, sncr_success, result);

  /* Test invalid broadcast address */
  ipv4_broadcast.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 10, 254);
  ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 10, 10);
  ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 0);
  result = snc_ipv4_broadcast_validate(&ipv4_broadcast, &ipv4_address, &ipv4_netmask);
  CuAssertIntEquals(tc, sncr_parseerror, result);

  /* Test invalid broadcast address */
  ipv4_broadcast.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 10, 255);
  ipv4_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 10, 10);
  ipv4_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 0, 0);
  result = snc_ipv4_broadcast_validate(&ipv4_broadcast, &ipv4_address, &ipv4_netmask);
  CuAssertIntEquals(tc, sncr_parseerror, result);
}


/** Test snc_net_config_validate() */
static void test_snc_net_config_validate(CuTest *tc)
{
  enum e_snc_result result;
  struct s_snc_net_config net_config;
  uint32_t valid_mask;
  uint32_t expected_valid_mask;

  /* Test a valid net_config */
  net_config.use_dhcp = false;
  net_config.ip_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 1, 100);
  net_config.ip_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 0, 0);
  net_config.ip_broadcast.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 255, 255);
  net_config.ip_gateway.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 5, 10);
  net_config.ip_dns[0].s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(172, 17, 1, 55);
  net_config.ip_dns[1].s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 6, 20);
  net_config.ip_dns[2].s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 7, 30);
  snprintf(net_config.search_domain, SNC_DOMAIN_NAME_MAX, "search.domain");
  snprintf(net_config.hostname, SNC_HOST_NAME_MAX, "hostname");
  snprintf(net_config.domain, SNC_DOMAIN_NAME_MAX, "test.domain");
  result = snc_net_config_validate(&net_config, &valid_mask, false);
  CuAssertIntEquals(tc, sncr_success, result);
  CuAssertIntEquals(tc, SNC_VALIDATE_VALID, valid_mask);

  /* Test a invalid net_config */
  net_config.use_dhcp = false;
  net_config.ip_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(224, 168, 1, 100);       /* Class D */
  net_config.ip_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 0);       /* Valid so other tests work */
  net_config.ip_broadcast.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 255, 255);
  net_config.ip_gateway.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 1, 1);         /* Off ip address subnet */
  net_config.ip_dns[0].s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(240, 17, 1, 55);          /* Class E */
  net_config.ip_dns[1].s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(241, 168, 6, 20);         /* Class E */
  net_config.ip_dns[2].s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(242, 168, 7, 30);         /* Class E */
  snprintf(net_config.search_domain, SNC_DOMAIN_NAME_MAX, "search.domain;");                /* Bad characters */
  snprintf(net_config.hostname, SNC_HOST_NAME_MAX, "hostname?");                            /* Bad characters */
  snprintf(net_config.domain, SNC_DOMAIN_NAME_MAX, "test.domain:");                         /* Bad characters */
  result = snc_net_config_validate(&net_config, &valid_mask, false);
  CuAssertIntEquals(tc, sncr_parseerror, result);

  expected_valid_mask = SNC_VALIDATE_IPV4_ADDRESS_INVALID | SNC_VALIDATE_IPV4_BROADCAST_INVALID | SNC_VALIDATE_IPV4_GATEWAY_INVALID |
                        SNC_VALIDATE_IPV4_DNS0_INVALID | SNC_VALIDATE_IPV4_DNS1_INVALID | SNC_VALIDATE_IPV4_DNS2_INVALID |
                        SNC_VALIDATE_SEARCHDOMAIN_INVALID | SNC_VALIDATE_HOSTNAME_INVALID | SNC_VALIDATE_DOMAIN_INVALID;
  CuAssertIntEquals(tc, expected_valid_mask, valid_mask);

  /* Test a invalid net_config, but with DHCP enabled */
  net_config.use_dhcp = true;
  net_config.ip_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(224, 168, 1, 100);       /* Class D */
  net_config.ip_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 0);       /* Valid so other tests work */
  net_config.ip_broadcast.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 255, 255);
  net_config.ip_gateway.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 1, 1);         /* Off ip address subnet */
  net_config.ip_dns[0].s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(240, 17, 1, 55);          /* Class E */
  net_config.ip_dns[1].s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(241, 168, 6, 20);         /* Class E */
  net_config.ip_dns[2].s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(242, 168, 7, 30);         /* Class E */
  snprintf(net_config.search_domain, SNC_DOMAIN_NAME_MAX, "search.domain;");                /* Bad characters */
  snprintf(net_config.hostname, SNC_HOST_NAME_MAX, "hostname");                             /* Valid */
  snprintf(net_config.domain, SNC_DOMAIN_NAME_MAX, "test.domain");                          /* Valid */
  result = snc_net_config_validate(&net_config, &valid_mask, false);
  CuAssertIntEquals(tc, sncr_success, result);
  CuAssertIntEquals(tc, SNC_VALIDATE_VALID, valid_mask);

  /* Test a invalid net_config, but DHCP enabled, and not ignoring dhcp fields */
  net_config.use_dhcp = true;
  net_config.ip_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(224, 168, 1, 100);       /* Class D */
  net_config.ip_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 0);       /* Valid so other tests work */
  net_config.ip_broadcast.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 255, 255);
  net_config.ip_gateway.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 1, 1);         /* Off ip address subnet */
  net_config.ip_dns[0].s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(240, 17, 1, 55);          /* Class E */
  net_config.ip_dns[1].s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(241, 168, 6, 20);         /* Class E */
  net_config.ip_dns[2].s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(242, 168, 7, 30);         /* Class E */
  snprintf(net_config.search_domain, SNC_DOMAIN_NAME_MAX, "search.domain;");                /* Bad characters */
  snprintf(net_config.hostname, SNC_HOST_NAME_MAX, "hostname");                             /* Valid */
  snprintf(net_config.domain, SNC_DOMAIN_NAME_MAX, "test.domain");                          /* Valid */
  result = snc_net_config_validate(&net_config, &valid_mask, true);
  CuAssertIntEquals(tc, sncr_parseerror, result);

  expected_valid_mask = SNC_VALIDATE_IPV4_ADDRESS_INVALID | SNC_VALIDATE_IPV4_BROADCAST_INVALID | SNC_VALIDATE_IPV4_GATEWAY_INVALID |
                        SNC_VALIDATE_IPV4_DNS0_INVALID | SNC_VALIDATE_IPV4_DNS1_INVALID | SNC_VALIDATE_IPV4_DNS2_INVALID |
                        SNC_VALIDATE_SEARCHDOMAIN_INVALID;
  CuAssertIntEquals(tc, expected_valid_mask, valid_mask);
}


/** Test the IPV4 address validation functions */
CuSuite *CuGetSuite_lsnc_ipv4_validate(void)
{
  CuSuite *suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_snc_ipv4_netmask_validate);
  SUITE_ADD_TEST(suite, test_snc_ipv4_single_destination_address_validate);
  SUITE_ADD_TEST(suite, test_snc_ipv4_host_address_validate);
  SUITE_ADD_TEST(suite, test_snc_ipv4_loopback_address_validate);
  SUITE_ADD_TEST(suite, test_snc_ipv4_gateway_address_validate);
  SUITE_ADD_TEST(suite, test_snc_ipv4_broadcast_validate);
  SUITE_ADD_TEST(suite, test_snc_net_config_validate);

  return suite;
}
