/*
 * lsnc_test_rcconf.c
 *
 *  Created on: 12 Jul. 2018
 *      Author: ian
 */
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <CuTest.h>
#include <setnetconfig.h>

#include "lsnc_test_rcconf.h"


#define TEST_TMPFILE_TEMPLATE "lsnc_test_tmpfileXXXXXX"


/** rc.conf with static IP configuration */
static const char rcconf_static_ip[] =
  "all_services=\"mount-proc-sys mdev udev hostname devfsd filesystems depmod modules syslog network inetd portmap dropbear sshd boa smb dhcpd settime qtopia watchdog gtk2 pango\"\n"
  "all_services_r=\"pango gtk2 watchdog qtopia settime dhcpd smb boa sshd dropbear portmap inetd network syslog modules depmod filesystems devfsd hostname udev mdev mount-proc-sys\"\n"
  "\n"
  "cfg_services=\" mount-proc-sys mdev hostname filesystems depmod modules syslog l000-502 network c500-500 rindevicefinder l900-501 boa\"\n"
  "\n"
  "cfg_services_r=\"boa l900-501 rindevicefinder c500-500 network l000-502 syslog modules depmod filesystems hostname mdev mount-proc-sys  \"\n"
  "\n"
  "upg_kill_services=\"l900-501 c500-500 inetd rindevicefinder  \"\n"
  "\n"
  "export NTP_SERVER=\"\"\n"
  "export MODLIST=\"\"\n"
  "export RAMDIRS=\"/tmp /var\"\n"
  "export TMPFS=\"ramfs\"\n"
  "export TMPFS_SIZE=\"512k\"\n"
  "export READONLY_FS=\"\"\n"
  "export INETD_ARGS=\"\"\n"
  "export BOA_ARGS=\"-c /etc\"\n"
  "export SMBD_ARGS=\"\"\n"
  "export NMBD_ARGS=\"\"\n"
  "export DHCP_ARG=\"\"\n"
  "export DEPLOYMENT_STYLE=\"JFFS2\"\n"
  "export SYSLOGD_OPTIONS=\"\"\n"
  "export KLOGD_OPTIONS=\"\"\n"
  "export SYSCFG_DHCPC_CMD=\"udhcpc --syslog -x hostname:$HOSTNAME --interface \"\n"
  "# net interface 0\n"
  "export INTERFACE0=\"eth0\"\n"
  "export HOSTNAME=\"c520-3477741\"\n"
  "export DOMAINNAME=\"domain.local\"\n"
  "export SYSCFG_IFACE0=\"y\"\n"
  "export IPADDR0=\"172.17.1.59\"\n"
  "export NETMASK0=\"255.255.255.0\"\n"
  "export BROADCAST0=\"172.17.1.255\"\n"
  "export GATEWAY0=\"172.17.1.1\"\n"
  "export NAMESERVER0=\"172.17.1.2\"\n"
  "export NAMESERVER0_2=\"0.0.0.0\"\n"
  "export NAMESERVER0_3=\"\"\n"
  "export SEARCHDOMAIN0=\"search.domain\"\n";


/** Expected rc.conf with static IP configuration */
static const char rcconf_static_ip_expected[] =
  "all_services=\"mount-proc-sys mdev udev hostname devfsd filesystems depmod modules syslog network inetd portmap dropbear sshd boa smb dhcpd settime qtopia watchdog gtk2 pango\"\n"
  "all_services_r=\"pango gtk2 watchdog qtopia settime dhcpd smb boa sshd dropbear portmap inetd network syslog modules depmod filesystems devfsd hostname udev mdev mount-proc-sys\"\n"
  "\n"
  "cfg_services=\" mount-proc-sys mdev hostname filesystems depmod modules syslog l000-502 network c500-500 rindevicefinder l900-501 boa\"\n"
  "\n"
  "cfg_services_r=\"boa l900-501 rindevicefinder c500-500 network l000-502 syslog modules depmod filesystems hostname mdev mount-proc-sys  \"\n"
  "\n"
  "upg_kill_services=\"l900-501 c500-500 inetd rindevicefinder  \"\n"
  "\n"
  "export NTP_SERVER=\"\"\n"
  "export MODLIST=\"\"\n"
  "export RAMDIRS=\"/tmp /var\"\n"
  "export TMPFS=\"ramfs\"\n"
  "export TMPFS_SIZE=\"512k\"\n"
  "export READONLY_FS=\"\"\n"
  "export INETD_ARGS=\"\"\n"
  "export BOA_ARGS=\"-c /etc\"\n"
  "export SMBD_ARGS=\"\"\n"
  "export NMBD_ARGS=\"\"\n"
  "export DHCP_ARG=\"\"\n"
  "export DEPLOYMENT_STYLE=\"JFFS2\"\n"
  "export SYSLOGD_OPTIONS=\"\"\n"
  "export KLOGD_OPTIONS=\"\"\n"
  "export SYSCFG_DHCPC_CMD=\"udhcpc --syslog -x hostname:$HOSTNAME --interface \"\n"
  "# net interface 0\n"
  "export INTERFACE0=\"eth0\"\n"
  "export SYSCFG_IFACE0=\"y\"\n"
  "export IPADDR0=\"192.168.100.108\"\n"
  "export NETMASK0=\"255.255.255.0\"\n"
  "export BROADCAST0=\"192.168.100.255\"\n"
  "export GATEWAY0=\"192.168.100.1\"\n"
  "export NAMESERVER0=\"192.168.100.2\"\n"
  "export NAMESERVER0_2=\"192.168.100.3\"\n"
  "export NAMESERVER0_3=\"\"\n"
  "export SEARCHDOMAIN0=\"test.search.domain\"\n"
  "export HOSTNAME=\"test-hostname\"\n"
  "export DOMAINNAME=\"test.domain\"\n";


/** rc.conf with DHCP configuration */
static const char rcconf_dhcp[] =
  "all_services=\"mount-proc-sys mdev udev hostname devfsd filesystems depmod modules syslog network inetd portmap dropbear sshd boa smb dhcpd settime qtopia watchdog gtk2 pango\"\n"
  "all_services_r=\"pango gtk2 watchdog qtopia settime dhcpd smb boa sshd dropbear portmap inetd network syslog modules depmod filesystems devfsd hostname udev mdev mount-proc-sys\"\n"
  "\n"
  "cfg_services=\" mount-proc-sys mdev hostname filesystems depmod modules syslog l000-502 network c500-500 rindevicefinder l900-501 boa\"\n"
  "\n"
  "cfg_services_r=\"boa l900-501 rindevicefinder c500-500 network l000-502 syslog modules depmod filesystems hostname mdev mount-proc-sys  \"\n"
  "\n"
  "upg_kill_services=\"l900-501 c500-500 inetd rindevicefinder  \"\n"
  "\n"
  "export NTP_SERVER=\"\"\n"
  "export MODLIST=\"\"\n"
  "export RAMDIRS=\"/tmp /var\"\n"
  "export TMPFS=\"ramfs\"\n"
  "export TMPFS_SIZE=\"512k\"\n"
  "export READONLY_FS=\"\"\n"
  "export INETD_ARGS=\"\"\n"
  "export BOA_ARGS=\"-c /etc\"\n"
  "export SMBD_ARGS=\"\"\n"
  "export NMBD_ARGS=\"\"\n"
  "export DHCP_ARG=\"\"\n"
  "export DEPLOYMENT_STYLE=\"JFFS2\"\n"
  "export SYSLOGD_OPTIONS=\"\"\n"
  "export KLOGD_OPTIONS=\"\"\n"
  "export SYSCFG_DHCPC_CMD=\"udhcpc --syslog -x hostname:$HOSTNAME --interface \"\n"
  "# net interface 0\n"
  "export INTERFACE0=\"eth0\"\n"
  "export HOSTNAME=\"c520-3477741\"\n"
  "export DOMAINNAME=\"domain.local\"\n"
  "export SYSCFG_IFACE0=\"y\"\n"
  "export IPADDR0=\"dhcp\"\n";


/** Expected rc.conf with DHCP configuration */
static const char rcconf_dhcp_expected[] =
  "all_services=\"mount-proc-sys mdev udev hostname devfsd filesystems depmod modules syslog network inetd portmap dropbear sshd boa smb dhcpd settime qtopia watchdog gtk2 pango\"\n"
  "all_services_r=\"pango gtk2 watchdog qtopia settime dhcpd smb boa sshd dropbear portmap inetd network syslog modules depmod filesystems devfsd hostname udev mdev mount-proc-sys\"\n"
  "\n"
  "cfg_services=\" mount-proc-sys mdev hostname filesystems depmod modules syslog l000-502 network c500-500 rindevicefinder l900-501 boa\"\n"
  "\n"
  "cfg_services_r=\"boa l900-501 rindevicefinder c500-500 network l000-502 syslog modules depmod filesystems hostname mdev mount-proc-sys  \"\n"
  "\n"
  "upg_kill_services=\"l900-501 c500-500 inetd rindevicefinder  \"\n"
  "\n"
  "export NTP_SERVER=\"\"\n"
  "export MODLIST=\"\"\n"
  "export RAMDIRS=\"/tmp /var\"\n"
  "export TMPFS=\"ramfs\"\n"
  "export TMPFS_SIZE=\"512k\"\n"
  "export READONLY_FS=\"\"\n"
  "export INETD_ARGS=\"\"\n"
  "export BOA_ARGS=\"-c /etc\"\n"
  "export SMBD_ARGS=\"\"\n"
  "export NMBD_ARGS=\"\"\n"
  "export DHCP_ARG=\"\"\n"
  "export DEPLOYMENT_STYLE=\"JFFS2\"\n"
  "export SYSLOGD_OPTIONS=\"\"\n"
  "export KLOGD_OPTIONS=\"\"\n"
  "export SYSCFG_DHCPC_CMD=\"udhcpc --syslog -x hostname:$HOSTNAME --interface \"\n"
  "# net interface 0\n"
  "export INTERFACE0=\"eth0\"\n"
  "export SYSCFG_IFACE0=\"y\"\n"
  "export IPADDR0=\"dhcp\"\n"
  "export HOSTNAME=\"test-hostname\"\n"
  "export DOMAINNAME=\"test.domain\"\n";


/** Test snc_get_net_config_rcconf() with static IP configuration */
static void test_snc_get_net_config_rcconf_static_ip(CuTest* tc)
{
  enum e_snc_result result;
  struct s_snc_net_config net_config;
  FILE *rcconf_file;
  char tmp_filename[] = TEST_TMPFILE_TEMPLATE;
  int rcconf_fd;

  rcconf_fd = mkstemp(tmp_filename);
  CuAssertTrue(tc, rcconf_fd >= 0);
  rcconf_file = fdopen(rcconf_fd, "r+");
  CuAssertPtrNotNull(tc, rcconf_file);

  fwrite(rcconf_static_ip, sizeof(char), strlen(rcconf_static_ip), rcconf_file);
  fflush(rcconf_file);

  result = snc_get_net_config_rcconf(tmp_filename, "eth0", &net_config);
  CuAssertIntEquals(tc, sncr_success, result);
  CuAssertIntEquals(tc, false, net_config.use_dhcp);
  CuAssertIntEquals(tc, SNC_IPV4_OCTETS_TO_INADDR_NETWORK(172, 17, 1, 59), net_config.ip_address.s_addr);
  CuAssertIntEquals(tc, SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 0), net_config.ip_netmask.s_addr);
  CuAssertIntEquals(tc, SNC_IPV4_OCTETS_TO_INADDR_NETWORK(172, 17, 1, 255), net_config.ip_broadcast.s_addr);
  CuAssertIntEquals(tc, SNC_IPV4_OCTETS_TO_INADDR_NETWORK(172, 17, 1, 1), net_config.ip_gateway.s_addr);
  CuAssertIntEquals(tc, SNC_IPV4_OCTETS_TO_INADDR_NETWORK(172, 17, 1, 2), net_config.ip_dns[0].s_addr);
  CuAssertIntEquals(tc, SNC_IPV4_OCTETS_TO_INADDR_NETWORK(0, 0, 0, 0), net_config.ip_dns[1].s_addr);
  CuAssertIntEquals(tc, SNC_IPV4_OCTETS_TO_INADDR_NETWORK(0, 0, 0, 0), net_config.ip_dns[2].s_addr);

  CuAssertStrEquals(tc, "search.domain", net_config.search_domain);
  CuAssertStrEquals(tc, "domain.local", net_config.domain);
  CuAssertStrEquals(tc, "c520-3477741", net_config.hostname);

  fclose(rcconf_file);
  close(rcconf_fd);
  unlink(tmp_filename);
}


/** Test snc_get_net_config_rcconf() with DHCP configuration */
static void test_snc_get_net_config_rcconf_dhcp(CuTest* tc)
{
  enum e_snc_result result;
  struct s_snc_net_config net_config;
  FILE *rcconf_file;
  char tmp_filename[] = TEST_TMPFILE_TEMPLATE;
  int rcconf_fd;

  rcconf_fd = mkstemp(tmp_filename);
  CuAssertTrue(tc, rcconf_fd >= 0);
  rcconf_file = fdopen(rcconf_fd, "r+");
  CuAssertPtrNotNull(tc, rcconf_file);

  fwrite(rcconf_dhcp, sizeof(char), strlen(rcconf_dhcp), rcconf_file);
  fflush(rcconf_file);

  result = snc_get_net_config_rcconf(tmp_filename, "eth0", &net_config);
  CuAssertIntEquals(tc, sncr_success, result);
  CuAssertIntEquals(tc, true, net_config.use_dhcp);
  CuAssertIntEquals(tc, SNC_IPV4_UNSET_INADDR_NETWORK, net_config.ip_address.s_addr);
  CuAssertIntEquals(tc, SNC_IPV4_UNSET_INADDR_NETWORK, net_config.ip_netmask.s_addr);
  CuAssertIntEquals(tc, SNC_IPV4_UNSET_INADDR_NETWORK, net_config.ip_broadcast.s_addr);
  CuAssertIntEquals(tc, SNC_IPV4_UNSET_INADDR_NETWORK, net_config.ip_gateway.s_addr);
  CuAssertIntEquals(tc, SNC_IPV4_UNSET_INADDR_NETWORK, net_config.ip_dns[0].s_addr);
  CuAssertIntEquals(tc, SNC_IPV4_UNSET_INADDR_NETWORK, net_config.ip_dns[1].s_addr);
  CuAssertIntEquals(tc, SNC_IPV4_UNSET_INADDR_NETWORK, net_config.ip_dns[2].s_addr);

  CuAssertStrEquals(tc, "", net_config.search_domain);
  CuAssertStrEquals(tc, "domain.local", net_config.domain);
  CuAssertStrEquals(tc, "c520-3477741", net_config.hostname);

  fclose(rcconf_file);
  close(rcconf_fd);
  unlink(tmp_filename);
}


/** Test snc_set_net_config_rcconf() with static IP configuration */
static void test_snc_set_net_config_rcconf_static_ip(CuTest* tc)
{
  enum e_snc_result result;
  struct s_snc_net_config net_config;
  FILE *rcconf_file;
  char tmp_filename[] = TEST_TMPFILE_TEMPLATE;
  int rcconf_fd;
  bool set_ifacecfg = true;
  bool set_hostname = true;
  bool set_domain = true;
  char *str;
  ssize_t readlen;
  long filelen;

  const struct s_snc_configuration snc_config =
  {
    .resolv_file = "",
    .route_cmd = "",
    .ifconfig_cmd = "",
    .ip_cmd = "",
    .rcconf_file = tmp_filename,
    .config_method = SNC_METHOD_RCCONF,
    .current_method = SNC_METHOD_IOCTL,
    .net_reconfigure_cmd = "",
    .hostname_reconfigure_cmd = ""
  };

  /* Setup existing rc.conf */
  rcconf_fd = mkstemp(tmp_filename);
  CuAssertTrue(tc, rcconf_fd >= 0);
  rcconf_file = fdopen(rcconf_fd, "r+");
  CuAssertPtrNotNull(tc, rcconf_file);

  fwrite(rcconf_static_ip, sizeof(char), strlen(rcconf_static_ip), rcconf_file);
  fclose(rcconf_file);
  rcconf_file = NULL;
  close(rcconf_fd);
  rcconf_fd = -1;

  /* Set these values in rc.conf */
  net_config.use_dhcp = false;
  net_config.ip_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 100, 108);
  net_config.ip_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 0);
  net_config.ip_broadcast.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 100, 255);
  net_config.ip_gateway.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 100, 1);
  net_config.ip_dns[0].s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 100, 2);
  net_config.ip_dns[1].s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 100, 3);
  net_config.ip_dns[2].s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(0, 0, 0, 0);
  snprintf(net_config.search_domain, SNC_DOMAIN_NAME_MAX, "test.search.domain");
  snprintf(net_config.hostname, SNC_HOST_NAME_MAX, "test-hostname");
  snprintf(net_config.domain, SNC_DOMAIN_NAME_MAX, "test.domain");

  result = snc_set_net_config_rcconf(&snc_config, "eth0", &net_config, &set_ifacecfg, &set_hostname, &set_domain);
  CuAssertIntEquals(tc, sncr_success, result);

  rcconf_file = fopen(tmp_filename, "r");
  fseek(rcconf_file, 0, SEEK_END);
  filelen = ftell(rcconf_file);
  fseek(rcconf_file, 0, SEEK_SET);
  str = malloc(filelen);
  readlen = fread(str, sizeof(char), filelen, rcconf_file);
  CuAssertIntEquals(tc, filelen, readlen);
  str[readlen] = '\0';

  CuAssertStrEquals(tc, rcconf_static_ip_expected, str);
  fclose(rcconf_file);
  free(str);
  unlink(tmp_filename);
}


/** Test snc_set_net_config_rcconf() with DHCP configuration */
static void test_snc_set_net_config_rcconf_dhcp(CuTest* tc)
{
  enum e_snc_result result;
  struct s_snc_net_config net_config;
  FILE *rcconf_file;
  char tmp_filename[] = TEST_TMPFILE_TEMPLATE;
  int rcconf_fd;
  bool set_ifacecfg = true;
  bool set_hostname = true;
  bool set_domain = true;
  char *str;
  ssize_t readlen;
  long filelen;

  const struct s_snc_configuration snc_config =
  {
    .resolv_file = "",
    .route_cmd = "",
    .ifconfig_cmd = "",
    .ip_cmd = "",
    .rcconf_file = tmp_filename,
    .config_method = SNC_METHOD_RCCONF,
    .current_method = SNC_METHOD_IOCTL,
    .net_reconfigure_cmd = "",
    .hostname_reconfigure_cmd = ""
  };

  /* Setup existing rc.conf */
  rcconf_fd = mkstemp(tmp_filename);
  CuAssertTrue(tc, rcconf_fd >= 0);
  rcconf_file = fdopen(rcconf_fd, "r+");
  CuAssertPtrNotNull(tc, rcconf_file);

  fwrite(rcconf_static_ip, sizeof(char), strlen(rcconf_static_ip), rcconf_file);
  fclose(rcconf_file);
  rcconf_file = NULL;
  close(rcconf_fd);
  rcconf_fd = -1;

  /* Set these values in rc.conf */
  net_config.use_dhcp = true;
  net_config.ip_address.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 100, 108);
  net_config.ip_netmask.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(255, 255, 255, 0);
  net_config.ip_broadcast.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 100, 255);
  net_config.ip_gateway.s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 100, 1);
  net_config.ip_dns[0].s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 100, 2);
  net_config.ip_dns[1].s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(192, 168, 100, 3);
  net_config.ip_dns[2].s_addr = SNC_IPV4_OCTETS_TO_INADDR_NETWORK(0, 0, 0, 0);
  snprintf(net_config.search_domain, SNC_DOMAIN_NAME_MAX, "test.search.domain");
  snprintf(net_config.hostname, SNC_HOST_NAME_MAX, "test-hostname");
  snprintf(net_config.domain, SNC_DOMAIN_NAME_MAX, "test.domain");

  result = snc_set_net_config_rcconf(&snc_config, "eth0", &net_config, &set_ifacecfg, &set_hostname, &set_domain);
  CuAssertIntEquals(tc, sncr_success, result);

  rcconf_file = fopen(tmp_filename, "r");
  fseek(rcconf_file, 0, SEEK_END);
  filelen = ftell(rcconf_file);
  fseek(rcconf_file, 0, SEEK_SET);
  str = malloc(filelen);
  readlen = fread(str, sizeof(char), filelen, rcconf_file);
  CuAssertIntEquals(tc, filelen, readlen);
  str[readlen] = '\0';

  CuAssertStrEquals(tc, rcconf_dhcp_expected, str);
  fclose(rcconf_file);
  free(str);
  unlink(tmp_filename);
}


/** Test the rcconf functions of libsetnetconfig */
CuSuite *CuGetSuite_lsnc_rcconf(void)
{
  CuSuite* suite = CuSuiteNew();

  SUITE_ADD_TEST(suite, test_snc_get_net_config_rcconf_static_ip);
  SUITE_ADD_TEST(suite, test_snc_get_net_config_rcconf_dhcp);
  SUITE_ADD_TEST(suite, test_snc_set_net_config_rcconf_static_ip);
  SUITE_ADD_TEST(suite, test_snc_set_net_config_rcconf_dhcp);

  return suite;
}
