#!/bin/sh
#
# Parses the settings out of the WE2111 debug file and applies them to the WE2111 the script is run on
#
# Upload this script plus the debug file to the WE2111, then run it like:
# debuginfo_to_we2111.sh <debugfile> -> extract all files to /root
# debuginfo_to_we2111.sh <debugfile> forreal -> extract all files and install to their correct location on the WE2111

#set -x
set -e

debugfile="$1"
action="$2"
extractto="/root"

echo "This script will parse the settings out of a WE2111 debug file"
echo "(eg WE2111_3440944_20150706_122023_debug_info.txt) and"
if [ "$action" == "forreal" ]; then
  echo "install them to this WE2111."
  echo "WARNING: This has the ability to destroy all settings on the"
  echo "unit, including factory calibration and ethernet MAC"
  echo "I'd suggest doing a full backup before running this."
  read -p "Are you really sure you want to continue? [y/n]"
  if [ "$REPLY" != "y" ]; then
    echo "Aborted."
    exit 1
  fi
else
  echo "copy them to $extractto for your perusal."
fi
 

facuboot="$extractto/facuboot.txt"

if [ "$action" == "forreal" ]; then
  fac1='/etc/rinstrum/factory1.rdb'
  fac2='/etc/rinstrum/factory2.rdb'
  user1='/etc/rinstrum/user1.rdb'
  user2='/etc/rinstrum/user2.rdb'
  rcconf='/etc/rc.d/rc.conf'
  uboot_eth_file='uboot'
  uboot_fac_file='uboot'
  uboot_eth='fw_setenv -s -'
  uboot_fac='fw_setenv -s -'
  mdevconf='/etc/mdev.conf'
  weconfig='/etc/rinstrum/weconfig.xml'
else
  fac1="$extractto/factory1.rdb"
  fac2="$extractto/factory2.rdb"
  user1="$extractto/user1.rdb"
  user2="$extractto/user2.rdb"
  rcconf="$extractto/rc.conf"
  uboot_eth_file="$extractto/uboot_ethernet.txt"
  uboot_fac_file="$extractto/uboot_factory.txt"
  uboot_eth="cat - > $uboot_eth_file"
  uboot_fac="cat - > $uboot_fac_file"
  mdevconf="$extractto/mdev.conf"
  weconfig="$extractto/weconfig.xml"
fi

# Factory database first copy
read -p "Install factory1.rdb to $fac1? [y/n]"
if [ "$REPLY" == "y" ]; then
  awk '
    /Contents of \"\/etc\/rinstrum\/factory1.rdb\"/ { m = 2; }
    /--------------------------------------------------------------------------------/ { m = 0; }
    {
      if (m == 2) m = 1;
      else if (m) print
    }
    BEGIN { m = 0; }
  ' $debugfile > $fac1
fi

# Factory database second copy
read -p "Install factory2.rdb to $fac2? [y/n]"
if [ "$REPLY" == "y" ]; then
  awk '
    /Contents of \"\/etc\/rinstrum\/factory2.rdb\"/ { m = 2; }
    /--------------------------------------------------------------------------------/ { m = 0; }
    {
      if (m == 2) m = 1;
      else if (m) print
    }
    BEGIN { m = 0; }
  ' $debugfile > $fac2
fi

# User database first copy
read -p "Install user1.rdb to $user1? [y/n]"
if [ "$REPLY" == "y" ]; then
  awk '
    /Contents of \"\/etc\/rinstrum\/user1.rdb\"/ { m = 2; }
    /--------------------------------------------------------------------------------/ { m = 0; }
    {
      if (m == 2) m = 1;
      else if (m) print
    }
    BEGIN { m = 0; }
  ' $debugfile > $user1
fi

# User database second copy
read -p "Install user2.rdb to $user2? [y/n]"
if [ "$REPLY" == "y" ]; then
  awk '
    /Contents of \"\/etc\/rinstrum\/user2.rdb\"/ { m = 2; }
    /--------------------------------------------------------------------------------/ { m = 0; }
    {
      if (m == 2) m = 1;
      else if (m) print
    }
    BEGIN { m = 0; }
  ' $debugfile > $user2
fi

# Uboot environment variables
awk '
  /Contents of \"fw_printenv\"/ { m = 2; }
  /--------------------------------------------------------------------------------/ { m = 0; }
  {
    if (m == 2) m = 1;
    else if (m) print
  }
  BEGIN { m = 0; }
' $debugfile > $facuboot

# Uboot factory database
read -p "Install fw_printenv factory database to $uboot_fac_file? [y/n]"
if [ "$REPLY" == "y" ]; then
  sed -n -r 's/^(factorydb_[^=]*)=(.*)/\1 \2/p' < $facuboot | eval $uboot_fac
fi

# Uboot MAC address
read -p "Install fw_printenv MAC address to $uboot_eth_file? [y/n]"
if [ "$REPLY" == "y" ]; then
  sed -n -r 's/^(ethaddr)=(.*)/\1 \2/p' < $facuboot | eval $uboot_eth
fi

# rc.conf
read -p "Install rc.conf to $rcconf? [y/n]"
if [ "$REPLY" == "y" ]; then
  awk '
    /Contents of \"\/etc\/rc.d\/rc.conf\"/ { m = 2; }
    /--------------------------------------------------------------------------------/ { m = 0; }
    {
      if (m == 2) m = 1;
      else if (m) print
    }
    BEGIN { m = 0; }
  ' $debugfile > $rcconf
fi

# mdev.conf
read -p "Install mdev.conf to $mdevconf? [y/n]"
if [ "$REPLY" == "y" ]; then
  awk '
    /Contents of \"\/etc\/mdev.conf\"/ { m = 2; }
    /--------------------------------------------------------------------------------/ { m = 0; }
    {
      if (m == 2) m = 1;
      else if (m) print
    }
    BEGIN { m = 0; }
  ' $debugfile > $mdevconf
fi

# weconfig.xml
read -p "Install weconfig.xml to $weconfig? [y/n]"
if [ "$REPLY" == "y" ]; then
  awk '
    /Contents of \"\/etc\/rinstrum\/weconfig.xml\"/ { m = 2; }
    /--------------------------------------------------------------------------------/ { m = 0; }
    {
      if (m == 2) m = 1;
      else if (m) print
    }
    BEGIN { m = 0; }
  ' $debugfile > $weconfig
fi

echo "Done"
exit 0

